using System.Text;

namespace Scan_Measurement_Example
{
    public abstract class Instrument
    {
        /* Accessed Window on Measurement Instrument */
        internal const int WINDOW1 = 1; /* Bargraph Window */
        internal const int WINDOW2 = 2; /* Diagramm Window */

        /* Switch into single sweep */
        internal const bool SWEEP_MODE_CONTINUOUS = false;

        /* Number of sweeps to take */
        internal const int NUMBER_OF_SWEEPS = 1;

        /* Number of scan ranges */
        internal const int SCAN_RANGE = 1;

        /* Trace for detector definition */
        internal const int TRACE = 1;

        /* Number of elements in the arrays which are specify as the array output parameters */
        internal const int ARRAY_SIZE = 10000;

        /* Sets the number of the final measurement */
        internal const int FINAL_MEASUREMENT = 1;

        public Instrument()
        {

        }

        ~Instrument()
        {

        }

        /* Resets the instrument */
        public abstract void resetInstrument(bool resetDevice);

        /* Switches the instrument into receiver mode */
        public abstract void receiverMode();

        /* Switches on/off the update of all display elements during remote control */
        public abstract void configureDisplayUpdate(bool displayEnable);

        /* Configures the scan table */
        public abstract void configureScanTable(InstrumentType instrumentType, StepMode stepMode, double startFrequency, double stopFrequency, double stepSize, double bandwidth, double measTime, double rfAttenuation);

        /* Switch to single scan */
        public abstract void configureAcquisition();

        /* Switches block data transmission during the scan on */
        public abstract void configureFeedControl();

        /* Selects the detector used in the scan measurement */
        public abstract void configureReceiverDetector(DetectorType detectorType);

        /* Switches the unit for power for the selected measurement window into dBuV */
        public abstract void configurePowerUnit();

        /* Runs the receiver measurement */
        public abstract void runReceiverMeasurement();

        /* Contains the commands for the querying status reporting system */
        public abstract void getStatusRegister(int statusRegistersQuery, out int registerValue);

        /* Returns the result of the scan measurement */
        public abstract void readScanMeasurement(out int traceStatus, out int numOfResults, StringBuilder statusData, double[] trace1, double[] trace2, double[] trace3, double[] trace4, double[] trace5, double[] trace6);

        /* Configures the peak search */
        public abstract void configurePeakSearch(PeakSearchMethod peakSearchMethod, int numberOfPeaksSubranges);

        /* Generates a peak list from the scan result */
        public abstract void searchPeaks();

        /* Returns the results of the peak search */
        public abstract void readPeakList(double[] frequency, double[] level, double[] deltaLevel, int returnedValues);

        /* Reads an error code and a message from the instrument's error queue */
        public abstract void error_query(out int errorCode, StringBuilder errorMessage);
    }
}