﻿using System;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

namespace Scan_Measurement_Example
{
    public partial class MainWindow : Form
    {
        private WaitDialog m_waitDialog;

        private MeasurementManager m_measurementManager;

        public MainWindow()
        {
            InitializeComponent();
        }

        ~MainWindow()
        {

        }

        /* Starts the measurement process */
        private void bt_Start_Click(object sender, EventArgs e)
        {
            m_measurementManager = new MeasurementManager();

            m_measurementManager.m_applicationStatus.setApplicationIsRunning(true);

            if (m_measurementManager.m_applicationStatus.getApplicationIsRunning() == true)
            {
                setSettings();

                runMeasurementManager();

                if (m_measurementManager.m_applicationStatus.getMeasurementIsRunning() == false)
                {
                    drawDiagramm();

                    writePeakList();
                }
            }

            m_measurementManager.m_applicationStatus.setApplicationIsRunning(false);
        }

        /* Sets the settings */
        private void setSettings()
        {
            m_measurementManager.setRemoteControlSettings(comboBox_InstrumentModel.SelectedIndex, textBox_ResourceDescriptor.Text, checkBox_IDQuery.Checked, checkBox_ResetDevice.Checked, checkBox_DisplayEnable.Checked);
            m_measurementManager.setMeasurementSettings(Convert.ToDouble(numericUpDown_StartFrequency.Value), Convert.ToDouble(numericUpDown_StopFrequency.Value), comboBox_StepMode.SelectedIndex, Convert.ToDouble(numericUpDown_StepSize.Value));
            m_measurementManager.setAdvancedMeasurementSettings(comboBox_Detector.SelectedIndex, Convert.ToDouble(comboBox_ResolutionBW.Text), Convert.ToDouble(numericUpDown_MeasurementTime.Value), Convert.ToDouble(numericUpDown_RFAttenuation.Value));
            m_measurementManager.setPeakSearchSettings(radioButton_Peaks.Checked, radioButton_Subranges.Checked, Convert.ToInt32(numericUpDown_NoOfPeaksSubranges.Value));
        }

        /* Runs the measurement manager */
        public void runMeasurementManager()
        {
            /* Starts the measurement management process */
            Thread sThread = new Thread(new ThreadStart(measurementManagerProcess));
            sThread.IsBackground = true;
            sThread.Start();

            /* Display the wait dialog */
            m_waitDialog = new WaitDialog();
            m_waitDialog.ShowDialog(this);
        }

        private void measurementManagerProcess()
        {
            m_measurementManager.runMeasurementManager();
            m_waitDialog.closeDialogNow = true;
        }

        /* Draws the diagramm from the scan result and the generated peak list */
        private void drawDiagramm()
        {
            m_measurementManager.m_applicationStatus.setDrawingDiagramIsRunning(true);

            DataPoint logarithmicItem = new DataPoint();
            DataPoint linearItem = new DataPoint();

            double xValue = m_measurementManager.m_settingsManager.m_measurementSettings.getStartFrequency();

            chartDiagramm.ChartAreas["ChartArea"].AxisX.IsLogarithmic = true;

            chartDiagramm.ChartAreas["ChartArea"].AxisX.Minimum = m_measurementManager.m_settingsManager.m_measurementSettings.getStartFrequency();
            chartDiagramm.ChartAreas["ChartArea"].AxisX.Maximum = m_measurementManager.m_settingsManager.m_measurementSettings.getStopFrequency();

            chartDiagramm.Series["Graph"].Points.Clear();
            chartDiagramm.Series["Peaks"].Points.Clear();

            /* Graph */
            for (int i = 0; i <= m_measurementManager.m_dataManager.m_instrumentData.getPointsToPrint() - 1; i++)
            {
                /* Step Mode: Linear */
                if (m_measurementManager.m_settingsManager.m_measurementSettings.getStepMode() == StepMode.m_linear)
                {
                    xValue = m_measurementManager.m_settingsManager.m_measurementSettings.getStartFrequency() + (i * m_measurementManager.m_settingsManager.m_measurementSettings.getStepSize());

                    linearItem = new DataPoint(xValue, m_measurementManager.m_dataManager.m_measurementData.getTraceData()[i]);

                    chartDiagramm.Series["Graph"].Points.Add(linearItem);
                }
                /* Step Mode: Logarithmic */
                else if (m_measurementManager.m_settingsManager.m_measurementSettings.getStepMode() == StepMode.m_logarithmic)
                {
                    if (xValue > m_measurementManager.m_settingsManager.m_measurementSettings.getStopFrequency())
                    {
                        logarithmicItem = new DataPoint(m_measurementManager.m_settingsManager.m_measurementSettings.getStopFrequency(), m_measurementManager.m_dataManager.m_measurementData.getTraceData()[i]);
                    }
                    else
                    {
                        logarithmicItem = new DataPoint(xValue, m_measurementManager.m_dataManager.m_measurementData.getTraceData()[i]);
                    }

                    xValue = xValue + ((m_measurementManager.m_settingsManager.m_measurementSettings.getStepSize() / 100) * xValue);

                    chartDiagramm.Series["Graph"].Points.Add(logarithmicItem);
                }
            }

            /* Peaks */
            for (int i = 0; i < m_measurementManager.m_settingsManager.m_peakSearchSettings.getNumberOfPeaksSubranges(); i++)
            {
                chartDiagramm.Series["Peaks"].Points.AddXY(m_measurementManager.m_dataManager.m_measurementData.getPeakFrequency()[i], m_measurementManager.m_dataManager.m_measurementData.getPeakLevel()[i]);
            }

            m_measurementManager.m_applicationStatus.setDrawingDiagramIsRunning(false);
        }

        /* Writes the generated peak list into the table */
        private void writePeakList()
        {
            m_measurementManager.m_applicationStatus.setWritingPeakListIsRunning(true);

            dataGridView_PeakListTabel.Rows.Clear();

            for (int i = 0; i < m_measurementManager.m_settingsManager.m_peakSearchSettings.getNumberOfPeaksSubranges(); i++)
            {
                if (m_measurementManager.m_dataManager.m_measurementData.getPeakFrequency()[i] != 0.0 && m_measurementManager.m_dataManager.m_measurementData.getPeakLevel()[i] != 0.0)
                {
                    dataGridView_PeakListTabel.Rows.Add();
                    dataGridView_PeakListTabel.Rows[i].Cells["Frequency"].Value = m_measurementManager.m_dataManager.m_measurementData.getPeakFrequency()[i];
                    dataGridView_PeakListTabel.Rows[i].Cells["Level"].Value = m_measurementManager.m_dataManager.m_measurementData.getPeakLevel()[i];
                }
            }

            m_measurementManager.m_applicationStatus.setWritingPeakListIsRunning(false);
        }

        private void comboBox_StepMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox_StepMode.SelectedIndex == 0) //Linear
            {
                numericUpDown_StepSize.Enabled = true;
                label_StepSize.Text = "Step Size [Hz]:";
                numericUpDown_StepSize.Maximum = 10000000000;
                numericUpDown_StepSize.Minimum = 0;
                numericUpDown_StepSize.Increment = 100000;
                numericUpDown_StepSize.Value = 100000;
            }
            else if (comboBox_StepMode.SelectedIndex == 1) //Logarithmic
            {
                numericUpDown_StepSize.Enabled = true;
                label_StepSize.Text = "Step Size [%]:";
                numericUpDown_StepSize.Maximum = 100;
                numericUpDown_StepSize.Minimum = 0;
                numericUpDown_StepSize.Increment = 10;
                numericUpDown_StepSize.Value = 10;
            }
        }

        /* Changes the design for the peak search method 'Peaks' */
        private void radioButton_Peaks_CheckedChanged(object sender, EventArgs e)
        {
            label_NoOfPeaksSubranges.Text = "No of Peaks:";
            numericUpDown_NoOfPeaksSubranges.Value = 25;
            numericUpDown_NoOfPeaksSubranges.Minimum = 1;
            numericUpDown_NoOfPeaksSubranges.Maximum = 500;
        }

        /* Changes the design for the peak search method 'Subranges' */
        private void radioButton_Subranges_CheckedChanged(object sender, EventArgs e)
        {
            label_NoOfPeaksSubranges.Text = "No of Subranges:";
            numericUpDown_NoOfPeaksSubranges.Value = 10;
            numericUpDown_NoOfPeaksSubranges.Minimum = 1;
            numericUpDown_NoOfPeaksSubranges.Maximum = 50;
        }

        private void MainWindow_Load(object sender, EventArgs e)
        {
            comboBox_InstrumentModel.Text = "ESU";
            comboBox_StepMode.Text = "Linear";
            comboBox_Detector.Text = "Max Peak";
            comboBox_ResolutionBW.Text = "9000";
        }

        private void numericUpDown_StartFrequency_ValueChanged(object sender, EventArgs e)
        {
            if (numericUpDown_StopFrequency.Value <= numericUpDown_StartFrequency.Value)
            {
                numericUpDown_StopFrequency.Value = numericUpDown_StartFrequency.Value + numericUpDown_StepSize.Value;
            }
        }

        private void numericUpDown_StopFrequency_ValueChanged(object sender, EventArgs e)
        {
            if (numericUpDown_StartFrequency.Value >= numericUpDown_StopFrequency.Value)
            {
                numericUpDown_StartFrequency.Value = numericUpDown_StopFrequency.Value - numericUpDown_StepSize.Value;
            }
        }

        private void numericUpDown_StepSize_ValueChanged(object sender, EventArgs e)
        {
            if (comboBox_StepMode.Text == "Linear")
            {
                numericUpDown_StopFrequency.Value = numericUpDown_StartFrequency.Value + numericUpDown_StepSize.Value;
            }
        }

        private void comboBox_InstrumentModel_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox_InstrumentModel.Text == "ESU" || comboBox_InstrumentModel.Text == "ESL")
            {
                comboBox_Detector.Items.Add("CISPR Average");
                comboBox_Detector.Items.Add("CISPR RMS");
            }
            else if (comboBox_InstrumentModel.Text == "ESCI" || comboBox_InstrumentModel.Text == "ESPI")
            {
                comboBox_Detector.Items.Remove("CISPR Average");
                comboBox_Detector.Items.Remove("CISPR RMS");
            }
        }
    }
}