﻿using System;
using System.Text;
using InstrumentDrivers;

namespace Scan_Measurement_Example
{
    public class rsesci_Instrument : Instrument
    {
        private rsesci m_instrument;

        public rsesci_Instrument(string m_resourceDescriptor, bool m_idQery, bool m_resetDevice)
        {
            /* Initialize driver */
            m_instrument = new rsesci(m_resourceDescriptor, m_idQery, m_resetDevice);
        }

        ~rsesci_Instrument()
        {

        }

        /* Resets the instrument */
        public override void resetInstrument(bool resetDevice)
        {
            if (resetDevice == true)
                m_instrument.reset();
        }

        /* Switches the instrument into receiver mode */
        public override void receiverMode()
        {
            m_instrument.SetInstrumentMode(rsesciConstants.InstrumentModeReceiver);
        }

        /* Switches on/off the update of all display elements during remote control */
        public override void configureDisplayUpdate(bool displayEnable)
        {
            m_instrument.SetDisplayUpdate(displayEnable);
        }

        /* Configures the scan table */
        public override void configureScanTable(InstrumentType instrumentType, StepMode stepMode, double startFrequency, double stopFrequency, double stepSize, double bandwidth, double measTime, double rfAttenuation)
        {
            /* Determines the number of ranges */
            m_instrument.SetScanRanges(SCAN_RANGE);

            /* Defines the start frequency of the overall scan in receiver mode */
            m_instrument.SetStartFrequency(WINDOW2, startFrequency);

            /* Defines the stop frequency of the overall scan in receiver mode */
            m_instrument.SetStopFrequency(WINDOW2, stopFrequency);

            /* Defines the start frequency of the scan range */
            m_instrument.SetScanStart(SCAN_RANGE, startFrequency);

            /* Defines the stop frequency of the scan range */
            m_instrument.SetScanStop(SCAN_RANGE, stopFrequency);

            if (instrumentType == InstrumentType.m_ESU)
            {
                /* Switches the scan table into 'Stepped Scan Table' */
                m_instrument.SetFrequencyMode(WINDOW2, rsesciConstants.FreqScan);
            }

            if (stepMode == StepMode.m_linear)
            {
                /* Switches the step mode into linear */
                m_instrument.SetSweepSpacing(WINDOW2, rsesciConstants.Lin);

                /* This function defines the step size for the frequency of the scan range */
                m_instrument.SetScanStepSize(SCAN_RANGE, stepSize);
            }
            else if (stepMode == StepMode.m_logarithmic)
            {
                /* Switches the step mode into logarithmic */
                m_instrument.SetSweepSpacing(WINDOW2, rsesciConstants.Log);

                /* This function defines the step size for the frequency of the scan range */
                m_instrument.SetScanStepSize(SCAN_RANGE, stepSize);
            }

            /* Defines the bandwidth for the receiver scan range */
            m_instrument.SetScanBandwidth(SCAN_RANGE, bandwidth);

            /* Defines the measurement time of the receiver subscan */
            m_instrument.SetScanMeasurementTime(SCAN_RANGE, measTime);

            /* Switches off the autoranging function */
            m_instrument.SetScanAutoRanging(SCAN_RANGE, false);

            /* Defines the RF attenuation */
            m_instrument.SetScanRFAttenuation(SCAN_RANGE, rfAttenuation);

            /* Switches off the preamplifier */
            m_instrument.SetScanPreamplifier(SCAN_RANGE, false);
        }

        /* Switch to single scan */
        public override void configureAcquisition()
        {
            m_instrument.SetSweepMode(WINDOW2, rsesciConstants.SingleSweep); ;
        }

        /* Switches block data transmission during the scan on */
        public override void configureFeedControl()
        {
            m_instrument.SetTraceFeedControl(SCAN_RANGE, rsesciConstants.TransAlways);
        }

        /* Selects the detector used in the scan measurement */
        public override void configureReceiverDetector(DetectorType detectorType)
        {
            if (detectorType == DetectorType.m_average) // Average
            {
                /* Sets the detector used in the bargraph measurement (optical reasons to counteract confusion) */
                m_instrument.SetReceiverDetectorWithCRMS(WINDOW2, false, false, false, true, false, false, false);

                /* Switches on the detector for the data acquisition */
                m_instrument.SetTraceDetector(WINDOW2, TRACE, rsesciConstants.TracedetAvg);
            }
            else if (detectorType == DetectorType.m_maxPeak)  // Max Peak
            {
                /* Sets the detector used in the bargraph measurement (optical reasons to counteract confusion) */
                m_instrument.SetReceiverDetectorWithCRMS(WINDOW2, true, false, false, false, false, false, false);

                /* Switches on the detector for the data acquisition */
                m_instrument.SetTraceDetector(WINDOW2, TRACE, rsesciConstants.TracedetPos);
            }
            else if (detectorType == DetectorType.m_minPeak)  // Min Peak
            {
                /* Sets the detector used in the bargraph measurement (optical reasons to counteract confusion) */
                m_instrument.SetReceiverDetectorWithCRMS(WINDOW2, false, true, false, false, false, false, false);

                /* Switches on the detector for the data acquisition */
                m_instrument.SetTraceDetector(WINDOW2, TRACE, rsesciConstants.TracedetNeg);
            }
            else if (detectorType == DetectorType.m_rms)  // RMS
            {
                /* Sets the detector used in the bargraph measurement (optical reasons to counteract confusion) */
                m_instrument.SetReceiverDetectorWithCRMS(WINDOW2, false, false, true, false, false, false, false);

                /* Switches on the detector for the data acquisition */
                m_instrument.SetTraceDetector(WINDOW2, TRACE, rsesciConstants.TracedetRms);
            }
            else if (detectorType == DetectorType.m_quasipeak)  // QPeak
            {
                m_instrument.SetReceiverDetectorWithCRMS(WINDOW2, false, false, false, false, true, false, false);

                /* Switches on the detector for the data acquisition */
                m_instrument.SetTraceDetector(WINDOW2, TRACE, rsesciConstants.TracedetQpeak);
            }
            else if (detectorType == DetectorType.m_cisprRMS)  // CRMS
            {
                m_instrument.SetReceiverDetectorWithCRMS(WINDOW2, false, false, false, false, false, false, true);

                /* Switches on the detector for the data acquisition */
                m_instrument.SetTraceDetector(WINDOW2, TRACE, rsesciConstants.TracedetCrms);
            }
            else if (detectorType == DetectorType.m_cisprAverage)  // CAverage
            {
                m_instrument.SetReceiverDetectorWithCRMS(WINDOW2, false, false, false, false, false, true, false);

                /* Switches on the detector for the data acquisition */
                m_instrument.SetTraceDetector(WINDOW2, TRACE, rsesciConstants.TracedetCavg);
            }
        }

        /* Switches the unit for power for the selected measurement window into dBuV */
        public override void configurePowerUnit()
        {
            m_instrument.SetPowerUnit(WINDOW2, rsesciConstants.UnitDbuv);
        }

        /* Runs the receiver measurement */
        public override void runReceiverMeasurement()
        {
            m_instrument.RunScan();
        }

        /* Contains the commands for the querying status reporting system */
        public override void getStatusRegister(int statusRegistersQuery, out int registerValue)
        {
            m_instrument.getStatusRegister(statusRegistersQuery, out registerValue);
        }

        /* Returns the result of the scan measurement */
        public override void readScanMeasurement(out int traceStatus, out int numOfResults, StringBuilder statusData, double[] trace1, double[] trace2, double[] trace3, double[] trace4, double[] trace5, double[] trace6)
        {
            m_instrument.ReadScanMeasurement(ARRAY_SIZE, out traceStatus, out numOfResults, statusData, trace1, trace2, trace3);
        }

        /* Configures the peak search */
        public override void configurePeakSearch(PeakSearchMethod peakSearchMethod, int numberOfPeaksSubranges)
        {
            /* Ends the current measurement */
            m_instrument.Abort();

            if (peakSearchMethod == PeakSearchMethod.m_peaks)
            {
                /* Switches the peak search method into peak */
                m_instrument.SetPeakSearchMethod(rsesciConstants.SearchPeak);

                /* Sets the number of peaks to be determined */
                m_instrument.SetPeakSearchSubranges(numberOfPeaksSubranges);
            }
            else if (peakSearchMethod == PeakSearchMethod.m_subranges)
            {
                /* Switches the peak search method into subranges */
                m_instrument.SetPeakSearchMethod(rsesciConstants.SearchSubr);

                /* Sets the number of subranges to be analyzed */
                m_instrument.SetPeakSearchSubranges(numberOfPeaksSubranges);
            }
        }

        /* Generates a peak list from the scan result */
        public override void searchPeaks()
        {
            m_instrument.PeakSearch();
        }

        /* Returns the results of the peak search */
        public override void readPeakList(double[] frequency, double[] level, double[] deltaLevel, int returnedValues)
        {
            m_instrument.ReadFinalMeasurement(TRACE, ARRAY_SIZE, out returnedValues, frequency, level, deltaLevel);
        }

        /* Reads an error code and a message from the instrument's error queue */
        public override void error_query(out int errorCode, StringBuilder errorMessage)
        {
            m_instrument.error_query(out errorCode, errorMessage);
        }
    }
}