using System;
using System.Text;
using InstrumentDrivers;

namespace Scan_Measurement_Example
{
    public class rsspecan_Instrument : Instrument
    {
        private rsspecan m_instrument;

        public rsspecan_Instrument(string m_resourceDescriptor, bool m_idQery, bool m_resetDevice)
        {
            /* Initialize Driver */
            m_instrument = new rsspecan(m_resourceDescriptor, m_idQery, m_resetDevice);
        }

        ~rsspecan_Instrument()
        {

        }

        /* Resets the instrument */
        public override void resetInstrument(bool resetDevice)
        {
            if (resetDevice == true)
                m_instrument.reset();
        }

        /* Switches the instrument into receiver mode */
        public override void receiverMode()
        {
            m_instrument.ReceiverMode();
        }

        /* Switches on/off the update of all display elements during remote control */
        public override void configureDisplayUpdate(bool displayEnable)
        {
            m_instrument.ConfigureDisplayUpdate(displayEnable);
        }

        /* Configures the scan table */
        public override void configureScanTable(InstrumentType instrumentType, StepMode stepMode, double startFrequency, double stopFrequency, double stepSize, double bandwidth, double measTime, double rfAttenuation)
        {
            /* Configures the number of scan ranges */
            m_instrument.ConfigureScanRanges(SCAN_RANGE);

            /* Defines the start and stop frequency of the overall scan in receiver mode */
            m_instrument.ConfigureFrequencyStartStop(WINDOW2, startFrequency, stopFrequency);

            if (stepMode == StepMode.m_linear)
            {
                /* Switches the step mode into linear */
                m_instrument.ConfigureReceiverStepMode(rsspecanConstants.SweepSpacLin);

                /* Defines the start and stop frequency and the step size of the scan range */
                m_instrument.ConfigureScanTableStartStop(SCAN_RANGE, startFrequency, stopFrequency, stepSize);
            }
            else if (stepMode == StepMode.m_logarithmic)
            {
                /* Switches the step mode into logarithmic */
                m_instrument.ConfigureReceiverStepMode(rsspecanConstants.SweepSpacLog);

                /* Defines the start and stop frequency and the step size of the scan range */
                m_instrument.ConfigureScanTableStartStop(SCAN_RANGE, startFrequency, stopFrequency, stepSize);
            }

            /* Defines the bandwidth for the receiver scan range */
            m_instrument.ConfigureScanTableResBandwidth(SCAN_RANGE, bandwidth);

            /* Defines the measurement time of the receiver subscan */
            m_instrument.ConfigureScanTableMeasTime(SCAN_RANGE, measTime);

            /* Switches off the autoranging function */
            m_instrument.ConfigureReceiverAutoRange(WINDOW2, false);

            /* Defines the RF attenuation */
            m_instrument.ConfigureScanTableAttenuation(SCAN_RANGE, false, rfAttenuation);

            /* Switches off the preamplifier */
            m_instrument.ConfigureReceiverPreamplifierState(WINDOW2, false);
        }

        /* Switch to single scan */
        public override void configureAcquisition()
        {
            m_instrument.ConfigureAcquisition(WINDOW2, SWEEP_MODE_CONTINUOUS, NUMBER_OF_SWEEPS);
        }

        /* Switches block data transmission during the scan on */
        public override void configureFeedControl()
        {
            m_instrument.ConfigureReceiverTraceFeedControl(rsspecanConstants.TracFeedAlways);
        }

        /* Selects the detector used in the scan measurement */
        public override void configureReceiverDetector(DetectorType detectorType)
        {
            if (detectorType == DetectorType.m_average) // Average
            {
                /* Sets the detector used in the bargraph measurement (optical reasons to counteract confusion) */
                m_instrument.ConfigureReceiverDetector(WINDOW1, TRACE, true, false, false, false, false, false, false);

                /* Switches on the detector for the data acquisition */
                m_instrument.SetInt32(rsspecanProperties.DetectorType, "Win0,TR1", rsspecanConstants.DetectorTypeAverage);
            }
            else if (detectorType == DetectorType.m_maxPeak)  // Max Peak
            {
                /* Sets the detector used in the bargraph measurement (optical reasons to counteract confusion) */
                m_instrument.ConfigureReceiverDetector(WINDOW1, TRACE, false, true, false, false, false, false, false);

                /* Switches on the detector for the data acquisition */
                m_instrument.SetInt32(rsspecanProperties.DetectorType, "Win0,TR1", rsspecanConstants.DetectorTypeMaxPeak);
            }
            else if (detectorType == DetectorType.m_minPeak)  // Min Peak
            {
                /* Sets the detector used in the bargraph measurement (optical reasons to counteract confusion) */
                m_instrument.ConfigureReceiverDetector(WINDOW1, TRACE, false, false, true, false, false, false, false);

                /* Switches on the detector for the data acquisition */
                m_instrument.SetInt32(rsspecanProperties.DetectorType, "Win0,TR1", rsspecanConstants.DetectorTypeMinPeak);
            }
            else if (detectorType == DetectorType.m_rms)  // RMS
            {
                /* Sets the detector used in the bargraph measurement (optical reasons to counteract confusion) */
                m_instrument.ConfigureReceiverDetector(WINDOW1, TRACE, false, false, false, true, false, false, false);

                /* Switches on the detector for the data acquisition */
                m_instrument.SetInt32(rsspecanProperties.DetectorType, "Win0,TR1", rsspecanConstants.DetectorTypeRms);
            }
            else if (detectorType == DetectorType.m_quasipeak)  // QPeak
            {
                /* Sets the detector used in the bargraph measurement (optical reasons to counteract confusion) */
                m_instrument.ConfigureReceiverDetector(WINDOW1, TRACE, false, false, false, false, true, false, false);

                /* Switches on the detector for the data acquisition */
                m_instrument.SetInt32(rsspecanProperties.DetectorType, "Win0,TR1", rsspecanConstants.DetectorTypeQpk);
            }
            else if (detectorType == DetectorType.m_cisprRMS)  // CRMS
            {
                /* Sets the detector used in the bargraph measurement (optical reasons to counteract confusion) */
                m_instrument.ConfigureReceiverDetector(WINDOW1, TRACE, false, false, false, false, false, true, false);

                /* Switches on the detector for the data acquisition */
                m_instrument.SetInt32(rsspecanProperties.DetectorType, "Win0,TR1", rsspecanConstants.DetectorTypeCrms);
            }
            else if (detectorType == DetectorType.m_cisprAverage)  // CAverage
            {
                /* Sets the detector used in the bargraph measurement (optical reasons to counteract confusion) */
                m_instrument.ConfigureReceiverDetector(WINDOW1, TRACE, false, false, false, false, false, false, true);

                /* Switches on the detector for the data acquisition */
                m_instrument.SetInt32(rsspecanProperties.DetectorType, "Win0,TR1", rsspecanConstants.DetectorTypeCav);
            }
        }

        /* Switches the unit for power for the selected measurement window into dBuV */
        public override void configurePowerUnit()
        {
            /* Configures the power unti for the bargraph window */
            m_instrument.ConfigureUnitPower(WINDOW1, rsspecanConstants.UnitDbuv);

            /* Configures the power unti for the spectrum window */
            m_instrument.ConfigureUnitPower(WINDOW2, rsspecanConstants.UnitDbuv);
        }

        /* Runs the receiver measurement */
        public override void runReceiverMeasurement()
        {
            m_instrument.RunReceiverMeasurement();
        }

        /* Contains the commands for the querying status reporting system */
        public override void getStatusRegister(int statusRegistersQuery, out int registerValue)
        {
            m_instrument.getStatusRegister(statusRegistersQuery, out registerValue);
        }

        /* Returns the result of the scan measurement */
        public override void readScanMeasurement(out int traceStatus, out int numOfResults, StringBuilder statusData, double[] trace1, double[] trace2, double[] trace3, double[] trace4, double[] trace5, double[] trace6)
        {
            m_instrument.ReadScanMeasurement(ARRAY_SIZE, out traceStatus, out numOfResults, statusData, trace1, trace2, trace3, trace4, trace5, trace6);
        }

        /* Configures the peak search */
        public override void configurePeakSearch(PeakSearchMethod peakSearchMethod, int numberOfPeaksSubranges)
        {
            if (peakSearchMethod == PeakSearchMethod.m_peaks)
            {
                /* Switches the peak search method into peak */
                m_instrument.ConfigureReceiverPeakSearchMethod(rsspecanConstants.RecPsearchMethodPeak);

                /* Sets the number of peaks to be determined */
                m_instrument.ConfigureReceiverNumberOfPeaksSubranges(numberOfPeaksSubranges);
            }
            else if (peakSearchMethod == PeakSearchMethod.m_subranges)
            {
                /* Switches the peak search method into subranges */
                m_instrument.ConfigureReceiverPeakSearchMethod(rsspecanConstants.RecPsearchMethodSubr);

                /* Sets the number of subranges to be analyzed */
                m_instrument.ConfigureReceiverNumberOfPeaksSubranges(numberOfPeaksSubranges);
            }
        }

        /* Generates a peak list from the scan result */
        public override void searchPeaks()
        {
            m_instrument.SEMSearchPeak();
        }

        /* Returns the results of the peak search */
        public override void readPeakList(double[] frequency, double[] level, double[] deltaLevel, int returnedValues)
        {
            m_instrument.ReadSEMPeakList(WINDOW2, FINAL_MEASUREMENT, 1024, frequency, level, deltaLevel, out returnedValues);
        }

        /* Reads an error code and a message from the instrument's error queue */
        public override void error_query(out int errorCode, StringBuilder errorMessage)
        {
            m_instrument.error_query(out errorCode, errorMessage);
        }
    }
}