namespace InstrumentDrivers
{
    using System;
    using System.Runtime.InteropServices;


    public class rsspecan : object, System.IDisposable
    {

        private System.IntPtr _handle;

        private bool _disposed = true;

        ~rsspecan() { Dispose(false); }


        /// <summary>
        /// This function performs the following initialization actions:
        /// 
        /// - Creates a new instrument driver session.
        /// 
        /// - Opens a session to the specified device using the interface and address you specify for the Resource Name parameter.
        /// 
        /// - If the ID Query parameter is set to VI_TRUE, this function queries the instrument ID and checks that it is valid for this instrument driver.
        /// 
        /// - If the Reset parameter is set to VI_TRUE, this function resets the instrument to a known state.
        /// 
        /// - Sends initialization commands to set the instrument to the state necessary for the operation of the instrument driver.
        /// 
        /// - Returns a ViSession handle that you use to identify the instrument in all subsequent instrument driver function calls.
        /// 
        /// Note:
        /// 
        /// This function creates a new session each time you invoke it. Although you can open more than one session for the same resource, it is best not to do so. You can use the same session in multiple program threads. You can use the rsspecan_LockSession and rsspecan_UnlockSession functions to protect sections of code that require exclusive access to the resource.
        /// 
        /// 
        /// Remote-control command(s):
        /// *IDN?
        /// </summary>
        /// <param name="Resource_Name">
        /// Pass the resource name of the device to initialize.
        /// 
        /// You can also pass the name of a virtual instrument or logical name that you configure with the VISA Configuration utility.  The virtual instrument identifies a specific device and specifies the initial settings for the session. A logical Name identifies a particular virtual instrument.
        /// 
        /// Refer to the following table below for the exact grammar to use for this parameter.  Optional fields are shown in square brackets ([]).
        /// 
        /// Syntax
        /// ------------------------------------------------------
        /// GPIB[board]::&lt;primary address&gt;[::secondary address]::INSTR
        /// VXI[board]::&lt;logical address&gt;::INSTR
        /// GPIB-VXI[board]::&lt;logical address&gt;::INSTR
        /// ASRL&lt;port&gt;::INSTR
        /// &lt;LogicalName&gt;
        /// &lt;DriverSession&gt;
        /// 
        /// If you do not specify a value for an optional field, the following values are used:
        /// 
        /// Optional Field - Value
        /// ------------------------------------------------------
        /// board - 0
        /// secondary address - none (31)
        /// 
        /// The following table contains example valid values for this parameter.
        /// 
        /// "Valid Value" - Description
        /// ------------------------------------------------------
        /// "GPIB::22::INSTR" - GPIB board 0, primary address 22 no
        ///                     secondary address
        /// "GPIB::22::5::INSTR" - GPIB board 0, primary address 22
        ///                        secondary address 5
        /// "GPIB1::22::5::INSTR" - GPIB board 1, primary address 22
        ///                         secondary address 5
        /// "VXI::64::INSTR" - VXI board 0, logical address 64
        /// "VXI1::64::INSTR" - VXI board 1, logical address 64
        /// "GPIB-VXI::64::INSTR" - GPIB-VXI board 0, logical address 64
        /// "GPIB-VXI1::64::INSTR" - GPIB-VXI board 1, logical address 64
        /// "ASRL2::INSTR" - COM port 2
        /// "SampleInstr" - Logical name "SampleInstr"
        /// "xyz432" - Driver Session "xyz432"
        /// 
        /// 
        /// Default Value:  "GPIB::14::INSTR"
        /// 
        /// </param>
        /// <param name="ID_Query">
        /// Specify whether you want the instrument driver to perform an ID Query.
        /// 
        /// Valid Range:
        /// VI_TRUE  (1) - Perform ID Query (Default Value)
        /// VI_FALSE (0) - Skip ID Query
        /// 
        /// When you set this parameter to VI_TRUE, the driver verifies that the instrument you initialize is a type that this driver supports.  
        /// 
        /// Circumstances can arise where it is undesirable to send an ID Query command string to the instrument.  When you set this parameter to VI_FALSE, the function initializes the instrument without performing an ID Query.
        /// </param>
        /// <param name="Reset_Device">
        /// Specify whether you want the to reset the instrument during the initialization procedure.
        /// 
        /// Valid Range:
        /// VI_TRUE  (1) - Reset Device (Default Value)
        /// VI_FALSE (0) - Don't Reset
        /// 
        /// 
        /// </param>
        /// <param name="Instrument_Handle">
        /// Returns a ViSession handle that you use to identify the instrument in all subsequent instrument driver function calls.
        /// 
        /// Notes:
        /// 
        /// (1) This function creates a new session each time you invoke it.  This is useful if you have multiple physical instances of the same type of instrument.  
        /// 
        /// (2) Avoid creating multiple concurrent sessions to the same physical instrument. Although you can create more than one session for the same resource, it is best not to do so. A better approach is to use the same session in multiple execution threads. You can use functions rsspecan_LockSession and rsspecan_UnlockSession to protect sections of code that require exclusive access to the resource.
        /// 
        /// </param>
        public rsspecan(string Resource_Name, bool ID_Query, bool Reset_Device)
        {
            int pInvokeResult = PInvoke.init(Resource_Name, System.Convert.ToUInt16(ID_Query), System.Convert.ToUInt16(Reset_Device), out this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            this._disposed = false;
        }

        /// <summary>
        /// This function performs the following initialization actions:
        /// 
        /// - Creates a new instrument driver session and optionally sets the initial state of the following session attributes:
        /// 
        ///     RSSPECAN_ATTR_RANGE_CHECK
        ///     RSSPECAN_ATTR_QUERY_INSTRUMENT_STATUS
        ///     RSSPECAN_ATTR_CACHE (currently not used)
        ///     RSSPECAN_ATTR_SIMULATE
        ///     RSSPECAN_ATTR_RECORD_COERCIONS (currently not used)
        /// 
        /// - Opens a session to the specified device using the interface and address you specify for the Resource Name parameter.
        /// 
        /// - If the ID Query parameter is set to VI_TRUE, this function queries the instrument ID and checks that it is valid for this instrument driver.
        /// 
        /// - If the Reset parameter is set to VI_TRUE, this function resets the instrument to a known state.
        /// 
        /// - Sends initialization commands to set the instrument to the state necessary for the operation of the instrument driver.
        /// 
        /// - Returns a ViSession handle that you use to identify the instrument in all subsequent instrument driver function calls.
        /// 
        /// Note(s):
        /// 
        /// (1) Simulation mode is not implemented.
        /// 
        /// 
        /// (2) This function creates a new session each time you invoke it. Although you can open more than one session for the same resource, it is best not to do so. You can use the same session in multiple program threads. You can use the rsspecan_LockSession and rsspecan_UnlockSession functions to protect sections of code that require exclusive access to the resource.
        /// 
        /// </summary>
        /// <param name="Resource_Name">
        /// Pass the resource name of the device to initialize.
        /// 
        /// You can also pass the name of a virtual instrument or logical name that you configure with the VISA Configuration utility.  The virtual instrument identifies a specific device and specifies the initial settings for the session. A logical Name identifies a particular virtual instrument.
        /// 
        /// Refer to the following table below for the exact grammar to use for this parameter.  Optional fields are shown in square brackets ([]).
        /// 
        /// Syntax
        /// ------------------------------------------------------
        /// GPIB[board]::&lt;primary address&gt;[::secondary address]::INSTR
        /// VXI[board]::&lt;logical address&gt;::INSTR
        /// GPIB-VXI[board]::&lt;logical address&gt;::INSTR
        /// ASRL&lt;port&gt;::INSTR
        /// &lt;LogicalName&gt;
        /// &lt;DriverSession&gt;
        /// 
        /// If you do not specify a value for an optional field, the following values are used:
        /// 
        /// Optional Field - Value
        /// ------------------------------------------------------
        /// board - 0
        /// secondary address - none (31)
        /// 
        /// The following table contains example valid values for this parameter.
        /// 
        /// "Valid Value" - Description
        /// ------------------------------------------------------
        /// "GPIB::22::INSTR" - GPIB board 0, primary address 22 no
        ///                     secondary address
        /// "GPIB::22::5::INSTR" - GPIB board 0, primary address 22
        ///                        secondary address 5
        /// "GPIB1::22::5::INSTR" - GPIB board 1, primary address 22
        ///                         secondary address 5
        /// "VXI::64::INSTR" - VXI board 0, logical address 64
        /// "VXI1::64::INSTR" - VXI board 1, logical address 64
        /// "GPIB-VXI::64::INSTR" - GPIB-VXI board 0, logical address 64
        /// "GPIB-VXI1::64::INSTR" - GPIB-VXI board 1, logical address 64
        /// "ASRL2::INSTR" - COM port 2
        /// "SampleInstr" - Logical name "SampleInstr"
        /// "xyz432" - Driver Session "xyz432"
        /// 
        /// 
        /// Default Value:  "GPIB::14::INSTR"
        /// 
        /// </param>
        /// <param name="ID_Query">
        /// Specify whether you want the instrument driver to perform an ID Query.
        /// 
        /// Valid Range:
        /// VI_TRUE  (1) - Perform ID Query (Default Value)
        /// VI_FALSE (0) - Skip ID Query
        /// 
        /// When you set this parameter to VI_TRUE, the driver verifies that the instrument you initialize is a type that this driver supports.  
        /// 
        /// Circumstances can arise where it is undesirable to send an ID Query command string to the instrument.  When you set this parameter to VI_FALSE, the function initializes the instrument without performing an ID Query.
        /// </param>
        /// <param name="Reset_Device">
        /// Specify whether you want the to reset the instrument during the initialization procedure.
        /// 
        /// Valid Range:
        /// VI_TRUE  (1) - Reset Device (Default Value)
        /// VI_FALSE (0) - Don't Reset
        /// 
        /// 
        /// </param>
        /// <param name="Option_String">
        /// You can use this control to set the initial value of certain attributes for the session.  The following table lists the attributes and the name you use in this parameter to identify the attribute.
        /// 
        /// Name              Attribute Defined Constant   
        /// --------------------------------------------
        /// RangeCheck        RSSPECAN_ATTR_RANGE_CHECK
        /// QueryInstrStatus  RSSPECAN_ATTR_QUERY_INSTRUMENT_STATUS   
        /// Cache             RSSPECAN_ATTR_CACHE   
        /// Simulate          RSSPECAN_ATTR_SIMULATE  
        /// RecordCoercions   RSSPECAN_ATTR_RECORD_COERCIONS
        /// 
        /// The format of this string is, "AttributeName=Value" where AttributeName is the name of the attribute and Value is the value to which the attribute will be set.  To set multiple attributes, separate their assignments with a comma.  
        /// 
        /// If you pass NULL or an empty string for this parameter, the session uses the default values for the attributes.   You can override the default values by assigning a value explicitly in a string you pass for this parameter.  You do not have to specify all of the attributes and may leave any of them out.  If you do not specify one of the attributes, its default value will be used.  
        /// 
        /// The default values for the attributes are shown below:
        /// 
        ///     Attribute Name     Default Value
        ///     ----------------   -------------
        ///     RangeCheck         VI_TRUE
        ///     QueryInstrStatus   VI_FALSE
        ///     Cache              VI_TRUE
        ///     Simulate           VI_FALSE
        ///     RecordCoercions    VI_FALSE
        ///     
        /// 
        /// The following are the valid values for ViBoolean attributes:
        /// 
        ///     True:     1, True, or VI_TRUE
        ///     False:    0, False, or VI_FALSE
        /// 
        /// 
        /// Default Value:
        ///        "Simulate=0,RangeCheck=1,QueryInstrStatus=1,Cache=1"
        /// 
        /// </param>
        /// <param name="Instrument_Handle">
        /// Returns a ViSession handle that you use to identify the instrument in all subsequent instrument driver function calls.
        /// 
        /// Notes:
        /// 
        /// (1) This function creates a new session each time you invoke it.  This is useful if you have multiple physical instances of the same type of instrument.  
        /// 
        /// (2) Avoid creating multiple concurrent sessions to the same physical instrument. Although you can create more than one RS session for the same resource, it is best not to do so. A better approach is to use the same RS session in multiple execution threads. You can use functions rsspecan_LockSession and rsspecan_UnlockSession to protect sections of code that require exclusive access to the resource.
        /// 
        /// 
        /// </param>
        public rsspecan(string Resource_Name, bool ID_Query, bool Reset_Device, string Option_String)
        {
            int pInvokeResult = PInvoke.InitWithOptions(Resource_Name, System.Convert.ToUInt16(ID_Query), System.Convert.ToUInt16(Reset_Device), Option_String, out this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            this._disposed = false;
        }

        /// <summary>
        /// This function selects the spectrum analyzer mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SAN_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument:SELect SAN
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SetSANMode()
        {
            int pInvokeResult = PInvoke.SetSANMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the frequency range of the spectrum analyzer using start frequency and stop frequency. If start frequency is equal to the stop frequency, then the spectrum analyzer is in time-domain mode.
        /// 
        /// Notes:
        /// 
        /// (1) In auto-coupled mode, resolution bandwidth (RBW), video bandwidth (VBW), and sweep time may be affected by this function.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FREQUENCY_START
        /// RSSPECAN_ATTR_FREQUENCY_STOP
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:STARt
        /// [SENSe&lt;1|2&gt;:]FREQuency:STOP
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Start_Frequency">
        /// The start frequency of the frequency sweep.
        /// 
        /// Units: Hertz
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_START
        /// </param>
        /// <param name="Stop_Frequency">
        /// The stop frequency of the frequency sweep.
        /// 
        /// Units: Hertz
        /// 
        /// Default Value: None
        /// 
        /// Attribute: 
        /// RSSPECAN_ATTR_FREQUENCY_STOP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFrequencyStartStop(int Window, double Start_Frequency, double Stop_Frequency)
        {
            int pInvokeResult = PInvoke.ConfigureFrequencyStartStop(this._handle, Window, Start_Frequency, Stop_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the center frequency 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FREQUENCY_CENTER
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Center_Frequency">
        /// The center frequency of the frequency sweep.
        /// 
        /// Units: Hertz
        /// 
        /// Default Value: None
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_CENTER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFrequencyCenter(int Window, double Center_Frequency)
        {
            int pInvokeResult = PInvoke.ConfigureFrequencyCenter(this._handle, Window, Center_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the frequency range of the spectrum analyzer using the center frequency and the frequency span.  If span corresponds to zero hertz, then the spectrum analyzer is in time-domain.  
        /// Notes:
        /// 
        /// (1) In auto-coupled mode, RBW, VBW and Sweep time may be affected by this function.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FREQUENCY_CENTER
        /// RSSPECAN_ATTR_FREQUENCY_SPAN
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer
        /// [SENSe&lt;1|2&gt;:]FREQuency:SPAN
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Center_Frequency">
        /// The center frequency of the frequency sweep.
        /// 
        /// Units: Hertz
        /// 
        /// Default Value: None
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_CENTER
        /// </param>
        /// <param name="Span">
        /// The frequency span of the frequency sweep.
        /// Units: Hertz
        /// 
        /// Default Value: None
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_SPAN
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFrequencyCenterSpan(int Window, double Center_Frequency, double Span)
        {
            int pInvokeResult = PInvoke.ConfigureFrequencyCenterSpan(this._handle, Window, Center_Frequency, Span);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the step width of the center frequency.
        /// 
        /// Notes:
        /// 
        /// (1) This function sets the frequency coupling to Off
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FREQUENCY_CENTER_LINK
        /// RSSPECAN_ATTR_FREQUENCY_CENTER_STEP
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer:STEP:LINK OFF
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer:STEP
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Step_Size">
        /// Sets the step width of the center frequency.
        /// 
        /// Units: Hertz
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_CENTER_STEP
        /// RSSPECAN_ATTR_FREQUENCY_CENTER_LINK
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFrequencyStepSize(int Window, double Step_Size)
        {
            int pInvokeResult = PInvoke.ConfigureFrequencyStepSize(this._handle, Window, Step_Size);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function couples the step size of the center frequency to the span (ON) or sets the value of the center frequency entered via SENSe&lt;1|2&gt;:]FREQuency:CENTer:STEP (OFF).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FREQUENCY_STEP_AUTO
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer:STEP:AUTO ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Step_Size_Auto">
        /// Couples the step size of the center frequency to the span (ON) or sets the value of the center frequency entered via SENSe&lt;1|2&gt;:]FREQuency:CENTer:STEP (OFF).
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_STEP_AUTO
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFrequencyStepSizeAuto(int Window, bool Step_Size_Auto)
        {
            int pInvokeResult = PInvoke.ConfigureFrequencyStepSizeAuto(this._handle, Window, System.Convert.ToUInt16(Step_Size_Auto));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function couples the step width of the center frequency to span (span &gt;0) or to the resolution bandwidth (span = 0) or cancels the couplings and sets the coupling factor.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FREQUENCY_CENTER_LINK
        /// RSSPECAN_ATTR_FREQUENCY_CENTER_LINK_FACTOR
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer:STEP:LINK OFF | SPAN | RBW 
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer:STEP:LINK:FACTor
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Coupling">
        /// Couples the step width of the center frequency to span (span &gt;0) or to the resolution bandwidth (span = 0) or cancels the couplings.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_CENT_FREQ_LINK_OFF  (0) - Off
        /// RSSPECAN_VAL_CENT_FREQ_LINK_SPAN (1) - Span
        /// RSSPECAN_VAL_CENT_FREQ_LINK_RBW  (2) - RBW
        /// RSSPECAN_VAL_CENT_FREQ_LINK_DIVT (3) - 10% of Span
        /// 
        /// Default Value: RSSPECAN_VAL_CENT_FREQ_LINK_SPAN (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_CENTER_LINK
        /// 
        /// Note(s):
        /// (1) Span is valid only for frequency domain (Span&gt;0)
        /// 
        /// (2) RBW is valid only for time domain (Span=0)
        /// 
        /// (3) 10% of Span - Coupling of the step size to 10% of the span. This option is available only for FSC and FSH
        /// </param>
        /// <param name="Coupling_Factor">
        /// Sets the coupling factor.
        /// 
        /// Units: %
        /// 
        /// Valid Values:
        /// 1 to 100 %
        /// 
        /// Default Value: 1 %
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_CENTER_LINK_FACTOR
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFrequencyCouplingFactor(int Window, int Coupling, int Coupling_Factor)
        {
            int pInvokeResult = PInvoke.ConfigureFrequencyCouplingFactor(this._handle, Window, Coupling, Coupling_Factor);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the frequency span in the analyzer mode to its maximum.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FREQUENCY_SPAN_FULL
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:SPAN:FULL
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFrequencySpanFull(int Window)
        {
            int pInvokeResult = PInvoke.ConfigureFrequencySpanFull(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the span to 0 Hz. The x-axis becomes the time axis with the grid lines corresponding to 1/10 of the current sweep time.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FREQUENCY_SPAN
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:SPAN
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFrequencyZeroSpan(int Window)
        {
            int pInvokeResult = PInvoke.ConfigureFrequencyZeroSpan(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches span mode AUTO on/off (Measure Mode: Channel Power or Occupied BW).
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AUTO_SPAN_MODE
        /// 
        /// Remote-control command(s):
        /// [SENSe:]FREQuency:SPAN:AUTO
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Span_Mode">
        /// This control switches span mode AUTO on/off (Measure Mode: Channel Power or Occupied BW).
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AUTO_SPAN_MODE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureAutoSpanMode(bool Span_Mode)
        {
            int pInvokeResult = PInvoke.ConfigureAutoSpanMode(this._handle, System.Convert.ToUInt16(Span_Mode));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the frequency offset attribute of the spectrum analyzer. This function affects the setting of the spectrum analyzer's absolute frequencies, such as start, stop, center, and marker. It does not affect values, which are the difference of frequencies, such as span and delta marker. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FREQUENCY_OFFSET
        /// 
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:OFFSet
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Frequency_Offset">
        /// Frequency offset.
        /// 
        /// Units: Hertz
        /// 
        /// Valid Values: -100.0e+9 to 100.0e+9
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_OFFSET
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFrequencyOffset(int Window, double Frequency_Offset)
        {
            int pInvokeResult = PInvoke.ConfigureFrequencyOffset(this._handle, Window, Frequency_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches between frequency domain (SWEep) and time domain (CW | FIXed) in the analyzer mode. For CW and FIXed, the frequency setting is via command FREQuency:CENTer. In the SWEep mode, the setting is via commands FREQuency:STARt, STOP, CENTer and SPAN. In EMI receiver mode, it switches between single frequency measurement (CW) and scan (SCAN).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FREQUENCY_MODE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:MODE CW | FIXed | SWEep | SCAN
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Frequency_Mode">
        /// Frequency mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FREQ_MODE_CW   (0) - CW
        /// RSSPECAN_VAL_FREQ_MODE_FIX  (1) - Fixed
        /// RSSPECAN_VAL_FREQ_MODE_SWE  (2) - Sweep
        /// RSSPECAN_VAL_FREQ_MODE_SCAN (3) - Scan
        /// 
        /// Default Value: RSSPECAN_VAL_FREQ_MODE_SWE  (2)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFrequencyMode(int Window, int Frequency_Mode)
        {
            int pInvokeResult = PInvoke.ConfigureFrequencyMode(this._handle, Window, Frequency_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the frequency mode. Select the Channel frequency mode only if you want to work with channel tables. The input of the center frequency in this case is not done as a
        /// frequency value, but as a channel number.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FREQUENCY_INPUT_MODE
        /// 
        /// Remote-control command(s):
        /// [SENSe:]FREQuency:INPut:MODE CHANnel | FREQuency
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Input_Mode">
        /// This control sets the frequency mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FREQ_INPUT_MODE_CHAN (0) - Channel
        /// RSSPECAN_VAL_FREQ_INPUT_MODE_FREQ (1) - Frequency
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_FREQ_INPUT_MODE_FREQ (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_INPUT_MODE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureFrequencyInputMode(int Input_Mode)
        {
            int pInvokeResult = PInvoke.ConfigureFrequencyInputMode(this._handle, Input_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the specified channel table for upcoming measurements.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CHANNEL_TABLE
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CHANnel:TABLe:SELect
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Channel_Table">
        /// This control sets the specified channel table for upcoming measurements.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CHANNEL_TABLE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureChannelTable(string Channel_Table)
        {
            int pInvokeResult = PInvoke.ConfigureChannelTable(this._handle, Channel_Table);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the sweep time values of the spectrum analyzer.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SWEEP_TIME_AUTO
        /// RSSPECAN_ATTR_SWEEP_TIME
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:TIME:AUTO ON | OFF
        /// [SENSe&lt;1|2&gt;:]SWEep:TIME
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Sweep_Time_Auto">
        /// This control sets automatic sweep time. The driver uses this value to set the RSSPECAN_ATTR_SWEEP_TIME_AUTO attribute.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SWEEP_TIME_AUTO
        /// </param>
        /// <param name="Sweep_Time">
        /// The length of time to complete one sweep. The driver uses this value to set the RSSPECAN_ATTR_SWEEP_TIME attribute.
        /// 
        /// Valid Values:
        /// 1.0e-6 s to 16.0e3 s
        /// 
        /// Default Value: 5.0e-3 s
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available if Sweep Time Auto is set to Off.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SWEEP_TIME
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSweepTime(int Window, bool Sweep_Time_Auto, double Sweep_Time)
        {
            int pInvokeResult = PInvoke.ConfigureSweepTime(this._handle, Window, System.Convert.ToUInt16(Sweep_Time_Auto), Sweep_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates auto coupling of selected coupling method.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RESOLUTION_BANDWIDTH_AUTO
        /// RSSPECAN_ATTR_VIDEO_BANDWIDTH_AUTO
        /// RSSPECAN_ATTR_SWEEP_TIME_AUTO
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth[:RESolution]:AUTO
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:VIDeo:AUTO ON | OFF
        /// [SENSe&lt;1|2&gt;:]SWEep:TIME:AUTO ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Sweep_Coupling">
        /// This control selects coupling type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_COUPLING_RBW        (0) - Resolution Bandwidth
        /// RSSPECAN_VAL_COUPLING_VBW        (1) - Video Bandwidth
        /// RSSPECAN_VAL_COUPLING_SWEEP_TIME (2) - Sweep Time
        /// 
        /// Default Value: RSSPECAN_VAL_COUPLING_RBW (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RESOLUTION_BANDWIDTH_AUTO
        /// RSSPECAN_ATTR_VIDEO_BANDWIDTH_AUTO
        /// RSSPECAN_ATTR_SWEEP_TIME_AUTO
        /// 
        /// Note(s):
        /// 
        /// (1) In  receiver  mode,  this  command  either  automatically couples the IF bandwidth of the R&amp;S ESL to the frequency range or cancels the coupling. Note  that  this  command  is  only  effective  in  conjunction with the quasipeak, the CISPR average or CISPR RMS detector.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSweepCouplingAuto(int Window, int Sweep_Coupling)
        {
            int pInvokeResult = PInvoke.ConfigureSweepCouplingAuto(this._handle, Window, Sweep_Coupling);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the coupling values of the spectrum analyzer.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RESOLUTION_BANDWIDTH
        /// RSSPECAN_ATTR_VIDEO_BANDWIDTH
        /// RSSPECAN_ATTR_SWEEP_TIME
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth[:RESolution]
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:VIDeo
        /// [SENSe&lt;1|2&gt;:]SWEep:TIME
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Sweep_Coupling">
        /// This control selects coupling type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_COUPLING_RBW        (0) - Resolution Bandwidth
        /// RSSPECAN_VAL_COUPLING_VBW        (1) - Video Bandwidth
        /// RSSPECAN_VAL_COUPLING_SWEEP_TIME (2) - Sweep Time
        /// 
        /// Default Value: RSSPECAN_VAL_COUPLING_RBW (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Video Bandwidth: Not available for R&amp;S FMU instrument. 
        /// </param>
        /// <param name="Coupling_Value">
        /// Resolution bandwidth:
        /// Specifies the measurement resolution bandwidth.
        /// 
        /// Video bandwidth:
        /// Specifies the video bandwidth of the post-detection filter. 
        /// 
        /// Sweep Time:
        /// The length of time to complete one sweep.
        /// 
        /// Valid Values:
        /// Resolution bandwidth: 10.0 Hz to 50.0E6 Hz
        /// Video bandwidth: 1.0 Hz to 10.0E6 Hz
        /// Sweep Time: 1.0e-6 s to 16.0E3 s
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RESOLUTION_BANDWIDTH
        /// RSSPECAN_ATTR_VIDEO_BANDWIDTH
        /// RSSPECAN_ATTR_SWEEP_TIME
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSweepCoupling(int Window, int Sweep_Coupling, double Coupling_Value)
        {
            int pInvokeResult = PInvoke.ConfigureSweepCoupling(this._handle, Window, Sweep_Coupling, Coupling_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures advanced parameters of coupling and sweeping.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RESOLUTION_BANDWIDTH_RATIO
        /// RSSPECAN_ATTR_VIDEO_BANDWIDTH_RATIO
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth[:RESolution]:RATio
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:VIDeo:RATio
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Ratio_Selection">
        /// This control selects bandwidth ratio to be set.
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_COUPLING_RATIO_RBW (0) - Resolution Bandwidth Ratio
        /// RSSPECAN_VAL_COUPLING_RATIO_VBW (1) - Video Bandwidth Ratio
        /// 
        /// Default Value: RSSPECAN_COUPLING_RATIO_RBW (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Video Bandwidth Ratio: Not available for R&amp;S FMU instrument.
        /// </param>
        /// <param name="Value">
        /// Video Bandwidth Ratio:
        /// Defines the ratio resolution bandwidth (Hz) / span (Hz). The ratio to be entered is reciprocal to the ratio span/RBW used in manual control.
        /// 
        /// Resolution Bandwidth Ratio:
        /// Defines the ratio video bandwidth (Hz) / resolution bandwidth (Hz). The ratio to be entered is reciprocal to the ratio RBW/VBW used in manual control.
        /// 
        /// Valid Values:
        /// Video Bandwidth Ratio: 0.01 to 1000.0
        /// Resolution Bandwidth Ratio: 0.0001 to 1.0
        /// 
        /// Default Value: 0.01
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VIDEO_BANDWIDTH_RATIO
        /// RSSPECAN_ATTR_RESOLUTION_BANDWIDTH_RATIO
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSweepCouplingAdvanced(int Window, int Ratio_Selection, double Value)
        {
            int pInvokeResult = PInvoke.ConfigureSweepCouplingAdvanced(this._handle, Window, Ratio_Selection, Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the bandwidth of the main PLL of the instrument synthesizer.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PLL_BANDWIDTH
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:PLL AUTO | HIGH | MEDium | LOW
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="PLL">
        /// Sets the bandwidth of the main PLL of the instrument synthesizer.
        /// 
        /// Valid Values:
        /// 
        /// RSSPECAN_VAL_PLL_AUTO (0) - Auto
        /// RSSPECAN_VAL_PLL_LOW  (1) - Narrow
        /// RSSPECAN_VAL_PLL_MED  (2) - Normal
        /// RSSPECAN_VAL_PLL_HIGH (3) - Wide
        /// 
        /// Default Value: RSSPECAN_VAL_PLL_AUTO (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PLL_BANDWIDTH
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMainPLLBwid(int Window, int PLL)
        {
            int pInvokeResult = PInvoke.ConfigureMainPLLBwid(this._handle, Window, PLL);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the filter type for the resolution bandwidth.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RESOLUTION_BANDWIDTH_FILTER_TYPE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth[:RESolution]:TYPE NORMal | FFT | CFILter | RRC | P5 | PULSe | NOISe | P5Digital
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Filter_Type">
        /// Sets the filter type for the resolution bandwidth.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FILT_TYPE_NORM  (0) - Normal
        /// RSSPECAN_VAL_FILT_TYPE_FFT   (1) - FFT
        /// RSSPECAN_VAL_FILT_TYPE_CFIL  (2) - Cosine Filter
        /// RSSPECAN_VAL_FILT_TYPE_RRC   (3) - Root Raised Cosine
        /// RSSPECAN_VAL_FILT_TYPE_P5    (4) - 5-pole
        /// RSSPECAN_VAL_FILT_TYPE_PULSE (5) - Pulse
        /// RSSPECAN_VAL_FILT_TYPE_NOISE (6) - Gaussian
        /// RSSPECAN_VAL_FILT_TYPE_P5D   (7) - 5-pole Digital
        /// 
        /// Default Value: RSSPECAN_VAL_FILT_TYPE_NORM (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RESOLUTION_BANDWIDTH_FILTER_TYPE
        /// 
        /// Note(s):
        /// 
        /// (1) The possible combinations of filter type and filter bandwidth are listed in Operating Manual in the table "List of available channel filters" in chapter 4, section "Setting Bandwidths and Sweep Time- Key BW".
        /// 
        /// (2) Value 5-pole and 5-pole Digital is not valid for R&amp;S FSL or R&amp;S ESL or R&amp;S FSV.
        /// 
        /// (3) Value Pulse is valid for R&amp;S FSL or R&amp;S ESL only.
        /// 
        /// (4) Value Gaussian is valid for Receiver only (R&amp;S ESL).
        /// 
        /// (5) Value 5-pole Digital is not valid for R&amp;S FSUP.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFilterType(int Window, int Filter_Type)
        {
            int pInvokeResult = PInvoke.ConfigureFilterType(this._handle, Window, Filter_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the position of the video filter in the signal path, provided that the resolution bandwidth is &lt;=100 kHz.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VIDEO_BANDWIDTH_TYPE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:VIDeo:TYPE LINear | LOGarithmic
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Video_Filter_Type">
        /// Set the position of the video filter in the signal path
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VBW_FILT_LIN   (0) - Linear
        /// RSSPECAN_VAL_VBW_FILT_LOG   (1) - Logarithmic
        /// 
        /// Default Value: RSSPECAN_VAL_VBW_FILT_LIN (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VIDEO_BANDWIDTH_TYPE
        /// 
        /// Note(s):
        /// 
        /// (1) Lin - the video filter is connected ahead of the logarithmic amplifier (default)
        /// 
        /// (2) Log - the video filter follows the logarithmic amplifier
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVBWMode(int Window, int Video_Filter_Type)
        {
            int pInvokeResult = PInvoke.ConfigureVBWMode(this._handle, Window, Video_Filter_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function allows to specify FFT filter.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FFT_FILTER_MODE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:FFT WIDE | AUTO | NARROW
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="FFT_Filter_Mode">
        /// Allows to specify FFT filter.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FFT_MODE_AUTO   (0) - Auto
        /// RSSPECAN_VAL_FFT_MODE_WIDE   (1) - Wide
        /// RSSPECAN_VAL_FFT_MODE_NARROW (2) - Narrow
        /// 
        /// Default Value: RSSPECAN_VAL_FFT_MODE_AUTO (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FFT_FILTER_MODE
        /// 
        /// Note(s):
        /// 
        /// (1) Auto - The firmware decides whether to use wide or narrow filters to achieve the best performance of the measurement.
        /// 
        /// (2) Wide - The FFT filters with the greater partial span (this is the span which is covered with one FFT analysis) are used always.
        /// 
        /// (3) Narrow - The FFT filters with the smaller partial span are used, this allows measurements near a carrier with reduced reference level, because of a more narrow analog prefilter.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFFTFilterMode(int Window, int FFT_Filter_Mode)
        {
            int pInvokeResult = PInvoke.ConfigureFFTFilterMode(this._handle, Window, FFT_Filter_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the average calculation for the selected trace in the selected measurement window.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AVG_STATE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]AVERage[:STATe&lt;1 to 3&gt;] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 1 to 4 for R&amp;S FSL or R&amp;S ESL
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// Switches on or off the average calculation for the selected trace in the selected measurement window.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AVG_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAveraging(int Window, int Trace, bool State)
        {
            int pInvokeResult = PInvoke.ConfigureAveraging(this._handle, Window, Trace, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the type of average function. If Video is selected, the logarithmic power is averaged and, if Linear is selected, the power values are averaged before they are converted to logarithmic values.
        /// The type of average calculation is equally set for all traces in one measurement window.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AVG_TYPE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]AVERage:TYPE VIDeo | LINear | POWer
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Averaging_Type">
        /// Selects the type of average function. If Video is selected, the logarithmic power is averaged and, if Linear is selected, the power values are averaged before they are converted to logarithmic values.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_AVG_LIN (0) - Linear
        /// RSSPECAN_VAL_AVG_VID (1) - Video
        /// RSSPECAN_VAL_AVG_POW (2) - Power
        /// 
        /// Default Value: RSSPECAN_VAL_AVG_LIN (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AVG_TYPE
        /// 
        /// Note(s):
        /// 
        /// (1) Power - Only available for R&amp;S FSV and R&amp;S FSL.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAveragingType(int Window, int Averaging_Type)
        {
            int pInvokeResult = PInvoke.ConfigureAveragingType(this._handle, Window, Averaging_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the averaging count. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AVG_COUNT
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]AVERage:COUNt
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Count">
        /// This control defines the number of measurements which contribute to the average value.
        /// 
        /// Valid Values: 
        /// 0 to 32767
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AVG_COUNT
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAveragingCount(int Window, int Count)
        {
            int pInvokeResult = PInvoke.ConfigureAveragingCount(this._handle, Window, Count);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the acquisition attributes of the spectrum analyzer. Single sweep, continuous sweep and sweep count can be set.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SWEEP_MODE_CONTINUOUS
        /// RSSPECAN_ATTR_NUMBER_OF_SWEEPS
        /// 
        /// Remote-control command(s):
        /// INITiate&lt;1|2&gt;:CONTinuous ON | OFF
        /// [SENSe&lt;1|2&gt;:]SWEep:COUNt
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Sweep_Mode_Continuous">
        /// Turns on/off continuous sweeping.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SWEEP_MODE_CONTINUOUS
        /// </param>
        /// <param name="Number_Of_Sweeps">
        /// The number of sweeps to take.
        /// 
        /// Valid Values:
        /// 0 to 32767
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NUMBER_OF_SWEEPS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAcquisition(int Window, bool Sweep_Mode_Continuous, int Number_Of_Sweeps)
        {
            int pInvokeResult = PInvoke.ConfigureAcquisition(this._handle, Window, System.Convert.ToUInt16(Sweep_Mode_Continuous), Number_Of_Sweeps);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the vertical scale of analyzer.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VERTICAL_SCALE
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y:SPACing
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Vertical_Scale">
        /// Pass the vertical scale that you want the spectrum analyzer to use.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TRACE_SPACE_LIN (0) - Linear
        /// RSSPECAN_VAL_TRACE_SPACE_LOG (1) - Logarithmic
        /// RSSPECAN_VAL_TRACE_SPACE_PCT (2) - Percent
        /// RSSPECAN_VAL_TRACE_SPACE_LDB (3) - LDB
        /// 
        /// Default Value: RSSPECAN_VAL_TRACE_SPACE_LOG (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VERTICAL_SCALE
        /// 
        /// Note:
        /// 
        /// (1) Value RSSPECAN_VAL_TRACE_SPACE_PCT is valid for R&amp;S FSL only
        /// 
        /// (2) Value RSSPECAN_VAL_TRACE_SPACE_LDB is valid for R&amp;S FSL or R&amp;S FSV or R&amp;S FSUP or R&amp;S FSMR only.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVerticalScale(int Window, int Vertical_Scale)
        {
            int pInvokeResult = PInvoke.ConfigureVerticalScale(this._handle, Window, Vertical_Scale);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function toggles between linear and logarithmic display of the x-axis.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_HORIZONTAL_SCALE
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACE&lt;1 to 3&gt;:X:SPACing LINear | LOGarithmic
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Horizontal_Scale">
        /// Toggles between linear and logarithmic display of the x-axis. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIN  (0) - Linear
        /// RSSPECAN_VAL_LOG  (1) - Logarithmic
        /// 
        /// Default Value: RSSPECAN_VAL_LOG (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_HORIZONTAL_SCALE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureHorizontalScale(int Window, int Horizontal_Scale)
        {
            int pInvokeResult = PInvoke.ConfigureHorizontalScale(this._handle, Window, Horizontal_Scale);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the number of measurement points for one sweep run. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SWEEP_POINTS
        /// 
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:POINts
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Sweep_Points">
        /// Sets defines the number of measurement points for one sweep run. 
        /// 
        /// Valid Values: 101 to 32001
        /// 
        /// Default Value: 625
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SWEEP_POINTS
        /// 
        /// Note(s):
        /// 
        /// (1) Valid values differs for each instrument. Please check operating manual for valid values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSweepPoints(int Window, int Sweep_Points)
        {
            int pInvokeResult = PInvoke.ConfigureSweepPoints(this._handle, Window, Sweep_Points);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the number of measurement points for one sweep run. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SWEEP_POINTS
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:POINts?
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// 
        /// Note(s):
        /// (1) This control is not valid for R&amp;S FSH4, FSH8 instruments.
        /// </param>
        /// <param name="Sweep_Points">
        /// Returns the number of measurement points for one sweep run. The driver uses this value to get the RSSPECAN_ATTR_SWEEP_POINTS attribute.
        /// 
        /// Valid Values: 101 to 32001
        /// 
        /// Note(s):
        /// 
        /// (1) Valid values differs for each instrument. Please check operating manual for valid values.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SWEEP_POINTS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QuerySweepPoints(int Window, out int Sweep_Points)
        {
            int pInvokeResult = PInvoke.QuerySweepPoints(this._handle, Window, out Sweep_Points);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the sweep type.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SWEEP_TYPE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:TYPE SWEep | AUTO | FFT
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Sweep_Type">
        /// Sets the sweep type.
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_SWEEP_TYPE_AUTO  (0) - Auto
        /// RSSPECAN_VAL_SWEEP_TYPE_SWEEP (1) - Sweep
        /// RSSPECAN_VAL_SWEEP_TYPE_FFT   (2) - FFT
        /// 
        /// Default Value: RSSPECAN_VAL_SWEEP_TYPE_AUTO  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SWEEP_TYPE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSweepType(int Sweep_Type)
        {
            int pInvokeResult = PInvoke.ConfigureSweepType(this._handle, Sweep_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the input attenuation.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_ATTENUATION_AUTO
        /// RSSPECAN_ATTR_ATTENUATION
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:ATTenuation:AUTO ON | OFF
        /// INPut&lt;1|2&gt;:ATTenuation
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Attenuation_Auto">
        /// Specifies whether the spectrum analyzer is enabled for auto attenuation.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ATTENUATION_AUTO
        /// </param>
        /// <param name="Attenuation">
        /// Specifies the input attenuation (in positive dB). This means that if 10dB is selected, the result is a reduction in the signal level of 10 dB.
        /// 
        /// Valid Values:
        /// 0, 5, 10, 15, 20, 25 ... 75 dB
        /// 
        /// Default Value: 10.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ATTENUATION
        /// 
        /// Notes:
        /// 
        /// (1) If auto attenuation is turned on, this value is ignored.
        /// 
        /// (2) Range depends on instrument.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAttenuation(int Window, bool Attenuation_Auto, double Attenuation)
        {
            int pInvokeResult = PInvoke.ConfigureAttenuation(this._handle, Window, System.Convert.ToUInt16(Attenuation_Auto), Attenuation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the reference level
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_REFERENCE_LEVEL
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1?3&gt;:Y[:SCALe]:RLEVel
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Reference_Level">
        /// The calibrated vertical position of the captured data used as a reference for amplitude measurements. This is typically set to a value slightly higher than the highest expected signal level. 
        /// 
        /// Valid Values:
        /// not checked, depends on units set by user with RSSPECAN_ATTR_AMPLITUDE_UNITS attribute.
        /// 
        /// Default Value: -20.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_LEVEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReferenceLevel(int Window, double Reference_Level)
        {
            int pInvokeResult = PInvoke.ConfigureReferenceLevel(this._handle, Window, Reference_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the offset value to the reference level.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:RLEVel:OFFSet
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Reference_Level_Offset">
        /// Specifies the offset value to the reference level. The driver uses this value to set the RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET attribute.
        /// 
        /// Units: dB
        /// 
        /// Valid Values: -200.0 to 200.0
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReferenceLevelOffset(int Window, double Reference_Level_Offset)
        {
            int pInvokeResult = PInvoke.ConfigureReferenceLevelOffset(this._handle, Window, Reference_Level_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the vertical attributes of the spectrum analyzer.  This corresponds to attributes like amplitude units, input attenuation, input impedance, reference level, and reference level offset.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AMPLITUDE_UNITS
        /// RSSPECAN_ATTR_REFERENCE_LEVEL
        /// RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET
        /// RSSPECAN_ATTR_INPUT_IMPEDANCE
        /// RSSPECAN_ATTR_ATTENUATION_AUTO
        /// RSSPECAN_ATTR_ATTENUATION
        /// 
        /// Remote-control command(s):
        /// UNIT&lt;1|2&gt;:POWer DBM | DBPW | WATT | DBUV | DBMV | VOLT | DBUA | AMPere | DB | PCT | UNITLESS | DBPT | DBUV_MHZ | DBMV_MHZ | DBUA_MHZ | DBUV_M | DBUA_M | DBUV_MMHZ | DBUA_MMHZ
        /// INPut&lt;1|2&gt;:IMPedance
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:RLEVel
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:RLEVel:OFFSet
        /// INPut&lt;1|2&gt;:ATTenuation:AUTO ON | OFF
        /// INPut&lt;1|2&gt;:ATTenuation
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Amplitude_Units">
        /// Specifies the amplitude units for input, output and display. Pass the units you want the spectrum analyzer to use.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_DBM       (0) - dBm 
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_DBMV      (1) - dBmV
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_DBUV      (2) - dBuV
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_VOLT      (3) - Volt
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_WATT      (4) - Watt
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_DBPW      (5) - dBpW
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_DBUA      (6) - dBuA
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_A         (7) - Ampere
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_DB        (8) - dB
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_PCT       (9) - %
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_UNIT      (10) - Unitless
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_DBPT      (11) - dBpT
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_DBUV_MHZ  (12) - dBuV/MHz
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_DBMV_MHZ  (13) - dBmV/MHz
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_DBUA_MHZ  (14) - dBuA/MHz
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_DBUV_M    (15) - dBuV/M
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_DBUA_M    (16) - dBuA/M
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_DBUV_MMHZ (17) - dBuV/MMHz
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_DBUA_MMHZ (18) - dBuA/MMHz
        /// 
        /// Default Value: RSSPECAN_VAL_AMPLITUDE_UNITS_DBM (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AMPLITUDE_UNITS
        /// 
        /// Note(s):
        /// 
        /// (1) dBm: Sets the spectrum analyzer to measure in decibels relative to 1 milliwatt.
        /// 
        /// (2) dBmV: Sets the spectrum analyzer to measure in decibels relative to 1 millivolt.
        /// 
        /// (3) dBuV: Sets the spectrum analyzer to measure in decibels relative to 1 microvolt.
        /// 
        /// (4) Volt: Sets the spectrum analyzer to measure in volts.
        /// 
        /// (5) Watt: Sets the spectrum analyzer to measure in watts.
        /// 
        /// (6) dBpW: Sets the spectrum analyzer to measure in pico watts. Not available for R&amp;S FSH4/8 instrument.
        /// 
        /// (7) dBuA: Sets the spectrum analyzer to measure in micro ampers. Not available for R&amp;S FSH4/8 instrument.
        /// 
        /// (8) Ampere: Sets the spectrum analyzer to measure in ampers. Not available for R&amp;S FSH4/8 instrument.
        /// 
        /// (9) These units are available only for R&amp;S ESL and R&amp;S FSUP instruments:
        /// DB, PCT, UNITLESS, DBPT, DBUV_MHZ, DBMV_MHZ, DBUA_MHZ, DBUV_M, DBUA_M, DBUV_MMHZ, DBUA_MMHZ
        /// </param>
        /// <param name="Input_Impedance">
        /// Specifies the input impedance.
        /// 
        /// Valid Values:
        /// 50, 75 Ohms
        /// 
        /// Default Value: 50 Ohms
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_INPUT_IMPEDANCE
        /// 
        /// Note(s): 
        /// 
        /// (1) Not available for R&amp;S FMU instrument.
        /// </param>
        /// <param name="Reference_Level">
        /// The calibrated vertical position of the captured data used as a reference for amplitude measurements. This is typically set to a value slightly higher than the highest expected signal level. 
        /// 
        /// Valid Values:
        /// not checked, depends on units set by user with RSSPECAN_ATTR_AMPLITUDE_UNITS attribute.
        /// 
        /// Default Value: -20.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_LEVEL
        /// </param>
        /// <param name="Reference_Level_Offset">
        /// Specifies the offset value to the reference level.
        /// 
        /// Units: dB
        /// 
        /// Valid Values: -200.0 to 200.0
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET
        /// </param>
        /// <param name="Attenuation_Auto">
        /// Specifies whether the spectrum analyzer is enabled for auto attenuation.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ATTENUATION_AUTO
        /// 
        /// Note(s): 
        /// 
        /// (1) Not available for R&amp;S FMU instrument.
        /// </param>
        /// <param name="Attenuation">
        /// Specifies the input attenuation (in positive dB). This means that if 10dB is selected, the result is a reduction in the signal level of 10 dB.
        /// 
        /// Valid Values:
        /// 0, 1, 2, 5, 10, 20 dB
        /// 
        /// Default Value: 10.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ATTENUATION
        /// 
        /// Notes:
        /// 
        /// (1) If auto attenuation is turned on, this value is ignored.
        /// 
        /// (2) Not available for R&amp;S FMU instrument.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLevel(int Window, int Amplitude_Units, double Input_Impedance, double Reference_Level, double Reference_Level_Offset, bool Attenuation_Auto, double Attenuation)
        {
            int pInvokeResult = PInvoke.ConfigureLevel(this._handle, Window, Amplitude_Units, Input_Impedance, Reference_Level, Reference_Level_Offset, System.Convert.ToUInt16(Attenuation_Auto), Attenuation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function swicthes the behaviour for the attenuator.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_ATTENUATION_MODE
        /// RSSPECAN_ATTR_ATTENUATION_MODE_PORTABLE
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:ATTenuation:AUTO:MODE NORMal | LDIStortion
        /// INPut:ATTenuation:MODE LNOise | LDIStortion
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Mode">
        /// This control switches the behaviour of the attenuator.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ATT_MODE_NORM (0) - Normal
        /// RSSPECAN_VAL_ATT_MODE_LDIS (1) - Low Distortion
        /// RSSPECAN_VAL_ATT_MODE_LNO  (2) - Low Noise
        /// 
        /// Default Value: RSSPECAN_VAL_ATT_MODE_NORM (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ATTENUATION_MODE
        /// 
        /// Note(s):
        /// 
        /// (1) Normal - mechanical attenuator is switch as early as possible ("Low Noise") 
        /// 
        /// (2) Low distortion - means the mechanical attenuator is switched as late as possible for Low Distortion.
        /// 
        /// (3) Low Noise - for FSH Instrument
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAttenuationMode(int Window, int Mode)
        {
            int pInvokeResult = PInvoke.ConfigureAttenuationMode(this._handle, Window, Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// In operating mode SPECTRUM this function selects the parameter coupling between the two measurement windows screen A and B.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for instruments supporting two measurement windows.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PARAMETER_COUPLING
        /// 
        /// Remote-control command(s):
        /// INSTrument:COUPle NONE | RLEVel | CF_B | CF_A
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Parameter_Coupling">
        /// Selects the parameter coupling between the two measurement windows screen A and B.
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_COUPLING_NONE (0) - None
        /// RSSPECAN_VAL_COUPLING_RLEV (1) - Reference Level
        /// RSSPECAN_VAL_COUPLING_CFB  (2) - Center Frequency B
        /// RSSPECAN_VAL_COUPLING_CFA  (3) - Center Frequency A
        /// 
        /// Default Value: RSSPECAN_VAL_COUPLING_NONE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PARAMETER_COUPLING
        /// 
        /// Note(s):
        /// 
        /// (1) None - No coupling. The two measurement windows are operated like two independent virtual devices.
        /// 
        /// (2) Reference Level - The reference levels of the two measurement windows are coupled.
        /// 
        /// (3) Center Frequency B - The center frequency of screen B is coupled to the frequency of marker 1 in screen A.
        /// 
        /// (4) Center Frequency A - The center frequency of screen A is coupled to the frequency of marker 1 in screen B.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureParameterCoupling(int Parameter_Coupling)
        {
            int pInvokeResult = PInvoke.ConfigureParameterCoupling(this._handle, Parameter_Coupling);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines wether the reference value for the Y axis is coupled to the reference level (default) or not.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_REF_VALUE_AUTO
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...4&gt;:Y[:SCALe]:RVALue:AUTO ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Coupled_Reference_Level">
        /// This control defines wether the reference value for the Y axis is coupled to the reference level (default) or not.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_REF_VALUE_AUTO
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReferenceLevelCoupling(int Window, bool Coupled_Reference_Level)
        {
            int pInvokeResult = PInvoke.ConfigureReferenceLevelCoupling(this._handle, Window, System.Convert.ToUInt16(Coupled_Reference_Level));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the source of the IF output between the demodulated signal (2 possible values) and the IF signal.
        /// 
        /// Note(s): 
        /// 
        /// (1) The AF output available at the frontpanel can only be used if the IF output source is set to video.
        /// 
        /// (2) This functions is only available for R&amp;S FSL or R&amp;S ESL or R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_IF_OUTPUT_SOURCE
        /// 
        /// Remote-control command(s):
        /// OUTPut&lt;1|2&gt;:IF[:SOURce] IF | VIDeo | HVIDeo
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Output_Source">
        /// This control switches the source of the IF output between the demodulated signal (2 possible values) and the IF signal.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_IF_OUTPUT_SOURCE_IF   (0) - IF
        /// RSSPECAN_VAL_IF_OUTPUT_SOURCE_VID  (1) - Video
        /// RSSPECAN_VAL_IF_OUTPUT_SOURCE_HVID (2) - Hight Video
        /// 
        /// Default Value: RSSPECAN_VAL_IF_OUTPUT_SOURCE_IF (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_IF_OUTPUT_SOURCE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureIFOutputSource(int Window, int Output_Source)
        {
            int pInvokeResult = PInvoke.ConfigureIFOutputSource(this._handle, Window, Output_Source);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the unit for power in the selected measurement window.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_UNIT_POWER
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:UNIT:POWer DBM | V | A | W | DB | PCT | UNITLESS | DBPW | WATT | DBUV | DBMV | VOLT | DBUA | AMPere | DBPT | DBUV_MHZ | DBMV_MHZ | DBUA_MHZ | DBUV_M | DBUA_M | DBUV_MMHZ | DBUA_MMHZ
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Unit_Power">
        /// Sets the unit for power in the selected measurement window.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DBM       (0) - dBm
        /// RSSPECAN_VAL_UNIT_DBPW      (1) - dBpW
        /// RSSPECAN_VAL_UNIT_WATT      (2) - W
        /// RSSPECAN_VAL_UNIT_DBUV      (3) - dBuV
        /// RSSPECAN_VAL_UNIT_DBMV      (4) - dBmV
        /// RSSPECAN_VAL_UNIT_VOLT      (5) - V
        /// RSSPECAN_VAL_UNIT_DBUA      (6) - dBuA
        /// RSSPECAN_VAL_UNIT_AMP       (7) - A
        /// RSSPECAN_VAL_UNIT_DB        (8) - dB
        /// RSSPECAN_VAL_UNIT_PCT       (13) - %
        /// RSSPECAN_VAL_UNIT_UNITLESS  (14) - Unitless
        /// RSSPECAN_VAL_UNIT_DBPT      (15) - dBpT
        /// RSSPECAN_VAL_UNIT_DBMV_MHZ  (16) - dBmV/MHz
        /// RSSPECAN_VAL_UNIT_DBUV_MHZ  (17) - dBuV/MHz
        /// RSSPECAN_VAL_UNIT_DBUA_MHZ  (18) - dBuA/MHz
        /// RSSPECAN_VAL_UNIT_DBMV_M    (19) - dBmV/M
        /// RSSPECAN_VAL_UNIT_DBUA_M    (20) - dBuA/M
        /// RSSPECAN_VAL_UNIT_DBMV_MMHZ (21) - dBmV/MMHz
        /// RSSPECAN_VAL_UNIT_DBUA_MMHZ (22) - dBuA/MMHz
        /// RSSPECAN_VAL_UNIT_DBUV_M    (23) - dBuV/M
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_DBM  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_UNIT_POWER
        /// 
        /// Note(s):
        /// 
        /// (1) For R&amp;S FSV only values 0 to 7 are available.
        /// 
        /// (2) For R&amp;S FSUP only these values are available: 0 to 7, 15, 20, 23.
        /// DBM | DBPW | WATT | DBUV | DBMV | VOLT | DBUA | AMPere | DBPT | DBUV_M | DBUA_M
        /// 
        /// (3) For R&amp;S FSMR only values 0 to 14 are available.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureUnitPower(int Window, int Unit_Power)
        {
            int pInvokeResult = PInvoke.ConfigureUnitPower(this._handle, Window, Unit_Power);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the instrument's default unit of length.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_UNIT_LENGTH
        /// 
        /// Remote-control command(s):
        /// UNIT:LENGth METer | FEET
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Unit_Length">
        /// This control sets the instrument's default unit of length.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_LENGTH_METER (0) - Meter
        /// RSSPECAN_VAL_UNIT_LENGTH_FEET  (1) - Feet
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_UNIT_LENGTH_METER (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_UNIT_LENGTH
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureUnitLength(int Unit_Length)
        {
            int pInvokeResult = PInvoke.ConfigureUnitLength(this._handle, Unit_Length);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the channel number if the frequency mode is set ot "channel".
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CHANNEL_FREQUENCY_MODE_CHANNEL_NUMBER
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CHANNel?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Number_Of_Channel">
        /// This control returns the channel number if the frequency mode is set ot "channel".
        /// 
        /// Valid Range: 37750 to 38249
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsfsh_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int GetChannelNumber(out int Number_Of_Channel)
        {
            int pInvokeResult = PInvoke.GetChannelNumber(this._handle, out Number_Of_Channel);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the trace to acquire.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TRACE_TYPE
        /// RSSPECAN_ATTR_TRACE_STATE
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:MODE WRITe | VIEW | AVERage | MAXHold | MINHold | RMS
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACE&lt;1 to 3&gt;[:STATe]
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 1 to 6 for R&amp;S FSL
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRACE_STATE
        /// </param>
        /// <param name="Trace_Type">
        /// The type of trace to take. The driver sets the
        /// RSSPECAN_ATTR_TRACE_TYPE attribute to this value.
        /// 
        /// Valid Values:
        /// RSSPECAN_TRAC_MOD_WRITE (0) - Clear/Write
        /// RSSPECAN_TRAC_MOD_VIEW  (1) - View
        /// RSSPECAN_TRAC_MOD_AVER  (2) - Average
        /// RSSPECAN_TRAC_MOD_MAXH  (3) - Max Hold
        /// RSSPECAN_TRAC_MOD_MINH  (4) - Min Hold
        /// RSSPECAN_TRAC_MOD_BLANK (5) - Blank
        /// 
        /// Default Value:  RSSPECAN_TRAC_MOD_WRITE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRACE_TYPE
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTrace(int Window, int Trace, int Trace_Type)
        {
            int pInvokeResult = PInvoke.ConfigureTrace(this._handle, Window, Trace, Trace_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the trace detector.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DETECTOR_TYPE_AUTO
        /// RSSPECAN_ATTR_DETECTOR_TYPE
        /// RSSPECAN_ATTR_DETECTOR_TYPE_PORTABLE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DETector&lt;1 to 3&gt;[:FUNCtion]:AUTO ON | OFF
        /// [SENSe&lt;1|2&gt;:]DETector&lt;1 to 3&gt;[:FUNCtion]
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 1 to 4 for R&amp;S FSL
        /// 1 to 6 for R&amp;S ESL
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Detector_Type_Auto">
        /// Turns auto detector on/off. The driver uses this value to set the RSSPECAN_ATTR_DETECTOR_TYPE_AUTO attribute.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DETECTOR_TYPE_AUTO
        /// </param>
        /// <param name="Detector_Type">
        /// Specifies the method of capturing and processing
        /// signal data. The driver uses this value to set the RSSPECAN_ATTR_DETECTOR_TYPE or RSSPECAN_ATTR_DETECTOR_TYPE_PORTABLE attribute. See the attribute description for more details.
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_DETECTOR_TYPE_AUTO_PEAK (0) - Auto Peak
        /// RSSPECAN_VAL_DETECTOR_TYPE_AVERAGE   (1) - Average
        /// RSSPECAN_VAL_DETECTOR_TYPE_MAX_PEAK  (2) - Maximum Peak
        /// RSSPECAN_VAL_DETECTOR_TYPE_MIN_PEAK  (3) - Minimum Peak
        /// RSSPECAN_VAL_DETECTOR_TYPE_SAMPLE    (4) - Sample
        /// RSSPECAN_VAL_DETECTOR_TYPE_RMS       (5) - RMS
        /// RSSPECAN_VAL_DETECTOR_TYPE_QPK       (6) - QPeak
        /// RSSPECAN_VAL_DETECTOR_TYPE_CRMS      (7) - Cispr RMS
        /// RSSPECAN_VAL_DETECTOR_TYPE_CAV       (8) - Cispr Average
        /// 
        /// Default Value: RSSPECAN_VAL_DETECTOR_TYPE_AUTO_PEAK (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DETECTOR_TYPE
        /// 
        /// Note(s):
        /// 
        /// (1) Auto Peak: Allows the detector to capture better readings by using both positive and negative peak values when noise is present.
        /// 
        /// (2) Average: Average value of samples taken within the bin for a dedicated point on the display. Not available for R&amp;S FSH4/8 instrument.
        /// 
        /// (3) Maximum Peak: Obtains the maximum video signal between the last display point and the present display point.
        /// 
        /// (4) Minimum Peak: Obtains the minimum video signal between the last display point and the present display point.
        /// 
        /// (5) Sample: Pick one point within a bin.
        /// 
        /// (6) RMS: RMS value of samples taken within the bin for a dedicated point on the display.
        /// 
        /// (7) QPeak: Performs a signal evaluation for EMC measurements. Not available for R&amp;S FMU and R&amp;S FSH4/8 instrument.
        /// 
        /// (8) Cispr RMS: Performs a signal evaluation for EMC measurements. Available only for R&amp;S ESL instrument.
        /// 
        /// (9) Cispr Average: Performs a signal evaluation for EMC measurements. Available only for R&amp;S ESL instrument.
        /// 
        /// (10) This control has effect only if Detector Type Auto is set Off.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTraceDetector(int Window, int Trace, bool Detector_Type_Auto, int Detector_Type)
        {
            int pInvokeResult = PInvoke.ConfigureTraceDetector(this._handle, Window, Trace, System.Convert.ToUInt16(Detector_Type_Auto), Detector_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies whether or not the traces with peak or minimum value detection are reset after specific parameter changes.
        /// Usually the measurement must be restarted after a parameter change, before an evaluation of the measurement results is performed (e.g. with a marker). In cases in which a change causes a compulsory new measurement, the trace is automatically reset in order to prevent erroneous measurements of previous measurement results (e.g. when the span changes). For applications in which this behavior is not desired, this mechanism can be switched off.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TRACE_RESET_BEHAVIOR
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:MODE:HCONtinuous ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 1 to 6 for R&amp;S FSL
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Reset_at_Change">
        /// Specifies whether or not the traces with peak or minimum value detection are reset after specific parameter changes.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRACE_RESET_BEHAVIOR
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTraceResetBehavior(int Window, int Trace, bool Reset_at_Change)
        {
            int pInvokeResult = PInvoke.ConfigureTraceResetBehavior(this._handle, Window, Trace, System.Convert.ToUInt16(Reset_at_Change));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function stores the selected trace (1 to 3) in the measurement window indicated by window in a file with ASCII format.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// MMEMory:STORe&lt;1|2&gt;:TRACe 1 to 3,&lt;file_name&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 1 to 6 for R&amp;S FSL
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="File_Name">
        /// Specifies full path of the file, where trace data will be stored.
        /// 
        /// Valid Values:
        /// any valid file name
        /// 
        /// Default Value: "a:\\test.txt"
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int StoreTraceToFile(int Window, int Trace, string File_Name)
        {
            int pInvokeResult = PInvoke.StoreTraceToFile(this._handle, Window, Trace, File_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function stores the current trace into the memory trace of the R&amp;S FSH.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TRACE_TO_MEMORY
        /// 
        /// Remote-control command(s):
        /// CALCulate:MATH&lt;1...2&gt;:COPY:MEMory
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int CopyTraceToMemory(int Trace)
        {
            int pInvokeResult = PInvoke.CopyTraceToMemory(this._handle, Trace);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates and deactivates the trace stored in the memory of the R&amp;S FSH. The suffix at TRACe selects the memory slot the trace is stored in.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEMORY_DISPLAY_TRACE_STATE
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow]:TRACe&lt;1|2&gt;:MEMory[:STATe]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// 1 to 6
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// This control activates and deactivates the trace stored in the memory of the R&amp;S FSH. The suffix at TRACe selects the memory slot the trace is stored in.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0) - Off
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsfsh_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureMemoryTraceState(int Trace, bool State)
        {
            int pInvokeResult = PInvoke.ConfigureMemoryTraceState(this._handle, Trace, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the settings of the analyzer hardware for the measurement of I/Q data. This allows setting the bandwidth of the analog filters in front of the A/D converter as well as setting the sample rate, trigger conditions and the record length.
        /// 
        /// Notes:
        /// 
        /// (1) Function Configure Trace IQ Data Acquisition (rsspecan_ConfigureTraceIQDataAcquisition) has to be switched ON before using this function.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRACe:IQ:SET
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Sampling_Rate">
        /// This control sets the sampling rate for the I/Q data acquisition.
        /// 
        /// Valid Values: 10.0e3 Hz to 100.0e6 Hz
        /// 
        /// Value range with bandwidth extension (R&amp;S FSQ-B72): 10.0e3 Hz to 326.4e6 Hz
        /// 
        /// 
        /// Default Value: 32.0e6 Hz
        /// 
        /// Notes:
        /// 
        /// (1) Possible values are: 15.625 kHz, 31.25 kHz, 62.5 kHz, 125 kHz, 250 kHz, 500 kHz, 1 MHz, 2 MHz, 4 MHz, 8 MHz, 16 MHz, 32 MHz.
        /// 
        /// (2) List of valid values depends on used instrument.
        /// </param>
        /// <param name="Trigger_Mode">
        /// This parameter selects the trigger source for starting data acquisition.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TRG_IMM  (0) - Immediate (Default Value)
        /// RSSPECAN_VAL_TRG_EXT  (1) - External
        /// RSSPECAN_VAL_TRG_IFP  (2) - IFPower
        /// RSSPECAN_VAL_TRG_VID  (3) - Video
        /// RSSPECAN_VAL_TRG_TIME (4) - Time
        /// 
        /// Note:
        /// 
        /// (1) IFPower is available as of model 03 of the detector board assembly.
        /// 
        /// (2) After selecting IFPower, the trigger threshold can be set with command TRIG:LEV:IFP.
        /// 
        /// (3) VIDeo - the next measurement is triggered by the detection of a signal at the video filter output (not available for FM DEMOD mode).
        /// 
        /// (4) Time - the next measurement is triggered by the time. This option is valid only for R&amp;S FSL since FW 1.60 or R&amp;S FSV.
        /// </param>
        /// <param name="Trigger_Slope">
        /// This parameter selects the slope of the trigger signal for external trigger.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_NEG (0) - Negative
        /// RSSPECAN_VAL_POS (1) - Positive 
        /// 
        /// Default Value: RSSPECAN_VAL_POS (1)
        /// 
        /// Note:
        /// 
        /// Positive value is currently the only value supported.
        /// 
        /// </param>
        /// <param name="Bandwidth">
        /// This control sets bandwidth of the analog filters in front of the A/D converter.
        /// 
        /// Valid Values: 10.0 to 50.0e6 Hz in steps of 1, 3, 10
        /// 
        /// Default Value: 10.0e6 Hz
        /// 
        /// Note:
        /// (1) For baseband input only 3.0e+6 is valid value.
        /// 
        /// </param>
        /// <param name="Pretrigger_Samples">
        /// This control sets the number of measurement values to be recorded before the trigger point.
        /// 
        /// Valid Values: -16744447 to 65023
        ///                 
        /// 
        /// Default Value: 0
        /// 
        /// Note:
        /// 
        /// For &lt;trigger mode&gt; = Immediate the value must be 0.
        /// 
        /// 
        /// Note:
        /// (1) Valid Values is calculated:
        ///            -((2^24)-1-512k)) to ((64*1024)-512-1)
        /// </param>
        /// <param name="No_of_Samples">
        /// This control sets the number of measurement values to record.
        /// 
        /// Valid Values: 
        /// 1 to 16776704 (=16*1024*1024 - 512)
        /// 1 to 469761536 with FSQ-B100 option
        /// 1 to 1409285632 with FSQ-B100 and FSQ-B102 options
        /// 
        /// Default Value: 1
        /// 
        /// Note(s):
        /// 
        /// (1) List of valid values depends on used instrument.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int TraceIQSet(int Window, double Sampling_Rate, int Trigger_Mode, int Trigger_Slope, double Bandwidth, int Pretrigger_Samples, int No_of_Samples)
        {
            int pInvokeResult = PInvoke.TraceIQSet(this._handle, Window, Sampling_Rate, Trigger_Mode, Trigger_Slope, Bandwidth, Pretrigger_Samples, No_of_Samples);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the sampling rate for the I/Q data acquisition. Thus the sampling rate can be modified without affecting the other settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_IQ_SAMPLE_RATE
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:IQ:SRATe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Sampling_Rate">
        /// This control sets the sampling rate for the I/Q data acquisition.
        /// 
        /// Valid Values: 
        /// 10.0e3 Hz to 100.0e6 Hz
        /// 400.0 Hz to 100.0e6 Hz - FMU instrument
        /// 
        /// Value range with bandwidth extension (R&amp;S FSQ-B72): 10.0e3 Hz to 326.4e6 Hz
        /// 
        /// 
        /// Default Value: 32.0e6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_IQ_SAMPLE_RATE
        /// 
        /// Notes:
        /// 
        /// (1) Possible values are: 15.625 kHz, 31.25 kHz, 62.5 kHz, 125 kHz, 250 kHz, 500 kHz, 1 MHz, 2 MHz, 4 MHz, 8 MHz, 16 MHz, 32 MHz.
        /// 
        /// (2) List of valid values depends on used instrument.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int TraceIQSamplingRate(int Window, double Sampling_Rate)
        {
            int pInvokeResult = PInvoke.TraceIQSamplingRate(this._handle, Window, Sampling_Rate);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the IQ data measurement averaging.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_IQ_DATA_AVER_STATE
        /// RSSPECAN_ATTR_IQ_DATA_AVER_COUNT
        /// 
        /// Remote-control command(s):
        /// TRACe:IQ:AVERage:STATe ON | OFF
        /// TRACe:IQ:AVERage:COUNt
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Average_State">
        /// This command switches the averaging function for the recorded I/Q data on or off. Preconditions are that the I/Q data recording function has been switched on previously and that the sampling rate has been set to 32 MHz. The driver uses this value to set the RSSPECAN_ATTR_IQ_DATA_AVER_STATE attribute.
        /// 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_IQ_DATA_AVER_STATE
        /// 
        /// Note:
        /// (1) Averaging is not supported for sampling rates &lt;&gt; 32 MHz or for trigger offsets &lt; 0.
        /// 
        /// 
        /// 
        /// </param>
        /// <param name="Average_Count">
        /// This command defines the number of averages to perform on the recorded I/Q data. The driver uses this value to set the RSSPECAN_ATTR_IQ_DATA_AVER_COUNT attribute.
        /// 
        /// Valid Values:
        ///  0 to 32767
        /// 
        /// Default Value:
        ///  0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_IQ_DATA_AVER_COUNT
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int TraceIQAveraging(int Window, bool Average_State, int Average_Count)
        {
            int pInvokeResult = PInvoke.TraceIQAveraging(this._handle, Window, System.Convert.ToUInt16(Average_State), Average_Count);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function allows measurement with extended usable signal bandwidth for sample rates in the range of 20.4 MHz to 40.8 MHz. 
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available in R&amp;S FSP, FSQ, FSU and FSMR instruments.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_IQ_EXT_BAND
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:IQ:FILTer:FLATness NORMal | WIDE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Filter">
        /// This control determines whether the extended usable signal bandwidth is used. The driver uses this value to set the RSSPECAN_ATTR_IQ_EXT_BAND attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_IQ_FILT_NORM (0) - Normal Operation 
        /// RSSPECAN_VAL_IQ_FILT_WIDE (1) - Extended Bandwidth
        /// 
        /// Default Value: RSSPECAN_VAL_IQ_FILT_NORM (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_IQ_EXT_BAND
        /// 
        /// Note(s):
        /// 
        /// (1) Sample Rate Range: 10.2 MHz &lt; Sample Rate &lt;= 20.4 MHz
        /// 
        /// Usable Bandwidth:
        /// - Normal Operation: 0.8 * Sample Rate
        /// - Extended Bandwidth: 0.9 * Sample Rate
        /// 
        /// (2) Sample Rate Range: 20.4 MHz &lt; Sample Rate &lt;= 40.8 MHz
        /// 
        /// Usable Bandwidth:
        /// - Normal Operation: 0.68 * Sample Rate
        /// - Extended Bandwidth: 0.8 * Sample Rate
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int TraceIQExtendedBandwidth(int Window, int Filter)
        {
            int pInvokeResult = PInvoke.TraceIQExtendedBandwidth(this._handle, Window, Filter);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the trace IQ data acquisition.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_IQ_DATA_STATE
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:IQ[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace_State">
        /// This control switches IQ data acquisition on or off. The driver uses this value to set the RSSPECAN_ATTR_IQ_DATA_STATE attribute.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_IQ_DATA_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTraceIQDataAcquisition(int Window, bool Trace_State)
        {
            int pInvokeResult = PInvoke.ConfigureTraceIQDataAcquisition(this._handle, Window, System.Convert.ToUInt16(Trace_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the synchronization of the start of I/Q data acquisition to the trigger event. Prior to this command, I/Q data acquisition has to be switched. The sampling rate must be 32 MHz. The synchronization ensures the correct calculation of I/Q average values as the data aquisition
        /// is always started with the same phase reference in relation to the trigger event.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available for R&amp;S FSU &amp; FSQ instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_IQ_START_SYNCHRONIZATION
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:IQ:SYNChronize[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Start_Synchronization">
        /// This control switches on or off the synchronization of the start of I/Q data acquisition to the trigger event.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_IQ_START_SYNCHRONIZATION
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTraceIQStartSynchronization(int Window, bool Start_Synchronization)
        {
            int pInvokeResult = PInvoke.ConfigureTraceIQStartSynchronization(this._handle, Window, System.Convert.ToUInt16(Start_Synchronization));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the Bandwidth Extension FSQ-B72 on or off.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_IQ_DATA_WBAND_STATE
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:IQ:WBANd[:STATe] ON|OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="BW_Extension_State">
        /// This control switches the Bandwidth Extension FSQ-B72 on or off.  The driver uses this value to set the RSSPECAN_ATTR_IQ_DATA_WBAND_STATE attribute.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_IQ_DATA_WBAND_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTraceIQBWExtension(int Window, bool BW_Extension_State)
        {
            int pInvokeResult = PInvoke.ConfigureTraceIQBWExtension(this._handle, Window, System.Convert.ToUInt16(BW_Extension_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the gate function for IQ data measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available for R&amp;S FSV instrument.
        /// 
        /// (2) This function is only valid for TRAC:IQ ON and trigger modes "EXT" and "IFP".
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_IQ_DATA_GATE_STATE
        /// RSSPECAN_ATTR_IQ_DATA_GATE_TYPE
        /// RSSPECAN_ATTR_IQ_DATA_GATE_LENGTH
        /// RSSPECAN_ATTR_IQ_DATA_GATE_GAP
        /// RSSPECAN_ATTR_IQ_DATA_GATE_NUMBER_OF_PERIODS
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1...4&gt;:IQ:EGATe ON | OFF
        /// TRACe&lt;1...4&gt;:IQ:EGATe:TYPE LEVel | EDGE
        /// TRACe&lt;1...4&gt;:IQ:EGATe:LENGth
        /// TRACe&lt;1...4&gt;:IQ:EGATe:GAP
        /// TRACe&lt;1...4&gt;:IQ:EGATe:NOFgateperiods
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Gate">
        /// This control activates or deactivates the gate function for IQ data measurements.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_IQ_DATA_GATE_STATE
        /// </param>
        /// <param name="Type">
        /// This control specifies the gate trigger type used for data collection.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GATE_TYPE_LEVEL (0) - Level
        /// RSSPECAN_VAL_GATE_TYPE_EDGE  (1) - Edge
        /// 
        /// Default Value: RSSPECAN_VAL_GATE_TYPE_EDGE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_IQ_DATA_GATE_TYPE
        /// </param>
        /// <param name="Gate_Length">
        /// This control defines the gate length in samples in edge mode.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 100
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_IQ_DATA_GATE_LENGTH
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available if Type is set to Edge.
        /// </param>
        /// <param name="Gate_Gap">
        /// This control defines the interval between several gate periods in samples.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 100
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_IQ_DATA_GATE_GAP
        /// </param>
        /// <param name="Number_Of_Gate_Periods">
        /// This control defines the number of gate periods after a trigger signal.
        /// 
        /// Valid Values: 
        /// 1 to 1023
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_IQ_DATA_GATE_NUMBER_OF_PERIODS
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTraceIQGate(bool Gate, int Type, int Gate_Length, int Gate_Gap, int Number_Of_Gate_Periods)
        {
            int pInvokeResult = PInvoke.ConfigureTraceIQGate(this._handle, System.Convert.ToUInt16(Gate), Type, Gate_Length, Gate_Gap, Number_Of_Gate_Periods);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the display type of the IQ data.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available for R&amp;S FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_IQ_DATA_DISPLAY_TYPE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1...4&gt;:FORMat RIMag | MAGNitude | FREQuency | VECTor
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Display_Type">
        /// This control defines the display type of the IQ data.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_IQ_DATA_TYPE_RIMAG (0) - IQ Data
        /// RSSPECAN_VAL_IQ_DATA_TYPE_MAGN  (1) - Magnitude
        /// RSSPECAN_VAL_IQ_DATA_TYPE_FREQ  (2) - Spectrum
        /// RSSPECAN_VAL_IQ_DATA_TYPE_VECT  (3) - IQ Vector
        /// 
        /// Default Value: RSSPECAN_VAL_IQ_DATA_TYPE_RIMAG (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_IQ_DATA_DISPLAY_TYPE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTraceIQDataDisplayType(int Display_Type)
        {
            int pInvokeResult = PInvoke.ConfigureTraceIQDataDisplayType(this._handle, Display_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the filter bandwidth of the resampling filter, which is dependent on the sampling rate. 
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available for R&amp;S FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_IQ_DATA_FILTER_BANDWIDTH
        /// 
        /// Remote-control command(s):
        /// TRACe:IQ:BWIDth?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Filter_Bandwidth">
        /// This control queries the filter bandwidth of the resampling filter.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_IQ_DATA_FILTER_BANDWIDTH
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryTraceIQFilterBandwidth(out double Filter_Bandwidth)
        {
            int pInvokeResult = PInvoke.QueryTraceIQFilterBandwidth(this._handle, out Filter_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function copies one trace array to another trace array.  Any data in the destination trace is over written. 
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:COPY TRACE1| TRACE2| TRACE3
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Destination_Trace">
        /// Selects the destination trace.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 1 to 6 for R&amp;S FSL or R&amp;S ESL
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Source_Trace">
        /// Selects the source trace.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 1 to 6 for R&amp;S FSL or R&amp;S ESL
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CopyTrace(int Window, int Destination_Trace, int Source_Trace)
        {
            int pInvokeResult = PInvoke.CopyTrace(this._handle, Window, Destination_Trace, Source_Trace);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures subtraction of traces.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TRACE_MATH_POSITION
        /// RSSPECAN_ATTR_TRACE_MATH_STATE
        /// RSSPECAN_ATTR_TRACE_MATH_MODE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MATH:POSition
        /// CALCulate&lt;1|2&gt;:MATH:STATe ON | OFF
        /// CALCulate&lt;1|2&gt;:MATH:MODE LINear | LOGarithmic | POWer
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Position">
        /// Defines the position of the result of the trace mathematics in the selected measurement window. The indication is in % of the screen height, with 100% corresponding to the upper diagram border. The driver uses this value to set the RSSPECAN_ATTR_TRACE_MATH_POSITION attribute.
        /// 
        /// Valid Values:
        /// -100.0 to 200.0 %
        /// 
        /// Default Value: 50.0 %
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRACE_MATH_POSITION
        /// </param>
        /// <param name="Mode">
        /// Selects linear or logarithmic (= video) calculation of the mathematical functions related to the traces. The calculation of the average is one of the affected functions. The setting is valid for all measurement windows. The driver uses this value to set the RSSPECAN_ATTR_TRACE_MATH_MODE attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIN  (0) - Linear
        /// RSSPECAN_VAL_LOG  (1) - Logarithmic
        /// RSSPECAN_VAL_MPOW (2) - Power
        /// 
        /// Default Value: RSSPECAN_VAL_LOG (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRACE_MATH_MODE
        /// 
        /// Note:
        /// 
        /// (1) This control is available from firmware 1.90 on R&amp;S ESL.
        /// 
        /// (2) Power is only available for R&amp;S ESL or R&amp;S FSV or R&amp;S FSL or R&amp;S FSUP.
        /// </param>
        /// <param name="Trace_Math_State">
        /// Switches the mathematical relation of traces on or off.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRACE_MATH_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSubtractTraces(int Window, double Position, int Mode, bool Trace_Math_State)
        {
            int pInvokeResult = PInvoke.ConfigureSubtractTraces(this._handle, Window, Position, Mode, System.Convert.ToUInt16(Trace_Math_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function subtracts the array elements of trace defined in argument Trace 2 from TRACE1 of the instrument and stores the result in the TRACE1.
        /// 
        /// Destination Trace = TRACE1 - {Trace 2}
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MATH[:EXPression][:DEFine]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace_2">
        /// Selects the trace for subtraction.
        /// 
        /// Valid Values:
        /// 2 to 3
        /// 2 to 6 for R&amp;S FSL or R&amp;S ESL
        /// 
        /// Default Value: 2
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SubtractTraces(int Window, int Trace_2)
        {
            int pInvokeResult = PInvoke.SubtractTraces(this._handle, Window, Trace_2);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function enables the active marker on the specified trace.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_ENABLED
        /// RSSPECAN_ATTR_MARKER_TRACE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;[:STATe]
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:TRACe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects delta marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 1..16 for R&amp;S FSV
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker_Enabled">
        /// Specifies whether the specified delta marker is enabled. 
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_ENABLED
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition. 
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 1 to 6 for R&amp;S FSL, R&amp;S FSV
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_TRACE
        /// 
        /// Note:
        /// (1) This control has effect only if Marker Enabled is set On.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMarker(int Window, int Marker, bool Marker_Enabled, int Trace)
        {
            int pInvokeResult = PInvoke.ConfigureMarker(this._handle, Window, Marker, System.Convert.ToUInt16(Marker_Enabled), Trace);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function turns configures signal-tracking. 
        /// 
        /// With signal track activated, the maximum signal is determined after each frequency sweep and the center frequency is set to the frequency of this signal. Thus with drifting signals the center frequency follows the signal.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SIGNAL_TRACK_ENABLED
        /// RSSPECAN_ATTR_SIGNAL_TRACK_BWID
        /// RSSPECAN_ATTR_SIGNAL_TRACK_THRESHOLD
        /// RSSPECAN_ATTR_SIGNAL_TRACK_TRACE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:STRack[:STATe] ON | OFF
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:STRack:BWIDth
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:STRack:THReshold
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:STRack:TRACe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Signal_Track_Enabled">
        /// Switches the signal-track function on or off for the selected measurement window. 
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SIGNAL_TRACK_ENABLED
        /// </param>
        /// <param name="Signal_Track_Bandwidth">
        /// Defines the bandwidth around the center frequency within which the largest signal is searched. 
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SIGNAL_TRACK_BWID
        /// 
        /// Note:
        /// (1) This control has effect only if Signal Track Enabled is set On
        /// </param>
        /// <param name="Signal_Track_Threshold">
        /// Defines the threshold above which the largest signal is searched for in the selected measurement window. 
        /// 
        /// Valid Values:
        /// -330.0 to 30.0 dBm
        /// 
        /// Default Value: -120.0 dBm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SIGNAL_TRACK_THRESHOLD
        /// 
        /// Note:
        /// (1) This control has effect only if Signal Track Enabled is set On
        /// </param>
        /// <param name="Trace">
        /// Defines the trace on which the largest signal is searched for in the selected measurement window. 
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 1 to 6 for R&amp;S FSL and R&amp;S FSV
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SIGNAL_TRACK_TRACE
        /// 
        /// Note:
        /// (1) This control has effect only if Signal Track Enabled is set On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSignalTrack(int Window, bool Signal_Track_Enabled, double Signal_Track_Bandwidth, double Signal_Track_Threshold, int Trace)
        {
            int pInvokeResult = PInvoke.ConfigureSignalTrack(this._handle, Window, System.Convert.ToUInt16(Signal_Track_Enabled), Signal_Track_Bandwidth, Signal_Track_Threshold, Trace);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function turns off all the markers in selected measurement window.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_AOFF
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:AOFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int DisableAllMarkers(int Window)
        {
            int pInvokeResult = PInvoke.DisableAllMarkers(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the type of marker search and performs the search.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_SEARCH_PEAK
        /// RSSPECAN_ATTR_MARKER_SEARCH_PEAK_NEXT
        /// RSSPECAN_ATTR_MARKER_SEARCH_PEAK_RIGHT
        /// RSSPECAN_ATTR_MARKER_SEARCH_PEAK_LEFT
        /// RSSPECAN_ATTR_MARKER_SEARCH_MIN
        /// RSSPECAN_ATTR_MARKER_SEARCH_MIN_NEXT
        /// RSSPECAN_ATTR_MARKER_SEARCH_MIN_RIGHT
        /// RSSPECAN_ATTR_MARKER_SEARCH_MIN_LEFT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:MAXimum[:PEAK]
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:MAXimum:NEXT
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:MAXimum:RIGHt
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:MAXimum:LEFT
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:MINimum[:PEAK]
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:MINimum:NEXT
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:MINimum:RIGHt
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:MINimum:LEFT
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 1..16 for R&amp;S FSV
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker_Search">
        /// Specifies the type of marker search type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MARKER_SEARCH_HIGHEST            (1) - Peak Maximum
        /// RSSPECAN_VAL_MARKER_SEARCH_MINIMUM            (2) - Peak Minimum
        /// RSSPECAN_VAL_MARKER_SEARCH_NEXT_PEAK          (3) - Next Maximum
        /// RSSPECAN_VAL_MARKER_SEARCH_NEXT_PEAK_LEFT     (4) - Left Maximum
        /// RSSPECAN_VAL_MARKER_SEARCH_NEXT_PEAK_RIGHT    (5) - Right Maximum
        /// RSSPECAN_VAL_MARKER_SEARCH_NEXT_MINIMUM       (6) - Next Minimum
        /// RSSPECAN_VAL_MARKER_SEARCH_NEXT_MINIMUM_LEFT  (7) - Left Minimum 
        /// RSSPECAN_VAL_MARKER_SEARCH_NEXT_MINIMUM_RIGHT (8) - Right Minimum
        /// 
        /// Default Value:  RSSPECAN_VAL_MARKER_SEARCH_HIGHEST
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_SEARCH_PEAK
        /// RSSPECAN_ATTR_MARKER_SEARCH_PEAK_NEXT
        /// RSSPECAN_ATTR_MARKER_SEARCH_PEAK_RIGHT
        /// RSSPECAN_ATTR_MARKER_SEARCH_PEAK_LEFT
        /// RSSPECAN_ATTR_MARKER_SEARCH_MIN
        /// RSSPECAN_ATTR_MARKER_SEARCH_MIN_NEXT
        /// RSSPECAN_ATTR_MARKER_SEARCH_MIN_RIGHT
        /// RSSPECAN_ATTR_MARKER_SEARCH_MIN_LEFT
        /// 
        /// Note(s):
        /// 
        /// (1) Peak Maximum - Sets marker search for the highest amplitude
        /// 
        /// (2) Next Maximum - Sets marker search for the next highest peak.
        /// 
        /// (3) Left Maximum - Sets marker search for the next peak left of the peak amplitude.
        /// 
        /// (4) Right Maximum - Sets marker search for the next peak right of the peak amplitude.
        /// 
        /// (5) Peak Minimum - Sets marker search for the minimum amplitude.
        /// 
        /// (6) Next Minimum - Sets marker search for the next lowest minimum.
        /// 
        /// (7) Left Minimum - Sets marker search for the next minimum left of the minimum amplitude.
        /// 
        /// (8) Right Minimum - Sets marker search for the next minimum right of the minimum amplitude.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int MarkerSearch(int Window, int Marker, int Marker_Search)
        {
            int pInvokeResult = PInvoke.MarkerSearch(this._handle, Window, Marker, Marker_Search);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures an automatic peak search action
        /// for Marker 1 at the end of each particular sweep. This function may be used during adjustments of a device under test to keep track of the actual peak marker position and level.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_SEARCH_PEAK_AUTO
        /// RSSPECAN_ATTR_MARKER_SEARCH_MIN_AUTO
        /// 
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:MAXimum:AUTO ON | OFF
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:MINimum:AUTO ON | OFF
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker_Search">
        /// Specifies the type of marker search type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MARKER_SEARCH_HIGHEST (1) - Peak Maximum
        /// RSSPECAN_VAL_MARKER_SEARCH_MINIMUM (2) - Peak Minimum
        /// 
        /// Default Value:  RSSPECAN_VAL_MARKER_SEARCH_HIGHEST
        /// 
        /// Note(s):
        /// 
        /// (1) Peak Maximum - Sets marker search for the highest amplitude
        /// 
        /// (2) Peak Minimum - Sets marker search for the minimum amplitude.
        /// 
        /// </param>
        /// <param name="Auto_Peak">
        /// Switches the peak auto search state. 
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_SEARCH_PEAK_AUTO
        /// RSSPECAN_ATTR_MARKER_SEARCH_MIN_AUTO
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int MarkerPeakAuto(int Window, int Marker_Search, bool Auto_Peak)
        {
            int pInvokeResult = PInvoke.MarkerPeakAuto(this._handle, Window, Marker_Search, System.Convert.ToUInt16(Auto_Peak));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function moves the active marker to the specified horizontal position.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_POSITION
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:X
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 1..16 for R&amp;S FSV
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker_Position">
        /// Horizontal position.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_POSITION
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int MoveMarker(int Window, int Marker, double Marker_Position)
        {
            int pInvokeResult = PInvoke.MoveMarker(this._handle, Window, Marker, Marker_Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function makes the selected marker frequency to be the center frequency or step width of center frequency. It can also make the active marker amplitude to be the reference level.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_TO_CENTER
        /// RSSPECAN_ATTR_MARKER_TO_STEP
        /// RSSPECAN_ATTR_MARKER_TO_REFERENCE
        /// RSSPECAN_ATTR_MARKER_TO_REFERENCE_PORTABLE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:CENTer
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:CSTep
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:REFerence
        /// CALCulate:MARKer&lt;1...6&gt;:FUNCtion:LEVel:ONCE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 1..16 for R&amp;S FSV
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Instrument_Setting">
        /// Specifies the instrument setting to be set from the marker position.
        /// 
        /// Defined Values:
        /// RSSPECAN_VAL_INSTRUMENT_SETTING_FREQUENCY_CENTER (1) - Center Frequency
        /// RSSPECAN_VAL_INSTRUMENT_SETTING_FREQUENCY_STEP   (2) - Step Width of Center Frequency
        /// RSSPECAN_VAL_INSTRUMENT_SETTING_REFERENCE_LEVEL  (3) - Reference Level
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_INSTRUMENT_SETTING_FREQUENCY_CENTER (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_TO_CENTER
        /// RSSPECAN_ATTR_MARKER_TO_STEP
        /// RSSPECAN_ATTR_MARKER_TO_REFERENCE
        /// 
        /// Note(s):
        /// (1) Center Frequency - sets the center frequency of the selected measurement window equal to the frequency of the indicated marker.
        /// If marker 2, 3 or 4 is selected and used as delta marker, it is switched to the marker mode.
        /// 
        /// (2) Step Width of Center Frequency - sets the step width of the center frequency in the selected measurement window to the X value of the current marker. If marker 2, 3 or 4 is selected and used as delta marker, it is switched to the marker mode. Not available for R&amp;S FMU instrument.
        /// 
        /// (3) Reference Level - sets the reference level in the selected measurement window to the power measured by the indicated marker. If marker 2, 3 or 4 is selected and used as delta marker, it is switched to marker mode.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SetInstrumentFromMarker(int Window, int Marker, int Instrument_Setting)
        {
            int pInvokeResult = PInvoke.SetInstrumentFromMarker(this._handle, Window, Marker, Instrument_Setting);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates or deactivates the zoom. The display is centered on the marker. The zoom factor is set with CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ZOOM (attribute RSSPECAN_ATTR_MARKER_ZOOM).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_ZOOM_STATE
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...6&gt;:X[:SCALe]:ZOOM ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Marker_Zoom_State">
        /// Activates or deactivates the zoom. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_ZOOM_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMarkerZoomState(bool Marker_Zoom_State)
        {
            int pInvokeResult = PInvoke.ConfigureMarkerZoomState(this._handle, System.Convert.ToUInt16(Marker_Zoom_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the range to be zoomed around marker 1 in the selected measurement window. Marker 1 is activated first, if necessary.
        /// The subsequent frequency sweep is stopped at the marker position and the frequency of the signal is counted. This frequency becomes the new center frequency, and the zoomed span is set. In order to recognize the end of the operation the synchronization to the sweep end should be activated. This is only possible in single sweep mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_ZOOM
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:ZOOM
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker_Zoom">
        /// Defines the range to be zoomed around marker 1 in the selected measurement window.
        /// 
        /// Units: Hz
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 10.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_ZOOM
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMarkerZoom(int Window, double Marker_Zoom, uint Timeout)
        {
            int pInvokeResult = PInvoke.ConfigureMarkerZoom(this._handle, Window, Marker_Zoom, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the marker zoom.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_ZOOM_STATE_PORTABLE
        /// RSSPECAN_ATTR_MARKER_ZOOM_FACTOR_PORTABLE
        /// RSSPECAN_ATTR_MARKER_ZOOM_X_PORTABLE
        /// 
        /// Remote-control command(s):
        /// DISPlay:ZOOM:AREA[:STATe]
        /// DISPlay:ZOOM:FACTor 2| 4 | 8
        /// DISPlay:ZOOM:X
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Zoom">
        /// This control sets the state of the marker zoom function in a Smith chart.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_ZOOM_STATE_PORTABLE
        /// </param>
        /// <param name="Zoom_Factor">
        /// This control sets the zoom factor of the marker zoom function in a Smith chart.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DISPLAY_ZOOM_FACTOR_2 (0) - 2
        /// RSSPECAN_VAL_DISPLAY_ZOOM_FACTOR_4 (1) - 4
        /// RSSPECAN_VAL_DISPLAY_ZOOM_FACTOR_8 (2) - 8
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_DISPLAY_ZOOM_FACTOR_2 (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_ZOOM_FACTOR_PORTABLE
        /// </param>
        /// <param name="X">
        /// This control sets the zoom window on the x-axis of the Smith chart. '0%' marks the center of the x-axis in the Smith chart.
        /// 
        /// Valid Values:
        /// -50.0 to 50.0
        /// 
        /// Default Value:
        /// 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_ZOOM_X_PORTABLE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureMarkerZoomPortable(bool Zoom, int Zoom_Factor, double X)
        {
            int pInvokeResult = PInvoke.ConfigureMarkerZoomPortable(this._handle, System.Convert.ToUInt16(Zoom), Zoom_Factor, X);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on and off a link of delta marker 1 to marker 1. If the link is switched on and the x-axis value of the marker1 is changed the delta marker 1 will follow on
        /// the same x-position.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_LINK
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:LINK ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1
        /// 1..16 for R&amp;S FSV
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// Specifies the marker and delta marker link state.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value:  VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_LINK
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int LinkMarkerAndDeltamarker(int Window, int Marker, bool State)
        {
            int pInvokeResult = PInvoke.LinkMarkerAndDeltamarker(this._handle, Window, Marker, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the horizontal position and the marker amplitude level of the selected marker.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_POSITION
        /// RSSPECAN_ATTR_MARKER_AMPLITUDE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:X?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:Y?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 1..16 for R&amp;S FSV
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker_Position">
        /// The horizontal position of the selected marker.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_POSITION
        /// </param>
        /// <param name="Marker_Amplitude">
        /// The amplitude value of the selected marker (Amplitude Units or relative amplitude units for delta markers).
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_AMPLITUDE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryMarker(int Window, int Marker, out double Marker_Position, out double Marker_Amplitude)
        {
            int pInvokeResult = PInvoke.QueryMarker(this._handle, Window, Marker, out Marker_Position, out Marker_Amplitude);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the step size of the rotary knob for marker or delta marker value changes.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_STEP_SIZE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:X:SSIZe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Step_Size">
        /// Defines the step size. 
        /// .
        /// 
        /// Valid Values: 
        /// 
        /// RSSPECAN_VAL_MARK_STEP_SIZE_STAN (0) - Standard
        /// RSSPECAN_VAL_MARK_STEP_SIZE_POIN (1) - Points
        /// 
        /// Default Value: RSSPECAN_VAL_MARK_STEP_SIZE_STAN (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_STEP_SIZE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMarkerStepSize(int Window, int Step_Size)
        {
            int pInvokeResult = PInvoke.ConfigureMarkerStepSize(this._handle, Window, Step_Size);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function positions the selected marker to the given probability.
        /// 
        /// Note(s): 
        /// 
        /// (1) This function is only available with the CCDF measurement switched on. The associated level value can be determined with the CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:X command.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_PROBABILITY
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:Y:PERCent
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 1..16 for R&amp;S FSV
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Probability">
        /// Defines the probability.
        /// 
        /// Units: %
        /// 
        /// Valid Values: 
        /// 
        /// 0.0 to 100.0
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_PROBABILITY
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ProbabilityMarkerPosition(int Window, int Marker, double Probability)
        {
            int pInvokeResult = PInvoke.ProbabilityMarkerPosition(this._handle, Window, Marker, Probability);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the marker frequency display mode.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_DISPLAY_MODE
        /// 
        /// Remote-control command(s):
        /// CALCulate:MARKer&lt;1...6&gt;:FREQuency:MODE FREQ | CHAN
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker number.
        /// 
        /// Valid Values:
        /// 1 to 6
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Display_Mode">
        /// This control sets the marker frequency display mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MARKER_DISPLAY_MODE_FREQ (0) - Frequency
        /// RSSPECAN_VAL_MARKER_DISPLAY_MODE_CHAN (1) - Channel
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_MARKER_DISPLAY_MODE_FREQ (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_DISPLAY_MODE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureMarkerDisplayMode(int Marker, int Display_Mode)
        {
            int pInvokeResult = PInvoke.ConfigureMarkerDisplayMode(this._handle, Marker, Display_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function couples the receiver frequency to the marker. With every update of the marker position, the receiver frequency is adjusted to the position of the marker. If deactivated, an update of the marker position has no effect on the receiver frequency.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_RECEIVER_FREQ_COUPLING
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:COUPled[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// Couples the receiver frequency to the marker.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value:  VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_RECEIVER_FREQ_COUPLING
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMarkerReceiverFrequencyCoupling(int Window, int Marker, bool State)
        {
            int pInvokeResult = PInvoke.ConfigureMarkerReceiverFrequencyCoupling(this._handle, Window, Marker, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function couples the receiver frequency settings of the corresponding subscans to the marker. With every update of the marker position, the receiver frequency is adjusted to the position of the marker. If deactivated, an update of the marker position has no effect on the receiver frequency.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_SUBSCAN_COUPLING
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:SCOupled[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// Couples the receiver frequency settings of the corresponding subscans to the marker.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value:  VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_SUBSCAN_COUPLING
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMarkerSubscanCoupling(int Window, int Marker, bool State)
        {
            int pInvokeResult = PInvoke.ConfigureMarkerSubscanCoupling(this._handle, Window, Marker, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures marker threshold values. The marker threshold specifies a lower bound for ALL marker search functions.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_THRESHOLD_STATE
        /// RSSPECAN_ATTR_MARKER_THRESHOLD
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:THReshold:STATe ON | OFF
        /// CALCulate&lt;1|2&gt;:THReshold
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Threshold_State">
        /// Switches on or off the threshold line in the selected measurement window.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_THRESHOLD_STATE
        /// </param>
        /// <param name="Marker_Threshold">
        /// Sets the threshold value for the maximum/minimum search of markers with marker search functions MAX PEAK, NEXT PEAK, etc in the selected measurement window. The associated display line is automatically switched on. 
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_THRESHOLD
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMarkerSearchThreshold(int Window, bool Threshold_State, double Marker_Threshold)
        {
            int pInvokeResult = PInvoke.ConfigureMarkerSearchThreshold(this._handle, Window, System.Convert.ToUInt16(Threshold_State), Marker_Threshold);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the marker peak excursion The marker peak excursion specifies the minimum amplitude variation that can be recognized as a peak or minimum by the marker.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PEAK_EXCURSION
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:PEXCursion
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Peak_Excursion">
        /// Sets the minimum amplitude variation of the signal in dB that the marker can identify as a peak. 
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 6.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PEAK_EXCURSION
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMarkerSearchPeakExcursion(int Window, double Peak_Excursion)
        {
            int pInvokeResult = PInvoke.ConfigureMarkerSearchPeakExcursion(this._handle, Window, Peak_Excursion);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures state of local oscillator supression.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_LOEX
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:LOEXclude ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Local_Oscillator_Supression">
        /// Switches the local oscillator suppression for peak search on or off. This setting is valid for all markers and delta markers in all measurement windows. 
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_LOEX
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMarkerSearchLocalOscillator(int Window, bool Local_Oscillator_Supression)
        {
            int pInvokeResult = PInvoke.ConfigureMarkerSearchLocalOscillator(this._handle, Window, System.Convert.ToUInt16(Local_Oscillator_Supression));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures marker search limits.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_SEARCH_LIMITS_STATE
        /// RSSPECAN_ATTR_MARKER_SEARCH_LIMITS_LEFT
        /// RSSPECAN_ATTR_MARKER_SEARCH_LIMITS_RIGHT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:X:SLIMits[:STATe] ON | OFF
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:X:SLIMits:LEFT
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:X:SLIMits:RIGHT
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Search_Limits">
        /// Switches between a limited (ON) and unlimited (OFF) search range in the selected measurement window. The function is independent of the selection of a marker. 
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_SEARCH_LIMITS_STATE
        /// </param>
        /// <param name="Search_Limit_Left">
        /// Sets the left limit of the search range for markers and delta markers in the selected measurement window. Depending on the x axis domain the indicated value defines a frequency (span &gt; 0) or time (span = 0). The function is independent of the selection of a marker.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_SEARCH_LIMITS_LEFT
        /// 
        /// Note(s):
        /// 
        /// (1) This control is valid only if Search Limits is set to On.
        /// </param>
        /// <param name="Search_Limit_Right">
        /// Sets the right limit of the search range for markers and delta markers in the selected measurement window. Depending on the x axis domain the indicated value defines a frequency (span &gt; 0) or time (span = 0). The function is independent of the selection of a marker.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_SEARCH_LIMITS_RIGHT
        /// 
        /// Note(s):
        /// 
        /// (1) This control is valid only if Search Limits is set to On.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMarkerSearchLimits(int Window, bool Search_Limits, double Search_Limit_Left, double Search_Limit_Right)
        {
            int pInvokeResult = PInvoke.ConfigureMarkerSearchLimits(this._handle, Window, System.Convert.ToUInt16(Search_Limits), Search_Limit_Left, Search_Limit_Right);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the limits of the search range for markers and delta markers to the zoom area. 
        /// 
        /// Note(s): 
        /// 
        /// (1) The function is only available if the search limit for marker and delta marker is switched on (CALCulate&lt;1|2&gt;:MARKer&lt;1...16&gt;:X:SLIMits[:STATe]).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_SEARCH_LIMITS_ZOOM
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...16&gt;:X:SLIMits:ZOOM[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Search_Limits_Zoom">
        /// Sets the limits of the search range for markers and delta markers to the zoom area.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_SEARCH_LIMITS_ZOOM
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMarkerSearchLimitsZoom(int Window, bool Search_Limits_Zoom)
        {
            int pInvokeResult = PInvoke.ConfigureMarkerSearchLimitsZoom(this._handle, Window, System.Convert.ToUInt16(Search_Limits_Zoom));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures marker demodulation parameters.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_DEMOD_STATE
        /// RSSPECAN_ATTR_MARKER_DEMOD_TYPE
        /// RSSPECAN_ATTR_MARKER_DEMOD_HOLDOFF
        /// RSSPECAN_ATTR_MARKER_DEMOD_CONT
        /// RSSPECAN_ATTR_MARKER_DEMOD_SQUELCH_STATE
        /// RSSPECAN_ATTR_MARKER_DEMOD_SQUELCH_LEVEL
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:DEModulation[:STATe] ON | OFF
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:DEModulation:SELect AM | FM
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:DEModulation:HOLDoff
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:DEModulation:CONTinuous ON | OFF
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:DEModulation:SQUelch[:STATe] ON | OFF
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:DEModulation:SQUelch:LEVel
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 1..16 for R&amp;S FSV
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// This control switches on or off the audio demodulator when the indicated marker is reached in the selected measurement window. In the frequency domain (span &gt; 0) the hold time can be defined at the corresponding marker position with Holdoff argument.
        /// In the time domain (span = 0) the demodulation is permanently active.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_DEMOD_STATE
        /// </param>
        /// <param name="Demodulation_Type">
        /// Selects the demodulation type for the audio demodulator. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MARKER_DEMOD_AM (0) - AM
        /// RSSPECAN_VAL_MARKER_DEMOD_FM (1) - FM
        /// 
        /// Default Value: RSSPECAN_VAL_MARKER_DEMOD_AM (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_DEMOD_TYPE
        /// </param>
        /// <param name="Marker_Stop_Time">
        /// Defines the hold time at the marker position for the demodulation in the frequency domain (span &gt; 0). 
        /// 
        /// Valid Values:
        /// 10.0e-3 to 1000 s
        /// 
        /// Default Value: 10.0e-3 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_DEMOD_HOLDOFF
        /// </param>
        /// <param name="Continuous_Demodulation">
        /// This control switches on or off the continuous demodulation in the frequency domain (span &gt;0) in the selected measurement window. Thus acoustic monitoring of the signals can be performed in the frequency domain. The function does not depend on the selected marker.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_DEMOD_CONT
        /// </param>
        /// <param name="Squelch_State">
        /// This control enables or disables the squelch function for the audio output.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_DEMOD_SQUELCH_STATE
        /// 
        /// Note:
        /// 
        /// (1) This control has no effect for R&amp;S FSL or R&amp;S FSV.
        /// </param>
        /// <param name="Squelch_Level">
        /// Sets the trigger level for the squelch function.
        /// 
        /// Valid Values:
        /// 0.0 to 100.0 %
        /// 
        /// Default Value: 50.0 %
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_DEMOD_SQUELCH_LEVEL
        /// 
        /// Note(s):
        /// 
        /// (1) This control is valid only if Squelch State is set to On.
        /// 
        /// (2) This control has no effect for R&amp;S FSL or R&amp;S FSV.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMarkerDemodulation(int Window, int Marker, bool State, int Demodulation_Type, double Marker_Stop_Time, bool Continuous_Demodulation, bool Squelch_State, double Squelch_Level)
        {
            int pInvokeResult = PInvoke.ConfigureMarkerDemodulation(this._handle, Window, Marker, System.Convert.ToUInt16(State), Demodulation_Type, Marker_Stop_Time, System.Convert.ToUInt16(Continuous_Demodulation), System.Convert.ToUInt16(Squelch_State), Squelch_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function searches the selected trace for the indicated number of maxima. The number of maxima found depends on the waveform and value set for the Peak Excursion parameter, however, a maximum number of 50 maxima are determined. Only the signals which exceed their surrounding values at least by the value indicated by the peak excursion parameter will be recognized as maxima. Therefore, the number of maxima found is not automatically the same as the number of maxima desired.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_PEAK_LIST_COUNT
        /// RSSPECAN_ATTR_MARKER_PEAK_LIST_SORT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:FPEaks[:IMMediate]
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:FPEaks:SORT X | Y
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 1..16 for R&amp;S FSV
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Peak_List_Count">
        /// Sets the number of maxima to be found. 
        /// 
        /// Valid Values:
        /// 1 to 50
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_PEAK_LIST_COUNT
        /// </param>
        /// <param name="Peak_List_Sort">
        /// Sets the sort mode for the search for maxima. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MARKER_SORT_X (0) - X
        /// RSSPECAN_VAL_MARKER_SORT_Y (1) - Y
        /// 
        /// Default Value: RSSPECAN_VAL_MARKER_SORT_X (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_PEAK_LIST_SORT
        /// 
        /// Note(s):
        /// 
        /// (1) X - the maxima are sorted in the list of responses according to increasing X values
        /// 
        /// (2) Y - the maxima are sorted in the list of responses according to decreasing Y values
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMarkerPeakList(int Window, int Marker, int Peak_List_Count, int Peak_List_Sort)
        {
            int pInvokeResult = PInvoke.ConfigureMarkerPeakList(this._handle, Window, Marker, Peak_List_Count, Peak_List_Sort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function stores the current marker peak list in a *.dat file. The file consists of a data section containing the peak list.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_STORE_PEAK_LIST
        /// 
        /// Remote-control command(s):
        /// MMEMory:STORe&lt;1|2&gt;:PEAK
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Path">
        /// This control specifies the file path and name
        /// 
        /// Valid Values: not tested
        /// 
        /// Default Value: "D:\\user\\data\\test"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_STORE_PEAK_LIST
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int StoreMarkerPeakList(string Path)
        {
            int pInvokeResult = PInvoke.StoreMarkerPeakList(this._handle, Path);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the number of maxima found during the search. If no search for maxima has been performed, 0 is returned.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_PEAK_LIST_FOUND
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:FPEaks:COUNt?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 1..16 for R&amp;S FSV
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Peak_List_Found">
        /// This control returns the number of maxima found during the search. If no search for maxima has been performed, 0 is returned.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_PEAK_LIST_FOUND
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryMarkerPeakListFound(int Window, int Marker, out int Peak_List_Found)
        {
            int pInvokeResult = PInvoke.QueryMarkerPeakListFound(this._handle, Window, Marker, out Peak_List_Found);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the list of X or Y values of the maxima found.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:FPEaks:X?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:FPEaks:Y?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 1..16 for R&amp;S FSV
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the size of peak list array. It should correspond with requested peak list count.
        /// 
        /// Valid Values:
        /// &gt; 0
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Peak_List_Selection">
        /// Selects the peak list values.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MARKER_SORT_X (0) - X
        /// RSSPECAN_VAL_MARKER_SORT_Y (1) - Y
        /// 
        /// Default Value: RSSPECAN_VAL_MARKER_SORT_X (0)
        /// </param>
        /// <param name="Peak_List">
        /// This control returns the list of X values of the maxima found.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryMarkerPeakList(int Window, int Marker, int Array_Size, int Peak_List_Selection, double[] Peak_List)
        {
            int pInvokeResult = PInvoke.QueryMarkerPeakList(this._handle, Window, Marker, Array_Size, Peak_List_Selection, Peak_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function saves the data of active markers to a file,
        /// 
        /// The file names &lt;file_name&gt; are indicated as string parameters with the commands being enclosed in quotation marks. They also comply with DOS conventions. DOS file names consist of max. 8 ASCII characters and an extension of up to three characters
        /// separated from the file name by a dot "." Both, the dot and the extension are optional. The dot is not part of the file name. DOS file names do not distinguish between uppercase and lowercase notation. All letters and digits are permitted as well as the special characters "_", "^", "$", "~", "!", "#", "%", "&amp;", "-", "{", "}", "(", ")", "@" and "`". Reserved file names are CLOCK$, CON, AUX, COM1 to COM4, LPT1 to LPT3,
        /// NUL and PRN. The two characters "*" and "?" have the function of so-called "wildcards", i.e., they are variables for
        /// selection of several files. The question mark "?" replaces exactly one character, the asterisk means any
        /// of the remaining characters in the file name. "*.*" thus means all files in a directory.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_FILE_EXPORT
        /// 
        /// Remote-control command(s):
        /// MMEMory:STORe&lt;1|2&gt;:MARKer
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Path">
        /// This control specifies the file path and name
        /// 
        /// Valid Values: not tested
        /// 
        /// Default Value: "D:\\user\\data\\marker.txt"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_FILE_EXPORT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int MarkerSaveToFile(string Path)
        {
            int pInvokeResult = PInvoke.MarkerSaveToFile(this._handle, Path);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function enables the active marker on the specified trace.
        /// 
        /// Note(s):
        /// 
        /// (1) Only marker state is available for R&amp;S FSH4/8.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_REFERENCE_MARKER_STATE
        /// RSSPECAN_ATTR_REFERENCE_MARKER_TRACE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;[:STATe] ON | OFF
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:TRACe
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Delta_Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 1..6  for R&amp;S FSH4/8
        /// 1..16 for R&amp;S FSV
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// Specifies whether the specified marker is enabled.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_MARKER_STATE
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition. 
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 1 to 6 for R&amp;S FSL, R&amp;S FSV
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_MARKER_TRACE
        /// 
        /// Note(s):
        /// 
        /// (1) This control is not available for R&amp;S FSH4/8 instrument.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDeltaMarker(int Window, int Delta_Marker, bool State, int Trace)
        {
            int pInvokeResult = PInvoke.ConfigureDeltaMarker(this._handle, Window, Delta_Marker, System.Convert.ToUInt16(State), Trace);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the delta marker position.
        /// 
        /// Note(s):
        /// 
        /// (1) Only position is available for R&amp;S FSH4/8.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_REFERENCE_MARKER_MODE
        /// RSSPECAN_ATTR_REFERENCE_MARKER_POSITION
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:MODE ABSolute | RELative
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:X
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Delta_Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 1..6  for R&amp;S FSH4/8
        /// 1..16 for R&amp;S FSV
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Mode">
        /// Switches between relative and absolute frequency input of the delta marker (or time with span = 0). It affects all delta markers independent of the measurement window.. 
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_ABS (0) - Absolute
        /// RSSPECAN_VAL_REL (1) - Relative 
        /// 
        /// Default Value: RSSPECAN_VAL_ABS (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_MARKER_MODE
        /// 
        /// Note(s):
        /// 
        /// (1) This control is not available for R&amp;S FSH4/8 instrument.
        /// </param>
        /// <param name="Position">
        /// Sets the position of selected delta marker in the indicated measurement window. 
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_MARKER_POSITION
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDeltaMarkerPosition(int Window, int Delta_Marker, int Mode, double Position)
        {
            int pInvokeResult = PInvoke.ConfigureDeltaMarkerPosition(this._handle, Window, Delta_Marker, Mode, Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function turns off all the delta markers in selected measurement window.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_REFERENCE_MARKER_AOFF
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:AOFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int DisableAllDeltaMarkers(int Window)
        {
            int pInvokeResult = PInvoke.DisableAllDeltaMarkers(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the type of delta marker search and performs the search.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_REFERENCE_MARKER_PEAK
        /// RSSPECAN_ATTR_REFERENCE_MARKER_PEAK_NEXT
        /// RSSPECAN_ATTR_REFERENCE_MARKER_PEAK_NEXT_RIGHT
        /// RSSPECAN_ATTR_REFERENCE_MARKER_PEAK_NEXT_LEFT
        /// RSSPECAN_ATTR_REFERENCE_MARKER_MIN
        /// RSSPECAN_ATTR_REFERENCE_MARKER_MIN_NEXT
        /// RSSPECAN_ATTR_REFERENCE_MARKER_MIN_NEXT_RIGHT
        /// RSSPECAN_ATTR_REFERENCE_MARKER_MIN_NEXT_LEFT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:MAXimum[:PEAK]
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:MAXimum:NEXT
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:MAXimum:RIGHt
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:MAXimum:LEFT
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:MINimum[:PEAK]
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:MINimum:NEXT
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:MINimum:RIGHt
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:MINimum:LEFT
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Delta_Marker">
        /// This control selects delta marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 1..6  for R&amp;S FSH4/8
        /// 1..16 for R&amp;S FSV
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker_Search">
        /// Specifies the type of marker search type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MARKER_SEARCH_HIGHEST            (1) - Highest
        /// RSSPECAN_VAL_MARKER_SEARCH_MINIMUM            (2) - Minimum
        /// RSSPECAN_VAL_MARKER_SEARCH_NEXT_PEAK          (3) - Next Peak
        /// RSSPECAN_VAL_MARKER_SEARCH_NEXT_PEAK_LEFT     (4) - Next Peak Left
        /// RSSPECAN_VAL_MARKER_SEARCH_NEXT_PEAK_RIGHT    (5) - Next Peak Right
        /// RSSPECAN_VAL_MARKER_SEARCH_NEXT_MINIMUM       (6) - Next Minimum
        /// RSSPECAN_VAL_MARKER_SEARCH_NEXT_MINIMUM_LEFT  (7) - Next Minimum Left
        /// RSSPECAN_VAL_MARKER_SEARCH_NEXT_MINIMUM_RIGHT (8) - Next Minimum Right
        /// 
        /// Default Value:  RSSPECAN_VAL_MARKER_SEARCH_HIGHEST
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_MARKER_PEAK
        /// RSSPECAN_ATTR_REFERENCE_MARKER_PEAK_NEXT
        /// RSSPECAN_ATTR_REFERENCE_MARKER_PEAK_NEXT_RIGHT
        /// RSSPECAN_ATTR_REFERENCE_MARKER_PEAK_NEXT_LEFT
        /// RSSPECAN_ATTR_REFERENCE_MARKER_MIN
        /// RSSPECAN_ATTR_REFERENCE_MARKER_MIN_NEXT
        /// RSSPECAN_ATTR_REFERENCE_MARKER_MIN_NEXT_RIGHT
        /// RSSPECAN_ATTR_REFERENCE_MARKER_MIN_NEXT_LEFT
        /// 
        /// Note(s):
        /// 
        /// (1) Highest - Sets marker search for the highest amplitude
        /// 
        /// (2) Next Peak - Sets marker search for the next highest peak.
        /// 
        /// (3) Next Peak Left - Sets marker search for the next peak left of the peak amplitude. Not available for R&amp;S FSH4/8 instrument.
        /// 
        /// (4) Next Peak Right - Sets marker search for the next peak right of the peak amplitude. Not available for R&amp;S FSH4/8 instrument.
        /// 
        /// (5) Minimum - Sets marker search for the minimum amplitude.
        /// 
        /// (6) Next Minimum - Sets marker search for the next lowest minimum. Not available for R&amp;S FSH4/8 instrument.
        /// 
        /// (7) Next Minimum Left - Sets marker search for the next minimum left of the minimum amplitude. Not available for R&amp;S FSH4/8 instrument.
        /// 
        /// (8) Next Minimum Right - Sets marker search for the next minimum right of the minimum amplitude. Not available for R&amp;S FSH4/8 instrument.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int DeltaMarkerSearch(int Window, int Delta_Marker, int Marker_Search)
        {
            int pInvokeResult = PInvoke.DeltaMarkerSearch(this._handle, Window, Delta_Marker, Marker_Search);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the horizontal position and the marker amplitude level of the selected delta marker.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_REFERENCE_MARKER_POSITION
        /// RSSPECAN_ATTR_REFERENCE_MARKER_REL_POSITION
        /// RSSPECAN_ATTR_REFERENCE_MARKER_AMPLITUDE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:X?
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:X:RELative?
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:Y?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Delta_Marker">
        /// This control selects delta marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 1..6  for R&amp;S FSH4/8
        /// 1..16 for R&amp;S FSV
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Mode">
        /// Switches between absolute and relative position value.
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_ABS (0) - Absolute
        /// RSSPECAN_VAL_REL (1) - Relative 
        /// 
        /// Default Value: RSSPECAN_VAL_ABS (0)
        /// </param>
        /// <param name="Position">
        /// Returns the absolute or relative position of selected delta marker.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_MARKER_POSITION
        /// RSSPECAN_ATTR_REFERENCE_MARKER_REL_POSITION
        /// </param>
        /// <param name="Amplitude">
        /// Returns the measured value of the selected delta marker in the indicated measurement window. The corresponding delta marker will be activated, if necessary. The output is always a relative value referred to marker 1 or to the reference position (reference fixed active). 
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_MARKER_AMPLITUDE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryDeltaMarker(int Window, int Delta_Marker, int Mode, out double Position, out double Amplitude)
        {
            int pInvokeResult = PInvoke.QueryDeltaMarker(this._handle, Window, Delta_Marker, Mode, out Position, out Amplitude);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures marker reference fixed state.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_REF_STATE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1to4&gt;:FUNCtion:FIXed[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Fixed_Reference">
        /// This control switches the relative measurement to a fixed reference value on or off. Marker 1 will be activated previously and a peak search will be performed, if necessary. If marker 1 is activated, its position becomes the reference point for the measurement.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_REF_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReferenceFixed(int Window, bool Fixed_Reference)
        {
            int pInvokeResult = PInvoke.ConfigureReferenceFixed(this._handle, Window, System.Convert.ToUInt16(Fixed_Reference));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures marker reference fixed position.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_REF_LEVEL
        /// RSSPECAN_ATTR_MARKER_REF_LEVEL_OFFSET
        /// RSSPECAN_ATTR_MARKER_REF_FREQ
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1to4&gt;:FUNCtion:FIXed:RPOint:Y
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1to4&gt;:FUNCtion:FIXed:RPOint:Y:OFFSet
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1to4&gt;:FUNCtion:FIXed:RPOint:X
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Ref_Point_Level">
        /// Defines a new reference point level for delta marker 2 in the selected measurement window.
        /// 
        /// Valid Values:
        /// -200.0 dB to 200.0 dB
        /// 
        /// Default Value: 0.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_REF_LEVEL
        /// 
        /// Note(s):
        /// 
        /// (1) This control is valid only if Fixed Reference is set to On.
        /// </param>
        /// <param name="Ref_Point_Level_Offset">
        /// Defines an additional level offset which is included in the display of delta marker 2 in the selected measurement window.
        /// 
        /// Valid Values:
        /// -200.0 dB to 200.0 dB
        /// 
        /// Default Value: 0.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_REF_LEVEL_OFFSET
        /// 
        /// Note(s):
        /// 
        /// (1) This control is valid only if Fixed Reference is set to On.
        /// </param>
        /// <param name="Ref_Point_Frequency_Time">
        /// Defines a new reference frequency or time for delta marker 2 in the selected measurement window.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 0.0 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_REF_FREQ
        /// 
        /// Note(s):
        /// 
        /// (1) This control is valid only if Fixed Reference is set to On.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReferenceFixedPoint(int Window, double Ref_Point_Level, double Ref_Point_Level_Offset, double Ref_Point_Frequency_Time)
        {
            int pInvokeResult = PInvoke.ConfigureReferenceFixedPoint(this._handle, Window, Ref_Point_Level, Ref_Point_Level_Offset, Ref_Point_Frequency_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the reference point level for all delta markers in the selected measurement window for a measurement with fixed reference point (CALC:DELT:FUNC:FIX:STAT ON) to the peak of the selected trace.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_REF_PEAK
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:FUNCtion:FIXed:RPOint:MAXimum[:PEAK]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReferenceFixedPeakSearch(int Window)
        {
            int pInvokeResult = PInvoke.ReferenceFixedPeakSearch(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the reference oscillator.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_ROSC_SOURCE
        /// RSSPECAN_ATTR_ROSC_FREQUENCY
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ROSCillator:SOURce INTernal | EXTernal | EAUTo
        /// [SENSe&lt;1|2&gt;:]ROSCillator:EXTernal:FREQuency
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Source">
        /// Sets the reference oscillator. If the external reference oscillator is selected, the reference signal must be connected to the rear panel of the instrument.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SOUR_INT  (0) - Internal
        /// RSSPECAN_VAL_SOUR_EXT  (1) - External
        /// RSSPECAN_VAL_SOUR_EAUT (2) - External Auto
        /// 
        /// Default Value: RSSPECAN_VAL_SOUR_INT (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ROSC_SOURCE
        /// 
        /// Note(s):
        /// 
        /// (1) Internal: The internal reference signal is used.
        /// 
        /// (2) External: The external reference signal is used.
        /// 
        /// (3) External Auto: The external reference signal is used, but the instrument automatically switches back. This value is not valid for R&amp;S FSH4, FSH8 instruments.
        /// </param>
        /// <param name="External_Frequency">
        /// Informs the instrument on the frequency of the external reference oscillator. This frequency is used as starting point for the synchronization of the internal reference frequencies.
        /// 
        /// Units: Hz
        /// 
        /// Valid Values: 1.0e+6 to 20.0e+6
        /// 
        /// Default Value: 1.0e+6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ROSC_FREQUENCY
        /// 
        /// Note(s):
        /// 
        /// (1) This control has effect only for source External.
        /// 
        /// (2) This control is not valid for R&amp;S FSH4, FSH8 instruments.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReferenceOscillator(int Source, double External_Frequency)
        {
            int pInvokeResult = PInvoke.ConfigureReferenceOscillator(this._handle, Source, External_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the current selection of the reference signal (internal / external) if the external reference source EXT [INT] is used. The function is only available, if SENS:ROSC:SOUR EAUT is selected. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_ROSC_SOURCE_EAUTO
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ROSCillator:SOURce:EAUTo?
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Source">
        /// Returns the source of the reference oscillator.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_SOUR_INT (0) - Internal
        /// RSSPECAN_VAL_SOUR_EXT (1) - External
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ROSC_SOURCE_EAUTO
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QuerySelectedReferenceOscillatorSource(out int Source)
        {
            int pInvokeResult = PInvoke.QuerySelectedReferenceOscillatorSource(this._handle, out Source);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function controls the PLL bandwidth used to synchronize with the external reference signal. It is only available if the external reference frequency is set to 10 MHz. The reference settings are not changed if a preset occurs to maintain the specific setup of a test system.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_ROSC_PLL_BWID
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ROSCillator:EXTernal:PLL NORMal | WIDE
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Bandwidth">
        /// Controls the PLL bandwidth used to synchronize with the external reference signal. It is only available if the external reference frequency is set to 10 MHz. The reference settings are not changed if a preset occurs to maintain the specific setup of a test system.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_PLL_BWID_NORM (0) - Normal
        /// RSSPECAN_VAL_PLL_BWID_WIDE (1) - Wide
        /// 
        /// Default Value: RSSPECAN_VAL_PLL_BWID_NORM (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ROSC_PLL_BWID
        /// 
        /// Note(s):
        /// 
        /// (1) Normal: The instrument uses a PLL bandwidth of about 3 Hz
        /// 
        /// (2) Wide: The instrument uses a PLL bandwidth of about 30 Hz.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReferenceOscillatorExternalPLLBandwidth(int Bandwidth)
        {
            int pInvokeResult = PInvoke.ConfigureReferenceOscillatorExternalPLLBandwidth(this._handle, Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the volume of the built-in loudspeaker for demodulated signals.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SYST_SPEAKER
        /// RSSPECAN_ATTR_SYST_SPEAKER_PORTABLE
        /// 
        /// Remote-control command(s):
        /// SYSTem:SPEaker:VOLume
        /// SYSTem:AUDio:VOLume
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Speaker_Volume">
        /// Sets the volume of the built-in loudspeaker for demodulated signals.
        /// 
        /// Units: %
        /// 
        /// Valid Values:
        /// 0.0 to 100.0
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SYST_SPEAKER
        /// RSSPECAN_ATTR_SYST_SPEAKER_PORTABLE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SpeakerVolume(double Speaker_Volume)
        {
            int pInvokeResult = PInvoke.SpeakerVolume(this._handle, Speaker_Volume);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the volume of the beeper, activated with
        /// CALCulate:LIMit&lt;1|2&gt;:BEEP[:STATe]. The unit is percent.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BEEPER_VOLUME
        /// 
        /// Remote-control command(s):
        /// SYSTem:BEEPer:VOLume
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Beeper_Volume">
        /// This control sets the volume of the beeper. The unit is percent.
        /// 
        /// Valid Values:
        /// 0.00 to 1.00
        /// 
        /// Default Value:
        /// 0.60
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BEEPER_VOLUME
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureBeeperVolume(double Beeper_Volume)
        {
            int pInvokeResult = PInvoke.ConfigureBeeperVolume(this._handle, Beeper_Volume);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the volume of the keyboard clicking. The unit is percent.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_KEY_VOLUME
        /// 
        /// Remote-control command(s):
        /// SYSTem:BEEPer:KEY:VOLume
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Key_Volume">
        /// This control sets the volume of the keyboard clicking. The unit is percent.
        /// 
        /// Valid Values:
        /// 0.00 to 1.00
        /// 
        /// Default Value:
        /// 0.30
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_KEY_VOLUME
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureKeyVolume(double Key_Volume)
        {
            int pInvokeResult = PInvoke.ConfigureKeyVolume(this._handle, Key_Volume);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the function the BNC socket indicated by the numeric suffix is used for.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BNC_MODE
        /// 
        /// Remote-control command(s):
        /// SYSTem:BNC&lt;1...2&gt;:MODE REFerence | TRIGger | BIAS | IF3 | IFDac
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="BNC">
        /// This control sets the BNC connector.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="BNC_Mode">
        /// This control sets the BNC socket indicated by the numeric suffix is used for.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_BNC_MODE_REF  (0) - Reference
        /// RSSPECAN_VAL_BNC_MODE_TRIG (1) - Trigger
        /// RSSPECAN_VAL_BNC_MODE_BIAS (2) - Bias
        /// RSSPECAN_VAL_BNC_MODE_IF3  (3) - IF 3
        /// RSSPECAN_VAL_BNC_MODE_IFD  (4) - IF DAC
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_BNC_MODE_TRIG (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BNC_MODE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureBNCMode(int BNC, int BNC_Mode)
        {
            int pInvokeResult = PInvoke.ConfigureBNCMode(this._handle, BNC, BNC_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the matching pad connected to the R&amp;S FSH.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_INPUT_IMPEDANCE_PAD
        /// 
        /// Remote-control command(s):
        /// INPut:IMPedance:PAD RAM | RAZ | HZTE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Pad">
        /// This control sets the matching pad connected to the R&amp;S FSH.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_IMPEDANCE_PAD_RAM  (0) - RAM
        /// RSSPECAN_VAL_IMPEDANCE_PAD_RAZ  (1) - RAZ
        /// RSSPECAN_VAL_IMPEDANCE_PAD_HZTE (2) - HZTE
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_IMPEDANCE_PAD_RAM  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_INPUT_IMPEDANCE_PAD
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureInputImpedancePad(int Pad)
        {
            int pInvokeResult = PInvoke.ConfigureInputImpedancePad(this._handle, Pad);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function controls the instrument behaviour when changing the active application, e.g from SPECTRUM to FM DEMOD and back from FM DEMOD to SPECTRUM. In the default state OFF a few
        /// parameters of the current analyzer setting are passed to the application (e.g. center frequency, level settings) or from the application back to the analyzer mode. If APPL SETUP RECOVERY is switched ON, the settings of the applications are independent of each other. Leaving the FM DEMOD application will restore the previous state of the ANALYZER.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_APPL_SETUP_REC
        /// 
        /// Remote-control command(s):
        /// SYSTem:APPLication:SRECovery[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Application_Setup_Recovery">
        /// This control sets the instrument behaviour when changing the active application.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_APPL_SETUP_REC
        /// 
        /// Note(s):
        /// 
        /// (1) Off: A few parameters of the current analyzer setting are passed to the application (e.g. center frequency, level settings) or from the application back to the analyzer mode.
        /// 
        /// (2) On: The settings of the applications are independent of each other. Leaving the application will restore the previous state of the analyzer.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ApplicationSetupRecovery(bool Application_Setup_Recovery)
        {
            int pInvokeResult = PInvoke.ApplicationSetupRecovery(this._handle, System.Convert.ToUInt16(Application_Setup_Recovery));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the shifting of the 1st IF.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_IF_SHIFT
        /// RSSPECAN_ATTR_IF_SHIFT_FREQ
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:IF:SHIFt[:MODE] OFF | A | B | AUTO
        /// [SENSe&lt;1|2&gt;:]SWEep:IF:SHIFt:FREQuency
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="IF_Shift">
        /// This control selects the shifting of the 1st IF. Input signals at a frequency of half the 1st IF (in the frequency range of 2270 MHz to 2350 MHz) will reduce the dynamic range of the analyzer. This problem only occurs for low RF attenuation values. It can be overcome by shifting the 1st IF. The 1st IF shifting is automatically done for ACP measurements if center frequency ( = signal frequency) is in the range of 2270 MHz to 2350 MHz. The IF SHIFT setting is therefore ignored for ACP measurements.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_IF_SHIFT_OFF  (0) - Off
        /// RSSPECAN_VAL_IF_SHIFT_A    (1) - A
        /// RSSPECAN_VAL_IF_SHIFT_B    (2) - B
        /// RSSPECAN_VAL_IF_SHIFT_AUTO (3) - Auto
        /// 
        /// Default Value: RSSPECAN_VAL_IF_SHIFT_OFF (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_IF_SHIFT
        /// 
        /// Note(s):
        /// 
        /// (1) Off: Normal operation.
        /// 
        /// (2) A: For signals from 2270MHz to 2310 MHz.
        /// 
        /// (3) B: For signals from 2310 MHz to 2350 MHz.
        /// 
        /// (4) Auto: Automatically selects the suitable 1st IF shifting.
        /// The signal frequency has to specified with control IF Shift Frequency.
        /// 
        /// </param>
        /// <param name="IF_Shift_Frequency">
        /// This control defines the signal frequency to automatically select the suitable 1st IF shifting for mode IF SHIFT AUTO (SENS:SWE:IF:SHIFT:MODE AUTO).
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 1.0e9 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_IF_SHIFT_FREQ
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available if IF Shift is set AUTO.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureIFShift(int IF_Shift, double IF_Shift_Frequency)
        {
            int pInvokeResult = PInvoke.ConfigureIFShift(this._handle, IF_Shift, IF_Shift_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the time when R&amp;S FSH automatically turns off when not operating (in battery mode).
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AUTO_OFF_TIME
        /// 
        /// Remote-control command(s):
        /// SYSTem:POWer:AUTO:OFF:TIME?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Auto_Off_Time">
        /// This control displays the time when R&amp;S FSH automatically turns off when not operating (in battery mode).
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AUTO_OFF_TIME
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rssmu_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int QueryAutoOffTime(out int Auto_Off_Time)
        {
            int pInvokeResult = PInvoke.QueryAutoOffTime(this._handle, out Auto_Off_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries whether R&amp;S FSH is battery-powered or line-powered.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_POWER_SOURCE
        /// 
        /// Remote-control command(s):
        /// SYSTem:POWer:SOURce?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Power_Source">
        /// This control displays whether R&amp;S FSH is battery-powered or line-powered.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_POWER_SOURCE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rssmu_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int QueryPowerSource(System.Text.StringBuilder Power_Source)
        {
            int pInvokeResult = PInvoke.QueryPowerSource(this._handle, Power_Source);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the trigger source that causes the spectrum analyzer to leave the Wait-for-Trigger state.  
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TRIGGER_SOURCE
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:SOURce IMMediate | EXTernal | VIDeo | IFPower | TIMe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trigger_Source">
        /// Specifies the source of the trigger signal that causes the analyzer to leave the Wait-For-Trigger state.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TRG_IMM  (0) - Immediate 
        /// RSSPECAN_VAL_TRG_EXT  (1) - External   
        /// RSSPECAN_VAL_TRG_IFP  (2) - IF Power
        /// RSSPECAN_VAL_TRG_VID  (3) - Video
        /// RSSPECAN_VAL_TRG_TIME (4) - Time
        /// 
        /// Default Value: RSSPECAN_VAL_TRG_IMM (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_SOURCE
        /// 
        /// Note(s):
        /// 
        /// (1) Valid values differs for each instrument and operating mode. Please check operating manual for valid values.
        /// 
        /// (2) Immediate - automatic triggering the next measurement at the end of the previous one. The value IMMediate corresponds to the FREE RUN setting.
        /// 
        /// (3) External - the next measurement is triggered by the signal at the external trigger input.
        /// 
        /// (4) Video - the next measurement is triggered by the detection of a signal at the video filter output (not available for FM DEMOD mode).
        /// 
        /// (5) IF Power - the next measurement is triggered by the detection of a signal at the instrument IF (10 MHz bandwidth)@
        /// 
        /// (6) Time - the next measurement is triggered by the time. This option is valid only for R&amp;S FSL since FW 1.60 or R&amp;S FSV.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTriggerSource(int Window, int Trigger_Source)
        {
            int pInvokeResult = PInvoke.ConfigureTriggerSource(this._handle, Window, Trigger_Source);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the trigger source that causes the spectrum analyzer to leave the Wait-for-Trigger state.  
        /// 
        /// Note:
        /// (1) This function is available only with FSP-B6 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TRIGGER_SOURCE_TVRF
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:SOURce RFPower | TV
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trigger_Source">
        /// Specifies the source of the trigger signal that causes the analyzer to leave the Wait-For-Trigger state.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TRG_B6_RFP (0) - RF Power
        /// RSSPECAN_VAL_TRG_B6_TV  (1) - TV
        /// 
        /// 
        /// Default value: RSSPECAN_VAL_TRG_B6_RFP (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_SOURCE_TVRF
        /// 
        /// Note:
        /// 
        /// (1) RSSPECAN_VAL_TRG_B6_RFP is not available for R&amp;S FSL
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTVRFPTriggerSource(int Window, int Trigger_Source)
        {
            int pInvokeResult = PInvoke.ConfigureTVRFPTriggerSource(this._handle, Window, Trigger_Source);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the delay and polarity for triggering.  
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TRIGGER_DELAY
        /// RSSPECAN_ATTR_TRIGGER_SLOPE
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:HOLDoff
        /// TRIGger&lt;1|2&gt;[:SEQuence]:SLOPe POSitive | NEGative
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trigger_Delay">
        /// Sets the length of the trigger delay.
        /// 
        /// Units: Volts
        /// 
        /// Valid Values: -100.0 to 100.0
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_DELAY
        /// 
        /// Note:
        /// (1) A negative delay time (pretrigger) can be set in the time domain (span &lt; 0 Hz) only.
        /// 
        /// </param>
        /// <param name="Trigger_Polarity">
        /// Specifies the polarity for triggering.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_NEG (0) - Negative
        /// RSSPECAN_VAL_POS (1) - Positive
        /// 
        /// Default value: RSSPECAN_VAL_POS (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_SLOPE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTrigger(int Window, double Trigger_Delay, int Trigger_Polarity)
        {
            int pInvokeResult = PInvoke.ConfigureTrigger(this._handle, Window, Trigger_Delay, Trigger_Polarity);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the external trigger level. This is applicable when the trigger source is set to external.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_EXTERNAL_TRIGGER_LEVEL
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel[:EXTernal]
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="External_Trigger_Level">
        /// Sets the external level for triggering. 
        /// 
        /// Units: Volts
        /// 
        /// Valid Values: 0.5 to 3.5
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_TRIGGER_LEVEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureExternalTrigger(int Window, double External_Trigger_Level)
        {
            int pInvokeResult = PInvoke.ConfigureExternalTrigger(this._handle, Window, External_Trigger_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the external trigger delay compensation. This is applicable when the trigger source is set to external.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_EXTERNAL_TRIGGER_DELAY_COMPENSATION
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:HOLDoff:ADJust:AUTO ON|OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Delay_Compensation">
        /// Defines whether the group delay of the bandwidth filters for the external trigger is compensated.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_TRIGGER_DELAY_COMPENSATION
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDelayCompensation(int Window, bool Delay_Compensation)
        {
            int pInvokeResult = PInvoke.ConfigureDelayCompensation(this._handle, Window, System.Convert.ToUInt16(Delay_Compensation));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the video level for triggering.  This is applicable when the trigger source is set to video.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VIDEO_TRIGGER_LEVEL
        /// 
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:VIDeo
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Video_Trigger_Level">
        /// Sets the videp level for triggering.
        /// 
        /// Units: %
        /// 
        /// Valid Values: 0.0 to 100.0
        /// 
        /// Default Value: 50.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VIDEO_TRIGGER_LEVEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVideoTrigger(int Window, double Video_Trigger_Level)
        {
            int pInvokeResult = PInvoke.ConfigureVideoTrigger(this._handle, Window, Video_Trigger_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the level of the IF power trigger source. This is applicable when the trigger source is set to IF Power.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TRIGGER_IFP_LEVEL
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:IFPower
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trigger_Level">
        /// Sets the level for triggering.
        /// 
        /// Units: dBm
        /// 
        /// Valid Values:
        /// -30.0 to -10.0
        /// -70.0 to +30.0 for Trace IQ subsystem extension
        /// 
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_IFP_LEVEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureIFPowerTrigger(int Window, double Trigger_Level)
        {
            int pInvokeResult = PInvoke.ConfigureIFPowerTrigger(this._handle, Window, Trigger_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the IF power trigger parameters. This is applicable when the trigger source is set to IF Power.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is valit for R&amp;S FSL, R&amp;S FSV, R&amp;S FSP, R&amp;S FSQ instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TRIGGER_IFP_LEVEL
        /// RSSPECAN_ATTR_TRIGGER_IFP_OFFSET
        /// RSSPECAN_ATTR_TRIGGER_IFP_HYSTERESIS
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:IFPower
        /// TRIGger&lt;1|2&gt;[:SEQuence]:IFPower:HOLDoff
        /// TRIGger&lt;1|2&gt;[:SEQuence]:IFPower:HYSTeresis
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trigger_Level">
        /// Sets the level for triggering.
        /// 
        /// Units: dBm
        /// 
        /// Valid Values: -30.0 to -10.0
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_IFP_LEVEL
        /// </param>
        /// <param name="Holdoff">
        /// This command sets the holding time before the next IF power trigger event. This command is available for R&amp;S FSL from firmware version 1.30.
        /// 
        /// Valid Values: 
        ///  150.0e-9 to 1000.0 s
        /// 
        /// Default Value: 150.0e-9
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_IFP_OFFSET
        /// </param>
        /// <param name="Hysteresis">
        /// This command sets the limit that the hysteresis value has to fall below in order to trigger the next measurement.
        /// 
        /// Valid Values: 
        ///  3.0 to 50.0 dB
        /// 
        /// Default Value: 3.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_IFP_HYSTERESIS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureIFPowerTriggerParameters(int Window, double Trigger_Level, double Holdoff, double Hysteresis)
        {
            int pInvokeResult = PInvoke.ConfigureIFPowerTriggerParameters(this._handle, Window, Trigger_Level, Holdoff, Hysteresis);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies sets the level of the RF power trigger source. This is applicable when the trigger source is set to RF Power.
        /// 
        /// Note:
        /// This function is available only with FSP-B6 option
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TRIGGER_RFP_LEVEL
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:RFPower
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trigger_Level">
        /// Sets the level for triggering.
        /// 
        /// Units: dBm
        /// 
        /// Valid Values: -50.0 to -10.0
        /// 
        /// Default Value: -20.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_RFP_LEVEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureRFPowerTrigger(int Window, double Trigger_Level)
        {
            int pInvokeResult = PInvoke.ConfigureRFPowerTrigger(this._handle, Window, Trigger_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the time trigger. This is applicable when the trigger source is set to time.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TRIGGER_TIME_INTERVAL
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:TIME:RINTerval
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Time_Interval">
        /// This control sets the time interval for the time trigger source.
        /// 
        /// Units: s
        /// 
        /// Valid Values: 
        ///  0.1 to 5000
        /// 
        /// Default Value: 1.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_TIME_INTERVAL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTimeTrigger(int Window, double Time_Interval)
        {
            int pInvokeResult = PInvoke.ConfigureTimeTrigger(this._handle, Window, Time_Interval);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the TV Trigger parameters. This is applicable when the trigger source is set to TV.
        /// 
        /// 
        /// Note:
        /// (1) This function is available only with R&amp;S FSL with option B6 or K10.
        /// 
        /// (2) Option TV Trigger, B6, is available from firmware version 1.10.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TRIGGER_TV_LINE_SYSTEM
        /// RSSPECAN_ATTR_TRIGGER_TV_VERTICAL_SIGNAL
        /// RSSPECAN_ATTR_TRIGGER_TV_HORIZONTAL_SIGNAL
        /// RSSPECAN_ATTR_TRIGGER_TV_VIDEO_POLARITY
        /// RSSPECAN_ATTR_TRIGGER_TV_VID_CONT
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:VIDeo:FORMat:LPFRame 525 | 625
        /// TRIGger&lt;1|2&gt;[:SEQuence]:VIDeo:FIELd:SELect ALL | ODD | EVEN
        /// TRIGger&lt;1|2&gt;[:SEQuence]:VIDeo:LINE:NUMBer
        /// TRIGger&lt;1|2&gt;[:SEQuence]:VIDeo:SSIGnal:POLarity
        /// TRIGger&lt;1|2&gt;[:SEQuence]:VIDeo:CONTinuous
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Line_System">
        /// This control sets the line system to be used.
        /// 
        /// Valid Values:
        /// 525, 625
        /// 
        /// Default Value: 625
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_TV_LINE_SYSTEM
        /// </param>
        /// <param name="Synchronization">
        /// Sets the trigger synchronization.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TRIG_TV_VFIEL_ALL  (0) - Vertical Sync
        /// RSSPECAN_VAL_TRIG_TV_VFIEL_ODD  (1) - Vertical Sync Odd Field
        /// RSSPECAN_VAL_TRIG_TV_VFIEL_EVEN (2) - Vertical Sync Even Field
        /// RSSPECAN_VAL_TRIG_TV_HORIZONTAL (3) - Horizontal Sync
        /// 
        /// Default Value: RSSPECAN_VAL_TRIG_TV_VFIEL_ALL (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_TV_VERTICAL_SIGNAL
        /// RSSPECAN_ATTR_TRIGGER_TV_HORIZONTAL_SIGNAL
        /// 
        /// Note(s):
        /// 
        /// (1) Vertical Sync - triggers on the frame repetition signal
        /// without distinction between the two fields.
        /// 
        /// (2) Vertical Sync Odd Field- sets the trigger on the vertical sync signal of the first field.
        /// 
        /// (3) Vertical Sync Even Field- sets the trigger on the vertical sync signal of the second field.
        /// 
        /// (4) Horizontal Sync - activates triggering at the horizontal sync signal of the indicated line number.
        /// </param>
        /// <param name="Horizontal_Sync_Line">
        /// This command on the horizontal sync signal and sets the corresponding line.
        /// 
        /// Valid Values: 
        ///  1 to 525 (for line system 525)
        ///  1 to 625 (for line system 625)
        /// 
        /// Default Value: 17
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_TV_HORIZONTAL_SIGNAL
        /// 
        /// Note:
        /// 
        /// (1) This control has effect only for Synchronization sets to Horizontal Sync
        /// </param>
        /// <param name="Polarity">
        /// This control selects the slope of the trigger signal only when trigger source is selected.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_NEG (0) - Negative 
        /// RSSPECAN_VAL_POS (1) - Positive 
        /// 
        /// Default Value: RSSPECAN_VAL_NEG (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_TV_VIDEO_POLARITY
        /// </param>
        /// <param name="TV_Free_Run_State">
        /// This control activates or deactivates the free run trigger mode. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_TV_VID_CONT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTVTrigger(int Window, int Line_System, int Synchronization, int Horizontal_Sync_Line, int Polarity, bool TV_Free_Run_State)
        {
            int pInvokeResult = PInvoke.ConfigureTVTrigger(this._handle, Window, Line_System, Synchronization, Horizontal_Sync_Line, Polarity, System.Convert.ToUInt16(TV_Free_Run_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the external gate parameters. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_EXTERNAL_GATE
        /// RSSPECAN_ATTR_EXTERNAL_GATE_TRIGGER_TYPE
        /// RSSPECAN_ATTR_EXTERNAL_GATE_HOLD
        /// RSSPECAN_ATTR_EXTERNAL_GATE_LENGTH
        /// RSSPECAN_ATTR_EXTERNAL_GATE_POLARITY
        /// RSSPECAN_ATTR_EXTERNAL_GATE_SIGNAL_SOURCE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe ON | OFF
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:TYPE LEVel | EDGE
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:HOLDoff
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:LENGth
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:POLarity POSitive | NEGative
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:SOURce EXTernal | IFPower | VIDeo
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Gating">
        /// This parameter switches on/off the gating.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_GATE
        /// </param>
        /// <param name="Gate_Source">
        /// This control toggles between external gate signal and RF power signal.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_EGAT_SOUR_IFP (0) - IF Power
        /// RSSPECAN_VAL_EGAT_SOUR_EXT (1) - External
        /// RSSPECAN_VAL_EGAT_SOUR_VID (2) - Video
        /// 
        /// Default Value: RSSPECAN_VAL_EGAT_SOUR_IFP (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_GATE_SIGNAL_SOURCE
        /// 
        /// Note:
        /// (1) This parameter has effect only for Gating set to On
        /// 
        /// (2) Video is only available on R&amp;S FSL or R&amp;S ESL or R&amp;S FSV.
        /// </param>
        /// <param name="Mode">
        /// Sets sets the type of triggering (level or edge) by the external gate signal. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_EGAT_TRIG_LEV  (0) - Level
        /// RSSPECAN_VAL_EGAT_TRIG_EDGE (1) - Edge
        /// 
        /// Default Value: RSSPECAN_VAL_EGAT_TRIG_EDGE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_GATE_TRIGGER_TYPE
        /// 
        /// Note:
        /// (1) This parameter has effect only for Gating set to On
        /// 
        /// 
        /// 
        /// </param>
        /// <param name="Polarity">
        /// Sets determines the polarity of the external gate signal.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_NEG  (0) - Negative
        /// RSSPECAN_VAL_POS  (1) - Positive
        /// 
        /// Default Value: RSSPECAN_VAL_POS  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_GATE_POLARITY
        /// 
        /// Note:
        /// (1) This parameter has effect only for Gating set to On
        /// 
        /// </param>
        /// <param name="Delay">
        /// Defines the delay time between the external gate signal and the continuation of the sweep. 
        /// 
        /// Valid Values:
        /// 0.0 to 100.0
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_GATE_HOLD
        /// 
        /// Note:
        /// (1) This parameter has effect only for Gating set to On
        /// 
        /// </param>
        /// <param name="Length">
        /// Specifies defines the gate time in the capture buffer in time units or samples. The range of this value is dependent on the measurement that was last run.
        /// 
        /// Units: s
        /// 
        /// Valid Values: 1.2e-7 to 100.0
        /// 
        /// Default Value: 1.25e-7
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_GATE_LENGTH
        /// 
        /// Note:
        /// (1) This parameter has effect only for Gating set to On and Mode is set to Edge
        /// 
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureExternalGate(int Window, bool Gating, int Gate_Source, int Mode, int Polarity, double Delay, double Length)
        {
            int pInvokeResult = PInvoke.ConfigureExternalGate(this._handle, Window, System.Convert.ToUInt16(Gating), Gate_Source, Mode, Polarity, Delay, Length);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the external gate trace parameters. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_EXTERNAL_GATE_TRACE_STATE
        /// RSSPECAN_ATTR_EXTERNAL_GATE_TRACE_START
        /// RSSPECAN_ATTR_EXTERNAL_GATE_TRACE_STOP
        /// RSSPECAN_ATTR_EXTERNAL_GATE_TRACE_PERIOD_LENGTH
        /// RSSPECAN_ATTR_EXTERNAL_GATE_TRACE_COMMENT
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:TRACe&lt;1...6&gt;[:STATe&lt;1...3&gt;] ON | OFF
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:TRACe&lt;1...6&gt;:STARt&lt;1...3&gt;
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:TRACe&lt;1...6&gt;:STOP&lt;1...3&gt;
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:TRACe&lt;1...6&gt;:PERiod
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:TRACe&lt;1...6&gt;:COMMent
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 1 to 6 for R&amp;S FSL
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Range">
        /// This control sets the external gate trace range.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace_State">
        /// This parameter activates or deactivates tracing for a specific range using gated triggering.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_GATE_TRACE_STATE
        /// </param>
        /// <param name="Comment">
        /// This control defines a comment for one of the traces for gated triggering.
        /// 
        /// Valid Values: not checked 
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_GATE_TRACE_COMMENT
        /// </param>
        /// <param name="Trace_Start">
        /// Defines the starting point for the range to be traced using gated triggering.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0.0 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_GATE_TRACE_START
        /// </param>
        /// <param name="Trace_Stop">
        /// Defines the stopping point for the range to be traced using gated triggering.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0.001 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_GATE_TRACE_STOP
        /// 
        /// 
        /// </param>
        /// <param name="Period_Length">
        /// Defines the length of the period to be traced using gated triggering.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0.0 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_GATE_TRACE_PERIOD_LENGTH
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureExternalGateTrace(int Trace, int Range, bool Trace_State, string Comment, double Trace_Start, double Trace_Stop, double Period_Length)
        {
            int pInvokeResult = PInvoke.ConfigureExternalGateTrace(this._handle, Trace, Range, System.Convert.ToUInt16(Trace_State), Comment, Trace_Start, Trace_Stop, Period_Length);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function turns external mixing on/off.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_ENABLED
        /// 
        /// Remote-control command(s):
        /// [SENSe:]MIXer[:STATe] ON | OFF
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="External_Mixing_Enabled">
        /// Specifies whether external mixing is enabled.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0) - Off
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_ENABLED
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureExternalMixerEnabled(bool External_Mixing_Enabled)
        {
            int pInvokeResult = PInvoke.ConfigureExternalMixerEnabled(this._handle, System.Convert.ToUInt16(External_Mixing_Enabled));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies a 2-port or a 3-port external mixer.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_NUMBER_OF_PORTS
        /// 
        /// Remote-control command(s):
        /// [SENSe:]MIXer:PORTs 2 | 3
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Number_of_Ports">
        /// Specifies a two-port mixer or a three-port mixer. 
        /// 
        /// Valid Values:
        /// 2, 3
        /// 
        /// Default Value: 2 
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_NUMBER_OF_PORTS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureExternalMixerNumberOfPorts(int Number_of_Ports)
        {
            int pInvokeResult = PInvoke.ConfigureExternalMixerNumberOfPorts(this._handle, Number_of_Ports);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the mixer harmonic and average conversion loss.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_HARMONIC
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_HARMONIC_TYPE
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_HARMONIC_BAND
        /// 
        /// Remote-control command(s):
        /// [SENSe:]MIXer:HARMonic
        /// [SENSe:]MIXer:HARMonic:TYPE ODD | EVEN | EODD
        /// [SENSe:]MIXer:HARMonic:BAND A|Q|U|V|E|W|F|D|G|Y|J|USER
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Band">
        /// Defines waveguide band in the Conversion Loss table.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_EXT_MIX_BAND_A    (0)  - A
        /// RSSPECAN_VAL_EXT_MIX_BAND_Q    (1)  - Q
        /// RSSPECAN_VAL_EXT_MIX_BAND_U    (2)  - U 
        /// RSSPECAN_VAL_EXT_MIX_BAND_V    (3)  - V
        /// RSSPECAN_VAL_EXT_MIX_BAND_E    (4)  - E
        /// RSSPECAN_VAL_EXT_MIX_BAND_W    (5)  - W
        /// RSSPECAN_VAL_EXT_MIX_BAND_F    (6)  - F
        /// RSSPECAN_VAL_EXT_MIX_BAND_D    (7)  - D
        /// RSSPECAN_VAL_EXT_MIX_BAND_G    (8)  - G
        /// RSSPECAN_VAL_EXT_MIX_BAND_Y    (9)  - Y
        /// RSSPECAN_VAL_EXT_MIX_BAND_J    (10) - J
        /// RSSPECAN_VAL_EXT_MIX_BAND_USER (11) - User
        /// 
        /// Default Value: RSSPECAN_VAL_EXT_MIX_BAND_U (2)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_HARMONIC_BAND
        /// </param>
        /// <param name="Harmonic">
        /// Specifies harmonic number. 
        /// 
        /// Valid Values:
        /// 2 to 64
        /// 2 to 65 for R&amp;S FSV
        /// 
        /// Default Value: 2
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_HARMONIC
        /// </param>
        /// <param name="Harmonic_Type">
        /// Sets the type of harmonic.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_EXT_MIX_HARM_ODD  (0) - Odd
        /// RSSPECAN_VAL_EXT_MIX_HARM_EVEN (1) - Even
        /// RSSPECAN_VAL_EXT_MIX_HARM_EODD (2) - Even&amp;Odd
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_EXT_MIX_HARM_EVEN (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_HARMONIC_TYPE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureExternalMixer(int Band, int Harmonic, int Harmonic_Type)
        {
            int pInvokeResult = PInvoke.ConfigureExternalMixer(this._handle, Band, Harmonic, Harmonic_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the frequency at which the external mixer band starts and stops.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available for R&amp;S FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GET_EXTERNAL_MIXER_RF_FREQUENCY_START
        /// RSSPECAN_ATTR_GET_EXTERNAL_MIXER_RF_FREQUENCY_STOP
        /// 
        /// Remote-control command(s):
        /// [SENSe:]MIXer:FREQuency:STARt?
        /// [SENSe:]MIXer:FREQuency:STOP?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="RF_Start">
        /// Queries the frequency at which the external mixer band starts.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GET_EXTERNAL_MIXER_RF_FREQUENCY_START
        /// </param>
        /// <param name="RF_Stop">
        /// Queries the frequency at which the external mixer band stops.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GET_EXTERNAL_MIXER_RF_FREQUENCY_STOP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryExternalMixerRFStartStop(out double RF_Start, out double RF_Stop)
        {
            int pInvokeResult = PInvoke.QueryExternalMixerRFStartStop(this._handle, out RF_Start, out RF_Stop);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the frequency at which the mixer switches from one range to the next (if two different ranges are selected). The handover frequency for each band can be selected freely within the overlapping frequency range.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available for R&amp;S FSV instrument.
        /// 
        /// (2) This function is only available if the external mixer is active.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_HANDOVER_FREQUENCY
        /// 
        /// Remote-control command(s):
        /// [SENSe:]MIXer:FREQuency:HANDover
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Handover_Frequency">
        /// Defines the frequency at which the mixer switches from one range to the next.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0.0 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_HANDOVER_FREQUENCY
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureExternalMixerHandoverFrequency(double Handover_Frequency)
        {
            int pInvokeResult = PInvoke.ConfigureExternalMixerHandoverFrequency(this._handle, Handover_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies whether the frequencies beyond the defined band limits are to be used.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available for R&amp;S FSV instrument.
        /// 
        /// (2) This function is only available if the external mixer is active.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_RF_OVERRANGE
        /// 
        /// Remote-control command(s):
        /// [SENSe:]MIXer:RFOVerrange[:STATe] ON | OFF
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="RF_Overrange">
        /// Specifies whether the frequencies beyond the defined band limits are to be used.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_RF_OVERRANGE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureExternalMixerRFOverrange(bool RF_Overrange)
        {
            int pInvokeResult = PInvoke.ConfigureExternalMixerRFOverrange(this._handle, System.Convert.ToUInt16(RF_Overrange));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function restores the preset frequency ranges for the standard waveguide bands.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available for R&amp;S FSV instrument.
        /// 
        /// (2) Changes to the band and mixer settings are maintained even after using this function. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_PRESET_BAND
        /// 
        /// Remote-control command(s):
        /// [SENSe:]MIXer:HARMonic:BAND:PRESet
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureExternalMixerPresetBand()
        {
            int pInvokeResult = PInvoke.ConfigureExternalMixerPresetBand(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies a second (high) harmonic.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available for R&amp;S FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_HIGH_HARMONIC_STATE
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_HIGH_HARMONIC
        /// 
        /// Remote-control command(s):
        /// [SENSe:]MIXer:HARMonic:HIGH:STATe ON | OFF
        /// [SENSe:]MIXer:HARMonic:HIGH[:VALue]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="High_Harmonic">
        /// Specifies whether a second (high) harmonic is to be used to cover the band's frequency range.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_HIGH_HARMONIC_STATE
        /// </param>
        /// <param name="Harmonic_Order">
        /// Specifies the harmonic order to be used for the high range. 
        /// 
        /// Valid Values:
        /// 2 to 65
        /// 
        /// Default Value: 2
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_HIGH_HARMONIC
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureExternalMixerHighHarmonic(bool High_Harmonic, int Harmonic_Order)
        {
            int pInvokeResult = PInvoke.ConfigureExternalMixerHighHarmonic(this._handle, System.Convert.ToUInt16(High_Harmonic), Harmonic_Order);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the Signal ID or Auto ID mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_SIGNAL_ID_MODE
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_THRESHOLD
        /// 
        /// Remote-control command(s):
        /// [SENSe:]MIXer:SIGNal ON | OFF | AUTO | ALL
        /// [SENSe:]MIXer:THReshold
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Mode">
        /// Sets the type of harmonic.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_STATE_OFF  (0) - Off
        /// RSSPECAN_VAL_STATE_ON   (1) - Signal ID
        /// RSSPECAN_VAL_STATE_AUTO (2) - Auto ID
        /// RSSPECAN_VAL_STATE_ALL  (3) - All
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_STATE_OFF  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_MODE
        /// </param>
        /// <param name="Auto_ID_Threshold">
        /// Determines the maximum permissible level difference between measurement and reference sweep in AUTO ID mode. 
        /// 
        /// Units:
        /// dBm
        /// 
        /// Valid Values:
        /// 0.1 to 100.0
        /// 
        /// Default Value:
        /// 10.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_THRESHOLD
        /// 
        /// Note(s):
        /// (1) This control has effect only if Mode is set to Auto ID
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureExternalMixerSignalIDMode(int Mode, double Auto_ID_Threshold)
        {
            int pInvokeResult = PInvoke.ConfigureExternalMixerSignalIDMode(this._handle, Mode, Auto_ID_Threshold);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the bias current for the low and high range.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_BIAS_LOW
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_BIAS_HIGH
        /// 
        /// Remote-control command(s):
        /// [SENSe:]MIXer:BIAS[:LOW]
        /// [SENSe:]MIXer:BIAS:HIGH
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Range">
        /// This control selects the conversion loss range.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Bias">
        /// Specifies the bias current.
        /// 
        /// Units: Amps
        /// 
        /// Valid Values: -10.0e-3 to 10.0e-3 
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_BIAS_LOW
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureExternalMixerBias(int Range, double Bias)
        {
            int pInvokeResult = PInvoke.ConfigureExternalMixerBias(this._handle, Range, Bias);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the conversion loss for selected band.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_CONVERSION_LOSS_LOW
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_CONVERSION_LOSS_HIGH
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_CONVERSION_LOSS_TABLE_LOW
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_CONVERSION_LOSS_TABLE_HIGH
        /// 
        /// Remote-control command(s):
        /// [SENSe:]MIXer:LOSS[:LOW]
        /// [SENSe:]MIXer:LOSS:HIGH
        /// [SENSe:]MIXer:LOSS:TABLe[:LOW]
        /// [SENSe:]MIXer:LOSS:TABLe:HIGH
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Range">
        /// This control selects the conversion loss range.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Conversion_Loss_Mode">
        /// This control selects the converstion loss mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MIXER_LOSS_AVER (0) - Average
        /// RSSPECAN_VAL_MIXER_LOSS_TABL (1) - Table
        /// 
        /// Default Value: RSSPECAN_VAL_MIXER_LOSS_AVER (0)
        /// </param>
        /// <param name="Conversion_Loss_Average">
        /// Specifies the average conversion loss.
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_CONVERSION_LOSS_LOW
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_CONVERSION_LOSS_HIGH
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available if Conversion Loss Mode is set to Average.
        /// </param>
        /// <param name="Conversion_Loss_Table">
        /// Sets the conversion loss table for the active band.
        /// 
        /// Valid Values: any string
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_CONVERSION_LOSS_TABLE_LOW
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_CONVERSION_LOSS_TABLE_HIGH
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available if Conversion Loss Mode is set to Table.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureExternalMixerConversionLoss(int Range, int Conversion_Loss_Mode, double Conversion_Loss_Average, string Conversion_Loss_Table)
        {
            int pInvokeResult = PInvoke.ConfigureExternalMixerConversionLoss(this._handle, Range, Conversion_Loss_Mode, Conversion_Loss_Average, Conversion_Loss_Table);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the LO level of the external mixer LO port from 13.0 dBm to 17.0 dBm in 0.1 dB steps.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_LO_POWER
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]MIXer:LOPower
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Low_Power">
        /// Sets the LO level of the external mixer LO port. 
        /// 
        /// Units: dBm
        /// 
        /// Valid Values:
        /// 13.0 to 17.0 in 0.1 steps
        /// 
        /// Default Value:
        /// 15.5
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_LO_POWER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureExternalMixerLowPower(double Low_Power)
        {
            int pInvokeResult = PInvoke.ConfigureExternalMixerLowPower(this._handle, Low_Power);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function creates or overwrites conversion loss table of external mixer.
        /// 
        /// Notes:
        /// 
        /// (1) This function is available only with option FSU-B21.
        /// 
        /// (2) If the external mixer is not turned on, this function will return an error.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CVL_NAME
        /// RSSPECAN_ATTR_CVL_MIXER
        /// RSSPECAN_ATTR_CVL_MIXER_SNUM
        /// RSSPECAN_ATTR_CVL_BAND
        /// RSSPECAN_ATTR_CVL_HARM
        /// RSSPECAN_ATTR_CVL_BIAS
        /// RSSPECAN_ATTR_CVL_COMMENT
        /// RSSPECAN_ATTR_CVL_PORT_MIXER
        /// no attribute for CVL Data
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CORRection:CVL:SELect
        /// [SENSe:]CORRection:CVL:MIXer
        /// [SENSe:]CORRection:CVL:SNUMber
        /// [SENSe:]CORRection:CVL:BAND A|Q|U|V|E|W|F|D|G|Y|J|USER
        /// [SENSe:]CORRection:CVL:HARMonic
        /// [SENSe:]CORRection:CVL:BIAS
        /// [SENSe:]CORRection:CVL:COMMent
        /// [SENSe:]CORRection:CVL:PORTs 2 | 3
        /// [SENSe:]CORRection:CVL:DATA
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Table_Name">
        /// Enter name of table (without extention) to edit or create. A new table will be created if the specified name does not exist.
        /// 
        /// Valid Values: max. 8 characters
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CVL_NAME
        /// 
        /// </param>
        /// <param name="Mixer_Type">
        /// Defines the type designation of the mixer in the Conversion Loss Table.
        /// 
        /// Valid Values: max. 16 characters
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CVL_MIXER
        /// </param>
        /// <param name="Band">
        /// Defines waveguide band in the Conversion Loss table.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_CVL_BAND_A    (0) - A
        /// RSSPECAN_VAL_CVL_BAND_Q    (1) - Q
        /// RSSPECAN_VAL_CVL_BAND_U    (2) - U 
        /// RSSPECAN_VAL_CVL_BAND_V    (3) - V
        /// RSSPECAN_VAL_CVL_BAND_E    (4) - E
        /// RSSPECAN_VAL_CVL_BAND_W    (5) - W
        /// RSSPECAN_VAL_CVL_BAND_F    (6) - F
        /// RSSPECAN_VAL_CVL_BAND_D    (7) - D
        /// RSSPECAN_VAL_CVL_BAND_G    (8) - G
        /// RSSPECAN_VAL_CVL_BAND_Y    (9) - Y
        /// RSSPECAN_VAL_CVL_BAND_J    (10) - J
        /// RSSPECAN_VAL_CVL_BAND_USER (11) - User
        /// 
        /// Default Value: RSSPECAN_VAL_CVL_BAND_U    (2)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CVL_BAND
        /// </param>
        /// <param name="Serial_Number">
        /// Enter the serial number of the mixer.
        /// 
        /// Valid Values: max. 16 characters
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CVL_MIXER_SNUM
        /// </param>
        /// <param name="Harmonic">
        /// Defines the harmonic of the mixer in the Conversion Loss Table. 
        /// 
        /// Valid Values: &gt;= 2
        /// 
        /// Default Value: 2
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CVL_HARM
        /// </param>
        /// <param name="Ports">
        /// Specifies the mixer type (two-port mixer or three-port mixer).
        /// 
        /// Valid Values: 2 to 3
        /// 
        /// Default Value: 2
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CVL_PORT_MIXER
        /// </param>
        /// <param name="Bias">
        /// Enters the bias current.
        /// 
        /// Units: A
        /// 
        /// Valid Values: -10.0e-3 to 10.0e-3
        /// 
        /// Default Value: 0.0A
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CVL_BIAS
        /// </param>
        /// <param name="Comment">
        /// Enter the conevrsion loss table comment.
        /// 
        /// Valid Values: max. 60 characters
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CVL_COMMENT
        /// </param>
        /// <param name="Number_Of_Values">
        /// Inputs number of reference value pairs to be send to the  instrument.
        /// 
        /// Valid Values: 1 to 50
        /// 
        /// Default Value: 50
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_MIXER_LO_POWER
        /// </param>
        /// <param name="Frequency_Values">
        /// Specifies array of frequency values in Hz. 
        /// 
        /// Valid Values: none
        /// 
        /// Default Value: none
        /// 
        /// Notes:
        /// 
        /// (1) The reference values must be entered in order of increasing frequencies.
        /// 
        /// (2) A maximum of 50 reference values can be entered.
        /// 
        /// </param>
        /// <param name="Conversion_Loss_Values">
        /// Specifies array of level values in dB.
        /// 
        /// Valid Values: none
        /// 
        /// Default Value: none
        /// 
        /// Notes:
        /// 
        /// (1) The reference values must be entered in order of increasing frequencies.
        /// 
        /// (2) A maximum of 50 reference values can be entered.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CreateExternalMixerConversionLossTable(int Window, string Table_Name, string Mixer_Type, int Band, string Serial_Number, int Harmonic, int Ports, double Bias, string Comment, int Number_Of_Values, double[] Frequency_Values, double[] Conversion_Loss_Values)
        {
            int pInvokeResult = PInvoke.CreateExternalMixerConversionLossTable(this._handle, Window, Table_Name, Mixer_Type, Band, Serial_Number, Harmonic, Ports, Bias, Comment, Number_Of_Values, Frequency_Values, Conversion_Loss_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects a conversion loss table of external mixer.
        /// 
        /// Notes:
        /// 
        /// (1) If the external mixer is not turned on, this function will return an error.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CVL_NAME
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CORRection:CVL:SELect
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Table_Name">
        /// Enter name of table (without extention) to delete.
        /// 
        /// Valid Values: max. 8 characters
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CVL_NAME
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ExternalMixerConversionLossTableSelect(int Window, string Table_Name)
        {
            int pInvokeResult = PInvoke.ExternalMixerConversionLossTableSelect(this._handle, Window, Table_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function deletes a conversion loss table of external mixer.
        /// 
        /// Notes:
        /// 
        /// (1) If the external mixer is not turned on, this function will return an error.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CVL_NAME
        /// RSSPECAN_ATTR_CVL_CLEAR
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CORRection:CVL:SELect
        /// [SENSe:]CORRection:CVL:CLEar
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Table_Name">
        /// Enter name of table (without extention) to delete.
        /// 
        /// Valid Values: max. 8 characters
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CVL_NAME
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ExternalMixerConversionLossTableDelete(int Window, string Table_Name)
        {
            int pInvokeResult = PInvoke.ExternalMixerConversionLossTableDelete(this._handle, Window, Table_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the names of all conversion loss tables saved on the hard disk.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:CVL:CATalog?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Number_of_CVL">
        /// This control returns number of the names of all digital standards available.
        /// 
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the string you specify for the catalog list parameter.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// Valid Values:
        /// any number
        /// 
        /// Default Value: 0
        /// 
        /// </param>
        /// <param name="CVL_Names_List">
        /// This control the names of all conversion loss tables saved on the hard disk.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// if you pass 0 for buffer Size, you can use VI_NULL for this parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ExternalMixerConversionLossTableCatalog(out int Number_of_CVL, int Buffer_Size, System.Text.StringBuilder CVL_Names_List)
        {
            int pInvokeResult = PInvoke.ExternalMixerConversionLossTableCatalog(this._handle, out Number_of_CVL, Buffer_Size, CVL_Names_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the input coupling of the RF input between AC and DC.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSQ, FSU, FSMR, FSG or FSV instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AMPL_RF_COUPLING
        /// 
        /// Remote-control command(s):
        /// INPut:COUPling AC | DC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="RF_Input_Coupling">
        /// This control selects selects the coupling of RF Input. The driver uses this value to set the RSSPECAN_ATTR_AMPL_RF_COUPLING attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DC (0) - DC
        /// RSSPECAN_VAL_AC (1) - AC
        /// 
        /// Default Value: RSSPECAN_VAL_DC (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AMPL_RF_COUPLING
        /// 
        /// Note(s):
        /// 
        /// (1) For available coupling values please consult instrument's operating manual.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureRFInputCoupling(int RF_Input_Coupling)
        {
            int pInvokeResult = PInvoke.ConfigureRFInputCoupling(this._handle, RF_Input_Coupling);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures electronic attenuator parameters.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is valid only in cojunction with option B25.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AMPL_EATT_STATE
        /// RSSPECAN_ATTR_AMPL_EATT_VALUE
        /// RSSPECAN_ATTR_AMPL_EATT_AUTO
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:EATT:STATe ON | OFF
        /// INPut&lt;1|2&gt;:EATT
        /// INPut&lt;1|2&gt;:EATT:AUTO ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Input">
        /// Defines the input number.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Mode">
        /// This control selects selects the state of electronic attenuator. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_EATT_OFF  (0) - Off
        /// RSSPECAN_VAL_EATT_AUTO (1) - Auto
        /// RSSPECAN_VAL_EATT_MAN  (2) - Manual
        /// 
        /// Default Value: RSSPECAN_VAL_EATT_OFF (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AMPL_EATT_STATE
        /// RSSPECAN_ATTR_AMPL_EATT_AUTO
        /// </param>
        /// <param name="Manual_Value">
        /// Programs the attenuation of the electronic input attenuator. The attenuation can be varied in 5 dB steps from 0 to 30 dB. Other entries are rounded to the next lower integer value. If the attenuation is programmed directly, the coupling to the reference level is switched off. If the defined reference level cannot be set with the given RF attenuation, this level is adapted to the maximum possible value.
        /// The electronic attenuator is switched off in the default state. 
        /// 
        /// Valid Values:
        /// 0,5,10,15,20,25,30 dB
        /// 0 to 25 dB for R&amp;S FSV (increment step is 1 dB)
        /// 
        /// Default Value:
        /// 0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AMPL_EATT_VALUE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureElectronicAttenuator(int Input, int Mode, int Manual_Value)
        {
            int pInvokeResult = PInvoke.ConfigureElectronicAttenuator(this._handle, Input, Mode, Manual_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures input mixer of the analyzer.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AMPL_MIXER_LEVEL
        /// RSSPECAN_ATTR_AMPL_MIXER_AUTO
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:MIXer[:POWer]
        /// INPut&lt;1|2&gt;:MIXer:AUTO ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Input">
        /// Defines the input number.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Automatic_Setup">
        /// Enables/disables the automatic setup of the mixer level.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AMPL_MIXER_AUTO
        /// </param>
        /// <param name="Power">
        /// Defines the desired power at the input mixer of the analyzer. On any change to the reference level the RF attenuation will be adjusted in a way that makes the difference between reference level and RF attenuation come as close to the desired mixer level as possible.
        /// 
        /// Valid Values:
        /// -100.0 to 5.0 dBm
        /// 
        /// Default Value: -25.0 dBm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AMPL_MIXER_LEVEL
        /// 
        /// Note:
        /// 
        /// (1) This control has effect only if Automatic setup is set Off
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureInputMixerLevel(int Input, bool Automatic_Setup, double Power)
        {
            int pInvokeResult = PInvoke.ConfigureInputMixerLevel(this._handle, Input, System.Convert.ToUInt16(Automatic_Setup), Power);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on the preamplifier for the instrument. The switchable gain is fixed to 20 dB.
        /// 
        /// Note(s):
        /// 
        /// (1) This function requires options B25 or B23. Option B22 is required for R&amp;S FSL or R&amp;S ESL or R&amp;S FSV instruments.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AMPL_PREAMPLIFIER
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:GAIN:STATe ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// Switches on the preamplifier for the instrument. The switchable gain is fixed to 20 dB.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AMPL_PREAMPLIFIER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePreamplifierState(int Window, bool State)
        {
            int pInvokeResult = PInvoke.ConfigurePreamplifierState(this._handle, Window, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the YIG filter for image rejection into the signal path (ON) or removes it from the signal path (OFF).
        /// 
        /// Note:
        /// (1) This function is available for FSQ or FSMR only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SERVICE_YIG_FILTER
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:FILTer:YIG[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="YIG_Filter">
        /// Switches the YIG filter for image rejection into the signal path (ON) or removes it from the signal path (OFF).
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SERVICE_YIG_FILTER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureYIGFilter(int Window, bool YIG_Filter)
        {
            int pInvokeResult = PInvoke.ConfigureYIGFilter(this._handle, Window, System.Convert.ToUInt16(YIG_Filter));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// The instrument is equipped with an overload protection mechanism. This mechanism becomes active as soon as the power at the input mixer exceeds a value of 27 dBm. It ensures that the connection between RF input and input mixer is cut off.
        /// The function resets the attenuator into the state that it had before the overload condition was detected. It re-connects the RF input with the input mixer.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AMPL_PROTECTION_RESET
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:ATTenuation:PROTection:RESet
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int InputProtectionReset(int Window)
        {
            int pInvokeResult = PInvoke.InputProtectionReset(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function creates new limit or selects existing one and configures basic parameters.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_NAME
        /// RSSPECAN_LIMIT_CONTROL_DOMAIN
        /// RSSPECAN_LIMIT_ASSIGN_TRACE
        /// RSSPECAN_LIMIT_COMMENT
        /// RSSPECAN_LIMIT_DELETE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:NAME
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:CONTrol:DOMain FREQuency | TIME
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:COMMent
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:TRACe
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:DELete
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Name">
        /// This control sets the limit line name.
        /// 
        /// Valid Values:
        /// up to 8 character string
        /// 
        /// Default Value: "limit"
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_NAME
        /// </param>
        /// <param name="Domain">
        /// This control defines the frequency or time domain for the x axis values.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FREQ (0) - Frequency
        /// RSSPECAN_VAL_TIME (1) - Time
        /// 
        /// Default Value: RSSPECAN_VAL_FREQ (0)
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_CONTROL_DOMAIN
        /// </param>
        /// <param name="Comment">
        /// This control sets the limit line comment.
        /// 
        /// Valid Values:
        /// up to 40 character string
        /// 
        /// Default Value: "Comment"
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_COMMENT
        /// </param>
        /// <param name="Assign_to_Trace">
        /// Assigns a limit line to a trace in the indicated measurement window.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 1 to 6 for R&amp;S FSL, R&amp;S FSV
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_ASSIGN_TRACE
        /// </param>
        /// <param name="Delete_Existing_Limit_Line">
        /// Removes specified limit line before new is created.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_DELETE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CreateLimitLine(int Window, int Limit, string Name, int Domain, string Comment, int Assign_to_Trace, bool Delete_Existing_Limit_Line)
        {
            int pInvokeResult = PInvoke.CreateLimitLine(this._handle, Window, Limit, Name, Domain, Comment, Assign_to_Trace, System.Convert.ToUInt16(Delete_Existing_Limit_Line));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures specified limit line.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_UNITS
        /// RSSPECAN_LIMIT_CONTROL_SPACING
        /// RSSPECAN_LIMIT_LOWER_SPACING
        /// RSSPECAN_LIMIT_UPPER_SPACING
        /// RSSPECAN_LIMIT_CONTROL_SCALING
        /// RSSPECAN_LIMIT_LOWER_SCALING
        /// RSSPECAN_LIMIT_UPPER_SCALING
        /// RSSPECAN_LIMIT_UPPER_MARGIN
        /// RSSPECAN_LIMIT_LOWER_MARGIN
        /// RSSPECAN_LIMIT_UPPER_THRESHOLD
        /// RSSPECAN_LIMIT_LOWER_THRESHOLD
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:UNIT
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:CONTrol:SPACing LINear | LOGarithmic
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:LOWer:SPACing LINear | LOGarithmic
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:UPPer:SPACing LINear | LOGarithmic
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:CONTrol:MODE RELative | ABSolute
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:LOWer:MODE RELative | ABSolute
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:UPPer:MODE RELative | ABSolute
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:UPPer:MARGin
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:LOWer:MARGin
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:UPPer:THReshold
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:LOWer:THReshold
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Type">
        /// Specifies a limit line to be either an upper or lower type limit line. An upper line will be used as the maximum allowable value when comparing with the data. A lower limit line defines the minimum value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIMIT_LOWER (0) - Lower
        /// RSSPECAN_VAL_LIMIT_UPPER (1) - Upper
        /// 
        /// Default Value: RSSPECAN_VAL_LIMIT_LOWER (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Lower - lower limit fails if the trace falls below the limit.
        /// 
        /// (2) Upper - upper limit fails if the trace exceeds the limit.
        /// 
        /// </param>
        /// <param name="Units">
        /// This control defines the unit of the selected limit line.
        /// The definition is valid independently of the measurement window.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DBM       (0) - dBm
        /// RSSPECAN_VAL_UNIT_DBPW      (1) - dBpW
        /// RSSPECAN_VAL_UNIT_WATT      (2) - W
        /// RSSPECAN_VAL_UNIT_DBUV      (3) - dBuV
        /// RSSPECAN_VAL_UNIT_DBMV      (4) - dBmV
        /// RSSPECAN_VAL_UNIT_VOLT      (5) - V
        /// RSSPECAN_VAL_UNIT_DBUA      (6) - dBuA
        /// RSSPECAN_VAL_UNIT_AMP       (7) - A
        /// RSSPECAN_VAL_UNIT_DB        (8) - dB
        /// RSSPECAN_VAL_UNIT_DEG       (9) - deg
        /// RSSPECAN_VAL_UNIT_RAD       (10) - rad
        /// RSSPECAN_VAL_UNIT_S         (11) - s
        /// RSSPECAN_VAL_UNIT_HZ        (12) - Hz
        /// RSSPECAN_VAL_UNIT_PCT       (13) - %
        /// RSSPECAN_VAL_UNIT_UNITLESS  (14) - Unitless
        /// RSSPECAN_VAL_UNIT_DBPT      (15) - dBpT
        /// RSSPECAN_VAL_UNIT_DBMV_MHZ  (16) - dBmV/MHz
        /// RSSPECAN_VAL_UNIT_DBUV_MHZ  (17) - dBuV/MHz
        /// RSSPECAN_VAL_UNIT_DBUA_MHZ  (18) - dBuA/MHz
        /// RSSPECAN_VAL_UNIT_DBMV_M    (19) - dBmV/M
        /// RSSPECAN_VAL_UNIT_DBUA_M    (20) - dBuA/M
        /// RSSPECAN_VAL_UNIT_DBMV_MMHZ (21) - dBmV/MMHz
        /// RSSPECAN_VAL_UNIT_DBUA_MMHZ (22) - dBuA/MMHz
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_DBM (0)
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_UNITS
        /// 
        /// Note(s):
        /// 
        /// (1) Upon selection of the unit DB the limit line is automatically switched to the relative mode. For units different from DB the limit line is  automatically switched to absolute mode.
        /// 
        /// (2) The units DEG, RAD, S, HZ, PCT are not available in SPECTRUM mode.
        /// 
        /// (3) The units DBPT is available for R&amp;S FSL and R&amp;S ESL only.
        /// 
        /// (4) The units dBmV/MHz, dBuV/MHz, dBuA/MHz, dBmV/M, dBuA/M, dBmV/MMHz, dBuA/MMHz are available for R&amp;S ESL only.
        /// </param>
        /// <param name="X_Axis_Interpolation">
        /// Selects linear or logarithmic interpolation for the calculation of limit lines from frequency points.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIN (0) - Linear
        /// RSSPECAN_VAL_LOG (1) - Logarithmic
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_LIN (0)
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_CONTROL_SPACING
        /// </param>
        /// <param name="Y_Axis_Interpolation">
        /// Selects linear or logarithmic interpolation for the upper or lower limit line.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIN (0) - Linear
        /// RSSPECAN_VAL_LOG (1) - Logarithmic
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_LIN (0)
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_LOWER_SPACING
        /// RSSPECAN_LIMIT_UPPER_SPACING
        /// </param>
        /// <param name="X_Axis_Scaling">
        /// Selects the relative or absolute scaling for the X axis of the selected limit line.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ABS (0) - Absolute
        /// RSSPECAN_VAL_REL (1) - Relative
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_ABS (0)
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_CONTROL_SCALING
        /// </param>
        /// <param name="Margin">
        /// Defines a margin to a lower limit line, at which out-of-limit values are signalled (if the limit check is active), but not handled as a violation of the limit value.
        /// 
        /// Units: dB
        /// 
        /// Valid Values:
        /// 0.0 to 200.0 dB
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_UPPER_MARGIN
        /// RSSPECAN_LIMIT_LOWER_MARGIN
        /// </param>
        /// <param name="Threshold">
        /// Defines an absolute threshold value for limit lines with relative Y axis scaling independently of the measurement window. The absolute threshold value is used in the limit check
        /// as soon as it exceeds the relative limit value.
        /// 
        /// Valid Values:
        /// -200.0 to 200.0 dB
        /// 
        /// Default Value: -200.0
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_UPPER_THRESHOLD
        /// RSSPECAN_LIMIT_LOWER_THRESHOLD
        /// 
        /// Note(s):
        /// 
        /// (1) This control has effect only for units dB.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLimitLine(int Limit, int Type, int Units, int X_Axis_Interpolation, int Y_Axis_Interpolation, int X_Axis_Scaling, double Margin, double Threshold)
        {
            int pInvokeResult = PInvoke.ConfigureLimitLine(this._handle, Limit, Type, Units, X_Axis_Interpolation, Y_Axis_Interpolation, X_Axis_Scaling, Margin, Threshold);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines specified limit line.
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:CONTrol[:DATA]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:LOWer[:DATA]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:UPPer[:DATA]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Type">
        /// Specifies a limit line to be either an upper or lower type limit line. An upper line will be used as the maximum allowable value when comparing with the data. A lower limit line defines the minimum value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIMIT_LOWER (0) - Lower
        /// RSSPECAN_VAL_LIMIT_UPPER (1) - Upper
        /// 
        /// Default Value: RSSPECAN_VAL_LIMIT_LOWER (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Lower - lower limit fails if the trace falls below the limit.
        /// 
        /// (2) Upper - upper limit fails if the trace exceeds the limit.
        /// 
        /// </param>
        /// <param name="Count">
        /// Specifies number of values in X Axis and Amplitude arrays.
        /// 
        /// Valid Values:
        /// &gt;= 1
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="X_Axis">
        /// Specifies the frequency or time value for a limit point. The units are depended on the attribute RSSPECAN_ATTR_LLINE_X_AXIS_UNITS.
        /// 
        /// You must pass a ViReal64 array with at least count values.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Amplitude">
        /// Specifies the amplitude value for the current limit point.
        /// 
        /// You must pass a ViReal64 array with at least count values.
        /// 
        /// Units: Are depended on the RSSPECAN_LIMIT_UPPER_SCALING or RSSPECAN_LIMIT_LOWER_SCALING attribute.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int DefineLimitLine(int Limit, int Type, int Count, double[] X_Axis, double[] Amplitude)
        {
            int pInvokeResult = PInvoke.DefineLimitLine(this._handle, Limit, Type, Count, X_Axis, Amplitude);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the limit line.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_NAME
        /// RSSPECAN_LIMIT_STATE
        /// 
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:NAME
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:STATe ON | OFF
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Name">
        /// This control sets the limit line name.
        /// 
        /// Valid Values:
        /// up to 8 character string
        /// 
        /// Default Value: "limit"
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_NAME
        /// </param>
        /// <param name="Limit_Enabled">
        /// Turns the limit line display on and off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SelectLimitLine(int Window, int Limit, string Name, bool Limit_Enabled)
        {
            int pInvokeResult = PInvoke.SelectLimitLine(this._handle, Window, Limit, Name, System.Convert.ToUInt16(Limit_Enabled));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the upper limit line.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LIMIT_UPPER_PORTABLE
        /// 
        /// Remote-control command(s):
        /// CALCulate:LIMit&lt;1|2&gt;:UPPer:SELect
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit_Line_Name">
        /// This control sets the upper limit line name.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LIMIT_UPPER_PORTABLE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int SelectLimitLineUpperPortable(string Limit_Line_Name)
        {
            int pInvokeResult = PInvoke.SelectLimitLineUpperPortable(this._handle, Limit_Line_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the lower limit line.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LIMIT_LOW_PORTABLE
        /// 
        /// Remote-control command(s):
        /// CALCulate:LIMit&lt;1|2&gt;:LOWer:SELect
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit_Line_Name">
        /// This control sets the lower limit line name.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LIMIT_LOW_PORTABLE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int SelectLimitLineLowerPortable(string Limit_Line_Name)
        {
            int pInvokeResult = PInvoke.SelectLimitLineLowerPortable(this._handle, Limit_Line_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the limit check and/or limit line for the selected limit line in the selected measurement window.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_STATE
        /// RSSPECAN_LIMIT_LOWER_STATE
        /// RSSPECAN_LIMIT_UPPER_STATE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:STATe ON | OFF
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:LOWer:STATe ON | OFF
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:UPPer:STATe ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Type">
        /// Specifies a limit line to be either an upper or lower type limit line. An upper line will be used as the maximum allowable value when comparing with the data. A lower limit line defines the minimum value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIMIT_LOWER (0) - Lower
        /// RSSPECAN_VAL_LIMIT_UPPER (1) - Upper
        /// 
        /// Default Value: RSSPECAN_VAL_LIMIT_LOWER (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Lower - lower limit fails if the trace falls below the limit.
        /// 
        /// (2) Upper - upper limit fails if the trace exceeds the limit.
        /// 
        /// </param>
        /// <param name="Limit_Enabled">
        /// Turns the limit line display on and off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_STATE
        /// 
        /// </param>
        /// <param name="Check_Enabled">
        /// Turns limit line testing on/off. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_LOWER_STATE
        /// RSSPECAN_LIMIT_UPPER_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int EnableLimitCheck(int Window, int Limit, int Type, bool Limit_Enabled, bool Check_Enabled)
        {
            int pInvokeResult = PInvoke.EnableLimitCheck(this._handle, Window, Limit, Type, System.Convert.ToUInt16(Limit_Enabled), System.Convert.ToUInt16(Check_Enabled));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function moves specified limit line.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_CONTROL_OFFSET
        /// RSSPECAN_LIMIT_CONTROL_SHIFT
        /// RSSPECAN_LIMIT_LOWER_OFFSET
        /// RSSPECAN_LIMIT_LOWER_SHIFT
        /// RSSPECAN_LIMIT_UPPER_OFFSET
        /// RSSPECAN_LIMIT_UPPER_SHIFT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:CONTrol:OFFSet
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:CONTrol:SHIFt
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:LOWer:OFFSet
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:LOWer:SHIFt
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:UPPer:OFFSet
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:UPPer:SHIFt
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Type">
        /// Specifies a limit line to be type to be moved.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIMIT_LOWER   (0) - Lower
        /// RSSPECAN_VAL_LIMIT_UPPER   (1) - Upper
        /// RSSPECAN_VAL_LIMIT_CONTROL (2) - X-Axis
        /// 
        /// Default Value: RSSPECAN_VAL_LIMIT_LOWER (0)
        /// </param>
        /// <param name="Method">
        /// This control selects the method of moving.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIMIT_MOVE_OFFSET (0) - Offset
        /// RSSPECAN_VAL_LIMIT_MOVE_SHIFT  (1) - Shift
        /// 
        /// Default Value: RSSPECAN_VAL_LIMIT_MOVE_OFFSET (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Offset - line is shifted by means of an additive offset
        /// 
        /// (2) Shift - line is shifted by modifying the individual x values.
        /// </param>
        /// <param name="Value">
        /// Defines the move value.
        /// 
        /// Valid Values:
        /// -200.0 to 200.0 dB for Lower and Upper
        /// -300.0e9 to -300.0e9 Hz for X-Axis
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_CONTROL_OFFSET
        /// RSSPECAN_LIMIT_CONTROL_SHIFT
        /// RSSPECAN_LIMIT_LOWER_OFFSET
        /// RSSPECAN_LIMIT_LOWER_SHIFT
        /// RSSPECAN_LIMIT_UPPER_OFFSET
        /// RSSPECAN_LIMIT_UPPER_SHIFT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int MoveLimitLine(int Limit, int Type, int Method, double Value)
        {
            int pInvokeResult = PInvoke.MoveLimitLine(this._handle, Limit, Type, Method, Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function copies specified limit line into another one.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_COPY
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:COPY
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Copy_To">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_COPY
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CopyLimitLine(int Limit, int Copy_To)
        {
            int pInvokeResult = PInvoke.CopyLimitLine(this._handle, Limit, Copy_To);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function removes specified limit line.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_DELETE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:DELete
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_DELETE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int DeleteLimitLine(int Limit)
        {
            int pInvokeResult = PInvoke.DeleteLimitLine(this._handle, Limit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function deletes the result of the current limit check.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_CLEAR
        /// 
        /// Remote-control command(s):
        /// CALCulate:LIMit&lt;1 to 6&gt;:CLEar[:IMMediate]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 6
        /// 1 to 8 for R&amp;S FSL
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_CLEAR
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ClearLimitLineResults(int Window, int Limit)
        {
            int pInvokeResult = PInvoke.ClearLimitLineResults(this._handle, Window, Limit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures state of specified display line.
        /// 
        /// Note(s):
        /// 
        /// (1) Only display line is available for R&amp;S FSH4/8.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_THRLINE_STATE
        /// RSSPECAN_ATTR_DLINE_STATE
        /// RSSPECAN_ATTR_FLINE_STATE
        /// RSSPECAN_ATTR_TLINE_STATE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:THReshold:STATe ON | OFF
        /// CALCulate&lt;1|2&gt;:DLINe&lt;1|2&gt;:STATe ON | OFF
        /// CALCulate&lt;1|2&gt;:FLINe&lt;1|2&gt;:STATe ON | OFF
        /// CALCulate&lt;1|2&gt;:TLINe&lt;1|2&gt;:STATe ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Line">
        /// This control selects line.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Type">
        /// Selects the line to be shown.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LINE_THR     (0) - Threshold Line
        /// RSSPECAN_VAL_LINE_DISPLAY (1) - Display Line
        /// RSSPECAN_VAL_LINE_FREQ    (2) - Frequency Line
        /// RSSPECAN_VAL_LINE_TIME    (3) - Time Line
        /// 
        /// Default Value: RSSPECAN_VAL_LIMIT_LOWER (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Threshold Line - This line indicates the threshold value for the maximum/minimum search of markers with marker search functions. This value is not available for R&amp;S FSH4/8.
        /// 
        /// (2) Display Line - These lines enable the user to mark any
        /// levels in the diagram.
        /// 
        /// (3) Frequency Line - The frequency lines mark the frequencies specified in the measurement window. Frequency lines are only available with SPAN &gt; 0. This value is not available for R&amp;S FSH4/8.
        /// 
        /// (4) Time Lines - The time lines mark the times specified in the measurement window. Time lines are only available with SPAN = 0. This value is not available for R&amp;S FSH4/8.
        /// </param>
        /// <param name="State">
        /// Specifies whether selected line is enabled.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_THRLINE_STATE
        /// RSSPECAN_ATTR_DLINE_STATE
        /// RSSPECAN_ATTR_FLINE_STATE
        /// RSSPECAN_ATTR_TLINE_STATE
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDisplayLineState(int Window, int Line, int Type, bool State)
        {
            int pInvokeResult = PInvoke.ConfigureDisplayLineState(this._handle, Window, Line, Type, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures position of specified display line.
        /// 
        /// Note(s):
        /// 
        /// (1) Only display line is available for R&amp;S FSH4/8.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_THRLINE_POSITION
        /// RSSPECAN_ATTR_DLINE_POSITION
        /// RSSPECAN_ATTR_FLINE_POSITION
        /// RSSPECAN_ATTR_TLINE_POSITION
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:THReshold
        /// CALCulate&lt;1|2&gt;:DLINe&lt;1|2&gt;
        /// CALCulate&lt;1|2&gt;:FLINe&lt;1|2&gt;
        /// CALCulate&lt;1|2&gt;:TLINe&lt;1|2&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Line">
        /// This control selects line.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Type">
        /// Selects the line to be shown.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LINE_THR     (0) - Threshold Line
        /// RSSPECAN_VAL_LINE_DISPLAY (1) - Display Line
        /// RSSPECAN_VAL_LINE_FREQ    (2) - Frequency Line
        /// RSSPECAN_VAL_LINE_TIME    (3) - Time Line
        /// 
        /// Default Value: RSSPECAN_VAL_LIMIT_LOWER (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Threshold Line - This line indicates the threshold value for the maximum/minimum search of markers with marker search functions. This value is not available for R&amp;S FSH4/8. 
        /// 
        /// (2) Display Line - These lines enable the user to mark any
        /// levels in the diagram.
        /// 
        /// (3) Frequency Line - The frequency lines mark the frequencies specified in the measurement window. Frequency lines are only available with SPAN &gt; 0. This value is not available for R&amp;S FSH4/8.
        /// 
        /// (4) Time Lines - The time lines mark the times specified in the measurement window. Time lines are only available with SPAN = 0. This value is not available for R&amp;S FSH4/8.
        /// </param>
        /// <param name="Position">
        /// Defines position of selected line.
        /// 
        /// Valid Values:
        /// Threshold Line: not checked
        /// Display Line: not checked
        /// Frequency Line: not checked, 0 to fmax
        /// Time Line: 0.0 to 1000.0 s
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_THRLINE_POSITION
        /// RSSPECAN_ATTR_DLINE_POSITION
        /// RSSPECAN_ATTR_FLINE_POSITION
        /// RSSPECAN_ATTR_TLINE_POSITION
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDisplayLinePosition(int Window, int Line, int Type, double Position)
        {
            int pInvokeResult = PInvoke.ConfigureDisplayLinePosition(this._handle, Window, Line, Type, Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates or deactivates the audio beep. If active, the R&amp;S FSH outputs a beep each time a limit is exceeded.
        /// The numeric suffix &lt;1|2&gt; is irrelevant.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LIMIT_AUDIO_BEEP
        /// 
        /// Remote-control command(s):
        /// CALCulate:LIMit&lt;1|2&gt;:BEEP[:STATe]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Audio_Beep">
        /// This control sets the audio beep state.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value:
        /// VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LIMIT_AUDIO_BEEP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureLimitLineAudioBeep(bool Audio_Beep)
        {
            int pInvokeResult = PInvoke.ConfigureLimitLineAudioBeep(this._handle, System.Convert.ToUInt16(Audio_Beep));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the names of all active limit lines.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ACTive?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="String_Size">
        /// This control sets the allocated length of Active Limit Lines string.
        /// 
        /// Default Value: 512
        /// </param>
        /// <param name="Active_Limit_Lines">
        /// This control returns the control lines of the user ports.
        /// 
        /// 
        /// </param>
        /// <param name="Returned_String_Length">
        /// Returns the length of string returned from the instrument.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetActiveLimitLines(int String_Size, System.Text.StringBuilder Active_Limit_Lines, out int Returned_String_Length)
        {
            int pInvokeResult = PInvoke.GetActiveLimitLines(this._handle, String_Size, Active_Limit_Lines, out Returned_String_Length);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the names of all limit lines stored on the hard disk.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is not available for R&amp;S FSL or R&amp;S ESL.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:CATalog?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Number_of_Limit_Lines">
        /// This control returns number of names of all limit lines.
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the string you specify for the catalog list parameter.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// Valid Values:
        /// any number
        /// 
        /// Default Value: 0
        /// 
        /// </param>
        /// <param name="Limit_Lines_List">
        /// This control returns the names of all limit lines stored on the hard disk.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// if you pass 0 for buffer Size, you can use VI_NULL for this parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetLimitLineCatalog(out int Number_of_Limit_Lines, int Buffer_Size, System.Text.StringBuilder Limit_Lines_List)
        {
            int pInvokeResult = PInvoke.GetLimitLineCatalog(this._handle, out Number_of_Limit_Lines, Buffer_Size, Limit_Lines_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the active measurement window.
        /// In FULL SCREEN mode, the measurements are only performed in the active measurement window. Measurements are therefore initiated in the active window and result queries (marker, trace data and other results) answered also in the active window.
        /// Initiating measurements and querying results in the inactive window yields an error message (execution error).
        /// In split screen mode, the selection of the active window for result queries is irrelevant.
        /// 
        /// Note(s):
        /// 
        /// (1) In FULL SCREEN mode, settings can also be performed in the inactive measurement window. They become effective as soon as the corresponding window becomes active.
        /// 
        /// (2) This function is not available in GSM/EDGE mode. In GSM/EDGE mode, the display is always set to FULL SCREEN, which corresponds to SCREEN A, which is automatically selected when the GSM/EDGE mode is activated with command INSTrument:SELect:MGSM.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SELECT_WINDOW
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:SELect
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int DisplaySetActiveWindow(int Window)
        {
            int pInvokeResult = PInvoke.DisplaySetActiveWindow(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the active measurement window.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_QUERY_ACTIVE_WINDOW
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:ACTive?
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control returns the active measurement window.
        /// 
        /// Returned Values:
        /// 1 - Screen A
        /// 2 - Screen B
        /// 3 - Screen C
        /// 4 - Screen D
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int DisplayGetActiveWindow(out int Window)
        {
            int pInvokeResult = PInvoke.DisplayGetActiveWindow(this._handle, out Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function controls the apperance of display elements.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_FREQ_STATE
        /// RSSPECAN_ATTR_DISP_LOGO_STATE
        /// RSSPECAN_ATTR_DISP_TIME_STATE
        /// 
        /// Remote-control command(s):
        /// DISPlay:ANNotation:FREQuency ON | OFF
        /// DISPlay:LOGO ON | OFF
        /// DISPlay[:WINDow&lt;1|2&gt;]:TIME ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Frequency">
        /// This control switches the x-axis annotation on or off.
        /// 
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_FREQ_STATE
        /// </param>
        /// <param name="Logo">
        /// This control switches the factory logo on the screen on or off.
        /// 
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_LOGO_STATE
        /// 
        /// Note:
        /// 
        /// (1) This control has no effect for R&amp;S FSL and Time set On
        /// </param>
        /// <param name="Time">
        /// This control switches on or off the screen display of date and time.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_TIME_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDisplayControl(bool Frequency, bool Logo, bool Time)
        {
            int pInvokeResult = PInvoke.ConfigureDisplayControl(this._handle, System.Convert.ToUInt16(Frequency), System.Convert.ToUInt16(Logo), System.Convert.ToUInt16(Time));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function controls the apperance of display title.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_COMMENT_STATE
        /// RSSPECAN_ATTR_DISP_COMMENT
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TEXT:STATe ON | OFF
        /// DISPlay[:WINDow&lt;1|2&gt;]:TEXT[:DATA]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// This control switches the x-axis annotation on or off. The driver uses this value to set the RSSPECAN_ATTR_DISP_COMMENT_STATE attribute.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_COMMENT_STATE
        /// </param>
        /// <param name="Comment">
        /// Defines a comment (max. 20 characters) which can be displayed on the screen in the selected measurement window.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_COMMENT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDisplayComment(int Window, bool State, string Comment)
        {
            int pInvokeResult = PInvoke.ConfigureDisplayComment(this._handle, Window, System.Convert.ToUInt16(State), Comment);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures power-save mode of the display. With the power-save mode activated the display including backlight is completely switched off after the elapse of the response time (see command DISPlay:PSAVe:HOLDoff).
        /// 
        /// Note: This mode is recommended for preserving the display  especially if the instrument is exclusively operated via remote control.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_PWR_SAVE_STATE
        /// RSSPECAN_ATTR_DISP_PWR_SAVE_HOLDOFF
        /// 
        /// Remote-control command(s):
        /// DISPlay:PSAVe[:STATe] ON | OFF
        /// DISPlay:PSAVe:HOLDoff
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="State">
        /// This control switches on or off the power-save mode of the display.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_PWR_SAVE_STATE
        /// </param>
        /// <param name="Holdoff">
        /// Sets the holdoff time for the power-save mode of the display.
        /// 
        /// Valid Values:
        /// 0 to 60 minutes
        /// 
        /// Default Value: 15 minutes
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_PWR_SAVE_HOLDOFF
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDisplayPowerSave(bool State, int Holdoff)
        {
            int pInvokeResult = PInvoke.ConfigureDisplayPowerSave(this._handle, System.Convert.ToUInt16(State), Holdoff);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the measurement result display between FULL SCREEN and SPLIT SCREEN.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_FORMAT
        /// 
        /// Remote-control command(s):
        /// DISPlay:FORMat SINGle | SPLit
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Format">
        /// Switches the measurement result display between FULL SCREEN and SPLIT SCREEN. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DISP_FORMAT_SINGLE (0) - Single
        /// RSSPECAN_VAL_DISP_FORMAT_SPLIT  (1) - Split
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_DISP_FORMAT_SINGLE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_FORMAT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDisplayFormat(int Format)
        {
            int pInvokeResult = PInvoke.ConfigureDisplayFormat(this._handle, Format);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the measurement window for channel and adjacent-channel power measurements to full screen or half screen.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_WINDOW_SIZE
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:SIZE LARGe | SMALl
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window_Size">
        /// Switches the measurement window for channel and adjacent-channel power measurements to full screen or half screen.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DISP_SIZE_LARGE (0) - Large
        /// RSSPECAN_VAL_DISP_SIZE_SMALL (1) - Small
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_DISP_SIZE_SMALL (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_WINDOW_SIZE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDisplayWindowSize(int Window_Size)
        {
            int pInvokeResult = PInvoke.ConfigureDisplayWindowSize(this._handle, Window_Size);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the display range of the Y axis (level axis) in the selected measurement window with logarithmic scaling (DISP:TRAC:Y:SPAC LOG).
        /// For linear scaling, (DISP:TRAC:Y:SPAC LIN | PERC) the display range is fixed and cannot be modified.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_LOG_RANGE
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Range">
        /// Defines the display range of the Y axis (level axis) in the selected measurement window with logarithmic scaling (DISP:TRAC:Y:SPAC LOG).
        /// 
        /// Valid Values:
        /// 10.0 to 200.0 dB
        /// 
        /// Default Value: 100.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_LOG_RANGE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDisplayLogRange(int Window, double Range)
        {
            int pInvokeResult = PInvoke.ConfigureDisplayLogRange(this._handle, Window, Range);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the scale type of the Y axis (absolute or relative) in the selected measurement window.
        /// When SYSTem:DISPlay is set to OFF, this command has no immediate effect on the screen.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_AMPLITUDE_GRID_MODE
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:MODE ABSolute | RELative
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Y_Axis_Grid_Mode">
        /// Defines the scale type of the Y axis (absolute or relative) in the selected measurement window.
        /// 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ABS (0) - Absolute
        /// RSSPECAN_VAL_REL (1) - Relative
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_ABS (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_AMPLITUDE_GRID_MODE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDisplayAmplitudeGridMode(int Window, int Y_Axis_Grid_Mode)
        {
            int pInvokeResult = PInvoke.ConfigureDisplayAmplitudeGridMode(this._handle, Window, Y_Axis_Grid_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// If the external generator control option (FSP-B10) is mounted and the normalization in the NETWORK mode is activated, this value defines the power value assigned to the reference position in the selected measurement window. This value corresponds to the parameter REFERENCE VALUE in manual operation.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_REF_VALUE
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACE&lt;1 to 3&gt;:Y[:SCALe]:RVALue
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Reference_Value">
        /// If the external generator control option (FSP-B10) is mounted and the normalization in the NETWORK mode is activated, this value defines the power value assigned to the reference position in the selected measurement window. This value corresponds to the parameter REFERENCE VALUE in manual operation.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_REF_VALUE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDisplayReferenceValue(int Window, double Reference_Value)
        {
            int pInvokeResult = PInvoke.ConfigureDisplayReferenceValue(this._handle, Window, Reference_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the position of the reference value in the selected measurement window.
        /// In operating mode NETWORK (Ext. Generator Option FSP-B10) with active normalization, RPOSition defines the reference point for the output of the normalized measurement results.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_REF_POSITION
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:RPOSition
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Reference_Position">
        /// Defines the position of the reference value in the selected measurement window.
        /// 
        /// Valid Values:
        /// 0.0 to 100.0 %
        /// 
        /// Default Value: 100.0 %
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_REF_POSITION
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDisplayReferencePosition(int Window, double Reference_Position)
        {
            int pInvokeResult = PInvoke.ConfigureDisplayReferencePosition(this._handle, Window, Reference_Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the behaviour of the display during a single sweep.
        /// INITiate:DISPlay OFF means that the display is switched off during the measurement, INITiate:DISPlay ON means that the display is switched on during the measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_SINGLE_SWEEP
        /// 
        /// Remote-control command(s):
        /// INITiate:DISPlay ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="State">
        /// This control configures the behaviour of the display during a single sweep.
        /// 
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_SINGLE_SWEEP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDisplaySingleSweep(bool State)
        {
            int pInvokeResult = PInvoke.ConfigureDisplaySingleSweep(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the update of all display elements during remote control.
        /// 
        /// Note:
        /// 
        /// The best performance is obtained when the display output is switched off during remote control.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_UPDATE
        /// 
        /// Remote-control command(s):
        /// SYSTem:DISPlay:UPDate ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Display_In_Remote">
        /// This control enables or disables instrument display when in remote control mode.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_UPDATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDisplayUpdate(bool Display_In_Remote)
        {
            int pInvokeResult = PInvoke.ConfigureDisplayUpdate(this._handle, System.Convert.ToUInt16(Display_In_Remote));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates or deactivates the display of the front panel keys on the screen. With the display activated, the instrument can be operated on the screen using the mouse by pressing the corresponding buttons. This may be useful if the instrument is operated in a detached station by means of a remote program such as PCANYWHERE.
        /// 
        /// Notes: 
        /// 
        /// (1) With the display of the front panel keys activated, the screen resolution of the unit is set to 1024x768. Thus, only a section of the whole screen is visible on the internal LCD display, which will be moved by mouse moves. For a full display of the user interface, an external monitor has to be connected to the rear panel. When the front panel display is deactivated, the original screen resolution is restored.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_FP_KEYS
        /// 
        /// Remote-control command(s):
        /// SYSTem:DISPlay:FPANel ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Soft_Frontpanel">
        /// This control enables or disables instrument display when in remote control mode.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_FP_KEYS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDisplaySoftFrontpanel(bool Soft_Frontpanel)
        {
            int pInvokeResult = PInvoke.ConfigureDisplaySoftFrontpanel(this._handle, System.Convert.ToUInt16(Soft_Frontpanel));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function adapts the diagram limits to the current measurement results. The value range depends on the selected display.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_TRACE_Y_AUTO
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:AUTO ONCE
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:ADJust
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDisplayAutoscaleOnce()
        {
            int pInvokeResult = PInvoke.ConfigureDisplayAutoscaleOnce(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the scaling of the y-axis in the currently selected unit.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_Y_AXIS_SCALING
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:PDIVision
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Scaling">
        /// Defines the scaling of the Y axis.
        /// 
        /// Valid Values: no checking
        /// 
        /// Default Value: 1.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_Y_AXIS_SCALING
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDisplayYAxisScaling(int Window, double Scaling)
        {
            int pInvokeResult = PInvoke.ConfigureDisplayYAxisScaling(this._handle, Window, Scaling);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the display settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function can be used only for R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_STATUS_BAR
        /// RSSPECAN_ATTR_DISP_SOFTKEY_BAR
        /// RSSPECAN_ATTR_DISP_TOOL_BAR
        /// RSSPECAN_ATTR_DISP_TOUCHSCREEN_STATE
        /// 
        /// Remote-control command(s):
        /// DISPlay:SBAR[:STATe] ON | OFF
        /// DISPlay:SKEYs[:STATe] ON | OFF
        /// DISPlay:TBAR[STATe] ON | OFF
        /// DISPlay:TOUChscreen:STATe ON | OFF | FRAMe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Status_Bar_State">
        /// This control switches the statusbar under the diagram on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_STATUS_BAR
        /// </param>
        /// <param name="Softkey_Bar_State">
        /// This control switches the softkey bar on the right side of the diagram on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_SOFTKEY_BAR
        /// </param>
        /// <param name="Tool_Bar_State">
        /// This control switches the toolbar above the diagram on or off. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_TOOL_BAR
        /// </param>
        /// <param name="Touchscreen_State">
        /// This control switches the touch screen functionality on, off, or partially on.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TOUCH_OFF  (0) - Off
        /// RSSPECAN_VAL_TOUCH_ON   (1) - On
        /// RSSPECAN_VAL_TOUCH_FRAM (2) - Diagram Off
        /// 
        /// Default Value: RSSPECAN_VAL_TOUCH_ON (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_TOUCHSCREEN_STATE
        /// 
        /// Note(s):
        /// 
        /// (1) Off: Touch screen function is active for entire screen.
        /// 
        /// (2) On: Touch screen is deactivated for entire screen.
        /// 
        /// (3) Diagram Off: Touch screen is deactivated for the diagram area of the screen, but active for the surrounding softkeys, toolbars and menus.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDisplaySettings(bool Status_Bar_State, bool Softkey_Bar_State, bool Tool_Bar_State, int Touchscreen_State)
        {
            int pInvokeResult = PInvoke.ConfigureDisplaySettings(this._handle, System.Convert.ToUInt16(Status_Bar_State), System.Convert.ToUInt16(Softkey_Bar_State), System.Convert.ToUInt16(Tool_Bar_State), Touchscreen_State);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function displays the selection list of available themes for the screen display.
        /// 
        /// Note(s):
        /// 
        /// (1) This function can be used only for R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// DISPlay:THEMe:CATalog?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Number_of_Themes">
        /// This control returns number of names of all themes.
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the string you specify for the catalog list parameter.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// Valid Values:
        /// any number
        /// 
        /// Default Value: 0
        /// 
        /// </param>
        /// <param name="Theme_List">
        /// This control returns the names of all themes stored on the hard disk.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// if you pass 0 for buffer Size, you can use VI_NULL for this parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetDisplayThemeCatalog(out int Number_of_Themes, int Buffer_Size, System.Text.StringBuilder Theme_List)
        {
            int pInvokeResult = PInvoke.GetDisplayThemeCatalog(this._handle, out Number_of_Themes, Buffer_Size, Theme_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects a theme for the screen display. The theme defines the colors used for keys and screen elements, for example. Use command DISPlay:THEMe:CATalog? to get a list of possible themes.
        /// 
        /// Note(s):
        /// 
        /// (1) This function can be used only for R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_SELECT_THEME
        /// 
        /// Remote-control command(s):
        /// DISPlay:THEMe:SELect
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Theme">
        /// This control selects a theme for the screen display.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_SELECT_THEME
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDisplayThemeSelect(string Theme)
        {
            int pInvokeResult = PInvoke.ConfigureDisplayThemeSelect(this._handle, Theme);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function toggles the display of the marker table. With automatic display, the table is displayed if 2 or more markers are active.
        /// 
        /// Note(s):
        /// 
        /// (1) This function can be used only for R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_MARKER_TABLE
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:MTABle ON | OFF | AUTO
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Marker_Table">
        /// This control toggles the display of the marker table. With automatic display, the table is displayed if 2 or more markers are active.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_STATE_OFF  (0) - Off
        /// RSSPECAN_VAL_STATE_ON   (1) - On
        /// RSSPECAN_VAL_STATE_AUTO (2) - Auto
        /// 
        /// Default Value: RSSPECAN_VAL_STATE_AUTO (2)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_MARKER_TABLE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDisplayMarkerTable(int Marker_Table)
        {
            int pInvokeResult = PInvoke.ConfigureDisplayMarkerTable(this._handle, Marker_Table);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the display date format.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_DATE_FORMAT
        /// 
        /// Remote-control command(s):
        /// DISPlay:DATE:FORMat DDMMyyyy | MMDDyyyy
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Date_Format">
        /// This control sets the display date format.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DISPLAY_DATE_FORMAT_DDMM (0) - DDMMYYYY
        /// RSSPECAN_VAL_DISPLAY_DATE_FORMAT_MMDD (1) - MMDDYYYY
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_DISPLAY_DATE_FORMAT_DDMM (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_DATE_FORMAT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureDisplayDateFormat(int Date_Format)
        {
            int pInvokeResult = PInvoke.ConfigureDisplayDateFormat(this._handle, Date_Format);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function makes a screenshot of the current trace.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_SCREENSHOT
        /// 
        /// Remote-control command(s):
        /// DISPlay:STORe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int CreateDisplayScreenshot()
        {
            int pInvokeResult = PInvoke.CreateDisplayScreenshot(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the display zooming parameters.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available for R&amp;S FSV instrument only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISPLAY_ZOOM_STATE
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1..4&gt;]:ZOOM[:STATe]
        /// DISPlay[:WINDow&lt;1..4&gt;]:ZOOM:AREA
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Zoom_State">
        /// This control activates and deactivates the zoom mode.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISPLAY_ZOOM_STATE
        /// </param>
        /// <param name="x1">
        /// Defines the x1 value of the area for which the display is enlarged. The area is defined via the coordinates of the lower left (x1, y1) and top right (x2,y2) corners of a rectangle, where x and y are percentages of the complete diagram. (0,0) defines the lower left corner of the diagram area, (100,100) defines the top right corner.
        /// 
        /// Valid Values:
        /// 0.0 to 100.0 %
        /// 
        /// Default Value: 0.0 %
        /// </param>
        /// <param name="y1">
        /// Defines the y1 value of the area for which the display is enlarged. The area is defined via the coordinates of the lower left (x1, y1) and top right (x2,y2) corners of a rectangle, where x and y are percentages of the complete diagram. (0,0) defines the lower left corner of the diagram area, (100,100) defines the top right corner.
        /// 
        /// Valid Values:
        /// 0.0 to 100.0 %
        /// 
        /// Default Value: 0.0 %
        /// </param>
        /// <param name="x2">
        /// Defines the x1 value of the area for which the display is enlarged. The area is defined via the coordinates of the lower left (x1, y1) and top right (x2,y2) corners of a rectangle, where x and y are percentages of the complete diagram. (0,0) defines the lower left corner of the diagram area, (100,100) defines the top right corner.
        /// 
        /// Valid Values:
        /// 0.0 to 100.0 %
        /// 
        /// Default Value: 100.0 %
        /// </param>
        /// <param name="y2">
        /// Defines the y2 value of the area for which the display is enlarged. The area is defined via the coordinates of the lower left (x1, y1) and top right (x2,y2) corners of a rectangle, where x and y are percentages of the complete diagram. (0,0) defines the lower left corner of the diagram area, (100,100) defines the top right corner.
        /// 
        /// Valid Values:
        /// 0.0 to 100.0 %
        /// 
        /// Default Value: 100.0 %
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureDisplayZoom(int Window, bool Zoom_State, double x1, double y1, double x2, double y2)
        {
            int pInvokeResult = PInvoke.ConfigureDisplayZoom(this._handle, Window, System.Convert.ToUInt16(Zoom_State), x1, y1, x2, y2);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function resets the screen colors of all display items to their default settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_COL_PRESET
        /// 
        /// Remote-control command(s):
        /// DISPlay:CMAP&lt;1 to 26&gt;:DEFault&lt;1|2&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Default_Setting">
        /// This control selects the predefined setting to be applied.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int DisplayColorDefault(int Default_Setting)
        {
            int pInvokeResult = PInvoke.DisplayColorDefault(this._handle, Default_Setting);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function resets the screen colors of all display items to their default settings, i.e. to the color scheme.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_COL_PRES_PORTABLE
        /// 
        /// Remote-control command(s):
        /// DISPlay:CMAP:DEFault
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int DisplayColorDefaultPortable()
        {
            int pInvokeResult = PInvoke.DisplayColorDefaultPortable(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the color table of the instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_BRIGHTNESS
        /// 
        /// Remote-control command(s):
        /// DISPlay:CMAP&lt;1 to 26&gt;:HSL
        /// DISPlay:BRIGhtness
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Color_Map">
        /// This control defines the color table of the instrument.
        /// 
        /// Valid Values:
        /// 1 - Background
        /// 2 - Grid
        /// 3 - Function field + status field + data entry text
        /// 4 - Function field LED on
        /// 5 - Function field LED warn
        /// 6 - Enhancement label text
        /// 7 - Status field background
        /// 8 - Trace 1 (Default Value)
        /// 9 - Trace 2
        /// 10 - Trace 3
        /// 11 - Marker
        /// 12 - Lines
        /// 13 - Measurement status + limit check pass
        /// 14 - Limit check fail
        /// 15 - Table + softkey background
        /// 16 - Table + softkey text
        /// 17 - Table selected field text
        /// 18 - Table selected field background
        /// 19 - Table + data entry field opaq titlebar
        /// 20 - Data entry field opaq text
        /// 21 - Data entry field opaq background
        /// 22 - 3D shade bright part
        /// 23 - 3D shade dark part
        /// 24 - Softkey state on
        /// 25 - Softkey state data entry
        /// 26 - Logo
        /// 
        /// Note:
        /// 
        /// (1) FSQ handles colour map different way than FSEx driver. Be careful when using this control. Below is the table with FSEx old values:
        /// 
        ///  1 - Trace 1
        ///  2 - Trace 2
        ///  3 - Trace 3
        ///  4 - Trace 4
        ///  5 - Marker
        ///  6 - Grid
        ///  7 - Softkey State On
        ///  8 - Softkey State Data Entry
        ///  9 - Softkey State OFF
        ///  10 - Softkey Shade
        ///  11 - Text
        ///  12 - Title
        ///  13 - Background
        /// 
        /// </param>
        /// <param name="TINT">
        /// This control sets the hue = TINT.
        /// 
        /// Valid Values:
        /// 0.0 to 100.0 %
        /// 
        /// Default Value: 30.0 %
        /// </param>
        /// <param name="SATURATION">
        /// This control sets the sat = SATURATION.
        /// 
        /// Valid Values:
        /// 0.0 to 100.0 %
        /// 
        /// Default Value: 80.0 %
        /// </param>
        /// <param name="BRIGHTNESS">
        /// This control sets the lum = BRIGHTNESS.
        /// 
        /// Valid Values: 0.0 to 100.0 %
        /// 
        /// Default Value: 100.0 %
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_BRIGHTNESS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDisplayColor(int Color_Map, double TINT, double SATURATION, double BRIGHTNESS)
        {
            int pInvokeResult = PInvoke.ConfigureDisplayColor(this._handle, Color_Map, TINT, SATURATION, BRIGHTNESS);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the color scheme of the display to either color or black and white.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_COL_PORTABLE
        /// 
        /// Remote-control command(s):
        /// DISPlay:CMAP COLor | BW
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Color_Scheme">
        /// This control sets the color scheme of the display to either color or black and white.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DISPLAY_COLOR_SCHEME_COL (0) - Color
        /// RSSPECAN_VAL_DISPLAY_COLOR_SCHEME_BW  (1) - Black and White
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_DISPLAY_COLOR_SCHEME_COL (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_COL_PORTABLE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureDisplayColorPortable(int Color_Scheme)
        {
            int pInvokeResult = PInvoke.ConfigureDisplayColorPortable(this._handle, Color_Scheme);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the color table of the instrument using predefined colors.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_COL_PREDEFINED
        /// 
        /// Remote-control command(s):
        /// DISPlay:CMAP&lt;1 to 26&gt;:PDEFined
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Color_Map">
        /// This control defines the color table of the instrument.
        /// 
        /// Valid Values:
        /// 1 - Background
        /// 2 - Grid
        /// 3 - Function field + status field + data entry text
        /// 4 - Function field LED on
        /// 5 - Function field LED warn
        /// 6 - Enhancement label text
        /// 7 - Status field background
        /// 8 - Trace 1 (Default Value)
        /// 9 - Trace 2
        /// 10 - Trace 3
        /// 11 - Marker
        /// 12 - Lines
        /// 13 - Measurement status + limit check pass
        /// 14 - Limit check fail
        /// 15 - Table + softkey background
        /// 16 - Table + softkey text
        /// 17 - Table selected field text
        /// 18 - Table selected field background
        /// 19 - Table + data entry field opaq titlebar
        /// 20 - Data entry field opaq text
        /// 21 - Data entry field opaq background
        /// 22 - 3D shade bright part
        /// 23 - 3D shade dark part
        /// 24 - Softkey state on
        /// 25 - Softkey state data entry
        /// 26 - Logo
        /// 
        /// Note:
        /// 
        /// (1) FSQ handles colour map different way than FSEx driver. Be careful when using this control. Below is the table with FSEx old values:
        /// 
        ///  1 - Trace 1
        ///  2 - Trace 2
        ///  3 - Trace 3
        ///  4 - Trace 4
        ///  5 - Marker
        ///  6 - Grid
        ///  7 - Softkey State On
        ///  8 - Softkey State Data Entry
        ///  9 - Softkey State OFF
        ///  10 - Softkey Shade
        ///  11 - Text
        ///  12 - Title
        ///  13 - Background
        /// 
        /// </param>
        /// <param name="Predefined_Colors">
        /// This control defines the color table of the instrument using predefined color values.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DISP_COL_BLAC (0) - Black
        /// RSSPECAN_VAL_DISP_COL_BLUE (1) - Blue
        /// RSSPECAN_VAL_DISP_COL_BROW (2) - Brown
        /// RSSPECAN_VAL_DISP_COL_GRE  (3) - Green
        /// RSSPECAN_VAL_DISP_COL_CYAN (4) - Cyan
        /// RSSPECAN_VAL_DISP_COL_RED  (5) - Red
        /// RSSPECAN_VAL_DISP_COL_MAG  (6) - Magenta
        /// RSSPECAN_VAL_DISP_COL_YELL (7) - Yellow 
        /// RSSPECAN_VAL_DISP_COL_WHIT (8) - White 
        /// RSSPECAN_VAL_DISP_COL_DGRA (9) - Dark Gray 
        /// RSSPECAN_VAL_DISP_COL_LGRA (10) - Light Gray 
        /// RSSPECAN_VAL_DISP_COL_LBLU (11) - Light Blue 
        /// RSSPECAN_VAL_DISP_COL_LGRE (12) - Light Green 
        /// RSSPECAN_VAL_DISP_COL_LCY  (13) - Light Cyan 
        /// RSSPECAN_VAL_DISP_COL_LRED (14) - Light Red 
        /// RSSPECAN_VAL_DISP_COL_LMAG (15) - Light Magenta
        /// 
        /// Default Value: RSSPECAN_VAL_DISP_COL_BLAC (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_COL_PREDEFINED
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDisplayColorByPredefined(int Color_Map, int Predefined_Colors)
        {
            int pInvokeResult = PInvoke.ConfigureDisplayColorByPredefined(this._handle, Color_Map, Predefined_Colors);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function generates a transducer factor &lt;name&gt; using normalized trace data. 
        /// 
        /// Note:
        /// 
        /// (1) The function is only available when normalization is switched on.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TFAC_SEL_NAME
        /// RSSPECAN_ATTR_TFAC_STATE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:SELect
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Transducer_State">
        /// This command switches the selected transducer factor on or off.
        /// 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On 
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TFAC_STATE
        /// </param>
        /// <param name="Transducer_Name">
        /// This command selects the transducer factor name.
        /// 
        /// 
        /// Valid Values: up to 8 characters
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TFAC_SEL_NAME
        /// 
        /// Note:
        /// (1) This control has effect only if State set to On
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SelectTransducerFactor(int Window, bool Transducer_State, string Transducer_Name)
        {
            int pInvokeResult = PInvoke.SelectTransducerFactor(this._handle, Window, System.Convert.ToUInt16(Transducer_State), Transducer_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the transducer factor. The transducer is stored on harddisk any may be activated by function SelectTransducerFactor.
        /// 
        /// Attributes(s):
        /// RSSPECAN_ATTR_TFAC_SEL_NAME
        /// RSSPECAN_ATTR_TFAC_SCALING
        /// -
        /// RSSPECAN_ATTR_TFAC_UNIT
        /// RSSPECAN_ATTR_TFAC_COMMENT
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:SELect
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:SCALing LINear | LOGarithmic
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:DATA 
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:UNIT DB | DBM | DBMV | DBUV | DBUV/M | DBUA | DBUA/M | DBPW | DBPT
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:COMMent
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Name">
        /// This command selects the transducer factor designated by &lt;name&gt;. If &lt;name&gt; does not exist yet, a new transducer factor is created.
        /// 
        /// Valid Values: up to 8 characters
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TFAC_SEL_NAME
        /// </param>
        /// <param name="Unit">
        /// This control defines the unit of the transducer factor selected.
        /// 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TFAC_UNIT_DB    (0) - DB 
        /// RSSPECAN_VAL_TFAC_UNIT_DBM   (1) - DBM
        /// RSSPECAN_VAL_TFAC_UNIT_DBMV  (2) - DBMV
        /// RSSPECAN_VAL_TFAC_UNIT_DBUV  (3) - DB?V
        /// RSSPECAN_VAL_TFAC_UNIT_DBUVM (4) - DB?V/M
        /// RSSPECAN_VAL_TFAC_UNIT_DBUA  (5) - DB?A
        /// RSSPECAN_VAL_TFAC_UNIT_DBUAM (6) - DB?A/M
        /// RSSPECAN_VAL_TFAC_UNIT_DBPW  (7) - DBPW
        /// RSSPECAN_VAL_TFAC_UNIT_DBPT  (8) - DBPT
        /// 
        /// Default Value: RSSPECAN_VAL_TFAC_UNIT_DB    (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TFAC_UNIT
        /// </param>
        /// <param name="Interpolation">
        /// This control defines whether the frequency scaling of the transducer factor is linear or logarithmic.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIN (0) - Logarithmic
        /// RSSPECAN_VAL_LOG (1) - Linear 
        /// 
        /// Default Value: RSSPECAN_VAL_LOG (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TFAC_SCALING
        /// </param>
        /// <param name="Comment">
        /// Sets the comment for the selected transducer factor. 
        /// 
        /// Valid Values: any string
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TFAC_COMMENT
        /// </param>
        /// <param name="No_of_Test_Points">
        /// Specifies the number of frequency,level pairs. This value corresponds with number of arrays members (Frequency and Level controls). 
        /// 
        /// 
        /// Valid Values:
        ///  0 to max memory
        /// 
        /// Default Value:
        ///  0
        /// 
        /// </param>
        /// <param name="Frequency_Data">
        /// This array defines the frequency part (in Hz) of the test points for the selected transducer factor.  The frequencies must be in ascending order.
        /// 
        /// Valid Values: not range checked
        /// 
        /// Default Value: none
        /// 
        /// </param>
        /// <param name="Level_Data">
        /// This array defines the level part of the test points for the selected transducer factor.
        /// The level values are sent as dimensionless numbers; the unit is specified by control Unit.
        /// 
        /// Valid Values: not range checked
        /// 
        /// Default Value: none
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTransducerFactor(int Window, string Name, int Unit, int Interpolation, string Comment, int No_of_Test_Points, double[] Frequency_Data, double[] Level_Data)
        {
            int pInvokeResult = PInvoke.ConfigureTransducerFactor(this._handle, Window, Name, Unit, Interpolation, Comment, No_of_Test_Points, Frequency_Data, Level_Data);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function allows automatic reference level offset adaption to restore the original dynamic range by also shifting the reference level by the maximum value of the transducer factor. 
        /// 
        /// Note:
        /// 
        /// Command SENS:CORR:TRAN:SEL has to be sent prior to this function.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TFAC_ADJ_STATE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:ADJust:RLEVel[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Ref_Lev_Adj">
        /// Allows automatic reference level offset adaption to restore the original dynamic range by also shifting the reference level by the maximum value of the transducer factor.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On 
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TFAC_ADJ_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTransducerFactorRefLevAdj(int Window, bool Ref_Lev_Adj)
        {
            int pInvokeResult = PInvoke.ConfigureTransducerFactorRefLevAdj(this._handle, Window, System.Convert.ToUInt16(Ref_Lev_Adj));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on the display of the active transducer factor or set.
        /// 
        /// Note:
        /// 
        /// (1) Prior to this command, the command SENS:CORR:TRAN:SEL must be sent.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TFAC_DISPLAY
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:VIEW ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// This control switches on or off the power-save mode of the display.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TFAC_DISPLAY
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDisplayTransducerState(int Window, bool State)
        {
            int pInvokeResult = PInvoke.ConfigureDisplayTransducerState(this._handle, Window, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the names of all transducer factors stored on the harddisk.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is not available for R&amp;S FSL, R&amp;S ESL or R&amp;S FMU or R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:CATalog?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Number_of_Transducer_Factors">
        /// This control returns number of names of all transducer factors.
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the string you specify for the catalog list parameter.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// Valid Values:
        /// any number
        /// 
        /// Default Value: 0
        /// 
        /// </param>
        /// <param name="Transducer_Factors_List">
        /// This control returns the names of all transducer factors stored on the hard disk.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// if you pass 0 for buffer Size, you can use VI_NULL for this parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetTransducerFactorCatalog(out int Number_of_Transducer_Factors, int Buffer_Size, System.Text.StringBuilder Transducer_Factors_List)
        {
            int pInvokeResult = PInvoke.GetTransducerFactorCatalog(this._handle, out Number_of_Transducer_Factors, Buffer_Size, Transducer_Factors_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function deletes the active transducer factor.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TFAC_SEL_NAME
        /// RSSPECAN_ATTR_TFAC_DELETE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:SELect
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:DELete
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Transducer_Name">
        /// This command selects the transducer factor name. The driver uses this value to set the RSSPECAN_ATTR_TFAC_SEL_NAME attribute.
        /// 
        /// 
        /// Valid Values: up to 8 characters
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TFAC_SEL_NAME
        /// 
        /// Note:
        /// (1) This control has effect only if State set to On
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int DeleteTransducerFactor(int Window, string Transducer_Name)
        {
            int pInvokeResult = PInvoke.DeleteTransducerFactor(this._handle, Window, Transducer_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the active transducer factor. Any empty string is returned, if no transducer factor is active.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TFAC_ACT_NAME
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:ACTive?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Active_Transducer">
        /// This control the active transducer factor. Any empty string is returned, if no transducer factor is active.
        /// 
        /// The minimal size of 512 is recomended for this value.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TFAC_ACT_NAME
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetTransducerActive(System.Text.StringBuilder Active_Transducer)
        {
            int pInvokeResult = PInvoke.GetTransducerActive(this._handle, Active_Transducer);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function if the currently selected transducer is an isotropic antenna.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TRACKING_GENERATOR_ISOTROPIC_ANTENA_STATE
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CORRection:TRANsducer&lt;1...2&gt;:ISOTropic[:STATe]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Transducer">
        /// This control sets the transducer.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="State">
        /// This control displays if the currently selected transducer is an isotropic antenna.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Transducer is not an isotropic antenna
        /// VI_TRUE  (1) - Transducer is an isotropic antenna
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsfsh_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int QueryIsotropicAntennaState(int Transducer, out bool State)
        {
            ushort StateAsUShort;
            int pInvokeResult = PInvoke.QueryIsotropicAntennaState(this._handle, Transducer, out StateAsUShort);
            State = System.Convert.ToBoolean(StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the printer output medium and the data format of the printout.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_HCOPY_DEVICE_DESTINATION
        /// RSSPECAN_ATTR_HCOPY_DEVICE_LANG_OUT_FORM
        /// RSSPECAN_ATTR_HCOPY_DEVICE_ORIENTATION
        /// 
        /// Remote-control command(s):
        /// HCOPy:DESTination&lt;1|2&gt; 'MMEM'|'SYST:COMM:PRIN'|'SYST:COMM:CLIP'
        /// HCOPy:DEVice:LANGuage&lt;1|2&gt; GDI | WMF | EWMF | BMP | JPG | PNG
        /// HCOPy:PAGE:ORIentation&lt;1|2&gt; LANDscape | PORTrait
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Device">
        /// Sets the device number.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Destination">
        /// Defines the output associated with device configuration.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_HCOPY_DEST_BMP (0) - Bitmap
        /// RSSPECAN_VAL_HCOPY_DEST_WMF (1) - Window Metafile
        /// RSSPECAN_VAL_HCOPY_DEST_EMF (2) - Enhanced Metafile
        /// RSSPECAN_VAL_HCOPY_DEST_CLP (3) - Clipboard
        /// RSSPECAN_VAL_HCOPY_DEST_PRN (4) - Printer
        /// RSSPECAN_VAL_HCOPY_DEST_JPG (5) - JPG
        /// RSSPECAN_VAL_HCOPY_DEST_PNG (6) - PNG
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_HCOPY_DEVICE_DESTINATION
        /// RSSPECAN_ATTR_HCOPY_DEVICE_LANG_OUT_FORM
        /// 
        /// Note(s):
        /// 
        /// (1) Bitmap - Directs the hardcopy to a file and sets the format to bitmap
        /// 
        /// (2) Window Matafile - Directs the hardcopy to a file and sets the format to WMF
        /// 
        /// (3) Enhanced Metafile - Directs the hardcopy to a file and sets the format to EWMF
        /// 
        /// (4) Clipboard - Directs the hardcopy to the clipboard and sets the format to EWMF
        /// 
        /// (5) Printer - Directs the hardcopy to the printer and sets the format to GDI. Printer is selected with function ... or attribute ...
        /// 
        /// (6) JPG - Directs the hardcopy to a file and sets the format to JPG
        /// 
        /// (7) PNG - Directs the hardcopy to a file and sets the format to PNG
        /// </param>
        /// <param name="Page_Orientation">
        /// The command selects the format of the output (portrait and landscape) (hardcopy unit 1 or 2).
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_HCOPY_DEVICE_ORIENT_LAND (0) - Landscape
        /// RSSPECAN_VAL_HCOPY_DEVICE_ORIENT_PORT (1) - Portrait 
        /// 
        /// Default Value: RSSPECAN_VAL_HCOPY_DEVICE_ORIENT_PORT (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_HCOPY_DEVICE_ORIENTATION
        /// 
        /// Note:
        /// 
        /// (1)  This setting will be ignored for 
        /// Destination &lt;&gt; RSSPECAN_VAL_HCOPY_DEST_PRN (Printer).
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureHardcopyDevice(int Device, int Destination, int Page_Orientation)
        {
            int pInvokeResult = PInvoke.ConfigureHardcopyDevice(this._handle, Device, Destination, Page_Orientation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function aborts a running hardcopy output.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_HCOPY_ABORT
        /// 
        /// Remote-control command(s):
        /// HCOPy:ABORt
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int HardcopyAbort()
        {
            int pInvokeResult = PInvoke.HardcopyAbort(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts a hardcopy output of the selected items.
        ///  
        /// Attribute(s):
        /// RSSPECAN_ATTR_HCOPY_PRINT_SCREEN
        /// RSSPECAN_ATTR_HCOPY_PRINT_TAB
        /// RSSPECAN_ATTR_HCOPY_PRINT_TRACE
        /// RSSPECAN_ATTR_HCOPY_PRINT
        /// 
        /// Remote-control command(s):
        /// HCOPy:ITEM:ALL
        /// HCOPy:ITEM:WINDow&lt;1|2&gt;:TABle:STATe ON | OFF
        /// HCOPy:ITEM:WINDow&lt;1|2&gt;:TRACe:STATe ON | OFF
        /// HCOPy[:IMMediate&lt;1|2&gt;]
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Device">
        /// Sets the device number.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// 
        /// Note(s):
        /// (1) This control is not valid for R&amp;S FSH4, FSH8 instruments.
        /// </param>
        /// <param name="Items">
        /// Selects the items to be output.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_HCOP_ALL   (0) - All
        /// RSSPECAN_VAL_HCOP_TABLE (1) - Table
        /// RSSPECAN_VAL_HCOP_TRACE (2) - Trace
        /// 
        /// Default Value: RSSPECAN_VAL_HCOP_ALL   (0)
        /// 
        /// Note(s):
        /// (1) All - outputs all items
        ///     Table - outputs table
        ///     Trace - outputs trace
        /// 
        /// (2) This control is not valid for R&amp;S FSH4, FSH8 instruments.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int HardcopyPrint(int Device, int Items)
        {
            int pInvokeResult = PInvoke.HardcopyPrint(this._handle, Device, Items);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts a hardcopy output. The numeric suffix selects which printer configuration (device 1 or 2) is to be used for the hardcopy output. If there is no suffix, configuration 1 is automatically selected. If the output is printed to a file (see HCOPy:DESTination&lt;1|2&gt; command), the file name used in the last saving process is automatically counted up to the next unused name.
        ///  
        /// Attribute(s):
        /// RSSPECAN_ATTR_HCOPY_PRINT_NEXT
        /// 
        /// Remote-control command(s):
        /// HCOPy[:IMMediate&lt;1|2&gt;]:NEXT
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Device">
        /// Sets the device number.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int HardcopyPrintNext(int Device)
        {
            int pInvokeResult = PInvoke.HardcopyPrintNext(this._handle, Device);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines a destination file for the hadcopy output.
        ///  
        /// Attribute(s):
        /// RSSPECAN_ATTR_HCOPY_FILE_NAME
        /// 
        /// Remote-control command(s):
        /// MMEMory:NAME
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Name">
        /// Sets the hardcopy file name.
        /// 
        /// Valid Values:
        /// any string
        /// 
        /// Default Value: "D:\\output"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_HCOPY_FILE_NAME
        /// 
        /// Note(s):
        /// (1) This value has effect only for file output (BMP, WMF, EWMF)
        /// 
        /// (2) If no file extension is set, the appropriate suffix will be set automatically. Suffix depends on file type set in rsspecan_ConfigureHardcopyDevice (Destination)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int HardcopySetFileName(string Name)
        {
            int pInvokeResult = PInvoke.HardcopySetFileName(this._handle, Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the list of printers under Windows NT.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_HCOPY_PRINTER_FIRST
        /// RSSPECAN_ATTR_HCOPY_PRINTER_NEXT
        /// 
        /// Remote-control command(s):
        /// SYSTem:COMMunicate:PRINter:ENUMerate:FIRSt?
        /// SYSTem:COMMunicate:PRINter:ENUMerate:NEXT?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Buffer_Size">
        /// This control indicates the allocated memory in Printer List.
        /// If not enought memory is allocated, function returns the warning in status code.
        /// 
        /// Valid Values:
        /// &gt; 0
        /// 
        /// Default Value: 512
        /// </param>
        /// <param name="Printer_List">
        /// This control returns list of printers available under  Windows NT system. Names are separated using commas (,).
        /// 
        /// Note:
        /// 
        /// If no printer is configured an empty string is output.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int HardcopyGetPrinterList(int Buffer_Size, System.Text.StringBuilder Printer_List)
        {
            int pInvokeResult = PInvoke.HardcopyGetPrinterList(this._handle, Buffer_Size, Printer_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects one of the printers installed under Windows NT. Prior to use this function get list of available printers using function Hardcopy Get Printer List.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_HCOPY_PRINTER
        /// 
        /// Remote-control command(s):
        /// SYSTem:COMMunicate:PRINter:SELect&lt;1|2&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Device">
        /// Sets the device number.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Printer_Name">
        /// This control sets printer name.
        /// 
        /// Valid Values: up to 128 chars
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_HCOPY_PRINTER
        /// 
        /// Note(s):
        /// 
        /// (1) Valid names are taken using function Hardcopy Get Printer List.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int HardcopySetPrinter(int Device, string Printer_Name)
        {
            int pInvokeResult = PInvoke.HardcopySetPrinter(this._handle, Device, Printer_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the comment text for measurement window.
        ///  
        /// Attribute(s):
        /// RSSPECAN_ATTR_HCOPY_COMM_SCR
        /// 
        /// Remote-control command(s):
        /// HCOPy:ITEM:WINDow&lt;1|2&gt;:TEXT
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Comment">
        /// Sets the hardcopy comment.
        /// 
        /// Valid Values:
        /// any valid string
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_HCOPY_COMM_SCR
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int HardcopyComment(string Comment)
        {
            int pInvokeResult = PInvoke.HardcopyComment(this._handle, Comment);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function includes or removes the time and date in the printout.
        ///  
        /// Attribute(s):
        /// RSSPECAN_ATTR_HCOPY_TIME_DATE
        /// 
        /// Remote-control command(s):
        /// HCOPy:TDSTamp:STATe&lt;1|2&gt; ON | OFF
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Device">
        /// Sets the device number.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Time_and_Date_Stamp">
        /// Includes or removes the time and date in the printout.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_HCOPY_TIME_DATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int HardcopyTimeDate(int Device, bool Time_and_Date_Stamp)
        {
            int pInvokeResult = PInvoke.HardcopyTimeDate(this._handle, Device, System.Convert.ToUInt16(Time_and_Date_Stamp));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function resets the colors for a hardcopy to the selected default settings. DEFault1(SCREEN COLORS, but background white), DEFault2 (OPTIMIZED COLOR SET) and DEFault3 (USER DEFINED). The numeric suffix in CMAP is not significant.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_HCOPY_COLOR
        /// RSSPECAN_ATTR_HCOPY_COLOR_DEF
        /// 
        /// Remote-control command(s):
        /// HCOPy:CMAP&lt;1 to 26&gt;:DEFault 1|2|3
        /// HCOPy:DEVice:COLor ON|OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Color">
        /// This control sets color or monochrome hardcopy of the screen.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_HCOPY_COLOR
        /// 
        /// </param>
        /// <param name="Default_Setting">
        /// This control selects the predefined setting to be applied.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_HCOPY_COLOR_DEF
        /// 
        /// Note(s):
        /// (1) 1 - SCREEN COLORS, but background white
        ///     2 - OPTIMIZED COLOR SET
        ///     3 - USER DEFINED
        /// 
        /// (2) This contorl has effect only if Color is set On.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int HardcopyColor(bool Color, int Default_Setting)
        {
            int pInvokeResult = PInvoke.HardcopyColor(this._handle, System.Convert.ToUInt16(Color), Default_Setting);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the color table in USER DEFINED COLORS mode.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// HCOPy:CMAP&lt;1 to 26&gt;:HSL &lt;hue&gt;,&lt;sat&gt;,&lt;lum&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Color_Map">
        /// This control defines the color table of the instrument.
        /// 
        /// Valid Values:
        /// 1 - Background
        /// 2 - Grid
        /// 3 - Function field + status field + data entry text
        /// 4 - Function field LED on
        /// 5 - Function field LED warn
        /// 6 - Enhancement label text
        /// 7 - Status field background
        /// 8 - Trace 1 (Default Value)
        /// 9 - Trace 2
        /// 10 - Trace 3
        /// 11 - Marker
        /// 12 - Lines
        /// 13 - Measurement status + limit check pass
        /// 14 - Limit check fail
        /// 15 - Table + softkey background
        /// 16 - Table + softkey text
        /// 17 - Table selected field text
        /// 18 - Table selected field background
        /// 19 - Table + data entry field opaq titlebar
        /// 20 - Data entry field opaq text
        /// 21 - Data entry field opaq background
        /// 22 - 3D shade bright part
        /// 23 - 3D shade dark part
        /// 24 - Softkey state on
        /// 25 - Softkey state data entry
        /// 26 - Logo
        /// 
        /// Note:
        /// 
        /// (1) FSQ handles colour map different way than FSEx driver. Be careful when using this control. Below is the table with FSEx old values:
        /// 
        ///  1 - Trace 1
        ///  2 - Trace 2
        ///  3 - Trace 3
        ///  4 - Trace 4
        ///  5 - Marker
        ///  6 - Grid
        ///  7 - Softkey State On
        ///  8 - Softkey State Data Entry
        ///  9 - Softkey State OFF
        ///  10 - Softkey Shade
        ///  11 - Text
        ///  12 - Title
        ///  13 - Background
        /// 
        /// </param>
        /// <param name="TINT">
        /// This control sets the hue = TINT.
        /// 
        /// Valid Values:
        /// 0.0 to 100.0 %
        /// 
        /// Default Value: 30.0 %
        /// </param>
        /// <param name="SATURATION">
        /// This control sets the sat = SATURATION.
        /// 
        /// Valid Values:
        /// 0.0 to 100.0 %
        /// 
        /// Default Value: 80.0 %
        /// </param>
        /// <param name="BRIGHTNESS">
        /// This control sets the lum = BRIGHTNESS.
        /// 
        /// Valid Values: 0.0 to 100.0 %
        /// 
        /// Default Value: 100.0 %
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureHardcopyColor(int Color_Map, double TINT, double SATURATION, double BRIGHTNESS)
        {
            int pInvokeResult = PInvoke.ConfigureHardcopyColor(this._handle, Color_Map, TINT, SATURATION, BRIGHTNESS);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the color table in USER DEFINED COLORS using predefined color values.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_HCOPY_COLOR_PREDEFINED
        /// 
        /// Remote-control command(s):
        /// HCOPy:CMAP&lt;1 to 26&gt;:PDEFined BLACk | BLUE | BROWn | GREen | CYAN | RED | MAGenta | YELLow | WHITe | DGRAy | LGRAy | LBLUe | LGREen | LCYan | LRED | LMAGenta
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Color_Map">
        /// This control defines the color table of the instrument.
        /// 
        /// Valid Values:
        /// 1 - Background
        /// 2 - Grid
        /// 3 - Function field + status field + data entry text
        /// 4 - Function field LED on
        /// 5 - Function field LED warn
        /// 6 - Enhancement label text
        /// 7 - Status field background
        /// 8 - Trace 1 (Default Value)
        /// 9 - Trace 2
        /// 10 - Trace 3
        /// 11 - Marker
        /// 12 - Lines
        /// 13 - Measurement status + limit check pass
        /// 14 - Limit check fail
        /// 15 - Table + softkey background
        /// 16 - Table + softkey text
        /// 17 - Table selected field text
        /// 18 - Table selected field background
        /// 19 - Table + data entry field opaq titlebar
        /// 20 - Data entry field opaq text
        /// 21 - Data entry field opaq background
        /// 22 - 3D shade bright part
        /// 23 - 3D shade dark part
        /// 24 - Softkey state on
        /// 25 - Softkey state data entry
        /// 26 - Logo
        /// 
        /// Note:
        /// 
        /// (1) FSQ handles colour map different way than FSEx driver. Be careful when using this control. Below is the table with FSEx old values:
        /// 
        ///  1 - Trace 1
        ///  2 - Trace 2
        ///  3 - Trace 3
        ///  4 - Trace 4
        ///  5 - Marker
        ///  6 - Grid
        ///  7 - Softkey State On
        ///  8 - Softkey State Data Entry
        ///  9 - Softkey State OFF
        ///  10 - Softkey Shade
        ///  11 - Text
        ///  12 - Title
        ///  13 - Background
        /// 
        /// </param>
        /// <param name="Predefined_Colors">
        /// This control defines the color table of the instrument using predefined color values.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DISP_COL_BLAC (0) - Black
        /// RSSPECAN_VAL_DISP_COL_BLUE (1) - Blue
        /// RSSPECAN_VAL_DISP_COL_BROW (2) - Brown
        /// RSSPECAN_VAL_DISP_COL_GRE  (3) - Green
        /// RSSPECAN_VAL_DISP_COL_CYAN (4) - Cyan
        /// RSSPECAN_VAL_DISP_COL_RED  (5) - Red
        /// RSSPECAN_VAL_DISP_COL_MAG  (6) - Magenta
        /// RSSPECAN_VAL_DISP_COL_YELL (7) - Yellow 
        /// RSSPECAN_VAL_DISP_COL_WHIT (8) - White 
        /// RSSPECAN_VAL_DISP_COL_DGRA (9) - Dark Gray 
        /// RSSPECAN_VAL_DISP_COL_LGRA (10) - Light Gray 
        /// RSSPECAN_VAL_DISP_COL_LBLU (11) - Light Blue 
        /// RSSPECAN_VAL_DISP_COL_LGRE (12) - Light Green 
        /// RSSPECAN_VAL_DISP_COL_LCY  (13) - Light Cyan 
        /// RSSPECAN_VAL_DISP_COL_LRED (14) - Light Red 
        /// RSSPECAN_VAL_DISP_COL_LMAG (15) - Light Magenta
        /// 
        /// Default Value: RSSPECAN_VAL_DISP_COL_BLAC (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_HCOPY_COLOR_PREDEFINED
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureHardcopyColorByPredefined(int Color_Map, int Predefined_Colors)
        {
            int pInvokeResult = PInvoke.ConfigureHardcopyColorByPredefined(this._handle, Color_Map, Predefined_Colors);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function initiates the acquisition of system error correction data. A "0" is returned if the acquisition was successful.
        /// 
        /// Note(s):
        /// 
        /// (1)During the acquisition of correction data the instrument does not accept any remote control commands, except
        /// *RST
        /// CALibration:ABORt
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CALIBRATION
        /// 
        /// Remote-control command(s):
        /// CALibration[:ALL] ALL
        /// CALibration[:ALL]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Sync">
        /// This control sets the function behavior. Sync On means, that the function will wait for calibration done.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Note(s):
        /// (1) On - function sends CAL?;*OPC command and waits to be calibration complete.
        ///     Off - function sends CAL?;*OPC command and doesn't wait to completation. This selection allows use Calibration Abort if necessary. Use viReadSTB to check Operation complete.
        /// </param>
        /// <param name="Timeout__ms_">
        /// Sets the timeout for the calibration routine to be finished and have obtained calibration result. If the length of time required for calibration exceeds the timeout value, then the function will return with a timeout error and the instrument will continue with calibration.
        /// 
        /// Valid Values: 0 ms to 600000 ms
        /// 
        /// Default Value: 180000 ms
        /// 
        /// </param>
        /// <param name="Result">
        /// This control returns the result of acquisition of system error correction data. A "0" is returned if the acquisition was successful.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Calibration(bool Sync, uint Timeout__ms_, out int Result)
        {
            int pInvokeResult = PInvoke.Calibration(this._handle, System.Convert.ToUInt16(Sync), Timeout__ms_, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function aborts the acquisition of correction data and restores the last complete correction data set.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CALIBRATION_ABORT
        /// 
        /// Remote-control command(s):
        /// CALibration:ABORt
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CalibrationAbort()
        {
            int pInvokeResult = PInvoke.CalibrationAbort(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads the results of the correction data acquisition. The lines of the result table (see manual section "Recording the correction data of FSQ - CAL key") are output as string data separated by commas:
        /// "Total Calibration Status: PASSED","Date (dd/mm/yyyy): 12/07/1999", "Time: 16:24:54","Runtime:00.06"
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALibration:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the size of array passed to Result argument.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 256
        /// </param>
        /// <param name="Result">
        /// This control returns the results of the correction data acquisition. The lines of the result table (see manual section "Recording the correction data of FSQ - CAL key") are output as string data separated by
        /// commas:
        /// "Total Calibration Status: PASSED","Date (dd/mm/yyyy): 12/07/1999", "Time: 16:24:54","Runtime:00.06"
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CalibrationResult(int Array_Size, System.Text.StringBuilder Result)
        {
            int pInvokeResult = PInvoke.CalibrationResult(this._handle, Array_Size, Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function determines whether the current calibration data are taken into account by the instrument (ON) or not (OFF).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CALIBRATION_STATE
        /// 
        /// Remote-control command(s):
        /// CALibration:STATe ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="State">
        /// Determines whether the current calibration data are taken into account by the instrument (ON) or not (OFF).
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CALIBRATION_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CalibrationState(bool State)
        {
            int pInvokeResult = PInvoke.CalibrationState(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the automatic correction of the YIG filter frequency drift.
        /// When correction is switched on, it is checked once per minute whether the temperature on the YIG filter has changed by more than 5K relative to the last instance of correction. If this is the case, the frequency of the YIG filter is - at the end of the next measurement - adjusted as required for the new temperature. For time-critical applications, the correction function can be switched off after an operating period of 30 minutes.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CORR_YIG
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:YIG:TEMPerature:AUTO ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// Switches on or off the automatic correction of the YIG filter frequency drift.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CORR_YIG
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int YIGCorrection(int Window, bool State)
        {
            int pInvokeResult = PInvoke.YIGCorrection(this._handle, Window, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the names of all probe data sets stored in the FMU.
        /// 
        /// Note(s):
        /// 
        /// (1) This attribute can be used only for R&amp;S FMU.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]PROBe:CATalog?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Number_of_Data_Sets">
        /// This control returns number of names of all data sets.
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the string you specify for the catalog list parameter.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// Valid Values:
        /// any number
        /// 
        /// Default Value: 0
        /// 
        /// </param>
        /// <param name="Data_Set_List">
        /// This control returns the names of all data sets stored on the hard disk.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// if you pass 0 for buffer Size, you can use VI_NULL for this parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetCalibrationProbeCatalog(out int Number_of_Data_Sets, int Buffer_Size, System.Text.StringBuilder Data_Set_List)
        {
            int pInvokeResult = PInvoke.GetCalibrationProbeCatalog(this._handle, out Number_of_Data_Sets, Buffer_Size, Data_Set_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates a selected probe data set.
        /// 
        /// Note(s):
        /// 
        /// (1) This attribute can be used only for R&amp;S FMU.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CAL_PROBE_DATA_SELECT
        /// RSSPECAN_ATTR_CAL_PROBE_STATE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]PROBe:SELect
        /// [SENSe&lt;1|2&gt;:]PROBe[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Probe_Data_Set_Name">
        /// This control selects the specified probe data set.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CAL_PROBE_DATA_SELECT
        /// </param>
        /// <param name="State">
        /// This control activates a selected probe data set.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CAL_PROBE_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CalibrationProbeState(string Probe_Data_Set_Name, bool State)
        {
            int pInvokeResult = PInvoke.CalibrationProbeState(this._handle, Probe_Data_Set_Name, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the probe mode, DC voltage and pulse frequency.
        /// 
        /// Note(s):
        /// 
        /// (1) This attribute can be used only for R&amp;S FMU.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CAL_PROBE_ADJUST_MODE
        /// RSSPECAN_ATTR_CAL_PROBE_DC_VOLTAGE
        /// RSSPECAN_ATTR_CAL_PROBE_PULSE_FREQ
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]PROBe:ADJust:MODE DC | PULSE | COMPensation
        /// [SENSe&lt;1|2&gt;:]PROBe:ADJust:DC 0V, 10mV, 20mV, 50mV, 100mV, 200mV, 500mV, 1V, 2V, 2.4V
        /// [SENSe&lt;1|2&gt;:]PROBe:ADJust:PULSe 10kHz, 62.5kHz, 100kHz, 102.4kHz, 200kHz, 250kHz, 500kHz, 1MHz, 2MHz, 4MHz, 8MHz
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Probe_Mode">
        /// This control selects the probe mode.
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_PROBE_MODE_DC   (0) - DC
        /// RSSPECAN_VAL_PROBE_MODE_PULS (1) - Pulse
        /// RSSPECAN_VAL_PROBE_MODE_COMP (2) - Compensation
        /// 
        /// Default Value: RSSPECAN_VAL_PROBE_MODE_DC   (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CAL_PROBE_ADJUST_MODE
        /// 
        /// Note(s):
        /// 
        /// (1) DC: activates DC voltage at the Prob Cal output
        /// 
        /// (2) Pulse: activates pulse at the Prob Cal output
        /// 
        /// (3) Compensation: activates a 1kHz frequency at the Compensation output
        /// </param>
        /// <param name="Value">
        /// This control sets the voltage or pulse frequency at the Probe Cal output.
        /// 
        /// Valid Values:
        /// for DC Voltage:
        /// 0 V, 10.0e-3 V, 20.0e-3 V, 50.0e-3 V, 100.0e-3 V, 200.0e-3 V, 500.0e-3 V, 1 V, 2 V, 2.4 V
        /// 
        /// for Pulse Frequency:
        /// 10.0e3 Hz, 62.5.0e3 Hz, 100.0e3 Hz, 102.4.0e3 Hz, 200.0e3 Hz, 250.0e3 Hz, 500.0e3 Hz, 1.0e6 Hz, 2.0e6 Hz, 4.0e6 Hz, 8.0e6 Hz
        /// 
        /// Default Value: 0.0 V
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CAL_PROBE_DC_VOLTAGE
        /// RSSPECAN_ATTR_CAL_PROBE_PULSE_FREQ
        /// 
        /// Note(s):
        /// 
        /// (1) DC Voltage values can be set when Probe Mode is set to DC.
        /// 
        /// (2) Pulse Frequency values can be set when Probe Mode is set to Probe.
        /// 
        /// (3) If the Probe Mode is set to Compensation, this control is ignored.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CalibrationProbeSettings(int Probe_Mode, double Value)
        {
            int pInvokeResult = PInvoke.CalibrationProbeSettings(this._handle, Probe_Mode, Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the calibration probe manual compensation.
        /// 
        /// Note(s):
        /// 
        /// (1) This function can be used only for R&amp;S FMU.
        /// 
        /// (2) This function is only available when I/Q Input is set to High (function rsspecan_ConfigureAnalogBasebandParameters)..
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CAL_PROBE_MAN_COMPENSATION
        /// 
        /// Remote-control command(s):
        /// CALibration:PROBe:COMPensation ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Manual_Probe_Adjustment">
        /// This control switches the manual probe adjustment as part of the probe calibration process on or off. If active the probe calibration process is interrupted and the bits 12/13 of the STATus:QUEStionable:SYNC register will signal the required manual operation. The calibration process has to be continued by "INIT:CONM" afterwards.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CAL_PROBE_MAN_COMPENSATION
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CalibrationProbeManualCompensation(bool Manual_Probe_Adjustment)
        {
            int pInvokeResult = PInvoke.CalibrationProbeManualCompensation(this._handle, System.Convert.ToUInt16(Manual_Probe_Adjustment));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the calibration probe compensation.
        /// 
        /// Note(s):
        /// 
        /// (1) This attribute can be used only for R&amp;S FMU.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CAL_PROBE_FREQ_RESP_COMPENSATION
        /// RSSPECAN_ATTR_CAL_PROBE_GAIN_ERROR_COMPENSATION
        /// 
        /// Remote-control command(s):
        /// CALibration:PROBe:FRESponse ON | OFF
        /// CALibration:PROBe:GAIN ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Frequency_Response_Comp">
        /// This control switches the frequency response calibration on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CAL_PROBE_FREQ_RESP_COMPENSATION
        /// </param>
        /// <param name="Gain_Error_Compensation">
        /// This control switches the gain error compensation on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CAL_PROBE_GAIN_ERROR_COMPENSATION
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CalibrationProbeCompensation(bool Frequency_Response_Comp, bool Gain_Error_Compensation)
        {
            int pInvokeResult = PInvoke.CalibrationProbeCompensation(this._handle, System.Convert.ToUInt16(Frequency_Response_Comp), System.Convert.ToUInt16(Gain_Error_Compensation));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts a probe calibration using the specified file name to generate a probe data set at the end of the calibration process. During the probe calibration process, manual operations may be required (see -chapter 4, softkey PROBE CAL START). The Status Questionable Sync Register bits 12 and 13 are set if related user operation has to take place. After adjusting the probes of input I or Q, the calibration process has to be continued by the command "INIT:CONM". The probe data set is automatically activated and can be switched off again using the SENS:PROBE:STATE ON | OFF command. 
        /// 
        /// Note(s):
        /// 
        /// (1) This attribute can be used only for R&amp;S FMU.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CAL_PROBE_START
        /// 
        /// Remote-control command(s):
        /// CALibration:PROBe[:STARt] &lt;file_name&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Probe_Data_Set_Name">
        /// This control specified a file name for generated probe data set.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CAL_PROBE_START
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int StartProbeCalibration(string Probe_Data_Set_Name)
        {
            int pInvokeResult = PInvoke.StartProbeCalibration(this._handle, Probe_Data_Set_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the status of the probe calibration.
        /// 
        /// Note(s):
        /// 
        /// (1) This attribute can be used only for R&amp;S FMU.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Probe_Calibration_Status_I">
        /// This control returns the status of the input I of the probe calibration.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Note(s):
        /// 
        /// (0) Off - The probe connected with input I can't be now adjusted.
        /// 
        /// (1) On - The probe connected with input I can be now adjusted. Continue the calibration process with command INIT:CONM (function rsspecan_Continue)
        /// 
        /// </param>
        /// <param name="Probe_Calibration_Status_Q">
        /// This control returns the status of the input Q of the probe calibration.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Note(s):
        /// 
        /// (0) Off - The probe connected with input Q can't be now adjusted.
        /// 
        /// (1) On - The probe connected with input Q can be now adjusted. Continue the calibration process with command INIT:CONM (function rsspecan_Continue)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetProbeCalibrationStatus(uint Timeout, out bool Probe_Calibration_Status_I, out bool Probe_Calibration_Status_Q)
        {
            ushort Probe_Calibration_Status_IAsUShort;
            ushort Probe_Calibration_Status_QAsUShort;
            int pInvokeResult = PInvoke.GetProbeCalibrationStatus(this._handle, Timeout, out Probe_Calibration_Status_IAsUShort, out Probe_Calibration_Status_QAsUShort);
            Probe_Calibration_Status_I = System.Convert.ToBoolean(Probe_Calibration_Status_IAsUShort);
            Probe_Calibration_Status_Q = System.Convert.ToBoolean(Probe_Calibration_Status_QAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries, if the calibration of the selected probe data set passed or failed.
        /// 
        /// Note(s):
        /// 
        /// (1) This attribute can be used only for R&amp;S FMU.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CAL_PROBE_DATA_SELECT
        /// RSSPECAN_ATTR_CAL_PROBE_CHECK
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]PROBe:SELect
        /// [SENSe&lt;1|2&gt;:]PROBe:FAIL?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Probe_Data_Set_Name">
        /// This control selects the specified probe data set.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CAL_PROBE_DATA_SELECT
        /// </param>
        /// <param name="Probe_Check">
        /// This control queries, if the calibration of the selected probe data set passed or failed.
        /// 
        /// Valid Values: 
        /// 0 - Pass
        /// 1 - Fail
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CAL_PROBE_CHECK
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CalibrationProbeCheck(string Probe_Data_Set_Name, out int Probe_Check)
        {
            int pInvokeResult = PInvoke.CalibrationProbeCheck(this._handle, Probe_Data_Set_Name, out Probe_Check);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the calibration result of the selected probe data set.
        /// 
        /// Note(s):
        /// 
        /// (1) This attribute can be used only for R&amp;S FMU.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CAL_PROBE_DATA_SELECT
        /// ---
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]PROBe:SELect
        /// [SENSe&lt;1|2&gt;:]PROBe:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Probe_Data_Set_Name">
        /// This control selects the specified probe data set.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CAL_PROBE_DATA_SELECT
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the size of array passed to Result argument.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 256
        /// </param>
        /// <param name="Result">
        /// This control queries the calibration result of the selected probe data set.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CalibrationProbeResult(string Probe_Data_Set_Name, int Array_Size, System.Text.StringBuilder Result)
        {
            int pInvokeResult = PInvoke.CalibrationProbeResult(this._handle, Probe_Data_Set_Name, Array_Size, Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function renames existing probe data sets.
        /// 
        /// Note(s):
        /// 
        /// (1) This attribute can be used only for R&amp;S FMU.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]PROBe:MOVE &lt;probe_data_set_source&gt;,&lt;probe_data_set_destination&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Probe_Data_Set_Source">
        /// This control sets the probe data set source.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// 
        /// </param>
        /// <param name="Probe_Data_Set_Destination">
        /// This control sets the probe data set destination.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CalibrationProbeRename(string Probe_Data_Set_Source, string Probe_Data_Set_Destination)
        {
            int pInvokeResult = PInvoke.CalibrationProbeRename(this._handle, Probe_Data_Set_Source, Probe_Data_Set_Destination);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function deletes the specified probe data set.
        /// 
        /// Note(s):
        /// 
        /// (1) This attribute can be used only for R&amp;S FMU.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CAL_PROBE_DELETE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]PROBe:DELete &lt;probe_data_set_name&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Probe_Data_Set_Name">
        /// This control deletes the specified probe data set.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CAL_PROBE_DELETE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CalibrationProbeDelete(string Probe_Data_Set_Name)
        {
            int pInvokeResult = PInvoke.CalibrationProbeDelete(this._handle, Probe_Data_Set_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function toggles the control lines of the user ports between INPut and OUTPut. With ON, the user port is switched to INPut, with OFF to OUTPut.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_INP_UPORT_STATE
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:UPORt:STATe
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// Toggles the control lines of the user ports between INPut and OUTPut.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Output
        /// VI_TRUE  (1) - Input 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_INP_UPORT_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureControlLinesState(int Window, bool State)
        {
            int pInvokeResult = PInvoke.ConfigureControlLinesState(this._handle, Window, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the control lines of the user ports.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_OUT_UPORT_VALUE
        /// 
        /// Remote-control command(s):
        /// OUTPut:UPORt[:VALue]
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="User_Ports">
        /// This control sets the control lines of the user ports.
        /// 
        /// Valid Values: "00000000" to "11111111"
        /// 
        /// Default Value: "00000000"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_INP_UPORT_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureOutputControlLines(int Window, string User_Ports)
        {
            int pInvokeResult = PInvoke.ConfigureOutputControlLines(this._handle, Window, User_Ports);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function toggles the control lines of the user ports between INPut and OUTPut. With ON, the user port is switched to INPut, with OFF to OUTPut.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_INP_UPORT_VALUE
        /// RSSPECAN_ATTR_OUT_UPORT_VALUE
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:UPORt[:VALue]?
        /// OUTPut:UPORt[:VALue]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// 
        /// Note:
        /// 
        /// (1) This control has effect only for Input Lines
        /// </param>
        /// <param name="Lines">
        /// This control selects the control lines to be returned.
        /// 
        /// Valid Values:
        /// 
        /// RSSPECAN_VAL_USER_PORT_INP (0) - Input
        /// RSSPECAN_VAL_USER_PORT_OUT (1) - Output
        /// 
        /// Default Value: RSSPECAN_VAL_USER_PORT_INP (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_INP_UPORT_VALUE
        /// RSSPECAN_ATTR_OUT_UPORT_VALUE
        /// </param>
        /// <param name="User_Ports">
        /// Returns the control lines of the user ports. 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetControlLines(int Window, int Lines, System.Text.StringBuilder User_Ports)
        {
            int pInvokeResult = PInvoke.GetControlLines(this._handle, Window, Lines, User_Ports);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the auto adjust settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available for R&amp;S FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AUTO_ADJUST_ALL
        /// RSSPECAN_ATTR_AUTO_ADJUST_FREQ
        /// RSSPECAN_ATTR_AUTO_ADJUST_LEVEL
        /// RSSPECAN_ATTR_AUTO_ADJUST_LEVEL_MEAS_TIME_MODE
        /// RSSPECAN_ATTR_AUTO_ADJUST_LEVEL_MEAS_TIME
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ADJust:ALL
        /// [SENSe&lt;1|2&gt;:]ADJust:FREQuency
        /// [SENSe&lt;1|2&gt;:]ADJust:LEVel
        /// [SENSe&lt;1|2&gt;:]ADJust:CONFigure:LEVel:DURation:MODE AUTO | MAN
        /// [SENSe&lt;1|2&gt;:]ADJust:CONFigure:LEVel:DURation
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Adjust_Settings">
        /// This control selects the auto adjust setting.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ADJ_ALL  (0) - All
        /// RSSPECAN_VAL_ADJ_FREQ (1) - Frequency Only
        /// RSSPECAN_VAL_ADJ_LEV  (2) - Level Only
        /// 
        /// Default Value: RSSPECAN_VAL_ADJ_ALL (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AUTO_ADJUST_ALL
        /// RSSPECAN_ATTR_AUTO_ADJUST_FREQ
        /// RSSPECAN_ATTR_AUTO_ADJUST_LEVEL
        /// 
        /// Note(s):
        /// 
        /// (1) All: Frequency and Level are both adjusted.
        /// </param>
        /// <param name="Measurement_Time_Mode">
        /// This control defines whether the duration of the level measurement used to determine the optimal reference level is determined automatically or manually.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_AUTO (0) - Automatic Length
        /// RSSPECAN_VAL_MAN  (1) - Manual Length
        /// 
        /// Default Value: RSSPECAN_VAL_AUTO (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AUTO_ADJUST_LEVEL_MEAS_TIME_MODE
        /// </param>
        /// <param name="Measurement_Time">
        /// This control defines the duration of the level measurement used to determine the optimal reference level automatically.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0.001 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AUTO_ADJUST_LEVEL_MEAS_TIME
        /// 
        /// Note(s):
        /// 
        /// (1) This value is only available if Measurement Time Mode is set to Manual Length.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAutoAdjust(int Adjust_Settings, int Measurement_Time_Mode, double Measurement_Time)
        {
            int pInvokeResult = PInvoke.ConfigureAutoAdjust(this._handle, Adjust_Settings, Measurement_Time_Mode, Measurement_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function initiates auto scaling for screens.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available for R&amp;S FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AUTO_SCALE_SINGLE_SCREEN
        /// RSSPECAN_ATTR_AUTO_SCALE_ALL_SCREENS
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1..4&gt;]:TRACe&lt;trace&gt;:Y[:SCALe]:AUTO[:VALue]
        /// DISPlay[:WINDow&lt;1..4&gt;]:TRACe&lt;trace&gt;:Y[:SCALe]:AUTO:ALL
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Auto_Scale_Screen">
        /// This control selects the auto scaling type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_AUTOSCALE_ALL (0) - All
        /// RSSPECAN_VAL_AUTOSCALE_A   (1) - A
        /// RSSPECAN_VAL_AUTOSCALE_B   (2) - B
        /// RSSPECAN_VAL_AUTOSCALE_C   (3) - C
        /// RSSPECAN_VAL_AUTOSCALE_D   (4) - D
        /// 
        /// Default Value: RSSPECAN_VAL_AUTOSCALE_ALL (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AUTO_SCALE_SINGLE_SCREEN
        /// RSSPECAN_ATTR_AUTO_SCALE_ALL_SCREENS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int YAxisAutoScale(int Auto_Scale_Screen)
        {
            int pInvokeResult = PInvoke.YAxisAutoScale(this._handle, Auto_Scale_Screen);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates and deactivates the GPS receiver (R&amp;S HA-Z240).
        /// 
        /// Note(s):
        /// (1) This function is available for R&amp;S FSH instruments.
        /// 
        /// (2) - GPS receiver only works if a connection between the R&amp;S FSH and a GPS signal transmitter is established.
        /// 
        /// Attribute(s): RSSPECAN_ATTR_POSITION_GPS_STATE
        /// 
        /// Remote-control command(s):
        /// SYSTem:POSition:GPS[:STATe]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="GPS">
        /// This control activates and deactivates the GPS receiver (R&amp;S HA-Z240).
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value:
        /// VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsfsh_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureGPSState(bool GPS)
        {
            int pInvokeResult = PInvoke.ConfigureGPSState(this._handle, System.Convert.ToUInt16(GPS));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries if the R&amp;S FSH is currently connected to the GPS receiver (R&amp;S HAZ240).
        /// 
        /// Note(s):
        /// (1) This function is available for R&amp;S FSH instruments.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GPS_CONNECTION_STATE
        /// 
        /// Remote-control command(s):
        /// SYSTem:POSition:GPS:CONNected?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="GPS_Connected">
        /// This control displays the GPS receiver (R&amp;S HA-Z240) state.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsfsh_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int getGPSConnected(out bool GPS_Connected)
        {
            ushort GPS_ConnectedAsUShort;
            int pInvokeResult = PInvoke.getGPSConnected(this._handle, out GPS_ConnectedAsUShort);
            GPS_Connected = System.Convert.ToBoolean(GPS_ConnectedAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the frequency correction factor. This factor is calculated from a reference signal provided by the GPS receiver R&amp;S HA-Z240. The reference signal is used to determine
        /// the deviation of the internal clock of the instrument, which can be turned into a correction factor for the measured frequency.
        /// 
        /// Note(s):
        /// (1) This function is available for R&amp;S FSH instruments.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GPS_FREQUENCY_CORRECTION
        /// 
        /// Remote-control command(s):
        /// SYSTem:POSition:GPS:CORRection:FREQuency?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Correction_Factor">
        /// This control displays the frequency correction factor.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsfsh_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int getGPSFrequencyCorrection(out double Correction_Factor)
        {
            int pInvokeResult = PInvoke.getGPSFrequencyCorrection(this._handle, out Correction_Factor);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the quality of the GPS signal.
        /// 
        /// Note(s):
        /// (1) This function is available for R&amp;S FSH instruments.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GPS_SIGNAL_QUALITY
        /// 
        /// Remote-control command(s):
        /// SYSTem:POSition:GPS:QUALity?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Signal_Quality">
        /// This control displays the quality of the GPS signal.
        /// 
        /// 0 - INSufficient
        /// 1 - LOW
        /// 2 - MEDium
        /// 3 - HIGh
        /// 4 - EXCellent
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsfsh_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int getGPSSignalQuality(out int Signal_Quality)
        {
            int pInvokeResult = PInvoke.getGPSSignalQuality(this._handle, out Signal_Quality);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the number of tracked satellites.
        /// 
        /// Note(s):
        /// (1) This function is available for R&amp;S FSH instruments.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GPS_SATELLITES
        /// 
        /// Remote-control command(s):
        /// SYSTem:POSition:GPS:SATellites?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Satellites">
        /// This control displays the number of tracked satellites.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsfsh_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int getGPSSatellites(out int Satellites)
        {
            int pInvokeResult = PInvoke.getGPSSatellites(this._handle, out Satellites);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the latitude of the current position of the R&amp;S FSH.
        /// 
        /// Note(s):
        /// (1) This function is available for R&amp;S FSH instruments.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GPS_LATITUDE
        /// 
        /// Remote-control command(s):
        /// SYSTem:POSition:LATitude?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Buffer_Size">
        /// This control defines the size of buffer passed to argument List.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 256
        /// </param>
        /// <param name="Latitude">
        /// This control displays the latitude of the current position of the R&amp;S FSH.
        /// 
        /// Note(s):
        /// (1) - &lt;sign&gt;&lt;degrees&gt;,&lt;minutes&gt;,&lt;seconds&gt;
        /// (2) - &lt;sign&gt; no sign = northern hemisphere, negative sign (-) = southern hemisphere
        /// (3) - &lt;degrees&gt; degrees of latitude (integer value)
        /// (4) - &lt;minutes&gt; minutes of latitude (integer value)
        /// (5) - &lt;seconds&gt; seconds of latitude (floating point value)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsfsh_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int getGPSLatitude(int Buffer_Size, System.Text.StringBuilder Latitude)
        {
            int pInvokeResult = PInvoke.getGPSLatitude(this._handle, Buffer_Size, Latitude);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the longitude of the current position of the R&amp;S FSH.
        /// 
        /// Note(s):
        /// (1) This function is available for R&amp;S FSH instruments.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GPS_LONGITUDE
        /// 
        /// Remote-control command(s):
        /// SYSTem:POSition:LONGitude?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Buffer_Size">
        /// This control defines the size of buffer passed to argument List.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 256
        /// </param>
        /// <param name="Longitude">
        /// This control displays the longitude of the current position of the R&amp;S FSH.
        /// 
        /// Note(s):
        /// (1) - &lt;sign&gt;&lt;degrees&gt;,&lt;minutes&gt;,&lt;seconds&gt;
        /// (2) - &lt;sign&gt; no sign = east, negative sign (-) = west
        /// (3) - &lt;degrees&gt; degrees of longitude (integer value)
        /// (4) - &lt;minutes&gt; minutes of longitude (integer value)
        /// (5) - &lt;seconds&gt; seconds of longitude (floating point value)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsfsh_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int getGPSLongitude(int Buffer_Size, System.Text.StringBuilder Longitude)
        {
            int pInvokeResult = PInvoke.getGPSLongitude(this._handle, Buffer_Size, Longitude);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the validity of the position information.
        /// 
        /// Note(s):
        /// (1) This function is available for R&amp;S FSH instruments.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GPS_POSITION_INFORMATION
        /// 
        /// Remote-control command(s):
        /// SYSTem:POSition:VALid?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Position_Information">
        /// This control displays the GPS receiver (R&amp;S HA-Z240) - validity of the position information.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - not valid
        /// VI_TRUE  (1) - valid
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsfsh_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int getGPSPositionInformation(out bool Position_Information)
        {
            ushort Position_InformationAsUShort;
            int pInvokeResult = PInvoke.getGPSPositionInformation(this._handle, out Position_InformationAsUShort);
            Position_Information = System.Convert.ToBoolean(Position_InformationAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function initiates a signal acquisition based on the present instrument configuration.  It then waits for the acquisition to complete, and returns the trace as an array of amplitude values. The amplitude array returns data that represent the amplitude of the signals of the sweep from the start frequency to the stop frequency (in frequency domain, in time domain the amplitude array is ordered from beginning of sweep to end).  The Amplitude Units attribute determines the units of the points in the amplitude array.  This function resets the sweep count.
        /// 
        /// Notes:
        /// 
        /// (1) If the spectrum analyzer did not complete the acquisition within the time period the user specified with the maxTime parameter, the function returns Max Time Exceeded error.
        /// 
        /// (2) If the data was captured in an uncalibrated spectrum analyzer mode, the function will generate a Measure Uncalibrated warning flag. 
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// INITiate[:IMMediate]
        /// TRACe&lt;1|2&gt;[:DATA] TRACE1| TRACE2| TRACE3 | TRACE4 | TRACE5 | TRACE6
        /// FORMat[:DATA] REAL,32
        /// *CLS
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 1 to 6 for R&amp;S FSL or R&amp;S ESL or R&amp;S FSV
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Maximum_Time__ms_">
        /// Pass the maximum length of time in which to allow the read waveform operation to complete.    
        /// 
        /// If the operation does not complete within this time interval, the function returns the RSSPECAN_ERROR_MAX_TIME_EXCEEDED (0xBFFA2003) error code.  When this occurs, you can call rsspecan_Abort to cancel the read waveform operation and return the oscilloscope to the Idle state.
        /// 
        /// Units:  milliseconds.  
        /// 
        /// Defined Values:
        /// RSSPECAN_VAL_MAX_TIME_INFINITE (-1)             RSSPECAN_VAL_MAX_TIME_IMMEDIATE (0)          
        /// 
        /// Default Value: 5000 (ms)
        /// 
        /// Notes:
        /// 
        /// (1) The Maximum Time parameter applies only to this function.  It has no effect on other timeout parameters or attributes.
        /// 
        /// </param>
        /// <param name="Array_Length">
        /// Number of array points requested.
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Actual_Points">
        /// Number of points actually returned in the Amplitude array and X Array array.
        /// </param>
        /// <param name="Amplitude">
        /// A user allocated buffer into which the trace amplitudes is stored.
        /// 
        /// Units: user specified in RSSPECAN_ATTR_AMPLITUDE_UNITS
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadYTrace(int Window, int Trace, uint Maximum_Time__ms_, int Array_Length, out int Actual_Points, double[] Amplitude)
        {
            int pInvokeResult = PInvoke.ReadYTrace(this._handle, Window, Trace, Maximum_Time__ms_, Array_Length, out Actual_Points, Amplitude);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the trace the spectrum analyzer acquires. The trace is from a previously initiated acquisition.  You use the rsspecan_Initiate function to start an acquisition.  
        /// 
        /// You can call the Read Y Trace function instead of the Initiate function. This function starts an acquisition, waits for the acquisition to complete, and returns the trace in one function call.
        /// 
        /// The array returns data that represents the amplitude of the signals of the sweep from the start frequency to the stop frequency (in frequency domain, in time domain the amplitude array is ordered from beginning of sweep to end).  The Amplitude Units attribute determines the units of the points in the amplitude array.
        /// 
        /// This function does not check the instrument status. Typically, the end-user calls this function only in a sequence of calls to other low-level driver functions. The sequence performs one operation. The end-user uses the low-level functions to optimize one or more aspects of interaction with the instrument. Call the rsspecan_error_query function at the conclusion of the sequence to check the instrument status. 
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;[:DATA] TRACE1| TRACE2| TRACE3, | SPURious | ABITstream | PWCDp | CTABle, &lt;block&gt; | &lt;numeric_value&gt;
        /// FORMat[:DATA] REAL,32
        /// *CLS
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 1 to 6 for R&amp;S FSL or R&amp;S ESL or R&amp;S FSV
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Array_Length">
        /// Number of array points requested.
        /// </param>
        /// <param name="Actual_Points">
        /// Number of points actually returned in the array.
        /// </param>
        /// <param name="Amplitude">
        /// A user allocated buffer into which the trace amplitudes is stored.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchYTrace(int Window, int Trace, int Array_Length, out int Actual_Points, double[] Amplitude)
        {
            int pInvokeResult = PInvoke.FetchYTrace(this._handle, Window, Trace, Array_Length, out Actual_Points, Amplitude);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the current X and Y results of the sweep. During a sweep the last measured values are read out.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is not available for R&amp;S FSL, R&amp;S FMU, R&amp;S ESL and R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:IMMediate:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="X_Value">
        /// Returns the X value result of the sweep.
        /// 
        /// </param>
        /// <param name="Y_Value">
        /// Returns the Y value result of the sweep.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetCurrentResults(int Window, out double X_Value, out double Y_Value)
        {
            int pInvokeResult = PInvoke.GetCurrentResults(this._handle, Window, out X_Value, out Y_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the current Y results of the sweep. During a sweep the last measured value is read out.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_Y_RESULT
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:IMMediate:LEVel?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Level">
        /// Returns the current Y results of the sweep.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetCurrentYResult(int Window, out double Level)
        {
            int pInvokeResult = PInvoke.GetCurrentYResult(this._handle, Window, out Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function will start a measurement and read I/Q trace data results. The result values are scaled linear in unit Volt and correspond to the voltage at the RF input of the instrument.
        /// 
        /// Notes:
        /// 
        /// (1) For large I/Q trace data is recommended to increase default timeout value to the appropriate value which allows instrument handle all pending tasks. To change default timeout value call following VISA function:
        /// 
        ///     viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 60000);
        /// 
        /// This attribute specifies the minimum timeout value to use (in milliseconds) when accessing the device associated with the given session.
        /// 
        /// (2) If there are not I/Q data available in memory because the corresponding measurement had not been started, the function will cause a Query Error.
        /// 
        /// (3) For FFT Analysis Mode data are scaled linearly with Volt as the unit and corresponds to the voltage at the RF input of the instrument, irrespective of the output format selected.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FORMat[:DATA] REAL,32
        /// TRACe&lt;1|2&gt;:IQ:DATA:FORMat IQBLock
        /// TRACe&lt;1|2&gt;:IQ:DATA?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of doubles in the array you specify for the real parts and imaginary parameters.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the data contains more elemens than you specify with the Buffer Size parameter, the function returns Buffer Size elements into the arrays and returns the buffer size you must pass to get the all data.  For example, if the data contains 16000 elements for real parts and 16000 elements for imaginary part and the Buffer Size is 10000, the function places 10000 elements into the arrays and returns 16000.
        /// 
        /// Notes:
        /// 
        /// (1) If VI_ATTR_INTF_TYPE is equal to VI_INTF_ASRL, then use this parameter to pass the expected number of points to be read from the instrument. Data points will be read until instruments output buffer is empty. If Buffer Size is lower than actual number of results available in the instruments output buffer, then this function returns an error code (BFFC0002  Parameter 2 (Buffer Size) out of range.) and measurement results will not be valid. This is due to limitation if the unit is remote-controlled via the RS-232-C interface.
        /// 
        /// (2) This function requires that all response data are read out completely before the instrument accepts further commands.
        /// 
        /// </param>
        /// <param name="No_of_Points">
        /// Returns the number of I/Q trace data points.
        /// 
        /// </param>
        /// <param name="Real_Parts__I_">
        /// Returns an array of trace data (I-value x, real part) point. The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter.
        /// 
        /// </param>
        /// <param name="Imaginary_Parts__Q_">
        /// Returns an array of trace data (Q-value y, imaginary part) point. The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadTraceIQData(int Window, uint Timeout, int Buffer_Size, out int No_of_Points, double[] Real_Parts__I_, double[] Imaginary_Parts__Q_)
        {
            int pInvokeResult = PInvoke.ReadTraceIQData(this._handle, Window, Timeout, Buffer_Size, out No_of_Points, Real_Parts__I_, Imaginary_Parts__Q_);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function permits the readout of previously acquired (and frequency response corrected) I/Q data from the memory, with indication of the offset related to the start of measurement and with indication of the number of measurement values. Therefore a previously acquired data set can be read out in smaller portions. The result values are scaled linear in unit Volt and correspond to the voltage at the RF input of the instrument.
        /// 
        /// Notes:
        /// 
        /// (1) For large I/Q trace data is recommended to increase default timeout value to the appropriate value which allows instrument handle all pending tasks. To change default timeout value call following VISA function:
        /// 
        ///     viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 60000);
        /// 
        /// This attribute specifies the minimum timeout value to use (in milliseconds) when accessing the device associated with the given session.
        /// 
        /// (2) If there are not I/Q data available in memory because the corresponding measurement had not been started, the function will cause a Query Error.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FORMat[:DATA] REAL,32
        /// TRACe&lt;1|2&gt;:IQ:DATA:MEMory[:RF]?
        /// *CLS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Offset_Samples">
        /// This control sets offset of the values to be read related to the start of the acquired data.
        /// 
        /// Valid Values: 0 to &lt;# of samples&gt; - 1
        /// 
        /// Default Value: 0
        /// 
        /// Note:
        /// 
        /// Where &lt;# of samples&gt; is the value indicated with function Trace I/Q Set (rsspecan_TraceIQSet).
        /// 
        /// </param>
        /// <param name="No_of_Samples">
        /// This control sets the number of measurement values to be read.
        /// 
        /// Valid Values: 1 to &lt;# of samples&gt; - &lt;offset samples&gt;
        /// 
        /// Default Value: 1
        /// 
        /// Note:
        /// 
        /// Where &lt;# of samples&gt; is the value indicated with function Trace I/Q Set (rsspecan_TraceIQSet).
        /// 
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of doubles in the array you specify for the real parts and imaginary parameters.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the data contains more elemens than you specify with the Buffer Size parameter, the function returns Buffer Size elements into the arrays and returns the buffer size you must pass to get the all data.  For example, if the data contains 16000 elements for real parts and 16000 elements for imaginary part and the Buffer Size is 10000, the function places 10000 elements into the arrays and returns 16000.
        /// 
        /// Notes:
        /// 
        /// (1) If VI_ATTR_INTF_TYPE is equal to VI_INTF_ASRL, then use this parameter to pass the expected number of points to be read from the instrument. Data points will be read until instruments output buffer is empty. If Buffer Size is lower than actual number of results available in the instruments output buffer, then this function returns an error code (BFFC0002  Parameter 2 (Buffer Size) out of range.) and measurement results will not be valid. This is due to limitation if the unit is remote-controlled via the RS-232-C interface.
        /// 
        /// (2) This function requires that all response data are read out completely before the instrument accepts further commands.
        /// 
        /// </param>
        /// <param name="No_of_Points">
        /// Returns the number of I/Q trace data points.
        /// 
        /// </param>
        /// <param name="Real_Parts__I_">
        /// Returns an array of trace data (I-value x, real part) point. The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter.
        /// 
        /// </param>
        /// <param name="Imaginary_Parts__Q_">
        /// Returns an array of trace data (Q-value y, imaginary part) point. The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchTraceIQData(int Offset_Samples, int No_of_Samples, int Buffer_Size, out int No_of_Points, double[] Real_Parts__I_, double[] Imaginary_Parts__Q_)
        {
            int pInvokeResult = PInvoke.FetchTraceIQData(this._handle, Offset_Samples, No_of_Samples, Buffer_Size, out No_of_Points, Real_Parts__I_, Imaginary_Parts__Q_);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the transfer parameters and reads the data header.
        /// 
        /// Note:
        /// (1) No error checking is done in this function. It is recommended to perform error checking after all data are read (with funciton rsspecan_ReadMemoryIQLargeDataBlock).
        /// 
        /// (2) This function starts measurement and for REAL,32 data format reads the data header. Data has to be read with funciton rsspecan_ReadMemoryIQLargeDataBlock. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_IQ_DATA_FORMAT
        /// 
        /// Remote-control command(s):
        /// FORMat[:DATA] ASCii | REAL,32
        /// TRACe&lt;1|2&gt;:IQ:DATA:FORMat COMPatible | IQBLock | IQPair
        /// TRACe&lt;1|2&gt;:IQ:DATA?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Format">
        /// Sets the format of transferred data.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FORMAT_ASC  (0) - ASCII
        /// RSSPECAN_VAL_FORMAT_REAL (1) - Real
        /// 
        /// Default Value: RSSPECAN_VAL_FORMAT_REAL (1) - Real
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_IQ_DATA_FORMAT
        /// </param>
        /// <param name="Data_Order">
        /// Sets the transferred data order.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TRACE_FORM_COMP (0) - Compatible
        /// RSSPECAN_VAL_TRACE_FORM_IQBL (1) - IQ Block
        /// RSSPECAN_VAL_TRACE_FORM_IQP  (2) - IQ Pair
        /// 
        /// Default Value: RSSPECAN_VAL_TRACE_FORM_COMP (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_IQ_DATA_FORMAT
        /// 
        /// Note(s):
        /// 
        /// (1) Compatible - 512k I data and 512k Q data is alternately transferred
        /// 
        /// (2) IQ Block - First, all I and then all Q data is transferred
        /// 
        /// (3) IQ Pair - I/Q data is transferred in pairs
        /// 
        /// (4) This control has no effect for R&amp;S FSL, IQ Block is set.
        /// </param>
        /// <param name="No_of_Points">
        /// Returns the number of I/Q trace data bytes.
        /// 
        /// Note:
        /// (1) For Format set to Real, number of bztes is returned.
        /// 
        /// (1) For Format set to ASCII, -1 is returned allways. No information about data size is available.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadMemoryIQLargeData(int Window, uint Timeout, int Format, int Data_Order, out int No_of_Points)
        {
            int pInvokeResult = PInvoke.ReadMemoryIQLargeData(this._handle, Window, Timeout, Format, Data_Order, out No_of_Points);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function will read I/Q trace data according format defined with function rsspecan_ReadMemoryIQLargeData.
        /// 
        /// Notes:
        /// 
        /// (1) Funcion rsspecan_ReadMemoryIQLargeData has to be called before this function.
        /// 
        /// (2) If all data are read, function returns VI_SUCCESS, else it returns warning 3FFF0006  The specified number of bytes was read. (VI_SUCCESS_MAX_CNT)
        /// 
        /// (3) No error checking is done. It is recommended to check error after all data are read out.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// no command, only data are read
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the array size you specify for the returned data parameter.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the data contains more elemens than you specify with the Buffer Size parameter, the function returns Buffer Size elements into the arrays and returns the buffer size you must pass to get the all data.  For example, if the data contains 16000 elements for real parts and 16000 elements for imaginary part and the Buffer Size is 10000, the function places 10000 elements into the arrays and returns 16000.
        /// 
        /// Valid Values: &gt;0
        /// 
        /// Default Value: 512
        /// 
        /// 
        /// </param>
        /// <param name="Returned_Data">
        /// Returns an byte array of trace data. The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter.
        /// 
        /// </param>
        /// <param name="Returned_Counts">
        /// Number of bytes actually transferred.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadMemoryIQLargeDataBlock(int Buffer_Size, System.Text.StringBuilder Returned_Data, out int Returned_Counts)
        {
            int pInvokeResult = PInvoke.ReadMemoryIQLargeDataBlock(this._handle, Buffer_Size, Returned_Data, out Returned_Counts);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the result of the limit check of the limit line indicated in the selected measurement window. It should be noted that a complete sweep must have been performed for obtaining a valid result. A synchronization with  *OPC, *OPC? or *WAI should therefore be provided.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_CHECK_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:FAIL?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// The result of the limit check responds with 0 for PASS, 1 for FAIL, and 2 for MARGIN.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetLimitCheckResult(int Window, int Limit, out int State)
        {
            int pInvokeResult = PInvoke.GetLimitCheckResult(this._handle, Window, Limit, out State);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function initiates an acquisition. After calling this function, the spectrum analyzer leaves the idle state and waits for a trigger.
        /// 
        /// This function does not check the instrument status. Typically, the end-user calls this function only in a sequence of calls to other low-level driver functions. The sequence performs one operation. The end-user uses the low-level functions to optimize one or more aspects of interaction with the instrument.
        /// 
        /// Notes:
        /// 
        /// (1) Call Acquisition Status to determine when the acquisition is complete.
        /// 
        /// (2) Call the Error Query function at the conclusion of the sequence to check the instrument status.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_INIT
        /// 
        /// Remote-control command(s):
        /// INITiate[:IMMediate]
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Initiate(int Window, uint Timeout)
        {
            int pInvokeResult = PInvoke.Initiate(this._handle, Window, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts a new spurious measurement and waits till sweeping is finished. 
        /// 
        /// Note(s): 
        /// 
        /// (1) Set sweep configuration with rsspecan_ConfigureAcquisition
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_INIT_SPURIOUS
        /// 
        /// Remote-control command(s):
        /// INITiate&lt;1|2&gt;:SPURious
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int InitiateSpurious(int Window, uint Timeout)
        {
            int pInvokeResult = PInvoke.InitiateSpurious(this._handle, Window, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts an automatic sequence consisting of the preliminary measurement, determination of the peak list and subsequent final measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S ESL.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_INIT_EMI_TEST
        /// 
        /// Remote-control command(s):
        /// INITiate&lt;1|2&gt;:EMITest
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int InitiateEMITest(int Window, uint Timeout)
        {
            int pInvokeResult = PInvoke.InitiateEMITest(this._handle, Window, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts the final measurement based on the peak list.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S ESL.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_INIT_FINAL_MEASUREMENT_TEST
        /// 
        /// Remote-control command(s):
        /// INITiate&lt;1|2&gt;:FMEasurement
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int InitiateFinalMeasurementTest(int Window, uint Timeout)
        {
            int pInvokeResult = PInvoke.InitiateFinalMeasurementTest(this._handle, Window, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts a Spectrum Emission Mask measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S FSL, R&amp;S ESL, R&amp;S FSV, R&amp;S FSP, R&amp;S FSQ, R&amp;S FSU instruments.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_INIT_SEM_MEAS
        /// 
        /// Remote-control command(s):
        /// INITiate&lt;1|2&gt;:ESPectrum
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int InitiateSEMMeasurement(int Window, uint Timeout)
        {
            int pInvokeResult = PInvoke.InitiateSEMMeasurement(this._handle, Window, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function aborts a previously initiated measurement and returns the spectrum analyzer to the idle state.
        /// 
        /// This function does not check the instrument status. Typically, the end-user calls this function only in a sequence of calls to other low-level driver functions. The sequence performs one operation. The end-user uses the low-level functions to optimize one or more aspects of interaction with the instrument. 
        /// 
        /// Notes:
        /// 
        /// (1) To check the instrument status, call the Error Query function at the conclusion of the sequence.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_ABORT
        /// 
        /// Remote-control command(s):
        /// ABORt
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Abort()
        {
            int pInvokeResult = PInvoke.Abort(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function interrupts a current scan measurement. The scan is resumed by INITiate&lt;1|2&gt;[:IMMediate] - attribute RSSPECAN_ATTR_INIT.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S ESL.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_HOLD
        /// 
        /// Remote-control command(s):
        /// HOLD
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Hold()
        {
            int pInvokeResult = PInvoke.Hold(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function continues a stopped measurement at the current position in single sweep mode. The function is useful especially for trace functions MAXHold, MINHold and AVERage if the previous results are not to be cleared with Sweep Count &gt; 0 or Average Count &gt; 0 on restarting the measurement (INIT:IMMediate resets the previous results on restarting the measurement).
        /// The single-sweep mode is automatically switched on. Synchronization to the end of the indicated number of measurements can then be performed with the command *OPC, *OPC? or *WAI. In the continuous-sweep mode, synchronization to the sweep end is not possible since the overall measurement never ends.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_INIT_CONMEAS
        /// 
        /// Remote-control command(s):
        /// INITiate&lt;1|2&gt;:CONMeas
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Continue(int Window, uint Timeout)
        {
            int pInvokeResult = PInvoke.Continue(this._handle, Window, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sends a command to trigger the SpecAn.  Call this function if the trigger source is set to software trigger.  
        /// 
        /// Notes:
        /// 
        /// (1) If both the trigger source are not set to software trigger, this function returns an error.
        /// 
        /// (2) This function does not check the instrument status.   Typically, you call this function only in a sequence of calls to other low-level driver functions.  The sequence performs one operation.  You use the low-level functions to optimize one or more aspects of interaction with the instrument.  If you want to check the instrument status, call the rsspecan_error_query function at the conclusion of the sequence.
        /// 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_INIT_SW_TRIGGER
        /// 
        /// Remote-control command(s):
        /// *TRG
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SendSoftwareTrigger(uint Timeout)
        {
            int pInvokeResult = PInvoke.SendSoftwareTrigger(this._handle, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the current number of started sweeps. A sweep count value should be set and the device should be in single sweep mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SWEEP_COUNT_CURRENT
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:COUNt:CURRent?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Number_Of_Sweeps">
        /// Returns the current sweep number.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SWEEP_COUNT_CURRENT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetSweepNumber(int Window, out int Number_Of_Sweeps)
        {
            int pInvokeResult = PInvoke.GetSweepNumber(this._handle, Window, out Number_Of_Sweeps);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the noise measurement on or off for all markers of the indicated measurement window. The noise power density is measured at the position of the markers.
        /// 
        /// Note: This function is not available for GSM/EDGE measurements modulation accuracy (MAC), phase/frequency error (PFE) and power versus time (PVT).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_NOISE_STATE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:NOISe[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 1..16 for R&amp;S FSV
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// This control switches the noise measurement on or off for all markers of the indicated measurement window. The noise power density is measured at the position of the markers. 
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_NOISE_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMarkerNoiseMeasurement(int Window, int Marker, bool State)
        {
            int pInvokeResult = PInvoke.ConfigureMarkerNoiseMeasurement(this._handle, Window, Marker, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the result of the noise measurement.
        /// A complete sweep with synchronization to the sweep end must be performed between switching on the function and querying the measured value in order to obtain a valid query result. This is only possible in single sweep mode.
        /// 
        /// Note: This function is not available for GSM/EDGE measurements modulation accuracy (MAC), phase/frequency error (PFE) and power versus time (PVT).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_NOISE_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:NOISe:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 1..16 for R&amp;S FSV
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Result">
        /// This control returns the result of the noise measurement.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryMarkerNoiseMeasurementResult(int Window, int Marker, out double Result)
        {
            int pInvokeResult = PInvoke.QueryMarkerNoiseMeasurementResult(this._handle, Window, Marker, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures phase noise marker measurement. To setup reference point use function rsspecan_ConfigureReferenceFixedPoint.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_REFERENCE_MARKER_PNO_STATE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1to4&gt;:FUNCtion:PNOise[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// This control switches on or off the phase-noise measurement with all act?ve delta markers in the selected measurement window. The correction values for the bandwidth and the log amplifier are taken into account in the measurement.
        /// Marker 1 will be activated, if necessary, and a peak search will be performed. If marker 1 is activated, its position becomes the reference point for the measurement.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_MARKER_PNO_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMarkerPhaseNoiseMeasurement(int Window, bool State)
        {
            int pInvokeResult = PInvoke.ConfigureMarkerPhaseNoiseMeasurement(this._handle, Window, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines a new reference point level for delta marker 2 in the selected measurement window.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_REF_PEAK
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1to4&gt;:FUNCtion:FIXed:RPOint:MAXimum[:PEAK]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int PhaseNoisePeakSearch(int Window)
        {
            int pInvokeResult = PInvoke.PhaseNoisePeakSearch(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sests an automatic peak search action for the reference fixed marker 1 at the end of each particular sweep. This function may be used for tracking of a drifting source whilst phase noise measurement. The delta marker 2 which shows the phase noise measurement result keeps the delta frequency value. Therefore the phase noise measurement in a certain offset is valid although the source is drifting. Only when the marker 2 is reaching the border of the span the delta marker value is adjusted to be within the span. Choose a larger span in such
        /// situations.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_REFERENCE_MARKER_PNO_AUTO
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:FUNCtion:PNOise:AUTO ON | OFF
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// This control switches an automatic peak search action for the reference fixed marker 1 at the end of each particular sweep.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_MARKER_PNO_AUTO
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int PhaseNoisePeakSearchAuto(int Window, bool State)
        {
            int pInvokeResult = PInvoke.PhaseNoisePeakSearchAuto(this._handle, Window, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the result of the phase-noise measurement in the selected measurement window. The measurement will be switched on, if necessary.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_REFERENCE_MARKER_PNO_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:FUNCtion:PNOise:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Phase_Noise_Result">
        /// Returns the result of the phase-noise measurement in the selected measurement window. The measurement will be switched on, if necessary.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryPhaseNoiseResult(int Window, out double Phase_Noise_Result)
        {
            int pInvokeResult = PInvoke.QueryPhaseNoiseResult(this._handle, Window, out Phase_Noise_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the marker frequency counter resolution and turns the marker frequency counter on/off.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_FREQUENCY_COUNTER_ENABLED
        /// RSSPECAN_ATTR_MARKER_FREQUENCY_COUNTER_RESOLUTION
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:COUNt ON | OFF
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:COUNt:RESolution
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 1..16 for R&amp;S FSV
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker_Frequency_Counter">
        /// Specifies whether the frequency counter is enabled.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_FREQUENCY_COUNTER_ENABLED
        /// </param>
        /// <param name="Frequency_Counter_Resolution">
        /// Specifies the frequency counter resolution.
        /// 
        /// Valid Values:
        /// 0.1, 1, 10, 100, 1000, 10000 Hz
        /// 
        /// Default Value: 1000.0 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_FREQUENCY_COUNTER_RESOLUTION
        /// 
        /// Notes:
        /// 
        /// (1) This value is ignored when markerFrequencyCounter is False.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMarkerFrequencyCounter(int Window, int Marker, bool Marker_Frequency_Counter, double Frequency_Counter_Resolution)
        {
            int pInvokeResult = PInvoke.ConfigureMarkerFrequencyCounter(this._handle, Window, Marker, System.Convert.ToUInt16(Marker_Frequency_Counter), Frequency_Counter_Resolution);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the result of the frequency counter for the indicated marker in the selected measurement window. Before the command, the frequency counter should be switched on and a complete measurement performed to obtain a valid count result. Therefore, a single sweep with synchronization must be performed between switching on the frequency counter and querying the count result.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_COUNT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:COUNt:FREQuency?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 1..16 for R&amp;S FSV
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Counter_Frequency">
        /// This control returns the result of the frequency counter for the indicated marker.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryMarkerFrequencyCounterResult(int Window, int Marker, out double Counter_Frequency)
        {
            int pInvokeResult = PInvoke.QueryMarkerFrequencyCounterResult(this._handle, Window, Marker, out Counter_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the N dB function.
        /// 
        /// Note: This function is not available for GSM/EDGE measurements modulation accuracy (MAC), phase/frequency error (PFE) and power versus time (PVT).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_NDB_STATE
        /// RSSPECAN_ATTR_MARKER_NDB_VAL
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:NDBDown:STATe ON | OFF
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:NDBDown
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="N_dB_State">
        /// Switches the "N dB Down" function on or off in the selected measurement window. Marker 1 is activated first, if necessary.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_NDB_STATE
        /// </param>
        /// <param name="N_dB_Level">
        /// Defines the level spacing of the two delta markers to the right and left of marker 1 in the selected measurement window.
        /// 
        /// Valid Values:
        /// -100.0 dB to 100.0 dB
        /// 
        /// Default Value: 6 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_NDB_VAL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureNdBPoints(int Window, bool N_dB_State, double N_dB_Level)
        {
            int pInvokeResult = PInvoke.ConfigureNdBPoints(this._handle, Window, System.Convert.ToUInt16(N_dB_State), N_dB_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the frequency spacing (bandwidth) of the N-dB-down markers in the selected measurement window.
        /// A complete sweep with synchronization to sweep end must be performed between switching on the function and querying the measured value in order to obtain a valid query result. This is only possible in single sweep mode.
        /// 
        /// Note: This function is not available for GSM/EDGE measurements modulation accuracy (MAC), phase/frequency error (PFE) and power versus time (PVT).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_NDB_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:NDBDown:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="N_dB_Result">
        /// Returns the frequency spacing (bandwidth) of the N-dB-down markers in the selected measurement window.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryNdBResult(int Window, out double N_dB_Result)
        {
            int pInvokeResult = PInvoke.QueryNdBResult(this._handle, Window, out N_dB_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the two frequencies of the N-dB-down marker in the selected measurement window. The two frequency values are output in ascending order.
        /// A complete sweep with synchronization to sweep end must be performed between switching on the function and querying the measured value to obtain a valid query result. This is only possible in single sweep mode.
        /// 
        /// Note: This function is not available for GSM/EDGE measurements modulation accuracy (MAC), phase/frequency error (PFE) and power versus time (PVT).
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:NDBDown:FREQuency?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="N_dB_Frequency_Lower">
        /// Returns the lower frequency of N dB down markers in the selected measurement window.
        /// </param>
        /// <param name="N_dB_Frequency_Higher">
        /// Returns the higher frequency of N dB down markers in the selected measurement window.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryNdBFrequencies(int Window, out double N_dB_Frequency_Lower, out double N_dB_Frequency_Higher)
        {
            int pInvokeResult = PInvoke.QueryNdBFrequencies(this._handle, Window, out N_dB_Frequency_Lower, out N_dB_Frequency_Higher);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the two times of the N-dB-down marker in the selected measurement window. 
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:NDBDown:TIME?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="N_dB_Time_Lower">
        /// Returns the lower frequency of N dB down markers in the selected measurement window.
        /// </param>
        /// <param name="N_dB_Time_Higher">
        /// Returns the higher frequency of N dB down markers in the selected measurement window.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryNdBTimes(int Window, out double N_dB_Time_Lower, out double N_dB_Time_Higher)
        {
            int pInvokeResult = PInvoke.QueryNdBTimes(this._handle, Window, out N_dB_Time_Lower, out N_dB_Time_Higher);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the Q factor (quality) of the measured bandwidth for span&gt;0. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_NDB_Q_FACTOR
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...16&gt;:FUNCtion:NDBDown:QFACtor?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="N_dB_Q_Factor">
        /// Returns the Q factor (quality) of the measured bandwidth for span&gt;0.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryNdBQFactor(int Window, out double N_dB_Q_Factor)
        {
            int pInvokeResult = PInvoke.QueryNdBQFactor(this._handle, Window, out N_dB_Q_Factor);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the average type of the sense list measurement. In linear mode voltages are averaged in logarithmic mode levels.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is not valid for R&amp;S FSL and R&amp;S FSV
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LIST_POW_TYPE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:POWer:SET:AVERage:TYPE LINear | LOGarithmic
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Type">
        /// Sets the average type of the sense list measurement
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIN (0) - Linear 
        /// RSSPECAN_VAL_LOG (1) - Logarithmic
        /// 
        /// Default Value: RSSPECAN_VAL_LIN (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LIST_POW_TYPE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureListPowerAverageType(int Window, int Type)
        {
            int pInvokeResult = PInvoke.ConfigureListPowerAverageType(this._handle, Window, Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the list of settings (max. 100 entries for an individual frequency point) for the multiple power measurement and starts a measurement sequence. To reduce the setting time, all indicated parameters are set up simultaneously at each test point.
        /// 
        /// Notes:
        /// 
        /// (1) Each one test point is defined on the same position in all parameter arrays.
        /// 
        /// (2) The measurement is performed in the time domain and therefore the span is set to 0 Hz. If the time domain is quit, the function is automatically switched off. The measurement is not compatible with other measurements, especially as far as marker, adjacent channel power measurement or statistics are concerned. The corresponding commands thus automatically deactivate the function. The function is only available in REMOTE operation. It is deactivated when switching the instrument back to LOCAL.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:POWer:SEQuence
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="No_of_List_Items">
        /// Set number of list items. This value must be equal to array size of each list parameter.
        /// 
        /// Valid Values: 1 to 100
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Analyzer_Frequency">
        /// Array of receive frequency for the signal to be measured (= center frequency in manual operation).
        /// 
        /// Valid Values: 0.0 Hz to max. analyzer frequency, depending on the instrument model.
        /// 
        /// Default Value: none
        /// 
        /// </param>
        /// <param name="Reference_Level">
        /// Array of reference level values for each list item.
        /// 
        /// Valid Values: -70.0 dBm to 30.0 dBm in 10 dB steps
        ///              -75.0 dBm to 30.0 dBm in 5 dB steps (Electronic 
        ///              Attenuator option FS-B25)
        /// 
        /// Default Value: none
        /// 
        /// </param>
        /// <param name="RF_Input_Attenuation">
        /// Array of RF input attenuation values for each list item.
        /// 
        /// Valid Values: 0.0 dB to 70.0 dB in 10 dB steps
        ///              0.0 dB to 75.0 dB in 5 dB steps (Electronic 
        ///              Attenuator Option FS-B25)
        /// 
        /// Default Value: none
        /// 
        /// </param>
        /// <param name="RF_Input_Electronic_Attn">
        /// Array of RF input attenuation values of electronic attenuator for each list item.
        /// 
        /// Valid Values: 0.0 dB to 30.0 dB in 10 dB steps
        /// 
        /// Default Value: none
        /// 
        /// Note:
        /// 
        /// If value is &lt; 0.0 (negative value) then electronic attenuator is not in the signal path (OFF). If option FS-B25 is missing, -1 (OFF) is to be given.
        /// 
        /// </param>
        /// <param name="Filter_Type">
        /// Array of filter types for each list item.
        /// 
        /// Valid Values:
        /// 0 - Normal
        /// 1 - CFilter
        /// 2 - RRC
        /// 3 - FFT
        /// 4 - Pulse
        /// 
        /// Default Value: none
        /// 
        /// Notes:
        /// 
        /// (1) Normal:
        /// Normal resolution filter.
        /// 
        /// (2) CFilter:
        /// Channel filter. These are especially steep-edged filters, which are used for example in Fast ACP measurement to ensure the band-limiting of a transmission channel in the time domain.
        /// 
        /// (3) RRC:
        /// Root Raised Cosine filter. This special filter form is used to determine the channel power for some mobile radio standards.
        /// 
        /// (4) FFT:
        /// FFT filters. This value is valid only for R&amp;S FSL
        /// 
        /// (5) Pulse:
        /// EMI (6dB) filters. This value is valid only for R&amp;S FSL from firmware version 1.30
        /// </param>
        /// <param name="Resolution_Bandwidth">
        /// Array of resolution bandwidth values for each list item.
        /// 
        /// Valid Values: 10.0 Hz to 10.0e6 Hz in 1, 3, 10 steps
        /// 
        /// Default Value: 0.0 Hz
        /// 
        /// Note:
        /// 
        /// Valid range also depends on the selected filter type.
        /// 
        /// </param>
        /// <param name="Video_Bandwidth">
        /// Array of video bandwidth values for each list item.
        /// 
        /// Valid Values: 1.0 Hz to 10.0e6 Hz in 1, 3, 10 steps.
        /// 
        /// Default Value: none
        /// 
        /// Note:
        /// 
        /// The value is ignored for &lt;filter type&gt; = CFilter or RRC.
        /// 
        /// </param>
        /// <param name="Meas_Time">
        /// Array of measurement time values for each list item.
        /// 
        /// Valid Values: 1.0e-6 s to 30.0 s
        /// 
        /// Default Value: none
        /// 
        /// </param>
        /// <param name="Trigger_Level">
        /// Array of trigger level values for each list item.
        /// 
        /// This value is Reserved. Must be set to 0 for each list item.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureListPowerSequence(int Window, int No_of_List_Items, double[] Analyzer_Frequency, double[] Reference_Level, double[] RF_Input_Attenuation, double[] RF_Input_Electronic_Attn, int[] Filter_Type, double[] Resolution_Bandwidth, double[] Video_Bandwidth, double[] Meas_Time, double[] Trigger_Level)
        {
            int pInvokeResult = PInvoke.ConfigureListPowerSequence(this._handle, Window, No_of_List_Items, Analyzer_Frequency, Reference_Level, RF_Input_Attenuation, RF_Input_Electronic_Attn, Filter_Type, Resolution_Bandwidth, Video_Bandwidth, Meas_Time, Trigger_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the constant settings for the list during multiple power measurement. Parameters Peak Meas, RMS Meas and AVG Meas define, which measurements are to be performed at the same time at the frequency point. Correspondingly, one, two or three results per frequency point are returned for the function Get List Power Sequence (actListPwrSeq). If all three parameters are set to OFF, the command generates an execution error.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:POWer:SET
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Peak_Meas">
        /// Activates (deactivates) the measurement of the peak power (peak
        /// detector).
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On (Default Value)
        /// 
        /// </param>
        /// <param name="RMS_Meas">
        /// Activates (deactivates) the measurement of the RMS power (RMS detector).
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On (Default Value)
        /// 
        /// </param>
        /// <param name="AVG_Meas">
        /// Activates (deactivates) the measurement of the average power (average detector).
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On (Default Value)
        /// 
        /// </param>
        /// <param name="Trigger_Mode">
        /// Selection of the trigger source used for the list measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TRG_IMM (0) - Immediate (Default Value)
        /// RSSPECAN_VAL_TRG_EXT (1) - External
        /// RSSPECAN_VAL_TRG_IFP (2) - Video
        /// RSSPECAN_VAL_TRG_VID (3) - IF Power
        /// 
        /// </param>
        /// <param name="Trigger_Slope">
        /// This control selects the slope of the trigger signal.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_NEG (0) - Negative
        /// RSSPECAN_VAL_POS (1) - Positive 
        /// 
        /// Default Value: RSSPECAN_VAL_POS (1)
        /// 
        /// 
        /// 
        /// </param>
        /// <param name="Trigger_Offset">
        /// This control sets the offset between the detection of the trigger signal and the start of the measurement at the next frequency point.
        /// 
        /// Valid Values: 0.0 s (0.125 s) to 100s
        /// 
        /// Default Value: 0.0
        /// 
        /// Notes:
        /// 
        /// (1) The value 0 s deactivates the use of GATED TRIGGER; other
        /// values activate the GATED TRIGGER function.
        /// 
        /// (2) Values &lt;&gt; 0 s are only possible if Trigger Mode is different from IMMediate. Otherwise, an execution error is triggered.
        /// 
        /// </param>
        /// <param name="Gate_Length">
        /// Gate length with Gated Sweep.
        /// 
        /// Valid Values: 0.0 s (0.125 s) to 100.0 s
        /// 
        /// Default Value: 0.0
        /// 
        /// Notes:
        /// 
        /// (1) The value 0 s deactivates the use of GATED TRIGGER; other
        /// values activate the GATED TRIGGER function.
        /// 
        /// (2) Values &lt;&gt; 0 s are only possible if Trigger Mode is different from IMMediate. Otherwise, an execution error is triggered.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureListPowerSet(int Window, bool Peak_Meas, bool RMS_Meas, bool AVG_Meas, int Trigger_Mode, int Trigger_Slope, double Trigger_Offset, double Gate_Length)
        {
            int pInvokeResult = PInvoke.ConfigureListPowerSet(this._handle, Window, System.Convert.ToUInt16(Peak_Meas), System.Convert.ToUInt16(RMS_Meas), System.Convert.ToUInt16(AVG_Meas), Trigger_Mode, Trigger_Slope, Trigger_Offset, Gate_Length);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function deactivates the list measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LIST_POW_STATE_OFF
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:POWer:STATe OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ListPowerMeasurementOff(int Window)
        {
            int pInvokeResult = PInvoke.ListPowerMeasurementOff(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the list of settings (max. 100 entries for an individual frequency point) for the multiple power measurement, starts a measurement sequence and returns the measured data. To reduce the setting time, all indicated parameters are set up simultaneously at each test point.
        /// 
        /// Notes:
        /// 
        /// (1) Each one test point is defined on the same position in all parameter arrays.
        /// 
        /// (2) The measurement is performed in the time domain and therefore the span is set to 0 Hz. If the time domain is quit, the function is automatically switched off. The measurement is not compatible with other measurements, especially as far as marker, adjacent channel power measurement or statistics are concerned. The corresponding commands thus automatically deactivate the function. The function is only available in REMOTE operation. It is deactivated when switching the instrument back to LOCAL.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:POWer:SEQuence?
        /// *CLS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="No_of_List_Items">
        /// Set number of list items. This value must be equal to array size of each list parameter.
        /// 
        /// Valid Values: 1 to 100
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Analyzer_Frequency">
        /// Array of receive frequency for the signal to be measured (= center frequency in manual operation).
        /// 
        /// Valid Values: 0.0 Hz to max. analyzer frequency, depending on the instrument model.
        /// 
        /// Default Value: none
        /// 
        /// </param>
        /// <param name="Reference_Level">
        /// Array of reference level values for each list item.
        /// 
        /// Valid Values: -70.0 dBm to 30.0 dBm in 10 dB steps
        ///              -75.0 dBm to 30.0 dBm in 5 dB steps (Electronic 
        ///              Attenuator option FS-B25)
        /// 
        /// Default Value: none
        /// 
        /// </param>
        /// <param name="RF_Input_Attenuation">
        /// Array of RF input attenuation values for each list item.
        /// 
        /// Valid Values: 0.0 dB to 70.0 dB in 10 dB steps
        ///              0.0 dB to 75.0 dB in 5 dB steps (Electronic 
        ///              Attenuator Option FS-B25)
        /// 
        /// Default Value: none
        /// 
        /// </param>
        /// <param name="RF_Input_Electronic_Attn">
        /// Array of RF input attenuation values of electronic attenuator for each list item.
        /// 
        /// Valid Values: 0.0 dB to 30.0 dB in 10 dB steps
        /// 
        /// Default Value: none
        /// 
        /// Note:
        /// 
        /// If value is &lt; 0.0 (negative value) then electronic attenuator is not in the signal path (OFF). If option FS-B25 is missing, -1 (OFF) is to be given.
        /// 
        /// </param>
        /// <param name="Filter_Type">
        /// Array of filter types for each list item.
        /// 
        /// Valid Values:
        /// 0 - Normal
        /// 1 - CFilter
        /// 2 - RRC
        /// 3 - FFT
        /// 4 - Pulse
        /// 
        /// Default Value: none
        /// 
        /// Notes:
        /// 
        /// (1) Normal:
        /// Normal resolution filter.
        /// 
        /// (2) CFilter:
        /// Channel filter. These are especially steep-edged filters, which are used for example in Fast ACP measurement to ensure the band-limiting of a transmission channel in the time domain.
        /// 
        /// (3) RRC:
        /// Root Raised Cosine filter. This special filter form is used to determine the channel power for some mobile radio standards.
        /// 
        /// (4) FFT:
        /// FFT filters. This value is valid only for R&amp;S FSL
        /// 
        /// (5) Pulse:
        /// EMI (6dB) filters. This value is valid only for R&amp;S FSL from firmware version 1.30
        /// </param>
        /// <param name="Resolution_Bandwidth">
        /// Array of resolution bandwidth values for each list item.
        /// 
        /// Valid Values: 10.0 Hz to 10.0e6 Hz in 1, 3, 10 steps
        /// 
        /// Default Value: 0.0 Hz
        /// 
        /// Note:
        /// 
        /// Valid range also depends on the selected filter type.
        /// 
        /// </param>
        /// <param name="Video_Bandwidth">
        /// Array of video bandwidth values for each list item.
        /// 
        /// Valid Values: 1.0 Hz to 10.0e6 Hz in 1, 3, 10 steps.
        /// 
        /// Default Value: none
        /// 
        /// Note:
        /// 
        /// The value is ignored for &lt;filter type&gt; = CFilter or RRC.
        /// 
        /// </param>
        /// <param name="Meas_Time">
        /// Array of measurement time values for each list item.
        /// 
        /// Valid Values: 1.0e-6 s to 30.0 s
        /// 
        /// Default Value: none
        /// 
        /// </param>
        /// <param name="Trigger_Level">
        /// Array of trigger level values for each list item.
        /// 
        /// This value is Reserved. Must be set to 0 for each list item.
        /// 
        /// </param>
        /// <param name="List_Power_Results">
        /// This control returns array of elements as the result of a previously configured and initiated list measurement.
        /// 
        /// Notes:
        /// 
        /// (1) Array size must be =&gt; No of List Items.
        /// 
        /// (2) The result list can be extended to 3 results per frequency point if the Peak, RMS and Average detectors are activated.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryListPowerSequence(int Window, int No_of_List_Items, double[] Analyzer_Frequency, double[] Reference_Level, double[] RF_Input_Attenuation, double[] RF_Input_Electronic_Attn, int[] Filter_Type, double[] Resolution_Bandwidth, double[] Video_Bandwidth, double[] Meas_Time, double[] Trigger_Level, double[] List_Power_Results)
        {
            int pInvokeResult = PInvoke.QueryListPowerSequence(this._handle, Window, No_of_List_Items, Analyzer_Frequency, Reference_Level, RF_Input_Attenuation, RF_Input_Electronic_Attn, Filter_Type, Resolution_Bandwidth, Video_Bandwidth, Meas_Time, Trigger_Level, List_Power_Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the result of a previous list measurement as configured and initiated with List Power Sequence. This function may be used to obtain measurement results in an asynchronous way, using the service request mechanism for synchronization with the end of the measurement. If no measurement results are available, the function will return a query error.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:POWer:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="No_of_Results">
        /// Expected number of values to be returned from the instrument. This value must be equal to the List Power results array number of items.
        /// 
        /// Valid Values: &gt; 0
        /// 
        /// Default Value: 256
        /// 
        /// Note:
        /// 
        /// This function will read data from the instrument till instruments buffer is empty or number of results is reached.
        /// 
        /// </param>
        /// <param name="List_Power_Results">
        /// This control returns array of elements as the result of a previous list measurement as configured and initiated with List Power Sequence (confListPwrSeq).
        /// 
        /// Note:
        /// 
        /// Array size must be =&gt; then number of elements sent from the instrument.
        /// 
        /// </param>
        /// <param name="Returned_Values">
        /// Returns the number of Power Results.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryListPowerResult(int Window, int No_of_Results, double[] List_Power_Results, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.QueryListPowerResult(this._handle, Window, No_of_Results, List_Power_Results, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the previously selected time domain power measurements. Thus one or several measurements can be first selected and then switched on and off together with this attribute.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_TDOM_STATE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:SUMMary[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// Switches on or off the previously selected time domain power measurements.
        /// 
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_TDOM_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTimeDomainPowerMeasurementState(int Window, bool State)
        {
            int pInvokeResult = PInvoke.ConfigureTimeDomainPowerMeasurementState(this._handle, Window, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures parameters of time domain power measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_TDOM_PEAK
        /// RSSPECAN_ATTR_MEAS_TDOM_RMS
        /// RSSPECAN_ATTR_MEAS_TDOM_MEAN
        /// RSSPECAN_ATTR_MEAS_TDOM_SDEV
        /// RSSPECAN_ATTR_MEAS_TDOM_AVG
        /// RSSPECAN_ATTR_MEAS_TDOM_MAX_HOLD
        /// RSSPECAN_ATTR_MEAS_TDOM_MODE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:SUMMary:PPEak[:STATe] ON | OFF
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:SUMMary:RMS[:STATe] ON | OFF
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:SUMMary:MEAN[:STATe] ON | OFF
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:SUMMary:SDEViation[:STATe] ON | OFF
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:SUMMary:AVERage ON | OFF
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:SUMMary:PHOLd ON | OFF
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:SUMMary:MODE ABSolute | RELative
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Peak">
        /// Sets the time domain power peak masurement.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_TDOM_PEAK
        /// </param>
        /// <param name="RMS">
        /// Sets the time domain power RMS masurement.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_TDOM_RMS
        /// </param>
        /// <param name="Mean">
        /// Sets the time domain power mean masurement.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_TDOM_MEAN
        /// </param>
        /// <param name="Standard_Deviation">
        /// Sets the time domain power standard deviation masurement.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_TDOM_SDEV
        /// 
        /// Note:
        /// (1) If this control has effect only if Mean is turned On.
        /// </param>
        /// <param name="Average">
        /// Switches on or off averaging for the active time domain power measurement in the indicated window.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_TDOM_AVG
        /// </param>
        /// <param name="Max_Hold">
        /// Switches on or off the peak-hold function for the active time domain power measurement in the indicated measurement window.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_TDOM_MAX_HOLD
        /// </param>
        /// <param name="Power">
        /// Selects absolute or relative time domain power measurement in the indicated measurement window.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ABS (0) - Absolute
        /// RSSPECAN_VAL_REL (1) - Relative
        /// 
        /// Default Value: RSSPECAN_VAL_ABS (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_TDOM_MODE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTimeDomainPowerMeasurement(int Window, bool Peak, bool RMS, bool Mean, bool Standard_Deviation, bool Average, bool Max_Hold, int Power)
        {
            int pInvokeResult = PInvoke.ConfigureTimeDomainPowerMeasurement(this._handle, Window, System.Convert.ToUInt16(Peak), System.Convert.ToUInt16(RMS), System.Convert.ToUInt16(Mean), System.Convert.ToUInt16(Standard_Deviation), System.Convert.ToUInt16(Average), System.Convert.ToUInt16(Max_Hold), Power);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches off all time domain measurements in the selected measurement window.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_TDOM_AOFF
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:SUMMary:AOFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int TimeDomainPowerMeasurementAllOff(int Window)
        {
            int pInvokeResult = PInvoke.TimeDomainPowerMeasurementAllOff(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// With this function the currently measured average value and RMS value are declared as reference values for relative measurements in the indicated measurement window.
        /// If the function of Averaging or Peak Hold is switched on, the current value is the accumulated measurement value at the time considered.
        /// 
        /// Note(s):
        /// (1) This function is not available for GSM/EDGE measurements modulation accuracy (MAC) and phase/frequency error (PFE).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_TDOM_SET_REFERENCE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:SUMMary:REFerence:AUTO ONCE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int TimeDomainPowerMeasurementSetReference(int Window)
        {
            int pInvokeResult = PInvoke.TimeDomainPowerMeasurementSetReference(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries selected time domain power measurement result in the indicated measurement window.
        /// 
        /// A complete sweep with synchronization to sweep end must be performed between switching on the function and querying the measured value to obtain a valid query result. This is only possible in single sweep mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_TDOM_PEAK_RESULT
        /// RSSPECAN_ATTR_MEAS_TDOM_AVG_PEAK_RESULT
        /// RSSPECAN_ATTR_MEAS_TDOM_PEAK_HOLD_RESULT
        /// RSSPECAN_ATTR_MEAS_TDOM_RMS_RESULT
        /// RSSPECAN_ATTR_MEAS_TDOM_AVG_RMS_RESULT
        /// RSSPECAN_ATTR_MEAS_TDOM_PEAK_HOLD_RMS_RESULT
        /// RSSPECAN_ATTR_MEAS_TDOM_MEAN_RESULT
        /// RSSPECAN_ATTR_MEAS_TDOM_AVG_MEAN_RESULT
        /// RSSPECAN_ATTR_MEAS_TDOM_PEAK_HOLD_MEAN_RESULT
        /// RSSPECAN_ATTR_MEAS_TDOM_SDEV_RESULT
        /// RSSPECAN_ATTR_MEAS_TDOM_AVG_SDEV_RESULT
        /// RSSPECAN_ATTR_MEAS_TDOM_PEAK_HOLD_SDEV_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:SUMMary:PPEak:RESult?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:SUMMary:PPEak:AVERage:RESult?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:SUMMary:PPEak:PHOLd:RESult?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:SUMMary:RMS:RESult?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:SUMMary:RMS:AVERage:RESult?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:SUMMary:RMS:PHOLd:RESult?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:SUMMary:MEAN:RESult?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:SUMMary:MEAN:AVERage:RESult?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:SUMMary:MEAN:PHOLd:RESult?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:SUMMary:SDEViation:RESult?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:SUMMary:SDEViation:AVERage:RESult?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:SUMMary:SDEViation:PHOLd:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Measurement">
        /// Selects the time domain power measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TDOM_PEAK (0) - Peak
        /// RSSPECAN_VAL_TDOM_RMS  (1) - RMS
        /// RSSPECAN_VAL_TDOM_MEAN (2) - Mean
        /// RSSPECAN_VAL_TDOM_SDEV (3) - Standard Deviation
        /// 
        /// Default Value: RSSPECAN_VAL_TDOM_PEAK (0)
        /// </param>
        /// <param name="Result_Type">
        /// Selects the time domain power measurement result type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TDOM_RESULT          (0) - Result
        /// RSSPECAN_VAL_TDOM_AVG_RESULT      (1) - Average Result
        /// RSSPECAN_VAL_TDOM_MAX_HOLD_RESULT (2) - Max Hold Result
        /// 
        /// Default Value: RSSPECAN_VAL_TDOM_RESULT (0)
        /// 
        /// Note(s):
        /// (1) Result - query the result of the measurement
        /// (2) Average Result - query the result of the averaged measurement
        /// (3) Max Hold Result - query the result of the measurement with active peak hold function
        /// </param>
        /// <param name="Result">
        /// Returns the selected result of time domain power measurement.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryTDomPowerResult(int Window, int Measurement, int Result_Type, out double Result)
        {
            int pInvokeResult = PInvoke.QueryTDomPowerResult(this._handle, Window, Measurement, Result_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function is used to determine the power of a sequence of signal pulses having the same interval, as are typical for the slots of a GSM signal, for example. The number of pulses to be measured as well as the measurement time and the period can be set. To define the position of the first pulse in the trace, a suitable offset can be entered.
        /// The evaluation is performed on the measurement data of a previously recorded trace. The data recorded during the set measurement time is combined to a measured value for each pulse according to the detector specified and the indicated number of results is output as a list.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:MSUMmary?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Time_Offset_of_First_Pulse">
        /// Defines the time offset of first pulse to define the position of the first pulse in the trace.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 0.0 s
        /// </param>
        /// <param name="Measurement_Time">
        /// Defines the measurement time.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 0.0 s
        /// </param>
        /// <param name="Period">
        /// Defines the period.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 0.0 s
        /// </param>
        /// <param name="Number_of_Pulses">
        /// Defines number of pulses to be measured.
        /// 
        /// Valid Values:
        /// &gt; 0
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Result">
        /// Returns the array of power of a sequence of signal pulses having the same interval.
        /// Array has to have size at least Number Of Pulses.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryTDomPowerPulseSequenceResult(int Window, double Time_Offset_of_First_Pulse, double Measurement_Time, double Period, int Number_of_Pulses, double[] Result)
        {
            int pInvokeResult = PInvoke.QueryTDomPowerPulseSequenceResult(this._handle, Window, Time_Offset_of_First_Pulse, Measurement_Time, Period, Number_of_Pulses, Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the power measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_POW_OFF
        /// RSSPECAN_ATTR_MEAS_POW_SELECT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:POWer[:STATe] OFF
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:POWer:SELect ACPower | CPOWer | MCACpower | OBANdwidth | OBWidth | CN | CN0
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// This control sets the state of selected power measurement.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_POW_OFF
        /// 
        /// </param>
        /// <param name="Power_Measurement">
        /// Selects the ACP (adjacent channel power) standard.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEAS_POW_ACP  (0) - Adjacent Channel Power
        /// RSSPECAN_VAL_MEAS_POW_CPOW (1) - Channel Power with a Single Carrier
        /// RSSPECAN_VAL_MEAS_POW_MCAC (2) - Mulit Carrier ACP
        /// RSSPECAN_VAL_MEAS_POW_OBAN (3) - Occupied Bandwidth
        /// RSSPECAN_VAL_MEAS_POW_CN   (4) - C / N
        /// RSSPECAN_VAL_MEAS_POW_CN0  (5) - C / N0
        /// 
        /// Default Value: RSSPECAN_VAL_MEAS_POW_ACP   (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_POW_SELECT
        /// 
        /// Note(s):
        /// Adjacent Channel Power - Adjacent-channel power measurement with a single carrier signal
        /// Channel Power with a Single Carrier - Channel power measurement with a single carrier
        /// signal (equivalent to adjacent-channel power
        /// measurement with NO. OF ADJ CHAN = 0)
        /// Mulit Carrier ACP - Channel/adjacent-channel power measurement with
        /// several carrier signals
        /// Occupied Bandwidth - Measurement of occupied bandwidth
        /// C / N - Measurement of carrier-to-noise ratio
        /// C / N0 - Measurement of carrier-to-noise ratio referenced to
        /// 1 Hz bandwidth
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SelectPowerMeasurement(int Window, bool State, int Power_Measurement)
        {
            int pInvokeResult = PInvoke.SelectPowerMeasurement(this._handle, Window, System.Convert.ToUInt16(State), Power_Measurement);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the adjacent power channels measurement.
        /// 
        /// Note:
        /// (1) This function is only available in the frequency domain (span &gt; 0).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_POW_ADJ_NUM
        /// RSSPECAN_ATTR_MEAS_POW_ADJ_MODE
        /// RSSPECAN_ATTR_MEAS_POW_MODE
        /// RSSPECAN_ATTR_MEAS_POW_MODE_PORTABLE
        /// RSSPECAN_ATTR_MEAS_POW_RESULT_MODE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:ACPairs
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:MODE ABSolute | RELative
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:POWer:MODE WRITe | MAXHold
        /// CALCulate:MARKer&lt;1...6&gt;:FUNCtion:CPOWer:MODE CLR | MAX
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:POWer:RESult:PHZ ON | OFF
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="No_Of_Adj_Channels">
        /// The start frequency of the frequency sweep.
        /// 
        /// Valid Values:
        /// 0 to 12
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_POW_ADJ_NUM
        /// </param>
        /// <param name="Channel_Mode">
        /// Sets absolute and relative adjacent channel measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ABS (0) - Absolute
        /// RSSPECAN_VAL_REL (1) - Relative
        /// 
        /// Default Value: RSSPECAN_VAL_ABS (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_POW_ADJ_MODE
        /// </param>
        /// <param name="Power_Mode">
        /// Sets the Clear Write or Maxhold for Channel Power values. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEAS_POW_MODE_WRITE (0) - Clear/Write
        /// RSSPECAN_VAL_MEAS_POW_MODE_MAXH (1) - Max Hold
        /// 
        /// Default Value: RSSPECAN_VAL_MEAS_POW_MODE_WRITE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_POW_MODE or
        /// RSSPECAN_ATTR_MEAS_POW_MODE_PORTABLE
        /// </param>
        /// <param name="Result_Mode">
        /// Sets the query response of the power measurement results in the indicated measurement window between output of absolute values (OFF) and output referred to the measurement bandwidth (ON).. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Power
        /// VI_TRUE  (1) - /Hz
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_POW_RESULT_MODE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePowerMeasurement(int Window, int No_Of_Adj_Channels, int Channel_Mode, int Power_Mode, bool Result_Mode)
        {
            int pInvokeResult = PInvoke.ConfigurePowerMeasurement(this._handle, Window, No_Of_Adj_Channels, Channel_Mode, Power_Mode, System.Convert.ToUInt16(Result_Mode));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the adjacent power channel spacing.
        /// 
        /// Note:
        /// (1) This function is only available in the frequency domain (span &gt; 0).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_POW_CHANNEL_SPACING
        /// RSSPECAN_ATTR_MEAS_POW_ADJ_CHANNEL_SPACING
        /// RSSPECAN_ATTR_MEAS_POW_ALT_CHANNEL_SPACING
        /// 
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:SPACing:CHANnel&lt;1 to 11&gt;
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:SPACing:ACHannel
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:SPACing:ALTernate&lt;1 to 11&gt;
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Channel">
        /// Selects the channel type. The driver uses this value to selects the  RSSPECAN_ATTR_POW_CHANNEL_SPACING or RSSPECAN_ATTR_POW_ADJ_CHANNEL_SPACING or 
        /// RSSPECAN_ATTR_POW_ALT_CHANNEL_SPACING attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ACP_TX  (0) - Separate Channel
        /// RSSPECAN_VAL_ACP_ADJ (1) - Adjacent Channel
        /// RSSPECAN_VAL_ACP_ALT (2) - Alternate Channel
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_ACP_TX  (0)
        /// </param>
        /// <param name="Channel_Number">
        /// Sets the channel number.
        /// 
        /// Valid Values:
        /// 1 to 11
        /// 
        /// Default Value:
        /// 1
        /// 
        /// Note:
        /// (1) This control is valid for Separate channels and Alternate Channels. For Adjacent Channel has no effect.
        /// 
        /// (2) For R&amp;S FSL is only 1 valid for Separate Channel.
        /// </param>
        /// <param name="Spacing">
        /// Sets the selected channel spacing. 
        /// 
        /// Units: Hz
        /// 
        /// Valid Values: 100.0 to 2.0e+9 
        /// 
        /// Default Value: 20.0e+3
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_POW_CHANNEL_SPACING
        /// RSSPECAN_ATTR_MEAS_POW_ADJ_CHANNEL_SPACING
        /// RSSPECAN_ATTR_MEAS_POW_ALT_CHANNEL_SPACING
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePowerChannelSpacing(int Window, int Channel, int Channel_Number, double Spacing)
        {
            int pInvokeResult = PInvoke.ConfigurePowerChannelSpacing(this._handle, Window, Channel, Channel_Number, Spacing);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the adjacent power channel bandwidth.
        /// 
        /// Note:
        /// (1) This function is only available in the frequency domain (span &gt; 0).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_POW_CHANNEL_BANDWIDTH
        /// RSSPECAN_ATTR_MEAS_CHAN_POW_BANDWIDTH
        /// RSSPECAN_ATTR_MEAS_POW_ADJ_CHANNEL_BANDWIDTH
        /// RSSPECAN_ATTR_MEAS_POW_ALT_CHANNEL_BANDWIDTH
        /// 
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:BANDwidth|BWIDth[:CHANnel]
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:BANDwidth|BWIDth:ACHannel
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:BANDwidth|BWIDth:ALTernate&lt;1 to 11&gt;
        /// CALCulate:MARKer&lt;1...6&gt;:FUNCtion:CPOWer:BANDwidth
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Channel">
        /// Selects the channel type. The driver uses this value to select the  RSSPECAN_ATTR_POW_CHANNEL_SPACING or RSSPECAN_ATTR_POW_ADJ_CHANNEL_SPACING or 
        /// RSSPECAN_ATTR_POW_ALT_CHANNEL_SPACING attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ACP_TX  (0) - Separate Channel
        /// RSSPECAN_VAL_ACP_ADJ (1) - Adjacent Channel
        /// RSSPECAN_VAL_ACP_ALT (2) - Alternate Channel
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_ACP_TX  (0)
        /// </param>
        /// <param name="Channel_Number">
        /// Sets the channel number.
        /// 
        /// Valid Values:
        /// 1 to 11
        /// 
        /// Default Value:
        /// 1
        /// 
        /// Note:
        /// (1) This control is valid for Alternate Channels. For TX Channel and Adjacent Channel has no effect.
        /// </param>
        /// <param name="Bandwidth">
        /// Sets the selected channel bandwidth.
        /// 
        /// Units: Hz
        /// 
        /// Valid Values: 100.0 to 1.0e+9 
        /// 
        /// Default Value: 14.0e+3
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_POW_CHANNEL_BANDWIDTH
        /// RSSPECAN_ATTR_MEAS_POW_ADJ_CHANNEL_BANDWIDTH
        /// RSSPECAN_ATTR_MEAS_POW_ALT_CHANNEL_BANDWIDTH
        /// RSSPECAN_ATTR_MEAS_CHAN_POW_BANDWIDTH
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePowerChannelBandwidth(int Window, int Channel, int Channel_Number, double Bandwidth)
        {
            int pInvokeResult = PInvoke.ConfigurePowerChannelBandwidth(this._handle, Window, Channel, Channel_Number, Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the unit used for the channel power measurement.
        /// 
        /// The numeric suffix &lt;1...6&gt; is irrelevant.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_POW_CHANNEL_UNIT
        /// 
        /// Remote-control command(s): CALCulate:MARKer&lt;1...6&gt;:FUNCtion:CPOWer:UNIT DBM | DBMV | DBUV | VOLT | WATT | DUVM | DUAM | V | W | V_M | W_M2
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Channel_Power_Unit">
        /// Defines the unit used for the channel power measurement.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_CPOW_UNIT_DBM  (0) - DBM
        /// RSSPECAN_VAL_CPOW_UNIT_DBMV (1) - DBMV
        /// RSSPECAN_VAL_CPOW_UNIT_DBUV (2) - DBUV
        /// RSSPECAN_VAL_CPOW_UNIT_VOLT (3) - VOLT
        /// RSSPECAN_VAL_CPOW_UNIT_WATT (4) - WATT
        /// RSSPECAN_VAL_CPOW_UNIT_DUVM (5) - DUVM
        /// RSSPECAN_VAL_CPOW_UNIT_DUAM (6) - DUAM
        /// RSSPECAN_VAL_CPOW_UNIT_V    (7) - V
        /// RSSPECAN_VAL_CPOW_UNIT_W    (8) - W
        /// RSSPECAN_VAL_CPOW_UNIT_V_M  (9) - V_M
        /// RSSPECAN_VAL_CPOW_UNIT_W_M2 (10) - W_M2
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_CPOW_UNIT_DBM (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_POW_CHANNEL_UNIT
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureChannelPowerUnit(int Channel_Power_Unit)
        {
            int pInvokeResult = PInvoke.ConfigureChannelPowerUnit(this._handle, Channel_Power_Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the correction of the instrument inherent noise for channel power measurement. On activating this function, a reference measurement of the instrument inherent noise is performed. The measured noise power is then subtracted from the power in the examined channel. The measurement of the instrument inherent noise is then repeated after any change of the center frequency, resolution bandwidth, sweep time and level setting.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available in the frequency domain (span &gt; 0).
        /// 
        /// (2) This function is not available for R&amp;S FSL or R&amp;S ESL.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_POW_NOISE_CORR
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:NCORrection ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Noise_Correction_State">
        /// Turns on or off the noise correction. The driver uses this value to set the RSSPECAN_ATTR_POW_NOISE_CORR attribute.
        /// 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_POW_NOISE_CORR
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureChannelPowerNoiseCorrection(int Window, bool Noise_Correction_State)
        {
            int pInvokeResult = PInvoke.ConfigureChannelPowerNoiseCorrection(this._handle, Window, System.Convert.ToUInt16(Noise_Correction_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function assigns the channel/adjacent channel power measurement of the indicated trace in the selected measurement window. The corresponding trace should be active, ie its states should differ from blank.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_POW_TRACE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:TRACe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace_Number">
        /// Selects trace number.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 1 to 6 for R&amp;S FSL or R&amp;S ESL or R&amp;S FSV
        /// 
        /// Default Value: 1
        /// 
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_POW_TRACE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureChannelPowerTrace(int Window, int Trace_Number)
        {
            int pInvokeResult = PInvoke.ConfigureChannelPowerTrace(this._handle, Window, Trace_Number);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the standard for channel power measurement.
        /// 
        /// Note:
        /// 
        /// FAST ACP is not available if a WLAN standard is selected.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_POW_STANDARD
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:POWer:PRESet  NONE | NADC | TETRA | PDC | PHS | CDPD  | F8CDma | R8CDma | F19CDma | R19CDma | FWCDma | RWCDma | FW3Gppcdma | RW3Gppcdma | S2CDma | D2CDma | M2CDma | FIS95C0 | RIS95C0 | FIS95C1 | RIS95C1 | TCDMa | AWLan | BWLan | WIM | WIBR | RTCDM | RFID14443 | GSM | EUTRa | REUTra
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="ACP_Standard">
        /// Selects the ACP (adjacent channel power) standard.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEAS_POW_STD_NONE    (0) - None
        /// RSSPECAN_VAL_MEAS_POW_STD_NADC    (1) - NADC
        /// RSSPECAN_VAL_MEAS_POW_STD_TETRA   (2) - TETRA
        /// RSSPECAN_VAL_MEAS_POW_STD_PDC     (3) - PDC
        /// RSSPECAN_VAL_MEAS_POW_STD_PHS     (4) - PHS
        /// RSSPECAN_VAL_MEAS_POW_STD_CDPD    (5) - CDPD
        /// RSSPECAN_VAL_MEAS_POW_STD_FIS95A  (6) - CDMA IS95A forward
        /// RSSPECAN_VAL_MEAS_POW_STD_RIS95A  (7) - CDMA IS95A reverse
        /// RSSPECAN_VAL_MEAS_POW_STD_FIS95C0 (8) - CDMA IS95C Class 0 forward
        /// RSSPECAN_VAL_MEAS_POW_STD_RIS95C0 (9) - CDMA IS95C Class 0 reverse
        /// RSSPECAN_VAL_MEAS_POW_STD_FJ008   (10) - CDMA J-STD008 forward
        /// RSSPECAN_VAL_MEAS_POW_STD_RJ008   (11) - CDMA J-STD008 reverse
        /// RSSPECAN_VAL_MEAS_POW_STD_FIS95C1 (12) - CDMA IS95C Class 1 forward
        /// RSSPECAN_VAL_MEAS_POW_STD_RIS95C1 (13) - CDMA IS95C Class 1 reverse
        /// RSSPECAN_VAL_MEAS_POW_STD_FWCD    (14) - W-CDMA 4.096 MHz forward
        /// RSSPECAN_VAL_MEAS_POW_STD_RWCD    (15) - W-CDMA 4.096 MHz reverse
        /// RSSPECAN_VAL_MEAS_POW_STD_FW3G    (16) - W-CDMA 3.84 MHz forward
        /// RSSPECAN_VAL_MEAS_POW_STD_RW3G    (17) - W-CDMA 3.84 MHz reverse
        /// RSSPECAN_VAL_MEAS_POW_STD_D2CD    (18) - CDMA 2000 direct sequence
        /// RSSPECAN_VAL_MEAS_POW_STD_S2CD    (19) - CDMA 2000 MC1 multi carrier with 1 carrier
        /// RSSPECAN_VAL_MEAS_POW_STD_M2CD    (20) - CDMA 2000 MC3 multi carrier with 3 carriers
        /// RSSPECAN_VAL_MEAS_POW_STD_TCDM    (21) - TD-SCDMA
        /// RSSPECAN_VAL_MEAS_POW_STD_AWL     (22) - WLAN 802.11a
        /// RSSPECAN_VAL_MEAS_POW_STD_BWL     (23) - WLAN 802.11b
        /// RSSPECAN_VAL_MEAS_POW_STD_WIM     (24) - WiMAX
        /// RSSPECAN_VAL_MEAS_POW_STD_WIBRO   (25) - WiBro 
        /// RSSPECAN_VAL_MEAS_POW_STD_RTCDMA  (26) - TD?SCDMA reverse 
        /// RSSPECAN_VAL_MEAS_POW_STD_RFID    (27) - RFID 14443
        /// RSSPECAN_VAL_MEAS_POW_STD_GSM     (28) - GSM
        /// RSSPECAN_VAL_MEAS_POW_STD_EUTR    (29) - EUTRA/LTE Square
        /// RSSPECAN_VAL_MEAS_POW_STD_REUT    (30) - EUTRA/LTE Square/RRC
        /// 
        /// Default Value: RSSPECAN_VAL_MEAS_POW_STD_NONE   (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_POW_STANDARD
        /// 
        /// Note(s):
        /// 
        /// (1) Value 26 and 27 are available for R&amp;S FSL or R&amp;S FSV only.
        /// 
        /// (2) Value 28 is available for R&amp;S FSV only.
        /// 
        /// (3) Values 29 and 30 are available for R&amp;S FSV, R&amp;S FSQ, R&amp;S FSP and R&amp;S FSU only.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureChannelPowerStandard(int Window, int ACP_Standard)
        {
            int pInvokeResult = PInvoke.ConfigureChannelPowerStandard(this._handle, Window, ACP_Standard);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns all predefined ACP standards and all user standards.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:POWer:STANdard:CATalog?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the string you specify for the catalog list parameter.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// Valid Values:
        /// any number
        /// 
        /// Default Value: 1024
        /// 
        /// </param>
        /// <param name="Power_Standard_Catalog">
        /// This control returns all predefined ACP standards and all user standards.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryChannelPowerStandardCatalog(int Buffer_Size, System.Text.StringBuilder Power_Standard_Catalog)
        {
            int pInvokeResult = PInvoke.QueryChannelPowerStandardCatalog(this._handle, Buffer_Size, Power_Standard_Catalog);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the user standard for channel power measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_ACP_USER_STD_SET
        /// RSSPECAN_ATTR_ACP_USER_STD_SAVE
        /// RSSPECAN_ATTR_ACP_USER_STD_DELETE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:POWer:PRESet
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:POWer:STANdard:SAVE
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:POWer:STANdard:DELete
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Operation">
        /// This control selects the operation to be performed with user standard.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_USTD_SET  (0) - Set User Standard
        /// RSSPECAN_VAL_USTD_SAVE (1) - Save User Standard
        /// RSSPECAN_VAL_USTD_DEL  (2) - Delete User Standard
        /// 
        /// Default Value: RSSPECAN_VAL_USTD_SET  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ACP_USER_STD_SET
        /// RSSPECAN_ATTR_ACP_USER_STD_SAVE
        /// RSSPECAN_ATTR_ACP_USER_STD_DELETE
        /// </param>
        /// <param name="User_Standard_Name">
        /// This control selects name of ACP user standard.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureChannelPowerUserStandard(int Window, int Operation, string User_Standard_Name)
        {
            int pInvokeResult = PInvoke.ConfigureChannelPowerUserStandard(this._handle, Window, Operation, User_Standard_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This sets the reference value for the relative measurement to the currently measured channel power.
        /// 
        /// Note(s):
        /// This function sets instrument to Channel Power with a Single Carrier measurement
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_POW_REF_VALUE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:REFerence:AUTO ONCE
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int PowerAdjustReferenceLevel(int Window, uint Timeout)
        {
            int pInvokeResult = PInvoke.PowerAdjustReferenceLevel(this._handle, Window, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function adjusts the frequency span, the measurement bandwidths and the detector as required for the number of channels, the channel bandwidths and the channel spacings selected in the active power measurement. If necessary, adjacent-channel power measurement is switched on prior to the adjustment.
        /// 
        /// Note:
        /// (1) This attribute is available only for measurements in the frequency domain (span &gt; 0).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_POW_ADJ_PRESET
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:PRESet ACPower | CPOWer | OBANdwidth | CN | CN0 | MCACpower
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Channel_Power_Type">
        /// Specifies the channel power standard. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ADJ_PRE_ACP  (0) - Adjacent Channel
        /// RSSPECAN_VAL_ADJ_PRE_CPOW (1) - Channel Power
        /// RSSPECAN_VAL_ADJ_PRE_MCAC (2) - Multi Carrier Adjacent Channel Power
        /// RSSPECAN_VAL_ADJ_PRE_OBAN (3) - Occupied Bandwidth
        /// RSSPECAN_VAL_ADJ_PRE_CN   (4) - C / N
        /// RSSPECAN_VAL_ADJ_PRE_CN0  (5) - C / No
        /// 
        /// Default Value: RSSPECAN_VAL_ADJ_PRE_ACP  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_POW_ADJ_PRESET
        /// 
        /// Notes:
        /// 
        /// (1) Adjacent Channel:
        /// Adjacent channel power measurement.
        /// 
        /// (2) Channel Power:
        /// Channel power measurement.
        /// 
        /// (3) Occupied Bandwidth:
        /// Occupied bandwidth power measurement.
        /// 
        /// (4) C / N
        /// Carrier/Noise measurement.
        /// 
        /// (5) C / No
        /// Carrier/Noise measurement normalized to 1 Hz bandwidth.
        /// 
        /// (6) Multi Carrier Adjacent Channel Power
        /// Adjacent channel power measurement with multiple triggers.
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int PowerPresetMeasurement(int Window, int Channel_Power_Type, uint Timeout)
        {
            int pInvokeResult = PInvoke.PowerPresetMeasurement(this._handle, Window, Channel_Power_Type, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function whether the current instrument state is still in compliance with the Standard currently in use. Modifying instrument settings that are not included in a Standard do
        /// not break Standard compliance; modifying settings that are included in a Standard, will break compliance.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_POW_STANDARD_CHECK
        /// 
        /// Remote-control command(s):
        /// CALCulate:MARKer:FUNCtion:POWer:PRESet:CHECk?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Check">
        /// This control displays whether the current instrument state is still in compliance with the Standard currently in use.
        /// 
        /// Valid Values:
        /// 0 - Instrument Settings are not compliant to the selected standard.
        /// 1 - Instrument settings are compliant to the selected standard.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_POW_STANDARD_CHECK
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int PowerStandardCheck(out int Check)
        {
            int pInvokeResult = PInvoke.PowerStandardCheck(this._handle, out Check);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the power measurement results.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:POWer:RESult? ACPower | CPOWer | MCACpower | OBANdwidth | OBWidth | CN | CN0
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Power_Measurement">
        /// Selects the ACP (adjacent channel power) standard. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEAS_POW_ACP  (0) - Adjacent Channel Power
        /// RSSPECAN_VAL_MEAS_POW_CPOW (1) - Channel Power with a Single Carrier
        /// RSSPECAN_VAL_MEAS_POW_MCAC (2) - Mulit Carrier ACP
        /// RSSPECAN_VAL_MEAS_POW_OBAN (3) - Occupied Bandwidth
        /// RSSPECAN_VAL_MEAS_POW_CN   (4) - C / N
        /// RSSPECAN_VAL_MEAS_POW_CN0  (5) - C / N0
        /// 
        /// Default Value: RSSPECAN_VAL_MEAS_POW_ACP   (0)
        /// 
        /// Note(s):
        /// Adjacent Channel Power - Adjacent-channel power measurement with a single carrier signal
        /// Channel Power with a Single Carrier - Channel power measurement with a single carrier
        /// signal (equivalent to adjacent-channel power
        /// measurement with NO. OF ADJ CHAN = 0)
        /// Mulit Carrier ACP - Channel/adjacent-channel power measurement with
        /// several carrier signals
        /// Occupied Bandwidth - Measurement of occupied bandwidth
        /// C / N - Measurement of carrier-to-noise ratio
        /// C / N0 - Measurement of carrier-to-noise ratio referenced to
        /// 1 Hz bandwidth
        /// </param>
        /// <param name="Array_Size">
        /// Defines the memory size allocated for Results. 
        /// 
        /// Valid Values: 
        /// any number
        /// 
        /// Default Value: 12
        /// </param>
        /// <param name="Results">
        /// Returns the measured values:
        /// 
        /// Returned Values:
        /// (1) ACPower: Adjacent-channel power measurement
        /// Results are output in the following sequence, separated by commas:
        /// 1. Power of transmission channel
        /// 2. Power of lower adjacent channel
        /// 3. Power of upper adjacent channel
        /// 4. Power of lower alternate channel 1
        /// 5. Power of upper alternate channel 1
        /// 6. Power of lower alternate channel 2
        /// 7. Power of upper alternate channel 2
        /// The number of measured values returned depends on the number of
        /// adjacent/alternate channels selected with
        /// SENSe:POWer:ACHannel:ACPairs.
        /// With logarithmic scaling (RANGE LOG), the power is output in the currently selected level unit; with linear scaling (RANGE LIN dB or LIN %), the power is output in W. If SENSe:POWer:ACHannel:MODE REL is selected, the adjacent/alternate-channel power is output in dB.
        /// 
        /// (2) CPOWer Channel power measurement - 
        /// With logarithmic scaling (RANGE LOG), the channel power is output in the currently selected level unit; with linear scaling (RANGE LIN dB or LIN %), the channel power is output in W.
        /// 
        /// (3) MCACpower: Channel/adjacent-channel power measurement with several carrier signals Results are output in the following sequence, separated by commas:
        /// 1. Power of carrier signal 1
        /// 2. Power of carrier signal 2
        /// 3. Power of carrier signal 3
        /// 4. Power of carrier signal 4
        /// 5. Total power of all carrier signals
        /// 6. Power of lower adjacent channel
        /// 7. Power of upper adjacent channel
        /// 8. Power of lower alternate channel 1
        /// 9. Power of upper alternate channel 1
        /// 10. Power of lower alternate channel 2
        /// 11. Power of upper alternate channel 2
        /// The number of measured values returned depends on the number of
        /// carrier signals and adjacent/alternate channels selected with
        /// SENSe:POWer:ACHannel:TXCHannel:COUNt and SENSe:POWer:ACHannel:ACPairs.
        /// If only one carrier signal is measured, the total value of all carrier signals will not be output. With logarithmic scaling (RANGE LOG), the power is output in dBm; 
        /// with linear scaling (RANGE LIN dB or LIN %), the power is output in W. If SENSe:POWer:ACHannel:MODE REL is selected, the
        /// adjacent/alternate-channel power is output in dB.
        /// 
        /// (4) OBANdwidth | OBWidth Measurement of occupied bandwidth
        /// The occupied bandwidth in Hz is returned.
        /// 
        /// (5) CN Measurement of carrier-to-noise ratio The carrier-to-noise ratio in dB is returned.
        /// 
        /// (6) CN0 Measurement of carrier-to-noise ratio referenced to 1 Hz bandwidth. The carrier-to-noise ratio in dB/Hz is returned.
        /// 
        /// </param>
        /// <param name="Returned_Values">
        /// Returns number of returned values in Results array.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryPowerResults(int Window, int Power_Measurement, int Array_Size, double[] Results, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.QueryPowerResults(this._handle, Window, Power_Measurement, Array_Size, Results, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the power channel weighting filters.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available for R&amp;S FSV, R&amp;S FSP, R&amp;S FSQ, R&amp;S FSU instruments.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_WEIGHT_FILTER_ALL_STATE
        /// RSSPECAN_ATTR_SEM_WEIGHT_FILTER_ALL_ALPHA
        /// RSSPECAN_ATTR_SEM_WEIGHT_FILTER_ACH_STATE
        /// RSSPECAN_ATTR_SEM_WEIGHT_FILTER_ACH_ALPHA
        /// RSSPECAN_ATTR_SEM_WEIGHT_FILTER_ALT_STATE
        /// RSSPECAN_ATTR_SEM_WEIGHT_FILTER_ALT_ALPHA
        /// RSSPECAN_ATTR_SEM_WEIGHT_FILTER_TX_CHANNEL_STATE
        /// RSSPECAN_ATTR_SEM_WEIGHT_FILTER_TX_CHANNEL_ALPHA
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:FILTer[:STATe]:ALL ON | OFF
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:FILTer:ALPHa[:ALL]
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:FILTer[:STATe]:ACHannel ON | OFF
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:FILTer:ALPHa:ACHannel
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:FILTer[:STATe]:ALTernate&lt;1..11&gt; ON | OFF
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:FILTer:ALPHa:ALTernate&lt;1..11&gt;
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:FILTer[:STATe]:CHANnel&lt;1..12&gt; ON | OFF
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:FILTer:ALPHa:CHANnel&lt;1..12&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Filter_Type">
        /// This control selects the type of the filter to be modified.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WFILTER_ALL (0) - All Channels
        /// RSSPECAN_VAL_WFILTER_ADJ (1) - Adjacent Channels
        /// RSSPECAN_VAL_WFILTER_ALT (2) - Alternate Channels
        /// RSSPECAN_VAL_WFILTER_TX  (3) - TX Channels
        /// 
        /// Default Value: RSSPECAN_VAL_WFILTER_ALL (0)
        /// </param>
        /// <param name="Filter_State">
        /// This control sets the state of the selected filter.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_WEIGHT_FILTER_ALL_STATE
        /// RSSPECAN_ATTR_SEM_WEIGHT_FILTER_ACH_STATE
        /// RSSPECAN_ATTR_SEM_WEIGHT_FILTER_ALT_STATE
        /// RSSPECAN_ATTR_SEM_WEIGHT_FILTER_TX_CHANNEL_STATE
        /// </param>
        /// <param name="Filter_Value">
        /// This control sets the weighting filter alpha value.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0.22
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_WEIGHT_FILTER_ALL_ALPHA
        /// RSSPECAN_ATTR_SEM_WEIGHT_FILTER_ACH_ALPHA
        /// RSSPECAN_ATTR_SEM_WEIGHT_FILTER_ALT_ALPHA
        /// RSSPECAN_ATTR_SEM_WEIGHT_FILTER_TX_CHANNEL_ALPHA
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available if Filter State is set to ON.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePowerChannelWeightingFilters(int Filter_Type, bool Filter_State, double Filter_Value)
        {
            int pInvokeResult = PInvoke.ConfigurePowerChannelWeightingFilters(this._handle, Filter_Type, System.Convert.ToUInt16(Filter_State), Filter_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the high-speed channel/adjacent channel power measurement.
        /// The measurement itself is performed in the time domain on the center frequencies of the individual channels. The function automatically switches to the time domain and back. Depending on the selected mobile radio standard, weighting filters with sqrt(cos) characteristic or very steep-sided channel filters are used for band limitation.
        /// 
        /// Note:
        /// 
        /// This function is only available in the frequency domain (span &gt; 0).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_ACP_HSP
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:HSPeed ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Fast_ACP_State">
        /// Turns on or off the high-speed channel/adjacent channel power measurement..
        /// 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_POW_HSP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureACPFastACPState(int Window, bool Fast_ACP_State)
        {
            int pInvokeResult = PInvoke.ConfigureACPFastACPState(this._handle, Window, System.Convert.ToUInt16(Fast_ACP_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function adapts the reference level to the measured channel power and - if required - switches on previously the adjacent channel power measurement. This ensures that the signal path of the instrument is not overloaded. Since the measurement bandwidth is significantly smaller than the signal bandwidth in channel power measurements, the signal path can be overloaded although the trace is still significantly below the reference level. If the measured channel power equals the reference level, the signal path is not overloaded.
        /// 
        /// Note:
        /// (1) The attribute is only available in the frequency domain (span &gt; 0).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_ACP_PRESET_REF_LEVEL
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:PRESet:RLEVel
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ACPChannelPowerAutoAdjust(int Window, uint Timeout)
        {
            int pInvokeResult = PInvoke.ACPChannelPowerAutoAdjust(this._handle, Window, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines an additional level offset to be taken into account for the analyzer's reference level setting. It is only available if Fast ACP is not active and no predefine standard is selected.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S FSP, R&amp;S FSQ, R&amp;S FSU instruments.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_ACP_PRESET_REF_LEVEL_OFFSET
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:PRESet:RLEVel:OFFSet
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Adjust_Ref_Level_Offset">
        /// Defines an additional level offset to be taken into account for the analyzer's reference level setting. It is only available if Fast ACP is not active and no predefine standard is selected.
        /// 
        /// Valid Values:
        /// 0.0 to 100.0
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_ACP_PRESET_REF_LEVEL_OFFSET
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ACPChannelPowerAdjustRefLevelOffset(int Window, double Adjust_Ref_Level_Offset)
        {
            int pInvokeResult = PInvoke.ACPChannelPowerAdjustRefLevelOffset(this._handle, Window, Adjust_Ref_Level_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function adapts the reference level to the measured channel power and - if required - switches on previously the adjacent channel power measurement. This ensures that the signal path of the instrument is not overloaded. Since the measurement bandwidth is significantly smaller than the signal bandwidth in channel power measurements, the signal path can be overloaded althought the trace is still significantly below the reference level. If the measured channel power equals the reference level, the signal path is not overloaded.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_ACP_PRESET_REF_LEVEL_ADJ
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:PRESet:RLEVel:ADJust
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ACPAdjustReferenceLevelToChannelPower(int Window, uint Timeout)
        {
            int pInvokeResult = PInvoke.ACPAdjustReferenceLevelToChannelPower(this._handle, Window, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on and off the limit check for adjacent channel power measurements in the selected measurement window.
        /// 
        /// Note:
        /// 
        /// This function is only available in the frequency domain (span &gt; 0).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_ACP_LIMIT_STATE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:ACPower[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// Switches on and off the limit check for adjacent channel power measurements.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ACP_LIMIT_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureACPLimitState(int Window, bool State)
        {
            int pInvokeResult = PInvoke.ConfigureACPLimitState(this._handle, Window, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the limit value of the adjacent channel.
        /// 
        /// Note:
        /// 
        /// This function is only available in the frequency domain (span &gt; 0).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_ACP_LIMIT_REL_STATE
        /// no attribute
        /// RSSPECAN_ATTR_MEAS_ACP_LIMIT_ABS_STATE
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:ACPower:ACHannel[:RELative]:STATe ON | OFF
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:ACPower:ACHannel[:RELative]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:ACPower:ACHannel:ABSolute:STATe ON | OFF
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:ACPower:ACHannel:ABSolute
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Relative_State">
        /// Switches the limit check for the relative limit value of the adjacent channel when adjacent channel power measurement is performed. 
        /// 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_ACP_LIMIT_REL_STATE
        /// </param>
        /// <param name="Relative_Value">
        /// Sets the relative limit value.
        /// 
        /// Units: dB
        /// 
        /// Valid Values: 0.0 to 100.0
        /// 
        /// Default Value: 0.0
        /// 
        /// </param>
        /// <param name="Absolute_State">
        /// Switches the limit check for the absolute limit value of the adjacent channel when adjacent channel power measurement is performed.
        /// 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ACP_LIMIT_ABS_STATE
        /// 
        /// Note(s):
        /// 
        /// (1) For R&amp;S FSL can be set Abs and Rel On together. If Relative State is set On, Absolute State is ignored.
        /// 
        /// </param>
        /// <param name="Absolute_Value">
        /// Sets the relative limit value.
        /// 
        /// Units: dBm
        /// 
        /// Valid Values: -200.0 to 200.0
        /// 
        /// Default Value: -200.0
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureACPAdjacentChannelLimit(int Window, bool Relative_State, double Relative_Value, bool Absolute_State, double Absolute_Value)
        {
            int pInvokeResult = PInvoke.ConfigureACPAdjacentChannelLimit(this._handle, Window, System.Convert.ToUInt16(Relative_State), Relative_Value, System.Convert.ToUInt16(Absolute_State), Absolute_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the limit value of the alternate adjacent channel.
        /// 
        /// Note:
        /// 
        /// This function is only available in the frequency domain (span &gt; 0).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_ACP_ALT_LIMIT_REL_STATE
        /// no attribute
        /// RSSPECAN_ATTR_MEAS_ACP_ALT_LIMIT_ABS_STATE
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to  8&gt;:ACPower:ALTernate&lt;1 to 11&gt;[:RELative]:STATe ON | OFF
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:ACPower:ALTernate&lt;1 to 11&gt;[:RELative] 
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:ACPower:ALTernate&lt;1 to 11&gt;:ABSolute:STATe ON | OFF
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:ACPower:ALTernate&lt;1 to 11&gt;:ABSolute
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Alternate_Channel_Number">
        /// Sets the channel number.
        /// 
        /// Valid Values:
        /// 1 to 11
        /// 
        /// Default Value:
        /// 1
        /// 
        /// 
        /// </param>
        /// <param name="Relative_State">
        /// Switches the limit check for the relative limit value of the alternate adjacent channel when adjacent channel power measurement is performed.  The driver uses this value to set the RSSPECAN_ATTR_MEAS_ACP_ALT_LIMIT_REL_STATE attribute.
        /// 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_ACP_ALT_LIMIT_REL_STATE
        /// 
        /// Note(s):
        /// 
        /// (1) For R&amp;S FSL can't be set both Relative and Absolute state On. If Absolute is set On, Relative is set Off automatically.
        /// 
        /// </param>
        /// <param name="Relative_Value">
        /// Sets the relative limit value.
        /// 
        /// Units: dB
        /// 
        /// Valid Values: 0.0 to 100.0
        /// 
        /// Default Value: 0.0
        /// 
        /// </param>
        /// <param name="Absolute_State">
        /// Switches the limit check for the absolute limit value of the alternate adjacent channel when adjacent channel power measurement is performed.
        /// 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_ACP_ALT_LIMIT_ABS_STATE
        /// 
        /// Note(s):
        /// 
        /// (1) For R&amp;S FSL can't be set both Relative and Absolute state On. If Absolute is set On, Relative is set Off automatically.
        /// </param>
        /// <param name="Absolute_Value">
        /// Sets the relative limit value.
        /// 
        /// Units: dBm
        /// 
        /// Valid Values: -200.0 to 200.0
        /// 
        /// Default Value: -200.0
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureACPAlternateChannelLimit(int Window, int Alternate_Channel_Number, bool Relative_State, double Relative_Value, bool Absolute_State, double Absolute_Value)
        {
            int pInvokeResult = PInvoke.ConfigureACPAlternateChannelLimit(this._handle, Window, Alternate_Channel_Number, System.Convert.ToUInt16(Relative_State), Relative_Value, System.Convert.ToUInt16(Absolute_State), Absolute_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the adjacent channel limit results.
        /// 
        /// Note:
        /// 
        /// This function is only available in the frequency domain (span &gt; 0).
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:ACPower:ACHannel:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Lower_Result">
        /// Returns the lower limit result.
        /// 
        /// Returned Values:
        /// 0 - Pass
        /// 1 - Fail
        /// 
        /// </param>
        /// <param name="Upper_Result">
        /// Returns the upper limit result.
        /// 
        /// Returned Values:
        /// 0 - Pass
        /// 1 - Fail
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryACPAdjacentChannelLimitCheckResults(int Window, out int Lower_Result, out int Upper_Result)
        {
            int pInvokeResult = PInvoke.QueryACPAdjacentChannelLimitCheckResults(this._handle, Window, out Lower_Result, out Upper_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the alternate channel limit results.
        /// 
        /// Note:
        /// 
        /// This function is only available in the frequency domain (span &gt; 0).
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:ACPower:ALTernate&lt;1 to 11&gt;:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Alternate_Channel_Number">
        /// Sets the channel number.
        /// 
        /// Valid Values:
        /// 1 to 11
        /// 
        /// Default Value:
        /// 1
        /// 
        /// 
        /// </param>
        /// <param name="Lower_Result">
        /// Returns the lower limit result.
        /// 
        /// Returned Values:
        /// 0 - Fail
        /// 1 - Pass
        /// 
        /// </param>
        /// <param name="Upper_Result">
        /// Returns the upper limit result.
        /// 
        /// Returned Values:
        /// 0 - Fail
        /// 1 - Pass
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryACPAlternateChannelLimitCheckResults(int Window, int Alternate_Channel_Number, out int Lower_Result, out int Upper_Result)
        {
            int pInvokeResult = PInvoke.QueryACPAlternateChannelLimitCheckResults(this._handle, Window, Alternate_Channel_Number, out Lower_Result, out Upper_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the adjacent power channels measurement.
        /// 
        /// Note:
        /// (1) This function is only available in the frequency domain (span &gt; 0).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_POW_ADJ_NUM
        /// RSSPECAN_ATTR_MEAS_POW_CARR_SIG_NUM
        /// RSSPECAN_ATTR_MEAS_POW_ADJ_MODE
        /// RSSPECAN_ATTR_MEAS_POW_MODE
        /// RSSPECAN_ATTR_MEAS_POW_RESULT_MODE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:ACPairs
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:TXCHannel:COUNt
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:MODE ABSolute | RELative
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:POWer:MODE WRITe | MAXHold
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:POWer:RESult:PHZ ON | OFF
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="No_Of_Adj_Channels">
        /// The start frequency of the frequency sweep.
        /// 
        /// Valid Values:
        /// 0 to 12
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_POW_ADJ_NUM
        /// </param>
        /// <param name="No_Of_TX_Channels">
        /// Sets the number of TX channels.
        /// 
        /// Valid Values:
        /// 1 to 12
        /// 
        /// Default Value: 4
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_POW_CARR_SIG_NUM
        /// </param>
        /// <param name="Channel_Mode">
        /// Sets absolute and relative adjacent channel measurement. The driver uses this value to set the RSSPECAN_ATTR_MEAS_POW_ADJ_MODE attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ABS (0) - Absolute
        /// RSSPECAN_VAL_REL (1) - Relative
        /// 
        /// Default Value: RSSPECAN_VAL_ABS (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_POW_ADJ_MODE
        /// 
        /// Note:
        /// (1) This control has effect only for No Of Adj Channels &gt; 0
        /// 
        /// 
        /// </param>
        /// <param name="Power_Mode">
        /// Sets the Clear Write or Maxhold for Channel Power values. The driver uses this value to set the RSSPECAN_ATTR_MEAS_POW_MODE attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEAS_POW_MODE_WRITE (0) - Clear/Write
        /// RSSPECAN_VAL_MEAS_POW_MODE_MAXH (1) - Max Hold
        /// 
        /// Default Value: RSSPECAN_VAL_MEAS_POW_MODE_WRITE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_POW_MODE
        /// 
        /// </param>
        /// <param name="Result_Mode">
        /// Sets the query response of the power measurement results in the indicated measurement window between output of absolute values (OFF) and output referred to the measurement bandwidth (ON).. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Power
        /// VI_TRUE  (1) - /Hz
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_POW_RESULT_MODE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMulticarrierACPPower(int Window, int No_Of_Adj_Channels, int No_Of_TX_Channels, int Channel_Mode, int Power_Mode, bool Result_Mode)
        {
            int pInvokeResult = PInvoke.ConfigureMulticarrierACPPower(this._handle, Window, No_Of_Adj_Channels, No_Of_TX_Channels, Channel_Mode, Power_Mode, System.Convert.ToUInt16(Result_Mode));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Activates the automatic selection of a transmission channel to be used as a reference channel in relative adjacent-channel power measurements. The transmission channel with the highest power, the transmission channel with the lowest power, or the transmission channel nearest to the adjacent channels can be defined as a reference channel.
        /// 
        /// Note:
        /// (1) The attribute is available only for multicarrier channel and adjacent-channel power measurements (CALC:MARK:FUNC:POW:SEL MCAC) in the frequency domain (span &gt; 0).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_POW_REF_CHAN_SEL_AUTO
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:REFerence:TXCHannel:AUTO MINimum | MAXimum | LHIGhest
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Reference">
        /// This control sets the type of reference selection.
        /// 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ADJ_RCHAN_AUTO_MIN  (0) - Minimum
        /// RSSPECAN_VAL_ADJ_RCHAN_AUTO_MAX  (1) - Maximum
        /// RSSPECAN_VAL_ADJ_RCHAN_AUTO_LHIG (2) - Highest
        /// 
        /// Default Value: RSSPECAN_VAL_ADJ_RCHAN_AUTO_MIN  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_POW_REF_CHAN_SEL_AUTO
        /// 
        /// Note(s):
        /// (1) Minimum  - Transmission channel with the lowest power
        /// 
        /// (2) Maximum - Transmission channel with the highest power
        /// 
        /// (3) Highest - Lowermost transmission channel for the lower adjacent channels, uppermost transmission channel for the upper adjacent channels
        /// 
        /// ,,
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureACPReferenceChannelAuto(int Window, int Reference)
        {
            int pInvokeResult = PInvoke.ConfigureACPReferenceChannelAuto(this._handle, Window, Reference);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects a transmission channel to be used as a reference channel in relative adjacent-channel power measurements.
        /// 
        /// Note:
        /// (1) The attribute is available only for multicarrier channel and adjacent-channel power measurements (CALC:MARK:FUNC:POW:SEL MCAC) in the frequency domain (span &gt; 0).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_POW_ADJ_REF_TXCHANNEL
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:REFerence:TXCHannel:MANual
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Channel">
        /// This control select the reference channel for relative adjacent channel power measurement.
        /// 
        /// Valid Values: 1 to 12
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_POW_ADJ_REF_TXCHANNEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureACPReferenceChannelMan(int Window, int Channel)
        {
            int pInvokeResult = PInvoke.ConfigureACPReferenceChannelMan(this._handle, Window, Channel);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the percentage of the power with respect to the total power. This value is the basis for the occupied bandwidth measurement (command: POWer:ACHannel:PRESet OBW).
        /// 
        /// Note:
        /// (1) The attribute is only available in the frequency domain (span &gt; 0).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_POW_BANDWIDTH
        /// RSSPECAN_ATTR_MEAS_POW_CHANNEL_BANDWIDTH
        /// RSSPECAN_ATTR_MEAS_POW_CHANNEL_BANDWIDTH_PORTABLE
        /// RSSPECAN_ATTR_MEAS_POW_BANDWIDTH_PORTABLE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:BANDwidth|BWIDth
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:BANDwidth|BWIDth[:CHANnel]
        /// CALCulate:MARKer&lt;1...6&gt;:FUNCtion:OBAN|OBW:BANDwidth
        /// CALCulate:MARKer&lt;1...6&gt;:FUNCtion:OBAN|OBW:BANDwidth:PCT
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Power_Bandwidth">
        /// Sets the percentage of the power with respect to the total power. The driver uses this value to set the RSSPECAN_ATTR_POW_BANDWIDTH and RSSPECAN_ATTR_MEAS_POW_CHANNEL_BANDWIDTH_PORTABLE attribute.
        /// 
        /// Units: %
        /// 
        /// Valid Values: 0.0 to 100
        /// 
        /// Default Value: 99
        /// </param>
        /// <param name="Channel_Bandwidth">
        /// Sets the selected channel bandwidth. The driver uses this value to set the RSSPECAN_ATTR_POW_CHANNEL_BANDWIDTH and RSSPECAN_ATTR_MEAS_POW_CHANNEL_BANDWIDTH_PORTABLE attribute.
        /// 
        /// Units: Hz
        /// 
        /// Valid Values: 100.0 to 1.0e+9 
        /// 
        /// Default Value: 14.0e+3
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureOccupiedBandwidth(int Window, double Power_Bandwidth, double Channel_Bandwidth)
        {
            int pInvokeResult = PInvoke.ConfigureOccupiedBandwidth(this._handle, Window, Power_Bandwidth, Channel_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures parameters of signal statistic measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_STAT_APD_STATE
        /// RSSPECAN_ATTR_MEAS_STAT_CCDF_STATE
        /// RSSPECAN_ATTR_MEAS_STAT_SAMPLES
        /// RSSPECAN_ATTR_RESOLUTION_BANDWIDTH
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:APD[:STATe] ON | OFF
        /// CALCulate:STATistics:CCDF[:STATe] ON | OFF
        /// CALCulate:STATistics:NSAMples
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth[:RESolution]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement">
        /// Selects the signal statistic measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SSTAT_OFF  (0) - Off
        /// RSSPECAN_VAL_SSTAT_APD  (1) - APD
        /// RSSPECAN_VAL_SSTAT_CCDF (2) - CCDF
        /// 
        /// Default Value: RSSPECAN_VAL_SSTAT_OFF (0)
        /// </param>
        /// <param name="Number_of_Samples">
        /// Defines the number of measurement points to be acquired for the statistical measurement functions.
        /// 
        /// Valid Values:
        /// 100 to 1000000000
        /// 
        /// Default Value: 100000
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_STAT_SAMPLES
        /// 
        /// Note(s):
        /// (1) This attribute has no effect for Measurement set to Off.
        /// </param>
        /// <param name="Resolution_Bandwidth">
        /// Specifies the measurement resolution bandwidth.
        /// 
        /// Valid Values:
        /// 10.0 Hz to 50.0E6 Hz
        /// 
        /// Default Value: 1.0E6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RESOLUTION_BANDWIDTH
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSignalStatisticMeasurement(int Measurement, int Number_of_Samples, double Resolution_Bandwidth)
        {
            int pInvokeResult = PInvoke.ConfigureSignalStatisticMeasurement(this._handle, Measurement, Number_of_Samples, Resolution_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures parameters of signal statistic measurement X-Axis scaling.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_STAT_X_REF
        /// RSSPECAN_ATTR_MEAS_STAT_X_RANGE
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:SCALe:X:RLEVel
        /// CALCulate:STATistics:SCALe:X:RANGe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="X_Axis_Ref_Level">
        /// Defines the reference level for the X axis of the measurement diagram. The setting is identical to the reference level setting using the command DISPlay:WINDow:TRACe:Y:RLEVel.
        /// With the reference level offset &lt;&gt; 0 the indicated value range of the reference level is modified by the offset.Selects the signal statistic measurement.
        /// 
        /// Valid Values:
        /// -130.0 to 30.0 dBm
        /// 
        /// Default Value: -20.0 dBm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_STAT_X_REF
        /// </param>
        /// <param name="X_Axis_Range">
        /// Defines the level range for the X axis of the measurement diagram. The setting is identical to the level range setting defined with the command DISPlay:WINDow:TRACe:Y:SCALe.
        /// 
        /// Valid Values:
        /// 10.0 to 200.0
        /// 
        /// Default Value: 100.0
        /// 
        /// Attribute:
        /// SSPECAN_ATTR_MEAS_STAT_X_RANGE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSignalStatisticMeasurementXAxis(double X_Axis_Ref_Level, double X_Axis_Range)
        {
            int pInvokeResult = PInvoke.ConfigureSignalStatisticMeasurementXAxis(this._handle, X_Axis_Ref_Level, X_Axis_Range);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures parameters of signal statistic measurement Y Axis scaling.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_STAT_Y_UNIT
        /// RSSPECAN_ATTR_MEAS_STAT_Y_MIN
        /// RSSPECAN_ATTR_MEAS_STAT_Y_MAX
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:SCALe:Y:UNIT PCT | ABS
        /// CALCulate:STATistics:SCALe:Y:LOWer
        /// CALCulate:STATistics:SCALe:Y:UPPer
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Y_Unit">
        /// Toggles the scaling of Y axis between percentage and absolute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ABS (0) - Absolute Values
        /// RSSPECAN_VAL_PCT (1) - Percentage
        /// 
        /// Default Value: RSSPECAN_VAL_ABS (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_STAT_Y_UNIT
        /// </param>
        /// <param name="Y_Axis_Max_Value">
        /// Defines the upper limit for the Y axis of the diagram in statitistical measurements. Since probabilities are specified on the Y axis, the entered numerical values are dimensionless. 
        /// 
        /// Valid Values:
        /// 1.0e-8 to 1.0
        /// 
        /// Default Value: 1.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_STAT_Y_MAX
        /// </param>
        /// <param name="Y_Axis_Min_Value">
        /// Defines the lower limit for the Y axis of the diagram in statistical measurements. Since probabilities are specified on the Y axis, the entered numerical values are dimensionless.
        /// 
        /// Valid Values:
        /// 1.0e-9 to 0.1
        /// 
        /// Default Value: 1.0e-6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_STAT_Y_MIN
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSignalStatisticMeasurementYAxis(int Y_Unit, double Y_Axis_Max_Value, double Y_Axis_Min_Value)
        {
            int pInvokeResult = PInvoke.ConfigureSignalStatisticMeasurementYAxis(this._handle, Y_Unit, Y_Axis_Max_Value, Y_Axis_Min_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the acquisition attributes of signal statistic.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SWEEP_MODE_CONTINUOUS
        /// 
        /// Remote-control command(s):
        /// INITiate&lt;1|2&gt;:CONTinuous ON | OFF
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Sweep_Mode_Continuous">
        /// Turns on/off continuous sweeping.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SWEEP_MODE_CONTINUOUS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSignalStatisticSweep(bool Sweep_Mode_Continuous)
        {
            int pInvokeResult = PInvoke.ConfigureSignalStatisticSweep(this._handle, System.Convert.ToUInt16(Sweep_Mode_Continuous));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the relative X position of the mean power value for the CCDF measurement. The default position is 0% (left corner of the grid). This function is only available if the statistics measurement function CCDF is switched on.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_STAT_MEAN_POWER_POS
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:STATistics:SCALe:X:MPOSition
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Mean_Power_Position">
        /// This control defines the relative X position of the mean power value for the CCDF measurement. The default position is 0% (left corner of the grid). This function is only available if the statistics measurement function CCDF is switched on. 
        /// 
        /// Valid Ranges:
        /// 0.0 to 100.0
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_STAT_MEAN_POWER_POS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSignalStatisticMeanPowerPosition(double Mean_Power_Position)
        {
            int pInvokeResult = PInvoke.ConfigureSignalStatisticMeanPowerPosition(this._handle, Mean_Power_Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function optimizes the level setting of the instrument depending on the measured peak power, in order to obtain maximum instrument sensitivity.
        /// To obtain maximum resolution, the level range is set as a function of the measured spacing between peak power and the minimum power for the APD measurement and of the spacing between peak power and mean power for the CCDF measurement. In addition, the probability scale for the number of test points is adapted.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_STAT_ADJ
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:SCALe:AUTO ONCE
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SignalStatisticMeasurementAdjustSettings(uint Timeout)
        {
            int pInvokeResult = PInvoke.SignalStatisticMeasurementAdjustSettings(this._handle, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function adjusts the reference level to get an optimized sensitivity. Only the reference level is changed all other setting parameters are kept.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_STAT_REF_LEVEL_ADJUST
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:PRESet:RLEVel
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to  4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SignalStatisticMeasurementRefLevelAdjust(uint Timeout)
        {
            int pInvokeResult = PInvoke.SignalStatisticMeasurementRefLevelAdjust(this._handle, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function resets the scaling of the X and Y axes in a statistical measurement. The following
        /// values are set:
        /// X axis ref level: -20 dBm
        /// X axis range APD: 100 dB
        /// X axis range CCDF: 20 dB
        /// Y axis upper limit: 1.0
        /// Y axis lower limit: 1E-6
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_STAT_PRESET
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:PRESet
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SignalStatisticMeasurementPresetScaling()
        {
            int pInvokeResult = PInvoke.SignalStatisticMeasurementPresetScaling(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the level values for the probabilities 0.1%, 1% and 10%.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:CCDF:X&lt;1 to 3&gt;? P0_1 | P1 | P10
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Trace">
        /// Selects the trace.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Probability">
        /// Selects the probability.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SSTAT_PROB_0_1  (0) - 0.1 %
        /// RSSPECAN_VAL_SSTAT_PROB_1    (1) - 1 %
        /// RSSPECAN_VAL_SSTAT_PROB_10   (2) - 10 %
        /// 
        /// Default Value: RSSPECAN_VAL_SSTAT_PROB_0_1 (0)
        /// </param>
        /// <param name="Result">
        /// Returns the selected level value.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QuerySignalStatisticCCDFLevel(int Trace, int Probability, out double Result)
        {
            int pInvokeResult = PInvoke.QuerySignalStatisticCCDFLevel(this._handle, Trace, Probability, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the results of statistical measurements of a recorded trace.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_STAT_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:RESult&lt;1 to 3&gt;? MEAN | PEAK | CFACtor | ALL
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Trace">
        /// Selects the trace.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 1 to 4 for R&amp;S FSL or R&amp;S ESL
        /// 1 to 6 for R&amp;S FSV
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Result_Type">
        /// Selects the signal statistic measurement result type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SSTAT_RESULT_MEAN  (0) - Mean
        /// RSSPECAN_VAL_SSTAT_RESULT_PEAK  (1) - Peak
        /// RSSPECAN_VAL_SSTAT_RESULT_CREST (2) - Crest Factor
        /// 
        /// Default Value: RSSPECAN_VAL_SSTAT_RESULT_MEAN (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Mean - Average (=RMS) power in dBm measured during the measurement time.
        /// 
        /// (2) Peak - Peak power in dBm measured during the measurement time.
        /// 
        /// (3) Crest Factor - Determined CREST factor (= ratio of peak power to average power) in dB.
        /// </param>
        /// <param name="Result">
        /// Returns the selected measurement.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QuerySignalStatisticResult(int Trace, int Result_Type, out double Result)
        {
            int pInvokeResult = PInvoke.QuerySignalStatisticResult(this._handle, Trace, Result_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on the measurement of the AM modulation depth. An AM-modulated carrier is required on the screen for correct operation. If necessary, marker 1 is previously activated and set to the largest signal available.
        /// The level value of marker 1 is regarded as the carrier level. On activating the function, marker 2 and marker 3 are automatically set as delta markers symetrically to the carrier to the adjacent maxima of the trace.
        /// If the position of delta marker 2 is changed, delta marker 3 is moved symetrically with respect to the reference marker (marker 1). If the position of delta marker 3 is changed, fine adjustment can be performed independently of delta marker 2.
        /// Instrument calculates the power at the marker positions from the measured levels.
        /// The AM modulation depth is calculated from the ratio of power values at the reference marker and the delta markers. If the two AM sidebands differ in power, the average value of the two power values is used for calculating the AM modulation depth.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_MDEPTH_STATE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:MDEPth:[:STATe]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// 
        /// Note(s):
        /// (1) This control is not valid for R&amp;S FSH4, FSH8 instruments.
        /// </param>
        /// <param name="State">
        /// Switches on the measurement of the AM modulation depth.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_MDEPTH_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureModDepthMeasurement(int Window, bool State)
        {
            int pInvokeResult = PInvoke.ConfigureModDepthMeasurement(this._handle, Window, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the AM modulation depth in the indicated measurement window.
        /// A complete sweep with synchronization to sweep end must be performed between switching on the function and querying the measured value to obtain a valid query result. This is only possible in single sweep mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_MDEPTH_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:MDEPth:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// 
        /// Note(s):
        /// (1) This control is not valid for R&amp;S FSH4, FSH8 instruments.
        /// </param>
        /// <param name="Modulation_Depth">
        /// Returns the AM modulation depth in the indicated measurement window.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryModulationDepth(int Window, out double Modulation_Depth)
        {
            int pInvokeResult = PInvoke.QueryModulationDepth(this._handle, Window, out Modulation_Depth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts the search of the signals required for the modulation depth measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_MDEPTH_SEARCH
        ///  
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:MDEPth:SEARchsignal ONCE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 1..16 for R&amp;S FSV
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ModulationDepthSignalSearch(int Window, int Marker)
        {
            int pInvokeResult = PInvoke.ModulationDepthSignalSearch(this._handle, Window, Marker);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function initiates the measurement of the third-order intercept point.
        /// A two-tone signal with equal carrier levels is expected at the RF input of the instrument. Marker 1 and marker 2 (both normal markers) are set to the maximum of the two signals. Delta marker 3 and delta marker 4 are positioned to the intermodulation products. The delta markers can be modified sperately afterwards.
        /// The third-order intercept is calculated from the level spacing between the normal markers and the delta markers.
        /// 
        /// Note: This attribute is not available during GSM measurements modulation accuracy (MAC), phase-frequency error (PFE), power vs. time (PVT) and carrier power (CPW).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_TOI_STATE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:TOI[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// Initiates the measurement of the third-order intercept point.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_TOI_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTOIMeasurement(int Window, bool State)
        {
            int pInvokeResult = PInvoke.ConfigureTOIMeasurement(this._handle, Window, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the third-order intercept point measurement in the indicated measurement window.
        /// 
        /// A complete sweep with synchronization to sweep end must be performed between switching on the function and querying the measured value to obtain a valid query result. This is only possible in single sweep mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_TOI_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:TOI:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="TOI_Measurement">
        /// Returns the third-order intercept point measurement in the indicated measurement window.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryTOI(int Window, out double TOI_Measurement)
        {
            int pInvokeResult = PInvoke.QueryTOI(this._handle, Window, out TOI_Measurement);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts the search of the signals required for the third order intercept measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_TOI_SEARCH
        ///  
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:TOI:SEARchsignal ONCE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 1..16 for R&amp;S FSV
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int TOISignalSearch(int Window, int Marker)
        {
            int pInvokeResult = PInvoke.TOISignalSearch(this._handle, Window, Marker);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function  controls the positioning of the intercept markers 3 and 4 for TOI measurement, based on the two carrier signals.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_TOI_POSITIONING
        ///  
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:TOI:MARKer CALCulate | SEARch
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Intercept_Markers_Positioning">
        /// Controls the positioning of the intercept markers 3 and 4 for TOI measurement, based on the two carrier signals.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TOI_POS_CALC (0) - Calculate
        /// RSSPECAN_VAL_TOI_POS_SEAR (1) - Search
        /// 
        /// Default Value: RSSPECAN_VAL_TOI_POS_CALC (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_TOI_POSITIONING
        /// 
        /// Note(s):
        /// 
        /// (1) Calculate: Calculates the intermodulation product frequencies.
        /// 
        /// (2) Search: Performs a local peak search near the expected intermodulation product frequencies.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTOIPositioning(int Window, int Intercept_Markers_Positioning)
        {
            int pInvokeResult = PInvoke.ConfigureTOIPositioning(this._handle, Window, Intercept_Markers_Positioning);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures harmonic distortion measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_HDIST_STATE
        /// RSSPECAN_ATTR_MEAS_HDIST_NOOFHARM
        /// RSSPECAN_ATTR_MEAS_HDIST_RBWAUTO
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:HARMonics[:STATe] ON | OFF
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:HARMonics:NHARmonics
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:HARMonics:BANDwidth:AUTO ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="State">
        /// Switches on or off the measurement of the harmonics of a carrier signal. The carrier signal is the first harmonic. The function is independent of the marker selection. It is only available in the frequency domain (span &gt; 0).
        /// If the measurement is started in the frequency domain (span &gt; 0), the last span will define the search range for the first harmonic. The level for the first harmonic will also be calculated in the frequency domain. However, the measurement can also be started in the time domain (span = 0), in which case the center frequency and the level used will remain unchanged. The driver uses this value to set the RSSPECAN_ATTR_MEAS_HDIST_STATE
        ///  attribute.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_HDIST_STATE
        /// </param>
        /// <param name="No_of_Harmonics">
        /// Defines the number of harmonics of a carrier signal to be measured.
        /// 
        /// Valid Values:
        /// 1 to 26
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_HDIST_NOOFHARM
        /// </param>
        /// <param name="Harmonic_RBW_Auto">
        /// Specifies whether the resolution bandwidth of the 2nd to the nth harmonic should be identical to the bandwidth of the first harmonic (OFF) or to the next largest bandwidth (corresponding to the harmonic) of the multiple of the bandwidth of the first harmonic (ON).
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_HDIST_RBWAUTO
        /// 
        /// Note(s):
        /// (1) This control is not valid for R&amp;S FSH4, FSH8 instruments.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureHDistMeasurement(bool State, int No_of_Harmonics, bool Harmonic_RBW_Auto)
        {
            int pInvokeResult = PInvoke.ConfigureHDistMeasurement(this._handle, System.Convert.ToUInt16(State), No_of_Harmonics, System.Convert.ToUInt16(Harmonic_RBW_Auto));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function optimizes the device settings depending on the mode in which the harmonic measurement was started:
        /// If the harmonic measurement was started in the frequency domain (span &gt; 0), the frequency and the
        /// level of the first harmonic are calculated, from which the measurement list is set up.
        /// If the measurement was started in the time domain (span = 0), the frequency of the first harmonic is
        /// not changed. The level, however, is not calculated.
        /// The function is independent of the marker selection.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_HDIST_PRESET
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:HARMonics:PRESet
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295 
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int AdjustHDistSettings(uint Timeout)
        {
            int pInvokeResult = PInvoke.AdjustHDistSettings(this._handle, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the results of the total harmonic distortion (THD).
        /// 
        /// A complete sweep with synchronization to sweep end must be performed between switching on the function and querying the measured value to obtain a valid query result. This is only possible in single sweep mode.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:HARMonics:DISTortion? TOTal
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Harmonic_Distortion_in_percent">
        /// Returns the results of the total harmonic distortion (THD) in %.
        /// </param>
        /// <param name="Harmonic_Distortion_in_dB">
        /// Returns the results of the total harmonic distortion (THD) in dB.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryHDist(out double Harmonic_Distortion_in_percent, out double Harmonic_Distortion_in_dB)
        {
            int pInvokeResult = PInvoke.QueryHDist(this._handle, out Harmonic_Distortion_in_percent, out Harmonic_Distortion_in_dB);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the list of harmonics. The first value is the absolute power of the first harmonic in the unit set via UNIT. The other values are relative to the carrier signal and are output in dB.
        /// 
        /// A complete sweep with synchronization to sweep end must be performed between switching on the function and querying the measured value to obtain a valid query result. This is only possible in single sweep mode.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:HARMonics:LIST?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Harmonics_List">
        /// Returns the list of harmonics. The first value is the absolute power of the first harmonic in the unit set via UNIT. The other values are relative to the carrier signal and are output in dB.
        /// 
        /// Number of values corresponds to that of the harmonics to be measured, which are defined with CALC:MARK:FUNC:HARM:NHARM (attribute RSSPECAN_ATTR_MEAS_HDIST_NOOFHARM).
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryHDistHarmonicsList(double[] Harmonics_List)
        {
            int pInvokeResult = PInvoke.QueryHDistHarmonicsList(this._handle, Harmonics_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the resolution bandwidth values used by the harmonic measurment. A separate value for every harmonic frequency is automatically calculated. It is possible to query the calculated values with this function.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:HARMonics:BANDwidth[:LIST]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// This parameter determines the size of array result values. The Result Values array must contain at least this number of items.
        /// 
        /// </param>
        /// <param name="Result_Values">
        /// Returns the result values.
        /// 
        /// </param>
        /// <param name="Returned_Values">
        /// Returns the number of returned values.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryHDistBandwidthList(int Array_Size, double[] Result_Values, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.QueryHDistBandwidthList(this._handle, Array_Size, Result_Values, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the instrument setup for multiple burst power measurement and starts a measurement sequence. To reduce the setting time, the setup is performed simultaneously for all selected parameters.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]MPOWer:SEQuence
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Analyzer_Frequency__Hz_">
        /// Receive frequency for the burst signals to be measured (= center frequency in manual operation).
        /// 
        /// Valid Values: 0.0 Hz to max. analyzer frequency, depending on 
        ///              instrument model
        /// 
        /// Default Value: 0.0 Hz
        /// 
        /// </param>
        /// <param name="Resolution_Bandwidth__Hz_">
        /// Resolution bandwidth for the measurement.
        /// 
        /// Valid Values: 10.0 Hz to 10.0e6 Hz in steps of 1, 3, 10
        /// 
        /// Default Value: 10.0e6 Hz
        /// 
        /// </param>
        /// <param name="Meas_Time__s_">
        /// Time span during which measurement samples are sampled for RMS / peak measurement The type of measurement is selected by Type of Meas.
        /// 
        /// Valid Values: 1.0e-6 s to 30.0 s
        /// 
        /// Default Value: 1.0e-6
        /// 
        /// </param>
        /// <param name="Trigger_Source">
        /// Trigger signal source.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TRG_EXT (1) - External (Default Value)
        /// RSSPECAN_VAL_TRG_VID (3) - Video
        /// 
        /// Default Value: RSSPECAN_VAL_TRG_EXT (1)
        /// 
        /// Notes:
        /// 
        /// (1) External:
        /// The trigger signal is fed from the "Ext. Trigger/Gate" input on the rear of the unit.
        /// 
        /// (2) Video:
        /// The internal video signal is used as trigger signal.
        /// 
        /// </param>
        /// <param name="Trigger_Level____">
        /// Signal level of the video signal as a percentage of the diagram
        /// height at which the trigger becomes active. If Trigger Source =
        /// External is selected, the value entered here is ignored, as in this case the trigger input uses TTL levels.
        /// 
        /// Valid Values: 0.0 % to 100.0 %
        /// 
        /// Default Value: 50.0 %
        /// 
        /// </param>
        /// <param name="Trigger_Offset__s_">
        /// Offset between the detection of the trigger signal and the start of the measurement.
        /// 
        /// Valid Values: 125.0e-9 s to 100.0 s
        /// 
        /// Default Value: 125.0e-6 s
        /// 
        /// </param>
        /// <param name="Type_of_Meas">
        /// Determines whether mean power (RMS) or peak power (PEAK) is to be measured. The FSQ detector is selected accordingly.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_BURS_MEAS_MEAN (0) - Mean
        /// RSSPECAN_VAL_BURS_MEAS_PEAK (1) - Peak
        /// 
        /// Default Value: RSSPECAN_VAL_BURS_MEAS_MEAN (0)
        /// 
        /// </param>
        /// <param name="No_of_Measurements">
        /// Number of individual bursts to be measured.
        /// 
        /// Valid Values: 1 to 501
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int BurstPowerSequence(int Window, double Analyzer_Frequency__Hz_, double Resolution_Bandwidth__Hz_, double Meas_Time__s_, int Trigger_Source, double Trigger_Level____, double Trigger_Offset__s_, int Type_of_Meas, int No_of_Measurements)
        {
            int pInvokeResult = PInvoke.BurstPowerSequence(this._handle, Window, Analyzer_Frequency__Hz_, Resolution_Bandwidth__Hz_, Meas_Time__s_, Trigger_Source, Trigger_Level____, Trigger_Offset__s_, Type_of_Meas, No_of_Measurements);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function the filter type for the measurement.
        /// 
        /// Note: 
        /// 
        /// (1) This function is available only with R&amp;S FSL.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MPOW_FTYPE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]MPOWer:FTYPe NORMal | CFILter | RRC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Filter_Type">
        /// This control defines the filter type for the measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MPOW_FTYPE_NORM (0) - Normal (Default Value)
        /// RSSPECAN_VAL_MPOW_FTYPE_CFIL (1) - CFilter
        /// RSSPECAN_VAL_MPOW_FTYPE_RRC  (2) - RRC
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MPOW_FTYPE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int BurstPowerFilterType(int Window, int Filter_Type)
        {
            int pInvokeResult = PInvoke.BurstPowerFilterType(this._handle, Window, Filter_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the instrument setup for multiple burst power measurement, starts a measurement sequence and reads the results. To reduce the setting time, the setup is performed simultaneously for all selected parameters.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]MPOWer:SEQuence?
        /// *CLS
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Analyzer_Frequency__Hz_">
        /// Receive frequency for the burst signals to be measured (= center frequency in manual operation).
        /// 
        /// Valid Values: 0.0 Hz to max. analyzer frequency, depending on 
        ///              instrument model
        /// 
        /// Default Value: 0.0 Hz
        /// 
        /// </param>
        /// <param name="Resolution_Bandwidth__Hz_">
        /// Resolution bandwidth for the measurement.
        /// 
        /// Valid Values: 10.0 Hz to 10.0e6 Hz in steps of 1, 3, 10
        /// 
        /// Default Value: 10.0e6 Hz
        /// 
        /// </param>
        /// <param name="Meas_Time__s_">
        /// Time span during which measurement samples are sampled for RMS / peak measurement The type of measurement is selected by Type of Meas.
        /// 
        /// Valid Values: 1.0e-6 s to 30.0 s
        /// 
        /// Default Value: 1.0e-6
        /// 
        /// </param>
        /// <param name="Trigger_Source">
        /// Trigger signal source.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TRG_EXT (1) - External (Default Value)
        /// RSSPECAN_VAL_TRG_VID (3) - Video
        /// 
        /// Notes:
        /// 
        /// (1) External:
        /// The trigger signal is fed from the "Ext. Trigger/Gate" input on the rear of the unit.
        /// 
        /// (2) Video:
        /// The internal video signal is used as trigger signal.
        /// 
        /// </param>
        /// <param name="Trigger_Level____">
        /// Signal level of the video signal as a percentage of the diagram
        /// height at which the trigger becomes active. If Trigger Source =
        /// External is selected, the value entered here is ignored, as in this case the trigger input uses TTL levels.
        /// 
        /// Valid Values: 0.0 % to 100.0 %
        /// 
        /// Default Value: 50.0 %
        /// 
        /// </param>
        /// <param name="Trigger_Offset__s_">
        /// Offset between the detection of the trigger signal and the start of the measurement.
        /// 
        /// Valid Values: 125.0e-9 s to 100.0 s
        /// 
        /// Default Value: 125.0e-6 s
        /// 
        /// </param>
        /// <param name="Type_of_Meas">
        /// Determines whether mean power (RMS) or peak power (PEAK) is to be measured. The FSQ detector is selected accordingly.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_BURS_MEAS_MEAN (0) - Mean
        /// RSSPECAN_VAL_BURS_MEAS_PEAK (1) - Peak
        /// 
        /// Default Value: RSSPECAN_VAL_BURS_MEAS_MEAN (0)
        /// 
        /// </param>
        /// <param name="No_of_Measurements">
        /// Number of individual bursts to be measured.
        /// 
        /// Valid Values: 1 to 501
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value for data readout in milliseconds.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 10000 ms
        /// 
        /// Note:
        /// 
        /// This parameter is not used.
        /// 
        /// </param>
        /// <param name="Burst_Power_Results">
        /// This control returns array of elements as the result of a previous list measurement as configured and initiated with List Power Sequence (confListPwrSeq).
        /// 
        /// Note:
        /// 
        /// Array size must be =&gt; then number of elements sent from the instrument.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetBurstPowerSequence(int Window, double Analyzer_Frequency__Hz_, double Resolution_Bandwidth__Hz_, double Meas_Time__s_, int Trigger_Source, double Trigger_Level____, double Trigger_Offset__s_, int Type_of_Meas, int No_of_Measurements, uint Timeout, double[] Burst_Power_Results)
        {
            int pInvokeResult = PInvoke.GetBurstPowerSequence(this._handle, Window, Analyzer_Frequency__Hz_, Resolution_Bandwidth__Hz_, Meas_Time__s_, Trigger_Source, Trigger_Level____, Trigger_Offset__s_, Type_of_Meas, No_of_Measurements, Timeout, Burst_Power_Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the results of a multiple burst power measurement as configured and initiated with Get Burst Power Result (actBurstPwrResult). The unit used for the return values is always dBm. This function may be used to obtain measurement results in an asynchronous way using the service request mechanism for synchronization with the end of the measurement. If no measurement results are available, the function will return a query error.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]MPOWer:RESult[:LIST]?
        /// *CLS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="No_of_Results">
        /// Expected number of values to be returned from the instrument. This value must be equal to the Burst Power results array number of items.
        /// 
        /// Valid Values: &gt; 0
        /// 
        /// Default Value: 256
        /// 
        /// Note:
        /// 
        /// This function will read data from the instrument till instruments buffer is empty or number of results is reached.
        /// 
        /// </param>
        /// <param name="Burst_Power_Results">
        /// This control returns array of elements as the results of a multiple burst power measurement as configured and initiated with Get Burst Power Result (actBurstPwrResult).
        /// 
        /// Note:
        /// 
        /// Array size must be =&gt; then number of elements sent from the instrument.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetBurstPowerResults(int Window, int No_of_Results, double[] Burst_Power_Results)
        {
            int pInvokeResult = PInvoke.GetBurstPowerResults(this._handle, Window, No_of_Results, Burst_Power_Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the minimum power value in a multiple burst power measurement as configured and initiated with Burst Power Sequence (confBurstPwrSeq). The unit used for the return values is always dBm. If no measurement result is available, the function will return a query error.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MPOW_MIN
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]MPOWer:RESult:MIN?
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Power_Result_Min">
        /// This control queries the minimum power value in a multiple burst power measurement as configured and initiated with Burst Power Sequence.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MPOW_MIN
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetBurstPwrResultMin(int Window, out double Power_Result_Min)
        {
            int pInvokeResult = PInvoke.GetBurstPwrResultMin(this._handle, Window, out Power_Result_Min);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the spurious emissions measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SWEEP_MODE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:MODE AUTO | LIST
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="State">
        /// Switches on or off the spurious emissions measurement.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SWEEP_MODE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSEState(bool State)
        {
            int pInvokeResult = PInvoke.ConfigureSEState(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the peak search for spurious emission measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SE_PEAK_SEARCH_MARGIN
        /// RSSPECAN_ATTR_SE_PEAK_SEARCH_PEAKS
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PEAKsearch:MARGin &lt;value&gt;
        /// CALCulate&lt;1|2&gt;:PEAKsearch:SUBRanges &lt;value&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Margin">
        /// Defines an overall margin which is subtracted from the limit line to make the peak search.
        /// 
        /// Units: dB
        /// 
        /// Valid Values: -200.0 to 200.0
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SE_PEAK_SEARCH_MARGIN
        /// </param>
        /// <param name="Peaks_Per_Range">
        /// Defines how many peaks are searched for within one range.
        /// 
        /// Valid Values: 1 to 50
        /// 
        /// Default Value: 25
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PEAK_SEARCH_SUBRANGE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSEPeakSearch(int Window, double Margin, int Peaks_Per_Range)
        {
            int pInvokeResult = PInvoke.ConfigureSEPeakSearch(this._handle, Window, Margin, Peaks_Per_Range);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the start and stop frequency for the specified range.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SE_LIST_RANG_START
        /// RSSPECAN_ATTR_SE_LIST_RANG_STOP
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1 to 20&gt;[:FREQuency]:STARt
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1 to 20&gt;[:FREQuency]:STOP
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Start_Frequency">
        /// Sets the start frequency of a range in the spurious measurement. 
        /// 
        /// Units: Hz
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 20.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SE_LIST_RANG_START
        /// </param>
        /// <param name="Stop_Frequency">
        /// Sets the stop frequency of a range in the spurious measurement.
        /// 
        /// Units: Hz
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 1.0e+9
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SE_LIST_RANG_STOP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSESweepListStartStop(int Range, double Start_Frequency, double Stop_Frequency)
        {
            int pInvokeResult = PInvoke.ConfigureSESweepListStartStop(this._handle, Range, Start_Frequency, Stop_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the filter type for the specified range.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SE_LIST_RANG_FILT_TYPE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1 to 20&gt;:FILTer:TYPE NORMal | CHANnel | RRC | P5 | PULSe | NOISe | P5Digital
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Filter">
        /// This control sets the filter in the spurious measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIST_FTYPE_NORM (0) - Normal/Gaussian
        /// RSSPECAN_VAL_LIST_FTYPE_CHAN (1) - Channel
        /// RSSPECAN_VAL_LIST_FTYPE_RRC  (2) - RRC
        /// RSSPECAN_VAL_LIST_FTYPE_P5   (3) - 5-Pole
        /// RSSPECAN_VAL_LIST_FTYPE_PULS (4) - EMI (6 dB)
        /// RSSPECAN_VAL_LIST_FTYPE_NOIS (5) - Gaussian filters
        /// RSSPECAN_VAL_LIST_FTYPE_P5D  (6) - 5-Pole Digital
        /// 
        /// Default Value: RSSPECAN_VAL_LIST_FTYPE_NORM (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SE_LIST_RANG_FILT_TYPE
        /// 
        /// Note(s):
        /// 
        /// (1) 5-Pole: This is not available for R&amp;S FSL and R&amp;S ESL instruments.
        /// 
        /// (2) EMI (6dB): This is only available for R&amp;S FSL and R&amp;S ESL and R&amp;S FSUP instruments.
        /// 
        /// (3) Guassian filters: This is not available for R&amp;S FSV instrument.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSESweepListFilterType(int Range, int Filter)
        {
            int pInvokeResult = PInvoke.ConfigureSESweepListFilterType(this._handle, Range, Filter);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the resolution and video bandwidth values for the specified range.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SE_LIST_RANG_BAND
        /// RSSPECAN_ATTR_SE_LIST_RANG_BAND_VID
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1 to 20&gt;:BANDwidth[:RESolution]
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1 to 20&gt;:BANDwidth:VIDeo
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Resolution_Bandwidth">
        /// Sets the the resolution bandwidth.
        /// 
        /// Valid Values: 10.0 Hz to 50.0e6 Hz
        /// 
        /// Default Value: 10.0e3 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SE_LIST_RANG_BAND
        /// </param>
        /// <param name="Video_Bandwidth">
        /// Sets the the video bandwidth.
        /// 
        /// Valid Values: 1.0 Hz to 10.0e6 Hz
        /// 
        /// Default Value: 30.0e3 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SE_LIST_RANG_BAND_VID
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSESweepListBandwidth(int Range, double Resolution_Bandwidth, double Video_Bandwidth)
        {
            int pInvokeResult = PInvoke.ConfigureSESweepListBandwidth(this._handle, Range, Resolution_Bandwidth, Video_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the sweep time values for the specified range.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SE_LIST_RANG_SWE_TIME_AUTO
        /// RSSPECAN_ATTR_SE_LIST_RANG_SWE_TIME
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1 to 20&gt;:SWEep:TIME:AUTO ON | OFF
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1 to 20&gt;:SWEep:TIME
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Sweep_Time_Auto">
        /// Sets the automatic coupling of the sweep duration on the frequency span and the bandwidth settings in a spurious measurement range.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SE_LIST_RANG_SWE_TIME_AUTO
        /// </param>
        /// <param name="Sweep_Time">
        /// Sets the sweep duration of a range in the spurious measurement.
        /// 
        /// Valid Values: 0.001 s to 16000.0 s
        ///  
        /// Default Value: 0.1 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SE_LIST_RANG_SWE_TIME
        /// 
        /// Note:
        /// (1) This control has effect only if Sweep Time Auto is set Off
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSESweepSweepListTime(int Range, bool Sweep_Time_Auto, double Sweep_Time)
        {
            int pInvokeResult = PInvoke.ConfigureSESweepSweepListTime(this._handle, Range, System.Convert.ToUInt16(Sweep_Time_Auto), Sweep_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the detector value for the specified range.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SE_LIST_RANG_DET
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1 to 20&gt;:DETector APEak | NEGative | POSitive | SAMPle | RMS | AVERage
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Detector">
        /// This control sets the detector in the spurious measurement. The driver sets the RSSPECAN_ATTR_SE_LIST_RANG_DET to this value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIST_RANG_DET_APE  (0) - Auto Peak
        /// RSSPECAN_VAL_LIST_RANG_DET_NEG  (1) - Negative
        /// RSSPECAN_VAL_LIST_RANG_DET_POS  (2) - Positive
        /// RSSPECAN_VAL_LIST_RANG_DET_SAMP (3) - Sample
        /// RSSPECAN_VAL_LIST_RANG_DET_RMS  (4) - RMS
        /// RSSPECAN_VAL_LIST_RANG_DET_AVER (5) - Average
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_LIST_RANG_DET_APE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SE_LIST_RANG_DET
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSESweepListDetector(int Range, int Detector)
        {
            int pInvokeResult = PInvoke.ConfigureSESweepListDetector(this._handle, Range, Detector);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the reference level for the specified range.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SE_LIST_RANG_RLEV
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1 to 20&gt;:RLEVel
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Ref_Level_Value">
        /// Specifies the reference level of a range in the spurious measurement.
        /// 
        /// Valid Values: -130.0 dBm to 30.0 dBm
        /// 
        /// Default Value: -30.0 dBm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SE_LIST_RANG_RLEV
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSESweepListReferenceLevel(int Range, double Ref_Level_Value)
        {
            int pInvokeResult = PInvoke.ConfigureSESweepListReferenceLevel(this._handle, Range, Ref_Level_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the RF attenuation settings for the specified range.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SE_LIST_RANG_INP_ATT_AUTO
        /// RSSPECAN_ATTR_SE_LIST_RANG_INP_ATT
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1 to 20&gt;:INPut:ATTenuation:AUTO ON | OFF
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1 to 20&gt;:INPut:ATTenuation
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Attenuator_Auto">
        /// Sets the autoranging of a range in the spurious measurement on or off.. The driver sets the RSSPECAN_ATTR_SE_LIST_RANG_INP_ATT_AUTO to this value.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SE_LIST_RANG_INP_ATT_AUTO
        /// </param>
        /// <param name="Manual_Attenuator_Value__dB_">
        /// This control sets the RF attenuation of a range.
        /// 
        /// Valid Values: 0.0 dB to 75.0 dB
        /// 
        /// Default Value: 15.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SE_LIST_RANG_INP_ATT
        /// 
        /// Note:
        /// (1) This control has effect only if Autoranging control is set Off
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSESweepListRFAttenuation(int Range, bool Attenuator_Auto, double Manual_Attenuator_Value__dB_)
        {
            int pInvokeResult = PInvoke.ConfigureSESweepListRFAttenuation(this._handle, Range, System.Convert.ToUInt16(Attenuator_Auto), Manual_Attenuator_Value__dB_);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures state of pre-amplifier for selected sweep range.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only if options FS-B23 or FS-B25 are installed.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SE_LIST_RANG_INP_GAIN_STAT
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1 to 20&gt;:INPut:GAIN:STATe ON | OFF
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Pre_amplifier">
        /// Switches the preamplifier of a range on and off. The driver sets the RSSPECAN_ATTR_LIST_RANG_INP_GAIN_STAT to this value.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SE_LIST_RANG_INP_GAIN_STAT
        /// 
        /// Note:
        /// (1) On value is available only with option B23 or B25
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSESweepListPreamplifier(int Range, bool Pre_amplifier)
        {
            int pInvokeResult = PInvoke.ConfigureSESweepListPreamplifier(this._handle, Range, System.Convert.ToUInt16(Pre_amplifier));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the sweep settings for the specified range.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SE_LIST_RANG_POINTS
        /// RSSPECAN_ATTR_SE_LIST_RANG_BREAK
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1 to 20&gt;:POINts
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1 to 20&gt;:BREAK ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Sweep_Points">
        /// Sets the number of sweep points of a range in the spurious measurement. The driver sets the RSSPECAN_ATTR_SE_LIST_RANG_POINTS to this value.
        /// 
        /// Valid Values: 101, 155, 313, 625, 1251, 1999, 2501, 5001, 10001, 20001
        /// 
        /// Default Value: 625
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SE_LIST_RANG_POINTS
        /// </param>
        /// <param name="Stop_After_Sweep">
        /// Determines whether a sweep is stopped in the spurious measurement when a range is changed. The driver sets the    RSSPECAN_ATTR_SE_LIST_RANG_BREAK to this value.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SE_LIST_RANG_BREAK
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSESweepListSweep(int Range, int Sweep_Points, bool Stop_After_Sweep)
        {
            int pInvokeResult = PInvoke.ConfigureSESweepListSweep(this._handle, Range, Sweep_Points, System.Convert.ToUInt16(Stop_After_Sweep));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures a transducer for the specified range.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SE_LIST_RANG_TRAN
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1 to 20&gt;:TRANsducer
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Transducer_Factor">
        /// Sets the transducer factor for a range in the spurious measurement.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SE_LIST_RANG_TRAN
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSESweepListTransducerFactor(int Range, string Transducer_Factor)
        {
            int pInvokeResult = PInvoke.ConfigureSESweepListTransducerFactor(this._handle, Range, Transducer_Factor);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the limits for the specified range.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SE_LIST_RANG_LIMIT_STATE
        /// RSSPECAN_ATTR_SE_LIST_RANG_LIMIT
        /// RSSPECAN_ATTR_SE_LIST_RANG_LIMIT_START
        /// RSSPECAN_ATTR_SE_LIST_RANG_LIMIT_STOP
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:LIMit:STATe
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:LIMit
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:LIMit:STARt
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:LIMit:STOP
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Limit_Checking">
        /// Activates/deactivates limit checking of spurious emission measurement LIST EVALUATION.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SE_LIST_RANG_LIMIT_STATE
        /// </param>
        /// <param name="Limit">
        /// This control defines the limit of spurious emission measurement LIST EVALUATION.
        /// 
        /// Valid Values: -200.0 to 200.0
        /// 
        /// Default Value: -13.0 dBm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SE_LIST_RANG_LIMIT
        /// 
        /// Note:
        /// (1) This control has effect only if Limit Checking is set On.
        /// 
        /// (2) This control is ignored on R&amp;S FSL or R&amp;S ESL or R&amp;S FSV instruments.
        /// 
        /// </param>
        /// <param name="Absolute_Limit_Start">
        /// This control sets an absolute limit value at the start frequency of the specified range. Different from manual operation, this setting is independently of the defined limit check type. The driver uses this value to set the RSSPECAN_ATTR_SE_LIST_RANG_LIMIT_START attribute.
        /// 
        /// Valid Values:
        /// -400.0 to 400.0 dBm
        /// 
        /// Default Value: -13.0 dBm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SE_LIST_RANG_LIMIT_START
        /// 
        /// Note(s):
        /// 
        /// (1) This control has effect only if Limit Checking is set On.
        /// 
        /// (2) This control is only available on R&amp;S FSL, R&amp;S ESL, R&amp;S FSV, R&amp;S FSP, R&amp;S FSQ, R&amp;S FSU instruments.
        /// </param>
        /// <param name="Absolute_Limit_Stop">
        /// This control sets an absolute limit value at the stop frequency of the specified range. Different from manual operation, this setting is independently of the defined limit check type.
        /// 
        /// Valid Values:
        /// -400.0 to 400.0 dBm
        /// 
        /// Default Value: -13.0 dBm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SE_LIST_RANG_LIMIT_STOP
        /// 
        /// Note(s):
        /// 
        /// (1) This control has effect only if Limit Checking is set On.
        /// 
        /// (2) This control is only available on R&amp;S FSL, R&amp;S ESL, R&amp;S FSV, R&amp;S FSP, R&amp;S FSQ, R&amp;S FSU instruments.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSESweepListLimits(int Range, bool Limit_Checking, double Limit, double Absolute_Limit_Start, double Absolute_Limit_Stop)
        {
            int pInvokeResult = PInvoke.ConfigureSESweepListLimits(this._handle, Range, System.Convert.ToUInt16(Limit_Checking), Limit, Absolute_Limit_Start, Absolute_Limit_Stop);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function deletes a range.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SE_LIST_RANG_DEL
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1 to 20&gt;:DELete
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Range">
        /// This control specifies the range that will be deleted.
        /// 
        /// Valid Values: 1 to 20
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SEDeleteRange(int Range)
        {
            int pInvokeResult = PInvoke.SEDeleteRange(this._handle, Range);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the number of defined ranges of the spurious measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SE_LIST_RANG_COUNT
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1 to 20&gt;:COUNt?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Number_Of_Ranges">
        /// Returns indicates the number of defined ranges of the spurious measurement. The driver sets the RSSPECAN_ATTR_LIST_RANG_COUNT to this value.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LIST_RANG_COUNT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetSENumberOfListRanges(out int Number_Of_Ranges)
        {
            int pInvokeResult = PInvoke.GetSENumberOfListRanges(this._handle, out Number_Of_Ranges);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function searches the peaks in spurious emissions data.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SE_PEAK_SEARCH
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PEAKsearch
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SESearchPeak()
        {
            int pInvokeResult = PInvoke.SESearchPeak(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function stores the peak list of selected trace in ASCII format to a file.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SE_STORE_FILE
        /// 
        /// Remote-control command(s):
        /// MMEMory:STORe:SPURious &lt;file_name&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="File_Name">
        /// Specifies full path of the file, where spurios emissions data will be stored.
        /// 
        /// Valid Values:
        /// any valid file name
        /// 
        /// Default Value: "a:\\test.txt"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SE_STORE_FILE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int StoreSEToFile(string File_Name)
        {
            int pInvokeResult = PInvoke.StoreSEToFile(this._handle, File_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the spurious emissions peak list.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;[:DATA] SPUR
        /// FORMat[:DATA] REAL,32
        /// *CLS
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="No_Of_Values">
        /// This control sets the number of values returned in arrays.
        /// The array size has to be at least equal to this number.
        /// 
        /// Valid Values: &gt; 0
        /// 
        /// Default Value: 10
        /// 
        /// </param>
        /// <param name="Frequency">
        /// Returns an array of frequency list in Hz.
        /// 
        /// </param>
        /// <param name="Level">
        /// Returns an array of level list in dBm.
        /// 
        /// </param>
        /// <param name="Delta_Limit">
        /// Returns an array of delta to limit line values in dB. A delta limit of +200dB indicates no limit check is active.
        /// 
        /// </param>
        /// <param name="Returned_Values">
        /// Returns number of value sets (frequency, level and delta to limit line) read (available to return) from the instrument. Number of values returned in arrays is limited with the parameter No of Values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadSEPeakList(int No_Of_Values, double[] Frequency, double[] Level, double[] Delta_Limit, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.ReadSEPeakList(this._handle, No_Of_Values, Frequency, Level, Delta_Limit, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates or deactivates spurious emission measurement LIST EVALUATION.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SE_LIST_EVAL_STATE
        /// 
        /// Remote-control command(s):
        /// CALCulate1:PEAKsearch:AUTO
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="State">
        /// Switches on or off the spurious emissions measurement.
        /// 
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SE_LIST_EVAL_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SEListEvaluation(bool State)
        {
            int pInvokeResult = PInvoke.SEListEvaluation(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the resulting peak list of the spurious emission measurement LIST EVALUATION
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:DATA? LIST
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="No_Of_Values">
        /// This control sets the number of values returned in arrays.
        /// The array size has to be at least equal to this number.
        /// 
        /// Valid Values: &gt; 0
        /// 
        /// Default Value: 10
        /// 
        /// </param>
        /// <param name="Range_Number">
        /// Returns an array of range numbers
        /// 
        /// 
        /// </param>
        /// <param name="Start_Frequency">
        /// Returns an array of range start frequency
        /// 
        /// </param>
        /// <param name="Stop_Frequency">
        /// Returns an array of range stop frequency.
        /// </param>
        /// <param name="Resolution_Bandwidth">
        /// Returns an array of resolution bandwidth
        /// </param>
        /// <param name="Frequency">
        /// Returns an array of rrequency of the peak in the range
        /// 
        /// 
        /// </param>
        /// <param name="Level_Abs">
        /// Returns an array of absolute peak power of the range in dBm
        /// 
        /// </param>
        /// <param name="Level_Rel">
        /// Returns an array of relative peak power of the range in dBm
        /// </param>
        /// <param name="Delta">
        /// Returns an array of delta of the peak power to the limit line in dB
        /// </param>
        /// <param name="Limit_Check">
        /// Returns an array of limit check state (0 = PASSED, 1 = FAILED
        /// </param>
        /// <param name="Reserved1">
        /// Reserved (0.0)
        /// 
        /// 
        /// </param>
        /// <param name="Reserved2">
        /// Reserved (0.0)
        /// 
        /// </param>
        /// <param name="Returned_Values">
        /// Returns number of value sets (frequency, level and delta to limit line) read (available to return) from the instrument. Number of values returned in arrays is limited with the parameter No of Values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadSEListEvaluation(int No_Of_Values, double[] Range_Number, double[] Start_Frequency, double[] Stop_Frequency, double[] Resolution_Bandwidth, double[] Frequency, double[] Level_Abs, double[] Level_Rel, double[] Delta, double[] Limit_Check, double[] Reserved1, double[] Reserved2, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.ReadSEListEvaluation(this._handle, No_Of_Values, Range_Number, Start_Frequency, Stop_Frequency, Resolution_Bandwidth, Frequency, Level_Abs, Level_Rel, Delta, Limit_Check, Reserved1, Reserved2, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function changes from Spectrum Analyzer to Spectrum Emission Mask measurement mode and back.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SWEEP_MODE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:MODE AUTO | ESPectrum
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="State">
        /// Switches on or off the spectrum emission mask measurement.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SWEEP_MODE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSEMState(bool State)
        {
            int pInvokeResult = PInvoke.ConfigureSEMState(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Spectrum EM Mask Measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_ESP_MODE
        /// RSSPECAN_ATTR_ESP_MAN_VAL
        /// RSSPECAN_ATTR_ESP_TRAN
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:ESPectrum:MODE AUTO | MANual | USER
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:ESPectrum:VALue &lt;numeric_value&gt; dBm
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:ESPectrum:TRANsition
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Limit_Line_Type">
        /// This control selects the Limit Line Type. The driver uses this value to set the RSSPECAN_ATTR_ESP_MODE attribute.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ESP_MODE_AUTO (0) - Auto
        /// RSSPECAN_VAL_ESP_MODE_USER (1) - User
        /// RSSPECAN_VAL_ESP_MODE_MAN  (2) - Manual
        /// 
        /// Default Value: RSSPECAN_VAL_ESP_MODE_AUTO
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ESP_MODE
        /// 
        /// Note(s):
        /// 
        /// (1) The user mode enables the creation of user-defined limit lines. The softkey opens the menus of the limit-line editor, which may be familiar from the basic unit. The limit lines created by the user are included in the table for the manual mode.
        /// 
        /// (2) Manual is not valid for options K83, K85
        /// </param>
        /// <param name="Limit_Line_Value">
        /// This control sets the Limit Line Value. The driver uses this value to set the RSSPECAN_ATTR_ESP_MAN_VAL attribute.
        /// 
        /// Units: dBm
        /// 
        /// Valid Values:
        /// None
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ESP_MAN_VAL
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available if the Limit Line Type is set to Manual.
        /// 
        /// (2) The limit line is selected by specifying the expected power as a value. One of the four possible limit lines is selected depending on the value entered:
        /// Specified value in dBm
        /// Value &gt;= 33; 28 &lt;= value &lt; 33; Value &lt; 28
        /// Selected limit line
        /// "P &gt;= 33"; "28 &lt;= P &lt; 33"; "P &lt; 28"
        /// Value when queried
        /// 33; 28; 0
        /// 
        /// (3) FSL limits:
        /// 43; 39; 31; 0
        /// </param>
        /// <param name="_30kHz_1MHz_Transition">
        /// This control sets the 30kHz/1MHz Transition. The driver uses this value to set the RSSPECAN_ATTR_ESP_TRAN attribute.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 100.0 to 1.0e+9
        /// 
        /// Default Value: 4.0e+6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ESP_TRAN
        /// 
        /// Note(s):
        /// 
        /// (1) This attribute specifies the offset frequency at which the resolution bandwidth is changed between 30k and 1MHz. For multicarrier spectrum emission mask measurement this value can be extended. It is recommended to define user specific limit lines in this case.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSEM(int Limit_Line_Type, double Limit_Line_Value, double _30kHz_1MHz_Transition)
        {
            int pInvokeResult = PInvoke.ConfigureSEM(this._handle, Limit_Line_Type, Limit_Line_Value, _30kHz_1MHz_Transition);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the peak search for spectrum emission mask measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_PEAK_SEARCH_MARGIN
        /// RSSPECAN_ATTR_SEM_PEAK_SEARCH_PEAKS
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PEAKsearch:MARGin &lt;value&gt;
        /// CALCulate&lt;1|2&gt;:PEAKsearch:SUBRanges &lt;value&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Margin">
        /// Defines an overall margin which is subtracted from the limit line to make the peak search.
        /// 
        /// Units: dB
        /// 
        /// Valid Values: -200.0 to 200.0
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_PEAK_SEARCH_MARGIN
        /// </param>
        /// <param name="Peaks_Per_Range">
        /// Defines how many peaks are searched for within one range.
        /// 
        /// Valid Values: 1 to 50
        /// 
        /// Default Value: 25
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PEAK_SEARCH_SUBRANGE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSEMPeakSearch(double Margin, int Peaks_Per_Range)
        {
            int pInvokeResult = PInvoke.ConfigureSEMPeakSearch(this._handle, Margin, Peaks_Per_Range);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function restores the standard limit lines for the spectrum emission mask measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_ESP_RESTORE
        /// 
        /// Remote-control command(s):
        /// CALCulate:LIMit:ESPectrum:RESTore
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int RestoreSEMStandardLines()
        {
            int pInvokeResult = PInvoke.RestoreSEMStandardLines(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function searches the peaks in spectrum emission data.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_ESP_PEAK_SEARCH
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PEAKsearch
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SEMSearchPeak()
        {
            int pInvokeResult = PInvoke.SEMSearchPeak(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function stores the peak list of selected trace in ASCII format to a file.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LIM_STORE_FILE
        /// 
        /// Remote-control command(s):
        /// MMEMory:STORe:FINal &lt;file_name&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="File_Name">
        /// Specifies full path of the file, where spectrum emission mask data will be stored.
        /// 
        /// Valid Values: 
        /// any valid file name
        /// 
        /// Default Value: "c:\\test.txt"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LIM_STORE_FILE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int StoreSEMToFile(string File_Name)
        {
            int pInvokeResult = PInvoke.StoreSEMToFile(this._handle, File_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the spectrum emission mask peak list.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;[:DATA] FINal1 | FINAL2 | FINAL3 | FINAL4 | FINAL5 | FINAL6
        /// FORMat[:DATA] REAL,32
        /// *CLS
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Final_Measurement">
        /// This control sets the number of the final measurement.
        /// 
        /// Valid Values:
        /// 1 to 6
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="No_Of_Values">
        /// This control sets the number of values returned in arrays.
        /// The array size has to be at least equal to this number.
        /// 
        /// Valid Values: &gt; 0
        /// 
        /// Default Value: 10
        /// 
        /// </param>
        /// <param name="Frequency">
        /// Returns an array of frequency list in Hz.
        /// 
        /// </param>
        /// <param name="Level">
        /// Returns an array of level list in dBm.
        /// 
        /// </param>
        /// <param name="Delta_Level">
        /// Returns an array of delta to limit line values in dB. A delta limit of +200dB indicates no limit check is active.
        /// 
        /// </param>
        /// <param name="Returned_Values">
        /// Returns number of value sets (frequency, level and delta to limit line) read (available to return) from the instrument. Number of values returned in arrays is limited with the parameter No of Values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadSEMPeakList(int Window, int Final_Measurement, int No_Of_Values, double[] Frequency, double[] Level, double[] Delta_Level, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.ReadSEMPeakList(this._handle, Window, Final_Measurement, No_Of_Values, Frequency, Level, Delta_Level, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the X/Y -value at the maximum overstepping of the spectrum emission mask limits.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_ESP_QUERY_X
        /// RSSPECAN_ATTR_ESP_QUERY_Y
        /// 
        /// Remote-control command(s):
        /// CALCulate:LIMit:ESPectrum:CHECk:X?
        /// CALCulate:LIMit:ESPectrum:CHECk:Y?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Selection">
        /// Selects the peak list values.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_X (0) - X
        /// RSSPECAN_VAL_Y (1) - Y
        /// 
        /// Default Value: RSSPECAN_VAL_X (0)
        /// </param>
        /// <param name="Value">
        /// Returns selected value.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetSEMValue(int Selection, out double Value)
        {
            int pInvokeResult = PInvoke.GetSEMValue(this._handle, Selection, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates or deactivates spectrum emission mask  LIST EVALUATION.
        /// 
        /// Note(s):
        /// 
        /// (1) In receiver mode (R&amp;S ESL), this attribute performs an automatic peak search after the prescan. For each range, exactly one peak value is calculated.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_LIST_EVAL_STATE
        /// 
        /// Remote-control command(s):
        /// CALCulate1:PEAKsearch:AUTO
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="State">
        /// Switches on or off the spectrum emission mask measurement.
        /// 
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_LIST_EVAL_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SEMListEvaluation(bool State)
        {
            int pInvokeResult = PInvoke.SEMListEvaluation(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the list evaluation results.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:DATA? LIST
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="No_Of_Values">
        /// This control sets the number of values returned in arrays.
        /// The array size has to be at least equal to this number.
        /// 
        /// Valid Values: &gt; 0
        /// 
        /// Default Value: 10
        /// 
        /// </param>
        /// <param name="Range_Number">
        /// Returns an array of range numbers
        /// 
        /// 
        /// </param>
        /// <param name="Start_Frequency">
        /// Returns an array of range start frequency
        /// 
        /// </param>
        /// <param name="Stop_Frequency">
        /// Returns an array of range stop frequency.
        /// </param>
        /// <param name="Resolution_Bandwidth">
        /// Returns an array of resolution bandwidth
        /// </param>
        /// <param name="Frequency">
        /// Returns an array of rrequency of the peak in the range
        /// 
        /// 
        /// </param>
        /// <param name="Power_Abs">
        /// Returns an array of absolute power in dBm of peak.
        /// 
        /// </param>
        /// <param name="Power_Rel">
        /// Returns the relative power in dBc (related to the channel
        /// power) of peak.
        /// </param>
        /// <param name="Delta">
        /// Returns an array of delta of the peak power to the limit line in dB
        /// </param>
        /// <param name="Limit_Check">
        /// Returns an array of limit check state (0 = PASSED, 1 = FAILED
        /// </param>
        /// <param name="Reserved1">
        /// Reserved (0.0)
        /// 
        /// 
        /// </param>
        /// <param name="Reserved2">
        /// Reserved (0.0)
        /// 
        /// </param>
        /// <param name="Returned_Values">
        /// Returns number of value sets (frequency, level and delta to limit line) read (available to return) from the instrument. Number of values returned in arrays is limited with the parameter No of Values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadSEMListEvaluationResults(int No_Of_Values, double[] Range_Number, double[] Start_Frequency, double[] Stop_Frequency, double[] Resolution_Bandwidth, double[] Frequency, double[] Power_Abs, double[] Power_Rel, double[] Delta, double[] Limit_Check, double[] Reserved1, double[] Reserved2, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.ReadSEMListEvaluationResults(this._handle, No_Of_Values, Range_Number, Start_Frequency, Stop_Frequency, Resolution_Bandwidth, Frequency, Power_Abs, Power_Rel, Delta, Limit_Check, Reserved1, Reserved2, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function marks all peaks with blue squares in the diagram.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S FSL or R&amp;S ESL or R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_MARK_ALL_PEAKS
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PSEarch|PEAKsearch:PSHow ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="State">
        /// Marks all peaks with blue squares in the diagram. 
        /// 
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_MARK_ALL_PEAKS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SEMMarkAllPeaks(bool State)
        {
            int pInvokeResult = PInvoke.SEMMarkAllPeaks(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the reference range.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S FSL or R&amp;S ESL or R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_POWER_REF_TYPE
        /// RSSPECAN_ATTR_SEM_BANDWIDTH
        /// RSSPECAN_ATTR_SEM_RRC_FILTER_STATE
        /// RSSPECAN_ATTR_SEM_RRC_FILTER_ALPHA_VALUE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ESPectrum:RTYPe PEAK | CPOWer
        /// [SENSe&lt;1|2&gt;:]ESPectrum:BWID
        /// [SENSe&lt;1|2&gt;:]ESPectrum:FILTer[:RRC][:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]ESPectrum:FILTer[:RRC]:ALPHa
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Power_Reference_Type">
        /// This control sets the power reference type.
        /// 
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_SEM_REF_TYPE_PEAK (0) - Peak Power
        /// RSSPECAN_VAL_SEM_REF_TYPE_CPOW (1) - Channel Power
        /// 
        /// Default Value: RSSPECAN_VAL_SEM_REF_TYPE_CPOW (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_POWER_REF_TYPE
        /// </param>
        /// <param name="Tx_Bandwidth">
        /// This control defines the bandwidth used for measuring the channel power (reference range). The driver uses this value to set the RSSPECAN_ATTR_SEM_BANDWIDTH attribute.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 3.84e+6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_BANDWIDTH
        /// 
        /// Note(s):
        /// 
        /// (1) This setting takes only effect if channel power is selected as power reference type.
        /// </param>
        /// <param name="RRC_Filter_State">
        /// This control activates or deactivates the use of an RRC filter. The driver uses this value to set the RSSPECAN_ATTR_SEM_RRC_FILTER_STATE attribute.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_RRC_FILTER_STATE
        /// 
        /// Note(s):
        /// 
        /// (1) This setting takes only effect if channel power is selected as power reference type.
        /// 
        /// </param>
        /// <param name="RRC_Filter_Alpha">
        /// This control sets the alpha value of the RRC filter.
        /// 
        /// Valid Values:
        /// 0.0 to 1.0
        /// 
        /// Default Value: 0.5
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_RRC_FILTER_ALPHA_VALUE
        /// 
        /// Note(s):
        /// 
        /// (1) This setting takes only effect if channel power is selected as power reference type and if the RRC filter is activated.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSEMReferenceRange(int Power_Reference_Type, double Tx_Bandwidth, bool RRC_Filter_State, double RRC_Filter_Alpha)
        {
            int pInvokeResult = PInvoke.ConfigureSEMReferenceRange(this._handle, Power_Reference_Type, Tx_Bandwidth, System.Convert.ToUInt16(RRC_Filter_State), RRC_Filter_Alpha);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the current position (number) of the reference range.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S FSL or R&amp;S ESL or R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_REF_RANGE_POSITION
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ESPectrum:RRANge?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Reference_Range_Position">
        /// This control returns the current position (number) of the reference range.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_REF_RANGE_POSITION
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QuerySEMReferenceRangePosition(out int Reference_Range_Position)
        {
            int pInvokeResult = PInvoke.QuerySEMReferenceRangePosition(this._handle, out Reference_Range_Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates Fast SEM mode to accelerate spectrum emission mask measurements.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_FAST_SEM
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ESPectrum:HighSPeed ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Fast_SEM">
        /// This control activates Fast SEM mode to accelerate spectrum emission mask measurements.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_FAST_SEM
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFastSEM(bool Fast_SEM)
        {
            int pInvokeResult = PInvoke.ConfigureFastSEM(this._handle, System.Convert.ToUInt16(Fast_SEM));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the specified XML file under C:\r_s\instr\sem_std. If the file is stored in a subdirectory, include the relative path.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S FSL or R&amp;S ESL or R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_PRESET_STANDARD
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ESPectrum:PRESet[:STANdard]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Preset_Standard">
        /// This control selects the specified XML file The driver uses this value to set the RSSPECAN_ATTR_SEM_PRESET_STANDARD attribute.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_PRESET_STANDARD
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSEMPresetStandard(string Preset_Standard)
        {
            int pInvokeResult = PInvoke.ConfigureSEMPresetStandard(this._handle, Preset_Standard);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function copies the XML files from the C:\R_S\instr\sem_backup folder to the C:\R_S\instr\sem_std folder. Files of the same name are overwritten.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S FSL or R&amp;S ESL or R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_RESTORE_STANDARD
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ESPectrum:PRESet:RESTore
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int RestoreSEMStandardFiles()
        {
            int pInvokeResult = PInvoke.RestoreSEMStandardFiles(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function stores the current settings as presettings in the specified XML file under C:\r_s\instr\sem_backup.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S FSV or R&amp;S FSL.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_STORE_PRESET
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ESPectrum:PRESet:STORe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Settings_File_Name">
        /// This control specifies XML file that stores the current settings as presettings.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_STORE_PRESET
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int StoreSEMPresetSettings(string Settings_File_Name)
        {
            int pInvokeResult = PInvoke.StoreSEMPresetSettings(this._handle, Settings_File_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function stores the current list evaluation results in a *.dat file. The file consists of a data section containing the list evaluation results.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S FSL or R&amp;S ESL or R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_LIST_FILE_EXPORT
        /// 
        /// Remote-control command(s):
        /// MMEMory:STORe&lt;1|2&gt;:LIST
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="File_Name">
        /// This control specifies the name.
        /// 
        /// Valid Values: not tested
        /// 
        /// Default Value: "test"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_LIST_FILE_EXPORT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SEMSaveListToFile(string File_Name)
        {
            int pInvokeResult = PInvoke.SEMSaveListToFile(this._handle, File_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the start and stop frequency for the specified range. In order to change the start/stop frequency of the first/last range, select the appropriate span. If you set a span that is smaller than the overall span of the ranges, the measurement includes only the ranges that lie within the defined span and have a minimum span of 20 Hz. The first and last range are adapted to the given span as long as the minimum span of 20 Hz is not violated.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_RANGE_FREQ_START
        /// RSSPECAN_ATTR_SEM_RANGE_FREQ_STOP
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ESPectrum:RANGe&lt;1...20&gt;[:FREQuency]:STARt
        /// [SENSe&lt;1|2&gt;:]ESPectrum:RANGe&lt;1...20&gt;[:FREQuency]:STOP
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Range">
        /// Selects the range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Range_Start">
        /// This control sets the start frequency for the specified range.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: -250.0e+6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_RANGE_FREQ_START
        /// </param>
        /// <param name="Range_Stop">
        /// This control sets the stop frequency for the specified range.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: -2.52e+6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_RANGE_FREQ_STOP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSEMRangeStartStop(int Range, double Range_Start, double Range_Stop)
        {
            int pInvokeResult = PInvoke.ConfigureSEMRangeStartStop(this._handle, Range, Range_Start, Range_Stop);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the filter type for the specified range.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_RANGE_FILTER_TYPE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ESPectrum:RANGe&lt;1...20&gt;:FILTer:TYPE NORMal | CFILter | RRC | PULSe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Range">
        /// Selects the range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Filter_Type">
        /// This control sets the filter type for the specified range.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SEM_FILTER_TYPE_NORM (0) - Gaussian
        /// RSSPECAN_VAL_SEM_FILTER_TYPE_CFIL (1) - Channel
        /// RSSPECAN_VAL_SEM_FILTER_TYPE_RRC  (2) - RRC
        /// RSSPECAN_VAL_SEM_FILTER_TYPE_PULS (3) - EMI (6 dB)
        /// 
        /// Default Value: RSSPECAN_VAL_SEM_FILTER_TYPE_NORM (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_RANGE_FILTER_TYPE
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSEMRangeFilterType(int Range, int Filter_Type)
        {
            int pInvokeResult = PInvoke.ConfigureSEMRangeFilterType(this._handle, Range, Filter_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the resolution and video bandwidth values for the specified range.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_RANGE_BANDWIDTH
        /// RSSPECAN_ATTR_SEM_RANGE_VBW
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ESPectrum:RANGe&lt;1...20&gt;:BANDwidth:RESolution
        /// [SENSe&lt;1|2&gt;:]ESPectrum:RANGe&lt;1...20&gt;:BANDwidth:VIDeo
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Range">
        /// Selects the range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Resolution_Bandwidth">
        /// This control sets the RBW value for the specified range. 
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 30.0e+3 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_RANGE_BANDWIDTH
        /// </param>
        /// <param name="Video_Bandwidth">
        /// This control sets the VBW value for the specified range.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 10.0e+6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_RANGE_VBW
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSEMRangeBandwidth(int Range, double Resolution_Bandwidth, double Video_Bandwidth)
        {
            int pInvokeResult = PInvoke.ConfigureSEMRangeBandwidth(this._handle, Range, Resolution_Bandwidth, Video_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the sweep time values for the specified range.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_RANGE_SWEEP_TIME_AUTO
        /// RSSPECAN_ATTR_SEM_RANGE_SWEEP_TIME
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ESPectrum:RANGe&lt;1...20&gt;:SWEep:TIME:AUTO ON | OFF
        /// [SENSe&lt;1|2&gt;:]ESPectrum:RANGe&lt;1...20&gt;:SWEep:TIME
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Range">
        /// Selects the range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Sweep_Time_Mode">
        /// This control activates or deactivates the automatic sweep time setting for the specified range. The driver uses this value to set the RSSPECAN_ATTR_SEM_RANGE_SWEEP_TIME_AUTO attribute.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Manual
        /// VI_TRUE  (1) - Auto
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_RANGE_SWEEP_TIME_AUTO
        /// </param>
        /// <param name="Sweep_Time">
        /// This control sets the sweep time for the specified range. Allowed values depend on the ratio of span to RBW and RBW to VBW. The driver uses this value to set the RSSPECAN_ATTR_SEM_RANGE_SWEEP_TIME attribute.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0.27 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_RANGE_SWEEP_TIME
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSEMRangeSweepTime(int Range, bool Sweep_Time_Mode, double Sweep_Time)
        {
            int pInvokeResult = PInvoke.ConfigureSEMRangeSweepTime(this._handle, Range, System.Convert.ToUInt16(Sweep_Time_Mode), Sweep_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the reference level for the specified range.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_RANGE_REF_LEVEL
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ESPectrum:RANGe&lt;1...20&gt;:RLEVel
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Range">
        /// Selects the range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Reference_Level">
        /// This control sets the reference level for the specified range.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: -20.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_RANGE_REF_LEVEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSEMRangeReferenceLevel(int Range, double Reference_Level)
        {
            int pInvokeResult = PInvoke.ConfigureSEMRangeReferenceLevel(this._handle, Range, Reference_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the RF attenuation setting for the specified range.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_RANGE_INPUT_ATTENUATION_AUTO
        /// RSSPECAN_ATTR_SEM_RANGE_INPUT_ATTENUATION
        /// RSSPECAN_ATTR_SEM_RANGE_PREAMPLIFIER
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ESPectrum:RANGe&lt;1...20&gt;:INPut:ATTenuation:AUTO ON | OFF
        /// [SENSe&lt;1|2&gt;:]ESPectrum:RANGe&lt;1...20&gt;:INPut:ATTenuation
        /// [SENSe&lt;1|2&gt;:]ESPectrum:RANGe&lt;1...20&gt;:INPut:GAIN:STATe ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Range">
        /// Selects the range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="RF_Attenuation_Mode">
        /// This control activates or deactivates the automatic RF attenuation setting for the specified range.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Manual
        /// VI_TRUE  (1) - Auto
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_RANGE_INPUT_ATTENUATION_AUTO
        /// </param>
        /// <param name="RF_Attenuator">
        /// This control sets the attenuation for the specified range.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_RANGE_INPUT_ATTENUATION
        /// </param>
        /// <param name="Preamplifiier">
        /// This control switches the preamplifier on or off for the specified range.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_RANGE_PREAMPLIFIER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSEMRangeRFAttenuation(int Range, bool RF_Attenuation_Mode, double RF_Attenuator, bool Preamplifiier)
        {
            int pInvokeResult = PInvoke.ConfigureSEMRangeRFAttenuation(this._handle, Range, System.Convert.ToUInt16(RF_Attenuation_Mode), RF_Attenuator, System.Convert.ToUInt16(Preamplifiier));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures a transducer for the specified range. You can only choose a transducer that fulfills the following conditions:
        /// - The transducer overlaps or equals the span of the range.
        /// - The x-axis is linear.
        /// - The unit is dB.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_RANGE_TRANSDUCER
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ESPectrum:RANGe&lt;1...20&gt;:TRANsducer
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Range">
        /// Selects the range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Transducer_Factor">
        /// This control sets a transducer for the specified range. The driver uses this value to set the RSSPECAN_ATTR_SEM_RANGE_TRANSDUCER attribute.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_RANGE_TRANSDUCER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSEMRangeTransducerFactor(int Range, string Transducer_Factor)
        {
            int pInvokeResult = PInvoke.ConfigureSEMRangeTransducerFactor(this._handle, Range, Transducer_Factor);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the limit setting for the specified range.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_RANGE_LIMIT_CHECK_TYPE
        /// RSSPECAN_ATTR_SEM_RANGE_START_FREQ_ABS_LIMIT
        /// RSSPECAN_ATTR_SEM_RANGE_STOP_FREQ_ABS_LIMIT
        /// RSSPECAN_ATTR_SEM_RANGE_START_FREQ_REL_LIMIT
        /// RSSPECAN_ATTR_SEM_RANGE_STOP_FREQ_REL_LIMIT
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ESPectrum:RANGe&lt;1...20&gt;:LIMit:STATe ABSolute | RELative | AND | OR
        /// [SENSe&lt;1|2&gt;:]ESPectrum:RANGe&lt;1...20&gt;:LIMit:ABSolute:STARt
        /// [SENSe&lt;1|2&gt;:]ESPectrum:RANGe&lt;1...20&gt;:LIMit:ABSolute:STOP
        /// [SENSe&lt;1|2&gt;:]ESPectrum:RANGe&lt;1...20&gt;:LIMit:RELative:STARt
        /// [SENSe&lt;1|2&gt;:]ESPectrum:RANGe&lt;1...20&gt;:LIMit:RELative:STOP
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Range">
        /// Selects the range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit_Check_Type">
        /// This control sets the type of limit check for all ranges.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SEM_LIM_CHECK_TYPE_ABS (0) - Absolute
        /// RSSPECAN_VAL_SEM_LIM_CHECK_TYPE_REL (1) - Relative
        /// RSSPECAN_VAL_SEM_LIM_CHECK_TYPE_AND (2) - Abs and Rel
        /// RSSPECAN_VAL_SEM_LIM_CHECK_TYPE_OR  (3) - Abs or Rel
        /// 
        /// Default Value: RSSPECAN_VAL_SEM_LIM_CHECK_TYPE_REL (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_RANGE_LIMIT_CHECK_TYPE
        /// 
        /// Note(s):
        /// 
        /// (1) Absolute: Checks only the absolute limits defined.
        /// 
        /// (2) Relative: Checks only the relative limits. Relative limits are defined as relative to the measured power in the reference range.
        /// 
        /// (3) Abs and Rel: Combines the absolute and relative limit. The limit check fails when both limits are violated.
        /// 
        /// (4) Abs or Rel: Combines the absolute and relative limit. The limit check fails when one of the limits is violated.
        /// </param>
        /// <param name="Absolute_Limit_Start">
        /// This control sets an absolute limit value at the start frequency of the specified range. Different from manual operation, this setting is independently of the defined limit check type.
        /// 
        /// Valid Values:
        /// -400.0 to 400.0 dBm
        /// 
        /// Default Value: -13.0 dBm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_RANGE_START_FREQ_ABS_LIMIT
        /// 
        /// Note(s):
        /// 
        /// (1) This control is ignored if the Limit Check Type is set to Relative.
        /// </param>
        /// <param name="Absolute_Limit_Stop">
        /// This control sets an absolute limit value at the stop frequency of the specified range. Different from manual operation, this setting is independently of the defined limit check type.
        /// 
        /// Valid Values:
        /// -400.0 to 400.0 dBm
        /// 
        /// Default Value: -13.0 dBm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_RANGE_STOP_FREQ_ABS_LIMIT
        /// 
        /// Note(s):
        /// 
        /// (1) This control is ignored if the Limit Check Type is set to Relative.
        /// </param>
        /// <param name="Relative_Limit_Start">
        /// This control sets a relative limit value at the start frequency of the specified range. Different from manual operation, this setting is independently of the defined limit check type. The driver uses this value to set the RSSPECAN_ATTR_SEM_RANGE_START_FREQ_REL_LIMIT attribute.
        /// 
        /// Valid Values:
        /// -400.0 to 400.0 dBc
        /// 
        /// Default Value: -50.0 dBc
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_RANGE_START_FREQ_REL_LIMIT
        /// 
        /// Note(s):
        /// 
        /// (1) This control is ignored if the Limit Check Type is set to Absolute.
        /// </param>
        /// <param name="Relative_Limit_Stop">
        /// This control sets a relative limit value at the stop frequency of the specified range. Different from manual operation, this setting is independently of the defined limit check type. The driver uses this value to set the RSSPECAN_ATTR_SEM_RANGE_STOP_FREQ_REL_LIMIT attribute.
        /// 
        /// Valid Values:
        /// -400.0 to 400.0 dBc
        /// 
        /// Default Value: -50.0 dBc
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_RANGE_STOP_FREQ_REL_LIMIT
        /// 
        /// Note(s):
        /// 
        /// (1) This control is ignored if the Limit Check Type is set to Absolute.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSEMRangeLimits(int Range, int Limit_Check_Type, double Absolute_Limit_Start, double Absolute_Limit_Stop, double Relative_Limit_Start, double Relative_Limit_Stop)
        {
            int pInvokeResult = PInvoke.ConfigureSEMRangeLimits(this._handle, Range, Limit_Check_Type, Absolute_Limit_Start, Absolute_Limit_Stop, Relative_Limit_Start, Relative_Limit_Stop);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the number of defined ranges.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_RANGE_COUNT
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ESPectrum:RANGe&lt;1...20&gt;:COUNt?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Range_Count">
        /// Returns the number of defined ranges.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetSEMRangeCount(out int Range_Count)
        {
            int pInvokeResult = PInvoke.GetSEMRangeCount(this._handle, out Range_Count);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function inserts a new range before or after the specified range. The range numbers are updated accordingly.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_INSERT_NEW_RANGE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ESPectrum:RANGe&lt;1...20&gt;:INSert AFTer | BEFore
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Range">
        /// Selects the range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Insert_Range">
        /// This control inserts a new range before or after the specified range. The range numbers are updated accordingly.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SEM_INSERT_RANGE_AFTER  (0) - After
        /// RSSPECAN_VAL_SEM_INSERT_RANGE_BEFORE (1) - Before
        /// 
        /// Default Value: RSSPECAN_VAL_SEM_INSERT_RANGE_AFTER (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_INSERT_NEW_RANGE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SEMInsertNewRange(int Range, int Insert_Range)
        {
            int pInvokeResult = PInvoke.SEMInsertNewRange(this._handle, Range, Insert_Range);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function deletes the specified range. The range numbers are updated accordingly. The reference range cannot be deleted. A minimum of three ranges is mandatory.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_DELETE_RANGE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ESPectrum:RANGe&lt;1...20&gt;:DELete
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Range">
        /// Selects the range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SEMDeleteRange(int Range)
        {
            int pInvokeResult = PInvoke.SEMDeleteRange(this._handle, Range);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the power classes used in the spectrum emission mask measurement. It is only possible to use power classes for which limits are defined. Also, either only one power class at a time or all power classes together can be selected.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S FSV or R&amp;S FSL.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_POWER_CLASS_USED
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ESPectrum:PCLass&lt;1...4&gt;[:EXCLusive] ON | OFF
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Power_Class_Number">
        /// This control selects the number of power class to be modified.
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Used_Power_Class">
        /// This control sets the power classes used in the spectrum emission mask measurement. 
        /// 
        /// Valid Values:
        /// VI_FALSE         (0) - Off
        /// VI_TRUE          (1) - On
        /// RSSPECAN_VAL_ALL (2) - All Power Classes
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_POWER_CLASS_USED
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSEMUsedPowerClasses(int Power_Class_Number, int Used_Power_Class)
        {
            int pInvokeResult = PInvoke.ConfigureSEMUsedPowerClasses(this._handle, Power_Class_Number, Used_Power_Class);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the number of power classes to be defined.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S FSV or R&amp;S FSL.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_POWER_CLASS_COUNT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ESPectrum:PCLass&lt;1...4&gt;:COUNt
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Count">
        /// This control sets the number of power classes to be defined.
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_POWER_CLASS_COUNT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSEMPowerClassCount(int Count)
        {
            int pInvokeResult = PInvoke.ConfigureSEMPowerClassCount(this._handle, Count);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the power class settings of the Spectrum Emission Mask measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S FSV or R&amp;S FSL.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_POWER_CLASS_LIMIT_STATE
        /// RSSPECAN_ATTR_SEM_POWER_CLASS_LOWER_LEVEL_LIMIT
        /// RSSPECAN_ATTR_SEM_POWER_CLASS_UPPER_LEVEL_LIMIT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ESPectrum:PCLass&lt;1...4&gt;:LIMit[:STATe] ABSolute | RELative | AND | OR
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ESPectrum:PCLass&lt;1...4&gt;:MINimum
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ESPectrum:PCLass&lt;1...4&gt;:MAXimum
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Power_Class_Number">
        /// This control selects the number of power class to be modified.
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit_State">
        /// This control defines which limits are evaluated in the measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SEM_LIM_CHECK_TYPE_ABS (0) - Absolute
        /// RSSPECAN_VAL_SEM_LIM_CHECK_TYPE_REL (1) - Relative
        /// RSSPECAN_VAL_SEM_LIM_CHECK_TYPE_AND (2) - Abs and Rel
        /// RSSPECAN_VAL_SEM_LIM_CHECK_TYPE_OR  (3) - Abs or Rel
        /// 
        /// Default Value: RSSPECAN_VAL_SEM_LIM_CHECK_TYPE_REL (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_POWER_CLASS_LIMIT_STATE
        /// 
        /// Note(s):
        /// 
        /// (1) Absolute: Evaluates only limit lines with absolute power values.
        /// 
        /// (2) Relative: Evaluates only limit lines with relative power values.
        /// 
        /// (3) Abs and Rel: Evaluates limit lines with relative and absolute power values. A negative result is returned if both limits fail.
        /// 
        /// (4) Abs or Rel: Evaluates limit lines with relative and absolute power values. A negative result is returned if at least one limit failed.
        /// </param>
        /// <param name="Lower_Level_Limit">
        /// This control sets the minimum lower level limit for one power class. The limit always start at -200 dBm, i.e. the first lower limit can not be set. If more than one power class is in use, the lower limit must equal the upper limit of the previous power class.
        /// 
        /// Valid Values:
        /// -200.0 to 200.0 dBm
        /// 
        /// Default Value: -200.0 dBm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_POWER_CLASS_LOWER_LEVEL_LIMIT
        /// </param>
        /// <param name="Upper_Level_Limit">
        /// This control sets the upper limit level for one power class. The limit always ends at 200 dBm, i.e. the upper limit of the last power class can not be set. If more than one power class is in use, the upper limit must equal the lower limit of the next power class.
        /// 
        /// Valid Values:
        /// -200.0 to 200.0 dBm
        /// 
        /// Default Value: 200.0 dBm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SEM_POWER_CLASS_UPPER_LEVEL_LIMIT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSEMPowerClassLimits(int Power_Class_Number, int Limit_State, double Lower_Level_Limit, double Upper_Level_Limit)
        {
            int pInvokeResult = PInvoke.ConfigureSEMPowerClassLimits(this._handle, Power_Class_Number, Limit_State, Lower_Level_Limit, Upper_Level_Limit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets up to 4 power classes in one step.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S FSV or R&amp;S FSL.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ESPectrum:LIMits
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Number_Of_Limits">
        /// This control sets the number of power class limits.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limits">
        /// This control sets up to 4 power classes in one step.
        /// 
        /// Valid Values:
        /// 1-3 numeric values between -200 and 200 dBm
        /// 
        /// Default Value: none
        /// 
        /// Note(s):
        /// 
        /// (1) The number of values has to match the value defined by the control Number Of Limits.
        /// 
        /// (2) Example: -50,50,70
        /// 
        /// Defines the following power classes:
        /// 
        /// &lt;-200, -50&gt;
        /// &lt;-50, 50&gt;
        /// &lt;50, 70&gt;
        /// &lt;70, 200&gt;
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSEMPowerClassAllLimits(int Number_Of_Limits, double[] Limits)
        {
            int pInvokeResult = PInvoke.ConfigureSEMPowerClassAllLimits(this._handle, Number_Of_Limits, Limits);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns up to 4 power classes in one step.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S FSV or R&amp;S FSL.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ESPectrum:LIMits?
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limits">
        /// This control returns up to 4 power classes in one step.
        /// 
        /// Valid Values:
        /// -200 and 200 dBm
        /// 
        /// Note(s):
        /// 
        /// (1) The size of this array should be at least 5 elements.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QuerySEMPowerClassAllLimits(double[] Limits)
        {
            int pInvokeResult = PInvoke.QuerySEMPowerClassAllLimits(this._handle, Limits);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the measurement time for the TDMA power measurement.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDMA_POW_MEAS_TIME
        /// 
        /// Remote-control command(s):
        /// CALCulate:MARKer&lt;1...6&gt;:FUNCtion:TDMA:BURSt
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker number.
        /// 
        /// Valid Values:
        /// 1 to 6
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Measurement_Time">
        /// Defines the measurement time for the TDMA power measurement.
        /// 
        /// Valid Range: 
        /// 31.7e-3 to 19.0 ms
        /// 
        /// Default Value: 500.0e-6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDMA_POW_MEAS_TIME
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureTDMAPowerMeasurementTime(int Marker, double Measurement_Time)
        {
            int pInvokeResult = PInvoke.ConfigureTDMAPowerMeasurementTime(this._handle, Marker, Measurement_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches instrument to EMI receiver mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument[:SELect] RECeiver
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReceiverMode()
        {
            int pInvokeResult = PInvoke.ReceiverMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function adds a receiver frequency to the peak list. Only frequencies in the currently displayed frequency range of the scan are allowed.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S ESL
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_ADD_RECEIVER_FREQUENCY
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PSEarch|PEAKsearch:ADD
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Receiver_Frequency">
        /// Adds a receiver frequency to the peak list. Only frequencies in the currently displayed frequency range of the scan are allowed.
        /// 
        /// 
        /// Valid Values: not checked 
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ADD_RECEIVER_FREQUENCY
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int AddReceiverFrequencyToPeakList(double Receiver_Frequency)
        {
            int pInvokeResult = PInvoke.AddReceiverFrequencyToPeakList(this._handle, Receiver_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function assigns a symbol to the peak values of a trace. For each trace another symbol is used (e.g. an x or a +).
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available for R&amp;S ESL.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_PEAK_LIST_DISPLAY_STATE
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...6&gt;:SYMBol CROSs | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Trace">
        /// Selects the trace.
        /// 
        /// Valid Values:
        /// 1 to 6
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Peak_Symbol">
        /// Assigns a symbol to the peak values of a trace. For each trace another symbol is used (e.g. an x or a +).
        /// 
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_SEM_PEAK_SYMB_OFF  (0) - Off
        /// RSSPECAN_VAL_SEM_PEAK_SYMB_CROS (1) - Cross
        /// 
        /// Default Value: RSSPECAN_VAL_SEM_PEAK_SYMB_CROS (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_PEAK_LIST_DISPLAY_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReceiverPeakListDisplayState(int Trace, int Peak_Symbol)
        {
            int pInvokeResult = PInvoke.ConfigureReceiverPeakListDisplayState(this._handle, Trace, Peak_Symbol);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the detector used to record measured values for a specific trace in the final measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S ESL
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_FINAL_MEAS_DETECTOR_TYPE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DETector&lt;1...6&gt;:FMEasurement NEGative | POSitive | RMS | AVERage | QPEak | CRMS | CAVerage
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// 1 to 6
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Detector_Type">
        /// Sets the detector used to record measured values for a specific trace in the final measurement.
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_DETECTOR_TYPE_AVERAGE   (1) - Average
        /// RSSPECAN_VAL_DETECTOR_TYPE_MAX_PEAK  (2) - Maximum Peak
        /// RSSPECAN_VAL_DETECTOR_TYPE_MIN_PEAK  (3) - Minimum Peak
        /// RSSPECAN_VAL_DETECTOR_TYPE_RMS       (5) - RMS
        /// RSSPECAN_VAL_DETECTOR_TYPE_QPK       (6) - QPeak
        /// RSSPECAN_VAL_DETECTOR_TYPE_CRMS      (7) - Cispr RMS
        /// RSSPECAN_VAL_DETECTOR_TYPE_CAV       (8) - Cispr Average
        /// 
        /// Default Value: RSSPECAN_VAL_DETECTOR_TYPE_QPK (6)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_FINAL_MEAS_DETECTOR_TYPE
        /// 
        /// Note(s):
        /// 
        /// (1) Average: Average value of samples taken within the bin for a dedicated point on the display.
        /// 
        /// (2) Maximum Peak: Obtains the maximum video signal between the last display point and the present display point.
        /// 
        /// (3) Minimum Peak: Obtains the minimum video signal between the last display point and the present display point.
        /// 
        /// (4) RMS: RMS value of samples taken within the bin for a dedicated point on the display.
        /// 
        /// (5) QPeak: Performs a signal evaluation for EMC measurements.
        /// 
        /// (6) Cispr RMS: Performs a signal evaluation for EMC measurements.
        /// 
        /// (7) Cispr Average: Performs a signal evaluation for EMC measurements.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFinalReceiverDetector(int Window, int Trace, int Detector_Type)
        {
            int pInvokeResult = PInvoke.ConfigureFinalReceiverDetector(this._handle, Window, Trace, Detector_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the detector used in the bargraph measurement. This function either couples the detector in the selected measurement window to the current trace setting or turns coupling off.
        /// 
        /// Four detectors can be activated at the same time.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S ESL
        /// 
        /// Attribute(s):
        /// none
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DETector&lt;1...6&gt;:RECeiver[:FUNCtion] POSitive | NEGative | RMS | AVERage | QPEak | CAVerage | CRMS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// 1 to 6
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Average">
        /// This control switches on or off average detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <param name="Max_Peak">
        /// This control switches on or off max peak detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <param name="Min_Peak">
        /// This control switches on or off min peak detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <param name="RMS">
        /// This control switches on or off RMS detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <param name="Quasipeak">
        /// This control switches on or off quasipeak detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <param name="CISPR_RMS">
        /// This control switches on or off CISPR RMS detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <param name="CISPR_Average">
        /// This control switches on or off CISPR Average detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReceiverDetector(int Window, int Trace, bool Average, bool Max_Peak, bool Min_Peak, bool RMS, bool Quasipeak, bool CISPR_RMS, bool CISPR_Average)
        {
            int pInvokeResult = PInvoke.ConfigureReceiverDetector(this._handle, Window, Trace, System.Convert.ToUInt16(Average), System.Convert.ToUInt16(Max_Peak), System.Convert.ToUInt16(Min_Peak), System.Convert.ToUInt16(RMS), System.Convert.ToUInt16(Quasipeak), System.Convert.ToUInt16(CISPR_RMS), System.Convert.ToUInt16(CISPR_Average));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the selected receiver mode detectors.
        /// 
        /// Three detectors can be activated at the same time.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S ESL
        /// 
        /// Attribute(s):
        /// none
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DETector&lt;1...6&gt;:RECeiver[:FUNCtion]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// 1 to 6
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Average">
        /// This control returns the state of average detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <param name="Max_Peak">
        /// This control returns the state of max peak detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <param name="Min_Peak">
        /// This control returns the state of min peak detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <param name="RMS">
        /// This control returns the state of RMS detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <param name="Quasipeak">
        /// This control returns the state of quasipeak detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <param name="CISPR_RMS">
        /// This control returns the state of CISPR RMS detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <param name="CISPR_Average">
        /// This control returns the state of CISPR Average detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryReceiverDetector(int Window, int Trace, out bool Average, out bool Max_Peak, out bool Min_Peak, out bool RMS, out bool Quasipeak, out bool CISPR_RMS, out bool CISPR_Average)
        {
            ushort AverageAsUShort;
            ushort Max_PeakAsUShort;
            ushort Min_PeakAsUShort;
            ushort RMSAsUShort;
            ushort QuasipeakAsUShort;
            ushort CISPR_RMSAsUShort;
            ushort CISPR_AverageAsUShort;
            int pInvokeResult = PInvoke.QueryReceiverDetector(this._handle, Window, Trace, out AverageAsUShort, out Max_PeakAsUShort, out Min_PeakAsUShort, out RMSAsUShort, out QuasipeakAsUShort, out CISPR_RMSAsUShort, out CISPR_AverageAsUShort);
            Average = System.Convert.ToBoolean(AverageAsUShort);
            Max_Peak = System.Convert.ToBoolean(Max_PeakAsUShort);
            Min_Peak = System.Convert.ToBoolean(Min_PeakAsUShort);
            RMS = System.Convert.ToBoolean(RMSAsUShort);
            Quasipeak = System.Convert.ToBoolean(QuasipeakAsUShort);
            CISPR_RMS = System.Convert.ToBoolean(CISPR_RMSAsUShort);
            CISPR_Average = System.Convert.ToBoolean(CISPR_AverageAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the type of analog demodulation. 
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S ESL
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_DEMOD_TYPE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DEMod OFF | AM | FM
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Demodulation_Type">
        /// This control selects the type of analog demodulation.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_REC_DEMOD_OFF (0) - Off
        /// RSSPECAN_VAL_REC_DEMOD_AM  (1) - AM
        /// RSSPECAN_VAL_REC_DEMOD_FM  (2) - FM
        /// 
        /// Default Value: RSSPECAN_VAL_REC_DEMOD_OFF (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_DEMOD_TYPE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReceiverDemodulationType(int Window, int Demodulation_Type)
        {
            int pInvokeResult = PInvoke.ConfigureReceiverDemodulationType(this._handle, Window, Demodulation_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the input of a level threshold below which the audible AF is cut off. The squelch function is associated with the internal trigger function (TRIGGER menu), which will be switched on automatically with the squelch.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S ESL
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_DEMOD_SQUELCH_STATE
        /// RSSPECAN_ATTR_VIDEO_TRIGGER_LEVEL
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DEMod:SQUelch[:STATe] ON | OFF
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:VIDeo
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Squelch_State">
        /// This control enables the input of a level threshold below which the audible AF is cut off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_DEMOD_SQUELCH_STATE
        /// </param>
        /// <param name="Trigger_Level">
        /// This control sets the trigger level.
        /// 
        /// Valid Values:
        /// 0.0 to 100.0 %
        /// 
        /// Default Value: 50.0 %
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VIDEO_TRIGGER_LEVEL
        /// 
        /// Note(s):
        /// 
        /// (1) This control is ignored if the squelch state is set to Off.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReceiverSquelch(int Window, bool Squelch_State, double Trigger_Level)
        {
            int pInvokeResult = PInvoke.ConfigureReceiverSquelch(this._handle, Window, System.Convert.ToUInt16(Squelch_State), Trigger_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches block data transmission during a scan on and off. The availability of data is reported in the STATus:OPERation-Register.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S ESL
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_TRACE_FEED_CONTROL
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:FEED:CONTrol ALWays | NEVer
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Feed_Control">
        /// This control switches block data transmission during a scan on and off. The availability of data is reported in the STATus:OPERation-Register.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TRAC_FEED_NEVER  (0) - Never
        /// RSSPECAN_VAL_TRAC_FEED_ALWAYS (1) - Always
        /// 
        /// Default Value: RSSPECAN_VAL_TRAC_FEED_NEVER (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_TRACE_FEED_CONTROL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReceiverTraceFeedControl(int Feed_Control)
        {
            int pInvokeResult = PInvoke.ConfigureReceiverTraceFeedControl(this._handle, Feed_Control);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the maximum number of measurement points which are transferred in one block after the query command TRACE? SCAN. The total amount of bytes which is transferred depends on the number of active traces.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S ESL
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_TRACE_POINTS
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:POINts LIMit, 1 to 10000
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Trace_Points">
        /// This control defines the maximum number of measurement points which are transferred in one block after the query command TRACE? SCAN. The total amount of bytes which is transferred depends on the number of active traces.
        /// 
        /// Valid Values:
        /// 1 to 10000
        /// 
        /// Default Value: 1000
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_TRACE_POINTS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReceiverTracePoints(int Trace_Points)
        {
            int pInvokeResult = PInvoke.ConfigureReceiverTracePoints(this._handle, Trace_Points);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function toggles between linear and logarithmic step modes of the receiver.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S ESL
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_STEP_MODE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:SPACing LINear | LOGarithmic | AUTO
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Step_Mode">
        /// This control toggles between linear and logarithmic step modes of the receiver.
        /// 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SWEEP_SPAC_LOG  (0) - Logarithmic
        /// RSSPECAN_VAL_SWEEP_SPAC_LIN  (1) - Linear
        /// RSSPECAN_VAL_SWEEP_SPAC_AUTO (2) - Auto
        /// 
        /// Default Value: RSSPECAN_VAL_SWEEP_SPAC_LIN  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_STEP_MODE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReceiverStepMode(int Step_Mode)
        {
            int pInvokeResult = PInvoke.ConfigureReceiverStepMode(this._handle, Step_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the method used to determine the peak levels of the scan.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available for R&amp;S ESL.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_PEAK_SEARCH_METHOD
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PSEarch|PEAKsearch:METHod PEAK | SUBRanges
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Method">
        /// Defines the method used to determine the peak levels of the scan.
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_REC_PSEARCH_METHOD_PEAK (0) - Peak
        /// RSSPECAN_VAL_REC_PSEARCH_METHOD_SUBR (1) - Subrange
        /// 
        /// Default Value: RSSPECAN_VAL_REC_PSEARCH_METHOD_PEAK (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_PEAK_SEARCH_METHOD
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReceiverPeakSearchMethod(int Method)
        {
            int pInvokeResult = PInvoke.ConfigureReceiverPeakSearchMethod(this._handle, Method);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function changes two settings, depending on which method to determine the peak levels is activated:
        /// 
        /// (1) Peaks - The function sets the number of peaks to be determined.
        ///  
        /// (2) Subranges - The function sets the number of subranges to be analyzed.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S ESL
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_NUMBER_OF_PEAKS_SUBRANGES
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PSEarch|PEAKsearch:SUBRanges
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Number_of_Peaks_Subranges">
        /// Sets the number of peaks/subranges to be determined.
        /// 
        /// Valid Values: 1 to 500
        /// 
        /// Default Value: 25
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_NUMBER_OF_PEAKS_SUBRANGES
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReceiverNumberOfPeaksSubranges(int Number_of_Peaks_Subranges)
        {
            int pInvokeResult = PInvoke.ConfigureReceiverNumberOfPeaksSubranges(this._handle, Number_of_Peaks_Subranges);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the peaks to be found in each subrange. It is used if the peak search method is set to Subranges.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available for R&amp;S ESL.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_PEAKS_PER_SUBRANGE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PSEarch|PEAKsearch:SUBRanges:PCOunt
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Peak_Per_Subrange">
        /// Sets the peaks to be found in each subrange. It is used if the peak search method is set to Subranges. 
        /// 
        /// 
        /// Valid Values: 
        /// 1 to 10
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_PEAKS_PER_SUBRANGE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReceiverPeaksPerSubrange(int Peak_Per_Subrange)
        {
            int pInvokeResult = PInvoke.ConfigureReceiverPeaksPerSubrange(this._handle, Peak_Per_Subrange);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the margin for the peak search.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S ESL
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SEM_PEAK_SEARCH_MARGIN
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PSEarch|PEAKsearch:MARGin
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Margin">
        /// Defines the margin for the peak search.
        /// 
        /// Units: dB
        /// 
        /// Valid Values:
        /// -200.0 to 200.0
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PEAK_SEARCH_MARGIN
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReceiverMargin(double Margin)
        {
            int pInvokeResult = PInvoke.ConfigureReceiverMargin(this._handle, Margin);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the peak excursion. 
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S ESL
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PEAK_EXCURSION
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:PEXCursion
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Peak_Excursion">
        /// Sets the minimum amplitude variation of the signal in dB that the marker can identify as a peak.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 6.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PEAK_EXCURSION
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReceiverPeakExcursion(double Peak_Excursion)
        {
            int pInvokeResult = PInvoke.ConfigureReceiverPeakExcursion(this._handle, Peak_Excursion);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures automatic or interactive final measurement and the final measurement time.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S ESL
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_FINAL_MEAS_TIME
        /// RSSPECAN_ATTR_RECEIVER_FINAL_MEAS_AUTO
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FMEasurement:TIME
        /// [SENSe&lt;1|2&gt;:]FMEasurement:AUTO ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Final_Measurement_Time">
        /// This control defines the measurement time in which the values indicated in the peak list (final measurement values) are re-checked.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 1.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_FINAL_MEAS_TIME
        /// </param>
        /// <param name="Final_Measurement">
        /// This control activates automatic or interactive final measurement.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Interactive
        /// VI_TRUE  (1) - Automatic
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_FINAL_MEAS_AUTO
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReceiverFinalMeasurementFinalTest(int Window, double Final_Measurement_Time, bool Final_Measurement)
        {
            int pInvokeResult = PInvoke.ConfigureReceiverFinalMeasurementFinalTest(this._handle, Window, Final_Measurement_Time, System.Convert.ToUInt16(Final_Measurement));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function performs an automatic peak search after the prescan. For each range, exactly one peak value is calculated.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S ESL
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_PEAK_SEARCH_AFTER_PRESCAN
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PSEarch|PEAKsearch:AUTO ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Peak_Search_After_Prescan">
        /// This control performs an automatic peak search after the prescan. For each range, exactly one peak value is calculated.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_PEAK_SEARCH_AFTER_PRESCAN
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReceiverPeakSearchAfterPrescan(bool Peak_Search_After_Prescan)
        {
            int pInvokeResult = PInvoke.ConfigureReceiverPeakSearchAfterPrescan(this._handle, System.Convert.ToUInt16(Peak_Search_After_Prescan));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the V-network to be controlled via the user port.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S ESL
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_PRE_V_NETWORK_TYPE
        /// RSSPECAN_ATTR_RECEIVER_FINAL_MEAS_V_NETWORK_TYPE
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:LISN[:TYPE] ESH3Z5 | ESH2Z5 | ENV4200 | ENV216 | OFF
        /// [SENSe&lt;1|2&gt;:]FMEasurement:LISN[:TYPE] ESH3Z5 | ESH2Z5 | ENV4200 | ENV216 | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Measurement">
        /// This control selects the receiver measurement type.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VNET_MEAS_PRESCAN (0) - Prescan
        /// RSSPECAN_VAL_VNET_MEAS_FINAL   (1) - Final
        /// 
        /// Default Value: RSSPECAN_VAL_VNET_MEAS_PRESCAN (0)
        /// </param>
        /// <param name="V_Network_Type">
        /// This control selects the V-network to be controlled via user port.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_REC_VNET_TYPE_OFF    (0) - Off
        /// RSSPECAN_VAL_REC_VNET_TYPE_ESH3Z5 (1) - ESH3 Z5
        /// RSSPECAN_VAL_REC_VNET_TYPE_ESH2Z5 (2) - ESH2 Z5
        /// RSSPECAN_VAL_REC_VNET_TYPE_4200   (3) - ENV 4200
        /// RSSPECAN_VAL_REC_VNET_TYPE_216    (4) - ENV 216
        /// 
        /// Default Value: RSSPECAN_VAL_REC_VNET_TYPE_OFF (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_PRE_V_NETWORK_TYPE
        /// RSSPECAN_ATTR_RECEIVER_FINAL_MEAS_V_NETWORK_TYPE
        /// 
        /// Note(s):
        /// 
        /// (1) Off: No network is selected.
        /// 
        /// (42) ESH3 Z5: R&amp;S ESH3-Z5 (two phases and protective earth are controllable)
        /// 
        /// (3) ESH2 Z5: R&amp;S ESH2-Z5 (four phases and protective earth are controllable)
        /// 
        /// (4) ENV 4200: R&amp;S ENV 4200 (four phases are controllable)
        /// 
        /// (5) ENV 216: R&amp;S ENV 216 (two phases and highpass are controllable)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReceiverVnetworkType(int Window, int Measurement, int V_Network_Type)
        {
            int pInvokeResult = PInvoke.ConfigureReceiverVnetworkType(this._handle, Window, Measurement, V_Network_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the setting for the high pass filter on the V-network with which the level measurement is performed in the preliminary or final measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for the R&amp;S ENV 216 Vnetwork (function rsspecan_ConfigureReceiverVnetworkType).
        /// 
        /// (2) This function is available only for R&amp;S ESL
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_PRE_HIGH_PASS_FILTER
        /// RSSPECAN_ATTR_RECEIVER_FINAL_MEAS_HIGH_PASS_FILTER
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:LISN:FILTer:HPAS[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]FMEasurement:FILTer:HPAS[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Measurement">
        /// This control selects the receiver measurement type.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VNET_MEAS_PRESCAN (0) - Prescan
        /// RSSPECAN_VAL_VNET_MEAS_FINAL   (1) - Final
        /// 
        /// Default Value: RSSPECAN_VAL_VNET_MEAS_PRESCAN (0)
        /// </param>
        /// <param name="High_Pass_Filter_State">
        /// This control selects the setting for the high pass filter on the V-network with which the level measurement is performed.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_PRE_HIGH_PASS_FILTER
        /// RSSPECAN_ATTR_RECEIVER_FINAL_MEAS_HIGH_PASS_FILTER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReceiverVnetworkHighPassFilter(int Window, int Measurement, bool High_Pass_Filter_State)
        {
            int pInvokeResult = PInvoke.ConfigureReceiverVnetworkHighPassFilter(this._handle, Window, Measurement, System.Convert.ToUInt16(High_Pass_Filter_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the phase of the V-network that is used and which is controlled via the user port. This setting is used in the receiver mode also for the preliminary or final measurement. 
        /// 
        /// Note(s):
        /// 
        /// (1)  L2 and L3 are only available with four-line V-networks (FMEasurement:LISN ENV4200).
        /// 
        /// (2) This function is available only for R&amp;S ESL
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_PRES_V_NETWORK_PHASE
        /// RSSPECAN_ATTR_RECEIVER_FINAL_MEAS_V_NETWORK_PHASE
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:LISN:PHASe L1 | L2 | L3 | N
        /// [SENSe&lt;1|2&gt;:]FMEasurement:LISN:PHASe L1 | L2 | L3 | N
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Measurement">
        /// This control selects the receiver measurement type.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VNET_MEAS_PRESCAN (0) - Prescan
        /// RSSPECAN_VAL_VNET_MEAS_FINAL   (1) - Final
        /// 
        /// Default Value: RSSPECAN_VAL_VNET_MEAS_PRESCAN (0)
        /// </param>
        /// <param name="Phase">
        /// This control selects the phase of the V-network on which level measurements are consecutively performed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_REC_VNET_PHAS_N  (0) - N
        /// RSSPECAN_VAL_REC_VNET_PHAS_L1 (1) - L1
        /// RSSPECAN_VAL_REC_VNET_PHAS_L2 (2) - L2
        /// RSSPECAN_VAL_REC_VNET_PHAS_L3 (3) - L3
        /// 
        /// Default Value: RSSPECAN_VAL_REC_VNET_PHAS_N (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_PRES_V_NETWORK_PHASE
        /// RSSPECAN_ATTR_RECEIVER_FINAL_MEAS_V_NETWORK_PHASE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReceiverVnetworkPhase(int Window, int Measurement, int Phase)
        {
            int pInvokeResult = PInvoke.ConfigureReceiverVnetworkPhase(this._handle, Window, Measurement, Phase);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the phase of the V-network that is used and which is controlled via the user port.
        /// 
        /// Note(s):
        /// 
        /// (1)  L2 and L3 are only available with four-line V-networks (FMEasurement:LISN ENV4200).
        /// 
        /// (2) This function is available only for R&amp;S ESL
        /// 
        /// Attribute(s):
        /// none
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FMEasurement:LISN:PHASe L1 | L2 | L3 | N[[,L1 | L2 | L3 | N],...]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Number_Of_Phases">
        /// This control sets the number of phases.
        /// 
        /// Valid Values:
        /// &gt;1
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Phases">
        /// This control sets the phases of the V-network on which level measurements are consecutively performed in the final measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_REC_VNET_PHAS_N  (0) - N
        /// RSSPECAN_VAL_REC_VNET_PHAS_L1 (1) - L1
        /// RSSPECAN_VAL_REC_VNET_PHAS_L2 (2) - L2
        /// RSSPECAN_VAL_REC_VNET_PHAS_L3 (3) - L3
        /// 
        /// Default Value: none
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReceiverVnetworkMultiPhaseFinalMeasurement(int Window, int Number_Of_Phases, int[] Phases)
        {
            int pInvokeResult = PInvoke.ConfigureReceiverVnetworkMultiPhaseFinalMeasurement(this._handle, Window, Number_Of_Phases, Phases);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the phases of the V-network on which level measurements are consecutively performed in the final measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S ESL
        /// 
        /// Attribute(s):
        /// none
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FMEasurement:LISN:PHASe?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Number_Of_Phasees">
        /// This control displays the number of phases.
        /// 
        /// Valid Values:
        /// &gt;1
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Phase">
        /// This control returns the phases of the V-network on which level measurements are consecutively performed in the final measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_REC_VNET_PHAS_N  (0) - N
        /// RSSPECAN_VAL_REC_VNET_PHAS_L1 (1) - L1
        /// RSSPECAN_VAL_REC_VNET_PHAS_L2 (2) - L2
        /// RSSPECAN_VAL_REC_VNET_PHAS_L3 (3) - L3
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryReceiverVnetworkMultiPhaseFinalMeasurement(int Window, out int Number_Of_Phasees, int[] Phase)
        {
            int pInvokeResult = PInvoke.QueryReceiverVnetworkMultiPhaseFinalMeasurement(this._handle, Window, out Number_Of_Phasees, Phase);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function searches the peaks in spurious emissions data. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SE_PEAK_SEARCH
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PEAKsearch
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReceiverPeakSearch()
        {
            int pInvokeResult = PInvoke.ReceiverPeakSearch(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the minimum and maximum level of the grid in the current unit of the scan display.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S ESL.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_AMPLITUDE_MIN_LEVEL
        /// RSSPECAN_ATTR_DISP_AMPLITUDE_MAX_LEVEL
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...6&gt;:Y[:SCALe]:BOTTom
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...6&gt;:Y[:SCALe]:TOP
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Minimum_Level">
        /// Sets the minimum level of the grid in the current unit of the scan display.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: -10.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_AMPLITUDE_MIN_LEVEL
        /// </param>
        /// <param name="Maximum_Level">
        /// Sets the maximum level of the grid in the current unit of the scan display.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 10.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_AMPLITUDE_MAX_LEVEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReceiverAmplitudeLevel(double Minimum_Level, double Maximum_Level)
        {
            int pInvokeResult = PInvoke.ConfigureReceiverAmplitudeLevel(this._handle, Minimum_Level, Maximum_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates or deactivates the bargraph maxhold. The bargraph maxhold indicates the maximum measured value of the signal.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S ESL
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_BARGRAPH_MAXHOLD
        /// 
        /// Remote-control command(s):
        /// DISPlay:BARGraph:PHOLd ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Maxhold_State">
        /// Activates or deactivates the bargraph maxhold. The bargraph maxhold indicates the maximum measured value of the signal.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_BARGRAPH_MAXHOLD
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReceiverBargraphMaxhold(bool Maxhold_State)
        {
            int pInvokeResult = PInvoke.ConfigureReceiverBargraphMaxhold(this._handle, System.Convert.ToUInt16(Maxhold_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function resets the bargraph maxhold value.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S ESL
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_BARGRAPH_MAXHOLD_RESET
        /// 
        /// Remote-control command(s):
        /// DISPlay:BARGraph:PHOLd:RESet
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ResetReceiverBargraphMaxhold()
        {
            int pInvokeResult = PInvoke.ResetReceiverBargraphMaxhold(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the lower and upper level of the bargraph.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S ESL
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_MIN_BARGRAPH_LEVEL
        /// RSSPECAN_ATTR_RECEIVER_MAX_BARGRAPH_LEVEL
        /// 
        /// Remote-control command(s):
        /// DISPlay:BARGraph:LEVel:LOWer?
        /// DISPlay:BARGraph:LEVel:UPPer?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Lower_Level">
        /// Queries the lower level of the bargraph.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_MIN_BARGRAPH_LEVEL
        /// </param>
        /// <param name="Upper_Level">
        /// Queries the upper level of the bargraph.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_MAX_BARGRAPH_LEVEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryReceiverBargraphLevel(out double Lower_Level, out double Upper_Level)
        {
            int pInvokeResult = PInvoke.QueryReceiverBargraphLevel(this._handle, out Lower_Level, out Upper_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the number of ranges.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_SCAN_RANGES
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:RANGes:COUNt
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Number_Of_Ranges">
        /// This control determines the number of ranges.
        /// 
        /// Valid Values:
        /// 1 to 10
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_SCAN_RANGES
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureScanRanges(int Number_Of_Ranges)
        {
            int pInvokeResult = PInvoke.ConfigureScanRanges(this._handle, Number_Of_Ranges);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the start and stop frequency and the step size of the selected receiver scan range.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_SCAN_START
        /// RSSPECAN_ATTR_RECEIVER_SCAN_STOP
        /// RSSPECAN_ATTR_RECEIVER_SCAN_STEP_SIZE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:STARt
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:STOP
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:STEP
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Values:
        /// 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Frequency_Start">
        /// This control sets the start frequency of the selected receiver scan range.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 150.0e+3 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_SCAN_START
        /// </param>
        /// <param name="Frequency_Stop">
        /// This control sets the stop frequency of the selected receiver scan range.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 30.0e+6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_SCAN_STOP
        /// </param>
        /// <param name="Step_Size">
        /// This control sets the step size of the receiver frequency of the selected receiver scan range.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 4.0e+3 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_SCAN_STEP_SIZE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureScanTableStartStop(int Scan_Range, double Frequency_Start, double Frequency_Stop, double Step_Size)
        {
            int pInvokeResult = PInvoke.ConfigureScanTableStartStop(this._handle, Scan_Range, Frequency_Start, Frequency_Stop, Step_Size);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the bandwidth for the selected receiver scan range.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_SCAN_BANDWIDTH
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:BANDwidth:RESolution
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Values:
        /// 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Bandwidth">
        /// This control sets the bandwidth for the selected receiver scan range.
        /// 
        /// Valid Values: 
        /// 10.0 to 10.0e+6 Hz
        /// 
        /// Default Value: 9.0e+3 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_SCAN_BANDWIDTH
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureScanTableResBandwidth(int Scan_Range, double Bandwidth)
        {
            int pInvokeResult = PInvoke.ConfigureScanTableResBandwidth(this._handle, Scan_Range, Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the subscan measurement time of the selected receiver scan range.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_SCAN_MEASUREMENT_TIME
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:TIME
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Values:
        /// 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Measurement_Time">
        /// This control sets the subscan measurement time of the selected receiver scan range. 
        /// 
        /// Valid Values: 
        /// 0.0 to 100.0e s
        /// 
        /// Default Value: 0.001 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_SCAN_MEASUREMENT_TIME
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureScanTableMeasTime(int Scan_Range, double Measurement_Time)
        {
            int pInvokeResult = PInvoke.ConfigureScanTableMeasTime(this._handle, Scan_Range, Measurement_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the autoranging function and attenuation value in the selected receiver scan range.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_SCAN_AUTO_RANGING
        /// RSSPECAN_ATTR_RECEIVER_SCAN_RF_ATTENUATION
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:INPut:ATTenuation:AUTO
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:INPut:ATTenuation
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Values:
        /// 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Auto_Ranging">
        /// This control activates or deactivates the autoranging function in the selcted receiver scan range.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_SCAN_AUTO_RANGING
        /// </param>
        /// <param name="RF_Attenuation">
        /// This control sets the RF attenuation for the selected receiver scan range.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 10.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_SCAN_RF_ATTENUATION
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureScanTableAttenuation(int Scan_Range, bool Auto_Ranging, double RF_Attenuation)
        {
            int pInvokeResult = PInvoke.ConfigureScanTableAttenuation(this._handle, Scan_Range, System.Convert.ToUInt16(Auto_Ranging), RF_Attenuation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the preamplifier in the autoranging function and the preamplifier in the selected receiver scan range.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_SCAN_AUTO_PREAMPLIFIER
        /// RSSPECAN_ATTR_RECEIVER_SCAN_PREAMPLIFIER
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:INPut:GAIN:AUTO ON | OFF
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:INPut:GAIN[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Values:
        /// 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Auto_Preamplifier">
        /// This control includes the preamplifier in the autoranging function of the selected receiver scan range.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_SCAN_AUTO_PREAMPLIFIER
        /// </param>
        /// <param name="Preamplifier">
        /// This control activates or deactivates the preamplifier in the selected scan range.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_SCAN_PREAMPLIFIER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureScanTablePreamplifier(int Scan_Range, bool Auto_Preamplifier, bool Preamplifier)
        {
            int pInvokeResult = PInvoke.ConfigureScanTablePreamplifier(this._handle, Scan_Range, System.Convert.ToUInt16(Auto_Preamplifier), System.Convert.ToUInt16(Preamplifier));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the RF attenuation.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_ATTENUATION
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:ATTenuation
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Attenuation">
        /// Specifies the input attenuation (in positive dB). This means that if 10dB is selected, the result is a reduction in the signal level of 10 dB.
        /// 
        /// Valid Values:
        /// 0, 5, 10, 15, 20, 25 ... 50 dB
        /// 
        /// Default Value: 10.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ATTENUATION
        /// 
        /// Notes:
        /// 
        /// (1) To protect the input mixer against inadvertent overload, 0 dB can only be switched on when the 10 dB Min softkey is switched off.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReceiverRFAttenuation(int Window, double Attenuation)
        {
            int pInvokeResult = PInvoke.ConfigureReceiverRFAttenuation(this._handle, Window, Attenuation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on the preamplifier for the instrument. Switching on the preamplifier reduces the total mark figure of the R&amp;S ESL and thus improves the sensitivity. The signal level of the subsequent mixer is 20 dB higher so that the maximum input level is reduced by the gain of the preamplifier. The use of the preamplifier is recommended when measurements with a maximum sensitivity are to be performed. On the other hand, if the measurement should be performed at maximum dynamic range, the preamplifier should be switched off. The gain of the preamplifier is automatically considered in the level display
        /// 
        /// Note(s):
        /// 
        /// (1) Option B22 is required for R&amp;S ESL instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AMPL_PREAMPLIFIER
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:GAIN:STATe ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// Switches on the preamplifier for the instrument. The switchable gain is fixed to 20 dB.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AMPL_PREAMPLIFIER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReceiverPreamplifierState(int Window, bool State)
        {
            int pInvokeResult = PInvoke.ConfigureReceiverPreamplifierState(this._handle, Window, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines whether the 0 dB position of the attenuator is to be used in manual or automatic adjustment. 10 dB Min ON is the default value, i.e. an RF attenuation of at least 10 dB is always set on the R&amp;S ESL to protect the input mixer. An attenuation of 0 dB cannot be set manually either. This avoids 0 dB being switched on inadvertently, particularly when DUTs with high RFI voltage are measured.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S ESL.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_PROTECTION_STATE
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:ATTenuation:PROTection[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="_10_dB_Min">
        /// Defines whether the 0 dB position of the attenuator is to be used in manual or automatic adjustment.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_PROTECTION_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReceiver10dBMin(int Window, bool _10_dB_Min)
        {
            int pInvokeResult = PInvoke.ConfigureReceiver10dBMin(this._handle, Window, System.Convert.ToUInt16(_10_dB_Min));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the autorange function on and off.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_ATTENUATION_AUTO
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:ATTenuation:AUTO ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Auto_Range">
        /// Switches the autorange function on and off.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ATTENUATION_AUTO
        /// 
        /// Note(s):
        /// 
        /// (1) On - The attenuation is automatically set so that a good S/N ratio is obtained without the receiver stages being overdriven.
        ///  
        /// (2) Off - The attenuation is set manually.
        ///  
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReceiverAutoRange(int Window, bool Auto_Range)
        {
            int pInvokeResult = PInvoke.ConfigureReceiverAutoRange(this._handle, Window, System.Convert.ToUInt16(Auto_Range));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the auto preamp function and or off.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for R&amp;S ESL.
        /// 
        /// (2) This function is available only when Auto Ranging is switched on (function rsspecan_ConfigureReceiverAutoRange).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_AUTO_PREAMPLIFIER
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:GAIN:AUTO ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// Switches the auto preamp function and or off. The driver uses this value to set the RSSPECAN_ATTR_RECEIVER_AUTO_PREAMPLIFIER attribute.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RECEIVER_AUTO_PREAMPLIFIER
        /// 
        /// Note(s):
        /// 
        /// (1) On - The preamplifier is considered in the autorange procedure. The preamplifier is cut in when the RF attenuation is reduced to the minimum settable value. 
        ///  
        /// (2) Off - The preamplifier is not considered in the autorange procedure.
        ///  
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureReceiverAutoPreamplifier(int Window, bool State)
        {
            int pInvokeResult = PInvoke.ConfigureReceiverAutoPreamplifier(this._handle, Window, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function runs the receiver measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) The measurement can be started only in second window (Window B). When setting up single/continuous sweep measurement with function rsspecan_ConfigureAcquisition, set Window parameter to B.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_INIT_WITHOUT_OPC
        /// 
        /// Remote-control command(s):
        /// INITiate2[:IMMediate]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int RunReceiverMeasurement()
        {
            int pInvokeResult = PInvoke.RunReceiverMeasurement(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function is possible to use only as a query for single measurements in the receiver mode. The values of all activated detectors are transferred. For inactive detectors, the value is omitted at the position(s) in question.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FORMat[:DATA] ASCii
        /// TRACe&lt;1|2&gt;[:DATA]? SINGle
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="First_Detector">
        /// Returns the value of first activated detector. Inactive detector returns zero.
        /// 
        /// </param>
        /// <param name="Second_Detector">
        /// Returns the value of second activated detector. Inactive detector returns zero.
        /// 
        /// </param>
        /// <param name="Third_Detector">
        /// Returns the value of third activated detector. Inactive detector returns zero.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadReceiverDetector(uint Timeout, out double First_Detector, out double Second_Detector, out double Third_Detector)
        {
            int pInvokeResult = PInvoke.ReadReceiverDetector(this._handle, Timeout, out First_Detector, out Second_Detector, out Third_Detector);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function is possible to use only as a query for single measurements in the receiver mode. The values of all activated detectors are transferred. For inactive detectors, the value is omitted at the position(s) in question.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FORMat[:DATA] ASCii
        /// TRACe&lt;1|2&gt;[:DATA]? SINGle
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="First_Detector">
        /// Returns the value of first activated detector. Inactive detector returns zero.
        /// 
        /// </param>
        /// <param name="Second_Detector">
        /// Returns the value of second activated detector. Inactive detector returns zero.
        /// 
        /// </param>
        /// <param name="Third_Detector">
        /// Returns the value of third activated detector. Inactive detector returns zero.
        /// 
        /// </param>
        /// <param name="Fourth_Detector">
        /// Returns the value of fourth activated detector. Inactive detector returns zero.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadReceiverDetectorExt(uint Timeout, out double First_Detector, out double Second_Detector, out double Third_Detector, out double Fourth_Detector)
        {
            int pInvokeResult = PInvoke.ReadReceiverDetectorExt(this._handle, Timeout, out First_Detector, out Second_Detector, out Third_Detector, out Fourth_Detector);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the result of the scan measurement. The number of transmitted measurement results depends on the scan settings.
        /// 
        /// Note(s):
        /// 
        /// (1) For a successful execution the Trace Feed Control has to be set to Always (set with function rsspecan_ConfigureReceiverTraceFeedControl to value  RSSPECAN_VAL_TRAC_FEED_ALWAYS), otherwise the read will fail.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FORMat[:DATA] REAL,32
        /// TRACe&lt;1|2&gt;[:DATA]? SCAN
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Array_Size">
        /// Pass the number of elements in the arrays you specify as the array output parameters.
        /// 
        /// If the data returned contains more elements that you indicate in this parameter, this function copies 'Array Size' elements into the output parameters.
        /// 
        /// </param>
        /// <param name="Trace_Status">
        /// Returns the trace status information (4 byte). Structure of data is:
        /// 
        /// - Bit 0 to 9: subscan
        /// - Bit 10: last block of subscan
        /// - Bit 11: last block of last subscan of scan
        /// - Bit 12: last of all blocks (for multiple scans after the
        ///           last scan)
        /// 
        /// </param>
        /// <param name="Number_of_Results">
        /// Returns the number of the transmitted measurement results of a trace.
        /// 
        /// </param>
        /// <param name="Status_Data">
        /// Returns the status information per measurement result during scan measurements.
        /// 
        /// One byte status information per measurement result is transmitted:
        /// 
        ///   bit 2: overrange occured
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least the number of elements defined with 'Array Size' parameter.
        /// 
        /// </param>
        /// <param name="Results_of_Trace_1">
        /// Returns an array of measurement results  of trace 1 if trace 1 is active.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least the number of elements defined with 'Array Size' parameter.
        /// 
        /// </param>
        /// <param name="Results_of_Trace_2">
        /// Returns an array of measurement results of trace 2 if trace 2 is active.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least the number of elements defined with 'Array Size' parameter.
        /// 
        /// </param>
        /// <param name="Results_of_Trace_3">
        /// Returns an array of measurement results  of trace 3 if trace 3 is active.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least the number of elements defined with 'Array Size' parameter.
        /// 
        /// </param>
        /// <param name="Results_of_Trace_4">
        /// Returns an array of measurement results  of trace 4 if trace 4 is active.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least the number of elements defined with 'Array Size' parameter.
        /// 
        /// </param>
        /// <param name="Results_of_Trace_5">
        /// Returns an array of measurement results of trace 5 if trace 5 is active.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least the number of elements defined with 'Array Size' parameter.
        /// 
        /// </param>
        /// <param name="Results_of_Trace_6">
        /// Returns an array of measurement results  of trace 6 if trace 6 is active.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least the number of elements defined with 'Array Size' parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadScanMeasurement(int Array_Size, out int Trace_Status, out int Number_of_Results, System.Text.StringBuilder Status_Data, double[] Results_of_Trace_1, double[] Results_of_Trace_2, double[] Results_of_Trace_3, double[] Results_of_Trace_4, double[] Results_of_Trace_5, double[] Results_of_Trace_6)
        {
            int pInvokeResult = PInvoke.ReadScanMeasurement(this._handle, Array_Size, out Trace_Status, out Number_of_Results, Status_Data, Results_of_Trace_1, Results_of_Trace_2, Results_of_Trace_3, Results_of_Trace_4, Results_of_Trace_5, Results_of_Trace_6);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the status information per measurement result during scan measurements.
        /// 
        /// Note(s):
        /// 
        /// (1) For a successful execution the Trace Feed Control has to be set to Always (set with function rsspecan_ConfigureReceiverTraceFeedControl to value  RSSPECAN_VAL_TRAC_FEED_ALWAYS), otherwise the read will fail.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FORMat[:DATA] REAL,32
        /// TRACe&lt;1|2&gt;[:DATA]? STATus
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Status_Data">
        /// Returns the status information per measurement result during scan measurements.
        /// 
        /// One byte status information per measurement result is transmitted:
        /// 
        ///   bit 2: overrange trace 1 to trace 6
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadScanMeasurementStatus(System.Text.StringBuilder Status_Data)
        {
            int pInvokeResult = PInvoke.ReadScanMeasurementStatus(this._handle, Status_Data);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the values of the maximum peak of all active detectors from successive measurements.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;[:DATA]? PHOLd
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Array_Size">
        /// Pass the number of elements in the arrays you specify as the array output parameters.
        /// 
        /// If the data returned contains more elements that you indicate in this parameter, this function copies 'Array Size' elements into the output parameters.
        /// 
        /// </param>
        /// <param name="Number_of_Results">
        /// Returns the number of the transmitted measurement results of a trace.
        /// 
        /// </param>
        /// <param name="Peak_Level">
        /// Returns the levels of the maximum peak of all active detectors from successive measurements.
        /// 
        /// 
        /// </param>
        /// <param name="Peak_Frequency">
        /// Returns the frequencies of the maximum peak of all active detectors from successive measurements.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadPeakResults(uint Timeout, int Array_Size, out int Number_of_Results, double[] Peak_Level, double[] Peak_Frequency)
        {
            int pInvokeResult = PInvoke.ReadPeakResults(this._handle, Timeout, Array_Size, out Number_of_Results, Peak_Level, Peak_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the Phase Noise mode.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_PHASE_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument[:SELect] PNOise
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SSAPhaseMode()
        {
            int pInvokeResult = PInvoke.SSAPhaseMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the measurements.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_MEAS_CONF
        /// 
        /// Remote-control command(s):
        /// CONFigure:TSETup INT1 | INT2 | INT3 | EXT1 | EXT2 | DUT1 | DCON1 | DCON2 | RPN1 | RPN2
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Test_Setup">
        /// This control configures the measurements.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_SSA_MEAS_CONF_INT1  (0) - PLL Mode / INT1
        /// RSSPECAN_VAL_SSA_MEAS_CONF_INT2  (1) - PLL Mode / INT2
        /// RSSPECAN_VAL_SSA_MEAS_CONF_INT3  (2) - PLL Mode / INT3
        /// RSSPECAN_VAL_SSA_MEAS_CONF_EXT1  (3) - PLL Mode / EXT1
        /// RSSPECAN_VAL_SSA_MEAS_CONF_EXT2  (4) - PLL Mode / EXT2
        /// RSSPECAN_VAL_SSA_MEAS_CONF_DUT1  (5) - PLL Mode / DUT1
        /// RSSPECAN_VAL_SSA_MEAS_CONF_DCON1 (6) - Down Converter 1
        /// RSSPECAN_VAL_SSA_MEAS_CONF_DCON2 (7) - Down Converter 2
        /// RSSPECAN_VAL_SSA_MEAS_CONF_RPN1  (8) - Residual Phase Noise 1
        /// RSSPECAN_VAL_SSA_MEAS_CONF_RPN2  (9) - Residual Phase Noise 2
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_MEAS_CONF_INT1  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_MEAS_CONF
        /// 
        /// Notes:
        /// 
        /// (1) If the external mixer is switched on in the spectrum mode, only Down Converter 1 will be available. If the external mixer is off, all configurations are selectable except Down Converter 1.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SSAMeasurementConfiguration(int Test_Setup)
        {
            int pInvokeResult = PInvoke.SSAMeasurementConfiguration(this._handle, Test_Setup);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase Signal Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FREQUENCY_CENTER
        /// RSSPECAN_ATTR_SSA_EXPECTED_POWER
        /// RSSPECAN_ATTR_SSA_SWEEP_FORWARD
        /// RSSPECAN_ATTR_SSA_VERIFY_FREQ
        /// RSSPECAN_ATTR_SSA_FREQ_TOL
        /// RSSPECAN_ATTR_SSA_LEVEL_TOL
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer
        /// [SENSe&lt;1|2&gt;:]POWer:RLEVel
        /// [SENSe&lt;1|2&gt;:]SWEep:FORWard ON | OFF
        /// [SENSe&lt;1|2&gt;:]FREQuency:VERify[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]FREQuency:VERify:TOLerance
        /// [SENSe&lt;1|2&gt;:]POWer:RLEVel:VERify:TOLerance
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Frequency">
        /// Specifies the Center Frequency of the signal to be measured. 
        /// 
        /// Units: Hz
        /// 
        /// Valid Range:
        /// not checked
        /// 
        /// Default Value:  100.e+6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_CENTER
        /// </param>
        /// <param name="Level">
        /// Specifies the expected level of the RF input signal.
        /// 
        /// Units: dB
        /// 
        /// Valid Range:
        /// -100.0 to 30.00
        /// 
        /// Default Value: -20.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_EXPECTED_POWER
        /// </param>
        /// <param name="Sweep_Forward">
        /// Selects the sweep direction for the current measurement.
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_SWEEP_FORWARD
        /// 
        /// Notes:
        /// 
        /// (1) If this attribute is set to ON then the tolerance that is set with the attribute Frequency Tolerance is specified in Hz. 
        /// 
        /// (2) If this attribute is set to OFF then the tolerance that is set with the attribute Frequency Tolerance is specified in %.
        /// </param>
        /// <param name="Verify_Frequency___Level">
        /// Enables a search across a frequency tolerance range, for the carrier of greatest magnitude. Carrier frequency and level are measured. If the level is within a level tolerance range, the measured level overrides the specified Level. Otherwise the measurement is aborted. 
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_VERIFY_FREQ
        /// 
        /// Notes:
        /// 
        /// (1) This should be used when the carrier frequency is not known
        /// precisely.
        /// 
        /// (2) Both attributes use the same value for enabling or disabling.
        /// 
        /// (3) When this control is set to OFF, Frequency Tolerance and Level Tolerance values are ignored.
        /// </param>
        /// <param name="Frequency_Tolerance">
        /// Used to verify the input signal frequency.
        /// 
        /// Units: 
        /// Hz - sweep direction is forward
        /// %  - sweep direction is reverse
        /// 
        /// Valid Range:
        /// 1.0 to 1.0e+6 Hz (sweep direction is forward)
        /// 1 to 100 %       (sweep direction is reverse)
        /// 
        /// Default Value: 1.0e+3 Hz or 1 %
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_FREQ_TOL
        /// 
        /// Notes:
        /// 
        /// (1) If the sweep direction is forward (control Sweep Forward set to ON), the Frequency Tolerance (absolute) parameter is used, which is the range, either side of the Signal Frequency, within which the carrier is known to be. In forward sweep there will be a frequency and level check once at the beginning of the measurement.
        /// 
        /// (2) If the sweep direction is reverse (control Sweep Forward set to OFF), the Frequency Tolerance (relative) parameter is used, which is the ratio of the subspan?s start frequency. In reverse sweep there will be a frequency and level check before each subsweep.
        /// 
        /// (3) The units are Hz if the sweep direction is forward (control Sweep Forward set to ON) and % if the sweep direction is set to reverse (control Sweep Forward set to OFF).
        /// </param>
        /// <param name="Level_Tolerance">
        /// Used to verify the Level of the input signal. 
        /// 
        /// Units: dB
        /// 
        /// Valid Range:
        /// 3.0 to 20.0
        /// 
        /// Default Value: 10.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_LEVEL_TOL
        /// 
        /// Notes:
        /// 
        /// (1) Specifies the maximum and minimum deviation from the specified Level setting that the input signal may vary by and still pass the verification i.e. measured level between (Level+Level_Tolerance) and (Level-Level_Tolerance) is accepted.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhaseSignalSettings(double Frequency, double Level, bool Sweep_Forward, bool Verify_Frequency___Level, double Frequency_Tolerance, double Level_Tolerance)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhaseSignalSettings(this._handle, Frequency, Level, System.Convert.ToUInt16(Sweep_Forward), System.Convert.ToUInt16(Verify_Frequency___Level), Frequency_Tolerance, Level_Tolerance);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase frequency multiplier.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_FREQ_MULTI
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:MULTiplier
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Frequency_Multiplier">
        /// Specifies a multiplier that is applied to all entered and displayed frequencies. By using this multiplier, the R&amp;S FSUP corrects the measured frequencies to take an external multiplier into account. 
        /// 
        /// Valid Range:
        /// 1 to 1024
        /// 
        /// Default Value:  1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_FREQ_MULTI
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhaseFrequencyMultiplier(int Frequency_Multiplier)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhaseFrequencyMultiplier(this._handle, Frequency_Multiplier);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase AFC Track Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_TRACK_FREQ
        /// RSSPECAN_ATTR_SSA_TRACK_LEVEL
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:TRACk ON | OFF
        /// [SENSe&lt;1|2&gt;:]POWer:TRACk ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Track_Frequency">
        /// Selects whether the center frequency should be tracked and adjusted whilst a measurement is running.
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_TRACK_FREQ
        /// 
        /// Notes:
        /// 
        /// (1) Track Frequency is only available if the Verify Frequency &amp;
        /// Level parameter is switched on.
        /// </param>
        /// <param name="Track_Level">
        /// Selects whether the level should be tracked and adjusted whilst a measurement is running.
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_TRACK_LEVEL
        /// 
        /// Notes:
        /// 
        /// (1) Track Level is only available if the Verify Frequency &amp; Level parameter is switched on.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhaseAFCTrackSettings(bool Track_Frequency, bool Track_Level)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhaseAFCTrackSettings(this._handle, System.Convert.ToUInt16(Track_Frequency), System.Convert.ToUInt16(Track_Level));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase AFC Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) This function is available only to Phase Noise PLL, Cross Correlation PLL and Phase Noise Analyzer measurements.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_AFC_STATE
        /// RSSPECAN_ATTR_SSA_AFC_TARGET_FREQ
        /// RSSPECAN_ATTR_SSA_AFC_TOL
        /// RSSPECAN_ATTR_SSA_AFC_ITER
        /// 
        /// Remote-control command(s):
        /// SOURce:VOLTage:CONTrol&lt;1|2&gt;:AFC[:STATe] ON | OFF
        /// SOURce:VOLTage:CONTrol&lt;1|2&gt;:AFC:TARGet
        /// SOURce:VOLTage:CONTrol&lt;1|2&gt;:AFC:TOLerance
        /// SOURce:VOLTage:CONTrol&lt;1|2&gt;:AFC:ITERations
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="AFC_State">
        /// Enables/disables the automatic frequency control (AFC).
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_AFC_STATE
        /// </param>
        /// <param name="Target_Frequency">
        /// Specifies the target frequency for automatic frequency control.
        /// 
        /// Valid Range:
        /// 20.0 Hz to 50.0e9 Hz
        /// 
        /// Default Value: 4.0e9 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_AFC_TARGET_FREQ
        /// 
        /// Note(s):
        /// 
        /// (1) If AFC State is stitched OFF, this control is ignored.
        /// </param>
        /// <param name="Tolerance">
        /// Specifies the tolerance on the target frequency for automatic frequency control, e.g. the maximum value by which the target frequency may be undercut or exceeded.
        /// 
        /// Valid Range:
        /// 1.0 Hz to 1.0e6 Hz
        /// 
        /// Default Value: 1.0e3 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_AFC_TOL
        /// 
        /// Note(s):
        /// 
        /// (1) If AFC State is stitched OFF, this control is ignored.
        /// </param>
        /// <param name="Iterations">
        /// Specifies the maximum number of iterations for automatic frequency control.
        /// 
        /// Valid Range:
        /// 1 to 99999
        /// 
        /// Default Value: 10
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_AFC_ITER
        /// 
        /// Note(s):
        /// 
        /// (1) If AFC State is stitched OFF, this control is ignored.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhaseAFCSettings(bool AFC_State, double Target_Frequency, double Tolerance, int Iterations)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhaseAFCSettings(this._handle, System.Convert.ToUInt16(AFC_State), Target_Frequency, Tolerance, Iterations);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase Display Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FREQUENCY_START
        /// RSSPECAN_ATTR_FREQUENCY_STOP
        /// RSSPECAN_ATTR_SSA_AUTO_SCALING
        /// RSSPECAN_ATTR_SSA_Y_AXIS_TOP
        /// RSSPECAN_ATTR_SSA_Y_AXIS_RANGE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:STARt
        /// [SENSe&lt;1|2&gt;:]FREQuency:STOP
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...8&gt;:Y[:SCALe]:AUTO ON | OFF
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...8&gt;:Y[:SCALe]:RLEVel
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...8&gt;:Y[:SCALe]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="X_Axis_Start">
        /// Specifies the min frequency for the X axis.
        /// 
        /// Units: Hz
        /// 
        /// Valid Range:
        /// not checked
        /// 
        /// Default Value: 1.0e+3 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_START
        /// 
        /// Note(s):
        /// 
        /// (1) Only discrete values in 1,3 steps are possible.
        /// </param>
        /// <param name="X_Axis_Stop">
        /// Specifies the max frequency for the X axis.
        /// 
        /// Units: Hz
        /// 
        /// Valid Range:
        /// not checked
        /// 
        /// Default Value: 1.0e+6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_STOP
        /// 
        /// Note(s):
        /// 
        /// (1) Only discrete values in 1,3 steps are possible.
        /// </param>
        /// <param name="Autoscale_Once">
        /// Switches the calculation from results of Y Axis scaling. 
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// VI_FALSE  (0) - Off
        /// VI_TRUE   (1) - Once
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_AUTO_SCALING
        /// 
        /// Notes:
        /// 
        /// (1) The autoscaling will only be done once in the first sweep. The subsequence sweeps will not auto scale the y-axis.
        /// 
        /// (2) When Autoscale Once is ON, Y Axis Top and Range parameters will be ignored. When it is OFF, Y Axis Top and Range parameters will be used.
        /// </param>
        /// <param name="Y_Axis_Top">
        /// Specifies the max Phase Noise level in y-axis for the trace results.
        /// 
        /// Units: dBc/Hz
        /// 
        /// Valid Range:
        /// -200.0 to 50.0
        /// 
        /// Default Value: -20.0 dBc/Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_Y_AXIS_TOP
        /// 
        /// Notes:
        /// 
        /// (1) When Autoscale Once is ON, Y Axis Top and Range parameters will be ignored.
        /// </param>
        /// <param name="Y_Axis_Range">
        /// Specifies the distance from the top to the origin in y-axis.
        /// 
        /// Units: dB
        /// 
        /// Valid Range:
        /// 10.0 to 200.0
        /// 
        /// Default Value: 100.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_Y_AXIS_RANGE
        /// 
        /// Notes:
        /// 
        /// (1) When Autoscale Once is ON, Y Axis Top and Range parameters will be ignored.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhaseDisplaySettings(double X_Axis_Start, double X_Axis_Stop, bool Autoscale_Once, double Y_Axis_Top, double Y_Axis_Range)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhaseDisplaySettings(this._handle, X_Axis_Start, X_Axis_Stop, System.Convert.ToUInt16(Autoscale_Once), Y_Axis_Top, Y_Axis_Range);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase Trace Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_TRACE_OFFSET
        /// RSSPECAN_ATTR_SSA_TRACE_SMOOTHING_APER
        /// RSSPECAN_ATTR_SSA_SMOOTHING_TYPE
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...8&gt;:Y[:SCALe]:RLEVel:OFFSet
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...8&gt;:SMOothing:APERture
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...8&gt;:SMOothing:TYPE LINear | LOGarithmic | MEDian
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Trace_Offset">
        /// Specifies the arithmetical reference level offset which is
        /// added to the y axis labelling. 
        /// 
        /// Units: dB
        /// 
        /// Valid Range:
        /// -200.0 to 200.0
        /// 
        /// Default Value: 0.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_TRACE_OFFSET
        /// </param>
        /// <param name="Trace_Smoothing">
        /// Specifies the % of the display width to be used as a window when a trace is smoothed. 
        /// 
        /// Units: %
        /// 
        /// Valid Range:
        /// 1 to 20
        /// 
        /// Default Value: 1 %
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_TRACE_SMOOTHING_APER
        /// 
        /// Notes:
        /// 
        /// (1) The larger the setting of the Smoothing parameter the greater the effect of smoothing.
        /// </param>
        /// <param name="Smoothing_Type">
        /// Select the smoothing type for the trace. 
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_SSA_SMOOTH_TYPE_LIN (0) - Linear
        /// RSSPECAN_VAL_SSA_SMOOTH_TYPE_LOG (1) - Logarithmic
        /// RSSPECAN_VAL_SSA_SMOOTH_TYPE_MED (2) - Median
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_SMOOTH_TYPE_LIN (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_SMOOTHING_TYPE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhaseTraceSettings(double Trace_Offset, int Trace_Smoothing, int Smoothing_Type)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhaseTraceSettings(this._handle, Trace_Offset, Trace_Smoothing, Smoothing_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase Residual Calculations.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_CALC_EVAL
        /// RSSPECAN_ATTR_SSA_CALC_EVAL_START
        /// RSSPECAN_ATTR_SSA_CALC_EVAL_STOP
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:EVALuation[:STATe] ON | OFF
        /// CALCulate&lt;1|2&gt;:EVALuation:STARt
        /// CALCulate&lt;1|2&gt;:EVALuation:STOP
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Use_Measurement_Settings">
        /// Specifies whether to use the whole measurement range or the user defined evaluation range for the residual calculations.
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_CALC_EVAL
        /// 
        /// Notes:
        /// 
        /// (1) If the Use Measurement Settings is switched on then  Evaluation From and Evaluation To are ignored and residual calculations are performed across the complete range of the measurement results.
        /// 
        /// (2) If the Use Measurement Settings is switched off then  Evaluate From and Evaluate To values are used and allow the range over which residual calculations are performed to be specified.
        /// </param>
        /// <param name="Evaluate_From">
        /// Specifies the start of the measurement range for which residual calculations are to be performed.
        /// 
        /// Units: Hz
        /// 
        /// Valid Range:
        /// not checked
        /// 
        /// Default Value: 1.0e+3 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_CALC_EVAL_START
        /// 
        /// Notes:
        /// 
        /// (1) The minimum value that can be specified for the Evaluate From setting is the value of the X Axis Start setting.
        /// 
        /// (2) The maximum value that can be specified for the Evaluate From setting is the value of the X Axis Stop setting.
        /// 
        /// (3) When the X Axis Start or X Axis Stop settings are modified
        /// the Evaluate From will be automatically adjusted to ensure that it is not outside the measurement range.
        /// 
        /// (4) The Evaluate From setting cannot be set higher than the Evaluate To settings.
        /// </param>
        /// <param name="Evaluate_To">
        /// Specifies the end of the measurement range for which residual calculations are to be performed.
        /// 
        /// Units: Hz
        /// 
        /// Valid Range:
        /// not checked
        /// 
        /// Default Value: 1.0e+6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_CALC_EVAL_STOP
        /// 
        /// Notes:
        /// 
        /// (1) The minimum value that can be specified for the Evauate To setting is the value of the X Axis Start setting.
        /// 
        /// (2) The maximum value that can be specified for the Evauate To setting is the value of the X Axis Stop setting.
        /// 
        /// (3) When the X Axis Start or X Axis Stop settings are modified the Evauate From will be automatically adjusted to ensure that it is not outside the measurement range.
        /// 
        /// (4) The Evauate To setting cannot be set lower than the Evauate from settings.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhaseResidualCalculations(bool Use_Measurement_Settings, double Evaluate_From, double Evaluate_To)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhaseResidualCalculations(this._handle, System.Convert.ToUInt16(Use_Measurement_Settings), Evaluate_From, Evaluate_To);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase Level Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) If the RF preamplifier (B23) or electronic attenuator (B25) option is not installed on the instrument, then control Preamplifier is not used.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_PREAMP_AND_ATTEN_AUTO
        /// RSSPECAN_ATTR_AMPL_PREAMPLIFIER
        /// RSSPECAN_ATTR_ATTENUATION
        /// 
        /// Remote-control command(s):
        /// INPut:GAIN:AUTO ON | OFF
        /// INPut:GAIN:STATe ON | OFF
        /// INPut:ATTenuation 0 to 75dB
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Preamplifier___Atten_Auto">
        /// Switches preamplifier and the RF attenuator handling into automatic mode. The required preamplifier and attenuation values are set automatically.
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_PREAMP_AND_ATTEN_AUTO
        /// </param>
        /// <param name="Preamplifier">
        /// Switches on the preamplifier for the instrument. The switchable gain is fixed to 20 dB. This control is only available if the automatic setting is switched off (Preamplifier &amp; Atten Auto).
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AMPL_PREAMPLIFIER
        /// 
        /// Note(s):
        /// 
        /// (1) The control is only available with the RF preamplifier (B23) or electronic attenuator (B25) option.
        /// </param>
        /// <param name="Attenuation">
        /// Sthe input attenuator. To protect the input mixer against damage from overloads, the setting 0 dB can be obtained by entering numerals, not by using the command DEC. The step width is 5 dB, the range is 0 dB to 75 dB. This control is only available if the automatic setting is switched off (Preamplifier &amp; Atten Auto).
        /// 
        /// Units: dB
        /// 
        /// Valid Range:
        /// 0.0 to 75.0 dB
        /// 
        /// Default Value: 40.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ATTENUATION
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhaseLevelSettings(bool Preamplifier___Atten_Auto, bool Preamplifier, double Attenuation)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhaseLevelSettings(this._handle, System.Convert.ToUInt16(Preamplifier___Atten_Auto), System.Convert.ToUInt16(Preamplifier), Attenuation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase Baseband Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) The baseband settings are available for Baseband Noise, Residual Phase Noise and AM Noise measurements.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AMPL_RF_COUPLING
        /// RSSPECAN_ATTR_SSA_INPUT_IMP
        /// 
        /// Remote-control command(s):
        /// INPut:COUPling AC | DC 
        /// INPut:IMPedance
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Coupling">
        /// Set the coupling to either AC or DC if the impedance is set to 400 Ohms. If the impedance is 50 Ohms, the coupling is automatically set to DC.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_DC (0) - DC
        /// RSSPECAN_VAL_AC (1) - AC
        /// 
        /// Default Value: RSSPECAN_VAL_DC (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AMPL_RF_COUPLING
        /// </param>
        /// <param name="Impedance">
        /// Specifies the input impedance.
        /// 
        /// Valid Values:
        /// 50, 400 Ohms
        /// 
        /// Default Value: 50 Ohms
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_INPUT_IMP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhaseBasebandSettings(int Coupling, double Impedance)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhaseBasebandSettings(this._handle, Coupling, Impedance);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase Spot Noise Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_CALC_SNO_STATE
        /// RSSPECAN_ATTR_SSA_CALC_SNO_X
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:SNOise&lt;1...5&gt;:STATe ON | OFF
        /// CALCulate&lt;1|2&gt;:SNOise&lt;1...5&gt;:X
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Enable">
        /// Allows spot noise calculations to be switched on and off.
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_CALC_SNO_STATE
        /// 
        /// Notes:
        /// 
        /// (1) If this parameter is turned off, the offset frequencies 1 to 5 are ignored.
        /// </param>
        /// <param name="Offset_Frequency_1">
        /// Allows the first frequency point to be specified at which spot noise calculations will be performed. 
        /// 
        /// Units: Hz
        /// 
        /// Valid Range:
        /// 1.0 to 1.0e+9
        /// 
        /// Default Value: 1.0e+3
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_CALC_SNO_X
        /// 
        /// Notes:
        /// 
        /// (1) If an offset frequency is specified which is outside the measurement frequency range then no results will be displayed for that offset frequency.
        /// 
        /// (2) If the parameter Enable is set to Off, the offset frequencies 1 to 5 are ignored.
        /// </param>
        /// <param name="Offset_Frequency_2">
        /// Allows the second frequency point to be specified at which spot noise calculations will be performed.
        /// 
        /// Units: Hz
        /// 
        /// Valid Range:
        /// 1.0 to 1.0e+9
        /// 
        /// Default Value: 10.0e+3
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_CALC_SNO_X
        /// 
        /// Notes:
        /// 
        /// (1) If an offset frequency is specified which is outside the measurement frequency range then no results will be displayed for that offset frequency.
        /// 
        /// (2) If the parameter Enable is set to Off, the offset frequencies 1 to 5 are ignored.
        /// </param>
        /// <param name="Offset_Frequency_3">
        /// Allows the third frequency point to be specified at which spot noise calculations will be performed. 
        /// 
        /// Units: Hz
        /// 
        /// Valid Range:
        /// 1.0 to 1.0e+9
        /// 
        /// Default Value: 100.0e+3
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_CALC_SNO_X
        /// 
        /// Notes:
        /// 
        /// (1) If an offset frequency is specified which is outside the measurement frequency range then no results will be displayed for that offset frequency.
        /// 
        /// (2) If the parameter Enable is set to Off, the offset frequencies 1 to 5 are ignored.
        /// </param>
        /// <param name="Offset_Frequency_4">
        /// Allows the forth frequency point to be specified at which spot noise calculations will be performed.
        /// 
        /// Units: Hz
        /// 
        /// Valid Range:
        /// 1.0 to 1.0e+9
        /// 
        /// Default Value: 1.0e+6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_CALC_SNO_X
        /// 
        /// Notes:
        /// 
        /// (1) If an offset frequency is specified which is outside the measurement frequency range then no results will be displayed for that offset frequency.
        /// 
        /// (2) If the parameter Enable is set to Off, the offset frequencies 1 to 5 are ignored.
        /// </param>
        /// <param name="Offset_Frequency_5">
        /// Allows the forth frequency point to be specified at which spot noise calculations will be performed.
        /// 
        /// Units: Hz
        /// 
        /// Valid Range:
        /// 1.0 to 1.0e+9
        /// 
        /// Default Value: 10.0e+6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_CALC_SNO_X
        /// 
        /// Notes:
        /// 
        /// (1) If an offset frequency is specified which is outside the measurement frequency range then no results will be displayed for that offset frequency.
        /// 
        /// (2) If the parameter Enable is set to Off, the offset frequencies 1 to 5 are ignored.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhaseSpotNoiseSettings(bool Enable, double Offset_Frequency_1, double Offset_Frequency_2, double Offset_Frequency_3, double Offset_Frequency_4, double Offset_Frequency_5)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhaseSpotNoiseSettings(this._handle, System.Convert.ToUInt16(Enable), Offset_Frequency_1, Offset_Frequency_2, Offset_Frequency_3, Offset_Frequency_4, Offset_Frequency_5);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase Internal Generator Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) The  Internal Generator settings are only available for Residual Noise measurements in conjunction with an external phase detector. The upper frequency limit for the internal generator is 8 GHz.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_INT_GEN_STATE
        /// RSSPECAN_ATTR_SSA_INT_GEN_FREQ
        /// RSSPECAN_ATTR_SSA_INT_GEN_OUTP_LEVEL
        /// 
        /// Remote-control command(s):
        /// SOURce:GENerator[:STATe] ON | OFF
        /// SOURce:GENerator:FREQuency
        /// SOURce:GENerator:LEVel&lt;1|2&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Enable">
        /// Switches the internal generator on and off.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_INT_GEN_STATE
        /// </param>
        /// <param name="Frequency">
        /// Sets the frequency of the internal generator. 
        /// 
        /// Units: Hz
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 1.0e+9 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_INT_GEN_FREQ
        /// </param>
        /// <param name="Gen_Out_1_Level">
        /// Sets the power level of the generated signal at Output number one.
        /// 
        /// Valid Range:
        /// -15 to 3 dBm
        /// 
        /// Default Value: 0.0 dBm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_INT_GEN_OUTP_LEVEL
        /// </param>
        /// <param name="Gen_Out_2_Level">
        /// Sets the power level of the generated signal at Output number two.
        /// 
        /// Valid Range:
        /// -15 to 3 dBm
        /// 
        /// Default Value: 0.0 dBm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_INT_GEN_OUTP_LEVEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhaseInternalGeneratorSettings(bool Enable, double Frequency, double Gen_Out_1_Level, double Gen_Out_2_Level)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhaseInternalGeneratorSettings(this._handle, System.Convert.ToUInt16(Enable), Frequency, Gen_Out_1_Level, Gen_Out_2_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets all the phase spot noise markers off.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_CALC_SNO_AOFF
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:SNOise&lt;1...5&gt;:AOFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SSAPhaseDisableAllSpotNoiseMarkers()
        {
            int pInvokeResult = PInvoke.SSAPhaseDisableAllSpotNoiseMarkers(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on the Phase Detector scope display if the Phase Noise PLL, Cross Correlation PLL or Spot Noise Tuning measurement is selected. 
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_PHASE_DET_DISP
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...8&gt;:SCOPe[:STATe]  ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Phase_Detector_Display">
        /// This control switches on the Phase Detector scope display
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_PHASE_DET_DISP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhaseDetectorScope(bool Phase_Detector_Display)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhaseDetectorScope(this._handle, System.Convert.ToUInt16(Phase_Detector_Display));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the measurement type for noise measurements.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_PHASE_NOISE_MEAS
        /// 
        /// Remote-control command(s):
        /// CONFigure:PNOise:MEASurement SPECtrum | PLL | CCORrelation | SNTuning | RESidual | BASeband
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Phase_Noise">
        /// This control configures the measurement type for noise measurements and baseband measurements. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SSA_PHAS_NOIS_MEAS_SPEC (0) - Phase Noise Analyzer
        /// RSSPECAN_VAL_SSA_PHAS_NOIS_MEAS_PLL  (1) - Phase Noise PLL
        /// RSSPECAN_VAL_SSA_PHAS_NOIS_MEAS_CCOR (2) - Cross Correlation PLL
        /// RSSPECAN_VAL_SSA_PHAS_NOIS_MEAS_SNT  (3) - Spot Noise Tuning
        /// RSSPECAN_VAL_SSA_PHAS_NOIS_MEAS_RES  (4) - Residual Phase Noise
        /// RSSPECAN_VAL_SSA_PHAS_NOIS_MEAS_BAS  (5) - Phase Noise Baseband
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_PHAS_NOIS_MEAS_PLL (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_PHASE_NOISE_MEAS
        /// 
        /// Notes:
        /// 
        /// (1) Phase Noise Analyzer: Activates the Phase Noise Analyzer measurement.
        /// 
        /// (2) Phase Noise PLL: all test setups except PLL MODE / int 3. 
        /// 
        /// (3) Cross Correlation PLL: PLL MODE / int 3. This measurement is only available if the option R&amp;S FSUP-B60 is installed.
        /// 
        /// (4) Spot Noise Tuning: During a succession of n measurements, the tuning voltage is altered. 5 frequency points, Offset Freq 1/2/3/4/5 - are evaluated. The measurement result shows the developing of these frequency points as 5 curves with different spot noise offsets.
        /// 
        /// (5) Residual Phase Noise: Activates a residual phase noise
        /// measurement.
        /// 
        /// (6) Phase Noise Baseband: Activates a baseband noise
        /// measurement.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhaseNoiseMeasurement(int Phase_Noise)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhaseNoiseMeasurement(this._handle, Phase_Noise);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase Sweep Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_SWEEP_MODE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:MODE FAST | NORMal | AVERaged | MANual
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Sweep_Mode">
        /// This control specifies the general sweep mode for the measurement. For each sub-sweep, the sweep mode provides specific values for RBW, Average and FFT settings. If the sweep mode is set to MANual, the sub-sweep settings can be specified. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SWE_MODE_FAST (0) - Fast
        /// RSSPECAN_VAL_SWE_MODE_NORM (1) - Normal
        /// RSSPECAN_VAL_SWE_MODE_AVER (2) - Average
        /// RSSPECAN_VAL_SWE_MODE_MAN  (3) - Manual
        /// 
        /// Default Value: RSSPECAN_VAL_SWE_MODE_NORM (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_SWEEP_MODE
        /// 
        /// Note(s):
        /// 
        /// (1) Fast - Not averaged. The measurement runs very fast. As the average column is set to 1 for all sub-bands.
        /// 
        /// (2) Normal - Normal averaged. The measurement runs slower then the fast mode, but the sub-bands are averaged more.
        /// 
        /// (3) Averaged - Highly averaged. The measurement runs very slow with high average in each sub-band for more accurate results.
        /// 
        /// (4) Manual - The RBW, Average and FFT columns can be set by the user.
        /// 
        /// (5) In fast, normal and averaged modes, the table is not editable, it is information only.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhaseSweepSettings(int Sweep_Mode)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhaseSweepSettings(this._handle, Sweep_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the FFT window for Phase Noise PLL measurements. For Noise Analyzer measurements the FFT window is always FLATTOP.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_FFT_WINDOW
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:FFTWindow CHEBychev | GAUSsian | RECTangular | Blackman HARris
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="FFT_Window">
        /// This control specifies the FFT window for Phase Noise PLL measurements. For Noise Analyzer measurements the FFT window is always FLATTOP. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SSA_FFT_WINDOW_CHEB (0) - Chebychev
        /// RSSPECAN_VAL_SSA_FFT_WINDOW_GAUS (1) - Gaussian
        /// RSSPECAN_VAL_SSA_FFT_WINDOW_RECT (2) - Rectangular
        /// RSSPECAN_VAL_SSA_FFT_WINDOW_BHAR (3) - Blackman Harris
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_FFT_WINDOW_CHEB (0) 
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_FFT_WINDOW
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhaseFFTWindowpSettings(int FFT_Window)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhaseFFTWindowpSettings(this._handle, FFT_Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase Span Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FREQUENCY_START
        /// RSSPECAN_ATTR_FREQUENCY_STOP
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:STARt
        /// [SENSe&lt;1|2&gt;:]FREQuency:STOP
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Start_Offset">
        /// Defines the start frequency of the measurement.
        /// 
        /// Units: Hz
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 1.0e+3 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_START
        /// 
        /// Notes:
        /// 
        /// (1) When this parameter changes the X-axis Start parameter in
        /// General Settings (Noise) view gets updated accordingly.
        /// 
        /// (2) The Carrier Frequency Offset Table is highlighted to show
        /// the selected span for the noise measurement.
        /// 
        /// (3) Only discrete values in 1,3 steps are possible.
        /// </param>
        /// <param name="Stop_Offset">
        /// Defines the stop frequency of the measurement.
        /// 
        /// Units: Hz
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 1.0e+6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_STOP
        /// 
        /// Notes:
        /// 
        /// (1) When this parameter changes the X-axis Stop parameter in General Settings (Noise) view gets updated accordingly.
        /// 
        /// (2) The Carrier Frequency Offset Table is highlighted to show the selected span for the noise measurement.
        /// 
        /// (3) Only discrete values in 1,3 steps are possible.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhaseSpanSettings(double Start_Offset, double Stop_Offset)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhaseSpanSettings(this._handle, Start_Offset, Stop_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Carrier Frequency Offset Table.
        /// 
        /// Notes:
        /// 
        /// (1) The Carrier Frequency Offset Table can be used only when the SSA Phase Sweep Settings is set to Manual value.
        /// 
        /// (2) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_LIST_RANG_BAND
        /// RSSPECAN_ATTR_SSA_LIST_RANG_SWE_COUN
        /// RSSPECAN_ATTR_SSA_LIST_RANG_FILT_TYPE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:BWIDth[:RESolution] 
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:SWEep:COUNt
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:FILTer:TYPE NORMal | FFT
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Sub_band">
        /// Specifies the sub-band for the RBW, Average and FFT.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="RBW">
        /// Sets the resolution filter bandwidth used for each sub-band.  
        /// 
        /// Units: Hz
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 10.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_LIST_RANG_BAND
        /// 
        /// Notes:
        /// 
        /// (1) Enter values in steps of 1/3/10.
        /// </param>
        /// <param name="Average">
        /// Sets the number of sweeps to average over for each sub-band.
        /// 
        /// Units: None
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_LIST_RANG_SWE_COUN
        /// </param>
        /// <param name="FFT">
        /// Allows the user to choose whether to use the FFT Resolution Filter or the conventional filter for each decade.
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_BAND_TYPE_NORM (0) - Normal
        /// RSSPECAN_VAL_BAND_TYPE_FFT  (1) - FFT
        /// 
        /// Default Value: RSSPECAN_VAL_BAND_TYPE_FFT (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_LIST_RANG_FILT_TYPE
        /// 
        /// Notes:
        /// 
        /// (1) FFT is only available for RBW values between 1 Hz and 10 kHz.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhaseCarrierFrequencyOffsetTable(int Sub_band, double RBW, int Average, int FFT)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhaseCarrierFrequencyOffsetTable(this._handle, Sub_band, RBW, Average, FFT);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase Tuning Settings.
        /// 
        /// Notes:
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) This function is available only in Spot Noise Tuning.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_TUN_MEAS_POINTS
        /// RSSPECAN_ATTR_SSA_SETTL_TIME_DC_PORT
        /// RSSPECAN_ATTR_SSA_SETTL_TIME_TUN_PORT
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]VCO:TUNing:POINts
        /// [SENSe&lt;1|2&gt;:]VCO:POWer:DELay
        /// [SENSe&lt;1|2&gt;:]VCO:CONTrol:DELay
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement_Points">
        /// Sets the number of points to be measured for the Spot
        /// Noise Tuning measurements.
        /// 
        /// Units: None
        /// 
        /// Valid Range: 3 to 1000
        /// 
        /// Default Value: 10
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_TUN_MEAS_POINTS
        /// </param>
        /// <param name="Settle_Time_Vdc">
        /// Sets the settling time for DC port 1.
        /// 
        /// Units: second
        /// 
        /// Valid Range: 0 s to 10 s
        /// 
        /// Default Value: 0.1 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_SETTL_TIME_DC_PORT
        /// </param>
        /// <param name="Settle_Time_Vtune">
        /// Sets the settling time for Tuning Port 1.
        /// 
        /// Units: second
        /// 
        /// Valid Range: 0 s to 10 s
        /// 
        /// Default Value: 0.1 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_SETTL_TIME_TUN_PORT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhaseTuningSettings(int Measurement_Points, double Settle_Time_Vdc, double Settle_Time_Vtune)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhaseTuningSettings(this._handle, Measurement_Points, Settle_Time_Vdc, Settle_Time_Vtune);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase Preset Settings.
        /// 
        /// Notes:
        /// 
        /// (1) The Preset Settings can be used only when the Configure Sweep Settings is set to Manual value.
        /// 
        /// (2) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_BAND_RAT
        /// RSSPECAN_ATTR_SSA_SWEEP_COUNT
        /// RSSPECAN_ATTR_SSA_BAND_FFT
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:BWIDth[:RESolution]:RATio
        /// [SENSe&lt;1|2&gt;:]LIST:SWEep:COUNt
        /// [SENSe&lt;1|2&gt;:]LIST:BWIDth[:RESolution]:TYPE NORMal | FFT
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="RBW">
        /// The RBW parameter sets all the RBW settings in the Carrier Frequency Offset table to a proportion of the start frequency for each sub-band.
        /// 
        /// Units: %
        /// 
        /// Valid Range:
        /// 0.1 to 30.0
        /// 
        /// Default Value: 10.0 %
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_BAND_RAT
        /// 
        /// Notes:
        /// 
        /// (1) The resulting RBW is rounded to the nearest 1/3/10 steps.
        /// </param>
        /// <param name="Average">
        /// Sets all the Average settings in the Carrier Frequency Offset table to the specified value. 
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// 1 to 10000
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_SWEEP_COUNT
        /// </param>
        /// <param name="FFT">
        /// Allows the user to choose whether to use the FFT Resolution Filter or the conventional filter for each decade.
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_BAND_TYPE_NORM (0) - Normal
        /// RSSPECAN_VAL_BAND_TYPE_FFT  (1) - FFT
        /// 
        /// Default Value: RSSPECAN_VAL_BAND_TYPE_FFT (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_BAND_FFT
        /// 
        /// Notes:
        /// 
        /// (1) FFT is only available for RBW values between 1 Hz and 10 kHz.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhasePresetSettings(double RBW, int Average, int FFT)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhasePresetSettings(this._handle, RBW, Average, FFT);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies whether the number of cross correlations is optimized. If switched off, the number of cross correlations is defined via the [SENSe]:LIST:RANGe&lt;1...20&gt;:SWEep:COUNt
        /// command. If switched on, the cross correlation is not be pursued further if it does not improve the measurement result (in order to shorten the measurement time). The number of correlations is just used as a maximum value.
        /// 
        /// Notes:
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) This function is only available with the R&amp;S FSUP-B60 option (Cross Correlation PLL measurements)
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_XOPTIMIZE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:XOPTimize ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Optimize_Cross_Correlation">
        /// Specifies whether the number of cross correlations is optimized.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_XOPTIMIZE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhaseOptimizeCrossCorrelation(bool Optimize_Cross_Correlation)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhaseOptimizeCrossCorrelation(this._handle, System.Convert.ToUInt16(Optimize_Cross_Correlation));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase Tracking.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) This function is only available for Phase Noise PLL, Cross Correlation PLL and Spot Noise Tuning measurements.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_TRACKING
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]VCO:TRACking
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Tracking">
        /// Sets the value of the tracking parameter for internally tuned Phase Noise PLL and Cross Correlation PLL measurements.
        /// 
        /// Units: Hz/s
        /// 
        /// Valid Range:
        /// 1.0e-3 Hz/s to 1.0e+6 Hz/s
        /// 
        /// Default Value:  100.0e+3 Hz/s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_TRACKING
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhaseTracking(double Tracking)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhaseTracking(this._handle, Tracking);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the pulsed frequency settings. 
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) The function is only available for Phase Noise Analyzer, Phase Noise PLL and Phase Noise Cross Correlation measurements.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_PULSED_FREQ_MEAS_MODE
        /// RSSPECAN_ATTR_SSA_PULSED_DUTY_CYCLE
        /// RSSPECAN_ATTR_SSA_PULSED_REPET_FREQ
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:PULSed[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]FREQuency:PULSed:DCYCle
        /// [SENSe&lt;1|2&gt;:]FREQuency:PULSed:PREPetetion
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Pulsed_Frequency_Meas_Mode">
        /// This control activates or deactivates the pulsed frequency measurement mode during a measurement sweep.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_PULSED_FREQ_MEAS_MODE
        /// </param>
        /// <param name="Duty_Cycle">
        /// This control sets the duty cycle percentage of the connected pulsed signal.
        /// 
        /// Valid Values: 1 to 100 %
        /// 
        /// Default Value: 10.0 %
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_PULSED_DUTY_CYCLE
        /// </param>
        /// <param name="Repetition_Frequency">
        /// This control sets the pulse repetition frequency of a connected pulsed signal.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 1.0e3 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_PULSED_REPET_FREQ
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhasePulsedFrequencySettings(bool Pulsed_Frequency_Meas_Mode, double Duty_Cycle, double Repetition_Frequency)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhasePulsedFrequencySettings(this._handle, System.Convert.ToUInt16(Pulsed_Frequency_Meas_Mode), Duty_Cycle, Repetition_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the VCO Carrier Search settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_AUTO_CARRIER_SEARCH
        /// RSSPECAN_ATTR_SSA_CARRIER_SEARCH_FREQ_STAR
        /// RSSPECAN_ATTR_SSA_CARRIER_SEARCH_FREQ_STOP
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]VCO:CSEarch:AUTO ON | OFF
        /// [SENSe&lt;1|2&gt;:]VCO:CSEarch:STARt
        /// [SENSe&lt;1|2&gt;:]VCO:CSEarch:STOP
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Autosearch">
        /// Activates or deactivates the auto carrier search function. 
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_AUTO_CARRIER_SEARCH
        /// 
        /// Notes:
        /// 
        /// (1)  If activated, the value is set automatically, determined via a pre-measurement.
        /// 
        /// (2) If deactivated, the search range is defined using the [SENSe&lt;1|2&gt;:]VCO:CSEarch:SPAN command.
        /// </param>
        /// <param name="Start_Frequency">
        /// Sets the start frequency for VCO measurements.
        /// 
        /// Units: Hz
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 1.0 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_CARRIER_SEARCH_FREQ_STAR
        /// </param>
        /// <param name="Stop_Frequency">
        /// Sets the stop frequency for VCO measurements.
        /// 
        /// Units: Hz
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 40.0e+9 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_CARRIER_SEARCH_FREQ_STOP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAVCOCarrierSearch(bool Autosearch, double Start_Frequency, double Stop_Frequency)
        {
            int pInvokeResult = PInvoke.ConfigureSSAVCOCarrierSearch(this._handle, System.Convert.ToUInt16(Autosearch), Start_Frequency, Stop_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the VCO Level Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) If the RF preamplifier (B23) or electronic attenuator (B25) option is not installed on the instrument, then control Preamplifier is not used.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_PREAMP_AND_ATTEN_AUTO
        /// RSSPECAN_ATTR_AMPL_PREAMPLIFIER
        /// RSSPECAN_ATTR_ATTENUATION
        /// 
        /// Remote-control command(s):
        /// INPut:GAIN:AUTO ON | OFF
        /// INPut:GAIN:STATe ON | OFF
        /// INPut:ATTenuation 0 to 75dB
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Preamplifier___Atten_Auto">
        /// Switches preamplifier and the RF attenuator handling into automatic mode. The required preamplifier and attenuation values are set automatically.
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_PREAMP_AND_ATTEN_AUTO
        /// </param>
        /// <param name="Preamplifier">
        /// Switches on the preamplifier for the instrument. The switchable gain is fixed to 20 dB. This control is only available if the automatic setting is switched off (Preamplifier &amp; Atten Auto).
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AMPL_PREAMPLIFIER
        /// 
        /// Note(s):
        /// 
        /// (1) The control is only available with the RF preamplifier (B23) or electronic attenuator (B25) option.
        /// </param>
        /// <param name="Attenuation">
        /// Sthe input attenuator. To protect the input mixer against damage from overloads, the setting 0 dB can be obtained by entering numerals, not by using the command DEC. The step width is 5 dB, the range is 0 dB to 75 dB. This control is only available if the automatic setting is switched off (Preamplifier &amp; Atten Auto).
        /// 
        /// Units: dB
        /// 
        /// Valid Range:
        /// 0.0 to 75.0 dB
        /// 
        /// Default Value: 40.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ATTENUATION
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAVCOLevelSettings(bool Preamplifier___Atten_Auto, bool Preamplifier, double Attenuation)
        {
            int pInvokeResult = PInvoke.ConfigureSSAVCOLevelSettings(this._handle, System.Convert.ToUInt16(Preamplifier___Atten_Auto), System.Convert.ToUInt16(Preamplifier), Attenuation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the harmonics to be measured in harmonic measurements. The numeric suffix &lt;1 to 4&gt; indicates the selected trace.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_HARMONICS
        /// 
        /// Remote-control command(s):
        /// CONFigure:VCO:TRACe&lt;1...4&gt;:HARMonic
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Trace">
        /// This control selects the trace.
        /// 
        /// Valid Range:
        /// 1 - 4
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Harmonics">
        /// This control sets the harmonics to be measured in harmonic measurements.
        /// 
        /// Valid Range:
        /// 0 to 10
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_HARMONICS
        /// 
        /// Note(s):
        /// 
        /// (1) 
        /// 0 = Off
        /// 1 = Fundamental
        /// 2 - 10 = 2nd harmonic to 10th harmonic
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAVCOTraceSettings(int Trace, int Harmonics)
        {
            int pInvokeResult = PInvoke.ConfigureSSAVCOTraceSettings(this._handle, Trace, Harmonics);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches automatic scaling of the y-axis on and off for VCO measurements. The ONCE setting causes the automatic scaling to be performed immediately, regardless of whether the automatic scaling is switched on or not.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_VCO_Y_AXIS_AUTOSCALE
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...8&gt;:Y:VCO[:SCALe]:AUTO ON | OFF | ONCE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Autoscale_Y">
        /// This control switches automatic scaling of the y-axis on and off for VCO measurements.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_SSA_VCO_AUTOSCALE_OFF  (0) - Off
        /// RSSPECAN_VAL_SSA_VCO_AUTOSCALE_ON   (1) - On
        /// RSSPECAN_VAL_SSA_VCO_AUTOSCALE_ONCE (2) - Once
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_VCO_AUTOSCALE_ON (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_VCO_Y_AXIS_AUTOSCALE
        /// 
        /// Note(s):
        /// 
        /// (1)  The ONCE setting causes the automatic scaling to be performed immediately, regardless of whether the automatic scaling is switched on or not.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAVCOYAxisAutoscaleY(int Autoscale_Y)
        {
            int pInvokeResult = PInvoke.ConfigureSSAVCOYAxisAutoscaleY(this._handle, Autoscale_Y);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the upper value and range of the y-axis for VCO measurements. Depending on the configured measurement display, this command takes Hz, dBm or Hz/V units. The automatic scaling must be switched off. The numeric suffix at WINDow&lt;1|2 is irrelevant. For DC Char measurements, the suffix at TRACe is used. TRACe1 is frequency and TRACe2 is level. For Tuning Char and harmonic measurements, the TRACe suffix is irrelevant.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_VCO_Y_AXIS_TOP
        /// RSSPECAN_ATTR_SSA_VCO_Y_AXIS_RANGE
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...8&gt;:Y:VCO[:SCALe]:TOP
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...8&gt;:Y:VCO[:SCALe]:RANGe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Trace">
        /// This control selects the trace.
        /// 
        /// Valid Range:
        /// 1 - 2
        /// 
        /// Default Value: 1
        /// 
        /// Note(s):
        /// 
        /// (1) For DC Char measurements, the suffix at TRACe is used. TRACe1 is frequency and TRACe2 is level.
        /// 
        /// (2) For Tuning Char and harmonic measurements, the TRACe suffix is irrelevant.
        /// </param>
        /// <param name="Top">
        /// This control sets the upper value of the y-axis for VCO measurements.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 20.0e9
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_VCO_Y_AXIS_TOP
        /// 
        /// Note(s):
        /// 
        /// (1) Depending on the configured measurement display, this function takes as unit:
        /// 
        /// Hz - VCO Measurement Frequency
        /// dBm - VCO Measurement Level
        /// Hz/V - VCO Measurement Frequency Per Volt
        /// </param>
        /// <param name="Range">
        /// This control sets the range of the y-axis for VCO measurements.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 10.0e9
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_VCO_Y_AXIS_RANGE
        /// 
        /// Note(s):
        /// 
        /// (1) Depending on the configured measurement display, this function takes as unit:
        /// 
        /// Hz - VCO Measurement Frequency
        /// dBm - VCO Measurement Level
        /// Hz/V - VCO Measurement Frequency Per Volt
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAVCOYAxisSettings(int Trace, double Top, double Range)
        {
            int pInvokeResult = PInvoke.ConfigureSSAVCOYAxisSettings(this._handle, Trace, Top, Range);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the frequency offset of the y-axis for VCO measurements.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_VCO_Y_AXIS_OFFS
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...8&gt;:Y:VCO[:SCALe]:OFFSet
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Trace_Offset">
        /// This control sets the frequency offset of the y-axis for VCO measurements.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 3.0e6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_VCO_Y_AXIS_OFFS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAVCOYAxisTraceOffset(double Trace_Offset)
        {
            int pInvokeResult = PInvoke.ConfigureSSAVCOYAxisTraceOffset(this._handle, Trace_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the measurement type for VCO measurements.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_VCO_MEAS_TYPE
        /// 
        /// Remote-control command(s):
        /// CONFigure:VCO:MEASurement TCHar | TSENsitivity | RFPower | TCURrent | HPOWer | DCCHar | SUMMary
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="VCO_Measurement_Type">
        /// This control configures the measurement type for VCO measurements.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_SSA_VCO_MEAS_TYPE_TCH  (0) - VCO Tuning Characteristics
        /// RSSPECAN_VAL_SSA_VCO_MEAS_TYPE_TSEN (1) - VCO Tuning Sensitivity
        /// RSSPECAN_VAL_SSA_VCO_MEAS_TYPE_RFP  (2) - VCO RF Power Characteristics
        /// RSSPECAN_VAL_SSA_VCO_MEAS_TYPE_TCUR (3) - VCO Tuning Current Characteristics
        /// RSSPECAN_VAL_SSA_VCO_MEAS_TYPE_HPOW (4) - Harmonic Power
        /// RSSPECAN_VAL_SSA_VCO_MEAS_TYPE_DCCH (5) - VCO DC Characteristics
        /// RSSPECAN_VAL_SSA_VCO_MEAS_TYPE_SUMM (6) - VCO Summary
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_VCO_MEAS_TYPE_TCH  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_VCO_MEAS_TYPE
        /// 
        /// Notes:
        /// 
        /// (1) VCO Tuning Characteristics: activates the VCO Tuning Characteristic measurement (f_DUT vs. U_Tune).
        /// 
        /// (2) VCO Tuning Sensitivity: activates the VCO Tuning Sensitivity measurement. The tuning sensitivity is determined by computing the differential quotient delta_f_DUT / delta_U_TUNE of two neighbored measurement points.
        /// 
        /// (3) VCO RF Power Characteristics: activates the VCO RF Power measurement (P_DUT vs. U_Tune).
        /// 
        /// (4) VCO Tuning Current Characteristics: activates the VCO Tuning Current measurement.
        /// 
        /// (5) Harmonic Power: activates the Harmonic Power measurement (DUT.s harmonic frequencies vs. tuning voltage).
        /// 
        /// (6) VCO DC Characteristics: activates the VCO DC Characteristic measurement (DUT.s characteristic vs. supply voltage). The measurement shows the impact of a DC supply voltage sweep on the VCO's center frequency and level.
        /// 
        /// (7) VCO Summary: activates a measurement that summarizes the VCO characteristic results.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAVCOMeasurementType(int VCO_Measurement_Type)
        {
            int pInvokeResult = PInvoke.ConfigureSSAVCOMeasurementType(this._handle, VCO_Measurement_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the x-axis unit to voltage or frequency for VCO measurements.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_VCO_X_AXIS_UNIT
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...8&gt;:X:UNIT VOLTage | FREQuency
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="VCO_Meas_Units">
        /// This control sets the x-axis unit to voltage or frequency for VCO measurements.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_SSA_VCO_X_AXIS_UNIT_VOLT (0) - Voltage
        /// RSSPECAN_VAL_SSA_VCO_X_AXIS_UNIT_FREQ (1) - Frequency
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_VCO_X_AXIS_UNIT_VOLT (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_VCO_X_AXIS_UNIT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAVCOMeasurementUnits(int VCO_Meas_Units)
        {
            int pInvokeResult = PInvoke.ConfigureSSAVCOMeasurementUnits(this._handle, VCO_Meas_Units);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the relative or absolute display of the Pushing/Pulling measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_VCO_PUSH_SCAL
        /// RSSPECAN_ATTR_SSA_VCO_PULL_SCAL
        /// 
        /// Remote-control command(s):
        /// CONFigure:VCO:PUSHing:SCALe ABSolute | RELative
        /// CONFigure:VCO:PULLing:SCALe ABSolute | RELative
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="VCO_Type">
        /// This control selects the VCO type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SSA_VCO_TYPE_PUSH (0) - Pushing
        /// RSSPECAN_VAL_SSA_VCO_TYPE_PULL (1) - Pulling
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_VCO_TYPE_PUSH (0)
        /// </param>
        /// <param name="Scale">
        /// This control sets the relative or absolute display of the Pushing/Pulling measurement.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_ABS (0) - Absolute
        /// RSSPECAN_VAL_REL (1) - Relative
        /// 
        /// Default Value: RSSPECAN_VAL_REL (1)
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_VCO_PUSH_SCAL
        /// RSSPECAN_ATTR_SSA_VCO_PULL_SCAL
        /// 
        /// Note(s):
        /// 
        /// (1) Absolute: The three curves are displayed with normal scaling.
        /// 
        /// (2) Relative: The relative setting increases the resolution to distinguish the three curves individually, if are too close together. The basic curve (DC set to nominal value) is used as a reference and is displayed as a straight line. The two other curves are displayed relative to the basic curve. The scaling is adapted accordingly. 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAVCOPushPullScale(int VCO_Type, int Scale)
        {
            int pInvokeResult = PInvoke.ConfigureSSAVCOPushPullScale(this._handle, VCO_Type, Scale);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the VCO Frequency Counter
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_VCO_FREQ_COUNT_COUPL
        /// RSSPECAN_ATTR_SSA_VCO_FREQ_COUNT_RES
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]VCO:FCOunter:COUPled ON | OFF
        /// [SENSe&lt;1|2&gt;:]VCO:FCOunter:RESolution
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Coupled">
        /// Determines whether the Frequency Counter Resolution is coupled or not.
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_VCO_FREQ_COUNT_COUPL
        /// </param>
        /// <param name="Resolution">
        /// Sets the Frequency Counter Resolution for VCO measurements.
        /// 
        /// Units: Hz
        /// 
        /// Valid Range: 
        /// 1.0 Hz to 10.0e+3 Hz
        /// 
        /// Default Value: 1.0e+3 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_VCO_FREQ_COUNT_RES
        /// 
        /// Note(s):
        /// 
        /// (1) If Coupled control is set to ON, this control is ignored.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAVCOFrequencyCounter(bool Coupled, double Resolution)
        {
            int pInvokeResult = PInvoke.ConfigureSSAVCOFrequencyCounter(this._handle, System.Convert.ToUInt16(Coupled), Resolution);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the VCO Measurement Points.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_TUN_MEAS_POINTS
        /// RSSPECAN_ATTR_SSA_PUSH_MEAS_POINTS
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]VCO:TUNing:POINts
        /// [SENSe&lt;1|2&gt;:]VCO:PUSHing:POINts
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Tuning_Meas">
        /// Sets the number of points to be measured for VCO Tuning Characteristics measurements.
        /// 
        /// Units: None
        /// 
        /// Valid Range: 3 to 1000
        /// 
        /// Default Value: 10
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_TUN_MEAS_POINTS
        /// </param>
        /// <param name="Pushing_Meas">
        /// Sets the number of points to be measured for VCO DC Characteristics measurements.
        /// 
        /// Units: None
        /// 
        /// Valid Range: 3 to 1000
        /// 
        /// Default Value: 10
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_PUSH_MEAS_POINTS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAVCOMeasurementPoints(int Tuning_Meas, int Pushing_Meas)
        {
            int pInvokeResult = PInvoke.ConfigureSSAVCOMeasurementPoints(this._handle, Tuning_Meas, Pushing_Meas);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the VCO Measurement Time.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_VCO_SWEEP_TIME
        /// RSSPECAN_ATTR_SSA_SETTL_TIME_DC_PORT
        /// RSSPECAN_ATTR_SSA_SETTL_TIME_TUN_PORT
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]VCO:SWEep:TIME
        /// [SENSe&lt;1|2&gt;:]VCO:POWer:DELay
        /// [SENSe&lt;1|2&gt;:]VCO:CONTrol:DELay
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Sweep_Time">
        /// Sets the sweep time for VCO measurements.
        /// 
        /// Units: second
        /// 
        /// Valid Range: 0 s to 10 s
        /// 
        /// Default Value: 0.1 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_VCO_SWEEP_TIME
        /// </param>
        /// <param name="Settle_Time_Vdc">
        /// Sets the settling time for DC port 1.
        /// 
        /// Units: second
        /// 
        /// Valid Range: 0 s to 10 s
        /// 
        /// Default Value: 0.1 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_SETTL_TIME_DC_PORT
        /// </param>
        /// <param name="Settle_Time_Vtune">
        /// Sets the settling time for Tuning Port 1.
        /// 
        /// Units: second
        /// 
        /// Valid Range: 0 s to 10 s
        /// 
        /// Default Value: 0.1 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_SETTL_TIME_TUN_PORT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAVCOMeasurementTime(double Sweep_Time, double Settle_Time_Vdc, double Settle_Time_Vtune)
        {
            int pInvokeResult = PInvoke.ConfigureSSAVCOMeasurementTime(this._handle, Sweep_Time, Settle_Time_Vdc, Settle_Time_Vtune);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Transient Signal Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FREQUENCY_CENTER
        /// RSSPECAN_ATTR_SSA_EXPECTED_POWER
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer
        /// [SENSe&lt;1|2&gt;:]POWer:RLEVel
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Frequency">
        /// Specifies the Center Frequency of the signal to be measured. 
        /// 
        /// Units: Hz
        /// 
        /// Valid Range:
        /// not checked
        /// 
        /// Default Value:  100.e+6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_CENTER
        /// </param>
        /// <param name="Level">
        /// Specifies the expected level of the RF input signal.
        /// 
        /// Units: dB
        /// 
        /// Valid Range:
        /// -100.0 to 30.00
        /// 
        /// Default Value: -20.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_EXPECTED_POWER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSATransientSignalSettings(double Frequency, double Level)
        {
            int pInvokeResult = PInvoke.ConfigureSSATransientSignalSettings(this._handle, Frequency, Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the VCO Level Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) If the RF preamplifier (B23) or electronic attenuator (B25) option is not installed on the instrument, then control Preamplifier is not used.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_PREAMP_AND_ATTEN_AUTO
        /// RSSPECAN_ATTR_AMPL_PREAMPLIFIER
        /// RSSPECAN_ATTR_ATTENUATION
        /// 
        /// Remote-control command(s):
        /// INPut:GAIN:AUTO ON | OFF
        /// INPut:GAIN:STATe ON | OFF
        /// INPut:ATTenuation 0 to 75dB
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Preamplifier___Atten_Auto">
        /// Switches preamplifier and the RF attenuator handling into automatic mode. The required preamplifier and attenuation values are set automatically.
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_PREAMP_AND_ATTEN_AUTO
        /// </param>
        /// <param name="Preamplifier">
        /// Switches on the preamplifier for the instrument. The switchable gain is fixed to 20 dB. This control is only available if the automatic setting is switched off (Preamplifier &amp; Atten Auto).
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AMPL_PREAMPLIFIER
        /// 
        /// Note(s):
        /// 
        /// (1) The control is only available with the RF preamplifier (B23) or electronic attenuator (B25) option.
        /// </param>
        /// <param name="Attenuation">
        /// Sthe input attenuator. To protect the input mixer against damage from overloads, the setting 0 dB can be obtained by entering numerals, not by using the command DEC. The step width is 5 dB, the range is 0 dB to 75 dB. This control is only available if the automatic setting is switched off (Preamplifier &amp; Atten Auto).
        /// 
        /// Units: dB
        /// 
        /// Valid Range:
        /// 0.0 to 75.0 dB
        /// 
        /// Default Value: 40.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ATTENUATION
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSATransientLevelSettings(bool Preamplifier___Atten_Auto, bool Preamplifier, double Attenuation)
        {
            int pInvokeResult = PInvoke.ConfigureSSATransientLevelSettings(this._handle, System.Convert.ToUInt16(Preamplifier___Atten_Auto), System.Convert.ToUInt16(Preamplifier), Attenuation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches automatic scaling of the y-axis on or off for Transient measurements. The ONCE setting causes the automatic scaling to be performed immediately, regardless of whether the automatic scaling is switched on or not. This command will update the selected Transient measurement display.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_TRANS_Y_AXIS_AUTOSCALE
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...8&gt;:Y:TRANsient[:SCALe]:AUTO ON | OFF | ONCE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Autoscale_Y">
        /// This control switches automatic scaling of the y-axis on or off for Transient measurements.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_SSA_VCO_AUTOSCALE_OFF  (0) - Off
        /// RSSPECAN_VAL_SSA_VCO_AUTOSCALE_ON   (1) - On
        /// RSSPECAN_VAL_SSA_VCO_AUTOSCALE_ONCE (2) - Once
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_VCO_AUTOSCALE_ON  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_TRANS_Y_AXIS_AUTOSCALE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSATransientYAxisAutoscaleY(int Autoscale_Y)
        {
            int pInvokeResult = PInvoke.ConfigureSSATransientYAxisAutoscaleY(this._handle, Autoscale_Y);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the upper value and range of the y-axis for Transient measurements. Depending on the configured measurement display, this command takes Hz, dBm, radius or percentage as unit. The automatic scaling must be switched off.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_TRANS_Y_AXIS_TOP
        /// RSSPECAN_ATTR_SSA_TRANS_Y_AXIS_RANGE
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...8&gt;:Y:TRANsient[:SCALe]:TOP
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...8&gt;:Y:TRANsient[:SCALe]:RANGe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Top">
        /// This control sets the upper value of the y-axis for Transient measurements.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 1.0e9
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_TRANS_Y_AXIS_TOP
        /// 
        /// Note(s):
        /// 
        /// (1) Depending on the configured measurement display, this function takes as unit:
        /// 
        /// Hz - Transient Measurement Frequency
        /// dBm - Transient Measurement Amplitude
        /// radius - Transient Measurement Phase
        /// percentage - Transient Measurement Power
        /// </param>
        /// <param name="Range">
        /// This control sets the range of the y-axis for Transient measurements.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 1.0e9
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_TRANS_Y_AXIS_RANGE
        /// 
        /// Note(s):
        /// 
        /// (1) Depending on the configured measurement display, this function takes as unit:
        /// 
        /// Hz - Transient Measurement Frequency
        /// dB - Transient Measurement Amplitude
        /// radius - Transient Measurement Phase
        /// percentage - Transient Measurement Power
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSATransientYAxisSettings(double Top, double Range)
        {
            int pInvokeResult = PInvoke.ConfigureSSATransientYAxisSettings(this._handle, Top, Range);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the offset of the y-axis for Transient measurements. Depending on the configured measurement display, this command takes Hz, dB, radius or percentage as unit. The automatic scaling must be switched off.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_TRANS_Y_AXIS_OFFS
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...8&gt;:Y:TRANsient[:SCALe]:OFFSet
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Trace_Offset">
        /// This control sets the offset of the y-axis for Transient measurements.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 3.0e6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_TRANS_Y_AXIS_OFFS
        /// 
        /// Note(s):
        /// 
        /// (1) Depending on the configured measurement display, this function takes as unit:
        /// 
        /// Hz - Transient Measurement Frequency
        /// dB - Transient Measurement Amplitude
        /// radius - Transient Measurement Phase
        /// percentage - Transient Measurement Power
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSATransientYAxisTraceOffset(double Trace_Offset)
        {
            int pInvokeResult = PInvoke.ConfigureSSATransientYAxisTraceOffset(this._handle, Trace_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Transient Units.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_TRANS_UNIT
        /// 
        /// Remote-control command(s):
        /// UNIT:ANGLE DEG | RAD
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Units">
        /// Selects the unit for angles.
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_UNIT_ANGLE_DEG (0) - Degrees
        /// RSSPECAN_VAL_UNIT_ANGLE_RAD (1) - Radians
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_ANGLE_RAD (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_TRANS_UNIT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSATransientUnits(int Units)
        {
            int pInvokeResult = PInvoke.ConfigureSSATransientUnits(this._handle, Units);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the measurement type for transient measurements.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_TRANS_MEAS_TYPE
        /// 
        /// Remote-control command(s):
        /// CONFigure:TRANsient:MEASurement FREQuency | PHASe | AMPLitude | POWer
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Transient_Measurement">
        /// This control configures the measurement type for transient measurements.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_SSA_TRANS_MEAS_FREQ (0) - Transient Frequency
        /// RSSPECAN_VAL_SSA_TRANS_MEAS_PHAS (1) - Transient Phase
        /// RSSPECAN_VAL_SSA_TRANS_MEAS_AMPL (2) - Transient Amplitude
        /// RSSPECAN_VAL_SSA_TRANS_MEAS_POW  (3) - Transient Power
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_TRANS_MEAS_FREQ (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_TRANS_MEAS_TYPE
        /// 
        /// Notes:
        /// 
        /// (1) Transient Frequency: activates the Transient Frequency measurement.
        /// 
        /// (2) Transient Phase: activates the Transient Phase measurement.
        /// 
        /// (3) Transient Amplitude: activates the Transient Amplitude measurement.
        /// 
        /// (4) Transient Power: activates the Transient Power measurement.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSATransientMeasurement(int Transient_Measurement)
        {
            int pInvokeResult = PInvoke.ConfigureSSATransientMeasurement(this._handle, Transient_Measurement);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Transient Bandwidth Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_DEMOD_BAND
        /// RSSPECAN_ATTR_SSA_REL_LPASS_FILT
        /// RSSPECAN_ATTR_SSA_MEAS_TIME
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth:DEMod
        /// [SENSe&lt;1|2&gt;:]FILTer:LPASs:FREQuency:RELative
        /// [SENSe&lt;1|2&gt;:]SWEep:TIME
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Demod_BW">
        /// This control specifies the demodulation bandwidth. This setting is used during Transient measurements.
        /// 
        /// Units: Hz
        /// 
        /// Valid Range:
        /// 100 Hz to 30.0e6 Hz
        /// 
        /// Default Value: 5.0e6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_DEMOD_BAND
        /// </param>
        /// <param name="Low_Pass">
        /// Specifies the relative lowpass filter.
        /// 
        /// Units: %
        /// 
        /// Valid Values:
        /// 5, 10, 25
        /// 
        /// Default Value: 5 %
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_REL_LPASS_FILT
        /// </param>
        /// <param name="Measurement_Time">
        /// Defines the measurement time of the FM demodulator.
        /// 
        /// Units: second
        /// 
        /// Valid Values: 
        /// 15.63e-9 s to 1069.0 s
        /// 
        /// Default Value: 62.5e-6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_MEAS_TIME
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSATransientBandwidthSettings(double Demod_BW, int Low_Pass, double Measurement_Time)
        {
            int pInvokeResult = PInvoke.ConfigureSSATransientBandwidthSettings(this._handle, Demod_BW, Low_Pass, Measurement_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Transient Trigger Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_TRIG_MODE
        /// RSSPECAN_ATTR_TRIGGER_DELAY
        /// RSSPECAN_ATTR_SSA_TRIG_LEVEL
        /// RSSPECAN_ATTR_EXTERNAL_TRIGGER_LEVEL
        /// RSSPECAN_ATTR_TRIGGER_SLOPE
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:MODE IMMediate | EXTernal | POWer
        /// TRIGger&lt;1|2&gt;[:SEQuence]:HOLDoff
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:POWer
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel[:EXTernal]
        /// TRIGger&lt;1|2&gt;[:SEQuence]:SLOPe POSitive | NEGative
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Trigger_Mode">
        /// Selects the trigger source for the start of a sweep.
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_SSA_TRIG_MODE_IMM (0) - Immediate
        /// RSSPECAN_VAL_SSA_TRIG_MODE_EXT (1) - External
        /// RSSPECAN_VAL_SSA_TRIG_MODE_POW (2) - Power
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_TRIG_MODE_IMM (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_TRIG_MODE
        /// 
        /// Note(s):
        /// 
        /// (1) IMMediate - No triggering is performed. The value IMMediate corresponds to the FREE RUN setting.
        /// 
        /// (2) EXTernal - The next measurement is triggered by the signal at the external trigger input, e.g. a gated trigger.
        /// 
        /// (3) POWer - Triggering of the measurement via signals that are outside the measurement channel.
        /// </param>
        /// <param name="Trigger_Offset">
        /// Defines the length of the trigger delay. A negative delay time (pretrigger) can be set in the time domain (span = 0 Hz) only.
        /// 
        /// Units: second
        /// 
        /// Valid Values: -100 to 100 s
        /// 
        /// Default Value: 10.0e-6 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_DELAY
        /// 
        /// Note(s):
        /// 
        /// (1) The Trigger Offset settings is only available if the Trigger Mode is set to External or Power.
        /// </param>
        /// <param name="Trigger_Level">
        /// Sets the input signal level at which the measurement will be triggered to start.
        /// 
        /// Units: dBm or V
        /// 
        /// Valid Values: 
        /// -70 to +30 dBm for Power Trigger Source
        /// 0.5 to 3.5 V for External Trigger Source
        /// 
        /// Default Value: 0.0 dBm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_TRIGGER_LEVEL
        /// RSSPECAN_ATTR_SSA_TRIG_LEVEL
        /// 
        /// Note(s):
        /// 
        /// (1) The Trigger Level settings is only available if the Trigger Mode is set to Power or External.
        /// </param>
        /// <param name="Trigger_Polarity">
        /// Set the polarity of the trigger source. The sweep starts after a positive or negative edge of the trigger signal.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_NEG (0) - Negative
        /// RSSPECAN_VAL_POS (1) - Positive
        /// 
        /// Default value: RSSPECAN_VAL_POS (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_SLOPE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSATransientTriggerSettings(int Trigger_Mode, double Trigger_Offset, double Trigger_Level, int Trigger_Polarity)
        {
            int pInvokeResult = PInvoke.ConfigureSSATransientTriggerSettings(this._handle, Trigger_Mode, Trigger_Offset, Trigger_Level, Trigger_Polarity);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Transient Demodulated Trigger Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) For triggering with the FM/PM/AM/RF SIGNAL trigger source to be successful, the measurement time must cover at least five periods of the modulated signal.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_TRIG_MODE
        /// RSSPECAN_ATTR_SSA_DEMOD_TRIG_LEV
        /// RSSPECAN_ATTR_TRIGGER_DELAY
        /// RSSPECAN_ATTR_TRIGGER_SLOPE
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:MODE AM | AMRelative | FM | PM
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:AM
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:AM:REL
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:FM
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:PM
        /// TRIGger&lt;1|2&gt;[:SEQuence]:HOLDoff
        /// TRIGger&lt;1|2&gt;[:SEQuence]:SLOPe POSitive | NEGative
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Demod_Trigger_Source">
        /// Select the modulation of the source signal.
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_SSA_TRIG_MODE_AM  (3) - AM
        /// RSSPECAN_VAL_SSA_TRIG_MODE_AMR (4) - RF
        /// RSSPECAN_VAL_SSA_TRIG_MODE_FM  (5) - FM
        /// RSSPECAN_VAL_SSA_TRIG_MODE_PM  (6) - PM
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_TRIG_MODE_FM  (5)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_TRIG_MODE
        /// 
        /// Note(s):
        /// 
        /// (1) FM: Select FM signal source to trigger the measurement when the input signal reaches a specific absolute frequency.
        /// 
        /// (2) PM: Select PM signal to trigger the measurement when the input signal reaches a specific absolute phase.
        /// 
        /// (3) AM: Select AM signal to trigger the measurement when the signal reaches a specific modulation depth. 
        /// 
        /// (4) RF: Select RF signal to trigger the measurement when the input signal reaches a specific level.
        /// </param>
        /// <param name="Demod_Trigger_Level">
        /// Set the signal level at which the measurement will be triggered after the demodulation of the signal. 
        /// 
        /// Units: dBm, Hz, RAD, none
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0.0 dBm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_DEMOD_TRIG_LEV
        /// 
        /// </param>
        /// <param name="Trigger_Offset">
        /// Defines the length of the trigger delay. A negative delay time (pretrigger) can be set in the time domain (span = 0 Hz) only.
        /// 
        /// Units: second
        /// 
        /// Valid Values: -100 to 100 s
        /// 
        /// Default Value: 10.0e-6 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_DELAY
        /// 
        /// Note(s):
        /// 
        /// (1) The Trigger Offset settings is only available if the Trigger Mode is set to External or Power.
        /// </param>
        /// <param name="Trigger_Polarity">
        /// Set the polarity of the trigger source. The sweep starts after a positive or negative edge of the trigger signal.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_NEG (0) - Negative
        /// RSSPECAN_VAL_POS (1) - Positive
        /// 
        /// Default value: RSSPECAN_VAL_POS (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_SLOPE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSATransientDemodulatedTriggerSettings(int Demod_Trigger_Source, double Demod_Trigger_Level, double Trigger_Offset, int Trigger_Polarity)
        {
            int pInvokeResult = PInvoke.ConfigureSSATransientDemodulatedTriggerSettings(this._handle, Demod_Trigger_Source, Demod_Trigger_Level, Trigger_Offset, Trigger_Polarity);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Transient Squelch Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) The squelch function allows the triggering only if the following trigger conditions apply:
        /// - the demodulation trigger is above its threshold
        /// - the input level is above the squelch threshold
        /// If the input level is too low, the squelch function prevents the demodulation of the signal as long as the input level is below the set threshold.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_SQUELCH_STATE
        /// RSSPECAN_ATTR_SSA_SQUELCH_LEVEL
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...8&gt;:SQUelch[:STATe] ON | OFF
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...8&gt;:SQUelch:LEVel
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Squelch_Enabled">
        /// Enable or disable the squelch function.
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_SQUELCH_STATE
        /// 
        /// </param>
        /// <param name="Squelch_Level">
        /// Set the threshold level of the squelch function. 
        /// 
        /// Units: dBm
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0.0 dBm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_SQUELCH_LEVEL
        /// 
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available when Squelch is enabled.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSATransientSquelchSettings(bool Squelch_Enabled, double Squelch_Level)
        {
            int pInvokeResult = PInvoke.ConfigureSSATransientSquelchSettings(this._handle, System.Convert.ToUInt16(Squelch_Enabled), Squelch_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Transient zero phase reference.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_ZERO_PHASE_REF
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ADEMod:PM:RPOint:[X]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Zero_Phase_Reference">
        /// Sets the zero phase reference.
        /// 
        /// Units: s
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0.0 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_ZERO_PHASE_REF
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSATransientZeroPhaseReference(double Zero_Phase_Reference)
        {
            int pInvokeResult = PInvoke.ConfigureSSATransientZeroPhaseReference(this._handle, Zero_Phase_Reference);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function enables/disables the DC power.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_DC_POW_STATE
        /// 
        /// Remote-control command(s):
        /// SOURce:VOLTage[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="State">
        /// Enables/disables the DC power.
        /// 
        /// Unit: None
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_DC_POW_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSADCPower(bool State)
        {
            int pInvokeResult = PInvoke.ConfigureSSADCPower(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the DC Ports Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_DC_PORT_STATE
        /// RSSPECAN_ATTR_SSA_DC_PORT_VOLT
        /// RSSPECAN_ATTR_SSA_DC_PORT_VOLT_MAX
        /// RSSPECAN_ATTR_SSA_DC_PORT_VOLT_MIN
        /// RSSPECAN_ATTR_SSA_DC_PORT_CURR_MAX
        /// RSSPECAN_ATTR_SSA_DC_PORT_PUSH_VOLT
        /// 
        /// Remote-control command(s):
        /// SOURce:VOLTage:POWer&lt;1|2&gt;:LEVel[:STATe] ON | OFF
        /// SOURce:VOLTage:POWer&lt;1|2&gt;:LEVel:AMPLitude
        /// SOURce:VOLTage:POWer&lt;1|2&gt;:LEVel:LIMit:HIGH
        /// SOURce:VOLTage:POWer&lt;1|2&gt;:LEVel:LIMit:LOW
        /// SOURce:CURRent:POWer&lt;1|2&gt;:LIMit:HIGH
        /// SOURce:VOLTage:POWer&lt;1|2&gt;:PUSHing
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Port_Select">
        /// Selects the port.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Enable_Port">
        /// Enables/disables the specified DC power supply port.
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_DC_PORT_STATE
        /// 
        /// Note(s):
        /// 
        /// (1) Only available if the DC Switching Sequence is deactivated.
        /// </param>
        /// <param name="Set_Voltage">
        /// Specifies the required voltage for the specified DC power supply port. The specified voltage has to range within the limits set by the Min Voltage and Max Voltage controls.
        /// 
        /// Units: V
        /// 
        /// Valid Values: 
        /// 0 V to 12.25 V
        /// 
        /// Default Value: 5 V
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_DC_PORT_VOLT
        /// </param>
        /// <param name="Min_Voltage">
        /// Specifies the minimum voltage for the specified DC power supply port.
        /// 
        /// Units: V
        /// 
        /// Valid Values: 
        /// 0 V to 12.25 V
        /// 
        /// Default Value: 4.75 V
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_DC_PORT_VOLT_MAX
        /// </param>
        /// <param name="Max_Voltage">
        /// Specifies the maximum voltage for the specified DC power supply port.
        /// 
        /// Units: V
        /// 
        /// Valid Values: 
        /// 0 V to 12.25 V
        /// 
        /// Default Value: 5.25 V
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_DC_PORT_VOLT_MIN
        /// </param>
        /// <param name="Vcc1_Pushing">
        /// Specifies the pushing value for the VCO Tuning measurements. The value for the maximum voltage (Vmax) depends on the specified required voltage, the maximum and minimum voltage.
        /// 
        /// Units: V
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 50.0e-3 V
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_DC_PORT_PUSH_VOLT
        /// </param>
        /// <param name="Max_Current">
        /// Specifies the maximum current for the specified DC power supply port.
        /// 
        /// Units: A
        /// 
        /// Valid Values: 
        /// 0 to 500.0e-3 A
        /// 
        /// Default Value: 50.0e-3 A
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_DC_PORT_CURR_MAX
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSADCPortsSettings(int Port_Select, bool Enable_Port, double Set_Voltage, double Min_Voltage, double Max_Voltage, double Vcc1_Pushing, double Max_Current)
        {
            int pInvokeResult = PInvoke.ConfigureSSADCPortsSettings(this._handle, Port_Select, System.Convert.ToUInt16(Enable_Port), Set_Voltage, Min_Voltage, Max_Voltage, Vcc1_Pushing, Max_Current);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Tuning Ports Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_DC_TUN_PORT_STATE
        /// RSSPECAN_ATTR_SSA_DC_TUN_PORT_VOLT
        /// RSSPECAN_ATTR_SSA_DC_TUN_PORT_VOLT_MAX
        /// RSSPECAN_ATTR_SSA_DC_TUN_PORT_VOLT_MIN
        /// 
        /// Remote-control command(s):
        /// SOURce:VOLTage:CONTrol&lt;1|2&gt;:LEVel[:STATe] ON | OFF
        /// SOURce:VOLTage:CONTrol&lt;1|2&gt;:LEVel:AMPLitude
        /// SOURce:VOLTage:CONTrol&lt;1|2&gt;:LEVel:LIMit:HIGH
        /// SOURce:VOLTage:CONTrol&lt;1|2&gt;:LEVel:LIMit:LOW
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Port_Select">
        /// Selects the port.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Enable_Port">
        /// Enables/disables the specified tuning port.
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_DC_TUN_PORT_STATE
        /// 
        /// Note(s):
        /// 
        /// (1) Only available if the DC Switching Sequence is deactivated.
        /// </param>
        /// <param name="Set_Voltage">
        /// Specifies the required voltage for the specified tuning port. The specified voltage has to range within the limits set by the
        /// Min Voltage and Max Voltage controls.
        /// 
        /// Units: V
        /// 
        /// Valid Values: 
        /// -10 V to 28 V
        /// 
        /// Default Value: 2.5 V
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_DC_TUN_PORT_VOLT
        /// </param>
        /// <param name="Min_Voltage">
        /// Specifies the minimum voltage for the specified tuning port.
        /// 
        /// Units: V
        /// 
        /// Valid Values: 
        /// -10 V to 27.5 V
        /// 
        /// Default Value: 0.5 V
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_DC_TUN_PORT_VOLT_MAX
        /// </param>
        /// <param name="Max_Voltage">
        /// Specifies the maximum voltage for the specified tuning port.
        /// 
        /// Units: V
        /// 
        /// Valid Values: 
        /// -9.5 V to 28 V
        /// 
        /// Default Value: 5 V
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_DC_TUN_PORT_VOLT_MIN
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSADCTuningPortsSettings(int Port_Select, bool Enable_Port, double Set_Voltage, double Min_Voltage, double Max_Voltage)
        {
            int pInvokeResult = PInvoke.ConfigureSSADCTuningPortsSettings(this._handle, Port_Select, System.Convert.ToUInt16(Enable_Port), Set_Voltage, Min_Voltage, Max_Voltage);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the AUX Port Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_DC_AUX_PORT_STATE
        /// RSSPECAN_ATTR_SSA_DC_AUX_PORT_VOLT
        /// RSSPECAN_ATTR_SSA_DC_AUX_PORT_CURR_MAX
        /// 
        /// Remote-control command(s):
        /// SOURce:VOLTage:AUX:LEVel[:STATe] ON | OFF
        /// SOURce:VOLTage:AUX:LEVel:AMPLitude
        /// SOURce:CURRent:AUX:LIMit:HIGH
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Enable_Port">
        /// Enables/disables the AUX port.
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_DC_AUX_PORT_STATE
        /// 
        /// Note(s):
        /// 
        /// (1) Only available if the DC Switching Sequence is deactivated.
        /// </param>
        /// <param name="Set_Voltage">
        /// Specifies the required voltage for the AUX port.
        /// 
        /// Units: V
        /// 
        /// Valid Values: 
        /// -10 V to 0 V
        /// 
        /// Default Value: -5.0 V
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_DC_AUX_PORT_VOLT
        /// </param>
        /// <param name="Max_Current">
        /// Specifies the maximum current for the AUX port.
        /// 
        /// Units: A
        /// 
        /// Valid Values: 
        /// -500.0e-3 to 0.0 A
        /// 
        /// Default Value: -50.0e-3 A
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_DC_AUX_PORT_CURR_MAX
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSADCAUXPortSettings(bool Enable_Port, double Set_Voltage, double Max_Current)
        {
            int pInvokeResult = PInvoke.ConfigureSSADCAUXPortSettings(this._handle, System.Convert.ToUInt16(Enable_Port), Set_Voltage, Max_Current);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function Enables or disables the switching sequence.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_DC_SWITCH_SEQ_STATE
        /// 
        /// Remote-control command(s):
        /// SOURce:VOLTage:SEQuence[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Switching_Seq_State">
        /// Enables or disables the switching sequence.
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_DC_SWITCH_SEQ_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSADCSwitchingSequenceState(bool Switching_Seq_State)
        {
            int pInvokeResult = PInvoke.ConfigureSSADCSwitchingSequenceState(this._handle, System.Convert.ToUInt16(Switching_Seq_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Switching Sequence List.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// SOURce:VOLTage:SEQuence:LIST &lt;port&gt;,&lt;delay&gt;,...
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Port_List">
        /// Specifies the order of ports for the switching sequence.
        /// 
        /// Units: None
        /// 
        /// Valid Values: 
        /// 0 - DC Port 1
        /// 1 - DC Port 2
        /// 2 - Tuning Port 1
        /// 3 - Tuning Port 2
        /// 4 - AUX Port
        /// 
        /// Default Value: None
        /// 
        /// Note(s):
        /// 
        /// (1) This array must contain 5 elements.
        /// 
        /// (2) Each enabled port must appear in the specified sequence once and once only.
        /// </param>
        /// <param name="Delay_List">
        /// Specifies the order of delays for the switching sequence, time to wait after associated port has been switched on.
        /// 
        /// Units: second
        /// 
        /// Valid Values: 
        /// 0 to 1 s
        /// 
        /// Default Value: None
        /// 
        /// Note(s):
        /// 
        /// (1) The array must contain 5 elements.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSADCSwitchingSequenceList(int[] Port_List, double[] Delay_List)
        {
            int pInvokeResult = PInvoke.ConfigureSSADCSwitchingSequenceList(this._handle, Port_List, Delay_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the DC Port to use during measurements.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) In PLL / Cross Correlation mode, only mode A type configurations support DC Port 2.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_USE_DC_PORT
        /// 
        /// Remote-control command(s):
        /// SOURce:VOLTage:USEPort
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Use_DC_Port">
        /// Specifies the DC Port to use during measurements.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_USE_DC_PORT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSADCMeasurementPort(int Use_DC_Port)
        {
            int pInvokeResult = PInvoke.ConfigureSSADCMeasurementPort(this._handle, Use_DC_Port);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the VCO Loop DUT Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FREQUENCY_CENTER
        /// RSSPECAN_ATTR_SSA_EXPECTED_POWER
        /// RSSPECAN_ATTR_SSA_DUT_DRIFT
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer
        /// [SENSe&lt;1|2&gt;:]POWer:RLEVel
        /// [SENSe&lt;1|2&gt;:]VCO:DRIFt
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="DUT_Center_Frequency">
        /// Specifies the center frequency of the DUT.
        /// 
        /// Units: Hz
        /// 
        /// Valid Range:
        /// not checked
        /// 
        /// Default Value:  100.0e+6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_CENTER
        /// </param>
        /// <param name="DUT_Level">
        /// Specifies the level of the DUT.
        /// 
        /// Units: dB
        /// 
        /// Valid Range:
        /// -100.0 to 30.00
        /// 
        /// Default Value: 1.0 dB.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_EXPECTED_POWER
        /// </param>
        /// <param name="DUT_Drift">
        /// Specifies the drift of the DUT with the fixed tuning voltage.
        /// 
        /// Units: Hz/s
        /// 
        /// Valid Values:
        /// -10.0e+6 Hz/s to 10.0e+6 Hz/s
        /// 
        /// Default Value: 300.0 Hz/s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_DUT_DRIFT
        /// 
        /// Note(s):
        /// 
        /// (1) The manual setting of the DUT drift takes only effect, if the DUT parameters are deactivated for the pre-measurement.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAVCOLoopDUTSettings(double DUT_Center_Frequency, double DUT_Level, double DUT_Drift)
        {
            int pInvokeResult = PInvoke.ConfigureSSAVCOLoopDUTSettings(this._handle, DUT_Center_Frequency, DUT_Level, DUT_Drift);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the VCO Loop DUT Premeasurement Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) This function is only available for Phase Noise PLL and Cross Correlation PLL measurements.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_DUT_PREMEAS_STATE
        /// RSSPECAN_ATTR_SSA_DUT_TUN_RANGE
        /// RSSPECAN_ATTR_SSA_DUT_SLOPE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]VCO:PREMeas[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]VCO:TRANge
        /// [SENSe&lt;1|2&gt;:]VCO:SLOPe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="DUT_Tuning_Range">
        /// Sets the DUT Tuning Range.
        /// 
        /// Units: Hz
        /// 
        /// Valid Range:
        /// -50.0e+9 Hz to 50.0e+9 Hz
        /// 
        /// Default Value:  10.e+6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_DUT_TUN_RANGE
        /// </param>
        /// <param name="DUT_Slope">
        /// Sets the DUT slope (=K_VCO).
        /// 
        /// Units: Hz/V
        /// 
        /// Valid Range:
        /// -10.0e+9 Hz/V to 10.0e+9 Hz/V
        /// 
        /// Default Value: 5.0e+6 Hz/V
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_DUT_SLOPE
        /// </param>
        /// <param name="Premeasurement">
        /// Defines whether the DUT parameters are determined automatically in the premeasurement, and initiates a pre-measurement sequence.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_DUT_PREMEAS_STATE
        /// 
        /// Note(s):
        /// 
        /// (1) ON: The DUT parameters are determined automatically in the pre-measurement.
        /// 
        /// (2) OFF: The DUT parameters are not determined automatically and the manual setting takes effect.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAVCOLoopDUTPremeasSettings(double DUT_Tuning_Range, double DUT_Slope, bool Premeasurement)
        {
            int pInvokeResult = PInvoke.ConfigureSSAVCOLoopDUTPremeasSettings(this._handle, DUT_Tuning_Range, DUT_Slope, System.Convert.ToUInt16(Premeasurement));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the VCO Loop Phase Detector Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) This function is only available for Phase Noise PLL and Cross Correlation PLL measurements.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_PDET_PREMEAS_STATE
        /// RSSPECAN_ATTR_SSA_PDET_SLOPE
        /// RSSPECAN_ATTR_SSA_PDET_LEVEL
        /// RSSPECAN_ATTR_SSA_PDET_OFFS
        /// RSSPECAN_ATTR_SSA_PDET_HARM_STATE
        /// RSSPECAN_ATTR_SSA_PDET_HARM
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]VCO:PDETector:PREMeas[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]VCO:PDETector:SLOPe
        /// [SENSe&lt;1|2&gt;:]VCO:PDETector:LEVel
        /// [SENSe&lt;1|2&gt;:]VCO:PDETector:OFFSet
        /// [SENSe&lt;1|2&gt;:]VCO:PDETector:HARMonic:STATe ON | OFF
        /// [SENSe&lt;1|2&gt;:]VCO:PDETector:HARMonic
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Phase_Detector_Premeasurement">
        /// If switched on, the Phase Detector parameters are calculated as part of the pre-measurement sequence. If switched off, the manual setting of the Phase Detector parameters takes effect.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_PDET_PREMEAS_STATE
        /// </param>
        /// <param name="Phase_Detector_Slope">
        /// Specifies the center frequency of the DUT. If Phase Detector Premeasurement is on and a pre-measurement is started, the slope is calculated automatically.
        /// 
        /// Units: V/rad
        /// 
        /// Valid Range:
        /// 0 V/rad to 1 V/rad
        /// 
        /// Default Value:  1.0 V/rad
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_PDET_SLOPE
        /// </param>
        /// <param name="Phase_Detector_Level">
        /// Sets the Phase Detector Level. If Phase Detector Premeasurement is on and a pre-measurement is started, the level is calculated automatically.
        /// 
        /// Units: dBm
        /// 
        /// Valid Range:
        /// -30 dBm to 20 dBm
        /// 
        /// Default Value: 10.0 dBm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_PDET_LEVEL
        /// </param>
        /// <param name="Phase_Detector_DC_Offset">
        /// Sets the Phase Detector DC Offset. If Phase Detector Premeasurement is on and a pre-measurement is started, the offset is calculated automatically.
        /// 
        /// Units: V
        /// 
        /// Valid Values:
        /// -1 V to 1 V
        /// 
        /// Default Value: 0.5 V
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_PDET_OFFS
        /// </param>
        /// <param name="Auto_Harmonic">
        /// If switched on, the used harmonic is selected automatically.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_PDET_HARM_STATE
        /// </param>
        /// <param name="Harmonic">
        /// Specifies which harmonic to be used in the pre-measurement calculations. If a premeasurement is started, the harmonic is
        /// calculated automatically.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 1 to 10
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_PDET_HARM
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAVCOLoopPhaseDetectorSettings(bool Phase_Detector_Premeasurement, double Phase_Detector_Slope, double Phase_Detector_Level, double Phase_Detector_DC_Offset, bool Auto_Harmonic, int Harmonic)
        {
            int pInvokeResult = PInvoke.ConfigureSSAVCOLoopPhaseDetectorSettings(this._handle, System.Convert.ToUInt16(Phase_Detector_Premeasurement), Phase_Detector_Slope, Phase_Detector_Level, Phase_Detector_DC_Offset, System.Convert.ToUInt16(Auto_Harmonic), Harmonic);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the VCO Loop Generator Slope.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) This function is only available for Phase Noise PLL and Cross Correlation PLL measurements.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_GEN_SLOPE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]VCO:LOOP:SLOPe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Generator_Slope">
        /// Sets the Generator's Slope (=K_GEN). If a pre-measurement is started, the slope is calculated automatically. The PLL Loop BW is calculate accordingly.
        /// 
        /// Units: Hz/V
        /// 
        /// Valid Range:
        /// 1.0 Hz/V to 10.0e+9 Hz/V
        /// 
        /// Default Value:  5.0e+6 Hz/V
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_GEN_SLOPE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAVCOLoopGeneratorSlope(double Generator_Slope)
        {
            int pInvokeResult = PInvoke.ConfigureSSAVCOLoopGeneratorSlope(this._handle, Generator_Slope);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the VCO Loop PLL Bandwidth.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) This function is only available for Phase Noise PLL, Cross Correlation PLL and Spot Noise Tuning measurements.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_PLL_BAND_AUTO
        /// RSSPECAN_ATTR_SSA_PLL_BAND
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]VCO:LOOP:BWIDth:AUTO ON | OFF
        /// [SENSe&lt;1|2&gt;:]VCO:LOOP:BWIDth
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="BW_Auto">
        /// Activates or deactivates the automatic loop bandwidth calculation.
        /// 
        /// Units: None
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_PLL_BAND_AUTO
        /// </param>
        /// <param name="PLL_Bandwidth">
        /// Sets the PLL Loop BW. With ONCE, the bandwidth is calculated automatically. If a premeasurement is started, the loop bandwidth is also calculated automatically. The Generator's Slope is calculated accordingly.
        /// 
        /// Units: Hz
        /// 
        /// Valid Range:
        /// 1 Hz to 30.0e+3 Hz
        /// 
        /// Default Value:  10.0e+3 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_PLL_BAND
        /// 
        /// Note(s):
        /// 
        /// (1) This control is ignored, if BW Auto is set to ON.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAVCOLoopPLLBandwidth(bool BW_Auto, double PLL_Bandwidth)
        {
            int pInvokeResult = PInvoke.ConfigureSSAVCOLoopPLLBandwidth(this._handle, System.Convert.ToUInt16(BW_Auto), PLL_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the VCO Loop Phase Detector Characteristic.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_PDET_VOLT_BOTH
        /// RSSPECAN_ATTR_SSA_PDET_VOLT_MAX
        /// RSSPECAN_ATTR_SSA_PDET_VOLT_MIN
        /// RSSPECAN_ATTR_SSA_PDET_CHAR
        /// RSSPECAN_ATTR_SSA_PDET_SLOPE
        /// RSSPECAN_ATTR_SSA_PDET_LEVEL
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]VCO:PDETector:VOLTage:BOTH
        /// [SENSe&lt;1|2&gt;:]VCO:PDETector:VOLTage:MAXimum
        /// [SENSe&lt;1|2&gt;:]VCO:PDETector:VOLTage:MINimum
        /// [SENSe&lt;1|2&gt;:]VCO:PDETector:CHARacteristics SINe | LINear
        /// [SENSe&lt;1|2&gt;:]VCO:PDETector:SLOPe
        /// [SENSe&lt;1|2&gt;:]VCO:PDETector:LEVel
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Use_Voltage">
        /// Controls whether only the maximum or both the maximum and the minimum phase detector offset voltages are measured and used.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SSA_USE_VOLT_MAX  (0) - Only Max Voltage
        /// RSSPECAN_VAL_SSA_USE_VOLT_BOTH (1) - Both Max And Min
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_USE_VOLT_BOTH (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_PDET_VOLT_BOTH
        /// </param>
        /// <param name="Max_Phase_Detector_Voltage">
        /// Sets the maximum phase detector voltage.
        /// 
        /// Units: V
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0 V
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_PDET_VOLT_MAX
        /// </param>
        /// <param name="Min_Phase_Detector_Voltage">
        /// Sets the minimum phase detector voltage.
        /// 
        /// Units: V
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0 V
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_PDET_VOLT_MIN
        /// </param>
        /// <param name="Phase_Detector_Characteristic">
        /// Sets the characteristics of the phase detector. Using a sinusoidal phase detector automatically adds 3 dB to the phase detector slope.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SSA_PDET_CHAR_SINE (0) - Sine
        /// RSSPECAN_VAL_SSA_PDET_CHAR_LIN  (1) - Linear
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_PDET_CHAR_SINE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_PDET_CHAR
        /// </param>
        /// <param name="Phase_Detector_Slope">
        /// Sets the phase detector slope. 
        /// 
        /// Valid Values: 0 V/rad to 1 V/rad
        /// 
        /// Default Value: 0 V/rad
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_PDET_SLOPE
        /// </param>
        /// <param name="Phase_Detector_Level">
        /// Sets the phase detector level.
        /// 
        /// Valid Values: -30 dBm to 20 dBm
        /// 
        /// Default Value: 0.0 dBm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_PDET_LEVEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAVCOLoopPhaseDetectorCharacteristic(int Use_Voltage, double Max_Phase_Detector_Voltage, double Min_Phase_Detector_Voltage, int Phase_Detector_Characteristic, double Phase_Detector_Slope, double Phase_Detector_Level)
        {
            int pInvokeResult = PInvoke.ConfigureSSAVCOLoopPhaseDetectorCharacteristic(this._handle, Use_Voltage, Max_Phase_Detector_Voltage, Min_Phase_Detector_Voltage, Phase_Detector_Characteristic, Phase_Detector_Slope, Phase_Detector_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the VCO Loop AM detector characteristic.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_ADET_CFILE
        /// RSSPECAN_ATTR_SSA_ADET_EXT_LNA_GAIN
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]VCO:ADETector:CFILe
        /// [SENSe&lt;1|2&gt;:]VCO:ADETector:LNA
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="File_Name">
        /// Select the calibration file to be used in amplitude noise measurements.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_ADET_CFILE
        /// </param>
        /// <param name="Exetrnal_LNA_Gain">
        /// Sets the external LNA gain for amplitude noise measurements.
        /// 
        /// Units: dB
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_ADET_EXT_LNA_GAIN
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAVCOLoopAMDetectorCharacteristic(string File_Name, double Exetrnal_LNA_Gain)
        {
            int pInvokeResult = PInvoke.ConfigureSSAVCOLoopAMDetectorCharacteristic(this._handle, File_Name, Exetrnal_LNA_Gain);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the VCO Loop modulation index.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) This function is only available for normalization measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_ADET_MINDEX
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]VCO:ADETector:MINDex
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Modulation_Index">
        /// Sets the modulation index expected by the amplitude noise normalization.
        /// 
        /// Units: %
        /// 
        /// Valid Range: 0.1 to 100 %
        /// 
        /// Default Value: 1.0 %
        /// 
        /// Attrubute:
        /// RSSPECAN_ATTR_SSA_ADET_MINDEX
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAVCOLoopModulationIndex(double Modulation_Index)
        {
            int pInvokeResult = PInvoke.ConfigureSSAVCOLoopModulationIndex(this._handle, Modulation_Index);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the VCO Loop AM detector gain.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_ADET_GAIN
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]VCO:ADETector:GAIN
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="AM_Detector_Gain">
        /// Sets the AM detector gain.
        /// 
        /// Units: dB
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_ADET_GAIN
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAVCOLoopAMDetectorGain(double AM_Detector_Gain)
        {
            int pInvokeResult = PInvoke.ConfigureSSAVCOLoopAMDetectorGain(this._handle, AM_Detector_Gain);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Spurs Display Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) This function is only available for the Phase Noise PLL, Cross Correlation PLL and Spot Noise Tuning measurements.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_SPURS_DISP_MODE
        /// RSSPECAN_ATTR_SSA_SPURS_HIGHLIGHT
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SPURs:SUPPress NONE | ALL | UNSelected | SELected
        /// [SENSe&lt;1|2&gt;:]SPURs:HIGHlight:STATe ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Display_Mode">
        /// Specifies the spurs display mode.
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_SSA_SPURS_DISP_MODE_ALL  (0) - Show All
        /// RSSPECAN_VAL_SSA_SPURS_DISP_MODE_NONE (1) - Suppress All
        /// RSSPECAN_VAL_SSA_SPURS_DISP_MODE_SEL  (2) - Show Selected
        /// RSSPECAN_VAL_SSA_SPURS_DISP_MODE_UNS  (3) - Suppress Selected
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_SPURS_DISP_MODE_ALL (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_SPURS_DISP_MODE
        /// 
        /// Note(s):
        /// 
        /// (1) Show Selected and Suppress Selected states can only be set if the Spurs Enabled is set to ON (in SSA Select Spurs Settings function).
        /// </param>
        /// <param name="Highlight_Spurs">
        /// Controls the spurs highlighting.
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_SPURS_HIGHLIGHT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSASpursDisplaySettings(int Display_Mode, bool Highlight_Spurs)
        {
            int pInvokeResult = PInvoke.ConfigureSSASpursDisplaySettings(this._handle, Display_Mode, System.Convert.ToUInt16(Highlight_Spurs));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Select Spurs Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) This function is only available for the Phase Noise PLL and Cross Correlation PLL measurements.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_SPURS_ENAB
        /// RSSPECAN_ATTR_SSA_SPURS_FUND_FREQ
        /// RSSPECAN_ATTR_SSA_SPURS_COUNT
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SPURs:SETup&lt;1|2&gt;:ENABle ON | OFF
        /// [SENSe&lt;1|2&gt;:]SPURs:SETup&lt;1|2&gt;:FUNDamental
        /// [SENSe&lt;1|2&gt;:]SPURs:SETup&lt;1|2&gt;:COUNt
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Group">
        /// Selects the spurs group.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Enable_Spurs">
        /// Enables the selected spurs group to be displayed.
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_SPURS_ENAB
        /// </param>
        /// <param name="Fundamental_Frequency">
        /// Specifies the fundamental frequency for the selected spurs group.
        /// 
        /// Units: Hz
        /// 
        /// Valid Values:
        /// 0 Hz to 1.0e+9 Hz
        /// 
        /// Default Value: 50.0 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_SPURS_FUND_FREQ
        /// </param>
        /// <param name="Max_Order">
        /// Specifies the max order for the selected spurs group.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 1 to 50
        /// 
        /// Default Value: 5
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_SPURS_COUNT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSASpursSettings(int Group, bool Enable_Spurs, double Fundamental_Frequency, int Max_Order)
        {
            int pInvokeResult = PInvoke.ConfigureSSASpursSettings(this._handle, Group, System.Convert.ToUInt16(Enable_Spurs), Fundamental_Frequency, Max_Order);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the spur threshold. Only spurs above this threshold are displayed.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_SPURS_THR
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SPURs:THReshold
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Spur_Threshold">
        /// Specifies the spur threshold. Only spurs above this threshold are displayed.
        /// 
        /// Valid Values:
        /// 0 to 50 dB
        /// 
        /// Default Value: 0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_SPURS_THR
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSASpursThreshold(int Spur_Threshold)
        {
            int pInvokeResult = PInvoke.ConfigureSSASpursThreshold(this._handle, Spur_Threshold);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function stores the spurs list in a file with ASCII format.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_STORE_SPURS_LIST
        /// 
        /// Remote-control command(s):
        /// MMEMory:STORe&lt;1|2&gt;:SPURs
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="File_Name">
        /// Specifies the the file name of the spur list.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_STORE_SPURS_LIST
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SSAStoreSpursList(string File_Name)
        {
            int pInvokeResult = PInvoke.SSAStoreSpursList(this._handle, File_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the low noise amplifier gain settings.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) This function is only available for Phase Noise PLL, Cross Correlation PLL and Spot Noise Tuning measurements.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_AUTO_LNA_GAIN
        /// RSSPECAN_ATTR_SSA_LNA_GAIN
        /// 
        /// Remote-control command(s):
        /// INPut:LNA:GAIN:AUTO ON | OFF
        /// INPut:LNA:GAIN
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Automatic_LNA_Gain">
        /// This control activates or deactivates the automatic LNA gain selection.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_AUTO_LNA_GAIN
        /// </param>
        /// <param name="Low_Noise_Amplifier_Gain">
        /// This control specifies the amplifier setting of the low noise amplifier in dB.
        /// 
        /// Valid Values: 0, 20, 40, 50 dB
        /// 
        /// Default Value: 0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_LNA_GAIN
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available when Automatic LNA Gain is switched off.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhaseInputLNAGain(bool Automatic_LNA_Gain, int Low_Noise_Amplifier_Gain)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhaseInputLNAGain(this._handle, System.Convert.ToUInt16(Automatic_LNA_Gain), Low_Noise_Amplifier_Gain);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function zooms the trace by the specified numeric level around marker 1.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) This function is only available for Phase Noise PLL, Cross Correlation PLL and Spot Noise Tuning measurements.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_ZOOM
        /// 
        /// Remote-control command(s):
        /// CALCulate1:MARKer1:FUNCtion:ZOOM
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Zoom_Factor">
        /// This control defines the numeric level for the zoom.
        /// 
        /// Valid Range: 
        /// 1 to 20
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_ZOOM
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhaseMarkerZoom(int Zoom_Factor)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhaseMarkerZoom(this._handle, Zoom_Factor);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function creates new limit or selects existing one and configures basic parameters.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_NAME
        /// RSSPECAN_LIMIT_COMMENT
        /// RSSPECAN_LIMIT_ASSIGN_TRACE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...100&gt;:NAME
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...100&gt;:COMMent
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...100&gt;:TRACe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 100
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Name">
        /// This control sets the limit line name.
        /// 
        /// Valid Values:
        /// up to 8 character string
        /// 
        /// Default Value: "REM1"
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_NAME
        /// </param>
        /// <param name="Comment">
        /// This control sets the limit line comment.
        /// 
        /// Valid Values:
        /// up to 40 character string
        /// 
        /// Default Value: "Comment"
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_COMMENT
        /// </param>
        /// <param name="Assign_to_Trace">
        /// Assigns a limit line to a trace in the indicated measurement window.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_ASSIGN_TRACE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SSACreateLimitLine(int Limit, string Name, string Comment, int Assign_to_Trace)
        {
            int pInvokeResult = PInvoke.SSACreateLimitLine(this._handle, Limit, Name, Comment, Assign_to_Trace);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines specified limit line.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...100&gt;:CONTrol[:DATA]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...100&gt;:LOWer[:DATA]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...100&gt;:UPPer[:DATA]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 100
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Type">
        /// Specifies a limit line to be either an upper or lower type limit line. An upper line will be used as the maximum allowable value when comparing with the data. A lower limit line defines the minimum value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIMIT_LOWER (0) - Lower
        /// RSSPECAN_VAL_LIMIT_UPPER (1) - Upper
        /// 
        /// Default Value: RSSPECAN_VAL_LIMIT_LOWER (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Lower - lower limit fails if the trace falls below the limit.
        /// 
        /// (2) Upper - upper limit fails if the trace exceeds the limit.
        /// 
        /// </param>
        /// <param name="Count">
        /// Specifies number of values in X Axis and Amplitude arrays.
        /// 
        /// Valid Values:
        /// &gt;= 1
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="X_Axis">
        /// Specifies the frequency or time value for a limit point.
        /// 
        /// You must pass a ViReal64 array with at least count values.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Amplitude">
        /// Specifies the amplitude value for the current limit point.
        /// 
        /// You must pass a ViReal64 array with at least count values.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SSADefineLimitLine(int Limit, int Type, int Count, double[] X_Axis, double[] Amplitude)
        {
            int pInvokeResult = PInvoke.SSADefineLimitLine(this._handle, Limit, Type, Count, X_Axis, Amplitude);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the frequency or time domain for the x axis values.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_CONTROL_DOMAIN
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...100&gt;:CONTrol:DOMain FREQuency | TIME
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 100
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Domain">
        /// Defines the frequency or time domain for the x axis values.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_FREQ (0) - Frequency
        /// RSSPECAN_VAL_TIME (1) - Time
        /// 
        /// Default Value: RSSPECAN_VAL_FREQ (0)
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_CONTROL_DOMAIN
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SSALimitLineDomain(int Limit, int Domain)
        {
            int pInvokeResult = PInvoke.SSALimitLineDomain(this._handle, Limit, Domain);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the SSA limit line units.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_UNITS
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...100&gt;:UNIT PCT | HZ | DBM | DBCHZ | RAD | DBC | DEG
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 100
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Units">
        /// This control selects units for the limit line operations.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_UNIT_DBM   (0) - dBm
        /// RSSPECAN_VAL_UNIT_DEG   (9) - Deg
        /// RSSPECAN_VAL_UNIT_RAD   (10) - Rad
        /// RSSPECAN_VAL_UNIT_HZ    (12) - Hz
        /// RSSPECAN_VAL_UNIT_PCT   (13) - Pct
        /// RSSPECAN_VAL_UNIT_DBC   (23) - dBc
        /// RSSPECAN_VAL_UNIT_DBCHZ (24) - dBc/Hz
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_DBCHZ (24)
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_UNITS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SSALimitLineUnits(int Limit, int Units)
        {
            int pInvokeResult = PInvoke.SSALimitLineUnits(this._handle, Limit, Units);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the limit check and/or limit line for the selected limit line in the selected measurement window.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_STATE
        /// RSSPECAN_LIMIT_LOWER_STATE
        /// RSSPECAN_LIMIT_UPPER_STATE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...100&gt;:STATe ON | OFF
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...100&gt;:LOWer:STATe ON | OFF
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...100&gt;:UPPer:STATe ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 100
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Type">
        /// Specifies a limit line to be either an upper or lower type limit line. An upper line will be used as the maximum allowable value when comparing with the data. A lower limit line defines the minimum value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIMIT_LOWER (0) - Lower
        /// RSSPECAN_VAL_LIMIT_UPPER (1) - Upper
        /// 
        /// Default Value: RSSPECAN_VAL_LIMIT_LOWER (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Lower - lower limit fails if the trace falls below the limit.
        /// 
        /// (2) Upper - upper limit fails if the trace exceeds the limit.
        /// 
        /// </param>
        /// <param name="Limit_Enabled">
        /// Turns the limit line display on and off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_STATE
        /// </param>
        /// <param name="Check_Enabled">
        /// Turns limit line testing on/off. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_LOWER_STATE
        /// RSSPECAN_LIMIT_UPPER_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SSAEnableLimitCheck(int Limit, int Type, bool Limit_Enabled, bool Check_Enabled)
        {
            int pInvokeResult = PInvoke.SSAEnableLimitCheck(this._handle, Limit, Type, System.Convert.ToUInt16(Limit_Enabled), System.Convert.ToUInt16(Check_Enabled));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function moves specified limit line.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_CONTROL_SHIFT
        /// RSSPECAN_LIMIT_LOWER_SHIFT
        /// RSSPECAN_LIMIT_UPPER_SHIFT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...100&gt;:CONTrol:SHIFt
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...100&gt;:LOWer:SHIFt
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...100&gt;:UPPer:SHIFt
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 100
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Type">
        /// Specifies a limit line to be type to be moved.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIMIT_LOWER   (0) - Lower
        /// RSSPECAN_VAL_LIMIT_UPPER   (1) - Upper
        /// RSSPECAN_VAL_LIMIT_CONTROL (2) - X-Axis
        /// 
        /// Default Value: RSSPECAN_VAL_LIMIT_LOWER (0)
        /// </param>
        /// <param name="Value">
        /// Defines the move value.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_CONTROL_SHIFT
        /// RSSPECAN_LIMIT_LOWER_SHIFT
        /// RSSPECAN_LIMIT_UPPER_SHIFT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SSAMoveLimitLine(int Limit, int Type, double Value)
        {
            int pInvokeResult = PInvoke.SSAMoveLimitLine(this._handle, Limit, Type, Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function copies specified limit line into another one.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_COPY
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...100&gt;:COPY
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 100
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Copy_To">
        /// This control selects the limit line target.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_COPY
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SSACopyLimitLine(int Limit, int Copy_To)
        {
            int pInvokeResult = PInvoke.SSACopyLimitLine(this._handle, Limit, Copy_To);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function removes specified limit line.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_DELETE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...100&gt;:DELete
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 100
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SSADeleteLimitLine(int Limit)
        {
            int pInvokeResult = PInvoke.SSADeleteLimitLine(this._handle, Limit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies whether smoothing of a particular trace is performed.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_SMOOTHING_STATE
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...8&gt;:SMOothing[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Trace">
        /// Selects the trace.
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Smoothing_State">
        /// Specifies whether smoothing of a particular trace is performed.
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_SMOOTHING_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSATraceSmoothingState(int Trace, bool Smoothing_State)
        {
            int pInvokeResult = PInvoke.ConfigureSSATraceSmoothingState(this._handle, Trace, System.Convert.ToUInt16(Smoothing_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the automatic power level detection. If switched on, the power level detection is performed at the start of each measurement sweep.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_AUTO_LEVEL
        /// 
        /// Remote-control command(s):
        /// CONFigure:POWer:AUTO ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Auto_Level">
        /// This control switches on or off the automatic power level detection.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On (Default Value)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_AUTO_LEVEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhaseAutoLevel(bool Auto_Level)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhaseAutoLevel(this._handle, System.Convert.ToUInt16(Auto_Level));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the input power level of the source signal that is supplied to the RF input.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_SIGNAL_LEVEL
        /// 
        /// Remote-control command(s):
        /// CONFigure:POWer:EXPected:RF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Signal_Level">
        /// This control specifies the input power level of the source signal that is supplied to the RF input.
        /// 
        /// Valid Range: 
        /// -100.0 dBm to 30.0 dBm
        /// 
        /// Default Value: -20.0 dBm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_SIGNAL_LEVEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhaseSignalLevelRF(double Signal_Level)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhaseSignalLevelRF(this._handle, Signal_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the preset compatibility.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_PRESET_COMP
        /// 
        /// Remote-control command(s):
        /// SYSTem:PRESet:COMPatible OFF | SANanalyzer
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Preset_Mode">
        /// Specifies the preset compatibility.
        /// 
        /// Units: None
        /// 
        /// Valid Range: 
        /// RSSPECAN_VAL_SSA_PRESET_PNO (0) - Phase Noise
        /// RSSPECAN_VAL_SSA_PRESET_SAN (1) - Spectrum Analyzer
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_PRESET_PNO (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_PRESET_COMP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPresetCompatibility(int Preset_Mode)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPresetCompatibility(this._handle, Preset_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads trace data out of the instrument. Number and format of the measurement values for Signal Source Analyser - Phase Noise is described in Trace Data parameter.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) The numeric suffix at TRACe&lt;1|2&gt; is irrelevant.
        /// 
        /// Remote-control command(s):
        /// FORMat ASCII
        /// TRACe&lt;1|2&gt;[:DATA]? TRACE1 | TRACE2 | TRACE3 | TRACE4 | TRACE5 | TRACE6 | TRACE7 | TRACE8
        /// FORMat REAL,32
        /// *CLS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Source_Trace">
        /// This parameter indicates which trace will be read out.
        /// 
        /// Valid Range:
        /// 1 - Trace 1
        /// 2 - Trace 2
        /// 3 - Trace 3
        /// 4 - Trace 4
        /// 5 - Trace 5
        /// 6 - Trace 6
        /// 7 - Trace 7
        /// 8 - Trace 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Array_Length">
        /// Number of array points requested.
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="No_of_Values">
        /// Returns the number of trace data values.
        /// 
        /// </param>
        /// <param name="Trace_Data">
        /// Returns an array of trace data. Number and format of the measurement values for Signal Source Analyser - Phase Noise:
        /// 
        /// All measured data related to the currently selected measurement type is returned in x,y,x,y,x,y...format. VCO measurements' trace data are as follows:
        /// 
        /// (1) 
        /// VCO Tuning Characteristics (pushing off) - 8 traces
        /// X-Axis: Volt or Frequency
        /// Y-Axis: Trace 1, 3, 5, 7 - Frequency or Volt
        ///         Trace 2, 4, 6, 8 - Sensitivity
        /// 
        /// (2)
        /// VCO Tuning Characteristics (pushing on) - 8 traces
        /// X-Axis: Volt or Frequency
        /// Y-Axis: Frequency or Volt
        /// 
        /// (3)
        /// VCO Tuning Sensitivity (pushing off) - 8 traces
        /// X-Axis: Volt or Frequency
        /// Y-Axis: Trace 1, 3, 5, 7 - Sensitivity
        ///         Trace 2, 4, 6, 8 - Level
        /// 
        /// (4)
        /// VCO Tuning Sensitivity (pushing on) - 8 traces
        /// X-Axis: Volt or Frequency
        /// Y-Axis: Sensitivity
        /// 
        /// (5)
        /// VCO RF Power Characteristics (pushing off) - 8 traces
        /// X-Axis: Volt or Frequency
        /// Y-Axis: Trace 1, 3, 5, 7 - Level
        ///         Trace 2, 4, 6, 8 - Frequency or Volt
        /// 
        /// (6)
        /// VCO RF Power Characteristics (pushing on) - 8 traces
        /// X-Axis: Volt or Frequency
        /// Y-Axis: Level
        /// 
        /// (7)
        /// VCO Harmonic Power - 8 traces
        /// X-Axis: Volt or Frequency
        /// Y-Axis: Level
        /// 
        /// (8)
        /// VCO DC Characteristics - 8 traces
        /// X-Axis: Volt
        /// Y-Axis: Trace 1, 3, 5, 7 - Frequency
        ///         Trace 2, 4, 6, 8 - Level
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadSSATraceData(int Source_Trace, int Array_Length, out int No_of_Values, double[] Trace_Data)
        {
            int pInvokeResult = PInvoke.ReadSSATraceData(this._handle, Source_Trace, Array_Length, out No_of_Values, Trace_Data);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches automatic scaling for the y-axis on and off. The ONCE setting causes the automatic scaling to be performed immediately, regardless of whether automatic scaling is switched on or not.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_AUTO_SCAL_ONCE
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...8&gt;:Y[:SCALe]:AUTO ONCE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SSAAutoscaleY()
        {
            int pInvokeResult = PInvoke.SSAAutoscaleY(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures and initiates a reference measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_REF_MEAS
        /// 
        /// Remote-control command(s):
        /// CONFigure:REFMeas ONCE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value for data readout in milliseconds.
        /// 
        /// Valid Range: 0 to 600000
        /// 
        /// Default Value: 10000 ms
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAPhaseReferenceMeasurement(int Timeout)
        {
            int pInvokeResult = PInvoke.ConfigureSSAPhaseReferenceMeasurement(this._handle, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the VCO Pushing or Pulling measurement. If VCO Pushing is on, VCO Pulling is switched off and vice versa.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_VCO_PUSH_STATE
        /// RSSPECAN_ATTR_SSA_VCO_PULL_STATE
        /// 
        /// Remote-control command(s):
        /// CONFigure:VCO:PUSHing[:STATe] ON | OFF
        /// CONFigure:VCO:PULLing[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="VCO_Type">
        /// This control selects the VCO type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SSA_VCO_TYPE_PUSH (0) - Pushing
        /// RSSPECAN_VAL_SSA_VCO_TYPE_PULL (1) - Pulling
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_VCO_TYPE_PUSH (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Pushing: Activates or deactivates a Pushing measurement. The measurement identifies the impact of a DC supply voltage change on the characteristic curve. Three consecutive characteristic measurements with variation in DC supply voltage are performed and displayed in one trace. The DC supply voltage variation is defined in the DC SETTINGS dialog box, delta_V_cc1 Pushing field.
        /// 
        /// (2) Pulling: Activates or deactivates a Pulling measurement. The measurement identifies the impact of a load change on the characteristic curve. Three consecutive characteristic measurements with a variation in load are performed and displayed in one trace.
        /// </param>
        /// <param name="State">
        /// This control switches on or off the VCO Pushing or Pulling measurement.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_VCO_PUSH_STATE
        /// RSSPECAN_ATTR_SSA_VCO_PULL_STATE
        /// 
        /// Notes:
        /// 
        /// (1) If VCO Pushing is on, VCO Pulling is switched off and vice versa.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAVCOPushPullState(int VCO_Type, bool State)
        {
            int pInvokeResult = PInvoke.ConfigureSSAVCOPushPullState(this._handle, VCO_Type, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts the premeasurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) This function is only available for Phase Noise PLL and Cross Correlation PLL measurements.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_START_PREMEAS
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]VCO:PREMeas[:STATe] ONCE
        /// *OPC?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Timeout__ms_">
        /// Sets the timeout for the triggering routine to be finished. If the length of time required for triggering exceeds the timeout value, then the function will return with a timeout error.
        /// 
        /// Valid Range: 0 ms to 600000 ms
        /// 
        /// Default Value: 180000 ms
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAVCOLoopStartPremeasurement(int Timeout__ms_)
        {
            int pInvokeResult = PInvoke.ConfigureSSAVCOLoopStartPremeasurement(this._handle, Timeout__ms_);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the VCO Loop Premeasurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_PREMEAS_NEXT
        /// RSSPECAN_ATTR_SSA_PREMEAS_SKIP
        /// RSSPECAN_ATTR_SSA_PREMEAS_FIN
        /// 
        /// Remote-control command(s):
        /// INITiate:NEXT
        /// INITiate:SKIP
        /// INITiate:FINish
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Action">
        /// Selects the premeasurement action.
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_SSA_PREMEAS_CONT (0) - Continue Step
        /// RSSPECAN_VAL_SSA_PREMEAS_SKIP (1) - Skip Step
        /// RSSPECAN_VAL_SSA_PREMEAS_FIN  (2) - Finish Premeasurement
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_PREMEAS_CONT (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Continue Step: Finalizes the current step of  the premeasurement and continues to the next step.
        /// 
        /// (2) Skip Step: Skips the current step of the premeasurement.
        /// 
        /// (3) Finish Premeasurement: Finishes the premeasurement, closes the dialog box and starts the measurement.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAVCOLoopPremeasurement(int Action)
        {
            int pInvokeResult = PInvoke.ConfigureSSAVCOLoopPremeasurement(this._handle, Action);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures amplitude noise measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_ANOISE_MEAS_STAT
        /// 
        /// Remote-control command(s):
        /// CONFigure:ANOise:MEASurement ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="State">
        /// This control switches on or off the amplitude noise measurement
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SSA_ANOISE_MEAS_STAT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAAMNoiseMeasurement(bool State)
        {
            int pInvokeResult = PInvoke.ConfigureSSAAMNoiseMeasurement(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function performs an amplitude noise operating point premeasurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_ANOISE_OPOINT_PREMEAS
        /// 
        /// Remote-control command(s):
        /// CONFigure:ANOise:PREMeas:OPOint ONCE
        /// *OPC?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Timeout__ms_">
        /// Sets the timeout for the triggering routine to be finished. If the length of time required for triggering exceeds the timeout value, then the function will return with a timeout error.
        /// 
        /// Valid Range: 0 ms to 600000 ms
        /// 
        /// Default Value: 180000 ms
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAAMNoiseOperatingPointPremeasurement(int Timeout__ms_)
        {
            int pInvokeResult = PInvoke.ConfigureSSAAMNoiseOperatingPointPremeasurement(this._handle, Timeout__ms_);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function performs an amplitude noise normalization premeasurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_ANOISE_NORM_PREMEAS
        /// 
        /// Remote-control command(s):
        /// CONFigure:ANOise:PREMeas:NORMalize ONCE
        /// *OPC?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Timeout__ms_">
        /// Sets the timeout for the triggering routine to be finished. If the length of time required for triggering exceeds the timeout value, then the function will return with a timeout error.
        /// 
        /// Valid Range: 0 ms to 600000 ms
        /// 
        /// Default Value: 180000 ms
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSSAAMNoiseNormalizationPremeasurement(int Timeout__ms_)
        {
            int pInvokeResult = PInvoke.ConfigureSSAAMNoiseNormalizationPremeasurement(this._handle, Timeout__ms_);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries whether the current marker is positioned on a spur.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_MARKER_POS_ON_SPUR
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:X:SPUR?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Marker_Number">
        /// Selects the marker.
        /// 
        /// Valid Range:
        /// 1 - 1 (Default Value)
        /// 2 - 2
        /// 3 - 3
        /// 4 - 4
        /// 
        /// </param>
        /// <param name="Marker_Position_On_Spur">
        /// This control queries whether the current marker is positioned on a spur.
        /// 
        /// Valid Range:
        /// 0 if the marker is not positioned on a spur
        /// 1 if the marker is positioned on a spur
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QuerySSAMarkerOnSpur(int Marker_Number, out int Marker_Position_On_Spur)
        {
            int pInvokeResult = PInvoke.QuerySSAMarkerOnSpur(this._handle, Marker_Number, out Marker_Position_On_Spur);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries whether the current delta marker is positioned on a spur.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_DELTA_MARKER_POS_ON_SPUR
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:X:SPUR?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Marker_Number">
        /// Selects the marker.
        /// 
        /// Valid Range:
        /// 1 - 1 (Default Value)
        /// 2 - 2
        /// 3 - 3
        /// 4 - 4
        /// 
        /// </param>
        /// <param name="DMarker_Position_On_Spur">
        /// This control queries whether the current delta marker is positioned on a spur.
        /// 
        /// Valid Range:
        /// 0 if the delta marker is not positioned on a spur
        /// 1 if the delta marker is positioned on a spur
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QuerySSADeltaMarkerOnSpur(int Marker_Number, out int DMarker_Position_On_Spur)
        {
            int pInvokeResult = PInvoke.QuerySSADeltaMarkerOnSpur(this._handle, Marker_Number, out DMarker_Position_On_Spur);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the result of the selected spot noise frequency point.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) This function is only available for Phase Noise PLL, Cross Correlation PLL and Spot Noise Tuning measurements.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_GET_SPOT_NOISE_Y_POS
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:SNOise&lt;1...5&gt;:Y?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Spot_Noise_Marker">
        /// Selects the spot noise marker to configure.
        /// 
        /// Valid Range:
        /// 1 to 5
        /// 
        /// Default Value:
        /// 1
        /// 
        /// </param>
        /// <param name="Position__y_">
        /// Returns the result of the selected spot noise frequency point.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QuerySSASpotNoiseValue(int Spot_Noise_Marker, out double Position__y_)
        {
            int pInvokeResult = PInvoke.QuerySSASpotNoiseValue(this._handle, Spot_Noise_Marker, out Position__y_);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the compatibility flag of the indicated limit line.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) A limit line is marked as not compatible if the limit line domain/units are not compatible with the current measurement type. In such cases, correct the domain/unit for the limit line or the current measurement type to make the limit line compatible.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_QUERY_LIMIT_LINE_COMP
        /// 
        /// Remote-control command(s):
        /// CALCulate:LIMit&lt;1...100&gt;:COMPatible?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 100
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit_Line_Compatibility">
        /// This control queries the compatibility flag of the indicated limit line. The result of the limit check responds with 0 for not compatible and 1 for compatible.
        /// 
        /// Valid Range:
        /// 0 - not compatible
        /// 1 - compatible
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QuerySSALimitLineCompatibility(int Limit, out int Limit_Line_Compatibility)
        {
            int pInvokeResult = PInvoke.QuerySSALimitLineCompatibility(this._handle, Limit, out Limit_Line_Compatibility);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function deletes the result of the current limit check.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_CLEAR
        /// 
        /// Remote-control command(s):
        /// CALCulate:LIMit&lt;1...100&gt;:CLEar[:IMMediate]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 100
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SSAClearLimitLineResults(int Limit)
        {
            int pInvokeResult = PInvoke.SSAClearLimitLineResults(this._handle, Limit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the number of sweep points currently held in trace 1.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_QUERY_SWEEP_POINTS
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:POINts?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Sweep_Points">
        /// Returns the number of sweep points currently held in trace 1.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QuerySSASweepPoints(out int Sweep_Points)
        {
            int pInvokeResult = PInvoke.QuerySSASweepPoints(this._handle, out Sweep_Points);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns results of complex measurement tasks.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_FETCH_PHASE_NOISE_RESULTS
        /// 
        /// Remote-control command(s):
        /// FETCh:PNOise&lt;1...8&gt;:RFM?
        /// FETCh:PNOise&lt;1...8&gt;:RPM?
        /// FETCh:PNOise&lt;1...8&gt;:RMS?
        /// FETCh:PNOise&lt;1...8&gt;:IPN?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Trace_Number">
        /// Selects the trace to configure.
        /// 
        /// Valid Range:
        /// 1 to 8
        /// 
        /// Default Value:
        /// 1
        /// 
        /// </param>
        /// <param name="Type">
        /// Sets the result type.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_SSA_PHAS_NOIS_RES_RFM (0) - Residual FM
        /// RSSPECAN_VAL_SSA_PHAS_NOIS_RES_RPM (1) - Residual PM
        /// RSSPECAN_VAL_SSA_PHAS_NOIS_RES_RMS (2) - Residual RMS
        /// RSSPECAN_VAL_SSA_PHAS_NOIS_RES_IPN (3) - Integrated Phase Noise Result
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_PHAS_NOIS_RES_RFM (0)
        /// 
        /// 
        /// </param>
        /// <param name="Result">
        /// Outputs the result of the measurement.
        /// 
        /// Note:
        /// 
        /// (1) Residual FM - returns the measured Residual FM result.
        /// 
        /// (2) Residual PM - returns the measured Residual PM result.
        /// 
        /// (3) Residual RMS - returns the measured Residual RMS result.
        /// 
        /// (4) Integrated Phase Noise Result - returns the measured Integrated Phase Noise result.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchSSAPhaseNoiseResults(int Trace_Number, int Type, out double Result)
        {
            int pInvokeResult = PInvoke.FetchSSAPhaseNoiseResults(this._handle, Trace_Number, Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the measured spurs results. The data is returned in x,y,x,y,x,y...format: x value for frequency, y value for power.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FETCh:PNOise&lt;1...8&gt;:SPURs?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Trace_Number">
        /// Selects the trace to configure.
        /// 
        /// Valid Range:
        /// 1 to 8
        /// 
        /// Default Value:
        /// 1
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// This control sets the size of arrays - controls Frequency Results and Power Results.
        /// 
        /// Valid Values: &gt;0
        /// 
        /// Default Value: 10
        /// </param>
        /// <param name="Frequency_Results">
        /// This control returns the measured spurs frequncy results.
        /// 
        /// Note(s):
        /// 
        /// (1) The number of results returned is set with control Array Size.
        /// 
        /// 
        /// </param>
        /// <param name="Power_Results">
        /// This control returns the measured spurs power results.
        /// 
        /// Note(s):
        /// 
        /// (1) The number of results returned is set with control Array Size.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchSSAPhaseNoiseSpursResults(int Trace_Number, int Array_Size, double[] Frequency_Results, double[] Power_Results)
        {
            int pInvokeResult = PInvoke.FetchSSAPhaseNoiseSpursResults(this._handle, Trace_Number, Array_Size, Frequency_Results, Power_Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns results of complex measurement tasks.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FETCh:VCO:TCHar:FREQuency?
        /// FETCh:VCO:TCHar:POWer?
        /// FETCh:VCO:TCHar:SENSitivity?
        /// FETCh:VCO:TCHar:CURRent?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Type">
        /// Sets the result type.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_SSA_VCO_TCH_RES_TYPE_FREQ (0) - Frequency
        /// RSSPECAN_VAL_SSA_VCO_TCH_RES_TYPE_POW  (1) - Power
        /// RSSPECAN_VAL_SSA_VCO_TCH_RES_TYPE_SENS (2) - Sensitivity
        /// RSSPECAN_VAL_SSA_VCO_TCH_RES_TYPE_CURR (3) - Current
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_VCO_TCH_RES_TYPE_FREQ (0)
        /// </param>
        /// <param name="Results">
        /// Outputs the results of the measurement.
        /// 
        /// Note:
        /// 
        /// (1) Frequency - returns a list of measured frequency results at the minimum, current and maximum tuning voltages.
        /// 
        /// (2) Power - returns a list of measured power results at minimum, current and maximum tuning voltages.
        /// 
        /// (3) Sensitivity - returns a list of measured sensitivity results at minimum, current and maximum tuning voltages.
        /// 
        /// (4) Current - returns a list of measured supply current results at minimum, current and maximum tuning voltages.
        /// 
        /// (5) The array must contain at least 3 elements.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchSSAVCOTuningCharResults(int Type, double[] Results)
        {
            int pInvokeResult = PInvoke.FetchSSAVCOTuningCharResults(this._handle, Type, Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns results of complex measurement tasks.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FETCh:VCO:PUSHing:FDEViation?
        /// FETCh:VCO:PUSHing:PDEViation?
        /// FETCh:VCO:PUSHing:SENSitivity?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Type">
        /// Sets the result type.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_SSA_VCO_PUSH_PULL_RES_FDEV (0) - Frequency Deviation
        /// RSSPECAN_VAL_SSA_VCO_PUSH_PULL_RES_PDEV (1) - Power Deviation
        /// RSSPECAN_VAL_SSA_VCO_PUSH_PULL_RES_SENS (2) - Sensitivity
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_VCO_PUSH_PULL_RES_FDEV (0)
        /// </param>
        /// <param name="Results">
        /// Outputs the results of the measurement.
        /// 
        /// Note:
        /// 
        /// (1) Frequency Deviation - returns a list of measured frequency deviation results at pushing voltages in the following format:
        /// - frequency deviation at (current tuning voltage + pushing voltage)
        /// - frequency deviation at (current tuning voltage - pushing voltage)
        /// 
        /// (2) Power Deviation - returns a list of measured power deviation results at pushing voltages in the following format:
        /// - power deviation at (current tuning voltage + pushing voltage)
        /// - power deviation at (current tuning voltage - pushing voltage)
        /// 
        /// (3) Sensitivity - returns a list of measured sensitivity results at pushing voltages in the following format:
        /// - sensitivity at (current tuning voltage + pushing voltage)
        /// - sensitivity at (current tuning voltage - pushing voltage)
        /// 
        /// (4) The array must contain at least 2 elements.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchSSAVCOPushingResults(int Type, double[] Results)
        {
            int pInvokeResult = PInvoke.FetchSSAVCOPushingResults(this._handle, Type, Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns results of complex measurement tasks.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FETCh:VCO:PULLing:FDEViation?
        /// FETCh:VCO:PULLing:PDEViation?
        /// FETCh:VCO:PULLing:SENSitivity?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Type">
        /// Sets the result type.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_SSA_VCO_PUSH_PULL_RES_FDEV (0) - Frequency Deviation
        /// RSSPECAN_VAL_SSA_VCO_PUSH_PULL_RES_PDEV (1) - Power Deviation
        /// RSSPECAN_VAL_SSA_VCO_PUSH_PULL_RES_SENS (2) - Sensitivity
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_VCO_PUSH_PULL_RES_FDEV (0)
        /// 
        /// 
        /// </param>
        /// <param name="Results">
        /// Outputs the results of the measurement.
        /// 
        /// Note:
        /// 
        /// (1) Frequency Deviation - returns a list of measured frequency deviation results at the trigger port in the following format:
        /// - frequency deviation (with termination 1)
        /// - frequency deviation (with termination 2)
        /// 
        /// (2) Power Deviation - returns a list of measured power deviation results at the trigger port in the following format:
        /// - power deviation (with termination 1)
        /// - power deviation (with termination 2)
        /// 
        /// (3) Sensitivity - returns a list of measured sensitivity deviation results at the trigger port in the following format:
        /// - sensitivity deviation (with termination 1)
        /// - sensitivity deviation (with termination 2)
        /// 
        /// (4) The array must contain at least 2 elements.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchSSAVCOPullingResults(int Type, double[] Results)
        {
            int pInvokeResult = PInvoke.FetchSSAVCOPullingResults(this._handle, Type, Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns results of complex measurement tasks.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FETCh:VCO:HCHar&lt;1...8&gt;:FREQuency?
        /// FETCh:VCO:HCHar&lt;1...8&gt;:POWer:ABSolute?
        /// FETCh:VCO:HCHar&lt;1...8&gt;:POWer:RELative?
        /// FETCh:VCO:HCHar&lt;1...8&gt;:SENSitivity?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Trace_Number">
        /// Selects the trace to configure.
        /// 
        /// Valid Range:
        /// 1 to 8
        /// 
        /// Default Value:
        /// 1
        /// 
        /// </param>
        /// <param name="Type">
        /// Sets the result type.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_SSA_VCO_HARM_RES_FREQ (0) - Frequency
        /// RSSPECAN_VAL_SSA_VCO_HARM_RES_APOW (1) - Absolute Power
        /// RSSPECAN_VAL_SSA_VCO_HARM_RES_RPOW (2) - Relative Power
        /// RSSPECAN_VAL_SSA_VCO_HARM_RES_SENS (3) - Sensitivity
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_VCO_HARM_RES_FREQ (0)
        /// </param>
        /// <param name="Results">
        /// Outputs the results of the measurement.
        /// 
        /// Note:
        /// 
        /// (1) Frequency - returns a list of measured harmonic frequency results at minimum, current and maximum tuning voltages.
        /// 
        /// (2) Absolute Power - returns a list of measured absolute harmonic power results at minimum, current and maximum tuning voltages.
        /// 
        /// (3) Relative Power - returns a list of measured relative power results at minimum, current and maximum tuning voltages.
        /// 
        /// (4) Sensitivity - returns a list of measured harmonic sensitivity results at minimum, current and maximum tuning voltages.
        /// 
        /// (5) The array must contain at least 3 elements.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchSSAVCOHarmonicResults(int Trace_Number, int Type, double[] Results)
        {
            int pInvokeResult = PInvoke.FetchSSAVCOHarmonicResults(this._handle, Trace_Number, Type, Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns results of complex measurement tasks.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FETCh:VCO:DCHar:FREQuency?
        /// FETCh:VCO:DCHar:POWer?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Type">
        /// Sets the result type.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_SSA_VCO_DCH_RES_FREQ (0) - Frequency
        /// RSSPECAN_VAL_SSA_VCO_DCH_RES_POW  (1) - Power
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_VCO_DCH_RES_FREQ (0)
        /// </param>
        /// <param name="Results">
        /// Outputs the results of the measurement.
        /// 
        /// Note:
        /// 
        /// (1) Frequency - returns a list of measured frequency results at minimum, current and maximum supply voltages.
        /// 
        /// (2) Power - returns a list of measured power results at minimum, current and maximum supply voltages.
        /// 
        /// (3) The array must contain at least 3 elements.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchSSAVCODCCharResults(int Type, double[] Results)
        {
            int pInvokeResult = PInvoke.FetchSSAVCODCCharResults(this._handle, Type, Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the VCO Transient modulation measurement summary.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) The suffixes for CALCulate, MARKer and RESult are ignored.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_DEMOD_SUMM_RES
        /// RSSPECAN_ATTR_SSA_FREQ_ERR_RES
        /// RSSPECAN_ATTR_SSA_CARR_POW_RES
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ADEMod:AM[:RESult&lt;1...3&gt;]? PPEak | MPEak | MIDDle |RMS 
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ADEMod:FM[:RESult&lt;1...3&gt;]? PPEak | MPEak | MIDDle |RMS 
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ADEMod:PM[:RESult&lt;1...3&gt;]? PPEak | MPEak | MIDDle |RMS 
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ADEMod:FERRor:RESult&lt;1...3&gt;?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ADEMod:CARRier:RESult&lt;1...3&gt;?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Modulation_Type">
        /// This control sets modulation type.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_SSA_MOD_AM (0) - AM
        /// RSSPECAN_VAL_SSA_MOD_FM (1) - FM
        /// RSSPECAN_VAL_SSA_MOD_PM (2) - PM
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_MOD_FM (1)
        /// 
        /// Notes:
        /// 
        /// (1) AM:
        /// AM modulation measurement results of the analog demodulation.
        /// 
        /// (2) FM:
        /// FM modulation measurement results of the analog demodulation.
        /// 
        /// (3) PM:
        /// PM modulation measurement results of the analog demodulation.
        /// 
        /// </param>
        /// <param name="Parameter_To_Read">
        /// Specifies the parameter to read.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_SSA_DET_PPE  (0) - Positive Peak
        /// RSSPECAN_VAL_SSA_DET_MPE  (1) - Negative Peak
        /// RSSPECAN_VAL_SSA_DET_MID  (2) - Average Peak
        /// RSSPECAN_VAL_SSA_DET_RMS  (3) - RMS
        /// RSSPECAN_VAL_SSA_DET_FERR (4) - Frequency Error
        /// RSSPECAN_VAL_SSA_DET_CARR (5) - Carrier Power
        /// 
        /// Default Value: RSSPECAN_VAL_SSA_DET_PPE  (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Frequency Error: This parameter is only available for traces in the FM and PM result displays. If any other result display is selected, this parameter is disabled in the instrument (the driver doesn't influence this).
        /// </param>
        /// <param name="Value">
        /// Returns measured value.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetSSAVCOTransientResults(int Modulation_Type, int Parameter_To_Read, out double Value)
        {
            int pInvokeResult = PInvoke.GetSSAVCOTransientResults(this._handle, Modulation_Type, Parameter_To_Read, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns a list of measured voltages from ports.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// SOURce:VOLTage:SEQuence:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Voltage_Results">
        /// Returns a list of measured voltages from ports in the following format:
        /// 
        /// DC port 1 voltage
        /// DC port 2 voltage
        /// Tuning port 1 voltage
        /// Tuning port 2 voltage
        /// AUX port voltage
        /// 
        /// Note(s):
        /// 
        /// (1) This array must contain at least 5 elements.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetSSADCVoltageResults(double[] Voltage_Results)
        {
            int pInvokeResult = PInvoke.GetSSADCVoltageResults(this._handle, Voltage_Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns a list of measured currents from ports.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// SOURce:CURRent:SEQuence:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Current_Results">
        /// Returns a list of measured currents from ports in the following format:
        /// 
        /// DC port 1 current
        /// DC port 2 current
        /// Tuning port 1 current
        /// Tuning port 2 current
        /// AUX port current
        /// 
        /// Note(s):
        /// 
        /// (1) This array must contain at least 5 elements.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetSSADCCurrentResults(double[] Current_Results)
        {
            int pInvokeResult = PInvoke.GetSSADCCurrentResults(this._handle, Current_Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns a list of measured powers from ports.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// SOURce:POWer:SEQuence:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Power_Results">
        /// Returns a list of measured powers from ports in the following format:
        /// 
        /// DC port 1 power
        /// Tuning port 1 power
        /// AUX port power
        /// 
        /// Note(s):
        /// 
        /// (1) This array must contain at least 3 elements.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetSSADCPowerResults(double[] Power_Results)
        {
            int pInvokeResult = PInvoke.GetSSADCPowerResults(this._handle, Power_Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the VCO Loop Low Phase Noise slope, level and offset.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// (2) This function is only available with R&amp;S FSUP-B60 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_LPN_SLOPE_RES
        /// RSSPECAN_ATTR_SSA_LPN_LEVEL_RES
        /// RSSPECAN_ATTR_SSA_LPN_DC_OFFSET_RES
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]VCO:PDETector:LPNoise:SLOPe?
        /// [SENSe&lt;1|2&gt;:]VCO:PDETector:LPNoise:LEVel?
        /// [SENSe&lt;1|2&gt;:]VCO:PDETector:LPNoise:OFFSet?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Phase_Detector_Slope">
        /// Returns the Phase Detector Slope (=K_PD) of the low phase noise board.
        /// </param>
        /// <param name="Phase_Detector_Level">
        /// Returns the Phase Detector Level measured on the low phase noise board.
        /// </param>
        /// <param name="Phase_Detector_DC_Offset">
        /// Returns the Phase Detector DC Offset of the low phase noise board.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetSSAVCOLoopLowPhaseNoise(out double Phase_Detector_Slope, out double Phase_Detector_Level, out double Phase_Detector_DC_Offset)
        {
            int pInvokeResult = PInvoke.GetSSAVCOLoopLowPhaseNoise(this._handle, out Phase_Detector_Slope, out Phase_Detector_Level, out Phase_Detector_DC_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the measured phase detector voltage in residual phase noise measurements.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_PDET_VOLT_RES
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]VCO:PDETector:VOLTage?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Phase_Detector_Voltage">
        /// Returns the measured phase detector voltage in residual phase noise measurements.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetSSAVCOLoopPhaseDetectorVoltage(out double Phase_Detector_Voltage)
        {
            int pInvokeResult = PInvoke.GetSSAVCOLoopPhaseDetectorVoltage(this._handle, out Phase_Detector_Voltage);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the estimated input level at the AM detector.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSUP.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SSA_ADET_LEVEL_RES
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]VCO:ADETector:LEVel?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Input_Level">
        /// Returns the estimated input level at the AM detector.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetSSAVCOLoopAMDetectorInputLevel(out double Input_Level)
        {
            int pInvokeResult = PInvoke.GetSSAVCOLoopAMDetectorInputLevel(this._handle, out Input_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the instrument to the measuring receiver mode.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument[:SELect] MRECeiver
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int MeasuringReceiverMode()
        {
            int pInvokeResult = PInvoke.MeasuringReceiverMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the measuring frequency of the receiver for span = 0.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s): 
        /// RSSPECAN_ATTR_FREQUENCY_CENTER
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Receiver_Frequency">
        /// Defines the measuring frequency of the receiver for span = 0.
        /// 
        /// Valid Range (Only for Frequency Offset = 0.0):
        /// FSMR 3  - 0.0 Hz to 3.6 GHz
        /// FSMR 26 - 0.0 Hz to 26.5 GHz
        /// FSMR 50 - 0.0 Hz to 50 GHz
        /// 
        /// Default Value: 100.0e6 Hz
        /// 
        /// Note:
        /// 
        /// (1) This control is not range checked.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverFrequency(double Receiver_Frequency)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverFrequency(this._handle, Receiver_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the step width of the center frequency.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FREQUENCY_CENTER_STEP
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer:STEP
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Frequency_Step_Size">
        /// Defines the step width of the center frequency.
        /// 
        /// Valid Range: 0.0 Hz to fmax Hz
        /// 
        /// Default Value: 50.0e3 Hz
        /// 
        /// Note:
        /// 
        /// (1) This control is not range checked.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverFrequencyStepSize(double Frequency_Step_Size)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverFrequencyStepSize(this._handle, Frequency_Step_Size);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the frequency range for displaying the RF spectrum.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_FREQ_SPAN
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ADEMod:SPECtrum:SPAN:ZOOM
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Frequency_Span">
        /// Sets the frequency range for displaying the RF spectrum.
        /// 
        /// Valid Range: 0.0 Hz to fmax Hz
        /// 
        /// Default Value: 5.0e6 Hz
        /// 
        /// Note:
        /// 
        /// (1) This control is not range checked.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverFrequencySpan(double Frequency_Span)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverFrequencySpan(this._handle, Frequency_Span);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the maximum frequency range for displaying the RF spectrum.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_FREQ_FULL_SPAN
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ADEMod:SPECtrum:SPAN:ZOOM MAX
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverFrequencyFullSpan()
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverFrequencyFullSpan(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the maximum frequency range for displaying the RF spectrum that was determined from the FM demodulation data. The maximum span corresponds to the measurement bandwidth of analog demodulation.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_FREQ_MAX_SPAN
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ADEMod:SPECtrum:SPAN[:MAXimum]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Frequency_Max_Span">
        /// Sets the maximum frequency range for displaying the RF spectrum that was determined from the FM demodulation data. The maximum span corresponds to the measurement bandwidth of analog demodulation
        /// 
        /// Valid Range: 
        ///   Span     ... Sampling rate
        /// 
        ///   10 MHz   ... 32 MHz
        ///   8 MHz    ... 16 MHz
        ///   5 MHz    ... 8 MHz
        ///   3 MHz    ... 4 MHz
        ///   1.6 MHz  ... 2 MHz
        ///   800 kHz  ... 1 MHz
        ///   400 kHz  ... 500 kHz
        ///   200 kHz  ... 250 kHz
        ///   100 kHz  ... 125 kHz
        ///   50 kHz   ... 62.5 kHz
        ///   25 kHz   ... 31.25 kHz
        ///   12.5 kHz ... 15.625 kHz
        /// 
        /// 
        /// Default Value: 5.0e6 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverFrequencyMaxSpan(double Frequency_Max_Span)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverFrequencyMaxSpan(this._handle, Frequency_Max_Span);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the center frequency and span for result display AF spectrum. The span is limited to half the measurement bandwidth of analog demodulation (SENS:ADEM:BAND).
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_AF_CENTER
        /// RSSPECAN_ATTR_FMDEM_AF_SPAN
        /// 
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ADEMod:AF:CENTer
        /// [SENSe&lt;1|2&gt;:]ADEMod:AF:SPAN
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="AF_Center">
        /// Sets the center frequency for result display AF spectrum.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 1.25e6 Hz
        /// 
        /// </param>
        /// <param name="AF_Span">
        /// Sets the span for result display AF spectrum.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 2.5e6 Hz
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverAFCenterSpan(double AF_Center, double AF_Span)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverAFCenterSpan(this._handle, AF_Center, AF_Span);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the start and stop frequencies for result display AF spectrum. The stop frequency is limited to half the measurement bandwidth of analog demodulation (SENS:ADEM:BAND).
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_AF_START
        /// RSSPECAN_ATTR_FMDEM_AF_STOP 
        /// 
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ADEMod:AF:STARt
        /// [SENSe&lt;1|2&gt;:]ADEMod:AF:STOP
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="AF_Start">
        /// Sets the start frequency for result display AF spectrum.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0 Hz
        /// 
        /// </param>
        /// <param name="AF_Stop">
        /// Sets the stop frequency for result display AF spectrum.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 2.5e6 Hz
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverAFStartStop(double AF_Start, double AF_Stop)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverAFStartStop(this._handle, AF_Start, AF_Stop);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the maximum span for result display AF spectrum.
        /// 
        /// The maximum span corresponds to half the measurement bandwidth of analog demodulation (SENS:ADEM:BAND).
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_AF_FULL_SPAN
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ADEMod:AF:SPAN:FULL
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverAFFullSpan()
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverAFFullSpan(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function executes an automatic signal search.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_AUTO_SIGNAL_SEARCH
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:CW:AFC ONCE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int MeasuringReceiverAutoSignalSearch()
        {
            int pInvokeResult = PInvoke.MeasuringReceiverAutoSignalSearch(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates a function to track a drifting signal. If the measured signal frequency is too far away from the tuned center frequency and the level is above the threshold, the center frequency of the receiver will be changed to the new signal frequency.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_SIGNAL_TRACK
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:CW:AFC ON
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int MeasuringReceiverSignalTrack()
        {
            int pInvokeResult = PInvoke.MeasuringReceiverSignalTrack(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the reference level.
        /// 
        /// The unit depends on the setting defined with CALCulate:UNIT.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_REF_LEVEL
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:RLEVel
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIN_A (1) - A
        /// RSSPECAN_VAL_WIN_B (2) - B
        /// 
        /// Default Value: RSSPECAN_VAL_WIN_A (1)
        /// </param>
        /// <param name="Ref_Level">
        /// Defines the reference level.
        /// 
        /// Valid Range: -130.0 dBm to 30.0 dBm
        /// 
        /// Default Value: -20.0 dBm
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverRefLevel(int Window, double Ref_Level)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverRefLevel(this._handle, Window, Ref_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the autorange function on and off.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_ATTENUATION_AUTO
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:ATTenuation:AUTO ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIN_A (1) - A
        /// RSSPECAN_VAL_WIN_B (2) - B
        /// 
        /// Default Value: RSSPECAN_VAL_WIN_A (1)
        /// </param>
        /// <param name="Auto_Range">
        /// Switches the autorange function on and off.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Note(s):
        /// 
        /// (1) On -  Activates the autorange function; attenuation, IF gain and, if necessary, preamplification, are automatically matched to the applied RF signal. 
        ///  
        /// (2) Off - The attenuation is set manually.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverAutoRange(int Window, bool Auto_Range)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverAutoRange(this._handle, Window, System.Convert.ToUInt16(Auto_Range));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function determines whether the attenuator may be switched to 0 dB in manual or automatic attenuation control.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_10_DB_MIN
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:ATTenuation:PROTection[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIN_A (1) - A
        /// RSSPECAN_VAL_WIN_B (2) - B
        /// 
        /// Default Value: RSSPECAN_VAL_WIN_A (1)
        /// </param>
        /// <param name="_10_dB_Min">
        /// Determines whether the attenuator may be switched to 0 dB in manual or automatic attenuation control.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiver10dBMin(int Window, bool _10_dB_Min)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiver10dBMin(this._handle, Window, System.Convert.ToUInt16(_10_dB_Min));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates the preamplifier for the autorange function.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only in conjunction with FSMR.
        /// 
        /// (2) This function is available only with the R&amp;S FSU-B23/B25 preamplifier option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_AUTO_PREAMPLIFIER
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:GAIN:AUTO ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIN_A (1) - A
        /// RSSPECAN_VAL_WIN_B (2) - B
        /// 
        /// Default Value: RSSPECAN_VAL_WIN_A (1)
        /// </param>
        /// <param name="Auto_Preamplifier">
        /// Function activates the preamplifier for the autorange function.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Notes:
        /// 
        /// (1) On:
        /// The autorange function takes the preamplifier into account. The preamplifier is activated only when the attenuation of the attenuator has been reduced to the minimum settable value.
        /// 
        /// (2) Off:
        /// The autoranging does not take into account the preamplifier.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverAutoPreamplifier(int Window, bool Auto_Preamplifier)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverAutoPreamplifier(this._handle, Window, System.Convert.ToUInt16(Auto_Preamplifier));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the scaling of the Y axis in the current unit.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is not available in the AF spectrum display. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_Y_AXIS_SCALING
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACE&lt;1 to 3&gt;:Y[:SCALe]:PDIVision
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIN_A (1) - A
        /// RSSPECAN_VAL_WIN_B (2) - B
        /// RSSPECAN_VAL_WIN_C (3) - C
        /// RSSPECAN_VAL_WIN_D (4) - D
        /// 
        /// Default Value: RSSPECAN_VAL_WIN_A (1)
        /// </param>
        /// <param name="Scale_Per_Div">
        /// Defines the scaling of the Y axis in the current unit.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 1.0
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverDisplayScalePerDiv(int Window, double Scale_Per_Div)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverDisplayScalePerDiv(this._handle, Window, Scale_Per_Div);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the position of the reference value in the measurement window.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_REF_POSITION
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:RPOSition
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIN_A (1) - A
        /// RSSPECAN_VAL_WIN_B (2) - B
        /// 
        /// Default Value: RSSPECAN_VAL_WIN_A (1)
        /// </param>
        /// <param name="Reference_Position">
        /// Defines the position of the reference value in the measurement window.
        /// 
        /// Valid Range: 0.0 % to 100.0 %
        /// 
        /// Default Value: 50.0 %
        /// 
        /// Notes:
        /// 
        /// (1) This function defines the position of the reference line for the phase or frequency deviation (FM/PM) or the modulation depth (AM) on the y axis of the diagram. In the analyzer's default setting, this line corresponds to a frequency deviation of 0 Hz (FM) or a phase deviation of 0 rad (PM) or a modulation depth of 0% (AM).
        /// 
        /// (2) The entry is in percentage of the diagram height, where 100% corresponds to the top edge of the diagram. The default setting is 50% (diagram center) for displaying the FM, PM or AM signal, and 100% (top edge of diagram) for displaying the AF spectrum of the signal.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverDisplayReferencePosition(int Window, double Reference_Position)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverDisplayReferencePosition(this._handle, Window, Reference_Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the frequency or phase deviation (FM/PM) or the modulation depth (AM) on the reference line of the y axis. The reference value is set separately both for each FM, PM and AM signal display and for the AF spectrum display of the FM, PM and AM signal.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_REF_VALUE
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACE&lt;1 to 3&gt;:Y[:SCALe]:RVALue
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIN_A (1) - A
        /// RSSPECAN_VAL_WIN_B (2) - B
        /// 
        /// Default Value: RSSPECAN_VAL_WIN_A (1)
        /// </param>
        /// <param name="Reference_Value">
        /// Defines the frequency or phase deviation (FM/PM) or the modulation depth (AM) on the reference line of the y axis.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverDisplayReferenceValue(int Window, double Reference_Value)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverDisplayReferenceValue(this._handle, Window, Reference_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates the input of the reference point, where the phase is set to 0 rad.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSMR.
        /// 
        /// (2) This function is only available in the PM display with DC 
        /// coupling. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_ZERO_PHASE_REF_POS
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ADEMod:PM:RPOint:X
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Reference_Point">
        /// The reference point, where the phase is set to 0 rad.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Vaule: 10.0e-6 s
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverZeroPhaseRefPosition(double Reference_Point)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverZeroPhaseRefPosition(this._handle, Reference_Point);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function toggles between linear and logarithmic display in the measurement window.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSMR.
        /// 
        /// (2) This function is only available if the FM, PM or AM signal is displayed in the AF spectrum.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VERTICAL_SCALE
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y:SPACing LINear | LOGarithmic
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIN_A (1) - A
        /// RSSPECAN_VAL_WIN_B (2) - B
        /// 
        /// Default Value: RSSPECAN_VAL_WIN_A (1)
        /// </param>
        /// <param name="Display_Spacing">
        /// Pass the vertical scale that you want the spectrum analyzer to use.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TRACE_SPACE_LIN (0) - Linear
        /// RSSPECAN_VAL_TRACE_SPACE_LOG (1) - Logarithmic
        /// 
        /// Default Value: RSSPECAN_VAL_TRACE_SPACE_LOG (1)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverDisplaySpacing(int Window, int Display_Spacing)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverDisplaySpacing(this._handle, Window, Display_Spacing);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates the phase wrapping from -PI to +PI (-180 to +180).
        /// 
        /// Note:
        /// 
        /// (1) This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_DISPLAY_PHASE_WRAP
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FORMat PHASe | UPHase
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Phase_Wrap">
        /// Activates / Deactivates the phase wrapping to -PI to +PI (-180 to +180).
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverDisplayPhaseWrap(bool Phase_Wrap)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverDisplayPhaseWrap(this._handle, System.Convert.ToUInt16(Phase_Wrap));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the unit for PM signal display.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_UNIT_PM
        /// 
        /// Remote-control command(s):
        /// UNIT&lt;1|2&gt;:ANGLe RAD | DEG
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="PM_Unit">
        /// Selects the unit for PM signals.
        /// 
        /// Units: None
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_UNIT_ANGLE_DEG (0) - Degrees
        /// RSSPECAN_VAL_UNIT_ANGLE_RAD (1) - Radians
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_ANGLE_RAD (1)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverUnitPM(int PM_Unit)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverUnitPM(this._handle, PM_Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects between % and dB for displaying the THD and SINAD measurement results.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_UNIT_THD_SINAD
        /// 
        /// Remote-control command(s):
        /// UNIT&lt;1|2&gt;:THD DB | PCT
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="THD_SINAD_Unit">
        /// Selects between % and dB for displaying the THD and SINAD measurement results.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DB  (8) - dB
        /// RSSPECAN_VAL_UNIT_PCT (13) - %
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_UNIT_THD_SINAD
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_DB (8)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverUnitTHD_SINAD(int THD_SINAD_Unit)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverUnitTHD_SINAD(this._handle, THD_SINAD_Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the input coupling of the RF input between AC and DC.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AMPL_RF_COUPLING
        /// 
        /// Remote-control command(s):
        /// INPut:COUPling AC | DC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Input_Coupling">
        /// This control switches the input coupling of the RF input between AC and DC.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DC (0) - DC
        /// RSSPECAN_VAL_AC (1) - AC
        /// 
        /// Default Value: RSSPECAN_VAL_DC (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverRFInputCoupling(int Input_Coupling)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverRFInputCoupling(this._handle, Input_Coupling);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the input attenuation.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_ATTENUATION_AUTO
        /// RSSPECAN_ATTR_ATTENUATION
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:ATTenuation:AUTO ON | OFF
        /// INPut&lt;1|2&gt;:ATTenuation
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIN_A (1) - A
        /// RSSPECAN_VAL_WIN_B (2) - B
        /// 
        /// Default Value: RSSPECAN_VAL_WIN_A (1)
        /// </param>
        /// <param name="Input_Attenuation_Auto">
        /// Automatically sets the RF attenuation depending on the set reference level (ON state) or switches the input attenuation to manual control (OFF state).
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// </param>
        /// <param name="Input_Attenuation">
        /// Defines the input attenuation.
        /// 
        /// Valid Range: 0.0 dB to 75.0 dB
        /// 
        /// Default Value: 10.0 dB
        /// 
        /// Note(s):
        /// 
        /// (1) If auto attenuation is turned on, this value is ignored.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverRFAttenuation(int Window, bool Input_Attenuation_Auto, double Input_Attenuation)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverRFAttenuation(this._handle, Window, System.Convert.ToUInt16(Input_Attenuation_Auto), Input_Attenuation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures input mixer of the analyzer.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AMPL_MIXER_AUTO
        /// RSSPECAN_ATTR_AMPL_MIXER_LEVEL
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:MIXer:AUTO ON | OFF
        /// INPut&lt;1|2&gt;:MIXer[:POWer]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIN_A (1) - A
        /// RSSPECAN_VAL_WIN_B (2) - B
        /// 
        /// Default Value: RSSPECAN_VAL_WIN_A (1)
        /// </param>
        /// <param name="Mixer_Level_Auto">
        /// Enables/disables the automatic setup of the mixer level.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// </param>
        /// <param name="Mixer_Level">
        /// Defines the desired power at the input mixer of the analyzer.
        /// 
        /// Valid Range: 
        /// -100.0 to 5.0 dBm
        /// 
        /// Default Value: -25.0 dBm
        /// 
        /// Note(s):
        /// 
        /// (1) This control has effect only if Mixer Level Auto is set Off
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverMixerLevel(int Window, bool Mixer_Level_Auto, double Mixer_Level)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverMixerLevel(this._handle, Window, System.Convert.ToUInt16(Mixer_Level_Auto), Mixer_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the nominal input impedance of the instrument. The set impedance is taken into account in all level indications of results.
        /// 
        /// The setting 75 Ohm should be selected, if the 50 Ohm input impedance is transformed to a higher impedance using a 75 Ohm adapter of the RAZ type (= 25 Ohm in series to the input impedance of the instrument). The correction value in this case is 1.76 dB = 10 log ( 75 Ohm / 50 Ohm).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_INPUT_IMPEDANCE
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:IMPedance 50 | 75
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIN_A (1) - A
        /// RSSPECAN_VAL_WIN_B (2) - B
        /// 
        /// Default Value: RSSPECAN_VAL_WIN_A (1)
        /// </param>
        /// <param name="Input_Impedance">
        /// Specifies the input impedance.
        /// 
        /// Valid Values:
        /// 50, 75 Ohms
        /// 
        /// Default Value: 50 Ohms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverInputImpedance(int Window, double Input_Impedance)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverInputImpedance(this._handle, Window, Input_Impedance);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the demodulation bandwidth settings.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_DEMOD_BW_AUTO
        /// RSSPECAN_ATTR_MRECEIVER_DEMOD_BW
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:DEMod:AUTO ON | OFF
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:DEMod
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIN_A (1) - A
        /// RSSPECAN_VAL_WIN_B (2) - B
        /// 
        /// Default Value: RSSPECAN_VAL_WIN_A (1)
        /// </param>
        /// <param name="Demod_BW_Auto">
        /// Activates automatic setting of the demodulation bandwidth.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// </param>
        /// <param name="Demod_BW">
        /// Defines the demodulation bandwidth of the instrument for analog demodulation.
        /// 
        /// The available demodulation bandwidths are determined by the existing sampling rates.
        /// 
        /// Rounded demodulation bandwidth ... Sampling rate
        /// 
        ///   10 MHz   ... 32 MHz
        ///   8 MHz    ... 16 MHz
        ///   5 MHz    ... 8 MHz
        ///   3 MHz    ... 4 MHz
        ///   1.6 MHz  ... 2 MHz
        ///   800 kHz  ... 1 MHz
        ///   400 kHz  ... 500 kHz
        ///   200 kHz  ... 250 kHz
        ///   100 kHz  ... 125 kHz
        ///   50 kHz   ... 62.5 kHz
        ///   25 kHz   ... 31.25 kHz
        ///   12.5 kHz ... 15.625 kHz
        /// 
        /// Default Value: 1.6e6 Hz
        /// 
        /// This control configures the demodulation bandwidth settings.
        /// 
        /// Valid Values:
        /// 1,3E+004, 2,5E+004, 5,0E+004, 1,0E+005, 2,0E+005, 4,0E+005, 8,0E+005, 1,6E+006, 3,0E+006, 5,0E+006, 8,0E+006, 1,0E+007
        /// 
        /// Default Value: 1.6e6Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MRECEIVER_DEMOD_BW
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available if Demod BW Auto is set to Off.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverDemodBW(int Window, bool Demod_BW_Auto, double Demod_BW)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverDemodBW(this._handle, Window, System.Convert.ToUInt16(Demod_BW_Auto), Demod_BW);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the IF bandwidth settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RESOLUTION_BANDWIDTH_AUTO
        /// RSSPECAN_ATTR_RESOLUTION_BANDWIDTH
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth[:RESolution]:AUTO ON | OFF
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth[:RESolution]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIN_A (1) - A
        /// RSSPECAN_VAL_WIN_B (2) - B
        /// 
        /// Default Value: RSSPECAN_VAL_WIN_A (1)
        /// </param>
        /// <param name="IF_BW_Auto">
        /// Define whether to couple IF bandwidth to the selected receive frequency.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// </param>
        /// <param name="IF_BW">
        /// Defines the IF bandwidth.
        /// 
        /// Valid Range: 10.0 Hz to 50.0e6 MHz
        /// 
        /// Default Value: 1.0e6 Hz
        /// 
        /// Note:
        /// 
        /// (1) Analog resolution filters of 10 Hz to 20 MHz in 1, 2, 3, 5, 10 steps are available. Additionally there is a 50 MHz resolution bandwidth. These filters are implemented as 5-circuit LC filters in the range from 300 kHz to 10 MHz and as digital filters with analog characteristic in the range of 10 Hz to 100 kHz.
        /// 
        /// (2) This control is only available if IF BW Auto is set to Off.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverIFBW(int Window, bool IF_BW_Auto, double IF_BW)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverIFBW(this._handle, Window, System.Convert.ToUInt16(IF_BW_Auto), IF_BW);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the measurement time settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SWEEP_TIME_AUTO
        /// RSSPECAN_ATTR_SWEEP_TIME
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:TIME:AUTO ON | OFF
        /// [SENSe&lt;1|2&gt;:]SWEep:TIME
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIN_A (1) - A
        /// RSSPECAN_VAL_WIN_B (2) - B
        /// 
        /// Default Value: RSSPECAN_VAL_WIN_A (1)
        /// </param>
        /// <param name="Meas_Time_Auto">
        /// Controls the automatic coupling of the sweep time to the frequency span and bandwidth settings.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// </param>
        /// <param name="Meas_Time">
        /// Defines the sweep time. 
        /// 
        /// Valid Range: 1.0e-6 s to 16.0E3 s
        /// 
        /// Default Value: 0.2 s
        /// 
        /// Note:
        /// 
        /// (1) The available time values are different in the frequency 
        /// domain (2.5 ms to 16000s with span &gt; 0) and in the time domain (1 us to 16000s with span = 0).
        /// 
        /// (2) This control is only available if Meas Time Auto is set to Off.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverMeasTime(int Window, bool Meas_Time_Auto, double Meas_Time)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverMeasTime(this._handle, Window, System.Convert.ToUInt16(Meas_Time_Auto), Meas_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures analog demodulator measuring time parameters.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_ANALOG_DEMOD_MEAS_TIME
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:MTIMe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measuring_Time">
        /// This control sets the measuring time for the analog Demodulation.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 62.625E-6 s
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverADemodMeasuringTime(double Measuring_Time)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverADemodMeasuringTime(this._handle, Measuring_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the resolution bandwidth for spectrum represetation that was determined from the analog demodulation data.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_RES_BW_MANUAL
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ADEMod:SPECtrum:BANDwidth|BWIDth[:RESolution]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Resolution_BW">
        /// Sets the resolution bandwidth for spectrum represetation.
        /// 
        /// Valid Range: 1.0 Hz to 10.0e6 MHz
        /// 
        /// Default Value: 61.2e3 Hz
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverResolutionBWManual(double Resolution_BW)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverResolutionBWManual(this._handle, Resolution_BW);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the measuring receiver trigger settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_TRIGGER_SOURCE
        /// RSSPECAN_ATTR_MRECEIVER_TRIGGER_OFFSET
        /// RSSPECAN_ATTR_TRIGGER_SLOPE
        /// RSSPECAN_ATTR_TRIGGER_IFP_LEVEL
        /// RSSPECAN_ATTR_MRECEIVER_TRIGGER_LEVEL_AM
        /// RSSPECAN_ATTR_ADEM_TRIGGER_FM_LEVEL
        /// RSSPECAN_ATTR_ADEM_TRIGGER_PM_LEVEL
        /// RSSPECAN_ATTR_VIDEO_TRIGGER_LEVEL
        /// RSSPECAN_ATTR_MRECEIVER_TRIGGER_LEVEL_AUDIO
        /// 
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:SOURce IMMediate | EXTernal | VIDeo | IFPower | FM | AM | PM | AUDio
        /// TRIGger&lt;1|2&gt;[:SEQuence]:HOLDoff
        /// TRIGger&lt;1|2&gt;[:SEQuence]:SLOPe POSitive | NEGative
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:IFPower
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:AM
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:FM
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:PM
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:VIDeo
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:AUDio
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Trigger_Source">
        /// Specifies the source of the trigger signal that causes the analyzer to leave the Wait-For-Trigger state.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TR_SRC_FREE  (0) - Free Run 
        /// RSSPECAN_VAL_TR_SRC_EXT   (1) - External   
        /// RSSPECAN_VAL_TR_SRC_IF    (2) - IF Power
        /// RSSPECAN_VAL_TR_SRC_FM    (3) - FM
        /// RSSPECAN_VAL_TR_SRC_PM    (4) - PM
        /// RSSPECAN_VAL_TR_SRC_AM    (5) - AM
        /// RSSPECAN_VAL_TR_SRC_VIDEO (6) - Video
        /// RSSPECAN_VAL_TR_SRC_AUDIO (7) - Audio
        /// 
        /// Default value: RSSPECAN_VAL_TR_SRC_FREE (0)
        /// </param>
        /// <param name="Trigger_Offset">
        /// Sets the length of the trigger delay.
        /// 
        /// Units: Volts
        /// 
        /// Valid Values: -100.0 to 100.0
        /// 
        /// Default Value: 0.0
        /// 
        /// Note:
        /// (1) A negative delay time (pretrigger) can be set in the time domain (span &lt; 0 Hz) only.
        /// 
        /// (2) This control has effect for Immediate, External and IF Power Trigger only.
        /// </param>
        /// <param name="Trigger_Polarity">
        /// Specifies the polarity for triggering.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_NEG (0) - Negative
        /// RSSPECAN_VAL_POS (1) - Positive
        /// 
        /// Default value: RSSPECAN_VAL_NEG (0)
        /// </param>
        /// <param name="Trigger_Value">
        /// This command sets the level of the video trigger source.
        /// 
        /// Valid Values: 
        ///  for IF Power
        ///    -50 to 30 dBm
        ///  for FM
        ///    -10.0e+6 to 10.0e+6 Hz
        ///  for PM
        ///    -1000 to +1000 rad|deg
        ///  for AM
        ///    -100 to +30 dBm
        ///  for Audio
        ///    -4 to +4 V
        ///  for Video
        ///    0 to 100 %
        ///  
        /// Default Value: 50.0
        /// 
        /// Note:
        /// 
        /// (1) This control has no effect for next triggers:
        /// Free Run
        /// External
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverTrigger(int Trigger_Source, double Trigger_Offset, int Trigger_Polarity, double Trigger_Value)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverTrigger(this._handle, Trigger_Source, Trigger_Offset, Trigger_Polarity, Trigger_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates the RF LEVEL mode (selective level measurement).
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_RF_LEVEL_MODE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:AC:STATe ON
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int MeasuringReceiverRFLevelMode()
        {
            int pInvokeResult = PInvoke.MeasuringReceiverRFLevelMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function carries out the various steps of absolute level calibration in the Tuned RF Level mode.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_CAL_ABS_POWER
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:COLLect[:ACQuire] INPut | PMETer | PSPLitter
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Cal_Abs_Power">
        /// Selects the various steps of absolute level calibration in the Tuned RF Level mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_CAL_ABS_POW_INPUT (0) - RF Input
        /// RSSPECAN_VAL_CAL_ABS_POW_METER (1) - Power Meter
        /// RSSPECAN_VAL_CAL_ABS_POW_SPLIT (2) - Power Splitter
        /// 
        /// Default Value: RSSPECAN_VAL_CAL_ABS_POW_INPUT (0)
        /// 
        /// Notes:
        /// 
        /// (1) RF Input:
        /// Measures the reference level with a power meter.
        /// 
        /// (2) Power Meter:
        /// Measures the comparison value at the receiver's RF input and corrects it to the reference level.
        /// 
        /// (3) Power Splitter:
        /// Measures the reference level and then the comparison value. For this measurement, the power meter and the receiver's RF input must be connected to the DUT via a power splitter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverCalibrationAbsPower(int Cal_Abs_Power)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverCalibrationAbsPower(this._handle, Cal_Abs_Power);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts a calibration of the adjacent range in order to extend the calibrated measurement range. The RECAL function is available if the measured level is in the range adjacent to the calibrated range. This condition is indicated by the STATus:OPERation:PCALibration register.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_EXPAND_CAL_RANGE
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:ATTenuation:AUTO RECal
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverExpandCalibratedRange()
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverExpandCalibratedRange(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts a single autoranging procedure. The input attenuation, IF gain and, if a preamplifier is installed, the preamplification are adjusted in order to optimally match the receiver sensitivity to the input signal to be measured.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_ADJUST_INPUT_RANGE
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:ATTenuation:AUTO ONCE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverAdjustInputRange()
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverAdjustInputRange(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads the names of all saved correction sets and the corresponding settings.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// No attributes
        /// 
        /// Remote-control command(s):
        /// MEMory[:CORRection]:CATalog?
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the string you specify for the catalog list parameter.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer.
        /// 
        /// 
        /// 
        /// </param>
        /// <param name="Correction_List">
        /// Returns the names of all saved correction sets and the corresponding settings.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryMeasuringReceiverCorrectionTable(int Buffer_Size, System.Text.StringBuilder Correction_List)
        {
            int pInvokeResult = PInvoke.QueryMeasuringReceiverCorrectionTable(this._handle, Buffer_Size, Correction_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function enables the administration of automatically saved correction sets in operating mode Tuned RF Level.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_CORR_SEL
        /// RSSPECAN_ATTR_MRECEIVER_CORR_DEL
        /// RSSPECAN_ATTR_MRECEIVER_CORR_DEL_ALL
        /// 
        /// Remote-control command(s):
        /// MEMory[:CORRection]:SELect
        /// MEMory[:CORRection]:DELete[:NAME]
        /// MEMory[:CORRection]:DELete:ALL
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Operation">
        /// This control selects operation to be performed.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_CORR_SEL     (0) - Select (Default Value)
        /// RSSPECAN_VAL_CORR_DEL     (1) - Delete
        /// RSSPECAN_VAL_CORR_DEL_ALL (2) - Delete All
        /// 
        /// Default Value: RSSPECAN_VAL_CORR_SEL (0)
        /// 
        /// Notes:
        /// 
        /// (1) Select:
        /// Activates a correction data set with the given name for Tuned RF Level of FSMR.
        /// 
        /// (2) Delete:
        /// Deletes a correction data set with the given name for Tuned RF Level of FSMR.
        /// 
        /// (3) Delete All:
        /// Deletes all correction data sets for Tuned RF Level of FSMR.
        /// 
        /// </param>
        /// <param name="Path">
        /// Select the path.
        /// 
        /// Valid Range: NULL pointer checking
        /// 
        /// Default Value: none
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverCorrection(int Operation, string Path)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverCorrection(this._handle, Operation, Path);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the relative level measurement in the RF LEVEL mode.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_REL_LEVEL_MEAS_STATE
        /// RSSPECAN_ATTR_MRECEIVER_REL_LEVEL_MEAS_VALUE
        /// RSSPECAN_ATTR_MRECEIVER_REL_LEVEL_MEAS_UNIT
        /// 
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:AC:REFerence[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]POWer:AC:REFerence
        /// UNIT&lt;1|2&gt;:POWer:RATio DB | PCT
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Relative_Level_Measurement">
        /// Switches on absolute or relative measured-value display in the RF LEVEL mode.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <param name="Reference_Value">
        /// Defines the reference value for the relative measured-value display in the RF LEVEL mode.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: -28.0 dBm
        /// 
        /// </param>
        /// <param name="Relative_Unit">
        /// Selects the unit for the display of the relative measurement results.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_REC_LEV_UNIT_DB  (0) - dB
        /// RSSPECAN_VAL_REC_LEV_UNIT_PCT (1) - PCT
        /// 
        /// Default Value: RSSPECAN_VAL_REC_LEV_UNIT_DB (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverRelativeLevelMeasurement(bool Relative_Level_Measurement, double Reference_Value, int Relative_Unit)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverRelativeLevelMeasurement(this._handle, System.Convert.ToUInt16(Relative_Level_Measurement), Reference_Value, Relative_Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function uses the current measured value as a reference for the relative measured-level display in the RF LEVEL mode.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_REL_LEVEL_MEAS
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:AC:REFerence:AUTO ONCE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverMeasuredLevelToReference()
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverMeasuredLevelToReference(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches averaging of the level measurement on and off. It also determines the number of level measurements across which the average value is formed (average count).
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attributes(s):
        /// RSSPECAN_ATTR_MRECEIVER_LEVEL_MEAS_AVER_STATE
        /// RSSPECAN_ATTR_MRECEIVER_LEVEL_MEAS_AVER_COUNT
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:AC:AVERage[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]POWer:AC:AVERage:COUNt
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Averaging">
        /// Switches on or off averaging of the measured level values.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <param name="Averaging_Count">
        /// Defines, the number of measurements that are to be started as "single measurements" and used to form an average.
        /// 
        /// Valid Range: 0 to 30000
        /// 
        /// Default Value: 10
        /// 
        /// Note:
        /// 
        ///   Average count = 0 indicates continuous averaging
        ///   Average count = 1 indicates no averaging
        ///   Average count &gt; 1 indicates averaging across the specified
        ///                     number of measurements
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverLevelMeasurementAveraging(bool Averaging, int Averaging_Count)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverLevelMeasurementAveraging(this._handle, System.Convert.ToUInt16(Averaging), Averaging_Count);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates/deactivates the auto average mode of the RF Level Mode.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_LEVEL_AUTO_AVER
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:AC:AVERage:AUTO
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="State">
        /// Sets the auto average mode state.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverLevelAutoAverage(bool State)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverLevelAutoAverage(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the entries of the auto averaging config table. The entries are defined in pairs of level and average count. Up to 20 entries are allowed in descending order of level values.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// No attributes.
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:AC:AVERage:DATA &lt;level 1&gt;, &lt;count 1&gt;,.....,&lt;level n&gt;, &lt;count n&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Array_Size">
        /// Sets the size of Level and Count arrays.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value: 20
        /// </param>
        /// <param name="Level">
        /// Sets the array of levels for averaging.
        /// 
        /// Valid Values:
        /// any array
        /// 
        /// Note:
        /// 
        /// (1) The number of elements has to be according Array Size value.
        /// </param>
        /// <param name="Count">
        /// Sets the array of averaging counts.
        /// 
        /// Valid Values:
        /// any array
        /// 
        /// Note:
        /// 
        /// (1) The number of elements has to be according Array Size value.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverLevelAutoAverageData(int Array_Size, double[] Level, int[] Count)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverLevelAutoAverageData(this._handle, Array_Size, Level, Count);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects between different detector functions. DETector NARRow activates an FFT in the set demodulation bandwidth. The displayed level value is derived from the FFT peak value. The effective measurement bandwidth for this type of measurement is approx. 4 / measurement time. During the
        /// measurement time, the signal must remain in the effective bandwidth. DETector WIDE activates a level measurement in the set demodulation bandwidth. The level is obtained by an RMS (root mean square) computation of all measurement points Select DETector WIDE if the signal sources are unstable
        /// (frequency drift, residual FM).
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_LEVEL_DETECTOR
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DETector[:FUNCtion] NARRow | WIDE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Detector_Type">
        /// Sets the detector type.
        /// 
        /// Valid Values:
        /// 
        /// RSSPECAN_VAL_DET_WD_NARROW (0) - Narrow (Default Value)
        /// RSSPECAN_VAL_DET_WD_WIDE   (1) - Wide
        /// 
        /// Default Value: RSSPECAN_VAL_DET_WD_NARROW (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverLevelDetector(int Detector_Type)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverLevelDetector(this._handle, Detector_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// $This function switches on/off the auto recal. If active a recal operation is performed automatically if the signal level reaches a recal range. 
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_AUTO_RECAL
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2:ATTenuation:RECal:AUTO[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Auto_Recal">
        /// Switches on/off the auto recal. If active a recal operation is performed automatically if the signal level reaches a recal range. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverAutoRecal(bool Auto_Recal)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverAutoRecal(this._handle, System.Convert.ToUInt16(Auto_Recal));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches measurements with a power sensor on or off.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RECEIVER_PWR_METER
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]PMETer[:STATe] ON | OFF
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="State">
        /// Sets the measurements with a power sensor on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int MeasuringReceiverPWRMeter(bool State)
        {
            int pInvokeResult = PInvoke.MeasuringReceiverPWRMeter(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the power meter measurement parameters.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_PWM_MEAS_TIME
        /// RSSPECAN_ATTR_MRECEIVER_PWM_MEAS_COUNT
        /// RSSPECAN_ATTR_MRECEIVER_PWM_MEAS_COUNT_MANUAL
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]PMETer:MTIMe SHORt | NORMal | LONG
        /// [SENSe&lt;1|2&gt;:]PMETer:MTIMe:AVERage:COUNt
        /// [SENSe&lt;1|2&gt;:]PMETer:MTIMe:AVERage[:STATe]
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement_Time">
        /// Sets the measurement time of the power sensor.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_PWMET_MEASTIME_NORM   (0) - Normal
        /// RSSPECAN_VAL_PWMET_MEASTIME_SHORT  (1) - Short
        /// RSSPECAN_VAL_PWMET_MEASTIME_LONG   (2) - Long
        /// RSSPECAN_VAL_PWMET_MEASTIME_MANUAL (3) - Manual
        /// 
        /// Default Value: RSSPECAN_VAL_PWMET_MEASTIME_NORM   (0)
        /// 
        /// </param>
        /// <param name="Number_Of_Readings">
        /// Sets the number of averages for the manual average mode of the power meter. 
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 0
        /// 
        /// Note:
        /// (1) This value is valid only if Measurement Time is set to Manual.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverPWMMeasurement(int Measurement_Time, int Number_Of_Readings)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverPWMMeasurement(this._handle, Measurement_Time, Number_Of_Readings);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function zeroes the power sensor.
        /// 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_PWM_ZERO
        /// 
        /// Remote-control command(s):
        /// CALibration:PMETer:ZERO:AUTO ONCE
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverPWMZero()
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverPWMZero(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the relative measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_PWM_RELATIVE_VALUE
        /// RSSPECAN_ATTR_MRECEIVER_PWM_RELATIVE_UNIT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PMETer:RELative:STATe ON
        /// CALCulate&lt;1|2&gt;:PMETer:RELative[:MAGNitude]
        /// UNIT&lt;1|2&gt;:PMETer:POWer:RATio DB | PCT
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Reference_Value">
        /// Sets the reference value for relative measurements.
        /// 
        /// Valid Values:
        /// -200.0 to 200.0
        /// 
        /// Default Value: 0.0 dB
        /// 
        /// </param>
        /// <param name="Unit">
        /// Sets the unit for relative measurements with a power sensor.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DB  (8)  - dB (Default Value)
        /// RSSPECAN_VAL_UNIT_PCT (13) - % 
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_DB (8)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverPWMRelative(double Reference_Value, int Unit)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverPWMRelative(this._handle, Reference_Value, Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the absolute measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_PWM_ABSOLUTE_UNIT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PMETer:RELative:STATe OFF
        /// UNIT&lt;1|2&gt;:PMETer:POWer DBM | W
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Unit">
        /// Sets the unit for absolute measurements with a power sensor.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DBM  (0) - dBm (Default Value)
        /// RSSPECAN_VAL_UNIT_WATT (2) - W
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_DBM (0)
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverPWMAbsolute(int Unit)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverPWMAbsolute(this._handle, Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function accepts the current result as a reference value for relative measurements.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_PWM_MEAS_TO_REFERENCE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PMETer:RELative[:MAGNitude]:AUTO ONCE
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverPWMMeasToRef()
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverPWMMeasToRef(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates and deactivates automatic consideration of a power splitter that splits the present RF signal between the power meter and the R&amp;S FSMR RF input. 
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_PWM_USE_POWER_SPLITTER
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CORRection:PLOSs:INPut[:STATe] ON | OFF 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="State">
        /// Activates and  deactivates automatic consideration of a power splitter.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverPWMUsePowerSplitter(bool State)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverPWMUsePowerSplitter(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the interpolation points for the frequency response correction for the power splitter on the path between the signal source and the power meter. The points are to be entered as a sequence of frequency/level pairs, with the frequencies to be specified in ascending order. 
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// No attributes
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:PLOSs:INPut &lt;freq&gt;,&lt;level&gt;,...,&lt;freq&gt;,&lt;level&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Array_Size">
        /// Sets the size of Frequency and Insertion Loss arrays.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value: 20
        /// </param>
        /// <param name="Frequency">
        /// Sets the array of frequencies for power splitter, the frequencies to be specified in ascending order.
        /// 
        /// Note:
        /// 
        /// (1) The number of elements has to be according Array Size value.
        /// </param>
        /// <param name="Insertion_Loss">
        /// Sets the array of insertion loss for power splitter.
        /// 
        /// Note:
        /// 
        /// (1) The number of elements has to be according Array Size value.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverPWMPowerSplitterInsertionLoss(int Array_Size, double[] Frequency, double[] Insertion_Loss)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverPWMPowerSplitterInsertionLoss(this._handle, Array_Size, Frequency, Insertion_Loss);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the insertion loss of the power splitter between the signal source and the RF input of the receiver. The value entered here only serves for plausibility checking during absolute value calibration. The correct value is then determined during absolute value calibration.  
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_PWM_PATH_2_INSERTION_LOSS
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CORRection:PLOSs:INPut:SPATh
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Insertion_Loss">
        /// Defines the insertion loss of the power splitter between the signal source and the RF input of the receiver.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0.0 dB
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverPWMPath2InsertionLoss(double Insertion_Loss)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverPWMPath2InsertionLoss(this._handle, Insertion_Loss);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates/deactivates the mismatch correction between the power sensor module NRPZ27 or NRP-Z37 and the FSMR RF input.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSMR.
        /// 
        /// (2) The function is only available under certain conditions: 
        /// -  The RF attenuation must be 10 dB or 30 dB. The RF attenuation 
        /// default setting is 30 dB. 
        /// - A correction file with VSWR values must be present on the internal memory. If the correction file is not present, the softkey will not be present.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_PWM_VSWR_CORRECTION
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:VSWR[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="State">
        /// Sets the auto average mode state.
        /// 
        /// Valid Values:
        /// 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On (Default Value)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverPWMVSWRCorrection(bool State)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverPWMVSWRCorrection(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the measurements with an external power meter.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_PWR_EXTERNAL_METER
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]PMETer:EXTern[:STATe] ON | OFF
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="State">
        /// This control switches on or off the measurements with an external power meter.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverPWRExternalMeter(bool State)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverPWRExternalMeter(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the remote settings of the power sensor.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_PWM_REMOTE_SET_TYPE
        /// RSSPECAN_ATTR_MERECEIVER_PWM_REMOTE_SET_ADDRESS
        /// 
        /// Remote-control command(s):
        /// SYSTem:COMMunicate:RDEVice:PMETer:TYPE
        /// SYSTem:COMMunicate:GPIB:RDEVice:PMETer:ADDRess
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Type">
        /// Selects the type of power meter used from a list. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_REMOTE_SET_TYPE_EPM441A  (0) - EPM441A
        /// RSSPECAN_VAL_REMOTE_SET_TYPE_HP437B   (1) - HP437B
        /// RSSPECAN_VAL_REMOTE_SET_TYPE_HP438A   (2) - HP438A
        /// RSSPECAN_VAL_REMOTE_SET_TYPE_MLL2438A (3) - ML2438A
        /// RSSPECAN_VAL_REMOTE_SET_TYPE_NRVD     (4) - NRVD
        /// RSSPECAN_VAL_REMOTE_SET_TYPE_NRVS     (5) - NRVS
        /// 
        /// Default Value: RSSPECAN_VAL_REMOTE_SET_TYPE_NRVD (4) - NRVD
        /// </param>
        /// <param name="GPIB_Address">
        /// Sets the IEC/IEEE bus address via which the external power meter is addressed for remote-control. 
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 5
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverPWMRemoteSettings(int Type, int GPIB_Address)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverPWMRemoteSettings(this._handle, Type, GPIB_Address);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the sensor calibration factor settings. 
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_PWM_SENSOR_CAL_FACTOR_SENSOR
        /// RSSPECAN_ATTR_MRECEIVER_PWM_SENSOR_CAL_FACTOR_A_LABEL
        /// RSSPECAN_ATTR_MRECEIVER_PWM_SENSOR_CAL_FACTOR_B_LABEL
        /// 
        /// Remote-control command(s):
        /// SYSTem:COMMunicate:RDEVice:PMETer:CFACtor:SELect ASENsor | BSENsor
        /// SYSTem:COMMunicate:RDEVice:PMETer:CFACtor:ASENsor
        /// SYSTem:COMMunicate:RDEVice:PMETer:CFACtor:ASENsor:LABel
        /// SYSTem:COMMunicate:RDEVice:PMETer:CFACtor:BSENsor 
        /// SYSTem:COMMunicate:RDEVice:PMETer:CFACtor:BSENsor:LABel
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Use_Sensor">
        /// Switches between the calibration factors of the sensors A and B. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SENSOR_A (1) - A
        /// RSSPECAN_VAL_SENSOR_B (2) - B
        /// 
        /// Default Value: RSSPECAN_VAL_SENSOR_A (1) - A
        /// </param>
        /// <param name="Sensor_Label">
        /// Defines a name for correction table for the external power meter. This name may designate a specific power sensor, for example.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// </param>
        /// <param name="Array_Size">
        /// Sets the size of Frequency and Cal Factor arrays.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value: 20
        /// </param>
        /// <param name="Frequency">
        /// Sets the array of frequencies for frequency response correction for the external power meter, the frequencies to be specified in ascending order.
        /// 
        /// Note:
        /// 
        /// (1) The number of elements has to be according Array Size value.
        /// </param>
        /// <param name="Cal_Factor">
        /// Sets the array of insertion loss for frequency response correction for the external power meter. The correction factors are percentage values. 
        /// 
        /// Note:
        /// 
        /// (1) The number of elements has to be according Array Size value.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverPWMSensorCalFactor(int Use_Sensor, string Sensor_Label, int Array_Size, double[] Frequency, double[] Cal_Factor)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverPWMSensorCalFactor(this._handle, Use_Sensor, Sensor_Label, Array_Size, Frequency, Cal_Factor);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the POWER REF reference source on the front panel.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_PWM_REF_OUTPUT
        /// 
        /// Remote-control command(s):
        /// OUTPut:REFerence:STATe ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Reference_Output">
        /// Switches on or off the POWER REF reference source on the front panel.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverPWMReferenceOutput(bool Reference_Output)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverPWMReferenceOutput(this._handle, System.Convert.ToUInt16(Reference_Output));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function is setting parameters of the AF filters in the audio signal path.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_AF_FIL_HP
        /// RSSPECAN_ATTR_MRECEIVER_AF_FIL_LP
        /// RSSPECAN_ATTR_MRECEIVER_AF_FIL_HP_FREQ
        /// RSSPECAN_ATTR_MRECEIVER_AF_FIL_LP_FREQ
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FILTer:HPASs[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]FILTer:HPASs:FREQuency 50 Hz | 300 Hz
        /// [SENSe&lt;1|2&gt;:]FILTer:LPASs[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]FILTer:LPASs:FREQuency 3 kHz | 15 kHz | 100 kHz
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Filter_Type">
        /// This control selects the filter type in the audio signal path.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_AF_HP (0) - High Pass AF Filter
        /// RSSPECAN_VAL_AF_LP (1) - Low Pass AF Filter
        /// 
        /// Default Value: RSSPECAN_VAL_AF_HP (0)
        /// 
        /// </param>
        /// <param name="Filter_State">
        /// This control activates the selected filter.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <param name="Filter_Frequency">
        /// Defines the cutoff frequency of the lowpass or highpass filter in the AF path.
        /// 
        /// Valid Values:
        ///  - High Pass filter: 50 Hz, 300 Hz
        ///  - Low Pass filter:  3 kHz, 15 kHz, 100 kHz
        /// 
        /// Default Value: 50.0 Hz
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfMeasuringReceiverAFFilter(int Filter_Type, bool Filter_State, double Filter_Frequency)
        {
            int pInvokeResult = PInvoke.ConfMeasuringReceiverAFFilter(this._handle, Filter_Type, System.Convert.ToUInt16(Filter_State), Filter_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function is setting deemphasis parameters in the audio signal path.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_DEEM_AF_FIL_STATE
        /// RSSPECAN_ATTR_MRECEIVER_DEEM_AF_FIL_VALUE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FILTer:DEMPhasis[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]FILTer:DEMPhasis:TCONstant 25 us | 50 us | 75 us | 750 us
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Filter_State">
        /// Switches on or off a deemphasis in the audio signal path.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <param name="Deemphasis_Value">
        /// Defines the time constant for the deemphasis.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_FIL_DEMP_CONS_25  (0) - 25 us
        /// RSSPECAN_VAL_FIL_DEMP_CONS_50  (1) - 50 us 
        /// RSSPECAN_VAL_FIL_DEMP_CONS_75  (2) - 75 us
        /// RSSPECAN_VAL_FIL_DEMP_CONS_750 (3) - 750 us
        /// 
        /// Default Value: RSSPECAN_VAL_FIL_DEMP_CONS_25 (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverDeemphasisAFFilter(bool Filter_State, int Deemphasis_Value)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverDeemphasisAFFilter(this._handle, System.Convert.ToUInt16(Filter_State), Deemphasis_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function is setting parameters of the weighting filters in the audio signal path.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_WEIGHTING_AF_FIL_CCIT
        /// RSSPECAN_ATTR_MRECEIVER_WEIGHTING_AF_FIL_CCIR
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FILTer:CCIT ON | OFF
        /// [SENSe&lt;1|2&gt;:]FILTer:CCIR ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Filter_Type">
        /// This control selects the filter type in the audio signal path.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_WEIGHTING_FIL_CCIT (0) - CCIT
        /// RSSPECAN_VAL_WEIGHTING_FIL_CCIR (1) - CCIR
        /// 
        /// Default Value: RSSPECAN_VAL_WEIGHTING_FIL_CCIT (0)
        /// 
        /// (1) Weighting AF Filter CCITT:
        /// Activates the weighting filter according to CCITT-recommendation in the audio signal path.
        /// 
        /// (2) Weighting AF Filter C-Message:
        /// Activates the C-message weighting filter according to CCITT-recommendation in the audio signal path.
        /// 
        /// </param>
        /// <param name="Filter_State">
        /// This control activates the selected filter.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverWeightingAFFilter(int Filter_Type, bool Filter_State)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverWeightingAFFilter(this._handle, Filter_Type, System.Convert.ToUInt16(Filter_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the coupling of the AF path of the analyzer.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_AF_INPUT_COUPLING
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ADEMod:AF:COUPling AC | DC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Input_Coupling">
        /// This control selects the coupling of the AF path of the analyzer.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DC (0) - DC
        /// RSSPECAN_VAL_AC (1) - AC
        /// 
        /// Default Value: RSSPECAN_VAL_DC (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverAFInputCoupling(int Input_Coupling)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverAFInputCoupling(this._handle, Input_Coupling);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates the demodulator of the instrument. The instrument will be set to time domain measurement (span = 0) at the current center frequency.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_MOD_MEAS_MODE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ADEMod[:STATe] ON
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int MeasuringReceiverModulationMeasurementMode()
        {
            int pInvokeResult = PInvoke.MeasuringReceiverModulationMeasurementMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates the modulation measurement (amplitude modulation, frequency modulation and phase modulation) measurement with its result displays.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_MODULATION_TYPE
        /// 
        /// Remote-control command(s):
        /// CALCulate2:FEED 'XTIM:AM:RELative[:TDOMain]'
        /// CALCulate2:FEED 'XTIM:FM[:TDOMain]'
        /// CALCulate2:FEED 'XTIM:PM[:TDOMain]'
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Modulation_Type">
        /// Selects the modulation measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MOD_TYPE_AM (0) - AM
        /// RSSPECAN_VAL_MOD_TYPE_FM (1) - FM
        /// RSSPECAN_VAL_MOD_TYPE_PM (2) - PM
        /// 
        /// Default Value: RSSPECAN_VAL_MOD_TYPE_AM (0)
        /// 
        /// Notes:
        /// 
        /// (1) AM:
        /// Activates the amplitude modulation measurement with its result displays.
        /// 
        /// (2) FM:
        /// Activates the frequency modulation measurement with its result displays. In the device default setting of the modulation measurement mode, the FM measurement is active.
        /// 
        /// (3) PM:
        /// Activates the phase modulation measurement with its result displays.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverModulationType(int Modulation_Type)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverModulationType(this._handle, Modulation_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function is selecting the detectors whose results are to be displayed in the modulation summary.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_DETECTOR_TYPE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:PPEak[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:MPEak[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:PAVerage[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:AVERage[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:RMS[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:SRMS[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:THD[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:SINad[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Detector_Type">
        /// This control selects the detector type.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_DET_TYPE_PLUS_PEAK  (0) - +PK
        /// RSSPECAN_VAL_DET_TYPE_MINUS_PEAK (1) - -PK
        /// RSSPECAN_VAL_DET_TYPE_AVG_PEAK   (2) - ?PK/2
        /// RSSPECAN_VAL_DET_TYPE_AVG        (3) - Average
        /// RSSPECAN_VAL_DET_TYPE_RMS        (4) - RMS
        /// RSSPECAN_VAL_DET_TYPE_SRMS       (5) - Sqrt(2) RMS
        /// RSSPECAN_VAL_DET_TYPE_THD        (6) - THD
        /// RSSPECAN_VAL_DET_TYPE_SINAD      (7) - SINAD
        /// 
        /// Default Value: RSSPECAN_VAL_DET_TYPE_PLUS_PEAK (0)
        /// 
        /// Notes:
        /// 
        /// (1) +PK:
        /// Result of measurement with +PK detector.
        /// 
        /// (2) -PK:
        /// Result of measurement with -PK detector.
        /// 
        /// (3) ?PK/2:
        /// Result of averaging ?PK/2, i.e. (plus peak + minus peak)/2.
        /// 
        /// (4) Average:
        /// Result of measurement with average detector
        /// 
        /// (5) RMS:
        /// Result of measurement with RMS detector
        /// 
        /// (6) Sqrt(2) RMS:
        /// Result of measurement with square root of 2 RMS detector
        /// 
        /// (7) THD:
        /// Total harmonic distortion.
        /// 
        /// (8) SINAD:
        /// Signal, noise and distortion.
        /// 
        /// </param>
        /// <param name="Detector_State">
        /// This control activates the selected detector.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverDetectorType(int Detector_Type, bool Detector_State)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverDetectorType(this._handle, Detector_Type, System.Convert.ToUInt16(Detector_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on the detectors whose results are to be displayed in the modulation summary.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:DETector  PPEak | MPEak | PAVerage | RMS  |  AVERage | SRMS | THD | SINad [[,PPEak | MPEak | PAVerage | RMS  | AVERage | SRMS | THD | SINad],...]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="_PK">
        /// This control activates the +PK detector.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// </param>
        /// <param name="_PK_2">
        /// This control activates the -PK detector.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <param name="_PK_2_2">
        /// This control activates the ?PK/2 detector.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <param name="Average">
        /// This control activates the average detector.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <param name="RMS">
        /// This control activates the RMS detector.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <param name="Sqrt_2__RMS">
        /// This control activates the Sqrt(2) RMS detector.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <param name="THD___SINAD">
        /// This control activates the THD &amp; SINAD detector.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverDetectorTypeMultiple(bool _PK, bool _PK_2, bool _PK_2_2, bool Average, bool RMS, bool Sqrt_2__RMS, bool THD___SINAD)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverDetectorTypeMultiple(this._handle, System.Convert.ToUInt16(_PK), System.Convert.ToUInt16(_PK_2), System.Convert.ToUInt16(_PK_2_2), System.Convert.ToUInt16(Average), System.Convert.ToUInt16(RMS), System.Convert.ToUInt16(Sqrt_2__RMS), System.Convert.ToUInt16(THD___SINAD));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function uses the current measured values as reference values for the relative measurement for all results displayed in the modulation summary.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_MOD_MEAS_TO_REF
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:REFerence:AUTO ONCE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverModulationMeasToRef()
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverModulationMeasToRef(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// For all selected detectors, this function additionally activates the display of the maximum value from several measurements. The highest results are displayed in addition to the current results.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_PEAK_HOLD
        /// 
        /// Notes:
        /// 
        /// (1) If the average-value measurement is switched on, the relative-value display is referenced to the average values rather than to the current measured values. If the Peak Hold display is active, this display has priority, and the relative-value display is referenced to the Peak Hold values.
        /// 
        /// (2) This function is available only in conjunction with FSMR.
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ADEMod:PHOLd[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Peak_Hold">
        /// Switches for all selected detectors display of the maximum value from several measurements.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverPeakHold(bool Peak_Hold)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverPeakHold(this._handle, System.Convert.ToUInt16(Peak_Hold));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the display of averaged results in the modulation summary. Averaged results are displayed in addition to current results.
        /// 
        /// Notes:
        /// 
        /// (1) If the average-value measurement is switched on, the relative-value display is referenced to the average values rather than to the current measured values. If the Peak Hold display is active, this display has priority, and the relative-value display is referenced to the Peak Hold values.
        /// 
        /// (2) This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_AVERAGING
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ADEMod:AVERage[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Average">
        /// Switches on or off the display of averaged results in the modulation summary.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverAveraging(bool Average)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverAveraging(this._handle, System.Convert.ToUInt16(Average));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the type of results to be displayed.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// No attributes.
        /// 
        /// Remote-control command(s):
        /// CALCulate1:FEED 'XTIM:RFPower:BARGraph'
        /// CALCulate1:FEED 'XTIM:AM:RELative[:TDOMain]'
        /// CALCulate1:FEED 'XTIM:FM[:TDOMain]'
        /// CALCulate1:FEED 'XTIM:PM[:TDOMain]'
        /// CALCulate1:FEED 'XTIM:AM:RELative:AFSPectrum&lt;1 to 3&gt;'
        /// CALCulate1:FEED 'XTIM:FM:AFSPectrum&lt;1 to 3&gt;'
        /// CALCulate1:FEED 'XTIM:PM:AFSPectrum&lt;1 to 3&gt;'
        /// CALCulate1:FEED 'XTIM:SPECtrum'
        /// CALCulate1:FEED 'NONE'
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Result_Display">
        /// Selects the type of results to be displayed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_RES_DISP_BARGRAF     (0) - Bargraph
        /// RSSPECAN_VAL_RES_DISP_TIME_DOMAIN (1) - Time Domain
        /// RSSPECAN_VAL_RES_DISP_MOD_SPECT   (2) - Modulation Spectrum
        /// RSSPECAN_VAL_RES_DISP_RF_SPECT    (3) - RF Spectrum
        /// 
        /// Default Value: RSSPECAN_VAL_RES_DISP_BARGRAF (0)
        /// 
        /// Notes:
        /// 
        /// (1) Bargraph:
        /// Inserts a numeric frequency and level display in the upper screen half as well as a graphic level display with a bargraph.
        /// 
        /// (2) Time Domain:
        /// Inserts a graphical display with the time characteristics of the demodulated signal in the upper screen half.
        /// 
        /// (3) Modulation Spectrum:
        /// Inserts in the upper screen half a graphical display with the AF spectrum of the demodulated signal. AF spectrum of audio signal in AUDIO mode.
        /// 
        /// (4) RF Spectrum:
        /// Inserts a graphical display with the RF spectrum of the received signal in the upper screen half. RF spectrum of signal generated from measured data by means of an FFT.
        /// 
        /// (5) None:
        /// Disables the RF spectrum in RF Level Mode.
        /// 
        /// </param>
        /// <param name="Modulation_Type">
        /// Selects the modulation type for Time Domain or Modulation Spectrum result display.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MOD_TYPE_AM (0) - AM
        /// RSSPECAN_VAL_MOD_TYPE_FM (1) - FM
        /// RSSPECAN_VAL_MOD_TYPE_PM (2) - PM
        /// 
        /// Default Value: RSSPECAN_VAL_MOD_TYPE_AM (0)
        /// 
        /// </param>
        /// <param name="Trace">
        /// This contyrol sets the trace.
        /// 
        /// Valid Values: 1 to 3
        /// 
        /// Default Value: 1
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available if Result Display is set to Modulation Spectrum.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverModulationResultDisplay(int Result_Display, int Modulation_Type, int Trace)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverModulationResultDisplay(this._handle, Result_Display, Modulation_Type, Trace);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on absolute or relative measurement for selected detector type in the modulation summary. Each time the relative measurement mode is switched on, the current measured value is taken as a reference for the subsequent relative measurements.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_REL_MEAS_PPEAK_MODE
        /// RSSPECAN_ATTR_MRECEIVER_REL_MEAS_MPEAK_MODE
        /// RSSPECAN_ATTR_MRECEIVER_REL_MEAS_PAVERAGE_MODE
        /// RSSPECAN_ATTR_MRECEIVER_REL_MEAS_AVERAGE_MODE
        /// RSSPECAN_ATTR_MRECEIVER_REL_MEAS_RMS_MODE
        /// RSSPECAN_ATTR_MRECEIVER_REL_MEAS_SRMS_MODE
        /// RSSPECAN_ATTR_MRECEIVER_REL_MEAS_ABSOLUTE_MODE
        /// RSSPECAN_ATTR_MRECEIVER_REL_LEVEL_MEAS_UNIT
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:PPEak:MODE ABSolute | RELative
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:MPEak:MODE ABSolute | RELative
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:PAVerage:MODE ABSolute | RELative
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:AVERage:MODE ABSolute | RELative
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:RMS:MODE ABSolute | RELative
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:SRMS:MODE ABSolute | RELative
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:REFerence AOFF
        /// UNIT&lt;1|2&gt;:POWer:RATio DB | PCT
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Detector_Type">
        /// This control selects the detector type.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_DET_TYPE_PPEAK    (0) - +PK
        /// RSSPECAN_VAL_DET_TYPE_MPEAK    (1) - -PK
        /// RSSPECAN_VAL_DET_TYPE_PAVERAGE (2) - ?PK/2
        /// RSSPECAN_VAL_DET_TYPE_AVERAGE  (3) - Average
        /// RSSPECAN_VAL_DET_TYPE_RMS      (4) - RMS
        /// RSSPECAN_VAL_DET_TYPE_SRMS     (5) - Sqrt(2) RMS
        /// 
        /// Default Value: RSSPECAN_VAL_DET_TYPE_PPEAK (0)
        /// 
        /// Notes:
        /// 
        /// (1) +PK:
        /// Result of measurement with +PK detector.
        /// 
        /// (2) -PK:
        /// Result of measurement with -PK detector.
        /// 
        /// (3) ?PK/2:
        /// Result of averaging ?PK/2, i.e. (plus peak + minus peak)/2.
        /// 
        /// (4) Average:
        /// Result of measurement with average detector
        /// 
        /// (5) RMS:
        /// Result of measurement with RMS detector
        /// 
        /// (6) Sqrt(2) RMS:
        /// Result of measurement with square root of 2 RMS detector
        /// 
        /// </param>
        /// <param name="Measurement_Mode">
        /// Switches on absolute or relative measurement for the selected detector type in the modulation summary. Each time the relative measurement mode is switched on, the current measured value is taken as a reference for the subsequent relative measurements.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_MEAS_MODE_ABS         (0) - Absolute
        /// RSSPECAN_VAL_MEAS_MODE_REL         (1) - Relative
        /// RSSPECAN_VAL_MEAS_MODE_REL_ALL_OFF (2) - Relative All Off
        /// 
        /// Default Value: RSSPECAN_VAL_MEAS_MODE_ABS (0)
        /// 
        /// Note:
        /// 
        /// Relative All Off switches from relative-value to absolute-value display for all measured values shown in the modulation summary. This is also the default setting of the detectors.
        /// 
        /// </param>
        /// <param name="Relative_Unit">
        /// Selects the unit for the display of the relative measurement results.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_REL_UNIT_DB  (0) - dB
        /// RSSPECAN_VAL_REL_UNIT_PTC (1) - PTC
        /// 
        /// Default Value: RSSPECAN_VAL_REL_UNIT_DB (0)
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only availabe if Measurement Mode is set to Relative.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverRelativeMeasurement(int Detector_Type, int Measurement_Mode, int Relative_Unit)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverRelativeMeasurement(this._handle, Detector_Type, Measurement_Mode, Relative_Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects whether as the reference value for the relative measurement in the modulation summary is used current measured value or sets the reference value manually.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_REL_MEAS_REF_AUTO
        /// RSSPECAN_ATTR_MRECEIVER_REL_MEAS_REF_VALUE
        /// 
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:PPEak:REFerence
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:PPEak:REFerence:AUTO ONCE
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:MPEak:REFerence
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:MPEak:REFerence:AUTO ONCE
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:PAVerage:REFerence
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:PAVerage:REFerence:AUTO ONCE
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:AVERage:REFerence
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:AVERage:REFerence:AUTO ONCE
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:RMS:REFerence
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:RMS:REFerence:AUTO ONCE
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:SRMS:REFerence
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:SRMS:REFerence:AUTO ONCE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Detector_Type">
        /// This control selects the detector type.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_DET_TYPE_PPEAK    (0) - +PK
        /// RSSPECAN_VAL_DET_TYPE_MPEAK    (1) - -PK
        /// RSSPECAN_VAL_DET_TYPE_PAVERAGE (2) - ?PK/2
        /// RSSPECAN_VAL_DET_TYPE_AVERAGE  (3) - Average
        /// RSSPECAN_VAL_DET_TYPE_RMS      (4) - RMS
        /// RSSPECAN_VAL_DET_TYPE_SRMS     (5) - Sqrt(2) RMS
        /// 
        /// Default Value: RSSPECAN_VAL_DET_TYPE_PPEAK (0)
        /// 
        /// Notes:
        /// 
        /// (1) +PK:
        /// Result of measurement with +PK detector.
        /// 
        /// (2) -PK:
        /// Result of measurement with -PK detector.
        /// 
        /// (3) ?PK/2:
        /// Result of averaging ?PK/2, i.e. (plus peak + minus peak)/2.
        /// 
        /// (4) Average:
        /// Result of measurement with average detector
        /// 
        /// (5) RMS:
        /// Result of measurement with RMS detector
        /// 
        /// (6) Sqrt(2) RMS:
        /// Result of measurement with square root of 2 RMS detector
        /// 
        /// </param>
        /// <param name="Reference_Auto">
        /// Selects whether as the reference value for the relative measurement is used current measured value (ON) or sets the reference value manually (OFF).
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <param name="Reference_Value">
        /// Sets the reference value for the relative measurement for the selected detector in the modulation summary if the reference auto parameter is set to OFF.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverRelativeMeasurementRef(int Detector_Type, bool Reference_Auto, double Reference_Value)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverRelativeMeasurementRef(this._handle, Detector_Type, System.Convert.ToUInt16(Reference_Auto), Reference_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the reference value of the relative measurement for the selected detector in the modulation summary.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_REL_MEAS_REF_VALUE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:PPEak:REFerence?
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:MPEak:REFerence?
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:PAVerage:REFerence?
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:AVERage:REFerence?
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:RMS:REFerence?
        /// [SENSe&lt;1|2&gt;:]ADEMod:DETector:SRMS:REFerence?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Detector_Type">
        /// This control selects the detector type.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_DET_TYPE_PPEAK    (0) - +PK
        /// RSSPECAN_VAL_DET_TYPE_MPEAK    (1) - -PK
        /// RSSPECAN_VAL_DET_TYPE_PAVERAGE (2) - ?PK/2
        /// RSSPECAN_VAL_DET_TYPE_AVERAGE  (3) - Average
        /// RSSPECAN_VAL_DET_TYPE_RMS      (4) - RMS
        /// RSSPECAN_VAL_DET_TYPE_SRMS     (5) - Sqrt(2) RMS
        /// 
        /// Default Value: RSSPECAN_VAL_DET_TYPE_PPEAK (0)
        /// 
        /// Notes:
        /// 
        /// (1) +PK:
        /// Result of measurement with +PK detector.
        /// 
        /// (2) -PK:
        /// Result of measurement with -PK detector.
        /// 
        /// (3) ?PK/2:
        /// Result of averaging ?PK/2, i.e. (plus peak + minus peak)/2.
        /// 
        /// (4) Average:
        /// Result of measurement with average detector
        /// 
        /// (5) RMS:
        /// Result of measurement with RMS detector
        /// 
        /// (6) Sqrt(2) RMS:
        /// Result of measurement with square root of 2 RMS detector
        /// 
        /// </param>
        /// <param name="Reference_Value">
        /// Returns the reference value of the relative measurement for the selected detector in the modulation summary.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryMeasuringReceiverRelativeMeasurementRef(int Detector_Type, out double Reference_Value)
        {
            int pInvokeResult = PInvoke.QueryMeasuringReceiverRelativeMeasurementRef(this._handle, Detector_Type, out Reference_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// $This function switches the audio input and thus the AUDIO mode on (AUDio) or off (RF).
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_AUDIO_INPUT_MODE
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:SELect AUDio | RF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Input">
        /// Switches the audio input and thus the AUDIO mode on (AUDio) or off (RF).
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_INPUT_TYPE_RF    (0) - RF
        /// RSSPECAN_VAL_INPUT_TYPE_AUDIO (1) - Audio
        /// 
        /// Default Value: RSSPECAN_VAL_INPUT_TYPE_RF (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int MeasuringReceiverAudioInputMode(int Input)
        {
            int pInvokeResult = PInvoke.MeasuringReceiverAudioInputMode(this._handle, Input);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the receiver audio input settings.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_AUDIO_INPUT_LEVEL
        /// RSSPECAN_ATTR_INPUT_IMPEDANCE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]VOLTage:AC:RANGe[:UPPer] 4 | 0.4
        /// INPut&lt;1|2&gt;:IMPedance 50 | 1000000
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Input_Level">
        /// Selects the input voltage range for the audio input.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_IN_LEV_4V    (0) - 4 V
        /// RSSPECAN_VAL_IN_LEV_400MV (1) - 400 mV
        /// 
        /// Default Value: RSSPECAN_VAL_IN_LEV_4V (0)
        /// 
        /// </param>
        /// <param name="Input_Impedance">
        /// Specifies the nominal input impedance.
        /// 
        /// Valid Range:
        /// 50.0 or 1000000.0
        /// 
        /// Default Value: 50.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverAudioInput(int Input_Level, double Input_Impedance)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverAudioInput(this._handle, Input_Level, Input_Impedance);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the type of results to be displayed in the  audio mode.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// No attributes.
        /// 
        /// Remote-control command(s):
        /// CALCulate1:FEED 'XTIM:AC[:TDOMain]'
        /// CALCulate1:FEED 'XTIM:AC:SPectrum'
        /// CALCulate1:FEED 'XTIM:AC:AFSPectrum&lt;1 to 3&gt;'
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Result_Display">
        /// Selects the type of results to be displayed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_INP_RES_DISP_TD    (0) - Time Domain
        /// RSSPECAN_VAL_INP_RES_DISP_SPECT (1) - Spectrum
        /// 
        /// Default Value: RSSPECAN_VAL_INP_RES_DISP_TD (0)
        /// 
        /// Notes:
        /// 
        /// (1) Time Domain:
        /// Audio signal in AUDIO mode.
        /// 
        /// (2) Spectrum:
        /// AF spectrum of audio signal in AUDIO mode.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverAudioInputResultDisplay(int Result_Display)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverAudioInputResultDisplay(this._handle, Result_Display);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on absolute or relative measured-value display and relative unit in the AUDIO mode.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_AUDIO_INPUT_RES_DISPLAY_MODE
        /// RSSPECAN_ATTR_MRECEIVER_REL_LEVEL_MEAS_UNIT
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]VOLTage:AC:REFerence[:STATe] ON | OFF
        /// UNIT&lt;1|2&gt;:POWer:RATio DB | PCT
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Display_Mode">
        /// Switches on absolute or relative measured-value display in the AUDIO mode.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_DISP_MODE_ABS (0) - Absolute
        /// RSSPECAN_VAL_DISP_MODE_REL (1) - Relative
        /// 
        /// Default Value: RSSPECAN_VAL_DISP_MODE_ABS (0)
        /// 
        /// </param>
        /// <param name="Relative_Unit">
        /// Selects the unit for the display of the relative measurement results.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_REL_UNIT_DB  (0) - dB
        /// RSSPECAN_VAL_REL_UNIT_PTC (1) - PCT
        /// 
        /// Default Value: RSSPECAN_VAL_REL_UNIT_DB (0)
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only availabe if Display Mode is set to Relative.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverAudioInputResultDisplayMode(int Display_Mode, int Relative_Unit)
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverAudioInputResultDisplayMode(this._handle, Display_Mode, Relative_Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function uses the current measured values as reference values for the relative measured value display for all detectors shown in the audio summary.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MRECEIVER_AUDIO_INPUT_MEAS_TO_REF
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]VOLTage:AC:REFerence:AUTO ONCE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMeasuringReceiverAudioInputMeasToRef()
        {
            int pInvokeResult = PInvoke.ConfigureMeasuringReceiverAudioInputMeasToRef(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns result of the selected modulation measurement. For this, a suitable result display mode must have been selected before with CALC:FEED.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_QUERY_MRECEIVER_MOD_MEASUREMENT_RES
        /// 
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ADEMod:AM[:WRITe][:RESult]? PPEak| MPEak| PAVerage| AVERage | RMS | SRMS
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ADEMod:FM[:WRITe][:RESult]? PPEak| MPEak| PAVerage| AVERage | RMS | SRMS
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ADEMod:PM[:WRITe][:RESult]? PPEak| MPEak| PAVerage| AVERage | RMS | SRMS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Detector_Type">
        /// This control selects the detector type.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_DET_TYPE_PPEAK    (0) - +PK
        /// RSSPECAN_VAL_DET_TYPE_MPEAK    (1) - -PK
        /// RSSPECAN_VAL_DET_TYPE_PAVERAGE (2) - ?PK/2
        /// RSSPECAN_VAL_DET_TYPE_AVERAGE  (3) - Average
        /// RSSPECAN_VAL_DET_TYPE_RMS      (4) - RMS
        /// RSSPECAN_VAL_DET_TYPE_SRMS     (5) - Sqrt(2) RMS
        /// 
        /// Default Value: RSSPECAN_VAL_DET_TYPE_PPEAK (0)
        /// 
        /// Notes:
        /// 
        /// (1) +PK:
        /// Result of measurement with +PK detector.
        /// 
        /// (2) -PK:
        /// Result of measurement with -PK detector.
        /// 
        /// (3) ?PK/2:
        /// Result of averaging ?PK/2, i.e. (plus peak + minus peak)/2.
        /// 
        /// (4) Average:
        /// Result of measurement with average detector
        /// 
        /// (5) RMS:
        /// Result of measurement with RMS detector
        /// 
        /// (6) Sqrt(2) RMS:
        /// Result of measurement with square root of 2 RMS detector
        /// 
        /// </param>
        /// <param name="Modulation_Type">
        /// Selects the modulation type of measurement result.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MOD_TYPE_AM (0) - AM
        /// RSSPECAN_VAL_MOD_TYPE_FM (1) - FM
        /// RSSPECAN_VAL_MOD_TYPE_PM (2) - PM
        /// 
        /// Default Value: RSSPECAN_VAL_MOD_TYPE_AM (0)
        /// 
        /// </param>
        /// <param name="Modulation_Measurement_Result">
        /// Returns result of the selected modulation measurement.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryMeasuringReceiverModulationMeasurementResults(int Detector_Type, int Modulation_Type, out double Modulation_Measurement_Result)
        {
            int pInvokeResult = PInvoke.QueryMeasuringReceiverModulationMeasurementResults(this._handle, Detector_Type, Modulation_Type, out Modulation_Measurement_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns averaged result of the selected modulation measurement. For this, a suitable result display mode must have been selected before with CALC:FEED.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_QUERY_MRECEIVER_MOD_AVERAGED_MEAS_RES
        /// 
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ADEMod:AM:AVERage[:RESult]? PPEak| MPEak| PAVerage| AVERage | RMS | SRMS
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ADEMod:FM:AVERage[:RESult]? PPEak| MPEak| PAVerage| AVERage | RMS | SRMS
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ADEMod:PM:AVERage[:RESult]? PPEak| MPEak| PAVerage| AVERage | RMS | SRMS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Detector_Type">
        /// This control selects the detector type.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_DET_TYPE_PPEAK    (0) - +PK
        /// RSSPECAN_VAL_DET_TYPE_MPEAK    (1) - -PK
        /// RSSPECAN_VAL_DET_TYPE_PAVERAGE (2) - ?PK/2
        /// RSSPECAN_VAL_DET_TYPE_AVERAGE  (3) - Average
        /// RSSPECAN_VAL_DET_TYPE_RMS      (4) - RMS
        /// RSSPECAN_VAL_DET_TYPE_SRMS     (5) - Sqrt(2) RMS
        /// 
        /// Default Value: RSSPECAN_VAL_DET_TYPE_PPEAK (0)
        /// 
        /// Notes:
        /// 
        /// (1) +PK:
        /// Result of measurement with +PK detector.
        /// 
        /// (2) -PK:
        /// Result of measurement with -PK detector.
        /// 
        /// (3) ?PK/2:
        /// Result of averaging ?PK/2, i.e. (plus peak + minus peak)/2.
        /// 
        /// (4) Average:
        /// Result of measurement with average detector
        /// 
        /// (5) RMS:
        /// Result of measurement with RMS detector
        /// 
        /// (6) Sqrt(2) RMS:
        /// Result of measurement with square root of 2 RMS detector
        /// 
        /// </param>
        /// <param name="Modulation_Type">
        /// Selects the modulation type of measurement result.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MOD_TYPE_AM (0) - AM
        /// RSSPECAN_VAL_MOD_TYPE_FM (1) - FM
        /// RSSPECAN_VAL_MOD_TYPE_PM (2) - PM
        /// 
        /// Default Value: RSSPECAN_VAL_MOD_TYPE_AM (0)
        /// 
        /// </param>
        /// <param name="Modulation_Measurement_Result">
        /// Returns result of the selected modulation measurement.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryMeasuringReceiverModulationAveragedMeasurementResults(int Detector_Type, int Modulation_Type, out double Modulation_Measurement_Result)
        {
            int pInvokeResult = PInvoke.QueryMeasuringReceiverModulationAveragedMeasurementResults(this._handle, Detector_Type, Modulation_Type, out Modulation_Measurement_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns highest result of the selected modulation measurement. For this, a suitable result display mode must have been selected before with CALC:FEED.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_QUERY_MRECEIVER_MOD_PEAK_HOLD_MEAS_RES
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ADEMod:AM:PHOLd[:RESult]? PPEak| MPEak| PAVerage| AVERage | RMS | SRMS
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ADEMod:FM:PHOLd[:RESult]? PPEak| MPEak| PAVerage| AVERage | RMS | SRMS
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ADEMod:PM:PHOLd[:RESult]? PPEak| MPEak| PAVerage| AVERage | RMS | SRMS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Detector_Type">
        /// This control selects the detector type.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_DET_TYPE_PPEAK    (0) - +PK
        /// RSSPECAN_VAL_DET_TYPE_MPEAK    (1) - -PK
        /// RSSPECAN_VAL_DET_TYPE_PAVERAGE (2) - ?PK/2
        /// RSSPECAN_VAL_DET_TYPE_AVERAGE  (3) - Average
        /// RSSPECAN_VAL_DET_TYPE_RMS      (4) - RMS
        /// RSSPECAN_VAL_DET_TYPE_SRMS     (5) - Sqrt(2) RMS
        /// 
        /// Default Value: RSSPECAN_VAL_DET_TYPE_PPEAK (0)
        /// 
        /// Notes:
        /// 
        /// (1) +PK:
        /// Result of measurement with +PK detector.
        /// 
        /// (2) -PK:
        /// Result of measurement with -PK detector.
        /// 
        /// (3) ?PK/2:
        /// Result of averaging ?PK/2, i.e. (plus peak + minus peak)/2.
        /// 
        /// (4) Average:
        /// Result of measurement with average detector
        /// 
        /// (5) RMS:
        /// Result of measurement with RMS detector
        /// 
        /// (6) Sqrt(2) RMS:
        /// Result of measurement with square root of 2 RMS detector
        /// 
        /// </param>
        /// <param name="Modulation_Type">
        /// Selects the modulation type of measurement result.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MOD_TYPE_AM (0) - AM
        /// RSSPECAN_VAL_MOD_TYPE_FM (1) - FM
        /// RSSPECAN_VAL_MOD_TYPE_PM (2) - PM
        /// 
        /// Default Value: RSSPECAN_VAL_MOD_TYPE_AM (0)
        /// 
        /// </param>
        /// <param name="Modulation_Measurement_Result">
        /// Returns result of the selected modulation measurement.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryMeasuringReceiverModulationPeakHoldMeasurementResults(int Detector_Type, int Modulation_Type, out double Modulation_Measurement_Result)
        {
            int pInvokeResult = PInvoke.QueryMeasuringReceiverModulationPeakHoldMeasurementResults(this._handle, Detector_Type, Modulation_Type, out Modulation_Measurement_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the audio frequency with analog demodulation.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_QUERY_MRECEIVER_MOD_AUDIO_FREQ_RES
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:ADEMod:AFRequency[:RESult]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Audio_Frequency">
        /// Returns the audio frequency with analog demodulation.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryMeasuringReceiverModulationAudioFrequencyResult(out double Audio_Frequency)
        {
            int pInvokeResult = PInvoke.QueryMeasuringReceiverModulationAudioFrequencyResult(this._handle, out Audio_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the frequency error of the carrier signal.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_QUERY_MRECEIVER_MOD_FREQ_ERROR_RES
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:ADEMod:FERRor[:RESult]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Frequency_Error">
        /// Returns the frequency error of the carrier signal.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryMeasuringReceiverModulationFrequencyErrorResult(out double Frequency_Error)
        {
            int pInvokeResult = PInvoke.QueryMeasuringReceiverModulationFrequencyErrorResult(this._handle, out Frequency_Error);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the measured value of the carrier power.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_QUERY_MRECEIVER_MOD_CAR_POWER_RES
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:ADEMod:CARRier[:RESult]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Carrier_Power">
        /// Returns the measured value of the carrier power.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryMeasuringReceiverModulationCarrierPowerResult(out double Carrier_Power)
        {
            int pInvokeResult = PInvoke.QueryMeasuringReceiverModulationCarrierPowerResult(this._handle, out Carrier_Power);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the SINAD measurement result.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_QUERY_MRECEIVER_MOD_SINAD_MEAS_RESULTS
        /// RSSPECAN_ATTR_QUERY_MRECEIVER_MOD_SINAD_MEAS_AVERAGE
        /// RSSPECAN_ATTR_QUERY_MRECEIVER_MOD_SINAD_MEAS_PHOLD
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:ADEMod:SINad[:WRITe]:RESult?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:ADEMod:SINad:AVERage:RESult?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:ADEMod:SINad:PHOLd:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Result_Type">
        /// This control selects the result type.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_RES_TYPE_CUR   (0) - Current
        /// RSSPECAN_VAL_RES_TYPE_AVER  (1) - Averaged
        /// RSSPECAN_VAL_RES_TYPE_PHOLD (2) - Peak Hold
        /// 
        /// Default Value: RSSPECAN_VAL_RES_TYPE_CUR (0)
        /// 
        /// Notes:
        /// 
        /// (1) Current:
        /// Current result of the SINAD measurement.
        /// 
        /// (2) Averaged:
        /// SINAD value averaged over several measurements.
        /// 
        /// (3) Peak Hold:
        /// Highest value obtained in the SINAD measurement.
        /// 
        /// </param>
        /// <param name="Measurement_Result">
        /// Returns result of the SINAD measurement.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryMeasuringReceiverModulationSINADMeasurementResults(int Result_Type, out double Measurement_Result)
        {
            int pInvokeResult = PInvoke.QueryMeasuringReceiverModulationSINADMeasurementResults(this._handle, Result_Type, out Measurement_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the THD measurement result.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_QUERY_MRECEIVER_MOD_THD_MEAS_RESULTS
        /// RSSPECAN_ATTR_QUERY_MRECEIVER_MOD_THD_MEAS_AVERAGE
        /// RSSPECAN_ATTR_QUERY_MRECEIVER_MOD_THD_MEAS_PHOLD
        /// 
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:ADEMod:THD[:WRITe]:RESult?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:ADEMod:THD:AVERage:RESult?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 
        /// 4&gt;:FUNCtion:ADEMod:THD:PHOLd:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Result_Type">
        /// This control selects the result type.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_RES_TYPE_CUR   (0) - Current
        /// RSSPECAN_VAL_RES_TYPE_AVER  (1) - Averaged
        /// RSSPECAN_VAL_RES_TYPE_PHOLD (2) - Peak Hold
        /// 
        /// Default Value: RSSPECAN_VAL_RES_TYPE_CUR (0)
        /// 
        /// Notes:
        /// 
        /// (1) Current:
        /// Current result of the THD measurement.
        /// 
        /// (2) Averaged:
        /// THD value averaged over several measurements.
        /// 
        /// (3) Peak Hold:
        /// Highest value obtained in the THD measurement.
        /// 
        /// </param>
        /// <param name="Measurement_Result">
        /// Returns result of the THD measurement.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryMeasuringReceiverModulationTHDMeasurementResults(int Result_Type, out double Measurement_Result)
        {
            int pInvokeResult = PInvoke.QueryMeasuringReceiverModulationTHDMeasurementResults(this._handle, Result_Type, out Measurement_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the standard uncertainty of the carrier power.
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_QUERY_MRECEIVER_MOD_STANDARD_UNCERTAINTY
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ADEMod:CARRier:SUNCertainty?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Standard_Uncertainty">
        /// Returns the standard uncertainty of the carrier power.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryMeasuringReceiverModulationStandardUncertainty(out double Standard_Uncertainty)
        {
            int pInvokeResult = PInvoke.QueryMeasuringReceiverModulationStandardUncertainty(this._handle, out Standard_Uncertainty);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the results of the audio measurement. 
        /// 
        /// Note:
        /// 
        /// This function is available only in conjunction with FSMR.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_QUERY_MRECEIVER_AUDIO_MEAS_RES
        /// 
        /// Remote-control command(s):
        /// CALCulate:MARKer:FUNCtion:VOLTage[:WRITe][:RESult]?  PPEak | RMS | THD | SINad
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Detector_Type">
        /// This control selects the detector type.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_AUDIO_MEAS_RES_PPEAK (0) - +PK
        /// RSSPECAN_VAL_AUDIO_MEAS_RES_RMS   (1) - RMS
        /// RSSPECAN_VAL_AUDIO_MEAS_RES_THD   (2) - THD
        /// RSSPECAN_VAL_AUDIO_MEAS_RES_SINAD (3) - SINAD
        /// 
        /// Default Value: RSSPECAN_VAL_AUDIO_MEAS_RES_PPEAK (0)
        /// 
        /// Notes:
        /// 
        /// (1) +PK:
        /// Result of measurement with +PK detector.
        /// 
        /// (2) RMS:
        /// Result of measurement with RMS detector.
        /// 
        /// (3) THD:
        /// Total harmonic distortion.
        /// 
        /// (4) SINAD: 
        /// Signal, noise and distortion.
        /// </param>
        /// <param name="Audio_Measurement_Result">
        /// Returns the results of the audio measurement.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryMeasuringReceiverAudioMeasurementResults(int Detector_Type, out double Audio_Measurement_Result)
        {
            int pInvokeResult = PInvoke.QueryMeasuringReceiverAudioMeasurementResults(this._handle, Detector_Type, out Audio_Measurement_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function triggers a measurement with the power sensor and then outputs the result.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_READ_MRECEIVER_PWM_RESULT
        /// 
        /// Remote-control command(s):
        /// READ&lt;1|2&gt;:PMETer?
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Result">
        /// Returns the result.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadMeasuringReceiverPWMResult(uint Timeout, out double Result)
        {
            int pInvokeResult = PInvoke.ReadMeasuringReceiverPWMResult(this._handle, Timeout, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function outputs the result of the power sensor.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FETCH_MRECEIVER_PWM_RESULT
        /// 
        /// Remote-control command(s):
        /// FETCh&lt;1|2&gt;:PMETer?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Result">
        /// Returns the result.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchMeasuringReceiverPWMResult(out double Result)
        {
            int pInvokeResult = PInvoke.FetchMeasuringReceiverPWMResult(this._handle, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the tracking generator.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TGEN_OUTPUT_LEVEL
        /// RSSPECAN_ATTR_TGEN_LEVEL_OFFSET
        /// RSSPECAN_ATTR_TGEN_FREQUENCY_OFFSET
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:POWer[:LEVel][:IMMediate][:AMPLitude]
        /// SOURce&lt;1|2&gt;:POWer[:LEVel][:IMMediate]:OFFSet
        /// SOURce&lt;1|2&gt;:FREQuency:OFFSet
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Source_Power">
        /// Specifies the output level of the tracking generator. The driver uses this value to set the RSSPECAN_ATTR_TGEN_OUTPUT_LEVEL attribute.
        /// 
        /// Units: dBm
        /// 
        /// Valid Values: -100.0 to 5.0
        /// 
        /// Default Value: -20.0
        /// </param>
        /// <param name="Power_Offset">
        /// Specifies a level offset for the tracking generator level. The driver uses this value to set the RSSPECAN_ATTR_TGEN_LEVEL_OFFSET attribute.
        /// 
        /// Units: dB
        /// 
        /// Valid Values: -200.0 to 200.0
        /// 
        /// Default Value: -20.0
        /// </param>
        /// <param name="Frequency_Offset">
        /// Sets a frequency offset of the tracking generator for the indicated measurement window. The driver uses this value to set the RRSSPECAN_ATTR_TGEN_FREQUENCY_OFFSET attribute.
        /// 
        /// Units: Hz
        /// 
        /// Valid Values: -150.0e+6 to 150.0e+6
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTrackingGenerator(int Window, double Source_Power, double Power_Offset, double Frequency_Offset)
        {
            int pInvokeResult = PInvoke.ConfigureTrackingGenerator(this._handle, Window, Source_Power, Power_Offset, Frequency_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the tracking generator state.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TGEN_STATE
        /// 
        /// Remote-control command(s):
        /// OUTPut&lt;1|2&gt;[:STATe] ON | OFF
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// Sets the tracking generator state. The driver uses this value to set the RSSPECAN_ATTR_TGEN_STATE attribute.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SetTrackingGeneratorState(int Window, bool State)
        {
            int pInvokeResult = PInvoke.SetTrackingGeneratorState(this._handle, Window, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the external modulations of the tracking generator.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TGEN_AM_STATE
        /// RSSPECAN_ATTR_TGEN_DM_STATE
        /// RSSPECAN_ATTR_TGEN_FM_STATE
        /// RSSPECAN_ATTR_TGEN_FM_DEVIATION
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:AM:STATe ON | OFF
        /// SOURce&lt;1|2&gt;:DM:STATe ON | OFF
        /// SOURce&lt;1|2&gt;:FM:STATe ON | OFF
        /// SOURce&lt;1|2&gt;:FM:DEViation
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Modulation">
        /// Sets a frequency offset of the tracking generator for the indicated measurement window.
        /// 
        /// Units: Hz
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_TGEN_EXT_AM (0) - Ext AM
        /// RSSPECAN_VAL_TGEN_EXT_FM (1) - Ext FM
        /// RSSPECAN_VAL_TGEN_EXT_IQ (2) - Ext I/Q
        /// 
        /// Default Value: RSSPECAN_VAL_TGEN_EXT_AM (0)
        /// </param>
        /// <param name="Mod_State">
        /// This command switches on or off selected modulation.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TGEN_AM_STATE
        /// RSSPECAN_ATTR_TGEN_DM_STATE
        /// RSSPECAN_ATTR_TGEN_FM_STATE
        /// </param>
        /// <param name="FM_Deviation">
        /// Sets the maximum frequency deviation at 1 V input voltage at the FM input of the tracking generator.
        /// 
        /// Units: Hz
        /// 
        /// Valid Values: 100.0 to 10.0e+6
        /// 
        /// Default Value: 100.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TGEN_FM_DEVIATION
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTrackingGeneratorExternalModulation(int Window, int Modulation, bool Mod_State, double FM_Deviation)
        {
            int pInvokeResult = PInvoke.ConfigureTrackingGeneratorExternalModulation(this._handle, Window, Modulation, System.Convert.ToUInt16(Mod_State), FM_Deviation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the power sweep of the tracking generator.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TGEN_POW_MODE
        /// RSSPECAN_ATTR_TGEN_POW_START
        /// RSSPECAN_ATTR_TGEN_POW_STOP
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:POWer:MODE FIXed | SWEep
        /// SOURce&lt;1|2&gt;:POWer:STARt
        /// SOURce&lt;1|2&gt;:POWer:STOP
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Power_Sweep">
        /// Activates or deactivates the power sweep.
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_TGEN_POW_FIX (0) - Fixed
        /// RSSPECAN_VAL_TGEN_POW_SWE (1) - Sweep
        /// 
        /// Default Value: RSSPECAN_VAL_TGEN_POW_FIX (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TGEN_POW_MODE
        /// </param>
        /// <param name="Power_Start">
        /// Sets the start power of the power sweep.
        /// 
        /// Units: dBm
        /// 
        /// Valid Values: 
        /// -30.0 to 5.0
        /// -100.0 to 5.0 with FSU-B12 option
        /// 
        /// Default Value: -30.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TGEN_POW_START
        /// </param>
        /// <param name="Power_Stop">
        /// Sets the stop power of the power sweep.
        /// 
        /// Units: dBm
        /// 
        /// Valid Values: 
        /// -30.0 to 5.0
        /// -100.0 to 5.0 with FSU-B12 option
        /// 
        /// Default Value: -30.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TGEN_POW_STOP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTrackingGeneratorPowerSweep(int Window, int Power_Sweep, double Power_Start, double Power_Stop)
        {
            int pInvokeResult = PInvoke.ConfigureTrackingGeneratorPowerSweep(this._handle, Window, Power_Sweep, Power_Start, Power_Stop);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the selected external generator.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_EXT_GEN_OUTPUT_POWER
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:EXTernal&lt;1|2&gt;:POWer[:LEVel]
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Generator">
        /// This control selects the external generator 1 or 2.
        /// 
        /// Valid Values: 1 to 2
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Power">
        /// Sets the output power of the selected generator 1 or 2 in the selected window.
        /// 
        /// Units: dBm
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: -20.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXT_GEN_OUTPUT_POWER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureExternalGeneratorPower(int Window, int Generator, double Power)
        {
            int pInvokeResult = PInvoke.ConfigureExternalGeneratorPower(this._handle, Window, Generator, Power);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the external generator source.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_EXT_GEN_STATE
        /// 
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:EXTernal&lt;1|2&gt;[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// This command switches on or off external source.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXT_GEN_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureExternalGeneratorSource(int Window, bool State)
        {
            int pInvokeResult = PInvoke.ConfigureExternalGeneratorSource(this._handle, Window, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the external generator.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_EXT_GEN_IEEE_ADDR
        /// RSSPECAN_ATTR_EXT_GEN_INTF_TYPE
        /// RSSPECAN_ATTR_EXT_GEN_TYPE
        /// 
        /// 
        /// Remote-control command(s):
        /// SYSTem:COMMunicate:GPIB:RDEVice:GENerator&lt;1|2&gt;:ADDRess
        /// SYSTem:COMMunicate:RDEVice:GENerator&lt;1|2&gt;:LINK GPIB | TTL
        /// SYSTem:COMMunicate:RDEVice:GENerator&lt;1|2&gt;:TYPE &lt;name&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Generator">
        /// This control selects the external generator 1 or 2.
        /// 
        /// Valid Values: 1 to 2
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Generator_Address">
        /// This control changes the IEC/IEEE-bus address of the device selected as external generator 1 or 2.
        /// 
        /// Valid Values: 0 to 30
        /// 
        /// Default Value: 28
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXT_GEN_IEEE_ADDR
        /// 
        /// Note:
        /// 
        /// If two generators are connected at the same time to IECBUS 2 their addresses must be different.
        /// 
        /// </param>
        /// <param name="Generator_Link">
        /// This control selects the interface type of the external generator 1 or 2.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_EXT_GEN_INTF_GPIB (0) - GPIB
        /// RSSPECAN_VAL_EXT_GEN_INTF_TTL  (1) - TTL
        /// 
        /// Default Value: RSSPECAN_VAL_EXT_GEN_INTF_GPIB (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXT_GEN_INTF_TYPE
        /// 
        /// Notes:
        /// 
        /// (1) IECBUS alone (= GPIB, for all the generators of other manufacturers and some Rohde &amp; Schwarz units).
        /// 
        /// (2) IECBUS and TTL interface for synchronization (= TTL, for most of the Rohde &amp; Schwarz generators).
        /// 
        /// (3) The difference between the two operating modes is the execution speed. While, during IECBUS operation, each settable frequency is transmitted separately to the generator, a whole frequency list can be programmed in one go if the TTL interface is also used. Frequency switching can then be performed per TTL handshake which results in considerable speed advantages.
        /// 
        /// (4) Only one of the two generators can be operated via the TTL interface at a time. The other generator must be configured for IECBUS (GPIB).
        /// 
        /// </param>
        /// <param name="Generator_Type">
        /// This control selects the type of external generator 1 or 2. The available generator types including the associated interface are listed in the documentation.
        /// 
        /// Valid Values: 
        /// SME02, SME03, SME06, SMG, SMGL, SMGU, SMH, SMHU, SMIQ02B, SMIQ02E, SMIQ03B, SMIQ03E, SMIQ04B, SMIQ06B, SML01, SMR20, SMR20B11, SMR27, SMR27B11, SMR30, SMR30B11, SMR40, SMR40B11, SMP02, SMP03, SMP04, SMP22, SMT02, SMT03, SMT06, SMU02, SMU02B31, SMU03, SMU03B31, SMU04, SMU04B31, SMU06, SMU06B31.
        /// 
        /// Default Value: "SME03"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXT_GEN_TYPE
        /// 
        /// Notes:
        /// 
        /// (1) Generators with TTL interface can also be operated via IECBUS (= GPIB) alone.
        /// 
        /// (2) With "NONE" selected, the corresponding generator 1 or 2 is deactivated.
        /// 
        /// (3) There is a minimum version requirement of the R&amp;S SMU firmware. Please install V1.10 or higher on the R&amp;S SMU!
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SelectExternalGenerator(int Generator, int Generator_Address, int Generator_Link, string Generator_Type)
        {
            int pInvokeResult = PInvoke.SelectExternalGenerator(this._handle, Generator, Generator_Address, Generator_Link, Generator_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function is used for setting the generator multiplier and the offset used to derive the generator frequency from the analyzer frequency.
        /// 
        /// Notes:
        /// 
        /// (1) If no external generator is selected, if the IECBUS address is not correct or the generator is not ready for operation, an execution error will be generated.
        /// 
        /// (2) This function is only valid in combination with option Ext. Generator Control FSQ-B10.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_EXT_GEN_FREQ_SWEEP_STATE
        /// RSSPECAN_ATTR_EXT_GEN_FREQ_OFFSET
        /// RSSPECAN_ATTR_EXT_GEN_NUMERATOR
        /// RSSPECAN_ATTR_EXT_GEN_DENOMINATOR
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:EXTernal&lt;1|2&gt;:FREQuency:NUMerator 
        /// SOURce&lt;1|2&gt;:EXTernal&lt;1|2&gt;:FREQuency:DENominator 
        /// SOURce&lt;1|2&gt;:EXTernal&lt;1|2&gt;:FREQuency:OFFSet
        /// SOURce&lt;1|2&gt;:EXTernal&lt;1|2&gt;:FREQuency:SWEep ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Generator">
        /// This control selects the external generator 1 or 2.
        /// 
        /// Valid Values: 1 to 2
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Frequency_Sweep">
        /// This control activates or deactivates the frequency sweep for generator 1 or 2 in the selected window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXT_GEN_FREQ_SWEEP_STATE
        /// </param>
        /// <param name="Frequency_Offset">
        /// This control defines the frequency offset of the selected generator 1 or 2 with reference to the receive frequency in the selected window.
        /// 
        /// Valid Values: none
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXT_GEN_FREQ_OFFSET
        /// 
        /// Note:
        /// 
        /// Select the multiplication factor so that the frequency range of the generator is not exceeded if the following formula:
        /// 
        /// F(generator) = F(analyzer) * (numerator/denominator) + F(offset)
        /// 
        /// is applied to the start and stop frequency of the analyzer.
        /// 
        /// This parameter is not range-checked.
        /// </param>
        /// <param name="Denominator">
        /// This control defines the denominator of the factor with which the analyzer frequency is multiplied in order to obtain the transmit frequency of the selected generator 1 or 2 in the selected window.
        /// 
        /// Valid Values: none
        /// 
        /// Default Value: 1.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXT_GEN_NUMERATOR
        /// 
        /// Note:
        /// 
        /// Select the multiplication factor so that the frequency range of the generator is not exceeded if the following formula:
        /// 
        /// F(generator) = F(analyzer) * (numerator/denominator) + F(offset)
        /// 
        /// is applied to the start and stop frequency of the analyzer.
        /// 
        /// This parameter is not range-checked.
        /// </param>
        /// <param name="Numerator">
        /// This control defines the numerator of the factor with which the analyzer frequency is multiplied to obtain the transmit frequency of the selected generator 1 or 2 in the selected window.
        /// 
        /// Valid Values: none
        /// 
        /// Default Value: 1.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXT_GEN_DENOMINATOR
        /// 
        /// Note:
        /// 
        /// Select the multiplication factor so that the frequency range of the generator is not exceeded if the following formula:
        /// 
        /// F(generator) = F(analyzer) * (numerator/denominator) + F(offset)
        /// 
        /// is applied to the start and stop frequency of the analyzer.
        /// 
        /// This parameter is not range-checked.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureExternalGeneratorFrequencySweep(int Window, int Generator, bool Frequency_Sweep, double Frequency_Offset, int Denominator, int Numerator)
        {
            int pInvokeResult = PInvoke.ConfigureExternalGeneratorFrequencySweep(this._handle, Window, Generator, System.Convert.ToUInt16(Frequency_Sweep), Frequency_Offset, Denominator, Numerator);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches between external and internal reference oscillator for the frequency processing of external generator 1 and 2.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_EXT_GEN_ROSC_SOURCE
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:EXTernal&lt;1|2&gt;:ROSCillator[:SOURce] INTernal | EXTernal
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Reference">
        /// This control sets the reference signal for the external
        /// generator.
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_SOUR_INT (0) - Internal
        /// RSSPECAN_VAL_SOUR_EXT (1) - External
        /// 
        /// Default Value: RSSPECAN_VAL_SOUR_INT (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXT_GEN_ROSC_SOURCE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureExternalGeneratorRefOscillator(int Window, int Reference)
        {
            int pInvokeResult = PInvoke.ConfigureExternalGeneratorRefOscillator(this._handle, Window, Reference);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function makes an external gain known to the analyzer, which will take it into account during the display of measurement results.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only with R&amp;S FSL with Tracking Generator (Models 13 and 16)
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CORR_EGA_INP
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:EGAin:INPut[:MAGNitude]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Gain">
        /// This control sets the antena gain
        /// 
        /// Units: dB
        /// 
        /// Valid Values: 
        /// -200.0 to 200.0
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CORR_EGA_INP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureExternalGain(int Window, double Gain)
        {
            int pInvokeResult = PInvoke.ConfigureExternalGain(this._handle, Window, Gain);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sends a command or query string to the device using the GPIB interface FSP-B10.
        /// Usually an external generator is connected to this interface.
        /// 
        /// Note: 
        /// 
        /// (1) Only basic I/O is possible with this device. The first parameter is the GPIB address of the device connected with GPIB interface of option FSP-B10.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// SYSTem:COMMunicate:GPIB:RDEVice:COMMand 0..30,&lt;command string&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Device_Address">
        /// This control sets the GPIB address
        /// 
        /// Valid Values: 0 to 30
        /// 
        /// Default Value: 28
        /// 
        /// Note:
        /// 
        /// If two generators are connected at the same time to IECBUS 2 their addresses must be different.
        /// 
        /// </param>
        /// <param name="Command_String">
        /// This control sethe command string, send to the device using GPIB interface.
        /// 
        /// Valid Values: 
        /// any command string.
        /// 
        /// Default Value: ""
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SendCommandToExternalGenerator(int Device_Address, string Command_String)
        {
            int pInvokeResult = PInvoke.SendCommandToExternalGenerator(this._handle, Device_Address, Command_String);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the tracking generator normalization.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TGEN_NORMALIZATION_STATE      
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection[:STATe] ON | OFF
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// Specifies whether the spectrum analyzer is enabled for auto attenuation.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TGEN_NORMALIZATION_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ExternalGeneratorNormalization(int Window, bool State)
        {
            int pInvokeResult = PInvoke.ExternalGeneratorNormalization(this._handle, Window, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates the selected tracking generator calibration.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TGEN_METHOD
        /// RSSPECAN_ATTR_TGEN_ACQUIRE
        /// 
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:METHod TRANsmission | REFLection
        /// [SENSe&lt;1|2&gt;:]CORRection:COLLect[:ACQuire] THRough | OPEN
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Method">
        /// This control selects the kind of measurement with active tracking generator (transmission/reflexion) and kind of measurement for the reference values of the normalization (response calibration).
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TGEN_CAL_TRAN  (0) - Transmission Calibration 
        /// RSSPECAN_VAL_TGEN_CAL_REFLO (1) - Reflection Calibration Open
        /// RSSPECAN_VAL_TGEN_CAL_REFLS (2) - Reflection Calibration Short
        /// 
        /// Default Value: RSSPECAN_VAL_TGEN_CAL_TRAN  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TGEN_METHOD
        /// RSSPECAN_ATTR_TGEN_ACQUIRE
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value for data readout in milliseconds.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 10000 ms
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ExternalGeneratorCalibration(int Window, int Method, uint Timeout)
        {
            int pInvokeResult = PInvoke.ExternalGeneratorCalibration(this._handle, Window, Method, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function restores the instrument setting that was valid for the measurement of the reference data, provided that the tracking generator is active.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TGEN_RECALL
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:RECall
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ExternalGeneratorRecall(int Window)
        {
            int pInvokeResult = PInvoke.ExternalGeneratorRecall(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function uses the normalized measurement data to
        /// generate a transducer factor with up to 625 points.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TFAC_GEN_NAME
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:GENerate &lt;name&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Name">
        /// This control sets the name of saved transducer factor.
        /// 
        /// Valid Values:
        /// any valid transducer factor name
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TFAC_GEN_NAME
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ExternalGeneratorSaveAsTrdFactor(int Window, string Name)
        {
            int pInvokeResult = PInvoke.ExternalGeneratorSaveAsTrdFactor(this._handle, Window, Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the Network analyzer for mobile and base station tests.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NAN_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument&lt;1|2&gt;[:SELect] NAN
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int NetworkAnalyzerMode()
        {
            int pInvokeResult = PInvoke.NetworkAnalyzerMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the network analyzer measurement.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NAN_MEAS_MODE
        /// RSSPECAN_ATTR_NAN_TRANSMISSION_STATE
        /// RSSPECAN_ATTR_NAN_MEAS_FORMAT
        /// RSSPECAN_ATTR_NAN_MEAS_PORT
        /// 
        /// Remote-control command(s):
        /// MEASurement:MODE SCALar | VECTor | VVMeter
        /// MEASurement:FUNCtion:TRANsmission[:STATe]
        /// MEASurement:FORMat MAGNitude | PHASe | MPHase | GDELay
        /// MEASurement:PORT 1 | 2
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement_Mode">
        /// This control sets the measurement mode for network analysis measurements and distance to fault measurements.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_NAN_MEAS_MODE_SCALAR     (0) - Scalar
        /// RSSPECAN_VAL_NAN_MEAS_MODE_VECTOR     (1) - Vector
        /// RSSPECAN_VAL_NAN_MEAS_MODE_VVM        (2) - VV Meter
        /// RSSPECAN_VAL_DTF_MEAS_MODE_DTF        (3) - DTF
        /// RSSPECAN_VAL_DTF_MEAS_MODE_REFLECTION (4) - Reflection
        /// RSSPECAN_VAL_DTF_MEAS_MODE_SPECTRUM   (5) - Spectrum
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_NAN_MEAS_MODE_VECTOR     (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NAN_MEAS_MODE
        /// </param>
        /// <param name="Transmission_State">
        /// This control sets the transmission measurement in network analysis mode.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value:
        /// VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NAN_TRANSMISSION_STATE
        /// </param>
        /// <param name="Measurement_Format">
        /// This control sets the graphical format in which the measurement result are presented.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEAS_FORMAT_MAGNITUDE  (0) - Magnitude
        /// RSSPECAN_VAL_MEAS_FORMAT_PHASE      (1) - Phase
        /// RSSPECAN_VAL_MEAS_FORMAT_MPHASE     (2) - M Phase
        /// RSSPECAN_VAL_MEAS_FORMAT_GDELAY     (7) - Group Delay
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_MEAS_FORMAT_MAGNITUDE  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NAN_MEAS_FORMAT
        /// </param>
        /// <param name="Measurement_Port">
        /// This control sets the measurement port used for two-port measurements.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value:
        /// 2
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NAN_MEAS_PORT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureNetworkAnalyzerMeasurement(int Measurement_Mode, bool Transmission_State, int Measurement_Format, int Measurement_Port)
        {
            int pInvokeResult = PInvoke.ConfigureNetworkAnalyzerMeasurement(this._handle, Measurement_Mode, System.Convert.ToUInt16(Transmission_State), Measurement_Format, Measurement_Port);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the reflection measurements state.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NAN_REFLECTION_STATE
        /// 
        /// Remote-control command(s):
        /// MEASurement:FUNCtion:REFLection[:STATe]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Reflection_State">
        /// This control sets the reflection measurements state.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value:
        /// VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NAN_REFLECTION_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureNetworkAnalyzerReflectionState(bool Reflection_State)
        {
            int pInvokeResult = PInvoke.ConfigureNetworkAnalyzerReflectionState(this._handle, System.Convert.ToUInt16(Reflection_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the electrical cable length state.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NAN_ELECTRICAL_CABLE_STATE
        /// 
        /// Remote-control command(s):
        /// CALCulate:TRACe:CABLe:LENGth[:STATe]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="State">
        /// This control sets the electrical cable length state.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value:
        /// VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NAN_ELECTRICAL_CABLE_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureElectricalCableState(bool State)
        {
            int pInvokeResult = PInvoke.ConfigureElectricalCableState(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the scaling of the y-axis for the Magnitude measurement format to either linear or logarithmic.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NAN_MAGN_SPACING
        /// 
        /// Remote-control command(s):
        /// DISPlay:MAGNitude:Y:SPACing LINear | LOGarithmic
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Spacing">
        /// This control sets the scaling of the y-axis for the Magnitude measurement format to either linear or logarithmic.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MAGNITUDE_SPACING_LIN (0) - Linear
        /// RSSPECAN_VAL_MAGNITUDE_SPACING_LOG (1) - Logarythmic
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_MAGNITUDE_SPACING_LOG (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NAN_MAGN_SPACING
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureMagnitudeSpacing(int Spacing)
        {
            int pInvokeResult = PInvoke.ConfigureMagnitudeSpacing(this._handle, Spacing);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the output format for the (complex) value of the marker.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NETWORK_ANALYZER_MARKER_OUTPUT_FORMAT
        /// 
        /// Remote-control command(s):
        /// CALCulate:MARKer&lt;1...6&gt;:MODE NORMal | RPDB | RPL | RSCalar | IMPedance | ADMittance | NIMPedance | NADMittance
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker number.
        /// 
        /// Valid Values:
        /// 1 to 6
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Output_Format">
        /// This control sets the output format for the (complex) value of the marker.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MARKER_OUTPUT_FORMAT_NORM (0) - Normal
        /// RSSPECAN_VAL_MARKER_OUTPUT_FORMAT_RPDB (1) - RPDB
        /// RSSPECAN_VAL_MARKER_OUTPUT_FORMAT_RPL  (2) - RPL
        /// RSSPECAN_VAL_MARKER_OUTPUT_FORMAT_RSC  (3) - RSC
        /// RSSPECAN_VAL_MARKER_OUTPUT_FORMAT_IMP  (4) - Impedance
        /// RSSPECAN_VAL_MARKER_OUTPUT_FORMAT_ADM  (5) - Admittance
        /// RSSPECAN_VAL_MARKER_OUTPUT_FORMAT_NIMP (6) - NIMP 
        /// RSSPECAN_VAL_MARKER_OUTPUT_FORMAT_NADM (7) - NADM
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_MARKER_OUTPUT_FORMAT_NORM (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NETWORK_ANALYZER_MARKER_OUTPUT_FORMAT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureMarkerOutputFormat(int Marker, int Output_Format)
        {
            int pInvokeResult = PInvoke.ConfigureMarkerOutputFormat(this._handle, Marker, Output_Format);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the reference impedance for the Smith chart measurement format. The impedance can be between 1mohm and 10 kohm.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NAN_SMITH_REF_IMPEDANCE
        /// 
        /// Remote-control command(s):
        /// DISPlay:IMPedance:REFerence:POSition
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Reference_Impedance">
        /// This control sets the reference impedance for the Smith chart measurement format.
        /// 
        /// Valid Values:
        /// 1.0e-3 to 10.0e+3
        /// 
        /// Default Value:
        /// 50.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NAN_SMITH_REF_IMPEDANCE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureSmithChartReferenceImpedance(double Reference_Impedance)
        {
            int pInvokeResult = PInvoke.ConfigureSmithChartReferenceImpedance(this._handle, Reference_Impedance);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the display range of the y-axis for the VSWR measurement format.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NAN_VSWR_Y_SCALE
        /// 
        /// Remote-control command(s):
        /// DISPlay:VSWR:Y:SCALe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Y_Scale">
        /// This control sets the display range of the y-axis for the VSWR measurement format.
        /// 
        /// Valid Values:
        /// 1.0 to 71.0
        /// 
        /// Default Value:
        /// 1.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NAN_VSWR_Y_SCALE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureVSWRYScale(double Y_Scale)
        {
            int pInvokeResult = PInvoke.ConfigureVSWRYScale(this._handle, Y_Scale);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function removes the restriction limiting the value range to +/- 180?. Phase can have values from 0? to 100000?.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NAN_PHASE_UNWRAP
        /// 
        /// Remote-control command(s):
        /// DISPlay:PHASe:Y:UNWRap
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Phase_Unwrap">
        /// This control removes the restriction limiting the value range to +/- 180?.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value:
        /// VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NAN_PHASE_UNWRAP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigurePhaseUnwrap(bool Phase_Unwrap)
        {
            int pInvokeResult = PInvoke.ConfigurePhaseUnwrap(this._handle, System.Convert.ToUInt16(Phase_Unwrap));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the unit for the reflection coefficient.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NAN_REFL_COEFFICIENT_UNIT
        /// 
        /// Remote-control command(s):
        /// DISPlay:REFLection:UNIT RHO | MRHO
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Unit">
        /// This control sets the unit for the reflection coefficient.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DISPLAY_REFLECTION_UNIT_RHO  (0) - RHO
        /// RSSPECAN_VAL_DISPLAY_REFLECTION_UNIT_MRHO (1) - MRHO
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_DISPLAY_REFLECTION_UNIT_MRHO (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NAN_REFL_COEFFICIENT_UNIT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureReflectionCoefficientUnit(int Unit)
        {
            int pInvokeResult = PInvoke.ConfigureReflectionCoefficientUnit(this._handle, Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function the tracking generator reference level.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NAN_CABLE_Y_SCALE
        /// RSSPECAN_ATTR_NAN_MAGN_Y_SCALE
        /// RSSPECAN_ATTR_NAN_PHASE_Y_SCALE
        /// RSSPECAN_ATTR_NAN_REFL_COEFFICIENT_Y_SCALE
        /// 
        /// Remote-control command(s):
        /// DISPlay:LOSS:Y:SCALe
        /// DISPlay:MAGNitude:Y:SCALe
        /// DISPlay:PHASe:Y:SCALe
        /// DISPlay:REFLection:Y:SCALe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Type">
        /// This control sets the type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TRACKING_GEN_TYPE_CLOSS (0) - Cable Loss
        /// RSSPECAN_VAL_TRACKING_GEN_TYPE_MAGN  (1) - Magnitude
        /// RSSPECAN_VAL_TRACKING_GEN_TYPE_PHAS  (2) - Phase
        /// RSSPECAN_VAL_TRACKING_GEN_TYPE_REFL  (3) - Reflection Coefficient
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_TRACKING_GEN_TYPE_CLOSS (0)
        /// </param>
        /// <param name="Y_Scale">
        /// This control sets the display range of the y-axis with logarithmic scaling.
        /// 
        /// Valid Values:
        /// 
        /// 
        /// Default Value:
        /// 100.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NAN_CABLE_Y_SCALE or
        /// RSSPECAN_ATTR_NAN_MAGN_Y_SCALE or
        /// RSSPECAN_ATTR_NAN_PHASE_Y_SCALE or
        /// RSSPECAN_ATTR_NAN_REFL_COEFFICIENT_Y_SCALE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureTrackingGeneratorYScale(int Type, double Y_Scale)
        {
            int pInvokeResult = PInvoke.ConfigureTrackingGeneratorYScale(this._handle, Type, Y_Scale);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the tracking generator reference level.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NAN_CABLE_REF_LEVEL
        /// RSSPECAN_ATTR_NAN_MAGN_REF_LEVEL
        /// RSSPECAN_ATTR_NAN_PHASE_REF_LEVEL
        /// RSSPECAN_ATTR_NAN_CABLE_REF_POSITION
        /// RSSPECAN_ATTR_NAN_MAGN_REF_POSITION
        /// RSSPECAN_ATTR_NAN_PHASE_REF_POSITION
        /// 
        /// Remote-control command(s):
        /// DISPlay:LOSS:REFerence
        /// DISPlay:MAGNitude:REFerence
        /// DISPlay:PHASe:REFerence
        /// DISPlay:LOSS:REFerence:POSition
        /// DISPlay:MAGNitude:REFerence:POSition
        /// DISPlay:PHASe:REFerence:POSition
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Type">
        /// This control sets the type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TRACKING_GEN_TYPE_CLOSS (0) - Cable Loss
        /// RSSPECAN_VAL_TRACKING_GEN_TYPE_MAGN  (1) - Magnitude
        /// RSSPECAN_VAL_TRACKING_GEN_TYPE_PHAS  (2) - Phase
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_TRACKING_GEN_TYPE_CLOSS (0)
        /// </param>
        /// <param name="Reference_Level">
        /// This control sets the reference level.
        /// 
        /// Valid Values:
        /// 
        /// 
        /// Default Value:
        /// 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NAN_CABLE_REF_LEVEL or
        /// RSSPECAN_ATTR_NAN_MAGN_REF_LEVEL or
        /// RSSPECAN_ATTR_NAN_PHASE_REF_LEVEL
        /// </param>
        /// <param name="Reference_Position">
        /// This control sets the position of the reference value on the display.
        /// 
        /// Valid Values:
        /// 1 to 10
        /// 
        /// Default Value:
        /// 10
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NAN_CABLE_REF_POSITION or
        /// RSSPECAN_ATTR_NAN_MAGN_REF_POSITION or
        /// RSSPECAN_ATTR_NAN_PHASE_REF_POSITION
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureTrackingGeneratorReferenceLevel(int Type, double Reference_Level, int Reference_Position)
        {
            int pInvokeResult = PInvoke.ConfigureTrackingGeneratorReferenceLevel(this._handle, Type, Reference_Level, Reference_Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the output level of the tracking generator.
        /// You can set the output level by selecting an attenuation value. The range is from 0 dB to 50 dB.
        /// Setting the attenuation to 0 dB results in an output level of 0 dBm. If you set the attenuation to 50 dB, the resulting output level is -50 dBm.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NAN_OUTPUT_LEVEL
        /// 
        /// Remote-control command(s):
        /// SOURce:TG:ATTenuation
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Output_Level">
        /// This control sets the output level of the tracking generator.
        /// 
        /// Valid Values: 
        /// 0.0 to 50.0
        /// 
        /// Default Value: 
        /// 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NAN_OUTPUT_LEVEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureTrackingGeneratorOutputLevel(double Output_Level)
        {
            int pInvokeResult = PInvoke.ConfigureTrackingGeneratorOutputLevel(this._handle, Output_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the measurement mode which is calibrated.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NAN_CALIBRATION_MODE
        /// 
        /// Remote-control command(s):
        /// CALibration:MODE?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Calibration_Mode">
        /// This control displays the measurement mode which is calibrated.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NAN_CALIBRATION_MODE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int QueryCalibrationMode(out int Calibration_Mode)
        {
            int pInvokeResult = PInvoke.QueryCalibrationMode(this._handle, out Calibration_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries whether the R&amp;S FSH is calibrated for the current measurement mode or not.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NAN_CALIBRATION_STATUS
        /// 
        /// Remote-control command(s):
        /// CALibration:STATus?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Calibration_Status">
        /// This control displays whether the R&amp;S FSH is calibrated for the current measurement mode or not.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NAN_CALIBRATION_STATUS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int QueryCalibrationStatus(System.Text.StringBuilder Calibration_Status)
        {
            int pInvokeResult = PInvoke.QueryCalibrationStatus(this._handle, Calibration_Status);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the electrical cable length.
        /// 
        /// Notes:
        /// 
        /// (1) This function is only available when measurement mode is Phase and zero span is not active.
        /// 
        /// (2) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NAN_CABLE_LENGTH
        /// 
        /// Remote-control command(s):
        /// CALCulate:TRACe:CABLe:LENGth:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Electrical_Cable_Length">
        /// Returns the electrical cable length.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NAN_CABLE_LENGTH
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int QueryElectricalCableLength(out double Electrical_Cable_Length)
        {
            int pInvokeResult = PInvoke.QueryElectricalCableLength(this._handle, out Electrical_Cable_Length);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the results of the limit check for the VSWR measurement format.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NAN_VSWR_FAIL
        /// 
        /// Remote-control command(s):
        /// CALCulate:TRACe:LIMit:VSWR:FAIL?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Fail">
        /// This control displays the results of the limit check for the VSWR measurement format.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NAN_VSWR_FAIL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int QueryVSWRFail(System.Text.StringBuilder Fail)
        {
            int pInvokeResult = PInvoke.QueryVSWRFail(this._handle, Fail);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the user calibration.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_USER_CALIBRATE_STATE
        /// RSSPECAN_ATTR_USER_CALIBRATE_LENGTH
        /// RSSPECAN_ATTR_USER_CALIBRATE_OFFS
        /// 
        /// Remote-control command(s):
        /// CALKit:USER[:STATe]
        /// CALKit:USER:LENGth
        /// CALKit:USER:OFFSet:LENGth
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="User_Calkit">
        /// This control sets the user calibration state.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value:
        /// VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_USER_CALIBRATE_STATE
        /// </param>
        /// <param name="User_Length">
        /// This control sets the electrical length of the user calibration.
        /// 
        /// Valid Values:
        /// 0.00 to 1000.00 mm
        /// 
        /// Default Value:
        /// 5.27 mm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_USER_CALIBRATE_LENGTH
        /// </param>
        /// <param name="Offset_Length">
        /// This control sets the electrical length offset.
        /// 
        /// Valid Values:
        /// 0.00 to 100000.00 mm
        /// 
        /// Default Value:
        /// 0.00 mm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_USER_CALIBRATE_OFFS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureUserCalibrate(bool User_Calkit, double User_Length, double Offset_Length)
        {
            int pInvokeResult = PInvoke.ConfigureUserCalibrate(this._handle, System.Convert.ToUInt16(User_Calkit), User_Length, Offset_Length);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the name for the user calibration to save the customized settings under a user defined name.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_USER_CALIBRATE_RENAME
        /// 
        /// Remote-control command(s):
        /// CALKit:USER:NAME
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Rename_User">
        /// This control sets the name for the user calibration to save the customized settings under a user defined name.
        /// 
        /// Valid Values:
        /// Not Checked
        /// 
        /// Default Value:
        /// ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_USER_CALIBRATE_RENAME
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int UserCalkitRename(string Rename_User)
        {
            int pInvokeResult = PInvoke.UserCalkitRename(this._handle, Rename_User);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the group delay.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GDEL_APERTURE_STEPS
        /// RSSPECAN_ATTR_GDEL_REF_LEVEL
        /// RSSPECAN_ATTR_GDEL_REF_POSITION
        /// RSSPECAN_ATTR_GDEL_Y_SCALE
        /// 
        /// Remote-control command(s):
        /// DISPlay:GDELay:APERture:STEP
        /// DISPlay:GDELay:REFerence
        /// DISPlay:GDELay:REFerence:POSition
        /// DISPlay:GDELay:Y:SCALe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Aperture_Steps">
        /// This control sets the aperture steps for Group Delay measurements.
        /// 
        /// Valid Values:
        /// 1 to 630
        /// 
        /// Default Value:
        /// 10
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GDEL_APERTURE_STEPS
        /// </param>
        /// <param name="Reference_Level">
        /// This control sets the reference level for the Cable Loss measurement format.
        /// 
        /// Valid Values:
        /// 1 to 10000 ns
        /// 
        /// Default Value:
        /// 1 ns
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GDEL_REF_LEVEL
        /// </param>
        /// <param name="Reference_Position">
        /// This control sets the position of the reference value on the display for the Group Delay measurement format.
        /// 
        /// Valid Values:
        /// 1 to 10
        /// 
        /// Default Value:
        /// 5
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GDEL_REF_POSITION
        /// </param>
        /// <param name="Y_Scale">
        /// This control sets the display range of the y-axis for the Group Delay measurement format.
        /// 
        /// Valid Values:
        /// Not Checked
        /// 
        /// Default Value:
        /// 100e-9 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GDEL_Y_SCALE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureGroupDelay(int Aperture_Steps, int Reference_Level, int Reference_Position, double Y_Scale)
        {
            int pInvokeResult = PInvoke.ConfigureGroupDelay(this._handle, Aperture_Steps, Reference_Level, Reference_Position, Y_Scale);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the current measurement values as reference values. The reference values can be used for comparison measurements.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Required Option(s):
        /// FSH-K45
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VVM_REF_STATE
        /// 
        /// Remote-control command(s):
        /// CALCulate:VVMeter:REFerence[:STATe]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="State">
        /// This control sets the current measurement values as reference values.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value:
        /// VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VVM_REF_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureVectorVoltmeterReferenceState(bool State)
        {
            int pInvokeResult = PInvoke.ConfigureVectorVoltmeterReferenceState(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the reference values for the return loss of the DUT in vector voltmeter mode.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Required Option(s):
        /// FSH-K45
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VVM_MAGN_REF_VALUE
        /// 
        /// Remote-control command(s):
        /// CALCulate:VVMeter:MAGNitude:REFerence?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Reference_Value">
        /// This control displays the reference values for the return loss of the DUT in vector voltmeter mode.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VVM_MAGN_REF_VALUE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int QueryVectorVoltmeterMagnitudeReferenceValue(out double Reference_Value)
        {
            int pInvokeResult = PInvoke.QueryVectorVoltmeterMagnitudeReferenceValue(this._handle, out Reference_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the numerical values for the return loss of the DUT in vector voltmeter mode.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Required Option(s):
        /// FSH-K45
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VVM_MAGN_VALUE
        /// 
        /// Remote-control command(s):
        /// CALCulate:VVMeter:MAGNitude:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Value">
        /// This control displays the numerical values for the return loss of the DUT in vector voltmeter mode.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VVM_MAGN_VALUE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int QueryVectorVoltmeterMagnitudeValue(out double Value)
        {
            int pInvokeResult = PInvoke.QueryVectorVoltmeterMagnitudeValue(this._handle, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the reference values for the phase of the DUT in vector voltmeter mode.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Required Option(s):
        /// FSH-K45
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VVM_PHASE_REF_VALUE
        /// 
        /// Remote-control command(s):
        /// CALCulate:VVMeter:PHASe:REFerence?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Value">
        /// This control displays the reference values for the phase of the DUT in vector voltmeter mode.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VVM_PHASE_REF_VALUE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int QueryVectorVoltmeterPhaseReferenceValue(out double Value)
        {
            int pInvokeResult = PInvoke.QueryVectorVoltmeterPhaseReferenceValue(this._handle, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the numerical values for the phase of the DUT in vector voltmeter mode.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Required Option(s):
        /// FSH-K45
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VVM_PHASE_VALUE
        /// 
        /// Remote-control command(s):
        /// CALCulate:VVMeter:PHASe:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Value">
        /// This control displays the numerical values for the phase of the DUT in vector voltmeter mode.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VVM_PHASE_VALUE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int QueryVectorVoltmeterPhaseValue(out double Value)
        {
            int pInvokeResult = PInvoke.QueryVectorVoltmeterPhaseValue(this._handle, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects distance to fault mode.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with FSH and FSC.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISTANCE_TO_FAULT_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument[:SELect] DTF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int DTFMode()
        {
            int pInvokeResult = PInvoke.DTFMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the maximum length of the cable that is used for distance-to-fault measurements.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DTF_CABLE_LENGTH
        /// 
        /// Remote-control command(s):
        /// CALCulate:DLINe:DTF:CABle:LENGth
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Cable_Length">
        /// Defines the cable length. The unit of the length can either be Meter or Feet depending on the Length Unit setting. The cable length is converted and rounded to meters internally.
        /// 
        /// Valid Range: not checked
        /// 
        ///   3.0 m  to 1000.0 m
        ///  10.0 ft to 3281.0 ft
        /// 
        /// Default Value: 20.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DTF_CABLE_LENGTH
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureCableLength(double Cable_Length)
        {
            int pInvokeResult = PInvoke.ConfigureCableLength(this._handle, Cable_Length);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the cable model that is used for distance-to-fault measurements.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DTF_CABLE_MODEL
        /// 
        /// Remote-control command(s):
        /// CALCulate:DLINe:DTF:CABle:PRESet
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Cable_Model">
        /// This control selects the cable model that is used for distance-to-fault measurements.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DTF_CABLE_MODEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureDistancetoFaultCableModel(string Cable_Model)
        {
            int pInvokeResult = PInvoke.ConfigureDistancetoFaultCableModel(this._handle, Cable_Model);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the threshold for the DTF list.
        /// All values which exceed the threshold are in the DTF list and can be queried with the CALCulate:MARKer&lt;1...6&gt;:FUNCtion:DTF:PEAK:RESult? command.
        /// The numeric suffix &lt;1...6&gt; is irrelevant.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DTF_THRESHOLD
        /// 
        /// Remote-control command(s):
        /// CALCulate:MARKer&lt;1...6&gt;:FUNCtion:DTF:PEAK:THReshold
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Threshold">
        /// This control set the threshold for the DTF list.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DTF_THRESHOLD
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigureDistancetoFaultThreshold(double Threshold)
        {
            int pInvokeResult = PInvoke.ConfigureDistancetoFaultThreshold(this._handle, Threshold);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the DTF list. It contains the return loss and distance from the measurement plane of all reflections that exceed the threshold level. The threshold level is defined with the CALCulate:MARKer&lt;1...6&gt;:FUNCtion:DTF:PEAK:THReshold
        /// command.
        /// The numeric suffix &lt;1..6&gt; is irrelevant.
        /// 
        /// Note:
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DTF_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate:MARKer&lt;1...6&gt;:FUNCtion:DTF:PEAK:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Result">
        /// This control displays the DTF list.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DTF_RESULT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int QueryDistancetoFaultResult(out double Result)
        {
            int pInvokeResult = PInvoke.QueryDistancetoFaultResult(this._handle, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches FM demodulation on or off.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FM_STATE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FM ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// Sets the FM demodulation on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FM_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFMDemodAnalogState(int Window, bool State)
        {
            int pInvokeResult = PInvoke.ConfigureFMDemodAnalogState(this._handle, Window, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the AF lowpass filters.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FM_LPAS_MAN
        /// RSSPECAN_ATTR_FM_LPAS_FILTER
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FM:FILTer[:LPASs]:AUTO ON | OFF
        /// [SENSe&lt;1|2&gt;:]FM:FILTer[:LPASs]:FREQuency
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="AF_Lowpass">
        /// Links the AF lowpass filters to the resolution bandwidth or deactivates coupling.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Auto
        /// VI_TRUE  (1) - Manual 
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FM_LPAS_MAN
        /// </param>
        /// <param name="Frequency">
        /// Sets the AF lowpass filters for FM demodulation.
        /// 
        /// Valid Values: 
        /// 30.0e+3,100.0e+3,300.0e+3,1.0e+6,5.0e+6 Hz
        /// 
        /// Default Value: 30.0e+3 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FM_LPAS_FILTER
        /// 
        /// Note:
        /// (1) This control has effect only if AF Lowpass is set Manual.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFMDemodAnalogLowpassFilter(int Window, bool AF_Lowpass, double Frequency)
        {
            int pInvokeResult = PInvoke.ConfigureFMDemodAnalogLowpassFilter(this._handle, Window, System.Convert.ToUInt16(AF_Lowpass), Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the AF highpass filters.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FM_HPAS_FILTER
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FM:FILTer:HPASs:FREQuency
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Frequency">
        /// Sets the AF highpass filters for disconnecting the DC component. 
        /// 
        /// Valid Values: 
        /// 0.0,10.0,100.0,1.0e+3 Hz
        /// 
        /// Default Value: 0.0 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FM_HPAS_FILTER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFMDemodAnalogHighpassFilter(int Window, double Frequency)
        {
            int pInvokeResult = PInvoke.ConfigureFMDemodAnalogHighpassFilter(this._handle, Window, Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the AF demodulation range.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FM_SPAN_AUTO
        /// RSSPECAN_ATTR_FM_SPAN
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FM[:DEViation]:RANGe:AUTO ON | OFF
        /// [SENSe&lt;1|2&gt;:]FM[:DEViation]:RANGe[:UPPer]
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Span_Auto_Manual">
        /// Links the FM demodulation span to the selected resolution bandwidth.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FM_SPAN_AUTO
        /// </param>
        /// <param name="Span">
        /// Sets the FM demodulation span. The driver uses this value to set the RSSPECAN_ATTR_FM_SPAN attribute.
        /// 
        /// Valid Values: 
        /// no range checking
        /// 
        /// Default Value: 50.0e+3
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FM_SPAN
        /// 
        /// Note:
        /// (1) This control has effect only if Span is set to Manual.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFMDemodAnalogRange(int Window, bool Span_Auto_Manual, double Span)
        {
            int pInvokeResult = PInvoke.ConfigureFMDemodAnalogRange(this._handle, Window, System.Convert.ToUInt16(Span_Auto_Manual), Span);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the system diagnostic
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FFT_CAL_SIG
        /// RSSPECAN_ATTR_FFT_CAL_SIG_SOUR
        /// RSSPECAN_ATTR_FFT_CAL_DC
        /// RSSPECAN_ATTR_FFT_CAL_FREQ
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:IQ:CALibration:DESTination IHIGh | ILOW | QHIGh | QLOW
        /// DIAGnostic&lt;1|2&gt;:SERVice:IQ:INPut IQ | GND | CALDc | CALPulse
        /// DIAGnostic&lt;1|2&gt;:SERVice:IQ:CALibration:DC 0 | 0.1 | 0.178 | 0.316 | 0.562 | 1.0
        /// DIAGnostic&lt;1|2&gt;:SERVice:IQ:CALibration:PULSe:PRATe
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Signal_Path">
        /// Selects the calibration signal.
        /// 
        /// Valid Values:
        /// 
        /// RSSPECAN_VAL_FFT_CAL_IHIGH (0) - I High
        /// RSSPECAN_VAL_FFT_CAL_ILOW  (1) - I Low
        /// RSSPECAN_VAL_FFT_CAL_QHIGH (2) - Q High
        /// RSSPECAN_VAL_FFT_CAL_QLOW  (3) - Q Low
        /// 
        /// Default Value: RSSPECAN_VAL_FFT_CAL_IHIGH
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FFT_CAL_SIG
        /// 
        /// Note:
        /// 
        /// (1) I High - Feed calibration signal to positive I path.
        ///     I Low  - Feed calibration signal to negative I path.
        ///     Q High - Feed calibration signal to positive Q path.
        ///     Q Low  - Feed calibration signal to negative Q path.
        /// 
        /// </param>
        /// <param name="Signal_Source">
        /// Selects the calibration signal.
        /// 
        /// Valid Values:
        /// 
        /// RSSPECAN_VAL_FFT_SIG_SOUR_IQ   (0) - IQ
        /// RSSPECAN_VAL_FFT_SIG_SOUR_GND  (1) - GND
        /// RSSPECAN_VAL_FFT_SIG_SOUR_CALD (2) - Cal DC
        /// RSSPECAN_VAL_FFT_SIG_SOUR_CALP (3) - Cal Pulse
        /// 
        /// Default Value: RSSPECAN_VAL_FFT_SIG_SOUR_IQ
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FFT_CAL_SIG
        /// 
        /// Note:
        /// 
        /// (1) IQ - The female I and Q connectors of the FSQ are the baseband signal sources.
        ///     GND - The baseband inputs are internally connected to ground.
        ///     Cal DC - The baseband signal source is the DC Cal signal.
        ///     Cal Pulse - The baseband signal source is the Pulse Cal signal. 
        /// 
        /// </param>
        /// <param name="Voltage">
        /// Selects the voltage for the DC Cal signal in volts.
        /// 
        /// Valid Values:
        /// 0.0, 0.1, 0.178, 0.316, 0.562, 1.0
        /// 
        /// Default Value:
        /// 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FFT_CAL_DC
        /// 
        /// Note:
        /// 
        /// (1) This control is used only for source Cal DC
        /// </param>
        /// <param name="Frequency">
        /// Selects the frequency of the Pulse Cal signal.
        /// 
        /// 
        /// Valid Values:
        /// 10.0e+3, 62.5e+3, 80.0e+3, 100.0e+3, 102.4e+3, 200.0e+3, 500.0e+3, 1.0e+6, 2.0e+6, 4.0e+6 Hz
        /// 
        /// Default Value:
        /// 62.5e+3
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FFT_CAL_FREQ
        /// 
        /// Note:
        /// 
        /// (1) This control is used only for source Cal Pulse
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFFTDiagnostic(int Signal_Path, int Signal_Source, double Voltage, double Frequency)
        {
            int pInvokeResult = PInvoke.ConfigureFFTDiagnostic(this._handle, Signal_Path, Signal_Source, Voltage, Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures analog baseband input.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BB_INPUT_SELECTION
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:SELect AIQ | RF
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Baseband_Inputs">
        /// Switches the baseband inputs on or off.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_BB_INPUT_RF  (0) - Baseband Input Off
        /// RSSPECAN_VAL_BB_INPUT_AIQ (1) - Analog Baseband Input
        /// 
        /// Default Value: RSSPECAN_VAL_BB_INPUT_RF (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BB_INPUT_SELECTION
        /// 
        /// Note(s):
        /// 
        /// (1) Baseband Input Off: This is not available for R&amp;S FMU.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAnalogBasebandInput(int Baseband_Inputs)
        {
            int pInvokeResult = PInvoke.ConfigureAnalogBasebandInput(this._handle, Baseband_Inputs);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the measurement range of the baseband input. The value range depends on the input impedance. The measurement range defines the measurable peak voltage (positive and negative).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BB_INPUT_MEAS_RANGE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]VOLTage:IQ:RANGe[:UPPer] 0.0316 | 0.0562 | 0.1 | 0.178 | 0.316 | 0.562 | 1 | 1.78 | 3.16 | 5.62
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement_Range">
        /// This control defines the measurement range of the baseband input. The value range depends on the input impedance. The measurement range defines the measurable peak voltage (positive and negative).
        /// 
        /// Valid Values:
        /// 0.0316, 0.0562, 0.1, 0.178, 0.316, 0.562, 1.0, 1.78, 3.16, 5.62
        /// 
        /// Default Value: 1.0 V
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BB_INPUT_MEAS_RANGE
        /// 
        /// Note(s):
        /// 
        /// (1) Values 3.16, 5.62 are valid only for low input impedance (function rsspecan_ConfigureAnalogBasebandParameters).
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAnalogBasebandInputMeasurementRange(double Measurement_Range)
        {
            int pInvokeResult = PInvoke.ConfigureAnalogBasebandInputMeasurementRange(this._handle, Measurement_Range);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures analog baseband input trigger.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BB_INPUT_TRIGGER
        /// 
        /// Remote-control command(s):
        /// TRIGger[:SEQuence]:SOURce IONLy | QONLy
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Trigger">
        /// Selects the basebant input trigger.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_BB_TRIG_IONLY (0) - I Level
        /// RSSPECAN_VAL_BB_TRIG_QONLY (1) - Q Level
        /// 
        /// Default Value: RSSPECAN_VAL_BB_TRIG_IONLY (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BB_INPUT_TRIGGER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAnalogBasebandTrigger(int Trigger)
        {
            int pInvokeResult = PInvoke.ConfigureAnalogBasebandTrigger(this._handle, Trigger);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures analog baseband input trigger level.
        /// 
        /// Note(s):
        /// 
        /// (1) It is only available in the Time Domain Voltage mode (function rsspecan_ConfigureFFTTimeDomain).
        /// 
        /// (2) This function is only available for R&amp;S FMU.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BB_INPUT_TRIGGER_I_LEVEL
        /// RSSPECAN_ATTR_BB_INPUT_TRIGGER_Q_LEVEL
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:IONLy
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:QONLy
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Trigger">
        /// Selects the basebant input trigger level.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_BB_TRIG_IONLY (0) - I Level
        /// RSSPECAN_VAL_BB_TRIG_QONLY (1) - Q Level
        /// 
        /// Default Value: RSSPECAN_VAL_BB_TRIG_IONLY (0)
        /// 
        /// </param>
        /// <param name="Trigger_Level">
        /// This control sets the level of the I level or Q level trigger source. The driver uses this value to set the RSSPECAN_ATTR_BB_INPUT_TRIGGER_I_LEVEL or RSSPECAN_ATTR_BB_INPUT_TRIGGER_Q_LEVEL attribute.
        /// 
        /// Valid Values:
        /// -7.9242 to +7.9242 V
        /// 
        /// Default Value: 0.0 V
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BB_INPUT_TRIGGER_I_LEVEL
        /// RSSPECAN_ATTR_BB_INPUT_TRIGGER_Q_LEVEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAnalogBasebandTriggerLevel(int Trigger, double Trigger_Level)
        {
            int pInvokeResult = PInvoke.ConfigureAnalogBasebandTriggerLevel(this._handle, Trigger, Trigger_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures analog baseband parameters.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BB_INPUT_BALANCED
        /// RSSPECAN_ATTR_BB_INPUT_IMPEDANCE
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:IQ:BALanced[:STATe] ON | OFF
        /// INPut&lt;1|2&gt;:IQ:IMPedance LOW | HIGH
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Balanced">
        /// This attribute toggles the baseband inputs between balanced and unbalanced.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off (Unbalanced)
        /// VI_TRUE  (1) - On  (Balanced)
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BB_INPUT_BALANCED
        /// </param>
        /// <param name="Impedance">
        /// Selects the impedance of the baseband inputs.
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_BB_INPUT_IMP_LOW  (0) - Low  (50 Ohm)
        /// RSSPECAN_VAL_BB_INPUT_IMP_HIGH (1) - High (1 kOhm)
        /// 
        /// Default Value: RSSPECAN_VAL_BB_INPUT_IMP_LOW (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BB_INPUT_IMPEDANCE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAnalogBasebandParameters(bool Balanced, int Impedance)
        {
            int pInvokeResult = PInvoke.ConfigureAnalogBasebandParameters(this._handle, System.Convert.ToUInt16(Balanced), Impedance);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the signal path.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BB_INPUT_SIGNAL_PATH
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:IQ:TYPE I | Q | IQ
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Path">
        /// Defines the input signal path used.
        /// 
        /// Valid Values:
        /// 
        /// RSSPECAN_VAL_BB_SIG_PATH_I  (0) - I Only
        /// RSSPECAN_VAL_BB_SIG_PATH_Q  (1) - Q Only
        /// RSSPECAN_VAL_BB_SIG_PATH_IQ (2) - I+j*Q
        /// 
        /// Default Value: RSSPECAN_VAL_BB_SIG_PATH_I
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BB_INPUT_SIGNAL_PATH
        /// 
        /// Note:
        /// (1) I Only - Real signal I only
        ///     Q Only - Real signal Q only
        ///     I+j*Q  - Complex signal I+j*Q
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAnalogBasebandSignalPath(int Path)
        {
            int pInvokeResult = PInvoke.ConfigureAnalogBasebandSignalPath(this._handle, Path);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the signal source low pass.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BB_INPUT_IQ_LPAS
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]IQ:LPASs[:STATe] ON | OFF
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Low_Pass">
        /// Inserts a 36 MHz filter into the I and Q paths of the baseband input.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BB_INPUT_IQ_LPAS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAnalogBasebandSignalSourceLowPass(bool Low_Pass)
        {
            int pInvokeResult = PInvoke.ConfigureAnalogBasebandSignalSourceLowPass(this._handle, System.Convert.ToUInt16(Low_Pass));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the signal source dither.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BB_INPUT_IQ_DITH
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]IQ:DITHer[:STATe] ON | OFF
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="IQ_Dither">
        /// Inserts a 2 MHz-wide noise signal at 42.67 MHz into the signal path of the baseband input.When measuring data with TRAC:IQ subsystem it appears in the spectrum at 38.92 MHz.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BB_INPUT_IQ_DITH
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAnalogBasebandSignalSourceDither(bool IQ_Dither)
        {
            int pInvokeResult = PInvoke.ConfigureAnalogBasebandSignalSourceDither(this._handle, System.Convert.ToUInt16(IQ_Dither));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the FFT Analyzer mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FFT
        /// 
        /// Remote-control command(s):
        /// INSTrument[:SELect] FANalyzer
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FFTSetFFTMode()
        {
            int pInvokeResult = PInvoke.FFTSetFFTMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the FFT Analyzer mode to the default state.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FFT_PRESET
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FFT:PRESet[:DEVice]
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FFTPreset()
        {
            int pInvokeResult = PInvoke.FFTPreset(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function restarts the analysis of captured data with new parameter settings, e.g. resolution bandwidth, window function or domain. This function is only available for CAPTURE BOTH DOMAIN ON , SINGLE SWEEP and Recalculation Auto is OFF
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FFT_CAPT_CALC
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FFT:CAPTureCALCulate
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FFTRecalc()
        {
            int pInvokeResult = PInvoke.FFTRecalc(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the vertical attributes of the FFT analyzer.  This corresponds to attributes like amplitude units,  reference level, and reference level offset.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AMPLITUDE_UNITS
        /// RSSPECAN_ATTR_REFERENCE_LEVEL
        /// RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET
        /// 
        /// 
        /// Remote-control command(s):
        /// UNIT&lt;1|2&gt;:POWer DBM | DBPW | WATT | DBUV | DBMV | VOLT | DBUA | AMPere
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:RLEVel
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:RLEVel:OFFSet
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Amplitude_Units">
        /// Specifies the amplitude units for input, output and display. Pass the units you want the spectrum analyzer to use.  
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_DBM  (0) - dBm 
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_DBMV (1) - dBmV
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_DBUV (2) - dBuV
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_VOLT (3) - Volt
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_WATT (4) - Watt
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_DBPW (5) - dBpW
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_DBUA (6) - dBuA
        /// RSSPECAN_VAL_AMPLITUDE_UNITS_A    (7) - Ampere
        /// 
        /// Default Value: RSSPECAN_VAL_AMPLITUDE_UNITS_DBM (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AMPLITUDE_UNITS
        /// 
        /// Note(s):
        /// 
        /// (1) dBm: Sets the spectrum analyzer to measure in decibels relative to 1 milliwatt.
        /// 
        /// (2) dBmV: Sets the spectrum analyzer to measure in decibels relative to 1 millivolt.
        /// 
        /// (3) dBuV: Sets the spectrum analyzer to measure in decibels relative to 1 microvolt.
        /// 
        /// (4) Volt: Sets the spectrum analyzer to measure in volts.
        /// 
        /// (5) Watt: Sets the spectrum analyzer to measure in watts.
        /// 
        /// (6) dBpW: Sets the spectrum analyzer to measure in pico watts.
        /// 
        /// (7) dBuA: Sets the spectrum analyzer to measure in micro ampers.
        /// 
        /// (8) Ampere: Sets the spectrum analyzer to measure in ampers.
        /// </param>
        /// <param name="Reference_Level">
        /// The calibrated vertical position of the captured data used as a reference for amplitude measurements. This is typically set to a value slightly higher than the highest expected signal level.
        /// 
        /// Valid Values:
        /// not checked, depends on units set by user with RSSPECAN_ATTR_AMPLITUDE_UNITS attribute.
        /// 
        /// Default Value: -20.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_LEVEL
        /// </param>
        /// <param name="Reference_Level_Offset">
        /// Specifies the offset value to the reference level.
        /// 
        /// Units: dB
        /// 
        /// Valid Values: -200.0 to 200.0
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFFTReferenceLevel(int Window, int Amplitude_Units, double Reference_Level, double Reference_Level_Offset)
        {
            int pInvokeResult = PInvoke.ConfigureFFTReferenceLevel(this._handle, Window, Amplitude_Units, Reference_Level, Reference_Level_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the parameters of frequency domain.
        /// 
        /// Note:
        /// (1) The frequency span has to be set &lt;&gt; 0
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FFT_FORM
        /// RSSPECAN_ATTR_FFT_WIN_TYPE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FORMat MAGNitude | RIMag | MPHase 
        /// [SENSe&lt;1|2&gt;:]WINDow:TYPE
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Type">
        /// Selects frequency domain type.
        /// 
        /// Valid Values:
        /// 
        /// RSSPECAN_VAL_FFT_MAGN (0) - Magnitude
        /// RSSPECAN_VAL_FFT_RIM  (3) - Real Image
        /// RSSPECAN_VAL_FFT_MPH  (4) - Magnitude Phase
        /// 
        /// Default Value: RSSPECAN_VAL_FFT_MAGN
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FFT_FORM
        /// 
        /// Note:
        /// 
        /// (1) Magnitude  - display of magnitude
        /// 
        ///     Real Image - display of real and imaginary component of spectrum.
        /// 
        ///     Magnitude Phase - display of magnitude and phase of spectrum.
        /// 
        /// </param>
        /// <param name="Window">
        /// Selects the type of the window function for the FFT input data in the spectrum display (span &gt; 0).
        /// 
        /// Valid Values:
        /// 
        /// RSSPECAN_VAL_FFT_WIND_FLAT (0) - Flattop
        /// RSSPECAN_VAL_FFT_WIND_EXP  (1) - Gaussian
        /// RSSPECAN_VAL_FFT_WIND_HAMM (2) - Hamming
        /// RSSPECAN_VAL_FFT_WIND_HANN (3) - Hann
        /// RSSPECAN_VAL_FFT_WIND_RECT (4) - Rect
        /// RSSPECAN_VAL_FFT_WIND_CHEB (5) - Chebychev
        /// 
        /// Default Value: RSSPECAN_VAL_FFT_WIND_HAMM
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FFT_WIN_TYPE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFFTFrequencyDomain(int Type, int Window)
        {
            int pInvokeResult = PInvoke.ConfigureFFTFrequencyDomain(this._handle, Type, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the parameters of time domain.
        /// 
        /// Note:
        /// (1) The frequency span will be set to zero to switch to time domain.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FREQUENCY_SPAN
        /// RSSPECAN_ATTR_FFT_FORM
        /// 
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:SPAN
        /// CALCulate&lt;1|2&gt;:FORMat MAGNitude | VOLTage
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Type">
        /// Selects frequency domain type.
        /// 
        /// Valid Values:
        /// 
        /// RSSPECAN_VAL_FFT_MAGN (0) - Magnitude
        /// 
        /// RSSPECAN_VAL_FFT_VOLT  (5) - Voltage
        /// 
        /// Default Value: RSSPECAN_VAL_FFT_MAGN
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FFT_FORM
        /// 
        /// Note:
        /// 
        /// (1) Magnitude  - display of magnitude
        /// 
        ///     Voltage - display of voltage.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFFTTimeDomain(int Type)
        {
            int pInvokeResult = PInvoke.ConfigureFFTTimeDomain(this._handle, Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the phase diagram.
        /// 
        /// Note:
        /// (1) This function is available only in the frequency domain of the magnitude/phase display when the FFT Analyzer mode is active.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FFT_FORM
        /// RSSPECAN_ATTR_FFT_OFFS_PHAS
        /// RSSPECAN_ATTR_FFT_UNIT
        /// 
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FORMat PHASe | UPHase
        /// [SENSe&lt;1|2&gt;:]CORRection:OFFSet:PHASe
        /// CALCulate&lt;1|2&gt;:UNIT:ANGLe DEG | RAD
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Phase_Wrap">
        /// Sets phase wrap on or off.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FFT_PHAS (1) - Wrap
        /// RSSPECAN_VAL_FFT_UPH  (2) - Unwrap
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_FFT_PHAS
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FFT_FORM
        /// 
        /// Note:
        /// 
        /// (1) Wrap  - display of phase with limitation to ?180 deg or . ? Phi 
        /// 
        ///     Unwrap - display of phase without limitation to ?180 deg or . ? Phi
        /// 
        /// 
        /// </param>
        /// <param name="Phase_Offset">
        /// Sets a reference value for the phase display in the FFT analyzer mode.
        /// 
        /// Valid Values: -36000 to 36000 deg
        ///               -0.0174533 to 0.0174533 rad
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FFT_OFFS_PHAS
        /// </param>
        /// <param name="Phase_Unit">
        /// Selects phase unit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_ANGLE_DEG (0) - Deg
        /// RSSPECAN_VAL_UNIT_ANGLE_RAD (1) - Rad
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_ANGLE_DEG
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FFT_UNIT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFFTPhaseDiagram(int Phase_Wrap, double Phase_Offset, int Phase_Unit)
        {
            int pInvokeResult = PInvoke.ConfigureFFTPhaseDiagram(this._handle, Phase_Wrap, Phase_Offset, Phase_Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the phase diagram.
        /// 
        /// Note:
        /// (1) This function is available only in the frequency domain of the magnitude/phase display when the FFT Analyzer mode is active.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FFT_PLINE_STATE
        /// RSSPECAN_ATTR_FFT_PLINE
        /// 
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PLINe&lt;1|2&gt;:STATe ON | OFF
        /// CALCulate&lt;1|2&gt;:PLINe&lt;1|2&gt;
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Line">
        /// This control selects line.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Phase_Line_State">
        /// Switches the display lines in the phase diagram on or off.
        /// 
        /// Valid Values: On (VI_TRUE); Off (VI_FALSE)
        /// Default Value: Off (VI_FALSE)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FFT_PLINE_STATE
        /// </param>
        /// <param name="Phase_Line_Position">
        /// Switches the display lines in the phase diagram on or off.
        /// 
        /// Valid Values: -36000 to 36000 deg
        ///               
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FFT_PLINE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFFTPhaseLine(int Window, int Line, bool Phase_Line_State, double Phase_Line_Position)
        {
            int pInvokeResult = PInvoke.ConfigureFFTPhaseLine(this._handle, Window, Line, System.Convert.ToUInt16(Phase_Line_State), Phase_Line_Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the signal source.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BB_INPUT_SIGNAL_PATH
        /// RSSPECAN_ATTR_BB_INPUT_IMPEDANCE
        /// RSSPECAN_ATTR_BB_INPUT_BALANCED
        /// RSSPECAN_ATTR_BB_INPUT_IQ_LPAS
        /// RSSPECAN_ATTR_BB_INPUT_IQ_DITH
        /// 
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:IQ:TYPE I | Q | IQ
        /// INPut&lt;1|2&gt;:IQ:IMPedance LOW | HIGH
        /// INPut&lt;1|2&gt;:IQ:BALanced[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]IQ:LPASs[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]IQ:DITHer[:STATe] ON | OFF
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Path">
        /// Defines the input signal path used. The driver uses this value to set the RSSPECAN_ATTR_BB_INPUT_SIGNAL_PATH attribute.
        /// 
        /// Valid Values:
        /// 
        /// RSSPECAN_VAL_BB_SIG_PATH_I  (0) - I Only
        /// RSSPECAN_VAL_BB_SIG_PATH_Q  (1) - Q Only
        /// RSSPECAN_VAL_BB_SIG_PATH_IQ (2) - I+j*Q
        /// 
        /// Default Value: RSSPECAN_VAL_BB_SIG_PATH_I
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BB_INPUT_SIGNAL_PATH
        /// 
        /// Note:
        /// (1) I Only - Real signal I only
        ///     Q Only - Real signal Q only
        ///     I+j*Q  - Complex signal I+j*Q
        /// </param>
        /// <param name="IQ_Input">
        /// Sets the specifies the input impedance for the IQ inputs. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_BB_INPUT_IMP_LOW  (0)  - 50 Ohm
        /// RSSPECAN_VAL_BB_INPUT_IMP_HIGH (1) - 1kOhm
        /// 
        /// Default Value: RSSPECAN_VAL_BB_INPUT_IMP_LOW
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BB_INPUT_IMPEDANCE
        /// </param>
        /// <param name="IQ_Balanced">
        /// Specifies whether the IQ inputs are symmetrical (balanced) or asymmetrical (unbalanced).
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BB_INPUT_BALANCED
        /// </param>
        /// <param name="Low_Pass">
        /// Inserts a 36 MHz filter into the I and Q paths of the baseband input.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BB_INPUT_IQ_LPAS
        /// </param>
        /// <param name="IQ_Dither">
        /// Inserts a 2 MHz-wide noise signal at 42.67 MHz into the signal path of the baseband input.When measuring data with TRAC:IQ subsystem it appears in the spectrum at 38.92 MHz.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BB_INPUT_IQ_DITH
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFFTSignalSource(int Path, int IQ_Input, bool IQ_Balanced, bool Low_Pass, bool IQ_Dither)
        {
            int pInvokeResult = PInvoke.ConfigureFFTSignalSource(this._handle, Path, IQ_Input, System.Convert.ToUInt16(IQ_Balanced), System.Convert.ToUInt16(Low_Pass), System.Convert.ToUInt16(IQ_Dither));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the data sweep and capture.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SWEEP_MODE_CONTINUOUS
        /// RSSPECAN_ATTR_NUMBER_OF_SWEEPS
        /// RSSPECAN_ATTR_SWEEP_POINTS
        /// RSSPECAN_ATTR_FFT_CAPT
        /// RSSPECAN_ATTR_FFT_CAPT_AUTO
        /// 
        /// Remote-control command(s):
        /// INITiate&lt;1|2&gt;:CONTinuous ON | OFF
        /// [SENSe&lt;1|2&gt;:]SWEep:COUNt
        /// [SENSe&lt;1|2&gt;:]SWEep:POINts
        /// [SENSe&lt;1|2&gt;:]FFT:CAPTure ON | OFF
        /// [SENSe&lt;1|2&gt;:]FFT:CAPTure:AUTO ON | OFF
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Sweep_Mode_Continuous">
        /// Turns on/off continuous sweeping.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE (1) - On
        /// 
        /// Default Value: VI_FALSE (0) - Off
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SWEEP_MODE_CONTINUOUS
        /// </param>
        /// <param name="Sweep_Count">
        /// The number of sweeps to take.
        /// 
        /// Valid Values:
        /// 0 to 32767
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NUMBER_OF_SWEEPS
        /// </param>
        /// <param name="Sweep_Points">
        /// The number of measurement points for one sweep run.
        /// 
        /// Valid Values:
        /// 155 to 10001
        /// 
        /// Default Value: 625
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SWEEP_POINTS
        /// </param>
        /// <param name="Capure_Both_Domain">
        /// Controls how data is captured in the FFT Analyzer mode. While the highest measurement speed is attained with OFF, ON and Single Sweep allow switching between the time and frequency domains even after the measurement without further data capture being necessary.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FFT_CAPT
        /// </param>
        /// <param name="Recalculate_Auto">
        /// Controls the automatic recalculation of measurement results after changing the instruments parameter. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FFT_CAPT_AUTO
        /// 
        /// Note:
        /// (1) This value is available only for capureBothDomain set On and sweepModeContinuous is set Off.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFFTSweep(bool Sweep_Mode_Continuous, int Sweep_Count, int Sweep_Points, bool Capure_Both_Domain, bool Recalculate_Auto)
        {
            int pInvokeResult = PInvoke.ConfigureFFTSweep(this._handle, System.Convert.ToUInt16(Sweep_Mode_Continuous), Sweep_Count, Sweep_Points, System.Convert.ToUInt16(Capure_Both_Domain), System.Convert.ToUInt16(Recalculate_Auto));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the bandwidth for FFT Analyze
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RESOLUTION_BANDWIDTH_AUTO
        /// RSSPECAN_ATTR_RESOLUTION_BANDWIDTH
        /// RSSPECAN_ATTR_RESOLUTION_BANDWIDTH_RATIO
        /// RSSPECAN_ATTR_FFT_BAND_STEP_MODE
        /// 
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth[:RESolution]:AUTO
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth[:RESolution]
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth[:RESolution]:RATio
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth[:RESolution]:STEP:MODE LIN | L1235
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Resolution_Bandwidth_Auto">
        /// Turns resolution bandwidth auto coupling on/off.
        /// 
        /// Valid Values: On (VI_TRUE); Off (VI_FALSE)
        /// Default Value: Off (VI_FALSE)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RESOLUTION_BANDWIDTH_AUTO
        /// 
        /// Note:
        /// (1) Only Off state is valid for Time Domain
        /// 
        /// </param>
        /// <param name="Resolution_Bandwidth">
        /// Specifies the measurement resolution bandwidth.
        /// 
        /// Units: Hertz
        /// 
        /// Default Value: 1.0e+6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RESOLUTION_BANDWIDTH
        /// 
        /// Notes:
        /// 
        /// (1) This value is ignored when resolutionBandwidthAuto is TRUE.
        /// </param>
        /// <param name="Resolution_Bandwidth_Ratio">
        /// Specifies the ratio resolution bandwidth (Hz) / span (Hz). The ratio to be entered is reciprocal to the ratio span/RBW used in manual control.
        /// 
        /// Units: no units
        /// 
        /// Default Value: 0.02
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RESOLUTION_BANDWIDTH_RATIO
        /// </param>
        /// <param name="Resolution_BW_1235">
        /// Controls the rounding of the settable bandwidth in the FFT Analyzer mode (in the frequency domain). The driver uses this value to set the RSSPECAN_ATTR_FFT_BAND_STEP_MODE attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_BSTEP_MODE_LIN   (0) - LIN
        /// RSSPECAN_VAL_BSTEP_MODE_L1235 (1) - L1235
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_BSTEP_MODE_LIN
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FFT_BAND_STEP_MODE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFFTBandwidth(bool Resolution_Bandwidth_Auto, double Resolution_Bandwidth, double Resolution_Bandwidth_Ratio, int Resolution_BW_1235)
        {
            int pInvokeResult = PInvoke.ConfigureFFTBandwidth(this._handle, System.Convert.ToUInt16(Resolution_Bandwidth_Auto), Resolution_Bandwidth, Resolution_Bandwidth_Ratio, Resolution_BW_1235);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures digital baseband input.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BB_INPUT_SELECTION
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:SELect DIQ | RF
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Baseband_Inputs">
        /// Switches the baseband inputs on or off.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_BB_INPUT_RF  (0) - Baseband Input Off
        /// RSSPECAN_VAL_BB_INPUT_DIQ (2) - Digital Baseband Input
        /// 
        /// Default Value: RSSPECAN_VAL_BB_INPUT_RF (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BB_INPUT_SELECTION
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDigitalBasebandInput(int Baseband_Inputs)
        {
            int pInvokeResult = PInvoke.ConfigureDigitalBasebandInput(this._handle, Baseband_Inputs);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures digital baseband input parameters.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DIGITAL_INPUT_RANGE
        /// RSSPECAN_ATTR_DIGITAL_INPUT_SRATE
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:DIQ:RANGe[:UPPer]
        /// INPut&lt;1|2&gt;:DIQ:SRATe
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Range">
        /// This control sets the voltage corresponding to the maximum input value of the digital baseband input.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 1.0 V
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DIGITAL_INPUT_RANGE
        /// </param>
        /// <param name="Sample_Rate">
        /// This control sets the input date sample rate read by the digital baseband input.  
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 81.6e+6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DIGITAL_INPUT_SRATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDigitalBasebandInputParameters(double Range, double Sample_Rate)
        {
            int pInvokeResult = PInvoke.ConfigureDigitalBasebandInputParameters(this._handle, Range, Sample_Rate);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the IQ data output of the Digital Baseband interface.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available for R&amp;S FSV instrument.
        /// 
        /// (2) This function requires the LVDS interface option (B17).
        /// 
        /// (3) Digital input and digital output cannot be used simultaneously.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DIGITAL_BASEBAND_OUTPUT_STATE
        /// 
        /// Remote-control command(s):
        /// OUTPut:DIQ ON | OFF
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Output">
        /// Configures the IQ data output of the Digital Baseband interface.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DIGITAL_BASEBAND_OUTPUT_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDigitalBasebandOutput(bool Output)
        {
            int pInvokeResult = PInvoke.ConfigureDigitalBasebandOutput(this._handle, System.Convert.ToUInt16(Output));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the full scale level.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available for R&amp;S FSV instrument.
        /// 
        /// (2) This function requires the LVDS interface option (B17).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DIGITAL_BASEBAND_FULL_SCALE_LEVEL
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:DIQ:FSCLlevel
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Full_Scale_Level">
        /// Configures the full scale level.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DIGITAL_BASEBAND_FULL_SCALE_LEVEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDigitalBasebandFullScaleLevel(double Full_Scale_Level)
        {
            int pInvokeResult = PInvoke.ConfigureDigitalBasebandFullScaleLevel(this._handle, Full_Scale_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the current configuration and status of the Digital Baseband input or output of option R&amp;S FSQ-B17. The returned string includes the name of the connected device, the serial number of this device, the port name and the sample rate, separated by comma, e.g. "AMU200A,100266,Out A, 80000000.0".
        /// 
        /// Note(s):
        /// 
        /// (1) The function is only available with the option baseband input B17.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_QUERY_DIGITAL_INPUT_CONF
        /// RSSPECAN_ATTR_QUERY_DIGITAL_OUTPUT_CONF
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:DIQ:CDEVice?
        /// OUTPut&lt;1|2&gt;:DIQ:CDEVice?
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Digital_Baseband_Select">
        /// This control selects the digital baseband.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DIQ_IN  (0) - Input
        /// RSSPECAN_VAL_DIQ_OUT (1) - Output
        /// 
        /// Default Value: RSSPECAN_VAL_DIQ_IN (0)
        /// </param>
        /// <param name="Array_Size">
        /// Defines size of array passed to 'Configuration'.
        /// 
        /// Valid Range: &gt; 0
        /// 
        /// Default Value: 256
        /// </param>
        /// <param name="Configuration">
        /// This control returns the current configuration and status of the Digital Baseband input of option R&amp;S FSQ-B17. The returned string includes the name of the connected device, the serial number of this device, the port name and the sample rate, separated by comma, e.g. "AMU200A,100266,Out A, 80000000.0".
        /// 
        /// Note:
        /// 
        /// Array size must be =&gt; then number of elements sent from the instrument.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryDigitalBasebandInputOutputConfiguration(int Digital_Baseband_Select, int Array_Size, System.Text.StringBuilder Configuration)
        {
            int pInvokeResult = PInvoke.QueryDigitalBasebandInputOutputConfiguration(this._handle, Digital_Baseband_Select, Array_Size, Configuration);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the trigger source for the start of a sweep.  
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available for R&amp;S FSV instrument.
        /// 
        /// (2) This function requires the Digital Baseband interface (B17).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DIGITAL_BASEBAND_INPUT_TRIGGER_SOURCE
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:SOURce BBPower
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Trigger_Source">
        /// Selects the trigger source for the start of a sweep.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TRG_BBP  (5) - Baseband Power
        /// 
        /// Default Value: RSSPECAN_VAL_TRG_BBP (5)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DIGITAL_BASEBAND_INPUT_TRIGGER_SOURCE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDigitalBasebandInputTriggerSource(int Trigger_Source)
        {
            int pInvokeResult = PInvoke.ConfigureDigitalBasebandInputTriggerSource(this._handle, Trigger_Source);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the Digital Baseband Input trigger parameters. This is applicable when the trigger source is set to Baseband Power.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available for R&amp;S FSV instrument.
        /// 
        /// (2) This function requires the Digital Baseband interface (B17).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DIGITAL_BASEBAND_INPUT_TRIGGER_LEVEL
        /// RSSPECAN_ATTR_DIGITAL_BASEBAND_INPUT_TRIGGER_HOLDOFF
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:BBPower
        /// TRIGger&lt;1|2&gt;[:SEQuence]:BBPower:HOLDoff
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Trigger_Level">
        /// Sets the level of the baseband power trigger source.
        /// 
        /// Units: dBm
        /// 
        /// Valid Values: -50.0 to -20.0
        /// 
        /// Default Value: -20.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DIGITAL_BASEBAND_INPUT_TRIGGER_LEVEL
        /// </param>
        /// <param name="Holdoff">
        /// Sets the holding time before the next BB power trigger event.
        /// 
        /// Valid Values: 
        /// 150.0e-9 to 1000.0 s
        /// 
        /// Default Value: 150.0e-9
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DIGITAL_BASEBAND_INPUT_TRIGGER_HOLDOFF
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureDigitalBasebandInputTriggerParameters(double Trigger_Level, double Holdoff)
        {
            int pInvokeResult = PInvoke.ConfigureDigitalBasebandInputTriggerParameters(this._handle, Trigger_Level, Holdoff);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the GSM/EDGE analyzer for mobile and base station tests.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument&lt;1|2&gt;[:SELect] MSGM
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GsmMode()
        {
            int pInvokeResult = PInvoke.GsmMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the GSM Trigger Adjustment.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_SYNC_ADJ_IMM
        /// RSSPECAN_ATTR_GSM_SYNC_ADJ_EXT
        /// RSSPECAN_ATTR_GSM_SYNC_ADJ_IFP
        /// RSSPECAN_ATTR_GSM_SYNC_ADJ_RFP
        /// 
        /// Remote-control command(s):
        /// TRIGger[:SEQuence]:SYNChronize:ADJust:IMMediate
        /// TRIGger[:SEQuence]:SYNChronize:ADJust:EXTernal
        /// TRIGger[:SEQuence]:SYNChronize:ADJust:IFPower
        /// TRIGger[:SEQuence]:SYNChronize:ADJust:RFPower
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Trigger_Source">
        /// Choose desired operation.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_TRIG_FRUN (0) - Free Run
        /// RSSPECAN_VAL_GSM_TRIG_EXT  (1) - External GSM Trigger
        /// RSSPECAN_VAL_GSM_TRIG_IFP  (2) - IF Power GSM Trigger
        /// RSSPECAN_VAL_GSM_TRIG_RFP  (3) - RF Power GSM Trigger
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_GSM_TRIG_FRUN
        /// 
        /// Notes:
        /// (1) Free Run - selects the GSM Trigger Free Run.
        /// 
        /// (2) External GSM Trigger:
        /// This command is a combination of 2 commands:
        /// 
        /// For one, the 'Extern' GSM trigger is selected. For all GSM measurements requiring a trigger signal and for which an external trigger is possible, the EXTernal trigger setting is used. If an external trigger is not possible, the IMMediate trigger setting is used (see manual of option).
        /// 
        /// For another, the correction value for the time offset of the external trigger from the beginning of the measured slot is defined (see manual of option). This correction value is needed in order to establish an exact time reference between the trigger event and the beginning of the slot if there is no midamble triggering. The correction value for the DUT in question can be determined in the GENERAL SETTINGS menu with the TRIGGER OFFSET softkey.
        /// 
        /// If the instrument is set to external trigger on switchover to GSM/EDGE mode, the 'Extern' GSM trigger is used, otherwise the 'IF Power' GSM trigger (default) is set.
        /// 
        /// When switching from GSM/EDGE mode to Analyzer mode, the GSM/EDGE trigger setting is maintained, i.e. IF power if 'IF Power' was set before and external trigger, if 'Extern' was set before.
        /// 
        /// (3) IF Power GSM Trigger:
        /// This command is a combination of 2 commands:
        /// 
        /// For one, the 'IF Power' GSM trigger is selected. For all GSM measurements for which an IF power trigger is possible, the IFPower trigger setting is used. If an IF power trigger is not possible, the IMMediate trigger setting is used (see manual of option).
        /// 
        /// For another, the correction value for the time offset of the IF power trigger from the beginning of the measured slot is defined (see manual of option). This correction value is needed in order to establish an exact time reference between the trigger event and the beginning of the slot if there is no midamble triggering. The correction value for the DUT in question can be determined in the GENERAL SETTINGS menu with the TRIGGER OFFSET softkey.
        /// 
        /// If the instrument is set to external trigger on switchover to GSM/EDGE mode, the 'Extern' GSM trigger is used, otherwise the 'IF Power' GSM trigger (default) is set.
        /// 
        /// (4) RF Power GSM Trigger
        /// This command is a combination of 2 commands:
        /// 
        /// For one, the ?RF Power? GSM trigger is selected. For all GSM measurements for which an RF power trigger is possible, the RFPower trigger setting is used. If an RF power trigger is not possible, the IMMediate trigger setting is used (see manual of option)
        /// 
        /// For another, the correction value for the time offset of the RF power trigger from the beginning of the first active slot is defined (see manual of option).
        /// 
        /// This correction value is needed in order to establish an exact time reference between the trigger event and the beginning of the slot if there is no midamble triggering. The correction value for the DUT in question can be determined in the GENERAL SETTINGS menu with the TRIGGER OFFSET softkey.
        /// 
        /// It is available only if the option FSP-B6 (TV- and RF-Trigger) is available.
        /// </param>
        /// <param name="Trigger_Value">
        /// This control sets selected adjust value (See also Trigger Adjust help).
        /// 
        /// Valid Values: -460.0e-6 to 8.0 s
        ///  
        /// Default Value: 0.0 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_SYNC_ADJ_EXT
        /// RSSPECAN_ATTR_GSM_SYNC_ADJ_IFP
        /// RSSPECAN_ATTR_GSM_SYNC_ADJ_RFP
        /// 
        /// Note(s):
        /// This value has no effect for Free Run trigger.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGsmTrigger(int Trigger_Source, double Trigger_Value)
        {
            int pInvokeResult = PInvoke.ConfigureGsmTrigger(this._handle, Trigger_Source, Trigger_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the the major parameters of the demodulator in the GSM/EDGE application.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_MTYP
        /// RSSPECAN_ATTR_GSM_SLOT_MULT
        /// RSSPECAN_ATTR_GSM_PRAT
        /// RSSPECAN_ATTR_GSM_SSE
        /// RSSPECAN_ATTR_GSM_BSE
        /// RSSPECAN_ATTR_GSM_BSTH
        /// RSSPECAN_ATTR_GSM_CHAN_TSC
        /// RSSPECAN_ATTR_GSM_TSC_USER
        /// 
        /// Remote-control command(s):
        /// CONFigure:MTYPe GMSK | EDGE
        /// CONFigure:CHANnel:SLOT:MULTi ACT1SYNC1 | ACT2SYNC1 | ACT2SYNC2 | ACT3SYNC1 | ACT3SYNC2 | ACT3SYNC3 |ACT4SYNC1 | ACT4SYNC2 | ACT4SYNC3 | ACT4SYNC4 | ACT8SYNC1 | ACT8SYNC2 |ACT8SYNC3 | ACT8SYNC4 | ACT8SYNC5 | ACT83SYNC6 | ACT8SYNC7 | ACT8SYNC8
        /// CONFigure:PRATe 4 | 8
        /// CONFigure:SSEarch ON | OFF
        /// CONFigure:BSEarch ON | OFF
        /// CONFigure:BSTHreshold
        /// CONFigure:CHANnel:TSC 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | USER | AB0 | AB1 | AB2
        /// CONFigure:CHANnel:TSC:USER
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Modulation_Type">
        /// Sets the modulation type GMSK or EDGE.
        /// A running GSM measurement is aborted when the modulation type is changed. 
        /// 
        /// Valid Values:
        /// 
        /// - RSSPECAN_VAL_GSM_MTYP_GMSK - selects modulation type GMSK
        /// 
        /// - RSSPECAN_VAL_GSM_MTYP_EDGE - selects modulation type EDGE
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_MTYP_GMSK
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_MTYP
        /// 
        /// Note:
        /// (1) For Midamble set to TS 0 to TS 2 (RSSPECAN_VAL_GSM_TSC_AB0 to RSSPECAN_VAL_GSM_TSC_AB2) has this control no effect, modulation is set to GMSK allways.
        /// </param>
        /// <param name="Multi_Slot">
        /// Selects the used slots of the mobile or base station. The multislot setting defines how many adjacent slots are active and which of the active slots should be used for synchronization.
        /// 
        /// Valid Values:
        /// 
        /// - RSSPECAN_VAL_GSM_SLOT_11 - Active 1 Sync 1
        /// - RSSPECAN_VAL_GSM_SLOT_21 - Active 2 Sync 1
        /// - RSSPECAN_VAL_GSM_SLOT_22 - Active 2 Sync 2
        /// - RSSPECAN_VAL_GSM_SLOT_31 - Active 3 Sync 1
        /// - RSSPECAN_VAL_GSM_SLOT_32 - Active 3 Sync 2
        /// - RSSPECAN_VAL_GSM_SLOT_33 - Active 3 Sync 3
        /// - RSSPECAN_VAL_GSM_SLOT_41 - Active 4 Sync 1
        /// - RSSPECAN_VAL_GSM_SLOT_42 - Active 4 Sync 2
        /// - RSSPECAN_VAL_GSM_SLOT_43 - Active 4 Sync 3
        /// - RSSPECAN_VAL_GSM_SLOT_44 - Active 4 Sync 4
        /// - RSSPECAN_VAL_GSM_SLOT_81 - Active 8 Sync 1
        /// - RSSPECAN_VAL_GSM_SLOT_82 - Active 8 Sync 2
        /// - RSSPECAN_VAL_GSM_SLOT_83 - Active 8 Sync 3
        /// - RSSPECAN_VAL_GSM_SLOT_84 - Active 8 Sync 4
        /// - RSSPECAN_VAL_GSM_SLOT_85 - Active 8 Sync 5
        /// - RSSPECAN_VAL_GSM_SLOT_86 - Active 8 Sync 6
        /// - RSSPECAN_VAL_GSM_SLOT_87 - Active 8 Sync 7
        /// - RSSPECAN_VAL_GSM_SLOT_88 - Active 8 Sync 8
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_SLOT_11
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_SLOT_MULT
        /// 
        /// Note(s):
        /// (1) For Midamble set to TS 0 to TS 2 (RSSPECAN_VAL_GSM_TSC_AB0 to RSSPECAN_VAL_GSM_TSC_AB2) has this control no effect.
        /// 
        /// (2)
        /// ACT1SYNC1 - 1 active slot synchronization to 1st active slot
        /// ACT2SYNC1 - 2 active slots synchronization to 1st active slot
        /// ACT2SYNC2 - 2 active slots synchronization to 2nd active slot
        /// ACT3SYNC1 - 3 active slots synchronization to 1st active slot
        /// ACT3SYNC2 - 3 active slots synchronization to 2nd active slot
        /// ACT3SYNC3 - 3 active slots synchronization to 3rd active slot
        /// ACT4SYNC1 - 4 active slots synchronization to 1st active slot
        /// ACT4SYNC2 - 4 active slots synchronization to 2nd active slot
        /// ACT4SYNC3 - 4 active slots synchronization to 3rd active slot
        /// ACT4SYNC4 - 4 active slots synchronization to 4th active slot
        /// ACT8SYNC1 - 8 active slots synchronization to 1st active slot
        /// ACT8SYNC2 - 8 active slots synchronization to 2nd active slot
        /// ACT8SYNC3 - 8 active slots synchronization to 3rd active slot
        /// ACT8SYNC4 - 8 active slots synchronization to 4th active slot
        /// ACT8SYNC5 - 8 active slots synchronization to 5th active slot
        /// ACT8SYNC6 - 8 active slots synchronization to 6th active slot
        /// ACT8SYNC7 - 8 active slots synchronization to 7th active slot
        /// ACT8SYNC8 - 8 active slots synchronization to 8th active slot
        /// 
        /// 
        /// </param>
        /// <param name="Samples_Per_Symbols">
        /// Sets the number of measurement points per symbol. (rate of points per symbols).
        /// 
        /// Valid Values: 4,8
        /// 
        /// Default Value: 4
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_PRAT
        /// </param>
        /// <param name="Sync_Search">
        /// Sets the IQ sync search to ON or OFF.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_SSE
        /// </param>
        /// <param name="Burst_Search">
        /// Sets the IQ burst search to ON or OFF. The driver uses this value to set the RSSPECAN_ATTR_GSM_BSE attribute.
        /// 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_BSE
        /// 
        /// Note(s):
        /// (1) For Midamble set to TS 0 to TS 2 (RSSPECAN_VAL_GSM_TSC_AB0 to RSSPECAN_VAL_GSM_TSC_AB2) has this control no effect, it is set to On allways.
        /// 
        /// </param>
        /// <param name="Burst_Search_Threshold">
        /// Sets the burst search threshold in dB.
        /// 
        /// Valid Values: -100.0 dB to 0.0 dB
        /// 
        /// Default Value: -35.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_BSTH
        /// </param>
        /// <param name="Midamble">
        /// Geographical region for the measurements.
        /// 
        /// Valid Values:
        /// 
        /// RSSPECAN_VAL_GSM_TSC_0   (0)  - TSC 0
        /// RSSPECAN_VAL_GSM_TSC_1   (1)  - TSC 1
        /// RSSPECAN_VAL_GSM_TSC_2   (2)  - TSC 2
        /// RSSPECAN_VAL_GSM_TSC_3   (3)  - TSC 3
        /// RSSPECAN_VAL_GSM_TSC_4   (4)  - TSC 4
        /// RSSPECAN_VAL_GSM_TSC_5   (5)  - TSC 5
        /// RSSPECAN_VAL_GSM_TSC_6   (6)  - TSC 6
        /// RSSPECAN_VAL_GSM_TSC_7   (7)  - TSC 7
        /// RSSPECAN_VAL_GSM_USER    (8)  - User
        /// RSSPECAN_VAL_GSM_TSC_AB0 (9)  - TS 0
        /// RSSPECAN_VAL_GSM_TSC_AB1 (10) - TS 1
        /// RSSPECAN_VAL_GSM_TSC_AB2 (11) - TS 2
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_TSC_0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_CHAN_TSC
        /// 
        /// Note:
        /// (1) For TS 0 to TS 2 (RSSPECAN_VAL_GSM_TSC_AB0 to RSSPECAN_VAL_GSM_TSC_AB2) is next setting done:
        /// Burst Search is set On
        /// Mutli Slot has no effect (not send to instrument)
        /// Modulation Type is set to GMSK
        /// </param>
        /// <param name="User_Sequence">
        /// Defines the bit pattern of the user midamble (training sequence).
        /// 
        /// Valid Values: 
        /// For modulation type GMSK:
        ///    26 characters of 0 and 1
        /// 
        /// For modulation type 8PSK (EDGE):
        ///    78 characters of 0 and 1
        /// 
        /// Default Value:
        ///     "00000000000000000000000000"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_TSC_USER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGsmDemodulation(int Modulation_Type, int Multi_Slot, int Samples_Per_Symbols, bool Sync_Search, bool Burst_Search, double Burst_Search_Threshold, int Midamble, string User_Sequence)
        {
            int pInvokeResult = PInvoke.ConfigureGsmDemodulation(this._handle, Modulation_Type, Multi_Slot, Samples_Per_Symbols, System.Convert.ToUInt16(Sync_Search), System.Convert.ToUInt16(Burst_Search), Burst_Search_Threshold, Midamble, User_Sequence);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Sets the multi carrier mode to ON or OFF.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_MCAR
        /// 
        /// Remote-control command(s):
        /// CONFigure:MCARrier ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Multi_Carrier_Mode">
        /// Sets the multi carrier mode to ON or OFF.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On 
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_MCAR
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGsmMultiCarrierMode(bool Multi_Carrier_Mode)
        {
            int pInvokeResult = PInvoke.ConfigureGsmMultiCarrierMode(this._handle, System.Convert.ToUInt16(Multi_Carrier_Mode));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the type of limit check of the measurement of the spectrum due to switching transients.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_SWIT_LIM
        /// 
        /// Remote-control command(s):
        /// CONFigure:SPECtrum:SWITching:LIMit ABS| REL
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Limit_Type">
        /// Selects the type of limit check.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ABS (0) - Absolute
        /// RSSPECAN_VAL_REL (1) - Relative
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_REL
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_SWIT_LIM
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGsmTransSpecLimitType(int Limit_Type)
        {
            int pInvokeResult = PInvoke.ConfigureGsmTransSpecLimitType(this._handle, Limit_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the force the IF-power or with FSP-B6 RF-power trigger. If the trigger source is set to IF- or RF-power and an IQ measurement like PFE/MAC or PVT is used the free run trigger is selected, because a synchronization can be done with sync and burst search. Now also for these measurements the IF or RF power trigger will be selected. Default state is OFF.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_SYNC_IQPOWER
        /// 
        /// Remote-control command(s):
        /// TRIGger[:SEQuence]:SYNChronize:IQPower OFF | ON
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="State">
        /// This control set the IF/RF Trigger as IQ Power state.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_SYNC_IQPOWER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGsmIFRFTriggerAsIQPower(bool State)
        {
            int pInvokeResult = PInvoke.ConfigureGsmIFRFTriggerAsIQPower(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects linear and logarithmic averaging in the modulation spectrum list measurement. In LIN mode voltages
        /// are averaged. In LOG mode levels.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_MODULATION_SPECTRUM_LIST_AVER_TYPE
        /// 
        /// Remote-control command(s):
        /// CONFigure:SPECtrum:MODulation:LIST:AVERage:TYPE LIN | LOG
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="List_Average">
        /// Selects the type of averaging. The driver uses this value to set the RSSPECAN_ATTR_GSM_MODULATION_SPECTRUM_LIST_AVER_TYPE attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIN (0) - Linear
        /// RSSPECAN_VAL_LOG (1) - Logarithmic
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_LOG
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_MODULATION_SPECTRUM_LIST_AVER_TYPE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGsmModulationSpectrumListAverage(int List_Average)
        {
            int pInvokeResult = PInvoke.ConfigureGsmModulationSpectrumListAverage(this._handle, List_Average);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function restores the GSM/EDGE limit lines.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_REST
        /// 
        /// Remote-control command(s):
        /// CONFigure:RESTore
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int RestoreGsmLimitLines()
        {
            int pInvokeResult = PInvoke.RestoreGsmLimitLines(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates the extended slot configuration.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_STATE
        /// 
        /// Remote-control command(s):
        /// CONFigure:ECONfigure[:STATe] ON | OFF
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="State">
        /// This control sets the extended slot configuration state.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value:
        /// VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGsmExtendedSlotState(bool State)
        {
            int pInvokeResult = PInvoke.ConfigureGsmExtendedSlotState(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the GSM Extended Slot for Multi 
        /// Slot measurements. These values are standard specific.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_STANDARD
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_TRIG_REFERENCE
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_MIDAMBLE_REFERENCE
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_ABS_LIMIT_STATE
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_ABS_LIMIT_VALUE
        /// 
        /// Remote-control command(s):
        /// CONFigure[:MS]:ECONfigure:STANdard:SELect
        /// CONFigure:ECONfigure:TREFerence
        /// CONFigure:ECONfigure:MREFerence
        /// CONFigure[:MS]:ECONfigure:SLOT&lt;0 | [1] ? 7&gt;:LIMit:ABSolute[:STATe]
        /// CONFigure[:MS]:ECONfigure:SLOT&lt;0 | [1] ? 7&gt;:LIMit:ABSolute:BASe&lt;[1] | 2&gt;
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Standard">
        /// This control sets the behaviour of the extended slot configuration mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_EXTSLOT_DYNAMIC (0) - Dynamic
        /// RSSPECAN_VAL_GSM_EXTSLOT_GSM900  (1) - MS - GSM 900
        /// RSSPECAN_VAL_GSM_EXTSLOT_GSM1800 (2) - MS - GSM 1800
        /// RSSPECAN_VAL_GSM_EXTSLOT_GSM1900 (3) - MS - GSM 1900
        /// RSSPECAN_VAL_GSM_EXTSLOT_GSM850  (4) - MS - GSM 850
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_GSM_EXTSLOT_DYNAMIC
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_STANDARD
        /// 
        /// Notes:
        /// (1)  RSSPECAN_VAL_GSM_EXTSLOT_GSM850 is not supported for model R&amp;S FSG with base system version 4.29.
        /// </param>
        /// <param name="Trigger_Reference">
        /// This control sets the slot used as trigger reference in the extended slot configuration mode.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value:
        /// 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_TRIG_REFERENCE
        /// </param>
        /// <param name="Reference_Midamble">
        /// This control sets the midamble of the reference slot in the extended slot configuration mode.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value:
        /// 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_MIDAMBLE_REFERENCE
        /// </param>
        /// <param name="Absolute_Level_State">
        /// This control specifies whether absolute limit base values should be taken into account or not upon calculation of the upper limit line(s).
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value:
        /// VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_OFRAME
        /// 
        /// Note:
        /// 
        /// (1) VI_FALSE -  the absolute values are ignored.
        /// 
        /// (2) This control has no effect if Standard is set to Dynamic
        /// </param>
        /// <param name="Limit_Base_1">
        /// This control sets specifies the value for the line area situated 28 us and more apart from either side of the useful part of the slot.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value:
        /// 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_ABS_LIMIT_VALUE
        /// 
        /// Note:
        /// 
        /// (1) This control has no effect if Standard is set to Dynamic or Absolute Level State is set Off.
        /// </param>
        /// <param name="Limit_Base_2">
        /// This control sets the value for the region located from
        /// 18 Ys to 28 Ys outside the useful part of the slot at either side.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value:
        /// 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_ABS_LIMIT_VALUE
        /// 
        /// Note:
        /// 
        /// (1) This control has no effect if Standard is set to Dynamic or Absolute Level State is set Off.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGsmExtendedSlot(int Standard, int Trigger_Reference, int Reference_Midamble, bool Absolute_Level_State, double Limit_Base_1, double Limit_Base_2)
        {
            int pInvokeResult = PInvoke.ConfigureGsmExtendedSlot(this._handle, Standard, Trigger_Reference, Reference_Midamble, System.Convert.ToUInt16(Absolute_Level_State), Limit_Base_1, Limit_Base_2);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the GSM Extended Slot for Multi Slot measurements. These configurations are common for all standards.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_LONG_SLOT
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_OFRAME
        /// 
        /// Remote-control command(s):
        /// CONFigure:ECONfigure:LSLot[:STATe] ON | OFF
        /// CONFigure:ECONfigure:LSLot:VALue &lt; slot_number &gt;, &lt;slot_number&gt;
        /// CONFigure:ECONfigure:OFRame ON | OFF
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Long_Slots">
        /// This control sets  whether a GSM frame contains long slots or not.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value:
        /// VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_LONG_SLOT
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the number of values in Long Slots Values array.
        /// 
        /// Valid Values: 
        /// 2
        /// 
        /// Default Value: 2
        /// 
        /// Note:
        /// 
        /// (1) Only two values are supported now.
        /// </param>
        /// <param name="Long_Slots_Values">
        /// This control defines the array of long slots of GSM frame.
        /// 
        /// Note:
        /// 
        /// (1) Only two slot numbers are supported now.
        /// 
        /// </param>
        /// <param name="Only_One_Frame">
        /// This control the limit line handling before begin of slot 0 and after end of slot 7 in the extended slot configuration mode. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value:
        /// VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_OFRAME
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGsmExtendedSlotCommonSettings(bool Long_Slots, int Array_Size, int[] Long_Slots_Values, bool Only_One_Frame)
        {
            int pInvokeResult = PInvoke.ConfigureGsmExtendedSlotCommonSettings(this._handle, System.Convert.ToUInt16(Long_Slots), Array_Size, Long_Slots_Values, System.Convert.ToUInt16(Only_One_Frame));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the slot specific parameters for GSM Extended Slot for Multi Slot measurements.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_MODULATION
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_RLEV_MODE
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_RLEV_VALUE
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_LIMIT_CLEV
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_LIMIT
        ///  
        /// Remote-control command(s):
        /// CONFigure:ECONfigure:SLOT&lt;0 | [1] ? 7&gt;:MODulation GMSK | EDGE | OFF
        /// CONFigure:ECONfigure:SLOT&lt;0 | [1] ? 7&gt;:RLEVel:MODe ABS | REL
        /// CONFigure:ECONfigure:SLOT&lt;0 | [1] ? 7&gt;:RLEVel:VALue
        /// CONFigure[:MS]:ECONfigure:SLOT&lt;0 | [1] ? 7&gt;:LIMit:CLEVel
        /// CONFigure:ECONfigure:SLOT&lt;0 | [1] ? 7&gt;:LIMit:&lt;LOWer | UPPer&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Slot_Number">
        /// This control sets the slot number to be configured.
        /// 
        /// Valid Values: 
        /// 0 to 7
        /// 
        /// Default Value: 0
        /// </param>
        /// <param name="Modulation">
        /// This control sets the modulation for the selected slot or defines the slot as inactive (OFF).
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_EXT_SLOT_MODUL_OFF  (0) - Off
        /// RSSPECAN_VAL_EXT_SLOT_MODUL_GMSK (1) - GMSK
        /// RSSPECAN_VAL_EXT_SLOT_MODUL_EDGE (2) - EDGE
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_EXT_SLOT_MODUL_OFF
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_MODULATION
        /// </param>
        /// <param name="Reference_Level_Mode">
        /// This control sets the Reference Level Mode for the selected slot in the extended slot configuration mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ABS (0) - Absolute
        /// RSSPECAN_VAL_REL (1) - Relative
        /// 
        /// Default Value: RSSPECAN_VAL_REL
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_RLEV_MODE
        /// </param>
        /// <param name="Reference_Level_Value">
        /// This control sets the midamble of the reference slot in the extended slot configuration mode..
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value:
        /// 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_RLEV_VALUE
        /// </param>
        /// <param name="Limit_Line_Ctrl_Level">
        /// This control sets which power control level should be taken into account upon calculation of the upper limit line(s).
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value:
        /// 2
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_LIMIT_CLEV
        /// </param>
        /// <param name="Limit_Line_Lower">
        /// This control selects the lower limit line 'string_value' for the selected slot in the extended slot configuration mode.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value:
        /// ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_LIMIT
        /// </param>
        /// <param name="Limit_Line_Upper">
        /// This control selects the upper limit line 'string_value' for the selected slot in the extended slot configuration mode.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value:
        /// ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_EXTENDED_SLOT_LIMIT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGsmExtendedSlotParameters(int Slot_Number, int Modulation, int Reference_Level_Mode, double Reference_Level_Value, int Limit_Line_Ctrl_Level, string Limit_Line_Lower, string Limit_Line_Upper)
        {
            int pInvokeResult = PInvoke.ConfigureGsmExtendedSlotParameters(this._handle, Slot_Number, Modulation, Reference_Level_Mode, Reference_Level_Value, Limit_Line_Ctrl_Level, Limit_Line_Lower, Limit_Line_Upper);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the GSM Measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_BURS_PFER
        /// RSSPECAN_ATTR_GSM_BURS_MACC
        /// RSSPECAN_ATTR_GSM_BURS_POW
        /// RSSPECAN_ATTR_GSM_BURS_PTEM
        /// RSSPECAN_ATTR_GSM_SPEC_MOD
        /// RSSPECAN_ATTR_GSM_SPEC_SWIT
        /// RSSPECAN_ATTR_GSM_SPUR
        /// 
        /// Remote-control command(s):
        /// CONFigure:BURSt:PFERror[:IMMediate]
        /// CONFigure:BURSt:MACCuracy[:IMMediate]
        /// CONFigure:BURSt:POWer[:IMMediate]
        /// CONFigure:BURSt:PTEMplate[:IMMediate]
        /// CONFigure:SPECtrum:MODulation[:IMMediate]
        /// CONFigure:SPECtrum:SWITching[:IMMediate]
        /// CONFigure:SPURious[:IMMediate]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement_Mode">
        /// Selects the measurement mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_MEAS_PFER     (0) - Phase and Frequency Error
        /// RSSPECAN_VAL_GSM_MEAS_MACC     (1) - Modulation Accuracy
        /// RSSPECAN_VAL_GSM_MEAS_CPW      (2) - Carrier Power
        /// RSSPECAN_VAL_GSM_MEAS_PVT      (3) - Power vs. Time
        /// RSSPECAN_VAL_GSM_MEAS_SPEC_MOD (4) - Modulation Spectrum
        /// RSSPECAN_VAL_GSM_MEAS_SPEC_TRA (5) - Transient Spectrum
        /// RSSPECAN_VAL_GSM_MEAS_SPUR     (6) - Spurious
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_GSM_BURS_PFER
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_BURS_PFER
        /// RSSPECAN_ATTR_GSM_BURS_MACC
        /// RSSPECAN_ATTR_GSM_BURS_POW
        /// RSSPECAN_ATTR_GSM_BURS_PTEM
        /// RSSPECAN_ATTR_GSM_SPEC_MOD
        /// RSSPECAN_ATTR_GSM_SPEC_SWIT
        /// RSSPECAN_ATTR_GSM_SPUR
        /// 
        /// Notes:
        /// (1) Phase and Frequency Error:
        /// Selects measurement of the phase and frequency error of the mobile or base station.
        /// 
        /// (2) Modulation Accuracy:
        /// Selects measurement of the modulation accuracy of the mobile or base station
        /// 
        /// (3) Carrier Power:
        /// Selects measurement of the carrier power of the mobile or base station.
        /// 
        /// (4) Power vs. Time:
        /// Selects measurement of power of the mobile or base station vs. time.
        /// 
        /// (5) Modulation Spectrum:
        /// Selects measurement of the spectrum due to modulation.
        /// 
        /// (6) Transient Spectrum:
        /// Selects measurement of the spectrum due to switching transients.
        /// 
        /// (7) Spurious:
        /// Selects measurement of spurious emissions.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGsmMeasurement(int Measurement_Mode)
        {
            int pInvokeResult = PInvoke.ConfigureGsmMeasurement(this._handle, Measurement_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the the major parameters of the demodulator in the GSM/EDGE application.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_BURS_PTEM_SEL
        /// RSSPECAN_ATTR_GSM_BURS_PTEM_TMHR
        /// RSSPECAN_ATTR_GSM_BURS_PTEM_FILT
        /// RSSPECAN_ATTR_GSM_BURS_PTEM_FRZ
        /// 
        /// Remote-control command(s):
        /// CONFigure:BURSt:PTEMplate:SELect FULL | TOP | RISing | FALLing
        /// CONFigure:BURSt:PTEMplate:TMHRes ON | OFF
        /// CONFigure:BURSt:PTEMplate:FILTer G500 | B600
        /// CONFigure:BURSt:PTEMplate:FRZoom
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Burst_Section">
        /// This control defines the burst section to be measured.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_BURST_SEC_FULL (0) - Full Burst (Default Value)
        /// RSSPECAN_VAL_GSM_BURST_SEC_TOP  (1) - Burst High Resolution
        /// RSSPECAN_VAL_GSM_BURST_SEC_RIS  (2) - Rising
        /// RSSPECAN_VAL_GSM_BURST_SEC_FALL (3) - Falling
        /// RSSPECAN_VAL_GSM_BURST_SEC_FRZ  (4) - Fall/Rise Zoom
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_BURS_PTEM_SEL
        /// 
        /// Note(s):
        /// 
        /// Fall/Rise Zoom is available if more than one slot is active.
        /// 
        /// </param>
        /// <param name="High_Resolution_Mode">
        /// Sets the Time Meas High Resolution mode to ON or OFF.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_BURS_PTEM_TMHR
        /// 
        /// Note:
        /// (1) This control has effect only if Burst Section is set to Full Burst.
        /// </param>
        /// <param name="Filter_Bandwidth">
        /// Sets changes between the default low pass filter with 600k bandwidth and the gaussian 500 kHz bandwidth filter. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_FILT_G500 - 500 kHz
        /// RSSPECAN_VAL_GSM_FILT_B600 - 600 kHz
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_FILT_G500
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_BURS_PTEM_FILT
        /// 
        /// Note(s):
        /// 
        /// (1) G500 - sets the gaussian 500 kHz bandwidth filter
        ///     B600 - sets the default low pass filter with 600k bandwidth
        /// 
        /// (2) This parameter has effect only if High Resolution Mode is set Off
        /// 
        /// </param>
        /// <param name="Transition_No">
        /// Defines the transition number where the zoom shall be carried out can be entered.
        /// 
        /// Valid Values:  1 to (active slots - 1)
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_BURS_PTEM_FRZ
        /// 
        /// Note:
        /// (1) This control has effect only if Burst Section is set to Fall/Rise Zoom
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGsmPVTMeasurement(int Burst_Section, bool High_Resolution_Mode, int Filter_Bandwidth, int Transition_No)
        {
            int pInvokeResult = PInvoke.ConfigureGsmPVTMeasurement(this._handle, Burst_Section, System.Convert.ToUInt16(High_Resolution_Mode), Filter_Bandwidth, Transition_No);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts a measurement sequence which automatically adjust the level and trigger timing of the input signal to the analyzer.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// READ:AUTO:LEVTime?
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Status">
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </param>
        /// <param name="Signal_Power__dBm_">
        /// This control returns the corrected level signal power (reference level is automatically set 3 dB above the signal power)
        /// </param>
        /// <param name="Trigger_Time__s_">
        /// This control returns the correction value for the time offset between the currently active GSM trigger and beginning of the slot.
        /// </param>
        /// <param name="Trigger_Level__dBm_V_">
        /// This control returns trigger level of the current active GSM trigger. For IF/RF the units are dBm, for the external trigger the unit is Volts.
        /// 
        /// </param>
        /// <param name="Reserved">
        /// For future use, always 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadGsmLevelTime(uint Timeout, out bool Status, out double Signal_Power__dBm_, out double Trigger_Time__s_, out double Trigger_Level__dBm_V_, out double Reserved)
        {
            ushort StatusAsUShort;
            int pInvokeResult = PInvoke.ReadGsmLevelTime(this._handle, Timeout, out StatusAsUShort, out Signal_Power__dBm_, out Trigger_Time__s_, out Trigger_Level__dBm_V_, out Reserved);
            Status = System.Convert.ToBoolean(StatusAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts the Phase Frequency Error measurement and returns the result.
        /// 
        /// Note(s):
        /// (1) This function is available only if Phase and Frequency Error is selected with attribute RSSPECAN_ATTR_GSM_BURS_PFER.
        /// 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_READ_BURST_PERR_RMS_AVER
        /// RSSPECAN_ATTR_GSM_READ_BURST_PERR_RMS_MAX
        /// RSSPECAN_ATTR_GSM_READ_BURST_PERR_PEAK_AVER
        /// RSSPECAN_ATTR_GSM_READ_BURST_PERR_PEAK_MAX
        /// RSSPECAN_ATTR_GSM_READ_BURST_FERR_AVER
        /// RSSPECAN_ATTR_GSM_READ_BURST_FERR_MAX
        /// RSSPECAN_ATTR_GSM_READ_BURST_IQOF_AVER
        /// RSSPECAN_ATTR_GSM_READ_BURST_IQOF_MAX
        /// RSSPECAN_ATTR_GSM_READ_BURST_IQIM_AVER
        /// RSSPECAN_ATTR_GSM_READ_BURST_IQIM_MAX
        /// 
        /// Remote-control command(s):
        /// READ:BURSt:PERRor:RMS:AVERage?
        /// READ:BURSt:PERRor:RMS:MAXimum?
        /// READ:BURSt:PERRor:PEAK:AVERage?
        /// READ:BURSt:PERRor:PEAK:MAXimum?
        /// READ:BURSt:FERRor:AVERage?
        /// READ:BURSt:FERRor:MAXimum?
        /// READ:BURSt:IQIMbalance:MAXimum?
        /// READ:BURSt:IQIMbalance:AVERage?
        /// READ:BURSt:IQOFfset:MAXimum?
        /// READ:BURSt:IQOFfset:AVERage?
        /// *OPC
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Measurement">
        /// Sets the type of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_PERR_RMS  (0) - RMS of Phase Error
        /// RSSPECAN_VAL_GSM_PERR_PEAK (1) - Peak of Phase Error
        /// RSSPECAN_VAL_GSM_FERR      (2) - Frequency Error
        /// RSSPECAN_VAL_GSM_IQOF      (3) - IQ Offset
        /// RSSPECAN_VAL_GSM_IQIM      (4) - IQ Imbalance
        /// 
        /// </param>
        /// <param name="Modifier">
        /// Sets the modifier of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// </param>
        /// <param name="Value">
        /// Returns the value. 
        /// 
        /// Note(s):
        /// 
        /// (1) RMS of Phase Error - returned in deg.
        /// (2) Peak of Phase Error - returned in deg.
        /// (3) Frequency Error - returned in Hz
        /// (4) IQ Offset - returned in %
        /// (5) IQ Imbalance - returned in %
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadGsmPhaseFreqErrorValues(uint Timeout, int Measurement, int Modifier, out double Value)
        {
            int pInvokeResult = PInvoke.ReadGsmPhaseFreqErrorValues(this._handle, Timeout, Measurement, Modifier, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the Phase Frequency Error measurement result.
        /// 
        /// Note(s):
        /// (1) This function is available only if Phase and Frequency Error is selected with attribute RSSPECAN_ATTR_GSM_BURS_PFER.
        /// 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_FETC_BURST_PERR_RMS_AVER
        /// RSSPECAN_ATTR_GSM_FETC_BURST_PERR_RMS_MAX
        /// RSSPECAN_ATTR_GSM_FETC_BURST_PERR_PEAK_AVER
        /// RSSPECAN_ATTR_GSM_FETC_BURST_PERR_PEAK_MAX
        /// RSSPECAN_ATTR_GSM_FETC_BURST_FERR_AVER
        /// RSSPECAN_ATTR_GSM_FETC_BURST_FERR_MAX
        /// RSSPECAN_ATTR_GSM_FETC_BURST_IQOF_AVER
        /// RSSPECAN_ATTR_GSM_FETC_BURST_IQOF_MAX
        /// RSSPECAN_ATTR_GSM_FETC_BURST_IQIM_AVER
        /// RSSPECAN_ATTR_GSM_FETC_BURST_IQIM_MAX
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:PERRor:RMS:AVERage?
        /// FETCh:BURSt:PERRor:RMS:MAXimum?
        /// FETCh:BURSt:PERRor:PEAK:AVERage?
        /// FETCh:BURSt:PERRor:PEAK:MAXimum?
        /// FETCh:BURSt:FERRor:AVERage?
        /// FETCh:BURSt:FERRor:MAXimum?
        /// FETCh:BURSt:IQIMbalance:MAXimum?
        /// FETCh:BURSt:IQIMbalance:AVERage?
        /// FETCh:BURSt:IQOFfset:MAXimum?
        /// FETCh:BURSt:IQOFfset:AVERage?
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement">
        /// Sets the type of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_PERR_RMS  (0) - RMS of Phase Error
        /// RSSPECAN_VAL_GSM_PERR_PEAK (1) - Peak of Phase Error
        /// RSSPECAN_VAL_GSM_FERR      (2) - Frequency Error
        /// RSSPECAN_VAL_GSM_IQOF      (3) - IQ Offset
        /// RSSPECAN_VAL_GSM_IQIM      (4) - IQ Imbalance
        /// 
        /// </param>
        /// <param name="Modifier">
        /// Sets the modifier of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// </param>
        /// <param name="Value">
        /// Returns the value. 
        /// 
        /// Note(s):
        /// 
        /// (1) RMS of Phase Error - returned in deg.
        /// (2) Peak of Phase Error - returned in deg.
        /// (3) Frequency Error - returned in Hz
        /// (4) IQ Offset - returned in %
        /// (5) IQ Imbalance - returned in %
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchGsmPhaseFreqErrorValues(int Measurement, int Modifier, out double Value)
        {
            int pInvokeResult = PInvoke.FetchGsmPhaseFreqErrorValues(this._handle, Measurement, Modifier, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts the Modulation Accuracy measurement and returns the result.
        /// 
        /// Note(s):
        /// (1) This function is available only if Modulation Accuracy is selected with attribute RSSPECAN_ATTR_GSM_BURS_MACC.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_READ_BURST_MACC_RMS_AVER
        /// RSSPECAN_ATTR_GSM_READ_BURST_MACC_RMS_MAX
        /// RSSPECAN_ATTR_GSM_READ_BURST_MACC_PEAK_AVER
        /// RSSPECAN_ATTR_GSM_READ_BURST_MACC_PEAK_MAX
        /// RSSPECAN_ATTR_GSM_READ_BURST_MACC_OSUP_AVER
        /// RSSPECAN_ATTR_GSM_READ_BURST_MACC_OSUP_MAX
        /// RSSPECAN_ATTR_GSM_READ_BURST_MACC_PERC_AVER
        /// RSSPECAN_ATTR_GSM_READ_BURST_MACC_PERC_MAX
        /// RSSPECAN_ATTR_GSM_READ_BURST_MACC_FREQ_AVER
        /// RSSPECAN_ATTR_GSM_READ_BURST_MACC_FREQ_MAX
        /// RSSPECAN_ATTR_GSM_READ_BURST_MACC_IQOF_AVER
        /// RSSPECAN_ATTR_GSM_READ_BURST_MACC_IQOF_MAX
        /// RSSPECAN_ATTR_GSM_READ_BURST_MACC_IQIM_AVER
        /// RSSPECAN_ATTR_GSM_READ_BURST_MACC_IQIM_MAX
        /// 
        /// Remote-control command(s):
        /// READ:BURSt:MACCuracy:RMS:AVERage?
        /// READ:BURSt:MACCuracy:RMS:MAXimum?
        /// READ:BURSt:MACCuracy:PEAK:AVERage?
        /// READ:BURSt:MACCuracy:PEAK:MAXimum?
        /// READ:BURSt:MACCuracy:OSUPpress:AVERage?
        /// READ:BURSt:MACCuracy:OSUPpress:MAXimum?
        /// READ:BURSt:MACCuracy:PERCentile:AVERage?
        /// READ:BURSt:MACCuracy:PERCentile:MAXimum?
        /// READ:BURSt:MACCuracy:FREQuency:AVERage?
        /// READ:BURSt:MACCuracy:FREQuency:MAXimum?
        /// READ:BURSt:MACCuracy:IQOFfset:AVERage?
        /// READ:BURSt:MACCuracy:IQOFfset:MAXimum?
        /// READ:BURSt:MACCuracy:IQIMbalance:AVERage?
        /// READ:BURSt:MACCuracy:IQIMbalance:MAXimum?
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Measurement">
        /// Sets the type of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_MACC_RMS  (0) - RMS of Error Vector Magnitude
        /// RSSPECAN_VAL_GSM_MACC_PEAK (1) - Peak of Error Vector Magnitude
        /// RSSPECAN_VAL_GSM_MACC_OSUP (2) - Original Offset Suppression
        /// RSSPECAN_VAL_GSM_MACC_PERC (3) - 95% Percentile
        /// RSSPECAN_VAL_GSM_MACC_FREQ (4) - Frequency Error
        /// RSSPECAN_VAL_GSM_MACC_IQOF (5) - IQ Offset
        /// RSSPECAN_VAL_GSM_MACC_IQIM (6) - IQ Imbalance
        /// 
        /// </param>
        /// <param name="Modifier">
        /// Sets the modifier of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// </param>
        /// <param name="Value">
        /// Returns the value. 
        /// 
        /// Note(s):
        /// 
        /// (1) RMS of Phase Error - returned in deg.
        /// (2) Peak of Phase Error - returned in deg.
        /// (3) Frequency Error - returned in Hz
        /// (4) IQ Offset - returned in %
        /// (5) IQ Imbalance - returned in %
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadGsmModulAccValues(uint Timeout, int Measurement, int Modifier, out double Value)
        {
            int pInvokeResult = PInvoke.ReadGsmModulAccValues(this._handle, Timeout, Measurement, Modifier, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the results of the Modulation Accuracy measurement.
        /// 
        /// Note(s):
        /// (1) This function is available only if Modulation Accuracy is selected with attribute RSSPECAN_ATTR_GSM_BURS_MACC.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_FETC_BURST_MACC_RMS_AVER
        /// RSSPECAN_ATTR_GSM_FETC_BURST_MACC_RMS_MAX
        /// RSSPECAN_ATTR_GSM_FETC_BURST_MACC_PEAK_AVER
        /// RSSPECAN_ATTR_GSM_FETC_BURST_MACC_PEAK_MAX
        /// RSSPECAN_ATTR_GSM_FETC_BURST_MACC_OSUP_AVER
        /// RSSPECAN_ATTR_GSM_FETC_BURST_MACC_OSUP_MAX
        /// RSSPECAN_ATTR_GSM_FETC_BURST_MACC_PERC_AVER
        /// RSSPECAN_ATTR_GSM_FETC_BURST_MACC_PERC_MAX
        /// RSSPECAN_ATTR_GSM_FETC_BURST_MACC_FREQ_AVER
        /// RSSPECAN_ATTR_GSM_FETC_BURST_MACC_FREQ_MAX
        /// RSSPECAN_ATTR_GSM_FETC_BURST_MACC_IQOF_AVER
        /// RSSPECAN_ATTR_GSM_FETC_BURST_MACC_IQOF_MAX
        /// RSSPECAN_ATTR_GSM_FETC_BURST_MACC_IQIM_AVER
        /// RSSPECAN_ATTR_GSM_FETC_BURST_MACC_IQIM_MAX
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:MACCuracy:RMS:AVERage?
        /// FETCh:BURSt:MACCuracy:RMS:MAXimum?
        /// FETCh:BURSt:MACCuracy:PEAK:AVERage?
        /// FETCh:BURSt:MACCuracy:PEAK:MAXimum?
        /// FETCh:BURSt:MACCuracy:OSUPpress:AVERage?
        /// FETCh:BURSt:MACCuracy:OSUPpress:MAXimum?
        /// FETCh:BURSt:MACCuracy:PERCentile:AVERage?
        /// FETCh:BURSt:MACCuracy:PERCentile:MAXimum?
        /// FETCh:BURSt:MACCuracy:FREQuency:AVERage?
        /// FETCh:BURSt:MACCuracy:FREQuency:MAXimum?
        /// FETCh:BURSt:MACCuracy:IQOFfset:AVERage?
        /// FETCh:BURSt:MACCuracy:IQOFfset:MAXimum?
        /// FETCh:BURSt:MACCuracy:IQIMbalance:AVERage?
        /// FETCh:BURSt:MACCuracy:IQIMbalance:MAXimum?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement">
        /// Sets the type of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_MACC_RMS  (0) - RMS of Error Vector Magnitude
        /// RSSPECAN_VAL_GSM_MACC_PEAK (1) - Peak of Error Vector Magnitude
        /// RSSPECAN_VAL_GSM_MACC_OSUP (2) - Original Offset Suppression
        /// RSSPECAN_VAL_GSM_MACC_PERC (3) - 95% Percentile
        /// RSSPECAN_VAL_GSM_MACC_FREQ (4) - Frequency Error
        /// RSSPECAN_VAL_GSM_MACC_IQOF (5) - IQ Offset
        /// RSSPECAN_VAL_GSM_MACC_IQIM (6) - IQ Imbalance
        /// 
        /// </param>
        /// <param name="Modifier">
        /// Sets the modifier of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// </param>
        /// <param name="Value">
        /// Returns the value. 
        /// 
        /// Note(s):
        /// 
        /// (1) RMS of Phase Error - returned in deg.
        /// (2) Peak of Phase Error - returned in deg.
        /// (3) Frequency Error - returned in Hz
        /// (4) IQ Offset - returned in %
        /// (5) IQ Imbalance - returned in %
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchGsmModulAccValues(int Measurement, int Modifier, out double Value)
        {
            int pInvokeResult = PInvoke.FetchGsmModulAccValues(this._handle, Measurement, Modifier, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts the premeasurement of power vs. time and read out the results.
        /// 
        /// Note(s):
        /// (1) This function is available only if Power Vs. Time is selected with attribute RSSPECAN_ATTR_GSM_BURS_PTEM.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// READ:BURSt:PTEMplate:REFerence[:IMMediate]?
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Returned_Values">
        /// This control returns the value in array.
        /// 
        /// Returned Values:
        /// Idx 0 - measured level
        /// Idx 1 - level corrected by means of the bandwidth
        /// Idx 2 - bandwidth
        /// 
        /// Note(s):
        /// (1) The array size has to be at least 3.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadGsmPtempRef(uint Timeout, double[] Returned_Values)
        {
            int pInvokeResult = PInvoke.ReadGsmPtempRef(this._handle, Timeout, Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts the Power Vs. Time measurement and returns the result.
        /// 
        /// Note(s):
        /// (1) This function is available only if Power Vs. Time is selected with attribute RSSPECAN_ATTR_GSM_BURS_PTEM.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_READ_BURST_PTEM_TRGS_AVER
        /// RSSPECAN_ATTR_GSM_READ_BURST_PTEM_TRGS_MAX
        /// 
        /// Remote-control command(s):
        /// READ:BURSt:PTEMplate:TRGS:AVERage?
        /// READ:BURSt:PTEMplate:TRGS:MAXimum?
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Measurement">
        /// Sets the type of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_PVT_TRGS (0) - Trigger To Start Symbol
        /// 
        /// Note(s):
        /// 
        /// (1) Trigger To Start Symbol -  returns the absolute time between external trigger and begin of the first symbol of the trainings sequence (midamble) in s over the selected number of bursts.
        /// 
        /// </param>
        /// <param name="Modifier">
        /// Sets the modifier of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// </param>
        /// <param name="Value">
        /// Returns the value. 
        /// 
        /// Note(s):
        /// 
        /// (1) RMS of Phase Error - returned in deg.
        /// (2) Peak of Phase Error - returned in deg.
        /// (3) Frequency Error - returned in Hz
        /// (4) IQ Offset - returned in %
        /// (5) IQ Imbalance - returned in %
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadGsmPowVsTimeValues(uint Timeout, int Measurement, int Modifier, out double Value)
        {
            int pInvokeResult = PInvoke.ReadGsmPowVsTimeValues(this._handle, Timeout, Measurement, Modifier, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts the pre-measurement of power vs time and reads out the result in the extended slot configuration mode.
        /// 
        /// Note(s):
        /// (1) This function is available only if Power Vs. Time is selected with attribute RSSPECAN_ATTR_GSM_BURS_PTEM.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// READ:BURSt:PTEMplate:REFerence:ECONfigure[:IMMediate]?
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the number of elements allocated in arrays. 
        /// 
        /// Valid Values: &gt;0
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Slot_Number">
        /// Returns indexed array of slot number.
        /// 
        /// </param>
        /// <param name="Measured_Level">
        /// Returns array of measured level.
        /// 
        /// </param>
        /// <param name="Corrected_Level">
        /// Returns array of level corrected by means of the bandwidth.
        /// 
        /// </param>
        /// <param name="Resolution_Bandwidth">
        /// Returns array of resolution bandwidth.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadGsmExtendedSlotPtempRef(uint Timeout, int Array_Size, int[] Slot_Number, double[] Measured_Level, double[] Corrected_Level, double[] Resolution_Bandwidth)
        {
            int pInvokeResult = PInvoke.ReadGsmExtendedSlotPtempRef(this._handle, Timeout, Array_Size, Slot_Number, Measured_Level, Corrected_Level, Resolution_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the results of the premeasurement of power vs. time.
        /// 
        /// Note(s):
        /// (1) This function is available only if Power Vs. Time is selected with attribute RSSPECAN_ATTR_GSM_BURS_PTEM.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:PTEMplate:REFerence?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Returned_Values">
        /// This control returns the value in array.
        /// 
        /// Returned Values:
        /// Idx 0 - measured level
        /// Idx 1 - level corrected by means of the bandwidth
        /// Idx 2 - bandwidth
        /// 
        /// Note(s):
        /// (1) The array size has to be at least 3.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchGsmPtempRef(double[] Returned_Values)
        {
            int pInvokeResult = PInvoke.FetchGsmPtempRef(this._handle, Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the Power Vs. Time measurement results.
        /// 
        /// Note(s):
        /// (1) This function is available only if Power Vs. Time is selected with attribute RSSPECAN_ATTR_GSM_BURS_PTEM.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_FETC_BURST_PTEM_TRGS_AVER
        /// RSSPECAN_ATTR_GSM_FETC_BURST_PTEM_TRGS_MAX
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:PTEMplate:TRGS:AVERage?
        /// FETCh:BURSt:PTEMplate:TRGS:MAXimum?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement">
        /// Sets the type of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_PVT_TRGS (0) - Trigger To Start Symbol
        /// 
        /// Note(s):
        /// 
        /// (1) Trigger To Start Symbol -  returns the absolute time between external trigger and begin of the first symbol of the trainings sequence (midamble) in s over the selected number of bursts.
        /// 
        /// </param>
        /// <param name="Modifier">
        /// Sets the modifier of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// </param>
        /// <param name="Value">
        /// Returns the value. 
        /// 
        /// Note(s):
        /// 
        /// (1) RMS of Phase Error - returned in deg.
        /// (2) Peak of Phase Error - returned in deg.
        /// (3) Frequency Error - returned in Hz
        /// (4) IQ Offset - returned in %
        /// (5) IQ Imbalance - returned in %
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchGsmPowVsTimeValues(int Measurement, int Modifier, out double Value)
        {
            int pInvokeResult = PInvoke.FetchGsmPowVsTimeValues(this._handle, Measurement, Modifier, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the result in the extended slot configuration mode.
        /// 
        /// Note(s):
        /// (1) This function is available only if Power Vs. Time is selected with attribute RSSPECAN_ATTR_GSM_BURS_PTEM.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:PTEMplate:REFerence:ECONfigure[:IMMediate]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the number of elements allocated in arrays. 
        /// 
        /// Valid Values: &gt;0
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Slot_Number">
        /// Returns indexed array of slot number.
        /// 
        /// </param>
        /// <param name="Measured_Level">
        /// Returns array of measured level.
        /// 
        /// </param>
        /// <param name="Corrected_Level">
        /// Returns array of level corrected by means of the bandwidth.
        /// 
        /// </param>
        /// <param name="Resolution_Bandwidth">
        /// Returns array of resolution bandwidth.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchGsmExtendedSlotPtempRef(int Array_Size, int[] Slot_Number, double[] Measured_Level, double[] Corrected_Level, double[] Resolution_Bandwidth)
        {
            int pInvokeResult = PInvoke.FetchGsmExtendedSlotPtempRef(this._handle, Array_Size, Slot_Number, Measured_Level, Corrected_Level, Resolution_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts the premeasurement of the modulation due to spectrum measurement and reads out the result.
        /// 
        /// Note(s):
        /// (1) This function is available only if modulation due to spectrum is selected with attribute RSSPECAN_ATTR_GSM_SPEC_MOD.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// READ:SPECtrum:MODulation:REFerence[:IMMediate]?
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Returned_Values">
        /// This control returns the value in array.
        /// 
        /// Returned Values:
        /// Idx 0 - measured level
        /// Idx 1 - level corrected by means of the bandwidth
        /// Idx 2 - bandwidth
        /// 
        /// Note(s):
        /// (1) The array size has to be at least 3.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadGsmSpecModRef(uint Timeout, double[] Returned_Values)
        {
            int pInvokeResult = PInvoke.ReadGsmSpecModRef(this._handle, Timeout, Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts the measurement of the modulation spectrum of the base station or mobile and reads out the result.
        /// 
        /// Note(s):
        /// (1) This function is available only if modulation due to spectrum is selected with attribute RSSPECAN_ATTR_GSM_SPEC_MOD.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// READ:SPECtrum:MODulation[:ALL]?
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the number of elements allocated in arrays. 
        /// 
        /// Valid Values: &gt;0
        /// 
        /// Default Value: 100
        /// 
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Index">
        /// Returns indexed array of measurement range (last number indicates sum of measurements).
        /// 
        /// </param>
        /// <param name="Start_Frequencies">
        /// Returns start frequencies of the measurement range or frequencies where the limit is exceeded.
        /// 
        /// </param>
        /// <param name="Stop_Frequencies">
        /// Returns stop frequencies of the measurement range or frequencies where the measured range is exceeded. This values are equal to the values of start frequencies, if either the measurement is performed in the time domain or the result value contains a limit excess.
        /// 
        /// </param>
        /// <param name="Levels">
        /// Returns array of measured maximum levels of the partial range or measured levels at the test point.
        /// 
        /// </param>
        /// <param name="Limits">
        /// Returns limits in the partial range or measured levels at the test point.
        /// 
        /// </param>
        /// <param name="Abs_Rel">
        /// Returns array of ABS/REL unit indicators. Meaning of numbers is following:
        /// 
        ///  ABS = 0
        ///  REL = 1
        /// 
        /// </param>
        /// <param name="Status">
        /// Limit checks array of results.
        /// 
        /// PASSED ... no limits exceeded
        /// FAILED ... limits exceeded
        /// MARGIN ... margin exceeded
        /// EXC ... limited excesses characterized as an exception
        /// 
        /// Meaning of numbers is:
        /// 
        /// PASSED = 0
        /// FAILED = 1
        /// MARGIN = 2
        /// EXC = 3 
        /// </param>
        /// <param name="Num_of_Results">
        /// This control returns number of results, received from the instrument.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadGsmModSpectrum(int Array_Size, uint Timeout, int[] Index, double[] Start_Frequencies, double[] Stop_Frequencies, double[] Levels, double[] Limits, int[] Abs_Rel, int[] Status, out int Num_of_Results)
        {
            int pInvokeResult = PInvoke.ReadGsmModSpectrum(this._handle, Array_Size, Timeout, Index, Start_Frequencies, Stop_Frequencies, Levels, Limits, Abs_Rel, Status, out Num_of_Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the results of the premeasurement of modulation due to spectrum measurement.
        /// 
        /// Note(s):
        /// (1) This function is available only if modulation due to spectrum is selected with attribute RSSPECAN_ATTR_GSM_SPEC_MOD.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FETCh:SPECtrum:MODulation:REFerence?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Returned_Values">
        /// This control returns the value in array.
        /// 
        /// Returned Values:
        /// Idx 0 - measured level
        /// Idx 1 - level corrected by means of the bandwidth
        /// Idx 2 - bandwidth
        /// 
        /// Note(s):
        /// (1) The array size has to be at least 3.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchGsmSpecModRef(double[] Returned_Values)
        {
            int pInvokeResult = PInvoke.FetchGsmSpecModRef(this._handle, Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts the measurement of the modulation spectrum of the base station or mobile and reads out the result.
        /// 
        /// Note(s):
        /// (1) This function is available only if modulation due to spectrum is selected with attribute RSSPECAN_ATTR_GSM_SPEC_MOD.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FETCh:SPECtrum:MODulation[:ALL]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the number of elements allocated in arrays. 
        /// 
        /// Valid Values: &gt;0
        /// 
        /// Default Value: 100
        /// </param>
        /// <param name="Index">
        /// Returns indexed array of measurement range (last number indicates sum of measurements).
        /// 
        /// </param>
        /// <param name="Start_Frequencies">
        /// Returns start frequencies of the measurement range or frequencies where the limit is exceeded.
        /// 
        /// </param>
        /// <param name="Stop_Frequencies">
        /// Returns stop frequencies of the measurement range or frequencies where the measured range is exceeded. This values are equal to the values of start frequencies, if either the measurement is performed in the time domain or the result value contains a limit excess.
        /// 
        /// </param>
        /// <param name="Levels">
        /// Returns array of measured maximum levels of the partial range or measured levels at the test point.
        /// 
        /// </param>
        /// <param name="Limits">
        /// Returns limits in the partial range or measured levels at the test point.
        /// 
        /// </param>
        /// <param name="Abs_Rel">
        /// Returns array of ABS/REL unit indicators. Meaning of numbers is following:
        /// 
        ///  ABS = 0
        ///  REL = 1
        /// 
        /// </param>
        /// <param name="Status">
        /// Limit checks array of results.
        /// 
        /// PASSED ... no limits exceeded
        /// FAILED ... limits exceeded
        /// MARGIN ... margin exceeded
        /// EXC ... limited excesses characterized as an exception
        /// 
        /// Meaning of numbers is:
        /// 
        /// PASSED = 0
        /// FAILED = 1
        /// MARGIN = 2
        /// EXC = 3 
        /// </param>
        /// <param name="Num_of_Results">
        /// This control returns number of results, received from the instrument.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchGsmModSpectrum(int Array_Size, int[] Index, double[] Start_Frequencies, double[] Stop_Frequencies, double[] Levels, double[] Limits, int[] Abs_Rel, int[] Status, out int Num_of_Results)
        {
            int pInvokeResult = PInvoke.FetchGsmModSpectrum(this._handle, Array_Size, Index, Start_Frequencies, Stop_Frequencies, Levels, Limits, Abs_Rel, Status, out Num_of_Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts the premeasurement of the modulation due to switching transients (TRA) measurement and reads out the result.
        /// 
        /// Note(s):
        /// (1) This function is available only if spectrum due to switching transients is selected with attribute RSSPECAN_ATTR_GSM_SPEC_SWIT.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// READ:SPECtrum:SWITching:REFerence[:IMMediate]?
        /// *OPC
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Returned_Values">
        /// This control returns the value in array.
        /// 
        /// Returned Values:
        /// Idx 0 - measured level
        /// Idx 1 - level corrected by means of the bandwidth
        /// Idx 2 - bandwidth
        /// 
        /// Note(s):
        /// (1) The array size has to be at least 3.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadGsmSpecTransRef(uint Timeout, double[] Returned_Values)
        {
            int pInvokeResult = PInvoke.ReadGsmSpecTransRef(this._handle, Timeout, Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts the measurement of the transient spectrum of the mobile or base station and reads out the result.
        /// 
        /// Note(s):
        /// (1) This function is available only if spectrum due to switching transients is selected with attribute RSSPECAN_ATTR_GSM_SPEC_SWIT.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// READ:SPECtrum:SWITching[:ALL]?
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the number of elements allocated in arrays. 
        /// 
        /// Valid Values: &gt;0
        /// 
        /// Default Value: 100
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Index">
        /// Returns indexed array of measurement range (last number indicates sum of measurements).
        /// 
        /// </param>
        /// <param name="Start_Frequencies">
        /// Returns start frequencies of the measurement range or frequencies where the limit is exceeded.
        /// 
        /// </param>
        /// <param name="Stop_Frequencies">
        /// Returns stop frequencies of the measurement range or frequencies where the measured range is exceeded. This values are equal to the values of start frequencies, if either the measurement is performed in the time domain or the result value contains a limit excess.
        /// 
        /// </param>
        /// <param name="Levels">
        /// Returns array of measured maximum levels of the partial range or measured levels at the test point.
        /// 
        /// </param>
        /// <param name="Limits">
        /// Returns limits in the partial range or measured levels at the test point.
        /// 
        /// </param>
        /// <param name="Abs_Rel">
        /// Returns array of ABS/REL unit indicators. Meaning of numbers is following:
        /// 
        ///  ABS = 0
        ///  REL = 1
        /// 
        /// </param>
        /// <param name="Status">
        /// Limit checks array of results.
        /// 
        /// PASSED ... no limits exceeded
        /// FAILED ... limits exceeded
        /// MARGIN ... margin exceeded
        /// EXC ... limited excesses characterized as an exception
        /// 
        /// Meaning of numbers is:
        /// 
        /// PASSED = 0
        /// FAILED = 1
        /// MARGIN = 2
        /// EXC = 3 
        /// </param>
        /// <param name="Num_of_Results">
        /// This control returns number of results, received from the instrument.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadGsmTransSpectrum(int Array_Size, uint Timeout, int[] Index, double[] Start_Frequencies, double[] Stop_Frequencies, double[] Levels, double[] Limits, int[] Abs_Rel, int[] Status, out int Num_of_Results)
        {
            int pInvokeResult = PInvoke.ReadGsmTransSpectrum(this._handle, Array_Size, Timeout, Index, Start_Frequencies, Stop_Frequencies, Levels, Limits, Abs_Rel, Status, out Num_of_Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the result of the measurement of the transient spectrum of the mobile or base station.
        /// 
        /// Note(s):
        /// (1) This function is available only if spectrum due to switching transients is selected with attribute RSSPECAN_ATTR_GSM_SPEC_SWIT.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FETCh:SPECtrum:SWITching[:ALL]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the number of elements allocated in arrays. 
        /// 
        /// Valid Values: &gt;0
        /// 
        /// Default Value: 100
        /// </param>
        /// <param name="Index">
        /// Returns indexed array of measurement range (last number indicates sum of measurements).
        /// 
        /// </param>
        /// <param name="Start_Frequencies">
        /// Returns start frequencies of the measurement range or frequencies where the limit is exceeded.
        /// 
        /// </param>
        /// <param name="Stop_Frequencies">
        /// Returns stop frequencies of the measurement range or frequencies where the measured range is exceeded. This values are equal to the values of start frequencies, if either the measurement is performed in the time domain or the result value contains a limit excess.
        /// 
        /// </param>
        /// <param name="Levels">
        /// Returns array of measured maximum levels of the partial range or measured levels at the test point.
        /// 
        /// </param>
        /// <param name="Limits">
        /// Returns limits in the partial range or measured levels at the test point.
        /// 
        /// </param>
        /// <param name="Abs_Rel">
        /// Returns array of ABS/REL unit indicators. Meaning of numbers is following:
        /// 
        ///  ABS = 0
        ///  REL = 1
        /// 
        /// </param>
        /// <param name="Status">
        /// Limit checks array of results.
        /// 
        /// PASSED ... no limits exceeded
        /// FAILED ... limits exceeded
        /// MARGIN ... margin exceeded
        /// EXC ... limited excesses characterized as an exception
        /// 
        /// Meaning of numbers is:
        /// 
        /// PASSED = 0
        /// FAILED = 1
        /// MARGIN = 2
        /// EXC = 3 
        /// </param>
        /// <param name="Num_of_Results">
        /// This control returns number of results, received from the instrument.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchGsmTransSpectrum(int Array_Size, int[] Index, double[] Start_Frequencies, double[] Stop_Frequencies, double[] Levels, double[] Limits, int[] Abs_Rel, int[] Status, out int Num_of_Results)
        {
            int pInvokeResult = PInvoke.FetchGsmTransSpectrum(this._handle, Array_Size, Index, Start_Frequencies, Stop_Frequencies, Levels, Limits, Abs_Rel, Status, out Num_of_Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the FM Demodulator mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument[:SELect] ADEM
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ADemodMode()
        {
            int pInvokeResult = PInvoke.ADemodMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates the FM demodulator of the instrument. The instrument will be set to timedomain measurement (span = 0) at the current center frequency. The detector will be set to
        /// SAMPle, the demodulator itself will be set up according to the parameters of command [SENSe:]ADEMod:SET.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_STATE
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="State">
        /// This control sets the analog demodulation state.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SetADemodState(bool State)
        {
            int pInvokeResult = PInvoke.SetADemodState(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects FM demodulator measurement.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FEED
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Operating_Mode">
        /// This control selects the type of evaluation for the measured data.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FM_FEED_AM_REL      (0) - Demodulated AM signal 
        /// RSSPECAN_VAL_FM_FEED_AM          (1) - Demodulated AM signal in level display
        /// RSSPECAN_VAL_FM_FEED_SPEC        (2) - RF spectrum
        /// RSSPECAN_VAL_FM_FEED_FM          (3) - Demodulated FM signal
        /// RSSPECAN_VAL_FM_FEED_PM          (4) - Demodulated PM signal
        /// RSSPECAN_VAL_FM_FEED_AMS_REL     (5) - AM-Summary, referenced to trace
        /// RSSPECAN_VAL_FM_FEED_AMS         (6) - AM-Summary in level display, referenced to trace
        /// RSSPECAN_VAL_FM_FEED_FMS         (7) - FM-Summary, referenced to trace
        /// RSSPECAN_VAL_FM_FEED_PMS         (8) - PM-Summary, referenced to trace
        /// RSSPECAN_VAL_FM_FEED_FM_AFSP     (9) - AF spectrum of FM signal, referenced to trace
        /// RSSPECAN_VAL_FM_FEED_PM_AFSP     (10) - AF spectrum of PM signal, referenced to trace
        /// RSSPECAN_VAL_FM_FEED_AM_REL_AFSP (11) - AF spectrum of AM Rel signal, referenced to trace
        /// RSSPECAN_VAL_FM_FEED_RFP_AFSP    (12) - AF spectrum of AM signal, referenced to trace
        /// RSSPECAN_VAL_FSM_FEED_SUMM       (13) - Summary Table
        /// 
        /// Default Value: RSSPECAN_VAL_FM_FEED_AM_REL (0)
        /// Notes:
        /// 
        /// (1) Demodulated AM signal in standardized display
        /// 
        /// (2) Demodulated AM signal in level display
        /// 
        /// (3) RF spectrum of FM signal determined from the measured data by means of FFT.
        /// 
        /// (4) Demodulated FM signal
        /// 
        /// (5) Demodulated PM signal
        /// 
        /// (6) AM results in standardized display, referenced to traces 1 to 4
        /// 
        /// (7) AM results in level display, referenced to traces 1 to 4
        /// 
        /// (8) FM results, referenced to traces 1 to 4
        /// 
        /// (9) PM results, referenced to traces 1 to 4
        /// 
        /// (10) AF spectrum of the demodulated FM signal, results referenced to traces 1 to 4
        /// 
        /// (11) AF spectrum of the demodulated PM signal, results referenced to traces 1 to 4
        /// 
        /// (12) AF spectrum of the demodulated AM signal in standardized display, results referenced to traces 1 to 4
        /// 
        /// (13) AF spectrum of the demodulated AM signal, results referenced to traces 1 to 3. This selection is not valid for R&amp;S FSL or R&amp;S ESL.
        /// </param>
        /// <param name="Trace">
        /// This control sets the reference trace for modes.
        /// 
        /// Valid Valuess:
        /// 1 to 3
        /// 1 to 6 for R&amp;S FSL or R&amp;S ESL or R&amp;S FSV
        /// 
        /// Default Value: 1
        /// 
        /// Note:
        /// 
        /// (1) This control has effect only for Operating Mode 6 to 13.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SelectADemodResult(int Operating_Mode, int Trace)
        {
            int pInvokeResult = PInvoke.SelectADemodResult(this._handle, Operating_Mode, Trace);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the FM Demodulation Trigger
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_TRIGGER_SOURCE
        /// RSSPECAN_ATTR_TRIGGER_DELAY
        /// RSSPECAN_ATTR_TRIGGER_SLOPE
        /// RSSPECAN_ATTR_TRIGGER_IFP_LEVEL
        /// RSSPECAN_ATTR_ADEM_TRIGGER_AM_LEVEL_ABS
        /// RSSPECAN_ATTR_ADEM_TRIGGER_AM_LEVEL_REL
        /// RSSPECAN_ATTR_ADEM_TRIGGER_FM_LEVEL
        /// RSSPECAN_ATTR_ADEM_TRIGGER_PM_LEVEL
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:SOURce IMMediate | EXTernal | IFPower | AF | FM | AM | PM
        /// TRIGger&lt;1|2&gt;[:SEQuence]:HOLDoff
        /// TRIGger&lt;1|2&gt;[:SEQuence]:SLOPe POSitive | NEGative
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:IFPower
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:AM[:ABSolute]
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:AM:RELative
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:FM
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:PM
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Trigger_Source">
        /// Specifies the source of the trigger signal that causes the analyzer to leave the Wait-For-Trigger state. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TRG_IMM (0) - Immediate 
        /// RSSPECAN_VAL_TRG_EXT (1) - External   
        /// RSSPECAN_VAL_TRG_IFP (2) - IF Power
        /// RSSPECAN_VAL_TRG_FM  (3) - FM
        /// RSSPECAN_VAL_TRG_AM  (4) - RF Power Signal
        /// RSSPECAN_VAL_TRG_PM  (5) - PM
        /// RSSPECAN_VAL_TRG_AMR (6) - AM
        /// 
        /// Default value: RSSPECAN_VAL_TRG_IMM (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_TRIGGER_SOURCE
        /// </param>
        /// <param name="Trigger_Offset">
        /// Sets the length of the trigger delay. 
        /// 
        /// Units: Volts
        /// 
        /// Valid Values: -100.0 to 100.0
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_DELAY
        /// 
        /// Note:
        /// (1) A negative delay time (pretrigger) can be set in the time domain (span &lt; 0 Hz) only.
        /// 
        /// (2) This control has effect for Immediate, External and IF Power Trigger only.
        /// </param>
        /// <param name="Trigger_Polarity">
        /// Specifies the polarity for triggering.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_NEG (0) - Negative
        /// RSSPECAN_VAL_POS (1) - Positive
        /// 
        /// Default value: RSSPECAN_VAL_POS (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_SLOPE
        /// </param>
        /// <param name="Trigger_Value">
        /// This command sets the level of the video trigger source.
        /// 
        /// Valid Values: 
        ///  for IF Power
        ///    -50 to 30 dBm
        ///  for FM
        ///    -10.0e+6 to 10.0e+6 Hz
        ///  for AM Relative:
        ///    -100 to 100 %
        ///  for PM
        ///    -1000 to +1000 rad|deg
        ///  for AM
        ///    -100 to +30 dBm
        ///  
        /// Default Value: 50.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_IFP_LEVEL
        /// RSSPECAN_ATTR_ADEM_TRIGGER_AM_LEVEL_ABS
        /// RSSPECAN_ATTR_ADEM_TRIGGER_AM_LEVEL_REL
        /// RSSPECAN_ATTR_ADEM_TRIGGER_FM_LEVEL
        /// RSSPECAN_ATTR_ADEM_TRIGGER_PM_LEVEL
        /// 
        /// Note:
        /// 
        /// (1) This control has no effect for next triggers:
        /// Free Run
        /// External
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureADemodTrigger(int Trigger_Source, double Trigger_Offset, int Trigger_Polarity, double Trigger_Value)
        {
            int pInvokeResult = PInvoke.ConfigureADemodTrigger(this._handle, Trigger_Source, Trigger_Offset, Trigger_Polarity, Trigger_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the FM demodulator.
        /// 
        /// Note:
        /// (1) This command is only available with option FS-K7 (FM Demodulator).
        /// 
        /// Attribute:
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:SET
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Sample_Rate">
        /// Selects the sampling rate for the data acquisition.
        /// 
        /// Valid Valuess:
        /// 122.0703125  to 32.0e+6 Hz
        /// 
        /// Default Value:
        /// 8.0e+6 Hz
        /// 
        /// 
        /// Note:
        /// 
        /// (1) For R&amp;S FSL are valid values: 32.0e+6, 16.0e+6, 8.0e+6, 4.0e+6, 2.0e+6, 1.0e+6,500.0e+3, 250.0e+3, 125.0e+3, 62.5e+, 31.25e+3, 15.625e+3, 7.8125e+3, 3.90625e+3, 1.953125e+3, 976,5625, 488.28125, 244.140625, 122.0703125 Hz for R&amp;S FSL
        /// 
        /// (2) For other instruments are valid values: 15.625e+3, 31.25e+3,62.5e+3,125.0e+3, 250.0e+3, 500.0e+3, 1.0e+6,4.0e+6, 8.0e+6, 16.0e+6, 32.0e+6
        /// 
        /// </param>
        /// <param name="Record_Length">
        /// Number of samples to be stored in I/Q memory.
        /// 
        /// Valid Valuess:
        ///  1 to 130560
        /// 
        ///  for R&amp;S FSL (or R&amp;S ESL): 
        ///    1 to 400001 with AF filter or AF trigger active
        ///    1 to 480001 with both AF filter and AF trigger deactive 
        /// 
        /// Default Value:
        ///  1
        /// 
        /// 
        /// </param>
        /// <param name="Trigger_Source">
        /// Specifies the source of the trigger signal that causes the analyzer to leave the Wait-For-Trigger state. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TRG_IMM (0) - Immediate 
        /// RSSPECAN_VAL_TRG_EXT (1) - External   
        /// RSSPECAN_VAL_TRG_IFP (2) - IF Power
        /// RSSPECAN_VAL_TRG_FM  (3) - FM
        /// RSSPECAN_VAL_TRG_AM  (4) - RF Power Signal
        /// RSSPECAN_VAL_TRG_PM  (5) - PM
        /// RSSPECAN_VAL_TRG_AMR (6) - AM
        /// 
        /// Default value: RSSPECAN_VAL_TRG_IMM (0)
        /// </param>
        /// <param name="Trigger_Slope">
        /// Selecs the trigger slope.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_NEG (0) - Negative
        /// RSSPECAN_VAL_POS (1) - Positive
        /// 
        /// Default Value: RSSPECAN_VAL_FMDEM_POS (1)
        /// </param>
        /// <param name="Pretrigger_Samples">
        /// Number of samples to be used as an offset to the trigger signal.
        /// 
        /// Valid Valuess:
        ///  -65024  to 130560
        /// 
        /// Default Value:
        ///  0
        /// 
        /// Note:
        /// (1) This value will be ingnored for trigger = 'Immediate'.
        /// </param>
        /// <param name="Number_Of_Meas">
        /// Number of measurements. The value indicated here is particularly necessary for the AVERage/MAXHold/MINHold function.
        /// 
        /// Valid Valuess:
        ///  0 to 32767
        /// 
        /// Default Value:
        ///  0
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureADemodSet(double Sample_Rate, int Record_Length, int Trigger_Source, int Trigger_Slope, int Pretrigger_Samples, int Number_Of_Meas)
        {
            int pInvokeResult = PInvoke.ConfigureADemodSet(this._handle, Sample_Rate, Record_Length, Trigger_Source, Trigger_Slope, Pretrigger_Samples, Number_Of_Meas);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects demodulation bandwidth AF filters of FM demodulation.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only in conjunction with option FS-K7.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_FILT_HPAS_STAT
        /// RSSPECAN_ATTR_FMDEM_FILT_HPAS_FREQ
        /// RSSPECAN_ATTR_FMDEM_FILT_LPAS_STAT
        /// RSSPECAN_ATTR_FMDEM_FILT_LPAS_FREQ
        /// RSSPECAN_ATTR_FMDEM_FILT_DEMP_STAT
        /// RSSPECAN_ATTR_FMDEM_FILT_DEMP_TCON
        /// RSSPECAN_ATTR_ADEM_FILT_LPAS_FREQ_REL
        /// 
        /// Remote-control command(s):
        /// [SENSe:]FILTer:HPASs[:STATe] ON | OFF
        /// [SENSe:]FILTer:HPASs:FREQuency 20 Hz | 50 Hz | 300 Hz
        /// [SENSe:]FILTer:LPASs[:STATe] ON | OFF
        /// [SENSe:]FILTer:LPASs:FREQuency 3 kHz | 15 kHz | 150 kHz
        /// [SENSe:]FILTer:DEMPhasis[:STATe] ON | OFF
        /// [SENSe:]FILTer:DEMPhasis:TCONstant 25 us | 50 us | 75 us | 750 us
        /// [SENSe&lt;1|2&gt;:]FILTer:LPASs:FREQuency:RELative 5 | 10 | 25
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Filter_Type">
        /// This control selects the filter type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FMDEM_FILTER_HP     (0) - High Pass AF Filter
        /// RSSPECAN_VAL_FMDEM_FILTER_LP     (1) - Low Pass AF Filter
        /// RSSPECAN_VAL_FMDEM_FILTER_DEEMP  (2) - AM/FM Deemphasis
        /// RSSPECAN_VAL_FMDEM_FILTER_LP_REL (3) - Low Pass Filter Relative
        /// 
        /// Default Value: RSSPECAN_VAL_FMDEM_FILTER_HP (0)
        /// 
        /// Notes:
        /// 
        /// (1) High Pass AF Filter:
        /// Activates the high-pass filter in the AF-branch for analog demodulation.
        /// 
        /// (2) Low Pass AF Filter:
        /// Activates the low-pass filter in the AF-branch with analog demodulation.
        /// 
        /// (3) AM/FM Deemphasis:
        /// Activates the selected de-emphasis for analog demodulation.
        /// 
        /// (4) Low Pass Filter Relative:
        /// Activates the relative low-pass filter. 
        /// 
        /// 
        /// </param>
        /// <param name="Filter_State">
        /// This control activates the selected filter.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_FILT_HPAS_STAT
        /// RSSPECAN_ATTR_FMDEM_FILT_LPAS_STAT
        /// RSSPECAN_ATTR_FMDEM_FILT_DEMP_STAT
        /// </param>
        /// <param name="Filter_Frequency">
        /// This control sets the filter frequency.
        /// 
        /// Valid Values:
        ///   for High Pass filter:  20 Hz, 50 Hz,   300 Hz
        ///   for Low Pass filter: 3000 Hz, 15000 Hz, 23000 Hz, 150000 Hz
        ///   for relative Low Pass filter: 5%, 10%, 25%z
        /// 
        /// Default Value: 50.0 Hz
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_FILT_HPAS_FREQ
        /// RSSPECAN_ATTR_FMDEM_FILT_LPAS_FREQ
        /// RSSPECAN_ATTR_ADEM_FILT_LPAS_FREQ_REL
        /// 
        /// Note(s):
        /// 
        /// (1) High Pass:
        /// 
        ///     20 Hz for 100 Hz &lt;= demodulation bandwidth &lt;= 1.6 MHz 
        ///     50 Hz for 200 Hz &lt;= demodulation bandwidth &lt;=  4 MHz
        ///    300 Hz     800 Hz &lt;= demodulation bandwidth &lt;= 16 MHz
        /// 
        /// (2) Low Pass:
        /// 
        ///     3000 Hz for 6.4 kHz &lt;= demodulation bandwidth &lt;=  4 MHz
        ///    15000 Hz      50 kHz &lt;= demodulation bandwidth &lt;= 16 MHz
        ///   150000 Hz     400 kHz &lt;= demodulation bandwidth &lt;= 16 MHz
        /// 
        /// </param>
        /// <param name="AM_FM_Deemphasis_Value">
        /// This control sets the time constant of the de-emphasis for analog demodulation.
        /// 
        /// Valid Values:
        /// 25.0E-6 s, 50.0E-6 s, 75.0E-6 s, 750.0E-6 s
        /// 
        /// Default Value: 25.0E-6 s
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_FILT_DEMP_TCON
        /// 
        /// Note(s):
        /// 
        /// (1) The de-emphasis is active in the following demodulation bandwidth range:
        /// 
        ///      25 us for  25 kHz &lt;= demodulation bandwidth &lt;= 30 MHz
        ///      50 us     6.4 kHz &lt;= demodulation bandwidth &lt;= 18 MHz
        ///      75 us     6.4 kHz &lt;= demodulation bandwidth &lt;= 18 MHz
        ///     750 us     800  Hz &lt;= demodulation bandwidth &lt;=  4 MHz
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureADemodFilter(int Filter_Type, bool Filter_State, double Filter_Frequency, double AM_FM_Deemphasis_Value)
        {
            int pInvokeResult = PInvoke.ConfigureADemodFilter(this._handle, Filter_Type, System.Convert.ToUInt16(Filter_State), Filter_Frequency, AM_FM_Deemphasis_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects weighting filter.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only in conjunction with option FS-K7.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_ADEM_FILT_CCITT
        /// RSSPECAN_ATTR_ADEM_FILT_CCIR
        /// 
        /// Remote-control command(s):
        /// [SENSe:]FILTer:CCITt[:STATe] ON | OFF
        /// [SENSe:]FILTer:CCIR[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="CCITT">
        /// This control activates/deactivate CCITT filter weightening. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE 
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ADEM_FILT_CCITT
        /// </param>
        /// <param name="CCIR">
        /// This control activates/deactivate CCIR filter weightening. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE 
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ADEM_FILT_CCIR
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureADemodFilterWeighting(bool CCITT, bool CCIR)
        {
            int pInvokeResult = PInvoke.ConfigureADemodFilterWeighting(this._handle, System.Convert.ToUInt16(CCITT), System.Convert.ToUInt16(CCIR));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets analog demodulation parameters such as zoom state, start time.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only in conjunction with option FS-K7.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_ZOOM
        /// RSSPECAN_ATTR_FMDEM_ZOOM_START
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:ZOOM[:STATe] ON | OFF
        /// [SENSe:]ADEMod:ZOOM:STARt
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Zoom_State">
        /// Turns on or off the zoom function for the trace data of the FM demodulator.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_ZOOM
        /// </param>
        /// <param name="Start_Time">
        /// Selects the start time for the display of the single measured values of the FM demodulator. 
        /// 
        /// Valid Values:
        /// 0.0 to Max s
        /// 
        /// Default Value: 0.0 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_ZOOM_START
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureADemodZoom(bool Zoom_State, double Start_Time)
        {
            int pInvokeResult = PInvoke.ConfigureADemodZoom(this._handle, System.Convert.ToUInt16(Zoom_State), Start_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the zoom length settings.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only in conjunction with option FS-K7.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_ZOOM_LENGTH_MODE
        /// RSSPECAN_ATTR_FMDEM_ZOOM_LENGTH
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:ZOOM:LENGth:MODE AUTO | MANual
        /// [SENSe:]ADEMod:ZOOM:LENGth
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Zoom_Length_Mode">
        /// Defines whether the length of the zoom area for the analog-demodulated measurement data is defined automatically or manually.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_AUTO (0) - Automatic
        /// RSSPECAN_VAL_MAN  (1) - Manual
        /// 
        /// Default Value: RSSPECAN_VAL_AUTO (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_ZOOM_LENGTH_MODE
        /// </param>
        /// <param name="Zoom_Length">
        /// Defines the length of the zoom area for the analog-demodulated measurement data manually.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0.0000625 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_ZOOM_LENGTH
        /// 
        /// Note(s):
        /// 
        /// (1) This value is only available when Zoom Length Mode is set to Manual.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureADemodZoomLength(int Zoom_Length_Mode, double Zoom_Length)
        {
            int pInvokeResult = PInvoke.ConfigureADemodZoomLength(this._handle, Zoom_Length_Mode, Zoom_Length);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects between % and dB for displaying the
        /// THD measurement result.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_ADEM_UNIT_THD
        /// 
        /// Remote-control command(s):
        /// UNIT:THD PCT | DB
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Unit">
        /// This control selects between % and dB for displaying the
        /// THD measurement result.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DB  (8) - dB
        /// RSSPECAN_VAL_UNIT_PCT (13) - %
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_DB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ADEM_UNIT_THD
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureADemodTHDUnit(int Unit)
        {
            int pInvokeResult = PInvoke.ConfigureADemodTHDUnit(this._handle, Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates an automatic signal search.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_ADEM_SIGNAL_SEARCH
        /// 
        /// Remote-control command(s):
        /// [SENSe:]FREQuency:CW:AFC ONCE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Timeout__ms_">
        /// Sets the timeout for signal search. If the length of time required for signal search exceeds the timeout value, then the function will return with a timeout error.
        /// 
        /// Valid Values: 0 ms to 600000 ms
        /// 
        /// Default Value: 15000 ms
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ADemAutoTune(uint Timeout__ms_)
        {
            int pInvokeResult = PInvoke.ADemAutoTune(this._handle, Timeout__ms_);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures analog demodulator bandwidth.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_BAND_DEM
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:BANDwidth:DEModulation
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Demodulation_Bandwidth">
        /// This control sets the bandwidth for analog demodulation. Depending on the selected demodulation bandwidth, the instrument selects the required sampling rate. The available values of the demodulation bandwidths are determined by the sampling rates.
        /// 
        /// Valid Values:
        /// 100 Hz to 50.0E6 Hz
        /// 
        /// Default Value: 5.0e6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_BAND_DEM
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureADemodBandwidth(double Demodulation_Bandwidth)
        {
            int pInvokeResult = PInvoke.ConfigureADemodBandwidth(this._handle, Demodulation_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures analog demodulator AF Coupling parameters.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_AF_COUP
        /// 
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:AF:COUPling AC | DC
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="AF_Coupling">
        /// This control selects selects the coupling of the AF path.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DC (0) - DC
        /// RSSPECAN_VAL_AC (1) - AC
        /// 
        /// Default Value: RSSPECAN_VAL_DC (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_AF_COUP
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureADemodAFCoupling(int AF_Coupling)
        {
            int pInvokeResult = PInvoke.ConfigureADemodAFCoupling(this._handle, AF_Coupling);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures analog demodulator measuring time parameters.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_MTIM
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:MTIMe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measuring_Time">
        /// This control sets the measuring time for the analog Demodulation.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 62.625E-6 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_MTIM
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureADemodMeasuringTime(double Measuring_Time)
        {
            int pInvokeResult = PInvoke.ConfigureADemodMeasuringTime(this._handle, Measuring_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the frequency range of the result display AF spectrum using start frequency and stop frequency.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_AF_START
        /// RSSPECAN_ATTR_FMDEM_AF_STOP
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:AF:STARt
        /// [SENSe:]ADEMod:AF:STOP
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Start_Frequency">
        /// Sets the start frequency for result display AF spectrum.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 0.0 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_AF_START
        /// </param>
        /// <param name="Stop_Frequency">
        /// Sets the stop frequency for result display AF spectrum.
        /// The stop frequency is limited to half the measurement bandwidth of analog demodulation.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 2.5E6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_AF_STOP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureADemodAFFrequencyStartStop(double Start_Frequency, double Stop_Frequency)
        {
            int pInvokeResult = PInvoke.ConfigureADemodAFFrequencyStartStop(this._handle, Start_Frequency, Stop_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the frequency range of the result display AF spectrum using the center frequency and the frequency span. This function modifies the AF Frequency Start and AF Frequency Stop attributes as follows:
        /// 
        ///     AF Frequency Start = CenterFrequency - span/2
        ///     AF Frequency Stop = CenterFrequency + span/2
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_AF_CENTER
        /// RSSPECAN_ATTR_FMDEM_AF_SPAN
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:AF:CENTer
        /// [SENSe:]ADEMod:AF:SPAN
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Center_Frequency">
        /// Sets the center frequency for result display AF spectrum.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 1.25E6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_AF_CENTER
        /// </param>
        /// <param name="Span">
        /// Sets the span for result display AF spectrum.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 2.5E6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_AF_SPAN
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureADemodAFFrequencyCenterSpan(double Center_Frequency, double Span)
        {
            int pInvokeResult = PInvoke.ConfigureADemodAFFrequencyCenterSpan(this._handle, Center_Frequency, Span);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the maximum span for the display of the AF spectrum.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_AF_FULL_SPAN
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:AF:SPAN:FULL
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureADemodAFFullSpan()
        {
            int pInvokeResult = PInvoke.ConfigureADemodAFFullSpan(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures bandwidth resolutionfor display of the AF spectrum.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_BAND_RES
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:SPECtrum:BANDwidth:RESolution
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Resolution_Bandwidth">
        /// This control sets the resolution bandwidth for display of the RF spectrum from the data of analog demodulation.
        /// The required record length is calculated from the sampling rate directly set via ADEM:SPEC:SPAN:MAX or ADEM:BAND. If the available record length is not sufficient for the given bandwidth, it is set to maximum and the resolution bandwidth is increased accordingly.
        /// 
        /// Valid Values:
        /// 1.0 Hz to 10.0E+6 Hz
        /// 511.20 to 15336107.60 Hz for R&amp;S FSV
        /// 
        /// Default Value: 61.2E+3 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_BAND_RES
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureADemodAFParam(double Resolution_Bandwidth)
        {
            int pInvokeResult = PInvoke.ConfigureADemodAFParam(this._handle, Resolution_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures bandwidth resolution and span for display of the RF spectrum.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_BAND_RES
        /// RSSPECAN_ATTR_FMDEM_SPEC_SPAN
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:SPECtrum:BANDwidth:RESolution
        /// [SENSe:]ADEMod:SPECtrum:SPAN[:MAXimum]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Resolution_Bandwidth">
        /// This control sets the resolution bandwidth for display of the RF spectrum from the data of analog demodulation.
        /// The required record length is calculated from the sampling rate directly set via ADEM:SPEC:SPAN:MAX or ADEM:BAND. If the available record length is not sufficient for the given bandwidth, it is set to maximum and the resolution bandwidth is increased accordingly.
        /// 
        /// Valid Values:
        /// 1.0 Hz to 10.0E+6 Hz
        /// 
        /// Default Value: 61.2E+3 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_BAND_RES
        /// </param>
        /// <param name="Span">
        /// This control sets the frequency range for display of the RF spectrum resulting from the data of FM demodulation.
        /// 
        /// Valid Values:
        /// 100.0 Hz to 50.0E+6 Hz
        /// 
        /// Default Value: 5.0E+6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_SPEC_SPAN
        /// 
        /// Note:
        /// (1) Maximum span corresponds to the measurement bandwidth of analog demodulation (SENS:ADEM:BAND).
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureADemodRFParam(double Resolution_Bandwidth, double Span)
        {
            int pInvokeResult = PInvoke.ConfigureADemodRFParam(this._handle, Resolution_Bandwidth, Span);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the frequency range for result display of RF spectrum determined from FM demodulation data.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_SPEC_ZOOM
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:SPECtrum:SPAN:ZOOM
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Zoom">
        /// This control sets the frequency range for display of the RF spectrum resulting from the data of FM demodulation.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 5.0E+6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_SPEC_ZOOM
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureADemodRFZoom(double Zoom)
        {
            int pInvokeResult = PInvoke.ConfigureADemodRFZoom(this._handle, Zoom);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets PM phase wrap
        /// 
        /// Note:
        /// 
        /// (1) This function is available only in conjunction with R&amp;S FSL or R&amp;S ESL with option FS-K7.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_ADEM_PHASE_WRAP
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FORMat
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Phase_Wrap">
        /// Activates / Deactivates the phase wrapping to -PI to +PI (-180 to +180). The driver uses this control to set RSSPECAN_ATTR_ADEM_PHASE_WRAP attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ADEM_UPHAS (0) - Off 
        /// RSSPECAN_VAL_ADEM_PHAS  (1) - On
        /// 
        /// Default Value: RSSPECAN_VAL_ADEM_UPHAS (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ADEM_PHASE_WRAP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAnalogDemodPMPhaseWrap(int Phase_Wrap)
        {
            int pInvokeResult = PInvoke.ConfigureAnalogDemodPMPhaseWrap(this._handle, Phase_Wrap);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets PM phase unit.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only in conjunction with R&amp;S FSL or R&amp;S ESL with option FS-K7.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_ADEM_UNIT_ANGLE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:UNIT:ANGLe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="PM_Units">
        /// Selects the unit for PM traces. The driver uses this control to set RSSPECAN_ATTR_ADEM_UNIT_ANGLE attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_ANGLE_RAD (0) - RAD
        /// RSSPECAN_VAL_UNIT_ANGLE_DEG (1) - DEG
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_ANGLE_RAD (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ADEM_UNIT_ANGLE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAnalogDemodPMPhaseUnits(int PM_Units)
        {
            int pInvokeResult = PInvoke.ConfigureAnalogDemodPMPhaseUnits(this._handle, PM_Units);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets PM zero phase reference point
        /// 
        /// Note:
        /// 
        /// (1) This function is available only in conjunction with option FS-K7.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_ADEM_PM_RPO_X
        /// 
        /// Remote-control command(s):
        /// [SENSe1:]ADEMod:PM:RPOint[:X]
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Reference_Point">
        /// The reference point, where the phase is set to 0 rad.
        /// 
        /// Valid Values: 
        /// none
        /// 
        /// Default Value: 10.0e-6 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ADEM_PM_RPO_X
        /// 
        /// Note:
        /// 
        /// (1) This control has effect only for AF Coupling set to 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAnalogDemodPMZeroPhaseReferencePoint(double Reference_Point)
        {
            int pInvokeResult = PInvoke.ConfigureAnalogDemodPMZeroPhaseReferencePoint(this._handle, Reference_Point);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the result types to be measured with the selected demodulation.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with option FS-K7.
        /// 
        /// (2) Settings other than OFF can be assigned only to one result type at a time.
        /// 
        /// (3) If all result types are set to OFF, the selected demodulation is not calculated.
        /// 
        /// (4) The result type "AF spectrum of AM demodulated signal" cannot be activated at the same time as "AF spectrum of FM demodulated signal".
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:AM[:ABSolute][:TDOMain][:TYPE]
        /// [SENSe:]ADEMod:AM[:ABSolute]:AFSPectrum[:TYPE]
        /// [SENSe:]ADEMod:FM[:TDOMain][:TYPE]
        /// [SENSe:]ADEMod:FM:AFSPectrum[:TYPE]
        /// [SENSe:]ADEMod:PM[:TDOMain][:TYPE]
        /// [SENSe:]ADEMod:PM:AFSPectrum[:TYPE]
        /// [SENSe:]ADEMod:SPECtrum[:TYPE]
        /// [SENSe:]ADEMod:AM:RELative[:TDOMain][:TYPE]
        /// [SENSe:]ADEMod:AM:RELative:AFSPectrum[:TYPE]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Demodulation_Type">
        /// This parameter selects the demodulation.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FMDEM_AM       (0) - AM 
        /// RSSPECAN_VAL_FMDEM_FM       (1) - FM
        /// RSSPECAN_VAL_FMDEM_PM       (2) - PM
        /// RSSPECAN_VAL_FMDEM_AFSAM    (3) - AF Spectrum of AM
        /// RSSPECAN_VAL_FMDEM_AFSFM    (4) - AF Spectrum of FM
        /// RSSPECAN_VAL_FMDEM_AFSPM    (5) - AF Spectrum of PM
        /// RSSPECAN_VAL_FMDEM_RF       (6) - RF Spectrum
        /// RSSPECAN_VAL_FMDEM_AMREL    (7) - AM Relative
        /// RSSPECAN_VAL_FMDEM_AFSAMREL (8) - AF Spectrum of AM Relative
        /// 
        /// Default Value: RSSPECAN_VAL_FMDEM_AM (0)
        /// 
        /// Note(s):
        /// (1) AM: Selects the result types to be measured in parallel by AM demodulation.
        /// 
        /// (2) FM: Selects the result types to be created in parallel by FM demodulation.
        /// 
        /// (3) PM: Selects the result types to be created in parallel by PM demodulation.
        /// 
        /// (4) AF Spectrum of AM: Selects the result types to be measured simultaneously with display of the AF spectrum of the AM
        /// demodulated signal. This selection is not valid for R&amp;S FSL 
        /// 
        /// (5) AF Spectrum of FM: Selects the result types to be measured simultaneously with display of the AF spectrum of the FM
        /// demodulated signal. 
        /// 
        /// (6) AF Spectrum of PM: Selects the result types to be measured simultaneously with display of the AF spectrum of the PM
        /// demodulated signal. 
        /// 
        /// (5) RF Spectrum: Selects the result types to be measured simultaneously with display of the RF spectrum
        /// 
        /// (6) AM Relative: Selects the result types to be measured simultaneously in the case of AM demodulation.
        /// 
        /// (7) AF Spectrum of AM Relative: AF spectrum result types of the AM-demodulated signal to be measured simultaneously.
        /// </param>
        /// <param name="Result_Types">
        /// Selects the trace mode result types, to be measured in parallel by selected demodulation.
        /// 
        /// Valid Values:
        /// array of next values
        /// for R&amp;S FSL 4 values are expected, for other instrument 3 values are expected. 
        /// 
        /// Values should be:
        /// RSSPECAN_VAL_FMDEM_RESULT_OFF  (0) - Off
        /// RSSPECAN_VAL_FMDEM_RESULT_CURR (1) - Current Meas Value
        /// RSSPECAN_VAL_FMDEM_RESULT_AVG  (2) - Average
        /// RSSPECAN_VAL_FMDEM_RESULT_MAX  (3) - Maximum
        /// RSSPECAN_VAL_FMDEM_RESULT_MIN  (4) - Minimum
        /// RSSPECAN_VAL_FMDEM_RESULT_VIEW (5) - View
        /// 
        /// Default Value: RSSPECAN_VAL_FMDEM_RESULT_OFF
        /// 
        /// Note(s):
        /// 
        /// (1) Off:
        /// The result type is not used.
        /// 
        /// (2) Current Meas Value:
        /// The current measured values are recorded.
        /// 
        /// (3) Average:
        /// The measurement results are averaged over the given number of measurements.
        /// 
        /// (4) Maximum:
        /// The maximum result over the given number of measurements is obtained.
        /// 
        /// (5) Minimum:
        /// The minimum result over the given number of measurements is obtained.
        /// 
        /// (6) View:
        /// The measurement result is frozen on the display, i.e. it is not recalculated in further measurements.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureADemodResultType(int Demodulation_Type, int[] Result_Types)
        {
            int pInvokeResult = PInvoke.ConfigureADemodResultType(this._handle, Demodulation_Type, Result_Types);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads selected value from instrument.
        /// 
        /// Note:
        /// (1) This command is only available with option FS-K7 (FM Demodulator).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_RLEN
        /// RSSPECAN_ATTR_FMDEM_SRATE
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:RLENgth?
        /// [SENSe:]ADEMod:SRATe?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Value">
        /// Selects the parameter to be read from instrument.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FMDEM_RLEN  (0) - Record Length
        /// RSSPECAN_VAL_FMDEM_SRATE (1) - Sampling Rate
        /// 
        /// Default Value: RSSPECAN_VAL_FMDEM_RLEN (0) 
        /// 
        /// Note(s):
        /// 
        /// (1) Record Length: reads the currently set record length for the analog demodulation.
        /// 
        /// (2) Sampling Rate: reads the currently set sampling rate for the analog demodulation.
        /// </param>
        /// <param name="Result">
        /// This control returns the value of selected parameter.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetADemodCurrentlySetValue(int Value, out double Result)
        {
            int pInvokeResult = PInvoke.GetADemodCurrentlySetValue(this._handle, Value, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns analog demodulation result values.
        /// 
        /// Notes:
        /// 
        /// (1) This function is available only in conjunction with option FS-K7.
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:AM[:ABSolute]:RESult? WRITe | AVERage | MAXHold | MINHold
        /// [SENSe:]ADEMod:FM:RESult? WRITe | AVERage | MAXHold | MINHold
        /// [SENSe:]ADEMod:PM:RESult? WRITe | AVERage | MAXHold | MINHold
        /// [SENSe:]ADEMod:AM[:ABSolute]:AFSPectrum:RESult? &lt;result type&gt;
        /// [SENSe:]ADEMod:FM:AFSPectrum:RESult? WRITe | AVERage | MAXHold | MINHold
        /// [SENSe:]ADEMod:PM:AFSPectrum:RESult? WRITe | AVERage | MAXHold | MINHold
        /// [SENSe:]ADEMod:SPECtrum:RESult? WRITe | AVERage | MAXHold | MINHold
        /// [SENSe:]ADEMod:AM:RELative:RESult? WRITe | AVERage | MAXHold | MINHold
        /// [SENSe:]ADEMod:AM:RELative:AFSPectrum:RESult? WRITe | AVERage | MAXHold | MINHold
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Demodulation_Type">
        /// This parameter selects the demodulation.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FMDEM_AM       (0) - AM 
        /// RSSPECAN_VAL_FMDEM_FM       (1) - FM
        /// RSSPECAN_VAL_FMDEM_PM       (2) - PM
        /// RSSPECAN_VAL_FMDEM_AFSAM    (3) - AF Spectrum of AM
        /// RSSPECAN_VAL_FMDEM_AFSFM    (4) - AF Spectrum of FM
        /// RSSPECAN_VAL_FMDEM_AFSPM    (5) - AF Spectrum of PM
        /// RSSPECAN_VAL_FMDEM_RF       (6) - RF Spectrum
        /// RSSPECAN_VAL_FMDEM_AMREL    (7) - AM Relative
        /// RSSPECAN_VAL_FMDEM_AFSAMREL (8) - AF Spectrum of AM Relative
        /// 
        /// Default Value: RSSPECAN_VAL_FMDEM_AM (0)
        /// 
        /// Note(s):
        /// (1) AM: Selects the result types to be measured in parallel by AM demodulation.
        /// 
        /// (2) FM: Selects the result types to be created in parallel by FM demodulation.
        /// 
        /// (3) PM: Selects the result types to be created in parallel by PM demodulation.
        /// 
        /// (4) AF Spectrum of AM: Selects the result types to be measured simultaneously with display of the AF spectrum of the AM
        /// demodulated signal.  This selection is not valid for R&amp;S FSL
        /// 
        /// (5) AF Spectrum of FM: Selects the result types to be measured simultaneously with display of the AF spectrum of the FM
        /// demodulated signal. 
        /// 
        /// (6) AF Spectrum of PM: Selects the result types to be measured simultaneously with display of the AF spectrum of the PM
        /// demodulated signal. 
        /// 
        /// (5) RF Spectrum: Selects the result types to be measured simultaneously with display of the RF spectrum
        /// 
        /// (6) AF Spectrum of AM Relative: Selects the result types to be measured AF spectrum result data of the AM?demodulated signal.
        /// 
        /// </param>
        /// <param name="Result_Type">
        /// Selects the result type to be measured in parallel by selected demodulation.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FMDEM_RESULT_CURR (1) - Current Meas Value
        /// RSSPECAN_VAL_FMDEM_RESULT_AVG  (2) - Average
        /// RSSPECAN_VAL_FMDEM_RESULT_MAX  (3) - Maximum
        /// RSSPECAN_VAL_FMDEM_RESULT_MIN  (4) - Minimum
        /// 
        /// Default Value: RSSPECAN_VAL_FMDEM_RESULT_CURR (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Current Meas Value:
        /// The current measured values are recorded.
        /// 
        /// (2) Average:
        /// The measurement results are averaged over the given number of measurements.
        /// 
        /// (3) Maximum:
        /// The maximum result over the given number of measurements is obtained.
        /// 
        /// (4) Minimum:
        /// The minimum result over the given number of measurements is obtained.
        /// 
        /// (5) The result type indicated must be one of those configured by Configure FM Demod Result Type. Otherwise a query error will be generated.
        /// 
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Array_Size">
        /// This parameter determines the size of array result values. The Result Values array must contain at least this number of items.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 0
        /// </param>
        /// <param name="Result_Values">
        /// Returns the result values.
        /// 
        /// </param>
        /// <param name="Returned_Values">
        /// Returns the number of returned values.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetADemodResultValues(int Demodulation_Type, int Result_Type, uint Timeout, int Array_Size, double[] Result_Values, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.GetADemodResultValues(this._handle, Demodulation_Type, Result_Type, Timeout, Array_Size, Result_Values, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function calculates the FM offset of the current measured data set.
        /// 
        /// Notes:
        /// 
        /// (1) This function is available only in conjunction with option FS-K7.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ADEM_FM_OFFSET
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:FM:OFFSet? IMMediate | AVERage
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Result_Type">
        /// This control determines the result type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FMDEM_IMM (0) - Immediate
        /// RSSPECAN_VAL_FMDEM_AVG (1) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_FMDEM_IMM (0)
        /// 
        /// Notes:
        /// 
        /// (1) Immediate:
        /// The current measurement results are used for calculating the FM offset.
        /// 
        /// (2) Average:
        /// The measurement results averaged over the given number of measurements are used for calculation of the FM offset.
        /// 
        /// (3) If no average measurement has been active during the last measurement sequence, only the type Immediate will return a valid result (data to calculate the FM offset are taken from the last measured data set). Type Average will cause a query error in this case.
        /// 
        /// </param>
        /// <param name="Value">
        /// Returns measured value.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetADemodOffset(int Result_Type, out double Value)
        {
            int pInvokeResult = PInvoke.GetADemodOffset(this._handle, Result_Type, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns analog modulation measurement summary.
        /// 
        /// Notes:
        /// 
        /// (1) This function is available only in conjunction with option FS-K7.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ADEM_SUMM_RES
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer:FUNCtion:ADEMod:AM[:RESult&lt;1...4&gt;?] PPEak | MPEak | MIDDle | RMS
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:ADEMod:FM:RESult&lt;1 to 4&gt;? PPEak | MPEak | MIDDle |RMS 
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:ADEMod:PM:RESult&lt;1 to 4&gt;? PPEak | MPEak | MIDDle |RMS 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Marker_Number">
        /// Selects the marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Demodulation_Type">
        /// This parameter selects the demodulation.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FMDEM_AM (0) - AM
        /// RSSPECAN_VAL_FMDEM_FM (1) - FM
        /// RSSPECAN_VAL_FMDEM_PM (2) - PM
        /// 
        /// Default Value: RSSPECAN_VAL_FMDEM_FM (0)
        /// 
        /// Note(s):
        /// (1) AM: Selects the result types to be created in parallel by AM demodulation.
        /// 
        /// (2) FM: Selects the result types to be created in parallel by FM demodulation.
        /// 
        /// (3) PM: Selects the result types to be created in parallel by PM demodulation.
        /// </param>
        /// <param name="Detector_Type">
        /// This control selects the detector type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FMDEM_DET_PPE (0) - +PK
        /// RSSPECAN_VAL_FMDEM_DET_MPE (1) - -PK
        /// RSSPECAN_VAL_FMDEM_DET_MID (2) - +/-PK/2
        /// RSSPECAN_VAL_FMDEM_DET_RMS (3) - RMS
        /// 
        /// Default Value: RSSPECAN_VAL_FMDEM_DET_PPE (0)
        /// 
        /// Notes:
        /// 
        /// (1) +PK:
        /// Result of measurement with +PK detector.
        /// 
        /// (2) -PK:
        /// Result of measurement with -PK detector.
        /// 
        /// (3) ?PK/2:
        /// Result of averaging ?PK/2, i.e. (plus peak + minus peak)/2.
        /// 
        /// (4) RMS:
        /// Result of measurement with RMS detector
        /// </param>
        /// <param name="Trace">
        /// Selects the trace.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 1 to 6 for R&amp;S FSL or R&amp;S FSV
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Value">
        /// Returns measured value.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetADemodMarkerModulationValue(int Marker_Number, int Demodulation_Type, int Detector_Type, int Trace, out double Value)
        {
            int pInvokeResult = PInvoke.GetADemodMarkerModulationValue(this._handle, Marker_Number, Demodulation_Type, Detector_Type, Trace, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns analog modulation measurement summary.
        /// 
        /// Notes:
        /// 
        /// (1) This function is available only in conjunction with option FS-K7.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_AFR_RES
        /// RSSPECAN_ATTR_FMDEM_FERR_RES
        /// RSSPECAN_ATTR_FMDEM_SIN_RES
        /// RSSPECAN_ATTR_FMDEM_THD_RES
        /// RSSPECAN_ATTR_FMDEM_CARR_RES
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:ADEMod:AFRequency[:RESult&lt;1 to 3&gt;]?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:ADEMod:FERRor[:RESult&lt;1 to 3&gt;]?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:ADEMod:SINad:RESult&lt;1 to 3&gt;?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:ADEMod:THD:RESult&lt;1 to 3&gt;?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:ADEMod:CARRier[:RESult&lt;1 to 3&gt;]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Marker_Number">
        /// Selects the marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Measurement">
        /// This parameter selects the measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FMDEM_MEAS_AFR (0) - Audio Frequency
        /// RSSPECAN_VAL_FMDEM_MEAS_FER (1) - Frequency Error
        /// RSSPECAN_VAL_FMDEM_MEAS_SIN (2) - SINAD
        /// RSSPECAN_VAL_FMDEM_MEAS_THD (3) - THD
        /// RSSPECAN_VAL_FMDEM_MEAS_CAR (4) - Carrier Power
        /// 
        /// Default Value: RSSPECAN_VAL_FMDEM_MEAS_AFR (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Audio Frequency: queries the audio frequency with analog demodulation.
        /// 
        /// (2) Frequency Error: queries the frequency error with FM demodulation.
        /// 
        /// (3) SINAD: queries the result of the SINAD measurement.
        /// 
        /// (4) THD: queries the result of the THD measurement.
        /// 
        /// (5) Carrier Power: queries the carrier power.
        /// </param>
        /// <param name="Trace">
        /// Selects the trace.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 1 to 6 for R&amp;S FSL or R&amp;S FSV
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Value">
        /// Returns measured value.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetADemodMarkerMeasValue(int Marker_Number, int Measurement, int Trace, out double Value)
        {
            int pInvokeResult = PInvoke.GetADemodMarkerMeasValue(this._handle, Marker_Number, Measurement, Trace, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the FM Stereo mode.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMS_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument[:SELect] SFM
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FMStereoMode()
        {
            int pInvokeResult = PInvoke.FMStereoMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches between Stereo FM and Spectrum mode.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMS_STATE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1...4&gt;:]SFM:STATe ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="State">
        /// This control switches between Stereo FM and Spectrum mode.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FM_STEREO_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SetFMStereoState(bool State)
        {
            int pInvokeResult = PInvoke.SetFMStereoState(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the FM Stereo trigger settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMS_TRIGGER_SOURCE
        /// RSSPECAN_ATTR_EXTERNAL_TRIGGER_LEVEL
        /// RSSPECAN_ATTR_TRIGGER_IFP_LEVEL
        /// RSSPECAN_ATTR_TRIGGER_TIME_INTERVAL
        /// RSSPECAN_ATTR_FMS_TRIGGER_LEVEL
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1...4&gt;[:SEQuence]:SOURce IMMediate | EXTern | IFPower | TIME | SLEFt | SRIGht | SMPX | SMONo | SSTereo | SRDS | SPILot
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel[:EXTernal]
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:IFPower
        /// TRIGger&lt;1|2&gt;[:SEQuence]:TIME:RINTerval
        /// TRIGger&lt;1...4&gt;[:SEQuence]:LEVel:SFM:LEFT
        /// TRIGger&lt;1...4&gt;[:SEQuence]:LEVel:SFM:RIGHt
        /// TRIGger&lt;1...4&gt;[:SEQuence]:LEVel:SFM:MPX
        /// TRIGger&lt;1...4&gt;[:SEQuence]:LEVel:SFM:MONO
        /// TRIGger&lt;1...4&gt;[:SEQuence]:LEVel:SFM:STEReo
        /// TRIGger&lt;1...4&gt;[:SEQuence]:LEVel:SFM:RDS
        /// TRIGger&lt;1...4&gt;[:SEQuence]:LEVel:SFM:PILot
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Trigger_Source">
        /// Specifies the source of the trigger signal that causes the analyzer to leave the Wait-For-Trigger state. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FMS_TRG_IMM  (0) - Free Run 
        /// RSSPECAN_VAL_FMS_TRG_EXT  (1) - External   
        /// RSSPECAN_VAL_FMS_TRG_IFP  (2) - IF Power
        /// RSSPECAN_VAL_FMS_TRG_TIME (3) - Time
        /// RSSPECAN_VAL_FMS_TRG_LEFT (4) - Left
        /// RSSPECAN_VAL_FMS_TRG_RIGH (5) - Right
        /// RSSPECAN_VAL_FMS_TRG_MPX  (6) - MPX
        /// RSSPECAN_VAL_FMS_TRG_MONO (7) - Mono
        /// RSSPECAN_VAL_FMS_TRG_STER (8) - Stereo
        /// RSSPECAN_VAL_FMS_TRG_RDS  (9) - RDS
        /// RSSPECAN_VAL_FMS_TRG_PIL  (10) - Pilot
        /// 
        /// Default value: RSSPECAN_VAL_FMS_TRG_IMM (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMS_TRIGGER_SOURCE
        /// </param>
        /// <param name="Trigger_Level">
        /// This command sets the level of the trigger source.
        /// 
        /// Valid Values: 
        /// 0.5 to 3.5           : for External
        /// -50 to 30 dBm        : for IF Power
        /// 0.1 to 5000 s        : for Time
        /// -10.0e6 to 10.0e6 Hz : for Left, Right, MPX, Mono, Stereo, RDS, Pilot
        ///  
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_TRIGGER_LEVEL
        /// RSSPECAN_ATTR_TRIGGER_IFP_LEVEL
        /// RSSPECAN_ATTR_TRIGGER_TIME_INTERVAL
        /// RSSPECAN_ATTR_FMS_TRIGGER_LEVEL
        /// 
        /// Note:
        /// 
        /// (1) This control has no effect for Free Run trigger source
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFMStereoTrigger(int Trigger_Source, double Trigger_Level)
        {
            int pInvokeResult = PInvoke.ConfigureFMStereoTrigger(this._handle, Trigger_Source, Trigger_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures phase noise marker measurement. To setup reference point use function rsspecan_ConfigureReferenceFixedPoint.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMS_PHASE_NOISE_MARKER_STATE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1...4&gt;:MARKer&lt;1...16&gt;:FUNCtion:PNOise ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1..16
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// This control switches the phase noise measurement for the specified marker on or off in the specified window. The phase noise power density is measured at the position of the markers. 
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMS_PHASE_NOISE_MARKER_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFMStereoPhaseNoiseMarker(int Window, int Marker, bool State)
        {
            int pInvokeResult = PInvoke.ConfigureFMStereoPhaseNoiseMarker(this._handle, Window, Marker, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures difference frequency distortion measurement. This function is only available for AF spectrum measurements.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMS_DIFFERENCE_FREQUENCY_DISTORTION_STATE
        /// RSSPECAN_ATTR_FMS_DIFFERENCE_FREQUENCY_DISTORTION_UNIT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1...4&gt;:MARKer:FUNCtion:DFD[:STATe] ON | OFF
        /// CALCulate&lt;1...4&gt;:MARKer:FUNCtion:DFD:UNIT PCT | DB
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// This control activates difference frequency distortion measurement.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMS_DIFFERENCE_FREQUENCY_DISTORTION_STATE
        /// </param>
        /// <param name="Unit">
        /// This control defines the unit for the difference frequency distortion measurement results. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DB  (8) - dB
        /// RSSPECAN_VAL_UNIT_PCT (13) - %
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_PCT (13)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMS_DIFFERENCE_FREQUENCY_DISTORTION_UNIT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFMStereoDifferenceFrequencyDistortion(int Window, bool State, int Unit)
        {
            int pInvokeResult = PInvoke.ConfigureFMStereoDifferenceFrequencyDistortion(this._handle, Window, System.Convert.ToUInt16(State), Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures intermodulation distortion measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMS_INTERMODULATION_DISTORTION_STATE
        /// RSSPECAN_ATTR_FMS_INTERMODULATION_DISTORTION_UNIT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1...4&gt;:MARKer:FUNCtion:IMD[:STATe] ON | OFF
        /// CALCulate&lt;1...4&gt;:MARKer:FUNCtion:IMD:UNIT PCT | DB
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// This control activates intermodulation distortion measurement.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMS_INTERMODULATION_DISTORTION_STATE
        /// </param>
        /// <param name="Unit">
        /// This control defines the unit for the difference frequency distortion measurement results. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DB  (8) - dB
        /// RSSPECAN_VAL_UNIT_PCT (13) - %
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_PCT (13)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMS_INTERMODULATION_DISTORTION_UNIT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFMStereoIntermodulationDistortion(int Window, bool State, int Unit)
        {
            int pInvokeResult = PInvoke.ConfigureFMStereoIntermodulationDistortion(this._handle, Window, System.Convert.ToUInt16(State), Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates automatic scaling of the y-axis for FM stereo measurements. Currently auto-scaling is only available for AF measurements. RF power and RF spectrum measurements are not affected by the auto-scaling.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMS_AF_AUTO_SCALE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1...4&gt;:]ADJust:SCALe:Y:AUTO[:CONTinuous] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="State">
        /// This control activates automatic scaling of the y-axis for FM stereo measurements.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMS_AF_AUTO_SCALE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFMStereoAFAutoScale(bool State)
        {
            int pInvokeResult = PInvoke.ConfigureFMStereoAFAutoScale(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the evaluation method of the measured data that is to be displayed.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISPLAY_STATE
        /// RSSPECAN_ATTR_FMS_DISPLAY_CONFIG
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;window&gt;]:STATe
        /// CALCulate&lt;1...4&gt;:FEED XTIM:SUMMary | XTIM:RFPower[:TDOMain] | XTIM:SPECtrum | XFRequency:SFM:LEFT | XFRequency:SFM:RIGHt | XFRequency:SFM:MPX | XFRequency:SFM:MONO | XFRequency:SFM:STEReo | XFRequency:SFM:RDS | XFRequency:SFM:PILot | XTIMe:SFM:LEFT | XTIMe:SFM:RIGHt | XTIMe:SFM:MPX | XTIMe:SFM:MONO | XTIMe:SFM:STEReo | XTIMe:SFM:RDS | XTIMe:SFM:PILot
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Display_Configuration">
        /// This control selects the evaluation method of the measured data that is to be displayed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FMS_DISP_OFF        (0) - Off
        /// RSSPECAN_VAL_FMS_DISP_SUMM       (1) - Summary
        /// RSSPECAN_VAL_FMS_DISP_TD_RF      (2) - RF Time Domain
        /// RSSPECAN_VAL_FMS_DISP_TD_LEFT    (3) - Left Time Domain
        /// RSSPECAN_VAL_FMS_DISP_TD_RIGHT   (4) - Right Time Domain
        /// RSSPECAN_VAL_FMS_DISP_TD_MPX     (5) - MPX Time Domain
        /// RSSPECAN_VAL_FMS_DISP_TD_MONO    (6) - Mono Time Domain
        /// RSSPECAN_VAL_FMS_DISP_TD_STER    (7) - Stereo Time Domain
        /// RSSPECAN_VAL_FMS_DISP_TD_RDS     (8) - RDS Time Domain
        /// RSSPECAN_VAL_FMS_DISP_TD_PILOT   (9) - Pilot Time Domain
        /// RSSPECAN_VAL_FMS_DISP_SPEC_RF    (10) - RF Spectrum
        /// RSSPECAN_VAL_FMS_DISP_SPEC_LEFT  (11) - Left Spectrum
        /// RSSPECAN_VAL_FMS_DISP_SPEC_RIGHT (12) - Right Spectrum
        /// RSSPECAN_VAL_FMS_DISP_SPEC_MPX   (13) - MPX Spectrum
        /// RSSPECAN_VAL_FMS_DISP_SPEC_MONO  (14) - Mono Spectrum
        /// RSSPECAN_VAL_FMS_DISP_SPEC_STER  (15) - Stereo Spectrum
        /// RSSPECAN_VAL_FMS_DISP_SPEC_RDS   (16) - RDS Spectrum
        /// RSSPECAN_VAL_FMS_DISP_SPEC_PILOT (17) - Pilot Spectrum
        /// 
        /// Default Value: RSSPECAN_VAL_FMS_DISP_SUMM       (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISPLAY_STATE
        /// RSSPECAN_ATTR_FMS_DISPLAY_CONFIG
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFMStereoDisplayConfig(int Window, int Display_Configuration)
        {
            int pInvokeResult = PInvoke.ConfigureFMStereoDisplayConfig(this._handle, Window, Display_Configuration);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the reference deviation required for relative deviation measurements.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMS_REFERENCE_DEVIATION
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1...4&gt;:]SFM:REFerence
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Reference_Deviation">
        /// This control defines the reference deviation required for relative deviation measurements.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 75.0e3
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMS_REFERENCE_DEVIATION
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFMStereoReferenceDeviation(double Reference_Deviation)
        {
            int pInvokeResult = PInvoke.ConfigureFMStereoReferenceDeviation(this._handle, Reference_Deviation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures result summary display.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMS_RESULT_SUMMARY_DETECTOR
        /// RSSPECAN_ATTR_FMS_RESULT_SUMMARY_MODE
        /// RSSPECAN_ATTR_FMS_RESULT_SUMMARY_COUPLED
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1...4&gt;:]SFM:&lt;ChannelType&gt;:RSUMmary:DETector[:FUNCtion] RMS | SRMS | PPEak | NPEak | PAVerage | QPEak | SQPeak
        /// [SENSe&lt;1...4&gt;:]SFM:&lt;ChannelType&gt;:RSUMmary:MODE WRITe | AVERage | PHOLd
        /// [SENSe&lt;1...4&gt;:]SFM:&lt;ChannelType&gt;:RSUMmary:COUPling ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Channel_Type">
        /// This control select the channel type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_LEFT  (0) - Left
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_RIGHT (1) - Right
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_MPX   (2) - MPX
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_MONO  (3) - Mono
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_STER  (4) - Stereo
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_RDS   (5) - RDS
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_PILOT (6) - Pilot
        /// 
        /// Default Value: RSSPECAN_VAL_FSM_CHAN_TYPE_LEFT  (0)
        /// </param>
        /// <param name="Detector">
        /// This control defines the detector used to determine the deviation value of the selected channel of the FM stereo signal in the result summary.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FMS_RSUM_DETEC_RMS  (0) - RMS
        /// RSSPECAN_VAL_FMS_RSUM_DETEC_SRMS (1) - RMS * SQRT2
        /// RSSPECAN_VAL_FMS_RSUM_DETEC_PPE  (2) - Positive Peak
        /// RSSPECAN_VAL_FMS_RSUM_DETEC_NPE  (3) - Negative Peak
        /// RSSPECAN_VAL_FMS_RSUM_DETEC_PAV  (4) - ?Peak/2
        /// RSSPECAN_VAL_FMS_RSUM_DETEC_QPE  (5) - Quasipeak CCIR
        /// RSSPECAN_VAL_FMS_RSUM_DETEC_SQP  (6) - Quasipeak * SQRT2
        /// 
        /// Default Value: RSSPECAN_VAL_FMS_RSUM_DETEC_RMS (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMS_RESULT_SUMMARY_DETECTOR
        /// </param>
        /// <param name="Mode">
        /// This control defines the result summary mode for the absolute and relative deviation. It does not affect the trace mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FMS_RSUM_MODE_WRIT (0) - Clear Write
        /// RSSPECAN_VAL_FMS_RSUM_MODE_AVER (1) - Average
        /// RSSPECAN_VAL_FMS_RSUM_MODE_PHOL (2) - Peak Hold
        /// 
        /// Default Value: RSSPECAN_VAL_FMS_RSUM_MODE_WRIT (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMS_RESULT_SUMMARY_MODE
        /// 
        /// Note(s):
        /// 
        /// (1) Clear Write: Overwrite mode; the summary is overwritten by each sweep.
        /// 
        /// (2) Average: The average is formed over several sweeps. The number of sweeps is defined by the sweep count.
        /// 
        /// (3) Peak hold: The maximum values are determined over several sweeps and displayed. The number of sweeps is defined by the sweep count.
        /// </param>
        /// <param name="Coupled">
        /// This control couples the channel settings to other channels, i.e. channels for which this control is set to "ON" are configured identically.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMS_RESULT_SUMMARY_COUPLED
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFMStereoResultSummarySetup(int Channel_Type, int Detector, int Mode, bool Coupled)
        {
            int pInvokeResult = PInvoke.ConfigureFMStereoResultSummarySetup(this._handle, Channel_Type, Detector, Mode, System.Convert.ToUInt16(Coupled));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function determines the reference deviation from the current channel measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMS_MEAS_TO_REF
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1...4&gt;:]SFM:&lt;ChannelType&gt;:RSUMmary:REFerence[:AUTO] ONCE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Channel_Type">
        /// This control select the channel type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_LEFT  (0) - Left
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_RIGHT (1) - Right
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_MPX   (2) - MPX
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_MONO  (3) - Mono
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_STER  (4) - Stereo
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_RDS   (5) - RDS
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_PILOT (6) - Pilot
        /// 
        /// Default Value: RSSPECAN_VAL_FSM_CHAN_TYPE_LEFT  (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFMStereoMeasToRef(int Channel_Type)
        {
            int pInvokeResult = PInvoke.ConfigureFMStereoMeasToRef(this._handle, Channel_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures analog demodulator measuring time parameters.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_MTIM
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:MTIMe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measuring_Time">
        /// This control sets the measuring time for the analog Demodulation.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 2.0e-3 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_MTIM
        /// 
        /// Note(s):
        /// 
        /// (1) The minimum measurement time is 2 ms.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFMStereoMeasurementTime(double Measuring_Time)
        {
            int pInvokeResult = PInvoke.ConfigureFMStereoMeasurementTime(this._handle, Measuring_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures different filter settings for each channel.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMS_AF_FILTER_HIGH_PASS_STATE
        /// RSSPECAN_ATTR_FMS_AF_FILTER_HIGH_PASS_FREQUENCY
        /// RSSPECAN_ATTR_FMS_AF_FILTER_LOW_PASS_STATE
        /// RSSPECAN_ATTR_FMS_AF_FILTER_LOW_PASS_FREQUENCY_ABSOLUTE
        /// RSSPECAN_ATTR_FMS_AF_FILTER_LOW_PASS_FREQUENCY_RELATIVE
        /// RSSPECAN_ATTR_FMS_AF_FILTER_CCITT
        /// RSSPECAN_ATTR_FMS_AF_FILTER_CCIR_UNWEIGHTED
        /// RSSPECAN_ATTR_FMS_AF_FILTER_CCIR_WEIGHTED
        /// RSSPECAN_ATTR_FMS_AF_FILTER_DEEMPHASIS_STATE
        /// RSSPECAN_ATTR_FMS_AF_FILTER_DEEMPHASIS_TIME_CONSTANT
        /// RSSPECAN_ATTR_FMS_AF_FILTER_COUPLED
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1...4&gt;:]SFM:&lt;ChannelType&gt;:FILTer:HPASs:STATe ON | OFF
        /// [SENSe&lt;1...4&gt;:]SFM:&lt;ChannelType&gt;:FILTer:HPASs:FREQuency
        /// [SENSe&lt;1...4&gt;:]SFM:&lt;ChannelType&gt;:FILTer:LPASs:STATe ON | OFF
        /// [SENSe&lt;1...4&gt;:]SFM:&lt;ChannelType&gt;:FILTer:LPASs:FREQuency
        /// [SENSe&lt;1...4&gt;:]SFM:&lt;ChannelType&gt;:FILTer:CCITt:STATe ON | OFF
        /// [SENSe&lt;1...4&gt;:]SFM:&lt;ChannelType&gt;:FILTer:CCIR[:UNWeighted][:STATe] ON | OFF
        /// [SENSe&lt;1...4&gt;:]SFM:&lt;ChannelType&gt;:FILTer:CCIR:WEIGhted[:STATe] ON | OFF
        /// [SENSe&lt;1...4&gt;:]SFM:&lt;ChannelType&gt;:FILTer:DEMPhasis:STATe ON | OFF
        /// [SENSe&lt;1...4&gt;:]SFM:&lt;ChannelType&gt;:FILTer:DEMPhasis:TCONstant
        /// [SENSe&lt;1...4&gt;:]SFM:&lt;ChannelType&gt;:FILTer:COUPling ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Channel_Type">
        /// This control select the channel type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_LEFT  (0) - Left
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_RIGHT (1) - Right
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_MPX   (2) - MPX
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_MONO  (3) - Mono
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_STER  (4) - Stereo
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_RDS   (5) - RDS
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_PILOT (6) - Pilot
        /// 
        /// Default Value: RSSPECAN_VAL_FSM_CHAN_TYPE_LEFT  (0)
        /// </param>
        /// <param name="High_Pass">
        /// This control selects the high pass filter type for the specified channel type.
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_FMS_HPAS_FILT_NONE  (0) - None
        /// RSSPECAN_VAL_FMS_HPAS_FILT_20HZ  (1) - 20 Hz
        /// RSSPECAN_VAL_FMS_HPAS_FILT_50HZ  (2) - 50 Hz
        /// RSSPECAN_VAL_FMS_HPAS_FILT_300HZ (3) - 300 Hz
        /// 
        /// Default Value: RSSPECAN_VAL_FMS_HPAS_FILT_NONE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMS_AF_FILTER_HIGH_PASS_STATE
        /// RSSPECAN_ATTR_FMS_AF_FILTER_HIGH_PASS_FREQUENCY
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available when Weighting is set to NONE.
        /// 
        /// (2) None: deactivates the selected high pass filter.
        /// 
        /// (3) The 20 Hz filter is designed as 3rd-order Butterworth filter (18 dB/octave).
        /// 
        /// (4) The 50 Hz and 300 Hz filters are designed as 2nd-order Butterworth filter (12 dB/octave). 
        /// </param>
        /// <param name="Low_Pass">
        /// This control selects the low pass filter type for the specified channel type.
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_FMS_LPAS_FILT_NONE   (0) - None
        /// RSSPECAN_VAL_FMS_LPAS_FILT_10PCT  (1) - 10 % DBW
        /// RSSPECAN_VAL_FMS_LPAS_FILT_15KHZ  (2) - 15 kHz
        /// RSSPECAN_VAL_FMS_LPAS_FILT_150KHZ (3) - 150 kHz
        /// RSSPECAN_VAL_FMS_LPAS_FILT_23KHZ  (4) - 23 kHz
        /// RSSPECAN_VAL_FMS_LPAS_FILT_25PCT  (5) - 25 % DBW
        /// RSSPECAN_VAL_FMS_LPAS_FILT_3KHZ   (6) - 3 kHz
        /// RSSPECAN_VAL_FMS_LPAS_FILT_5PCT   (7) - 5 % DBW
        /// 
        /// Default Value: RSSPECAN_VAL_FMS_LPAS_FILT_NONE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMS_AF_FILTER_LOW_PASS_STATE
        /// RSSPECAN_ATTR_FMS_AF_FILTER_LOW_PASS_FREQUENCY_ABSOLUTE
        /// RSSPECAN_ATTR_FMS_AF_FILTER_LOW_PASS_FREQUENCY_RELATIVE
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available when Weighting is set to NONE.
        /// 
        /// (2) None: deactivates the selected low pass filter.
        /// 
        /// (3) The 3 kHz, 15 kHz and 23 kHz filters are designed as 5th-order Butterworth filters (30 dB/octave).
        /// 
        /// (4) The 150 kHz filter is designed as 8th-order Butterworth filter (48 dB/octave). 
        /// 
        /// (5) The 5, 10 and 25 % filters are designed as 5th-order Butterworth filter (30 dB/octave) and active for all demodulation bandwidths.
        /// </param>
        /// <param name="Weighting">
        /// This control activates/deactivates the weighted filter for the specified channel type. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FMS_WEIGHTING_NONE  (0) - None
        /// RSSPECAN_VAL_FMS_WEIGHTING_CCIT  (1) - CCIT
        /// RSSPECAN_VAL_FMS_WEIGHTING_CCIRU (2) - CCIR Unweighted
        /// RSSPECAN_VAL_FMS_WEIGHTING_CCIRW (3) - CCIR Weighted
        /// 
        /// Default Value: RSSPECAN_VAL_FMS_WEIGHTING_NONE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMS_AF_FILTER_CCITT
        /// RSSPECAN_ATTR_FMS_AF_FILTER_CCIR_UNWEIGHTED
        /// RSSPECAN_ATTR_FMS_AF_FILTER_CCIR_WEIGHTED
        /// 
        /// Note(s):
        /// 
        /// (1) None: Deactivates the weighting filter.
        /// 
        /// (2) CCIT: Switches on a CCIT P.53 weighting filter.
        /// 
        /// (3) CCIR Unweighted: Switches on the CCIR unweighted filter, which is the combination of the 20 Hz highpass and 23 kHz low pass filter.
        /// 
        /// (4) CCIR Weighted: Switches on the CCIR weighted filter.
        /// </param>
        /// <param name="Deemphasis">
        /// This control selects the deemphasis for the specified channel type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FMS_DEEMPH_NONE (0) - None
        /// RSSPECAN_VAL_FMS_DEEMPH_25   (1) - 25 us
        /// RSSPECAN_VAL_FMS_DEEMPH_50   (2) - 50 us
        /// RSSPECAN_VAL_FMS_DEEMPH_75   (3) - 75 us
        /// RSSPECAN_VAL_FMS_DEEMPH_750  (4) - 750 us
        /// 
        /// Default Value: RSSPECAN_VAL_FMS_DEEMPH_NONE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMS_AF_FILTER_DEEMPHASIS_STATE
        /// RSSPECAN_ATTR_FMS_AF_FILTER_DEEMPHASIS_TIME_CONSTANT
        /// 
        /// Note(s):
        /// 
        /// (1) None: Deactivates the deemphasis filter.
        /// </param>
        /// <param name="Coupled">
        /// This control couples the channel settings to other channels, i.e. channels for which this control is set to "ON" are configured identically.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMS_AF_FILTER_COUPLED
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFMStereoAFFilter(int Channel_Type, int High_Pass, int Low_Pass, int Weighting, int Deemphasis, bool Coupled)
        {
            int pInvokeResult = PInvoke.ConfigureFMStereoAFFilter(this._handle, Channel_Type, High_Pass, Low_Pass, Weighting, Deemphasis, System.Convert.ToUInt16(Coupled));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches all AF Filters off at once.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMS_ALL_AF_FILTERS_OFF
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1...4&gt;:]FILTer:AOFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int AllFMStereoAFFiltersOff()
        {
            int pInvokeResult = PInvoke.AllFMStereoAFFiltersOff(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures deviation settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VERTICAL_SCALE
        /// RSSPECAN_ATTR_DISP_Y_AXIS_SCALING
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y:SPACing
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:PDIVision
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Deviation">
        /// Switches between logarithmic and linear display.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TRACE_SPACE_LIN (0) - Linear
        /// RSSPECAN_VAL_TRACE_SPACE_LOG (1) - Logarithmic
        /// 
        /// Default Value: RSSPECAN_VAL_TRACE_SPACE_LOG (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VERTICAL_SCALE
        /// </param>
        /// <param name="Dev_per_Division">
        /// Defines the deviation per division.
        /// 
        /// Valid Values: no checking
        /// 
        /// Default Value: 1.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_Y_AXIS_SCALING
        /// 
        /// Note(s):
        /// 
        /// (1) This control is not available if Deviation is set to logarithmic display.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFMStereoAFRange(int Deviation, double Dev_per_Division)
        {
            int pInvokeResult = PInvoke.ConfigureFMStereoAFRange(this._handle, Deviation, Dev_per_Division);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects between % and dB for displaying the
        /// THD measurement result.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_ADEM_UNIT_THD
        /// 
        /// Remote-control command(s):
        /// UNIT:THD PCT | DB
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Unit">
        /// This control selects between % and dB for displaying the
        /// THD measurement result.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DB  (8) - dB
        /// RSSPECAN_VAL_UNIT_PCT (13) - %
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_DB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ADEM_UNIT_THD
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFMStereoAFRangeTHDUnit(int Unit)
        {
            int pInvokeResult = PInvoke.ConfigureFMStereoAFRangeTHDUnit(this._handle, Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the deviation units.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMS_AF_RANGE_DEV_UNIT_ABSOLUTE
        /// RSSPECAN_ATTR_FMS_AF_RANGE_DEV_UNIT_RELATIVE
        /// 
        /// Remote-control command(s):
        /// UNIT:ADEV HZ | DBM
        /// UNIT:RDEV DB | PCT
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Absolute_Dev_Unit">
        /// Selects the unit for absolute deviation measurements.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DBM (0) - dBm
        /// RSSPECAN_VAL_UNIT_HZ  (12) - Hz
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_HZ (12)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMS_AF_RANGE_DEV_UNIT_ABSOLUTE
        /// </param>
        /// <param name="Relative_Dev_Unit">
        /// Selects the unit for relative deviation measurements.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DB  (8) - dB
        /// RSSPECAN_VAL_UNIT_PCT (13) - %
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_DB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMS_AF_RANGE_DEV_UNIT_RELATIVE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFMStereoAFRangeDevUnit(int Absolute_Dev_Unit, int Relative_Dev_Unit)
        {
            int pInvokeResult = PInvoke.ConfigureFMStereoAFRangeDevUnit(this._handle, Absolute_Dev_Unit, Relative_Dev_Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activate and configure the zoom function.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_ZOOM
        /// RSSPECAN_ATTR_FMDEM_ZOOM_START
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:ZOOM[:STATe] ON | OFF
        /// [SENSe:]ADEMod:ZOOM:STARt
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Zoom_State">
        /// Turns on or off the zoom function for the trace data of the FM demodulator.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_ZOOM
        /// </param>
        /// <param name="Start_Time">
        /// Selects the start time for the display of the single measured values of the FM demodulator. 
        /// 
        /// Valid Values:
        /// 0.0 to Max s
        /// 
        /// Default Value: 0.0 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_ZOOM_START
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFMStereoTimeDomainZoom(bool Zoom_State, double Start_Time)
        {
            int pInvokeResult = PInvoke.ConfigureFMStereoTimeDomainZoom(this._handle, System.Convert.ToUInt16(Zoom_State), Start_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the zoom length settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_ZOOM_LENGTH_MODE
        /// RSSPECAN_ATTR_FMDEM_ZOOM_LENGTH
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:ZOOM:LENGth:MODE AUTO | MANual
        /// [SENSe:]ADEMod:ZOOM:LENGth
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Zoom_Length_Mode">
        /// Defines whether the length of the zoom area for the analog-demodulated measurement data is defined automatically or manually.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_AUTO (0) - Automatic
        /// RSSPECAN_VAL_MAN  (1) - Manual
        /// 
        /// Default Value: RSSPECAN_VAL_AUTO (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_ZOOM_LENGTH_MODE
        /// </param>
        /// <param name="Zoom_Length">
        /// Defines the length of the zoom area for the analog-demodulated measurement data manually.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0.0000625 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_ZOOM_LENGTH
        /// 
        /// Note(s):
        /// 
        /// (1) This value is only available when Zoom Length Mode is set to Manual.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFMStereoTimeDomainZoomLength(int Zoom_Length_Mode, double Zoom_Length)
        {
            int pInvokeResult = PInvoke.ConfigureFMStereoTimeDomainZoomLength(this._handle, Zoom_Length_Mode, Zoom_Length);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the carrier frequency of the stereo measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_QUERY_FMS_CARRIER_FREQUENCY
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1...4&gt;:MARKer:FUNCtion:SFM[:RESult&lt;1...6&gt;]? FCARrier
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Carrier_Frequency">
        /// Returns the carrier frequency of the stereo measurement.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_QUERY_FMS_CARRIER_FREQUENCY
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryFMStereoCarrierFrequency(out double Carrier_Frequency)
        {
            int pInvokeResult = PInvoke.QueryFMStereoCarrierFrequency(this._handle, out Carrier_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the trace modes of the FM stereo to be measured simultaneously. For each trace a mode can be defined, however only if the specified channel is currently displayed in one of the four screens. If a trace mode is set for a channel that is not displayed, a query error is generated.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1...4&gt;:]SFM:&lt;ChannelType&gt;:AFSPectrum:TYPE
        /// [SENSe&lt;1...4&gt;:]SFM:&lt;ChannelType&gt;:TDOMain:TYPE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Channel_Type">
        /// This control select the channel type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_LEFT  (0) - Left
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_RIGHT (1) - Right
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_MPX   (2) - MPX
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_MONO  (3) - Mono
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_STER  (4) - Stereo
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_RDS   (5) - RDS
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_PILOT (6) - Pilot
        /// 
        /// Default Value: RSSPECAN_VAL_FSM_CHAN_TYPE_LEFT  (0)
        /// </param>
        /// <param name="Measurement_Type">
        /// This parameter selects the measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FMS_AFSP (0) - AF Spectrum
        /// RSSPECAN_VAL_FMS_TDOM (1) - Time Domain
        /// 
        /// Default Value: RSSPECAN_VAL_FMS_AFSP (0)
        /// 
        /// Note(s):
        /// (1) AF Spectrum: Selects the trace modes of the FM stereo AF spectrum to be measured simultaneously.
        /// 
        /// (2) Time Domain: Selects the trace modes of the FM stereo signal to be measured simultaneously in zero span.
        /// </param>
        /// <param name="Trace_Modes">
        /// Selects the trace modes.
        /// 
        /// Valid Values:
        /// array of next values
        /// 6 values are expected. 
        /// 
        /// Values should be:
        /// RSSPECAN_VAL_FMS_RESULT_OFF  (0) - Off
        /// RSSPECAN_VAL_FMS_RESULT_CURR (1) - Current Meas Value
        /// RSSPECAN_VAL_FMS_RESULT_AVG  (2) - Average
        /// RSSPECAN_VAL_FMS_RESULT_MAX  (3) - Maximum
        /// RSSPECAN_VAL_FMS_RESULT_MIN  (4) - Minimum
        /// RSSPECAN_VAL_FMS_RESULT_VIEW (5) - View
        /// 
        /// Default Value: RSSPECAN_VAL_FMS_RESULT_OFF
        /// 
        /// Note(s):
        /// 
        /// (1) Off:
        /// The result type is not used.
        /// 
        /// (2) Current Meas Value:
        /// The current measured values are recorded.
        /// 
        /// (3) Average:
        /// The measurement results are averaged over the given number of measurements.
        /// 
        /// (4) Maximum:
        /// The maximum result over the given number of measurements is obtained.
        /// 
        /// (5) Minimum:
        /// The minimum result over the given number of measurements is obtained.
        /// 
        /// (6) View:
        /// The measurement result is frozen on the display, i.e. it is not recalculated in further measurements.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureFMStereoTraceModes(int Channel_Type, int Measurement_Type, int[] Trace_Modes)
        {
            int pInvokeResult = PInvoke.ConfigureFMStereoTraceModes(this._handle, Channel_Type, Measurement_Type, Trace_Modes);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts the search of the signals required for the difference frequency distortion measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMS_DIFFERENCE_FREQUENCY_DISTORTION_SEARCH_SIGNAL
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1...4&gt;:MARKer:FUNCtion:DFD:SEARchsignal ONCE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Timeout">
        /// Sets the timeout for signal search. If the length of time required for signal search exceeds the timeout value, then the function will return with a timeout error.
        /// 
        /// Valid Values: 0 ms to 600000 ms
        /// 
        /// Default Value: 15000 ms
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FMStereoDifferenceFrequencyDistortionSearchSignal(int Window, uint Timeout)
        {
            int pInvokeResult = PInvoke.FMStereoDifferenceFrequencyDistortionSearchSignal(this._handle, Window, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts the search of the signals required for the intermodulation distortion measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMS_INTERMODULATION_DISTORTION_SEARCH_SIGNAL
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1...4&gt;:MARKer:FUNCtion:IMD:SEARchsignal ONCE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Timeout">
        /// Sets the timeout for signal search. If the length of time required for signal search exceeds the timeout value, then the function will return with a timeout error.
        /// 
        /// Valid Values: 0 ms to 600000 ms
        /// 
        /// Default Value: 15000 ms
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FMStereoIntermodulationDistortionSearchSignal(int Window, uint Timeout)
        {
            int pInvokeResult = PInvoke.FMStereoIntermodulationDistortionSearchSignal(this._handle, Window, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the result of the phase noise measurement in the selected measurement window. The measurement will be switched on, if necessary.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_QUERY_FMS_PHASE_NOISE_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1...4&gt;:MARKer&lt;1...16&gt;:FUNCtion:PNOise:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1..16
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Phase_Noise_Result">
        /// Returns the result of the phase-noise measurement in the selected measurement window. The measurement will be switched on, if necessary.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_QUERY_FMS_PHASE_NOISE_RESULT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryFMStereoPhaseNoiseResult(int Window, int Marker, out double Phase_Noise_Result)
        {
            int pInvokeResult = PInvoke.QueryFMStereoPhaseNoiseResult(this._handle, Window, Marker, out Phase_Noise_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the result of the difference frequency distortion measurement in the specified window.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1...4&gt;:MARKer:FUNCtion:DFD[:RESult&lt;1...6&gt;]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Difference_Frequency_Distortion">
        /// Returns the difference frequency distortion factors of 2nd and 3rd order.
        /// 
        /// Note(s): 
        /// 
        /// (1) The array must contain at least 2 elements.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryFMStereoDifferenceFrequencyDistortionResult(int Window, double[] Difference_Frequency_Distortion)
        {
            int pInvokeResult = PInvoke.QueryFMStereoDifferenceFrequencyDistortionResult(this._handle, Window, Difference_Frequency_Distortion);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the result of the intermodulation distortion measurement in the specified window.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1...4&gt;:MARKer:FUNCtion:IMD[:RESult&lt;1...6&gt;]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Intermodulation_Distortion">
        /// Returns the intermodulation distortion factors of 2nd and 3rd order.
        /// 
        /// Note(s): 
        /// 
        /// (1) The array must contain at least 2 elements.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryFMStereoIntermodulationDistortionResult(int Window, double[] Intermodulation_Distortion)
        {
            int pInvokeResult = PInvoke.QueryFMStereoIntermodulationDistortionResult(this._handle, Window, Intermodulation_Distortion);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the results of the stereo measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1...4&gt;:MARKer:FUNCtion:SFM[:RESult&lt;1...6&gt;]? SUMMary
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Absolute_Deviation">
        /// Returns the absolute deviation values for all channels.
        /// 
        /// Valid Values: 
        /// the channels are in this order:
        /// Left, Right, MPX, Mono, Stereo, RDS, Pilot
        /// 
        /// Note(s):
        /// 
        /// (1) The array must contain at least 7 elements.
        /// 
        /// (2) If one of the result values is not available, RSSPECAN_NAN (9.91E+37) is inserted for the missing value.
        /// 
        /// </param>
        /// <param name="Relative_Deviation">
        /// Returns the relative deviation values for all channels.
        /// 
        /// Valid Values: 
        /// the channels are in this order:
        /// Left, Right, MPX, Mono, Stereo, RDS, Pilot
        /// 
        /// Note(s):
        /// 
        /// (1) The array must contain at least 7 elements.
        /// 
        /// (2) If one of the result values is not available, RSSPECAN_NAN (9.91E+37) is inserted for the missing value.
        /// </param>
        /// <param name="SINAD">
        /// Returns the Signal-to-noise-and-distortion values for all channels.
        /// 
        /// Valid Values: 
        /// the channels are in this order:
        /// Left, Right, MPX, Mono, Stereo, RDS, Pilot
        /// 
        /// Note(s):
        /// 
        /// (1) The array must contain at least 7 elements.
        /// 
        /// (2) If one of the result values is not available, RSSPECAN_NAN (9.91E+37) is inserted for the missing value.
        /// </param>
        /// <param name="THD">
        /// Returns the total harmonic distortion values for all channels.
        /// 
        /// Valid Values: 
        /// the channels are in this order:
        /// Left, Right, MPX, Mono, Stereo, RDS, Pilot
        /// 
        /// Note(s):
        /// 
        /// (1) The array must contain at least 7 elements.
        /// 
        /// (2) If one of the result values is not available, RSSPECAN_NAN (9.91E+37) is inserted for the missing value.
        /// </param>
        /// <param name="Modulation_Frequency">
        /// Returns the modulation frequency values for all channels.
        /// 
        /// Valid Values: 
        /// the channels are in this order:
        /// Left, Right, MPX, Mono, Stereo, RDS, Pilot
        /// 
        /// Note(s):
        /// 
        /// (1) The array must contain at least 7 elements.
        /// 
        /// (2) If one of the result values is not available, RSSPECAN_NAN (9.91E+37) is inserted for the missing value.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryFMStereoMeasurementResults(double[] Absolute_Deviation, double[] Relative_Deviation, double[] SINAD, double[] THD, double[] Modulation_Frequency)
        {
            int pInvokeResult = PInvoke.QueryFMStereoMeasurementResults(this._handle, Absolute_Deviation, Relative_Deviation, SINAD, THD, Modulation_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the results of the measurement type for the selected channel in the specified window.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_QUERY_FMS_CHANNEL_TYPE_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1...4&gt;:MARKer:FUNCtion:SFM:&lt;ChannelType&gt;[:RESult&lt;1...6&gt;]? ADEV | RDEV | SINad | THD | AFRequency
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Channel_Type">
        /// This control select the channel type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_LEFT  (0) - Left
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_RIGHT (1) - Right
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_MPX   (2) - MPX
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_MONO  (3) - Mono
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_STER  (4) - Stereo
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_RDS   (5) - RDS
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_PILOT (6) - Pilot
        /// 
        /// Default Value: RSSPECAN_VAL_FSM_CHAN_TYPE_LEFT  (0)
        /// </param>
        /// <param name="Measurement_Type">
        /// This control selects the measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FMS_MEAS_TYPE_ADEV  (0) - Absolute Deviation
        /// RSSPECAN_VAL_FMS_MEAS_TYPE_RDEV  (1) - Relative Deviation
        /// RSSPECAN_VAL_FMS_MEAS_TYPE_SINAD (2) - SINAD
        /// RSSPECAN_VAL_FMS_MEAS_TYPE_THD   (3) - THD
        /// RSSPECAN_VAL_FMS_MEAS_TYPE_AFR   (4) - Audio Frequency
        /// 
        /// Default Value: RSSPECAN_VAL_FMS_MEAS_TYPE_ADEV (0)
        /// </param>
        /// <param name="Result">
        /// Returns the results of the measurement type for the selected channel in the specified window.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_QUERY_FMS_CHANNEL_TYPE_RESULT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryFMStereoChannelTypeResults(int Window, int Channel_Type, int Measurement_Type, out double Result)
        {
            int pInvokeResult = PInvoke.QueryFMStereoChannelTypeResults(this._handle, Window, Channel_Type, Measurement_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries all available measurement values for the selected channel in the specified window.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1...4&gt;:MARKer:FUNCtion:SFM:&lt;ChannelType&gt;[:RESult&lt;1...6&gt;]? ALL
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// 
        /// Note(s):
        /// 
        /// (1) For future use.
        /// </param>
        /// <param name="Channel_Type">
        /// This control select the channel type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_LEFT  (0) - Left
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_RIGHT (1) - Right
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_MPX   (2) - MPX
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_MONO  (3) - Mono
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_STER  (4) - Stereo
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_RDS   (5) - RDS
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_PILOT (6) - Pilot
        /// 
        /// Default Value: RSSPECAN_VAL_FSM_CHAN_TYPE_LEFT  (0)
        /// </param>
        /// <param name="Result">
        /// Returns all available measurement values for the selected channel in the specified window.
        /// 
        /// Note(s):
        /// 
        /// (1) If all result values are queried, and one of them is not available, RSSPECAN_NAN (9.91E+37) is inserted for the missing value.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryFMStereoChannelTypeAllResults(int Window, int Channel_Type, double[] Result)
        {
            int pInvokeResult = PInvoke.QueryFMStereoChannelTypeAllResults(this._handle, Window, Channel_Type, Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads the result data of the FM stereo signal in the specified trace mode. 
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with option K7S (FM Stereo).
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1...4&gt;:]SFM:&lt;ChannelType&gt;:AFSPectrum:RESult? WRITe | AVERage | MAXHold | MINHold
        /// [SENSe&lt;1...4&gt;:]SFM:&lt;ChannelType&gt;:TDOMain:RESult? WRITe | AVERage | MAXHold | MINHold
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Channel_Type">
        /// This control select the channel type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_LEFT  (0) - Left
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_RIGHT (1) - Right
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_MPX   (2) - MPX
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_MONO  (3) - Mono
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_STER  (4) - Stereo
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_RDS   (5) - RDS
        /// RSSPECAN_VAL_FSM_CHAN_TYPE_PILOT (6) - Pilot
        /// 
        /// Default Value: RSSPECAN_VAL_FSM_CHAN_TYPE_LEFT  (0)
        /// </param>
        /// <param name="Measurement_Type">
        /// This parameter selects the measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FMS_AFSP (0) - AF Spectrum
        /// RSSPECAN_VAL_FMS_TDOM (1) - Time Domain
        /// 
        /// Default Value: RSSPECAN_VAL_FMS_AFSP (0)
        /// 
        /// Note(s):
        /// (1) AF Spectrum: Selects the trace modes of the FM stereo AF spectrum to be measured simultaneously.
        /// 
        /// (2) Time Domain: Selects the trace modes of the FM stereo signal to be measured simultaneously in zero span.
        /// </param>
        /// <param name="Trace_Mode">
        /// Selects the trace mode to be measured in parallel by selected demodulation.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FMS_RESULT_CURR (1) - Current Meas Value
        /// 
        /// Default Value: RSSPECAN_VAL_FMS_RESULT_CURR (1)
        /// 
        /// Note(s):
        /// 
        /// (1) Current Meas Value:
        /// The current measured values are recorded.
        /// 
        /// (2) The trace mode indicated must be one of those configured by Configure FM Stereo Trace Mode. Otherwise a query error will be generated.
        /// 
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Array_Size">
        /// This parameter determines the size of array result values. The Result Values array must contain at least this number of items.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 0
        /// </param>
        /// <param name="Result_Values">
        /// Returns the result values.
        /// 
        /// </param>
        /// <param name="Returned_Values">
        /// Returns the number of returned values.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryFMStereoResultValues(int Channel_Type, int Measurement_Type, int Trace_Mode, uint Timeout, int Array_Size, double[] Result_Values, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.QueryFMStereoResultValues(this._handle, Channel_Type, Measurement_Type, Trace_Mode, Timeout, Array_Size, Result_Values, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the Bluetooth analyzer mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BTO_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument&lt;1|2&gt;:SELect BTOoth
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int BTOMode()
        {
            int pInvokeResult = PInvoke.BTOMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the common parameters of measurement according Bluetooth standard.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BTO_GEOG
        /// RSSPECAN_ATTR_BTO_CHANNEL
        /// RSSPECAN_ATTR_BTO_PCL
        /// RSSPECAN_ATTR_BTO_PRAT
        /// RSSPECAN_ATTR_BTO_PTYPE
        /// RSSPECAN_ATTR_BTO_EGA_INP
        /// 
        /// Remote-control command(s):
        /// CONFigure:BTOoth:GEOGraphy EURope | USA | FRANce
        /// CONFigure:BTOoth:CHANnel
        /// CONFigure:BTOoth:PCLass
        /// CONFigure:BTOoth:PRATe
        /// CONFigure:BTOoth:PTYPe DH1 | DH3 | DH5 | AUTO
        /// [SENSe&lt;1|2&gt;:]CORRection:EGAin:INPut[:MAGNitude]
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Geographical_Region">
        /// Geographical region for the measurements.
        /// 
        /// Valid Values:
        /// 
        /// - RSSPECAN_BTO_GEOG_EUR - sets the region to Europe
        /// 
        /// - RSSPECAN_BTO_GEOG_USA - sets the region to USA
        /// 
        /// - RSSPECAN_BTO_GEOG_FRAN - sets the region to France
        /// 
        /// Default Value: RSPECAN_BTO_GEOG_EUR
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_GEOG
        /// </param>
        /// <param name="Channel">
        /// Frequency channel for the measurement.
        /// 
        /// Valid Values: 
        /// Depends on Geographical Region 
        ///  - Europe - 0 to 78
        ///  - USA - 0 to 78
        ///  - France - 0 to 22
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_CHANNEL
        /// </param>
        /// <param name="Output_Power_Class">
        /// Output power class defines the limits for the Output Power measurement.
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_PCL
        /// </param>
        /// <param name="Samples_Per_Symbols">
        /// Sets the number of measurement samples (points) per symbol for the measurement.
        /// 
        /// Valid Values: 2,4,8,16,32
        /// Default Value: 4
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_PRAT
        /// </param>
        /// <param name="Packet_Type">
        /// Selects packet type to be measured.
        /// 
        /// Valid Values:
        /// 
        /// RSSPECAN_VAL_BTO_PTYPE_AUTO (0) - Selects automatic packet type
        /// 
        /// RSSPECAN_VAL_BTO_PTYPE_DH1  (1) - 1 slot packet
        /// 
        /// RSSPECAN_VAL_BTO_PTYPE_DH3  (2) - 3 slot packet
        /// 
        /// RSSPECAN_VAL_BTO_PTYPE_DH5  (3) - 5 slot packet
        /// 
        /// Default Value: RSSPECAN_VAL_BTO_PTYPE_DH1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_PTYPE
        /// </param>
        /// <param name="Antenna_Gain">
        /// Sets an external gain known to the analyzer, which will take it into account during the display of measurement results.
        /// 
        /// Valid Values: -200.0 to 200.0
        /// 
        /// Default Value: 0.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_EGA_INP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBTOMeasurement(int Geographical_Region, int Channel, int Output_Power_Class, int Samples_Per_Symbols, int Packet_Type, double Antenna_Gain)
        {
            int pInvokeResult = PInvoke.ConfigureBTOMeasurement(this._handle, Geographical_Region, Channel, Output_Power_Class, Samples_Per_Symbols, Packet_Type, Antenna_Gain);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the averaging area of the average power measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BTO_POW_AVER
        /// 
        /// Remote-control command(s):
        /// CONFigure:BTOoth:POWer:AVERage:STARt
        /// CONFigure:BTOoth:POWer:AVERage:STOP
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Start_Position">
        /// Defines the start position for the calculation of the average power of a burst.
        /// 
        /// Default Value: 20
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_POW_AVER
        /// </param>
        /// <param name="Stop_Position">
        /// Defines the end position for the calculation of the average power of a burst.
        /// 
        /// Default Value: 80
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_POW_AVER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBTOAveragingMethod(double Start_Position, double Stop_Position)
        {
            int pInvokeResult = PInvoke.ConfigureBTOAveragingMethod(this._handle, Start_Position, Stop_Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the synchronization of the measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BTO_SEARCH_SYNC
        /// RSSPECAN_ATTR_BTO_SYNC_OFFS
        /// RSSPECAN_ATTR_BTO_SEARCH_PULS
        /// RSSPECAN_ATTR_BTO_PULS_OFFS
        /// RSSPECAN_ATTR_BTO_SEARCH_TIME_AUTO
        /// RSSPECAN_ATTR_BTO_SEARCH_TIME
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:SYNC[:STATe]
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:SYNC:OFFSet
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:PULSe[:STATe]
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:PULSe:OFFSet
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:TIME:AUTO
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:TIME
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Find_Sync">
        /// Switches the search for a sync pattern on or off.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_SEARCH_SYNC
        /// </param>
        /// <param name="Sync_Offset">
        /// Defines the number of bits to be recorded before the first preamble bit is detected.
        /// 
        /// Units: bits
        /// 
        /// Valid Values: 
        /// -10000 to 10000
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_SYNC_OFFS
        /// 
        /// </param>
        /// <param name="Find_Burst">
        /// Switches switches the search for a signal burst on or off.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_SEARCH_PULS
        /// </param>
        /// <param name="Burst_Offset">
        /// Defines the time to be recorded before a signal burst is recognized.
        /// 
        /// Units: s
        /// 
        /// Valid Values: 
        /// -10.0e-3 to 10.0e-3
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_PULS_OFFS
        /// </param>
        /// <param name="Search_Length_Auto">
        /// Activates the automatic setting of the record length for the sync word and burst search, depending on the selected packet type.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_SEARCH_TIME_AUTO
        /// </param>
        /// <param name="Search_Length">
        /// Defines the time to be recorded before a signal burst is recognized.
        /// 
        /// Units: s
        /// 
        /// Valid Values: 
        /// 100.0e-6 to 130560.0e-6/points per symbol
        /// 
        /// Default Value: 1875.0e-6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_SEARCH_TIME
        /// 
        /// Notes:
        /// 
        /// (1) This value is ignored when searchLengthAuto is TRUE.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBTOSynchronization(bool Find_Sync, int Sync_Offset, bool Find_Burst, double Burst_Offset, bool Search_Length_Auto, double Search_Length)
        {
            int pInvokeResult = PInvoke.ConfigureBTOSynchronization(this._handle, System.Convert.ToUInt16(Find_Sync), Sync_Offset, System.Convert.ToUInt16(Find_Burst), Burst_Offset, System.Convert.ToUInt16(Search_Length_Auto), Search_Length);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Lowest 24 Bits of device address for synchronization of the measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BTO_LAP
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:SYNC:LAP
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Lowest_24_Bits_of_Dev__Address">
        /// Sets the lowest 24 bits of Bluetooth Device Address. These bits are used to determine the begin of the signal packet.
        /// 
        /// Valid Values: 0x0000000 to 0xFFFFFF
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_LAP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBTOLAP(int Lowest_24_Bits_of_Dev__Address)
        {
            int pInvokeResult = PInvoke.ConfigureBTOLAP(this._handle, Lowest_24_Bits_of_Dev__Address);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the current measurement according to the Bluetooth standard.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BTO
        /// 
        /// Remote-control command(s):
        /// CONFigure:BTOoth:MEASurement OPOWer | ACLR | MCHar | ICFTolerance | CFDRift | RTPower | IBSemissions | DPENcoding | CFSTability
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement_Mode">
        /// This control selects the current measurement according to the Bluetooth standard.
        /// 
        /// Valid Values:
        /// RSSPECAN_BTO_MEAS_MODE_OPOW  (0) - Output Power
        /// RSSPECAN_BTO_MEAS_MODE_ACLR  (1) - TX Output Spectrum - Adjacent Channel Power
        /// RSSPECAN_BTO_MEAS_MODE_MCH   (2) - Modulation Characteristics
        /// RSSPECAN_BTO_MEAS_MODE_ICFT  (3) - Initial Carrier Frequency Tolerance
        /// RSSPECAN_BTO_MEAS_MODE_CFDR  (4) - Carrier Frequency Drift
        /// RSSPECAN_BTO_MEAS_MODE_RTP   (5) - Relative Transmit Power (EDR)
        /// RSSPECAN_BTO_MEAS_MODE_IBS   (6) - Spurious Emissions (EDR)
        /// RSSPECAN_BTO_MEAS_MODE_DPEN  (7) - Differential Phase Encoding (EDR)
        /// RSSPECAN_BTO_MEAS_MODE_CFST  (8) - Carrier Frequency Stability and Modulation Accuracy (EDR)
        /// 
        /// Default Value: RSSPECAN_BTO_MEAS_MODE_OPOW
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO
        /// 
        /// Note(s):
        /// 
        /// (1) Values RSSPECAN_BTO_MEAS_MODE_RTP to RSSPECAN_BTO_MEAS_MODE_CFST are valid for R&amp;S FSL only
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBTOMeasurementMode(int Measurement_Mode)
        {
            int pInvokeResult = PInvoke.ConfigureBTOMeasurementMode(this._handle, Measurement_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the number of adjacent channel pairs during the measurement of the adjacent channel power. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BTO_ACLR_ACP
        /// 
        /// 
        /// Remote-control command(s):
        /// CONFigure:BTOoth:ACLR:ACPairs
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Number_of_Adjacent_Channels">
        /// Defines number of adjacent channel pairs during the measurement of the adjacent channel power. The number of adjacent channels will be limited as soon as the border of the Bluetooth frequency band is reached.
        /// 
        /// Valid Values: 1 to 78
        /// 
        /// Default Value: 78
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_ACLR_ACP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBTONumberOfAdjacentChannels(int Number_of_Adjacent_Channels)
        {
            int pInvokeResult = PInvoke.ConfigureBTONumberOfAdjacentChannels(this._handle, Number_of_Adjacent_Channels);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the receive filter for the signal to measure.
        /// 
        /// Note:
        /// (1) This function is available only for next measurements
        /// Modulation Characteristics
        /// Initial Carrier Freq Tol
        /// Carrier Freq Drift
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BTO_FILT_MEAS
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:FILTer:MEASurement OFF | BTOoth
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Filter">
        /// Selects the receive filter for the signal to measure.
        /// 
        /// Valid Values:
        /// RSSPECAN_BTO_MEASFILT_OFF (0) - Bluetooth measurement filter off
        /// RSSPECAN_BTO_MEASFILT_BTO (1) - Bluetooth measurement filter on
        /// 
        /// Default Value: RSSPECAN_BTO_MEASFILT_OFF (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_FILT_MEAS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBTOReceiveFilter(int Filter)
        {
            int pInvokeResult = PInvoke.ConfigureBTOReceiveFilter(this._handle, Filter);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates auto coupling of selected coupling method.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BTO_BWID_AUTO
        /// RSSPECAN_ATTR_BTO_VID_AUTO
        /// RSSPECAN_ATTR_BTO_SWE_TIME_AUTO
        /// 
        /// Remote-control command(s):
        /// CONFigure:BTOoth:BANDwidth|BWIDth[:RESolution]:AUTO ON | OFF
        /// CONFigure:BTOoth:BANDwidth|BWIDth:VIDeo:AUTO ON | OFF
        /// CONFigure:BTOoth:SWEep:TIME:AUTO ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Sweep_Coupling">
        /// This control selects coupling type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_COUPLING_RBW        (0) - Resolution Bandwidth
        /// RSSPECAN_VAL_COUPLING_VBW        (1) - Video Bandwidth
        /// RSSPECAN_VAL_COUPLING_SWEEP_TIME (2) - Sweep Time
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_COUPLING_RBW (0)
        /// 
        /// Note(s):
        /// (1) Video Bandwidth is available only for next measurements:
        /// TX Output Spectrum Mode
        /// 
        /// (2) Resolution Bandwidth is available only for next measurements:
        /// Output Power
        /// Modulation Characteristics
        /// Initial Carrier Freq Tol
        /// Carrier Freq Drift
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBTOSweepCouplingAuto(int Sweep_Coupling)
        {
            int pInvokeResult = PInvoke.ConfigureBTOSweepCouplingAuto(this._handle, Sweep_Coupling);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the coupling values of the currently selected Bluetooth measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BTO_BWID
        /// RSSPECAN_ATTR_BTO_VID
        /// RSSPECAN_ATTR_BTO_SWE_TIME
        /// RSSPECAN_ATTR_BTO_BWID_AUTO
        /// RSSPECAN_ATTR_BTO_VID_AUTO
        /// RSSPECAN_ATTR_BTO_SWE_TIME_AUTO
        /// 
        /// Remote-control command(s):
        /// CONFigure:BTOoth:BANDwidth|BWIDth[:RESolution]
        /// CONFigure:BTOoth:BANDwidth|BWIDth:VIDeo
        /// CONFigure:BTOoth:SWEep:TIME
        /// CONFigure:BTOoth:BANDwidth|BWIDth[:RESolution]:AUTO OFF
        /// CONFigure:BTOoth:BANDwidth|BWIDth:VIDeo:AUTO OFF
        /// CONFigure:BTOoth:SWEep:TIME:AUTO OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Sweep_Coupling">
        /// This control selects coupling type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_COUPLING_RBW        (0) - Resolution Bandwidth
        /// RSSPECAN_VAL_COUPLING_VBW        (1) - Video Bandwidth
        /// RSSPECAN_VAL_COUPLING_SWEEP_TIME (2) - Sweep Time
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_COUPLING_RBW (0)
        /// 
        /// Note(s):
        /// (1) Video Bandwidth is available only for next measurements:
        /// TX Output Spectrum Mode
        /// 
        /// (2) Resolution Bandwidth is available only for next measurements:
        /// Output Power
        /// Modulation Characteristics
        /// Initial Carrier Freq Tol
        /// Carrier Freq Drift
        /// 
        /// </param>
        /// <param name="Coupling_Value">
        /// Resolution bandwidth:
        /// Specifies the measurement resolution bandwidth.
        /// 
        /// Video bandwidth:
        /// Specifies the video bandwidth of the post-detection filter.
        /// 
        /// Sweep Time:
        /// The length of time to complete one sweep.
        /// 
        /// Valid Values:
        /// Resolution bandwidth: 300.0e+3 Hz to 10.0e+6 Hz
        /// Video bandwidth: 1.0 Hz to 10.0E6 Hz
        /// Sweep Time: 1.0e-6 s to 16.0E3 s
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_BWID
        /// RSSPECAN_ATTR_BTO_VID
        /// RSSPECAN_ATTR_BTO_SWE_TIME
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBTOSweepCoupling(int Sweep_Coupling, double Coupling_Value)
        {
            int pInvokeResult = PInvoke.ConfigureBTOSweepCoupling(this._handle, Sweep_Coupling, Coupling_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the acquisition attributes of the currently selected Bluetooth measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SWEEP_MODE_CONTINUOUS
        /// RSSPECAN_ATTR_BTO_SWE
        /// 
        /// Remote-control command(s):
        /// INITiate&lt;1|2&gt;:CONTinuous ON | OFF
        /// CONFigure:BTOoth:SWEep:COUNt
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Sweep_Mode_Continuous">
        /// Turns on/off continuous sweeping.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SWEEP_MODE_CONTINUOUS
        /// </param>
        /// <param name="Number_Of_Sweeps">
        /// The number of sweeps to take.
        /// 
        /// Valid Values:
        /// 0 to 32767
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_SWE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBTOAcquisition(bool Sweep_Mode_Continuous, int Number_Of_Sweeps)
        {
            int pInvokeResult = PInvoke.ConfigureBTOAcquisition(this._handle, System.Convert.ToUInt16(Sweep_Mode_Continuous), Number_Of_Sweeps);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the measurement curve for evaluation of the modulation characteristics.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BTO_TRAC
        /// 
        /// Remote-control command(s):
        /// CONFigure:BTOoth:TRACe&lt;1 to 3&gt;:SELect
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Trace">
        /// Sets the trace. The driver uses this value to set the repeated capability 'Trace' of the attributes.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SelectBTOTrace(int Trace)
        {
            int pInvokeResult = PInvoke.SelectBTOTrace(this._handle, Trace);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the averaging area of the average power measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BTO_TRAC_MOD
        /// RSSPECAN_ATTR_BTO_TRAC_DET
        /// 
        /// Remote-control command(s):
        /// CONFigure:BTOoth:TRACe&lt;1 to 3&gt;:MODE WRITe | VIEW | AVERage | BLANk | MAXHold | MINHold
        /// CONFigure:BTOoth:DETector&lt;1 to 3&gt; APEak | NEGative | POSitive | SAMPle | RMS | AVERage
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Trace">
        /// Sets the trace. The driver uses this value to set the repeated capability Trace of the attributes.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Trace_Mode">
        /// Defines the way of displaying and evaluating the measurement curves for the selected Bluetooth measurement.
        /// 
        /// 
        /// Valid Values:
        /// RSSPECAN_TRAC_MOD_WRITE (0) - Write
        /// RSSPECAN_TRAC_MOD_VIEW  (1) - View  
        /// RSSPECAN_TRAC_MOD_AVER  (2) - Average
        /// RSSPECAN_TRAC_MOD_MAXH  (3) - Maximum Hold
        /// RSSPECAN_TRAC_MOD_MINH  (4) - Minimum Hold
        /// RSSPECAN_TRAC_MOD_BLANK (5) - Blank
        ///  
        /// Default Value:  RSSPECAN_TRAC_MOD_WRITE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_TRAC_MOD
        /// </param>
        /// <param name="Trace_Detector">
        /// Selects the detector for the currently selected Bluetooth measurement.
        /// 
        /// 
        /// Valid Values:
        ///   RSSPECAN_BTO_TRAC_DET_APEAK  (0) - Auto Peak
        ///   RSSPECAN_BTO_TRAC_DET_NEG    (1) - Negative Peak  
        ///   RSSPECAN_BTO_TRAC_DET_POS    (2) - Positive Peak
        ///   RSSPECAN_BTO_TRAC_DET_SAMPLE (3) - Sample
        ///   RSSPECAN_BTO_TRAC_DET_RMS    (4) - RMS
        ///   RSSPECAN_BTO_TRAC_DET_AVER   (5) - Average
        ///  
        /// Default Value:  RSSPECAN_BTO_TRAC_DET_APEAK
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_TRAC_DET
        /// 
        /// Note(s):
        /// 
        /// (1) Auto Peak - displays both the positive peak value and the negative peak value when noise is present. The positive peak value is displayed when one signal is present.
        /// 
        /// (2) Negative Peak - sets detector to negative peak.
        /// 
        /// (3) Positive Peak - sets detector to positive peak.
        /// 
        /// (4) Sample  - sets detector to sample
        /// 
        /// (5) RMS -  - sets detector to RMS. This selection is not valid for R&amp;S FSL
        /// 
        /// (6) Average -  - sets detector to average 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBTOTrace(int Trace, int Trace_Mode, int Trace_Detector)
        {
            int pInvokeResult = PInvoke.ConfigureBTOTrace(this._handle, Trace, Trace_Mode, Trace_Detector);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the number of payload bytes that are transmitted in a packet.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_PBSC
        /// 
        /// Remote-control command(s):
        /// CONFigure:BTOoth:PBSCo
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Packet_Bytes_SCO">
        /// Selects the number of payload bytes that are transmitted in a packet.
        /// 
        /// Valid Values:
        ///  1 to 1000
        /// 
        /// Default Value:
        ///  1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_PBSC
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBTOPacketBytesSCO(int Packet_Bytes_SCO)
        {
            int pInvokeResult = PInvoke.ConfigureBTOPacketBytesSCO(this._handle, Packet_Bytes_SCO);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the scaling of the y?axis in the currently selected unit.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BTO_PDIV
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:PDIVision
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Scaling">
        /// Defines the scaling of the Y axis.
        /// 
        /// Valid Values: no checking
        /// 
        /// Default Value: 1.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_PDIV
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBTODisplayYAxisScaling(double Scaling)
        {
            int pInvokeResult = PInvoke.ConfigureBTODisplayYAxisScaling(this._handle, Scaling);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets analog demodulation parameters such as zoom state, start time.
        /// 
        /// Attributes:
        /// RSSPECAN_ATTR_BTO_ZOOM_STATE
        /// RSSPECAN_ATTR_BTO_ZOOM_START
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:ZOOM:STATe ON | OFF
        /// [SENSe:]ADEMod:ZOOM:STARt
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Zoom_State">
        /// Turns on or off the zoom function for the trace data of the FM demodulator.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_ZOOM_STATE
        /// 
        /// Notes:
        /// 
        /// (1) This control is active only if Demodulation Parameter is set to Zoom State.
        /// 
        /// </param>
        /// <param name="Start_Time">
        /// Selects sets the start time for the display of individual measured values.
        /// 
        /// Valid Values:
        /// 0.0 to Max s 
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_ZOOM_START
        /// 
        /// Notes:
        /// (1) Max depends on the measuring time set in the instrument.
        /// 
        /// (2) This control has effect only for Zoom State set On
        /// 
        /// 
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBTOAnalogDemodulationZoom(bool Zoom_State, double Start_Time)
        {
            int pInvokeResult = PInvoke.ConfigureBTOAnalogDemodulationZoom(this._handle, System.Convert.ToUInt16(Zoom_State), Start_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the status of a preceeding measurement.
        /// 
        /// Note(s): 
        /// 
        /// (1) This function is only available, if a measurement was started via INIT:IMMediate before and if this measurement is completed. With a missing or incomplete measurement the function will cause a query error.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BTO_STAT
        /// 
        /// Remote-control command(s):
        /// CALCulate:BTOoth:STATus?
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Status">
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetBTOMesurementStatus(out int Status)
        {
            int pInvokeResult = PInvoke.GetBTOMesurementStatus(this._handle, out Status);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the packet info of of the packet analyzed by the preceeding measurement.
        /// 
        /// Note(s): 
        /// 
        /// (1) This function is only available, if a measurement was started via INIT:IMMediate before and if this measurement is completed. With a missing or incomplete measurement the function will cause a query error.
        /// 
        /// (2) This function is only available with measurement of Output Power ((CONF:BTO:MEAS OPOW), Modulation Characteristics (CONF:BTO:MEAS MCH), Initial Carrier Frequency Tolerance
        /// (CONF:BTO:MEAS ICFT) and Carrier Frequency Drift (CONF:BTO:MEAS CFDR).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BTO_PTYP
        /// RSSPECAN_ATTR_BTO_PLEN
        /// 
        /// Remote-control command(s):
        /// CALCulate:BTOoth:PLENgth?
        /// CALCulate:BTOoth:PTYPe?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Type">
        /// Returns the type of the packet analyzed by a preceeding measurement.The driver uses this output to return the RSSPECAN_ATTR_BTO_PTYP attribute value.
        /// 
        /// Returned Values:
        /// RSSPECAN_BTO_PTYPE_DH1   (1)  - DH1
        /// RSSPECAN_BTO_PTYPE_DH3   (2)  - DH3
        /// RSSPECAN_BTO_PTYPE_DH5   (3)  - DH5
        /// RSSPECAN_BTO_PTYPE_DM1   (4)  - DM1
        /// RSSPECAN_BTO_PTYPE_DM3   (5)  - DM3
        /// RSSPECAN_BTO_PTYPE_DM5   (6)  - DM5
        /// RSSPECAN_BTO_PTYPE_FHS   (7)  - FHS
        /// RSSPECAN_BTO_PTYPE_HV1   (8)  - HV1
        /// RSSPECAN_BTO_PTYPE_HV2   (9)  - HV2
        /// RSSPECAN_BTO_PTYPE_HV3   (10) - HV3
        /// RSSPECAN_BTO_PTYPE_DV    (11) - DV
        /// RSSPECAN_BTO_PTYPE_NULL  (12) - NULL
        /// RSSPECAN_BTO_PTYPE_POLL  (13) - POLL
        /// RSSPECAN_BTO_PTYPE_AUX1  (14) - AUX1
        /// RSSPECAN_BTO_PTYPE_UNDEF (15) - UNDEF
        /// </param>
        /// <param name="Length">
        /// Returns the length of the packet analyzed by the preceeding measurement. The driver uses this output to return the RSSPECAN_ATTR_BTO_PLEN attribute value.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetBTOPacketInfo(out int Type, out int Length)
        {
            int pInvokeResult = PInvoke.GetBTOPacketInfo(this._handle, out Type, out Length);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the output power results.
        /// 
        /// Note(s): 
        /// 
        /// (1) This function is only available with active Output Power Measurement (command CONF:BTO:MEAS OPOW). With all other measurements it will lead to a query error.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BTO_OPOW_PEAK
        /// RSSPECAN_ATTR_BTO_OPOW_AVER
        /// 
        /// 
        /// Remote-control command(s):
        /// CALCulate:BTOoth:OPOWer[:PEAK]?
        /// CALCulate:BTOoth:OPOWer:AVERage? MINimum | MAXimum
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Type">
        /// Sets the output power measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Average Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Average Maximum
        /// RSSPECAN_VAL_MEASTYPE_PEAK (3) - Peak
        /// 
        /// Note(s):
        /// 
        /// (1) Peak - the driver returns value of RSSPECAN_ATTR_BTO_OPOW_PEAK attribute.
        /// 
        /// (2) Average Minimum - the driver returns value of RSSPECAN_ATTR_BTO_OPOW_AVER attribute with repeated capability Min.
        /// 
        /// (3) Average Maximum - the driver returns value of RSSPECAN_ATTR_BTO_OPOW_AVER_MAX with repeated capability Max..
        /// </param>
        /// <param name="Power">
        /// Returns the peak/average value of the Output Power Measurement according to the BLUETOOTH standard. The driver uses this output to return the attribute value according selscted type.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetBTOOutputPower(int Type, out double Power)
        {
            int pInvokeResult = PInvoke.GetBTOOutputPower(this._handle, Type, out Power);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the number of exceptions which occured during the adjacent channel power measurement according to the Bluetooth specification.
        /// 
        /// Note(s): 
        /// 
        /// (1) This function is only available with active Adjacent Channel Power measurement (command CONF:BTO:MEAS ACLR). With all other measurements this function will lead to a query error.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BTO_ACLR_EXC
        /// 
        /// Remote-control command(s):
        /// CALCulate:BTOoth:ACLR:EXCeptions?
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Number_Of_Exceptions">
        /// Returns the number of exceptions which occured during the adjacent channel power measurement. The driver uses this output to return the RSSPECAN_ATTR_BTO_ACLR_EXC attribute value.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetBTONumberOfExceptions(out int Number_Of_Exceptions)
        {
            int pInvokeResult = PInvoke.GetBTONumberOfExceptions(this._handle, out Number_Of_Exceptions);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the power of selected adjacent channels of the Bluetooth standard.
        /// 
        /// Notes:
        /// 
        /// (1) This function returns correct values only if Adjacent Channel Power Measurement is selected with function Configure BTO Measurement Mode (ConfigureBTOMeasurementMode). 
        /// 
        /// Attribute(s):
        /// No attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate:BTOoth:ACLR:LIST?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// This parameter determines the number of item in array list.
        /// 
        /// This value mustn't be great than items allocated in Power List.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Power">
        /// Returns the power of selected channels.
        /// </param>
        /// <param name="Returned_Values">
        /// Returns number of values, that were read from instrument.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetBTOPowerOfChannels(int Array_Size, double[] Power, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.GetBTOPowerOfChannels(this._handle, Array_Size, Power, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the value of frequency deviation for varying bit patterns of the payload.
        /// 
        /// Note(s): 
        /// 
        /// (1) The function is only available with active Modulation Characteristics measurement (command CONF:BTO:MEAS MCH). With all other measurements this function will lead to a query error.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BTO_MCH_DF_MAX
        /// RSSPECAN_ATTR_BTO_MCH_DF_AVER
        /// 
        /// 
        /// Remote-control command(s):
        /// CALCulate:BTOoth:MCHar:DF&lt;1|2&gt;:MAXimum? MINimum | MAXimum | AVERage
        /// CALCulate:BTOoth:MCHar:DF&lt;1|2&gt;:AVERage? MINimum | MAXimum
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Frequency_Deviation">
        /// Sets the frequency. The driver uses this value to set the repeated capability Frequency of the attributes.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Type">
        /// Sets the frequency deviation type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MAXMIN  (0) - Max. Frequency Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAXMAX  (1) - Max. Frequency Maximum
        /// RSSPECAN_VAL_MEASTYPE_MAXAVER (2) - Max. Frequency Average
        /// RSSPECAN_VAL_MEASTYPE_AVERMIN (3) - Aver. Frequency Minimum
        /// RSSPECAN_VAL_MEASTYPE_AVERMAX (4) - Aver. Frequency Maximum
        /// 
        /// Note(s):
        /// 
        /// (1) Max. Frequency Minimum - the driver returns value of RSSPECAN_ATTR_BTO_MCH_DF_MAX attribute with repeated capability Min.
        /// 
        /// (2) Max. Frequency Maximum - the driver returns value of RSSPECAN_ATTR_BTO_MCH_DF_MAX attribute with repeated capability Max.
        /// 
        /// (3) Max. Frequency Average - the driver returns value of RSSPECAN_ATTR_BTO_MCH_DF_MAX attribute with repeated capability Aver.
        /// 
        /// (4) Aver. Frequency Minimum - the driver returns value of RSSPECAN_ATTR_BTO_MCH_DF_AVER attribute with repeated capability Min.
        /// 
        /// (5) Aver. Frequency Maximum - the driver returns value of RSSPECAN_ATTR_BTO_MCH_DF_AVER attribute with repeated capability Max.
        /// 
        /// </param>
        /// <param name="Value">
        /// Returns the peak/average value of the Output Power Measurement according to the BLUETOOTH standard. The driver uses this output to return the attribute value according selected type.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetBTOFrequencyDeviation(int Frequency_Deviation, int Type, out double Value)
        {
            int pInvokeResult = PInvoke.GetBTOFrequencyDeviation(this._handle, Frequency_Deviation, Type, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the value of ratio of the average frequency deviations for varying bit patterns of the payload.
        /// 
        /// Note(s): 
        /// 
        /// (1) The function is only available with active Modulation Characteristics measurement (command CONF:BTO:MEAS MCH). With all other measurements this function will lead to a query error.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BTO_MCH_RAT
        /// 
        /// Remote-control command(s):
        /// CALCulate:BTOoth:MCHar:RATio? MINimum | MAXimum | AVERage 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Type">
        /// Sets the ratio of average frequency deviation type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Note(s):
        /// 
        /// (1) Minimum - the driver returns value of RSSPECAN_ATTR_BTO_MCH_RAT attribute with repeated capability Min.
        /// 
        /// (2) Maximum - the driver returns value of RSSPECAN_ATTR_BTO_MCH_RAT attribute with repeated capability Max.
        /// 
        /// (3) Average - the driver returns value of RSSPECAN_ATTR_BTO_MCH_RAT attribute with repeated capability Aver.
        /// 
        /// 
        /// </param>
        /// <param name="Value">
        /// Returns the peak/average value of the Output Power Measurement according to the BLUETOOTH standard. The driver uses this output to return the attribute value according selscted type.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetBTORatioAverageFrequencyDeviation(int Type, out double Value)
        {
            int pInvokeResult = PInvoke.GetBTORatioAverageFrequencyDeviation(this._handle, Type, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the percentage of measurements of the frequency deviation.
        /// 
        /// Note(s): 
        /// 
        /// (1) The function is only available with active Modulation Characteristics measurement (command CONF:BTO:MEAS MCH). With all other measurements this function will lead to a query error.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BTO_MCH_DF2_PERC
        /// 
        /// Remote-control command(s):
        /// CALCulate:BTOoth:MCHar:DF2:PERCent?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Value">
        /// Returns the percentage of measurements of the frequency deviation.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetBTOPercentageDeviation(out double Value)
        {
            int pInvokeResult = PInvoke.GetBTOPercentageDeviation(this._handle, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the packet data bits.
        /// 
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRACe:DATA? BTOBits
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Maximum_Time__ms_">
        /// Pass the maximum length of time in which to allow the read waveform operation to complete.    
        /// 
        /// If the operation does not complete within this time interval, the function returns the RSSPECAN_ERROR_MAX_TIME_EXCEEDED (0xBFFA2003) error code.  When this occurs, you can call rsspecan_Abort to cancel the read waveform operation and return the oscilloscope to the Idle state.
        /// 
        /// Units:  milliseconds.  
        /// 
        /// Defined Values:
        /// RSSPECAN_VAL_MAX_TIME_INFINITE (-1)             RSSPECAN_VAL_MAX_TIME_IMMEDIATE (0)          
        /// 
        /// Default Value: 5000 (ms)
        /// 
        /// Notes:
        /// 
        /// (1) The Maximum Time parameter applies only to this function.  It has no effect on other timeout parameters or attributes.
        /// 
        /// </param>
        /// <param name="Array_Length">
        /// Number of array points requested.
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Actual_Points">
        /// Number of points actually returned in the Amplitude array and X Array array.
        /// </param>
        /// <param name="Values">
        /// A user allocated buffer into which the trace amplitudes is stored.
        /// 
        /// Units: user specified in RSSPECAN_ATTR_AMPLITUDE_UNITS
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetBTOPacketDataBits(uint Maximum_Time__ms_, int Array_Length, out int Actual_Points, double[] Values)
        {
            int pInvokeResult = PInvoke.GetBTOPacketDataBits(this._handle, Maximum_Time__ms_, Array_Length, out Actual_Points, Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns FM trace with selected oversampling factor (Pointer per Symbol).
        /// 
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRACe:DATA? BTOFm
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Maximum_Time__ms_">
        /// Pass the maximum length of time in which to allow the read waveform operation to complete.    
        /// 
        /// If the operation does not complete within this time interval, the function returns the RSSPECAN_ERROR_MAX_TIME_EXCEEDED (0xBFFA2003) error code.  When this occurs, you can call rsspecan_Abort to cancel the read waveform operation and return the oscilloscope to the Idle state.
        /// 
        /// Units:  milliseconds.  
        /// 
        /// Defined Values:
        /// RSSPECAN_VAL_MAX_TIME_INFINITE (-1)             RSSPECAN_VAL_MAX_TIME_IMMEDIATE (0)          
        /// 
        /// Default Value: 5000 (ms)
        /// 
        /// Notes:
        /// 
        /// (1) The Maximum Time parameter applies only to this function.  It has no effect on other timeout parameters or attributes.
        /// 
        /// </param>
        /// <param name="Array_Length">
        /// Number of array points requested.
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Actual_Points">
        /// Number of points actually returned in the Amplitude array and X Array array.
        /// </param>
        /// <param name="Amplitude">
        /// A user allocated buffer into which the trace amplitudes is stored.
        /// 
        /// Units: user specified in RSSPECAN_ATTR_AMPLITUDE_UNITS
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadBTOFMTrace(uint Maximum_Time__ms_, int Array_Length, out int Actual_Points, double[] Amplitude)
        {
            int pInvokeResult = PInvoke.ReadBTOFMTrace(this._handle, Maximum_Time__ms_, Array_Length, out Actual_Points, Amplitude);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns value of the Initial Carrier Frequency Tolerance.
        /// 
        /// Note(s): 
        /// 
        /// (1) The function is only available with active Initial Carrier Frequency Tolerance measurement (command CONF:BTO:MEAS ICFT). With all other measurements this function will lead to a query error.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BTO_ICFT
        /// 
        /// Remote-control command(s):
        /// CALCulate:BTOoth:ICFTolerance? MINimum | MAXimum | AVERage
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Type">
        /// Sets the type of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Note(s):
        /// 
        /// (1) Minimum - the driver returns value of RSSPECAN_ATTR_BTO_ICFT attribute with repeated capability Min.
        /// 
        /// (2) Maximum - the driver returns value of RSSPECAN_ATTR_BTO_ICFT attribute with repeated capability Max.
        /// 
        /// (3) Average - the driver returns value of RSSPECAN_ATTR_BTO_ICFT attribute with repeated capability Aver.
        /// 
        /// </param>
        /// <param name="Value">
        /// Returns the peak/average value of the Output Power Measurement according to the BLUETOOTH standard. The driver uses this output to return the attribute value according selscted type.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetBTOInitCarrFreqTol(int Type, out double Value)
        {
            int pInvokeResult = PInvoke.GetBTOInitCarrFreqTol(this._handle, Type, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the Carrier Frequency Drift.
        /// 
        /// Note(s):
        /// (1) This function is available only with active Carrier Frequency Drift measurement (command CONF:BTO:MEAS CFDR). With all other measurements this function will lead to a query error.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BTO_CFDR_RATE
        /// RSSPECAN_ATTR_BTO_CFDR_MAX
        /// 
        /// Remote-control command(s):
        /// CALCulate:BTOoth:CFDRift:RATE?
        /// CALCulate:BTOoth:CFDRift[:MAXimum]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Type">
        /// Sets the modifier of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_NONE (0) - Rate
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// 
        /// Note(s):
        /// 
        /// (1) Rate - the driver returns value of RSSPECAN_ATTR_BTO_CFDR_RATE attribute.
        /// 
        /// (2) Maximum - the driver returns value of RSSPECAN_ATTR_BTO_CFDR_MAX attribute.
        /// 
        /// 
        /// </param>
        /// <param name="Value">
        /// Returns the Carrier Frequency Drift value. 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetBTOCarrFreqDrift(int Type, out double Value)
        {
            int pInvokeResult = PInvoke.GetBTOCarrFreqDrift(this._handle, Type, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the number of adjacent channel pairs during the In-band Spurious Emissions measurement. 
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_IBS_ACP
        /// 
        /// Remote-control command(s):
        /// CONFigure:BTOoth:IBSemissions:ACPairs
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Number">
        /// Sets the number of adjacent channels.
        /// 
        /// Valid Values:
        ///  1  to 78
        /// 
        /// Default Value:
        /// 20
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_IBS_ACP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBTOSENumAdjacentChannels(int Number)
        {
            int pInvokeResult = PInvoke.ConfigureBTOSENumAdjacentChannels(this._handle, Number);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function adjusts the gate settings for the In-band Spurious Emissions automatically.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_IBS_GATE_AUTO
        /// 
        /// Remote-control command(s):
        /// CONFigure:BTOoth:IBSemissions:GATE:AUTO
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int AdjustBTOSEGateSetting()
        {
            int pInvokeResult = PInvoke.AdjustBTOSEGateSetting(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the power of the TX channel in the In-band Spurious Emissions measurement..
        /// 
        /// Attributes:
        /// RSSPECAN_ATTR_BTO_IBS_TXCHANNEL
        /// RSSPECAN_ATTR_BTO_IBS_HADJ
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:BTOoth:IBSemissions:TXReference?
        /// CALCulate&lt;1|2&gt;:BTOoth:IBSemissions:HADJacent? UPPer | LOWer
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Parameter">
        /// Selects the measured value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_BTO_SE_TX       (0) - TX Channel Reference
        /// RSSPECAN_VAL_BTO_SE_HADJ_LOW (1) - Frequency Band Lower
        /// RSSPECAN_VAL_BTO_SE_HADJ_UPP (2) - Frequency Band Upper
        /// 
        /// 
        /// Notes:
        /// (1) TX Channel Reference:
        /// Reads the root mean square (RMS) of the differential error vector magnitude (DEVM) in the Carrier Frequency Stability and Modulation Accuracy measurement.
        /// 
        /// (2) Frequency Band Lower:
        /// Reads 99% of the differential error vector magnitude (DEVM) in the Carrier Frequency Stability and Modulation Accuracy measurement.
        /// 
        /// (3) Frequency Band Upper:
        /// Reads the peak of the differential error vector magnitude (DEVM) in the Carrier Frequency Stability and Modulation Accuracy measurement.
        /// 
        /// </param>
        /// <param name="Value">
        /// Returns measured value.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetBTOChannelPowerOfSEMeasurement(int Parameter, out double Value)
        {
            int pInvokeResult = PInvoke.GetBTOChannelPowerOfSEMeasurement(this._handle, Parameter, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the power of all adjacent channels in the In?band Spurious Emissions measurement.
        /// 
        /// Notes:
        /// 
        /// (1) This function returns correct values only if Spurious Emissions (EDR] measurement is selected with function Configure BTO Measurement Mode (ConfigureBTOMeasurementMode). 
        /// 
        /// Attribute(s):
        /// No attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate:BTOoth:IBSemissions:[List]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// This parameter determines the number of item in array list.
        /// 
        /// This value mustn't be great than items allocated in Power List.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Power">
        /// Returns the power of selected channels.
        /// </param>
        /// <param name="Returned_Values">
        /// Returns number of values, that were read from instrument.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetBTOSEPowerOfAdjacentChannels(int Array_Size, double[] Power, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.GetBTOSEPowerOfAdjacentChannels(this._handle, Array_Size, Power, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the number of results that exceeded the specified limits in the In-band Spurious Emissions measurement.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_IBS_EXC
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:BTOoth:IBSemissions:EXCeptions?
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Exceptions">
        /// Returns the number of exceptions.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetBTOSENumberOfExceptions(out int Exceptions)
        {
            int pInvokeResult = PInvoke.GetBTOSENumberOfExceptions(this._handle, out Exceptions);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the number of blocks to be measured.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_CFST_BLOCK_COUNT
        /// 
        /// Remote-control command(s):
        /// CONFigure:BTOoth:CFSTability:BCOunt
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Block_Count">
        /// Sets the number of blocks to be measured.
        /// 
        /// Valid Values:
        ///  0 to 1000000
        /// 
        /// Default Value:
        ///  200
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BTO_CFST_BLOCK_COUNT
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBTOCFSBlockCount(int Block_Count)
        {
            int pInvokeResult = PInvoke.ConfigureBTOCFSBlockCount(this._handle, Block_Count);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns value of the Carrier Frequency Stability Frequency Error.
        /// 
        /// Note(s): 
        /// 
        /// (1) The function is only available with active Initial Carrier Frequency Stability measurement (command CONF:BTO:MEAS CFST). With all other measurements this function will lead to a query error.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BTO_CFST_FERR_INITIAL
        /// RSSPECAN_ATTR_BTO_CFST_FERR_BLOCK
        /// RSSPECAN_ATTR_BTO_CFST_FERR_TOTAL
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:BTOoth:CFSTability:FERRor:INITial? MINimum | MAXimum | AVERage
        /// CALCulate&lt;1|2&gt;:BTOoth:CFSTability:FERRor:BLOCk? MINimum | MAXimum | AVERage
        /// CALCulate&lt;1|2&gt;:BTOoth:CFSTability:FERRor:[TOTal]? MINimum | MAXimum | AVERage
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Parameter">
        /// Selects the measured value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_BTO_CFS_FERR_INITIAL (0) - Initial Frequency Error
        /// RSSPECAN_VAL_BTO_CFS_FERR_BLOCK   (1) - Frequency Error / Block
        /// RSSPECAN_VAL_BTO_CFS_FERR_TOTAL   (2) - Total Frequency Error
        /// 
        /// 
        /// Notes:
        /// (1) Initial Frequency Error:
        /// Reads the combined frequency deviation of all packets and all blocks in the Carrier Frequency Stability and Modulation Accuracy measurement.
        /// 
        /// (2) Frequency Error / Block:
        /// Reads the maximum frequency deviation of all blocks in the Carrier Frequency Stability and Modulation Accuracy measurement.
        /// 
        /// (3) Total Frequency Error:
        /// Reads the carrier frequency deviation of all packets in the Carrier Frequency Stability and Modulation Accuracy measurement.
        /// 
        /// </param>
        /// <param name="Type">
        /// Sets the type of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Note(s):
        /// 
        /// (1) Minimum - the driver returns value of selected attribute with repeated capability Min.
        /// 
        /// (2) Maximum - the driver returns value of selected attribute with repeated capability Max.
        /// 
        /// (3) Average - the driver returns value of selected attribute with repeated capability Aver.
        /// 
        /// </param>
        /// <param name="Value">
        /// Returns the peak/average value of the Output Power Measurement according to the BLUETOOTH standard. The driver uses this output to return the attribute value according selscted type.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetBTOCFSDeviation(int Parameter, int Type, out double Value)
        {
            int pInvokeResult = PInvoke.GetBTOCFSDeviation(this._handle, Parameter, Type, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns value of the Carrier Frequency Stability Modulation Accuracy.
        /// 
        /// Note(s): 
        /// 
        /// (1) The function is only available with active Initial Carrier Frequency Stability measurement (command CONF:BTO:MEAS CFST). With all other measurements this function will lead to a query error.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BTO_CFST_DEVM_RMS
        /// RSSPECAN_ATTR_BTO_CFST_DEVM_99PCT
        /// RSSPECAN_ATTR_BTO_CFST_DEVM_PEAK
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:BTOoth:CFSTability:DEVM:[RMS]? MINimum | MAXimum | AVERage
        /// CALCulate&lt;1|2&gt;:BTOoth:CFSTability:DEVM:D99Pct?
        /// CALCulate&lt;1|2&gt;:BTOoth:CFSTability:DEVM:PEAK?
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Parameter">
        /// Selects the measured value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_BTO_CFS_DEVM_RMS   (0) - RMS of Differential EVM
        /// RSSPECAN_VAL_BTO_CFS_DEVM_99PCT (1) - 99% of Differential EVM
        /// RSSPECAN_VAL_BTO_CFS_DEVM_PEAK  (2) - Peak of Differential EVM
        /// 
        /// 
        /// Notes:
        /// (1) RMS of Differential EVM:
        /// Reads the root mean square (RMS) of the differential error vector magnitude (DEVM) in the Carrier Frequency Stability and Modulation Accuracy measurement.
        /// 
        /// (2) 99% of Differential EVM:
        /// Reads 99% of the differential error vector magnitude (DEVM) in the Carrier Frequency Stability and Modulation Accuracy measurement.
        /// 
        /// (3) Peak of Differential EVM:
        /// Reads the peak of the differential error vector magnitude (DEVM) in the Carrier Frequency Stability and Modulation Accuracy measurement.
        /// </param>
        /// <param name="Type">
        /// Sets the type of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Note(s):
        /// 
        /// (1) Minimum - the driver returns value of selected attribute with repeated capability Min.
        /// 
        /// (2) Maximum - the driver returns value of selected attribute with repeated capability Max.
        /// 
        /// (3) Average - the driver returns value of selected attribute with repeated capability Aver.
        /// 
        /// (4) This control has effect only for RMS of Differential EVM
        /// 
        /// </param>
        /// <param name="Value">
        /// Returns the peak/average value of the Output Power Measurement according to the BLUETOOTH standard. The driver uses this output to return the attribute value according selscted type.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetBTOCFSModulAccuracy(int Parameter, int Type, out double Value)
        {
            int pInvokeResult = PInvoke.GetBTOCFSModulAccuracy(this._handle, Parameter, Type, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the measured valeus of the differential phase encoding.
        /// 
        /// 
        /// Attributes:
        /// RSSPECAN_ATTR_BTO_DPEN_PACKETS
        /// RSSPECAN_ATTR_BTO_DPEN_BER
        /// RSSPECAN_ATTR_BTO_DPEN_PACKETS_PASSED
        /// 
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:BTOoth:DPENcoding:[TOTal]?
        /// CALCulate&lt;1|2&gt;:BTOoth:DPENcoding:BER?
        /// CALCulate&lt;1|2&gt;:BTOoth:DPENcoding:NERRor?
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Parameter">
        /// Selects the measured value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_BTO_DPEN_TESTED (0) - Number of Tested Packets
        /// RSSPECAN_VAL_BTO_DPEN_BER    (1) - Bit Error Rate
        /// RSSPECAN_VAL_BTO_DPEN_PASSED (2) - Number of Passed Packets
        /// 
        /// 
        /// Notes:
        /// (1) Number of Tested Packets:
        /// Reads the number of tested packets in the Differential Phase Encoding measurement.
        /// 
        /// (2) Bit Error Rate:
        /// Reads the bit error rate (BER) in the Differential Phase Encoding measurement.
        /// 
        /// (3) Number of Passed Packets:
        /// Reads reads the number of passed packets in the Differential Phase Encoding measurement.
        /// </param>
        /// <param name="Value">
        /// Returns measured value.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetBTODifferentialPhaseEncodingValues(int Parameter, out int Value)
        {
            int pInvokeResult = PInvoke.GetBTODifferentialPhaseEncodingValues(this._handle, Parameter, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configurest the start and stop time for power measurement on slected packet sections.
        /// 
        /// Attributes:
        /// RSSPECAN_ATTR_BTO_RTP_TIME
        /// 
        /// Remote-control command(s):
        /// CONFigure:BTOoth:RTPower:DAVerage:STARt
        /// CONFigure:BTOoth:RTPower:DAVerage:STOPt
        /// CONFigure:BTOoth:RTPower:GAVerage:STARt
        /// CONFigure:BTOoth:RTPower:GAVerage:STOPt
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Packet_Section">
        /// This control selects the packet section.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_BTO_RTP_GFSK (0) - GFSK
        /// RSSPECAN_VAL_BTO_RTP_DPSK (1) - DPSK
        /// 
        /// Default Value: RSSPECAN_VAL_BTO_RTP_GFSK (0)
        /// </param>
        /// <param name="Start">
        /// Sets the start time for power measurement of the selected packet sections.
        /// 
        /// Valid Values:
        ///  0 to 100%
        /// 
        /// Default Value:
        ///  10
        /// 
        /// </param>
        /// <param name="Stop">
        /// Sets the stop time for power measurement of the selected packet sections.
        /// 
        /// Valid Values:
        ///  0 to 100%
        /// 
        /// Default Value:
        ///  90
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBTORTPMeasurement(int Packet_Section, double Start, double Stop)
        {
            int pInvokeResult = PInvoke.ConfigureBTORTPMeasurement(this._handle, Packet_Section, Start, Stop);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the transmission power of the Relative Transmission Power measurement.
        /// 
        /// Attributes:
        /// RSSPECAN_ATTR_BTO_RTP
        /// RSSPECAN_ATTR_BTO_RTP_RATIO
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:BTOoth:RTPower:[DPSK]? MINimum | MAXimum | AVERage
        /// CALCulate&lt;1|2&gt;:BTOoth:RTPower:GFSK? MINimum | MAXimum | AVERage
        /// CALCulate&lt;1|2&gt;:BTOoth:RTPower:RATio? MINimum | MAXimum | AVERage
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Parameter">
        /// Selects the measured value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_BTO_RTP_DPSK  (0) - Average TP for DPSK
        /// RSSPECAN_VAL_BTO_RTP_GFSK  (1) - Transmission Power for GFSK
        /// RSSPECAN_VAL_BTO_RTP_RATIO (2) - Ratio TP for DPSK and GFSK
        /// 
        /// 
        /// Notes:
        /// (1) Average TP for DPSK:
        /// Reads the average transmission power for the DPSK sections of the packet in the Relative Transmission Power measurement. With a sweep count &lt;= 1, the command returns the current value, irrespective of the parameter. 
        /// 
        /// (2) Transmission Power for GFSK:
        /// Reads the transmission power for the GFSK sections of the packet in the Relative Transmission Power measurement. With a sweep count &lt;= 1, the command returns thecurrent value, irrespective of the parameter.
        /// 
        /// (3) Ratio TP for DPSK and GFSK:
        /// Reads the ratio of the transmission power for the GFSK and DPSK modulation in the Relative Transmission Power measurement (PDPSK/PGFSK). With a sweep count &lt;= 1, the command returns the current value, irrespective of the parameter.
        /// </param>
        /// <param name="Modificator">
        /// Selects the modificator of the command.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MIN  (1)
        /// 
        /// </param>
        /// <param name="Value">
        /// Returns power value.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetBTORelativeTransmissionPower(int Parameter, int Modificator, out double Value)
        {
            int pInvokeResult = PInvoke.GetBTORelativeTransmissionPower(this._handle, Parameter, Modificator, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches measurements with a power sensor on or off.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PMET_STATE
        /// RSSPECAN_ATTR_PMET_MODE
        /// RSSPECAN_ATTR_SAN_MODE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]PMETer[:STATe] ON | OFF
        /// INSTrument[:SELect] PM
        /// INSTrument[:SELect] SAN
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// Sets the measurements with a power sensor on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On 
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PMET_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int PWMMode(int Window, bool State)
        {
            int pInvokeResult = PInvoke.PWMMode(this._handle, Window, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the power meter measurement parameters.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PMET_MEAS_TIME
        /// RSSPECAN_ATTR_PMET_AVERAGE_AUTO
        /// RSSPECAN_ATTR_PMET_AVERAGE_COUNT
        /// RSSPECAN_ATTR_PMET_COUPLING
        /// RSSPECAN_ATTR_PMET_FREQ
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]PMETer:MTIMe SHORt | NORMal | LONG
        /// [SENSe&lt;1|2&gt;:]PMETer:MTIMe:AVERage[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]PMETer:MTIMe:AVERage:COUNt
        /// [SENSe&lt;1|2&gt;:]PMETer:FREQuency:LINK CENTer | MARKer1 | OFF
        /// [SENSe&lt;1|2&gt;:]PMETer:FREQuency
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Measurement_Time">
        /// Sets the measurement time of the power sensor. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_PWMET_MEASTIME_NORM   (0) - Normal
        /// RSSPECAN_VAL_PWMET_MEASTIME_SHORT  (1) - Short
        /// RSSPECAN_VAL_PWMET_MEASTIME_LONG   (2) - Long
        /// RSSPECAN_VAL_PWMET_MEASTIME_MANUAL (3) - Manual
        /// 
        /// Default Value: RSSPECAN_VAL_PWMET_MEASTIME_NORM   (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PMET_MEAS_TIME
        /// </param>
        /// <param name="Coupling">
        /// Sets the coupling for the frequency of the power sensor.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_PMET_COUP_OFF  (0) - Off (Default Value)
        /// RSSPECAN_VAL_PMET_COUP_CENT (1) - Center
        /// RSSPECAN_VAL_PMET_COUP_MARK (2) - Marker 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PMET_COUPLING
        /// 
        /// Note:
        /// (1) Off - couples the frequency to the center frequency of the analyzer
        /// Center - couples the frequency to the center frequency of the analyzer
        /// Marker 1 - couples the frequency to the position of marker 1
        /// 
        /// </param>
        /// <param name="Frequency">
        /// Sets the frequency of the power sensor.
        /// 
        /// Valid Values:
        /// fmin to fmax - no range checking
        /// 
        /// Default Value: 50.0e+6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PMET_FREQ
        /// 
        /// Note:
        /// (1) This value is valid only if coupling is set off.
        /// 
        /// </param>
        /// <param name="Average_Count">
        /// Sets the number of averages for the manual average mode of the power meter.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PMET_AVERAGE_COUNT
        /// 
        /// Note:
        /// (1) This value is valid only if Measurement Time is set to Manual.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePWMMeasurement(int Window, int Measurement_Time, int Coupling, double Frequency, int Average_Count)
        {
            int pInvokeResult = PInvoke.ConfigurePWMMeasurement(this._handle, Window, Measurement_Time, Coupling, Frequency, Average_Count);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the relative measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PMET_RELATIVE
        /// RSSPECAN_ATTR_PMET_REL
        /// RSSPECAN_ATTR_PMET_UNIT_REL
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PMETer:RELative:STATe ON
        /// CALCulate&lt;1|2&gt;:PMETer:RELative[:MAGNitude]
        /// UNIT&lt;1|2&gt;:PMETer:POWer:RATio DB | PCT
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Reference_Value">
        /// Sets the reference value for relative measurements.
        /// 
        /// Valid Values:
        /// -200.0 to 200.0
        /// 
        /// Default Value: 0.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PMET_REL
        /// </param>
        /// <param name="Unit">
        /// Sets the unit for relative measurements with a power sensor.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DB  (8)  - dB (Default Value)
        /// RSSPECAN_VAL_UNIT_PCT (13) - % 
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PMET_RELATIVE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePWMRelative(int Window, double Reference_Value, int Unit)
        {
            int pInvokeResult = PInvoke.ConfigurePWMRelative(this._handle, Window, Reference_Value, Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the relative measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PMET_RELATIVE
        /// RSSPECAN_ATTR_PMET_UNIT_ABS
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PMETer:RELative:STATe OFF
        /// UNIT&lt;1|2&gt;:PMETer:POWer DBM | W | DB | VSWR
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Unit">
        /// Sets the unit for absolute measurements with a power sensor.
        /// 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DBM     (0) - dBm (Default Value)
        /// RSSPECAN_VAL_UNIT_WATT    (2) - W
        /// RSSPECAN_VAL_ABSUNIT_DB   (3) - dB (Return Loss)
        /// RSSPECAN_VAL_ABSUNIT_VSWR (4) - VSWR
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PMET_UNIT_ABS
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePWMAbsolute(int Window, int Unit)
        {
            int pInvokeResult = PInvoke.ConfigurePWMAbsolute(this._handle, Window, Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines whether the reference level offset set for the analyzer is taken into account for the measured power or not.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PMET_REF_LEVEL_OFFSET_STATE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]PMETer:ROFFset[:STATe]
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// Switches using reference level offset on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PMET_REF_LEVEL_OFFSET_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePWMReferenceLevelOffsetState(int Window, bool State)
        {
            int pInvokeResult = PInvoke.ConfigurePWMReferenceLevelOffsetState(this._handle, Window, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the external power trigger settings. 
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available in conjunction with a NRP-Z81 power sensor.
        /// 
        /// (2) This function is only available for R&amp;S ESL and R&amp;S FSV and R&amp;S FSL.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PWM_EXTERNAL_POWER_TRIGGER_STATE
        /// RSSPECAN_ATTR_PWR_EXTERNAL_POWER_TRIGGER_LEVEL
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]PMETer:TRIGger[:STATe]
        /// [SENSe&lt;1|2&gt;:]PMETer:TRIGger:LEVel
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// Switches the external power trigger on or off. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PWM_EXTERNAL_POWER_TRIGGER_STATE
        /// </param>
        /// <param name="Level">
        /// Sets the power level of the external power trigger.
        /// 
        /// Valid Values:
        /// -20.0 to 20.0
        /// 
        /// Default Value: -20.0 dBm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PWR_EXTERNAL_POWER_TRIGGER_LEVEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePWMExternalPowerTrigger(int Window, bool State, double Level)
        {
            int pInvokeResult = PInvoke.ConfigurePWMExternalPowerTrigger(this._handle, Window, System.Convert.ToUInt16(State), Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches bargraph display on or off for measurements with a power sensor. If bargraph display is switched off, the measurement value appears in the Marker Info field.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only in conjunction with option K9.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PMET_BARGRAPH_STATE
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:PMETer:STATe ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Bar_Graph_View">
        /// Switches bargraph display on or off for measurements with a power sensor. If bargraph display is switched off, the measurement value appears in the Marker Info field.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PMET_BARGRAPH_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePWMBarGraphView(bool Bar_Graph_View)
        {
            int pInvokeResult = PInvoke.ConfigurePWMBarGraphView(this._handle, System.Convert.ToUInt16(Bar_Graph_View));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the duty cycle correction.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only in conjunction with option K9.
        /// 
        /// (2) This function is available only in R&amp;S FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PMET_DUTY_CYCLE_STATE
        /// RSSPECAN_ATTR_PMET_DUTY_CYCLE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1?4&gt;:]PMETer:DCYCle:STATe ON | OFF
        /// [SENSe&lt;1?4&gt;:]PMETer:DCYCle:VALue
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Duty_Cycle">
        /// Configures the duty cycle correction.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PMET_DUTY_CYCLE_STATE
        /// </param>
        /// <param name="Duty_Cycle_Value">
        /// Sets the duty cycle to a percent value for the correction of pulse modulated signals. With the correction activated, the sensor calculates the signal pulse power from this value and the mean power.
        /// 
        /// Valid Values: 
        /// 0.001 to 99.999 %
        /// 
        /// Default Value: 99.999 %
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PMET_DUTY_CYCLE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePWMDutyCycle(bool Duty_Cycle, double Duty_Cycle_Value)
        {
            int pInvokeResult = PInvoke.ConfigurePWMDutyCycle(this._handle, System.Convert.ToUInt16(Duty_Cycle), Duty_Cycle_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the absolute measurement unit.
        /// 
        /// Note(s):
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments.
        /// 
        /// (2) Listed remote command take effect only if the power sensors R&amp;S FSH-Z1, R&amp;S FSH-Z18, R&amp;S FSH-Z14 or R&amp;S FSH-Z44 are installed.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PMET_UNIT_ABS
        /// 
        /// Remote-control command(s):
        /// UNIT&lt;1|2&gt;:PMETer:POWer DBM | W | DB | VSWR
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Unit">
        /// Sets the unit for absolute measurements with a power sensor.
        /// 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DBM     (0) - dBm (Default Value)
        /// RSSPECAN_VAL_UNIT_WATT    (2) - W
        /// RSSPECAN_VAL_ABSUNIT_DB   (3) - dB (Return Loss)
        /// RSSPECAN_VAL_ABSUNIT_VSWR (4) - VSWR
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PMET_UNIT_ABS
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePWMAbsoluteUnit(int Window, int Unit)
        {
            int pInvokeResult = PInvoke.ConfigurePWMAbsoluteUnit(this._handle, Window, Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the forward power display of the power sensor.
        /// 
        /// Note(s):
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments.
        /// 
        /// (2) Listed remote command take effect only if the power sensors R&amp;S FSH-Z1, R&amp;S FSH-Z18, R&amp;S FSH-Z14 or R&amp;S FSH-Z44 are installed.
        /// 
        /// Attribute(s): 
        /// RSSPECAN_ATTR_PMET_DISPLAY
        /// 
        /// Remote-control command(s):
        /// [SENSe:]PMETer:DETector[:FUNCtion]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Power_Display">
        /// This control selects the forward power display of the power sensor.
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_POWERSENSOR_PWR_DISP_AVER (0) - Average Power
        /// RSSPECAN_VAL_POWERSENSOR_PWR_DISP_PENV (1) - Peak Envelope Power
        /// 
        /// Default Value: RSSPECAN_VAL_POWERSENSOR_PWR_DISP_AVER (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_POWER_SNESOR_POWER_DISPLAY
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsfsh_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigurePowerSensorPowerDisplay(int Power_Display)
        {
            int pInvokeResult = PInvoke.ConfigurePowerSensorPowerDisplay(this._handle, Power_Display);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the standard used for the power sensor R&amp;S FSH-Z44 measurements.
        /// 
        /// Note(s):
        /// (1) This function is not compatible with FSH4 &amp; FSH8 instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PMET_STANDARD
        /// 
        /// Remote-control command(s):
        /// CALCulate:PMETer:PRESet:SELect
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Channel_Power_Standard">
        /// This control defines the standard used for the power sensor R&amp;S FSH-Z44 measurements.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_PMET_STD_USER   (0) - User
        /// RSSPECAN_VAL_PMET_STD_GSM    (1) - GSM
        /// RSSPECAN_VAL_PMET_STD_EDGE   (2) - EDGE
        /// RSSPECAN_VAL_PMET_STD_WCDMA  (3) - 3GPP WCDMA
        /// RSSPECAN_VAL_PMET_STD_CDMA1  (4) - cdmaOne
        /// RSSPECAN_VAL_PMET_STD_CDMA2K (5) - cdma2000 2x
        /// RSSPECAN_VAL_PMET_STD_DVBT   (6) - DBV-T
        /// RSSPECAN_VAL_PMET_STD_DAB    (7) - DAB
        /// RSSPECAN_VAL_PMET_STD_TETRA  (8) - Tetra
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_PMET_STD_GSM    (1)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsfsh_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigurePowerSensorStandard(int Channel_Power_Standard)
        {
            int pInvokeResult = PInvoke.ConfigurePowerSensorStandard(this._handle, Channel_Power_Standard);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates and deactivates the usage of a standard for power sensor measurements.
        /// 
        /// Note(s):
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments.
        /// 
        /// (2) Listed remote command take effect only if the power sensors R&amp;S FSH-Z1, R&amp;S FSH-Z18, R&amp;S FSH-Z14 or R&amp;S FSH-Z44 are installed.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PMET_USE_STANDARD
        /// 
        /// Remote-control command(s):
        /// CALCulate:PMETer:PRESet[:STATe]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="State">
        /// This control activates and deactivates the usage of a standard for power sensor measurements.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value:
        /// VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsfsh_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigurePowerSensorUseStandard(bool State)
        {
            int pInvokeResult = PInvoke.ConfigurePowerSensorUseStandard(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function accepts the current result as a reference value for relative measurements.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PMET_REL_AUTO
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PMETer:RELative[:MAGNitude]:AUTO ONCE
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePWMMeasToRef(int Window)
        {
            int pInvokeResult = PInvoke.ConfigurePWMMeasToRef(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function zeroes the power sensor.
        /// 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PMET_ZERO
        /// 
        /// Remote-control command(s):
        /// CALibration:PMETer:ZERO:AUTO ONCE
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePWMZero()
        {
            int pInvokeResult = PInvoke.ConfigurePWMZero(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the offset of the reference level for relative measurements.
        /// 
        /// Note(s):
        /// (1) This function is available for R&amp;S FSH4, FSH8 instruments.
        /// 
        /// (2) Listed remote command take effect only if the power sensors R&amp;S FSH-Z1, R&amp;S FSH-Z18, R&amp;S FSH-Z14 or R&amp;S FSH-Z44 are installed.
        /// 
        /// Attribute(s):RSSPECAN_ATTR_PWM_RELATIVE_OFFSET
        /// 
        /// Remote-control command(s):
        /// CALCulate:PMETer:RELative[:MAGNitude]:OFFSet
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Offset">
        /// Defines the offset of the reference level for relative measurements.
        /// 
        /// Valid Range: -100 to 100
        /// 
        /// Default Value: 0
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsfsh_error_message function.
        ///           
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// BFFC0002  Parameter 2 (Resolution Bandwidth) out of range.
        /// BFFC0003  Parameter 3 (Video Bandwidth) out of range.
        /// 
        /// BFFC09F0  Unknown Error.
        /// BFFC09F1  Syntax Error.
        /// BFFC09F2  Execution Error.
        /// BFFC09F3  Dataset Storage Full.
        /// BFFC09F4  Not Allowed.
        /// BFFC09F5  Out of Range.
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        ///           
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// </returns>
        public int ConfigurePowerSensorReferenceLevelOffset(double Offset)
        {
            int pInvokeResult = PInvoke.ConfigurePowerSensorReferenceLevelOffset(this._handle, Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function triggers a measurement with the power sensor and then outputs the result.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PMET_READ
        /// 
        /// Remote-control command(s):
        /// READ&lt;1|2&gt;:PMETer?
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Result">
        /// Returns the result. The driver uses this value to get the RSSPECAN_ATTR_PMET_READ attribute.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadPWMResult(int Window, uint Timeout, out double Result)
        {
            int pInvokeResult = PInvoke.ReadPWMResult(this._handle, Window, Timeout, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function outputs the result of the power sensor.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PMET_FETCH
        /// 
        /// Remote-control command(s):
        /// FETCh&lt;1|2&gt;:PMETer?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Result">
        /// Returns the result. The driver uses this value to get the RSSPECAN_ATTR_PMET_FETCH attribute.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchPWMResult(int Window, out double Result)
        {
            int pInvokeResult = PInvoke.FetchPWMResult(this._handle, Window, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the GSM/EDGE analyzer for mobile and base station tests.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_K10_MODE
        /// RSSPECAN_ATTR_GSM_K10_MODE_FSV
        /// 
        /// Remote-control command(s):
        /// INSTrument[:SELect] GSM
        /// INSTrument[:SELect] MGSM
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GSMK10Mode()
        {
            int pInvokeResult = PInvoke.GSMK10Mode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the type of device to be measured.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_K10_DUT
        /// 
        /// Remote-control command(s):
        /// CONFigure[:MS]:DEVice:TYPE BTSNormal | BTSMicro | BTSPico | MSNormal | MSSMall 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Device_Under_Test">
        /// This control specifies the type of device to be measured.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_K10_BTS_NORMAL (0) - BTS Normal
        /// RSSPECAN_VAL_GSM_K10_BTS_MICRO  (1) - BTS Micro 
        /// RSSPECAN_VAL_GSM_K10_BTS_PICO   (2) - BTS Pico 
        /// RSSPECAN_VAL_GSM_K10_MS_NORMAL  (3) - MS Normal
        /// RSSPECAN_VAL_GSM_K10_MS_SMALL   (4) - MS Small 
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_K10_BTS_NORMAL (0)
        /// 
        /// Note(s):
        /// 
        /// (1) BTS Normal: BTS, TRX power class Normal 
        /// 
        /// (2) BTS Micro: BTS, TRX power class Micro 
        /// 
        /// (3) BTS Pico: BTS, TRX power class Pico 
        /// 
        /// (4) MS Normal: MS, normal type 
        /// 
        /// (5) MS Small: MS, small type 
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_K10_DUT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGSMK10DUT(int Device_Under_Test)
        {
            int pInvokeResult = PInvoke.ConfigureGSMK10DUT(this._handle, Device_Under_Test);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the signal characteristics. To set frequency directly in Hz instead of absolute RF channel number, please use function rsspecan_ConfigureFrequencyCenter.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_K10_SIG_CHAR_MODE
        /// RSSPECAN_ATTR_GSM_K10_SIG_CHAR_BAND
        /// RSSPECAN_ATTR_GSM_K10_SIG_CHAR_BAND_ARFC
        /// 
        /// Remote-control command(s):
        /// CONFigure[:MS]:NETWork[:TYPE] PGSM | EGSM | DCS | PCS | TGSM | RGSM | GSM 
        /// CONFigure[:MS]:NETWork:FREQuency[:BAND] 380 | 410 | 450 | 480 | 710 | 750 | 810 | 850 | 900 | 1800 | 1900  
        /// CONFigure[:MS]:ARFCn
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Band_Class">
        /// Sets the frequency band.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_K10_FREQ_BAND_TGSM_380 (0) - T-GSM 380
        /// RSSPECAN_VAL_GSM_K10_FREQ_BAND_TGSM_410 (1) - T-GSM 410
        /// RSSPECAN_VAL_GSM_K10_FREQ_BAND_GSM_450  (2) - GSM 450 
        /// RSSPECAN_VAL_GSM_K10_FREQ_BAND_GSM_480  (3) - GSM 480 
        /// RSSPECAN_VAL_GSM_K10_FREQ_BAND_GSM_710  (4) - GSM 710 
        /// RSSPECAN_VAL_GSM_K10_FREQ_BAND_GSM_750  (5) - GSM 750 
        /// RSSPECAN_VAL_GSM_K10_FREQ_BAND_TGSM_810 (6) - T-GSM 810
        /// RSSPECAN_VAL_GSM_K10_FREQ_BAND_GSM_850  (7) - GSM 850 
        /// RSSPECAN_VAL_GSM_K10_FREQ_BAND_PGSM_900 (8) - P-GSM 900
        /// RSSPECAN_VAL_GSM_K10_FREQ_BAND_EGSM_900 (9) - E-GSM 900
        /// RSSPECAN_VAL_GSM_K10_FREQ_BAND_RGSM_900 (10) - R-GSM 900
        /// RSSPECAN_VAL_GSM_K10_FREQ_BAND_TGSM_900 (11) - T-GSM 900
        /// RSSPECAN_VAL_GSM_K10_FREQ_BAND_DCS_1800 (12) - DCS 1800 
        /// RSSPECAN_VAL_GSM_K10_FREQ_BAND_PCS_1900 (13) - PCS 1900 
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_K10_FREQ_BAND_PGSM_900 (8)
        /// 
        /// Note(s):
        /// 
        /// (1) 
        /// Value - Description:
        ///  
        /// PGSM  - Primary GSM 
        /// EGSM  - Extended GSM 
        /// DCS   - DCS 
        /// PCS   - PCS 
        /// TGSM  - T-GSM 
        /// RGSM  - Railway GSM 
        /// GSM   - GSM
        /// 
        /// Attributes:
        /// RSSPECAN_ATTR_GSM_K10_SIG_CHAR_MODE
        /// RSSPECAN_ATTR_GSM_K10_SIG_CHAR_BAND
        /// </param>
        /// <param name="ARFCN">
        /// Sets the Absolute Radio Frequency Channel Number. Setting the ARFCN will update the Frequency. Some values may not be allowed depending on the selected frequency band. To set frequency directly in Hz instead of absolute RF channel number, please use function rsspecan_ConfigureFrequencyCenter.
        /// 
        /// Valid Values: 
        /// 0 to 1023
        /// 
        /// Default Value: None
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_K10_SIG_CHAR_BAND_ARFC
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGSMK10SignalCharacteristics(int Band_Class, int ARFCN)
        {
            int pInvokeResult = PInvoke.ConfigureGSMK10SignalCharacteristics(this._handle, Band_Class, ARFCN);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the level settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_REFERENCE_LEVEL
        /// RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET
        /// RSSPECAN_ATTR_GSM_K10_LEVEL_SETTINGS_PWR_CLASS
        /// RSSPECAN_ATTR_GSM_K10_LEVEL_SETTINGS_POW_STAT
        /// RSSPECAN_ATTR_GSM_K10_AUTO_PWR_LEV
        /// RSSPECAN_ATTR_GSM_K10_AUTO_PWR_LEV_ONCE
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:RLEVel
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:RLEVel:OFFSet
        /// CONFigure[:MS]:POWer:CLASs 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | E1 | E2 | E3 | M1 | M2 | M3 | P1 
        /// CONFigure[:MS]:POWer:STATic
        /// CONFigure[:MS]:POWer:AUTO ON | OFF | ONCE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Automatic_Reference_Level">
        /// This control is used to switch on or off automatic power level detection. When switched on, power level detection is performed at the start of each measurement sweep. Using the ONCE argument starts the auto level measurement immediately.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_K10_AUTO_OFF  (0) - Off
        /// RSSPECAN_VAL_GSM_K10_AUTO_ON   (1) - On
        /// RSSPECAN_VAL_GSM_K10_AUTO_ONCE (2) - Once
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_K10_AUTO_OFF (0)
        /// 
        /// Attributes:
        /// RSSPECAN_ATTR_GSM_K10_AUTO_PWR_LEV
        /// RSSPECAN_ATTR_GSM_K10_AUTO_PWR_LEV_ONCE
        /// 
        /// </param>
        /// <param name="Reference_Level">
        /// Sets the current internal instrument reference level for RF input used when performing measurements.  
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: -20.0 dBm
        /// 
        /// Note(s):
        /// 
        /// (1) This control is ignored if Automatic Reference Level is different from Off
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_LEVEL
        /// </param>
        /// <param name="External_Attenuation">
        /// Specifies the external attenuation or gain applied to measurements.
        /// 
        /// Units: dB
        /// 
        /// Valid Values: -200.0 to 200.0
        /// 
        /// Default Value: 0.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET
        /// </param>
        /// <param name="Power_Class">
        /// Specifies the power class of the device under test.
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_GSM_K10_POW_CLASS_1  (0) - 1 
        /// RSSPECAN_VAL_GSM_K10_POW_CLASS_2  (1) - 2 
        /// RSSPECAN_VAL_GSM_K10_POW_CLASS_3  (2) - 3 
        /// RSSPECAN_VAL_GSM_K10_POW_CLASS_4  (3) - 4 
        /// RSSPECAN_VAL_GSM_K10_POW_CLASS_5  (4) - 5 
        /// RSSPECAN_VAL_GSM_K10_POW_CLASS_6  (5) - 6 
        /// RSSPECAN_VAL_GSM_K10_POW_CLASS_7  (6) - 7 
        /// RSSPECAN_VAL_GSM_K10_POW_CLASS_8  (7) - 8 
        /// RSSPECAN_VAL_GSM_K10_POW_CLASS_E1 (8) - E1 
        /// RSSPECAN_VAL_GSM_K10_POW_CLASS_E2 (9) - E2 
        /// RSSPECAN_VAL_GSM_K10_POW_CLASS_E3 (10) - E3 
        /// RSSPECAN_VAL_GSM_K10_POW_CLASS_M1 (11) - M1 
        /// RSSPECAN_VAL_GSM_K10_POW_CLASS_M2 (12) - M2 
        /// RSSPECAN_VAL_GSM_K10_POW_CLASS_M3 (13) - M3 
        /// RSSPECAN_VAL_GSM_K10_POW_CLASS_P1 (14) - P1 
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_K10_POW_CLASS_2 (1)
        /// 
        /// Note(s):
        /// 
        /// (1) 
        /// Value - Description:
        /// 
        /// 1     - MS and BTS power class 1 
        /// 2     - MS and BTS power class 2 
        /// 3     - MS and BTS power class 3 
        /// 4     - MS and BTS power class 4 
        /// 5     - MS and BTS power class 5 
        /// 6     - BTS power class 6 
        /// 7     - BTS power class 7 
        /// 8     - BTS power class 8 
        /// E1    - MS power class E1 
        /// E2    - MS power class E2 
        /// E3    - MS power class E3 
        /// M1    - BTS power class M1 (Micro) 
        /// M2    - BTS power class M2 (Micro) 
        /// M3    - BTS power class M3 (Micro) 
        /// P1    - BTS power class P1 (Pico)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_K10_LEVEL_SETTINGS_PWR_CLASS
        /// </param>
        /// <param name="Static_PCL">
        /// Specifies the static power control level. The static PCL is only  available when a BTS is selected as the device type.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0
        /// 
        /// Attributes:
        /// RSSPECAN_ATTR_GSM_K10_LEVEL_SETTINGS_POW_STAT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGSMK10LevelSettings(int Automatic_Reference_Level, double Reference_Level, double External_Attenuation, int Power_Class, int Static_PCL)
        {
            int pInvokeResult = PInvoke.ConfigureGSMK10LevelSettings(this._handle, Automatic_Reference_Level, Reference_Level, External_Attenuation, Power_Class, Static_PCL);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the capture settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BB_INPUT_SELECTION
        /// RSSPECAN_ATTR_SWEEP_TIME
        /// RSSPECAN_ATTR_TRIGGER_SOURCE
        /// RSSPECAN_ATTR_TRIGGER_DELAY
        /// RSSPECAN_ATTR_EXTERNAL_TRIGGER_LEVEL
        /// RSSPECAN_ATTR_TRIGGER_IFP_LEVEL
        /// RSSPECAN_ATTR_NUMBER_OF_SWEEPS
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:SELect RF
        /// [SENSe&lt;1|2&gt;:]SWEep:TIME
        /// TRIGger&lt;1|2&gt;[:SEQuence]:SOURce IMMediate | EXTernal | IFPower
        /// TRIGger&lt;1|2&gt;[:SEQuence]:HOLDoff
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel[:EXTernal]
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:IFPower
        /// [SENSe&lt;1|2&gt;:]SWEep:COUNt
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Signal_Source">
        /// Selects the signal source.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_BB_INPUT_RF  (0) - RF
        /// 
        /// Default Value: RSSPECAN_VAL_BB_INPUT_RF (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BB_INPUT_SELECTION
        /// </param>
        /// <param name="Capture_Time">
        /// Specifies the capture time to be used for measurements. 
        /// 
        /// Valid Values:
        /// 1.0e-6 s to 16.0e3 s
        /// 
        /// Default Value: 0.02 s
        /// 
        /// Note(s):
        /// 
        /// (1) Currently the capture time is set to 10 ms and can not be changed by the user. 
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SWEEP_TIME
        /// </param>
        /// <param name="Trigger_Mode">
        /// Selects the trigger source for the start of a sweep.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TRG_IMM  (0) - Immediate 
        /// RSSPECAN_VAL_TRG_EXT  (1) - External   
        /// RSSPECAN_VAL_TRG_IFP  (2) - Power
        /// 
        /// Default Value: RSSPECAN_VAL_TRG_IMM (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Immediate - free run; the instrument starts capturing immediate after the capture was requested by the user.
        /// 
        /// (2) External - External trigger via the rear BNC trigger connector.
        /// 
        /// (3) Power - Trigger on the RF or IF power of the signal
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_SOURCE
        /// </param>
        /// <param name="Trigger_Offset">
        /// Sets the length of the trigger delay.
        /// 
        /// Units: s
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_DELAY
        /// </param>
        /// <param name="Trigger_Level">
        /// Sets the trigger level.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0.0
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available if Trigger Mode is set to External or Power.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_EXTERNAL_TRIGGER_LEVEL
        /// RSSPECAN_ATTR_TRIGGER_IFP_LEVEL
        /// </param>
        /// <param name="Statistic_Count">
        /// Sets the number of frames to be measured
        /// 
        /// Valid Values:
        /// 0 to 32767
        /// 
        /// Default Value: 200
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NUMBER_OF_SWEEPS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGSMK10CaptureSettings(int Signal_Source, double Capture_Time, int Trigger_Mode, double Trigger_Offset, double Trigger_Level, int Statistic_Count)
        {
            int pInvokeResult = PInvoke.ConfigureGSMK10CaptureSettings(this._handle, Signal_Source, Capture_Time, Trigger_Mode, Trigger_Offset, Trigger_Level, Statistic_Count);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the synchronization settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_K10_SYNCHRONIZATION
        /// RSSPECAN_ATTR_GSM_K10_MEASURE_ONLY_ON_SYNC
        /// 
        /// Remote-control command(s):
        /// CONFigure[:MS]:SYNC:MODE ALL | TSC | BURSt | NONE
        /// CONFigure[:MS]:SYNC:ONLY ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Synchronization">
        /// Sets the synchronization mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_K10_SYNC_ALL   (0) - Burst + TSC
        /// RSSPECAN_VAL_GSM_K10_SYNC_TSC   (1) - TSC
        /// RSSPECAN_VAL_GSM_K10_SYNC_BURS  (2) - Burst
        /// RSSPECAN_VAL_GSM_K10_SYNC_NONE  (3) - None
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_K10_SYNC_ALL   (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_K10_SYNCHRONIZATION
        /// </param>
        /// <param name="Measure_Only_On_Sync">
        /// If activated, only results from frames (slots) where the "Slot to measure" was found are displayed and taken into account in the averaging of the results.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_K10_MEASURE_ONLY_ON_SYNC
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGSMK10Synchronization(int Synchronization, bool Measure_Only_On_Sync)
        {
            int pInvokeResult = PInvoke.ConfigureGSMK10Synchronization(this._handle, Synchronization, System.Convert.ToUInt16(Measure_Only_On_Sync));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the sweep time for auto level measurements or swept measurements and the capture time for auto detection.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_K10_CAP_SET_TRACK_TIME
        /// 
        /// Remote-control command(s):
        /// CONFigure[:MS]:POWer:AUTO:SWEep:TIME
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Auto_Track_Time">
        /// Specify the sweep time for auto level measurements or swept measurements and the capture time for auto detection.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 1.0e-3 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_K10_CAP_SET_TRACK_TIME
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGSMK10AutoTrackTime(double Auto_Track_Time)
        {
            int pInvokeResult = PInvoke.ConfigureGSMK10AutoTrackTime(this._handle, Auto_Track_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the parameters to be set automatically.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_K10_AUTO_SET_LEVEL
        /// RSSPECAN_ATTR_GSM_K10_AUTO_SET_LEVEL_ONCE
        /// RSSPECAN_ATTR_GSM_K10_AUTO_SET_FRAME
        /// RSSPECAN_ATTR_GSM_K10_AUTO_SET_FRAME_ONCE
        /// RSSPECAN_ATTR_GSM_K10_AUTO_SET_TRIGGER
        /// RSSPECAN_ATTR_GSM_K10_AUTO_SET_TRIGGER_ONCE
        /// 
        /// Remote-control command(s):
        /// CONFigure[:MS]:AUTO:LEVel OFF | ON | ONCE
        /// CONFigure[:MS]:AUTO:FRAMe OFF | ON | ONCE
        /// CONFigure[:MS]:AUTO:TRIGger OFF | ON | ONCE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Level">
        /// Switches on or off automatic level detection while running auto set. When switched on, level detection is performed on auto set. Using the ONCE argument starts one auto level measurement immediately. 
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_GSM_K10_LEVEL_OFF  (0) - Off
        /// RSSPECAN_VAL_GSM_K10_LEVEL_ON   (1) - On
        /// RSSPECAN_VAL_GSM_K10_LEVEL_ONCE (2) - Once
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_K10_LEVEL_ON (1)
        /// 
        /// Attributes:
        /// RSSPECAN_ATTR_GSM_K10_AUTO_SET_LEVEL
        /// RSSPECAN_ATTR_GSM_K10_AUTO_SET_LEVEL_ONCE
        /// </param>
        /// <param name="Frame_Configuration">
        /// Switches on or off automatic frame configuration detection while running auto set. When switched on, frame configuration detection is performed on auto set. Using the ONCE argument starts one auto frame configuration measurement immediately.  
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_GSM_K10_LEVEL_OFF  (0) - Off
        /// RSSPECAN_VAL_GSM_K10_LEVEL_ON   (1) - On
        /// RSSPECAN_VAL_GSM_K10_LEVEL_ONCE (2) - Once
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_K10_LEVEL_ON (1)
        /// 
        /// Attributes:
        /// RSSPECAN_ATTR_GSM_K10_AUTO_SET_FRAME
        /// RSSPECAN_ATTR_GSM_K10_AUTO_SET_FRAME_ONCE
        /// </param>
        /// <param name="Trigger">
        /// Switches on or off automatic trigger (offset / level) detection while running auto set. When switched on, trigger detection is performed on auto set. Using the ONCE argument starts one auto trigger measurement immediately.  
        /// 
        /// Valid Range:
        /// RSSPECAN_VAL_GSM_K10_LEVEL_OFF  (0) - Off
        /// RSSPECAN_VAL_GSM_K10_LEVEL_ON   (1) - On
        /// RSSPECAN_VAL_GSM_K10_LEVEL_ONCE (2) - Once
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_K10_LEVEL_ON (1)
        /// 
        /// Attributes:
        /// RSSPECAN_ATTR_GSM_K10_AUTO_SET_TRIGGER
        /// RSSPECAN_ATTR_GSM_K10_AUTO_SET_TRIGGER_ONCE
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGSMK10AutoSettings(int Level, int Frame_Configuration, int Trigger)
        {
            int pInvokeResult = PInvoke.ConfigureGSMK10AutoSettings(this._handle, Level, Frame_Configuration, Trigger);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the slot to be measured in single-slot measurements relative to the GSM frame boundary.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_K10_DEMOD_SINGLE_SLOT_MEAS
        /// 
        /// Remote-control command(s):
        /// CONFigure[:MS]:CHANnel:MSLots:MEASure
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Slot_To_Measure">
        /// Specifies the slot to be measured in single-slot measurements 
        /// relative to the GSM frame boundary. 
        /// 
        /// Valid Range:
        /// 0 to 7
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_K10_DEMOD_SINGLE_SLOT_MEAS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGSMK10DemodulationSingleSlotMeasurements(int Slot_To_Measure)
        {
            int pInvokeResult = PInvoke.ConfigureGSMK10DemodulationSingleSlotMeasurements(this._handle, Slot_To_Measure);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the parameters for the measurement interval for multi-slot measurements.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_K10_DEM_MULTI_SLOT_MEAS_SLOT_NUM
        /// RSSPECAN_ATTR_GSM_K10_DEM_MULTI_SLOT_MEAS_FIRST_SLOT
        /// 
        /// Remote-control command(s):
        /// CONFigure[:MS]:CHANnel:MSLots:NOFSlots
        /// CONFigure[:MS]:CHANnel:MSLots:OFFSet
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Number_Of_Slots_To_Measure">
        /// Specifies the Number of Slots to measure for the measurement interval of multi-slot measurements, i.e. the "Power vs Time" and "Transient Spectrum" measurements.
        /// 
        /// Valid Range:
        /// 1 to 8
        /// 
        /// Default Value: 8
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_K10_DEM_MULTI_SLOT_MEAS_SLOT_NUM
        /// </param>
        /// <param name="First_Slot_To_Measure">
        /// Specifies the start for the measurement interval for multi-slot measurements, i.e. the "Power vs Time" and "Transient Spectrum" measurements, relative to the GSM frame boundary. 
        /// 
        /// Valid Range:
        /// 0 to 7
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_K10_DEM_MULTI_SLOT_MEAS_FIRST_SLOT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGSMK10DemodulationMultiSlotMeasurements(int Number_Of_Slots_To_Measure, int First_Slot_To_Measure)
        {
            int pInvokeResult = PInvoke.ConfigureGSMK10DemodulationMultiSlotMeasurements(this._handle, Number_Of_Slots_To_Measure, First_Slot_To_Measure);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function if activated, all slots of a frame have the same length (8 x 156.26 normal symbol periods). If deactivated, slots number 0 and 4 of a frame have a longer duration, all other a shorter duration compared to the "equal slot length" (157, 156, 156, 156, 157, 156, 156, 156 normal symbol periods). 
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_K10_DEMODULATION_EQUAL_TIMESLOT_LENGTH
        /// 
        /// Remote-control command(s):
        /// CONFigure[:MS]:CHANnel:FRAMe:EQUal ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Equal_Timeslot_Length">
        /// Configures the time for the "Power vs Time" masks of all slots. 
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_K10_DEMODULATION_EQUAL_TIMESLOT_LENGTH
        /// 
        /// Note(s):
        /// 
        /// (1) If activated, all slots of a frame have the same length (8 x 156.26 normal symbol periods).
        /// 
        /// (2) If deactivated, slots number 0 and 4 of a frame have a longer duration, all other a shorter duration compared to the "equal slot length" (157, 156, 156, 156, 157, 156, 156, 156 normal symbol periods).
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGSMK10DemodulationEqualTimeslotLength(bool Equal_Timeslot_Length)
        {
            int pInvokeResult = PInvoke.ConfigureGSMK10DemodulationEqualTimeslotLength(this._handle, System.Convert.ToUInt16(Equal_Timeslot_Length));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures a burst at a single slot.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_K10_BURST_SLOT_STATE
        /// RSSPECAN_ATTR_GSM_K10_BURST_TYPE_BURST
        /// RSSPECAN_ATTR_GSM_K10_BURST_PCL
        /// RSSPECAN_ATTR_GSM_K10_BURST_MOD
        /// RSSPECAN_ATTR_GSM_K10_BURST_FILTER
        /// RSSPECAN_ATTR_GSM_K10_BURST_TSC
        /// 
        /// Remote-control command(s):
        /// CONFigure[:MS]:CHANnel:SLOT&lt;0..7&gt;[:STATe]
        /// CONFigure[:MS]:CHANnel:SLOT&lt;0..7&gt;:TYPE NB | HB
        /// CONFigure[:MS]:CHANnel:SLOT&lt;0..7&gt;:PCL
        /// CONFigure[:MS]:CHANnel:SLOT&lt;0..7&gt;:MTYPe GMSK | QPSK | PSK8 | QAM16 | QAM32
        /// CONFigure[:MS]:CHANnel:SLOT&lt;0..7&gt;:FILTer GMSK | LINearised | NARRow | WIDE 
        /// CONFigure[:MS]:CHANnel:SLOT&lt;0..7&gt;:TSC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Slot">
        /// Selects the slot. 
        /// 
        /// Valid Range:
        /// 0 to 7
        /// 
        /// Default Value: 0
        /// 
        /// </param>
        /// <param name="Active">
        /// Activates or deactivates selected slot.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_K10_BURST_SLOT_STATE
        /// </param>
        /// <param name="Burst_Type">
        /// Specifies the type of the burst.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_K10_BURST_NORMAL      (0) - Normal Burst
        /// RSSPECAN_VAL_GSM_K10_BURST_HIGHER_RATE (1) - Higher Symbol Rate
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_K10_BURST_NORMAL (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_K10_BURST_TYPE_BURST
        /// </param>
        /// <param name="PCL">
        /// Specifies the Power Control Level (PCL) / Dynamic Power Control Level (Dynamic PCL) for this specific slot which is relevant to calculate the correct Power vs Time limit for the slot. If the Device Type is a BTS the Dynamic PCL is used whereas MS uses PCL. 
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_K10_BURST_PCL
        /// </param>
        /// <param name="Modulation">
        /// Specifies the modulation type.
        /// 
        /// Valid Values:
        /// For normal burst:
        /// RSSPECAN_VAL_GSM_K10_MOD_GMSK  (0) - GMSK
        /// RSSPECAN_VAL_GSM_K10_MOD_8PSK  (2) - 8PSK
        /// RSSPECAN_VAL_GSM_K10_MOD_16QAM (3) - 16QAM
        /// RSSPECAN_VAL_GSM_K10_MOD_32QAM (4) - 32QAM
        /// 
        /// For Higher Symbol Rate Burst
        /// RSSPECAN_VAL_GSM_K10_MOD_QPSK  (1) - QPSK
        /// RSSPECAN_VAL_GSM_K10_MOD_16QAM (3) - 16QAM
        /// RSSPECAN_VAL_GSM_K10_MOD_32QAM (4) - 32QAM
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_K10_MOD_GMSK (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_K10_BURST_MOD
        /// </param>
        /// <param name="Filter">
        /// Specifies the pulse shape of the ideal modulator. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_K10_FILTER_NARROW   (2) - Narrow Pulse
        /// RSSPECAN_VAL_GSM_K10_FILTER_WIDE     (3) - Wide Pulse
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_K10_FILTER_NARROW (0)
        /// 
        /// Note(s):
        /// 
        /// (1) This control is valid only for 'Higher Symbol Rate' burst type. For Normal burst filter is selected accordingly with respect to modulation.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_K10_BURST_FILTER
        /// </param>
        /// <param name="TSC">
        /// Selects the training sequence used by the mobile or base station.
        /// 
        /// Valid Values:
        /// 0 to 7
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_K10_BURST_TSC
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGSMK10Burst(int Slot, bool Active, int Burst_Type, int PCL, int Modulation, int Filter, int TSC)
        {
            int pInvokeResult = PInvoke.ConfigureGSMK10Burst(this._handle, Slot, System.Convert.ToUInt16(Active), Burst_Type, PCL, Modulation, Filter, TSC);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the GSM Measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_K10_MEAS_BURST_TYPE
        /// RSSPECAN_ATTR_GSM_K10_MEAS_SPECT_TYPE
        /// 
        /// Remote-control command(s):
        /// CONFigure:BURSt:MACCuracy[:IMMediate]
        /// CONFigure:BURSt:PFERror[:IMMediate]
        /// CONFigure:BURSt:ETIMe[:IMMediate]
        /// CONFigure:BURSt:MERRor[:IMMediate]
        /// CONFigure:BURSt:CONStellation[:IMMediate]
        /// CONFigure:BURSt:PTEMplate[:IMMediate]
        /// CONFigure[:MS]:SPECtrum:MODulation[:IMMediate]
        /// CONFigure[:MS]:SPECtrum:SWITching[:IMMediate]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement_Mode">
        /// Selects the measurement mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_K10_MACC  (0) - Modulation Accuracy
        /// RSSPECAN_VAL_GSM_K10_PFER  (1) - Phase Error vs Time
        /// RSSPECAN_VAL_GSM_K10_ETIM  (2) - EVM vs Time
        /// RSSPECAN_VAL_GSM_K10_MERR  (3) - Magnitude Error
        /// RSSPECAN_VAL_GSM_K10_CONST (4) - Constellation
        /// RSSPECAN_VAL_GSM_K10_PTEM  (5) - Power vs. Time
        /// RSSPECAN_VAL_GSM_K10_MOD   (6) - Modulation Spectrum
        /// RSSPECAN_VAL_GSM_K10_SWIT  (7) - Transient Spectrum
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_K10_MACC (0)
        /// 
        /// 
        /// Notes:
        /// (1) Modulation Accuracy:
        /// Starts the Modulation Accuracy measurement.
        /// 
        /// (2) Phase Error vs Time:
        /// Starts either the Phase Error vs Time measurement.
        /// 
        /// (3) EVM vs Time:
        /// Starts either the EVM vs Time measurement.
        /// 
        /// (4) Magnitude Error:
        /// Starts the magnitude error measurement.
        /// 
        /// (5) Constellation:
        /// Starts the constellation measurement. 
        /// 
        /// (6) Power vs. Time:
        /// Starts the Power vs Time measurement
        /// 
        /// (7) Modulation Spectrum:
        /// Starts the Modulation Spectrum measurement.
        /// 
        /// (8) Transient Spectrum:
        /// Starts the Transient Spectrum measurement.
        /// 
        /// Attributes:
        /// RSSPECAN_ATTR_GSM_K10_MEAS_BURST_TYPE
        /// RSSPECAN_ATTR_GSM_K10_MEAS_SPECT_TYPE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGSMK10Measurement(int Measurement_Mode)
        {
            int pInvokeResult = PInvoke.ConfigureGSMK10Measurement(this._handle, Measurement_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the burst section to be measured.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_K10_PWR_VS_TIME_MEAS
        /// 
        /// Remote-control command(s):
        /// CONFigure:BURSt:PTEMplate:SELect FULL | TOP | FRISing
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Burst_Section">
        /// This control defines the burst section to be measured.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_K10_BURST_SEL_FULL   (0) - Full Burst
        /// RSSPECAN_VAL_GSM_K10_BURST_SEL_TH_RES (1) - Top High Resolution
        /// RSSPECAN_VAL_GSM_K10_BURST_SEL_RS     (2) - Rising and Falling
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_K10_BURST_SEL_FULL (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_K10_PWR_VS_TIME_MEAS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGsmK10PVTMeasurement(int Burst_Section)
        {
            int pInvokeResult = PInvoke.ConfigureGsmK10PVTMeasurement(this._handle, Burst_Section);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the filter which shall be used for the Power vs Time measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// (2) The function is only available if "Multi Carrier BTS" is switched off.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_K10_POWER_VS_TIME_FILTER
        /// 
        /// Remote-control command(s):
        /// CONFigure:BURSt:PTEMplate:FILTer G1000 | G500 | B600
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Power_Vs_Time_Filter">
        /// This control defines the filter which shall be used for the Power vs Time measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_K10_FILTER_1MHZ   (0) - 1 MHz Gauss
        /// RSSPECAN_VAL_GSM_K10_FILTER_500KHZ (1) - 500 kHz Gauss
        /// RSSPECAN_VAL_GSM_K10_FILTER_600KHZ (2) - 600 kHz Lowpass
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_K10_FILTER_1MHZ (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_K10_POWER_VS_TIME_FILTER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGSMK10PVTFilter(int Power_Vs_Time_Filter)
        {
            int pInvokeResult = PInvoke.ConfigureGSMK10PVTFilter(this._handle, Power_Vs_Time_Filter);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the limit check of the spectrum trace.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// (2) This function effects the Modulation Spectrum and Transient
        /// Spectrum measurements.
        /// 
        /// (3) For measurements on multi-carrier signals, using either the check on the left or right side allows you to measure the spectrum of the left or right-most channel while ignoring the side where adjacent channels are located.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_K10_MODULATION_TRANSIENT_SPECTRUM_ENABLE_LEFT_LIMIT
        /// RSSPECAN_ATTR_GSM_K10_MODULATION_TRANSIENT_SPECTRUM_ENABLE_RIGHT_LIMIT
        /// 
        /// Remote-control command(s):
        /// CONFigure:SPECtrum:LIMit:LEFT ON | OFF
        /// CONFigure:SPECtrum:LIMit:RIGHt ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Enable_Left_Limit">
        /// Controls the left limit check of the spectrum trace (spectrum graph measurement) and which offset frequencies in the table (spectrum list measurement) are checked against the limit.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_K10_MODULATION_TRANSIENT_SPECTRUM_ENABLE_LEFT_LIMIT
        /// </param>
        /// <param name="Enable_Right_Limit">
        /// Controls the right limit check of the spectrum trace (spectrum graph measurement) and which offset frequencies in the table (spectrum list measurement) are checked against the limit.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_K10_MODULATION_TRANSIENT_SPECTRUM_ENABLE_RIGHT_LIMIT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGSMK10ModulationTransientSpectrum(bool Enable_Left_Limit, bool Enable_Right_Limit)
        {
            int pInvokeResult = PInvoke.ConfigureGSMK10ModulationTransientSpectrum(this._handle, System.Convert.ToUInt16(Enable_Left_Limit), System.Convert.ToUInt16(Enable_Right_Limit));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the settings related to measurements on multi-carrier base stations.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// (2) This function is only available if the "Device Type" is a "BTS" type.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_GSM_K10_MULTI_CARRIER_BTS
        /// RSSPECAN_ATTR_GSM_K10_MULTI_CARRIER_NUMBER_OF_ACTIVE_CHANNELS
        /// RSSPECAN_ATTR_GSM_K10_MULTI_CARRIER_BTS_CLASS
        /// RSSPECAN_ATTR_GSM_K10_MULTI_CARRIER_FILTER
        /// 
        /// Remote-control command(s):
        /// CONFigure[:MS]:MCARrier:MCBTs ON | OFF
        /// CONFigure[:MS]:MCARrier:ACTCarriers
        /// CONFigure[:MS]:MCARrier:BTSClass 
        /// CONFigure[:MS]:MCARrier:FILTer MC400 | MC300
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Multi_Carrier_BTS">
        /// This control switches on or off the  multi-carrier signal.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_K10_MULTI_CARRIER_BTS
        /// </param>
        /// <param name="Number_Of_Active_Carriers">
        /// This control specifies the total number of active carriers of the multi-carrier BTS to be measured.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_K10_MULTI_CARRIER_NUMBER_OF_ACTIVE_CHANNELS
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available if Multi Carrier BTS is switched ON.
        /// </param>
        /// <param name="BTS_Class">
        /// This control defines the base station class.
        /// 
        /// Valid Values: 
        /// 1, 2
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_K10_MULTI_CARRIER_BTS_CLASS
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available if Multi Carrier BTS is switched ON.
        /// </param>
        /// <param name="PvT_Filter">
        /// This control selects the filter used to reduce the measurement bandwidth for multi-carrier "Power vs Time" measurements.
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_GSM_K10_MCAR_FILT_400 (0) - 400 kHz MC
        /// RSSPECAN_VAL_GSM_K10_MCAR_FILT_300 (1) - 300 kHz MC
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_K10_MCAR_FILT_400 (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_GSM_K10_MULTI_CARRIER_FILTER
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available if Multi Carrier BTS is switched ON.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureGSMK10MultiCarrier(bool Multi_Carrier_BTS, int Number_Of_Active_Carriers, int BTS_Class, int PvT_Filter)
        {
            int pInvokeResult = PInvoke.ConfigureGSMK10MultiCarrier(this._handle, System.Convert.ToUInt16(Multi_Carrier_BTS), Number_Of_Active_Carriers, BTS_Class, PvT_Filter);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts the Modulation Accuracy measurement and returns the result.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_READ_GSM_K10_MEAS_RES
        /// RSSPECAN_ATTR_READ_GSM_K10_MEAS_RES_95_PERC
        /// 
        /// Remote-control command(s):
        /// READ:BURSt[:MACCuracy][:EVM]:RMS:AVERage?
        /// READ:BURSt[:MACCuracy][:EVM]:RMS:CURRent?
        /// READ:BURSt[:MACCuracy][:EVM]:RMS:MAXimum?
        /// READ:BURSt[:MACCuracy][:EVM]:RMS:SDEViation?
        /// READ:BURSt[:MACCuracy][:EVM]:PEAK:AVERage?
        /// READ:BURSt[:MACCuracy][:EVM]:PEAK:CURRent?
        /// READ:BURSt[:MACCuracy][:EVM]:PEAK:MAXimum?
        /// READ:BURSt[:MACCuracy][:EVM]:PEAK:SDEViation?
        /// READ:BURSt[:MACCuracy]:MERRor:RMS:AVERage?
        /// READ:BURSt[:MACCuracy]:MERRor:RMS:CURRent?
        /// READ:BURSt[:MACCuracy]:MERRor:RMS:MAXimum?
        /// READ:BURSt[:MACCuracy]:MERRor:RMS:SDEViation?
        /// READ:BURSt[:MACCuracy]:MERRor:PEAK:AVERage?
        /// READ:BURSt[:MACCuracy]:MERRor:PEAK:CURRent?
        /// READ:BURSt[:MACCuracy]:MERRor:PEAK:MAXimum?
        /// READ:BURSt[:MACCuracy]:MERRor:PEAK:SDEViation?
        /// READ:BURSt[:MACCuracy]:PERRor:RMS:AVERage?
        /// READ:BURSt[:MACCuracy]:PERRor:RMS:CURRent?
        /// READ:BURSt[:MACCuracy]:PERRor:RMS:MAXimum?
        /// READ:BURSt[:MACCuracy]:PERRor:RMS:SDEViation?
        /// READ:BURSt[:MACCuracy]:PERRor:PEAK:AVERage?
        /// READ:BURSt[:MACCuracy]:PERRor:PEAK:CURRent?
        /// READ:BURSt[:MACCuracy]:PERRor:PEAK:MAXimum?
        /// READ:BURSt[:MACCuracy]:PERRor:PEAK:SDEViation?
        /// READ:BURSt[:MACCuracy]:OSUPpress:AVERage?
        /// READ:BURSt[:MACCuracy]:OSUPpress:CURRent?
        /// READ:BURSt[:MACCuracy]:OSUPpress:MAXimum?
        /// READ:BURSt[:MACCuracy]:OSUPpress:SDEViation?
        /// READ:BURSt[:MACCuracy]:IQOFfset:AVERage?
        /// READ:BURSt[:MACCuracy]:IQOFfset:CURRent?
        /// READ:BURSt[:MACCuracy]:IQOFfset:MAXimum?
        /// READ:BURSt[:MACCuracy]:IQOFfset:SDEViation?
        /// READ:BURSt[:MACCuracy]:IQIMbalance:AVERage?
        /// READ:BURSt[:MACCuracy]:IQIMbalance:CURRent?
        /// READ:BURSt[:MACCuracy]:IQIMbalance:MAXimum?
        /// READ:BURSt[:MACCuracy]:IQIMbalance:SDEViation?
        /// READ:BURSt[:MACCuracy]:FREQuency:AVERage?
        /// READ:BURSt[:MACCuracy]:FREQuency:CURRent?
        /// READ:BURSt[:MACCuracy]:FREQuency:MAXimum?
        /// READ:BURSt[:MACCuracy]:FREQuency:SDEViation?
        /// READ:BURSt[:MACCuracy]:BPOWer:AVERage?
        /// READ:BURSt[:MACCuracy]:BPOWer:CURRent?
        /// READ:BURSt[:MACCuracy]:BPOWer:MAXimum?
        /// READ:BURSt[:MACCuracy]:BPOWer:SDEViation?
        /// READ:BURSt[:MACCuracy]:ADRoop:AVERage?
        /// READ:BURSt[:MACCuracy]:ADRoop:CURRent?
        /// READ:BURSt[:MACCuracy]:ADRoop:MAXimum?
        /// READ:BURSt[:MACCuracy]:ADRoop:SDEViation?
        /// READ:BURSt[:MACCuracy]:PERCentile:EVM?
        /// READ:BURSt[:MACCuracy]:PERCentile:MERRor?
        /// READ:BURSt[:MACCuracy]:PERCentile:PERRor?
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Measurement">
        /// Sets the type of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_K10_MEAS_EVM_RMS   (0) - RMS of Error Vector Magnitude
        /// RSSPECAN_VAL_GSM_K10_MEAS_EVM_PEAK  (1) - Peak of Error Vector Magnitude
        /// RSSPECAN_VAL_GSM_K10_MEAS_MERR_RMS  (2) - RMS of Magnitude Error
        /// RSSPECAN_VAL_GSM_K10_MEAS_MERR_PEAK (3) - Peak of Magnitude Error
        /// RSSPECAN_VAL_GSM_K10_MEAS_PERR_RMS  (4) - RMS of Phase Error
        /// RSSPECAN_VAL_GSM_K10_MEAS_PERR_PEAK (5) - Peak of Phase Error
        /// RSSPECAN_VAL_GSM_K10_MEAS_OSUPP     (6) - Original Offset Suppression
        /// RSSPECAN_VAL_GSM_K10_MEAS_IQOF      (7) - IQ Offset
        /// RSSPECAN_VAL_GSM_K10_MEAS_IQIM      (8) - IQ Imbalance
        /// RSSPECAN_VAL_GSM_K10_MEAS_FREQ      (9) - Frequency Error
        /// RSSPECAN_VAL_GSM_K10_MEAS_BPOW      (10) - Burst Power
        /// RSSPECAN_VAL_GSM_K10_MEAS_ADR       (11) - Amplitude Droop
        /// RSSPECAN_VAL_GSM_K10_MEAS_PERC_EVM  (12) - 95% Percentile of Error Vector Magnitude
        /// RSSPECAN_VAL_GSM_K10_MEAS_PERC_MERR (13) - 95% Percentile of Magnitude Error
        /// RSSPECAN_VAL_GSM_K10_MEAS_PERC_PERR (14) - 95% Percentile of Phase Error
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_K10_MEAS_EVM_RMS (0)
        /// </param>
        /// <param name="Modifier">
        /// Sets the modifier of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_K10_MOD_AVG (0) - Average
        /// RSSPECAN_VAL_GSM_K10_MOD_CUR (1) - Current
        /// RSSPECAN_VAL_GSM_K10_MOD_MAX (2) - Maximum
        /// RSSPECAN_VAL_GSM_K10_MOD_DEV (3) - Standard Deviation
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_K10_MOD_AVG (0)
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns the value. 
        /// 
        /// Note(s):
        /// 
        /// (1) RMS of Phase Error - returned in deg.
        /// (2) Peak of Phase Error - returned in deg.
        /// (3) Frequency Error - returned in Hz
        /// (4) IQ Offset - returned in %
        /// (5) IQ Imbalance - returned in %
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_READ_GSM_K10_MEAS_RES
        /// RSSPECAN_ATTR_READ_GSM_K10_MEAS_RES_95_PERC
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadGSMK10MeasurementResults(uint Timeout, int Measurement, int Modifier, out double Result)
        {
            int pInvokeResult = PInvoke.ReadGSMK10MeasurementResults(this._handle, Timeout, Measurement, Modifier, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts the Modulation Accuracy measurement and returns the result.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FETCH_GSM_K10_MEAS_RES
        /// RSSPECAN_ATTR_FETCH_GSM_K10_MEAS_RES_95_PERC
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt[:MACCuracy][:EVM]:RMS:AVERage?
        /// FETCh:BURSt[:MACCuracy][:EVM]:RMS:CURRent?
        /// FETCh:BURSt[:MACCuracy][:EVM]:RMS:MAXimum?
        /// FETCh:BURSt[:MACCuracy][:EVM]:RMS:SDEViation?
        /// FETCh:BURSt[:MACCuracy][:EVM]:PEAK:AVERage?
        /// FETCh:BURSt[:MACCuracy][:EVM]:PEAK:CURRent?
        /// FETCh:BURSt[:MACCuracy][:EVM]:PEAK:MAXimum?
        /// FETCh:BURSt[:MACCuracy][:EVM]:PEAK:SDEViation?
        /// FETCh:BURSt[:MACCuracy]:MERRor:RMS:AVERage?
        /// FETCh:BURSt[:MACCuracy]:MERRor:RMS:CURRent?
        /// FETCh:BURSt[:MACCuracy]:MERRor:RMS:MAXimum?
        /// FETCh:BURSt[:MACCuracy]:MERRor:RMS:SDEViation?
        /// FETCh:BURSt[:MACCuracy]:MERRor:PEAK:AVERage?
        /// FETCh:BURSt[:MACCuracy]:MERRor:PEAK:CURRent?
        /// FETCh:BURSt[:MACCuracy]:MERRor:PEAK:MAXimum?
        /// FETCh:BURSt[:MACCuracy]:MERRor:PEAK:SDEViation?
        /// FETCh:BURSt[:MACCuracy]:PERRor:RMS:AVERage?
        /// FETCh:BURSt[:MACCuracy]:PERRor:RMS:CURRent?
        /// FETCh:BURSt[:MACCuracy]:PERRor:RMS:MAXimum?
        /// FETCh:BURSt[:MACCuracy]:PERRor:RMS:SDEViation?
        /// FETCh:BURSt[:MACCuracy]:PERRor:PEAK:AVERage?
        /// FETCh:BURSt[:MACCuracy]:PERRor:PEAK:CURRent?
        /// FETCh:BURSt[:MACCuracy]:PERRor:PEAK:MAXimum?
        /// FETCh:BURSt[:MACCuracy]:PERRor:PEAK:SDEViation?
        /// FETCh:BURSt[:MACCuracy]:OSUPpress:AVERage?
        /// FETCh:BURSt[:MACCuracy]:OSUPpress:CURRent?
        /// FETCh:BURSt[:MACCuracy]:OSUPpress:MAXimum?
        /// FETCh:BURSt[:MACCuracy]:OSUPpress:SDEViation?
        /// FETCh:BURSt[:MACCuracy]:IQOFfset:AVERage?
        /// FETCh:BURSt[:MACCuracy]:IQOFfset:CURRent?
        /// FETCh:BURSt[:MACCuracy]:IQOFfset:MAXimum?
        /// FETCh:BURSt[:MACCuracy]:IQOFfset:SDEViation?
        /// FETCh:BURSt[:MACCuracy]:IQIMbalance:AVERage?
        /// FETCh:BURSt[:MACCuracy]:IQIMbalance:CURRent?
        /// FETCh:BURSt[:MACCuracy]:IQIMbalance:MAXimum?
        /// FETCh:BURSt[:MACCuracy]:IQIMbalance:SDEViation?
        /// FETCh:BURSt[:MACCuracy]:FREQuency:AVERage?
        /// FETCh:BURSt[:MACCuracy]:FREQuency:CURRent?
        /// FETCh:BURSt[:MACCuracy]:FREQuency:MAXimum?
        /// FETCh:BURSt[:MACCuracy]:FREQuency:SDEViation?
        /// FETCh:BURSt[:MACCuracy]:BPOWer:AVERage?
        /// FETCh:BURSt[:MACCuracy]:BPOWer:CURRent?
        /// FETCh:BURSt[:MACCuracy]:BPOWer:MAXimum?
        /// FETCh:BURSt[:MACCuracy]:BPOWer:SDEViation?
        /// FETCh:BURSt[:MACCuracy]:ADRoop:AVERage?
        /// FETCh:BURSt[:MACCuracy]:ADRoop:CURRent?
        /// FETCh:BURSt[:MACCuracy]:ADRoop:MAXimum?
        /// FETCh:BURSt[:MACCuracy]:ADRoop:SDEViation?
        /// FETCh:BURSt[:MACCuracy]:PERCentile:EVM?
        /// FETCh:BURSt[:MACCuracy]:PERCentile:MERRor?
        /// FETCh:BURSt[:MACCuracy]:PERCentile:PERRor?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement">
        /// Sets the type of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_K10_MEAS_EVM_RMS   (0) - RMS of Error Vector Magnitude
        /// RSSPECAN_VAL_GSM_K10_MEAS_EVM_PEAK  (1) - Peak of Error Vector Magnitude
        /// RSSPECAN_VAL_GSM_K10_MEAS_MERR_RMS  (2) - RMS of Magnitude Error
        /// RSSPECAN_VAL_GSM_K10_MEAS_MERR_PEAK (3) - Peak of Magnitude Error
        /// RSSPECAN_VAL_GSM_K10_MEAS_PERR_RMS  (4) - RMS of Phase Error
        /// RSSPECAN_VAL_GSM_K10_MEAS_PERR_PEAK (5) - Peak of Phase Error
        /// RSSPECAN_VAL_GSM_K10_MEAS_OSUPP     (6) - Original Offset Suppression
        /// RSSPECAN_VAL_GSM_K10_MEAS_IQOF      (7) - IQ Offset
        /// RSSPECAN_VAL_GSM_K10_MEAS_IQIM      (8) - IQ Imbalance
        /// RSSPECAN_VAL_GSM_K10_MEAS_FREQ      (9) - Frequency Error
        /// RSSPECAN_VAL_GSM_K10_MEAS_BPOW      (10) - Burst Power
        /// RSSPECAN_VAL_GSM_K10_MEAS_ADR       (11) - Amplitude Droop
        /// RSSPECAN_VAL_GSM_K10_MEAS_PERC_EVM  (12) - 95% Percentile of Error Vector Magnitude
        /// RSSPECAN_VAL_GSM_K10_MEAS_PERC_MERR (13) - 95% Percentile of Magnitude Error
        /// RSSPECAN_VAL_GSM_K10_MEAS_PERC_PERR (14) - 95% Percentile of Phase Error
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_K10_MEAS_EVM_RMS (0)
        /// </param>
        /// <param name="Modifier">
        /// Sets the modifier of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_K10_MOD_AVG (0) - Average
        /// RSSPECAN_VAL_GSM_K10_MOD_CUR (1) - Current
        /// RSSPECAN_VAL_GSM_K10_MOD_MAX (2) - Maximum
        /// RSSPECAN_VAL_GSM_K10_MOD_DEV (3) - Standard Deviation
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_K10_MOD_AVG (0)
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns the value. 
        /// 
        /// Note(s):
        /// 
        /// (1) RMS of Phase Error - returned in deg.
        /// (2) Peak of Phase Error - returned in deg.
        /// (3) Frequency Error - returned in Hz
        /// (4) IQ Offset - returned in %
        /// (5) IQ Imbalance - returned in %
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FETCH_GSM_K10_MEAS_RES
        /// RSSPECAN_ATTR_FETCH_GSM_K10_MEAS_RES_95_PERC
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchGSMK10MeasurementResults(int Measurement, int Modifier, out double Result)
        {
            int pInvokeResult = PInvoke.FetchGSMK10MeasurementResults(this._handle, Measurement, Modifier, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts the measurement and returns all the results. 
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// READ:BURSt[:MACCuracy]:ALL?
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Array_Size">
        /// This control defines size of array in 'Result' argument.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 51
        /// </param>
        /// <param name="Result">
        /// Returns the values. 
        /// 
        /// Order of elements in array:
        /// 
        /// Error Vector Magnitude RMS &lt;Average, Current, Peak, Standard Deviation&gt; 
        /// Error Vector Magnitude Peak &lt;Average, Current, Peak, Standard Deviation&gt; 
        /// Magnitude Error RMS &lt;Average, Current, Peak, Standard Deviation&gt; 
        /// Magnitude Error Peak &lt;Average, Current, Peak, Standard Deviation&gt; 
        /// Phase Error RMS &lt;Average, Current, Peak, Standard Deviation&gt; 
        /// Phase Error Peak &lt;Average, Current, Peak, Standard Deviation&gt; 
        /// Origin Offset Suppresion &lt;Average, Current, Peak, Standard Deviation&gt; 
        /// IQ Offset &lt;Average, Current, Peak, Standard Deviation&gt; 
        /// IQ Imbalance &lt;Average, Current, Peak, Standard Deviation&gt; 
        /// Frequency Error &lt;Average, Current, Peak, Standard Deviation&gt; 
        /// Burst Power &lt;Average, Current, Peak, Standard Deviation&gt; 
        /// Amplitude Droop &lt;Average, Current, Peak, Standard Deviation&gt; 
        /// 95%tile &lt;EVM, Mag Error, Phase Error&gt;
        /// 
        /// This array must contain at least 51 elements.
        /// </param>
        /// <param name="Returned_Values">
        /// Returns number of results returned from the instrument.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadGSMK10ModulationAccuracyResults(uint Timeout, int Array_Size, double[] Result, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.ReadGSMK10ModulationAccuracyResults(this._handle, Timeout, Array_Size, Result, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns all the results of the Modulation Accuracy table.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt[:MACCuracy]:ALL?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// This control defines size of array in 'Result' argument.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 51
        /// </param>
        /// <param name="Result">
        /// Returns the values. 
        /// 
        /// Order of elements in array:
        /// 
        /// Error Vector Magnitude RMS &lt;Average, Current, Peak, Standard Deviation&gt; 
        /// Error Vector Magnitude Peak &lt;Average, Current, Peak, Standard Deviation&gt; 
        /// Magnitude Error RMS &lt;Average, Current, Peak, Standard Deviation&gt; 
        /// Magnitude Error Peak &lt;Average, Current, Peak, Standard Deviation&gt; 
        /// Phase Error RMS &lt;Average, Current, Peak, Standard Deviation&gt; 
        /// Phase Error Peak &lt;Average, Current, Peak, Standard Deviation&gt; 
        /// Origin Offset Suppresion &lt;Average, Current, Peak, Standard Deviation&gt; 
        /// IQ Offset &lt;Average, Current, Peak, Standard Deviation&gt; 
        /// IQ Imbalance &lt;Average, Current, Peak, Standard Deviation&gt; 
        /// Frequency Error &lt;Average, Current, Peak, Standard Deviation&gt; 
        /// Burst Power &lt;Average, Current, Peak, Standard Deviation&gt; 
        /// Amplitude Droop &lt;Average, Current, Peak, Standard Deviation&gt; 
        /// 95%tile &lt;EVM, Mag Error, Phase Error&gt;
        /// 
        /// This array must contain at least 51 elements.
        /// </param>
        /// <param name="Returned_Values">
        /// Returns number of results returned from the instrument.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchGSMK10ModulationAccuracyResults(int Array_Size, double[] Result, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.FetchGSMK10ModulationAccuracyResults(this._handle, Array_Size, Result, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts a measurement and returns the result of the burst power for the selected slot.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_READ_GSM_K10_BURST_SLOT_RES
        /// 
        /// Remote-control command(s):
        /// READ:BURSt:SPOWer:SLOT&lt;0..7&gt;:ALL:AVERage?
        /// READ:BURSt:SPOWer:SLOT&lt;0..7&gt;:ALL:CRESt?
        /// READ:BURSt:SPOWer:SLOT&lt;0..7&gt;:ALL:MAXimum?
        /// READ:BURSt:SPOWer:SLOT&lt;0..7&gt;:CURRent:AVERage?
        /// READ:BURSt:SPOWer:SLOT&lt;0..7&gt;:CURRent:CRESt?
        /// READ:BURSt:SPOWer:SLOT&lt;0..7&gt;:CURRent:MAXimum?
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Slot">
        /// Selects the slot. 
        /// 
        /// Valid Range:
        /// 0 to 7
        /// 
        /// Default Value: 0
        /// 
        /// </param>
        /// <param name="Burst_Selection">
        /// Sets the type of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_K10_BURST_ALL  (0) - All Bursts
        /// RSSPECAN_VAL_GSM_K10_BURST_CURR (1) - Current Burst
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_K10_BURST_ALL (0)
        /// </param>
        /// <param name="Modifier">
        /// Sets the modifier of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_K10_BURST_MOD_AVG_PWR (0) - Average Power
        /// RSSPECAN_VAL_GSM_K10_BURST_MOD_CREST   (1) - Crest Factor
        /// RSSPECAN_VAL_GSM_K10_BURST_MOD_MAX_PWR (2) - Maximum Power
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_K10_BURST_MOD_AVG_PWR (0)
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns the value. 
        /// 
        /// Note(s):
        /// 
        /// (1) Average and Maximum Power - returned in dBm.
        /// (2) Crest Factor - returned in dB.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_READ_GSM_K10_BURST_SLOT_RES
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadGSMK10BurstSlotResults(uint Timeout, int Slot, int Burst_Selection, int Modifier, out double Result)
        {
            int pInvokeResult = PInvoke.ReadGSMK10BurstSlotResults(this._handle, Timeout, Slot, Burst_Selection, Modifier, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the result of the burst power for the selected slot.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FETCH_GSM_K10_BURST_SLOT_RES
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:SPOWer:SLOT&lt;0..7&gt;:ALL:AVERage?
        /// FETCh:BURSt:SPOWer:SLOT&lt;0..7&gt;:ALL:CRESt?
        /// FETCh:BURSt:SPOWer:SLOT&lt;0..7&gt;:ALL:MAXimum?
        /// FETCh:BURSt:SPOWer:SLOT&lt;0..7&gt;:CURRent:AVERage?
        /// FETCh:BURSt:SPOWer:SLOT&lt;0..7&gt;:CURRent:CRESt?
        /// FETCh:BURSt:SPOWer:SLOT&lt;0..7&gt;:CURRent:MAXimum?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Slot">
        /// Selects the slot. 
        /// 
        /// Valid Range:
        /// 0 to 7
        /// 
        /// Default Value: 0
        /// 
        /// </param>
        /// <param name="Burst_Selection">
        /// Sets the type of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_K10_BURST_ALL  (0) - All Bursts
        /// RSSPECAN_VAL_GSM_K10_BURST_CURR (1) - Current Burst
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_K10_BURST_ALL (0)
        /// </param>
        /// <param name="Modifier">
        /// Sets the modifier of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_GSM_K10_BURST_MOD_AVG_PWR (0) - Average Power
        /// RSSPECAN_VAL_GSM_K10_BURST_MOD_CREST   (1) - Crest Factor
        /// RSSPECAN_VAL_GSM_K10_BURST_MOD_MAX_PWR (2) - Maximum Power
        /// 
        /// Default Value: RSSPECAN_VAL_GSM_K10_BURST_MOD_AVG_PWR (0)
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns the value. 
        /// 
        /// Note(s):
        /// 
        /// (1) Average and Maximum Power - returned in dBm.
        /// (2) Crest Factor - returned in dB.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FETCH_GSM_K10_BURST_SLOT_RES
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchGSMK10BurstSlotResults(int Slot, int Burst_Selection, int Modifier, out double Result)
        {
            int pInvokeResult = PInvoke.FetchGSMK10BurstSlotResults(this._handle, Slot, Burst_Selection, Modifier, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts the measurement and reads out the result of the measurement of the modulation spectrum of the mobile or base station.  
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// (2) This function is available only when modulation spectrum measurement is selected (rsspecan_ConfigureGSMK10Measurement).
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// READ:SPECtrum:MODulation[:ALL?]
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the number of elements allocated in arrays. 
        /// 
        /// Valid Values: &gt;0
        /// 
        /// Default Value: 100
        /// 
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Index">
        /// Returns indexed array of measurement range (last number indicates sum of measurements).
        /// 
        /// </param>
        /// <param name="Start_Frequencies">
        /// Returns start frequencies of the measurement range or frequencies where the limit is exceeded.
        /// 
        /// </param>
        /// <param name="Stop_Frequencies">
        /// Returns stop frequencies of the measurement range or frequencies where the measured range is exceeded. This values are equal to the values of start frequencies, if either the measurement is performed in the time domain or the result value contains a limit excess.
        /// 
        /// </param>
        /// <param name="Levels">
        /// Returns array of measured maximum levels of the partial range or measured levels at the test point.
        /// 
        /// </param>
        /// <param name="Limits">
        /// Returns limits in the partial range or measured levels at the test point.
        /// 
        /// </param>
        /// <param name="Abs_Rel">
        /// Returns array of ABS/REL unit indicators. Meaning of numbers is following:
        /// 
        ///  ABS = 0
        ///  REL = 1
        /// 
        /// </param>
        /// <param name="Status">
        /// Limit checks array of results.
        /// 
        /// PASSED ... no limits exceeded
        /// FAILED ... limits exceeded
        /// MARGIN ... margin exceeded
        /// EXC ... limited excesses characterized as an exception
        /// 
        /// Meaning of numbers is:
        /// 
        /// PASSED = 0
        /// FAILED = 1
        /// MARGIN = 2
        /// EXC = 3 
        /// </param>
        /// <param name="Num_of_Results">
        /// This control returns number of results, received from the instrument.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadGSMK10ModulationSpectrumResults(int Array_Size, uint Timeout, int[] Index, double[] Start_Frequencies, double[] Stop_Frequencies, double[] Levels, double[] Limits, int[] Abs_Rel, int[] Status, out int Num_of_Results)
        {
            int pInvokeResult = PInvoke.ReadGSMK10ModulationSpectrumResults(this._handle, Array_Size, Timeout, Index, Start_Frequencies, Stop_Frequencies, Levels, Limits, Abs_Rel, Status, out Num_of_Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the result of the measurement of the modulation spectrum of the mobile or base station.  
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// (2) This function is available only when modulation spectrum measurement is selected (rsspecan_ConfigureGSMK10Measurement).
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FETCh:SPECtrum:MODulation[:ALL?]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the number of elements allocated in arrays. 
        /// 
        /// Valid Values: &gt;0
        /// 
        /// Default Value: 100
        /// 
        /// </param>
        /// <param name="Index">
        /// Returns indexed array of measurement range (last number indicates sum of measurements).
        /// 
        /// </param>
        /// <param name="Start_Frequencies">
        /// Returns start frequencies of the measurement range or frequencies where the limit is exceeded.
        /// 
        /// </param>
        /// <param name="Stop_Frequencies">
        /// Returns stop frequencies of the measurement range or frequencies where the measured range is exceeded. This values are equal to the values of start frequencies, if either the measurement is performed in the time domain or the result value contains a limit excess.
        /// 
        /// </param>
        /// <param name="Levels">
        /// Returns array of measured maximum levels of the partial range or measured levels at the test point.
        /// 
        /// </param>
        /// <param name="Limits">
        /// Returns limits in the partial range or measured levels at the test point.
        /// 
        /// </param>
        /// <param name="Abs_Rel">
        /// Returns array of ABS/REL unit indicators. Meaning of numbers is following:
        /// 
        ///  ABS = 0
        ///  REL = 1
        /// 
        /// </param>
        /// <param name="Status">
        /// Limit checks array of results.
        /// 
        /// PASSED ... no limits exceeded
        /// FAILED ... limits exceeded
        /// MARGIN ... margin exceeded
        /// EXC ... limited excesses characterized as an exception
        /// 
        /// Meaning of numbers is:
        /// 
        /// PASSED = 0
        /// FAILED = 1
        /// MARGIN = 2
        /// EXC = 3 
        /// </param>
        /// <param name="Num_of_Results">
        /// This control returns number of results, received from the instrument.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchGSMK10ModulationSpectrumResults(int Array_Size, int[] Index, double[] Start_Frequencies, double[] Stop_Frequencies, double[] Levels, double[] Limits, int[] Abs_Rel, int[] Status, out int Num_of_Results)
        {
            int pInvokeResult = PInvoke.FetchGSMK10ModulationSpectrumResults(this._handle, Array_Size, Index, Start_Frequencies, Stop_Frequencies, Levels, Limits, Abs_Rel, Status, out Num_of_Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts the measurement and reads out the result of the measurement of the transient spectrum of the mobile or base station.   
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// (2) This function is available only when transient spectrum measurement is selected (rsspecan_ConfigureGSMK10Measurement).
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// READ:SPECtrum:SWITching[:ALL?]
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the number of elements allocated in arrays. 
        /// 
        /// Valid Values: &gt;0
        /// 
        /// Default Value: 100
        /// 
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Index">
        /// Returns indexed array of measurement range (last number indicates sum of measurements).
        /// 
        /// </param>
        /// <param name="Start_Frequencies">
        /// Returns start frequencies of the measurement range or frequencies where the limit is exceeded.
        /// 
        /// </param>
        /// <param name="Stop_Frequencies">
        /// Returns stop frequencies of the measurement range or frequencies where the measured range is exceeded. This values are equal to the values of start frequencies, if either the measurement is performed in the time domain or the result value contains a limit excess.
        /// 
        /// </param>
        /// <param name="Levels">
        /// Returns array of measured maximum levels of the partial range or measured levels at the test point.
        /// 
        /// </param>
        /// <param name="Limits">
        /// Returns limits in the partial range or measured levels at the test point.
        /// 
        /// </param>
        /// <param name="Abs_Rel">
        /// Returns array of ABS/REL unit indicators. Meaning of numbers is following:
        /// 
        ///  ABS = 0
        ///  REL = 1
        /// 
        /// </param>
        /// <param name="Status">
        /// Limit checks array of results.
        /// 
        /// PASSED ... no limits exceeded
        /// FAILED ... limits exceeded
        /// MARGIN ... margin exceeded
        /// EXC ... limited excesses characterized as an exception
        /// 
        /// Meaning of numbers is:
        /// 
        /// PASSED = 0
        /// FAILED = 1
        /// MARGIN = 2
        /// EXC = 3 
        /// </param>
        /// <param name="Num_of_Results">
        /// This control returns number of results, received from the instrument.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadGSMK10TransientSpectrumResults(int Array_Size, uint Timeout, int[] Index, double[] Start_Frequencies, double[] Stop_Frequencies, double[] Levels, double[] Limits, int[] Abs_Rel, int[] Status, out int Num_of_Results)
        {
            int pInvokeResult = PInvoke.ReadGSMK10TransientSpectrumResults(this._handle, Array_Size, Timeout, Index, Start_Frequencies, Stop_Frequencies, Levels, Limits, Abs_Rel, Status, out Num_of_Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the result of the measurement of the transient spectrum of the mobile or base station.   
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjunction with K10 option.
        /// 
        /// (2) This function is available only when transient spectrum measurement is selected (rsspecan_ConfigureGSMK10Measurement).
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FETCh:SPECtrum:SWITching[:ALL?]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the number of elements allocated in arrays. 
        /// 
        /// Valid Values: &gt;0
        /// 
        /// Default Value: 100
        /// 
        /// </param>
        /// <param name="Index">
        /// Returns indexed array of measurement range (last number indicates sum of measurements).
        /// 
        /// </param>
        /// <param name="Start_Frequencies">
        /// Returns start frequencies of the measurement range or frequencies where the limit is exceeded.
        /// 
        /// </param>
        /// <param name="Stop_Frequencies">
        /// Returns stop frequencies of the measurement range or frequencies where the measured range is exceeded. This values are equal to the values of start frequencies, if either the measurement is performed in the time domain or the result value contains a limit excess.
        /// 
        /// </param>
        /// <param name="Levels">
        /// Returns array of measured maximum levels of the partial range or measured levels at the test point.
        /// 
        /// </param>
        /// <param name="Limits">
        /// Returns limits in the partial range or measured levels at the test point.
        /// 
        /// </param>
        /// <param name="Abs_Rel">
        /// Returns array of ABS/REL unit indicators. Meaning of numbers is following:
        /// 
        ///  ABS = 0
        ///  REL = 1
        /// 
        /// </param>
        /// <param name="Status">
        /// Limit checks array of results.
        /// 
        /// PASSED ... no limits exceeded
        /// FAILED ... limits exceeded
        /// MARGIN ... margin exceeded
        /// EXC ... limited excesses characterized as an exception
        /// 
        /// Meaning of numbers is:
        /// 
        /// PASSED = 0
        /// FAILED = 1
        /// MARGIN = 2
        /// EXC = 3 
        /// </param>
        /// <param name="Num_of_Results">
        /// This control returns number of results, received from the instrument.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchGSMK10TransientSpectrumResults(int Array_Size, int[] Index, double[] Start_Frequencies, double[] Stop_Frequencies, double[] Levels, double[] Limits, int[] Abs_Rel, int[] Status, out int Num_of_Results)
        {
            int pInvokeResult = PInvoke.FetchGSMK10TransientSpectrumResults(this._handle, Array_Size, Index, Start_Frequencies, Stop_Frequencies, Levels, Limits, Abs_Rel, Status, out Num_of_Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates or deactivates the Spectrogram Measurement option.
        /// 
        /// Note: 
        /// 
        /// (1) This function is available only with R&amp;S FSL with FS-K14 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SPEM_STATE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:SPECtrogram[:STATe] OFF | ON
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="State">
        /// Toggles the Spectrogram Measurement option. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off  
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SPEM_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SpectrogramMode(bool State)
        {
            int pInvokeResult = PInvoke.SpectrogramMode(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts a continuous sweep measurement or stops it and switches to single sweep mode.
        /// 
        /// Note: 
        /// 
        /// (1) This function is available only with R&amp;S FSL with FS-K14 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SWEEP_MODE_CONTINUOUS
        /// 
        /// Remote-control command(s):
        /// INITiate&lt;1|2&gt;:CONTinuous OFF | ON
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Continuous">
        /// This control toggles the sweep.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On  (Default Value)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SWEEP_MODE_CONTINUOUS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSpectrogramSweep(bool Continuous)
        {
            int pInvokeResult = PInvoke.ConfigureSpectrogramSweep(this._handle, System.Convert.ToUInt16(Continuous));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures Spectogram display parameters.
        /// 
        /// Note: 
        /// 
        /// (1) This function is available only with R&amp;S FSL with FS-K14 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SPEM_COLOR
        /// RSSPECAN_ATTR_SPEM_SIZE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:SPECtrogram:COLor COLor | RADar | GRAYscale
        /// CALCulate&lt;1|2&gt;:SPECtrogram:SIZE SMALl | LARGe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Color">
        /// This control defines the frame color setting.
        /// 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SPEM_COL_COLOR (0) - Color RSSPECAN_VAL_SPEM_COL_RADAR (1) - Radar
        /// RSSPECAN_VAL_SPEM_COL_GRAY  (2) - Greyscale
        /// 
        /// Default Value: RSSPECAN_VAL_SPEM_COL_COLOR (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SPEM_COLOR
        /// 
        /// Note:
        /// 
        /// (1) Color: RGB colors
        /// 
        /// (2) Radar: black ? green ? white
        /// 
        /// (3) Grayscale: black and white
        /// </param>
        /// <param name="Size">
        /// This control changes the screen layout.
        /// 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DISP_SIZE_LARGE (0) - Large
        /// RSSPECAN_VAL_DISP_SIZE_SMALL (1) - Small
        /// 
        /// Default Value: RSSPECAN_VAL_DISP_SIZE_SMALL (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SPEM_SIZE
        /// 
        /// Note:
        /// 
        /// (1) Large: Enlarges the spectrogram result display and scales down the spectrum analyzer result display.
        /// 
        /// (2) Small: Divides the screen in two equally sized panes.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSpectrogramDisplayParameters(int Color, int Size)
        {
            int pInvokeResult = PInvoke.ConfigureSpectrogramDisplayParameters(this._handle, Color, Size);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures Spectogram measurement parameters.
        /// 
        /// Note: 
        /// 
        /// (1) This function is available only with R&amp;S FSL with FS-K14 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SPEM_HDEP
        /// RSSPECAN_ATTR_SPEM_TIME_STAMP_STATE
        /// RSSPECAN_ATTR_SPEM_FRAME_COUNT
        /// RSSPECAN_ATTR_SPEM_CONTINUOUS
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:SPECtrogram:HDEPth
        /// CALCulate&lt;1|2&gt;:SPECtrogram:TSTamp[:STATe] OFF | ON
        /// CALCulate&lt;1|2&gt;:SPECtrogram:FRAMe:COUNt
        /// CALCulate&lt;1|2&gt;:SPECtrogram:CONTinuous OFF | ON
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="History_Depth">
        /// This control sets the number of frames to be stored in the history buffer.
        /// 
        /// Valid Values:
        /// 282 to 20000
        /// 
        /// Default Value: 3000
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SPEM_HDEP
        /// </param>
        /// <param name="Time_Stamp">
        /// Toggles the Spectrogram Measurement option.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off  
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0) 
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SPEM_TIME_STAMP_STATE
        /// </param>
        /// <param name="Frame_Count">
        /// This control sets the number of frames recorded in a single sweep measurement.
        /// 
        /// Valid Values:
        /// 1 to &lt;max value, depending on history depth&gt;
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SPEM_FRAME_COUNT
        /// </param>
        /// <param name="Continue_Frame">
        /// Determines whether the results of the last measurement are deleted before starting a new measurement in single sweep mode.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off  
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SPEM_CONTINUOUS
        /// 
        /// Note:
        /// 
        /// (1) Off: Deletes the last measurement results before performing a single sweep measurement.
        /// 
        /// (2) On: Repeats the single sweep measurement without deleting the spectrogram results of the last measurement. One of the following trace modes is to be used: Max Hold, Min Hold, Average.
        /// 
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSpectrogramMeasurementParameters(int History_Depth, bool Time_Stamp, int Frame_Count, bool Continue_Frame)
        {
            int pInvokeResult = PInvoke.ConfigureSpectrogramMeasurementParameters(this._handle, History_Depth, System.Convert.ToUInt16(Time_Stamp), Frame_Count, System.Convert.ToUInt16(Continue_Frame));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function deletes the spectrogram result display and the history buffer.
        /// 
        /// Note: 
        /// 
        /// (1) This function is available only with R&amp;S FSL with FS-K14 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SPEM_CLEAR
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:SPECtrogram:CLEar[:IMMediate]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SpectrogramClear()
        {
            int pInvokeResult = PInvoke.SpectrogramClear(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the frame number of the specified marker.
        /// 
        /// Note:
        /// 
        /// This function is only available with the R&amp;S FSL with FS-K14 option. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SPEM_MARK_FRAME_NUMBER
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:SPECtrogram:FRAMe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Marker_Number">
        /// Selects the marker to configure.
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Frame_Number">
        /// This control defines the frame number of the specified marker.
        /// 
        /// Valid Values: not checked
        /// (&lt;number of recorded frames&gt; to 0)
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SPEM_MARK_FRAME_NUMBER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSpectrogramMarkerFrameNumber(int Marker_Number, int Frame_Number)
        {
            int pInvokeResult = PInvoke.ConfigureSpectrogramMarkerFrameNumber(this._handle, Marker_Number, Frame_Number);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the frame number of the specified marker.
        /// 
        /// Note:
        /// 
        /// This function is only available with the R&amp;S FSL with FS-K14 option. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SPEM_MARK_FRAME_NUMBER
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:SPECtrogram:FRAMe?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Marker_Number">
        /// Selects the marker to configure.
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Frame_Number">
        /// Returns the frame number of marker.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetSpectrogramMarkerFrameNumber(int Marker_Number, out int Frame_Number)
        {
            int pInvokeResult = PInvoke.GetSpectrogramMarkerFrameNumber(this._handle, Marker_Number, out Frame_Number);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the search area of the specified marker.
        /// 
        /// Note:
        /// 
        /// This function is only available with the R&amp;S FSL with FS-K14 option. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SPEM_MARKER_SEARCH_AREA
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:SPECtrogram:SARea VISible | MEMory
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Marker_Number">
        /// Selects the marker to configure.
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Search_Area">
        /// This control sets teh search area.
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_SPEM_SAR_MEMORY  (0) - Memory
        /// RSSPECAN_VAL_SPEM_SAR_VISIBLE (1) - Visible
        /// 
        /// Default Value: RSSPECAN_VAL_SPEM_SAR_VISIBLE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SPEM_MARKER_SEARCH_AREA
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSpectrogramMarkerSearchArea(int Marker_Number, int Search_Area)
        {
            int pInvokeResult = PInvoke.ConfigureSpectrogramMarkerSearchArea(this._handle, Marker_Number, Search_Area);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function positions the specified marker on the specified place.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_SEARCH_PEAK
        /// RSSPECAN_ATTR_MARKER_SEARCH_PEAK_NEXT
        /// RSSPECAN_ATTR_MARKER_SEARCH_PEAK_RIGHT
        /// RSSPECAN_ATTR_MARKER_SEARCH_PEAK_LEFT
        /// RSSPECAN_ATTR_MARKER_SEARCH_MIN
        /// RSSPECAN_ATTR_MARKER_SEARCH_MIN_NEXT
        /// RSSPECAN_ATTR_MARKER_SEARCH_MIN_RIGHT
        /// RSSPECAN_ATTR_MARKER_SEARCH_MIN_LEFT
        /// RSSPECAN_ATTR_SPEM_MARKER_SEARCH_XY
        /// RSSPECAN_ATTR_SPEM_MARKER_SEARCH_Y
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2|3&gt;:MARKer&lt;1..4&gt;:MAX:PEAK | NEXT | LEFT | RIGHt
        /// CALCulate&lt;1|2|3&gt;:MARKer&lt;1..4&gt;:MIN:PEAK | NEXT | LEFT | RIGHt
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:SPECtrogram:Y:MAXimum[:PEAK]
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:SPECtrogram:Y:MAXimum:NEXT
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:SPECtrogram:Y:MAXimum:ABOVe
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:SPECtrogram:Y:MAXimum:BELow
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:SPECtrogram:Y:MINimum[:PEAK]
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:SPECtrogram:Y:MINimum:NEXT
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:SPECtrogram:Y:MINimum:ABOVe
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:SPECtrogram:Y:MINimum:BELow
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:SPECtrogram:XY:MAXimum[:PEAK]
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:SPECtrogram:XY:MINimum[:PEAK]
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Marker_Number">
        /// Selects the marker.
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Search_Mode">
        /// This control positions the marker to the desired position in the trace memory.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SPECM_MARK_MAX_PEAK     (0) - X Peak Maximum 
        /// RSSPECAN_VAL_SPECM_MARK_MAX_NEXT     (1) - X Next Maximum
        /// RSSPECAN_VAL_SPECM_MARK_MAX_LEFT     (2) - X Left Maximum
        /// RSSPECAN_VAL_SPECM_MARK_MAX_RIGHT    (3) - X Right Maximum
        /// RSSPECAN_VAL_SPECM_MARK_MIN_PEAK     (4) - X Peak Minimum
        /// RSSPECAN_VAL_SPECM_MARK_MIN_NEXT     (5) - X Next Minimum
        /// RSSPECAN_VAL_SPECM_MARK_MIN_LEFT     (6) - X Left Minimum
        /// RSSPECAN_VAL_SPECM_MARK_MIN_RIGHT    (7) - X Right Minimum
        /// RSSPECAN_VAL_SPECM_MARK_Y_MAX_PEAK   (8) - Y Peak Maximum
        /// RSSPECAN_VAL_SPECM_MARK_Y_MAX_NEXT   (9) - Y Next Maximum
        /// RSSPECAN_VAL_SPECM_MARK_Y_MAX_ABOVE (10) - Y Above Maximum
        /// RSSPECAN_VAL_SPECM_MARK_Y_MAX_BELOW (11) - Y Below Maximum
        /// RSSPECAN_VAL_SPECM_MARK_Y_MIN_PEAK  (12) - Y Peak Minimum
        /// RSSPECAN_VAL_SPECM_MARK_Y_MIN_NEXT  (13) - Y Next Minimum
        /// RSSPECAN_VAL_SPECM_MARK_Y_MIN_ABOVE (14) - Y Above Minimum
        /// RSSPECAN_VAL_SPECM_MARK_Y_MIN_BELOW (15) - Y Below Minimum
        /// RSSPECAN_VAL_SPECM_MARK_XY_MAX      (16) - XY Maximum
        /// RSSPECAN_VAL_SPECM_MARK_XY_MIN      (17) - XY Minimum
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_SEARCH_PEAK
        /// RSSPECAN_ATTR_MARKER_SEARCH_PEAK_NEXT
        /// RSSPECAN_ATTR_MARKER_SEARCH_PEAK_RIGHT
        /// RSSPECAN_ATTR_MARKER_SEARCH_PEAK_LEFT
        /// RSSPECAN_ATTR_MARKER_SEARCH_MIN
        /// RSSPECAN_ATTR_MARKER_SEARCH_MIN_NEXT
        /// RSSPECAN_ATTR_MARKER_SEARCH_MIN_RIGHT
        /// RSSPECAN_ATTR_MARKER_SEARCH_MIN_LEFT
        /// RSSPECAN_ATTR_SPEM_MARKER_SEARCH_XY
        /// RSSPECAN_ATTR_SPEM_MARKER_SEARCH_Y
        /// 
        /// Note(s):
        /// 
        /// (1) Y Peak Maximum: Positions the specified marker on the current maximum in y?axis direction (constant x?axis value).
        /// 
        /// (2) Y Next Maximum: Positions the specified marker on the next maximum below or above its current position (constant x?axis value). 
        /// 
        /// (3) Y Above Maximum: Positions the specified marker on the next maximum above its current position (constant x?axis value).
        /// 
        /// (4) Y Below Maximum: Positions the specified marker on the next maximum below its current position. (constant x?axis value). 
        /// 
        /// (5) Y Peak Minimum: Positions the specified marker on the current minimum in y?axis direction (constant x?axis value).
        /// 
        /// (6) Y Next Minimum: Positions the specified marker on the next minimum below or above its current position (constant x?axis value). 
        /// 
        /// (7) Y Above Minimum: Positions the specified marker on the next minimum above its current position (constant x?axis value).
        /// 
        /// (8) Y Below Minimum: Positions the specified marker on the next minimum below its current position. (constant x?axis value). 
        /// 
        /// (9) XY Maximum: Positions the specified marker on the current maximum of the search area.
        /// 
        /// (10) XY Minimum: Positions the specified marker on the current minimum of the search area.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SpectrogramMarkerSearch(int Marker_Number, int Search_Mode)
        {
            int pInvokeResult = PInvoke.SpectrogramMarkerSearch(this._handle, Marker_Number, Search_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the frame number of the specified delta marker.
        /// 
        /// Note:
        /// 
        /// This function is only available with the R&amp;S FSL with FS-K14 option. 
        /// 
        /// Attribute (s):
        /// RSSPECAN_ATTR_SPEM_DELT_FRAME_NUMBER
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:SPECtrogram:FRAMe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Delta_Marker_Number">
        /// Selects the delta marker to configure.
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Frame_Number">
        /// This control defines the frame number of the specified delta marker.
        /// 
        /// Valid Values: not checked
        /// (0 to &lt;number of recorded frames&gt;)
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SPEM_DELT_FRAME_NUMBER
        /// 
        /// Note:
        /// (1) Value can be positive or negative depending on reference marker position.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSpectrogramDeltaMarkerFrameNumber(int Delta_Marker_Number, int Frame_Number)
        {
            int pInvokeResult = PInvoke.ConfigureSpectrogramDeltaMarkerFrameNumber(this._handle, Delta_Marker_Number, Frame_Number);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the frame number of the specified delta marker.
        /// 
        /// Note:
        /// 
        /// This function is only available with the R&amp;S FSL with FS-K14 option. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SPEM_DELT_FRAME_NUMBER
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:SPECtrogram:FRAMe?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Delta_Marker">
        /// Selects the delta marker to configure.
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Frame_Number">
        /// Returns the frame number of marker.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetSpectrogramDeltaMarkerFrameNumber(int Delta_Marker, out int Frame_Number)
        {
            int pInvokeResult = PInvoke.GetSpectrogramDeltaMarkerFrameNumber(this._handle, Delta_Marker, out Frame_Number);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the search area of the specified delta marker.
        /// 
        /// Note:
        /// 
        /// This function is only available with the R&amp;S FSL with FS-K14 option. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SPEM_DELT_SEARCH_AREA
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:SPECtrogram:SARea
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Delta_Marker">
        /// Selects the marker to configure.
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Search_Area">
        /// This control sets teh search area.
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_SPEM_SAR_MEMORY  (0) - Memory
        /// RSSPECAN_VAL_SPEM_SAR_VISIBLE (1) - Visible 
        /// 
        /// Default Value: RSSPECAN_VAL_SPEM_SAR_VISIBLE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SPEM_DELT_SEARCH_AREA
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureSpectrogramDeltaMarkerSearchArea(int Delta_Marker, int Search_Area)
        {
            int pInvokeResult = PInvoke.ConfigureSpectrogramDeltaMarkerSearchArea(this._handle, Delta_Marker, Search_Area);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function positions the specified delta marker on the specified place.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_REFERENCE_MARKER_PEAK
        /// RSSPECAN_ATTR_REFERENCE_MARKER_PEAK_NEXT
        /// RSSPECAN_ATTR_REFERENCE_MARKER_PEAK_NEXT_RIGHT
        /// RSSPECAN_ATTR_REFERENCE_MARKER_PEAK_NEXT_LEFT
        /// RSSPECAN_ATTR_REFERENCE_MARKER_MIN
        /// RSSPECAN_ATTR_REFERENCE_MARKER_MIN_NEXT
        /// RSSPECAN_ATTR_REFERENCE_MARKER_MIN_NEXT_RIGHT
        /// RSSPECAN_ATTR_REFERENCE_MARKER_MIN_NEXT_LEFT
        /// RSSPECAN_ATTR_SPEM_DELT_SEARCH_Y
        /// RSSPECAN_ATTR_SPEM_DELT_SEARCH_XY
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:MAXimum[:PEAK]
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:MAXimum:NEXT
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:MAXimum:RIGHt
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:MAXimum:LEFT
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:MINimum[:PEAK]
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:MINimum:NEXT
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:MINimum:RIGHt
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:MINimum:LEFT
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:SPECtrogram:Y:MAXimum[:PEAK]
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:SPECtrogram:Y:MAXimum:NEXT
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:SPECtrogram:Y:MAXimum:ABOVe
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:SPECtrogram:Y:MAXimum:BELow
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:SPECtrogram:Y:MINimum[:PEAK]
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:SPECtrogram:Y:MINimum:NEXT
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:SPECtrogram:Y:MINimum:ABOVe
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:SPECtrogram:Y:MINimum:BELow
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:SPECtrogram:XY:MAXimum[:PEAK]
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:SPECtrogram:XY:MINimum[:PEAK]
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Delta_Marker">
        /// Selects the marker.
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Search_Mode">
        /// This control positions the delta marker to the desired position in the trace memory.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SPECM_MARK_MAX_PEAK     (0) - X Peak Maximum 
        /// RSSPECAN_VAL_SPECM_MARK_MAX_NEXT     (1) - X Next Maximum
        /// RSSPECAN_VAL_SPECM_MARK_MAX_LEFT     (2) - X Left Maximum
        /// RSSPECAN_VAL_SPECM_MARK_MAX_RIGHT    (3) - X Right Maximum
        /// RSSPECAN_VAL_SPECM_MARK_MIN_PEAK     (4) - X Peak Minimum
        /// RSSPECAN_VAL_SPECM_MARK_MIN_NEXT     (5) - X Next Minimum
        /// RSSPECAN_VAL_SPECM_MARK_MIN_LEFT     (6) - X Left Minimum
        /// RSSPECAN_VAL_SPECM_MARK_MIN_RIGHT    (7) - X Right Minimum
        /// RSSPECAN_VAL_SPECM_MARK_Y_MAX_PEAK   (8) - Y Peak Maximum
        /// RSSPECAN_VAL_SPECM_MARK_Y_MAX_NEXT   (9) - Y Next Maximum
        /// RSSPECAN_VAL_SPECM_MARK_Y_MAX_ABOVE (10) - Y Above Maximum
        /// RSSPECAN_VAL_SPECM_MARK_Y_MAX_BELOW (11) - Y Below Maximum
        /// RSSPECAN_VAL_SPECM_MARK_Y_MIN_PEAK  (12) - Y Peak Minimum
        /// RSSPECAN_VAL_SPECM_MARK_Y_MIN_NEXT  (13) - Y Next Minimum
        /// RSSPECAN_VAL_SPECM_MARK_Y_MIN_ABOVE (14) - Y Above Minimum
        /// RSSPECAN_VAL_SPECM_MARK_Y_MIN_BELOW (15) - Y Below Minimum
        /// RSSPECAN_VAL_SPECM_MARK_XY_MAX      (16) - XY Maximum
        /// RSSPECAN_VAL_SPECM_MARK_XY_MIN      (17) - XY Minimum
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_MARKER_PEAK
        /// RSSPECAN_ATTR_REFERENCE_MARKER_PEAK_NEXT
        /// RSSPECAN_ATTR_REFERENCE_MARKER_PEAK_NEXT_RIGHT
        /// RSSPECAN_ATTR_REFERENCE_MARKER_PEAK_NEXT_LEFT
        /// RSSPECAN_ATTR_REFERENCE_MARKER_MIN
        /// RSSPECAN_ATTR_REFERENCE_MARKER_MIN_NEXT
        /// RSSPECAN_ATTR_REFERENCE_MARKER_MIN_NEXT_RIGHT
        /// RSSPECAN_ATTR_REFERENCE_MARKER_MIN_NEXT_LEFT
        /// RSSPECAN_ATTR_SPEM_DELT_SEARCH_Y
        /// RSSPECAN_ATTR_SPEM_DELT_SEARCH_XY
        /// 
        /// Note(s):
        /// 
        /// (1) Y Peak Maximum: Positions the specified delta marker on the current maximum in y?axis direction (constant x?axis value).
        /// 
        /// (2) Y Next Maximum: Positions the specified delta marker on the next maximum below or above its current position (constant x?axis value). 
        /// 
        /// (3) Y Above Maximum: Positions the specified delta marker on the next maximum above its current position (constant x?axis value).
        /// 
        /// (4) Y Below Maximum: Positions the specified delta marker on the next maximum below its current position. (constant x?axis value). 
        /// 
        /// (5) Y Peak Minimum: Positions the specified delta marker on the current minimum in y?axis direction (constant x?axis value).
        /// 
        /// (6) Y Next Minimum: Positions the specified delta marker on the next minimum below or above its current position (constant x?axis value). 
        /// 
        /// (7) Y Above Minimum: Positions the specified delta marker on the next minimum above its current position (constant x?axis value).
        /// 
        /// (8) Y Below Minimum: Positions the specified delta marker on the next minimum below its current position. (constant x?axis value). 
        /// 
        /// (9) XY Maximum: Positions the specified delta marker on the current maximum of the search area.
        /// 
        /// (10) XY Minimum: Positions the specified delta marker on the current minimum of the search area.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SpectrogramDeltaMarkerSearch(int Delta_Marker, int Search_Mode)
        {
            int pInvokeResult = PInvoke.SpectrogramDeltaMarkerSearch(this._handle, Delta_Marker, Search_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the time stamp of the frames.
        /// 
        /// Note: 
        /// 
        /// (1) This function is available only with R&amp;S FSL with FS-K14 option.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:SPECtrogram:TSTamp:DATA? CURRent | ALL
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Frame">
        /// Defimes the frame for time stamp.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SPECM_TSTAMP_CURR (0) - Current (Default Value)
        /// RSSPECAN_VAL_SPECM_TSTAMP_ALL  (1) - All
        /// 
        /// Note:
        /// 
        /// (1) Current - returns the time stamp of the current frame.
        /// 
        /// (2) All - returns the time stamp of all frames sorted in a descending order, starting with the current frame.
        /// </param>
        /// <param name="Array_Size">
        /// This control sets the number of values returned in arrays.
        /// The array size has to be at least equal to this number.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Part_1">
        /// Returns array of part 1 od time stamp of frame(s). If All frames is selected, all frames sorted in a descending order, starting with the current frame is returned.
        /// 
        /// The value indicates the number of seconds from 1.1.1970
        /// </param>
        /// <param name="Part_2">
        /// Returns array of part 2 od time stamp of frame(s). If All frames is selected, all frames sorted in a descending order, starting with the current frame is returned.
        /// 
        /// The value indicates the miliseconds of time stamp.
        /// </param>
        /// <param name="Part_3">
        /// Returns array of part 3 od time stamp of frame(s). If All frames is selected, all frames sorted in a descending order, starting with the current frame is returned.
        /// 
        /// </param>
        /// <param name="Part_4">
        /// Returns array of part 4 od time stamp of frame(s). If All frames is selected, all frames sorted in a descending order, starting with the current frame is returned.
        /// 
        /// </param>
        /// <param name="Returned_Values">
        /// Returns the number of time stamps returned from the instrument.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetSpectrogramTimeStamp(int Frame, int Array_Size, int[] Part_1, int[] Part_2, int[] Part_3, int[] Part_4, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.GetSpectrogramTimeStamp(this._handle, Frame, Array_Size, Part_1, Part_2, Part_3, Part_4, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the frame results. The frames are sorted according to their time stamp in descending order (starting with the current frame).
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S FSL with FS-K14 option.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FORMat[:DATA] REAL,32
        /// TRACe&lt;1|2&gt;[:DATA]? SPECtogram
        /// *CLS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Array_Size">
        /// This control sets the number of values returned in arrays.
        /// The array size has to be at least equal to this number.
        /// 
        /// Valid Values: &gt; 0
        /// 
        /// Default Value: 1024
        /// 
        /// 
        /// </param>
        /// <param name="Results">
        /// Returns an array of frame results. The frames are sorted according to their time stamp in descending order (starting with the current frame).
        /// </param>
        /// <param name="Returned_Values">
        /// Returns number of value returned from the instrument. Number of values returned in arrays is limited with the parameter Array Size.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadSpectrogramTraceData(int Array_Size, double[] Results, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.ReadSpectrogramTraceData(this._handle, Array_Size, Results, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the avionics measurements operating mode.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the FS-K15 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AVI_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument[:SELect] AVIonics
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int AvionicsMode()
        {
            int pInvokeResult = PInvoke.AvionicsMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the avionics demodulation mode.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the R&amp;S FS-K15 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AVI_DEMOD_MODE
        /// 
        /// Remote-control command(s):
        /// CALCulate:AVIonics[:STANdard] VOR | ILS
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Demodulation_Mode">
        /// This control selects the avionics demodulation mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_AVI_STAN_VOR (0) - VOR demodulation
        /// RSSPECAN_VAL_AVI_STAN_ILS (1) - ILS demodulation
        /// 
        /// Default Value: RSSPECAN_VAL_AVI_STAN_VOR (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AVI_DEMOD_MODE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAvionicsDemodulationMode(int Demodulation_Mode)
        {
            int pInvokeResult = PInvoke.ConfigureAvionicsDemodulationMode(this._handle, Demodulation_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the type of evaluation of the measured data. 
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the FS-K15 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AVI_DISTORTION_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FEED &lt;string&gt;
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Type">
        /// This control sets the type of evaluation of the measured data. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_AVI_FEED_SPEC (0) - FFT display of the modulation spectrum
        /// RSSPECAN_VAL_AVI_FEED_AMS  (1) - AM-demodulation results in tables
        /// 
        /// Default Value: RSSPECAN_VAL_AVI_FEED_AMS  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AVI_DISTORTION_RESULT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAvionicsDistortionResult(int Type)
        {
            int pInvokeResult = PInvoke.ConfigureAvionicsDistortionResult(this._handle, Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects between % and dB for displaying the THD measurement result.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the FS-K15 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_ADEM_UNIT_THD
        /// 
        /// Remote-control command(s):
        /// UNIT:THD PCT | DB
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Unit">
        /// This control selects between % and dB for displaying the
        /// THD measurement result.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DB  (8) - dB
        /// RSSPECAN_VAL_UNIT_PCT (13) - %
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_DB (8)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ADEM_UNIT_THD
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAvionicsTHDUnit(int Unit)
        {
            int pInvokeResult = PInvoke.ConfigureAvionicsTHDUnit(this._handle, Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the unit for ILS DDM reading.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the FS-K15 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AVI_DDM_UNIT
        /// 
        /// Remote-control command(s):
        /// UNIT&lt;1|2&gt;:DDM UNITless | PCT
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Unit">
        /// This control selects the unit for ILS DDM reading.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_PCT      (13) - %
        /// RSSPECAN_VAL_UNIT_UNITLESS (14) - Unitless
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_UNITLESS (14)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AVI_DDM_UNIT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAvionicsDDMUnit(int Unit)
        {
            int pInvokeResult = PInvoke.ConfigureAvionicsDDMUnit(this._handle, Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the audio input - and thus the BASEBAND mode - on (AUDio) or off (RF).
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the FS-K15 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AVI_INPUT
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:SELect AIQ | AUDio | RF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Input">
        /// This control selects the audio input - and thus the BASEBAND mode - on (AUDio) or off (RF).
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_BB_INPUT_RF  (0) - RF
        /// RSSPECAN_VAL_BB_INPUT_AIQ (1) - AIQ
        /// RSSPECAN_VAL_BB_INPUT_AUD (3) - Audio
        /// 
        /// Default Value: RSSPECAN_VAL_BB_INPUT_RF (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AVI_INPUT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAvionicsInput(int Window, int Input)
        {
            int pInvokeResult = PInvoke.ConfigureAvionicsInput(this._handle, Window, Input);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the demodulation bandwidth settings.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the FS-K15 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AVI_DEMOD_BWID_AUTO
        /// RSSPECAN_ATTR_AVI_DEMOD_BWID
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:DEMod:AUTO ON | OFF
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:DEMod
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Auto">
        /// Enables or disables the automatic setting of the demodulation bandwidth.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AVI_DEMOD_BWID_AUTO
        /// </param>
        /// <param name="Demodulation_Bandwidth">
        /// This control defines the demodulation bandwidth of the instrument for analog demodulation. The required sampling rate is automatically set depending on the selected demodulation bandwidth. The available demodulation bandwidths are determined by the selected standard (ILS, VOR). Setting a value is only available in ILS demodulation mode. The bandwidth can be set to 12.5 kHz, 3.2 kHz and 800 Hz. This function can be used to suppress adjacent transmitters.
        /// 
        /// Valid Values: 800 Hz to 12.5 kHz
        /// 
        /// Default Value: 800.0 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AVI_DEMOD_BWID
        /// 
        /// Note(s):
        /// 
        /// (1) This control is ignored if Auto control is set to ON.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAvionicsDemodulationBandwidth(int Window, bool Auto, double Demodulation_Bandwidth)
        {
            int pInvokeResult = PInvoke.ConfigureAvionicsDemodulationBandwidth(this._handle, Window, System.Convert.ToUInt16(Auto), Demodulation_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects 4 V or 400 mV as the input voltage range for the audio input and the input voltage for 100% modulation depth at the audio input.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the R&amp;S FS-K15 option and FSMR instrument model.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AVI_BB_INPUT_RANGE
        /// RSSPECAN_ATTR_AVI_BB_INPUT_SENS
        /// 
        /// Remote-control command(s):
        /// [SENSe:]VOLTage:AC:RANGe[:UPPer]
        /// [SENSe:]VOLTage:AC:SENSitivity
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Level">
        /// This control selects 4 V or 400 mV as the input voltage range for the audio input.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_AVI_INPUT_VOL_4V    (0) - 4V
        /// RSSPECAN_VAL_AVI_INPUT_VOL_400MV (1) - 400mV
        /// 
        /// Default Value: RSSPECAN_VAL_AVI_INPUT_VOL_4V (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AVI_BB_INPUT_RANGE
        /// </param>
        /// <param name="Sensitivity">
        /// This control sets the sensitivity of an (external) AM modulator that could be or is driven by the applied baseband signal. The entered sensitivity [Vpeak/100%AM] is used to convert the measured voltages into modulation depth. 1 V peak voltage corresponds to 100 % AM.
        /// 
        /// Valid Values: 0.1 V to 5V
        /// 
        /// Default Value: 1.0 V
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AVI_BB_INPUT_SENS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAvionicsBasebandInput(int Level, double Sensitivity)
        {
            int pInvokeResult = PInvoke.ConfigureAvionicsBasebandInput(this._handle, Level, Sensitivity);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the frequency range of the result display AF spectrum using start frequency and stop frequency.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the FS-K15 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_AF_START
        /// RSSPECAN_ATTR_FMDEM_AF_STOP
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:AF:STARt
        /// [SENSe:]ADEMod:AF:STOP
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Start_Frequency">
        /// Sets the start frequency for result display AF spectrum.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 0.0 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_AF_START
        /// </param>
        /// <param name="Stop_Frequency">
        /// Sets the stop frequency for result display AF spectrum.
        /// The stop frequency is limited to half the measurement bandwidth of demodulation.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 12.5e3 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_AF_STOP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAvionicsAFFrequencyStartStop(double Start_Frequency, double Stop_Frequency)
        {
            int pInvokeResult = PInvoke.ConfigureAvionicsAFFrequencyStartStop(this._handle, Start_Frequency, Stop_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the frequency range of the result display AF spectrum using the center frequency and the frequency span. This function modifies the AF Frequency Start and AF Frequency Stop attributes as follows:
        /// 
        ///     AF Frequency Start = CenterFrequency - span/2
        ///     AF Frequency Stop = CenterFrequency + span/2
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the FS-K15 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_AF_CENTER
        /// RSSPECAN_ATTR_FMDEM_AF_SPAN
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:AF:CENTer
        /// [SENSe:]ADEMod:AF:SPAN
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Center_Frequency">
        /// Sets the center frequency for result display AF spectrum.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 6.25e3 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_AF_CENTER
        /// </param>
        /// <param name="Span">
        /// Sets the span for result display AF spectrum.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 12.5e3
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_AF_SPAN
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAvionicsAFFrequencyCenterSpan(double Center_Frequency, double Span)
        {
            int pInvokeResult = PInvoke.ConfigureAvionicsAFFrequencyCenterSpan(this._handle, Center_Frequency, Span);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the maximum span for the display of the AF spectrum.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the FS-K15 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_AF_FULL_SPAN
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:AF:SPAN:FULL
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAvionicsAFFullSpan()
        {
            int pInvokeResult = PInvoke.ConfigureAvionicsAFFullSpan(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures bandwidth resolutionfor display of the AF spectrum.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the FS-K15 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FMDEM_BAND_RES
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:SPECtrum:BANDwidth:RESolution
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Resolution_Bandwidth">
        /// This control sets the resolution bandwidth for spectrum representation that was determined from the analog demodulation data. The recording time required is calculated from bandwidth.  If the available recording time is not sufficient for the given bandwidth, the recording time is set to is maximum and the resolution bandwidth is enlarged to the resulting bandwidth.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 3.8 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FMDEM_BAND_RES
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAvionicsAFParam(double Resolution_Bandwidth)
        {
            int pInvokeResult = PInvoke.ConfigureAvionicsAFParam(this._handle, Resolution_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the distortion measurement on or off. Configures the measurement frequency of the harmonic distortion measurement to a single signal within the audio frequency range.
        /// Also configures the fundamental frequency of the harmonic distortion measurement and the maximum frequency of the harmonic distortion measurement.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the FS-K15 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AVI_HARM_DIST_STATE
        /// RSSPECAN_ATTR_AVI_HARM_DIST_FREQ
        /// RSSPECAN_ATTR_AVI_THD_FUND_FREQ
        /// RSSPECAN_ATTR_AVI_THD_MAX_FREQ
        /// 
        /// Remote-control command(s):
        /// CALCulate:AVIonics:SHD[:STATe]
        /// CALCulate:AVIonics:SHD:FREQuency
        /// CALCulate:AVIonics:THD:FREQuency:FUNDament
        /// CALCulate:AVIonics:THD:FREQuency:UPPer
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="SHD_State">
        /// Enables or disables the distortion measurement.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0) 
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AVI_HARM_DIST_STATE
        /// </param>
        /// <param name="SHD_Harmonic_Frequency">
        /// This control sets the measurement frequency of the harmonic distortion measurement to a single signal within the audio frequency range. The limit value fmax is half of the demodulation bandwidth.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 750.0 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AVI_HARM_DIST_FREQ
        /// </param>
        /// <param name="THD_Fundamental_Frequency">
        /// This control sets the fundamental frequency of the harmonic distortion measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_AVI_THD_FUND_FREQ_90     (0) - ILS 90 Hz
        /// RSSPECAN_VAL_AVI_THD_FUND_FREQ_150    (1) - ILS 150 Hz
        /// RSSPECAN_VAL_AVI_THD_FUND_FREQ_90_150 (2) - Average of ILS 90 and 150 Hz
        /// RSSPECAN_VAL_AVI_THD_FUND_FREQ_ID     (3) - Identifier Signal (300 to 4000 Hz)
        /// 
        /// Default Value: RSSPECAN_VAL_AVI_THD_FUND_FREQ_90_150 (2)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AVI_THD_FUND_FREQ
        /// </param>
        /// <param name="THD_Max_Frequency">
        /// This control sets the maximum frequency of the harmonic distortion measurement.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 750.0 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AVI_THD_MAX_FREQ
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureAvionicsDistortionMeasurement(bool SHD_State, double SHD_Harmonic_Frequency, int THD_Fundamental_Frequency, double THD_Max_Frequency)
        {
            int pInvokeResult = PInvoke.ConfigureAvionicsDistortionMeasurement(this._handle, System.Convert.ToUInt16(SHD_State), SHD_Harmonic_Frequency, THD_Fundamental_Frequency, THD_Max_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads RF frequency.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K15 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AVI_RF_FREQ_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate1:AVIonics:RFFRequency:RESult?
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="RF_Frequency_Result">
        /// Returns the result value. The driver uses this control to get the RSSPECAN_ATTR_AVI_RF_FREQ_RESULT attribute.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryAvionicsRFFrequency(out double RF_Frequency_Result)
        {
            int pInvokeResult = PInvoke.QueryAvionicsRFFrequency(this._handle, out RF_Frequency_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads Carrier offset, this mean frequency offset between measured frequency and tuned frequency.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K15 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AVI_CARR_OFFS_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate1:AVIonics:FERRor:RESult?
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Carrier_Offset_Result">
        /// Returns the result value. The driver uses this control to get the RSSPECAN_ATTR_AVI_CARR_OFFS_RESULT attribute.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryAvionicsCarrierOffset(out double Carrier_Offset_Result)
        {
            int pInvokeResult = PInvoke.QueryAvionicsCarrierOffset(this._handle, out Carrier_Offset_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads the RF level.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K15 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AVI_RF_LEVEL_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate1:AVIonics:CARRier:RESult?
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="RF_Level_Result">
        /// Returns the result value. The driver uses this control to get the RSSPECAN_ATTR_AVI_RF_LEVEL_RESULT attribute.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryAvionicsRFLevel(out double RF_Level_Result)
        {
            int pInvokeResult = PInvoke.QueryAvionicsRFLevel(this._handle, out RF_Level_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the modulation depth and AF frequency measurement at the frequency modulated VOR/ILS signal component.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the FS-K15 option.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate1:AVIonics:AM:DEPTh? 'string'
        /// CALCulate1:AVIonics:AM:FREQuency? 'string'
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="AM_Depth">
        /// This control selects the modulation depth of VOR/ILS signal component.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAR_AVI_DEMOD_SUMM_30     (0) - 30 Hz (VOR)
        /// RSSPECAN_VAR_AVI_DEMOD_SUMM_996    (1) - 9.96 kHz (VOR)
        /// RSSPECAN_VAR_AVI_DEMOD_SUMM_ID     (2) - Identifier Signal (VOR + ILS)
        /// RSSPECAN_VAR_AVI_DEMOD_SUMM_90     (3) - 90 Hz (ILS)
        /// RSSPECAN_VAR_AVI_DEMOD_SUMM_150    (4) - 150 Hz (ILS)
        /// RSSPECAN_VAR_AVI_DEMOD_SUMM_90_150 (5) - 90 + 150 Hz (ILS)
        /// 
        /// Default Value:RSSPECAN_VAR_AVI_DEMOD_SUMM_30     (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Values 30 Hz, 9.96 kHz and Identifier Signal are only avaible when VOR demodulation is switched on.
        /// 
        /// (2) Values 90 Hz, 150 Hz, 90 + 150 Hz and Identifier Signal are only avaible when ILS demodulation is switched on.
        /// </param>
        /// <param name="AM_Depth_Result">
        /// Returns the result value for AM depth.
        /// </param>
        /// <param name="AM_Frequency">
        /// This control selects the AF frequency measurement at the frequency modulated VOR/ILS signal component.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAR_AVI_DEMOD_SUMM_30     (0) - 30 Hz (VOR)
        /// RSSPECAN_VAR_AVI_DEMOD_SUMM_996    (1) - 9.96 kHz (VOR)
        /// RSSPECAN_VAR_AVI_DEMOD_SUMM_ID     (2) - Identifier Signal (VOR + ILS)
        /// RSSPECAN_VAR_AVI_DEMOD_SUMM_90     (3) - 90 Hz (ILS)
        /// RSSPECAN_VAR_AVI_DEMOD_SUMM_150    (4) - 150 Hz (ILS)
        /// 
        /// Default Value:RSSPECAN_VAR_AVI_DEMOD_SUMM_30     (0) 
        /// 
        /// Note(s):
        /// 
        /// (1) Values 30 Hz, 9.96 kHz and Identifier Signal are only avaible when VOR demodulation is switched on.
        /// 
        /// (2) Values 90 Hz, 150 Hz and Identifier Signal are only avaible when ILS demodulation is switched on.
        /// </param>
        /// <param name="AM_Frequency_Result">
        /// Returns the result value AM frequency.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryAvionicsAM(int AM_Depth, out double AM_Depth_Result, int AM_Frequency, out double AM_Frequency_Result)
        {
            int pInvokeResult = PInvoke.QueryAvionicsAM(this._handle, AM_Depth, out AM_Depth_Result, AM_Frequency, out AM_Frequency_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the distortion measurement.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the FS-K15 option.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate1:AVIonics:THD:RESult? 'string'
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="THD">
        /// This control selects the THD signal component.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAR_AVI_DEMOD_SUMM_30     (0) - 30 Hz (VOR)
        /// RSSPECAN_VAR_AVI_DEMOD_SUMM_ID     (2) - Identifier Signal (VOR)
        /// RSSPECAN_VAR_AVI_DEMOD_SUMM_90_150 (5) - 90 + 150 Hz (ILS)
        /// 
        /// Default Value:RSSPECAN_VAR_AVI_DEMOD_SUMM_30     (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Values 30 Hz and Identifier Signal are only avaible when VOR demodulation is switched on.
        /// 
        /// (2) Value 90 + 150 Hz is only avaible when ILS demodulation is switched on.
        /// </param>
        /// <param name="THD_Result">
        /// Returns the result value.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryAvionicsTHD(int THD, out double THD_Result)
        {
            int pInvokeResult = PInvoke.QueryAvionicsTHD(this._handle, THD, out THD_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads DDM (= difference in depth of modulation) between 90 Hz and 150 Hz AM signal (m90 Hz - m150 Hz). Only available when ILS demodulation is switched on.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K15 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AVI_DDM_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate1:AVIonics:DDM?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="DDM_Result">
        /// Returns the result value. The driver uses this control to get the RSSPECAN_ATTR_AVI_DDM_RESULT attribute.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryAvionicsDifferenceInDepth(out double DDM_Result)
        {
            int pInvokeResult = PInvoke.QueryAvionicsDifferenceInDepth(this._handle, out DDM_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads frequency deviation and AF frequency measurement at the frequency modulated VOR 30 Hz signal component. Only available when VOR demodulation is switched on.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K15 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AVI_FM_DEVIATION_RESULT
        /// RSSPECAN_ATTR_AVI_FM_FREQ_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate1:AVIonics:FM:DEViation?
        /// CALCulate1:AVIonics:FM:FREQuency?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="FM_Deviation_Result">
        /// Returns the result value. The driver uses this control to get the RSSPECAN_ATTR_AVI_FM_DEVIATION_RESULT attribute.
        /// </param>
        /// <param name="FM_Frequency_Result">
        /// Returns the result value. The driver uses this control to get the RSSPECAN_ATTR_AVI_FM_FREQ_RESULT attribute.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryAvionicsFM(out double FM_Deviation_Result, out double FM_Frequency_Result)
        {
            int pInvokeResult = PInvoke.QueryAvionicsFM(this._handle, out FM_Deviation_Result, out FM_Frequency_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads VOR or ILS Phase.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K15 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AVI_PHASE_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate1:AVIonics:PHASe?
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Phase_Result">
        /// Returns the result value. The driver uses this control to get the RSSPECAN_ATTR_AVI_PHASE_RESULT attribute.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryAvionicsPhase(out double Phase_Result)
        {
            int pInvokeResult = PInvoke.QueryAvionicsPhase(this._handle, out Phase_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads input level(Pk).
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K15 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AVI_INPUT_LEVEL_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate1:AVIonics:POWer:AC:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Input_Level_Result">
        /// Returns the result value. The driver uses this control to get the RSSPECAN_ATTR_AVI_INPUT_LEVEL_RESULT attribute.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryAvionicsInputLevel(out double Input_Level_Result)
        {
            int pInvokeResult = PInvoke.QueryAvionicsInputLevel(this._handle, out Input_Level_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads selective measurement of the sum-modulation depth (SDM = Sum in Depth of Modulation); arithmetic sum of the modulation depth of the 90 Hz and the 150 Hz components without any influence of the phase between the components.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K15 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AVI_SDM_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate1:AVIonics:SDM?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="SDM_Result">
        /// Returns the result value. The driver uses this control to get the RSSPECAN_ATTR_AVI_SDM_RESULT attribute.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryAvionicsSDM(out double SDM_Result)
        {
            int pInvokeResult = PInvoke.QueryAvionicsSDM(this._handle, out SDM_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads the result of the manual distortion measurement.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K15 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AVI_SHD_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate1:AVIonics:SHD:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="SHD_Result">
        /// Returns the result value. The driver uses this control to get the RSSPECAN_ATTR_AVI_SHD_RESULT attribute.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryAvionicsSHD(out double SHD_Result)
        {
            int pInvokeResult = PInvoke.QueryAvionicsSHD(this._handle, out SHD_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the Cable TV Measurements operating mode.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument:SELect CATV
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CATVMode()
        {
            int pInvokeResult = PInvoke.CATVMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects TV channel table.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_TV_CHANNEL
        /// RSSPECAN_ATTR_CATV_CHANNEL
        /// 
        /// Remote-control command(s):
        /// CONFigure:TV:CTABle:SELect
        /// [SENSe&lt;1|2&gt;:]FREQuency:CHANnel
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Channel_Table_Name">
        /// This control sets the channel table.
        /// 
        /// Valid Values:
        /// any file name (max. 45 chars)
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_TV_CHANNEL
        /// 
        /// Note:
        /// 
        /// (1) "" (Empty string) means none channel table is selected.
        /// 
        /// (2) If table with selected name doesn't exist, it will be created
        /// </param>
        /// <param name="Measurement_Channel">
        /// This control sets the active measurement channel from channels. Valid values depend on selected Channel Table.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_CHANNEL
        /// 
        /// Note:
        /// 
        /// (1) This control has no effect if Channel Table Name is "" (empty string)
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVChannelSetup(string Channel_Table_Name, int Measurement_Channel)
        {
            int pInvokeResult = PInvoke.ConfigureCATVChannelSetup(this._handle, Channel_Table_Name, Measurement_Channel);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures TV channel frequency.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_RF_FREQUENCY
        /// RSSPECAN_ATTR_CATV_SWEEP_SPACING
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:RF
        /// [SENSe&lt;1|2&gt;:]SWEep:SPACing ABSolute | RELative
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="RF_Frequency">
        /// This control sets RF frequency. The interpretation of the RF frequency depends on the signal type. For details refer instrument operating manual, chapter "Instrument Functions", section "Cable TV Measurement Option (K20)", RF softkey.
        /// 
        /// Valid Values:
        /// 5.0e+6 to 1.5e+9 Hz
        /// 
        /// Default Value: 48.25e+6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_RF_FREQUENCY
        /// </param>
        /// <param name="Axis_Labeling">
        /// This control sets the frequency axis labeling.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ABS (0) - Abs
        /// RSSPECAN_VAL_REL (1) - Rel  
        /// 
        /// Default Value: RSSPECAN_VAL_REL (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_SWEEP_SPACING
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVChannelFrequency(double RF_Frequency, int Axis_Labeling)
        {
            int pInvokeResult = PInvoke.ConfigureCATVChannelFrequency(this._handle, RF_Frequency, Axis_Labeling);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures TV channel frequency step size.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// (2) This function is not available if &lt;none&gt; channel is selected
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_RF_FREQUENCY_STEP_SIZE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:STEP
        /// *WAI
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Step_Size">
        /// This control sets the step size of the center frequency.
        /// 
        /// Valid Values:
        /// 1.0 to fmax, no checking
        /// 
        /// Default Value: 1.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_RF_FREQUENCY_STEP_SIZE
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVChannelFrequencyStepSize(double Step_Size)
        {
            int pInvokeResult = PInvoke.ConfigureCATVChannelFrequencyStepSize(this._handle, Step_Size);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures modulation standard.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_MODUL_STANDARD
        /// RSSPECAN_ATTR_CATV_MODUL_STANDARD_SIG_TYPE
        /// RSSPECAN_ATTR_CATV_SIDE_BAND
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]TV:MSTandard:NAME
        /// [SENSe&lt;1|2&gt;:]TV:MSTandard:STYPe ATV | DTV
        /// [SENSe&lt;1|2&gt;:]DDEMod:SBANd AUTO | NORMal | INVerse
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Modulation_Standard">
        /// This control defines the name for a modulation standard.
        /// 
        /// Valid Values:
        /// any string (max. 15 chars)
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_MODUL_STANDARD
        /// </param>
        /// <param name="Signal_Type">
        /// This control sets the signal type of a modulation standard.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_CATV_SIG_TYPE_ATV (0) - Analog TV
        /// RSSPECAN_VAL_CATV_SIG_TYPE_DTV (1) - Digital TV
        /// 
        /// Default Value: RSSPECAN_VAL_CATV_SIG_TYPE_ATV (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_MODUL_STANDARD_SIG_TYPE
        /// </param>
        /// <param name="Sideband_Position">
        /// This control sets the sideband position.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_CATV_SBAND_NORM (0) - Normal
        /// RSSPECAN_VAL_CATV_SBAND_INV  (1) - Inverse
        /// RSSPECAN_VAL_CATV_SBAND_AUTO (2) - Auto 
        /// 
        /// Default Value: RSSPECAN_VAL_CATV_SBAND_NORM (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_SIDE_BAND
        /// 
        /// Note:
        /// 
        /// (1) Auto is available only for Digital TV measurements.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVModulationStandard(string Modulation_Standard, int Signal_Type, int Sideband_Position)
        {
            int pInvokeResult = PInvoke.ConfigureCATVModulationStandard(this._handle, Modulation_Standard, Signal_Type, Sideband_Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function adjusts the input attenuator.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_ACP_PRESET_REF_LEVEL
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:PRESet:RLEVel
        /// *WAI
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CATVChannelAdjustAtt()
        {
            int pInvokeResult = PInvoke.CATVChannelAdjustAtt(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates the automatic scaling for the Hum and Tilt measurement.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_Y_SCALE_AUTO
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...4&gt;:Y[:SCALe]:AUTO
        /// *OPC
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CATVAutoscale()
        {
            int pInvokeResult = PInvoke.CATVAutoscale(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function enables or disables selected marker.
        /// 
        /// Note:
        /// 
        /// This function is only available with the R&amp;S FSL with FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_ENABLED
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Marker_Number">
        /// Selects the marker to configure.
        /// 
        /// Valid Values:
        /// 1 to 4 
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Marker_State">
        /// Enables or disables the selected marker.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_ENABLED
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVMarkerState(int Marker_Number, bool Marker_State)
        {
            int pInvokeResult = PInvoke.ConfigureCATVMarkerState(this._handle, Marker_Number, System.Convert.ToUInt16(Marker_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function enables or disables selected delta marker.
        /// 
        /// Note:
        /// 
        /// This function is only available with the R&amp;S FSL with FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_REFERENCE_MARKER_STATE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;[:STATe]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Delta_Marker_Number">
        /// Selects the marker to configure.
        /// 
        /// Valid Values:
        /// 1 - 4
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Delta_Marker_State">
        /// Enables or disables the selected marker.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_MARKER_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVDeltaMarkerState(int Delta_Marker_Number, bool Delta_Marker_State)
        {
            int pInvokeResult = PInvoke.ConfigureCATVDeltaMarkerState(this._handle, Delta_Marker_Number, System.Convert.ToUInt16(Delta_Marker_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the analog TV measurement type.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_MEAS
        /// 
        /// Remote-control command(s):
        /// CONFigure:ATV:MEASurement ASPectrum | CARRiers | CN | HUM | VMODulation | CTB | CSO | VSCope
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement">
        /// This control selects the measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ATV_MEAS_ASP  (0) - Spectrum (Default Value)
        /// RSSPECAN_VAL_ATV_MEAS_CARR (1) - Carrier Power
        /// RSSPECAN_VAL_ATV_MEAS_CN   (2) - Carrier to Noise Ratio
        /// RSSPECAN_VAL_ATV_MEAS_CSO  (3) - Carrier to Second Order Beat Ratio
        /// RSSPECAN_VAL_ATV_MEAS_CTB  (4) - Carrier to Composite Triple Beat Ratio
        /// RSSPECAN_VAL_ATV_MEAS_HUM  (5) - Hum
        /// RSSPECAN_VAL_ATV_MEAS_VMOD (6) - Vision Modulation
        /// RSSPECAN_VAL_ATV_MEAS_VSC  (7) - Video Scope
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_ATV_MEAS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVAMeasurement(int Measurement)
        {
            int pInvokeResult = PInvoke.ConfigureCATVAMeasurement(this._handle, Measurement);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the analog TV standard options.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_STANDARD
        /// RSSPECAN_ATTR_CATV_ATV_SOUND_SYSTEM
        /// RSSPECAN_ATTR_CATV_MODUL_STANDARD_GDELAY
        /// RSSPECAN_ATTR_CATV_ATV_COLOR
        /// RSSPECAN_ATTR_CATV_ATV_TRIGGER_BAR_FIELD
        /// RSSPECAN_ATTR_CATV_ATV_TRIGGER_BAR_LINE
        /// RSSPECAN_ATTR_CATV_ATV_TRIGGER_BAR_LINE_TYPE
        /// RSSPECAN_ATTR_CATV_ATV_TRIGGER_QLINE_FIELD
        /// RSSPECAN_ATTR_CATV_ATV_TRIGGER_QUIET_LINE
        /// 
        /// Remote-control command(s):
        /// SETup:TV:STANdard BG | DK | I | K1 | L | M | N
        /// SETup:TV:STANdard:AUDio FM55NICAM585 | FM55FM5742 | FM55MONO | FM65NICAM585 | FM65FM6258 | FM65FM6742 | FM65MONO | FM60NICAM6552 | FM60MONO | AM65NICAM585 | AM60MONO | FM45BTSC | FM45EIA_J | FM45FM4742 | FM45MONO
        /// [SENSe&lt;1|2&gt;:]TV:MSTandard:FILTer:GDELay  GENeral | GHALf | AUSTralia | DANMark | NEWZealand | NORWay | SWEDenfull | FLAT | OIRT | CHINa | CCIR | TDF | FFC
        /// SETup:TV:STANdard:COLor PAL | NTSC | SECam
        /// TRIGger&lt;1|2&gt;[:SEQuence]:VIDeo:BFIeld[:NUMBer]
        /// TRIGger&lt;1|2&gt;[:SEQuence]:VIDeo:BLINe[:NUMBer]
        /// TRIGger&lt;1|2&gt;[:SEQuence]:VIDeo:BLINe:TYPE CCIR17 | FCC | NTC7
        /// TRIGger&lt;1|2&gt;[:SEQuence]:VIDeo:QFIeld[:NUMBer]
        /// TRIGger&lt;1|2&gt;[:SEQuence]:VIDeo:QLINe[:NUMBer]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="TV_Standard">
        /// This control sets sets the TV standard.
        /// 
        /// Valid Values:
        /// 
        /// RSSPECAN_VAL_ATV_STAN_BG (0) - B/G 
        /// RSSPECAN_VAL_ATV_STAN_DK (1) - D/K
        /// RSSPECAN_VAL_ATV_STAN_I  (2) - I
        /// RSSPECAN_VAL_ATV_STAN_K1 (3) - K1
        /// RSSPECAN_VAL_ATV_STAN_L  (4) - L
        /// RSSPECAN_VAL_ATV_STAN_M  (5) - M
        /// RSSPECAN_VAL_ATV_STAN_N  (6) - N
        /// 
        /// Default Value: RSSPECAN_VAL_ATV_STAN_BG (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_ATV_STANDARD
        /// </param>
        /// <param name="Sound_System">
        /// This control sets the signal type of a modulation standard.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ATV_AUDIO_FM55NICAM585  (0) - FM 5.5 / NICAM 5.85 
        /// RSSPECAN_VAL_ATV_AUDIO_FM55FM5742    (1) - FM 5.5 / FM 5.742
        /// RSSPECAN_VAL_ATV_AUDIO_FM55MONO      (2) - FM 5.5 / MONO
        /// RSSPECAN_VAL_ATV_AUDIO_FM65NICAM585  (3) - FM 6.5 / NICAM 5.85
        /// RSSPECAN_VAL_ATV_AUDIO_FM65FM6258    (4) - FM 6.5 / FM 6.258
        /// RSSPECAN_VAL_ATV_AUDIO_FM65FM6742    (5) - FM 6.5 / FM 6.742
        /// RSSPECAN_VAL_ATV_AUDIO_FM65MONO      (6) - FM 6.5 / MONO
        /// RSSPECAN_VAL_ATV_AUDIO_FM60NICAM6552 (7) - FM 6.0 / NICAM 6.552
        /// RSSPECAN_VAL_ATV_AUDIO_FM60MONO      (8) - FM 6.0 / MONO
        /// RSSPECAN_VAL_ATV_AUDIO_AM65NICAM585  (9) - AM 6.5 / NICAM 5.85
        /// RSSPECAN_VAL_ATV_AUDIO_AM60MONO     (10) - AM 6.5 / MONO
        /// RSSPECAN_VAL_ATV_AUDIO_FM45BTSC     (11) - FM 4.5 BTSC
        /// RSSPECAN_VAL_ATV_AUDIO_FM45EIA_J    (12) - FM 4.5  EIA_J
        /// RSSPECAN_VAL_ATV_AUDIO_FM45FM4742   (13) - FM 4.5 / FM 4.724
        /// RSSPECAN_VAL_ATV_AUDIO_FM45MONO     (14) - FM 4.5 / MONO
        /// 
        /// Default Value: RSSPECAN_VAL_ATV_AUDIO_FM55NICAM585  (0)
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_SOUND_SYSTEM
        /// 
        /// Note:
        /// 
        /// (1) TV Standard B/G: values 0 to 2 are available
        /// 
        /// (2) TV Standard I: values 7 to 8 are available
        /// 
        /// (3) TV Standard D/K, K1: values 3 to 6 are available
        /// 
        /// (4) TV Standard L: values 9 to 10 are available
        /// 
        /// (5) TV Standard M, N: values 11 to 14 are available
        /// </param>
        /// <param name="Group_Delay">
        /// This control defines the group delay for a modulation standard.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_CATV_GDELAY_GENERAL    (0) - General 
        /// RSSPECAN_VAL_CATV_GDELAY_GHALF      (1) - General half
        /// RSSPECAN_VAL_CATV_GDELAY_AUSTRALIA  (2) - Australia
        /// RSSPECAN_VAL_CATV_GDELAY_DANMARK    (3) - Danmark
        /// RSSPECAN_VAL_CATV_GDELAY_NEWZELAND  (4) - New Zealand
        /// RSSPECAN_VAL_CATV_GDELAY_NORWAY     (5) - Norway
        /// RSSPECAN_VAL_CATV_GDELAY_SWEDENFULL (6) - Sweden full
        /// RSSPECAN_VAL_CATV_GDELAY_FLAT       (7) - Flat
        /// RSSPECAN_VAL_CATV_GDELAY_OIRT       (8) - OIRT
        /// RSSPECAN_VAL_CATV_GDELAY_CHINA      (9) - China
        /// RSSPECAN_VAL_CATV_GDELAY_CCIR      (10) - CCIR
        /// RSSPECAN_VAL_CATV_GDELAY_TDF       (11) - TDF
        /// RSSPECAN_VAL_CATV_GDELAY_FFC       (12) - FCC
        /// 
        /// Default Value: RSSPECAN_VAL_CATV_GDELAY_GENERAL    (0)
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_MODUL_STANDARD_GDELAY
        /// 
        /// Note:
        /// 
        /// (1) TV Standard B/G: values 0 to 7 are available
        /// 
        /// (2) TV Standard D/K: values 7 to 10 are available
        /// 
        /// (3) TV Standard I: value 7 is available
        /// 
        /// (4) TV Standard K1 and L: values 7 and 11 are available
        /// 
        /// (5) TV Standard M and N: values 7 and 12 are available
        /// 
        /// </param>
        /// <param name="Color_System">
        /// This control sets the color system for analog TV measurements.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ATV_COLOR_SYSTEM_PAL   (0) - PAL
        /// RSSPECAN_VAL_ATV_COLOR_SYSTEM_NTSC  (1) - NTSC
        /// RSSPECAN_VAL_ATV_COLOR_SYSTEM_SECAM (2) - SECAM
        /// 
        /// Default Value: RSSPECAN_VAL_ATV_COLOR_SYSTEM_PAL   (0)
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_COLOR
        /// 
        /// Note:
        /// 
        /// (1) TV Standard B/G, D/K, : values 0 and 2 are available
        /// 
        /// (2) TV Standard I, N: value 0 is available
        /// 
        /// (3) TV Standard K1, L: value 2 is available
        /// 
        /// (4) TV Standard M: values 0 and 1 are available
        /// 
        /// </param>
        /// <param name="Bar_Field">
        /// This control sets the bar field parameter
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_TRIGGER_BAR_FIELD
        /// 
        /// Notes:
        /// 
        /// (1) Value 2 is available only for TV Standard M and Color System NTSC
        /// 
        /// </param>
        /// <param name="Bar_Line">
        /// This control sets the bar line parameter
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 18
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_TRIGGER_BAR_LINE
        /// 
        /// </param>
        /// <param name="Bar_Line_Type">
        /// This control sets the bar line type
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ATV_BLIN_TYPE_CCIR17 (0) - CCIR17
        /// RSSPECAN_VAL_ATV_BLIN_TYPE_FCC    (1) - FCC
        /// RSSPECAN_VAL_ATV_BLIN_TYPE_NTC7   (2) - NTC7
        /// 
        /// Default Value: RSSPECAN_VAL_ATV_BLIN_TYPE_CCIR17 (0)
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_TRIGGER_BAR_LINE_TYPE
        /// 
        /// Note:
        /// 
        /// (1) Values 1 to 2 are available only for TV Standard M, value 0 is not available for TV STandard M.
        /// 
        /// </param>
        /// <param name="Quiet_Field">
        /// This control sets the quiet line field.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_TRIGGER_QLINE_FIELD
        /// 
        /// Notes:
        /// 
        /// (1) Value 2 is available only for TV Standard M and Color System NTSC.
        /// 
        /// 
        /// </param>
        /// <param name="Quiet_Line">
        /// This control sets the quiet line parameter
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 22
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_TRIGGER_QUIET_LINE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVAModulStandard(int TV_Standard, int Sound_System, int Group_Delay, int Color_System, int Bar_Field, int Bar_Line, int Bar_Line_Type, int Quiet_Field, int Quiet_Line)
        {
            int pInvokeResult = PInvoke.ConfigureCATVAModulStandard(this._handle, TV_Standard, Sound_System, Group_Delay, Color_System, Bar_Field, Bar_Line, Bar_Line_Type, Quiet_Field, Quiet_Line);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Video Scope line 
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_TRIGGER_VIDEO_SCOPE_LINE
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:VIDeo:LINE:[NUMBer]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Line">
        /// This control sets triggering at the horizontal sync signal of the indicated line number
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_ATV_TRIGGER_VIDEO_SCOPE_LINE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVAVideoScopeLine(int Line)
        {
            int pInvokeResult = PInvoke.ConfigureCATVAVideoScopeLine(this._handle, Line);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Video Scope field 
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// (2) This function is available only for TV Standard M, color system NTSC.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_TRIGGER_VIDEO_SCOPE_FIELD
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:VIDeo:FIELd:[NUMBer]
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Active_Field">
        /// This control sets the active field.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_ATV_TRIGGER_VIDEO_SCOPE_FIELD
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVAVideoScopeField(int Active_Field)
        {
            int pInvokeResult = PInvoke.ConfigureCATVAVideoScopeField(this._handle, Active_Field);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Carrier to Noise Ratio measurement 
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_CARR_MEAS
        /// RSSPECAN_ATTR_CATV_ATV_MEAS_MODE
        /// RSSPECAN_ATTR_CATV_ATV_BWID
        /// 
        /// Remote-control command(s):
        /// CONFigure:ATV:CN:MEASurement CARRier | NOISe
        /// [SENSe&lt;1|2&gt;:]ATV:CN:MEASurement:MODE ISER | OSER | QLINE
        /// [SENSe&lt;1|2&gt;:]ATV:CN:BWIDth
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Meas_Carrier">
        /// This control configures a new carrier or noise measurement with the next sweep for the measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ATV_MEAS_CARR_CARR (0) - Carrier
        /// RSSPECAN_VAL_ATV_MEAS_CARR_NOIS (1) - Noise
        /// 
        /// Default Value: RSSPECAN_VAL_ATV_MEAS_CARR_CARR (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_ATV_CARR_MEAS
        /// </param>
        /// <param name="Measurement_Method">
        /// This control sets the measurement method for the Carrier-to-Noise Ratio measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ATV_MEAS_MODE_INSERVICE  (0) - In-Service
        /// RSSPECAN_VAL_ATV_MEAS_MODE_OFFSERVICE (1) - Off-Sevice
        /// RSSPECAN_VAL_ATV_MEAS_MODE_QUIETLINE  (2) - Quiet Line
        /// 
        /// Default Value: RSSPECAN_VAL_ATV_MEAS_MODE_INSERVICE  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_ATV_MEAS_MODE
        /// 
        /// Note:
        /// 
        /// (1) In-Service : While an analog TV signal is present, a gap between two consecutive channels, where only noise exists, is used to determine the noise power. A SENSe Subsystem (CATV, K20) R&amp;S FSL 1300.2519.12 6.378 E-7 noise marker is placed there, and the noise power is determined. 
        /// 
        /// (2) Off-Service: The current measurement channel is switched off by the provider. In this channel, a noise marker is placed, and the noise power is determined.
        /// 
        /// (3) Quiet Line: While an analog TV signal is present, the carrier-to-noise ratio is measured using the so-called "Quiet Line". The measurement is fully automated.
        /// </param>
        /// <param name="Noise_Reference_Bandwidth">
        /// This control sets the noise reference bandwidth in the C/N measurement.
        /// 
        /// Valid Values:
        /// 100.0 to 20.0e+6 Hz
        /// 
        /// Default Value: 5.0e+6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_ATV_BWID
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVACNMeasurement(int Meas_Carrier, int Measurement_Method, double Noise_Reference_Bandwidth)
        {
            int pInvokeResult = PInvoke.ConfigureCATVACNMeasurement(this._handle, Meas_Carrier, Measurement_Method, Noise_Reference_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the reference power for Carrier to Noise Ratio measurement 
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_REF_POWER_MODE
        /// RSSPECAN_ATTR_CATV_ATV_REF_CHANNEL_MANUAL
        /// RSSPECAN_ATTR_CATV_ATV_REF_POWER_MANUAL
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ATV:CN:POWer:REFerence:MODE RCHannel | MCHannel | MANual
        /// [SENSe&lt;1|2&gt;:]ATV:CN:POWer:REFerence:CHANnel:MANual
        /// [SENSe&lt;1|2&gt;:]ATV:CN:POWer:REFerence:MANual
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Reference_Power">
        /// This control selects the reference power method.
        /// 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ATV_RPOW_MODE_RCH (0) - Reference Channel 
        /// RSSPECAN_VAL_ATV_RPOW_MODE_MCH (1) - Measurement Channel RSSPECAN_VAL_ATV_RPOW_MODE_MAN (2) - Manual Reference Power
        /// 
        /// Default Value: RSSPECAN_VAL_ATV_RPOW_MODE_MCH (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_ATV_REF_POWER_MODE
        /// </param>
        /// <param name="Reference_Channel">
        /// This control defines the reference channel.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_ATV_REF_CHANNEL_MANUAL
        /// 
        /// Note: 
        /// 
        /// (1) This control has effect only if Reference Power set to Reference Channel.
        /// </param>
        /// <param name="Power_Manual">
        /// This control defines the manual reference power.
        /// 
        /// Valid Values:
        /// -200.0 to 200.0 dB
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_ATV_REF_POWER_MANUAL
        /// 
        /// Note: 
        /// 
        /// (1) This control has effect only if Reference Power set to Manual Reference Power.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVACNReferencePower(int Reference_Power, int Reference_Channel, double Power_Manual)
        {
            int pInvokeResult = PInvoke.ConfigureCATVACNReferencePower(this._handle, Reference_Power, Reference_Channel, Power_Manual);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the measurement frequencies (center frequencies and span values), and activates or deactivates them for the C/N measurement.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// (2) This function is not avaiable for Quiet Line Measurement Method.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ATV:CN:TABle&lt;1...10&gt;:MFRequencies
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Table_Row">
        /// This control defines the table row
        /// 
        /// Valid Values:
        /// 1 to 10
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="State">
        /// This control activates or deactivates frequency for C/N measurement.
        /// 
        /// Valid Values:
        /// 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On (Default Value)
        /// </param>
        /// <param name="Center_Frequency">
        /// This control defines the center frequency
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 1.25e+3 Hz
        /// 
        /// </param>
        /// <param name="Span">
        /// This control defines the span.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 1.0e+6 Hz
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVACNMeasurementFrequencies(int Table_Row, bool State, double Center_Frequency, double Span)
        {
            int pInvokeResult = PInvoke.ConfigureCATVACNMeasurementFrequencies(this._handle, Table_Row, System.Convert.ToUInt16(State), Center_Frequency, Span);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the noise floor correction
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_NOISE_FLOOR_CORRECTION
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ATV:CN:POWer:NCORrection ON | OFF
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Noise_Floor_Correction">
        /// This control activates or deactivates the noise floor correction.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_ATV_NOISE_FLOOR_CORRECTION
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVACNNoiseFloorCorrection(bool Noise_Floor_Correction, uint Timeout)
        {
            int pInvokeResult = PInvoke.ConfigureCATVACNNoiseFloorCorrection(this._handle, System.Convert.ToUInt16(Noise_Floor_Correction), Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches from one frequency range to the next according to the defined measurement frequencies.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_FREQ_NEXT
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ATV:CN:CFRelative:NEXT
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CATVACNNextMeasFrequency()
        {
            int pInvokeResult = PInvoke.CATVACNNextMeasFrequency(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Carrier to Second Order Beat Ratio measurement.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_CARR_MEAS
        /// RSSPECAN_ATTR_CATV_ATV_MEAS_MODE
        /// 
        /// Remote-control command(s):
        /// CONFigure:ATV:CSO:MEASurement CARRier | NOISe
        /// [SENSe&lt;1|2&gt;:]ATV:CSO:MEASurement:MODE OSER | QLIN
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Meas_Carrier">
        /// This control configures a new carrier or noise measurement with the next sweep for the measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ATV_MEAS_CARR_CARR (0) - Carrier
        /// RSSPECAN_VAL_ATV_MEAS_CARR_NOIS (1) - Noise
        /// 
        /// Default Value: RSSPECAN_VAL_ATV_MEAS_CARR_CARR (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_ATV_CARR_MEAS
        /// </param>
        /// <param name="Measurement_Method">
        /// This control sets the measurement method for the Carrier-to-Noise Ratio measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ATV_MEAS_MODE_INSERVICE  (0) - In-Service
        /// RSSPECAN_VAL_ATV_MEAS_MODE_OFFSERVICE (1) - Off-Sevice
        /// RSSPECAN_VAL_ATV_MEAS_MODE_QUIETLINE  (2) - Quiet Line
        /// 
        /// Default Value: RSSPECAN_VAL_ATV_MEAS_MODE_INSERVICE  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_ATV_MEAS_MODE
        /// 
        /// Note:
        /// 
        /// (1) In-Service : While an analog TV signal is present, a gap between two consecutive channels, where only noise exists, is used to determine the noise power. A SENSe Subsystem (CATV, K20) R&amp;S FSL 1300.2519.12 6.378 E-7 noise marker is placed there, and the noise power is determined. 
        /// 
        /// (2) Off-Service: The current measurement channel is switched off by the provider. In this channel, a noise marker is placed, and the noise power is determined.
        /// 
        /// (3) Quiet Line: While an analog TV signal is present, the carrier-to-noise ratio is measured using the so-called "Quiet Line". The measurement is fully automated.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVACSOMeasurement(int Meas_Carrier, int Measurement_Method)
        {
            int pInvokeResult = PInvoke.ConfigureCATVACSOMeasurement(this._handle, Meas_Carrier, Measurement_Method);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the reference power for Carrier to Noise Ratio measurement 
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_REF_POWER_MODE
        /// RSSPECAN_ATTR_CATV_ATV_REF_CHANNEL_MANUAL
        /// RSSPECAN_ATTR_CATV_ATV_REF_POWER_MANUAL
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ATV:CSO:POWer:REFerence:MODE RCHannel | MCHannel | MANual
        /// [SENSe&lt;1|2&gt;:]ATV:CSO:POWer:REFerence:CHANnel:MANual
        /// [SENSe&lt;1|2&gt;:]ATV:CSO:POWer:REFerence:MANual
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Reference_Power">
        /// This control selects the reference power method.
        /// 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ATV_RPOW_MODE_RCH (0) - Reference Channel 
        /// RSSPECAN_VAL_ATV_RPOW_MODE_MCH (1) - Measurement Channel RSSPECAN_VAL_ATV_RPOW_MODE_MAN (2) - Manual Reference Power
        /// 
        /// Default Value: RSSPECAN_VAL_ATV_RPOW_MODE_MCH (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_ATV_REF_POWER_MODE
        /// </param>
        /// <param name="Reference_Channel">
        /// This control defines the reference channel.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_ATV_REF_CHANNEL_MANUAL
        /// 
        /// Note: 
        /// 
        /// (1) This control has effect only if Reference Power set to Reference Channel.
        /// </param>
        /// <param name="Power_Manual">
        /// This control defines the manual reference power.
        /// 
        /// Valid Values:
        /// -200.0 to 200.0 dB
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_ATV_REF_POWER_MANUAL
        /// 
        /// Note: 
        /// 
        /// (1) This control has effect only if Reference Power set to Manual Reference Power.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVACSOReferencePower(int Reference_Power, int Reference_Channel, double Power_Manual)
        {
            int pInvokeResult = PInvoke.ConfigureCATVACSOReferencePower(this._handle, Reference_Power, Reference_Channel, Power_Manual);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the measurement frequencies (center frequencies and span values), and activates or deactivates them for the CSO measurement.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ATV:CSO:TABle&lt;1...10&gt;:MFRequencies
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Table_Row">
        /// This control defines the table row
        /// 
        /// Valid Values:
        /// 1 to 10
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="State">
        /// This control activates or deactivates frequency for C/N measurement.
        /// 
        /// Valid Values:
        /// 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On (Default Value)
        /// </param>
        /// <param name="Center_Frequency">
        /// This control defines the center frequency
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 1.25e+3 Hz
        /// 
        /// </param>
        /// <param name="Span">
        /// This control defines the span.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 1.0e+6 Hz
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVACSOMeasurementFrequencies(int Table_Row, bool State, double Center_Frequency, double Span)
        {
            int pInvokeResult = PInvoke.ConfigureCATVACSOMeasurementFrequencies(this._handle, Table_Row, System.Convert.ToUInt16(State), Center_Frequency, Span);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the noise floor correction
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_NOISE_FLOOR_CORRECTION
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ATV:CSO:POWer:NCORrection ON | OFF
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Noise_Floor_Correction">
        /// This control activates or deactivates the noise floor correction.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_ATV_NOISE_FLOOR_CORRECTION
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVACSONoiseFloorCorrection(bool Noise_Floor_Correction, uint Timeout)
        {
            int pInvokeResult = PInvoke.ConfigureCATVACSONoiseFloorCorrection(this._handle, System.Convert.ToUInt16(Noise_Floor_Correction), Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches from one frequency range to the next according to the defined measurement frequencies.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_FREQ_NEXT
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ATV:CSO:CFRelative:NEXT
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CATVACSONextMeasFrequency()
        {
            int pInvokeResult = PInvoke.CATVACSONextMeasFrequency(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Carrier to Composite Triple Beat Ratio measurement.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_CARR_MEAS
        /// 
        /// Remote-control command(s):
        /// CONFigure:ATV:CTB:MEASuremen CARRier | NOISe
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Meas_Carrier">
        /// This control configures a new carrier or noise measurement with the next sweep for the measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ATV_MEAS_CARR_CARR (0) - Carrier
        /// RSSPECAN_VAL_ATV_MEAS_CARR_NOIS (1) - Noise
        /// 
        /// Default Value: RSSPECAN_VAL_ATV_MEAS_CARR_CARR (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_ATV_CARR_MEAS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVACTBMeasurement(int Meas_Carrier)
        {
            int pInvokeResult = PInvoke.ConfigureCATVACTBMeasurement(this._handle, Meas_Carrier);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the reference power for Carrier to Composite Triple Beat Ratio measurement 
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_REF_POWER_MODE
        /// RSSPECAN_ATTR_CATV_ATV_REF_CHANNEL_MANUAL
        /// RSSPECAN_ATTR_CATV_ATV_REF_POWER_MANUAL
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ATV:CTB:POWer:REFerence:MODE RCHannel | MCHannel | MANual
        /// [SENSe&lt;1|2&gt;:]ATV:CTB:POWer:REFerence:CHANnel:MANual
        /// [SENSe&lt;1|2&gt;:]ATV:CTB:POWer:REFerence:MANual
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Reference_Power">
        /// This control selects the reference power method.
        /// 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ATV_RPOW_MODE_RCH (0) - Reference Channel 
        /// RSSPECAN_VAL_ATV_RPOW_MODE_MCH (1) - Measurement Channel RSSPECAN_VAL_ATV_RPOW_MODE_MAN (2) - Manual Reference Power
        /// 
        /// Default Value: RSSPECAN_VAL_ATV_RPOW_MODE_MCH (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_ATV_REF_POWER_MODE
        /// </param>
        /// <param name="Reference_Channel">
        /// This control defines the reference channel.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_ATV_REF_CHANNEL_MANUAL
        /// 
        /// Note: 
        /// 
        /// (1) This control has effect only if Reference Power set to Reference Channel.
        /// </param>
        /// <param name="Power_Manual">
        /// This control defines the manual reference power.
        /// 
        /// Valid Values:
        /// -200.0 to 200.0 dB
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_ATV_REF_POWER_MANUAL
        /// 
        /// Note: 
        /// 
        /// (1) This control has effect only if Reference Power set to Manual Reference Power.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVACTBReferencePower(int Reference_Power, int Reference_Channel, double Power_Manual)
        {
            int pInvokeResult = PInvoke.ConfigureCATVACTBReferencePower(this._handle, Reference_Power, Reference_Channel, Power_Manual);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the measurement frequencies (center frequencies and span values), and activates or deactivates them for the CTB measurement.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ATV:CSO:TABle&lt;1...10&gt;:MFRequencies
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Table_Row">
        /// This control defines the table row
        /// 
        /// Valid Values:
        /// 1 to 10
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="State">
        /// This control activates or deactivates frequency for C/N measurement.
        /// 
        /// Valid Values:
        /// 
        /// VI_FALSE (0) - Off
        /// Vi_TRUE  (1) - On (Default Value)
        /// </param>
        /// <param name="Center_Frequency">
        /// This control defines the center frequency
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 1.25e+3 Hz
        /// 
        /// </param>
        /// <param name="Span">
        /// This control defines the span.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 1.0e+6 Hz
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVACTBMeasurementFrequencies(int Table_Row, bool State, double Center_Frequency, double Span)
        {
            int pInvokeResult = PInvoke.ConfigureCATVACTBMeasurementFrequencies(this._handle, Table_Row, System.Convert.ToUInt16(State), Center_Frequency, Span);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the noise floor correction
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_NOISE_FLOOR_CORRECTION
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ATV:CTB:POWer:NCORrection ON | OFF
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Noise_Floor_Correction">
        /// This control activates or deactivates the noise floor correction.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_ATV_NOISE_FLOOR_CORRECTION
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVACTBNoiseFloorCorrection(bool Noise_Floor_Correction, uint Timeout)
        {
            int pInvokeResult = PInvoke.ConfigureCATVACTBNoiseFloorCorrection(this._handle, System.Convert.ToUInt16(Noise_Floor_Correction), Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches from one frequency range to the next according to the defined measurement frequencies.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_FREQ_NEXT
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ATV:CTB:CFRelative:NEXT
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CATVACTBNextMeasFrequency()
        {
            int pInvokeResult = PInvoke.CATVACTBNextMeasFrequency(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the limit of the carrier power measurement
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_CARR_LIM_VCF
        /// RSSPECAN_ATTR_CATV_ATV_CARR_LIM_VCP
        /// RSSPECAN_ATTR_CATV_ATV_CARR_LIM_FREQ_OFFSET
        /// RSSPECAN_ATTR_CATV_ATV_CARR_LIM_REL_POWER
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:CARRiers:VCFoffset:LOWer
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:CARRiers:VCFoffset:UPPer
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:CARRiers:VCPabsolute:LOWer
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:CARRiers:VCPabsolute:UPPer
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:CARRiers:S1IFoffset:LOWer
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:CARRiers:S1IFoffset:UPPer
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:CARRiers:S1PRelative:LOWer
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:CARRiers:S1PRelative:UPPer
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:CARRiers:S2IFoffset:LOWer
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:CARRiers:S2IFoffset:UPPer
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:CARRiers:S2PRelative:LOWer
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:CARRiers:S2PRelative:UPPer
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement">
        /// This control selects the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ATV_VCP   (0) - Vision Carrier Power Absolute 
        /// RSSPECAN_VAL_ATV_VCF   (1) - Vision Carrier Frequency Offset
        /// RSSPECAN_VAL_ATV_SC1PR (2) - Sound Carrier 1 Power Relative
        /// RSSPECAN_VAL_ATV_SC1IF (3) - Sound Carrier 1 Intercarrier Frequency Offset
        /// RSSPECAN_VAL_ATV_SC2PR (4) - Sound Carrier 2 Power Relative
        /// RSSPECAN_VAL_ATV_SC2IF (5) - Sound Carrier 2 Intercarrier Frequency Offset
        /// 
        /// Default Value: RSSPECAN_VAL_ATV_VCP   (0)
        /// 
        /// </param>
        /// <param name="Limit_Type">
        /// This control selects the type of the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIM_LOW (0) - Lower
        /// RSSPECAN_VAL_LIM_UPP (1) - Upper
        /// 
        /// Default Value: RSSPECAN_VAL_LIM_LOW (0)
        /// 
        /// </param>
        /// <param name="Limit_Value">
        /// This control sets the limit value.
        /// 
        /// Valid Values: 
        /// 
        /// for Power Absolute: -200 to 200 dB
        /// for Frequency Offset: -999.999 to 999.999 Hz
        /// for Sound Carrier Power Relative: -200 to 200 dB
        /// for Sound Carrier Intercarrier Frequency Offset: -999,999 to 999,999 Hz
        /// 
        /// Default Value: -60.0
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_CARR_LIM_VCF
        /// RSSPECAN_ATTR_CATV_ATV_CARR_LIM_VCP
        /// RSSPECAN_ATTR_CATV_ATV_CARR_LIM_FREQ_OFFSET
        /// RSSPECAN_ATTR_CATV_ATV_CARR_LIM_REL_POWER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVACarrierLimit(int Measurement, int Limit_Type, double Limit_Value)
        {
            int pInvokeResult = PInvoke.ConfigureCATVACarrierLimit(this._handle, Measurement, Limit_Type, Limit_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the limit for the carrier-to-noise, carrier-to-second order beat or carrier-to-composite triple beat ratio.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_LIM_CN
        /// RSSPECAN_ATTR_CATV_ATV_LIM_CSO
        /// RSSPECAN_ATTR_CATV_ATV_LIM_CTB
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:CN:CN[:LOWer]
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:CSO:CSO[:LOWer]
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:CTB:CTB[:LOWer]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement">
        /// This control selects the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ATV_MEAS_CN  (2) - C/N
        /// RSSPECAN_VAL_ATV_MEAS_CSO (3) - CSO
        /// RSSPECAN_VAL_ATV_MEAS_CTB (4) - CTB
        /// 
        /// Default Value: RSSPECAN_VAL_ATV_MEAS_CN  (2)
        /// 
        /// Note:
        /// 
        /// (1) C/N - lower limit for the carrier-to-noise ratio.
        /// 
        /// (2) CSO - lower limit for the carrier-to-second order beat ratio.
        /// 
        /// (3) CTB - lower limit for the carrier-to-composite triple beat ratio.
        /// </param>
        /// <param name="Limit_Type">
        /// This control selects the type of the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIM_LOW (0) - Lower
        /// RSSPECAN_VAL_LIM_UPP (1) - Upper
        /// 
        /// Default Value: RSSPECAN_VAL_LIM_LOW (0)
        /// 
        /// </param>
        /// <param name="Limit_Value">
        /// This control sets the limit value.
        /// 
        /// Valid Values: 
        /// 
        /// -200.0 to 200.0 dB
        /// 
        /// 
        /// Default Value: 40.0 dB
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_LIM_CN
        /// RSSPECAN_ATTR_CATV_ATV_LIM_CSO
        /// RSSPECAN_ATTR_CATV_ATV_LIM_CTB
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVACarrierRatioLimit(int Measurement, int Limit_Type, double Limit_Value)
        {
            int pInvokeResult = PInvoke.ConfigureCATVACarrierRatioLimit(this._handle, Measurement, Limit_Type, Limit_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the limit for the hum values
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_HUM_UNIT
        /// RSSPECAN_ATTR_CATV_ATV_LIM_HUM_LOW
        /// RSSPECAN_ATTR_CATV_ATV_LIM_HUM_UPP
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:ATV:UNIT:POWer:HUM DB | PCT
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:HUM:HUM:LOWer
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:HUM:HUM:UPPer
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit_Type">
        /// This control selects the type of the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIM_LOW (0) - Lower
        /// RSSPECAN_VAL_LIM_UPP (1) - Upper
        /// 
        /// Default Value: RSSPECAN_VAL_LIM_LOW (0)
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_LIM_HUM_LOW
        /// RSSPECAN_ATTR_CATV_ATV_LIM_HUM_UPP
        /// 
        /// </param>
        /// <param name="Limit_Value">
        /// This control sets the limit value.
        /// 
        /// Valid Values: 
        /// 
        /// for Lower Limit Type: -60 to 60 dB
        /// for Upper Limit Type: 0.1 to 100.0 %
        /// 
        /// Default Value: 40.0 dB
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_LIM_HUM_LOW
        /// RSSPECAN_ATTR_CATV_ATV_LIM_HUM_UPP
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVAHumLimit(int Limit_Type, double Limit_Value)
        {
            int pInvokeResult = PInvoke.ConfigureCATVAHumLimit(this._handle, Limit_Type, Limit_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the limit for the vision modulation.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_LIM_VMOD_VCP
        /// RSSPECAN_ATTR_CATV_ATV_LIM_VMOD_RPC
        /// RSSPECAN_ATTR_CATV_ATV_LIM_VMOD_MDEP
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:VMODulation:VCPower:LOWer
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:VMODulation:VCPower:UPPer
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:VMODulation:RPC:LOWer
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:VMODulation:RPC:UPPer
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:VMODulation:MDEPth:LOWer
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:VMODulation:MDEPth:UPPer
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ATV_VCP  (0) - Vision Carrier Power
        /// RSSPECAN_VAL_ATV_RPC  (1) - Residual Picture
        /// RSSPECAN_VAL_ATV_MDEP (2) - Modulation Depth 
        /// 
        /// Default Value: RSSPECAN_VAL_ATV_VCP  (0)
        /// 
        /// </param>
        /// <param name="Limit_Type">
        /// This control selects the type of the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIM_LOW (0) - Lower
        /// RSSPECAN_VAL_LIM_UPP (1) - Upper
        /// 
        /// Default Value: RSSPECAN_VAL_LIM_LOW (0)
        /// 
        /// </param>
        /// <param name="Limit_Value">
        /// This control sets the limit value.
        /// 
        /// Valid Values: 
        /// 
        /// for Modulation Depth: 0 to 100 %
        /// for Residual Picture: 0 to 100 %
        /// for Vision Carrier Power: -200 to 200 dBm
        /// 
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_LIM_VMOD_VCP
        /// RSSPECAN_ATTR_CATV_ATV_LIM_VMOD_RPC
        /// RSSPECAN_ATTR_CATV_ATV_LIM_VMOD_MDEP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVAVisionModLimit(int Limit, int Limit_Type, double Limit_Value)
        {
            int pInvokeResult = PInvoke.ConfigureCATVAVisionModLimit(this._handle, Limit, Limit_Type, Limit_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the unit of the absolute vision carrier power.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_VCP_UNIT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:ATV:UNIT:POWer:VCPabsolute DBM | DBUV | DBUA | DBMV | DBPW | V | W | A
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Unit">
        /// This control selects the unit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DBM  (0) - dBm 
        /// RSSPECAN_VAL_UNIT_DBPW (1) - dBpW
        /// RSSPECAN_VAL_UNIT_WATT (2) - W
        /// RSSPECAN_VAL_UNIT_DBUV (3) - dBuV
        /// RSSPECAN_VAL_UNIT_DBMV (4) - dBmV
        /// RSSPECAN_VAL_UNIT_VOLT (5) - V
        /// RSSPECAN_VAL_UNIT_DBUA (6) - dBuA
        /// RSSPECAN_VAL_UNIT_AMP  (7) - A
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_DBM  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_ATV_VCP_UNIT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVAVisionModulationCarrierPowerUnit(int Unit)
        {
            int pInvokeResult = PInvoke.ConfigureCATVAVisionModulationCarrierPowerUnit(this._handle, Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the unit of the Hum measurement values.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_HUM_UNIT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:ATV:UNIT:POWer:HUM DB | PCT
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Unit">
        /// This control selects the unit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DB  (8) - dB 
        /// RSSPECAN_VAL_UNIT_PCT (13) - %
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_DB  (8)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_ATV_HUM_UNIT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVAHumUnit(int Unit)
        {
            int pInvokeResult = PInvoke.ConfigureCATVAHumUnit(this._handle, Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the digital measurement type.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_DTV_MEAS
        /// 
        /// Remote-control command(s):
        /// CONFigure:DTV:MEASurement DSPectrum | CHPower | CONS | MERRors | EPATtern | APD | CCDF | OVERview
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement">
        /// This control selects the measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DTV_MEAS_DSP   (0) - Spectrum 
        /// RSSPECAN_VAL_DTV_MEAS_CHP   (1) - Channel Power
        /// RSSPECAN_VAL_DTV_MEAS_CONS  (2) - Constellation Diagram
        /// RSSPECAN_VAL_DTV_MEAS_MERR  (3) - Modulation Errors
        /// RSSPECAN_VAL_DTV_MEAS_EPATT (4) - Echo Pattern
        /// RSSPECAN_VAL_DTV_MEAS_APD   (5) - Amplitude Probability Density Function
        /// RSSPECAN_VAL_DTV_MEAS_CCDF  (6) - Complementary Cumulative Distribution Function
        /// RSSPECAN_VAL_DTV_MEAS_OVER  (7) - Overview
        /// 
        /// Default Value: RSSPECAN_VAL_DTV_MEAS_DSP   (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_DTV_MEAS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVDMeasurement(int Measurement)
        {
            int pInvokeResult = PInvoke.ConfigureCATVDMeasurement(this._handle, Measurement);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the modulation properties of a TV signal (as used in a TV channel). The signal type is the characteristic parameter of this set.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_DTV_STANDARD
        /// RSSPECAN_ATTR_CATV_DTV_FILTER_ALPHA
        /// RSSPECAN_ATTR_CATV_DTV_FORMAT
        /// RSSPECAN_ATTR_CATV_DTV_SRATE
        /// 
        /// Remote-control command(s):
        /// SETup:TV:STANdard J83A | J83B | J83C
        /// [SENSe&lt;1|2&gt;:]DDEMod:FILTer:ALPHa R012 | R013 | R015 | R018
        /// SOURce:DM:FORMat QAM4 | QAM16 | QAM32 | QAM64 | QAM128 | QAM256 | QAM512 | QAM1024
        /// [SENSe&lt;1|2&gt;:]DDEMod:SRATe
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="TV_Standard">
        /// This control sets sets the TV standard.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DTV_STAN_J83A (0) - QAM J.83/A 
        /// RSSPECAN_VAL_DTV_STAN_J83B (1) - QAM J.83/B
        /// RSSPECAN_VAL_DTV_STAN_J83C (2) - QAM J.83/C
        /// 
        /// Default Value: RSSPECAN_VAL_DTV_STAN_J83A (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_DTV_STANDARD
        /// 
        /// </param>
        /// <param name="Roll_Off">
        /// This control sets the roll-off.
        /// 
        /// Valid Values:
        /// 
        /// RSSPECAN_VAL_CATV_DTV_ALPHA_R012 (0) - 0.120
        /// RSSPECAN_VAL_CATV_DTV_ALPHA_R013 (1) - 0.130
        /// RSSPECAN_VAL_CATV_DTV_ALPHA_R015 (2) - 0.150 
        /// RSSPECAN_VAL_CATV_DTV_ALPHA_R018 (3) - 0.180
        /// 
        /// Default Value: RSSPECAN_VAL_CATV_DTV_ALPHA_R015 (2)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_DTV_FILTER_ALPHA
        /// </param>
        /// <param name="Constellation_Parameter">
        /// This control sets the constellation parameter.
        /// 
        /// Valid Values:
        /// 
        /// RSSPECAN_VAL_DTV_FORMAT_QAM4    (0) - 4QAM
        /// RSSPECAN_VAL_DTV_FORMAT_QAM16   (1) - 16QAM
        /// RSSPECAN_VAL_DTV_FORMAT_QAM32   (2) - 32QAM
        /// RSSPECAN_VAL_DTV_FORMAT_QAM64   (3) - 64QAM  
        /// RSSPECAN_VAL_DTV_FORMAT_QAM128  (4) - 128QAM
        /// RSSPECAN_VAL_DTV_FORMAT_QAM256  (5) - 256QAM
        /// RSSPECAN_VAL_DTV_FORMAT_QAM512  (6) - 512QAM
        /// RSSPECAN_VAL_DTV_FORMAT_QAM1024 (7) - 1024QAM
        /// 
        /// Default Value: RSSPECAN_VAL_DTV_FORMAT_QAM64   (3)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_DTV_FORMAT
        /// </param>
        /// <param name="Symbol_Rate">
        /// This control sets the symbol rate.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value:
        /// 6.9e+6 Symb/s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_DTV_SRATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVDModulStandard(int TV_Standard, int Roll_Off, int Constellation_Parameter, double Symbol_Rate)
        {
            int pInvokeResult = PInvoke.ConfigureCATVDModulStandard(this._handle, TV_Standard, Roll_Off, Constellation_Parameter, Symbol_Rate);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Shoulder Attenuation for Spectrum measurement 
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// (2) This funciton is valid only for Spectrum measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_DTV_SATT
        /// 
        /// Remote-control command(s):
        /// CONFigure:DTV:MEASurement:SATTenuation OFF | ON
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Shoulder_Attenuation">
        /// This control activates or deactivates the shoulder measurement in accordance to ETSI TR 101290 standard.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On (Default Value)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_DTV_SATT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVDShoulderAttenuation(bool Shoulder_Attenuation)
        {
            int pInvokeResult = PInvoke.ConfigureCATVDShoulderAttenuation(this._handle, System.Convert.ToUInt16(Shoulder_Attenuation));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Echo Pattern measurement
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_EPAT_ZOOM_STATE
        /// RSSPECAN_ATTR_CATV_EPAT_ZOOM_FACTOR
        /// RSSPECAN_ATTR_CATV_DTV_EPAT_RVEL
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:ZOOM:EPATtern:STATe
        /// DISPlay[:WINDow&lt;1|2&gt;]:ZOOM:EPATtern
        /// [SENSe&lt;1|2&gt;:]CORRection:RVELocity
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Zoom_State">
        /// This control activates or deactivates the zoom.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_EPAT_ZOOM_STATE
        /// </param>
        /// <param name="Zoom_Factor">
        /// This control sets the zoom factor. 
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_EPAT_ZOOM_FACTOR
        /// </param>
        /// <param name="Velocity_Factor">
        /// This control sets the velocity of propagation of the signal in the transmission channel (e.g. cable), relative to speed of light.
        /// 
        /// Valid Values:
        /// 0.0001 to 1.0
        /// 
        /// Default Value:
        /// 1.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_DTV_EPAT_RVEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVDEchoPattern(bool Zoom_State, int Zoom_Factor, double Velocity_Factor)
        {
            int pInvokeResult = PInvoke.ConfigureCATVDEchoPattern(this._handle, System.Convert.ToUInt16(Zoom_State), Zoom_Factor, Velocity_Factor);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Modulation Errors Zoom.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_MERR_ZOOM
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:ZOOM:MERRors NONE | IMBalance | QERRor | SUPPression | PJITter | MERRms | MERPeak | EVMRms | EVMPeak
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Zoom">
        /// This control selects the measurement to be displayed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_CATV_MERR_ZOOM_NONE (0) - None
        /// RSSPECAN_VAL_CATV_MERR_ZOOM_MERR (1) - Mod. Err. Rate RMS 
        /// RSSPECAN_VAL_CATV_MERR_ZOOM_MERP (2) - Mod. Err. Rate Peak
        /// RSSPECAN_VAL_CATV_MERR_ZOOM_EVMR (3) - Err. Vector Magnitude RMS
        /// RSSPECAN_VAL_CATV_MERR_ZOOM_EVMP (4) - Err. Vector Magnitude Peak
        /// RSSPECAN_VAL_CATV_MERR_ZOOM_IMB  (5) - Aplitude Imbalance
        /// RSSPECAN_VAL_CATV_MERR_ZOOM_QERR (6) - Quadrature Error
        /// RSSPECAN_VAL_CATV_MERR_ZOOM_SUP  (7) - Carrier Suppression
        /// RSSPECAN_VAL_CATV_MERR_ZOOM_PJIT (8) - Phase Jitter
        /// 
        /// Default Value: RSSPECAN_VAL_CATV_MERR_ZOOM_NONE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_MERR_ZOOM
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVDModulErrorZoom(int Zoom)
        {
            int pInvokeResult = PInvoke.ConfigureCATVDModulErrorZoom(this._handle, Zoom);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Overview Zoom.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_OVER_ZOOM
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:ZOOM:OVERview NONE | MERRms | MERPeak | EVMRms | EVMPeak | CFOFfset | SROFfset
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Zoom">
        /// This control selects the measurement to be displayed for zoom
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_CATV_OVER_ZOOM_NONE (0) - None 
        /// RSSPECAN_VAL_CATV_OVER_ZOOM_MERR (1) - Mod. Err. Rate RMS 
        /// RSSPECAN_VAL_CATV_OVER_ZOOM_MERP (2) - Mod. Err. Rate Peak
        /// RSSPECAN_VAL_CATV_OVER_ZOOM_EVMR (3) - Err. Vector Magnitude RMS
        /// RSSPECAN_VAL_CATV_OVER_ZOOM_EVMP (4) - Err. Vector Magnitude Peak
        /// RSSPECAN_VAL_CATV_OVER_ZOOM_CFOF (5) - Carrier Frequency Offset
        /// RSSPECAN_VAL_CATV_OVER_ZOOM_SROF (6) - Symbol Rate Offset
        /// 
        /// Default Value: RSSPECAN_VAL_CATV_OVER_ZOOM_NONE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_OVER_ZOOM
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVDOverviewZoom(int Zoom)
        {
            int pInvokeResult = PInvoke.ConfigureCATVDOverviewZoom(this._handle, Zoom);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Constellatin Diagram Zoom.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_QUAD_ZOOM
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:ZOOM:QUADrant
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Zoom_Diagram">
        /// This control selects the constellation diagram to be displayed for zoom
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_CATV_QUAD_ZOOM_NONE (0) - Whole diagram
        /// RSSPECAN_VAL_CATV_QUAD_ZOOM_1    (1) - Quadrant 1
        /// RSSPECAN_VAL_CATV_QUAD_ZOOM_2    (2) - Quadrant 2
        /// RSSPECAN_VAL_CATV_QUAD_ZOOM_3    (3) - Quadrant 3
        /// RSSPECAN_VAL_CATV_QUAD_ZOOM_4    (4) - Quadrant 4
        /// 
        /// Default Value: 0
        /// .
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_QUAD_ZOOM
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVDConstellationDiagramZoom(int Zoom_Diagram)
        {
            int pInvokeResult = PInvoke.ConfigureCATVDConstellationDiagramZoom(this._handle, Zoom_Diagram);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the digital TV equalizer.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_DTV_EQUALIZER_STATE
        /// RSSPECAN_ATTR_CATV_DTV_EQUALIZER_FREEZE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:EQUalizer[:STATe]
        /// [SENSe&lt;1|2&gt;:]DDEMod:EQUalizer:FREeze[:STATe]
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="State">
        /// This control activates or deactivates the equalizer.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On (Default Value)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_DTV_EQUALIZER_STATE
        /// </param>
        /// <param name="Freeze">
        /// This control defines the change of the equalizer coefficients.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On 
        /// 
        /// Note:
        /// 
        /// (1) This control has effect only if State is On.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_DTV_EQUALIZER_FREEZE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVDEqualizer(bool State, bool Freeze)
        {
            int pInvokeResult = PInvoke.ConfigureCATVDEqualizer(this._handle, System.Convert.ToUInt16(State), System.Convert.ToUInt16(Freeze));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the number of measurement points to be acquired for the statistical measurement functions.
        /// 
        /// Note:
        /// 
        /// This function is only available with the R&amp;S FSL with FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_STAT_SAMPLES
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:NSAMples
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="No_of_Samples">
        /// This control sets the number of test points to be recorded for the statistical measurement functions.
        /// 
        /// Valid Values: 100 to 1000000000
        /// 
        /// Default Value: 100000
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_STAT_SAMPLES
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVDSignalStatisticsNoSamples(int No_of_Samples)
        {
            int pInvokeResult = PInvoke.ConfigureCATVDSignalStatisticsNoSamples(this._handle, No_of_Samples);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function changes the scaling parameters for statistical
        /// measurements.
        /// 
        /// Note:
        /// 
        /// This function is only available with the R&amp;S FSL with FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_STAT_PRESET
        /// RSSPECAN_ATTR_MEAS_STAT_ADJ
        /// RSSPECAN_ATTR_MEAS_STAT_X_REF
        /// RSSPECAN_ATTR_MEAS_STAT_X_RANGE
        /// RSSPECAN_ATTR_MEAS_STAT_Y_MAX
        /// RSSPECAN_ATTR_MEAS_STAT_Y_MIN
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:PRESet
        /// CALCulate:STATistics:SCALe:AUTO ONCE
        /// CALCulate:STATistics:SCALe:X:RLEVel
        /// CALCulate:STATistics:SCALe:X:RANGe
        /// CALCulate:STATistics:SCALe:Y:UPPer
        /// CALCulate:STATistics:SCALe:Y:LOWer
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Scaling_Mode">
        /// This control selects scaling mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_STAT_DEFAULT (0) - Default Settings 
        /// RSSPECAN_VAL_STAT_ADJUST  (1) - Adjust Settings
        /// RSSPECAN_VAL_STAT_XRLEV   (2) - X-axis Ref Level
        /// RSSPECAN_VAL_STAT_XRANGE  (3) - X-axis Range
        /// RSSPECAN_VAL_STAT_YMAX    (4) - Y-axis Max Value
        /// RSSPECAN_VAL_STAT_YMIN    (5) - Y-axis Min Value
        /// 
        /// Default Value: RSSPECAN_VAL_STAT_DEFAULT (0)
        /// 
        /// Notes:
        /// 
        /// (1) Default Settings:
        /// This command resets the scaling of the X and Y axes in a statistical measurement. The following values are set:
        /// x-axis ref level: -20 dBm
        /// x-axis range APD: 100 dB
        /// x-axis range CCDF: 20 dB
        /// y-axis upper limit: 1.0
        /// y-axis lower limit: 1E-6
        /// 
        /// (2) Adjust Settings:
        /// This command optimizes the level setting of the instrument depending on the measured peak power to obtain maximum instrument sensitivity. To obtain maximum resolution, the level range is set as a function of the measured spacing between peak power and the minimum power with the APD measurement and of the spacing between the peak power and the mean power with CCDF measurement. In addition, the probability scale for the number of test points is adapted.
        /// 
        /// (3) X-axis Ref Level:
        /// This command defines the reference level for the X-axis of the measurement diagram. The setting is identical to the reference level setting using the command DISPlay:WINDow:TRACe:Y:RLEVel.
        /// With the reference level offset &lt;&gt; 0 the indicated value range of the reference level is modified by the offset.
        /// 
        /// (4) X-axis Range:
        /// This command defines the level range for the X-axis of the measurement diagram.
        /// 
        /// (5) Y-axis Max Value:
        /// This command defines the upper limit for the Y-axis of the diagram in statitistical measurements. Since probabilities are specified on the Y-axis, the entered numerical values are dimensionless.
        /// 
        /// (6) Y-axis Min Value:
        /// This command defines the lower limit for th Y-axis of the diagram in statistical measurements. Since probabilities are specified on the Y-axis, the entered numerical values are dimensionless.
        /// 
        /// </param>
        /// <param name="Scaling_Value">
        /// This control sets the scaling values for the selected scaling mode.
        /// 
        /// Valid Valuess:
        ///  X-axis Ref Level: -130.0 dBm to 30.0 dBm
        ///  X-axis Range: 10.0 dB to 200.0 dB
        ///  Y-axis Max Value: 1.0E-8 to 1.0
        ///  Y-axis Min Value: 1.0E-9 to 0.1
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_STAT_PRESET
        /// RSSPECAN_ATTR_MEAS_STAT_ADJ
        /// RSSPECAN_ATTR_MEAS_STAT_X_REF
        /// RSSPECAN_ATTR_MEAS_STAT_X_RANGE
        /// RSSPECAN_ATTR_MEAS_STAT_Y_MAX
        /// RSSPECAN_ATTR_MEAS_STAT_Y_MIN
        /// 
        /// Note:
        /// 
        /// (1) This control has no effect if Scaling Mode is set to:
        ///     Default Settings
        ///     Adjust Settings
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVDSignalStatisticsScaling(int Scaling_Mode, double Scaling_Value)
        {
            int pInvokeResult = PInvoke.ConfigureCATVDSignalStatisticsScaling(this._handle, Scaling_Mode, Scaling_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function positions the selected marker in the selected window to the given probability.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the R&amp;S FSL with FS-K20 option.
        /// 
        /// (2) This command is only available with the CCDF measurement switched on.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_PROBABILITY
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:Y:PERCent
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Marker_Number">
        /// Selects the marker to configure.
        /// 
        /// Valid Values:
        /// 1 - 4 
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Position_Value">
        /// This control positions the selected marker in the selected window to the given probability.
        /// 
        /// Valid Values: 0.0 to 100.0 %
        /// 
        /// Default Value: 0.0 %
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_PROBABILITY
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVDCCDFPercentMark(int Marker_Number, double Position_Value)
        {
            int pInvokeResult = PInvoke.ConfigureCATVDCCDFPercentMark(this._handle, Marker_Number, Position_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function resets the equalizer and all equalizer parameters are set to their default values.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_DTV_EQUALIZER_RESET
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:EQUalizer:RESet
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CATVDResetEqualizer()
        {
            int pInvokeResult = PInvoke.CATVDResetEqualizer(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the limit of the overview measurement
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_DTV_ERROR_UNIT
        /// RSSPECAN_ATTR_CATV_DTV_LIM_ERR_LOW
        /// RSSPECAN_ATTR_CATV_DTV_LIM_ERR_UPP
        /// RSSPECAN_ATTR_CATV_DTV_CARR_LIM_FREQ_OFFSET
        /// RSSPECAN_ATTR_CATV_DTV_LIM_SYMB_RATE_OFFSET
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DTV:UNIT:POWer:MERRms DB | PCT
        /// CALCulate:DTV:LIMit:MERRms:LOWer
        /// CALCulate:DTV:LIMit:MERRms:UPPer
        /// CALCulate&lt;1|2&gt;:DTV:UNIT:POWer:MERPeak DB | PCT
        /// CALCulate:DTV:LIMit:MERPeak:LOWer
        /// CALCulate:DTV:LIMit:MERPeak:UPPer
        /// CALCulate&lt;1|2&gt;:DTV:UNIT:POWer:EVMRms DB | PCT
        /// CALCulate:DTV:LIMit:EVMRms:LOWer
        /// CALCulate:DTV:LIMit:EVMRms:UPPer
        /// CALCulate&lt;1|2&gt;:DTV:UNIT:POWer:EVMPeak DB | PCT
        /// CALCulate:DTV:LIMit:EVMPeak:LOWer
        /// CALCulate:DTV:LIMit:EVMPeak:UPPer
        /// CALCulate:DTV:LIMit:CFOFfset:LOWer
        /// CALCulate:DTV:LIMit:CFOFfset:UPPer
        /// CALCulate:DTV:LIMit:SROFfset:LOWer
        /// CALCulate:DTV:LIMit:SROFfset:UPPer
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement">
        /// This control selects the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DTV_MERR (0) - Mod. Err. Rate RMS 
        /// RSSPECAN_VAL_DTV_MERP (1) - Mod. Err. Rate Peak
        /// RSSPECAN_VAL_DTV_EVMR (2) - Err. Vector Magnitude RMS
        /// RSSPECAN_VAL_DTV_EVMP (3) - Err. Vector Magnitude Peak
        /// RSSPECAN_VAL_DTV_CFOF (4) - Carrier Frequency Offset
        /// RSSPECAN_VAL_DTV_SROF (5) - Symbol Rate Offset
        /// 
        /// Default Value: RSSPECAN_VAL_DTV_MERR (0)
        /// 
        /// </param>
        /// <param name="Limit_Type">
        /// This control selects the type of the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIM_LOW (0) - Lower
        /// RSSPECAN_VAL_LIM_UPP (1) - Upper
        /// 
        /// Default Value: RSSPECAN_VAL_LIM_LOW (0)
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_DTV_ERROR_UNIT
        /// 
        /// </param>
        /// <param name="Limit_Value">
        /// This control sets the limit value.
        /// 
        /// Valid Values: 
        /// Mod. Err. Rate RMS: 
        /// Mod. Err. Rate Peak:
        /// Err. Vector Magnitude RMS: 
        /// Err. Vector Magnitude Peak: 
        ///                     0 to 80 dB for Lower
        ///                     0.01 to 100% for Upper 
        /// Carrier Frequency Offset: -99.999 to 99.999 Hz
        /// Symbol Rate Offset: -99999 to 99999 symb/s
        /// 
        /// Default Value: 60.0
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_DTV_LIM_ERR_LOW
        /// RSSPECAN_ATTR_CATV_DTV_LIM_ERR_UPP
        /// RSSPECAN_ATTR_CATV_DTV_CARR_LIM_FREQ_OFFSET
        /// RSSPECAN_ATTR_CATV_DTV_LIM_SYMB_RATE_OFFSET
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVDOverLimit(int Measurement, int Limit_Type, double Limit_Value)
        {
            int pInvokeResult = PInvoke.ConfigureCATVDOverLimit(this._handle, Measurement, Limit_Type, Limit_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the limit of the channel Power measurement
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_DTV_LIM_CHANNEL_POWER
        /// 
        /// Remote-control command(s):
        /// CALCulate:DTV:LIMit:CHPower:UPPer
        /// CALCulate:DTV:LIMit:CHPower:LOWer
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit_Type">
        /// This control selects the type of the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIM_LOW (0) - Lower
        /// RSSPECAN_VAL_LIM_UPP (1) - Upper
        /// 
        /// Default Value: RSSPECAN_VAL_LIM_LOW (0)
        /// 
        /// </param>
        /// <param name="Limit_Value">
        /// This control sets the limit value.
        /// 
        /// Valid Values: 
        /// -200.0 to 200.0 dBm
        /// 
        /// Default Value: 30.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_DTV_LIM_CHANNEL_POWER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVDChanPowLimit(int Limit_Type, double Limit_Value)
        {
            int pInvokeResult = PInvoke.ConfigureCATVDChanPowLimit(this._handle, Limit_Type, Limit_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the limit of the modulation errors measurement
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_DTV_ERROR_UNIT
        /// RSSPECAN_ATTR_CATV_DTV_LIM_ERR_LOW
        /// RSSPECAN_ATTR_CATV_DTV_LIM_ERR_UPP
        /// RSSPECAN_ATTR_CATV_DTV_LIM_IMB
        /// RSSPECAN_ATTR_CATV_DTV_LIM_QERR
        /// RSSPECAN_ATTR_CATV_DTV_LIM_SUPP
        /// RSSPECAN_ATTR_CATV_DTV_LIM_PJIT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DTV:UNIT:POWer:MERRms DB | PCT
        /// CALCulate:DTV:LIMit:MERRms:LOWer
        /// CALCulate:DTV:LIMit:MERRms:UPPer
        /// CALCulate&lt;1|2&gt;:DTV:UNIT:POWer:MERPeak DB | PCT
        /// CALCulate:DTV:LIMit:MERPeak:LOWer
        /// CALCulate:DTV:LIMit:MERPeak:UPPer
        /// CALCulate&lt;1|2&gt;:DTV:UNIT:POWer:EVMRms DB | PCT
        /// CALCulate:DTV:LIMit:EVMRms:LOWer
        /// CALCulate:DTV:LIMit:EVMRms:UPPer
        /// CALCulate&lt;1|2&gt;:DTV:UNIT:POWer:EVMPeak DB | PCT
        /// CALCulate:DTV:LIMit:EVMPeak:LOWer
        /// CALCulate:DTV:LIMit:EVMPeak:UPPer
        /// CALCulate:DTV:LIMit:IMBalance:LOWer
        /// CALCulate:DTV:LIMit:IMBalance:UPPer
        /// CALCulate:DTV:LIMit:QERRor:LOWer
        /// CALCulate:DTV:LIMit:QERRor:UPPer
        /// CALCulate:DTV:LIMit:CSUPpression[:LOWer]
        /// CALCulate:DTV:LIMit:PJITter[:UPPer]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement">
        /// This control selects the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DTV_MERR (0) - Mod. Err. Rate RMS 
        /// RSSPECAN_VAL_DTV_MERP (1) - Mod. Err. Rate Peak
        /// RSSPECAN_VAL_DTV_EVMR (2) - Err. Vector Magnitude RMS
        /// RSSPECAN_VAL_DTV_EVMP (3) - Err. Vector Magnitude Peak
        /// RSSPECAN_VAL_DTV_IMB  (4) - Aplitude Imbalance
        /// RSSPECAN_VAL_DTV_QERR (5) - Quadrature Error
        /// RSSPECAN_VAL_DTV_SUP  (6) - Carrier Suppression
        /// RSSPECAN_VAL_DTV_PJIT (7) - Phase Jitter
        /// 
        /// Default Value: RSSPECAN_VAL_DTV_MERR (0)
        /// </param>
        /// <param name="Limit_Type">
        /// This control selects the type of the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIM_LOW (0) - Lower
        /// RSSPECAN_VAL_LIM_UPP (1) - Upper
        /// 
        /// Default Value: RSSPECAN_VAL_LIM_LOW (0)
        /// 
        /// Note:
        /// 
        /// (1) Upper is not available for Carrier Suppression
        /// 
        /// (2) Lower is not available for Phase Jitter
        /// </param>
        /// <param name="Limit_Value">
        /// This control sets the limit value.
        /// 
        /// Valid Values: 
        /// Mod. Err. Rate RMS: 
        /// Mod. Err. Rate Peak:
        /// Err. Vector Magnitude RMS: 
        /// Err. Vector Magnitude Peak: 
        ///                     0 to 80 dB for Lower
        ///                     0.01 to 100% for Upper 
        /// Amplitude Imbalance: -100.0 to 100.0 %
        /// Quadrature Error: -180? to 180?
        /// Carrier Suppression: 0 to 200 dB
        /// Phase Jitter: 0? to 180?
        /// 
        /// Default Value: 60.0
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_DTV_ERROR_UNIT
        /// RSSPECAN_ATTR_CATV_DTV_LIM_ERR_LOW
        /// RSSPECAN_ATTR_CATV_DTV_LIM_ERR_UPP
        /// RSSPECAN_ATTR_CATV_DTV_LIM_IMB
        /// RSSPECAN_ATTR_CATV_DTV_LIM_QERR
        /// RSSPECAN_ATTR_CATV_DTV_LIM_SUPP
        /// RSSPECAN_ATTR_CATV_DTV_LIM_PJIT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVDModErrLimit(int Measurement, int Limit_Type, double Limit_Value)
        {
            int pInvokeResult = PInvoke.ConfigureCATVDModErrLimit(this._handle, Measurement, Limit_Type, Limit_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the unit of the Echo Pattern measurement.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_DTV_EPAT_UNIT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DTV:UNIT:POWer:EPATtern s | m
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Unit">
        /// This control selects the unit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_SEC (0) - Seconds 
        /// RSSPECAN_VAL_UNIT_MIN (1) - Minutes
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_SEC (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_DTV_EPAT_UNIT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVDEchoPatternUnit(int Unit)
        {
            int pInvokeResult = PInvoke.ConfigureCATVDEchoPatternUnit(this._handle, Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the unit of measured values in Overview/Modulation Errors measurement
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_DTV_ERROR_UNIT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DTV:UNIT:POWer:EVMPeak DB | PCT
        /// CALCulate&lt;1|2&gt;:DTV:UNIT:POWer:EVMRms DB | PCT
        /// CALCulate&lt;1|2&gt;:DTV:UNIT:POWer:MERPeak DB | PCT
        /// CALCulate&lt;1|2&gt;:DTV:UNIT:POWer:MERRms DB | PCT
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement">
        /// This control selects the measured value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DTV_MERR (0) - Mod. Err. Rate RMS 
        /// RSSPECAN_VAL_DTV_MERP (1) - Mod. Err. Rate Peak
        /// RSSPECAN_VAL_DTV_EVMR (2) - Err. Vector Magnitude RMS
        /// RSSPECAN_VAL_DTV_EVMP (3) - Err. Vector Magnitude Peak
        /// 
        /// Default Value: RSSPECAN_VAL_DTV_MERR (0)
        /// </param>
        /// <param name="Unit">
        /// This control selects the unit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DB   (8) - dB 
        /// RSSPECAN_VAL_UNIT_PCT (13) - %
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_PCT (13)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_DTV_ERROR_UNIT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVDUnit(int Measurement, int Unit)
        {
            int pInvokeResult = PInvoke.ConfigureCATVDUnit(this._handle, Measurement, Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the TV analyzer measurement type.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_TV_MEAS
        /// 
        /// Remote-control command(s):
        /// CONFigure:TV:MEASurement TILT
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement">
        /// This control selects the measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TV_MEAS_TILT (0) - Tilt 
        /// 
        /// Default Value: RSSPECAN_VAL_TV_MEAS_TILT (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_CATV_TV_MEAS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVTVMeasurement(int Measurement)
        {
            int pInvokeResult = PInvoke.ConfigureCATVTVMeasurement(this._handle, Measurement);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates/deactivates a modulation standard for the Tilt measurement.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_TV_MOD_STANDARD_TILT_ACTIVATE
        /// RSSPECAN_ATTR_CATV_TV_MOD_STANDARD_TILT_DEACTIVATE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]TV:TILT:MSTandard:CDISable
        /// [SENSe&lt;1|2&gt;:]TV:TILT:MSTandard:CENable
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Modulation_Standard">
        /// This control defines the name of modulation standard.
        /// 
        /// Valid Values:
        /// any string (max. 15 chars)
        /// 
        /// Default Value: ""
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_TV_MOD_STANDARD_TILT_ACTIVATE
        /// RSSPECAN_ATTR_CATV_TV_MOD_STANDARD_TILT_DEACTIVATE
        /// </param>
        /// <param name="State">
        /// This control activates/deactivates selected standard for Titl measurement
        /// 
        /// Valid Values:
        /// 
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureCATVTVStandardForTitlMeasurement(string Modulation_Standard, bool State)
        {
            int pInvokeResult = PInvoke.ConfigureCATVTVStandardForTitlMeasurement(this._handle, Modulation_Standard, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads the measurement values (power, frequency) of the Carrier Power measurements.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_CARR_VCPA
        /// RSSPECAN_ATTR_CATV_ATV_CARR_VCF_VALUE
        /// RSSPECAN_ATTR_CATV_ATV_CARR_REL_POWER_VALUE
        /// RSSPECAN_ATTR_CATV_ATV_CARR_FREQ_OFFSET_VALUE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:ATV:RESult:CARRiers? VCPabsolute | VCFoffset | S1PRelative | S1IFoffset | S2PRelative | S2IFoffset
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement">
        /// This control selects the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ATV_VCP   (0) - Vision Carrier Power Absolute 
        /// RSSPECAN_VAL_ATV_VCF   (1) - Vision Carrier Frequency Offset
        /// RSSPECAN_VAL_ATV_SC1PR (2) - Sound Carrier 1 Power Relative
        /// RSSPECAN_VAL_ATV_SC1IF (3) - Sound Carrier 1 Intercarrier Frequency Offset
        /// RSSPECAN_VAL_ATV_SC2PR (4) - Sound Carrier 2 Power Relative
        /// RSSPECAN_VAL_ATV_SC2IF (5) - Sound Carrier 2 Intercarrier Frequency Offset
        /// 
        /// Default Value: RSSPECAN_VAL_ATV_VCP   (0)
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns the measured value.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryCATVACarrierResult(int Measurement, out double Result)
        {
            int pInvokeResult = PInvoke.QueryCATVACarrierResult(this._handle, Measurement, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads all measurement values (power, frequency) of the Carrier Power measurements.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:ATV:RESult:CARRiers? ALL
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Results">
        /// Returns array of all measured values in next order.
        /// 
        /// Idx 0 - vision carrier power absolute
        /// Idx 1 -  vision carrier frequency offset
        /// Idx 2 -  sound carrier 1 power relative
        /// Idx 3 -  sound carrier 1 intercarrier frequency offset
        /// Idx 4 -  sound carrier 2 power relative
        /// Idx 5 -  sound carrier 2 intercarrier frequency offset
        /// 
        /// Note:
        /// 
        /// (1) At least six element array has to be allocated for this control.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryCATVACarrierAllResults(double[] Results)
        {
            int pInvokeResult = PInvoke.QueryCATVACarrierAllResults(this._handle, Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the resultfor the carrier-to-noise, carrier-to-second order beat or carrier-to-composite triple beat ratio measurement.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_VALUE
        /// RSSPECAN_ATTR_CATV_ATV_CN_CN_VALUE
        /// RSSPECAN_ATTR_CATV_ATV_CSO_CSO_VALUE
        /// RSSPECAN_ATTR_CATV_ATV_CTB_CTB_VALUE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:ATV:RESult:CN? RPOWer | NRBW | NFCorrection | MFRelative | CN  
        /// CALCulate&lt;1|2&gt;:ATV:RESult:CSO? RPOWer | NFCorrection | MFRelative | CSO
        /// CALCulate&lt;1|2&gt;:ATV:RESult:CTB? RPOWer | NFCorrection | MFRelative | CTB
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement">
        /// This control selects the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ATV_MEAS_CN  (2) - C/N
        /// RSSPECAN_VAL_ATV_MEAS_CSO (3) - CSO
        /// RSSPECAN_VAL_ATV_MEAS_CTB (4) - CTB
        /// 
        /// Default Value: RSSPECAN_VAL_ATV_MEAS_CN  (2)
        /// 
        /// Note:
        /// 
        /// (1) C/N - lower limit for the carrier-to-noise ratio.
        /// 
        /// (2) CSO - lower limit for the carrier-to-second order beat ratio.
        /// 
        /// (3) CTB - lower limit for the carrier-to-composite triple beat ratio.
        /// </param>
        /// <param name="Result_Type">
        /// This control selects the type of the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ATV_RES_RPOW (0) - Reference Power
        /// RSSPECAN_VAL_ATV_RES_NFC  (1) - Noise Floor Corection
        /// RSSPECAN_VAL_ATV_RES_MFR  (2) - Measurement Frequency Relative
        /// RSSPECAN_VAL_ATV_RES_CR   (3) - Carrier To Ratio
        /// RSSPECAN_VAL_ATV_RES_NRBW (4) - Noise Reference BW
        /// 
        /// Default Value: RSSPECAN_VAL_ATV_RES_RPOW (0)
        /// 
        /// Note:
        /// 
        /// (1) Reference Power - returns reference power result value
        /// 
        /// (2) Noise Floor Corection - returns noise floor correction. 
        /// 
        /// (3) Measurement Frequency Relative - returns measurement frequency (relative) result value
        /// 
        /// (4) Carrier To Ratio - returns result value according selected measurement:
        ///     CN : returns carrier-to-noise ratio
        ///     CSO: returns carrier-to-second order beat ratio
        ///     CTB: returns carrier-to-composite triple beat ratio
        /// 
        /// (5) Noise Reference BW - returns noise reference bandwidth result value This selection is valid for CN measurement only.
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns the measured value.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryCATVACarrierRatioResult(int Measurement, int Result_Type, out double Result)
        {
            int pInvokeResult = PInvoke.QueryCATVACarrierRatioResult(this._handle, Measurement, Result_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads all measurement values.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:ATV:RESult:CN? ALL
        /// CALCulate&lt;1|2&gt;:ATV:RESult:CSO? ALL
        /// CALCulate&lt;1|2&gt;:ATV:RESult:CTB? ALL
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement">
        /// This control selects the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ATV_MEAS_CN  (2) - C/N
        /// RSSPECAN_VAL_ATV_MEAS_CSO (3) - CSO
        /// RSSPECAN_VAL_ATV_MEAS_CTB (4) - CTB
        /// 
        /// Default Value: RSSPECAN_VAL_ATV_MEAS_CN  (2)
        /// 
        /// Note:
        /// 
        /// (1) C/N - lower limit for the carrier-to-noise ratio.
        /// 
        /// (2) CSO - lower limit for the carrier-to-second order beat ratio.
        /// 
        /// (3) CTB - lower limit for the carrier-to-composite triple beat ratio.
        /// </param>
        /// <param name="Results">
        /// Returns array of all measured values in next order.
        /// 
        /// For CN measurement:
        /// Idx 0 - reference power
        /// Idx 1 - noise reference bandwidth
        /// Idx 2 - noise floor correction
        /// Idx 3 - measurement frequency (relative)
        /// Idx 4 - carrier-to-noise ratio
        /// 
        /// For CSO measurement:
        /// Idx 0 - reference power
        /// Idx 1 - noise reference bandwidth
        /// Idx 2 - measurement frequency relative
        /// Idx 3 - carrier-to-second order beat ratio
        /// 
        /// For CTB measurement:
        /// Idx 0 - reference power
        /// Idx 1 - noise reference bandwidth
        /// Idx 2 - measurement frequency relative
        /// Idx 3 - carrier-to-composite triple beat ratio
        /// 
        /// Note:
        /// 
        /// (1) Appropriate number of elements has to be allocated for this array control.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryCATVACarrierAllRatioResults(int Measurement, double[] Results)
        {
            int pInvokeResult = PInvoke.QueryCATVACarrierAllRatioResults(this._handle, Measurement, Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads the measurement values of the Hum measurement.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_HUM_VALUE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:ATV:RESult:HUM?
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Result">
        /// Returns the result value.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryCATVAHumResult(out double Result)
        {
            int pInvokeResult = PInvoke.QueryCATVAHumResult(this._handle, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads the measurement values of the Vision Modulation measurements.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_VMOD_RESULT_VALUE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:ATV:RESult:VMODulation? VCPower | RPC | MDEPth
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement">
        /// This control selects the measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ATV_VCP  (0) - Vision Carrier Power
        /// RSSPECAN_VAL_ATV_RPC  (1) - Residual Picture
        /// RSSPECAN_VAL_ATV_MDEP (2) - Modulation Depth 
        /// 
        /// Default Value: RSSPECAN_VAL_ATV_VCP  (0)
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns measured value.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryCATVAVisionModulationResult(int Measurement, out double Result)
        {
            int pInvokeResult = PInvoke.QueryCATVAVisionModulationResult(this._handle, Measurement, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads the measurement values of the Vision Modulation measurements.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:ATV:RESult:VMODulation?  ALL
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Results">
        /// Returns all vision modulation measured values in next order.
        /// 
        /// Returned Values:
        /// 
        /// Idx 0 - Vision Carrier Power
        /// Idx 1 - Residual Picature Carrier
        /// Idx 2 - Modulation Depth
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryCATVAVisionAllModulationResults(double[] Results)
        {
            int pInvokeResult = PInvoke.QueryCATVAVisionAllModulationResults(this._handle, Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the check limit result.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_CARR_LIM_VCF_RESULT
        /// RSSPECAN_ATTR_CATV_ATV_CARR_LIM_VCP_RESULT
        /// RSSPECAN_ATTR_CATV_ATV_CARR_LIM_FREQ_OFFSET_RESULT
        /// RSSPECAN_ATTR_CATV_ATV_CARR_LIM_REL_POWER_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:RESult:CARRiers:VCFoffset:LOWer?
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:RESult:CARRiers:VCFoffset:UPPer?
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:RESult:CARRiers:VCPabsolute:LOWer?
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:RESult:CARRiers:VCPabsolute:UPPer?
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:RESult:CARRiers:S1IFoffset:LOWer?
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:RESult:CARRiers:S1IFoffset:UPPer?
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:RESult:CARRiers:S1PRelative:LOWer?
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:RESult:CARRiers:S1PRelative:UPPer?
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:RESult:CARRiers:S2IFoffset:LOWer?
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:RESult:CARRiers:S2IFoffset:UPPer?
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:RESult:CARRiers:S2PRelative:LOWer?
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:RESult:CARRiers:S2PRelative:UPPer?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement">
        /// This control selects the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ATV_VCP   (0) - Vision Carrier Power Absolute 
        /// RSSPECAN_VAL_ATV_VCF   (1) - Vision Carrier Frequency Offset
        /// RSSPECAN_VAL_ATV_SC1PR (2) - Sound Carrier 1 Power Relative
        /// RSSPECAN_VAL_ATV_SC1IF (3) - Sound Carrier 1 Intercarrier Frequency Offset
        /// RSSPECAN_VAL_ATV_SC2PR (4) - Sound Carrier 2 Power Relative
        /// RSSPECAN_VAL_ATV_SC2IF (5) - Sound Carrier 2 Intercarrier Frequency Offset
        /// 
        /// Default Value: RSSPECAN_VAL_ATV_VCP   (0)
        /// 
        /// </param>
        /// <param name="Limit_Type">
        /// This control selects the type of the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIM_LOW (0) - Lower
        /// RSSPECAN_VAL_LIM_UPP (1) - Upper
        /// 
        /// Default Value: RSSPECAN_VAL_LIM_LOW (0)
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns the limit check result.
        /// 
        /// Returned Values:
        /// 
        /// 0 - FAIL
        /// 1 - PASS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryCATVACarrierLimitResult(int Measurement, int Limit_Type, out int Result)
        {
            int pInvokeResult = PInvoke.QueryCATVACarrierLimitResult(this._handle, Measurement, Limit_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the check limit result.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_LIM_CN_RESULT
        /// RSSPECAN_ATTR_CATV_ATV_LIM_CSO_RESULT
        /// RSSPECAN_ATTR_CATV_ATV_LIM_CTB_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:RESult:CN:CN[:LOWer]?
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:RESult:CSO:CSO[:LOWer]?
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:RESult:CTB:CTB[:LOWer]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement">
        /// This control selects the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ATV_MEAS_CN  (2) - C/N
        /// RSSPECAN_VAL_ATV_MEAS_CSO (3) - CSO
        /// RSSPECAN_VAL_ATV_MEAS_CTB (4) - CTB
        /// 
        /// Default Value: RSSPECAN_VAL_ATV_MEAS_CN  (2)
        /// 
        /// Note:
        /// 
        /// (1) C/N - lower limit for the carrier-to-noise ratio.
        /// 
        /// (2) CSO - lower limit for the carrier-to-second order beat ratio.
        /// 
        /// (3) CTB - lower limit for the carrier-to-composite triple beat ratio.
        /// </param>
        /// <param name="Limit_Type">
        /// This control selects the type of the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIM_LOW (0) - Lower
        /// 
        /// Default Value: RSSPECAN_VAL_LIM_LOW (0)
        /// 
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns the limit check result.
        /// 
        /// Returned Values:
        /// 
        /// 0 - FAIL
        /// 1 - PASS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryCATVACarrierRatioLimitResult(int Measurement, int Limit_Type, out int Result)
        {
            int pInvokeResult = PInvoke.QueryCATVACarrierRatioLimitResult(this._handle, Measurement, Limit_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the limit check for the Hum measurement.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_LIM_HUM_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:RESult:HUM:HUM:LOWer?
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:RESult:HUM:HUM:UPPer?
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit_Type">
        /// This control selects the type of the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIM_LOW (0) - Lower
        /// RSSPECAN_VAL_LIM_UPP (1) - Upper
        /// 
        /// Default Value: RSSPECAN_VAL_LIM_LOW (0)
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns the limit check result.
        /// 
        /// Returned Values:
        /// 
        /// 0 - FAIL
        /// 1 - PASS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryCATVAHumLimitResult(int Limit_Type, out int Result)
        {
            int pInvokeResult = PInvoke.QueryCATVAHumLimitResult(this._handle, Limit_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the check limit result.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_ATV_LIM_VMOD_MDEP_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:RESult:VMODulation:MDEPth:LOWer?
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:RESult:VMODulation:MDEPth:UPPer?
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:RESult:VMODulation:RPC:LOWer?
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:RESult:VMODulation:RPC:UPPer?
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:RESult:VMODulation:VCPower:LOWer?
        /// CALCulate&lt;1|2&gt;:ATV:LIMit:RESult:VMODulation:VCPower:UPPer?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement">
        /// This control selects the measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ATV_VCP  (0) - Vision Carrier Power
        /// RSSPECAN_VAL_ATV_RPC  (1) - Residual Picture
        /// RSSPECAN_VAL_ATV_MDEP (2) - Modulation Depth 
        /// 
        /// Default Value: RSSPECAN_VAL_ATV_VCP  (0)
        /// 
        /// </param>
        /// <param name="Limit_Type">
        /// This control selects the type of the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIM_LOW (0) - Lower
        /// RSSPECAN_VAL_LIM_UPP (1) - Upper
        /// 
        /// Default Value: RSSPECAN_VAL_LIM_LOW (0)
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns the limit check result.
        /// 
        /// Returned Values:
        /// 
        /// 0 - FAIL
        /// 1 - PASS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryCATVAVisionModulationLimitResult(int Measurement, int Limit_Type, out int Result)
        {
            int pInvokeResult = PInvoke.QueryCATVAVisionModulationLimitResult(this._handle, Measurement, Limit_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads the measurement value of spectrum.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_DTV_SPEC
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DTV:RESult? SALower | SAUPper
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement">
        /// This control selects the measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DTV_SPEC_SAL  (0) - Shoulder Attenuation Lower 
        /// RSSPECAN_VAL_DTV_SPEC_SAUP (1) - Shoulder Attenuation Upper
        /// 
        /// Default Value: RSSPECAN_VAL_DTV_SPEC_SAL  (0)
        /// </param>
        /// <param name="Result">
        /// Returns the measured value.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryCATVDSpectrumResult(int Measurement, out double Result)
        {
            int pInvokeResult = PInvoke.QueryCATVDSpectrumResult(this._handle, Measurement, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the result value of Channel Power measurement.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_DTV_CHANNEL_POWER_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer:FUNCtion:POWer:RESult? CPOWer
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Result">
        /// Returns the result value.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryCATVDChanelPowerResult(out double Result)
        {
            int pInvokeResult = PInvoke.QueryCATVDChanelPowerResult(this._handle, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the result value.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_DTV_ERROR
        /// RSSPECAN_ATTR_CATV_DTV_OVER_CARR_FREQ_OFFSET
        /// RSSPECAN_ATTR_CATV_DTV_OVER_SRATE_OFFSET
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DTV:RESult? MERRms | MERPeak | EVMRms | EVMPeak |  CFOFfset | SROFfset
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement">
        /// This control selects the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DTV_MERR (0) - Mod. Err. Rate RMS 
        /// RSSPECAN_VAL_DTV_MERP (1) - Mod. Err. Rate Peak
        /// RSSPECAN_VAL_DTV_EVMR (2) - Err. Vector Magnitude RMS
        /// RSSPECAN_VAL_DTV_EVMP (3) - Err. Vector Magnitude Peak
        /// RSSPECAN_VAL_DTV_CFOF (4) - Carrier Frequency Offset
        /// RSSPECAN_VAL_DTV_SROF (5) - Symbol Rate Offset
        /// 
        /// Default Value: RSSPECAN_VAL_DTV_MERR (0)
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns the result value.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryCATVDOverviewResult(int Measurement, out double Result)
        {
            int pInvokeResult = PInvoke.QueryCATVDOverviewResult(this._handle, Measurement, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the result values.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DTV:RESult? ALL
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Results">
        /// Returns array of all measured values in next order.
        /// 
        /// Idx 0 - root mean square of the modulation error rate
        /// Idx 1 - peak of the modulation error rate
        /// Idx 2 - root mean square of the error vector magnitude
        /// Idx 3 - peak of the error vector magnitude
        /// Idx 4 - carrier frequency offset
        /// Idx 5 - symbol rate offset
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryCATVDOverviewAllResults(double[] Results)
        {
            int pInvokeResult = PInvoke.QueryCATVDOverviewAllResults(this._handle, Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the result value.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_DTV_ERROR
        /// RSSPECAN_ATTR_CATV_DTV_MERR_IMB
        /// RSSPECAN_ATTR_CATV_DTV_MERR_QERR
        /// RSSPECAN_ATTR_CATV_DTV_MERR_CSUP
        /// RSSPECAN_ATTR_CATV_DTV_MERR_PJIT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DTV:RESult? MERRms | MERPeak | EVMRms | EVMPeak | IMBalance | QERRor | CSUPpression | PJITter
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement">
        /// This control selects the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DTV_MERR (0) - Mod. Err. Rate RMS 
        /// RSSPECAN_VAL_DTV_MERP (1) - Mod. Err. Rate Peak
        /// RSSPECAN_VAL_DTV_EVMR (2) - Err. Vector Magnitude RMS
        /// RSSPECAN_VAL_DTV_EVMP (3) - Err. Vector Magnitude Peak
        /// RSSPECAN_VAL_DTV_IMB  (4) - Aplitude Imbalance
        /// RSSPECAN_VAL_DTV_QERR (5) - Quadrature Error
        /// RSSPECAN_VAL_DTV_SUP  (6) - Carrier Suppression
        /// RSSPECAN_VAL_DTV_PJIT (7) - Phase Jitter
        /// 
        /// Default Value: RSSPECAN_VAL_DTV_MERR (0)
        /// </param>
        /// <param name="Result">
        /// Returns the result value.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryCATVDModulationErrorsResult(int Measurement, out double Result)
        {
            int pInvokeResult = PInvoke.QueryCATVDModulationErrorsResult(this._handle, Measurement, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the all result value.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DTV:RESult? ALL
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Results">
        /// Returns array of all measured values in next order.
        /// 
        /// Idx 0 - amplitude imbalance
        /// Idx 1 - quadrature error
        /// Idx 2 - carrier suppression
        /// Idx 3 - phase jitter
        /// Idx 4 - MER (rms)
        /// Idx 5 - MER (peak)
        /// Idx 6 - EVM (rms)
        /// Idx 7 - EVM (peak)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryCATVDModulationErrorsAllResults(double[] Results)
        {
            int pInvokeResult = PInvoke.QueryCATVDModulationErrorsAllResults(this._handle, Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the results of statistical measurements of a recorded trace.
        /// 
        /// Note:
        /// 
        /// This function is only available with the R&amp;S FSL with FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_STAT_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistic:RESult&lt;1...4&gt;? MEAN | PEAK | CFAC
        /// *CLS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Trace_Number">
        /// Selects the trace.
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Result_Type">
        /// Selects the result.
        /// 
        /// Valid Values:
        /// 
        /// RSSPECAN_VAL_STAT_MEAN (0) - Mean (Default Value)
        /// RSSPECAN_VAL_STAT_PEAK (1) - Peak
        /// RSSPECAN_VAL_STAT_CFAC (2) - Crest Factor
        /// </param>
        /// <param name="Result">
        /// Results of all measurements &lt;mean power&gt;,&lt;peak power&gt;,&lt;crest factor&gt;.
        /// 
        ///  MEAN - Average power in dBm measured over the measurement time.
        ///  PEAK - Peak power in dBm measured over the measurement time.
        ///  CFACtor - Determined CREST factor (= ratio of peak power to    
        ///            average power) in dB.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryCATVDSignalStatisticsResult(int Trace_Number, int Result_Type, out double Result)
        {
            int pInvokeResult = PInvoke.QueryCATVDSignalStatisticsResult(this._handle, Trace_Number, Result_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the results of statistical measurements of a recorded trace.
        /// 
        /// Note:
        /// 
        /// This function is only available with the R&amp;S FSL with FS-K20 option.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistic:RESult&lt;1...4&gt;? ALL
        /// *CLS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Trace_Number">
        /// Selects the trace.
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Results">
        /// Results of all measurements &lt;mean power&gt;,&lt;peak power&gt;,&lt;crest factor&gt;.
        /// 
        ///  MEAN - Average power in dBm measured over the measurement time.
        ///  PEAK - Peak power in dBm measured over the measurement time.
        ///  CFACtor - Determined CREST factor (= ratio of peak power to    
        ///            average power) in dB.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryCATVDSignalStatisticsAllResults(int Trace_Number, double[] Results)
        {
            int pInvokeResult = PInvoke.QueryCATVDSignalStatisticsAllResults(this._handle, Trace_Number, Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the check limit result.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_DTV_LIM_ERROR_RESULT
        /// RSSPECAN_ATTR_CATV_DTV_CARR_LIM_FREQ_OFFSET_RESULT
        /// RSSPECAN_ATTR_CATV_DTV_LIM_SYMB_RATE_OFFSET_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate:DTV:LIMit:RESult:MERRms:LOWer?
        /// CALCulate:DTV:LIMit:RESult:MERRms:UPPer?
        /// CALCulate:DTV:LIMit:RESult:MERPeak:LOWer?
        /// CALCulate:DTV:LIMit:RESult:MERPeak:UPPer?
        /// CALCulate:DTV:LIMit:RESult:EVMRms:LOWer?
        /// CALCulate:DTV:LIMit:RESult:EVMRms:UPPer?
        /// CALCulate:DTV:LIMit:RESult:EVMPeak:LOWer?
        /// CALCulate:DTV:LIMit:RESult:EVMPeak:UPPer?
        /// CALCulate:DTV:LIMit:RESult:CFOFfset:LOWer?
        /// CALCulate:DTV:LIMit:RESult:CFOFfset:UPPer?
        /// CALCulate:DTV:LIMit:RESult:SROFfset:LOWer?
        /// CALCulate:DTV:LIMit:RESult:SROFfset:UPPer?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement">
        /// This control selects the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DTV_MERR (0) - Mod. Err. Rate RMS 
        /// RSSPECAN_VAL_DTV_MERP (1) - Mod. Err. Rate Peak
        /// RSSPECAN_VAL_DTV_EVMR (2) - Err. Vector Magnitude RMS
        /// RSSPECAN_VAL_DTV_EVMP (3) - Err. Vector Magnitude Peak
        /// RSSPECAN_VAL_DTV_CFOF (4) - Carrier Frequency Offset
        /// RSSPECAN_VAL_DTV_SROF (5) - Symbol Rate Offset
        /// 
        /// Default Value: RSSPECAN_VAL_DTV_MERR (0)
        /// 
        /// </param>
        /// <param name="Limit_Type">
        /// This control selects the type of the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIM_LOW (0) - Lower
        /// RSSPECAN_VAL_LIM_UPP (1) - Upper
        /// 
        /// Default Value: RSSPECAN_VAL_LIM_LOW (0)
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns the limit check result.
        /// 
        /// Returned Values:
        /// 
        /// 0 - FAIL
        /// 1 - PASS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryCATVDOverviewLimitResult(int Measurement, int Limit_Type, out int Result)
        {
            int pInvokeResult = PInvoke.QueryCATVDOverviewLimitResult(this._handle, Measurement, Limit_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the check limit result.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_DTV_LIM_CHANNEL_POWER_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate:DTV:LIMit:RESult:CHPower?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Result">
        /// Returns the limit check result.
        /// 
        /// Returned Values:
        /// 
        /// 0 - FAIL
        /// 1 - PASS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryCATVDChannelPowerLimitResult(out int Result)
        {
            int pInvokeResult = PInvoke.QueryCATVDChannelPowerLimitResult(this._handle, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the check limit result.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with R&amp;S FSL and FS-K20 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_CATV_DTV_LIM_ERROR_RESULT
        /// RSSPECAN_ATTR_CATV_DTV_LIM_IMB_RESULT
        /// RSSPECAN_ATTR_CATV_DTV_LIM_QERR_RESULT
        /// RSSPECAN_ATTR_CATV_DTV_CARR_LIM_CARR_SUPP_RESULT
        /// RSSPECAN_ATTR_CATV_DTV_LIM_PJIT_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate:DTV:LIMit:RESult:MERRms:LOWer?
        /// CALCulate:DTV:LIMit:RESult:MERRms:UPPer?
        /// CALCulate:DTV:LIMit:RESult:MERPeak:LOWer?
        /// CALCulate:DTV:LIMit:RESult:MERPeak:UPPer?
        /// CALCulate:DTV:LIMit:RESult:EVMRms:LOWer?
        /// CALCulate:DTV:LIMit:RESult:EVMRms:UPPer?
        /// CALCulate:DTV:LIMit:RESult:EVMPeak:LOWer?
        /// CALCulate:DTV:LIMit:RESult:EVMPeak:UPPer?
        /// CALCulate:DTV:LIMit:RESult:IMBalance:LOWer?
        /// CALCulate:DTV:LIMit:RESult:IMBalance:UPPer?
        /// CALCulate:DTV:LIMit:RESult:QERRor:LOWer?
        /// CALCulate:DTV:LIMit:RESult:QERRor:UPPer?
        /// CALCulate:DTV:LIMit:RESult:CSUPpression[:LOWer]?
        /// CALCulate:DTV:LIMit:RESult:PJITter[:UPPer]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement">
        /// This control selects the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DTV_MERR (0) - Mod. Err. Rate RMS 
        /// RSSPECAN_VAL_DTV_MERP (1) - Mod. Err. Rate Peak
        /// RSSPECAN_VAL_DTV_EVMR (2) - Err. Vector Magnitude RMS
        /// RSSPECAN_VAL_DTV_EVMP (3) - Err. Vector Magnitude Peak
        /// RSSPECAN_VAL_DTV_IMB  (4) - Aplitude Imbalance
        /// RSSPECAN_VAL_DTV_QERR (5) - Quadrature Error
        /// RSSPECAN_VAL_DTV_SUP  (6) - Carrier Suppression
        /// RSSPECAN_VAL_DTV_PJIT (7) - Phase Jitter
        /// 
        /// Default Value: RSSPECAN_VAL_DTV_MERR (0)
        /// </param>
        /// <param name="Limit_Type">
        /// This control selects the type of the limit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIM_LOW (0) - Lower
        /// RSSPECAN_VAL_LIM_UPP (1) - Upper
        /// 
        /// Default Value: RSSPECAN_VAL_LIM_LOW (0)
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns the limit check result.
        /// 
        /// Returned Values:
        /// 
        /// 0 - FAIL
        /// 1 - PASS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryCATVDModulationErrorsLimitResult(int Measurement, int Limit_Type, out int Result)
        {
            int pInvokeResult = PInvoke.QueryCATVDModulationErrorsLimitResult(this._handle, Measurement, Limit_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the Noise Figure and Gain Measurement mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NOISE_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument[:SELect] NOISe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int NoiseMode()
        {
            int pInvokeResult = PInvoke.NoiseMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Noise Frequency Settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FREQUENCY_START
        /// RSSPECAN_ATTR_FREQUENCY_STOP
        /// RSSPECAN_ATTR_NOISE_FREQ_STEP
        /// RSSPECAN_ATTR_NOISE_CONF_SYST_LOSC
        /// RSSPECAN_ATTR_NOISE_CONF_MODE_DUT
        /// RSSPECAN_ATTR_NOISE_CONF_SYST_LOSC_FREQ
        /// RSSPECAN_ATTR_NOISE_CONF_SYST_INT_FREQ
        /// RSSPECAN_ATTR_NOISE_CORR_EREJ
        /// RSSPECAN_ATTR_NOISE_FREQ_FIXED
        /// 
        /// Remote-control command(s):
        /// [SENSe:]FREQuency:STARt[?] &lt;numeric value&gt; [Hz]
        /// [SENSe:]FREQuency:STOP[?] &lt;numeric value&gt; [Hz]
        /// [SENSe:]FREQuency:STEP[?] &lt;numeric value&gt; [Hz]
        /// [SENSe:]CONFigure:MODE:SYSTem:LOSCillator[?] FIXed | VARiable
        /// [SENSe:]CONFigure:MODE:DUT[?] AMPLifier | DOWNc | UPConv
        /// [SENSe:]CONFigure:MODE:SYSTem:LOSCillator:FREQuency[?] &lt;numeric value&gt; [Hz]
        /// [SENSe:]CONFigure:MODE:SYSTem:IF:FREQuency[?] &lt;numeric value&gt; [Hz]
        /// [SENSe:]CORRection:IREJection[?] &lt;numeric value &gt; [dB]
        /// [SENSe:]FREQuency[:CW|:FIXed][?] &lt;numeric value&gt; [Hz]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Start_Frequency">
        /// The start frequency of the frequency sweep.
        /// 
        /// Units: Hertz
        /// 
        /// Valid Values:
        /// 0.0 to 8.0e+9
        /// 
        /// Default Value: 220.0e+6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_START
        /// </param>
        /// <param name="Stop_Frequency">
        /// The stop frequency of the frequency sweep.
        /// 
        /// Units: Hertz
        /// 
        /// Valid Values:
        /// 0.0 to 8.0e+9
        /// 
        /// Default Value: 230.0e+6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_STOP
        /// </param>
        /// <param name="Step_Frequency">
        /// The step frequency of the frequency sweep.
        /// 
        /// Units: Hertz
        /// 
        /// Valid Values:
        /// 1.0 to 8.0e+9
        /// 
        /// Default Value: 10.0e+6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_STOP
        /// </param>
        /// <param name="Mode">
        /// Select the measurement mode according to the type of Device Under Test.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_NOISE_DUT_MODE_DIRECT        (0) - Direct
        /// RSSPECAN_VAL_NOISE_DUT_MODE_FIXED_IF_PLUS (1) - Fixed IF, LO = RF + IF
        /// RSSPECAN_VAL_NOISE_DUT_MODE_FIXED_IF_ABS  (2) - Fixed IF, LO = abs(RF - IF)
        /// RSSPECAN_VAL_NOISE_DUT_MODE_FIXED_LO_PLUS (3) - Fixed LO, IF = RF + LO
        /// RSSPECAN_VAL_NOISE_DUT_MODE_FIXED_LO_ABS  (4) - Fixed LO, IF = abs(RF - LO)
        /// 
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_NOISE_DUT_MODE_DIRECT
        /// 
        /// Attribute:
        /// RS_ATTR_INSTRUMENT_MODEL
        /// 
        /// Notes:
        /// 
        /// (0) The "Direct" mode sets the attribute RSSPECAN_ATTR_NOISE_CONF_MODE_DUT to AMPL.
        /// 
        /// (1) The "Fixed IF, LO = RF + IF" mode sets the attribute RSSPECAN_ATTR_NOISE_CONF_MODE_DUT to DOWN and RSSPECAN_ATTR_NOISE_CONF_SYST_LOSC to VAR. This value is not valid for R&amp;S FSL, R&amp;S FSV or R&amp;S ESL.
        /// 
        /// (2) The "Fixed IF, LO = abs(RF - IF)" mode sets the attribute RSSPECAN_ATTR_NOISE_CONF_MODE_DUT to UP and RSSPECAN_ATTR_NOISE_CONF_SYST_LOSC to VAR. This value is not valid for R&amp;S FSL, R&amp;S FSV or R&amp;S ESL.
        /// 
        /// (3) The "Fixed LO, IF = RF + LO" mode sets the attribute RSSPECAN_ATTR_NOISE_CONF_MODE_DUT to UP and RSSPECAN_ATTR_NOISE_CONF_SYST_LOSC to FIX.
        /// 
        /// (4) The "Fixed LO, IF = abs(RF - LO)" mode sets the attribute RSSPECAN_ATTR_NOISE_CONF_MODE_DUT to DOWN and RSSPECAN_ATTR_NOISE_CONF_SYST_LOSC to FIX.
        /// </param>
        /// <param name="Fixed_LO">
        /// Specifies a fixed local oscillator frequency for a new frequency measurement list that requires computing. This value will form the basis of one of the input criteria for computing a new frequency list.
        /// 
        /// Units: Hertz
        /// 
        /// Valid Values:
        /// 0.0 to 999.99e+9
        /// 
        /// Default Value: 0.0 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_CONF_SYST_LOSC_FREQ
        /// 
        /// Notes:
        /// 
        /// (1) The attribute will not have any immediate effect if a direct frequency list is being used (which is set by the RSSPECAN_ATTR_NOISE_CONF_MODE_DUT - AMPL attribute) or a variable intermediate frequency has been specified (which is set by the RSSPECAN_ATTR_NOISE_CONF_SYST_LOSC - FIX attribute).
        /// </param>
        /// <param name="Fixed_IF">
        /// Specifies a fixed intermediate frequency for a new frequency measurement list that requires computing. This value will form the basis of one of the input criteria for computing a new frequency list.
        /// 
        /// Units: Hertz
        /// 
        /// Valid Values:
        /// 0.0 to 8.0e+9
        /// 
        /// Default Value: 0.0 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_CONF_SYST_INT_FREQ
        /// 
        /// Notes:
        /// 
        /// (1) The attribute will not have any immediate effect if a direct frequency list is being used (which is set by the RSSPECAN_ATTR_NOISE_CONF_MODE_DUT - AMPL attribute) or a variable intermediate frequency has been specified (which is set by the RSSPECAN_ATTR_NOISE_CONF_SYST_LOSC - FIX attribute).
        /// </param>
        /// <param name="Image_Rejection">
        /// Allows an image rejection value associated with DUT to be entered which will be effective across the complete range of measurement frequencies.
        /// 
        /// Units: dB
        /// 
        /// Valid Values:
        /// 0.0 to 999.99
        /// 
        /// Default Value: 999.99 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_CORR_EREJ
        /// 
        /// Notes:
        /// 
        /// (1) The value entered is applied across the complete frequency range. The default value of 999.99 dB means that the second sideband does not noticeably affect the measurement result because a suppression of 999.99 dB is applied to it. This corresponds to the generally used single-sideband (SSB) measurement. 
        /// An entry of 0 dB would mean that both sidebands are converted to the same extent - this corresponds to a double-sideband (DSB) measurement. 
        /// Thus, for a SSB mixer, Image Rejection should be set to 999.99 dB. For a DSB mixer (one without any image rejection), the value should be 0 dB. Using an Image Rejection value of 999.99 dB for a DSB measurement will produce measurement errors: the measured noise figure will be 3 dB lower than the actual noise figure and the measured gain will be 3 dB higher than the actual gain. For comparison with noise test systems of other manufacturers, the
        /// Image Rejection should be set to 999.99 dB (SSB measurement) as this is the setting implicitly used by almost all manufacturers.
        /// </param>
        /// <param name="Fixed_Frequency">
        /// Specifies a single fixed frequency to measure noise and gain continuously. The value specified with this attribute is only used when a single frequency measurement is initiated.
        /// 
        /// Units: Hertz
        /// 
        /// Valid Values:
        /// 0.0 to 8.0e+9
        /// 
        /// Default Value: 550.0e+6 Hz 
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_FREQ_FIXED
        /// 
        /// Notes:
        /// 
        /// (1) Available when results are displayed in tabular
        /// form (via ConfigureNoiseResultSettings or attribute RSSPECAN_ATTR_NOISE_DISP_TABLE ON) and a valid result is selected in the table of results.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureNoiseFrequencySettings(double Start_Frequency, double Stop_Frequency, double Step_Frequency, int Mode, double Fixed_LO, double Fixed_IF, double Image_Rejection, double Fixed_Frequency)
        {
            int pInvokeResult = PInvoke.ConfigureNoiseFrequencySettings(this._handle, Start_Frequency, Stop_Frequency, Step_Frequency, Mode, Fixed_LO, Fixed_IF, Image_Rejection, Fixed_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Noise Frequency Table.
        /// 
        /// Attribute(s):
        /// None
        /// 
        /// Remote-control command(s):
        /// [SENSe:]FREQuency:LIST:DATA[?] &lt;numeric value&gt; [Hz],&lt;numeric value&gt; [Hz],&lt;numeric value&gt;[Hz], ... 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Number_Of_Values">
        /// Specify the number of values supplied for the frequency table in the controls RF Frequency Values, Local Oscilator Frequency Values and Inetermediate Frequency Values.
        /// 
        /// Valid Values:
        /// 1 to 100
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Receiver_Frequency_Values">
        /// Enter the Receiver frequency values for the frequency table.
        /// 
        /// Units: Hertz
        /// 
        /// Valid Values:
        /// 0.0 to 999.99e+9
        /// 
        /// Default Value:
        /// None
        /// 
        /// Notes:
        /// 
        /// (1) The number of values for the Receiver Frequency, Local Oscilator Frequency and the corresponding Intermediate Frequency have to be identical.
        /// </param>
        /// <param name="Local_Oscilator_Frequency_Val_">
        /// Enter the Local Oscilator frequency values for the frequency table.
        /// 
        /// Units: Hertz
        /// 
        /// Valid Values:
        /// 0.0 to 999.99e+9
        /// 
        /// Default Value:
        /// None
        /// 
        /// Notes:
        /// 
        /// (1) The number of values for the Receiver Frequency, Local Oscilator Frequency and the corresponding Intermediate Frequency have to be identical.
        /// </param>
        /// <param name="Intermediate_Frequency_Values">
        /// Enter the Intermediate frequency values for the frequency table.
        /// 
        /// Units: Hertz
        /// 
        /// Valid Values:
        /// 0.0 to 8.0e+9
        /// 
        /// Default Value:
        /// None
        /// 
        /// Notes:
        /// 
        /// (1) The number of values for the Receiver Frequency, Local Oscilator Frequency and the corresponding Intermediate Frequency have to be identical.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureNoiseFrequencyTable(int Number_Of_Values, double[] Receiver_Frequency_Values, double[] Local_Oscilator_Frequency_Val_, double[] Intermediate_Frequency_Values)
        {
            int pInvokeResult = PInvoke.ConfigureNoiseFrequencyTable(this._handle, Number_Of_Values, Receiver_Frequency_Values, Local_Oscilator_Frequency_Val_, Intermediate_Frequency_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Noise Measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NOISE_CONF_CORR
        /// RSSPECAN_ATTR_NOISE_CONF_LIST_CONT
        /// RSSPECAN_ATTR_NOISE_CONF_LIST_SING
        /// RSSPECAN_ATTR_NOISE_CONF_SING
        /// 
        /// Remote-control command(s):
        /// CONFigure:CORRection
        /// CONFigure:LIST:CONTinuous
        /// CONFigure:LIST:SINGle
        /// CONFigure:SINGle
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement_Mode">
        /// Select the measurement mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_NOISE_SECOND_STAGE_CORR_MEAS (0) - Second Stage Correction Measurement
        /// RSSPECAN_VAL_NOISE_CONT_FREQ_LIST_MEAS    (1) - Continuous Frequency List Measurement
        /// RSSPECAN_VAL_NOISE_SINGLE_FREQ_LIST_MEAS  (2) - Single Frequency List Measurement
        /// RSSPECAN_VAL_NOISE_SINGLE_FREQ_MEAS       (3) - Single Frequency Measurement
        /// 
        /// Default Value:
        /// Second Stage Correction Measurement
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NOISE_CONF_CORR
        /// RSSPECAN_ATTR_NOISE_CONF_LIST_CONT
        /// RSSPECAN_ATTR_NOISE_CONF_LIST_SING
        /// RSSPECAN_ATTR_NOISE_CONF_SING
        /// 
        /// Notes:
        /// 
        /// (1) Second Stage Correction Measurement - After this type has been selected the second stage correction measurement will be the measurement started when the user issues the INITiate command.
        /// 
        /// (2) Continuous Frequency List Measurement - After this type has been selected a continuous frequency list measurement will be the measurement started when the user issues the INITiate command.
        /// 
        /// (3) Single Frequency List Measurement - After this type has been selected a single frequency list measurement will be the measurement started when the user issues the INITiate command.
        /// 
        /// (4) Single Frequency Measurement - After this type has been selected a single frequency measurement will be the measurement started when the user issues the INITiate command.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureNoiseMeasurementMode(int Measurement_Mode)
        {
            int pInvokeResult = PInvoke.ConfigureNoiseMeasurementMode(this._handle, Measurement_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Noise Measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NOISE_CORR_STAT
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CORRection[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="_2nd_Stage_Correction">
        /// Select the measurement mode.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_CORR_STAT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureNoiseCalibration(bool _2nd_Stage_Correction)
        {
            int pInvokeResult = PInvoke.ConfigureNoiseCalibration(this._handle, System.Convert.ToUInt16(_2nd_Stage_Correction));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Noise Analyzer Settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_RESOLUTION_BANDWIDTH
        /// RSSPECAN_ATTR_SWEEP_TIME
        /// RSSPECAN_ATTR_NOISE_CONF_DUT_SETT_TIME 
        /// RSSPECAN_ATTR_NUMBER_OF_SWEEPS
        /// RSSPECAN_ATTR_ATTENUATION
        /// RSSPECAN_ATTR_NOISE_CONF_DUT_GAIN
        /// RSSPECAN_ATTR_NOISE_DISP_TRAC_REF_LEV_STAT
        /// RSSPECAN_ATTR_REFERENCE_LEVEL
        /// RSSPECAN_ATTR_NOISE_INP_PRES_STAT
        /// RSSPECAN_ATTR_NOISE_INP_GAIN_STAT
        /// 
        /// Remote-control command(s):
        /// [SENSe1:]BANDwidth|BWIDth[:RESolution][?] &lt;numeric value&gt; [Hz]
        /// [SENSe1:]SWEep:TIME[?] &lt;numeric value&gt; [s]
        /// SYSTemCONFigure:DUT:STIMe[?] &lt;numeric value&gt; [s]
        /// [SENSe1:]SWEep:COUNt[?] &lt;numeric value&gt;
        /// INPut1:ATTenuation[?] &lt;numeric_value&gt; [dB]
        /// SYSTemCONFigure:DUT:GAIN[?] &lt;numeric value&gt; [dB]
        /// DISPlay[:WINDow&lt;1&gt;]:TRACe&lt;1|2&gt;:Y[:SCALe]:RLEVel:AUTO[?] ON | OFF
        /// DISPlay[:WINDow&lt;1&gt;]:TRACe&lt;1|2&gt;:Y[:SCALe]:RLEVel[?] &lt;numeric_value&gt; [dBm]
        /// INPut:PRESelection[:STATe][?] ON | OFF
        /// INPut:GAIN:STATe[?] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Resolution_Bandwidth">
        /// Specifies the measurement resolution bandwidth.
        /// 
        /// Units: Hertz
        /// 
        /// Valid Values:
        /// 10 to 50e+6
        /// 
        /// Default Value: 1.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RESOLUTION_BANDWIDTH
        /// </param>
        /// <param name="Sweep_Time">
        /// The length of time to complete one sweep.
        /// 
        /// Units: seconds
        /// 
        /// Valid Values:
        /// 1e-6 to 16e+3
        /// 
        /// Default Value: 1.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SWEEP_TIME
        /// </param>
        /// <param name="Settling_Time">
        /// This control allows the DUT settling time to be modified. It represents the time to wait for the DUT to settle after a noise source has been turned on or off.
        /// 
        /// Units: seconds
        /// 
        /// Valid Values:
        /// 0 to 20
        /// 
        /// Default Value: 0.05
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_CONF_DUT_SETT_TIME
        /// </param>
        /// <param name="Average">
        /// This control specifies the number of sweeps over which the measurement results are to be averaged. The higher the number of sweeps averaged the more accurate the results however the more sweeps required the longer the measurement takes to perform.
        /// 
        /// Valid Values:
        /// 1 to 32767 
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NUMBER_OF_SWEEPS
        /// </param>
        /// <param name="RF_Attenuation">
        /// Specifies the input attenuation (in positive dB). This means that if 10dB is selected, the result is a reduction in the signal level of 10 dB.
        /// 
        /// Units: dB
        /// 
        /// Valid Values: 
        /// 0 to 75
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ATTENUATION
        /// 
        /// Note(s):
        /// 
        /// (1) Not available for R&amp;S FMU instrument.
        /// </param>
        /// <param name="Range">
        /// This control allows the maximum gain of the DUT to be specified. 
        /// 
        /// Units: dB
        /// 
        /// Valid Values:
        /// 10 to 999.99
        /// 
        /// Default Value: 30.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_CONF_DUT_GAIN
        /// </param>
        /// <param name="Automatic_Ref_Level">
        /// This control enables or disables automatic reference level detection. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_DISP_TRAC_REF_LEV_STAT
        /// </param>
        /// <param name="Ref_Level">
        /// The calibrated vertical position of the captured data used as a reference for amplitude measurements. This is typically set to a value slightly higher than the highest expected signal level.  
        /// 
        /// Units: Set by user with RSSPECAN_ATTR_AMPLITUDE_UNITS attribute.
        /// 
        /// Valid Values:
        /// -130 to 5
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_LEVEL
        /// </param>
        /// <param name="Pre_selector">
        /// This control switches the preselection on or off when option B2 is installed.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_INP_PRES_STAT
        /// 
        /// Notes:
        /// 
        /// (1) If both B2 and B25 options are installed only one
        /// pre-amplifier may be switched on. In this case the setting of
        /// this control determines which of the Pre-amplifiers may
        /// be switched on:
        /// Pre-selector On - Pre-amplifier for the pre-selector B2 can be controlled.
        /// Pre-selector Off - Pre-amplifier for the electronic
        /// attenuator B25 can be controlled.
        /// </param>
        /// <param name="Pre_amplifier__B2_B25_">
        /// This control switches the pre-amplifier on and off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_INP_GAIN_STAT
        /// 
        /// Notes:
        /// 
        /// (1) If only the B2 or B25 option is present then the pre-amplifier of the installed option is controlled. If both the B2 and B25 options are present then the pre-amplifier controlled by this attribute depends on the setting for the RSSPECAN_ATTR_NOISE_INP_PRES_STAT attribute. If the RSSPECAN_ATTR_NOISE_INP_PRES_STAT is set to ON then the
        /// pre-amplifier for the B2 option is controlled by this function, otherwise the pre-amplifier of the B25 option is controlled.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureNoiseAnalyzerSettings(double Resolution_Bandwidth, double Sweep_Time, double Settling_Time, int Average, double RF_Attenuation, double Range, bool Automatic_Ref_Level, double Ref_Level, bool Pre_selector, bool Pre_amplifier__B2_B25_)
        {
            int pInvokeResult = PInvoke.ConfigureNoiseAnalyzerSettings(this._handle, Resolution_Bandwidth, Sweep_Time, Settling_Time, Average, RF_Attenuation, Range, System.Convert.ToUInt16(Automatic_Ref_Level), Ref_Level, System.Convert.ToUInt16(Pre_selector), System.Convert.ToUInt16(Pre_amplifier__B2_B25_));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Noise Generator Settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NOISE_CONF_GEN_CONT_STAT
        /// RSSPECAN_ATTR_NOISE_EXT_POW
        /// RSSPECAN_ATTR_NOISE_GEN_TYPE
        /// RSSPECAN_ATTR_NOISE_GPIB_GEN_ADDR
        /// RSSPECAN_ATTR_NOISE_CONF_GEN_INIT_AUTO
        /// 
        /// Remote-control command(s):
        /// SYSTem:CONFigure:GENerator:CONTrol:STATe[?] ON | OFF
        /// SOURce:EXTernal:POWer[:LEVel][?] &lt;numeric value&gt; [dB]
        /// SYSTem:COMMunicate:RDEVice:GENerator&lt;1|2&gt;:TYPE[?] &lt;name&gt;
        /// SYSTem:COMMunicate:GPIB:RDEVice:GENerator:ADDRess &lt;0 to 30&gt;
        /// SYSTem:CONFigure:GENerator:INITialise:AUTO[?] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Automatic_Control">
        /// Automatic Control defines whether R&amp;S FS-K30 controls the
        /// signal generator via GPIB commands (ON) or whether the
        /// signal generator is controlled by the user (OFF). The driver uses this value to set the RSSPECAN_ATTR_NOISE_CONF_GEN_CONT_STAT attribute. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_CONF_GEN_CONT_STAT
        /// 
        /// Notes:
        /// 
        /// (1) When Automatic Control is set to OFF, no GPIB commands will be issued to the signal generator. R&amp;S FS-K30 will assume that the user sets up the signal generator correctly. None of the other parameters in the Generator Settings group will be available for editing when Automatic Control is set to OFF.
        /// 
        /// (2) The Manual setting only really makes sense when the Measurement Mode is one of the Fixed LO cases in which the signal generator settings do not need to change during the measurement. In the Fixed IF Measurement Modes, meaningful results will only be possible if Automatic Control is set to ON, since R&amp;S FS-K30 will expect the LO frequency to be automatically tuned to the test RF frequency.
        /// 
        /// (3) When Automatic Control is set to ON, R&amp;S FS-K30 will issue GPIB commands during measurement execution in order to control the signal generator.
        /// 
        /// (4) Note that option B10 must be installed on the spectrum analyzer for GPIB control of an external signal generator to be possible.
        /// </param>
        /// <param name="Level">
        /// Level is the desired output power level of the external signal generator.
        /// 
        /// Units: dB
        /// 
        /// Default Value: 5.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_EXT_POW
        /// 
        /// Notes:
        /// 
        /// (1) This attribute is only available when Automatic Control is set to ON. This attribute is not available when the B10
        /// option is not installed
        /// </param>
        /// <param name="Generator_Select">
        /// Selects the type of external signal generator.
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_GEN_TYPE
        /// 
        /// Notes:
        /// 
        /// (1) The following list shows the available generator types:
        /// 
        /// SME02, SME03, SME06, SMG, SMGL, SMGU, SMH, SMHU, SMIQ02B, SMIQ02E, SMIQ03B, SMIQ03E, SMIQ04B, SMIQ06B, SML01, SMR20, SMR20B11, SMR27, SMR27B11, SMR30, SMR30B11, SMR40, SMR40B11, SMP02, SMP03, SMP04, SMP22, SMT02, SMT03, SMT06, SMX, SMY01, SMY02, HP8340A, HP ESG-A Series 1000A, HP ESG-A Series 2000A, HP ESG-A Series 3000A, HP ESG-A Series 4000A, HP ESG-D SERIES E4432B
        /// 
        /// (2) The list of signal generators is defined within the spectrum analyzer software - refer to the spectrum analyzer and B10 option user manuals for details on how to edit this list and add new signal generators.
        /// 
        /// (3) This attribute is only available when Automatic Control is
        /// set to ON. This attribute is not available when the B10 option is not installed.
        /// </param>
        /// <param name="GPIB_Address">
        /// Set the address used to control the external signal generator via GPIB.
        /// 
        /// Valid Values:
        /// 0 to 30
        /// 
        /// Default Value: 28
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_GPIB_GEN_ADDR
        /// 
        /// Notes:
        /// 
        /// (1) This attribute is only available when Automatic Control is
        /// set to ON. This attribute is not available when the B10 option is not installed.
        /// </param>
        /// <param name="Init_Before_Measurement">
        /// This control is used to specify whether the external signal
        /// generator is initialised before each measurement or not.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_CONF_GEN_INIT_AUTO
        /// 
        /// Notes:
        /// 
        /// (1) When Init Before Meas is set to On, an initialisation sequence of GPIB commands will be sent to the signal generator before each measurement. This ensures that the signal generator will be in the correct state to receive control commands during measurement execution. However, this will add a time overhead to the overall measurement completion.
        /// 
        /// (2) When Init Before Meas is set to Off, no initialisation sequence of GPIB commands will be sent to the signal generator when a measurement is run.
        /// 
        /// (3) The INIT GEN softkey in the main measurement results
        /// view and measurements settings view can be pressed to get
        /// R&amp;S FS-K30 to send the initialisation sequence of GPIB
        /// commands to the signal generator at any time.
        /// 
        /// (4) This attribute is only available when Automatic Control is
        /// set to ON. This attribute is not displayed when the B10
        /// option is not installed.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureNoiseGeneratorSettings(bool Automatic_Control, double Level, string Generator_Select, int GPIB_Address, bool Init_Before_Measurement)
        {
            int pInvokeResult = PInvoke.ConfigureNoiseGeneratorSettings(this._handle, System.Convert.ToUInt16(Automatic_Control), Level, Generator_Select, GPIB_Address, System.Convert.ToUInt16(Init_Before_Measurement));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Noise Generator Frequency.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NOISE_EXT_FREQ_OFFS
        /// RSSPECAN_ATTR_NOISE_EXT_FREQ_NUM
        /// RSSPECAN_ATTR_NOISE_EXT_FREQ_DEN
        /// 
        /// Remote-control command(s):
        /// SOURce:EXTernal:FREQuency:OFFSet&lt;1|2&gt;[?] &lt;numeric value&gt; [Hz]
        /// SOURce:EXTernal:FREQuency[:FACTor]:NUMerator[?] &lt;numeric value&gt;
        /// SOURce:EXTernal:FREQuency[:FACTor]:DENominator[?] &lt;numeric value&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Offset_Nr__1">
        /// Defines the frequency offset number 1 of the selected generator with reference to the receive frequency.
        /// 
        /// Unit: Hertz
        /// 
        /// Valid Values:
        /// None
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_EXT_FREQ_OFFS
        /// 
        /// Notes:
        /// 
        /// (1) The following formula is used to calculate the generator frequency (LO is Local Oscillator frequency):
        /// Generator Frequency = [(LO + Offset 1) * Numerator / Denominator)] + Offset 2
        /// 
        /// (2) This function is only valid in combination with option Ext. Generator Control R&amp;S FSP-B10.
        /// </param>
        /// <param name="Offset_Nr__2">
        /// Defines the frequency offset number 2 of the selected generator with reference to the receive frequency. 
        /// 
        /// Unit: Hertz
        /// 
        /// Valid Values:
        /// None
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_EXT_FREQ_OFFS
        /// 
        /// Notes:
        /// 
        /// (1) The following formula is used to calculate the generator frequency (LO is Local Oscillator frequency):
        /// Generator Frequency = [(LO + Offset 1) * Numerator / Denominator)] + Offset 2
        /// 
        /// (2) This function is only valid in combination with option Ext. Generator Control R&amp;S FSP-B10.
        /// </param>
        /// <param name="Numerator">
        /// Defines the numerator of the factor with which the analyzer frequency is multiplied in order to obtain the transmit frequency of the selected generator.
        /// 
        /// Valid Values:
        /// None
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_EXT_FREQ_NUM
        /// 
        /// Notes:
        /// 
        /// (1) The following formula is used to calculate the generator frequency (LO is Local Oscillator frequency):
        /// Generator Frequency = [(LO + Offset 1) * Numerator / Denominator)] + Offset 2
        /// 
        /// (2) This function is only valid in combination with option Ext. Generator Control R&amp;S FSP-B10.
        /// </param>
        /// <param name="Denominator">
        /// Defines the denominator of the factor with which the analyzer frequency is multiplied in order to obtain the transmit frequency of the selected generator. 
        /// 
        /// Valid Values:
        /// None
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_EXT_FREQ_DEN
        /// 
        /// Notes:
        /// 
        /// (1) The following formula is used to calculate the generator frequency (LO is Local Oscillator frequency):
        /// Generator Frequency = [(LO + Offset 1) * Numerator / Denominator)] + Offset 2
        /// 
        /// (2) This function is only valid in combination with option Ext. Generator Control R&amp;S FSP-B10.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureNoiseGeneratorFrequency(double Offset_Nr__1, double Offset_Nr__2, int Numerator, int Denominator)
        {
            int pInvokeResult = PInvoke.ConfigureNoiseGeneratorFrequency(this._handle, Offset_Nr__1, Offset_Nr__2, Numerator, Denominator);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Noise ENR Settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NOISE_CORR_ENR_MODE
        /// RSSPECAN_ATTR_NOISE_CORR_ENR_SPOT
        /// RSSPECAN_ATTR_NOISE_CORR_TEMP
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CORRection:ENR:MODE[?] TABLe | SPOT
        /// [SENSe:]CORRection:ENR:SPOT[?] &lt;numeric_value&gt; [dB]
        /// [SENSe:]CORRection:TEMPerature[?] &lt;numeric_value&gt; [K]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Selection_Mode">
        /// Specify whether a single ENR value applies for all measured frequencies or whether an ENR list table is to be used (ENR values specified at specific input frequencies).
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_NOISE_ERN_MODE_SEL_SPOT  (0) - Constant
        /// RSSPECAN_VAL_NOISE_ERN_MODE_SEL_TABLE (1) - Table
        /// 
        /// Default Value:
        /// Constant
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_CORR_ENR_MODE
        /// </param>
        /// <param name="ENR_Constant">
        /// Specifies the ENR value applicable to the Noise source for all input frequencies. This value provides a simple mechanism to enter an ENR value that is applicable throughout a range of measurement frequencies.
        /// 
        /// Units: dB
        /// 
        /// Valid Values:
        /// -999.99 to 999.99
        /// 
        /// Default Value: 15 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_CORR_ENR_SPOT
        /// 
        /// Notes:
        /// 
        /// (1) This attribute will however not be effective if
        /// attribute RSSPECAN_ATTR_NOISE_CORR_ENR_MODE - TABL was previously entered.
        /// </param>
        /// <param name="Room_Temperature">
        /// Specifies the room temperature of the operating environment. This value will be taken into account when calculating noise results.
        /// 
        /// Units: Kelvin
        /// 
        /// Valid Values:
        /// 10 to 1000
        /// 
        /// Default Value: 293 K
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_CORR_TEMP
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureNoiseENRSettings(int Selection_Mode, double ENR_Constant, double Room_Temperature)
        {
            int pInvokeResult = PInvoke.ConfigureNoiseENRSettings(this._handle, Selection_Mode, ENR_Constant, Room_Temperature);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Noise ENR Table.
        /// 
        /// Attribute(s):
        /// None
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CORRection:ENR[:MEASurement]:TABLe:DATA[?] &lt;numeric_value&gt; [Hz],&lt;numeric_value&gt; [dB],...
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Number_Of_Values">
        /// Specify the number of values supplied for the ENR table in the controls Receiver Frequency Values and ENR Values.
        /// 
        /// Valid Values:
        /// 1 to 100
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Receiver_Frequency_Values">
        /// Enter the receiver frequency values for the ENR table.
        /// 
        /// Units: Hertz
        /// 
        /// Valid Values:
        /// 0.0 to 999.99e+9
        /// 
        /// Default Value:
        /// None
        /// 
        /// Notes:
        /// 
        /// (1) The number of values for the Receiver Frequency and the corresponding ENR values have to be identical.
        /// </param>
        /// <param name="ENR_Values">
        /// Enter the ENR values for the ENR table.
        /// 
        /// Units: dB
        /// 
        /// Valid Values:
        /// -999.99 to 999.99
        /// 
        /// Default Value:
        /// None
        /// 
        /// Notes:
        /// 
        /// (1) The number of values for the Receiver Frequency and the corresponding ENR values have to be identical.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureNoiseENRTable(int Number_Of_Values, double[] Receiver_Frequency_Values, double[] ENR_Values)
        {
            int pInvokeResult = PInvoke.ConfigureNoiseENRTable(this._handle, Number_Of_Values, Receiver_Frequency_Values, ENR_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Noise Loss Input Settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NOISE_CORR_LOSS_MODE
        /// RSSPECAN_ATTR_NOISE_CORR_LOSS_SPOT
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CORRection:LOSS:INPut:MODE[?] SPOT | TABLe
        /// [SENSe:]CORRection:LOSS:INPut:SPOT[?] &lt;numeric value&gt; [dB]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Selection_Mode">
        /// Specify whether a single input loss value applies for all measured frequencies or whether an input loss list is to be used. (Loss Input values specified at specific input frequencies).
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_NOISE_LOSS_MODE_SPOT  (0) - Constant
        /// RSSPECAN_VAL_NOISE_LOSS_MODE_TABLE (1) - Table
        /// 
        /// Default Value:
        /// Constant
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_CORR_LOSS_MODE
        /// </param>
        /// <param name="Loss_Input_Constant">
        /// Specifies the loss input constant to be set to a value applicable for all input frequencies to be measured. This attribute provides a simple mechanism to enter a loss input
        /// value that is applicable throughout a range of measurement frequencies.
        /// 
        /// Units: dB
        /// 
        /// Valid Values:
        /// -999.99 to 999.99
        /// 
        /// Default Value: 15 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_CORR_LOSS_SPOT
        /// 
        /// Notes:
        /// 
        /// (1) This attribute will however not be effective if
        /// attribute RSSPECAN_ATTR_NOISE_CORR_LOSS_MODE - TABL was previously entered.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureNoiseLossInputSettings(int Selection_Mode, double Loss_Input_Constant)
        {
            int pInvokeResult = PInvoke.ConfigureNoiseLossInputSettings(this._handle, Selection_Mode, Loss_Input_Constant);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Noise Loss Input Table.
        /// 
        /// Attribute(s):
        /// None
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CORRection:LOSS:INPut:TABLe[?] &lt;numeric value&gt; [Hz],&lt;numeric value&gt; [dB],...
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Number_Of_Values">
        /// Specify the number of values supplied for the LOSS Input table in the controls Receiver Frequency Values and Loss Input Values.
        /// 
        /// Valid Values:
        /// 1 to 100
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Receiver_Frequency_Values">
        /// Enter the receiver frequency values for the LOSS Input table.
        /// 
        /// Units: Hertz
        /// 
        /// Valid Values:
        /// 0.0 to 999.99e+9
        /// 
        /// Default Value:
        /// None
        /// 
        /// Notes:
        /// 
        /// (1) The number of values for the Receiver Frequency and the corresponding LOSS Input values have to be identical.
        /// 
        /// (2) This attribute will however not be effective if the attribute RSSPECAN_ATTR_NOISE_CORR_LOSS_INP_MODE - SPOT was previously entered.
        /// </param>
        /// <param name="Loss_Input_Values">
        /// Enter the Loss Input values for the LOSS Input table.
        /// 
        /// Units: dB
        /// 
        /// Valid Values:
        /// -999.99 to 999.99
        /// 
        /// Default Value:
        /// None
        /// 
        /// Notes:
        /// 
        /// (1) The number of values for the Receiver Frequency and the corresponding LOSS Input values have to be identical.
        /// 
        /// (2) This attribute will however not be effective if the attribute RSSPECAN_ATTR_NOISE_CORR_LOSS_INP_MODE - SPOT was previously entered.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureNoiseLossInputTable(int Number_Of_Values, double[] Receiver_Frequency_Values, double[] Loss_Input_Values)
        {
            int pInvokeResult = PInvoke.ConfigureNoiseLossInputTable(this._handle, Number_Of_Values, Receiver_Frequency_Values, Loss_Input_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Noise Loss Output Settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NOISE_CORR_LOSS_MODE
        /// RSSPECAN_ATTR_NOISE_CORR_LOSS_SPOT
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CORRection:LOSS:OUTPut:MODE[?] SPOT | TABLe
        /// [SENSe:]CORRection:LOSS:OUTPut:SPOT[?] &lt;numeric value&gt; [dB]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Selection_Mode">
        /// Specify whether a single output loss value applies for all measured frequencies or whether an output loss list is to be used. (Loss Output values specified at specific input frequencies).
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_NOISE_LOSS_MODE_SPOT  (0) - Constant
        /// RSSPECAN_VAL_NOISE_LOSS_MODE_TABLE (1) - Table
        /// 
        /// Default Value:
        /// Constant
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_CORR_LOSS_MODE
        /// </param>
        /// <param name="Loss_Output_Constant">
        /// Specifies the loss output constant to be set to a value applicable for all output frequencies to be measured. This attribute provides a simple mechanism to enter a loss output
        /// value that is applicable throughout a range of measurement frequencies.
        /// 
        /// Units: dB
        /// 
        /// Valid Values:
        /// -999.99 to 999.99
        /// 
        /// Default Value: 15 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_CORR_LOSS_SPOT
        /// 
        /// Notes:
        /// 
        /// (1) This attribute will however not be effective if
        /// attribute RSSPECAN_ATTR_NOISE_CORR_LOSS_MODE - TABL was previously entered.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureNoiseLossOutputSettings(int Selection_Mode, double Loss_Output_Constant)
        {
            int pInvokeResult = PInvoke.ConfigureNoiseLossOutputSettings(this._handle, Selection_Mode, Loss_Output_Constant);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Noise Loss Output Table.
        /// 
        /// Attribute(s):
        /// None
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CORRection:LOSS:OUTPut:TABLe[?] &lt;numeric value&gt; [Hz],&lt;numeric value&gt; [dB],...
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Number_Of_Values">
        /// Specify the number of values supplied for the LOSS Output table in the controls Receiver Frequency Values and Loss Output Values.
        /// 
        /// Valid Values:
        /// 1 to 100
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Receiver_Frequency_Values">
        /// Enter the receiver frequency values for the LOSS Output table.
        /// 
        /// Units: Hertz
        /// 
        /// Valid Values:
        /// 0.0 to 999.99e+9
        /// 
        /// Default Value:
        /// None
        /// 
        /// Notes:
        /// 
        /// (1) The number of values for the Receiver Frequency and the corresponding LOSS Output values have to be identical.
        /// 
        /// (2) This attribute will however not be effective if the attribute RSSPECAN_ATTR_NOISE_CORR_LOSS_OUT_MODE - SPOT was previously entered.
        /// </param>
        /// <param name="Loss_Output_Values">
        /// Enter the Loss Output values for the LOSS Output table.
        /// 
        /// Units: dB
        /// 
        /// Valid Values:
        /// -999.99 to 999.99
        /// 
        /// Default Value:
        /// None
        /// 
        /// Notes:
        /// 
        /// (1) The number of values for the Receiver Frequency and the corresponding LOSS Output values have to be identical.
        /// 
        /// (2) This attribute will however not be effective if the attribute RSSPECAN_ATTR_NOISE_CORR_LOSS_OUT_MODE - SPOT was previously entered.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureNoiseLossOutputTable(int Number_Of_Values, double[] Receiver_Frequency_Values, double[] Loss_Output_Values)
        {
            int pInvokeResult = PInvoke.ConfigureNoiseLossOutputTable(this._handle, Number_Of_Values, Receiver_Frequency_Values, Loss_Output_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Noise Display Result Settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NOISE_DISP_FORM
        /// RSSPECAN_ATTR_NOISE_DISP_TABLE
        /// 
        /// Remote-control command(s):
        /// DISPlay:FORMat[?] SINGle | SPLit
        /// DISPlay[:WINDow&lt;1&gt;]:TABLe[?] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Combined_Trace_Display">
        /// Select either combined or splitted trace to be displayed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DISP_FORMAT_SINGLE  (0) - Single
        /// RSSPECAN_VAL_DISP_FORMAT_SPLIT   (1) - Split
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_DISP_FORMAT_SINGLE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_FORMAT
        /// </param>
        /// <param name="Table_Form_Results">
        /// Toggle the display of results in graphical or tabular form. Selecting Off will place the results in a graphical form, On in a tabular form.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_DISP_TABLE
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureNoiseResultSettings(int Combined_Trace_Display, bool Table_Form_Results)
        {
            int pInvokeResult = PInvoke.ConfigureNoiseResultSettings(this._handle, Combined_Trace_Display, System.Convert.ToUInt16(Table_Form_Results));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Noise Trace Settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NOISE_DISP_DATA_TRAC
        /// RSSPECAN_ATTR_NOISE_DISP_TRAC_STAT
        /// RSSPECAN_ATTR_NOISE_DISP_SYMB
        /// RSSPECAN_ATTR_NOISE_DISP_AUTO_SCAL
        /// RSSPECAN_ATTR_NOISE_DISP_TRAC_BOTTOM
        /// RSSPECAN_ATTR_NOISE_DISP_TRAC_TOP
        /// 
        /// Remote-control command(s):
        /// DISPlay:DATA:TRACe&lt;1&gt;[?] NFIGure | TEFFective
        /// DISPlay[:WINDow&lt;1&gt;]:TRACe&lt;1|2&gt;[:STATe][?] ON | OFF
        /// DISPlay[:WINDow&lt;1&gt;]:TRACe&lt;1|2&gt;:SYMBols[?] ON | OFF
        /// DISPlay[:WINDow&lt;1&gt;]:TRACe&lt;1|2&gt;:Y[:SCALe]:AUTO[?] ON | OFF
        /// DISPlay[:WINDow&lt;1&gt;]:TRACe&lt;1|2&gt;:Y[:SCALe]:BOTTom[?] &lt;numeric_value&gt; [dB]
        /// DISPlay[:WINDow&lt;1&gt;]:TRACe&lt;1|2&gt;:Y[:SCALe]:TOP[?] &lt;numeric_value&gt; [dB]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Y_Axis">
        /// Specify the type of data to be displayed in trace 1. Select either to display either Noise Figure results (NFIGure) or Noise temperature (TEFFective - to keep compatibility with the NFA).
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_NOISE_DISP_DATA_OFF     (0) - Off 
        /// RSSPECAN_VAL_NOISE_DISP_DATA_TEFF    (1) - Noise Temperature
        /// RSSPECAN_VAL_NOISE_DISP_DATA_NFIGURE (2) - Noise Figure
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_NOISE_DISP_DATA_NFIGURE (2)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_DISP_DATA_TRAC
        /// 
        /// Notes:
        /// (1) Trace 1 is always noise results and Trace 2 is always for Gain results.
        /// </param>
        /// <param name="Symbols">
        /// Switches on or off the display of the symbols to mark the measurement points for the specified trace i.e. Noise or Gain. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_DISP_SYMB
        /// 
        /// Notes:
        /// 
        /// (1) This attribute will only have a noticeable effect if the required trace is active as set by the attribute RSSPECAN_ATTR_NOISE_DISP_TRAC_STAT ON.
        /// </param>
        /// <param name="Automatic_Scaling">
        /// Switches on or off automatic scaling of the Y-axis for the specified trace display. Automatic scaling set the Y-axis to automatically scale to best fit the measurement results.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_DISP_AUTO_SCAL
        /// 
        /// Notes:
        /// 
        /// (1) This attribute will only have a noticeable effect if the required trace is active as set by the attribute RSSPECAN_ATTR_NOISE_DISP_TRAC_STAT ON.
        /// </param>
        /// <param name="Min_Y_Axis">
        /// Sets the minimum (bottom) Y-axis display value for the specified trace display.
        /// 
        /// Units: dB or Kelvin
        /// 
        /// Valid Values:
        /// -75.0 to 75 dB
        /// -999990000 to 999990000 K
        /// 
        /// Default Value:
        /// 0.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_DISP_TRAC_BOTTOM
        /// 
        /// Notes:
        /// 
        /// (1) This attribute has no affect if automatic scaling of the Y-axis is enabled or the specified trace is not currently active.
        /// 
        /// (2) When the attribute RSSPECAN_ATTR_NOISE_DISP_DATA_TRAC is set to Noise Figure, then the units of this attribute are dB. When the attribute RSSPECAN_ATTR_NOISE_DISP_DATA_TRAC is set to Noise Temperature, then the units of the this attribute are Kelvins.
        /// </param>
        /// <param name="Max_Y_Axis">
        /// Sets the maximum (top) Y-axis display value for the specified trace display.
        /// 
        /// Units: dB or Kelvin
        /// 
        /// Valid Values:
        /// -75.0 to 75 dB
        /// -999990000 to 999990000 K
        /// 
        /// Default Value:
        /// 20.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_DISP_TRAC_TOP
        /// 
        /// Notes:
        /// 
        /// (1) This attribute has no affect if automatic scaling of the Y-axis is enabled or the specified trace is not currently active.
        /// 
        /// (2) When the attribute RSSPECAN_ATTR_NOISE_DISP_DATA_TRAC is set to Noise Figure, then the units of this attribute are dB. When the attribute RSSPECAN_ATTR_NOISE_DISP_DATA_TRAC is set to Noise Temperature, then the units of the this attribute are Kelvins.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureNoiseTraceSettings(int Y_Axis, bool Symbols, bool Automatic_Scaling, double Min_Y_Axis, double Max_Y_Axis)
        {
            int pInvokeResult = PInvoke.ConfigureNoiseTraceSettings(this._handle, Y_Axis, System.Convert.ToUInt16(Symbols), System.Convert.ToUInt16(Automatic_Scaling), Min_Y_Axis, Max_Y_Axis);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Gain Trace Settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NOISE_DISP_TRAC_STAT
        /// RSSPECAN_ATTR_NOISE_DISP_SYMB
        /// RSSPECAN_ATTR_NOISE_DISP_AUTO_SCAL
        /// RSSPECAN_ATTR_NOISE_DISP_TRAC_BOTTOM
        /// RSSPECAN_ATTR_NOISE_DISP_TRAC_TOP
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1&gt;]:TRACe&lt;1|2&gt;[:STATe][?] ON | OFF
        /// DISPlay[:WINDow&lt;1&gt;]:TRACe&lt;1|2&gt;:SYMBols[?] ON | OFF
        /// DISPlay[:WINDow&lt;1&gt;]:TRACe&lt;1|2&gt;:Y[:SCALe]:AUTO[?] ON | OFF
        /// DISPlay[:WINDow&lt;1&gt;]:TRACe&lt;1|2&gt;:Y[:SCALe]:BOTTom[?] &lt;numeric_value&gt; [dB]
        /// DISPlay[:WINDow&lt;1&gt;]:TRACe&lt;1|2&gt;:Y[:SCALe]:TOP[?] &lt;numeric_value&gt; [dB]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Y_Axis">
        /// Switches on or off the display of the corresponding trace and related information in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_DISP_TRAC_STAT
        /// 
        /// </param>
        /// <param name="Symbols">
        /// Switches on or off the display of the symbols to mark the measurement points for the specified trace i.e. Noise or Gain. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_DISP_SYMB
        /// 
        /// Notes:
        /// 
        /// (1) This attribute will only have a noticeable effect if the required trace is active as set by the attribute RSSPECAN_ATTR_NOISE_DISP_TRAC_STAT ON.
        /// </param>
        /// <param name="Automatic_Scaling">
        /// Switches on or off automatic scaling of the Y-axis for the specified trace display. Automatic scaling set the Y-axis to automatically scale to best fit the measurement results.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_DISP_AUTO_SCAL
        /// 
        /// Notes:
        /// 
        /// (1) This attribute will only have a noticeable effect if the required trace is active as set by the attribute RSSPECAN_ATTR_NOISE_DISP_TRAC_STAT ON.
        /// </param>
        /// <param name="Min_Y_Axis">
        /// Sets the minimum (bottom) Y-axis display value for the specified trace display.
        /// 
        /// Units: dB
        /// 
        /// Valid Values:
        /// -75.0 to 75 dB
        /// 
        /// Default Value:
        /// 0.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_DISP_TRAC_BOTTOM
        /// 
        /// Notes:
        /// 
        /// (1) This attribute has no affect if automatic scaling of the Y-axis is enabled or the specified trace is not currently active.
        /// </param>
        /// <param name="Max_Y_Axis">
        /// Sets the maximum (top) Y-axis display value for the specified trace display.
        /// 
        /// Units: dB
        /// 
        /// Valid Values:
        /// -75.0 to 75 dB
        /// 
        /// Default Value:
        /// 20.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_DISP_TRAC_TOP
        /// 
        /// Notes:
        /// 
        /// (1) This attribute has no affect if automatic scaling of the Y-axis is enabled or the specified trace is not currently active.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureNoiseGainTraceSettings(bool Y_Axis, bool Symbols, bool Automatic_Scaling, double Min_Y_Axis, double Max_Y_Axis)
        {
            int pInvokeResult = PInvoke.ConfigureNoiseGainTraceSettings(this._handle, System.Convert.ToUInt16(Y_Axis), System.Convert.ToUInt16(Symbols), System.Convert.ToUInt16(Automatic_Scaling), Min_Y_Axis, Max_Y_Axis);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the display of trace memory &lt;n&gt; on or off.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only in conjunction with R&amp;S FSL with option FS-K30.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NOISE_DISP_TRACE_MEM
        /// 
        /// Remote-control command(s):
        /// DISPlay:ARRay:MEMory&lt;1...3&gt;[:STATe]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Memory">
        /// Selects the trace memory
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// Toggles the display of traces memory state.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_DISP_TRACE_MEM
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureNoiseTraceMemoryDisplay(int Memory, bool State)
        {
            int pInvokeResult = PInvoke.ConfigureNoiseTraceMemoryDisplay(this._handle, Memory, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the display of the current measurement results on or off.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only in conjunction with R&amp;S FSL with option FS-K30.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NOISE_DISPL_CURR_RESULTS
        /// 
        /// Remote-control command(s):
        /// DISPlay:CURRent:DATA[:STATe]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="State">
        /// Toggles the display of current measuremnt results state.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_DISPL_CURR_RESULTS
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureNoiseCurrentResultsDisplay(bool State)
        {
            int pInvokeResult = PInvoke.ConfigureNoiseCurrentResultsDisplay(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the frequency to be displayed on the x-axis if the DUT is not an amplifier, i.e. in a frequency-converting measurement mode.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only in conjunction with option FS-K30.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NOISE_DISP_X_AXIS
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1&gt;]:TRACe&lt;1|2&gt;:X[:SCALe]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement_Results">
        /// Indicates the measurement results (trace).
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_NOISE_NOISE (1) - Noise 
        /// RSSPECAN_VAL_NOISE_GAIN  (2) - Gain
        /// 
        /// Default Value: RSSPECAN_VAL_NOISE_NOISE (1)
        /// 
        /// </param>
        /// <param name="Frequency">
        /// Selects the frequency to be displayed on the x-axis.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_NOISE_XAXIS_IF (0) - IF
        /// RSSPECAN_VAL_NOISE_XAXIS_RF (1) - RF 
        /// 
        /// Default Value: RSSPECAN_VAL_NOISE_XAXIS_RF
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_DISP_X_AXIS
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureXAxisFrequencyDisplay(int Measurement_Results, int Frequency)
        {
            int pInvokeResult = PInvoke.ConfigureXAxisFrequencyDisplay(this._handle, Measurement_Results, Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function creates new limit or selects existing one and configures basic parameters.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_NAME
        /// RSSPECAN_LIMIT_COMMENT
        /// 
        /// Remote-control command(s):
        /// CALCulate:LIMit&lt;1 to 6&gt;:NAME
        /// CALCulate:LIMit&lt;1 to 6&gt;:COMMent
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 6
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Name">
        /// This control sets the limit line name.
        /// 
        /// Valid Values:
        /// up to 8 character string
        /// 
        /// Default Value: "limit"
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_NAME
        /// </param>
        /// <param name="Comment">
        /// This control sets the limit line comment.
        /// 
        /// Valid Values:
        /// up to 40 character string
        /// 
        /// Default Value: "Comment"
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_COMMENT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CreateNoiseLimitLine(int Limit, string Name, string Comment)
        {
            int pInvokeResult = PInvoke.CreateNoiseLimitLine(this._handle, Limit, Name, Comment);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines specified limit line.
        /// 
        /// Attributes:
        /// RSSPECAN_ATTR_NOISE_CALC_LIM_TRAC
        /// 
        /// Remote-control command(s):
        /// CALCulate:LIMit&lt;1 to 6&gt;:TRACe NFIGure | TEFFective | GAIN
        /// CALCulate:LIMit&lt;1 to 6&gt;:CONTrol[:DATA]
        /// CALCulate:LIMit&lt;1 to 6&gt;:LOWer[:DATA]
        /// CALCulate:LIMit&lt;1 to 6&gt;:UPPer[:DATA]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 6
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Result_Type">
        /// Select the result type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_NOISE_CALC_LIM_LINE_NFIGURE    (0) - Noise Figure
        /// RSSPECAN_VAL_NOISE_CALC_LIM_LINE_TEFFECTIVE (1) - Noise Temperature
        /// RSSPECAN_VAL_NOISE_CALC_LIM_LINE_GAIN       (2) - Gain
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_NOISE_CALC_LIM_LINE_NFIGURE
        /// </param>
        /// <param name="Type">
        /// Specifies a limit line to be either an upper or lower type limit line. An upper line will be used as the maximum allowable value when comparing with the data. A lower limit line defines the minimum value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIMIT_LOWER (0) - Lower
        /// RSSPECAN_VAL_LIMIT_UPPER (1) - Upper
        /// 
        /// Default Value: RSSPECAN_VAL_LIMIT_LOWER (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_CALC_LIM_TRAC
        /// 
        /// Note(s):
        /// 
        /// (1) Lower - lower limit fails if the trace falls below the limit.
        /// 
        /// (2) Upper - upper limit fails if the trace exceeds the limit.
        /// 
        /// </param>
        /// <param name="Count">
        /// Specifies number of values in X Axis and Amplitude arrays.
        /// 
        /// Valid Values:
        /// &gt;= 1
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="X_Axis">
        /// Specifies the frequency or time value for a limit point. The units are depended on the attribute RSSPECAN_ATTR_LLINE_X_AXIS_UNITS.
        /// 
        /// You must pass a ViReal64 array with at least count values.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Amplitude">
        /// Specifies the amplitude value for the current limit point.
        /// 
        /// You must pass a ViReal64 array with at least count values.
        /// 
        /// Units: Are depended on the RSSPECAN_LIMIT_UPPER_SCALING or RSSPECAN_LIMIT_LOWER_SCALING attribute.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int DefineNoiseLimitLine(int Limit, int Result_Type, int Type, int Count, double[] X_Axis, double[] Amplitude)
        {
            int pInvokeResult = PInvoke.DefineNoiseLimitLine(this._handle, Limit, Result_Type, Type, Count, X_Axis, Amplitude);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the limit check and/or limit line for the selected limit line in the selected measurement window.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_STATE
        /// RSSPECAN_LIMIT_LOWER_STATE
        /// RSSPECAN_LIMIT_UPPER_STATE
        /// 
        /// Remote-control command(s):
        /// CALCulate:LIMit&lt;1 to 6&gt;:STATe ON | OFF
        /// CALCulate:LIMit&lt;1 to 6&gt;:LOWer:STATe ON | OFF
        /// CALCulate:LIMit&lt;1 to 6&gt;:UPPer:STATe ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 6
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Type">
        /// Specifies a limit line to be either an upper or lower type limit line. An upper line will be used as the maximum allowable value when comparing with the data. A lower limit line defines the minimum value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIMIT_LOWER (0) - Lower
        /// RSSPECAN_VAL_LIMIT_UPPER (1) - Upper
        /// 
        /// Default Value: RSSPECAN_VAL_LIMIT_LOWER (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Lower - lower limit fails if the trace falls below the limit.
        /// 
        /// (2) Upper - upper limit fails if the trace exceeds the limit.
        /// 
        /// </param>
        /// <param name="Limit_Enabled">
        /// Turns the limit line display on and off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_STATE
        /// </param>
        /// <param name="Check_Enabled">
        /// Turns limit line testing on/off. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_LOWER_STATE
        /// RSSPECAN_LIMIT_UPPER_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int EnableNoiseLimitCheck(int Limit, int Type, bool Limit_Enabled, bool Check_Enabled)
        {
            int pInvokeResult = PInvoke.EnableNoiseLimitCheck(this._handle, Limit, Type, System.Convert.ToUInt16(Limit_Enabled), System.Convert.ToUInt16(Check_Enabled));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function moves specified limit line.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_CONTROL_SHIFT
        /// RSSPECAN_LIMIT_LOWER_SHIFT
        /// RSSPECAN_LIMIT_UPPER_SHIFT
        /// 
        /// Remote-control command(s):
        /// CALCulate:LIMit&lt;1 to 6&gt;:CONTrol:SHIFt
        /// CALCulate:LIMit&lt;1 to 6&gt;:LOWer:SHIFt
        /// CALCulate:LIMit&lt;1 to 6&gt;:UPPer:SHIFt
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 6
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Type">
        /// Specifies a limit line to be type to be moved.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIMIT_LOWER   (0) - Lower
        /// RSSPECAN_VAL_LIMIT_UPPER   (1) - Upper
        /// RSSPECAN_VAL_LIMIT_CONTROL (2) - X-Axis
        /// 
        /// Default Value: RSSPECAN_VAL_LIMIT_LOWER (0)
        /// </param>
        /// <param name="Value">
        /// Defines the move value.
        /// 
        /// Valid Values:
        /// -200.0 to 200.0 dB for Lower and Upper
        /// -300.0e9 to 300.0e9 Hz for X-Axis
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_CONTROL_SHIFT
        /// RSSPECAN_LIMIT_LOWER_SHIFT
        /// RSSPECAN_LIMIT_UPPER_SHIFT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int MoveNoiseLimitLine(int Limit, int Type, double Value)
        {
            int pInvokeResult = PInvoke.MoveNoiseLimitLine(this._handle, Limit, Type, Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function copies specified limit line into another one.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_COPY
        /// 
        /// Remote-control command(s):
        /// CALCulate:LIMit&lt;1 to 6&gt;:COPY
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 6
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Copy_To">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 6
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_COPY
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CopyNoiseLimitLine(int Limit, int Copy_To)
        {
            int pInvokeResult = PInvoke.CopyNoiseLimitLine(this._handle, Limit, Copy_To);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function deletes the result of the current limit check.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_CLEAR
        /// 
        /// Remote-control command(s):
        /// CALCulate:LIMit&lt;1 to 6&gt;:CLEar[:IMMediate]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 6
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ClearNoiseLimitLineResults(int Limit)
        {
            int pInvokeResult = PInvoke.ClearNoiseLimitLineResults(this._handle, Limit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function removes specified limit line.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_DELETE
        /// 
        /// Remote-control command(s):
        /// CALCulate:LIMit&lt;1 to 6&gt;:DELete
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 6
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int DeleteNoiseLimitLine(int Limit)
        {
            int pInvokeResult = PInvoke.DeleteNoiseLimitLine(this._handle, Limit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function saves the current trace results (noise figure, noise temperature and noise gain traces) to trace memory &lt;n&gt;.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only with R&amp;S FSL or R&amp;S ESL (FW 1.60) with option FS-K30.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NOISE_ARRAY_MEMORY
        /// 
        /// Remote-control command(s):
        /// CONFigure:ARRay:MEMory&lt;1...3&gt; ONCE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Memory">
        /// Selects the trace memory.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SaveNoiseTraceDataToMemory(int Memory)
        {
            int pInvokeResult = PInvoke.SaveNoiseTraceDataToMemory(this._handle, Memory);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the noise measurement results.
        /// 
        /// Attribute(s):
        /// None
        /// 
        /// Remote-control command(s):
        /// FETCh:ARRay:NOISe:FIGure|TEMPerature|GAIN?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Modifier">
        /// Sets the modifier of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_NOISE_FETC_FIGURE      (0) - Figure
        /// RSSPECAN_VAL_NOISE_FETC_TEMPERATURE (1) - Temperature
        /// RSSPECAN_VAL_NOISE_FETC_GAIN        (2) - Gain
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// This parameter determines the number of item in array list.
        /// 
        /// Valid Values:
        /// 1 to 100
        /// 
        /// Default Value:
        /// 100
        /// </param>
        /// <param name="Values">
        /// Returns the values. 
        /// 
        /// Note(s):
        /// 
        /// (1) There can be up to 100 values returned.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchNoiseArray(int Modifier, int Array_Size, double[] Values)
        {
            int pInvokeResult = PInvoke.FetchNoiseArray(this._handle, Modifier, Array_Size, Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the noise measurement results.
        /// 
        /// Attribute(s):
        /// None
        /// 
        /// Remote-control command(s):
        /// FETCh:SCALar:NOISe:FIGure|TEMPerature|GAIN?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Modifier">
        /// Sets the modifier of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_NOISE_FETC_FIGURE      (0) - Figure
        /// RSSPECAN_VAL_NOISE_FETC_TEMPERATURE (1) - Temperature
        /// RSSPECAN_VAL_NOISE_FETC_GAIN        (2) - Gain
        /// 
        /// </param>
        /// <param name="Value">
        /// Returns the value. 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchNoiseScalar(int Modifier, out double Value)
        {
            int pInvokeResult = PInvoke.FetchNoiseScalar(this._handle, Modifier, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the measurement results of the selected memory.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only in conjunction with R&amp;S FSL with option FS-K30.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FETCh:ARRay:MEMory&lt;1...3&gt;:NOISe:FIGure?
        /// FETCh:ARRay:MEMory&lt;1...3&gt;:NOISe:GAIN?
        /// FETCh:ARRay:MEMory&lt;1...3&gt;:NOISe:TEMPerature?
        /// *CLS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Modifier">
        /// Sets the modifier of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_NOISE_FETC_FIGURE      (0) - Figure
        /// RSSPECAN_VAL_NOISE_FETC_TEMPERATURE (1) - Temperature
        /// RSSPECAN_VAL_NOISE_FETC_GAIN        (2) - Gain
        /// 
        /// </param>
        /// <param name="Memory">
        /// Selects the trace memory
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Array_Size">
        /// This control sets the array size. 
        /// 
        /// Valid Values:
        /// 1 to 100
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Meas_Result">
        /// Returns the last recorded noise measurement results. The results will be returned as an array of up to 100 elements.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchNoiseMemoryArray(int Modifier, int Memory, int Array_Size, double[] Meas_Result)
        {
            int pInvokeResult = PInvoke.FetchNoiseMemoryArray(this._handle, Modifier, Memory, Array_Size, Meas_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the result of the limit check of the limit line indicated in the selected measurement window. It should be noted that a complete sweep must have been performed for obtaining a valid result. A synchronization with  *OPC, *OPC? or *WAI should therefore be provided.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_CHECK_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate:LIMit&lt;1 to 6&gt;:FAIL?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 6
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// The result of the limit check responds with 0 for PASS, 1 for FAIL, and 2 for MARGIN.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetNoiseLimitCheckResult(int Limit, out int State)
        {
            int pInvokeResult = PInvoke.GetNoiseLimitCheckResult(this._handle, Limit, out State);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the Phase Noise mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PHASE_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument[:SELect] PNOise
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int PhaseMode()
        {
            int pInvokeResult = PInvoke.PhaseMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase Signal Settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FREQUENCY_CENTER
        /// RSSPECAN_ATTR_PHASE_CONF_POW_EXP_RF
        /// RSSPECAN_ATTR_PHASE_SWE_FORW
        /// RSSPECAN_ATTR_PHASE_FREQ_VER
        /// RSSPECAN_ATTR_PHASE_FREQ_VER_TOL
        /// RSSPECAN_ATTR_PHASE_POW_RLEV_VER_TOL
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer
        /// CONFigure:POWer:EXPected:RF
        /// [SENSe:]SWEep:FORWard ON | OFF
        /// [SENSe:]FREQuency:VERify[:STATe] ON | OFF
        /// [SENSe:]FREQuency:VERify:TOLerance
        /// [SENSe:]POWer:RLEVel:VERify:TOLerance
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Frequency">
        /// Specifies the Center Frequency of the signal to be measured.
        /// 
        /// Units: Hz
        /// 
        /// Valid Values:
        /// 0.0 to 7.0e+9
        /// 
        /// Default Value:  20.0 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_CENTER
        /// </param>
        /// <param name="Level">
        /// Specifies the expected level of the RF input signal.
        /// 
        /// Units: dBm
        /// 
        /// Valid Values:
        /// -100.0 to 30.00
        /// 
        /// Default Value: -20.0 dBm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_CONF_POW_EXP_RF
        /// </param>
        /// <param name="Sweep_Forward">
        /// Selects the sweep direction for the current measurement.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_SWE_FORW
        /// 
        /// Notes:
        /// 
        /// (1) If this attribute is set to ON then the tolerance that is set with the attribute RSSPECAN_ATTR_PHASE_FREQ_VER_TOL is specified in Hz. 
        /// 
        /// (2) If this attribute is set to OFF then the tolerance that is set with the attribute RSSPECAN_ATTR_PHASE_FREQ_VER_TOL is specified in %.
        /// </param>
        /// <param name="Verify_Frequency___Level">
        /// Enables a search across a frequency tolerance range, for the carrier of greatest magnitude. Carrier frequency and level are measured. If the level is within a level tolerance range, the measured level overrides the specified Level. Otherwise the measurement is aborted. 
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_FREQ_VER
        /// 
        /// Notes:
        /// 
        /// (1) This should be used when the carrier frequency is not known
        /// precisely.
        /// 
        /// (2) Both attributes use the same value for enabling or disabling.
        /// 
        /// (3) When this control is set to OFF, Frequency Tolerance and Level Tolerance values are ignored.
        /// </param>
        /// <param name="Frequency_Tolerance">
        /// Used to verify the input signal frequency.
        /// 
        /// Units: 
        /// Hz - sweep direction is forward
        /// %  - sweep direction is reverse
        /// 
        /// Valid Values:
        /// 1.0 to 1.0e+6 Hz (sweep direction is forward)
        /// 1 to 100 %       (sweep direction is reverse)
        /// 
        /// Default Value: 1.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_FREQ_VER_TOL
        /// 
        /// Notes:
        /// 
        /// (1) If the sweep direction is forward (control Sweep Forward set to ON), the Frequency Tolerance (absolute) parameter is used, which is the range, either side of the Signal Frequency, within which the carrier is known to be. In forward sweep there will be a frequency and level check once at the beginning of the measurement.
        /// 
        /// (2) If the sweep direction is reverse (control Sweep Forward set to OFF), the Frequency Tolerance (relative) parameter is used, which is the ratio of the subspan?s start frequency. In reverse sweep there will be a frequency and level check before each subsweep.
        /// 
        /// (3) The units are Hz if the sweep direction is forward (control Sweep Forward set to ON) and % if the sweep direction is set to reverse (control Sweep Forward set to OFF).
        /// </param>
        /// <param name="Level_Tolerance">
        /// Used to verify the Level of the input signal.
        /// 
        /// Units: dB
        /// 
        /// Valid Values:
        /// 3.0 to 20.0
        /// 
        /// Default Value: 10.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_POW_RLEV_VER_TOL
        /// 
        /// Notes:
        /// 
        /// (1) Specifies the maximum and minimum deviation from the specified Level setting that the input signal may vary by and still pass the verification i.e. measured level between (Level+Level_Tolerance) and (Level-Level_Tolerance) is accepted.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePhaseSignalSettings(double Frequency, double Level, bool Sweep_Forward, bool Verify_Frequency___Level, double Frequency_Tolerance, double Level_Tolerance)
        {
            int pInvokeResult = PInvoke.ConfigurePhaseSignalSettings(this._handle, Frequency, Level, System.Convert.ToUInt16(Sweep_Forward), System.Convert.ToUInt16(Verify_Frequency___Level), Frequency_Tolerance, Level_Tolerance);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the absolute frequency tolerance. This functions automatically sets the Verify Frequency to ON. This is required for the Frequency Tolerance to be enabled and set.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PHASE_FREQ_VER
        /// RSSPECAN_ATTR_PHASE_FREQ_VER_TOL_ABS
        /// 
        /// Remote-control command(s):
        /// [SENSe:]FREQuency:VERify[:STATe] ON
        /// [SENSe:]FREQuency:VERify:TOLerance:ABSolute
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Absolute_Frequency_Tolerance">
        /// Sets the absolute frequency tolerance.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 1.0e3 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_FREQ_VER_TOL_ABS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePhaseFrequencyTolerance(double Absolute_Frequency_Tolerance)
        {
            int pInvokeResult = PInvoke.ConfigurePhaseFrequencyTolerance(this._handle, Absolute_Frequency_Tolerance);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase AFC Settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PHASE_FREQ_TRACK
        /// RSSPECAN_ATTR_PHASE_POW_TRACK
        /// 
        /// Remote-control command(s):
        /// [SENSe:]FREQuency:TRACk ON | OFF
        /// [SENSe:]POWer:TRACk ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Track_Frequency">
        /// Selects whether the center frequency should be tracked and adjusted whilst a measurement is running.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_FREQ_TRACK
        /// 
        /// Notes:
        /// 
        /// (1) Track Frequency is only available if the Verify Frequency &amp;
        /// Level parameter is switched on (attribute RSSPECAN_ATTR_PHASE_FREQ_VER or RSSPECAN_ATTR_PHASE_POW_RLEV_VER)
        /// </param>
        /// <param name="Track_Level">
        /// Selects whether the level should be tracked and adjusted whilst a measurement is running.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_POW_TRACK
        /// 
        /// Notes:
        /// 
        /// (1) Track Level is only available if the Verify Frequency &amp; Level parameter is switched on (attribute RSSPECAN_ATTR_PHASE_FREQ_VER or RSSPECAN_ATTR_PHASE_POW_RLEV_VER)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePhaseAFCSettings(bool Track_Frequency, bool Track_Level)
        {
            int pInvokeResult = PInvoke.ConfigurePhaseAFCSettings(this._handle, System.Convert.ToUInt16(Track_Frequency), System.Convert.ToUInt16(Track_Level));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase Display Settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FREQUENCY_START
        /// RSSPECAN_ATTR_FREQUENCY_STOP
        /// RSSPECAN_ATTR_PHASE_DISP_TRACE_Y_AUTO_ONCE
        /// RSSPECAN_ATTR_PHASE_DISP_TRACE_Y_RLEV
        /// RSSPECAN_ATTR_PHASE_DISP_TRACE_Y
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:STARt
        /// [SENSe&lt;1|2&gt;:]FREQuency:STOP
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:AUTO ONCE
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:RLEVel
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="X_Axis_Start">
        /// Specifies the min frequency for the X axis.
        /// 
        /// Units: Hz
        /// 
        /// Valid Values:
        /// 1.0, 3.0, 10.0, 30.0, 100.0, 300.0, 1.0e+3, 3.0e+3, 10.0e+3, 30.0e+3, 100.0e+3, 300.0e+3, 1.0e+6, 3.0e+6, 10.0e+6, 30.0e+6, 100.0e+6, 300.0e+6
        /// 
        /// Default Value: 1.0e+3 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_START
        /// </param>
        /// <param name="X_Axis_Stop">
        /// Specifies the max frequency for the X axis.
        /// 
        /// Units: Hz
        /// 
        /// Valid Values:
        /// 3.0, 10.0, 30.0, 100.0, 300.0, 1.0e+3, 3.0e+3, 10.0e+3, 30.0e+3, 100.0e+3, 300.0e+3, 1.0e+6, 3.0e+6, 10.0e+6, 100.0e+6, 300.0e+6, 1.0e+9
        /// 
        /// Default Value: 1.0e+6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_STOP
        /// 
        /// </param>
        /// <param name="Autoscale_Once">
        /// Switches the calculation from results of Y Axis scaling. 
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE  (0) - Off
        /// VI_TRUE   (1) - Once
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_DISP_TRACE_Y_AUTO_ONCE
        /// 
        /// Notes:
        /// 
        /// (1) The autoscaling will only be done once in the first sweep. The subsequence sweeps will not auto scale the y-axis.
        /// 
        /// (2) When Autoscale Once is ON, Y Axis Top and Range parameters will be ignored. When it is OFF, Y Axis Top and Range parameters will be used.
        /// 
        /// (3) When Autoscale Once is ON, the attribute RSSPECAN_ATTR_PHASE_DISP_TRACE_Y_AUTO_ONCE is called. When it is OFF, the attribute RSSPECAN_ATTR_PHASE_DISP_TRACE_Y_AUTO is called with VI_FALSE parameter.
        /// </param>
        /// <param name="Y_Axis_Top">
        /// Specifies the max Phase Noise level in y-axis for the trace results.
        /// 
        /// Units: dBc/Hz
        /// 
        /// Valid Values:
        /// -200.0 to 0.0
        /// 
        /// Default Value: -20.0 dBc/Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_DISP_TRACE_Y_RLEV
        /// 
        /// Notes:
        /// 
        /// (1) When Autoscale Once is ON, Y Axis Top and Range parameters will be ignored.
        /// </param>
        /// <param name="Y_Axis_Range">
        /// Specifies the distance from the top to the origin in y-axis.
        /// 
        /// Units: dB
        /// 
        /// Valid Values:
        /// 1.0 to 200.0
        /// 
        /// Default Value: 140.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_DISP_TRACE_Y
        /// 
        /// Notes:
        /// 
        /// (1) When Autoscale Once is ON, Y Axis Top and Range parameters will be ignored.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePhaseDisplaySettings(double X_Axis_Start, double X_Axis_Stop, bool Autoscale_Once, double Y_Axis_Top, double Y_Axis_Range)
        {
            int pInvokeResult = PInvoke.ConfigurePhaseDisplaySettings(this._handle, X_Axis_Start, X_Axis_Stop, System.Convert.ToUInt16(Autoscale_Once), Y_Axis_Top, Y_Axis_Range);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase Trace Settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PHASE_DISP_TRACE_Y_RLEV_OFFS
        /// RSSPECAN_ATTR_PHASE_DISP_TRACE_SMO_APER
        /// RSSPECAN_ATTR_PHASE_DISP_TRACE_SMO_TYPE
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:RLEVel:OFFSet
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:SMOothing:APERture
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:SMOothing:TYPE[?] LINear | LOGarithmic
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Trace_Offset">
        /// Specifies the arithmetical reference level offset which is added to the y axis labelling.
        /// 
        /// Units: dB
        /// 
        /// Valid Values:
        /// -200.0 to 200.0
        /// 
        /// Default Value: 0.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_DISP_TRACE_Y_RLEV_OFFS
        /// </param>
        /// <param name="Trace_Smoothing">
        /// Specifies the % of the display width to be used as a window when a trace is smoothed.
        /// 
        /// Units: %
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value: 1 %
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_DISP_TRACE_SMO_APER
        /// 
        /// Notes:
        /// 
        /// (1) The larger the setting of the Smoothing parameter the greater the effect of smoothing.
        /// </param>
        /// <param name="Smoothing_Type">
        /// Select the smoothing type for the trace. 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_PHASE_SMO_TYPE_LIN (0) - Linear
        /// RSSPECAN_VAL_PHASE_SMO_TYPE_LOG (1) - Logarithmic
        /// 
        /// Default Value: RSSPECAN_VAL_PHASE_SMO_TYPE_LIN
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_DISP_TRACE_SMO_TYPE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePhaseTraceSettings(double Trace_Offset, int Trace_Smoothing, int Smoothing_Type)
        {
            int pInvokeResult = PInvoke.ConfigurePhaseTraceSettings(this._handle, Trace_Offset, Trace_Smoothing, Smoothing_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase Residual Calculations.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PHASE_CALC_EVAL
        /// RSSPECAN_ATTR_PHASE_CALC_EVAL_STAR
        /// RSSPECAN_ATTR_PHASE_CALC_EVAL_STOP
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:EVALuation[:STATe]
        /// CALCulate&lt;1|2&gt;:EVALuation:STARt
        /// CALCulate&lt;1|2&gt;:EVALuation:STOP
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Use_Measurement_Settings">
        /// Specifies whether to use the whole measurement range or the user defined evaluation range for the residual calculations.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_CALC_EVAL
        /// 
        /// Notes:
        /// 
        /// (1) If the Use Measurement Settings is switched on then  Evaluation From and Evaluation To are ignored and residual calculations are performed across the complete range of the measurement results.
        /// 
        /// (2) If the Use Measurement Settings is switched off then  Evaluate From and Evaluate To values are used and allow the range over which residual calculations are performed to be specified.
        /// </param>
        /// <param name="Evaluate_From">
        /// Specifies the start of the measurement range for which residual calculations are to be performed.
        /// 
        /// Units: Hz
        /// 
        /// Valid Values:
        /// 1.0e+3 to 1.0e+6
        /// 
        /// Default Value: 1.0e+3 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_CALC_EVAL_STAR
        /// 
        /// Notes:
        /// 
        /// (1) The minimum value that can be specified for the Evaluate From setting is the value of the X Axis Start setting.
        /// 
        /// (2) The maximum value that can be specified for the Evaluate From setting is the value of the X Axis Stop setting.
        /// 
        /// (3) When the X Axis Start or X Axis Stop settings are modified
        /// the Evaluate From will be automatically adjusted to ensure that it is not outside the measurement range.
        /// 
        /// (4) The Evaluate From setting cannot be set higher than the Evaluate To settings.
        /// </param>
        /// <param name="Evaluate_To">
        /// Specifies the end of the measurement range for which residual calculations are to be performed.
        /// 
        /// Units: Hz
        /// 
        /// Valid Values:
        /// 1.0e+3 to 1.0e+6
        /// 
        /// Default Value: 1.0e+6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_CALC_EVAL_STOP
        /// 
        /// Notes:
        /// 
        /// (1) The minimum value that can be specified for the Evauate To setting is the value of the X Axis Start setting.
        /// 
        /// (2) The maximum value that can be specified for the Evauate To setting is the value of the X Axis Stop setting.
        /// 
        /// (3) When the X Axis Start or X Axis Stop settings are modified the Evauate From will be automatically adjusted to ensure that it is not outside the measurement range.
        /// 
        /// (4) The Evauate To setting cannot be set lower than the Evauate from settings.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePhaseResidualCalculations(bool Use_Measurement_Settings, double Evaluate_From, double Evaluate_To)
        {
            int pInvokeResult = PInvoke.ConfigurePhaseResidualCalculations(this._handle, System.Convert.ToUInt16(Use_Measurement_Settings), Evaluate_From, Evaluate_To);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase Spot Noise Settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PHASE_CALC_SNO_STAT
        /// RSSPECAN_ATTR_PHASE_CALC_SNO_X
        /// 
        /// Remote-control command(s):
        /// CALCulate:SNOise&lt;1 to 4&gt;:STATe ON | OFF
        /// CALCulate:SNOise&lt;1 to 4&gt;:X
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Enable">
        /// Allows spot noise calculations to be switched on and off.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_CALC_SNO_STAT
        /// 
        /// Notes:
        /// 
        /// (1) If this parameter is turned off, the offset frequencies 1 to 4 are ignored.
        /// </param>
        /// <param name="Offset_Frequency_1">
        /// Allows the first frequency point to be specified at which spot noise calculations will be performed.
        /// 
        /// Units: Hz
        /// 
        /// Valid Values:
        /// 1.0 to 1.0e+9
        /// 
        /// Default Value: 1.0e+3
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_CALC_SNO_X
        /// 
        /// Notes:
        /// 
        /// (1) If an offset frequency is specified which is outside the measurement frequency range then no results will be displayed fo that offset frequency.
        /// 
        /// (2) If the parameter Enable is set to Off, the offset frequencies 1 to 4 are ignored.
        /// </param>
        /// <param name="Offset_Frequency_2">
        /// Allows the second frequency point to be specified at which spot noise calculations will be performed.
        /// 
        /// Units: Hz
        /// 
        /// Valid Values:
        /// 1.0 to 1.0e+9
        /// 
        /// Default Value: 10.0e+3
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_CALC_SNO_X
        /// 
        /// Notes:
        /// 
        /// (1) If an offset frequency is specified which is outside the measurement frequency range then no results will be displayed fo that offset frequency.
        /// 
        /// (2) If the parameter Enable is set to Off, the offset frequencies 1 to 4 are ignored.
        /// </param>
        /// <param name="Offset_Frequency_3">
        /// Allows the third frequency point to be specified at which spot noise calculations will be performed.
        /// 
        /// Units: Hz
        /// 
        /// Valid Values:
        /// 1.0 to 1.0e+9
        /// 
        /// Default Value: 100.0e+3
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_CALC_SNO_X
        /// 
        /// Notes:
        /// 
        /// (1) If an offset frequency is specified which is outside the measurement frequency range then no results will be displayed fo that offset frequency.
        /// 
        /// (2) If the parameter Enable is set to Off, the offset frequencies 1 to 4 are ignored.
        /// </param>
        /// <param name="Offset_Frequency_4">
        /// Allows the forth frequency point to be specified at which spot noise calculations will be performed.
        /// 
        /// Units: Hz
        /// 
        /// Valid Values:
        /// 1.0 to 1.0e+9
        /// 
        /// Default Value: 1.0e+6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_CALC_SNO_X
        /// 
        /// Notes:
        /// 
        /// (1) If an offset frequency is specified which is outside the measurement frequency range then no results will be displayed fo that offset frequency.
        /// 
        /// (2) If the parameter Enable is set to Off, the offset frequencies 1 to 4 are ignored.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePhaseSpotNoiseSettings(bool Enable, double Offset_Frequency_1, double Offset_Frequency_2, double Offset_Frequency_3, double Offset_Frequency_4)
        {
            int pInvokeResult = PInvoke.ConfigurePhaseSpotNoiseSettings(this._handle, System.Convert.ToUInt16(Enable), Offset_Frequency_1, Offset_Frequency_2, Offset_Frequency_3, Offset_Frequency_4);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase Analyzer Settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NOISE_INP_PRES_STAT
        /// RSSPECAN_ATTR_NOISE_INP_GAIN_STAT
        /// 
        /// Remote-control command(s):
        /// INPut:PRESelection[:STATe][?] ON | OFF
        /// INPut:GAIN:STATe[?] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Pre_selector">
        /// This control switches the preselection on or off when option B2 is installed.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_INP_PRES_STAT
        /// 
        /// Notes:
        /// 
        /// (1) If both B2 and B25 options are installed only one
        /// pre-amplifier may be switched on. In this case the setting of
        /// this control determines which of the Pre-amplifiers may
        /// be switched on:
        /// Pre-selector On - Pre-amplifier for the pre-selector B2 can be controlled.
        /// Pre-selector Off - Pre-amplifier for the electronic
        /// attenuator B25 can be controlled.
        /// </param>
        /// <param name="Pre_amplifier__B2_B25_B23_">
        /// This control switches the pre-amplifier on and off. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NOISE_INP_GAIN_STAT
        /// 
        /// Notes:
        /// 
        /// (1) The B23 option is always required. If only the B2 or B25 option is present in addition to B23 then the pre-amplifier of the installed option is controlled. If both the B2 and B25 options are present then the pre-amplifier controlled by this attribute depends on the setting for the RSSPECAN_ATTR_NOISE_INP_PRES_STAT attribute. If the RSSPECAN_ATTR_NOISE_INP_PRES_STAT is set to ON then the
        /// pre-amplifier for the B2 option is controlled by this function, otherwise the pre-amplifier of the B25 option is controlled.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePhaseAnalyzerSettings(bool Pre_selector, bool Pre_amplifier__B2_B25_B23_)
        {
            int pInvokeResult = PInvoke.ConfigurePhaseAnalyzerSettings(this._handle, System.Convert.ToUInt16(Pre_selector), System.Convert.ToUInt16(Pre_amplifier__B2_B25_B23_));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets all the phase spot noise markers off.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PHASE_CALC_SNO_AOFF
        /// 
        /// Remote-control command(s):
        /// CALCulate:SNOise:AOFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SetPhaseAllSpotNoiseMarkersOff()
        {
            int pInvokeResult = PInvoke.SetPhaseAllSpotNoiseMarkersOff(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase Sweep Settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PHASE_SWEEP_MODE
        /// 
        /// Remote-control command(s):
        /// [SENSe:]SWEep:MODe FAST | NORMal | AVERaged | MANual
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Sweep_Mode">
        /// Updates the Carrier Frequency Offset Table from the instruments default settings.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SWE_MODE_FAST (0) - Fast
        /// RSSPECAN_VAL_SWE_MODE_NORM (1) - Normal
        /// RSSPECAN_VAL_SWE_MODE_AVER (2) - Averaged
        /// RSSPECAN_VAL_SWE_MODE_MAN  (3) - Manual
        /// 
        /// Default Value: RSSPECAN_VAL_SWE_MODE_MAN
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_SWEEP_MODE
        /// 
        /// Notes:
        /// 
        /// (1) Fast - Not averaged. The measurement runs very fast. As the average column is set to 1 for all sub-bands.
        /// 
        /// (2) Normal - Normal averaged. The measurement runs slower then the fast mode, but the sub-bands are averaged more.
        /// 
        /// (3) Averaged - Highly averaged. The measurement runs very slow with high average in each sub-band for more accurate results.
        /// 
        /// (4) Manual - The RBW, Average and FFT columns can be set by the user.
        /// 
        /// (5) In fast, normal and avaeraged modes, the table is not editable, it is information only.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePhaseSweepSettings(int Sweep_Mode)
        {
            int pInvokeResult = PInvoke.ConfigurePhaseSweepSettings(this._handle, Sweep_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase Span Settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FREQUENCY_START
        /// RSSPECAN_ATTR_FREQUENCY_STOP
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:STARt
        /// [SENSe&lt;1|2&gt;:]FREQuency:STOP
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Start_Offset">
        /// Defines the start frequency of the measurement.
        /// 
        /// Units: Hz
        /// 
        /// Valid Values:
        /// 1.0, 3.0, 10.0, 30.0, 100.0, 300.0,,1.0e+3, 3.0e+3, 10.0e+3, 30.0e+3, 100.0e+3, 300.0e+3, 1.0e+6, 3.0e+6, 10.0e+6, 30.0e+6, 100.0e+6, 300.0e+6
        /// 
        /// Default Value: 1.0e+3 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_START
        /// 
        /// Notes:
        /// 
        /// (1) When this parameter changes the X-axis Start parameter in
        /// General Settings (Noise) view gets updated accordingly.
        /// 
        /// (2) The Carrier Frequency Offset Table is highlighted to show
        /// the selected span for the noise measurement.
        /// </param>
        /// <param name="Stop_Offset">
        /// Defines the stop frequency of the measurement.
        /// 
        /// Units: Hz
        /// 
        /// Valid Values:
        /// 3.0, 10.0, 30.0, 100.0, 300.0, 1.0e+3, 3.0e+3, 10.0e+3, 30.0e+3, 100.0e+3, 300.0e+3, 1.0e+6, 3.0e+6, 10.0e+6, 100.0e+6, 300.0e+6, 1.0e+9
        /// 
        /// Default Value: 1.0e+6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_STOP
        /// 
        /// Notes:
        /// 
        /// (1) When this parameter changes the X-axis Stop parameter in General Settings (Noise) view gets updated accordingly.
        /// 
        /// (2) The Carrier Frequency Offset Table is highlighted to show the selected span for the noise measurement.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePhaseSpanSettings(double Start_Offset, double Stop_Offset)
        {
            int pInvokeResult = PInvoke.ConfigurePhaseSpanSettings(this._handle, Start_Offset, Stop_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Carrier Frequency Offset Table.
        /// 
        /// Notes:
        /// 
        /// (1) The Carrier Frequency Offset Table can be used only when the Configure Sweep Settings is set to Manual value.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PHASE_LIST_RANG_BAND
        /// RSSPECAN_ATTR_PHASE_LIST_RANG_SWE_COUN
        /// RSSPECAN_ATTR_PHASE_LIST_RANG_FILT_TYPE
        /// 
        /// Remote-control command(s):
        /// [SENSe:]LIST:RANGe&lt;1 to 18&gt;:BANDwidth[:RESolution] &lt;numeric_value&gt; [Hz]
        /// [SENSe:]LIST:RANGe&lt;1 to 18&gt;:SWEep:COUNt
        /// [SENSe:]LIST:RANGe&lt;1 to 18&gt;:FILTer:TYPE NORMal | FFT
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Sub_band">
        /// Specifies the sub-band for the RBW, Average and FFT.
        /// 
        /// Valid Values:
        /// 1 to 18
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="RBW">
        /// Sets the resolution filter bandwidth used for each sub-band.  
        /// 
        /// Units: Hz
        /// 
        /// Valid Values:
        /// 1.0 to 300.0e+6
        /// 
        /// Default Value: 10.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_LIST_RANG_BAND
        /// 
        /// Notes:
        /// 
        /// (1) Enter values in steps of 1/3/10.
        /// </param>
        /// <param name="Average">
        /// Sets the number of sweeps to average over for each sub-band.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 1 to 10000
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_LIST_RANG_SWE_COUN
        /// 
        /// </param>
        /// <param name="FFT">
        /// Allows the user to choose whether to use the FFT Resolution Filter or the conventional filter for each decade.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_BAND_TYPE_NORM (0) - Normal
        /// RSSPECAN_VAL_BAND_TYPE_FFT  (1) - FFT
        /// 
        /// Default Value: RSSPECAN_VAL_BAND_TYPE_FFT
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_LIST_RANG_FILT_TYPE
        /// 
        /// Notes:
        /// 
        /// (1) FFT is only available for RBW values between
        /// 1 Hz and 30 kHz.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePhaseCarrierFrequencyOffsetTable(int Sub_band, double RBW, int Average, int FFT)
        {
            int pInvokeResult = PInvoke.ConfigurePhaseCarrierFrequencyOffsetTable(this._handle, Sub_band, RBW, Average, FFT);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase Preset Settings.
        /// 
        /// Notes:
        /// 
        /// (1) The Preset Settings can be used only when the Configure Sweep Settings is set to Manual value.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PHASE_BAND_RAT
        /// RSSPECAN_ATTR_PHASE_SWEEP_COUNT
        /// RSSPECAN_ATTR_PHASE_BAND_TYPE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:BWIDth[:RESolution]:RATio
        /// [SENSe&lt;1|2&gt;:]LIST:SWEep:COUNt
        /// [SENSe&lt;1|2&gt;:]LIST:BWIDth[:RESolution]:TYPE NORMal | FFT
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="RBW">
        /// The RBW parameter sets all the RBW settings in the Carrier Frequency Offset table to a proportion of the start frequency for each sub-band.
        /// 
        /// Units: %
        /// 
        /// Valid Values:
        /// 0.1 to 30.0
        /// 
        /// Default Value: 10.0 %
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_BAND_RAT
        /// 
        /// Notes:
        /// 
        /// (1) The resulting RBW is rounded to the nearest 1/3/10 steps.
        /// </param>
        /// <param name="Average">
        /// Sets all the Average settings in the Carrier Frequency Offset table to the specified value.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 1 to 10000
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_SWEEP_COUNT
        /// </param>
        /// <param name="FFT">
        /// Sets all the FFT settings in the Carrier Frequency Offset table to the specified value.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_BAND_TYPE_NORM (0) - Normal
        /// RSSPECAN_VAL_BAND_TYPE_FFT  (1) - FFT
        /// 
        /// Default Value: RSSPECAN_VAL_BAND_TYPE_FFT
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_BAND_TYPE
        /// 
        /// Notes:
        /// 
        /// (1) Only those sub-bands where FFT filters are available will have FFT filters enabled if this parameter is switched on.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigurePhasePresetSettings(double RBW, int Average, int FFT)
        {
            int pInvokeResult = PInvoke.ConfigurePhasePresetSettings(this._handle, RBW, Average, FFT);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function creates new limit or selects existing one and configures basic parameters.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_NAME
        /// RSSPECAN_LIMIT_ASSIGN_TRACE
        /// RSSPECAN_LIMIT_COMMENT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:NAME
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:COMMent
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:TRACe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Name">
        /// This control sets the limit line name.
        /// 
        /// Valid Values:
        /// up to 8 character string
        /// 
        /// Default Value: "limit"
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_NAME
        /// 
        /// </param>
        /// <param name="Comment">
        /// This control sets the limit line comment.
        /// 
        /// Valid Values:
        /// up to 40 character string
        /// 
        /// Default Value: "Comment"
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_COMMENT
        /// </param>
        /// <param name="Assign_to_Trace">
        /// Assigns a limit line to a trace in the indicated measurement window.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_ASSIGN_TRACE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CreatePhaseLimitLine(int Window, int Limit, string Name, string Comment, int Assign_to_Trace)
        {
            int pInvokeResult = PInvoke.CreatePhaseLimitLine(this._handle, Window, Limit, Name, Comment, Assign_to_Trace);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines specified limit line.
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:CONTrol[:DATA]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:LOWer[:DATA]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:UPPer[:DATA]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Type">
        /// Specifies a limit line to be either an upper or lower type limit line. An upper line will be used as the maximum allowable value when comparing with the data. A lower limit line defines the minimum value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIMIT_LOWER (0) - Lower
        /// RSSPECAN_VAL_LIMIT_UPPER (1) - Upper
        /// 
        /// Default Value: RSSPECAN_VAL_LIMIT_LOWER (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Lower - lower limit fails if the trace falls below the limit.
        /// 
        /// (2) Upper - upper limit fails if the trace exceeds the limit.
        /// 
        /// </param>
        /// <param name="Count">
        /// Specifies number of values in X Axis and Amplitude arrays.
        /// 
        /// Valid Values:
        /// &gt;= 1
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="X_Axis">
        /// Specifies the frequency or time value for a limit point. The units are depended on the attribute RSSPECAN_ATTR_LLINE_X_AXIS_UNITS.
        /// 
        /// You must pass a ViReal64 array with at least count values.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Amplitude">
        /// Specifies the amplitude value for the current limit point.
        /// 
        /// You must pass a ViReal64 array with at least count values.
        /// 
        /// Units: Are depended on the RSSPECAN_LIMIT_UPPER_SCALING or RSSPECAN_LIMIT_LOWER_SCALING attribute.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int DefinePhaseLimitLine(int Limit, int Type, int Count, double[] X_Axis, double[] Amplitude)
        {
            int pInvokeResult = PInvoke.DefinePhaseLimitLine(this._handle, Limit, Type, Count, X_Axis, Amplitude);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the limit check and/or limit line for the selected limit line in the selected measurement window.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_STATE
        /// RSSPECAN_LIMIT_LOWER_STATE
        /// RSSPECAN_LIMIT_UPPER_STATE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:STATe ON | OFF
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:LOWer:STATe ON | OFF
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:UPPer:STATe ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Type">
        /// Specifies a limit line to be either an upper or lower type limit line. An upper line will be used as the maximum allowable value when comparing with the data. A lower limit line defines the minimum value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIMIT_LOWER (0) - Lower
        /// RSSPECAN_VAL_LIMIT_UPPER (1) - Upper
        /// 
        /// Default Value: RSSPECAN_VAL_LIMIT_LOWER (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Lower - lower limit fails if the trace falls below the limit.
        /// 
        /// (2) Upper - upper limit fails if the trace exceeds the limit.
        /// 
        /// </param>
        /// <param name="Limit_Enabled">
        /// Turns the limit line display on and off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_STATE
        /// </param>
        /// <param name="Check_Enabled">
        /// Turns limit line testing on/off. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_LOWER_STATE
        /// RSSPECAN_LIMIT_UPPER_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int EnablePhaseLimitCheck(int Window, int Limit, int Type, bool Limit_Enabled, bool Check_Enabled)
        {
            int pInvokeResult = PInvoke.EnablePhaseLimitCheck(this._handle, Window, Limit, Type, System.Convert.ToUInt16(Limit_Enabled), System.Convert.ToUInt16(Check_Enabled));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function moves specified limit line.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_CONTROL_SHIFT
        /// RSSPECAN_LIMIT_LOWER_SHIFT
        /// RSSPECAN_LIMIT_UPPER_SHIFT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:CONTrol:SHIFt
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:LOWer:SHIFt
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:UPPer:SHIFt
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Type">
        /// Specifies a limit line to be type to be moved.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LIMIT_LOWER   (0) - Lower
        /// RSSPECAN_VAL_LIMIT_UPPER   (1) - Upper
        /// RSSPECAN_VAL_LIMIT_CONTROL (2) - X-Axis
        /// 
        /// Default Value: RSSPECAN_VAL_LIMIT_LOWER (0)
        /// </param>
        /// <param name="Value">
        /// Defines the move value.
        /// 
        /// Valid Values:
        /// -200.0 to 200.0 dB for Lower and Upper
        /// -300.0e9 to -300.0e9 Hz for X-Axis
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_CONTROL_SHIFT
        /// RSSPECAN_LIMIT_LOWER_SHIFT
        /// RSSPECAN_LIMIT_UPPER_SHIFT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int MovePhaseLimitLine(int Limit, int Type, double Value)
        {
            int pInvokeResult = PInvoke.MovePhaseLimitLine(this._handle, Limit, Type, Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function copies specified limit line into another one.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_COPY
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:COPY
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Copy_To">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_LIMIT_COPY
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CopyPhaseLimitLine(int Limit, int Copy_To)
        {
            int pInvokeResult = PInvoke.CopyPhaseLimitLine(this._handle, Limit, Copy_To);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function deletes the result of the current limit check.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_CLEAR
        /// 
        /// Remote-control command(s):
        /// CALCulate:LIMit&lt;1 to 8&gt;:CLEar[:IMMediate]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 6
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ClearPhaseLimitLineResults(int Limit)
        {
            int pInvokeResult = PInvoke.ClearPhaseLimitLineResults(this._handle, Limit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function removes specified limit line.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_DELETE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:DELete
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int DeletePhaseLimitLine(int Limit)
        {
            int pInvokeResult = PInvoke.DeletePhaseLimitLine(this._handle, Limit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Phase Trace Settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TRACE_STATE
        /// RSSPECAN_ATTR_TRACE_TYPE
        /// RSSPECAN_ATTR_PHASE_DISP_TRACE_SMO
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;[:STATe] ON | OFF
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:MODE WRITe | VIEW | AVERage | MAXHold | MINHold
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:SMOothing[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Trace_Number">
        /// This control selects the trace number.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Trace_State">
        /// Switches on or off the display of the corresponding trace.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRACE_STATE
        /// </param>
        /// <param name="Trace_Mode">
        /// Defines the type of display and the evaluation of the traces.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_TRAC_MOD_WRITE (0) - Clear/Write
        /// RSSPECAN_TRAC_MOD_VIEW  (1) - View
        /// RSSPECAN_TRAC_MOD_AVER  (2) - Average
        /// RSSPECAN_TRAC_MOD_MAXH  (3) - Maxhold
        /// RSSPECAN_TRAC_MOD_MINH  (4) - Minhold
        /// 
        /// Default Value: RSSPECAN_TRAC_MOD_WRITE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRACE_TYPE
        /// 
        /// Notes:
        /// 
        /// (1) WRITE corresponds to the Clr/Write mode of manual operation.
        /// 
        /// (2) The trace is switched off (= BLANK in manual operation) with RSSPECAN_ATTR_TRACE_STATE OFF.
        /// 
        /// (3) The number of measurements for AVERage is defined with the attribute RSSPECAN_ATTR_NUMBER_OF_SWEEPS.
        /// 
        /// (4) The synchronization to the end of the indicated number of measurements is only possible in single sweep mode.
        /// </param>
        /// <param name="Smoothing">
        /// specifies whether smoothing of a particular trace is carried out.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_PHASE_DISP_TRACE_SMO
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int TracePhaseSettings(int Trace_Number, bool Trace_State, int Trace_Mode, bool Smoothing)
        {
            int pInvokeResult = PInvoke.TracePhaseSettings(this._handle, Trace_Number, System.Convert.ToUInt16(Trace_State), Trace_Mode, System.Convert.ToUInt16(Smoothing));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures and initiates a reference measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PHASE_CONF_REFM
        /// 
        /// Remote-control command(s):
        /// CONFigure:REFMeas ONCE
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value for data readout in milliseconds.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 10000 ms
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReferencePhaseMeasurement(uint Timeout)
        {
            int pInvokeResult = PInvoke.ReferencePhaseMeasurement(this._handle, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the Phase Sweep Estimated Measurement Time.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SWEEP_TIME
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:TIME
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Estimated_Measurement_Time">
        /// Returns the estimated measurement time. The driver uses this value to set the RSSPECAN_ATTR_SWEEP_TIME attribute.
        /// 
        /// Units: seconds
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetPhaseEstimatedMeasurementTime(out double Estimated_Measurement_Time)
        {
            int pInvokeResult = PInvoke.GetPhaseEstimatedMeasurementTime(this._handle, out Estimated_Measurement_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out trace data from the instrument.
        /// 
        /// Notes:
        /// 
        /// (1) If the FM demodulator (option FS-K7) is active, only the displayed trace data is read out and recalled. A portion of the measurement data that can be called by means of a marker, however, is calculated from the raw measurement data. These results are no longer available after recalling a trace; the associated queries generate a query error.
        /// 
        /// (2) The associated measurement window is selected with the function Set Active Window (rsspecan_DisplaySetActiveWindow).
        /// 
        /// Remote-control command(s):
        /// FORMat[:DATA] ASCII
        /// TRACe&lt;1|2|3&gt;? TRACE1 | TRACE2 | TRACE3
        /// FORMat[:DATA] REAL,32
        /// *CLS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Source_Trace">
        /// This control sets the source trace data to be read from instrument.
        /// 
        /// Valid Values:
        /// 0 - Trace 1 (Default Value)
        /// 1 - Trace 2
        /// 2 - Trace 3
        /// 
        /// </param>
        /// <param name="No_of_Points">
        /// Returns the number of trace data points.
        /// 
        /// Note:
        /// 
        /// You can use this parameter to pass the expected number of points to be read from the instrument. Data points will be read until either number of data points are consumed or instruments output buffer is empty.
        /// 
        /// </param>
        /// <param name="Trace_Data_X">
        /// Trace data array.
        /// 
        /// Note:
        /// 
        /// The returned values are scaled in the current level unit. Returned FM-modulated measurement values (activated option FS-K7 or FS-K8 (BLUETOOTH-Analyzer)) are scaled in Hz.
        /// 
        /// </param>
        /// <param name="Trace_Data_Y">
        /// Trace data array.
        /// 
        /// Note:
        /// 
        /// The returned values are scaled in the current level unit. Returned FM-modulated measurement values (activated option FS-K7 or FS-K8 (BLUETOOTH-Analyzer)) are scaled in Hz.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadPhaseTraceData(int Source_Trace, out int No_of_Points, double[] Trace_Data_X, double[] Trace_Data_Y)
        {
            int pInvokeResult = PInvoke.ReadPhaseTraceData(this._handle, Source_Trace, out No_of_Points, Trace_Data_X, Trace_Data_Y);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the phase measurement results.
        /// 
        /// Attribute(s):
        /// None
        /// 
        /// Remote-control command(s):
        /// FETCh:PNOise:RFM?
        /// FETCh:PNOise:RPM?
        /// FETCh:PNOise:RMS?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Modifier">
        /// Sets the modifier of returned value.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_PHASE_RFM (0) - Residual FM result
        /// RSSPECAN_VAL_PHASE_RPM (1) - Residual PM result
        /// RSSPECAN_VAL_PHASE_RMS (2) - Residual RMS result
        /// 
        /// </param>
        /// <param name="Value">
        /// Returns the value. 
        /// 
        /// Units:
        /// Hz - RFM
        /// degrees - RPM
        /// seconds - RMS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchPhaseResidualResults(int Modifier, out double Value)
        {
            int pInvokeResult = PInvoke.FetchPhaseResidualResults(this._handle, Modifier, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the Phase Spot Noise Y Position.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_PHASE_CALC_SNO_Y
        /// 
        /// Remote-control command(s):
        /// CALCulate:SNOise&lt;1 to 4&gt;:Y?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Spot_Noise_Marker_Number">
        /// This control sets the Spot Noise Marker Number.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Spot_Noise_Marker_Result">
        /// Returns the measured spot noise marker result. The driver uses this value to set the RSSPECAN_ATTR_PHASE_CALC_SNO_Y attribute.
        /// 
        /// Units: dBc/Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetPhaseSpotNoiseYPosition(int Spot_Noise_Marker_Number, out double Spot_Noise_Marker_Result)
        {
            int pInvokeResult = PInvoke.GetPhaseSpotNoiseYPosition(this._handle, Spot_Noise_Marker_Number, out Spot_Noise_Marker_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the result of the limit check of the limit line indicated in the selected measurement window. It should be noted that a complete sweep must have been performed for obtaining a valid result. A synchronization with  *OPC, *OPC? or *WAI should therefore be provided.
        /// 
        /// Attribute(s):
        /// RSSPECAN_LIMIT_CHECK_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:FAIL?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// This control selects limit.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// The result of the limit check responds with 0 for PASS, 1 for FAIL, and 2 for MARGIN.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetPhaseLimitCheckResult(int Window, int Limit, out int State)
        {
            int pInvokeResult = PInvoke.GetPhaseLimitCheckResult(this._handle, Window, Limit, out State);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the instrument into Vector Signal Analysis mode. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument[:SELect] DDEMod
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SetVSAMode()
        {
            int pInvokeResult = PInvoke.SetVSAMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function restores the factory settings of the parameters for the R&amp;S FSQ-K70 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_FACTORY_DEFAULTS
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:FACTory GST
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Defaults">
        /// Selects the item to be set to factory default.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VSA_FACTORY_GST  (0) - Groups 
        /// RSSPECAN_VAL_VSA_FACTORY_STAN (1) - Standards
        /// RSSPECAN_VAL_VSA_FACTORY_MAPP (2) - Mapping
        /// RSSPECAN_VAL_VSA_FACTORY_PATT (3) - Patterns
        /// RSSPECAN_VAL_VSA_FACTORY_FILT (4) - Filters
        /// RSSPECAN_VAL_VSA_FACTORY_ALL  (5) - Both Standards and Patterns
        /// 
        /// Default Value: RSSPECAN_VAL_VSA_FACTORY_GST (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_FACTORY_DEFAULTS
        /// 
        /// Note(s):
        /// 
        /// (1) Both Standards and Patterns: only available for R&amp;S FSV.
        /// 
        /// (2) R&amp;S FSV instrument supports only: Both Standards and Patterns, Standards, Pattern
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SetVSAFactoryDefaults(int Defaults)
        {
            int pInvokeResult = PInvoke.SetVSAFactoryDefaults(this._handle, Defaults);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function initiates automatic setting of the RF attenuation and IF gain to the level of the applied signal.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_REF_LEVEL_ADJUST
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:PRESet:RLEVel
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int VSARefLevelAdjust(int Window, uint Timeout)
        {
            int pInvokeResult = PInvoke.VSARefLevelAdjust(this._handle, Window, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function enables or disables selected marker.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_ENABLED
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker_Number">
        /// Selects the marker to configure.
        /// 
        /// Valid Values:
        /// 1 to 4 
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Marker_State">
        /// Enables or disables the selected marker.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_ENABLED
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAMarkerState(int Window, int Marker_Number, bool Marker_State)
        {
            int pInvokeResult = PInvoke.ConfigureVSAMarkerState(this._handle, Window, Marker_Number, System.Convert.ToUInt16(Marker_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function couples markers between several screens, i.e. uses the same stimulus. All screens can be linked with an X-axis scaled in symbols or time, except those showing the capture buffer.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_SCREEN_COUPLING
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1..4&gt;:MARKer&lt;1 to 4&gt;:LINK ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker_Number">
        /// Selects the marker to configure.
        /// 
        /// Valid Values:
        /// 1 to 4 
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="State">
        /// Specifies the state of the screen coupling.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value:  VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_SCREEN_COUPLING
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAScreenCoupling(int Window, int Marker_Number, bool State)
        {
            int pInvokeResult = PInvoke.ConfigureVSAScreenCoupling(this._handle, Window, Marker_Number, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures whether the marker search works on the real or the imag trace.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_MARKER_SEARCH
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1..4&gt;:MARKer&lt;1 to 4&gt;:SEARch REAL | IMAG
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Search">
        /// Specifies whether the marker search works on the real or the imag trace.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VSA_MARKER_SEARCH_REAL (0) - Real
        /// RSSPECAN_VAL_VSA_MARKER_SEARCH_IMAG (1) - Imag
        /// 
        /// Default Value:  RSSPECAN_VAL_VSA_MARKER_SEARCH_REAL (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_MARKER_SEARCH
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAMarkerSearch(int Window, int Search)
        {
            int pInvokeResult = PInvoke.ConfigureVSAMarkerSearch(this._handle, Window, Search);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function searches for the max absolute peak of a trace (independent of its sign). This is especially usefull for measurements where a zero value is best, and either large positive or large negative values indicate a bad signal.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_MARKER_PEAK_SEARCH
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1..4&gt;:MARKer&lt;1 to 4&gt;:MAXimum:APEak
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker_Number">
        /// Selects the marker to configure.
        /// 
        /// Valid Values:
        /// 1 to 4 
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAMarkerPeakSearch(int Window, int Marker_Number)
        {
            int pInvokeResult = PInvoke.ConfigureVSAMarkerPeakSearch(this._handle, Window, Marker_Number);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function adjusts the reference level for digital input to the full scale level automatically if the fullscale level changes.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_ADJUST_REFERENCE_LEVEL_TO_FULL_SCALE_LEVEL
        /// 
        /// Remote-control command(s):
        /// INPut:DIQ:RANGe:COUPling ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Adjust_Reference_Level">
        /// If enabled, the reference level for digital input is adjusted to the full scale level automatically if the fullscale level changes.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_ADJUST_REFERENCE_LEVEL_TO_FULL_SCALE_LEVEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int VSAAdjustReferenceLevelToFullScaleLevel(bool Adjust_Reference_Level)
        {
            int pInvokeResult = PInvoke.VSAAdjustReferenceLevelToFullScaleLevel(this._handle, System.Convert.ToUInt16(Adjust_Reference_Level));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the unit of the full scale level.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_FULL_SCALE_LEVEL_UNIT
        /// 
        /// Remote-control command(s):
        /// INPut:DIQ:RANGe[:UPPer]:UNIT V | dBm | dBpW | W | dBmV | dBuV | dBuA | A
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Full_Scale_Level_Unit">
        /// Defines the unit of the full scale level.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DBM       (0) - dBm
        /// RSSPECAN_VAL_UNIT_DBPW      (1) - dBpW
        /// RSSPECAN_VAL_UNIT_WATT      (2) - W
        /// RSSPECAN_VAL_UNIT_DBUV      (3) - dBuV
        /// RSSPECAN_VAL_UNIT_DBMV      (4) - dBmV
        /// RSSPECAN_VAL_UNIT_VOLT      (5) - V
        /// RSSPECAN_VAL_UNIT_DBUA      (6) - dBuA
        /// RSSPECAN_VAL_UNIT_AMP       (7) - A
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_VOLT  (5)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_FULL_SCALE_LEVEL_UNIT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAFullScaleLevelUnit(int Full_Scale_Level_Unit)
        {
            int pInvokeResult = PInvoke.ConfigureVSAFullScaleLevelUnit(this._handle, Full_Scale_Level_Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function presets the screens.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_PRESET_SCREENS
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:PRESet:CALC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int PresetVSAScreens(int Window)
        {
            int pInvokeResult = PInvoke.PresetVSAScreens(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects an automatic setting of all modulation parameters according to a standardized transmission method or a user-defined transmission method. The standardized
        /// transmission methods are available in the unit (predefined standard). The transmission methods are set via a file name without extension (string data with quotation marks). Some predefined transmission methods can be set via a value also (character data without quotation marks) for reasons of compatibility with former instrument models.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_DIGITAL_STANDARD
        /// RSSPECAN_ATTR_VSA_FREQUENCY_STEP_AUTO
        /// RSSPECAN_ATTR_FREQUENCY_CENTER_STEP
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:PRESet[:STANdard] &lt;file_name&gt; | GSM | EDGE | FW3Gppcdma | RW3Gppcdma | FQCDma | F95Cdma | RQCDma | R95Cdma | CDMa2000 | F1CDma2000 | R1CDma2000 | FNADc | RNADc | PDCDown | PDCup | PHS | TETRa | DECT | DCS1800 | PCS1900
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer:STEP:AUTO ON | OFF
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer:STEP
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Standard">
        /// Selects an automatic setting of all modulation parameters according to a standardized transmission method or a user-defined transmission method. The predefined transmission methods are either set via a value (simple string) or via a file name (strings must always be entered in single quotation marks ('...'). User-defined transmission methods can only be set via a file name (single quotes are required, for e.g. '3G_WCDMA_FWD').
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: "GSM"
        /// 
        /// Attribte:
        /// RSSPECAN_ATTR_VSA_DIGITAL_STANDARD_SAVE
        /// 
        /// The following predefined standards are available (must be passed as simple string without the single quotes):
        /// 
        /// File name     Description                         Value
        /// ------------------------------------------------------------
        /// GSM_NB        GSM, Normal Burst                   GSM
        /// GSM_SB        GSM, Synchronization Burst          -
        /// GSM_FB        GSM, Frequency Correction Burst     -
        /// GSM_AB        GSM, Access Burst                   -
        /// EDGE_NB       Edge                                EDGE
        /// 3G_WCDMA_FWD  3GPP Wcdma forward                  FW3G
        /// 3G_WCDMA_REV  3GPP Wcdma reverse                  RW3G
        /// CDMAONE_FWD   CdmaOne for.according to st.IS95    FQCD
        /// CDMAONE_REV   CdmaOne rev.according to st.IS95    RQCD
        /// CDMA2K_1X_FWD Cdma 2000 forward, spreading rate 1 F1CDma2000
        /// CDMA2K_1X_REV Cdma 2000 reverse, spreading rate 1 R1CDma2000
        /// IS95_FWD      IS95 forward according to st.IS95   F95Cdma
        /// IS95_REV      IS95 reverse according to st.IS95   R95Cdma
        /// NADC_FWD      Nadc forward                        FNADc
        /// NADC_REV      Nadc reverse                        RNADc
        /// PDC_DOWN      Pdc down                            PDCDown
        /// PDC_UP        Pdc up                              PDCup
        /// PHS_COMM      PHS (data burst)                    PHS
        /// PHS_CTRL      PHS (control burst)                 -
        /// TETRA_NDDOWN  Tetra (data burst)                  TETRa
        /// TETRA_NCDOWN  Tetra (control burst)               -
        /// DECT_FP       DECT                                DECT
        /// GSM_NB        DCS 1800                            DCS1800
        /// GSM_NB        PCS 1900                            PCS1900
        /// BLUETOOTH_DH1 Bluetooth, High data rate, slot   1 -
        /// BLUETOOTH_DH3 Bluetooth, High data rate, slot   3 -
        /// BLUETOOTH_DH5 Bluetooth, High data rate, slot   5 -
        /// APCO25CQPSK                                       APCO25CQPSK
        /// </param>
        /// <param name="Step_Width_Auto">
        /// Links the step width to the current standard (ON) or sets the step width entered With the FREQ:CENT:STEP command (OFF). The driver uses this value to set the RSSPECAN_ATTR_VSA_FREQUENCY_STEP_AUTO attribute.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_FREQUENCY_STEP_AUTO
        /// </param>
        /// <param name="Step_Size">
        /// Sets the step width of the center frequency. 
        /// 
        /// Units: Hertz
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_CENTER_STEP
        /// 
        /// Note:
        /// (1) This control has effect only if Step Width Auto is set Off.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSADigitalStandard(int Window, string Standard, bool Step_Width_Auto, double Step_Size)
        {
            int pInvokeResult = PInvoke.ConfigureVSADigitalStandard(this._handle, Window, Standard, System.Convert.ToUInt16(Step_Width_Auto), Step_Size);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function stores the current settings of the vector signal analysis as a new user-defined digital standard. If the name of the digital standard is already in use, an error message is output and a new name has to be selected. It is recommended to define a group, a comment and a prefix before storing the standard.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_DIGITAL_STANDARD_SAVE
        /// RSSPECAN_ATTR_VSA_DIGITAL_STANDARD_GROUP
        /// RSSPECAN_ATTR_VSA_DIGITAL_STANDARD_COMMENT
        /// RSSPECAN_ATTR_VSA_DIGITAL_STANDARD_PREFIX
        /// 
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:STANdard:SAVE
        /// [SENSe&lt;1|2&gt;:]DDEMod:STANdard:GROup &lt;string&gt;
        /// [SENSe&lt;1|2&gt;:]DDEMod:STANdard:COMMent
        /// [SENSe&lt;1|2&gt;:]DDEMod:STANdard:PREfix
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="User_Defined_Standard">
        /// Define name of the user-defined standard. If the name of the digital standard is already in use, an error message is output and a new name has to be selected.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_DIGITAL_STANDARD_SAVE
        /// 
        /// </param>
        /// <param name="Group">
        /// Enters the group for a new standard. The group is stored with the standard and is only displayed in the selection menu (manual operation). When remote control is used, the string is deleted after the standard has been stored, allowing a new group to be entered for the next standard.
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_DIGITAL_STANDARD_GROUP
        /// 
        /// Note(s):
        /// 
        /// (1) This control is not available for R&amp;S FSV.
        /// </param>
        /// <param name="Comment">
        /// Enters the comment for a new standard. The comment is stored with the standard and is only displayed in the selection menu (manual operation). When remote control is used, the string is deleted after the standard has been stored, allowing a new comment to be entered for the next standard.
        ///  
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_DIGITAL_STANDARD_COMMENT
        /// </param>
        /// <param name="Prefix">
        /// Defines the prefix of the sync pattern for a standard. When a sync pattern for the standard is selected later, the patterns containing this string at the start are offered with priority in the selection table (only relevant to manual operation). When remote control is used, the string is deleted after the standard has been stored, allowing a new prefix to be entered for the next standard.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_DIGITAL_STANDARD_PREFIX
        /// 
        /// Note(s):
        /// 
        /// (1) This control is not available for R&amp;S FSV.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int VSADigitalStandardSave(int Window, string User_Defined_Standard, string Group, string Comment, string Prefix)
        {
            int pInvokeResult = PInvoke.VSADigitalStandardSave(this._handle, Window, User_Defined_Standard, Group, Comment, Prefix);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function deletes the selected digital standard if it is a user-defined standard. If the standard is predefined, it cannot be deleted (error message).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_DIGITAL_STANDARD_DELETE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:STANdard:DELete &lt;file_name&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Digital_Standard">
        /// Select digital standard if it is a user-defined standard. If the standard is predefined, it cannot be deleted (error message).
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_DIGITAL_STANDARD_DELETE
        /// 
        /// The following predefined standards are available:
        /// 
        /// Value             Description
        /// 
        /// GSM               GSM, normal burst
        /// SGSM              GSM, syncronisation burst
        /// FGSM              GSM, frequency correction burst
        /// AGSM              GSM, access burst
        /// EDGE              Edge
        /// FW3Gppcdma        3GPP Wcdma forward
        /// RW3Gppcdma        3GPP Wcdma reverse
        /// FQCDma or F95Cdma Cdma One forward according to IS95 Standard
        /// RQCDma or R95Cdma Cdma One reverse according to IS95 Standard
        /// F1CDma2000        Cdma 2000 forward, spreading rate 1
        /// R1CDma2000        Cdma 2000 reverse, spreading rate 1
        /// R3CDma2000        Cdma 2000 reverse, spreading rate 3
        /// FNADc             Nadc forward
        /// RNADc             Nadc reverse
        /// PDCDown           Pdc down
        /// PDCup             Pdc up
        /// PHS               Phs
        /// TETRa             Tetra
        /// CTETra            Tetra
        /// DECT              DECT
        /// H1BLuetooth       Bluetooth, High data rate, slot length 1
        /// H3BLuetooth       Bluetooth, High data rate, slot length 3
        /// H5BLuetooth       Bluetooth, High data rate, slot length 5
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int VSADigitalStandardDelete(int Window, string Digital_Standard)
        {
            int pInvokeResult = PInvoke.VSADigitalStandardDelete(this._handle, Window, Digital_Standard);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function restores the default settings of the currently selected standard.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_DIGITAL_STANDARD_DEFAULTS
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:STANdard:PREset
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int VSADigitalStandardDefaults(int Window)
        {
            int pInvokeResult = PInvoke.VSADigitalStandardDefaults(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads the names of all digital standards available.
        /// 
        /// Notes:
        /// 
        /// (1) Driver returns an instrument specific error, when catalog list is empty.
        /// 
        /// (2) This function is not available for R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:STANdard:CATalog?
        /// *OPC?
        /// *CLS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Number_of_Digital_Standards">
        /// This control returns number of the names of all digital standards available.
        /// 
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the string you specify for the catalog list parameter.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 0
        /// 
        /// </param>
        /// <param name="Digital_Standards_List">
        /// This control returns array of the names of all digital standards available in CSV form.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// if you pass 0 for buffer Size, you can use VI_NULL for this parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetVSADigitalStandardCatalog(int Window, out int Number_of_Digital_Standards, int Buffer_Size, System.Text.StringBuilder Digital_Standards_List)
        {
            int pInvokeResult = PInvoke.GetVSADigitalStandardCatalog(this._handle, Window, out Number_of_Digital_Standards, Buffer_Size, Digital_Standards_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function links the burst search to the type of signal. When a signal is marked as bursted, burst search is switched on automatically.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only on R&amp;S FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_AUTO
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:BURSt:AUTO AUTO | MAN
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Burst_Search">
        /// Enables or disables burst searches. If "Auto" is selected, burst search is enabled only if "Bursted Signal" is selected.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_AUTO (0) - Auto
        /// RSSPECAN_VAL_MAN  (1) - Manual
        /// 
        /// Default Value: RSSPECAN_VAL_AUTO (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_AUTO
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSABurstSearchAuto(int Window, int Burst_Search)
        {
            int pInvokeResult = PInvoke.ConfigureVSABurstSearchAuto(this._handle, Window, Burst_Search);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the search for a signal burst.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_STATE
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_MODE
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_THRESHOLD_AUTO
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_THRESHOLD_MODE
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_THRESHOLD_LEVEL
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:BURSt[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:BURSt:MODE MEAS | BURSt
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:BURSt:THReshold:AUTO ON | OFF
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:BURSt:THReshold:MODE RELative | ABSolute
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:BURSt:THReshold[:LEVel]
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Search_State">
        /// Switches the search for a signal burst on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_STATE
        /// </param>
        /// <param name="Burst_Search_Mode">
        /// Switches the vector analyzer so that a measurement is performed only if a burst is found (BURSt). The driver uses this value to set the RSSPECAN_ATTR_VSA_BURST_SEARCH_MODE attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VSA_BURS_MODE_MEAS (0) - Meas
        /// RSSPECAN_VAL_VSA_BURS_MODE_BURS (1) - Burs
        /// 
        /// Default Value: RSSPECAN_VAL_VSA_BURS_MODE_BURS (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_MODE
        /// 
        /// Note(s):
        /// This control is valid only if Search State is set On.
        /// 
        /// 
        /// </param>
        /// <param name="Threshold_Auto">
        /// Defines which value is to be used as the threshold. ON corresponds to a relative threshold value defined by the standard.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_THRESHOLD_AUTO
        /// 
        /// Note(s):
        /// 
        /// (1) This control is not available for R&amp;S FSV.
        /// </param>
        /// <param name="Threshold_Mode">
        /// Sets the threshold mode. The driver uses this value to set the RSSPECAN_ATTR_VSA_BURST_SEARCH_THRESHOLD_MODE attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ABS (0) - Absolute
        /// RSSPECAN_VAL_REL (1) - Relative
        /// 
        /// Default Value: RSSPECAN_VAL_REL (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_THRESHOLD_MODE
        /// 
        /// Note(s):
        /// 
        /// (1) This control has effext only if Threshold Auto is set Off
        /// 
        /// (2) This control is not available for R&amp;S FSV.
        /// </param>
        /// <param name="Threshold_Level">
        /// Sets the threshold for the burst search.. The driver uses this value to set the RSSPECAN_ATTR_VSA_BURST_SEARCH_THRESHOLD_LEVEL attribute.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_THRESHOLD_LEVEL
        /// 
        /// Note(s):
        /// 
        /// (1) This control has effect only if Threshold Auto is set Off
        /// 
        /// (2) This control is not available for R&amp;S FSV.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSABurstSearch(int Window, bool Search_State, int Burst_Search_Mode, bool Threshold_Auto, int Threshold_Mode, double Threshold_Level)
        {
            int pInvokeResult = PInvoke.ConfigureVSABurstSearch(this._handle, Window, System.Convert.ToUInt16(Search_State), Burst_Search_Mode, System.Convert.ToUInt16(Threshold_Auto), Threshold_Mode, Threshold_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the manual hysteresis level for a signal burst search.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is not available for R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_HYSTERESIS_LEVEL
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:BURSt:HYSTeresis[:LEVel]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Hysteresis_Level">
        /// Sets the trigger hysteresis to ensure that the end of the burst is reliably.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_HYSTERESIS_LEVEL
        /// 
        /// Note:
        /// (1) This control has effext only if Hysteresis Auto is set Off
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSABurstSearchHysteresis(int Window, double Hysteresis_Level)
        {
            int pInvokeResult = PInvoke.ConfigureVSABurstSearchHysteresis(this._handle, Window, Hysteresis_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the settings that are
        /// used to accurately control the behaviour of the burst search even under difficult measurement conditions.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_LENGTH_MIN
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_LENGTH_MAX
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_SKIP_RISE
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_SKIP_FALL
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_GLEN
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:BURSt:LENGth[:MINimum]
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:BURSt:LENGth[:MAXimum]
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:BURSt:SKIP:RISing
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:BURSt:SKIP:FALLing
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:BURSt:GLENgth[:MINimum]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Useful_Length">
        /// Sets the minimum length of a burst.
        /// 
        /// Units: Symbol
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 16
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_LENGTH_MIN
        /// 
        /// </param>
        /// <param name="Max_Length">
        /// Sets the maximum length of a burst. Only those bursts will be recognized that fall below this length.
        /// 
        /// Units: Symbol
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 160
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_LENGTH_MAX
        /// </param>
        /// <param name="Run_In">
        /// Sets the length of the rising burst edge which was not considered when evaluating the result.
        /// 
        /// Units: Symbol
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_SKIP_RISE
        /// </param>
        /// <param name="Run_Out">
        /// Sets the length of the falling burst edge which is not considered when evaluating the result.
        /// 
        /// Units: Symbol
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_SKIP_FALL
        /// </param>
        /// <param name="Min_Gaplen">
        /// Sets the minimum time between two bursts. A minimum time with decreased level must occur between two bursts. A new burst is recognized only after this time.
        /// 
        /// Units: Symbol
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 2
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_GLEN
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSABurstSearchExpertSettings(int Window, int Useful_Length, int Max_Length, int Run_In, int Run_Out, int Min_Gaplen)
        {
            int pInvokeResult = PInvoke.ConfigureVSABurstSearchExpertSettings(this._handle, Window, Useful_Length, Max_Length, Run_In, Run_Out, Min_Gaplen);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configure the search tolerance and the min gap length and their default values.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only on R&amp;S FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_SEARCH_BURST_AUTO_CONFIGURATION
        /// RSSPECAN_ATTR_VSA_SEARCH_BURST_TOLERANCE
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_GLEN
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:BURSt:CONFigure:AUTO ON | OFF
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:BURSt:TOLerance
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:BURSt:GLENgth[:MINimum]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Auto_Configuration">
        /// Enables or disables burst searches. If switched ON, burst search is enabled only if "Bursted Signal" is selected.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_SEARCH_BURST_AUTO_CONFIGURATION
        /// </param>
        /// <param name="Tolerance">
        /// Sets burst search tolerance.
        /// 
        /// Units: Symbol
        /// 
        /// Valid Values: 0 to 100000
        /// 
        /// Default Value: 4
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_SEARCH_BURST_TOLERANCE
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available if Auto Configuration is set to Off.
        /// </param>
        /// <param name="Min_Gaplen">
        /// Sets the minimum time between two bursts. A minimum time with decreased level must occur between two bursts. A new burst is recognized only after this time.
        /// 
        /// Units: Symbol
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 2
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_GLEN
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available if Auto Configuration is set to Off.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSABurstSearchAutoConfiguration(int Window, bool Auto_Configuration, int Tolerance, int Min_Gaplen)
        {
            int pInvokeResult = PInvoke.ConfigureVSABurstSearchAutoConfiguration(this._handle, Window, System.Convert.ToUInt16(Auto_Configuration), Tolerance, Min_Gaplen);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries whether burst is available in the signal. 
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// STATus:QUEStionable:SYNC:CONDition?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Found">
        /// Returns the burst found result. 
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetVSABurstFound(out bool Found)
        {
            ushort FoundAsUShort;
            int pInvokeResult = PInvoke.GetVSABurstFound(this._handle, out FoundAsUShort);
            Found = System.Convert.ToBoolean(FoundAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function links the pattern search to the type of signal. When a signal is marked as patterned, pattern search is switched on automatically.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only on R&amp;S FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_SYNC_SEARCH_AUTO
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:SYNC:AUTO AUTO | MAN
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Pattern_Search">
        /// Enables or disables pattern searches. If switched ON, pattern search is enabled automatically if "Pattern" is selected.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_AUTO (0) - Auto
        /// RSSPECAN_VAL_MAN  (1) - Manual
        /// 
        /// Default Value: RSSPECAN_VAL_AUTO (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_SYNC_SEARCH_AUTO
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAPatternSearchAuto(int Window, int Pattern_Search)
        {
            int pInvokeResult = PInvoke.ConfigureVSAPatternSearchAuto(this._handle, Window, Pattern_Search);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the search for a signal burst.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_SYNC_SEARCH_STATE
        /// RSSPECAN_ATTR_VSA_SYNC_SEARCH_MODE
        /// -
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:SYNC[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:SYNC:MODE MEAS | SYNC
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:SYNC:SELect &lt;string&gt;,&lt;string&gt;,...
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Search_State">
        /// Switches the search for a signal burst on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_SYNC_SEARCH_STATE
        /// </param>
        /// <param name="Pattern_Search_Mode">
        /// Switches the vector analyzer so that a measurement is performed only if a burst is found (BURSt).
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VSA_SYNC_MODE_MEAS (0) - Meas
        /// RSSPECAN_VAL_VSA_SYNC_MODE_SYNC (1) - Sync
        /// 
        /// Default Value: RSSPECAN_VAL_VSA_SYNC_MODE_SYNC (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_BURST_SEARCH_MODE
        /// 
        /// Note(s):
        /// This control is valid only if Search State is set On.
        /// 
        /// 
        /// </param>
        /// <param name="Pattern">
        /// Sets a predefined sync pattern file. Up to 16 sync pattern files may be given.
        /// 
        /// Valid Values: any string
        /// 
        /// Default Value: ""
        /// 
        /// Note:
        /// (1) String format is "PAT_GSM_1,PAT_GSM_5,PAT_GSM_8"
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAPatternSearch(int Window, bool Search_State, int Pattern_Search_Mode, string Pattern)
        {
            int pInvokeResult = PInvoke.ConfigureVSAPatternSearch(this._handle, Window, System.Convert.ToUInt16(Search_State), Pattern_Search_Mode, Pattern);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the selected pattern.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_SYNC_NAME
        /// RSSPECAN_ATTR_VSA_SYNC_TEXT
        /// RSSPECAN_ATTR_VSA_SYNC_COMMENT
        /// RSSPECAN_ATTR_VSA_SYNC_NSTAT
        /// -
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:SYNC:NAME
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:SYNC:TEXT
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:SYNC:COMMent
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:SYNC:NSTate
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:SYNC:DATA
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Pattern_Name">
        /// This control selects a predefined channel table file.
        /// 
        /// Valid Values: up to 8 characters
        /// 
        /// Default Value: "RECENT"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_SYNC_NAME
        /// </param>
        /// <param name="Description">
        /// Sets a text to explain the pattern. The text is displayed only in the selection menu (manual control). This text is supposed to be short and concise. Detailed information about the pattern is given in the comment.
        /// 
        /// Valid Values:
        /// any string
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_SYNC_TEXT
        /// </param>
        /// <param name="Comment">
        /// Defines a comment to a sync pattern. 
        /// 
        /// Valid Values:
        /// any string
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_SYNC_COMMENT
        /// </param>
        /// <param name="Degree_of_Modulation">
        /// Sets the degree of modulation (number of permitted states).
        /// 
        /// Valid Values:
        /// 2 | 4 | 8 | 16 | 32 | 64 | 128 | 256 | 512 | 1024
        /// 
        /// Default Value: 2
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_SYNC_NSTATE
        /// </param>
        /// <param name="Pattern_Data">
        /// Sets a sync sequence for the sync pattern. Four values represent a symbol. The value range of a symbol depends on the degree of modulation. FFFF entries are interpreted as "Don't Care Bits".
        /// 
        /// Valid Values:
        /// any string
        /// 
        /// Default Value: ""
        /// 
        /// Note:
        /// (1) With a degree of modulation of 4, all symbols have a value range of: 0000, 0001, 0002, 0003; with a degree of modulation of 8: 0000, 0001, 0002, 0003, 0004, 0005, 0006, 0007
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAPatternSettings(int Window, string Pattern_Name, string Description, string Comment, int Degree_of_Modulation, string Pattern_Data)
        {
            int pInvokeResult = PInvoke.ConfigureVSAPatternSettings(this._handle, Window, Pattern_Name, Description, Comment, Degree_of_Modulation, Pattern_Data);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function performs the selected operation with channel table.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_SYNC_PATT_ADD
        /// RSSPECAN_ATTR_VSA_SYNC_NAME
        /// RSSPECAN_ATTR_VSA_SYNC_DELETE
        /// RSSPECAN_ATTR_VSA_SYNC_PATT_REMOVE
        /// RSSPECAN_ATTR_VSA_SYNC_PATT_REMOVE_ALL
        /// RSSPECAN_ATTR_VSA_SYNC_COPY
        /// 
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:SYNC:PATTern:ADD
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:SYNC:NAME
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:SYNC:DELete
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:SYNC:PATTern:REMove &lt;string&gt;
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:SYNC:PATTern:REMove ALL
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:SYNC:COPY
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Operation">
        /// This control selects the type of memory operation to be performed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SYNC_PATT_ADD        (0) - Add Pattern
        /// RSSPECAN_VAL_SYNC_PATT_SELECT     (1) - Select Pattern
        /// RSSPECAN_VAL_SYNC_PATT_DELETE     (2) - Delete Pattern 
        /// RSSPECAN_VAL_SYNC_PATT_REMOVE     (3) - Remove Pattern
        /// RSSPECAN_VAL_SYNC_PATT_REMOVE_ALL (4) - Remove Pattern From Standard
        /// RSSPECAN_VAL_SYNC_PATT_COPY       (5) - Copy Pattern
        /// 
        /// 
        /// Notes:
        /// 
        /// (1) Add Pattern
        /// Adds a pattern to the current standard. With the DDEM:SEAR:SEL command, only those patterns can be selected which belong to the current standard.
        /// 
        /// (2) Select Pattern:
        /// Selects a sync pattern for editing or for a new entry.
        /// 
        /// (3) Delete Pattern
        /// deletes a sync sequence. The sync sequence to be deleted must have been selected before with the DDEM:SEARch:SYNC:NAME command.
        /// 
        /// (4) Remove Pattern From Standard:
        /// Deletes the selected channel table.
        /// 
        /// (5) Copy Pattern
        /// Copies active channel table onto another one.
        /// </param>
        /// <param name="Pattern_Name">
        /// This control selects a predefined channel table file.
        /// 
        /// Valid Values: up to 8 characters
        /// 
        /// Default Value: "RECENT"
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_SYNC_PATT_ADD
        /// RSSPECAN_ATTR_VSA_SYNC_NAME
        /// RSSPECAN_ATTR_VSA_SYNC_DELETE
        /// RSSPECAN_ATTR_VSA_SYNC_PATT_REMOVE
        /// RSSPECAN_ATTR_VSA_SYNC_PATT_REMOVE_ALL
        /// RSSPECAN_ATTR_VSA_SYNC_COPY
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAPatternOperations(int Window, int Operation, string Pattern_Name)
        {
            int pInvokeResult = PInvoke.ConfigureVSAPatternOperations(this._handle, Window, Operation, Pattern_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the pattern search and the IQ Correlation Threshold setting.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only on R&amp;S FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_SYNC_SEARCH_PATTERN_AUTO_CONFIGURATION
        /// RSSPECAN_ATTR_VSA_SYNC_SEARCH_IQ_CORRELATION_THRESHOLD
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:PATTern:CONFiguration:AUTO ON | OFF
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:SYNC:IQCThreshold
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Auto_Configuration">
        /// Sets the IQ correlation threshold to its default value.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_SYNC_SEARCH_PATTERN_AUTO_CONFIGURATION
        /// </param>
        /// <param name="IQ_Correlation_Threshold">
        /// Sets the IQ correlation threshold for pattern matching in percent. A high level means stricter matching.
        /// 
        /// Valid Values: 10.0 to 100.0 %
        /// 
        /// Default Value: 90.0 %
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_SYNC_SEARCH_IQ_CORRELATION_THRESHOLD
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available if Auto Configuration is set to Off.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAPatternSearchAutoConfiguration(int Window, bool Auto_Configuration, double IQ_Correlation_Threshold)
        {
            int pInvokeResult = PInvoke.ConfigureVSAPatternSearchAutoConfiguration(this._handle, Window, System.Convert.ToUInt16(Auto_Configuration), IQ_Correlation_Threshold);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries whether one of the selected sync patterns (DDEM:SEAR:SYNC:SEL) is available in the signal. Up to 16 sync pattern files can be selected.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:SYNC:FOUNd?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the string you specify for the found parameter.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 1024
        /// </param>
        /// <param name="Found">
        /// Returns the sync pattern found result. 
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetVSAPatternFound(int Window, int Buffer_Size, System.Text.StringBuilder Found)
        {
            int pInvokeResult = PInvoke.GetVSAPatternFound(this._handle, Window, Buffer_Size, Found);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads the names of all patterns stored on the hard disk.
        /// 
        /// Notes:
        /// 
        /// (1) Driver returns an instrument specific error, when catalog list is empty.
        /// 
        /// (2) This function is available only in conjunction with option FS-K70.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:SYNC:CATalog? CURRent | ALL
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Selection">
        /// Sets the pattern selection.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VSA_PATTERN_ALL     (0) - All
        /// RSSPECAN_VAL_VSA_PATTERN_CURRENT (1) - Current
        /// 
        /// Default Value: RSSPECAN_VAL_VSA_PATTERN_ALL     (0)
        /// 
        /// Note(s):
        /// (1) All - Reading out all patterns available on hard disk
        /// Current - Reading out patterns that belong to the current standard
        /// 
        /// </param>
        /// <param name="Number_of_Patterns">
        /// This control returns number of the names of all digital standards available.
        /// 
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the string you specify for the catalog list parameter.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 0
        /// 
        /// 
        /// </param>
        /// <param name="Patterns_List">
        /// This control returns array of the names of all digital standards available in CSV form.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// if you pass 0 for buffer Size, you can use VI_NULL for this parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetVSAPatternCatalog(int Window, int Selection, out int Number_of_Patterns, int Buffer_Size, System.Text.StringBuilder Patterns_List)
        {
            int pInvokeResult = PInvoke.GetVSAPatternCatalog(this._handle, Window, Selection, out Number_of_Patterns, Buffer_Size, Patterns_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the parameters of an adaptive equalizer filter.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is not available for R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_EQUALIZER_SPEED_CONVERSION
        /// RSSPECAN_ATTR_VSA_EQUALIZER_LENGTH
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:EQUalize:CNVRange 0..1
        /// [SENSe&lt;1|2&gt;:]DDEMod:EQUalize:LENGth 1..100
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Speed_of_Conversion">
        /// Defines the speed of conversion of the adaptive filter . 
        /// 
        /// Valid Values:
        /// 0.0 to 1.0
        /// 
        /// Default Value: 0.01
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_EQUALIZER_SPEED_CONVERSION
        /// </param>
        /// <param name="Length">
        /// Defines the length of the equalizer.
        /// 
        /// Units: Symbols
        /// 
        /// Valid Values: 1 to 100
        /// 
        /// Default Value: 20
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_EQUALIZER_LENGTH
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAEqualizer(int Window, double Speed_of_Conversion, int Length)
        {
            int pInvokeResult = PInvoke.ConfigureVSAEqualizer(this._handle, Window, Speed_of_Conversion, Length);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the mode of an adaptive equalizer filter.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is not available for R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_EQUALIZER_STATE
        /// RSSPECAN_ATTR_VSA_EQUALIZER_LEARNING
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:EQUalize[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]DDEMod:EQUalize:ADAPt ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// Switches the equalizer in or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_EQUALIZER_STATE
        /// </param>
        /// <param name="Training">
        /// Switches the learning phase of the equalizer on or off. Each sweep between commands DDEM:EQU ON and DDEM:EQU OFF are used for the calculation of the adaptive filter.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_EQUALIZER_LEARNING
        /// 
        /// Note(s):
        /// 
        /// (1) Off - Freeze
        /// 
        /// (2) On - Training
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAEqualizerMode(int Window, bool State, bool Training)
        {
            int pInvokeResult = PInvoke.ConfigureVSAEqualizerMode(this._handle, Window, System.Convert.ToUInt16(State), System.Convert.ToUInt16(Training));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function performs the selected operation with equalizer.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is not available for R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_EQUALIZER_LOAD
        /// RSSPECAN_ATTR_VSA_EQUALIZER_SAVE
        /// RSSPECAN_ATTR_VSA_EQUALIZER_DELETE
        /// RSSPECAN_ATTR_VSA_EQUALIZER_RESET
        /// 
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:EQUalize:LOAD &lt;Equalizer_Name&gt;
        /// [SENSe&lt;1|2&gt;:]DDEMod:EQUalize:SAVE &lt;Equalizer_Name&gt;
        /// [SENSe&lt;1|2&gt;:]DDEMod:EQUalize:DELete &lt;Equalizer_Name&gt;
        /// [SENSe&lt;1|2&gt;:]DDEMod:EQUalize:RESet
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Operation">
        /// This control selects the type of operation to be performed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VSA_EQUA_LOAD   (0) - Load
        /// RSSPECAN_VAL_VSA_EQUA_SAVE   (1) - Save
        /// RSSPECAN_VAL_VSA_EQUA_DELETE (2) - Delete
        /// RSSPECAN_VAL_VSA_EQUA_RESET  (3) - Reset
        /// 
        /// 
        /// Notes:
        /// 
        /// (1) Load:
        /// Loads a previously stored equalizer. .
        /// 
        /// (2) Save:
        /// Stores the equalizer.
        /// 
        /// (3) Delete:
        /// Deletes a previously stored equalizer.
        /// 
        /// (4) Reset:
        /// Deletes the data of the currently selected equalizer. . After deletion, a new adaptive filter can be calculated using command EQU:ADAP ON.
        /// </param>
        /// <param name="Equalizer_Name">
        /// This control selects a equalizer name
        /// 
        /// Valid Values: up to 8 characters
        /// 
        /// Default Value: ""
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_EQUALIZER_LOAD
        /// RSSPECAN_ATTR_VSA_EQUALIZER_SAVE
        /// RSSPECAN_ATTR_VSA_EQUALIZER_DELETE
        /// RSSPECAN_ATTR_VSA_EQUALIZER_RESET
        /// 
        /// Note(s):
        /// (1) This control has no effect for Operation Reset.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int VSAEqualizerOperations(int Window, int Operation, string Equalizer_Name)
        {
            int pInvokeResult = PInvoke.VSAEqualizerOperations(this._handle, Window, Operation, Equalizer_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the instrument to look for a bursted or a continuous signal.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only on R&amp;S FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_SIGNAL_TYPE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:SIGNal[:VALue] CONTinuous | BURSted
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Signal_Type">
        /// This control sets the instrument to look for a bursted or a continuous signal.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VSA_SIGNAL_TYPE_CONT  (0) - Continuous
        /// RSSPECAN_VAL_VSA_SIGNAL_TYPE_BURST (1) - Burst Signal
        /// 
        /// Default Value: RSSPECAN_VAL_VSA_SIGNAL_TYPE_CONT
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_SIGNAL_TYPE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSASignalType(int Window, int Signal_Type)
        {
            int pInvokeResult = PInvoke.ConfigureVSASignalType(this._handle, Window, Signal_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the instrument to look for a pattern.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only on R&amp;S FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_PATTERN_SIGNAL_STATE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:SIGNal:PATTern ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Pattern_Signal">
        /// Sets the instrument to look for a pattern.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_PATTERN_SIGNAL_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAPatternSignalState(int Window, bool Pattern_Signal)
        {
            int pInvokeResult = PInvoke.ConfigureVSAPatternSignalState(this._handle, Window, System.Convert.ToUInt16(Pattern_Signal));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the pattern offset. If the position of
        /// the pattern within the burst is known, it is recommended that you define the offset. That will accelerate the pattern search and enhance the accuracy of the burst search.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only on R&amp;S FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_PATTERN_OFFSET_STATE
        /// RSSPECAN_ATTR_VSA_FIT_SYNC_OFFSET
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:STANdard:SYNC:OFFSet:STATe ON | OFF
        /// [SENSe&lt;1|2&gt;:]DDEMod:STANdard:SYNC:OFFSet[:VALue]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Pattern_Offset_State">
        /// This control (de)activates the pattern offset.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_PATTERN_OFFSET_STATE
        /// </param>
        /// <param name="Pattern_Offset">
        /// Defines a number of symbols which are ignored before the comparison with the pattern starts.
        /// 
        /// Units: Symbols
        /// 
        /// Valid Values: no checking
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_FIT_SYNC_OFFSET
        /// 
        /// Note(s): 
        /// 
        /// (1) This control is only available if Pattern Offset State is set to On.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAPatternOffset(int Window, bool Pattern_Offset_State, int Pattern_Offset)
        {
            int pInvokeResult = PInvoke.ConfigureVSAPatternOffset(this._handle, Window, System.Convert.ToUInt16(Pattern_Offset_State), Pattern_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the modulation.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_MODULATION_SRATE
        /// RSSPECAN_ATTR_VSA_MODULATION_FORMAT
        /// RSSPECAN_ATTR_VSA_MODULATION_QPSK_FORMAT
        /// RSSPECAN_ATTR_VSA_MODULATION_PSK_FORMAT
        /// RSSPECAN_ATTR_VSA_MODULATION_MSK_FORMAT
        /// RSSPECAN_ATTR_VSA_MODULATION_PSK_NSTAT
        /// RSSPECAN_ATTR_VSA_MODULATION_QAM_FORMAT
        /// RSSPECAN_ATTR_VSA_MODULATION_QAM_NSTAT
        /// RSSPECAN_ATTR_VSA_MODULATION_FSK_NSTAT
        /// RSSPECAN_ATTR_VSA_MODULATION_VSB_NSTAT
        /// RSSPECAN_ATTR_VSA_MODULATION_MAPPING
        /// RSSPECAN_ATTR_VSA_MODULATION_FILTER_STATE
        /// RSSPECAN_ATTR_VSA_MODULATION_ALPHABT
        /// RSSPECAN_ATTR_VSA_MODULATION_PRATE
        /// RSSPECAN_ATTR_VSA_MODULATION_UQAM_MAPPING
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:SRATe
        /// [SENSe&lt;1|2&gt;:]DDEMod:FORMat QPSK | PSK | MSK |QAM | FSK | VSB | UQAM
        /// [SENSe&lt;1|2&gt;:]DDEMod:QPSK:FORMat NORMal | DIFFerential | OFFSet | DPI4
        /// [SENSe&lt;1|2&gt;:]DDEMod:PSK:FORMat NORMal | DIFFerential| N3Pi8
        /// [SENSe&lt;1|2&gt;:]DDEMod:MSK:FORMat TYPE1 | TYPE2 | NORMal | DIFFerential
        /// [SENSe&lt;1|2&gt;:]DDEMod:PSK:NSTate 2 | 8
        /// [SENSe&lt;1|2&gt;:]DDEMod:QAM:FORMat NORMal | DIFFerential
        /// [SENSe&lt;1|2&gt;:]DDEMod:QAM:NSTate 16 | 32 | 64 | 128 | 256
        /// [SENSe&lt;1|2&gt;:]DDEMod:FSK:NSTate 2 | 4
        /// [SENSe&lt;1|2&gt;:]DDEMod:VSB:NSTate 8
        /// [SENSe&lt;1|2&gt;:]DDEMod:MAPPing
        /// [SENSe&lt;1|2&gt;:]DDEMod:FILTer[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]DDEMod:FILTer:ALPHa
        /// [SENSe&lt;1|2&gt;:]DDEMod:PRATe 1 | 2 | 4 | 8 | 16
        /// [SENSe&lt;1|2&gt;:]DDEMod:UQAM:FORMat
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Symbol_Rate">
        /// Sets the symbol rate.
        /// 
        /// Units: Hz
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 1.0e+3
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_MODULATION_SRATE
        /// </param>
        /// <param name="Mode">
        /// Selects the digital demodulation mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VSA_MODULATION_FORMAT_QPSK     (0) - Quad Phase Shift Key
        /// RSSPECAN_VAL_VSA_MODULATION_FORMAT_DQPSK    (1) - Differential QPSK
        /// RSSPECAN_VAL_VSA_MODULATION_FORMAT_OQPSK    (2) - Offset QPSK
        /// RSSPECAN_VAL_VSA_MODULATION_FORMAT_DPI4QPSK (3) - PI/4 QPSK
        /// RSSPECAN_VAL_VSA_MODULATION_FORMAT_BPSK     (4) - Phase Shift Key
        /// RSSPECAN_VAL_VSA_MODULATION_FORMAT_8PSK     (5) - Normal PSK
        /// RSSPECAN_VAL_VSA_MODULATION_FORMAT_D8PSK    (6) - Differential PSK
        /// RSSPECAN_VAL_VSA_MODULATION_FORMAT_N3PI8PSK (7) - 3PI/8 - 8PSK
        /// RSSPECAN_VAL_VSA_MODULATION_FORMAT_MSK      (8) - Minimum Shift Key
        /// RSSPECAN_VAL_VSA_MODULATION_FORMAT_DMSK     (9) - Differemtial MSK
        /// RSSPECAN_VAL_VSA_MODULATION_FORMAT_16QAM    (10) - 16 QAM
        /// RSSPECAN_VAL_VSA_MODULATION_FORMAT_32QAM    (11) - 32 QAM
        /// RSSPECAN_VAL_VSA_MODULATION_FORMAT_64QAM    (12) - 64 QAM
        /// RSSPECAN_VAL_VSA_MODULATION_FORMAT_128QAM   (13) - 128 QAM
        /// RSSPECAN_VAL_VSA_MODULATION_FORMAT_256QAM   (14) - 256 QAM
        /// RSSPECAN_VAL_VSA_MODULATION_FORMAT_2FSK     (15) - Frequency Shift Key (2)
        /// RSSPECAN_VAL_VSA_MODULATION_FORMAT_4FSK     (16) - Frequency Shift Key (4)
        /// RSSPECAN_VAL_VSA_MODULATION_FORMAT_8VSB     (17) - Vestigial Sideband (8)
        /// RSSPECAN_VAL_VSA_MODULATION_FORMAT_16DQAM    (18) - 16 DQAM
        /// RSSPECAN_VAL_VSA_MODULATION_FORMAT_32DQAM    (19) - 32 DQAM
        /// RSSPECAN_VAL_VSA_MODULATION_FORMAT_64DQAM    (20) - 64 DQAM
        /// RSSPECAN_VAL_VSA_MODULATION_FORMAT_128DQAM   (21) - 128 DQAM
        /// RSSPECAN_VAL_VSA_MODULATION_FORMAT_256DQAM   (22) - 256 DQAM
        /// RSSPECAN_VAL_VSA_MODULATION_FORMAT_USERQAM   (23) - User QAM
        /// 
        /// Default Value: RSSPECAN_VAL_VSA_MODULATION_FORMAT_QPSK (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_MODULATION_FORMAT
        /// RSSPECAN_ATTR_VSA_MODULATION_QPSK_FORMAT RSSPECAN_ATTR_VSA_MODULATION_PSK_FORMAT 
        /// RSSPECAN_ATTR_VSA_MODULATION_MSK_FORMAT RSSPECAN_ATTR_VSA_MODULATION_PSK_NSTAT
        /// RSSPECAN_ATTR_VSA_MODULATION_QAM_NSTAT
        /// RSSPECAN_ATTR_VSA_MODULATION_FSK_NSTAT
        /// RSSPECAN_ATTR_VSA_MODULATION_VSB_NSTAT
        /// </param>
        /// <param name="Mapping_Name">
        /// Selects the mapping designated by Mapping Name for the digital demodulation.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_MODULATION_MAPPING
        /// </param>
        /// <param name="Signal_Filtering">
        /// Selects whether the input signal that is used to evaluate the measurement is filtered. If the filter is switched off, the input signal is only corrected in frequency and time. 
        /// 
        /// Valid Values:
        /// VI_FALSE  (0) - Off
        /// VI_TRUE   (1) - On
        /// 
        /// Default Value: VI_TRUE   (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_MODULATION_FILTER_STATE
        /// 
        /// Note:
        /// (1) This control has no effect in MSK and FSK mode since the demodulation is always carried out with unfiltered signals.
        /// 
        /// </param>
        /// <param name="Points_Symbol">
        /// Sets the number of points per symbol.
        /// 
        /// 
        /// Valid Values: 1, 2, 4, 8, 16
        /// 
        /// Default Value: 4
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_MODULATION_PRATE
        /// 
        /// Note(s):
        /// 
        /// (1) For R&amp;S FSV instrument this value is fixed to 4.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAModulation(int Window, double Symbol_Rate, int Mode, string Mapping_Name, bool Signal_Filtering, int Points_Symbol)
        {
            int pInvokeResult = PInvoke.ConfigureVSAModulation(this._handle, Window, Symbol_Rate, Mode, Mapping_Name, System.Convert.ToUInt16(Signal_Filtering), Points_Symbol);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the FSK modulation parameters
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_MODULATION_FSK_DEV_COMP
        /// RSSPECAN_ATTR_VSA_MODULATION_FSK_DEV
        /// 
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FSK:DEViation:COMPensation ON | OFF
        /// CALCulate&lt;1|2&gt;:FSK:DEViation:REFerence
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Deviation_Compensation">
        /// Selects the method for calculating the frequency error for FSK modulation.
        /// 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_MODULATION_FSK_DEV_COMP
        /// 
        /// Note: 
        /// (1) On -  Scales the reference signal to the actual deviation of the measurement signal
        /// Off -  Uses the entered nominal deviation for the reference signal
        /// </param>
        /// <param name="FSK_Deviation">
        /// Sets the reference value of the frequency deviation for FSK modulation.
        /// 
        /// Units: Hz
        /// 
        /// Valid Values: no checking
        /// 
        /// Default Value: 10.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_MODULATION_FSK_DEV
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAFSKModulation(int Window, bool Deviation_Compensation, double FSK_Deviation)
        {
            int pInvokeResult = PInvoke.ConfigureVSAFSKModulation(this._handle, Window, System.Convert.ToUInt16(Deviation_Compensation), FSK_Deviation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the EVM modulation parameters
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_MODULATION_EVM_CALCULATION
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:ECALc SYMBol | SIGNal | MACP | MECP
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Calculation">
        /// Selects the calculation formula for EVM modulation.
        /// 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VSA_MOD_EVM_SYMBOL (0) - Symbol (Maximum Ref Power)
        /// RSSPECAN_VAL_VSA_MOD_EVM_SIGNAL (1) - Signal (Mean Ref Power)
        /// RSSPECAN_VAL_VSA_MOD_EVM_MACP   (2) - Max Constellation Power
        /// RSSPECAN_VAL_VSA_MOD_EVM_MECP   (3) - Mean Constellation Power
        /// 
        /// Default Value: RSSPECAN_VAL_VSA_MOD_EVM_SYMBOL (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_MODULATION_EVM_CALCULATION
        /// 
        /// Note: 
        /// (1) Symbol -  Calculation normalized to the maximum power at
        /// symbol times.
        /// 
        /// Signal -  Calculation normalized to the average power within the
        /// measurement range.
        /// 
        /// Max Constellation Power - The maximum expected power power of the measurement signal at the symbol instants. Available on R&amp;S FSV only.
        /// 
        /// Mean Constellation Power - The mean expected power power of the measurement signal at the symbol instants. Available on R&amp;S FSV only.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAEVMModulation(int Window, int Calculation)
        {
            int pInvokeResult = PInvoke.ConfigureVSAEVMModulation(this._handle, Window, Calculation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the calculation of the error vector magnitude trace for Offset-QPSK only. It has no effect for all other modulations. It has no effect on results based on the MEAS or the REF signal.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is not available for R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_MODULATION_EVM_OFFSET
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:ECALc:OFFSet ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Calculation">
        /// Selects the calculation of the error vector magnitude trace for Offset-QPSK only.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_MODULATION_EVM_OFFSET
        /// 
        /// Note: 
        /// (1) On - The error vector magnitude is calculated at the symbol instants of the I-part and the symbol instants of the Q-part of the Offset-QPSK signal. In other words:The half symbol duration delay of the Q-part is compensated. In firmware versions prior to the introduction of this softkey, this was method was always used for Offset-QPSK.
        /// 
        /// (2) Off - The error vector magnitude is calculated at the symbol instants of the I-part and the corresponding sample of the Q-part of the signal. But the latter is not a symbol instant.
        /// In other words: The Offset-QPSK signal is treated like a QPSK signal for the error vector magnitude calculation, the Q-delay is not compensated.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAEVMOffsetState(int Window, bool Calculation)
        {
            int pInvokeResult = PInvoke.ConfigureVSAEVMOffsetState(this._handle, Window, System.Convert.ToUInt16(Calculation));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the modulation filters.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is not available for R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:FILTer:MODulation &lt;TX Filter&gt;,&lt;ISI Filter&gt;, &lt;MEAS Filter&gt;
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="TX_Filter">
        /// Select TX filter.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: "NONE"
        /// 
        /// </param>
        /// <param name="Receive_Filter">
        /// Select ISI filter.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: "NONE"
        /// 
        /// </param>
        /// <param name="Meas_Filter">
        /// Select Meas filter.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: "NONE"
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAModulationFilters(int Window, string TX_Filter, string Receive_Filter, string Meas_Filter)
        {
            int pInvokeResult = PInvoke.ConfigureVSAModulationFilters(this._handle, Window, TX_Filter, Receive_Filter, Meas_Filter);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the filter characteristic Alpha/BT.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_MODULATION_ALPHABT
        /// 
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:FILTer:ALPHa
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Alpha_BT">
        /// Sets the filter characteristic (ALPHA/BT). The resolution is 0.05.
        /// 
        /// 
        /// Valid Values: 0.20 to 1.00
        /// 
        /// Default Value: 0.5
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_MODULATION_ALPHABT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAFilterCharacteristic(int Window, double Alpha_BT)
        {
            int pInvokeResult = PInvoke.ConfigureVSAFilterCharacteristic(this._handle, Window, Alpha_BT);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads the names of all available filters.
        /// 
        /// Notes:
        /// 
        /// (1) Driver returns an instrument specific error, when catalog list is empty.
        /// 
        /// (2) This function is available only in conjunction with option FS-K70.
        /// 
        /// (3) This function is not available for R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:FILTer:CATalog?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Number_of_Filters">
        /// This control returns number of the names of all digital standards available.
        /// 
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the string you specify for the catalog list parameter.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 0
        /// </param>
        /// <param name="Modulation_Filters_List">
        /// This control returns array of the names of all digital standards available in CSV form.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// if you pass 0 for buffer Size, you can use VI_NULL for this parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetVSAModulationFilterCatalog(int Window, out int Number_of_Filters, int Buffer_Size, System.Text.StringBuilder Modulation_Filters_List)
        {
            int pInvokeResult = PInvoke.GetVSAModulationFilterCatalog(this._handle, Window, out Number_of_Filters, Buffer_Size, Modulation_Filters_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads the names of all mappings stored on the hard disk. A mapping describes the assignment of constellation points to symbols.
        /// 
        /// Notes:
        /// 
        /// (1) Driver returns an instrument specific error, when catalog list is empty.
        /// 
        /// (2) This function is available only in conjunction with option FS-K70.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:MAPPing:CATalog?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Number_of_Mappings">
        /// This control returns number of the names of all digital standards available.
        /// 
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the string you specify for the catalog list parameter.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 0
        /// 
        /// </param>
        /// <param name="Mappings_List">
        /// This control returns array of the names of all digital standards available in CSV form.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// if you pass 0 for buffer Size, you can use VI_NULL for this parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetVSAModulationMappingCatalog(int Window, out int Number_of_Mappings, int Buffer_Size, System.Text.StringBuilder Mappings_List)
        {
            int pInvokeResult = PInvoke.GetVSAModulationMappingCatalog(this._handle, Window, out Number_of_Mappings, Buffer_Size, Mappings_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the specific modulation level for FQAM.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_MODULATION_UQAM_NSTAT
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:UQAM:NSTate?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Specific_Level">
        /// This control returns the specific modulation level for FQAM.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetVSAModulationUQAMLevel(int Window, out int Specific_Level)
        {
            int pInvokeResult = PInvoke.GetVSAModulationUQAMLevel(this._handle, Window, out Specific_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the TX filter.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_TX_FILTER_TYPE
        /// RSSPECAN_ATTR_VSA_TX_FILTER_STATE
        /// RSSPECAN_ATTR_VSA_TX_FILTER_ALPHA
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:TFILter:NAME
        /// [SENSe&lt;1|2&gt;:]DDEMod:TFILter[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]DDEMod:TFILter:ALPHa
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Type">
        /// Defines the type of transmit filter.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VSA_TX_FILTER_TYPE_NONE        (0) - None
        /// RSSPECAN_VAL_VSA_TX_FILTER_TYPE_RC          (1) - RC
        /// RSSPECAN_VAL_VSA_TX_FILTER_TYPE_RRC         (2) - RRC
        /// RSSPECAN_VAL_VSA_TX_FILTER_TYPE_GAUSS       (3) - Gauss
        /// RSSPECAN_VAL_VSA_TX_FILTER_TYPE_GMSK        (4) - GMSK
        /// RSSPECAN_VAL_VSA_TX_FILTER_TYPE_LIN_GMSK    (5) - Linearized GMSK
        /// RSSPECAN_VAL_VSA_TX_FILTER_TYPE_APCO25      (6) - APCO25
        /// RSSPECAN_VAL_VSA_TX_FILTER_TYPE_CDMA2K_FWD  (7) - CDMA2000 1x Forward
        /// RSSPECAN_VAL_VSA_TX_FILTER_TYPE_CDMA2K_REV  (8) - CDMA2000 1x Reverse
        /// RSSPECAN_VAL_VSA_TX_FILTER_TYPE_RECT        (9) - Rectangular
        /// RSSPECAN_VAL_VSA_TX_FILTER_TYPE_HALF_SINE   (10) - Half Sine
        /// 
        /// Default Value: RSSPECAN_VAL_VSA_TX_FILTER_TYPE_GAUSS (3)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_TX_FILTER_TYPE
        /// RSSPECAN_ATTR_VSA_TX_FILTER_STATE
        /// 
        /// Note(s):
        /// 
        /// (1) None: No filter.
        /// 
        /// (2) RC: Raised cosine
        /// 
        /// (3) RRC: Root raised cosine
        /// 
        /// (4) Gauss: Gauss filter
        /// 
        /// (5) GMSK: Gauss filter convolved with a rectangular filter; typically used for MSK
        /// 
        /// (6) Linearized GMSK: Linearized Gauss filter, convolved with a rectangular filter; used as a transmit filter for GSM EDGE
        /// 
        /// (7) APCO25: APCO25 C4FM filter.
        /// 
        /// (8) CDMA2000 1x Forward/Reverse: Filter for CDMA ONE forward/reverse link (TIA/EIA/IS-95-A May 1995) and CDMA 2000 1x forward/reverse link
        /// 
        /// (9) Rectangular: Rectangular filter
        /// 
        /// (10) Half Sine: Half Sine filter.
        /// </param>
        /// <param name="Alpha_BT">
        /// Determines the filter characteristic (ALPHA/BT).
        /// 
        /// Valid Values:
        /// 0.1 to 1.0
        /// 
        /// Default Value: 0.22
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_TX_FILTER_ALPHA
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSATXFilter(int Window, int Type, double Alpha_BT)
        {
            int pInvokeResult = PInvoke.ConfigureVSATXFilter(this._handle, Window, Type, Alpha_BT);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the modulation.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_DEMODULATION_REC_LENGTH_AUTO
        /// RSSPECAN_ATTR_VSA_DEMODULATION_REC_LENGTH_SYM
        /// RSSPECAN_ATTR_VSA_DEMODULATION_REC_LENGTH_S
        /// RSSPECAN_ATTR_VSA_DEMODULATION_MULTIBURST_STATE
        /// RSSPECAN_ATTR_VSA_DEMODULATION_SIDE_BAND
        /// RSSPECAN_ATTR_VSA_DEMODULATION_NORMALIZE
        /// RSSPECAN_ATTR_VSA_MODULATION_PRATE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:RLENgth:AUTO ON | OFF
        /// [SENSe&lt;1|2&gt;:]DDEMod:RLENgth &lt;value&gt;SYM
        /// [SENSe&lt;1|2&gt;:]DDEMod:RLENgth &lt;value&gt;S
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:MBURst[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]DDEMod:SBANd NORMal | INVerse
        /// [SENSe&lt;1|2&gt;:]DDEMod:NORMalize ON | OFF
        /// [SENSe&lt;1|2&gt;:]DDEMod:PRATe 1 | 2 | 4 | 8 | 16
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Recording_Length">
        /// Selects whether or not is automatic adaptation of recording length used. To manually set recording length, select if Symbols or Time units are applied (automatic adaptation of recording length will be switched to off).
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VSA_DEM_RLEN_AUTO_ON  (0) - Auto On
        /// RSSPECAN_VAL_VSA_DEM_RLEN_SYMBOL   (1) - Symbols
        /// RSSPECAN_VAL_VSA_DEM_RLEN_TIME     (2) - Time
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_VSA_DEM_RLEN_AUTO_ON  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_DEMODULATION_REC_LENGTH_AUTO
        /// RSSPECAN_ATTR_VSA_DEMODULATION_REC_LENGTH_SYM RSSPECAN_ATTR_VSA_DEMODULATION_REC_LENGTH_S
        /// 
        /// </param>
        /// <param name="Record_Length">
        /// Sets defines the recording length for further processing, e.g. for burst search.
        /// 
        /// Units: symbol or s
        /// 
        /// Valid Values: no checking
        /// 
        /// Default Value: 1.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_DEMODULATION_REC_LENGTH_S
        /// RSSPECAN_ATTR_VSA_DEMODULATION_REC_LENGTH_SYM
        /// 
        /// Note:
        /// (1) This control has effect only if Recording Length is not set to Auto On.
        /// 
        /// </param>
        /// <param name="Multiburst_Search">
        /// Switches elimination of IQ offset on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_DEMODULATION_MULTIBURST_STATE
        /// 
        /// </param>
        /// <param name="Side_Band">
        /// Sets the sideband for the demodulation.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VSA_SBAND_NORM (0) - Normal
        /// RSSPECAN_VAL_VSA_SBAND_INV  (1) - Inverse
        /// 
        /// Default Value: RSSPECAN_VAL_VSA_SBAND_NORM  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_DEMODULATION_SIDE_BAND
        /// </param>
        /// <param name="Normalize">
        /// Switches the compensation of the IQ offset on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_DEMODULATION_NORMALIZE
        /// </param>
        /// <param name="Points_Symbol">
        /// Sets the number of points per symbol.
        /// 
        /// Valid Values: 1, 2, 4, 8, 16
        /// 
        /// Default Value: 4
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_MODULATION_PRATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSADemodulation(int Window, int Recording_Length, double Record_Length, bool Multiburst_Search, int Side_Band, bool Normalize, int Points_Symbol)
        {
            int pInvokeResult = PInvoke.ConfigureVSADemodulation(this._handle, Window, Recording_Length, Record_Length, System.Convert.ToUInt16(Multiburst_Search), Side_Band, System.Convert.ToUInt16(Normalize), Points_Symbol);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the tesult length od modulation
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_DEMODULATION_RESULT_LENGTH
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:TIME
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Result_Length">
        /// Sets the number of displayed symbols (result length).
        /// 
        /// Units: symbol
        /// 
        /// Valid Values: no checking
        /// 
        /// Default Value: 80
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_DEMODULATION_RESULT_LENGTH
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAResultLength(int Window, int Result_Length)
        {
            int pInvokeResult = PInvoke.ConfigureVSAResultLength(this._handle, Window, Result_Length);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the evaluation lines state
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_EVAL_LINE_STATE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:ELIN&lt;1|2&gt;:STATe ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Evaluation_Lines_State">
        /// Switches switches both evaluation lines on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_EVAL_LINE_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAEvaluationLinesState(int Window, bool Evaluation_Lines_State)
        {
            int pInvokeResult = PInvoke.ConfigureVSAEvaluationLinesState(this._handle, Window, System.Convert.ToUInt16(Evaluation_Lines_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the evaluation lines for measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_EVAL_LINE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:ELIN&lt;1|2&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Evaluation_Line">
        /// Selects the evaluation line. 
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Line_Value">
        /// Defines the position of the evaluation line in the diagram.
        /// 
        /// Units: Symbols
        /// 
        /// Valid Values: no checking
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_EVAL_LINE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAEvaluationLines(int Window, int Evaluation_Line, int Line_Value)
        {
            int pInvokeResult = PInvoke.ConfigureVSAEvaluationLines(this._handle, Window, Evaluation_Line, Line_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the use of signal path of option Bandwidth Extension FSQ-B72 for symbol rates below 25
        /// MHz.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only in conjuction with FSQ-B72 option.
        /// 
        /// (2) This function is not available for R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_DEMODULATION_WBAN_STATE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:WBANwidth:STATe ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// Activates the use of signal path of option ?Bandwidth Extension FSQ-B72? for symbol rates below 25 MHz. This option is automatically activated when using sample rates above 100 MHz (= symbol rate 25 MHz * 5 points/symbol).
        /// If control is switched on the B72 signal path is also used for sample rate below 25 MHz. The softkey is available down to symbol rate 5.1MHz (at 4 points/symbol). Reducing the symbol rate below 5.1 MHz will automatically switch off this function.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_DEMODULATION_WBAN_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAForceWBPath(int Window, bool State)
        {
            int pInvokeResult = PInvoke.ConfigureVSAForceWBPath(this._handle, Window, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the compensation during demodulation.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only on R&amp;S FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_COMPENSATE_IQ_OFFSET
        /// RSSPECAN_ATTR_VSA_COMPENSATE_IQ_IMBALANCE
        /// RSSPECAN_ATTR_VSA_COMPENSATE_AMPLITUDE_DROOP
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:NORMalize:IQOFfset ON | OFF
        /// [SENSe&lt;1|2&gt;:]DDEMod:NORMalize:IQIMbalance ON | OFF
        /// [SENSe&lt;1|2&gt;:]DDEMod:NORMalize:ADRoop ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Compensate_For">
        /// This control selects which distortion will be compensated during demodulation.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VSA_COMPENSATE_IQ_OFFSET (0) - IQ Offset
        /// RSSPECAN_VAL_VSA_COMPENSATE_IQ_IMBAL  (1) - IQ Imbalance
        /// RSSPECAN_VAL_VSA_COMPENSATE_ADROOP    (2) - Amplitude Droop
        /// 
        /// Default Value: RSSPECAN_VAL_VSA_COMPENSATE_IQ_OFFSET (0)
        /// </param>
        /// <param name="Compensation_State">
        /// Sets the state of the compensation.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_COMPENSATE_IQ_OFFSET
        /// RSSPECAN_ATTR_VSA_COMPENSATE_IQ_IMBALANCE
        /// RSSPECAN_ATTR_VSA_COMPENSATE_AMPLITUDE_DROOP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSACompensation(int Window, int Compensate_For, bool Compensation_State)
        {
            int pInvokeResult = PInvoke.ConfigureVSACompensation(this._handle, Window, Compensate_For, System.Convert.ToUInt16(Compensation_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the measurement filter.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_AUTO_MEASUREMENT_FILTER
        /// RSSPECAN_ATTR_VSA_MEASUREMENT_FILTER_TYPE
        /// RSSPECAN_ATTR_VSA_MEASUREMENT_FILTER_STATE
        /// RSSPECAN_ATTR_VSA_MEASUREMENT_FILTER_ALPHA
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:MFILter:AUTO ON | OFF
        /// [SENSe&lt;1|2&gt;:]DDEMod:MFILter:NAME
        /// [SENSe&lt;1|2&gt;:]DDEMod:MFILter[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]DDEMod:MFILter:ALPHa
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Auto_Measurement_Filter">
        /// Sets wheater the measurement filter is defined automatically depending on the TX filter (when set to On).
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_AUTO_MEASUREMENT_FILTER
        /// </param>
        /// <param name="Type">
        /// Defines the measurement filter type, if the Auto setting is not enabled.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VSA_MEAS_FILTER_TYPE_NONE       (0) - NONE
        /// RSSPECAN_VAL_VSA_MEAS_FILTER_TYPE_RRC        (1) - RRC
        /// RSSPECAN_VAL_VSA_MEAS_FILTER_TYPE_EDGE       (2) - EDGE NSR
        /// RSSPECAN_VAL_VSA_MEAS_FILTER_TYPE_APCO25     (3) - APCO25
        /// RSSPECAN_VAL_VSA_MEAS_FILTER_TYPE_CDMA2K_FWD (4) - CDMA2000 1x Forward
        /// RSSPECAN_VAL_VSA_MEAS_FILTER_TYPE_CDMA2K_REV (5) - CDMA2000 1x Reverse
        /// 
        /// Default Value: RSSPECAN_VAL_VSA_MEAS_FILTER_TYPE_RRC (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_MEASUREMENT_FILTER_TYPE
        /// RSSPECAN_ATTR_VSA_MEASUREMENT_FILTER_STATE
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available if Auto Measurement Filter is switched Off.
        /// 
        /// (2) NONE: No measurement filter is used.
        /// 
        /// (3) RRC: Root Raised Cosine Filter. The roll-off parameter Alpha/BT is set according to the TX Filter if the Auto Measurement Filter option is enabled. Otherwise it must be set manually. If the TX filter is also a Root Raised Cosine filter with the same roll-off parameter, the resulting system is inter-symbol interference free.
        /// 
        /// (4) EDGE NSR: Measurement filter required for the "EDGE, Normal Symbol Rate" standard. The resulting system is NOT inter-symbol interference free.
        /// 
        /// (5) APCO25: APCO25 C4FM filter.
        /// 
        /// (6) CDMA2000 1X Forward/Reverse: Filter for CDMA ONE forward/reverse link (TIA/EIA/IS-95-A May 1995) and CDMA 2000 1x forward/reverse link
        /// </param>
        /// <param name="Alpha_BT">
        /// Defines the roll-off factor (Alpha) or the filter bandwidth (BT).
        /// 
        /// Valid Values:
        /// 0.1 to 1.0
        /// 
        /// Default Value: 0.22
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_MEASUREMENT_FILTER_ALPHA
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available if Auto Measurement Filter is switched Off.
        /// 
        /// (2) This control is only available if Type is set to RRC.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAMeasurementFilter(int Window, bool Auto_Measurement_Filter, int Type, double Alpha_BT)
        {
            int pInvokeResult = PInvoke.ConfigureVSAMeasurementFilter(this._handle, Window, System.Convert.ToUInt16(Auto_Measurement_Filter), Type, Alpha_BT);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the default unit (symbols or seconds) for the x axis.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_UNIT_TIME
        /// 
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:X:UNIT:TIME S | SYM
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Unit">
        /// Sets the time unit.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VSA_UNIT_TIME_S   (0) - Seconds
        /// RSSPECAN_VAL_VSA_UNIT_TIME_SYM (1) - Symbols
        /// 
        /// Default Value: RSSPECAN_VAL_VSA_UNIT_TIME_S   (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_UNIT_TIME
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAUnitTime(int Window, int Unit)
        {
            int pInvokeResult = PInvoke.ConfigureVSAUnitTime(this._handle, Window, Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function the default unit for angles.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_UNIT_ANGLE
        /// 
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;: UNIT:ANGLe DEG | RAD
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Unit_Angle">
        /// Sets the default unit for angles.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_ANGLE_DEG (0) - Degree
        /// RSSPECAN_VAL_UNIT_ANGLE_RAD (1) - Radian
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_ANGLE_RAD (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_UNIT_ANGLE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAUnitAngle(int Window, int Unit_Angle)
        {
            int pInvokeResult = PInvoke.ConfigureVSAUnitAngle(this._handle, Window, Unit_Angle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the unit for power in the selected measurement window.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_UNIT_POWER
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:UNIT:POWer DBM | V | A | W | DB | PCT | UNITLESS | DBPW | WATT | DBUV | DBMV | VOLT | DBUA | AMPere | DBPT | DBUV_MHZ | DBMV_MHZ | DBUA_MHZ | DBUV_M | DBUA_M | DBUV_MMHZ | DBUA_MMHZ
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Unit_Power">
        /// Sets the unit for power in the selected measurement window.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DBM       (0) - dBm
        /// RSSPECAN_VAL_UNIT_DBPW      (1) - dBpW
        /// RSSPECAN_VAL_UNIT_WATT      (2) - W
        /// RSSPECAN_VAL_UNIT_DBUV      (3) - dBuV
        /// RSSPECAN_VAL_UNIT_DBMV      (4) - dBmV
        /// RSSPECAN_VAL_UNIT_VOLT      (5) - V
        /// RSSPECAN_VAL_UNIT_DBUA      (6) - dBuA
        /// RSSPECAN_VAL_UNIT_AMP       (7) - A
        /// RSSPECAN_VAL_UNIT_DB        (8) - dB
        /// RSSPECAN_VAL_UNIT_PCT       (13) - %
        /// RSSPECAN_VAL_UNIT_UNITLESS  (14) - Unitless
        /// RSSPECAN_VAL_UNIT_DBPT      (15) - dBpT
        /// RSSPECAN_VAL_UNIT_DBMV_MHZ  (16) - dBmV/MHz
        /// RSSPECAN_VAL_UNIT_DBUV_MHZ  (17) - dBuV/MHz
        /// RSSPECAN_VAL_UNIT_DBUA_MHZ  (18) - dBuA/MHz
        /// RSSPECAN_VAL_UNIT_DBMV_M    (19) - dBmV/M
        /// RSSPECAN_VAL_UNIT_DBUA_M    (20) - dBuA/M
        /// RSSPECAN_VAL_UNIT_DBMV_MMHZ (21) - dBmV/MMHz
        /// RSSPECAN_VAL_UNIT_DBUA_MMHZ (22) - dBuA/MMHz
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_DBM  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_UNIT_POWER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAUnitPower(int Window, int Unit_Power)
        {
            int pInvokeResult = PInvoke.ConfigureVSAUnitPower(this._handle, Window, Unit_Power);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the measurements results alignment on the display.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_FIT_ALIGN
        /// RSSPECAN_ATTR_VSA_FIT_ALIGN_VALUE
        /// RSSPECAN_ATTR_VSA_FIT_ALIGN_OFFSET
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:TRACe&lt;1 to 3&gt;:ADJust:ALIGnment LEFT | CENTer | RIGHt
        /// CALCulate&lt;1|2&gt;:TRACe&lt;1 to 3&gt;:ADJust:ALIGnment:VALue
        /// CALCulate&lt;1|2&gt;:TRACe&lt;1 to 3&gt;:ADJust:ALIGnment:OFFset
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// 1 to 1
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Alignment">
        /// Defines where the relevant event (reference point) is to appear in the diagram.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FIT_ALIGN_LEFT   (0) - Left
        /// RSSPECAN_VAL_FIT_ALIGN_CENTER (1) - Center
        /// RSSPECAN_VAL_FIT_ALIGN_RIGHT  (2) - Right
        /// 
        /// Default Value: RSSPECAN_VAL_FIT_ALIGN_CENTER (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_FIT_ALIGN
        /// 
        /// Note(s):
        /// (1) Left - The reference point is displayed at the left edge of the display.
        /// Center - The reference point is displayed in the middle of the display.
        /// Right - The reference point is displayed at the right edge of the display.
        /// </param>
        /// <param name="Align_Value">
        /// Defines shift of the reference point in the displayed range. The setting is made in percent relative to the size of the display range. This display range is normalized to 0% (left edge) and 100% (right edge). If the result length is changed, the percentage remains valid, and the absolute value in symbols for this shift is modified correspondingly.
        /// 
        /// Units: %
        /// 
        /// Valid Values:
        /// 0.0 to 100.0
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_FIT_ALIGN_VALUE
        /// 
        /// Note(s):
        /// 
        /// (1) This control is not available for R&amp;S FSV.
        /// </param>
        /// <param name="Align_Offset">
        /// Defines shifts the display range (relative to the reference time) by the number of given symbols. The resolution is 1 symbol. A value &gt;0 results in a shift towards the right, and a value &lt;0 results in a shift towards the left.
        /// 
        /// Units: Symbols
        /// 
        /// Valid Values: no checking
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_FIT_ALIGN_OFFSET
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSADisplayAlignment(int Window, int Trace, int Alignment, double Align_Value, int Align_Offset)
        {
            int pInvokeResult = PInvoke.ConfigureVSADisplayAlignment(this._handle, Window, Trace, Alignment, Align_Value, Align_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the positioning the measurements results on the display.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_FIT_REF_POINT
        /// RSSPECAN_ATTR_VSA_FIT_SYMBOL_NUMBER
        /// RSSPECAN_ATTR_VSA_FIT_SYNC_OFFSET
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:TRACe&lt;1 to 3&gt;:ADJust TRIGGer | BURSt | PATTern
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:X[:SCALe]:VOFFset
        /// [SENSe&lt;1|2&gt;:]DDEMod:STANdard:SYNC:OFFSet
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// 1 to 1
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Fit">
        /// Selects the reference point.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VSA_FIT_TRIGGER (0) - Trigger
        /// RSSPECAN_VAL_VSA_FIT_BURST   (1) - Burst
        /// RSSPECAN_VAL_VSA_FIT_PATTERN (2) - Pattern
        /// 
        /// Default Value: RSSPECAN_VAL_VSA_FIT_TRIGGER (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_FIT_REF_POINT
        /// </param>
        /// <param name="Symbol_Number">
        /// Sets the symbol number at the right edge of the X axis.
        /// 
        /// Units: Symbols
        /// 
        /// Valid Values: no checking
        /// 
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_FIT_SYMBOL_NUMBER
        /// </param>
        /// <param name="Pattern_Position">
        /// Sets defines the number of bits to be reoffset of the pattern in symbols with reference to the start of the burst.
        /// 
        /// Units: Symbols
        /// 
        /// Valid Values: no checking
        /// 
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_FIT_SYNC_OFFSET
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSADisplayFitTrace(int Window, int Trace, int Fit, int Symbol_Number, int Pattern_Position)
        {
            int pInvokeResult = PInvoke.ConfigureVSADisplayFitTrace(this._handle, Window, Trace, Fit, Symbol_Number, Pattern_Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the displaying of the trace.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_DISPLAY_TRACE_SYMBOL
        /// RSSPECAN_ATTR_VSA_TRACE_DISPLAY_SPACING
        /// 
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:SYMBol DOTS | BARS | OFF
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y:SPACing LINear | LOGarithmic | LDB
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// 1 to 1
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Symbol">
        /// Defines the display of the decision instants on the trace.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VSA_TRACE_OFF  (0) - Off
        /// RSSPECAN_VAL_VSA_TRACE_DOTS (1) - Dots
        /// RSSPECAN_VAL_VSA_TRACE_BARS (2) - Bars
        /// 
        /// Default Value: RSSPECAN_VAL_VSA_TRACE_OFF  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_DISPLAY_TRACE_SYMBOL
        /// </param>
        /// <param name="Spacing">
        /// Switches between a linear in %, linear in dB and a logarithmic scale in the selected measurement window.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TRACE_SPACE_LIN    (0) - Linear %
        /// RSSPECAN_VAL_TRACE_SPACE_LOG    (1) - Logarithmic
        /// RSSPECAN_VAL_TRACE_SPACE_LDB    (3) - Linear dB
        /// 
        /// Default Value: RSSPECAN_VAL_TRACE_SPACE_LOG    (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_TRACE_DISPLAY_SPACING
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSADisplayTrace(int Window, int Trace, int Symbol, int Spacing)
        {
            int pInvokeResult = PInvoke.ConfigureVSADisplayTrace(this._handle, Window, Trace, Symbol, Spacing);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the scaling of the x axis for statistical displays.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_SCALE_X_BAR_COUNT
        /// RSSPECAN_ATTR_VSA_SCALE_X_PDIV
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:SCALe:X:BCOunt
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:X[:SCALe]:PDIVision
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// 1 to 1
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Quantize">
        /// Defines defines the number of bars for the statistical functions.
        /// 
        /// Valid Values: 
        /// 1 to 1024
        /// 2 to 1024 for R&amp;S FSV
        /// 
        /// Default Value: 101
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_SCALE_X_BAR_COUNT
        /// </param>
        /// <param name="Scaling">
        /// Defines the scaling of the X axis. 
        /// 
        /// Units: Symbols
        /// 
        /// Valid Values: no checking
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_SCALE_X_PDIV
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSADisplayXAxis(int Window, int Trace, int Quantize, int Scaling)
        {
            int pInvokeResult = PInvoke.ConfigureVSADisplayXAxis(this._handle, Window, Trace, Quantize, Scaling);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the scaling referenece value of the x axis for I/Q and statistical displays.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_SCALE_X_RVAL
        /// RSSPECAN_ATTR_VSA_SCALE_X_RPOS
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:X[:SCALe]:RVALue
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:X[:SCALe]: RPOSition 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// 1 to 1
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Reference_Value">
        /// Defines the reference value for the X axis of the measurement diagram.
        /// 
        /// Units: Symbols
        /// 
        /// Valid Values: no checking
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_SCALE_X_RVAL
        /// </param>
        /// <param name="Reference_Position">
        /// Defines the position of the reference value for the X axis.
        /// 
        /// Units: %
        /// 
        /// Valid Values: 0 to 100
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_SCALE_X_RPOS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSADisplayXAxisReference(int Window, int Trace, int Reference_Value, int Reference_Position)
        {
            int pInvokeResult = PInvoke.ConfigureVSADisplayXAxisReference(this._handle, Window, Trace, Reference_Value, Reference_Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function control the zero point and scaling of the y
        /// axis for I/Q, statistical and time displays:.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_SCALE_Y_PDIV
        /// RSSPECAN_ATTR_DISP_REF_VALUE
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:PDIVision
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:RVALue
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Scaling">
        /// Defines the scaling of the Y axis.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0.0999
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_SCALE_Y_PDIV
        /// </param>
        /// <param name="Reference_Value">
        /// Defines the reference value for the Y axis of the measurement diagram.
        /// 
        /// 
        /// Valid Values: no checking
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_REF_VALUE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSADisplayYAxis(int Window, double Scaling, double Reference_Value)
        {
            int pInvokeResult = PInvoke.ConfigureVSADisplayYAxis(this._handle, Window, Scaling, Reference_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function controls the reference position of Y axis.
        /// 
        /// Attribute(s):
        /// 
        /// RSSPECAN_ATTR_DISP_REF_POSITION
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:RPOSition
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Reference_Position">
        /// Defines the position of the reference value for the X axis.
        /// 
        /// Units: %
        /// 
        /// Valid Values: 0.0 to 100.0
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_REF_POSITION
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSADisplayYAxisRefPosition(int Window, double Reference_Position)
        {
            int pInvokeResult = PInvoke.ConfigureVSADisplayYAxisRefPosition(this._handle, Window, Reference_Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the points per symbol settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_CONFIGURE_DISPLAY_POINTS_PER_SYMBOL_AUTO
        /// RSSPECAN_ATTR_VSA_CONFIGURE_DISPLAY_POINTS_PER_SYMBOL
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;window&gt;]:PRATe:AUTO
        /// DISPlay[:WINDow&lt;window&gt;]:PRATe[:VALue]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Points_Per_Symbol">
        /// This control turns auto mode of points per symbol on or off.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_AUTO (0) - Auto
        /// RSSPECAN_VAL_MAN  (1) - Manual
        /// 
        /// Default Value: RSSPECAN_VAL_AUTO (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_CONFIGURE_DISPLAY_POINTS_PER_SYMBOL_AUTO
        /// </param>
        /// <param name="Number_of_Points_Per_Symbol">
        /// This control determines the number of points to be displayed per symbol if manual mode is selected.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 4
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_CONFIGURE_DISPLAY_POINTS_PER_SYMBOL
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available if Points Per Symbol is set to Manual.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSADisplayPointPerSymbol(int Window, int Points_Per_Symbol, int Number_of_Points_Per_Symbol)
        {
            int pInvokeResult = PInvoke.ConfigureVSADisplayPointPerSymbol(this._handle, Window, Points_Per_Symbol, Number_of_Points_Per_Symbol);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures whether only the symbol points or all points are considered for the statistical calculations.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_CONFIGURE_DISPLAY_POINTS_PER_SYMBOL_AUTO
        /// RSSPECAN_ATTR_VSA_CAPTURE_OVERSAMPLING
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;window&gt;]:PRATe:AUTO
        /// CALCulate&lt;1..4&gt;:STATistics:MODE SONLy | INFinite
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Oversampling">
        /// This control sets the oversampling state.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_AUTO (0) - Auto
        /// RSSPECAN_VAL_MAN  (1) - Manual
        /// 
        /// Default Value: RSSPECAN_VAL_AUTO (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_CONFIGURE_DISPLAY_POINTS_PER_SYMBOL_AUTO
        /// </param>
        /// <param name="Statistical_Mode">
        /// This control defines whether only the symbol points or all points are considered for the statistical calculations.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_STAT_MODE_SONLY    (0) - Symbol Points Only
        /// RSSPECAN_VAL_STAT_MODE_INFINITE (1) - All Points Used
        /// 
        /// Default Value: RSSPECAN_VAL_STAT_MODE_SONLY (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_CAPTURE_OVERSAMPLING
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available if Oversampling is set to Manual.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSACaptureOversampling(int Window, int Oversampling, int Statistical_Mode)
        {
            int pInvokeResult = PInvoke.ConfigureVSACaptureOversampling(this._handle, Window, Oversampling, Statistical_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures whether the trace displays the evaluation of the measured signal or the reference signal (if "Meas &amp; Ref Signal" is used as the signal source).
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with FSV instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_TRACE_EVALUATION
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1..4&gt;:TRACe&lt;1..6&gt;[:VALue] MEAS | REF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace_Evaluation">
        /// This control defines whether the trace displays the evaluation of the measured signal or the reference signal.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TRAC_EVAL_MEAS (0) - Measurement
        /// RSSPECAN_VAL_TRAC_EVAL_REF  (1) - Reference
        /// 
        /// Default Value: RSSPECAN_VAL_TRAC_EVAL_MEAS (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_TRACE_EVALUATION
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSATraceEvaluation(int Window, int Trace_Evaluation)
        {
            int pInvokeResult = PInvoke.ConfigureVSATraceEvaluation(this._handle, Window, Trace_Evaluation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates/deactivates the window.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISPLAY_STATE
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1..4&gt;]:STATe OFF | ON
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Window_State">
        /// This control activates/deactivates the window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISPLAY_STATE
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSADisplayWindowState(int Window, bool Window_State)
        {
            int pInvokeResult = PInvoke.ConfigureVSADisplayWindowState(this._handle, Window, System.Convert.ToUInt16(Window_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the VSA multiple evaluation.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_MBURST_CAPTURE_AUTO
        /// RSSPECAN_ATTR_VSA_MBURST_STATE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:MBURst:CAPTure:AUTO ON | OFF
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:MBURst[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Capture_Auto">
        /// Switches the mode for writing to the capture RAM. If the state is ON, a new data capture operation will be initiated when the end of the capture RAM is reached (and DDEM:SEAR:MBUR:FIND:NEXT occurs again). This command is available only in single sweep
        /// mode.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_MBURST_CAPTURE_AUTO
        /// 
        /// Note(s):
        /// 
        /// (1) This control is not available for R&amp;S FSV.
        /// </param>
        /// <param name="Multiburst_State">
        /// Sets the multi-burst search. First, a large amount of data is captured in the capture RAM. This data can then be demodulated and measured. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_MBURST_STATE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAMultipleEvaluation(int Window, bool Capture_Auto, bool Multiburst_State)
        {
            int pInvokeResult = PInvoke.ConfigureVSAMultipleEvaluation(this._handle, Window, System.Convert.ToUInt16(Capture_Auto), System.Convert.ToUInt16(Multiburst_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the VSA multiple evaluation zoom parameters.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is not available for R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_MBURST_START
        /// RSSPECAN_ATTR_VSA_MBURST_LENGTH
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:MBURst:STARt
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:MBURst:LENGth 10 to 32000
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Zoom_Start">
        /// Sets defines the start of the visible range in the Magnitude Capture display.
        /// 
        /// Units: Symbols
        /// 
        /// Valid Values: no checking
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_MBURST_START
        /// </param>
        /// <param name="Zoom_Length">
        /// Sets the length of the visible range in the Magnitude Capture display.
        /// 
        /// Units: Symbols
        /// 
        /// Valid Values: 10 to 32000
        /// 
        /// Default Value: 100
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_MBURST_LENGTH
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAMultipleEvaluationZoom(int Window, int Zoom_Start, int Zoom_Length)
        {
            int pInvokeResult = PInvoke.ConfigureVSAMultipleEvaluationZoom(this._handle, Window, Zoom_Start, Zoom_Length);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function controls demodulation in multiple evaluation and searches the burst/sync pattern in the Capture Ram.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is not available for R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_MBURST_SEARCH_FIRST
        /// RSSPECAN_ATTR_VSA_MBURST_SEARCH_NEXT
        /// RSSPECAN_ATTR_VSA_MBURST_SEARCH_START
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:MBURst:FIND:FIRSt
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:MBURst:FIND:NEXT
        /// [SENSe&lt;1|2&gt;:]DDEMod:SEARch:MBURst:FIND:STARt
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Action">
        /// Sets the action to be performed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VSA_MBURS_SEARCH_FIRST (0) - Search First
        /// RSSPECAN_VAL_VSA_MBURS_SEARCH_NEXT  (1) - Search Next
        /// RSSPECAN_VAL_VSA_MBURS_SEARCH_START (2) - Search Start
        /// 
        /// Default Value: RSSPECAN_VAL_VSA_MBURS_SEARCH_FIRST (0)
        /// 
        /// Note(s):
        /// (1) Search First - searches the first burst/sync pattern in the Capture Ram.
        /// 
        /// (2) Search Next - searches the next burst/sync pattern in the Capture Ram.
        /// 
        /// (3) Search Start - searches for the first burst/sync pattern that occurs starting at the position that was defined via the command DDEM:SEAR:MBUR:STAR. This is the position starting at which the signal is demodulated in the capture RAM. (Finds first sync patt starting at start position)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int VSASearchBurstInCaptureRam(int Window, int Action)
        {
            int pInvokeResult = PInvoke.VSASearchBurstInCaptureRam(this._handle, Window, Action);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function transfers the following from other FSQ analyzers or from external programs (MAPWIZ, FILTWIZ) via file operations:
        /// - STANDARDS
        /// - MAPPINGS
        /// - PATTERNS
        /// - FILTERS
        /// - EQUALIZER FILTER
        /// 
        /// Note(s):
        /// 
        /// (1) This function is not available for R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:IMPort:STANdard &lt;name&gt;,&lt;path&gt;
        /// [SENSe&lt;1|2&gt;:]DDEMod:IMPort:MAPPing &lt;name&gt;,&lt;path&gt;
        /// [SENSe&lt;1|2&gt;:]DDEMod:IMPort:PATTern &lt;name&gt;,&lt;path&gt;
        /// [SENSe&lt;1|2&gt;:]DDEMod:IMPort:FILTer &lt;name&gt;,&lt;path&gt;
        /// [SENSe&lt;1|2&gt;:]DDEMod:IMPort:EQUalizer &lt;name&gt;,&lt;path&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Type">
        /// Selects the item to be import.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VSA_IMPEXP_STANDARDS  (0) - Standards
        /// RSSPECAN_VAL_VSA_IMPEXP_MAPPINGS   (1) - Mappings
        /// RSSPECAN_VAL_VSA_IMPEXP_PATTERNS   (2) - Patterns
        /// RSSPECAN_VAL_VSA_IMPEXP_FILTERS    (3) - Filters
        /// RSSPECAN_VAL_VSA_IMPEXP_EQUALIZERS (4) - Equalizers
        /// 
        /// Default Value: RSSPECAN_VAL_VSA_IMPEXP_STANDARDS  (0)
        /// 
        /// Note(s):
        /// (1) Standards - copies the selected external modulation standard file from the specified directory to the internal hard disk. The associated patterns, mappings, filters and limit lines are saved along with the standard.
        /// 
        /// (2) Mappings - copies the selected external mappings file from the specified directory to the internal hard disk.
        /// 
        /// (3) Patterns - copies the selected external pattern file from the specified direc tory to the internal hard disk.
        /// 
        /// (4) Filters - copies the selected external filter file from the specified directory to the internal hard disk.
        /// 
        /// (5) Equalizer - copies the selected external equalizer file from the specified directory to the internal hard disk..
        /// </param>
        /// <param name="Name">
        /// Selects name of external file.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// 
        /// </param>
        /// <param name="Path">
        /// Selects path to which the internal files are copied
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int VSAImport(int Type, string Name, string Path)
        {
            int pInvokeResult = PInvoke.VSAImport(this._handle, Type, Name, Path);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function transfers the following from other R&amp;S FSQ analyzers or from external programs (MAPWIZ, FILTWIZ) via file operations:
        /// - STANDARDS
        /// - MAPPINGS
        /// - PATTERNS
        /// - FILTERS
        /// - EQUALIZER FILTER
        /// 
        /// Note(s):
        /// 
        /// (1) This function is not available for R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DDEMod:EXPort:STANdard &lt;name&gt;,&lt;path&gt;
        /// [SENSe&lt;1|2&gt;:]DDEMod:EXPort:MAPPing &lt;name&gt;,&lt;path&gt;
        /// [SENSe&lt;1|2&gt;:]DDEMod:EXPort:PATTern &lt;name&gt;,&lt;path&gt;
        /// [SENSe&lt;1|2&gt;:]DDEMod:EXPort:FILTer &lt;name&gt;,&lt;path&gt;
        /// [SENSe&lt;1|2&gt;:]DDEMod:EXPort:EQUalizer &lt;name&gt;,&lt;path&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Type">
        /// Selects the item to be import.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VSA_IMPEXP_STANDARDS  (0) - Standards
        /// RSSPECAN_VAL_VSA_IMPEXP_MAPPINGS   (1) - Mappings
        /// RSSPECAN_VAL_VSA_IMPEXP_PATTERNS   (2) - Patterns
        /// RSSPECAN_VAL_VSA_IMPEXP_FILTERS    (3) - Filters
        /// RSSPECAN_VAL_VSA_IMPEXP_EQUALIZERS (4) - Equalizers
        /// 
        /// Default Value: RSSPECAN_VAL_VSA_IMPEXP_STANDARDS  (0)
        /// 
        /// Note(s):
        /// (1) Standards - copies the selected internal modulation standard file to the specified directory. The associated patterns, mappings, filters and limit lines are saved along with the standard.
        /// 
        /// (2) Mappings - copies the selected internal mappings file to the specified directory.
        /// 
        /// (3) Patterns - copies the selected internal Pattern file to the specified directory.
        /// 
        /// (4) Filters - copies the selected internal filter file to the specified directory.
        /// 
        /// (5) Equalizer - copies the selected internal equakizer file to the specified directory.
        /// </param>
        /// <param name="Name">
        /// Selects name of internal file.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// 
        /// </param>
        /// <param name="Path">
        /// Selects path to which the internal files are copied
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int VSAExport(int Type, string Name, string Path)
        {
            int pInvokeResult = PInvoke.VSAExport(this._handle, Type, Name, Path);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function exports the data to file in defined format.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FILE_DEC_SEPARATOR
        /// RSSPECAN_ATTR_VSA_FILE_EXP_HEADER
        /// RSSPECAN_ATTR_VSA_FILE_DATA_MODE
        /// RSSPECAN_ATTR_VSA_FILE_DATA_RAW_MODE
        /// -
        /// -
        /// 
        /// Remote-control command(s):
        /// FORMat:DEXPort:DSEParator POINt | COMMa
        /// FORMat:DEXPort:HEADer ON | OFF
        /// FORMat:DEXPort:MODE RAW | TRACe
        /// FORMat:DEXPort:RAW:FORMat ASCii | WAVeform | BINary
        /// FORMat[:DATA] ASCii
        /// MMEMory:STORe&lt;1|2&gt;:TRACe 1 to 3,&lt;file_name&gt;
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Decimal_Separator">
        /// Defines which decimal separator (decimal point or comma) is to be used for outputting measurement data to the file in ASCII format.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DEC_SEP_POIN  (0) - Point
        /// RSSPECAN_VAL_DEC_SEP_COMMA (1) - Comma
        /// 
        /// Default Value: RSSPECAN_VAL_DEC_SEP_POIN (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FILE_DEC_SEPARATOR
        /// </param>
        /// <param name="Header">
        /// Defines if a file header (including start frequency, sweep time, detector, etc.) is created or not. A small header with the instrument model, the version and the date is always transferred.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_FILE_EXP_HEADER
        /// </param>
        /// <param name="Data_Mode">
        /// Defines which data are transferred, raw I/Q data or trace data. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_EXP_MODE_RAW   (0) - Raw I/Q Data
        /// RSSPECAN_VAL_EXP_MODE_TRACE (1) - Trace Data
        /// 
        /// Default Value: RSSPECAN_VAL_EXP_MODE_TRACE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_FILE_DATA_MODE
        /// </param>
        /// <param name="Data_Raw">
        /// Defines the output format of the RAW data file export function. The driver uses this value to set the   RSSPECAN_ATTR_VSA_FILE_DATA_RAW_MODE attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_EXP_RAW_MODE_ASC (0) - ASCII
        /// RSSPECAN_VAL_EXP_RAW_MODE_WAV (1) - Waveform
        /// RSSPECAN_VAL_EXP_RAW_MODE_BIN (2) - Binary
        /// 
        /// Default Value: RSSPECAN_VAL_EXP_RAW_MODE_ASC
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_FILE_DATA_RAW_MODE
        /// 
        /// Note:
        /// 
        /// (1) This control has effect only for Data Mode set to Raw.
        /// 
        /// (2) This control is not available for R&amp;S FSV.
        /// </param>
        /// <param name="File_Name">
        /// Specifies full path of the file, where trace data will be stored.
        /// 
        /// Valid Values:
        /// any valid file name
        /// 
        /// Default Value: "a:\\test.txt"
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAFileExport(int Window, int Trace, int Decimal_Separator, bool Header, int Data_Mode, int Data_Raw, string File_Name)
        {
            int pInvokeResult = PInvoke.ConfigureVSAFileExport(this._handle, Window, Trace, Decimal_Separator, System.Convert.ToUInt16(Header), Data_Mode, Data_Raw, File_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function loads I/Q RAW data files. The file name includes indication of the path and the drive name. The path name complies with DOS conventions. This function is only available if RAW data with binary format is selected (":FORM:DEXP:MODE RAW",  ":FORM:DEXP:RAW:FORM BIN" ). 
        /// The function needs two different files, created with the MMEM:STOR:TRAC command:
        /// ? &lt;file_name&gt;.VAV VSA settings 
        /// ? &lt;file_name&gt;.bin I/Q RAW data
        /// The import of the I/Q RAW data requires the MULTI mode to be active. This mode is automatically switched on with start of the load process. 
        /// 
        /// Note(s):
        /// 
        /// (1) This function is not available for R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_FILE_DATA_MODE
        /// RSSPECAN_ATTR_VSA_FILE_DATA_RAW_MODE
        /// -
        /// 
        /// Remote-control command(s):
        /// FORMat:DEXPort:MODE RAW
        /// FORMat:DEXPort:RAW:FORMat BINary
        /// MMEMory:LOAD:TRACe 1,&lt;file_name&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="File_Name">
        /// Specifies full path of the file, where trace data are stored.
        /// 
        /// Valid Values:
        /// any valid file name
        /// 
        /// Default Value: "a:\\test.txt"
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAFileImport(string File_Name)
        {
            int pInvokeResult = PInvoke.ConfigureVSAFileImport(this._handle, File_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the VSA measurement or reference signal result displays.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_RESULT 
        /// RSSPECAN_ATTR_VSA_RESULT_FORMAT
        /// RSSPECAN_ATTR_DISP_AMPLITUDE_GRID_MODE
        /// RSSPECAN_ATTR_VSA_SPECTRUM_STATE
        /// RSSPECAN_ATTR_MEAS_STAT_CCDF_STATE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FEED 'XTIM:DDEM:MEAS' | 'XTIM:DDEM:REF'
        /// CALCulate&lt;1|2&gt;:FORMat MAGNitude | PHASe | UPHase | RIMag | FREQuency | IEYE | QEYE | COMP | CONS | CONVersion
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:MODE
        /// CALCulate&lt;1|2&gt;:DDEM:SPECtrum[:STATe] ON | OFF
        /// CALCulate:STATistics:CCDF[:STATe] ON | OFF
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Measurement_Signal">
        /// Selects the measured data that are to be displayed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VSA_DATA_SIGNAL_MEAS (0) - Measurement Signal
        /// RSSPECAN_VAL_VSA_DATA_SIGNAL_REF  (1) - Reference Signal
        /// 
        /// Default Value: RSSPECAN_VAL_VSA_DATA_SIGNAL_MEAS (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_RESULT
        /// </param>
        /// <param name="Result_Display">
        /// Sets the displayed measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_RESULT_DISP_FREQ_ABS   (0) - Frequency (Absolute)
        /// RSSPECAN_VAL_RESULT_DISP_FREQ_REL   (1) - Frequency (Relative)
        /// RSSPECAN_VAL_RESULT_DISP_RIM        (2) - Real/Imag
        /// RSSPECAN_VAL_RESULT_DISP_IQ_CONST   (3) - I/Q (Const)
        /// RSSPECAN_VAL_RESULT_DISP_IQ_VECT    (4) - I/Q (Vect)
        /// RSSPECAN_VAL_RESULT_DISP_MAGN_ABS   (5) - Magnitude (Absolute)
        /// RSSPECAN_VAL_RESULT_DISP_MAGN_REL   (6) - Magnitude (Relative)
        /// RSSPECAN_VAL_RESULT_DISP_PHAS_WRAP  (7) - Phase (Wrap)
        /// RSSPECAN_VAL_RESULT_DISP_PHAS_UWRAP (8) - Phase (Unwrap)
        /// RSSPECAN_VAL_RESULT_DISP_EYE_I      (9) - Eye (I)
        /// RSSPECAN_VAL_RESULT_DISP_EYE_Q      (10) - Eye (Q)
        /// 
        /// Default Value: RSSPECAN_VAL_RESULT_DISP_MAGN_ABS   (5)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_RESULT_FORMAT RSSPECAN_ATTR_DISP_AMPLITUDE_GRID_MODE
        /// 
        /// 
        /// Note(s):
        /// (1) Frequency (Absolute) and Freqeuncy (Relative) are available only only for modulation modes FSK and MSK.
        /// 
        /// </param>
        /// <param name="Spectrum">
        /// Switches the set result display to a spectral evaluation of the result parameter.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_SPECTRUM_STATE
        /// 
        /// Note:
        /// (1) This control has effect only if Result display is set to:
        /// Magnitude
        /// Phase
        /// Frequency
        /// Real/Imag
        /// </param>
        /// <param name="Statistics">
        /// switches the measurement of the statistical distribution of MAGNITUDE on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_STAT_CCDF_STATE
        /// 
        /// Note:
        /// (1) This control has effect only if Result display is set to:
        /// Magnitude
        /// Phase
        /// Frequency
        /// 
        /// (2) This control has effect only if Spectrum is set Off.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAMeasurementResults(int Window, int Measurement_Signal, int Result_Display, bool Spectrum, bool Statistics)
        {
            int pInvokeResult = PInvoke.ConfigureVSAMeasurementResults(this._handle, Window, Measurement_Signal, Result_Display, System.Convert.ToUInt16(Spectrum), System.Convert.ToUInt16(Statistics));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the VSA error result displays.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_RESULT
        /// RSSPECAN_ATTR_VSA_RESULT_FORMAT
        /// RSSPECAN_ATTR_DISP_AMPLITUDE_GRID_MODE
        /// RSSPECAN_ATTR_VSA_SPECTRUM_STATE
        /// RSSPECAN_ATTR_MEAS_STAT_CCDF_STATE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FEED 'XTIM:DDEM:ERR:MPH' | 'XTIM:DDEM:ERR:VECT'
        /// CALCulate&lt;1|2&gt;:FORMat MAGNitude | PHASe | UPHase | RIMag | FREQuency | IEYE | QEYE | COMP | CONS | CONVersion
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:MODE
        /// CALCulate&lt;1|2&gt;:DDEM:SPECtrum[:STATe] ON | OFF
        /// CALCulate:STATistics:CCDF[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Result_Display">
        /// Sets the displayed measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_RESULT_DISP_FREQ_ABS   (0) - Frequency (Absolute)
        /// RSSPECAN_VAL_RESULT_DISP_FREQ_REL   (1) - Frequency (Relative)
        /// RSSPECAN_VAL_RESULT_DISP_RIM        (2) - Real/Imag
        /// RSSPECAN_VAL_RESULT_DISP_IQ_CONST   (3) - I/Q (Const)
        /// RSSPECAN_VAL_RESULT_DISP_IQ_VECT    (4) - I/Q (Vect)
        /// RSSPECAN_VAL_RESULT_DISP_MAGN_ERR   (5) - Magnitude Error
        /// RSSPECAN_VAL_RESULT_DISP_PHAS_ERR   (6) - Phase Error
        /// RSSPECAN_VAL_RESULT_DISP_AMPM_CONV  (7) - AM/PM Conversion
        /// RSSPECAN_VAL_RESULT_DISP_EVM        (8) - EVM
        /// 
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_RESULT_DISP_MAGN_ERR   (5)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_RESULT_FORMAT
        /// RSSPECAN_ATTR_DISP_AMPLITUDE_GRID_MODE
        /// 
        /// Note(s):
        /// (1) Frequency (Absolute) and Freqeuncy (Relative) are available only only for modulation modes FSK and MSK.
        /// 
        /// </param>
        /// <param name="Spectrum">
        /// Switches the set result display to a spectral evaluation of the result parameter.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_SPECTRUM_STATE
        /// 
        /// Note:
        /// (1) This control has effect only if Result display is set to:
        /// Magnitude
        /// Phase
        /// Frequency
        /// Real/Imag
        /// Error Vector Magnitude 
        /// </param>
        /// <param name="Statistics">
        /// switches the measurement of the statistical distribution of MAGNITUDE on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_STAT_CCDF_STATE
        /// 
        /// Note:
        /// (1) This control has effect only if Result display is set to:
        /// Magnitude
        /// Phase
        /// Frequency
        /// 
        /// (2) This control has effect only if Spectrum is set Off. If Spectrum is set On, Statistics is set Off allways.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAErrorResults(int Window, int Result_Display, bool Spectrum, bool Statistics)
        {
            int pInvokeResult = PInvoke.ConfigureVSAErrorResults(this._handle, Window, Result_Display, System.Convert.ToUInt16(Spectrum), System.Convert.ToUInt16(Statistics));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the VSA symbol table result displays.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FEED 'XTIM:DDEM:SYMB'
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSASymbolTableResults(int Window)
        {
            int pInvokeResult = PInvoke.ConfigureVSASymbolTableResults(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures adaptive equalizer filter results.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_RESULT
        /// RSSPECAN_ATTR_VSA_RESULT_FORMAT
        /// RSSPECAN_ATTR_VERTICAL_SCALE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FEED 'XTIM:DDEMod:IMPuls' | 'XFR:DDEMod:RATio' | 'XFR:DDEMod:IRATio'
        /// CALCulate&lt;1|2&gt;:FORMat MAGNitude | PHASe | UPHase | RIMag | FREQuency | IEYE | QEYE | COMP | CONS | CONVersion | GDEL
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y:SPACing
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Result_Display">
        /// Sets the displayed measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_RESULT_EQDISP_MAGN_LIN    (0) - Magnitude (Lin)
        /// RSSPECAN_VAL_RESULT_EQDISP_MAGN_LOG    (1) - Magnitude (Log)
        /// RSSPECAN_VAL_RESULT_EQDISP_PHAS_WRAP   (2) - Phase (Wrap)
        /// RSSPECAN_VAL_RESULT_EQDISP_PHAS_UWRAP  (3) - Phase (Unwrap)
        /// RSSPECAN_VAL_RESULT_EQDISP_RIM         (4) - Real/Imag
        /// RSSPECAN_VAL_RESULT_EQDISP_GDELAY      (5) - Group Delay
        /// RSSPECAN_VAL_RESULT_EQDISP_PRESP_WRAP  (6) - Phase Response (Wrap)
        /// RSSPECAN_VAL_RESULT_EQDISP_PRESP_UWRAP (7) - Phase Response (Unwrap)
        /// RSSPECAN_VAL_RESULT_EQDISP_FRESP_LIN   (8) - Frequency Response (Lin)
        /// RSSPECAN_VAL_RESULT_EQDISP_FRESP_LOG   (9) - Frequency Response (Log)
        /// RSSPECAN_VAL_RESULT_EQDISP_CHRESP_LIN  (10) - Channel Response (Lin)
        /// RSSPECAN_VAL_RESULT_EQDISP_CHRESP_LOG  (11) - Channel Response (Log)
        /// 
        /// Default Value: RSSPECAN_VAL_RESULT_EQDISP_MAGN_LIN   (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_RESULT_FORMAT
        /// RSSPECAN_ATTR_VERTICAL_SCALE
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAEqualizerResults(int Window, int Result_Display)
        {
            int pInvokeResult = PInvoke.ConfigureVSAEqualizerResults(this._handle, Window, Result_Display);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the VSA measurement or reference signal result displays.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_RESULT
        /// RSSPECAN_ATTR_VSA_RESULT_FORMAT
        /// RSSPECAN_ATTR_DISP_AMPLITUDE_GRID_MODE
        /// RSSPECAN_ATTR_VSA_SPECTRUM_STATE
        /// RSSPECAN_ATTR_MEAS_STAT_CCDF_STATE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FEED 'TCAP'
        /// CALCulate&lt;1|2&gt;:FORMat MAGNitude | PHASe | UPHase | RIMag | FREQuency | IEYE | QEYE | COMP | CONS | CONVersion
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:MODE
        /// CALCulate&lt;1|2&gt;:DDEM:SPECtrum[:STATe] ON | OFF
        /// CALCulate:STATistics:CCDF[:STATe] ON | OFF
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Result_Display">
        /// Sets the displayed measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_RESULT_DISP_FREQ_ABS   (0) - Frequency (Absolute)
        /// RSSPECAN_VAL_RESULT_DISP_FREQ_REL   (1) - Frequency (Relative)
        /// RSSPECAN_VAL_RESULT_DISP_RIM        (2) - Real/Imag
        /// RSSPECAN_VAL_RESULT_DISP_MAGN_RAW   (3) - Magnitude
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_RESULT_DISP_MAGN_RAW   (3)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_RESULT_FORMAT RSSPECAN_ATTR_DISP_AMPLITUDE_GRID_MODE
        /// 
        /// Note(s):
        /// (1) Frequency (Absolute) and Freqeuncy (Relative) are available only only for modulation modes FSK and MSK.
        /// 
        /// </param>
        /// <param name="Spectrum">
        /// Switches the set result display to a spectral evaluation of the result parameter.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_SPECTRUM_STATE
        /// 
        /// Note:
        /// (1) This control has effect only if Result display is set to:
        /// Magnitude
        /// Frequency
        /// Real/Imag
        /// </param>
        /// <param name="Statistics">
        /// switches the measurement of the statistical distribution of MAGNITUDE on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_STAT_CCDF_STATE
        /// 
        /// Note:
        /// (1) This control has effect only if Result display is set to:
        /// Magnitude
        /// Frequency
        /// Real/Image
        /// 
        /// (2) This control has effect only if Spectrum is set Off.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSARawSignal(int Window, int Result_Display, bool Spectrum, bool Statistics)
        {
            int pInvokeResult = PInvoke.ConfigureVSARawSignal(this._handle, Window, Result_Display, System.Convert.ToUInt16(Spectrum), System.Convert.ToUInt16(Statistics));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the VSA signal source to modulation accuracy.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FEED 'XTIM:DDEM:MACC'
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSAModulationAccuracy(int Window)
        {
            int pInvokeResult = PInvoke.ConfigureVSAModulationAccuracy(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the first value of the X axis in symbols or time, depending on the unit setting for the X axis.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_X_FIRST_VALUE_QUERY
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:X[:SCALe]:STARt?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// 1 to 1
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Result">
        /// Returns the first X-Axis value.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetVSAXAxisFirstValue(int Window, int Trace, out double Result)
        {
            int pInvokeResult = PInvoke.GetVSAXAxisFirstValue(this._handle, Window, Trace, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the results of the error measurement carried out for digital demodulation.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:DDEM:STATistic:MPOWer? &lt;none&gt; | PEAK | ASYM | RMS | AVG | SDEV | TPEak
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:DDEM:STATistic:DTTStart? &lt;none&gt;|RMS| AVG| SDEV
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:DDEM:STATistic:ADRoop? &lt;none&gt;| RMS| AVG| SDEV
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:DDEM:STATistic:CFERror? &lt;none&gt; | RMS | AVG | SDEV | TPEak
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:DDEM:STATistic:EVM? &lt;none&gt; | PEAK | ASYM | RMS | AVG | SDEV | PCTL | TPEak
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:DDEM:STATistic:FSK:CFDRift? &lt;none&gt; | RMS | AVG | SDEV
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:DDEM:STATistic:FSK:DERRor? &lt;none&gt; | PEAK | ASYM | RMS | AVG | SDEV | TPEak
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:DDEM:STATistic:FSK:MDEViation? none&gt; | RMS | AVG | SDEV
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:DDEM:STATistic:IQIMbalance? &lt;none&gt; | RMS | AVG | SDEV
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:DDEM:STATistic:GIMBalance? &lt;none&gt; | RMS | AVG | SDEV
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:DDEM:STATistic:MERRor? &lt;none&gt; | PEAK | ASYM | RMS | AVG | SDEV | TPEak
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:DDEM:STATistic:OOFFset? &lt;none&gt; | RMS | AVG | SDEV
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:DDEM:STATistic:PERRor? &lt;none&gt; | PEAK | ASYM | RMS | AVG | SDEV | TPEak
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:DDEM:STATistic:PLERror? &lt;none&gt; | RMS | AVG | SDEV
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:DDEM:STATistic:QERRor? &lt;none&gt; | RMS | AVG | SDEV
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:DDEM:STATistic:RHO? &lt;none&gt; | RMS | AVG | SDEV
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:DDEM:STATistic:SNR? &lt;none&gt; | RMS | AVG | SDEV
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Type">
        /// This control sets the result.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VSA_MPOW     (0) - Power
        /// RSSPECAN_VAL_VSA_DTTS     (1) - Trigger Delay
        /// RSSPECAN_VAL_VSA_ADR      (2) - Amplitude Droop Error
        /// RSSPECAN_VAL_VSA_CFER     (3) - Carrier Frequency Error
        /// RSSPECAN_VAL_VSA_EVM      (4) - Error Vector Magnitude
        /// RSSPECAN_VAL_VSA_FSK_CDR  (5) - Frequency Error Maximum
        /// RSSPECAN_VAL_VSA_FSK_DERR (6) - FSK Deviation Error
        /// RSSPECAN_VAL_VSA_FSK_MDEV (7) - FSK Deviation
        /// RSSPECAN_VAL_VSA_IQIM     (8) - I/Q Imbalance Error
        /// RSSPECAN_VAL_VSA_GIMB     (9) - Gain Imbalance Error
        /// RSSPECAN_VAL_VSA_MERR     (10) - Magnitude Error
        /// RSSPECAN_VAL_VSA_OOFF     (11) - I/Q Offset Error
        /// RSSPECAN_VAL_VSA_PERR     (12) - Phase Error
        /// RSSPECAN_VAL_VSA_PLER     (13) - Pilot Level Error
        /// RSSPECAN_VAL_VSA_QERR     (14) - Quadrature Error
        /// RSSPECAN_VAL_VSA_RHO      (15) - Rho Factor Error
        /// RSSPECAN_VAL_VSA_SNR      (16) - Modulation Error Ratio
        /// 
        /// Default Value: RSSPECAN_VAL_VSA_MPOW     (0)
        /// 
        /// Note(s):
        /// (1) Power - queries the results of the power measurement of digital demodulation. The output values are the same values as those offered when selecting modulation accuracy (softkey SYMBOLS &amp; MOD ACC). All modifiers are valid for this value.
        /// 
        /// (2) Trigger Delay - queries the results of the trigger delay having an effect on the sync sequence of digital demodulation. The output values are the same values as those offered when selecting modulation accuracy (softkey SYMBOLS &amp; MOD ACC).
        /// Modifiers None, RMS, AVG, Standard Deviation are allowed for this selection.
        /// 
        /// (3) Amplitude Droop Error - queries the results of the amplitude droop error measurement performed for digital demodulation. The output values are the same values as those offered when selecting modulation accuracy (softkey SYMBOLS &amp; MOD ACC). Modifiers None, RMS, AVG, Standard Deviation are allowed for this selection.
        /// 
        /// (4) Carrier Frequency Error - queries the results of the carrier frequency error measurement performed for digital demodulation. The output values are the same values as those offered when selecting modulation accuracy (softkey SYMBOLS &amp; MOD ACC). Modifiers None, RMS, AVG, Standard Deviation and Extreme Value are allowed for this selection.
        /// 
        /// (5) Error Vector Magnitude - queries the results of the error vector magnitude measurement of digital demodulation. The output values are the same values as those offered when selecting
        /// modulation accuracy (softkey SYMBOLS &amp; MOD ACC). For FSK demodulation, this command is not available. All modifiers are valid for this value.
        /// 
        /// (6) Frequency Error Maximum - queries the results of the frequency error maximum of digital demodulation. The
        /// output values are the same values as those offered when selecting modulation accuracy (softkey SYMBOLS &amp; MOD ACC).
        /// This command is only available for FSK demodulation. Modifiers None, RMS, AVG, Standard Deviation are allowed for this selection.
        /// 
        /// (7) FSK Deviation Error - queries the results of the FSK deviation error of digital demodulation. The output
        /// values are the same values as those offered when selecting modulation accuracy (softkey SYMBOLS &amp; MOD ACC).
        /// This command is only available for FSK demodulation. All modifiers are valid for this value.
        /// 
        /// (8) FSK Deviation - queries the results of the FSK deviation of digital demodulation. The output values are the same values as those offered when selecting modulation accuracy (softkey
        /// SYMBOLS &amp; MOD ACC). This command is only available for FSK demodulation. Modifiers None, RMS, AVG, Standard Deviation are allowed for this selection.
        /// 
        /// (9) I/Q Imbalance Error - queries the results of the I/Q imbalance error measurement of digital demodulation. The output values are the same values as those offered when selecting
        /// modulation accuracy (softkey SYMBOLS &amp; MOD ACC).
        /// For FSK demodulation, this command is not available. Modifiers None, RMS, AVG, Standard Deviation are allowed for this selection.
        /// 
        /// (10) Gain Imbalance Error - queries the results of the Gain Imbalance error measurement of digital demodulation. The output values are the same values as those offered when selecting
        /// modulation accuracy (softkey SYMBOLS &amp; MOD ACC). For FSK demodulation, this command is not available. Modifiers None, RMS, AVG, Standard Deviation are allowed for this selection.
        /// 
        /// (11) Magnitude Error - queries the results of the magnitude error measurement of digital demodulation. The output values are the same values as those offered when selecting modulation accuracy (softkey SYMBOLS &amp; MOD ACC). All modifiers are valid for this value.
        /// 
        /// (12) Origin Offset Error - queries the results of the origin offset error measurement performed for digital
        /// demodulation. The output values are the same values as those offered when selecting modulation accuracy (softkey SYMBOLS &amp; MOD ACC). This command is not available for FSK demodulation.
        /// For FSK demodulation, this command is not available. Modifiers None, RMS, AVG, Standard Deviation are allowed for this selection.
        /// 
        /// (13) Phase Error - queries the results of the phase error measurement performed for digital demodulation. The output values are the same values as those offered when selecting
        /// modulation accuracy (softkey SYMBOLS &amp; MOD ACC). For FSK demodulation, this command is not available. All modifiers are valid for this value.
        /// 
        /// (14) Pilot Level Error - queries the results of the pilot level error measurement performed for digital demodulation. The output values are the same values as those offered when selecting
        /// modulation accuracy (softkey SYMBOLS &amp; MOD ACC). This command is only available for VSB demodulation. Modifiers None, RMS, AVG, Standard Deviation are allowed for this selection.
        /// 
        /// (15) Quadrature Error - queries the results of the Quadratur error measurement performed for digital demodulation. The output values are the same values as those offered when selecting
        /// modulation accuracy (softkey SYMBOLS &amp; MOD ACC). For FSK demodulation, this command is not available. Modifiers None, RMS, AVG, Standard Deviation are allowed for this selection.
        /// 
        /// (16) RHo Factor Error - queries the results of the RHo factor error measurement performed for digital demodulation. The output values are the same as those offered when selecting modulation
        /// accuracy (softkey SYMBOLS &amp; MOD ACC). For FSK demodulation, this command is not available. Modifiers None, RMS, AVG, Standard Deviation are allowed for this selection.
        /// 
        /// (17) SNR Error - queries the results of the SNR error measurement performed for digital demodulation. The output values are the same as those offered when selecting modulation
        /// accuracy (softkey SYMBOLS &amp; MOD ACC). For FSK demodulation, this command is not available. Modifiers None, RMS, AVG, Standard Deviation are allowed for this selection.
        /// </param>
        /// <param name="Modifier">
        /// This control sets the result. Available values depend on instrument used, refer to the notes below.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VSA_MODIF_NONE     (0) - None
        /// RSSPECAN_VAL_VSA_MODIF_RMS      (1) - RMS
        /// RSSPECAN_VAL_VSA_MODIF_AVG      (2) - AVG
        /// RSSPECAN_VAL_VSA_MODIF_SDEV     (3) - Standard Deviation
        /// RSSPECAN_VAL_VSA_MODIF_TPE      (4) - Extreme Value
        /// RSSPECAN_VAL_VSA_MODIF_PEAK     (5) - Maximum
        /// RSSPECAN_VAL_VSA_MODIF_ASYM     (6) - Maximum Symbol Number
        /// RSSPECAN_VAL_VSA_MODIF_PCTL     (7) - 95% of Cumulative Distribution
        /// RSSPECAN_VAL_VSA_MODIF_FSV_CURR (8) - Current (RMS/-)
        /// RSSPECAN_VAL_VSA_MODIF_FSV_AVG  (9) - Mean (RMS/-)
        /// RSSPECAN_VAL_VSA_MODIF_FSV_RPE  (10) - Peak (RMS/-)
        /// RSSPECAN_VAL_VSA_MODIF_FSV_SDEV (11) - Standard Deviation (RMS/-)
        /// RSSPECAN_VAL_VSA_MODIF_FSV_PCTL (12) - 95 Percentile (RMS/-)
        /// RSSPECAN_VAL_VSA_MODIF_FSV_PEAK (13) - Current (Peak only)
        /// RSSPECAN_VAL_VSA_MODIF_FSV_PAVG (14) - Mean (Peak only)
        /// RSSPECAN_VAL_VSA_MODIF_FSV_TPE  (15) - Peak (Peak only)
        /// RSSPECAN_VAL_VSA_MODIF_FSV_PSD  (16) - Standard Deviation (Peak only)
        /// RSSPECAN_VAL_VSA_MODIF_FSV_PPCT (17) - 95 Percentile (Peak only)
        /// 
        /// Default Value: RSSPECAN_VAL_VSA_MODIF_NONE (0)
        /// 
        /// Note(s):
        /// (1) 
        /// &lt;none&gt; - Vector error in %rms.
        /// PEAK - Vector error maximum in %pk.
        /// ASYM - Symbol number for which the vector error maximum has occurred.
        /// RMS - Vector error in %, evaluating the rms over several sweeps.
        /// AVG - Vector error in %, evaluating the linear average value over several sweeps.
        /// SDEV - Standard deviation of vector error in %.
        /// PCTL - 95% of cumulative distribution function.
        /// TPEak - Extreme value of all vector error maxima.
        /// 
        /// (2) For Power 
        /// only values &lt;none&gt;, RMS, AVG, Standard Deviation, Extreme Value, Maximum, Maximum Symbol Number are available.
        /// 
        /// (3) For Trigger Delay
        ///         Amplitude Droop Error
        ///         Frequency Error Maximum
        ///         FSK Deviation
        ///         Magnitude Error
        ///         Origin Offset Error
        ///         Pilot Level Error
        ///         Quadrature Error
        ///         RHo Factor Error
        ///         SNR Error
        ///         I/Q Imbalance Error
        ///         Gain Imbalance Error
        /// only values &lt;none&gt;, RMS, AVG, Standard Deviation are available
        /// 
        /// (3) For Carrier Frequency Error
        ///         FSK Deviation Error
        ///         Phase Error
        /// only values &lt;none&gt;, RMS, AVG, Standard Deviation, Extreme Value are available.
        /// 
        /// (4) For Error Vector Magnitude 
        /// all values are available
        /// 
        /// (5) Parameters 8 to 17 are desinged for FSV instrument only:
        /// 
        /// Current - Value for the current sweep
        /// Mean - The linear mean of the values of the Current value.
        /// Peak - The maximum value that occurred during the sweeps.
        /// Standard Deviation - The value for the standard deviation is calculated on the linear values and then converted to the displayed unit.
        /// 95 Percentile - 95% of cumulative distribution function.
        /// 
        /// (6) FSV instrument:
        ///     For Power 
        ///         Amplitude Droop Error
        ///         I/Q Offset Error
        ///         Quadrature Error
        ///         RHo Factor Error
        ///         I/Q Imbalance Error
        ///         Gain Imbalance Error
        ///         Carrier Frequency Error
        ///         Trigger Delay
        ///         Frequency Error Maximum
        ///         FSK Deviation
        ///         FSK Deviation Error
        ///         Pilot Level Error
        /// only values marked with (RMS/-) are available.
        /// 
        ///     For Error Vector Magnitude
        ///         Modulation Error Ratio 
        ///         Magnitude Error
        ///         Phase Error
        /// all values are available
        /// </param>
        /// <param name="Result">
        /// Returns the measured result. 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetVSAResult(int Window, int Type, int Modifier, out double Result)
        {
            int pInvokeResult = PInvoke.GetVSAResult(this._handle, Window, Type, Modifier, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries all results of the result summary.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with FSV instrument.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:DDEM:STATistic:ALL?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Results">
        /// Returns all result summary results. 
        /// 
        /// Order of results:
        /// EVM (RMS) Current, EVM (RMS) Mean, EVM (RMS) Peak, EVM (RMS) StdDev, EVM (RMS) 95%ile, EVM (Peak) Current, EVM (Peak) Mean, EVM (Peak) Peak, EVM (Peak) StdDev, EVM (Peak) 95%ile, MER (RMS) Current, MER (RMS) Mean, MER (RMS) Peak, MER (RMS) StdDev, MER (RMS) 95%ile, MER (Peak) Current, MER (Peak) Mean, MER (Peak) Peak, MER (Peak) StdDev, MER (Peak) 95%ile, Phase Error (RMS) Current, Phase Error (RMS) Mean, Phase Error (RMS) Peak, Phase Error (RMS) StdDev, Phase Error (RMS) 95%ile, Phase Error (Peak) Current, Phase Error (Peak) Mean, Phase Error (Peak) Peak, Phase Error (Peak) StdDev, Phase Error (Peak) 95%ile, Magnitude Error (RMS) Current, Magnitude Error (RMS) Mean, Magnitude Error (RMS) Peak, Magnitude Error (RMS) StdDev, Magnitude Error (RMS) 95%ile, Magnitude Error (Peak) Current, Magnitude Error (Peak) Mean, Magnitude Error (Peak) Peak, Magnitude Error (Peak) StdDev, Magnitude Error (Peak) 95%ile, Carrier Frequency Error Current, Carrier Frequency Error Mean, Carrier Frequency Error Peak, Carrier Frequency Error StdDev, Carrier Frequency Error 95%ile, Rho Current, Rho Mean, Rho Peak, Rho StdDev, Rho 95%ile, IQ Offset Current, IQ Offset Mean, IQ Offset Peak, IQ Offset StdDev, IQ Offset 95%ile, IQ Imbalance Current, IQ Imbalance Mean, IQ Imbalance Peak, IQ Imbalance StdDev, IQ Imbalance 95%ile, Gain Imbalance Current, Gain Imbalance Mean, Gain Imbalance Peak, Gain Imbalance StdDev, Gain Imbalance 95%ile, Quadrature Error Current, Quadrature Error Mean, Quadrature Error Peak, Quadrature Error StdDev, Quadrature Error 95%ile, Amplitude Droop Current, Amplitude Droop Mean, Amplitude Droop Peak, Amplitude Droop StdDev, Amplitude Droop 95%ile, Power Current, Power Mean, Power Peak, Power StdDev, Power 95%ile.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetVSAAllResults(int Window, double[] Results)
        {
            int pInvokeResult = PInvoke.GetVSAAllResults(this._handle, Window, Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the absolute X marker position for AM &amp; PM conversion measurement. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_X_MARKER_POSITION
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:X:CONVersion:ABSolute?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// Selects the marker
        /// 
        /// Valid Values:
        /// 1 to 1
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Position">
        /// Returns the X marker position.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetVSAXMarkerPosition(int Window, int Marker, out double Position)
        {
            int pInvokeResult = PInvoke.GetVSAXMarkerPosition(this._handle, Window, Marker, out Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns all the measured data that relates to the currently selected measurement type. The returned data is particular to the currently selected measurement type and is specified in the Trace Data parameter description.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2|3|4&gt;[:DATA]? TRACE1 | TRACE2 | TRACE3 
        /// FORMat[:DATA] REAL,32
        /// *CLS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Source_Trace">
        /// This parameter indicates which trace will be read out.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// Number of array points requested.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 256
        /// </param>
        /// <param name="Trace_Data">
        /// Returns an array of trace data. Number and format of the measurement values for operating mode Wireless LAN (WLAN):
        /// 
        /// --- Cartesian diagrams
        /// In all cartesian diagrams (Magnitude, Phase, Frequency, Real/Imag, Eye) only the Y-values are transferred. The number of Y-valiues is the product of RESULT LENGTH and POINTS PER
        /// SYMBOL. The X-value related to the first Y-value is queried with command DISP:TRAC:X:START?.
        /// Note: In the case of the eye pattern, results are simply superimposed in the display, ie the EYE representation is the same as the REAL/IMAG representation.
        /// 
        /// --- Polar diagrams
        /// In the polar diagrams (Polar Constellation, Polar Vector) the real and the imaginary component are transferred as a pair for each result. The number of value pairs is the product of RESULT
        /// LENGTH and POINTS PER SYMBOL for POLAR VECTOR, and the RESULT LENGTH for POLAR CONSTELLATION.
        /// 
        /// --- Symbols &amp; Modulation Accuracy
        /// Symbol Table:
        /// The displayed symbols can be read out as TRACE1. The data is transferred in symbols in UINT16 format. The value range depends on the specific demodulation mode (for 8PSK the value range for a symbol is 0..7)
        /// 
        /// Modulation Accuracy:
        /// The symbol accuracy is read out as TRACE2. Modulation type FSK, number of measurement values for FSK: 42, format:
        /// &lt;1: FSK Dev Error (Result)&gt;,&lt;2: FSK Dev Error (Peak)&gt;,
        /// &lt;3: FSK Dev Error (atSym)&gt;,&lt;4: FSK Dev Error (Rms)&gt;,
        /// &lt;5: FSK Dev Error (Avg)&gt;,&lt;6: FSK Dev Error (StdDev)&gt;,
        /// &lt;7: FSK Dev Error (Total Peak)&gt;,&lt;8: FSK Meas Dev (Result)&gt;,
        /// &lt;9: FSK Meas Dev (Rms)&gt;,&lt;10: FSK Meas Dev (Avg)&gt;,
        /// &lt;11: FSK Meas Dev (StdDev)&gt;,&lt;12: FSK Rev Dev (Result)&gt;,
        /// &lt;13: Carrier Freq Err (Result)&gt;,&lt;14: Carrier Freq Err (Rms)&gt;,
        /// &lt;15: Carrier Freq Err (Avg)&gt;,&lt;16: Carrier Freq Err (StdDev)&gt;,
        /// &lt;17: Carrier Freq Drift (Result)&gt;,&lt;18: Carrier Freq Drift (Rms)&gt;,
        /// &lt;19: Carrier Freq Drift (Avg)&gt;,&lt;20: Carrier Freq Drift (StdDev)&gt;,
        /// &lt;21: Mag Err (Result)&gt;,&lt;22: Mag Err (Peak)&gt;,
        /// &lt;23: Mag Err (atSym)&gt;,&lt;24: Mag Err (Rms)&gt;,
        /// &lt;25: Mag Err (Avg)&gt;,&lt;26: Mag Err (StdDev)&gt;,
        /// &lt;27: Mag Err (Total Peak)&gt;,&lt;28: Ampt Droop (Result)&gt;,
        /// &lt;29: Ampt Droop (Rms)&gt;,&lt;30: Ampt Droop (Avg)&gt;,
        /// &lt;31: Ampt Droop (StdDev)&gt;,&lt;32: Mean Power (Result)&gt;,
        /// &lt;33: Mean Power (Peak)&gt;,&lt;34: Mean Power (atSym)&gt;,
        /// &lt;35: Mean Power (Rms)&gt;,&lt;36: Mean Power (Avg)&gt;,
        /// &lt;37: Mean Power (StdDev)&gt;,&lt;38: Mean Power (Total Peak)&gt;,
        /// &lt;39: Trigger to Sync (Result)&gt;,&lt;40: Trigger to Sync (Rms)&gt;,
        /// &lt;41: Trigger to Sync (Avg)&gt;,&lt;42: Trigger to Sync (StdDev)&gt;,
        /// 
        /// Modulation type VSB, number of measurement values: 57
        /// &lt;1: EVM (Result)&gt;,&lt;2: EVM (Peak)&gt;,
        /// &lt;3: EVM (atSym)&gt;,&lt;4: EVM (Rms)&gt;,
        /// &lt;5: EVM (Avg)&gt;,&lt;6: EVM (StdDev)&gt;,
        /// &lt;7: EVM (95Pctl)&gt;,&lt;8: EVM (Total Peak)&gt;,
        /// &lt;9: Mag Err (Result)&gt;,&lt;10: Mag Err (Peak)&gt;,
        /// &lt;11: Mag Err (atSym)&gt;,&lt;12: Mag Err (Rms)&gt;,
        /// &lt;13: Mag Err (Avg)&gt;,&lt;14: Mag Err (StdDev)&gt;,
        /// &lt;15: Mag Err (Total Peak)&gt;,&lt;16: Phase Err (Result)&gt;,
        /// &lt;17: Phase Err (Peak)&gt;,&lt;18: Phase Err (atSym)&gt;,
        /// &lt;19: Phase Err (Rms)&gt;,&lt;20: Phase Err (Avg)&gt;,
        /// &lt;21: Phase Err (StdDev)&gt;,&lt;22: Phase Err (Total Peak)&gt;,
        /// &lt;23: Carrier Freq Err (Result)&gt;,&lt;24: Carrier Freq Err (Rms)&gt;,
        /// &lt;25: Carrier Freq Err (Avg)&gt;,&lt;26: Carrier Freq Err (StdDev)&gt;,
        /// &lt;27: Ampt Droop (Result)&gt;,&lt;28: Ampt Droop (Rms)&gt;,
        /// &lt;29: Ampt Droop (Avg)&gt;,&lt;30: Ampt Droop (StdDev)&gt;,
        /// &lt;31: Origin Offset (Result)&gt;,&lt;32: Origin Offset (Rms)&gt;,
        /// &lt;33: Origin Offset (Avg)&gt;,&lt;34: Origin Offset (StdDev)&gt;,
        /// &lt;35: IQ Imbalance (Result)&gt;,&lt;36: IQ Imbalance (Rms)&gt;,
        /// &lt;37: IQ Imbalance (Avg)&gt;,&lt;38: IQ Imbalance (StdDev)&gt;,
        /// &lt;39: Mean Power (Result)&gt;,&lt;40: Mean Power (Peak)&gt;,
        /// &lt;41: Mean Power (atSym)&gt;,&lt;42: Mean Power (Rms)&gt;,
        /// &lt;43: Mean Power (Avg)&gt;,&lt;44: Mean Power (StdDev)&gt;,
        /// &lt;45: Mean Power (Total Peak)&gt;, &lt;46: RHO (Result)&gt;,
        /// &lt;47: RHO (Rms)&gt;,&lt;48: RHO (Avg)&gt;,
        /// &lt;49: RHO (StdDev)&gt;, &lt;50: Trigger to Sync (Result)&gt;,
        /// &lt;51: Trigger to Sync (Rms)&gt;,&lt;52: Trigger to Sync (Avg)&gt;,
        /// &lt;53: Trigger to Sync (StdDev)&gt;, &lt;54: Pilot Level Error (Result)&gt;,
        /// &lt;55: Pilot Level Error (Rms)&gt;,&lt;56: Pilot Level Error (Avg)&gt;,
        /// &lt;57: Pilot Level Error (StdDev)&gt;
        /// 
        /// Modulation types other than FSK and VSB, number of measurement values: 53
        /// &lt;1: EVM (Result)&gt;,&lt;2: EVM (Peak)&gt;,
        /// &lt;3: EVM (atSym)&gt;,&lt;4: EVM (Rms)&gt;,
        /// &lt;5: EVM (Avg)&gt;,&lt;6: EVM (StdDev)&gt;,
        /// &lt;7: EVM (95Pctl)&gt;,&lt;8: EVM (Total Peak)&gt;,
        /// &lt;9: Mag Err (Result)&gt;,&lt;10: Mag Err (Peak)&gt;,
        /// &lt;11: Mag Err (atSym)&gt;,&lt;12: Mag Err (Rms)&gt;,
        /// &lt;13: Mag Err (Avg)&gt;,&lt;14: Mag Err (StdDev)&gt;,
        /// &lt;15: Mag Err (Total Peak)&gt;,&lt;16: Phase Err (Result)&gt;,
        /// &lt;17: Phase Err (Peak)&gt;,&lt;18: Phase Err (atSym)&gt;,
        /// &lt;19: Phase Err (Rms)&gt;,&lt;20: Phase Err (Avg)&gt;,
        /// &lt;21: Phase Err (StdDev)&gt;,&lt;22: Phase Err (Total Peak)&gt;,
        /// &lt;23: Carrier Freq Err (Result)&gt;,&lt;24: Carrier Freq Err (Rms)&gt;,
        /// &lt;25: Carrier Freq Err (Avg)&gt;,&lt;26: Carrier Freq Err (StdDev)&gt;,
        /// &lt;27: Ampt Droop (Result)&gt;,&lt;28: Ampt Droop (Rms)&gt;,
        /// &lt;29: Ampt Droop (Avg)&gt;,&lt;30: Ampt Droop (StdDev)&gt;,
        /// &lt;31: Origin Offset (Result)&gt;,&lt;32: Origin Offset (Rms)&gt;,
        /// &lt;33: Origin Offset (Avg)&gt;,&lt;34: Origin Offset (StdDev)&gt;,
        /// &lt;35: IQ Imbalance (Result)&gt;,&lt;36: IQ Imbalance (Rms)&gt;,
        /// &lt;37: IQ Imbalance (Avg)&gt;,&lt;38: IQ Imbalance (StdDev)&gt;,
        /// &lt;39: Mean Power (Result)&gt;,&lt;40: Mean Power (Peak)&gt;,
        /// &lt;41: Mean Power (atSym)&gt;,&lt;42: Mean Power (Rms)&gt;,
        /// &lt;43: Mean Power (Avg)&gt;,&lt;44: Mean Power (StdDev)&gt;,
        /// &lt;45: Mean Power (Total Peak)&gt;, &lt;46: RHO (Result)&gt;,
        /// &lt;47: RHO (Rms)&gt;,&lt;48: RHO (Avg)&gt;,
        /// &lt;49: RHO (StdDev)&gt;, &lt;50: Trigger to Sync (Result)&gt;,
        /// &lt;51: Trigger to Sync (Rms)&gt;,&lt;52: Trigger to Sync (Avg)&gt;,
        /// &lt;53: Trigger to Sync (StdDev)&gt;,
        /// </param>
        /// <param name="No_of_Values">
        /// Returns the number of trace data values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadVSATraceData(int Window, int Source_Trace, int Array_Size, double[] Trace_Data, out int No_of_Values)
        {
            int pInvokeResult = PInvoke.ReadVSATraceData(this._handle, Window, Source_Trace, Array_Size, Trace_Data, out No_of_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function compression point measurement. 
        /// 
        /// Note(s):
        /// 
        /// (1) This function is valid only in the AM/PM diagram.
        /// 
        /// (2) This function is not available for R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_CPOINT_STATE
        /// RSSPECAN_ATTR_VSA_CPOINT_VALUE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:CPOint[:STATe] ON | OFF
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:CPOint:VALue 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// Sets compression point measurement state.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_CPOINT_STATE
        /// </param>
        /// <param name="Value">
        /// Sets the compression value of the compression point measurement.
        /// 
        /// Units: dB
        /// 
        /// Valid Values: 0.0 to 5.0
        /// 
        /// Default Value: 1.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_VSA_CPOINT_VALUE
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureVSACompressionPointMeasurement(int Window, bool State, double Value)
        {
            int pInvokeResult = PInvoke.ConfigureVSACompressionPointMeasurement(this._handle, Window, System.Convert.ToUInt16(State), Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns teh compression point measurement results.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is not available for R&amp;S FSV.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_VSA_CPOINT_PHASE_SHIFT
        /// RSSPECAN_ATTR_VSA_CPOINT_POWER
        /// -
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:CPOint:PHDiff?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:CPOint:POWer?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:CPOint:DATA?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 - A
        /// 2 - B
        /// 3 - C
        /// 4 - D
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Type">
        /// This control sets the result.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_VSA_CPOINT_PSHIFT (0) - Phase Shift
        /// RSSPECAN_VAL_VSA_CPOINT_POWER  (1) - Power
        /// RSSPECAN_VAL_VSA_CPOINT_DATA   (2) - Data
        /// 
        /// Default Value: RSSPECAN_VAL_VSA_CPOINT_PSHIFT (0)
        /// </param>
        /// <param name="Results">
        /// Returns the measured results. 
        /// 
        /// Note:
        /// (1) For Phase shift and Power only one value is returned.
        /// 
        /// (2) For Data following values are returned in specied order:
        /// Ref Mean Pwr - theoretical power of the ref signal + additional gain from AM/PM diagr.
        /// Meas Mean Pwr - measured mean power
        /// Cmp Mean Pwr - power loss at current modulation
        /// Ref Crest - crest factor of the ideal signal
        /// Meas Crest - crest factor of the measured signal at current modulation
        /// Cmp Crest - difference of the crest factors
        /// &lt;reserved1&gt; reserved for expansion
        /// &lt;reserved2&gt; reserved for expansion
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetVSACompressionPointResults(int Window, int Type, double[] Results)
        {
            int pInvokeResult = PInvoke.GetVSACompressionPointResults(this._handle, Window, Type, Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the instrument into 3GPP FDD BS mode. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BWCD_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument[:SELect] BWCD
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Set3GPPBSMode()
        {
            int pInvokeResult = PInvoke.Set3GPPBSMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the 3GPP FDD UE measurement
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_3GBS_SCODE
        /// RSSPECAN_ATTR_3GBS_WCDP_POW_REF
        /// RSSPECAN_ATTR_3GBS_WCDP_COMPARE_MODE
        /// RSSPECAN_ATTR_3GBS_WCDP_SYNC_TYPE
        /// RSSPECAN_ATTR_3GBS_WCDP_SIDE_BAND
        /// RSSPECAN_ATTR_3GBS_WCDP_NORMALIZE
        /// RSSPECAN_ATTR_3GBS_WCDP_HSDPAUPA_STATE
        /// RSSPECAN_ATTR_3GBS_WCDP_INACT_CHAN_THR
        /// RSSPECAN_ATTR_3GBS_WCDP_INVERT_Q
        /// RSSPECAN_ATTR_3GBS_ANTENNA
        /// RSSPECAN_ATTR_3GBS_WCDP_POW_DIFFERENCE
        /// RSSPECAN_ATTR_3GBS_WCDP_CONST_B
        /// RSSPECAN_ATTR_3GBS_TIMING_OFFSET
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:LCODe[:VALue]
        /// [SENSe:]CDPower:PREFerence TOTal | CPICh
        /// [SENSe&lt;1|2&gt;:]CDPower:PCONtrol PILot | SLOT
        /// [SENSe&lt;1|2&gt;:]CDPower:STYPe CPICh | SCHannel
        /// [SENSe:]CDPower:SBANd NORMal | INVers
        /// [SENSe:]CDPower:NORMalize ON | OFF
        /// [SENSe:]CDPower:HSDPamode ON|OFF
        /// [SENSe:]CDPower:ICTReshold
        /// [SENSe:]CDPower:QINVert ON | OFF
        /// [SENSe&lt;1|2&gt;:]CDPower:ANTenna OFF | 1 | 2
        /// [SENSe&lt;1|2&gt;:]CDPower:PDIFf ON|OFF
        /// [SENSe:]CDPower:CPB
        /// CONFigure:WCDPower[:BTS]:CTABle:TOFFset PREDefine|MEASurement
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Scrambling_Code">
        /// Sets the scrambling code.
        /// 
        /// Valid Values:
        /// 0 to 8191
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GUE_CDP_SCR_CODE
        /// </param>
        /// <param name="Power_Reference">
        /// Switches between switches between the use of total power or CPICH power as the reference for the relative CDP measurement values.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_3GPP_BS_SETTINGS_POW_REF_TOT  (0) - Total
        /// RSSPECAN_VAL_3GPP_BS_SETTINGS_POW_REF_CPIC (1) - CPICH
        /// 
        /// Default Value: RSSPECAN_VAL_3GPP_BS_SETTINGS_POW_REF_TOT  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GBS_WCDP_POW_REF
        /// </param>
        /// <param name="Compare_Mode">
        /// Sets the power control measurement position. The slot power can be averaged from the beginning of the slot to the end of the slot (param. SLOT) or from the beginning of the pilot symbols of the previous slot to the beginning of the pilot symbols of the actual slot. (param. PILOT). Furthermore, an enhanced channel search is used to consider the properties of compressed mode channels.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_3GPP_BS_SETTINGS_COMP_MODE_PILOT (0) - Pilot
        /// RSSPECAN_VAL_3GPP_BS_SETTINGS_COMP_MODE_SLOT  (1) - Slot
        /// 
        /// Default Value: RSSPECAN_VAL_3GPP_BS_SETTINGS_COMP_MODE_PILOT (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GBS_WCDP_COMPARE_MODE
        /// 
        /// Note(s):
        /// 
        /// (1) This control has no effect for R&amp;S FSL.
        /// </param>
        /// <param name="Sync_Type">
        /// Sets the type of synchronization. CPICH Synchronization to CPICH (default setting). For this type of synchronization, the CPICH must be present in the transmit signal.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_3GPP_BS_SETTINGS_SYNC_TYPE_CPIC (0) - CPICH
        /// RSSPECAN_VAL_3GPP_BS_SETTINGS_SYNC_TYPE_SCH  (1) - Sync. without CPICH
        /// 
        /// Default Value: RSSPECAN_VAL_3GPP_BS_SETTINGS_SYNC_TYPE_CPIC (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GBS_WCDP_SYNC_TYPE
        /// </param>
        /// <param name="Side_Band">
        /// Sets interchanging the left and the right sideband.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_3GPP_SBAND_NORM (0) - Normal
        /// RSSPECAN_VAL_3GPP_SBAND_INV  (1) - Inverse
        /// 
        /// Default Value: RSSPECAN_VAL_3GPP_SBAND_NORM  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GBS_WCDP_SIDE_BAND
        /// </param>
        /// <param name="Normalize">
        /// Switches elimination of IQ offset on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GBS_WCDP_NORMALIZE
        /// </param>
        /// <param name="HSDPA_UPA">
        /// Switches the HSUPA/HSDPA channel detection On or Off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GBS_WCDP_HSDPAUPA_STATE
        /// 
        /// Note(s):
        /// 
        /// (1) This control works only with option K74.
        /// </param>
        /// <param name="Inactive_Channel_Treshold">
        /// Sets the threshold value from which a channel is treated as active. The level entered refers to the total signal power.
        /// 
        /// Units: dB
        /// 
        /// Valid Values: -100.0 to 10.0
        /// 
        /// Default Value: -60.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GBS_WCDP_INACT_CHAN_THR
        /// 
        /// </param>
        /// <param name="Invert_Q">
        /// Inverts the Q component of the signal.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GUE_CDP_INVERT_Q
        /// </param>
        /// <param name="Antenna">
        /// Activates or deactivates the antenna diversity mode and selects the antenna to be used.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_3GPP_BS_SETTINGS_ANTENNA_OFF (0) - Off
        /// RSSPECAN_VAL_3GPP_BS_SETTINGS_ANTENNA_1   (1) - 1
        /// RSSPECAN_VAL_3GPP_BS_SETTINGS_ANTENNA_2   (2) - 2
        /// 
        /// Default Value: RSSPECAN_VAL_3GPP_BS_SETTINGS_ANTENNA_OFF (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GBS_ANTENNA
        /// </param>
        /// <param name="Power_Difference">
        /// Activates or deactivates the slot power difference calculation of the POWER VERSUS SLOT display.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GBS_WCDP_POW_DIFFERENCE
        /// 
        /// Note(s):
        /// 
        /// (1) This control has no effect for R&amp;S FSL.
        /// </param>
        /// <param name="Constellation_Parameter_B">
        /// Sets selects the constellation parameter B. 
        /// 
        /// Valid Values:
        /// 0 to 3
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GBS_WCDP_CONST_B
        /// 
        /// Note(s):
        /// 
        /// (1) This control has no effect for R&amp;S FSL.
        /// </param>
        /// <param name="Timing_Offset">
        /// Sets whether the timing offset and pilot length are measured or if the values are taken from the predefined table.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_3GPP_TOFFSET_PRED (0) - Predefine
        /// RSSPECAN_VAL_3GPP_TOFFSET_MEAS (1) - Measurement
        /// 
        /// Default Value: RSSPECAN_VAL_3GPP_TOFFSET_PRED  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GBS_TIMING_OFFSET
        /// 
        /// Note(s):
        /// 
        /// (1) This control has no effect for R&amp;S FSL.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Configure3GPPBSMeasurement(int Scrambling_Code, int Power_Reference, int Compare_Mode, int Sync_Type, int Side_Band, bool Normalize, bool HSDPA_UPA, double Inactive_Channel_Treshold, bool Invert_Q, int Antenna, bool Power_Difference, int Constellation_Parameter_B, int Timing_Offset)
        {
            int pInvokeResult = PInvoke.Configure3GPPBSMeasurement(this._handle, Scrambling_Code, Power_Reference, Compare_Mode, Sync_Type, Side_Band, System.Convert.ToUInt16(Normalize), System.Convert.ToUInt16(HSDPA_UPA), Inactive_Channel_Treshold, System.Convert.ToUInt16(Invert_Q), Antenna, System.Convert.ToUInt16(Power_Difference), Constellation_Parameter_B, Timing_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the 3GPP FDD BS Multiframe.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is not available for R&amp;S FSL
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_3GBS_WCDP_MULTIFRAME_CAPTURE_LENGTH
        /// RSSPECAN_ATTR_3GBS_WCDP_MULTIFRAME_FRAME_ANALYZE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CDPower:IQLength
        /// [SENSe&lt;1|2&gt;:]CDPower:FRAMe[:VALue]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Capture_Length">
        /// Sets the number of frames that are captured by one sweep.
        /// 
        /// Valid Values:
        /// R&amp;S FSU / R&amp;S FSP-B70 (free run): [1  to  2]
        /// R&amp;S FSU / R&amp;S FSP-B70 (ext. trig): [1  to  3]
        /// R&amp;S FSQ: [1  to  100]
        /// R&amp;S FSQ with B100: [1 to 1000]
        /// R&amp;S FSQ with B100 and B102: [1 to 3000]
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GBS_WCDP_MULTIFRAME_CAPTURE_LENGTH
        /// </param>
        /// <param name="Frame_To_Analyze">
        /// Sets  the frame number to be analyzed.
        /// 
        /// Valid Values:
        /// no checking
        /// 0  to  CAPTURE_LENGTH - 1
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GBS_WCDP_MULTIFRAME_FRAME_ANALYZE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Configure3GPPBSMultiframe(int Capture_Length, int Frame_To_Analyze)
        {
            int pInvokeResult = PInvoke.Configure3GPPBSMultiframe(this._handle, Capture_Length, Frame_To_Analyze);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the 3GPP FDD BS Multicarrier
        /// 
        /// Note(s):
        /// 
        /// (1) This function is not available for R&amp;S FSL
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_3GBS_WCDP_MCACLR
        /// RSSPECAN_ATTR_3GBS_WCDP_AUTO_SCALE
        /// 
        /// Remote-control command(s):
        /// CONFigure:WCDPower[:BTS]:MCARier:STATe ON | OFF
        /// CONFigure:WCDPower[:BTS]:ASCale:STATe ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Multicarrier_State">
        /// Switches multicarrier state on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GBS_WCDP_MCACLR
        /// 
        /// Note(s):
        /// (1) On - Adjust reference level considering multi carrier conditions.
        /// 
        /// (2) Off - Adjust reference level considering single carrier conditions
        /// </param>
        /// <param name="Autoscale">
        /// Switches the autoscale state  on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GBS_WCDP_AUTO_SCALE
        /// 
        /// Note(s):
        /// (1) This control has effect only if Multicarrier State is set On.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Configure3GPPBSMulticarrier(bool Multicarrier_State, bool Autoscale)
        {
            int pInvokeResult = PInvoke.Configure3GPPBSMulticarrier(this._handle, System.Convert.ToUInt16(Multicarrier_State), System.Convert.ToUInt16(Autoscale));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the 3GPP FDD BS results.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_3GBS_RESULTS_SELECT_CHAN
        /// RSSPECAN_ATTR_3GBS_RESULTS_SELECT_CPICH_SLOT
        /// RSSPECAN_ATTR_3GBS_RESULTS_SFAC
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:CODE
        /// [SENSe:]CDPower:SLOT
        /// [SENSe:]CDPower:SFACtor 4 | 8 | 16 | 32 | 64 | 128 | 256 | 512
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Channel">
        /// Sets the code number. The code number refers to code class 9 (spreading factor 512).
        /// 
        /// Valid Values:
        /// 0 to 511
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GBS_RESULTS_SELECT_CHAN
        /// </param>
        /// <param name="CPICH_Slot">
        /// Sets sets the slot number of the common pilot channels (CPICH). 
        /// 
        /// Valid Values:
        /// 0 to 14
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GBS_RESULTS_SELECT_CPICH_SLOT
        /// </param>
        /// <param name="Spreading_Factor">
        /// Defines the spreading factor.
        /// 
        /// Valid Values:
        /// 4, 8, 16, 32, 64, 128, 256, 512
        /// 
        /// Default Value: 512
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GBS_RESULTS_SFAC
        /// 
        /// Note:
        /// (1) This control is only significant for display mode PEAK CODE DOMAIN ERROR.
        /// 
        /// (2) This control has no effect for R&amp;S FSL.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Configure3GPPBSResults(int Channel, int CPICH_Slot, int Spreading_Factor)
        {
            int pInvokeResult = PInvoke.Configure3GPPBSResults(this._handle, Channel, CPICH_Slot, Spreading_Factor);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the 3GPP FDD BS result type.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_3GBS_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FEED
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Result_Type">
        /// Selects the measured data that are to be displayed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_3GBS_RES_CDP       (0) - Code Domain Power
        /// RSSPECAN_VAL_3GBS_RES_CDPRAT    (1) - Code Domain Power Ratio
        /// RSSPECAN_VAL_3GBS_RES_CDEP      (2) - Code Domain Error Power
        /// RSSPECAN_VAL_3GBS_RES_SUM       (3) - Result Summary
        /// RSSPECAN_VAL_3GBS_RES_CTAB      (4) - Channel Assignment Table
        /// RSSPECAN_VAL_3GBS_RES_PCD       (5) - Peak Code Domain Error
        /// RSSPECAN_VAL_3GBS_RES_MACC      (6) - Composite Error Vector Magnitude
        /// RSSPECAN_VAL_3GBS_RES_PVSLA     (7) - Power vs Slot Absolute
        /// RSSPECAN_VAL_3GBS_RES_PVSLR     (8) - Power vs Slot Relative
        /// RSSPECAN_VAL_3GBS_RES_BSTR      (9) - Bit Stream
        /// RSSPECAN_VAL_3GBS_RES_SYMBCONST (10) - Symbol Constellation
        /// RSSPECAN_VAL_3GBS_RES_SYMBEVM   (11) - Symbol Error Vector Magnitude
        /// RSSPECAN_VAL_3GBS_RES_FVSL      (12) - Frequency Error Versus Slot
        /// RSSPECAN_VAL_3GBS_RES_PVSY      (13) - Power Versus Symbol
        /// RSSPECAN_VAL_3GBS_RES_EVM_MAGN  (14) - Symbol Magnitude Error
        /// RSSPECAN_VAL_3GBS_RES_EVM_PHAS  (15) - Symbol Phase Error 
        /// RSSPECAN_VAL_3GBS_RES_COMP_CONS (16) - Composite Constellation
        /// RSSPECAN_VAL_3GBS_RES_CHIP_EVM  (17) - EVM vs. Chip
        /// RSSPECAN_VAL_3GBS_RES_CHIP_MAGN (18) - Magnitude Error vs. Chip
        /// RSSPECAN_VAL_3GBS_RES_CHIP_PHAS (19) - Phase Error vs. Chip
        /// RSSPECAN_VAL_3GBS_RES_PSVSL     (20) - Phase Discontinuity Versus Slot
        /// RSSPECAN_VAL_3GBS_RES_CDPOVER   (21) - Code Domain Power Overview
        /// 
        /// Default Value: 
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GBS_RESULT
        /// 
        /// Note(s):
        /// 
        /// (1) Code Domain Peak - Result display of code domain power as bargraph
        /// 
        /// (2) Code Domain Power Ratio - Result display of code domain power ratio as bargraph
        /// 
        /// (3) Code Domain Error Power - Result display of code domain error power as bar graph
        /// 
        /// (4) Result Summary - Result display in tabular form 
        /// 
        /// (5) Channel Assignment Table - Result display of channel assignment table
        /// 
        /// (6) Peak Code Domain Error - Result display of peak code domain error
        /// 
        /// (7) Composite Error Vector Magnitude - Result display of composite EVM (error vector magnitude referred to the overall signal)
        /// 
        /// (8) Power vs Slot Absolute - Result display of power versus slot (absolute)
        /// 
        /// (9) Power vs Slot Relative - Result display of power versus slot (relative)
        /// 
        /// (10) Bit Stream - Result display of bit stream
        /// 
        /// (11) Symbol Constellation - Result display of symbol constellation
        /// 
        /// (12) Error Vector Magnitude - Result display of symbol error vector magnitude
        /// 
        /// (13) Frequency Error Versus Slot - Result display of frequency versus slot
        /// 
        /// (14) Power Versus Symbol - Result display of of power versus symbol 
        /// 
        /// (15) Symbol Magnitude Error - Result display of symbol magnitude error.
        /// 
        /// (16) Symbol Phase Error - Result display of symbol phase error.
        /// 
        /// (17) Composite Constellation - Result display of composite constellation
        /// 
        /// (18) EVM vs. Chip - Result display of vector error data 
        /// 
        /// (19) Magnitude Error vs. Chip - Result display of magnitude error data 
        /// 
        /// (20) Phase Error vs. Chip - Result display of phase error data
        /// 
        /// (21) For R&amp;S FSL with option K72 are valid only values RSSPECAN_VAL_3GBS_RES_CDP, RSSPECAN_VAL_3GBS_RES_CDPRAT, RSSPECAN_VAL_3GBS_RES_SUM, RSSPECAN_VAL_3GBS_RES_CTAB 
        /// 
        /// (22) Phase Discontinuity Versus Slot - Result display of phase discontinuity versus slot. The softkey is available in frame mode of FS-K73. Only available for R&amp;S FSV instrument.
        /// 
        /// (23) Code Domain Power Overview - Result display of code domain power (both I and Q component) as bargraph. Only available for R&amp;S FSV instrument.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Configure3GPPBSResultType(int Result_Type)
        {
            int pInvokeResult = PInvoke.Configure3GPPBSResultType(this._handle, Result_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the view of the Code Domain Result Summary.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_3GBS_RESULT_SUM_VIEW
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CDPower:RSUMmary NORMal| EXTended
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="View">
        /// This control sets the view of the Code Domain Result Summary.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_3GPP_RES_SUM_NORMAL   (0) - Normal 
        /// RSSPECAN_VAL_3GPP_RES_SUM_EXTENDED (1) - Extended
        /// 
        /// Default Value: RSSPECAN_VAL_3GPP_RES_SUM_NORMAL
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GBS_RESULT_SUM_VIEW
        /// 
        /// Notes:
        /// 
        /// (1) Normal - some of the most important results are shown in the table on the R&amp;S FSL display..
        /// 
        /// (2) Extended - all available result are shown in the table on the R&amp;S FSL display.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Configure3GPPBSResultSummaryView(int View)
        {
            int pInvokeResult = PInvoke.Configure3GPPBSResultSummaryView(this._handle, View);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the scaling of the y-axis in dB.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the R&amp;S FSL and FS-K72 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_3GPP_PDIV
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1..3&gt;:Y[:SCALe]:PDIVision
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Y_Scale_Division">
        /// This control sets the scaling of the y-axis in dB.
        /// 
        /// Valid Values: 0.01 to 100.0 dB
        /// 
        /// Default Value: 7.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GPP_PDIV
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Configure3GPPBSYScaleDivision(double Y_Scale_Division)
        {
            int pInvokeResult = PInvoke.Configure3GPPBSYScaleDivision(this._handle, Y_Scale_Division);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts the following sequence:
        /// - Adjusts the reference level.
        /// - Searches automatically the scrambling code that leads to the highest signal power and stores it
        /// as new scrambling code.
        /// - Changes into the Result Summary result display.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only with R&amp;S FSL with FS-K72  option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_3GPP_ASEQ
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CDPower:ASEQuence
        /// *OPC
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Set3GPPBSASequence()
        {
            int pInvokeResult = PInvoke.Set3GPPBSASequence(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function searches automatically the scrambling code that leads to the highest signal power and stores it as new scrambling code for further measurements.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only with R&amp;S FSL with FS-K72  option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_3GPP_SCODE_SEARCH
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CDPower:LCODe:SEARch[:IMMediate]
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Search3GPPBSScramblingCode()
        {
            int pInvokeResult = PInvoke.Search3GPPBSScramblingCode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function performs the selected operation with channel table.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is not available for R&amp;S FSL
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_3GBS_WCDP_CTAB
        /// RSSPECAN_ATTR_3GBS_WCDP_CTAB_SEL
        /// RSSPECAN_ATTR_3GBS_WCDP_CTAB_NAME
        /// RSSPECAN_ATTR_3GBS_WCDP_CTAB_DEL
        /// RSSPECAN_ATTR_3GBS_WCDP_CTAB_COPY
        /// 
        /// 
        /// Remote-control command(s):
        /// CONFigure:WCDPower[:BTS]:CTABle[:STATe] ON | OFF
        /// CONFigure:WCDPower[:BTS]:CTABle:SELect
        /// CONFigure:WCDPower[:BTS]:CTABle:NAME
        /// CONFigure:WCDPower[:BTS]:CTABle:DELete
        /// CONFigure:WCDPower[:BTS]:CTABle:COPY
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Operation">
        /// This control selects the type of memory operation to be performed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_CDP_TABLE_OFF    (0) - Off
        /// RSSPECAN_VAL_CDP_TABLE_SELECT (1) - Select Table File
        /// RSSPECAN_VAL_CDP_TABLE_NEW    (2) - New/Select
        /// RSSPECAN_VAL_CDP_TABLE_DELETE (3) - Delete
        /// RSSPECAN_VAL_CDP_TABLE_COPY   (4) - Copy
        /// 
        /// 
        /// Notes:
        /// 
        /// (1) Off
        /// Switches the channel table Off
        /// 
        /// (2) Select Table File:
        /// Selects a channel table file.
        /// 
        /// (3) New/Select
        /// Selects an existing channel table or creates the name of a new channel table.
        /// 
        /// (4) Delete:
        /// Deletes the selected channel table.
        /// 
        /// (5) Copy
        /// Copies active channel table onto another one.
        /// </param>
        /// <param name="Channel_Table">
        /// This control selects a predefined channel table file.
        /// 
        /// Valid Values: up to 8 characters
        /// 
        /// Default Value: "RECENT"
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Configure3GPPFDDBSChannelTableOperations(int Operation, string Channel_Table)
        {
            int pInvokeResult = PInvoke.Configure3GPPFDDBSChannelTableOperations(this._handle, Operation, Channel_Table);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the values of the selected channel table in the code domain power measurement mode. Eight values are specified for a line of table.
        /// 
        /// Prior to this function, the name of the channel table has to be defined with function Configure 3GPP FDD BS WCDP Channel Table Operations (Configure3GPPFDDBSChannelTableOperations)
        /// 
        /// Note(s):
        /// 
        /// (1) This function is not available for R&amp;S FSL
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// CONFigure:WCDPower[:BTS]:CTABle:DATA
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Array_Size">
        /// Sets the size of arryas.
        /// 
        /// Valid Values: &gt;0
        /// 
        /// Default Value: 1
        /// 
        /// Note(s):
        /// (1) All arrays has to have this number of items.
        /// </param>
        /// <param name="Code_Class">
        /// Sets array of code class of channel.
        /// 
        /// Valid Values: 2 to 9
        /// 
        /// 
        /// </param>
        /// <param name="Code_Number">
        /// Sets array of code number.
        /// 
        /// Valid Values: 0 to 511
        /// 
        /// 
        /// </param>
        /// <param name="TFCI">
        /// Sets array of the TFCI using.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Not used
        /// VI_TRUE  (1) - Used
        /// 
        /// </param>
        /// <param name="Timing_Offset">
        /// Sets array of timing offset.
        /// 
        /// Valid Values: 
        /// 0 to 38400 - for code class 9, the step width is 512
        /// 256 - otherwise
        /// 
        /// 
        /// </param>
        /// <param name="Pilot_Length">
        /// Sets array pilot length of channel DPCCH.
        /// 
        /// Valid Values: not checked
        /// 
        /// code class 9: 4
        /// code class 8: 2, 4, 8
        /// code class 7: 4, 8
        /// code class 5/6: 8
        /// code class 2/3/4 16
        /// </param>
        /// <param name="Channel_Type">
        /// Sets array of channel type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_CDP_TABLE_CHANNEL_DPCH (0): DPCH Dedicated Physical Channel of a standard WCDMA Frame
        /// RSSPECAN_VAL_CDP_TABLE_CHANNEL_PICH (1): PICH Paging Indication Channel
        /// RSSPECAN_VAL_CDP_TABLE_CHANNEL_SCCPCH (2): SCCPCH Secondary Common Control Physical Channel
        /// RSSPECAN_VAL_CDP_TABLE_CHANNEL_HSSCCH (3): HS_SCCH HSDPA: High Speed Shared Control Channel
        /// RSSPECAN_VAL_CDP_TABLE_CHANNEL_HSPDSCH (4): HS_PDSCH HSDPA: High Speed Physical Downlink Shared Channel
        /// RSSPECAN_VAL_CDP_TABLE_CHANNEL_CHAN (5): CHAN any other QPSK modulated channel without pilot symbols
        /// RSSPECAN_VAL_CDP_TABLE_CHANNEL_CPRSD (10): CPRSD Dedicated Physical Channel (DPCH) in compressed mode
        /// RSSPECAN_VAL_CDP_TABLE_CHANNEL_CPRTPC (11): CPR-TPC DPCH in compressed mode TPC symbols are sent in the first slot of the gap.
        /// RSSPECAN_VAL_CDP_TABLE_CHANNEL_CPRSF (12): CPR-SF/2 DPCH in compressed mode using half spreading factor (SF/2).
        /// RSSPECAN_VAL_CDP_TABLE_CHANNEL_CPRSFTPC (13): CPR-SF/2-TPC DPCH in compressed mode using half spreading factor (SF/2).TPC symbols are sent in the first slot of the gap.
        /// RSSPECAN_VAL_CDP_TABLE_CHANNEL_EHICH (14): E-HICH: Enhanced HARQ Hybrid Acknowledgement Indicator Channel, E-RGCH: Enhanced Relative Grant Channel
        /// RSSPECAN_VAL_CDP_TABLE_CHANNEL_EAGCH (15): EAGCH E-AGCH: Enhanced Absolute Grant Channel
        /// RSSPECAN_VAL_CDP_TABLE_CHANNEL_SCPICH (16): Secondary Common Pilot Channel
        /// 
        /// Note:
        /// (1) Channels PICH, CPICH and PCCPCH may only be defined once. If channel CPICH or PCCPCH is missing in the command, it is automatically added at the end of the table.
        /// </param>
        /// <param name="Status">
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </param>
        /// <param name="CDP_Relative">
        /// Sets array of CDP relative.
        /// 
        /// Valid Values: no checking
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Configure3GPPFDDBSChannelTableData(int Array_Size, int[] Code_Class, int[] Code_Number, ushort[] TFCI, int[] Timing_Offset, int[] Pilot_Length, int[] Channel_Type, ushort[] Status, double[] CDP_Relative)
        {
            int pInvokeResult = PInvoke.Configure3GPPFDDBSChannelTableData(this._handle, Array_Size, Code_Class, Code_Number, TFCI, Timing_Offset, Pilot_Length, Channel_Type, Status, CDP_Relative);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the marker to selected channel
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_3GBS_MARKER_TO_CHANNEL
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:FUNCtion:CPICh
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:FUNCtion:PCCPch
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Channel">
        /// Specifies the type of marker search type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_3GPP_MARKER_CPICH  (0) - CPICH
        /// RSSPECAN_VAL_3GPP_MARKER_PCCPCH (1) - PCCPCH
        /// 
        /// Default Value:  RSSPECAN_VAL_3GPP_MARKER_CPICH (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GBS_MARKER_TO_CHANNEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Set3GPPBSMarkerToChannel(int Window, int Channel)
        {
            int pInvokeResult = PInvoke.Set3GPPBSMarkerToChannel(this._handle, Window, Channel);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the marker to selected channel
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_3GBS_MARKER_CPICH
        /// RSSPECAN_ATTR_3GBS_MARKER_PCCP
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:FUNCtion:CPICh
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:FUNCtion:PCCPch
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Channel">
        /// Specifies the type of marker search type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_3GPP_MARKER_CPICH  (0) - CPICH
        /// RSSPECAN_VAL_3GPP_MARKER_PCCPCH (1) - PCCPCH
        /// 
        /// Default Value:  RSSPECAN_VAL_3GPP_MARKER_CPICH (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Set3GPPBSDeltaMarkerToChannel(int Window, int Channel)
        {
            int pInvokeResult = PInvoke.Set3GPPBSDeltaMarkerToChannel(this._handle, Window, Channel);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the 3GPP FDD BS additional measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_3GBS_MEAS
        /// 
        /// Remote-control command(s):
        /// CONFigure&lt;1&gt;:WCDPower[:BTS]:MEASurement POWer | ACLR| MCAClr | ESPectrum | OBWidth | WCDPower | CCDF | RFCombi | FDOM | TDOM
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement">
        /// Selects the 3GPP FDD UE additional measurement..
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_3GPPBS_MEAS_POW  (0) - Power
        /// RSSPECAN_VAL_3GPPBS_MEAS_ACLR (1) - ACLR
        /// RSSPECAN_VAL_3GPPBS_MEAS_ESP  (2) - Spectrum Emission Mask
        /// RSSPECAN_VAL_3GPPBS_MEAS_OBW  (3) - Occupied Bandwidth
        /// RSSPECAN_VAL_3GPPBS_MEAS_WCDP (4) - Code Domain Power
        /// RSSPECAN_VAL_3GPPBS_MEAS_CCDF (5) - CCDF
        /// RSSPECAN_VAL_3GPPBS_MEAS_MCAC (6) - Multi Carrier ACP
        /// RSSPECAN_VAL_3GPPBS_MEAS_RFC  (7) - RF Combination
        /// RSSPECAN_VAL_3GPPBS_MEAS_FDOM (8) - FDOMain
        /// RSSPECAN_VAL_3GPPBS_MEAS_TDOM (9) - TDOMain
        /// 
        /// Default Value: RSSPECAN_VAL_3GPPBS_MEAS_WCDP (4)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GBS_MEAS
        /// 
        /// Note(s):
        /// 
        /// (1) Power - Channel power measurement (standard 3GPP 3GPP FDD Forward) with predefined settings
        /// 
        /// (2) ACLR - Adjacent channel power measurement (standard 3GPP 3GPP FDD Forward) with predefined settings
        /// 
        /// (3) Spectrum Emission Mask - Measurement of spectrum emission mask
        /// 
        /// (4) Occupied Bandwidth - Measurement of occupied power bandwidth
        /// 
        /// (5) Code Domain Power - This selection has the same effect as command INSTrument:SELect WCDPower.
        /// 
        /// (6) CCDF - Measurement of Complementary Cumulative Distribution
        /// Function
        /// 
        /// (7) Multi Carrier ACP - selects the 3GPP FDD base station test
        /// for measurement of multi carrier adjacent channel leakage power ratio (MCACLR).
        /// 
        /// (8) RF Combination - RF combination measurement of adjacent channel leackage error (ACLR), spectrum emission mask (SEM) and occupied bandwidth (OBW).
        /// 
        /// (9) FDOMain - Overview measurement in the frequency domain with predefined settings.
        /// 
        /// (10) TDOMain - Measurement of crest factor in the time domain with predefined settings.
        /// 
        /// (11) For R&amp;S FSL are valid only values RSSPECAN_VAL_3GPPBS_MEAS_POW, RSSPECAN_VAL_3GPPBS_MEAS_ACLR, RSSPECAN_VAL_3GPPBS_MEAS_ESP, RSSPECAN_VAL_3GPPBS_MEAS_WCDP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Set3GPPBSMeasurement(int Measurement)
        {
            int pInvokeResult = PInvoke.Set3GPPBSMeasurement(this._handle, Measurement);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines a comment for the selected channel table in the code domain power measurement mode.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is not available for R&amp;S FSL
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_3GBS_WCDP_CTAB_HEAD_COMM
        /// 
        /// Remote-control command(s):
        /// CONFigure:WCDPower[:BTS]:CTABle:COMMent
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Channel_Table_Comment">
        /// This control defines a comment for the selected channel table.
        /// 
        /// Valid Values: up to 128 characters
        /// 
        /// Default Value: "Comment for table 1"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GBS_WCDP_CTAB_HEAD_COMM
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Configure3GPPFDDBSChannelTableComment(string Channel_Table_Comment)
        {
            int pInvokeResult = PInvoke.Configure3GPPFDDBSChannelTableComment(this._handle, Channel_Table_Comment);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the names of all channel tables stored on the harddisk in the code domain power measurement mode.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is not available for R&amp;S FSL
        /// 
        /// Attribute(s):
        /// - 
        /// 
        /// Remote-control command(s):
        /// CONFigure:WCDPower[:BTS]:CTABle:CATalog?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Number_of_Channel_Tables">
        /// This control returns number of channel tables, stored on harddisk.
        /// 
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the string you specify for the catalog list parameter.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 0
        /// 
        /// </param>
        /// <param name="Channel_Tables_List">
        /// This control returns array of channel tables, stored on harddisk.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// if you pass 0 for buffer Size, you can use VI_NULL for this parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Query3GPPFDDBSChannelTableCatalog(out int Number_of_Channel_Tables, int Buffer_Size, System.Text.StringBuilder Channel_Tables_List)
        {
            int pInvokeResult = PInvoke.Query3GPPFDDBSChannelTableCatalog(this._handle, out Number_of_Channel_Tables, Buffer_Size, Channel_Tables_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns data that represent the values in array.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;[:DATA]? TRACE1 | TRACE2 | ABITstream | CWCDp | TPVSlot | CTABle | PWCDp 
        /// FORMat[:DATA] REAL,32
        /// *CLS
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_3GBS_TRAC_CDP       (0) - Code Domain Power
        /// RSSPECAN_VAL_3GBS_TRAC_CDPRAT    (1) - Code Domain Power Ratio
        /// RSSPECAN_VAL_3GBS_TRAC_CDEP      (2) - Code Domain Error Power
        /// RSSPECAN_VAL_3GBS_TRAC_CTAB      (3) - Channel Assignment Table
        /// RSSPECAN_VAL_3GBS_TRAC_CWCD      (4) - Code Power (CWCD)
        /// RSSPECAN_VAL_3GBS_TRAC_PWCD      (5) - Code Power (PWCD)
        /// RSSPECAN_VAL_3GBS_TRAC_PCD       (6) - Peak Code Domain Error
        /// RSSPECAN_VAL_3GBS_TRAC_MACC      (7) - Composite Error Vector Magnitude
        /// RSSPECAN_VAL_3GBS_TRAC_PVSLA     (8) - Power vs Slot Absolute
        /// RSSPECAN_VAL_3GBS_TRAC_PVSLR     (9) - Power vs Slot Relative
        /// RSSPECAN_VAL_3GBS_TRAC_BSTR      (10) - Bit Stream
        /// RSSPECAN_VAL_3GBS_TRAC_SYMBCONST (11) - Symbol Constellation
        /// RSSPECAN_VAL_3GBS_TRAC_SYMBEVM   (12) - Symbol Error Vector Magnitude
        /// RSSPECAN_VAL_3GBS_TRAC_FVSL      (13) - Frequency Error Versus Slot
        /// RSSPECAN_VAL_3GBS_TRAC_PVSY      (14) - Power Versus Symbol
        /// RSSPECAN_VAL_3GBS_TRAC_EVM_MAGN  (15) - Symbol Magnitude Error
        /// RSSPECAN_VAL_3GBS_TRAC_EVM_PHAS  (16) - Symbol Phase Error 
        /// RSSPECAN_VAL_3GBS_TRAC_COMP_CONS (17) - Composite Constellation
        /// RSSPECAN_VAL_3GBS_TRAC_CHIP_EVM  (18) - EVM vs. Chip
        /// RSSPECAN_VAL_3GBS_TRAC_CHIP_MAGN (19) - Magnitude Error vs. Chip
        /// RSSPECAN_VAL_3GBS_TRAC_CHIP_PHAS (20) - Phase Error vs. Chip
        /// RSSPECAN_VAL_3GBS_TRAC_ATRACE2   (21) - Frequency Error Versus Slot Result
        /// 
        /// Default Value: RSSPECAN_VAL_3GBS_TRAC_CWCD (4)
        /// </param>
        /// <param name="Array_Length">
        /// Number of array points requested.
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Actual_Points">
        /// Number of points actually returned from the instrument.
        /// </param>
        /// <param name="Values">
        /// A user allocated buffer into which the trace values are stored.
        /// 
        /// Returned Values:
        /// 
        /// CODE PWR ABSOLUTE / RELATIVE, CHANNEL TABLE (PWCDp)
        /// PWCPp can be set if CODE PWR ABSOLUTE / RELATIVE, CHANNEL TABLE is selected for
        /// trace 1. The pilot length is transmitted in addition to the values as transmitted for trace 1. The
        /// pilot length is specified in symbols.
        /// Six values are transmitted for each channel:
        /// &lt; class&gt;,&lt;channel number&gt;,&lt;absolute level&gt;,&lt;relative level&gt;,&lt;timing offset&gt; or &lt;I/Q-mapping&gt;,
        /// &lt;pilot length&gt;,...
        /// 
        /// CODE PWR ABSOLUTE / RELATIVE, CHANNEL TABLE (CTABle)
        /// CTABle can be set if CODE PWR ABSOLUTE / RELATIVE, CHANNEL TABLE is selected for trace 1. The pilot length and the channel state (active, inactive) are transmitted in addition to the
        /// values transmitted for trace 1. The pilot length is specified in symbols. 
        /// Seven values are transmitted for each channel:
        /// &lt; class&gt;,&lt;channel number&gt;,&lt;absolute level&gt;,&lt;relative level&gt;,&lt;timing offset&gt;,&lt;pilot length&gt;,
        /// &lt;active|inactive&gt;...
        /// 
        /// CODE PWR ABSOLUTE / RELATIVE, CHANNEL TABLE (CWCDp)
        /// CWCDp can be set if CODE PWR ABSOLUTE / RELATIVE, CHANNEL TABLE is selected for trace 1. The pilot length, channel state, channel type, modulation type and a reserved value are
        /// transmitted in addition to the values transmitted for trace 1. For each channel 10 values are tranmitted.The range of channel types 13. &lt;code class&gt;,&lt;channel number&gt;,&lt;absolute level&gt;,&lt;relative level&gt;, &lt;timing offset&gt;, &lt;pilot length&gt;, &lt;active flag&gt;, &lt;channel type&gt;, &lt;modulation type&gt;, &lt;reserved&gt;...
        /// The channels are output in ascending order sorted by code number, i.e. in the same sequence they are displayed on screen.
        /// No. Parameter Range Unit Explanation 
        /// 1) &lt;code class&gt; {2  to  9} [1] Code class of the channel.
        /// 2) &lt;channel number&gt; {0  to  511} [1] Code number of the channel.
        /// 3) &lt;absolute level&gt; {-inf  to  inf} [dBm] Absolute level of the code channel at the selected channel slot. (The channel slot
        /// can be marked by the SELECTED CPICH slot.)
        /// 4) &lt; relative level &gt; {-inf  to  inf} [dB] Relative level of the code channel at the selected channel slot referenced to CPICH
        /// or total power. (The channel slot can be marked by the SELECTED CPICH slot.)
        /// 5) &lt;timing offset&gt; {0  to  38400} [chips] Timing offset of the code channel to the frame start. The value is measured in
        /// chips. The step width is 256 chips in the case of code class 2 to 8, and 512 chips in the case of code class 9.
        /// 6) &lt;pilot length&gt; {0,2,4,8,16} [symbols] Pilot length of the code channel. According to the 3GPP standard, the pilot length
        /// range depends on the code class.
        /// 7) &lt;active flag&gt; {0,1} [1] Flag to indicate whether a channel is active
        /// 0 - channel not active
        /// 1 - channel active
        /// 
        /// 8) &lt;channel type&gt; {0  to  13} 
        /// 0 - DPCH Dedicated Physical Channel of a standard frame
        /// 1 - PICH Paging Indication Channel
        /// 2 - CPICH Common Pilot Channel
        /// 3 - PSCH Primary Synchronization Channel
        /// 4 - SSCH Secondary Synchronization Channel
        /// 5 - PCCPCH Primary Common Control Physical Channel
        /// 6 - SCCPCH Secondary Common Control Physical Channel
        /// 7 - HS_SCCH HSDPA: High Speed Shared Control Channel
        /// 8 - HS_PDSCH HSDPA: High Speed Physical Downlink Shared Channel
        /// 9 - CHAN Channel without any pilot symbols (QPSK modulated)
        /// 10 - CPRSD Dedicated Physical Channel in compressed mode
        /// 11 - CPR-TPC Dedicated Physical Channel in compressed mode
        /// TPC symbols are sent in the first slot of the gap.
        /// 12 - CPR-SF/2 Dedicated Physical Channel in compressed mode using half spreading factor (SF/2).
        /// 13 - CPR-SF/2-TPC Dedicated Physical Channel in compressed mode using half spreading factor (SF/2).
        /// TPC symbols are sent in the first slot of the gap.
        /// 14 - EHICH-ERGCH E-HICH: Enhanced HARQ Hybrid Acknowledgement
        /// Indicator Channel E-RGCH: Enhanced Relative Grant Channel
        /// 15 - EAGCH E-AGCH: Enhanced Absolute Grant Channel
        /// 16 - SCPICH SCPICH: Secondary Common Pilot Channel
        /// 
        /// 9) &lt;modulation type&gt;
        /// {2,4,15} [1] Modulation type of the code channel at the selected
        /// channel slot. (The channel slot can be marked by
        /// adjusting SELECT CPICH slot.)
        /// 2 - QPSK Modulation type QPSK.
        /// 4 - 16QAM Modulation type 16QAM.
        /// 15 - NONE There is no power in the selected channel slot [slot is switched OFF]. (According to 3GPP, the power of an
        /// HSDPA channel can be switched every 2 ms, i.e. 3 slots.)
        /// 10) &lt;reserved&gt; {0} [1] Reserved for future functionality.
        /// 
        /// POWER VS SLOT (TPVSlot)
        /// Sixteen (16) pairs of slots (slot number of CPICH) and level values (for 16 slots) are always transferred.
        /// The query is possible no matter which evaluation is selected in the code domain analyzer.
        /// &lt;slot number&gt;, &lt;level value in dBm&gt;, &lt;slot number&gt;, &lt;level value in dBm&gt;, ....
        /// 
        /// BITSTREAM (ABITstream)
        /// ABITstream can be set only if CALC2:FEED "XTIM:CDP:BSTReam" is selected (in the lower bit
        /// stream window). This command returns the bit streams of all 15 slots one after the other. The output
        /// format may be REAL, UINT or ASCII. The number of bits of a 16QAM-modulated channel is twice that of a QPSK-modulated channel. The output format is equal to that of the ":TRAC1:DATA? TRAC2" command in case of an activated bitstream display. The only difference is the number of symbols which are evaluated. The ABITSTREAM command evaluates all symbols of one frame. One value is transferred per bit (range
        /// 0,1,). Each symbol contains of two (QPSK) or four (16QAM) consecutive bits. The number of symbols is not constant and may vary depending on the selected channel and its symbol modulation
        /// type. The bit stream may contain invalid (symbols without power). 
        /// If the analyzer is equipped with R&amp;S FS-K72 only, the values and numbers of the bits are as follows:
        /// Unit: []
        /// Range: {0,1,6,9}
        /// Bits per symbol: NBitPerSymb = 2
        /// Number of symbols: NSymb = 150*2(8-Code Class)
        /// Number of bits: NBit = NSymb * NBitPerSymb
        /// Format: Bit00,Bit01,Bit10,Bit11,Bit20,Bit21,.... ,
        /// Bit NSymb 0,Bit NSymb 1
        /// Explanation: 0 - Low state of a transmitted bit
        /// 1 - High state of a transmitted bit
        /// 6 - Bit of a symbol of a suppressed slot of a DPCH
        /// in Compressed Mode (DPCH-CPRSD)
        /// 9 - Bit of a suppressed symbol of a DPCH
        /// (e.g. TFCI off)
        /// If the analyzer is equipped additionally with R&amp;S FS-K74, the values and numbers of the bits are as follows:
        /// Unit: []
        /// Range: {0,1,6,7,8,9}
        /// Bits per symbol: NBitPerSymb = {2,4}
        /// Max. bits per symbol: 
        /// Symbols per slot: NSymb_Slot = 10*2(8-Code Class)
        /// Symbols per frame: NSymb_Frame = 15* NSymb_Slot = 150*2(8-Code Class)
        /// Number of bits: NBit = NSymb_Frame * NBitPerSymb_MAX
        /// Format (QPSK): Bit00,Bit01,Bit10,Bit11,Bit20,Bit21, .... ,
        /// Bit NSymb_Frame 0,Bit NSymb_Frame 1
        /// Format (16QAM): Bit00,Bit01,Bit02,Bit03,Bit10,Bit11,Bit12,Bit13,.... ... ,
        /// Bit NSymb_Frame 0,Bit NSymb_Frame 1,Bit NSymb_Frame 2,
        /// Bit NSymb_Frame 3
        /// Explanation: 0 - Low state of a transmitted bit
        /// 1 - High state of a transmitted bit
        /// 6 - Bit of a symbol of a suppressed slot of a DPCH in
        /// Compressed Mode (DPCH-CPRSD)
        /// 7 - Bit of a switched-off symbol of an HS-PDSCH channel
        /// 8 - Reserved value of a QPSK symbol bit in a frame which
        /// includes also 16QAM symbols.
        /// 9 - Bit of a suppressed symbol of a DPCH
        /// (e.g. TFCI off)
        /// The values 7 and 8 are only used in case of a varying modulation type of an HS-PDSCH channel. In this case the number of bits per symbol (NBitPerSymb) varies too. However, the length of the transmitted bit vector (NBit) depends only of the maximum number of bits per symbol in that frame. That means, if the modulation type changes throughout the frame this will not influence the number of biots being transmitted.
        /// 
        /// FREQUENCY ERROR VS SLOT (ATRACe2)
        /// Returns a list of slot number / absolute frequency error vs slot for all slots.
        /// Output: List of slot number and absolute frequency error values of all slots
        /// Format: SlotNumber 0, FreqError 0, ...., SlotNumber 14, FreqError 14
        /// Unit: [Hz]
        /// Quantity: 15
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Get3GPPBSTrace(int Window, int Trace, int Array_Length, out int Actual_Points, double[] Values)
        {
            int pInvokeResult = PInvoke.Get3GPPBSTrace(this._handle, Window, Trace, Array_Length, out Actual_Points, Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the WCDP measurement Results
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1&gt;:MARKer&lt;1&gt;:FUNCtion:WCDPower[:BTS]:RESult? PTOTal | FERRor | TFRame | TOFFset | MACCuracy | PCDerror | EVMRms | EVMPeak | CERRor | CSLot | SRATe | CHANnel | CDPabsolute | CDPRelative | IQOFfset | IQIMbalance | MTYPe | RHO | PSYMbol | ACHannels | MPIC | RCDerror | ARCDerror | RCDQam64
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Type">
        /// This control sets the result.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WCDP_RES_PTOT (0) - Total Power
        /// RSSPECAN_VAL_WCDP_RES_FERR (1) - Frequency Error
        /// RSSPECAN_VAL_WCDP_RES_TFR  (2) - Trigger to Frame
        /// RSSPECAN_VAL_WCDP_RES_MACC (3) - Composite EVM
        /// RSSPECAN_VAL_WCDP_RES_PCD  (4) - Error Peak Code Domain Error
        /// RSSPECAN_VAL_WCDP_RES_EVMR (5) - Error Vector Magnitude RMS
        /// RSSPECAN_VAL_WCDP_RES_EVMP (6) - Error Vector Magnitude Peak
        /// RSSPECAN_VAL_WCDP_RES_CERR (7) - Chip Rate Error
        /// RSSPECAN_VAL_WCDP_RES_CSL  (8) - Channel Slot Number
        /// RSSPECAN_VAL_WCDP_RES_SRAT (9) - Symbol Rate
        /// RSSPECAN_VAL_WCDP_RES_CHAN (10) - Channel Number
        /// RSSPECAN_VAL_WCDP_RES_CDPA (11) - Channel Power Absolute
        /// RSSPECAN_VAL_WCDP_RES_CDPR (12) - Channel Power Relative
        /// RSSPECAN_VAL_WCDP_RES_IQOF (13) - IQ Offset
        /// RSSPECAN_VAL_WCDP_RES_IQIM (14) - IQ Imbalance
        /// RSSPECAN_VAL_WCDP_RES_MTYP (15) - Modulation Type
        /// RSSPECAN_VAL_WCDP_RES_RHO  (16) - RHO Value
        /// RSSPECAN_VAL_WCDP_RES_TOFF (17) - Timing Offset
        /// RSSPECAN_VAL_WCDP_RES_PSYM (18) - Number of pilot bits
        /// RSSPECAN_VAL_WCDP_RES_ACH  (19) - Number of active channels
        /// RSSPECAN_VAL_WCDP_RES_MPIC (20) - Average Power of Inactive Codes
        /// RSSPECAN_VAL_WCDP_RES_RCDE (21) - Relative Code Domain Error
        /// RSSPECAN_VAL_WCDP_RES_ARCD (22) - Average Relative Code Domain Error
        /// RSSPECAN_VAL_WCDP_RES_RCDQ (23) - Average RCDE for 64QAM
        /// 
        /// Default Value: RSSPECAN_VAL_CDP_RES_PTOT (0)
        /// </param>
        /// <param name="Result">
        /// Returns the measured result. 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Get3GPPBSResult(int Type, out double Result)
        {
            int pInvokeResult = PInvoke.Get3GPPBSResult(this._handle, Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the power result.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:POWer:RESult? ACPower | CPOWer | MCACpower | OBANdwidth
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Type">
        /// This control sets the result.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_POW_RES_ACP  (0) - Adjacent Channel Power
        /// RSSPECAN_VAL_POW_RES_CPOW (1) - Channel Power
        /// RSSPECAN_VAL_POW_RES_MCAC (2) - Multi Carrier Signal
        /// RSSPECAN_VAL_POW_RES_OBAN (3) - Occupied Bandwidth
        /// 
        /// Default Value: RSSPECAN_VAL_POW_RES_ACP  (0)
        /// </param>
        /// <param name="Array_Size">
        /// Defines the memory size allocated for Results. 
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 12
        /// </param>
        /// <param name="Results">
        /// Returns the measured results. 
        /// 
        /// Returned Values:
        /// 
        /// Adjacent Channel Power:
        /// 1. Power of transmission channel
        /// 2. Power of lower adjacent channel
        /// 3. Power of upper adjacent channel
        /// 4. Power of lower alternate channel 1
        /// 5. Power of upper alternate channel 1
        /// 6. Power of lower alternate channel 2
        /// 7. Power of upper alternate channel 2
        /// The number of measured values returned depends on the number of
        /// adjacent/alternate channels selected with SENSe:POWer:ACHannel:ACPairs.
        /// 
        /// Channel Power:
        /// With logarithmic scaling (RANGE LOG), the channel power is output in the currently selected level unit; with linear scaling (RANGE LIN dB or LIN %), the channel power is output in W.
        /// 
        /// Multi Carrier: 
        /// Power of carrier signal 1
        /// Power of carrier signal 2
        /// Power of carrier signal 3
        /// Power of carrier signal 4
        /// Power of carrier signal 5
        /// Power of carrier signal 6
        /// Power of carrier signal 7
        /// Power of carrier signal 8
        /// Power of carrier signal 9
        /// Power of carrier signal 10
        /// Power of carrier signal 11
        /// Power of carrier signal 12
        /// Total power of all carrier signals
        /// Power of lower adjacent channel
        /// Power of upper adjacent channel
        /// Power of lower alternate channel 1
        /// Power of upper alternate channel 1
        /// Power of lower alternate channel 2
        /// Power of upper alternate channel 2
        /// 
        /// Occupied Bandwidth:
        /// Occupied bandwidth in Hz is returned.
        /// 
        /// 
        /// </param>
        /// <param name="Returned_Values">
        /// Returns number of returned values.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Get3GPPBSCDPResult(int Type, int Array_Size, double[] Results, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.Get3GPPBSCDPResult(this._handle, Type, Array_Size, Results, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns result table of the automatic search sequence containing the highest power values calculated and the corresponding scrambling codes.
        /// 
        /// Notes:
        /// 
        /// (1) This function is available only with R&amp;S FSL or R&amp;S FSV with FS-K72.
        /// 
        /// Attribute(s):
        /// no attribute 
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CDPower:LCODe:SEARch:LIST?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Array_Size">
        /// Defines the number of items in arrays Scrambilng Code and  Power.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Scrambing_Code">
        /// Returns scrambling code values.
        /// 
        /// </param>
        /// <param name="Power">
        /// Returns power of selected channels.
        /// 
        /// </param>
        /// <param name="Returned_Values">
        /// Returned number of values readed from instrument.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Get3GPPBSCDPScramblingCodeResult(int Array_Size, int[] Scrambing_Code, double[] Power, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.Get3GPPBSCDPScramblingCodeResult(this._handle, Array_Size, Scrambing_Code, Power, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the list evaluation results. All results are float values.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only with R&amp;S FSL or FSV with FS-K72.
        /// 
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRACe1:DATA? LIST
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="No_Of_Values">
        /// This control sets the number of values returned in arrays.
        /// The array size has to be at least equal to this number.
        /// 
        /// Valid Values: &gt; 0
        /// 
        /// Default Value: 10
        /// 
        /// </param>
        /// <param name="Range_Number">
        /// Returns an array of range numbers
        /// 
        /// 
        /// </param>
        /// <param name="Start_Frequency">
        /// Returns an array of range start frequency
        /// 
        /// </param>
        /// <param name="Stop_Frequency">
        /// Returns an array of range stop frequency.
        /// </param>
        /// <param name="Resolution_Bandwidth">
        /// Returns an array of resolution bandwidth
        /// </param>
        /// <param name="Frequency">
        /// Returns an array of rrequency of the peak in the range
        /// 
        /// 
        /// </param>
        /// <param name="Power_Abs">
        /// Returns an array of absolute peak power of the range in dBm
        /// 
        /// </param>
        /// <param name="Power_Rel">
        /// Reserved (0.0)
        /// </param>
        /// <param name="Delta">
        /// Returns an array of delta of the peak power to the limit line in dB
        /// </param>
        /// <param name="Limit_Check">
        /// Returns an array of limit check state (0 = PASSED, 1 = FAILED)
        /// </param>
        /// <param name="Reserved1">
        /// Reserved (0.0)
        /// 
        /// 
        /// </param>
        /// <param name="Reserved2">
        /// Reserved (0.0)
        /// 
        /// </param>
        /// <param name="Returned_Values">
        /// Returns number of value sets (frequency, level and delta to limit line) read (available to return) from the instrument. Number of values returned in arrays is limited with the parameter No of Values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Get3GPPBSCDPListEvaluationResult(int No_Of_Values, double[] Range_Number, double[] Start_Frequency, double[] Stop_Frequency, double[] Resolution_Bandwidth, double[] Frequency, double[] Power_Abs, double[] Power_Rel, double[] Delta, int[] Limit_Check, double[] Reserved1, double[] Reserved2, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.Get3GPPBSCDPListEvaluationResult(this._handle, No_Of_Values, Range_Number, Start_Frequency, Stop_Frequency, Resolution_Bandwidth, Frequency, Power_Abs, Power_Rel, Delta, Limit_Check, Reserved1, Reserved2, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the instrument into 3GPP FDD UE mode. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_3GUE_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument[:SELect] MWCDpower
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Set3GPPUEMode()
        {
            int pInvokeResult = PInvoke.Set3GPPUEMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the 3GPP FDD UE measurement
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_3GUE_CDP_SCR_CODE
        /// RSSPECAN_ATTR_3GUE_CDP_SCR_TYPE
        /// RSSPECAN_ATTR_3GUE_CDP_BASE
        /// RSSPECAN_ATTR_3GUE_CDP_SLOT
        /// RSSPECAN_ATTR_3GUE_CDP_SIDEBAND
        /// RSSPECAN_ATTR_3GUE_CDP_NORM
        /// RSSPECAN_ATTR_3GUE_RRC_FILTER
        /// RSSPECAN_ATTR_3GUE_CDP_HSDPAUPA_STATE
        /// RSSPECAN_ATTR_3GUE_CDP_ELIMINATE_TAIL_CHIPS
        /// RSSPECAN_ATTR_3GUE_CDP_INACT_CHAN_THR
        /// RSSPECAN_ATTR_3GUE_CDP_INVERT_Q
        /// RSSPECAN_ATTR_3GUE_CDP_EVM_MEAS_INTERVAL
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:LCODe[:VALue]
        /// [SENSe:]CDPower:LCODe:TYPE LONG | SHORt
        /// [SENSe:]CDPower:BASE
        /// [SENSe:]CDPower:SLOT 0 to 14
        /// [SENSe:]CDPower:SBANd NORMal | INVers
        /// [SENSe:]CDPower:NORMalize ON | OFF
        /// [SENSe:]CDPower:FILTer ON|OFF
        /// [SENSe:]CDPower:HSDPamode ON|OFF
        /// [SENSe:]CDPower:ETCHips ON|OFF
        /// [SENSe:]CDPower:ICTReshold
        /// [SENSe:]CDPower:QINVert ON | OFF
        /// [SENSe:]CDPower:EINTerval SLOT | MEAS | FHALf | SHALf
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Scrambling_Code">
        /// Sets the scrambling code.
        /// 
        /// Valid Values:
        /// 0 to 8191
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GUE_CDP_SCR_CODE
        /// </param>
        /// <param name="Scrambling_Type">
        /// Switches between long and short scrambling code.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_3GUE_CDP_SCR_LONG  (0) - Long
        /// RSSPECAN_VAL_3GUE_CDP_SCR_SHORT (1) - Short
        /// 
        /// Default Value: RSSPECAN_VAL_3GUE_CDP_SCR_LONG  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GUE_CDP_SCR_TYPE
        /// </param>
        /// <param name="Measurement_Base">
        /// Sets the base of the CDP analysis.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_3GUE_CDP_BASE_SLOT  (0) - Slot
        /// RSSPECAN_VAL_3GUE_CDP_BASE_FRAME (1) - Frame
        /// 
        /// Default Value: RSSPECAN_VAL_3GUE_CDP_BASE_SLOT  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GUE_CDP_BASE
        /// </param>
        /// <param name="Slot">
        /// Sets the slot number.
        /// 
        /// Valid Values:
        /// 0 to 14
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GUE_CDP_SLOT
        /// 
        /// Note:
        /// (1) This control is valid only if Measurement Base is set to Frame
        /// 
        /// 
        /// </param>
        /// <param name="Side_Band">
        /// Sets interchanging the left and the right sideband.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_3GPP_SBAND_NORM (0) - Normal
        /// RSSPECAN_VAL_3GPP_SBAND_INV  (1) - Inverse
        /// 
        /// Default Value: RSSPECAN_VAL_3GPP_SBAND_NORM  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GUE_CDP_SIDEBAND
        /// </param>
        /// <param name="Normalize">
        /// Switches elimination of IQ offset on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GUE_CDP_NORM
        /// </param>
        /// <param name="RRC_Filter">
        /// Switches the RRC filter ON and OFF. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GUE_RRC_FILTER
        /// </param>
        /// <param name="HSDPA_UPA">
        /// Switches the HSUPA/HSDPA channel detection On or Off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GUE_CDP_HSDPAUPA_STATE
        /// </param>
        /// <param name="Eliminate_Tail_Chips">
        /// Selects length of the measurement interval for calculation of error vector magnitude (EVM).
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GUE_CDP_ELIMINATE_TAIL_CHIPS
        /// </param>
        /// <param name="Inactive_Channel_Treshold">
        /// Sets the threshold value from which a channel is treated as active. The level entered refers to the total signal power.
        /// 
        /// Units: dB
        /// 
        /// Valid Values: -100.0 to 10.0
        /// 
        /// Default Value: -60.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GUE_CDP_INACT_CHAN_THR
        /// 
        /// </param>
        /// <param name="Invert_Q">
        /// Inverts the Q component of the signal.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GUE_CDP_INVERT_Q
        /// </param>
        /// <param name="EVM_Meas_Interval">
        /// Sets the mode of determining the average interval of the root mean square (RMS) calculation for error vector magnitude (EVMrms) versus slot. The softkey influences the display of COMPOSITE EVM (RMS).
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_3GUE_CDP_EVM_INT_SLOT (0) - Slot
        /// RSSPECAN_VAL_3GUE_CDP_EVM_INT_MEAS (1) - Meas
        /// RSSPECAN_VAL_3GUE_CDP_EVM_INT_FHAL (2) - FHalf
        /// RSSPECAN_VAL_3GUE_CDP_EVM_INT_SHAL (3) - SHalf
        /// 
        /// Default Value: RSSPECAN_VAL_3GUE_CDP_EVM_INT_SLOT (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GUE_CDP_EVM_MEAS_INTERVAL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Configure3GPPUEMeasurement(int Scrambling_Code, int Scrambling_Type, int Measurement_Base, int Slot, int Side_Band, bool Normalize, bool RRC_Filter, bool HSDPA_UPA, bool Eliminate_Tail_Chips, double Inactive_Channel_Treshold, bool Invert_Q, int EVM_Meas_Interval)
        {
            int pInvokeResult = PInvoke.Configure3GPPUEMeasurement(this._handle, Scrambling_Code, Scrambling_Type, Measurement_Base, Slot, Side_Band, System.Convert.ToUInt16(Normalize), System.Convert.ToUInt16(RRC_Filter), System.Convert.ToUInt16(HSDPA_UPA), System.Convert.ToUInt16(Eliminate_Tail_Chips), Inactive_Channel_Treshold, System.Convert.ToUInt16(Invert_Q), EVM_Meas_Interval);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the 3GPP FDD BS Multiframe.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_3GUE_CDP_MULTIFRAME_CAPTURE_LENGTH
        /// RSSPECAN_ATTR_3GUE_CDP_MULTIFRAME_FRAME_ANALYZE
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CDPower:IQLength
        /// [SENSe&lt;1|2&gt;:]CDPower:FRAMe[:VALue]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Capture_Length">
        /// Sets the number of frames that are captured by one sweep.
        /// 
        /// Valid Values:
        /// R&amp;S FSU / R&amp;S FSP-B70 (free run): [1  to  2]
        /// R&amp;S FSU / R&amp;S FSP-B70 (ext. trig): [1  to  3]
        /// R&amp;S FSQ: [1  to  100]
        /// R&amp;S FSQ with B100: [1 to 1000]
        /// R&amp;S FSQ with B100 and B102: [1 to 3000]
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GUE_CDP_MULTIFRAME_CAPTURE_LENGTH
        /// </param>
        /// <param name="Frame_To_Analyze">
        /// Sets  the frame number to be analyzed.
        /// 
        /// Valid Values:
        /// no checking
        /// 0  to  CAPTURE_LENGTH - 1
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GUE_WCDP_MULTIFRAME_FRAME_ANALYZE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Configure3GPPUEMultiframe(int Capture_Length, int Frame_To_Analyze)
        {
            int pInvokeResult = PInvoke.Configure3GPPUEMultiframe(this._handle, Capture_Length, Frame_To_Analyze);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the 3GPP FDD UE results.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_3GUE_CDP_SEL_COMP
        /// RSSPECAN_ATTR_3GUE_CDP_CHAN_CODE
        /// RSSPECAN_ATTR_3GUE_CDP_SFAC
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:MAPPing
        /// [SENSe:]CDPower:CODE
        /// [SENSe:]CDPower:SFACtor 4 | 8 | 16 | 32 | 64 | 128 | 256
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Select_I_Q">
        /// Switches between I and Q component of the signal.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_I (0) - I
        /// RSSPECAN_VAL_Q (1) - Q
        /// 
        /// Default Value: RSSPECAN_VAL_I (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GUE_CDP_SEL_COMP
        /// </param>
        /// <param name="Channel">
        /// Sets the code number. The code number refers to code class 8 (spreading factor 256).
        /// 
        /// Valid Values:
        /// 0 to 255
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GUE_CDP_CHAN_CODE
        /// </param>
        /// <param name="Spreading_Factor">
        /// Sets defines the spreading factor. The code number refers to code class 8 (spreading factor 256).
        /// 
        /// Valid Values:
        /// 4, 8, 16, 32, 64, 128, 256
        /// 
        /// Default Value: 256
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GUE_CDP_SFAC
        /// 
        /// Note:
        /// (1) This control is only significant for display mode PEAK CODE DOMAIN ERROR.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Configure3GPPUEResults(int Select_I_Q, int Channel, int Spreading_Factor)
        {
            int pInvokeResult = PInvoke.Configure3GPPUEResults(this._handle, Select_I_Q, Channel, Spreading_Factor);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the 3GPP FDD UE result type.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_3GUE_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FEED
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Result_Type">
        /// Selects the measured data that are to be displayed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_3GUE_RES_CDP       (0) - Code Domain Power
        /// RSSPECAN_VAL_3GUE_RES_CDPRAT    (1) - Code Domain Power Ratio
        /// RSSPECAN_VAL_3GUE_RES_CDPOVER   (2) - Code Domain Power Overview
        /// RSSPECAN_VAL_3GUE_RES_CDEP      (3) - Code Domain Error Power
        /// RSSPECAN_VAL_3GUE_RES_SUM       (4) - Result Summary
        /// RSSPECAN_VAL_3GUE_RES_CTAB      (5) - Channel Assignment Table
        /// RSSPECAN_VAL_3GUE_RES_PCD       (6) - Peak Code Domain Error
        /// RSSPECAN_VAL_3GUE_RES_MACC      (7 - Composite Error Vector Magnitude
        /// RSSPECAN_VAL_3GUE_RES_PVSLA     (8) - Power vs Slot Absolute
        /// RSSPECAN_VAL_3GUE_RES_PVSLR     (9) - Power vs Slot Relative
        /// RSSPECAN_VAL_3GUE_RES_BSTR      (10) - Bit Stream
        /// RSSPECAN_VAL_3GUE_RES_PVSY      (11) - Power Versus Symbol
        /// RSSPECAN_VAL_3GUE_RES_SYMBCONST (12) - Symbol Constellation
        /// RSSPECAN_VAL_3GUE_RES_SYMBEVM   (13) - Symbol Error Vector Magnitude
        /// RSSPECAN_VAL_3GUE_RES_FVSL      (14) - Frequency Error Versus Slot
        /// RSSPECAN_VAL_3GUE_RES_PSVSL     (15) - Phase Discontinuity Versus Slot
        /// RSSPECAN_VAL_3GUE_RES_EVM_MAGN  (16) - Symbol Magnitude Error
        /// RSSPECAN_VAL_3GUE_RES_EVM_PHAS  (17) - Symbol Phase Error 
        /// RSSPECAN_VAL_3GUE_RES_COMP_CONS (18) - Composite Constellation
        /// RSSPECAN_VAL_3GUE_RES_CHIP_EVM  (19) - EVM vs. Chip
        /// RSSPECAN_VAL_3GUE_RES_CHIP_MAGN (20) - Magnitude Error vs. Chip
        /// RSSPECAN_VAL_3GUE_RES_CHIP_PHAS (21) - Phase Error vs. Chip 
        /// 
        /// Default Value: 
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GUE_RESULT
        /// 
        /// Note(s):
        /// (1) Code Domain Power - Result display of code domain power as bargraph
        /// 
        /// (2) Code Domain Power Ratio - Result display of code domain power ratio as bargraph
        /// 
        /// (3) Code Domain Power Overview - Result display of code domain power (both I and Q component) as bargraph
        /// 
        /// (4) Code Domain Error Power - Result display of code domain error power as bargraph.
        /// 
        /// (5) Result Summary - Result display in tabular form 
        /// 
        /// (6) Channel Assignment Table - Result display of channel assignment table
        /// 
        /// (7) Peak Code Domain Error - Result display of peak code domain error. Frame mode only.
        /// 
        /// (8) Composite Error Vector Magnitude - Result display of composite EVM (error vector magnitude referred to the overall signal) The softkey is available in frame mode of FS-K73.
        /// 
        /// (9) Power vs Slot Absolute - Result display of power versus slot (absolute) The softkey is available in frame mode of FS-K73.
        /// 
        /// (10) Power vs Slot Relative - Result display of power versus slot (relative) The softkey is available in frame mode of FS-K73.
        /// 
        /// (11) Bit Stream - Result display of bit stream
        /// 
        /// (12) Power Versus Symbol - Result display of power versus symbol
        /// 
        /// (13) Symbol Constellation - Result display of symbol constellation
        /// 
        /// (14) Error Vector Magnitude - Result display of symbol error vector magnitude
        /// 
        /// (15) Frequency Error Versus Slot - Result display of frequency error versus slot. The softkey is available in frame mode of FS-K73.
        /// 
        /// (16) Phase Discontinuity Versus Slot - Result display of phase discontinuity versus slot. The softkey is available in frame mode of FS-K73.
        /// 
        /// (17) Symbol Magnitude Error - Result display of symbol magnitude error.
        /// 
        /// (18) Symbol Phase Error - Result display of symbol phase error.
        /// 
        /// (19) Composite Constellation - Result display of composite constellation
        /// 
        /// (20) EVM vs. Chip - Result display of vector error data 
        /// 
        /// (21) Magnitude Error vs. Chip - Result display of magnitude error data 
        /// 
        /// (22) Phase Error vs. Chip - Result display of phase error data
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Configure3GPPUEResultType(int Result_Type)
        {
            int pInvokeResult = PInvoke.Configure3GPPUEResultType(this._handle, Result_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function performs the selected operation with channel table.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_3GUE_WCDP_CTAB
        /// RSSPECAN_ATTR_3GUE_WCDP_CTAB_SEL
        /// RSSPECAN_ATTR_3GUE_WCDP_CTAB_NAME
        /// RSSPECAN_ATTR_3GUE_WCDP_CTAB_DEL
        /// RSSPECAN_ATTR_3GUE_WCDP_CTAB_COPY
        /// 
        /// Remote-control command(s):
        /// CONFigure&lt;1&gt;:WCDPower:MS:CTABle[:STATe] ON | OFF
        /// CONFigure&lt;1&gt;:WCDPower:MS:CTABle:SELect
        /// CONFigure&lt;1&gt;:WCDPower:MS:CTABle:NAME
        /// CONFigure&lt;1&gt;:WCDPower:MS:CTABle:DELete
        /// CONFigure&lt;1&gt;:WCDPower:MS:CTABle:COPY
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Operation">
        /// This control selects the type of memory operation to be performed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_CDP_TABLE_OFF    (0) - Off
        /// RSSPECAN_VAL_CDP_TABLE_SELECT (1) - Select Table File
        /// RSSPECAN_VAL_CDP_TABLE_NEW    (2) - New/Select
        /// RSSPECAN_VAL_CDP_TABLE_DELETE (3) - Delete
        /// RSSPECAN_VAL_CDP_TABLE_COPY   (4) - Copy
        /// 
        /// 
        /// Notes:
        /// 
        /// (1) Off
        /// Switches the channel table Off
        /// 
        /// (2) Select Table File:
        /// Selects a channel table file.
        /// 
        /// (3) New/Select:
        /// Selects an existing channel table or creates the name of a new channel table.
        /// 
        /// (4) Delete:
        /// Deletes the selected channel table.
        /// 
        /// (5) Copy
        /// Copies active channel table onto another one.
        /// </param>
        /// <param name="Channel_Table">
        /// This control selects a predefined channel table file.
        /// 
        /// Valid Values: up to 8 characters
        /// 
        /// Default Value: "RECENT"
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_3GUE_WCDP_CTAB
        /// RSSPECAN_ATTR_3GUE_WCDP_CTAB_SEL
        /// RSSPECAN_ATTR_3GUE_WCDP_CTAB_NAME
        /// RSSPECAN_ATTR_3GUE_WCDP_CTAB_DEL
        /// RSSPECAN_ATTR_3GUE_WCDP_CTAB_COPY
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Configure3GPPFDDUEChannelTableOperations(int Operation, string Channel_Table)
        {
            int pInvokeResult = PInvoke.Configure3GPPFDDUEChannelTableOperations(this._handle, Operation, Channel_Table);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the values of the selected channel table in the code domain power measurement mode. Six values are specified for a line of table.
        /// 
        /// Prior to this function, the name of the channel table has to be defined with function Configure 3GPP FDD UE CDP Channel Table Operations (Configure3GPPFDDUEChannelTableOperations).
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// CONFigure:WCDPower:MS:CTABle:DATA
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Array_Size">
        /// Sets the size of arryas.
        /// 
        /// Valid Values: &gt;0
        /// 
        /// Default Value: 1
        /// 
        /// Note(s):
        /// (1) All arrays has to have this number of items.
        /// </param>
        /// <param name="Code_Class">
        /// Sets code class of channel.
        /// 
        /// Valid Values: 2 to 8
        /// 
        /// Default Value: None
        /// 
        /// </param>
        /// <param name="Code_Number">
        /// Sets code number.
        /// 
        /// Valid Values: 0 to 255
        /// 
        /// Default Value: None
        /// 
        /// </param>
        /// <param name="Channel_Mapping">
        /// Sets the channel mapping.
        /// 
        /// Valid Values:
        /// 0 = I branch
        /// 1 = Q branch
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Pilot_Length">
        /// Sets pilot length of channel DPCCH.
        /// 
        /// Valid Values: -1,3,4,5,6,7,8
        /// 
        /// Default Value: None
        /// 
        /// </param>
        /// <param name="Status">
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </param>
        /// <param name="CDP_Relative">
        /// Sets CDP relative.
        /// 
        /// Valid Values: no checking
        /// 
        /// Default Value: None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Configure3GPPFDDUEChannelTableData(int Array_Size, int[] Code_Class, int[] Code_Number, int[] Channel_Mapping, int[] Pilot_Length, ushort[] Status, double[] CDP_Relative)
        {
            int pInvokeResult = PInvoke.Configure3GPPFDDUEChannelTableData(this._handle, Array_Size, Code_Class, Code_Number, Channel_Mapping, Pilot_Length, Status, CDP_Relative);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates/deactivates the high speed channel in active channel table.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_3GUE_WCDP_CTAB_HSDP
        /// 
        /// Remote-control command(s):
        /// CONFigure:WCDPower:MS:CTABle:DATA:HSDPcch ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="State">
        /// Activates/deactivates the high speed channel.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GUE_WCDP_CTAB_HSDP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Configure3GPPFDDUEChannelTableHighSpeedChannel(bool State)
        {
            int pInvokeResult = PInvoke.Configure3GPPFDDUEChannelTableHighSpeedChannel(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the 3GPP FDD UE additional measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_3GUE_MEAS
        /// 
        /// Remote-control command(s):
        /// CONFigure&lt;1&gt;:WCDPower:MS:MEASurement POWer | ACLR | ESPectrum | OBWidth | CCDF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement">
        /// Selects the 3GPP FDD UE additional measurement..
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_3GPPUE_MEAS_POW  (0) - Power
        /// RSSPECAN_VAL_3GPPUE_MEAS_ACLR (1) - ACLR
        /// RSSPECAN_VAL_3GPPUE_MEAS_ESP  (2) - Spectrum Emission Mask
        /// RSSPECAN_VAL_3GPPUE_MEAS_OBW  (3) - Occupied Bandwidth
        /// RSSPECAN_VAL_3GPPUE_MEAS_WCDP (4) - Code Domain Power
        /// RSSPECAN_VAL_3GPPUE_MEAS_CCDF (5) - CCDF
        /// 
        /// Default Value: RSSPECAN_VAL_3GPPUE_MEAS_WCDP (4)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GUE_MEAS
        /// 
        /// Note(s):
        /// Power - Channel power measurement (standard 3GPP 3GPP FDD Forward) with predefined settings
        /// 
        /// ACLR - Adjacent channel power measurement (standard 3GPP 3GPP FDD Forward) with predefined settings
        /// 
        /// Spectrum Emission Mask - Measurement of spectrum emission mask
        /// 
        /// Occupied Bandwidth - Measurement of occupied power bandwidth
        /// 
        /// Code Domain Power - This selection has the same effect as command INSTrument:SELect WCDPower.
        /// 
        /// CCDF - Measurement of Complementary Cumulative Distribution
        /// Function
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Set3GPPUEMeasurement(int Measurement)
        {
            int pInvokeResult = PInvoke.Set3GPPUEMeasurement(this._handle, Measurement);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines a comment for the selected channel table in the code domain power measurement mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_3GUE_WCDP_CTAB_COMMENT
        /// 
        /// Remote-control command(s):
        /// CONFigure:WCDPower:MS:CTABle:COMMent
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Channel_Table_Comment">
        /// This control defines a comment for the selected channel table.
        /// 
        /// Valid Values: up to 128 characters
        /// 
        /// Default Value: "Comment for table 1"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_3GUE_WCDP_CTAB_COMMENT
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Configure3GPPFDDUEChannelTableComment(string Channel_Table_Comment)
        {
            int pInvokeResult = PInvoke.Configure3GPPFDDUEChannelTableComment(this._handle, Channel_Table_Comment);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the names of all channel tables stored on the harddisk in the code domain power measurement mode.
        /// 
        /// 
        /// Attribute(s):
        /// - 
        /// 
        /// Remote-control command(s):
        /// CONFigure:WCDPower:MS:CTABle:CATalog?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Number_of_Channel_Tables">
        /// This control returns number of channel tables, stored on harddisk.
        /// 
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the string you specify for the catalog list parameter.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 0
        /// 
        /// </param>
        /// <param name="Channel_Tables_List">
        /// This control returns array of channel tables, stored on harddisk.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// if you pass 0 for buffer Size, you can use VI_NULL for this parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Query3GPPFDDUEChannelTableCatalog(out int Number_of_Channel_Tables, int Buffer_Size, System.Text.StringBuilder Channel_Tables_List)
        {
            int pInvokeResult = PInvoke.Query3GPPFDDUEChannelTableCatalog(this._handle, out Number_of_Channel_Tables, Buffer_Size, Channel_Tables_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the trace as an array of  values. 
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;[:DATA] ABITstream | CWCDp | TPVSlot | CTABle
        /// FORMat[:DATA] REAL,32
        /// *CLS
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_3GUE_TRAC_CDP       (0) - Code Domain Power
        /// RSSPECAN_VAL_3GUE_TRAC_CDPRAT    (1) - Code Domain Power Ratio
        /// RSSPECAN_VAL_3GUE_TRAC_CDPOVER   (2) - Code Domain Power Overview
        /// RSSPECAN_VAL_3GUE_TRAC_CDEP      (3) - Code Domain Error Power
        /// RSSPECAN_VAL_3GUE_TRAC_CTAB      (4) - Channel Assignment Table
        /// RSSPECAN_VAL_3GUE_TRAC_PCD       (5) - Peak Code Domain Error
        /// RSSPECAN_VAL_3GUE_TRAC_MACC      (6) - Composite Error Vector Magnitude
        /// RSSPECAN_VAL_3GUE_TRAC_PVSLA     (7) - Power vs Slot Absolute
        /// RSSPECAN_VAL_3GUE_TRAC_PVSLR     (8) - Power vs Slot Relative
        /// RSSPECAN_VAL_3GUE_TRAC_BSTR      (9) - Bit Stream
        /// RSSPECAN_VAL_3GUE_TRAC_PVSY      (10) - Power Versus Symbol
        /// RSSPECAN_VAL_3GUE_TRAC_SYMBCONST (11) - Symbol Constellation
        /// RSSPECAN_VAL_3GUE_TRAC_SYMBEVM   (12) - Symbol Error Vector Magnitude
        /// RSSPECAN_VAL_3GUE_TRAC_FVSL      (13) - Frequency Error Versus Slot
        /// RSSPECAN_VAL_3GUE_TRAC_PSVSL     (14) - Phase Discontinuity Versus Slot
        /// RSSPECAN_VAL_3GUE_TRAC_EVM_MAGN  (15) - Symbol Magnitude Error
        /// RSSPECAN_VAL_3GUE_TRAC_EVM_PHAS  (16) - Symbol Phase Error 
        /// RSSPECAN_VAL_3GUE_TRAC_COMP_CONS (17) - Composite Constellation
        /// RSSPECAN_VAL_3GUE_TRAC_CHIP_EVM  (18) - EVM vs. Chip
        /// RSSPECAN_VAL_3GUE_TRAC_CHIP_MAGN (19) - Magnitude Error vs. Chip
        /// RSSPECAN_VAL_3GUE_TRAC_CHIP_PHAS (20) - Phase Error vs. Chip 
        /// RSSPECAN_VAL_3GUE_TRAC_EVM_RMS   (21) - RMS Value of EVM
        /// 
        /// Default Value: RSSPECAN_VAL_3GUE_TRAC_BSTR (9)
        /// </param>
        /// <param name="Array_Length">
        /// Number of array points requested.
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Actual_Points">
        /// Number of points actually returned in the Amplitude array and X Array array.
        /// </param>
        /// <param name="Values">
        /// A user allocated buffer into which the trace values are stored.
        /// 
        /// Returned Values:
        /// 
        /// Frame bit-stream:
        /// Unit: []
        /// Range: {0,1,7,9}
        /// Bits per symbol: NBitPerSymb = 2
        /// Number of symbols: NSymb = 150*2(8-Code Class)
        /// Number of bits: NBit = NSymb * NBitPerSymb
        /// Format: Bit00,Bit01,Bit10,Bit11,Bit20,Bit21,.... ,
        /// Bit NSymb 0,Bit NSymb 1
        /// Explanation: 0 - Low state of a transmitted bit
        /// 1 - High state of a transmitted bit
        /// 7 - suppressed symbol of a HS-DPCCH slot
        /// 9 - Bit of an inactive channel
        /// 
        /// 
        /// Code Power:
        /// Can be set if CODE PWR ABSOLUTE / RELATIVE, CHANNEL TABLE is selected for trace 1. The pilot length, channel state, channel type, modulation type and a reserved value
        /// are transmitted in addition to the values transmitted for trace 1. For each channel, 11 values
        /// are transmitted
        /// &lt;code class&gt;, &lt;channel number&gt;, &lt;IQ component&gt;, &lt;absolute level&gt;, &lt;relative level&gt;,&lt;timing offset&gt;,&lt;pilot length&gt;,&lt;active flag&gt;, &lt;channel type&gt;, &lt;modulation type&gt;, &lt;reserved&gt;...
        /// 
        /// No. Parameter Range Unit Explanation
        /// - &lt;code class&gt; {2  to  8} [1] Code class of the channel.
        /// - &lt;channel number&gt; {0  to  255} [1] Code number of the channel.
        /// - &lt;IQ component&gt; {0,1} [1] IQ component of the channel.
        ///    0 - Q component channel symbols (Sn) sent at quadrate component only imaginary part of Sn is used. [Re {Sn} = 0 Im {Sn} &lt;&gt; 0] 
        ///    1 - I component channel symbols (Sn) sent at In phase component only real part of Sn is used. [Re {Sn} &lt;&gt; 0 Im {Sn} = 0]
        ///  - &lt;absolute level&gt; [dBm] Absolute level of the code channel at the selected channel slot. (The channel slot can be marked by the SELECTED CPICH slot.)
        ///  - &lt; relative level &gt; [dB] Relative level of the code channel at the selected channel slot referenced to CPICH
        /// or total power. (The channel slot can be marked by the SELECTED CPICH slot.)
        ///  - &lt;timing offset&gt; {0  to  2560} [chips] Timing offset of the HS-DPCCH to the frame start. The value is measured in
        /// chips. The step width is 256 chips. For all other data channels, the timing offset is zero
        ///  - &lt;pilot length&gt; {0 to 8} [symbols] Pilot length of the DPCCH.
        ///  - &lt;active flag&gt; {0,1} [1] Flag to indicate whether a channel is active
        ///     0 - channel not active
        ///     1 - channel active
        ///  - &lt;channel type&gt; {0  to  2} [1] Channel type indication
        ///     0 - DPDCH Dedicated Physical Data Channel
        ///     1 - DPCCH Dedicated Physical Control Channel
        ///     2 - HS-DPCCH High Speed Dedicated Physical Control Channel
        ///     3 - E-DPCCH Enhanced Dedicated Physical Control Channel
        ///     4 - E-DPDCH Enhanced Dedicated Physical Data Channel
        ///  - &lt;modulation type&gt; {0,1,15} [1] Modulation type of the code channel
        /// 0 - BPSK-I Modulation type BPSK I - Branch
        /// 1 - BPSK-Q Modulation type BPSK Q - Branch
        /// 15 - None no power within the channel slot
        ///  - &lt;reserved&gt; {0} [1] Reserved for future functionality.
        /// 
        /// Total Power Versus Slot:
        /// 15 pairs of slot (slotnumber of CPICH) and level values (for 15 slots) are always transferred. The
        /// query is only possible in frame mode and not in slot mode. But it is possible no matter which
        /// evaluation is selected in the code domain analyzer.
        /// &lt;slot number&gt;, &lt;level value in dBm&gt;,&lt;slot number&gt;,&lt;level value in dBm&gt;,.....
        /// 
        /// RMS Value od EVM:
        /// 15 groups with 6 values per group are returned:
        /// &lt;slot0&gt;,&lt;EVM0&gt;, &lt;BeginMeas0&gt;,&lt;EndMeas0&gt;,&lt;Reserved_A0&gt;,&lt;Reserved_B0&gt;
        /// &lt;slot1&gt;,&lt;EVM1&gt;, &lt;BeginMeas1&gt;,&lt;EndMeas1&gt;,&lt;Reserved_A1&gt;,&lt;Reserved_B1&gt;
        /// ...
        /// &lt;slot14&gt;,&lt;EVM14&gt;, &lt;BeginMeas14&gt;,&lt;EndMeas14&gt;,&lt;Reserved_A14&gt;,&lt;Reserved_B14&gt;
        /// 
        /// Where: &lt;field&gt; [unitf] {range} - explanation
        ///        &lt;slotn&gt; [1] {0 ? 14} - slot number
        ///        &lt;EVMn&gt; [%] {0 ? 100} - RMS value of error vector magnitude
        ///        &lt;BeginMeasn&gt; [chip] {0 ? 1278} - Begin of the measurement interval for EVMrms value
        ///        &lt;EndMeasn&gt; [chip] {0 ? 2559} - End of the measurement interval for EVMrms value
        ///        &lt; Reserved_A n&gt; [] {0} - Reserved for possible additional information in future FW versions
        ///        &lt; Reserved_B n&gt; [] {0} - Reserved for possible additional information in future FW versions
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Get3GPPUETrace(int Window, int Trace, int Array_Length, out int Actual_Points, double[] Values)
        {
            int pInvokeResult = PInvoke.Get3GPPUETrace(this._handle, Window, Trace, Array_Length, out Actual_Points, Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the measured and calculated results of the 3GPP FDD code domain power measurement.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:FUNCtion:WCDPower:MS:RESult? PTOTal | FERRor | TFRame | MACCuracy | PCDerror | EVMRms | EVMPeak | CERRor | CSLot | SRATe | CHANnel | CDPabsolute | CDPRelative | IQOFfset | IQIMbalance | CMAPping | PSYMbol | RHO | TOFFset | EVMBegin | EVMend |MTYPe | ACHannels | MPIC | ARCDerror
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Type">
        /// This control sets the result.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_CDP_RES_PTOT (0) - Total Power
        /// RSSPECAN_VAL_CDP_RES_FERR (1) - Frequency Error
        /// RSSPECAN_VAL_CDP_RES_TFR  (2) - Trigger to Frame
        /// RSSPECAN_VAL_CDP_RES_MACC (3) - Composite EVM
        /// RSSPECAN_VAL_CDP_RES_PCD  (4) - Error Peak Code Domain Error
        /// RSSPECAN_VAL_CDP_RES_EVMR (5) - Error Vector Magnitude RMS
        /// RSSPECAN_VAL_CDP_RES_EVMP (6) - Error Vector Magnitude Peak
        /// RSSPECAN_VAL_CDP_RES_CERR (7) - Chip Rate Error
        /// RSSPECAN_VAL_CDP_RES_CSL  (8) - Channel Slot Number
        /// RSSPECAN_VAL_CDP_RES_SRAT (9) - Symbol Rate
        /// RSSPECAN_VAL_CDP_RES_CHAN (10) - Channel Number
        /// RSSPECAN_VAL_CDP_RES_CDPA (11) - Channel Power Absolute
        /// RSSPECAN_VAL_CDP_RES_CDPR (12) - Channel Power Relative
        /// RSSPECAN_VAL_CDP_RES_IQOF (13) - IQ Offset
        /// RSSPECAN_VAL_CDP_RES_IQIM (14) - IQ Imbalance
        /// RSSPECAN_VAL_CDP_RES_CMAP (15) - Channel Component
        /// RSSPECAN_VAL_CDP_RES_PSYM (16) - Number of Pilot Bits
        /// RSSPECAN_VAL_CDP_RES_RHO  (17) - RHO Value
        /// RSSPECAN_VAL_CDP_RES_TOFF (18) - Timing Offset
        /// RSSPECAN_VAL_CDP_RES_EVMB (19) - Begin of EVM (RMS)
        /// RSSPECAN_VAL_CDP_RES_EVM  (20) - End of EVM (RMS)
        /// RSSPECAN_VAL_CDP_RES_MTYP (21) - Modulation Type
        /// RSSPECAN_VAL_CDP_RES_ACH  (22) - Number of active channels
        /// RSSPECAN_VAL_CDP_RES_MPIC (23) - Average Power of Inactive Codes
        /// RSSPECAN_VAL_CDP_RES_ARCD (24) - 4PAM Average of residual code domain errors
        /// 
        /// Default Value: RSSPECAN_VAL_CDP_RES_PTOT (0)
        /// </param>
        /// <param name="Result">
        /// Returns the measured result. 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Get3GPPUEResult(int Type, out double Result)
        {
            int pInvokeResult = PInvoke.Get3GPPUEResult(this._handle, Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the TD-SCDMA BS (BTDScdma) mode. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDBS_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument[:SELect] BTDScdma
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SetTDSBSMode()
        {
            int pInvokeResult = PInvoke.SetTDSBSMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the measurement of the Application Firmware R&amp;S FS-K76, TD-SCDMA base station test. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDBS_MEAS
        /// 
        /// Remote-control command(s):
        /// CONFigure&lt;1&gt;:CDPower[:BTS]:MEASurement POWer | ACLR | ESPectrum | OBANdwith | PVT | CDPower | CCDF | MCAC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement">
        /// Selects the TDS BS additional measurement..
        /// 
        /// Valid Values:
        /// 
        /// RSSPECAN_VAL_TDS_MEAS_POW   (0) - Power
        /// RSSPECAN_VAL_TDS_MEAS_ACLR  (1) - ACLR
        /// RSSPECAN_VAL_TDS_MEAS_ESP   (2) - Spectrum Emission Mask
        /// RSSPECAN_VAL_TDS_MEAS_OBAN  (3) - Occupied Bandwidth
        /// RSSPECAN_VAL_TDS_MEAS_PVT   (4) - Power Vs. Time
        /// RSSPECAN_VAL_TDS_MEAS_CDP   (5) - Code Domain Analyzer
        /// RSSPECAN_VAL_TDS_MEAS_CCDF  (6) - Signal Statistic
        /// RSSPECAN_VAL_TDS_MEAS_MCAC  (7) - Multi Carrier ACLR
        /// 
        /// Default Value: RSSPECAN_VAL_TDS_MEAS_CDP (5)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_MEAS
        /// 
        /// Note(s):
        /// (1) Power - Channel power measurement (TD-SCDMA forward standard) with predefined settings
        /// 
        /// (2) ACLR - Adjacent channel power measurement (TD-SCDMA forward standard) with predefined settings
        /// 
        /// (3) Spectrum Emission Mask - Check of signal power (spectrum emission mask)
        /// 
        /// (4) Occupied Bandwidth - Measurement of occupied bandwidth
        /// 
        /// (5) Code Domain Power
        /// 
        /// (6) Power Vs. Time - Measurement of power versus time
        /// 
        /// (7) Code Domain Analyzer - Code domain analyzer measurement
        /// 
        /// (8) CCDF - Signal statistic measurements
        /// 
        /// (9) Multi Carrier ACLR - Multi Carrier Adjacent Channel Leakage Power Ratio measurement
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SetTDSBSMeasurement(int Measurement)
        {
            int pInvokeResult = PInvoke.SetTDSBSMeasurement(this._handle, Measurement);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the gated sweep mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDBS_ADAPT_SIGN_SLOT_START
        /// RSSPECAN_ATTR_TDBS_ADAPT_SIGN_SLOT_STOP
        /// 
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:SLOT:STARt
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:SLOT:STOP
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Start_Slot">
        /// Sets the start slot for gated sweep mode.
        /// 
        /// Valid Values:
        /// 1 to 7
        /// 
        /// Default Value: 4
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_ADAPT_SIGN_SLOT_START
        /// </param>
        /// <param name="Stop_Slot">
        /// Sets the stop slot for gated sweep mode.
        /// 
        /// Valid Values:
        /// 1 to 7
        /// 
        /// Default Value: 6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_ADAPT_SIGN_SLOT_STOP
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTDSBSAdaptToSignal(int Start_Slot, int Stop_Slot)
        {
            int pInvokeResult = PInvoke.ConfigureTDSBSAdaptToSignal(this._handle, Start_Slot, Stop_Slot);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the TDS BS capture settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDBS_RESULT_SET_COUNT
        /// RSSPECAN_ATTR_TDBS_RESULT_SET_VALUE
        /// RSSPECAN_ATTR_TDBS_CAPTURE_LENGTH
        /// RSSPECAN_ATTR_TDBS_RESULT_SEL_CHAN
        /// RSSPECAN_ATTR_TDBS_RESULT_SEL_SLOT
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:SET:COUNt
        /// [SENSe:]CDPower:SET[:VALue]
        /// [SENSe:]CDPower:IQLength
        /// [SENSe:]CDPower:CODE
        /// [SENSe:]CDPower:SLOT
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Set_Count">
        /// Sets the number of SETS of 63 slots that shall be captured consecutively into the IQ RAM of the R&amp;S FSQ.
        /// 
        /// Valid Values:
        /// 1 to 190
        /// 1 to 1900 with FSQ-B100 option
        /// 1 to 5700 with FSQ-B100 and FSQ-B102 option
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_RESULT_SET_COUNT
        /// </param>
        /// <param name="Set_Value">
        /// Sets the SET for which the results are evaluated.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_RESULT_SET_VALUE
        /// 
        /// Note:
        /// (1) This control has effect only if Set Count &gt; 1
        /// </param>
        /// <param name="Capture_Length">
        /// Sets the IQ capture length in multiples of slots.
        /// 
        /// Valid Values:
        /// 2 to 63
        /// 
        /// Default Value: 7
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_CAPTURE_LENGTH
        /// 
        /// Note:
        /// (1) This control has effect only if Set Count = 1
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTDSBSCapture(int Set_Count, int Set_Value, int Capture_Length)
        {
            int pInvokeResult = PInvoke.ConfigureTDSBSCapture(this._handle, Set_Count, Set_Value, Capture_Length);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the TDS BS capture settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDBS_RESULT_SET_COUNT
        /// RSSPECAN_ATTR_TDBS_RESULT_SET_VALUE
        /// RSSPECAN_ATTR_TDBS_CAPTURE_LENGTH
        /// RSSPECAN_ATTR_TDBS_RESULT_SEL_CHAN
        /// RSSPECAN_ATTR_TDBS_RESULT_SEL_SLOT
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:SET:COUNt
        /// [SENSe:]CDPower:SET[:VALue]
        /// [SENSe:]CDPower:IQLength
        /// [SENSe:]CDPower:CODE
        /// [SENSe:]CDPower:SLOT
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Set_Count">
        /// Sets the number of SETS of 63 slots that shall be captured consecutively into the IQ RAM of the R&amp;S FSQ.
        /// 
        /// Valid Values:
        /// 1 to 190
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_RESULT_SET_COUNT
        /// </param>
        /// <param name="Set_Value">
        /// Sets the SET for which the results are evaluated.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_RESULT_SET_VALUE
        /// 
        /// Note:
        /// (1) This control has effect only if Set Count &gt; 1
        /// </param>
        /// <param name="Capture_Length">
        /// Sets the IQ capture length in multiples of slots.
        /// 
        /// Valid Values:
        /// 2 to 63
        /// 
        /// Default Value: 7
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_CAPTURE_LENGTH
        /// 
        /// Note:
        /// (1) This control has effect only if Set Count = 1
        /// 
        /// </param>
        /// <param name="Channel">
        /// Sets the code number.
        /// 
        /// Valid Values:
        /// 1 to 16
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_RESULT_SEL_CHAN
        /// </param>
        /// <param name="Slot">
        /// Sets the slot.
        /// 
        /// Valid Values:
        /// no range checking (0 to IQLength-1)
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_RESULT_SEL_SLOT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTDSBSCDACapture(int Set_Count, int Set_Value, int Capture_Length, int Channel, int Slot)
        {
            int pInvokeResult = PInvoke.ConfigureTDSBSCDACapture(this._handle, Set_Count, Set_Value, Capture_Length, Channel, Slot);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the TDS BS results
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDBS_RESULT_TYPE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FEED
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Result_Type">
        /// Selects the measured data that are to be displayed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TDS_RES_CDP       (0) - Code Domain Power
        /// RSSPECAN_VAL_TDS_RES_CDPRAT    (1) - Code Domain Power Ratio
        /// RSSPECAN_VAL_TDS_RES_CDEP      (2) - Code Domain Error Power
        /// RSSPECAN_VAL_TDS_RES_MACC      (3) - Composite EVM
        /// RSSPECAN_VAL_TDS_RES_PVSLA    (4) - Power vs Slot Absolute
        /// RSSPECAN_VAL_TDS_RES_PVSLR     (5) - Power vs Slot Relative
        /// RSSPECAN_VAL_TDS_RES_PVSYM     (6) - Power vs Symbol
        /// RSSPECAN_VAL_TDS_RES_BSTR      (7) - Bit Stream
        /// RSSPECAN_VAL_TDS_RES_SUM       (8) - Summary
        /// RSSPECAN_VAL_TDS_RES_CTAB      (9) - Channel Assignment Table
        /// RSSPECAN_VAL_TDS_RES_PCD       (10) - Peak Code Domain Error
        /// RSSPECAN_VAL_TDS_RES_SYMBCONST (11) - Symbol Constellation
        /// RSSPECAN_VAL_TDS_RES_SYMBEVM   (12) - Symbol Error Vector Magnitude
        /// RSSPECAN_VAL_TDS_RES_COMPCONST (13) - Composite Constellation
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_TDS_RES_CDP       (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_RESULT_TYPE
        /// 
        /// Note(s):
        /// (1) Code Domain Power - Result display of code domain power as bargraph
        /// 
        /// (2) Code Domain Power Ratio - Result display of code domain power ratio as bargraph
        /// 
        /// (3) Code Domain Error Power - Result display of code domain power (both I and Q component) as bargraph
        /// 
        /// (4) Composite EVM - Result display of composite EVM (error vector magnitude referred to the overall signal)
        /// 
        /// (5) Power vs Slot Absolute - Result display of power versus slot (absolute)
        /// 
        /// (6) Power vs Slot Relative - Result display of power versus slot (relative)
        /// 
        /// (7) Power vs Symbol - Result display of power versus symbol
        /// 
        /// (8) Bit Stream - Result display of bit stream
        /// 
        /// (9) Summary - Result display in tabular form 
        /// 
        /// (10) Channel Assignment Table - Result display of channel assignment table
        /// 
        /// (11) Peak Code Domain Error - Result display of peak code domain error
        /// 
        /// (12) Symbol Constellation - Result display of symbol constellation
        /// 
        /// (13) Error Vector Magnitude - Result display of symbol error vector magnitude
        /// 
        /// (14) Composite Constellation - Result display of composite constellation
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTDSBSResults(int Result_Type)
        {
            int pInvokeResult = PInvoke.ConfigureTDSBSResults(this._handle, Result_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts the autorange routine for the reference level. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDBS_ADAPT_SIGN_AUT_LEVEL_TIME
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:AUTO:LTIMe
        /// *WAI
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int TDSBSAdaptSignalAutoLevelTime(uint Timeout)
        {
            int pInvokeResult = PInvoke.TDSBSAdaptSignalAutoLevelTime(this._handle, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function initiates automatic setting of the RF attenuation and IF gain to the level of the applied signal. The unit is put into RF ATTEN MANUAL mode to optimize RF attenuation and IF gain independently of each other. This mode is retained after changing from TD-SCDMA BTS mode to SPECTRUM mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDBS_CDP_LEV_ADJUST
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:LEVel:ADJust
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int TDSBSCDPLevelAdjust(uint Timeout)
        {
            int pInvokeResult = PInvoke.TDSBSCDPLevelAdjust(this._handle, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function performs the selected operation with channel table.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDBS_CTAB_STATE
        /// RSSPECAN_ATTR_TDBS_CTAB_SEL
        /// RSSPECAN_ATTR_TDBS_CTAB_NAME
        /// RSSPECAN_ATTR_TDBS_CTAB_DEL
        /// RSSPECAN_ATTR_TDBS_CTAB_COPY
        /// 
        /// Remote-control command(s):
        /// CONFigure&lt;1&gt;:CDPower[:BTS]:CTABle[:STATe] ON | OFF
        /// CONFigure&lt;1&gt;:CDPower[:BTS]:CTABle:SELect
        /// CONFigure:CDPower[:BTS]:CTABle:NAME
        /// CONFigure:CDPower[:BTS]:CTABle:DELete
        /// CONFigure:CDPower[:BTS]:CTABle:COPY
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Operation">
        /// This control selects the type of memory operation to be performed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TDS_TABLE_OFF    (0) - Off
        /// RSSPECAN_VAL_TDS_TABLE_SELECT (1) - Select Table File
        /// RSSPECAN_VAL_TDS_TABLE_NEW    (2) - New/Select
        /// RSSPECAN_VAL_TDS_TABLE_DELETE (3) - Delete
        /// RSSPECAN_VAL_TDS_TABLE_COPY   (4) - Copy
        /// 
        /// 
        /// Notes:
        /// 
        /// (1) Off
        /// Switches the channel table Off
        /// 
        /// (2) SelectTable File:
        /// Selects a channel table file.
        /// 
        /// (3) New/Select:
        /// Selects an existing channel table or creates the name of a new channel table.
        /// 
        /// (4) Delete:
        /// Deletes the selected channel table.
        /// 
        /// (5) Copy
        /// Copies active channel table onto another one.
        /// </param>
        /// <param name="Channel_Table">
        /// This control selects a predefined channel table file.
        /// 
        /// Valid Values: up to 8 characters
        /// 
        /// Default Value: "RECENT"
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDBS_CTAB_STATE
        /// RSSPECAN_ATTR_TDBS_CTAB_SEL
        /// RSSPECAN_ATTR_TDBS_CTAB_NAME
        /// RSSPECAN_ATTR_TDBS_CTAB_DEL
        /// RSSPECAN_ATTR_TDBS_CTAB_COPY
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int TDSBSChannelTableOperations(int Operation, string Channel_Table)
        {
            int pInvokeResult = PInvoke.TDSBSChannelTableOperations(this._handle, Operation, Channel_Table);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines a comment for the selected channel table in the code domain power measurement mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDBS_CTAB_COMM
        /// 
        /// Remote-control command(s):
        /// CONFigure:CDPower[:BTS]:CTABle:COMMent
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Channel_Table_Comment">
        /// This control defines a comment for the selected channel table.
        /// 
        /// Valid Values: up to 128 characters
        /// 
        /// Default Value: "Comment for table 1"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_CTAB_COMM
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTDSBSChannelTableComment(string Channel_Table_Comment)
        {
            int pInvokeResult = PInvoke.ConfigureTDSBSChannelTableComment(this._handle, Channel_Table_Comment);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines a channel table. The whole table is defined in one operation. The inactive channels (INACtive) do not have to be defined.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// CONFigure:CDPower[:BTS]:CTABle:DATA &lt; Channel type &gt;, &lt;Code class&gt;, &lt;Code number&gt;, &lt;Modulation type&gt;, &lt;Midamble shift&gt;, &lt;Status&gt;, &lt;Reserved 1&gt;, &lt;Reserved 2&gt;, ....
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Array_Size">
        /// Sets the size of arryas.
        /// 
        /// Valid Values: &gt;0
        /// 
        /// Default Value: 1
        /// 
        /// Note(s):
        /// (1) All arrays has to have this number of items.
        /// </param>
        /// <param name="Channel_Type">
        /// Sets array of channel type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TDS_TABLE_CHANNEL_MID    (1) - Midamble
        /// RSSPECAN_VAL_TDS_TABLE_CHANNEL_DPCH   (2) - DPCH
        /// RSSPECAN_VAL_TDS_TABLE_CHANNEL_PCCPCH (3) - P-CCPCH
        /// RSSPECAN_VAL_TDS_TABLE_CHANNEL_SCCPCH (4) - S-CCPCH
        /// RSSPECAN_VAL_TDS_TABLE_CHANNEL_FPACH  (5) - FPACH
        /// RSSPECAN_VAL_TDS_TABLE_CHANNEL_PRACH  (6) - PRACH
        /// 
        /// 
        /// </param>
        /// <param name="Code_Class">
        /// Sets array of code class of channel.
        /// 
        /// Valid Values: 0 to 4
        /// 
        /// 
        /// </param>
        /// <param name="Code_Number">
        /// Sets array of code number.
        /// 
        /// Valid Values: 1 to 16
        /// 
        /// 
        /// </param>
        /// <param name="Modulation_Type">
        /// Sets array of modulation type
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_TDS_MOD_INVALID (0) - invalid (for midamble)
        /// RSSPECAN_VAL_TDS_MOD_QPSK    (1) - QPSK
        /// RSSPECAN_VAL_TDS_MOD_8PSK    (2) - 8PSK
        /// RSSPECAN_VAL_TDS_MOD_16QAM   (3) - 16QAM
        /// 
        /// </param>
        /// <param name="Midamble_Shift">
        /// Sets array of midamble shift.
        /// 
        /// Valid Values: 1 to 16
        /// </param>
        /// <param name="Status">
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </param>
        /// <param name="Reserved_1">
        /// Always 0, reserved for additions
        /// 
        /// Valid Values: 0
        /// 
        /// 
        /// </param>
        /// <param name="Reserved_2">
        /// Always 0, reserved for additions
        /// 
        /// Valid Values: 0
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTDSBSChannelTableData(int Array_Size, int[] Channel_Type, int[] Code_Class, int[] Code_Number, int[] Modulation_Type, int[] Midamble_Shift, ushort[] Status, int[] Reserved_1, int[] Reserved_2)
        {
            int pInvokeResult = PInvoke.ConfigureTDSBSChannelTableData(this._handle, Array_Size, Channel_Type, Code_Class, Code_Number, Modulation_Type, Midamble_Shift, Status, Reserved_1, Reserved_2);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects sorting of the channel table in code order or midamble order.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDBS_CTAB_ORDER
        /// 
        /// Remote-control command(s):
        /// CONFigure&lt;1&gt;:CDPower[:BTS]:CTABle:ORDer CODE | MIDamble
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Order">
        /// Selects sorting of the channel table in code order or midamble order.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TDS_CTAB_ORDER_CODE (0) - Code
        /// RSSPECAN_VAL_TDS_CTAB_ORDER_MID  (1) - Midamble
        /// 
        /// Default Value: RSSPECAN_VAL_TDS_CTAB_ORDER_CODE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_CTAB_ORDER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTDSBSChannelTableOrder(int Order)
        {
            int pInvokeResult = PInvoke.ConfigureTDSBSChannelTableOrder(this._handle, Order);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the maximum number of midamble shifts in the channel table.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDBS_CTAB_MSHIFT
        /// 
        /// Remote-control command(s):
        /// CONFigure:CDPower[:BTS]:CTABle:MSHift 2 | 4 | 6 | 8 | 10 | 12 | 14 | 16
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Maximum_of_Midamble_Shifts">
        /// Sets the maximum number of midamble shifts in the channel table.
        /// 
        /// Valid Values:
        /// 2,4,6,8,10,12,14,16
        /// 
        /// Default Value: 16
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_CTAB_MSHIFT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTDSBSChannelTableMidambleShift(int Maximum_of_Midamble_Shifts)
        {
            int pInvokeResult = PInvoke.ConfigureTDSBSChannelTableMidambleShift(this._handle, Maximum_of_Midamble_Shifts);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the names of all channel tables stored on the harddisk in the code domain power measurement mode.
        /// 
        /// 
        /// Attribute(s):
        /// - 
        /// 
        /// Remote-control command(s):
        /// CONFigure:CDPower[:BTS]:CTABle:CATalog?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Number_of_Channel_Tables">
        /// This control returns number of channel tables, stored on harddisk.
        /// 
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the string you specify for the catalog list parameter.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 0
        /// </param>
        /// <param name="Channel_Tables_List">
        /// This control returns array of channel tables, stored on harddisk.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// if you pass 0 for buffer Size, you can use VI_NULL for this parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetTDSBSChannelTableCatalog(out int Number_of_Channel_Tables, int Buffer_Size, System.Text.StringBuilder Channel_Tables_List)
        {
            int pInvokeResult = PInvoke.GetTDSBSChannelTableCatalog(this._handle, out Number_of_Channel_Tables, Buffer_Size, Channel_Tables_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the TDS BS measurement
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDBS_STANDARD
        /// RSSPECAN_ATTR_TDBS_SCODE
        /// RSSPECAN_ATTR_TDBS_CELL_SHIFTS
        /// RSSPECAN_ATTR_TDBS_SIDE_BAND
        /// RSSPECAN_ATTR_TDBS_NORMALIZE
        /// RSSPECAN_ATTR_TDBS_INACT_CHAN_THR
        /// RSSPECAN_ATTR_TDBS_INVERT_Q
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:STANdard GPP | TSM
        /// [SENSe:]CDPower:LCODe[:VALue]
        /// [SENSe:]CDPower:MSHift
        /// [SENSe:]CDPower:SBANd NORMal | INVers
        /// [SENSe:]CDPower:NORMalize ON | OFF
        /// [SENSe:]CDPower:ICTReshold
        /// [SENSe:]CDPower:QINVert ON | OFF
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Standard">
        /// Switches switches between the standard 3GPP and TSM. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TDBS_SETTINGS_STANDARD_GPP (0) - GPP
        /// RSSPECAN_VAL_TDBS_SETTINGS_STANDARD_TSM (1) - TSM
        /// 
        /// Default Value: RSSPECAN_VAL_TDBS_SETTINGS_STANDARD_GPP  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_STANDARD
        /// </param>
        /// <param name="Scrambling_Code">
        /// Sets the scrambling code of the base station.
        /// 
        /// Valid Values:
        /// 0 to 127
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_SCODE
        /// </param>
        /// <param name="Midamble_Shift_Cells">
        /// Sets the maximum number of midamble shifts.
        /// 
        /// Valid Values:
        /// 2,4,6,8,10,12,14,16
        /// 
        /// Default Value: 16
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_CELL_SHIFTS
        /// 
        /// Note:
        /// 
        /// (1) This control has effect only if no Channel Table is active (rsspecan_TDSBSChannelTableOperations - Operation is set Off)
        /// 
        /// 
        /// 
        /// </param>
        /// <param name="Side_Band">
        /// Sets interchanging the left and the right sideband.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TDS_CDP_SBAND_NORM (0) - Normal
        /// RSSPECAN_VAL_TDS_CDP_SBAND_INV  (1) - Inverse
        /// 
        /// Default Value: RSSPECAN_VAL_TDS_CDP_SBAND_NORM  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_SIDE_BAND
        /// </param>
        /// <param name="Normalize">
        /// Switches elimination of IQ offset on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_NORMALIZE
        /// </param>
        /// <param name="Inactive_Channel_Treshold">
        /// Sets the threshold above which a channel is regarded as active. The level refers to total signal power. The driver uses this value to set the RSSPECAN_ATTR_TDBS_INACT_CHAN_THR attribute.
        /// 
        /// Units: dB
        /// 
        /// Valid Values: -100.0 to 0.0
        /// 
        /// Default Value: -40.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_INACT_CHAN_THR
        /// </param>
        /// <param name="Invert_Q">
        /// Inverts the sign of the signal Q component.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_INVERT_Q
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTDSBSMeasurement(int Standard, int Scrambling_Code, int Midamble_Shift_Cells, int Side_Band, bool Normalize, double Inactive_Channel_Treshold, bool Invert_Q)
        {
            int pInvokeResult = PInvoke.ConfigureTDSBSMeasurement(this._handle, Standard, Scrambling_Code, Midamble_Shift_Cells, Side_Band, System.Convert.ToUInt16(Normalize), Inactive_Channel_Treshold, System.Convert.ToUInt16(Invert_Q));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the TDS BS sync to slot. By default the R&amp;S FS-K76 determines the phase reference for all downlink data slots from the downlink pilot channel (DwPCH). For e.g. beamforming or repeater measurements it might be necessary to apply different phase offsets to each time slot.Using the DwPCH as phase reference leads to rotated constellation diagrams and bad EVM values in these time slots. By activating the new setting 'SYNC TO SLOT' the R&amp;S FS-K76 determines the phase reference from the midamble of the selected slot. Thus the data slots can be phase rotated to each other without degrading the EVM results. The selected slot must contain at least one data channel with sufficient power for successful synchronization.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDBS_SYNC_TO_SLOT
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:STSLot ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Sync_To_Slot">
        /// Selects the phase reference to be used.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_SYNC_TO_SLOT
        /// 
        /// Note(s):
        /// 
        /// (1) On: Selects the midamble of the selected slot as phase reference.
        /// 
        /// (2) Off: Selects the downlink pilot channel (DwPCH) as phase reference.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTDSBSSyncToSlot(bool Sync_To_Slot)
        {
            int pInvokeResult = PInvoke.ConfigureTDSBSSyncToSlot(this._handle, System.Convert.ToUInt16(Sync_To_Slot));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the TDS BS user defined CPICH.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDBS_CPICH_STATE
        /// RSSPECAN_ATTR_TDBS_CPICH_CODE
        /// RSSPECAN_ATTR_TDBS_CPICH_PATTERN
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:UCPich[:STATe] ON | OFF
        /// [SENSe:]CDPower:UCPich:CODE 0 to 255
        /// [SENSe:]CDPower:UCPich:PATTern OFF | 1 | 2
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="User_Defined_CPICH">
        /// Defines if the user defined CPICH settings are valid and shall be used for signal analysis.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_CPICH_STATE
        /// 
        /// Note(s):
        /// 
        /// (1) If this control is set to OFF, then CPICH Code and CPICH Pattern controls are ignored.
        /// </param>
        /// <param name="CPICH_Code">
        /// Sets the code number of the user defined CPICH used for signal analysis.
        /// 
        /// Valid Values:
        /// 0 to 255
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_CPICH_CODE
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available if User Defined CPICH is set to ON.
        /// </param>
        /// <param name="CPICH_Pattern">
        /// Defines which pattern for the user defined CPICH is used for signal analysis. OFF leads to a pettern selection according to the antenna selection, 1 leads to the fixed usage of "Pattern 1" and 2 leads to the fixed usage of "Pattern 2" according to standard.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TDS_CPICH_PATT_OFF (0) - Off
        /// RSSPECAN_VAL_TDS_CPICH_PATT_1   (1) - 1
        /// RSSPECAN_VAL_TDS_CPICH_PATT_2   (2) - 2
        /// 
        /// Default Value: RSSPECAN_VAL_TDS_CPICH_PATT_OFF (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_CPICH_PATTERN
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available if User Defined CPICH is set to ON.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTDSBSUCPICH(bool User_Defined_CPICH, int CPICH_Code, int CPICH_Pattern)
        {
            int pInvokeResult = PInvoke.ConfigureTDSBSUCPICH(this._handle, System.Convert.ToUInt16(User_Defined_CPICH), CPICH_Code, CPICH_Pattern);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the TDS BS Power Vs. Time measurement
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDBS_MEAS_PVT_SWITCH_POINT
        /// RSSPECAN_ATTR_TDBS_MEAS_PVT_SFRAMES
        /// RSSPECAN_ATTR_TDBS_MEAS_PVT_HDYN 
        /// 
        /// 
        /// Remote-control command(s):
        /// CONFigure:CDPower[:BTS]:PVTime:SPOint
        /// CONFigure:CDPower[:BTS]:PVTime:SFRames
        /// CONFigure:CDPower:BTS:PVTime:HDYNamic ON|OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Switching_Point">
        /// Sets the switching point between uplink and downlink slots.
        /// 
        /// Valid Values:
        /// 1 to 7
        /// 
        /// Default Value: 3
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_MEAS_PVT_SWITCH_POINT
        /// </param>
        /// <param name="Subframes">
        /// Sets the number of subframes for averaging.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 100
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_CELL_SHIFTS
        /// </param>
        /// <param name="High_Dynamic_State">
        /// Sets the number of subframes for averaging.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDBS_MEAS_PVT_HDYN
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTDSBSPVTMeasurement(int Switching_Point, int Subframes, bool High_Dynamic_State)
        {
            int pInvokeResult = PInvoke.ConfigureTDSBSPVTMeasurement(this._handle, Switching_Point, Subframes, System.Convert.ToUInt16(High_Dynamic_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function adjusts the reference level and returns the result.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:FUNCtion:CDPower:RESult? SLOT | PDATa | PD1 | PD2 | PMIDamble | RHO | MACCuracy | PCDerror | FERRor | CERRor | TFRame I IQOFfset | IQIMbalance | ACTive | SRATe | CHANnel | SFACtor | CDPabsolute | CDPRelative | EVMRms | EVMPeak | MCACpower
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Type">
        /// This control sets the result type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TDS_RESULT_SLOT (0) - Slot number
        /// RSSPECAN_VAL_TDS_RESULT_PDAT (1) - Power data fields
        /// RSSPECAN_VAL_TDS_RESULT_PD1  (2) - Power data field 1
        /// RSSPECAN_VAL_TDS_RESULT_PD2  (3) - Power data field 2 
        /// RSSPECAN_VAL_TDS_RESULT_PMID (4) - Power midamble
        /// RSSPECAN_VAL_TDS_RESULT_RHO  (5) - RHO
        /// RSSPECAN_VAL_TDS_RESULT_MACC (6) - Composite EVM
        /// RSSPECAN_VAL_TDS_RESULT_PCD  (7) - Peak code domain error 
        /// RSSPECAN_VAL_TDS_RESULT_FERR (8) - Frequency error
        /// RSSPECAN_VAL_TDS_RESULT_CERR (9) - Chip rate error
        /// RSSPECAN_VAL_TDS_RESULT_TFR  (10) - Trigger to frame 
        /// RSSPECAN_VAL_TDS_RESULT_IQIM (11) - IQ Imbalance
        /// RSSPECAN_VAL_TDS_RESULT_IQOF (12) - IQ Offset
        /// RSSPECAN_VAL_TDS_RESULT_ACT  (13) - Number of active channels
        /// RSSPECAN_VAL_TDS_RESULT_SRAT (14) - Data rate
        /// RSSPECAN_VAL_TDS_RESULT_CHAN (15) - Channel number
        /// RSSPECAN_VAL_TDS_RESULT_SFAC (16) - Spreading factor of channel
        /// RSSPECAN_VAL_TDS_RESULT_CDPR (17) - Channel power relative
        /// RSSPECAN_VAL_TDS_RESULT_CDP  (18) - Channel power absolute
        /// RSSPECAN_VAL_TDS_RESULT_EVMR (19) - Error vector magnitude RMS
        /// RSSPECAN_VAL_TDS_RESULT_EVMP (20) - Error vector magnitude Peak
        /// 
        /// Default Value: RSSPECAN_VAL_TDS_RESULT_SLOT  (0)
        /// 
        /// Note(s):
        /// (1) Slot number
        /// 
        /// (2) Power data fields in dBm 
        /// 
        /// (3)Power data field 1 in dBm 
        /// 
        /// (4) Power data field 2 in dBm 
        /// 
        /// (5) Power midamble in dBm 
        /// 
        /// (6) RHO 
        /// 
        /// (7) Composite EVM in % 
        /// 
        /// (8) Peak code domain error in dB
        /// 
        /// (9) Frequency error in Hz
        /// 
        /// (10) Chip rate error in ppm
        /// 
        /// (11) Trigger to frame
        /// 
        /// (12) IQ imbalance in %
        /// 
        /// (13) IQ offset in %
        /// 
        /// (14) Number of active channels
        /// 
        /// (15) Data rate in kbps
        /// 
        /// (16) Channel number
        /// 
        /// (17) Spreading factor of channel
        /// 
        /// (18) Channel power relative in dB
        /// 
        /// (19) Channel power absolute in dBm
        /// 
        /// (20) Error vector magnitude RMS in %
        /// 
        /// (21) Error vector magnitude Peak in %
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Result">
        /// Returns the measured values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetTDSBSResult(int Type, uint Timeout, out double Result)
        {
            int pInvokeResult = PInvoke.GetTDSBSResult(this._handle, Type, Timeout, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function adjusts the refernce level and returns the result.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDBS_ADJ_RLEV_QUERY
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:PRESet:RLEVel?
        /// *OPC
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Result">
        /// Returns the measured result. 
        /// 
        /// Returned Values:
        /// RSSPECAN_VAL_RES_FAIL (0) - Failed
        /// RSSPECAN_VAL_RES_PASS (1) - Passed
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetTDSBSReferenceLevelAdjustmentResult(uint Timeout, out int Result)
        {
            int pInvokeResult = PInvoke.GetTDSBSReferenceLevelAdjustmentResult(this._handle, Timeout, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the measured values of TDS BS measurement.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRACe[:DATA] TRACE1| TRACE2
        /// FORMat[:DATA] REAL,32
        /// *CLS
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Array_Length">
        /// Number of array points requested.
        /// </param>
        /// <param name="Actual_Points">
        /// Number of points actually returned in the array.
        /// </param>
        /// <param name="Values">
        /// A user allocated buffer into which the returned values are stored.
        /// 
        /// Returned Values:
        /// CODE DOMAIN POWER ABSOLUTE / CODE DOMAIN POWER RELATIVE (TRACE1)
        /// The following is output for each channel:
        /// &lt;Code class&gt;, &lt;Code number&gt;, &lt;Level&gt;, &lt;Power detection&gt;
        /// A maximum of 16 channels are read out, channels belonging together are read out as one channel.
        /// 
        /// CODE DOMAIN ERROR POWER (TRACE1)
        /// The following is output for each channel:
        /// &lt;Code class&gt;, &lt;Code number&gt;, &lt;Level&gt;, &lt;Power detection&gt;
        /// An error power is output for the CDEP, so consolidation of the power values is not meaningful.
        /// The number of output codes therefore generally corresponds to 
        /// the spreading factor 16.
        /// 
        /// CHANNEL TABLE (TRACE1)
        /// The following is output for each channel:
        /// &lt;Channel type&gt;, &lt;Code class&gt;, &lt;Code number&gt;, &lt;Modulation type&gt;, &lt;Absolute level in dBm&gt;,
        /// &lt;Relative level in dB&gt;, &lt;Midamble shift&gt;, &lt;?MidD1&gt;, &lt;?MidD2&gt;, &lt;Reserved 1&gt;, &lt;Reserved 2&gt;
        /// 
        /// Channel type The channel type is coded by numbers as follows:
        /// 0 = inactive
        /// 1 = midamble
        /// 2 = DPCH
        /// 3 = P-CCPCH
        /// 4 = S-CCPCH
        /// 5 = FPACH
        /// 6 = PDSCH
        /// 7 = PICH
        /// 
        /// In code sorting (CONF:CDP:CTAB:ORD CODE), first all midambles are output, then the control channels, and finally the data channels in ascending order of code number.
        /// In midamble sorting (CONF:CDP:CTAB:ORD MID), first the midamble is output and then its control and data channels.
        /// 
        /// RESULT SUMMARY (TRACE2)
        /// This command polls the measured and calculated values of code domain power analysis. The results are provided for the channel to which the code selected by the CDPower:CODe command belongs.
        /// &lt;SLOT&gt;, &lt;PDATa&gt;, &lt;PD1&gt;, &lt;PD2&gt;, &lt;PMIDamble&gt;, &lt;RHO&gt;, &lt;MACCuracy&gt;, &lt;PCDerror&gt;, &lt;FERRor&gt;, &lt;CERRor&gt;, &lt;TFRame&gt;, &lt;IQIMbalance&gt;, &lt;IQOFfset&gt;, &lt;ACTive&gt;, &lt;SRATe&gt;, &lt;CHANnel&gt;, &lt;SFACtor&gt;, &lt;CDPRelative&gt;, &lt;CDPabsolute&gt;, &lt;EVMRms&gt;, &lt;EVMPeak&gt;,&lt;Reserved 1&gt;,&lt;Reserved 2&gt;,&lt;Reserved 3&gt;,&lt;Reserved 4&gt;
        /// 
        /// SLOT Slot number
        /// PDATa Power data fields in dBm FERRor Frequency error in Hz
        /// PD1 Power data field 25.40 mm dBm CERRor Chip rate error in ppm
        /// PD2 Power data field 2 in dBm TFRame Trigger to frame
        /// PMIDamble Power midamble in dBm IQIMbalance IQ imbalance in %
        /// RHO RHO IQOFfset IQ offset in %
        /// MACCuracy Composite EVM in % ACTive Number of active channels
        /// PCDerror Peak code domain error in dB
        /// Channel results:
        /// SRATe Data rate in kbps
        /// CHANnel Channel number
        /// SFACtor Spreading factor of channel
        /// CDPRelative Channel power relative in dB
        /// CDPabsolute Channel power absolute in dBm
        /// EVMRms Error vector magnitude RMS in %
        /// EVMPeak Error vector magnitude Peak in %
        /// 
        /// 
        /// POWER VS SLOT ABS / REL (TRACE2)
        /// The number of returned value tripels corresponds to the IQ capture length.
        /// 
        /// &lt;Slot number&gt;,&lt;Level&gt;,&lt;Validity&gt;,...
        /// 
        /// Validity is coded as follows:
        /// 0 = inactive (channel not occupied)
        /// 1 = active (channel occupied)
        /// 2 = alias (code class of channel &lt; 4, i.e. several channels belong together)
        /// 
        /// 
        /// PK CODE DOM ERR and COMPOSITE EVM (TRACE2)
        /// The number of returned value pairs corresponds to the IQ capture length.
        /// PK CODE DOM ERR: &lt;Slot number&gt;, &lt;Level in dB&gt;, .....;
        /// COMPOSITE EVM: &lt;Slot number&gt;, &lt;Value in %&gt;, .....;
        /// 
        /// 
        /// SYMBOL EVM (TRACE2)
        /// The number of values depends on the spreading factor:
        /// Spreading factor 16: 44 values Spreading factor 8: 88 values
        /// Spreading factor 4: 176 values Spreading factor 2: 352 values
        /// Spreading factor 1: 704 values
        /// &lt;Value in % symbol 0&gt;, &lt;Value in % symbol 1&gt;,.....;
        /// 
        /// 
        /// POWER VS SYMBOL (TRACE2)
        /// The number of values depends on the spreading factor:
        /// Spreading factor 16: 44 values Spreading factor 8: 88 values
        /// Spreading factor 4: 176 values Spreading factor 2: 352 values
        /// Spreading factor 1: 704 values
        /// &lt;Value in dBm symbol 0&gt;, &lt;Value in dBm symbol 1&gt;,.....;
        /// 
        /// SYMBOL CONST (TRACE2)
        /// The number of value pairs depends on the spreading factor:
        /// Spreading factor 16: 44 values Spreading factor 8: 88 values
        /// Spreading factor 4: 176 values Spreading factor 2: 352 values
        /// Spreading factor 1: 704 values
        /// Real and imaginary parts are transferred as value pairs.
        /// &lt;re 0&gt;,&lt;im 0&gt;,&lt;re 1&gt;,&lt;im 1&gt;,.....&lt;re n&gt;, &lt;im n&gt;
        /// 
        /// COMPOSITE CONST (TRACe2):
        /// The number of value pairs corresponds to the chip number of 704 data chips in a slot. Real and imaginary parts are transferred as value pairs:
        /// &lt;re chip 0&gt;, &lt;im chip 0&gt;, &lt;re chip 1&gt;, &lt;im chip 1&gt;,.....;
        /// 
        /// BIT STREAM (TRACE2)
        /// The bit stream of a channel is output. A value is output for each bit (range 0, 1), each symbol consists of two bits for QPSK channels and three bits for 8PSK channels.
        /// 
        /// The number of values depends on the spreading factor in QPSK:
        /// Spreading factor 16: 88 values Spreading factor 2: 704 values
        /// Spreading factor 8: 176 values Spreading factor 1: 1408 values
        /// Spreading factor 4: 352 values
        /// In 8PSK:
        /// Spreading factor 16: 132 values Spreading factor 2: 1056 values
        /// Spreading factor 8: 264 values Spreading factor 1: 2112 values
        /// Spreading factor 4: 528 values
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchTDSBSTrace(int Trace, int Array_Length, out int Actual_Points, double[] Values)
        {
            int pInvokeResult = PInvoke.FetchTDSBSTrace(this._handle, Trace, Array_Length, out Actual_Points, Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the TD-SCDMA MS (MTDScdma) mode. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDUE_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument[:SELect] MTDScdma
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SetTDSUEMode()
        {
            int pInvokeResult = PInvoke.SetTDSUEMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the measurement of the Application Firmware R&amp;S FS-K77, TD-SCDMA mobile station test.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDUE_MEAS
        /// 
        /// Remote-control command(s):
        /// CONFigure&lt;1&gt;:CDPower:MEASurement POWer | ACLR | ESPectrum | OBANdwith | OBWidth | CDPower | CCDF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement">
        /// Selects the 3GPP FDD UE additional measurement..
        /// 
        /// Valid Values:
        /// 
        /// RSSPECAN_VAL_TDS_MEAS_POW   (0) - Power
        /// RSSPECAN_VAL_TDS_MEAS_ACLR  (1) - ACLR
        /// RSSPECAN_VAL_TDS_MEAS_ESP   (2) - Spectrum Emission Mask
        /// RSSPECAN_VAL_TDS_MEAS_OBAN  (3) - Occupied Bandwidth
        /// RSSPECAN_VAL_TDS_MEAS_PVT   (4) - Power Vs. Time
        /// RSSPECAN_VAL_TDS_MEAS_CDP   (5) - Code Domain Analyzer
        /// RSSPECAN_VAL_TDS_MEAS_CCDF  (6) - Signal Statistic
        /// 
        /// Default Value: RSSPECAN_VAL_TDS_MEAS_CDP (5)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDUE_MEAS
        /// 
        /// Note(s):
        /// (1) Power - Channel power measurement (TD-SCDMA forward standard) with predefined settings
        /// 
        /// (2) ACLR - Adjacent channel power measurement (TD-SCDMA forward standard) with predefined settings
        /// 
        /// (3) Spectrum Emission Mask - Check of signal power (spectrum emission mask)
        /// 
        /// (4) Occupied Bandwidth - Measurement of occupied bandwidth
        /// 
        /// (5) Code Domain Power
        /// 
        /// (6) Power Vs. Time - Measurement of power versus time
        /// 
        /// (7) Code Domain Analyzer - Code domain analyzer measurement
        /// 
        /// (8) CCDF - Signal statistic measurements
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SetTDSUEMeasurement(int Measurement)
        {
            int pInvokeResult = PInvoke.SetTDSUEMeasurement(this._handle, Measurement);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the gated sweep mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDUE_ADAPT_SIGN_SLOT_START
        /// RSSPECAN_ATTR_TDUE_ADAPT_SIGN_SLOT_STOP
        /// 
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:SLOT:STARt
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:SLOT:STOP
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Start_Slot">
        /// Sets the start slot for gated sweep mode.
        /// 
        /// Valid Values:
        /// 1 to 7
        /// 
        /// Default Value: 4
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDUE_ADAPT_SIGN_SLOT_START
        /// </param>
        /// <param name="Stop_Slot">
        /// Sets the stop slot for gated sweep mode.
        /// 
        /// Valid Values:
        /// 1 to 7
        /// 
        /// Default Value: 6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDUE_ADAPT_SIGN_SLOT_STOP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTDSUEAdaptToSignal(int Start_Slot, int Stop_Slot)
        {
            int pInvokeResult = PInvoke.ConfigureTDSUEAdaptToSignal(this._handle, Start_Slot, Stop_Slot);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the TDS BS capture settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDUE_RESULT_SET_COUNT
        /// RSSPECAN_ATTR_TDUE_RESULT_SET_VALUE
        /// RSSPECAN_ATTR_TDUE_CAPTURE_LENGTH
        /// RSSPECAN_ATTR_TDUE_RESULT_SEL_CHAN
        /// RSSPECAN_ATTR_TDUE_RESULT_SEL_SLOT
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:SET:COUNt
        /// [SENSe:]CDPower:SET[:VALue]
        /// [SENSe:]CDPower:IQLength
        /// [SENSe:]CDPower:CODE
        /// [SENSe:]CDPower:SLOT
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Set_Count">
        /// Sets the number of SETS of 63 slots that shall be captured consecutively into the IQ RAM of the R&amp;S FSQ.
        /// 
        /// Valid Values:
        /// 1 to 190
        /// 1 to 1900 with FSQ-B100 option
        /// 1 to 5700 with FSQ-B100 and FSQ-B102 option
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDUE_RESULT_SET_COUNT
        /// 
        /// </param>
        /// <param name="Set_Value">
        /// Sets the SET for which the results are evaluated.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDUE_RESULT_SET_VALUE
        /// 
        /// Note:
        /// (1) This control has effect only if Set Count &gt; 1
        /// </param>
        /// <param name="Capture_Length">
        /// Sets the IQ capture length in multiples of slots.
        /// 
        /// Valid Values:
        /// 2 to 63
        /// 
        /// Default Value: 7
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDUE_CAPTURE_LENGTH
        /// 
        /// Note:
        /// (1) This control has effect only if Set Count = 1
        /// 
        /// </param>
        /// <param name="Channel">
        /// Sets the code number.
        /// 
        /// Valid Values:
        /// 1 to 16
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDUE_RESULT_SEL_CHAN
        /// 
        /// </param>
        /// <param name="Slot">
        /// Sets the slot.
        /// 
        /// Valid Values:
        /// no range checking (0 to IQLength-1)
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDUE_RESULT_SEL_SLOT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTDSUECapture(int Set_Count, int Set_Value, int Capture_Length, int Channel, int Slot)
        {
            int pInvokeResult = PInvoke.ConfigureTDSUECapture(this._handle, Set_Count, Set_Value, Capture_Length, Channel, Slot);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the TDS UE results
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDUE_RESULT_TYPE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FEED
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Result_Type">
        /// Selects the measured data that are to be displayed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TDS_RES_CDP       (0) - Code Domain Power
        /// RSSPECAN_VAL_TDS_RES_CDPRAT    (1) - Code Domain Power Ratio
        /// RSSPECAN_VAL_TDS_RES_CDEP      (2) - Code Domain Error Power
        /// RSSPECAN_VAL_TDS_RES_MACC      (3) - Composite EVM
        /// RSSPECAN_VAL_TDS_RES_PVSLA     (4) - Power vs Slot Absolute
        /// RSSPECAN_VAL_TDS_RES_PVSLR     (5) - Power vs Slot Relative
        /// RSSPECAN_VAL_TDS_RES_PVSYM     (6) - Power vs Symbol
        /// RSSPECAN_VAL_TDS_RES_BSTR      (7) - Bit Stream
        /// RSSPECAN_VAL_TDS_RES_SUM       (8) - Summary
        /// RSSPECAN_VAL_TDS_RES_CTAB      (9) - Channel Assignment Table
        /// RSSPECAN_VAL_TDS_RES_PCD       (10) - Peak Code Domain Error
        /// RSSPECAN_VAL_TDS_RES_SYMBCONST (11) - Symbol Constellation
        /// RSSPECAN_VAL_TDS_RES_SYMBEVM   (12) - Symbol Error Vector Magnitude
        /// RSSPECAN_VAL_TDS_RES_COMPCONST (13) - Composite Constellation
        /// 
        /// Default Value: RSSPECAN_VAL_TDS_RES_CDP       (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDUE_RESULT_TYPE
        /// 
        /// Note(s):
        /// (1) Code Domain Power - Result display of code domain power as bargraph
        /// 
        /// (2) Code Domain Power Ratio - Result display of code domain power ratio as bargraph
        /// 
        /// (3) Code Domain Error Power - Result display of code domain power (both I and Q component) as bargraph
        /// 
        /// (4) Composite EVM - Result display of composite EVM (error vector magnitude referred to the overall signal)
        /// 
        /// (5) Power vs Slot Absolute - Result display of power versus slot (absolute)
        /// 
        /// (6) Power vs Slot Relative - Result display of power versus slot (relative)
        /// 
        /// (7) Power vs Symbol - Result display of power versus symbol
        /// 
        /// (8) Bit Stream - Result display of bit stream
        /// 
        /// (9) Summary - Result display in tabular form 
        /// 
        /// (10) Channel Assignment Table - Result display of channel assignment table
        /// 
        /// (11) Peak Code Domain Error - Result display of peak code domain error
        /// 
        /// (12) Symbol Constellation - Result display of symbol constellation
        /// 
        /// (13) Error Vector Magnitude - Result display of symbol error vector magnitude
        /// 
        /// (14) Composite Constellation - Result display of composite constellation
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTDSUEResults(int Result_Type)
        {
            int pInvokeResult = PInvoke.ConfigureTDSUEResults(this._handle, Result_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts the autorange routine for the reference level. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDUE_ADAPT_SIGN_AUT_LEVEL_TIME
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:AUTO:LTIMe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int TDSUEAdaptSignalAutoLevelTime()
        {
            int pInvokeResult = PInvoke.TDSUEAdaptSignalAutoLevelTime(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function initiates automatic setting of the RF attenuation and IF gain to the level of the applied signal. The unit is put into RF ATTEN MANUAL mode to optimize RF attenuation and IF gain independently of each other. This mode is retained after changing from TD-SCDMA BTS mode to SPECTRUM mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDUE_CDP_LEV_ADJUST
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:LEVel:ADJust
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int TDSUECDPLevelAdjust(uint Timeout)
        {
            int pInvokeResult = PInvoke.TDSUECDPLevelAdjust(this._handle, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function performs the selected operation with channel table.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDUE_CTAB_STATE
        /// RSSPECAN_ATTR_TDUE_CTAB_SEL
        /// RSSPECAN_ATTR_TDUE_CTAB_NAME
        /// RSSPECAN_ATTR_TDUE_CTAB_DEL
        /// RSSPECAN_ATTR_TDUE_CTAB_COPY
        /// 
        /// Remote-control command(s):
        /// CONFigure&lt;1&gt;:CDPower:CTABle[:STATe] ON | OFF
        /// CONFigure&lt;1&gt;:CDPower:CTABle:SELect
        /// CONFigure:CDPower:CTABle:NAME
        /// CONFigure:CDPower:CTABle:DELete
        /// CONFigure:CDPower:CTABle:COPY
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Operation">
        /// This control selects the type of memory operation to be performed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TDS_TABLE_OFF    (0) - Off
        /// RSSPECAN_VAL_TDS_TABLE_SELECT (1) - Select Table File
        /// RSSPECAN_VAL_TDS_TABLE_NEW    (2) - New/Select 
        /// RSSPECAN_VAL_TDS_TABLE_DELETE (3) - Delete
        /// RSSPECAN_VAL_TDS_TABLE_COPY   (4) - Copy
        /// 
        /// 
        /// Notes:
        /// 
        /// (1) Off
        /// Switches the channel table Off
        /// 
        /// (2) Select Table File:
        /// Selects a channel table file.
        /// 
        /// (3) New/Select:
        /// Selects an existing channel table or creates the name of a new channel table.
        /// 
        /// (4) Delete:
        /// Deletes the selected channel table.
        /// 
        /// (5) Copy
        /// Copies active channel table onto another one.
        /// </param>
        /// <param name="Channel_Table">
        /// This control selects a predefined channel table file.
        /// 
        /// Valid Values: up to 8 characters
        /// 
        /// Default Value: "RECENT"
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDUE_CTAB_STATE
        /// RSSPECAN_ATTR_TDUE_CTAB_SEL
        /// RSSPECAN_ATTR_TDUE_CTAB_NAME
        /// RSSPECAN_ATTR_TDUE_CTAB_DEL
        /// RSSPECAN_ATTR_TDUE_CTAB_COPY
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int TDSUEChannelTableOperations(int Operation, string Channel_Table)
        {
            int pInvokeResult = PInvoke.TDSUEChannelTableOperations(this._handle, Operation, Channel_Table);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines a comment for the selected channel table in the code domain power measurement mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDUE_CTAB_COMM
        /// 
        /// Remote-control command(s):
        /// CONFigure:CDPower:CTABle:COMMent
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Channel_Table_Comment">
        /// This control defines a comment for the selected channel table.
        /// 
        /// Valid Values: up to 128 characters
        /// 
        /// Default Value: "Comment for table 1"
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDUE_CTAB_COMM
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTDSUEChannelTableComment(string Channel_Table_Comment)
        {
            int pInvokeResult = PInvoke.ConfigureTDSUEChannelTableComment(this._handle, Channel_Table_Comment);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines a channel table. The whole table is defined in one operation. The inactive channels (INACtive) do not have to be defined.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// CONFigure:CDPower:CTABle:DATA &lt; Channel type &gt;, &lt;Code class&gt;, &lt;Code number&gt;, &lt;Modulation type&gt;, &lt;Midamble shift&gt;, &lt;Status&gt;, &lt;Reserved 1&gt;, &lt;Reserved 2&gt;, ....
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Array_Size">
        /// Sets the size of arryas.
        /// 
        /// Valid Values: &gt;0
        /// 
        /// Default Value: 1
        /// 
        /// Note(s):
        /// (1) All arrays has to have this number of items.
        /// </param>
        /// <param name="Channel_Type">
        /// Sets array of channel type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TDS_TABLE_CHANNEL_MID    (1) - Midamble
        /// RSSPECAN_VAL_TDS_TABLE_CHANNEL_DPCH   (2) - DPCH
        /// RSSPECAN_VAL_TDSUE_TABLE_CHANNEL_PRACH  (8) - PRACH
        /// RSSPECAN_VAL_TDSUE_TABLE_CHANNEL_PUSCH  (9) - PUSCH
        /// 
        /// 
        /// </param>
        /// <param name="Code_Class">
        /// Sets array of code class of channel.
        /// 
        /// Valid Values: 0 to 4
        /// 
        /// 
        /// </param>
        /// <param name="Code_Number">
        /// Sets array of code number.
        /// 
        /// Valid Values: 1 to 16
        /// 
        /// 
        /// </param>
        /// <param name="Modulation_Type">
        /// Sets array of modulation type
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_TDS_MOD_INVALID (0) - invalid (for midamble)
        /// RSSPECAN_VAL_TDS_MOD_QPSK    (1) - QPSK
        /// RSSPECAN_VAL_TDS_MOD_8PSK    (2) - 8PSK
        /// 
        /// 
        /// </param>
        /// <param name="Midamble_Shift">
        /// Sets array of midamble shift.
        /// 
        /// Valid Values: 1 to 16
        /// </param>
        /// <param name="Status">
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </param>
        /// <param name="Reserved_1">
        /// Always 0, reserved for additions
        /// 
        /// Valid Values: 0
        /// 
        /// 
        /// </param>
        /// <param name="Reserved_2">
        /// Always 0, reserved for additions
        /// 
        /// Valid Values: 0
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTDSUEChannelTableData(int Array_Size, int[] Channel_Type, int[] Code_Class, int[] Code_Number, int[] Modulation_Type, int[] Midamble_Shift, ushort[] Status, int[] Reserved_1, int[] Reserved_2)
        {
            int pInvokeResult = PInvoke.ConfigureTDSUEChannelTableData(this._handle, Array_Size, Channel_Type, Code_Class, Code_Number, Modulation_Type, Midamble_Shift, Status, Reserved_1, Reserved_2);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects sorting of the channel table in code order or midamble order.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDUE_CTAB_ORDER
        /// 
        /// Remote-control command(s):
        /// CONFigure&lt;1&gt;:CDPower:CTABle:ORDer CODE | MIDamble
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Order">
        /// Selects sorting of the channel table in code order or midamble order.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TDS_CTAB_ORDER_CODE (0) - Code
        /// RSSPECAN_VAL_TDS_CTAB_ORDER_MID  (1) - Midamble
        /// 
        /// Default Value: RSSPECAN_VAL_TDS_CTAB_ORDER_CODE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDUE_CTAB_ORDER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTDSUEChannelTableOrder(int Order)
        {
            int pInvokeResult = PInvoke.ConfigureTDSUEChannelTableOrder(this._handle, Order);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the maximum number of midamble shifts in the channel table.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDUE_CTAB_MSHIFT
        /// 
        /// Remote-control command(s):
        /// CONFigure:CDPower:CTABle:MSHift 2 | 4 | 6 | 8 | 10 | 12 | 14 | 16
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Maximum_of_Midamble_Shifts">
        /// Sets the maximum number of midamble shifts in the channel table.
        /// 
        /// Valid Values:
        /// 2,4,6,8,10,12,14,16
        /// 
        /// Default Value: 16
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDUE_CTAB_MSHIFT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTDSUEChannelTableMidambleShift(int Maximum_of_Midamble_Shifts)
        {
            int pInvokeResult = PInvoke.ConfigureTDSUEChannelTableMidambleShift(this._handle, Maximum_of_Midamble_Shifts);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the names of all channel tables stored on the harddisk in the code domain power measurement mode.
        /// 
        /// 
        /// Attribute(s):
        /// - 
        /// 
        /// Remote-control command(s):
        /// CONFigure:CDPower:CTABle:CATalog?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Number_of_Channel_Tables">
        /// This control returns number of channel tables, stored on harddisk.
        /// 
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the string you specify for the catalog list parameter.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 0
        /// 
        /// </param>
        /// <param name="Channel_Tables_List">
        /// This control returns array of channel tables, stored on harddisk.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// if you pass 0 for buffer Size, you can use VI_NULL for this parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetTDSUEChannelTableCatalog(out int Number_of_Channel_Tables, int Buffer_Size, System.Text.StringBuilder Channel_Tables_List)
        {
            int pInvokeResult = PInvoke.GetTDSUEChannelTableCatalog(this._handle, out Number_of_Channel_Tables, Buffer_Size, Channel_Tables_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the TDS UE measurement
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDUE_SCODE    
        /// RSSPECAN_ATTR_TDUE_CELL_SHIFTS
        /// RSSPECAN_ATTR_TDUE_SIDE_BAND
        /// RSSPECAN_ATTR_TDUE_NORMALIZE
        /// RSSPECAN_ATTR_TDUE_INACT_CHAN_THR
        /// RSSPECAN_ATTR_TDUE_INVERT_Q
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:LCODe[:VALue]
        /// [SENSe:]CDPower:MSHift
        /// [SENSe:]CDPower:SBANd NORMal | INVers
        /// [SENSe:]CDPower:NORMalize ON | OFF
        /// [SENSe:]CDPower:ICTReshold
        /// [SENSe:]CDPower:QINVert ON | OFF
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Scrambling_Code">
        /// Sets the scrambling code of the base station.
        /// 
        /// Valid Values:
        /// 0 to 127
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDUE_SCODE
        /// </param>
        /// <param name="Midamble_Shift_Cells">
        /// Sets the maximum number of midamble shifts.
        /// 
        /// Valid Values:
        /// 2,4,6,8,10,12,14,16
        /// 
        /// Default Value: 16
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDUE_CELL_SHIFTS
        /// 
        /// </param>
        /// <param name="Side_Band">
        /// Sets interchanging the left and the right sideband.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TDS_CDP_SBAND_NORM (0) - Normal
        /// RSSPECAN_VAL_TDS_CDP_SBAND_INV  (1) - Inverse
        /// 
        /// Default Value: RSSPECAN_VAL_TDS_CDP_SBAND_NORM  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDUE_SIDE_BAND
        /// </param>
        /// <param name="Normalize">
        /// Switches elimination of IQ offset on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDUE_NORMALIZE
        /// </param>
        /// <param name="Inactive_Channel_Treshold">
        /// Sets the threshold above which a channel is regarded as active. The level refers to total signal power.
        /// 
        /// Units: dB
        /// 
        /// Valid Values: -100.0 to 0.0
        /// 
        /// Default Value: -40.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDUE_INACT_CHAN_THR
        /// </param>
        /// <param name="Invert_Q">
        /// Inverts the sign of the signal Q component.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDUE_INVERT_Q
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTDSUEMeasurement(int Scrambling_Code, int Midamble_Shift_Cells, int Side_Band, bool Normalize, double Inactive_Channel_Treshold, bool Invert_Q)
        {
            int pInvokeResult = PInvoke.ConfigureTDSUEMeasurement(this._handle, Scrambling_Code, Midamble_Shift_Cells, Side_Band, System.Convert.ToUInt16(Normalize), Inactive_Channel_Treshold, System.Convert.ToUInt16(Invert_Q));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the TDS UE sync to slot. By default the R&amp;S FS-K76 determines the phase reference for all downlink data slots from the downlink pilot channel (DwPCH). For e.g. beamforming or repeater measurements it might be necessary to apply different phase offsets to each time slot.Using the DwPCH as phase reference leads to rotated constellation diagrams and bad EVM values in these time slots. By activating the new setting 'SYNC TO SLOT' the R&amp;S FS-K76 determines the phase reference from the midamble of the selected slot. Thus the data slots can be phase rotated to each other without degrading the EVM results. The selected slot must contain at least one data channel with sufficient power for successful synchronization.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDUE_SYNC_TO_SLOT
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:STSLot ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Sync_To_Slot">
        /// Selects the phase reference to be used.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDUE_SYNC_TO_SLOT
        /// 
        /// Note(s):
        /// 
        /// (1) On: Selects the midamble of the selected slot as phase reference.
        /// 
        /// (2) Off: Selects the downlink pilot channel (DwPCH) as phase reference.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTDSUESyncToSlot(bool Sync_To_Slot)
        {
            int pInvokeResult = PInvoke.ConfigureTDSUESyncToSlot(this._handle, System.Convert.ToUInt16(Sync_To_Slot));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the TDS UE user defined CPICH.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDUE_CPICH_STATE
        /// RSSPECAN_ATTR_TDUE_CPICH_CODE
        /// RSSPECAN_ATTR_TDUE_CPICH_PATTERN
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:UCPich[:STATe] ON | OFF
        /// [SENSe:]CDPower:UCPich:CODE 0 to 255
        /// [SENSe:]CDPower:UCPich:PATTern OFF | 1 | 2
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="User_Defined_CPICH">
        /// Defines if the user defined CPICH settings are valid and shall be used for signal analysis.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDUE_CPICH_STATE
        /// 
        /// Note(s):
        /// 
        /// (1) If this control is set to OFF, then CPICH Code and CPICH Pattern controls are ignored.
        /// </param>
        /// <param name="CPICH_Code">
        /// Sets the code number of the user defined CPICH used for signal analysis.
        /// 
        /// Valid Values:
        /// 0 to 255
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDUE_CPICH_CODE
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available if User Defined CPICH is set to ON.
        /// </param>
        /// <param name="CPICH_Pattern">
        /// Defines which pattern for the user defined CPICH is used for signal analysis. OFF leads to a pettern selection according to the antenna selection, 1 leads to the fixed usage of "Pattern 1" and 2 leads to the fixed usage of "Pattern 2" according to standard.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TDS_CPICH_PATT_OFF (0) - Off
        /// RSSPECAN_VAL_TDS_CPICH_PATT_1   (1) - 1
        /// RSSPECAN_VAL_TDS_CPICH_PATT_2   (2) - 2
        /// 
        /// Default Value: RSSPECAN_VAL_TDS_CPICH_PATT_OFF (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDUE_CPICH_PATTERN
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available if User Defined CPICH is set to ON.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTDSUEUCPICH(bool User_Defined_CPICH, int CPICH_Code, int CPICH_Pattern)
        {
            int pInvokeResult = PInvoke.ConfigureTDSUEUCPICH(this._handle, System.Convert.ToUInt16(User_Defined_CPICH), CPICH_Code, CPICH_Pattern);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the TDS UE Power Vs. Time measurement
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDUE_MEAS_PVT_SFRAMES
        /// RSSPECAN_ATTR_TDUE_MEAS_PVT_HDYN 
        /// 
        /// 
        /// Remote-control command(s):
        /// CONFigure:CDPower:PVTime:SFRames
        /// CONFigure:CDPower:PVTime:HDYNamic ON|OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Subframes">
        /// Sets the number of subframes for averaging.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 100
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDUE_CELL_SHIFTS
        /// 
        /// 
        /// </param>
        /// <param name="High_Dynamic_State">
        /// Sets the number of subframes for averaging. The driver uses this value to set the RSSPECAN_ATTR_TDUE_MEAS_PVT_HDYN attribute.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TDUE_MEAS_PVT_HDYN
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTDSUEPVTMeasurement(int Subframes, bool High_Dynamic_State)
        {
            int pInvokeResult = PInvoke.ConfigureTDSUEPVTMeasurement(this._handle, Subframes, System.Convert.ToUInt16(High_Dynamic_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function adjusts the refernce level and returns the result.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:FUNCtion:CDPower:RESult? SLOT | PDATa | PD1 | PD2 | PMIDamble | RHO | MACCuracy | PCDerror | FERRor | CERRor | TFRame I IQOFfset | IQIMbalance | ACTive | SRATe | CHANnel | SFACtor | CDPabsolute | CDPRelative | EVMRms | EVMPeak
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Type">
        /// This control sets the result type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TDS_RESULT_SLOT (0) - Slot number
        /// RSSPECAN_VAL_TDS_RESULT_PDAT (1) - Power data fields
        /// RSSPECAN_VAL_TDS_RESULT_PD1  (2) - Power data field 1
        /// RSSPECAN_VAL_TDS_RESULT_PD2  (3) - Power data field 2 
        /// RSSPECAN_VAL_TDS_RESULT_PMID (4) - Power midamble
        /// RSSPECAN_VAL_TDS_RESULT_RHO  (5) - RHO
        /// RSSPECAN_VAL_TDS_RESULT_MACC (6) - Composite EVM
        /// RSSPECAN_VAL_TDS_RESULT_PCD  (7) - Peak code domain error 
        /// RSSPECAN_VAL_TDS_RESULT_FERR (8) - Frequency error
        /// RSSPECAN_VAL_TDS_RESULT_CERR (9) - Chip rate error
        /// RSSPECAN_VAL_TDS_RESULT_TFR  (10) - Trigger to frame 
        /// RSSPECAN_VAL_TDS_RESULT_IQIM (11) - IQ Imbalance
        /// RSSPECAN_VAL_TDS_RESULT_IQOF (12) - IQ Offset
        /// RSSPECAN_VAL_TDS_RESULT_ACT  (13) - Number of active channels
        /// RSSPECAN_VAL_TDS_RESULT_SRAT (14) - Data rate
        /// RSSPECAN_VAL_TDS_RESULT_CHAN (15) - Channel number
        /// RSSPECAN_VAL_TDS_RESULT_SFAC (16) - Spreading factor of channel
        /// RSSPECAN_VAL_TDS_RESULT_CDPR (17) - Channel power relative
        /// RSSPECAN_VAL_TDS_RESULT_CDP  (18) - Channel power absolute
        /// RSSPECAN_VAL_TDS_RESULT_EVMR (19) - Error vector magnitude RMS
        /// RSSPECAN_VAL_TDS_RESULT_EVMP (20) - Error vector magnitude Peak
        /// 
        /// Default Value: RSSPECAN_VAL_TDS_RESULT_SLOT  (0)
        /// 
        /// Note(s):
        /// (1) Slot number
        /// 
        /// (2) Power data fields in dBm 
        /// 
        /// (3)Power data field 1 in dBm 
        /// 
        /// (4) Power data field 2 in dBm 
        /// 
        /// (5) Power midamble in dBm 
        /// 
        /// (6) RHO 
        /// 
        /// (7) Composite EVM in % 
        /// 
        /// (8) Peak code domain error in dB
        /// 
        /// (9) Frequency error in Hz
        /// 
        /// (10) Chip rate error in ppm
        /// 
        /// (11) Trigger to frame
        /// 
        /// (12) IQ imbalance in %
        /// 
        /// (13) IQ offset in %
        /// 
        /// (14) Number of active channels
        /// 
        /// (15) Data rate in kbps
        /// 
        /// (16) Channel number
        /// 
        /// (17) Spreading factor of channel
        /// 
        /// (18) Channel power relative in dB
        /// 
        /// (19) Channel power absolute in dBm
        /// 
        /// (20) Error vector magnitude RMS in %
        /// 
        /// (21) Error vector magnitude Peak in %
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Result">
        /// Returns the measured result. 
        /// 
        /// Returned Values:
        /// RSSPECAN_VAL_RES_FAIL (0) - Failed
        /// RSSPECAN_VAL_RES_PASS (1) - Passed
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetTDSUEResult(int Type, uint Timeout, out double Result)
        {
            int pInvokeResult = PInvoke.GetTDSUEResult(this._handle, Type, Timeout, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function adjusts the refernce level and returns the result.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TDUE_ADJ_RLEV_QUERY
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:PRESet:RLEVel?
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <param name="Result">
        /// Returns the measured result. 
        /// 
        /// Returned Values:
        /// RSSPECAN_VAL_RES_FAIL (0) - Failed
        /// RSSPECAN_VAL_RES_PASS (1) - Passed
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetTDSUEReferenceLevelAdjustmentResult(uint Timeout, out int Result)
        {
            int pInvokeResult = PInvoke.GetTDSUEReferenceLevelAdjustmentResult(this._handle, Timeout, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the measured values of TDS BS measurement.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRAC[:DATA] TRACE1| TRACE2
        /// FORMat[:DATA] REAL,32
        /// *CLS
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Array_Length">
        /// Number of array points requested.
        /// </param>
        /// <param name="Actual_Points">
        /// Number of points actually returned in the array.
        /// </param>
        /// <param name="Values">
        /// A user allocated buffer into which the returned values are stored.
        /// 
        /// Returned Values:
        /// CODE DOMAIN POWER ABSOLUTE / CODE DOMAIN POWER RELATIVE (TRACE1)
        /// Thus four values are transferred for all channels:
        /// &lt;Code class&gt;, &lt;Code number&gt;, &lt;Level&gt;, &lt;Power detection&gt;
        /// 
        /// CODE DOMAIN ERROR POWER (TRACE1)
        /// Thus four values are transferred for all channels:
        /// &lt;Code class&gt;, &lt;Code number&gt;, &lt;Level&gt;, &lt;Power detection&gt;
        /// 
        /// CHANNEL TABLE (TRACE1)
        /// Thus 11 values are transferred for all channels:
        /// &lt;Channel type&gt;, &lt;Code class&gt;, &lt;Code number&gt;, &lt;Modulation type&gt;, &lt;Absolute level in dBm&gt;,&lt;Relative level in dB&gt;, &lt;Midamble shift&gt;, &lt;deltaMidD1&gt;, &lt;deltaMidD2&gt;, &lt;Reserved 1&gt;, &lt;Reserved 2&gt;
        /// 
        /// RESULT SUMMARY (TRACE2)
        /// The results of RESULT SUMMARY are output in the following order:
        /// &lt;SLOT&gt;, &lt;PDATa&gt;, &lt;PD1&gt;, &lt;PD2&gt;, &lt;PMIDamble&gt;, &lt;RHO&gt;, &lt;MACCuracy&gt;, &lt;PCDerror&gt;, &lt;FERRor&gt;, &lt;CERRor&gt;, &lt;TFRame&gt;, &lt;IQIMbalance&gt;, &lt;IQOFfset&gt;, &lt;ACTive&gt;, &lt;SRATe&gt;, &lt;CHANnel&gt;, &lt;SFACtor&gt;, &lt;CDPRelative&gt;, &lt;CDPabsolute&gt;, &lt;EVMRms&gt;, &lt;EVMPeak&gt;,&lt;Reserved 1&gt;,&lt;Reserved 2&gt;,&lt;Reserved 3&gt;,&lt;Reserved 4&gt;
        /// 
        /// POWER VS SLOT ABS / REL (TRACE2)
        /// The number of returned value tripels corresponds to the IQ capture length.
        /// (See CDPower:IQLength command, range 2 to 63.)
        /// POWER VS SLOT ABS: &lt;Slot number&gt;,&lt;Level in dBm&gt;,&lt;Validity&gt;,.....;
        /// POWER VS SLOT REL: &lt;Slot number&gt;,&lt;Level in dB&gt;,&lt;Validity&gt;,.....;
        /// 
        /// PK CODE DOM ERR and COMPOSITE EVM (TRACE2)
        /// The number of returned value pairs corresponds to the IQ capture length.
        /// (See CDPower:IQLength command, range 2 to 63.)
        /// PK CODE DOM ERR: &lt;Slot number&gt;, &lt;Level in dB&gt;, .....;
        /// COMPOSITE EVM: &lt;Slot number&gt;, &lt;Value in %&gt;, .....;
        /// 
        /// SYMBOL EVM (TRACE2)
        /// The number of values depends on the spreading factor:
        /// Spreading factor 16: 44 values Spreading factor 8: 88 values
        /// Spreading factor 4: 176 values Spreading factor 2: 352 values
        /// Spreading factor 1: 704 values
        /// &lt;Value in % symbol 0&gt;, &lt;Value in % symbol 1&gt;,.....;
        /// 
        /// POWER VS SYMBOL (TRACE2)
        /// The number of values depends on the spreading factor:
        /// Spreading factor 16: 44 values Spreading factor 8: 88 values
        /// Spreading factor 4: 176 values Spreading factor 2: 352 values
        /// Spreading factor 1: 704 values
        /// &lt;Value in dBm symbol 0&gt;, &lt;Value in dBm symbol 1&gt;,.....;
        /// 
        /// SYMBOL CONST (TRACE2)
        /// The number of value pairs depends on the spreading factor:
        /// Spreading factor 16: 44 values Spreading factor 8: 88 values
        /// Spreading factor 4: 176 values Spreading factor 2: 352 values
        /// Spreading factor 1: 704 values 
        /// Real and imaginary parts are transferred as value pairs.
        /// &lt;re 0&gt;,&lt;im 0&gt;,&lt;re 1&gt;,&lt;im 1&gt;,.....&lt;re n&gt;, &lt;im n&gt;
        /// 
        /// COMPOSITE CONST (TRACe2):
        /// The number of value pairs corresponds to the chip number of 704 data chips in a slot. Real and imaginary parts are transferred as value pairs:
        /// &lt;re chip 0&gt;, &lt;im chip 0&gt;, &lt;re chip 1&gt;, &lt;im chip 1&gt;,.....;
        /// 
        /// BIT STREAM (TRACE2)
        /// The bit stream of a channel is output. A value is output for each bit (range 0, 1), each symbol
        /// consists of two bits for QPSK channels and three bits for 8PSK channels.
        /// The number of values depends on the spreading factor in QPSK:
        /// Spreading factor 16: 88 values Spreading factor 2: 704 values
        /// Spreading factor 8: 176 values Spreading factor 1: 1408 values
        /// Spreading factor 4: 352 values
        /// In 8PSK:
        /// Spreading factor 16: 132 values Spreading factor 2: 1056 values
        /// Spreading factor 8: 264 values Spreading factor 1: 2112 values
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchTDSUETrace(int Trace, int Array_Length, out int Actual_Points, double[] Values)
        {
            int pInvokeResult = PInvoke.FetchTDSUETrace(this._handle, Trace, Array_Length, out Actual_Points, Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function adjusts the frequency span, the measurement bandwidths and the detector as required for the number of channels, the channel bandwidths and the channel spacings selected in the active power measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_ACH_PRES
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:PRESet ACPower | CPOWer | MCACpower | OBANdwidth
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Adjust_Settings">
        /// This control selects the Adjust Settings.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_ACH_PRES_ACP   (0) - ACPower
        /// RSSPECAN_VAL_C2K_ACH_PRES_CPOW  (1) - CPOWer
        /// RSSPECAN_VAL_C2K_ACH_PRES_MCAC  (2) - MCACpower
        /// RSSPECAN_VAL_C2K_ACH_PRES_OBW   (3) - OBWidth
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_ACH_PRES_ACP
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_ACH_PRES
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int AdjustC2KSettings(int Window, int Adjust_Settings, uint Timeout)
        {
            int pInvokeResult = PInvoke.AdjustC2KSettings(this._handle, Window, Adjust_Settings, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function adapts the reference level to the measured channel power and - if required - switches on previously the adjacent channel power measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_ADJUST_REF_LEVEL
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:PRESet:RLEVel
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int AdjustC2KReferenceLevel(int Window, uint Timeout)
        {
            int pInvokeResult = PInvoke.AdjustC2KReferenceLevel(this._handle, Window, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function restores the standard limit lines for the spectrum emission mask measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_LIM_RESTORE
        /// 
        /// Remote-control command(s):
        /// CALCulate:LIMit:ESPectrum:RESTore
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int RestoreC2KSEMStandardLines()
        {
            int pInvokeResult = PInvoke.RestoreC2KSEMStandardLines(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures parameters of signal statistic measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MEAS_STAT_APD_STATE
        /// RSSPECAN_ATTR_MEAS_STAT_CCDF_STATE
        /// RSSPECAN_ATTR_MEAS_STAT_SAMPLES
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:APD[:STATe] ON | OFF
        /// CALCulate:STATistics:CCDF[:STATe] ON | OFF
        /// CALCulate:STATistics:NSAMples
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement">
        /// Selects the signal statistic measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SSTAT_OFF  (0) - Off
        /// RSSPECAN_VAL_SSTAT_APD  (1) - APD
        /// RSSPECAN_VAL_SSTAT_CCDF (2) - CCDF
        /// 
        /// Default Value: RSSPECAN_VAL_SSTAT_OFF (0)
        /// </param>
        /// <param name="Number_of_Samples">
        /// Defines the number of measurement points to be acquired for the statistical measurement functions.
        /// 
        /// Valid Values:
        /// 100 to 1000000000
        /// 
        /// Default Value: 100000
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MEAS_STAT_SAMPLES
        /// 
        /// Note(s):
        /// (1) This attribute has no effect for Measurement set to Off.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureC2KSignalStatisticMeasurement(int Measurement, int Number_of_Samples)
        {
            int pInvokeResult = PInvoke.ConfigureC2KSignalStatisticMeasurement(this._handle, Measurement, Number_of_Samples);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the percent marker.
        /// 
        /// Note(s): 
        /// 
        /// (1)The command is only available with the CCDF measurement switched on.
        /// 
        /// (2) The associated level value can be determined with the CALC:MARK:X? command.
        /// 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_PERCENT_MARKER
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:Y:PERCent 0 to100%
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects delta marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Percent_Marker">
        /// This control positions the selected marker in the selected window to the given probability.
        /// 
        /// Valid Values: 
        /// 0.0 to 100.0 %
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_PERCENT_MARKER
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureC2KSignalStatisticsPercentMarker(int Window, int Marker, double Percent_Marker)
        {
            int pInvokeResult = PInvoke.ConfigureC2KSignalStatisticsPercentMarker(this._handle, Window, Marker, Percent_Marker);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function performs the selected operation with channel table.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_CTABLE_STATE
        /// RSSPECAN_ATTR_C2K_CTABLE_SELECT
        /// RSSPECAN_ATTR_C2K_CTABLE_NAME
        /// RSSPECAN_ATTR_C2K_CTABLE_COPY
        /// RSSPECAN_ATTR_C2K_CTABLE_DELETE
        /// 
        /// Remote-control command(s):
        /// CONFigure&lt;1&gt;:CDPower[:BTS]:CTABle[:STATe] ON | OFF
        /// CONFigure&lt;1&gt;:CDPower[:BTS]:CTABle:SELect &lt;string&gt;
        /// CONFigure&lt;1&gt;:CDPower[:BTS]:CTABle:NAME &lt;file_name&gt;
        /// CONFigure&lt;1&gt;:CDPower[:BTS]:CTABle:COPY &lt;file_name&gt;
        /// CONFigure&lt;1&gt;:CDPower[:BTS]:CTABle:DELete
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Operation">
        /// This control selects the type of memory operation to be performed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TABLE_OFF    (0) - Off
        /// RSSPECAN_VAL_TABLE_SELECT (1) - Select Table File
        /// RSSPECAN_VAL_TABLE_NEW    (2) - New/Select
        /// RSSPECAN_VAL_TABLE_DELETE (3) - Delete
        /// RSSPECAN_VAL_TABLE_COPY   (4) - Copy
        /// 
        /// 
        /// Notes:
        /// 
        /// (1) Off
        /// Switches the channel table Off
        /// 
        /// (2) Select Table File:
        /// Selects a channel table file.
        /// 
        /// (3) New/Select:
        /// Selects an existing channel table or creates the name of a new channel table.
        /// 
        /// (4) Delete:
        /// Deletes the selected channel table.
        /// 
        /// (5) Copy
        /// Copies active channel table onto another one.
        /// </param>
        /// <param name="Channel_Table">
        /// This control selects a predefined channel table file.
        /// 
        /// Valid Values: up to 8 characters
        /// 
        /// Default Value: "RECENT"
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_CTABLE_STATE
        /// RSSPECAN_ATTR_C2K_CTABLE_SELECT
        /// RSSPECAN_ATTR_C2K_CTABLE_NAME
        /// RSSPECAN_ATTR_C2K_CTABLE_COPY
        /// RSSPECAN_ATTR_C2K_CTABLE_DELETE
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureC2KChannelTableOperations(int Operation, string Channel_Table)
        {
            int pInvokeResult = PInvoke.ConfigureC2KChannelTableOperations(this._handle, Operation, Channel_Table);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function restores the predefined tables. They are restored to the values they had when the device was delivered. With this restore feature an undo of overwriting these channel tables is possible.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_CTABLE_RESTORE
        /// 
        /// Remote-control command(s):
        /// CONFigure:CDPower:CTABle:RESTore
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int RestoreC2KChannelTable()
        {
            int pInvokeResult = PInvoke.RestoreC2KChannelTable(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the basic settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_BCLASS
        /// RSSPECAN_ATTR_C2K_CDP_TPMEAS
        /// RSSPECAN_ATTR_C2K_CDP_ICTHRES
        /// RSSPECAN_ATTR_C2K_CDP_QINVERT
        /// RSSPECAN_ATTR_C2K_CDP_SBAND
        /// RSSPECAN_ATTR_C2K_CDP_NORMALIZE
        /// 
        /// Remote-control command(s):
        /// CONFigure:CDPower[:BTS]:BCLass 0 to 12
        /// [SENSe:]CDPower:TPMeas ON | OFF
        /// [SENSe:]CDPower:ICTReshold -100 dB  to 0 dB
        /// [SENSe:]CDPower:QINVert ON | OFF
        /// [SENSe:]CDPower:SBANd NORMal | INVers
        /// [SENSe:]CDPower:NORMalize ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Band_Class">
        /// This control selects the band class.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0 - 800 MHz Band
        /// 1 - 1900 MHz Band
        /// 2 - TACS Band
        /// 3 - JTACS Band
        /// 4 - Korean PCS Band
        /// 5 - 450 MHz Band
        /// 6 - 2 GHz Band
        /// 7 - 700 MHz Band
        /// 8 - 1800 MHz Band
        /// 9 - 900 MHz Band
        /// 10 - Secondary 800 MHz Band
        /// 11 - 400 MHz European PAMR Band
        /// 12 - 800 MHz PAMR Band
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_BCLASS
        /// </param>
        /// <param name="Time_Phase">
        /// This control allows specific activation and deactivation of the timing and phase offset evaluation of the channels for the pilot channel.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE  (0) - Off
        /// VI_TRUE   (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_CDP_TPMEAS
        /// </param>
        /// <param name="Inact_Channel_Threshold">
        /// This control sets the threshold value from which a channel is regarded as active.
        /// 
        /// Units: dB
        /// 
        /// Valid Values:
        /// -100.0 to 0.0
        /// 
        /// Default Value: -60.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_CDP_ICTHRES
        /// </param>
        /// <param name="Invert_Q">
        /// This control inserts the sign of the signal Q-component.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE  (0) - Off
        /// VI_TRUE   (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_CDP_QINVERT
        /// </param>
        /// <param name="Side_Band">
        /// This control is used to switch the left and right sideband.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_SBAND_INVERS  (0) - Invers
        /// RSSPECAN_VAL_C2K_SBAND_NORMAL  (1) - Normal
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_SBAND_NORMAL
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_CDP_SBAND
        /// </param>
        /// <param name="Normalize">
        /// This control enables and disables elimination of the IQ offset. 
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE  (0) - Off
        /// VI_TRUE   (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_CDP_NORMALIZE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureC2KBasicSettings(int Band_Class, bool Time_Phase, double Inact_Channel_Threshold, bool Invert_Q, int Side_Band, bool Normalize)
        {
            int pInvokeResult = PInvoke.ConfigureC2KBasicSettings(this._handle, Band_Class, System.Convert.ToUInt16(Time_Phase), Inact_Channel_Threshold, System.Convert.ToUInt16(Invert_Q), Side_Band, System.Convert.ToUInt16(Normalize));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the scaling of the Y axis in the current unit.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_PDIV
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACE:Y[:SCALe]:PDIVision &lt;numeric_value&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Y_Scale_Div">
        /// This control defines the scaling of the Y axis in the current unit.
        /// 
        /// Units: dB
        /// 
        /// Valid Values: 
        /// 0.1 to 20.0
        /// 
        /// Default Value: 1.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_PDIV
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureC2KAmplitudeYScaleDivision(double Y_Scale_Div)
        {
            int pInvokeResult = PInvoke.ConfigureC2KAmplitudeYScaleDivision(this._handle, Y_Scale_Div);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function adjusts reference level.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_CDP_LEVEL_ADJUST
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CDPower:LEVel:ADJust
        /// *OPC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Timeout">
        /// This control sets the timeout value.
        /// 
        /// Valid Values: 0 to 4294967295
        /// 
        /// Default Value: 15000 ms
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int AdjustC2KAmplitudeReferenceLevel(uint Timeout)
        {
            int pInvokeResult = PInvoke.AdjustC2KAmplitudeReferenceLevel(this._handle, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the CDMA2000 Measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_MEAS_SELECT
        /// 
        /// Remote-control command(s):
        /// CONFigure&lt;1&gt;:CDPower[:BTS]:MEASurement POWer | ACLR | MCAClr | ESPectrum | OBWidth | CDPower | CCDF | PVTime
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement_Select">
        /// This control selects the Measurement.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_MEAS_SEL_POWER       (0) - Power
        /// RSSPECAN_VAL_C2K_MEAS_SEL_ACLR        (1) - ACLR
        /// RSSPECAN_VAL_C2K_MEAS_SEL_MC_ACLR     (2) - Multi Carrier ACLR
        /// RSSPECAN_VAL_C2K_MEAS_SEL_ESP         (3) - Spectrum EM Mask
        /// RSSPECAN_VAL_C2K_MEAS_SEL_OBW         (4) - Occupied Bandwidth
        /// RSSPECAN_VAL_C2K_MEAS_SEL_CDP         (5) - Code Domain Analyzer
        /// RSSPECAN_VAL_C2K_MEAS_SEL_CCDF        (6) - Signal Statistic
        /// RSSPECAN_VAL_C2K_MEAS_SEL_PVTIME      (7) - Power vs Time
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_MEAS_SEL_POWER
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_MEAS_SELECT
        /// 
        /// Note(s):
        /// 
        /// (1) Power vs Time - is available only for option K84.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SelectC2KMeasurement(int Measurement_Select)
        {
            int pInvokeResult = PInvoke.SelectC2KMeasurement(this._handle, Measurement_Select);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the PCG Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for option K83 and R&amp;S FSQ.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_SET_COUNT
        /// RSSPECAN_ATTR_C2K_SET_TO_ANALYZE
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:SET:COUNt
        /// [SENSe:]CDPower:SET:VALUE (0..SET COUNT - 1)
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Set_Count">
        /// This control defines how many SETS of the SLOTS shall be captured consecutively into the IQ RAM.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 1 to 38 for CDMA2000
        /// 1 to 380 for CDMA2000 with FSQ-B100
        /// 1 to 1140 for CDMA2000 with FSQ-B100 and FSQ-B102
        /// 1 to 57 for EVDO
        /// 1 to 570 for EVDO with FSQ-B100
        /// 1 to 1710 for EVDO with FSQ-B100 and FSQ-B102
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_SET_COUNT
        /// </param>
        /// <param name="Set_To_Analyze">
        /// This control defines the set for which the results are calculated.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0 .. SET_COUNT -1
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_SET_TO_ANALYZE
        /// 
        /// Note(s):
        /// 
        /// (1) This value is ignored if the instrument is other than FSQ.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureC2KPCGSettings(int Set_Count, int Set_To_Analyze)
        {
            int pInvokeResult = PInvoke.ConfigureC2KPCGSettings(this._handle, Set_Count, Set_To_Analyze);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines a comment for the selected channel table in the code domain power measurement mode.
        /// 
        /// Prior to this function, the name of the channel table has to be defined with function Configure C2K Channel Table Operations (ConfigureC2KChannelTableOperations) and the values of the table have to be defined with one of the functions Configure C2K BST Channel Table Data (ConfigureBC2KChannelTableData), Configure C2K MST Channel Table Data (ConfigureM2KChannelTableData), Configure EV-DO BST Channel Table Data (ConfigureBDOChannelTableData), or Configure EV-DO MST Channel Table Data (ConfigureMDOChannelTableData).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_CTABLE_COMMENT
        /// 
        /// Remote-control command(s):
        /// CONFigure:CDPower[:BTS]:CTABle:COMMent &lt;string&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Channel_Table_Comment">
        /// This control defines a comment for the selected channel table. 
        /// 
        /// Valid Values: up to 128 characters
        /// 
        /// Default Value: "Comment for table 1"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_CTABLE_COMMENT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureC2KChannelTableComment(string Channel_Table_Comment)
        {
            int pInvokeResult = PInvoke.ConfigureC2KChannelTableComment(this._handle, Channel_Table_Comment);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the names of all channel tables stored on the harddisk in the code domain power measurement mode.
        /// 
        /// Notes:
        /// 
        /// (1) Driver returns an instrument specific error, when catalog list is empty.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// CONFigure:CDPower[:BTS]:CTABle:CATalog?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Number_of_Channel_Tables">
        /// This control returns number of channel tables, stored on harddisk.
        /// 
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the string you specify for the catalog list parameter.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 0
        /// 
        /// </param>
        /// <param name="Channel_Tables_List">
        /// This control returns array of channel tables, stored on harddisk.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// If you pass 0 for buffer Size, you can use VI_NULL for this parameter.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryC2KChannelTableCatalog(out int Number_of_Channel_Tables, int Buffer_Size, System.Text.StringBuilder Channel_Tables_List)
        {
            int pInvokeResult = PInvoke.QueryC2KChannelTableCatalog(this._handle, out Number_of_Channel_Tables, Buffer_Size, Channel_Tables_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads trace data out of the instrument. Only TRACE1 or TRACE2 can be queried depending on the display mode.
        /// 
        /// Notes:
        /// 
        /// (1) This function is available for option FS-K82, FS-K83, FS-K84 or FS-K85 only.
        /// 
        /// (2) The associated measurement window is selected with the function Display Set Active Window (rsspecan_DisplaySetActiveWindow).
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;[:DATA]? TRACE1 | TRACE2
        /// FORMat[:DATA] ASCII
        /// FORMat[:DATA] REAL,32
        /// *CLS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Source_Trace">
        /// This parameter indicates which trace will be read out.
        /// 
        /// Valid Values:
        /// 0 - Trace 1 (Default Value)
        /// 1 - Trace 2
        /// 
        /// </param>
        /// <param name="Array_Length">
        /// Number of array points requested.
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Actual_Points">
        /// Number of points actually returned in the Amplitude array and X Array array.
        /// </param>
        /// <param name="Values">
        /// Returns an array of trace data. Only TRACE1 or TRACE2 can be queried depending on the display mode. The following measured values are transferred depending on the display mode.
        /// 
        /// For more dedails please refer also to the user manual.
        /// 
        /// 
        /// ---------- Option FS-K82 ----------
        /// 
        /// 
        /// CODE DOMAIN POWER ABSOLUTE/CODE DOMAIN POWER RELATIVE (TRACE1):
        /// 
        /// The following is output for each channel:
        /// 
        /// Code class:
        ///   Code class of the channel. With Hadamard order, normally the
        ///   code class corresponding to the base spreading factor. With
        ///   BitReverse order, values between 2 and 7.
        /// 
        /// Code number:
        ///   Code number of the channel, values between 0 and 127.
        /// 
        /// Level:
        ///   - For CODE DOMAIN POWER ABSOLUTE in dBm
        ///   - For CODE DOMAIN POWER RELATIVE in dB
        ///     (referred to the total or pilot power)
        /// 
        ///   In Hadamard order, power values of the different codes are
        ///   normally specified, in BitReverse order the consolidated
        ///   channel power is returned.
        /// 
        /// Power detection:
        ///   0 - inactive channel
        ///   1 - power of own antenna
        ///   2 - alias power of own antenna
        ///   3 - alias power of other antenna
        ///   4 - alias power of own and other antenna.
        /// 
        ///   Alias power can be avoided by analysis when the base
        ///   spreading factor is selected correctly.
        /// 
        /// Thus four values are transferred for all channels:
        /// 
        ///   &lt;Code class&gt;,&lt;Code number&gt;,&lt;Signal level&gt;,&lt;Power ID&gt;,...
        /// 
        /// For sorting the channels and consolidation, Hadamard order or BitReverse order is important.
        /// 
        /// With Hadamard, the different codes are output in ascending order with their code power. The number of output codes corresponds to the base spreading factor.
        /// 
        /// With BitReverse, codes belonging to a channel are next to each other and are therefore output in the class of the channel with output of the consolidated channel power. The maximum number of output codes or channels cannot be higher than the base spreading factor, but decreases with every concentrated channel.
        /// 
        /// CODE DOMAIN ERROR POWER (TRACE1):
        /// 
        /// The following is output for each channel:
        /// 
        /// Code class:
        ///   Code class of the channel normally corresponding to the class
        ///   for the CDEP that belongs to the base spreading factor.
        /// 
        /// Code number:
        ///   Code number of the channel, values between 0 and 127.
        /// 
        /// Error power:
        ///   In dB
        ///   No difference of power between the Hadamard and BitReverse
        ///   orders.
        /// 
        /// Power ID:
        ///   0 - inactive channel
        ///   1 - power of own antenna
        ///   2 - alias power of own antenna
        ///   3 - alias power of other antenna
        ///   4 - alias power of own and other antenna.
        /// 
        ///   Alias power can be avoided by analysis when the base
        ///   spreading factor is selected correctly.
        /// 
        /// Thus four values are transferred for all channels:
        /// 
        ///   &lt;Code class&gt;,&lt;code number&gt;,&lt;signal level&gt;,&lt;power ID&gt;,...
        /// 
        /// For sorting the channels, the Hadamard order or BitReverse order is important.
        /// 
        /// In Hadamard order, the different codes are sorted in ascending order before being output. With BitReverse order, codes belonging to any one channel are next to each other. Because an error power is output with the code domain error power, a consolidation of the power values is not sensible.
        /// 
        /// The number of output codes therefore generally corresponds to the base spreading factor.
        /// 
        /// CHANNEL TABLE (TRACE1):
        /// 
        /// The following is output for each channel:
        /// 
        /// Channel type:
        ///   The channel type is coded with numbers as follows:
        ///   0 = PICH
        ///   1 = SYNC
        ///   2 = PCH
        ///   3 = TDPICH
        ///   4 = APICH
        ///   5 = ATDPICH
        ///   6 = BCH
        ///   7 = CPCCH
        ///   8 = CACH
        ///   9 = CCCH
        ///   10 = CHAN
        ///   11 = INACTIVE
        /// 
        /// Code class:
        ///   Code class of the channel, values between 2 and 7.
        /// 
        /// Code number:
        ///   Code number of the channel, values between 0 and 127.
        /// 
        /// Radio configuration:
        ///   0 for special channels, 1-5 for CHAN (with CODE CHAN
        ///   AUTOSEARCH '1' is returned for RCs 1&amp;2 and '3' for RCs 3 to
        ///   5)
        /// 
        /// Absolute level:
        ///   In dBm
        /// 
        /// Relative level:
        ///   In dB, referred to the total or pilot power.
        /// 
        /// Timing offset:
        ///   Referred to the pilot in seconds.
        /// 
        /// Phase offset:
        ///   Referred to the pilot in rad. If the evaluation of the timing
        ///   and phase offset is not active or more than 50 active
        ///   channels are in the signal, the value 9 is returned. For
        ///   inactive channels, the value 9 is returned.
        /// 
        /// The class specifies the spreading factor:
        /// 
        /// Class 7 corresponds to the highest spreading factor (128, symbol rate 9.6 ksps), and class 2 to the lowest allowed spreading factor (4, symbol rate 307.2 ksps).
        /// 
        /// Thus eight values are transferred for all channels:
        /// 
        ///   &lt;Channel type&gt;,&lt;Code class&gt;,&lt;Code number&gt;,&lt;Radio
        ///   configuration&gt;,&lt;Absolute level&gt;,&lt;Relative level&gt;,&lt;Timing
        ///   offset&gt;,&lt;Phase offset&gt;,...
        /// 
        /// All detected special channels are sorted first, followed by the data channels, in ascending order by code class and within the code class in ascending order by code number. The unassigned codes appear at the bottom of the list with the code class of the base spreading factor.
        /// 
        /// RESULT SUMMARY (TRACE2):
        /// 
        /// The results of the RESULT SUMMARY are output in the following order:
        /// 
        ///   &lt;SLOT&gt;,&lt;PTOTal&gt;,&lt;PPICh&gt;,&lt;RHO&gt;,&lt;MACCuracy&gt;,&lt;PCDerror&gt;,
        ///   &lt;ACTive&gt;,&lt;FERRor&gt;,&lt;FERPpm&gt;,&lt;TFRame&gt;,&lt;CERRor&gt;,&lt;IQOFfset&gt;,
        ///   &lt;IQIMbalance&gt;,&lt;SRATe&gt;,&lt;CHANnel&gt;,&lt;SFACtor&gt;,&lt;TOFFset&gt;,
        ///   &lt;POFFset&gt;,&lt;CDPRelative&gt;,&lt;CDPabsolute&gt;,&lt;EVMRms&gt;,&lt;EVMPeak&gt;
        /// 
        /// The results have the following meanings and units:
        /// 
        /// Global results of the selected PCG:
        /// 
        ///   SLOT        PCG number
        ///   PTOTal      Total power in dBm
        ///   PPICh       Pilot power in dBm
        ///   RHO         RHO
        ///   MACCuracy   Composite EVM in %
        ///   PCDerror    Peak code domain error in dB
        ///   IQOFfset    IQ offset in %
        ///   IQIMbalance IQ imbalance in %
        /// 
        /// Global results of all PCGs:
        /// 
        ///   FERRor      Frequency error in Hz
        ///   FERPpm      Frequency error in ppm
        ///   CERRor      Chip rate error in ppm
        ///   TFRame      Trigger to frame
        ///   ACTive      Number of active channels
        /// 
        /// Channel results:
        /// 
        ///   SRATe       Symbol rate in ksps
        ///   CHANnel     Channel number
        ///   SFACtor     Spreading factor of the channel
        ///   CDPRelative Channel power relative in dB
        ///   EVMRms      Error vector magnitude rms in %
        ///   TOFFset     Timing Offset in s
        ///   POFFset     Phase Offset in rad
        ///   CDPabsolute Channel power abs. in dBm
        ///   EVMPeak     Error vector mag. peak in %
        /// 
        /// Hint:
        /// 
        /// The value trigger to frame (TFRame) delivers a '9' if the trigger is set to FREE RUN. The values timing/phase offset (TOFFset/POFFset) deliver a '9' if the timing and phase offset measurement is switched off (refer to CDP:TPM)or the number of active channel exceeds 50.
        /// 
        /// POWER VS PCG, PEAK CODE DOMAIN ERR and COMPOSITE EVM (TRACE2):
        /// 
        /// The number of returned value pairs corresponds to the Capture length.
        /// 
        ///   POWER VS PCG: &lt;Slot number&gt;,&lt;Level value in dB&gt;,&lt;Slot
        ///                 number&gt;,&lt;Level value in dB&gt;,.....;
        ///   PEAK CODE DOMAIN ERROR: &lt;Slot number&gt;,&lt;Level value in dB&gt;,
        ///                           .....;
        ///   COMPOSITE EVM: &lt;Slot number&gt;,&lt;Value in %&gt;, .....;
        /// 
        /// SYMBOL EVM (TRACE2):
        /// 
        /// The number of values depends on the spreading factor:
        /// 
        ///   Spreading factor 128:12 values; spreading factor 64:24 values
        ///   Spreading factor 32:48 values; spreading factor 16:96 values
        ///   Spreading factor 8:192 values; spreading factor 4:384 values
        /// 
        /// (With transmit diversity on the number of symbols are reduced to the half.)
        /// 
        ///   &lt;Value in % symbol 0&gt;,&lt;value in % symbol 1&gt;,.....;
        /// 
        /// POWER VS SYMBOL (TRACE2):
        /// 
        /// The number of values depends on the spreading factor:
        /// 
        ///   Spreading factor 128:12 values; spreading factor 64:24 values
        ///   Spreading factor 32:48 values; spreading factor 16:96 values
        ///   Spreading factor 8:192 values; spreading factor 4:384 values
        /// 
        /// (With transmit diversity on the number of values are reduced to the half.)
        /// 
        ///   &lt;Value in dBm symbol 0&gt;,&lt;value in dBm symbol 1&gt;,.....;
        /// 
        /// SYMBOL CONST (TRACE2):
        /// 
        /// The number of value pairs depends on the spreading factor:
        /// 
        ///   Spreading factor 128:12 pairs; spreading factor 64:24 pairs
        ///   Spreading factor 32:48 pairs: spreading factor 16:96 pairs
        ///   Spreading factor 8:192 pairs; spreading factor 4:384 pairs
        /// 
        /// (With transmit diversity on the number of pairs are reduced to the half.)
        /// 
        /// Real and imaginary parts are transferred as value pairs.
        /// 
        ///   &lt;re 0&gt;,&lt;im 0&gt;,&lt;re 1&gt;,&lt;im 1&gt;,.....&lt;re n&gt;,&lt;im n&gt;
        /// 
        /// COMPOSITE CONST (TRACE2):
        /// 
        /// The number of values corresponds to the chip number of 1536 chips in a power control group.
        /// 
        ///   &lt;Value in % chip 0&gt;,&lt;value in % chip 1&gt;,.....;
        /// 
        /// BITSTREAM (TRACE2):
        /// 
        /// The bitstream of a slot is output. A value is output for each bit (range 0,1), each symbol consists of 2 bits for QPSK channels and of one bit for BPSK channels.
        /// 
        /// The number of values depends on the spreading factor with QPSK:
        /// 
        ///   Spreading factor 128:24 values; spreading factor 64:48 values
        ///   Spreading factor 32:96 values; spreading factor 16:192 values
        ///   Spreading factor 8:384 values; spreading factor 4:768 values
        /// 
        /// For BPSK:
        /// 
        ///   Spreading factor 128:12 values; spreading factor 64:24 values
        ///   Spreading factor 32:48 values; spreading factor 16:96 values
        ///   Spreading factor 8:192 values; spreading factor 4:384 values
        /// 
        /// (With transmit diversity on the number of values are reduced to the half.)
        /// 
        /// Example of bitstream trace: 0, 0, 1, 0, 1, 1, 0 ....
        /// 
        /// 
        /// ---------- Option FS-K83 ----------
        /// 
        /// 
        /// CODE DOMAIN POWER ABSOLUTE/CODE DOMAIN POWER RELATIVE (TRACE1):
        /// 
        /// The following is output for each channel:
        /// 
        /// Code class:
        ///   Code class of the channel; with the Hadamard order it is the
        ///   code class which corresponds to the base spreading factor.
        ///   With the BitReverse order, values are between 1..6Code number 
        ///   Code number of the channel, values are between 0..63.
        /// 
        /// Level:
        ///   - With CODE DOMAIN POWER ABSOLUTE, units are dBm
        ///   - With CODE DOMAIN POWER RELATIVE, units are dB
        ///     (referred to the total or pilot power, refer to the command
        ///     CDPower:PREFerence)
        /// 
        ///   In the Hadamard order, power values of the
        ///   individual codes are usually specified; in the BitReverse 
        ///   order, the consolidated channel power is returned.
        /// 
        /// Power ID:
        ///   0 - Inactive channel
        ///   1 - Power of device antenna
        ///   2 - Alias power of device antenna
        ///   3 - Quasi-inactive channel (on the analyzed branch, the
        ///       channel is not assigned, but an active channel exists on
        ///       the other branch)
        /// 
        ///   Alias power can be avoided by ensuring that the correct base
        ///   spreading factor is selected when performing the analysis.
        ///   (Refer to the CDPower:SFACtor command)
        /// 
        /// Four values are thus transferred for all channels:
        /// 
        ///   &lt;Code class&gt;, &lt;Code number&gt;, &lt;Level&gt;, &lt;Power ID&gt;, ...
        /// 
        /// The Hadamard or BitReverse order is important for sorting the channels and consolidation (refer to the command CDPower:ORDer). With Hadamard, the individual codes are output in ascending order with their code power. The number of codes which are output corresponds to the base spreading factor.
        /// 
        /// With BitReverse, codes which belong to a particular channel are adjacent to each other and are therefore output in the class of the channel together with the channel power. The maximum number of codes or channels that are output cannot be higher than the base spreading factor, and decreases with each bunched channel.
        /// 
        /// CODE DOMAIN ERROR POWER (TRACE1):
        /// 
        /// The following is output for each channel:
        /// 
        /// Code class:
        ///   Code class of the channel, which with the CDEP generally
        ///   corresponds to the class which belongs to the base spreading
        ///   factor.
        /// 
        /// Code number:
        ///   Code number of the channel, values are between 0..63
        /// 
        /// Error power:
        ///   In dB. No distinction is made for power between the Hadamard
        ///   and BitReverse order.
        /// 
        /// Power ID:
        ///   0 - Inactive channel
        ///   1 - Power of device antenna
        ///   2 - Alias power of device antenna
        ///   3 - Quasi-inactive channel (on the analyzed branch, the
        ///       channel is not assigned, but an active channel exists on
        ///       the other branch)
        /// 
        /// Alias power can be avoided by ensuring that the correct base
        /// spreading factor is selected when performing the analysis. (Refer to the CDPower:SFACtor command)
        /// 
        /// Four values are thus transferred for all channels:
        /// 
        ///   &lt;Code class&gt;, &lt;Code number&gt;, &lt;Level&gt;, &lt;Power ID&gt;, ...
        /// 
        /// The Hadamard or BitReverse order is important for sorting the channels (refer to the command CDPower:ORDer). With Hadamard, the individual codes are output in ascending order. With BitReverse, codes which belong to a particular channel are adjacent to each other. Since an error power is output for Code Domain Error Power, consolidation of the power values is not appropriate. The number of codes that are output therefore generally corresponds to the base spreading factor.
        /// 
        /// CHANNEL TABLE (TRACE1):
        /// 
        /// The following is output for each channel:
        /// 
        /// Channel type:
        ///   The channel type is numerically coded as follows:
        ///   0 = PICH
        ///   1 = EACH
        ///   2 = CCCH
        ///   3 = DCCH
        ///   4 = ACKCH
        ///   5 = CQICH
        ///   6 = FCH
        ///   7 = S1CH
        ///   8 = S2CH
        ///   9 = INACTIVE
        /// 
        /// Code class:
        ///   Code class of the channel, values are between 1..6
        /// 
        /// Code number:
        ///   Code number of the channel, values are between 0..63
        /// 
        /// Mapping:
        ///   0 = I branch
        ///   1 = Q branch
        /// 
        /// Absolute level:
        ///   In dBmRelative level In dB, referred to the total or pilot 
        ///   power (refer to the command CDPower:PREFerence)
        /// 
        /// Timing offset:
        ///   Referred to the pilot in seconds. Phase offset Referred to the 
        ///   pilot in rad. If the analysis of the timing and phase offset
        ///   is not active (refer to CDPower:TPMeas) or there are more than
        ///   50 active channels in the signal, a value of 9 is returned. A
        ///   value of 9 is generally returned for inactive channels.
        /// 
        /// The class gives the spreading factor of the channel:
        ///   Class 6 corresponds to the highest spreading factor (64,
        ///   symbol rate 19.2 ksps), class 1 to the lowest permissible
        ///   spreading factor (2, symbol rate 614.4 ksps).
        /// 
        /// This means that 8 values are transmitted for all active channels:
        /// 
        ///   &lt;Channel type&gt;, &lt;Code class&gt;, &lt;Code number&gt;, &lt;Mapping&gt;,
        ///   &lt;Absolute level&gt;, &lt;Relative level&gt;, &lt;Timing offset&gt;, &lt;Phase
        ///   offset&gt;, etc.
        /// 
        /// All detected active channels are output first, followed by the inactive or quasi-active channels. The channels are sorted in ascending code number order (with identical code numbers: the I branch first, followed by the Q branch). The unassigned codes are displayed together with the code factor of the base spreading factor.
        /// 
        /// RESULT SUMMARY (TRACE2):
        /// 
        /// The results of the RESULT SUMMARY are output in the following order:
        /// 
        ///   &lt;SLOT&gt;, &lt;PTOTal&gt;, &lt;PPICh&gt;, &lt;RHO&gt;, &lt;MACCuracy&gt;, &lt;PCDerror&gt;,
        ///   &lt;ACTive&gt;, &lt;FERRor&gt;, &lt;FERPpm&gt;, &lt;TFRame&gt;, &lt;CERRor&gt;, &lt;IQOFfset&gt;,
        ///   &lt;IQIMbalance&gt;, &lt;SRATe&gt;, &lt;CHANnel&gt;, &lt;SFACtor&gt;, &lt;TOFFset&gt;,
        ///   &lt;POFFset&gt;, &lt;CDPRelative&gt;, &lt;CDPabsolute&gt;, &lt;EVMRms&gt;, &lt;EVMPeak&gt;
        /// 
        /// In this instance the results have the following meaning and units:
        /// 
        /// Global results of the selected PCG:
        ///   SLOT - PCG number
        ///   PTOTal - Total power in dBm
        ///   PPICh - Pilot power in dBm
        ///   RHO - RHO
        ///   MACCuracy - Composite EVM in %
        ///   PCDerror - Peak code domain error in dB
        ///   IQOFfset - IQ offset in %
        ///   IQIMbalance - IQ imbalance in %
        /// 
        /// Global results for all PCGs:
        ///   FERRor - Frequency error in Hz
        ///   FERPpm - Frequency error in ppm
        ///   CERRor - Chip rate error in ppm
        ///   TFRame - Trigger to frame
        ///   ACTive - Number of active channels
        /// 
        /// Channel results:
        ///   SRATe - Symbol rate in ksps
        ///   CHANnel - Channel number
        ///   SFACtor - Spreading factor of channel
        ///   CDPRelative - Channel power relative in dB (relative to total
        ///                 or PICH power, refer to command CDP:PREF)
        ///   EVMRms - Error vector magnitude RMS in %
        ///   TOFFset - Timing offset in s
        ///   POFFset - Phase offset in rad
        ///   CDPabsolute - Channel power absolute in dBm
        ///   EVMPeak - Error vector mag. peak in %
        /// 
        /// POWER VS PCG, PEAK CODE DOMAIN ERR and COMPOSITE EVM (TRACE2):
        /// 
        /// The number of returned value pairs corresponds to the IQ capture length. (Refer to the command CDPower:IQLength).
        /// 
        ///   POWER VS PCG: &lt;Slot number&gt;,&lt;Level value in dB&gt;, &lt;Slot
        ///                 number&gt;, &lt;Level value in dB&gt;,.....;
        /// 
        ///   PEAK CODE DOMAIN ERROR: &lt;Slot number&gt;, &lt;Level value in dB&gt;,
        ///                           .....;
        /// 
        ///   COMPOSITE EVM: &lt;Slot number&gt;, &lt;Value in %&gt;, .....;
        /// 
        /// SYMBOL EVM (TRACE2):
        /// 
        /// The number of values depends on the spreading factor:
        /// 
        /// Spreading factor 64 : 24 values Spreading factor 32 : 48 values:
        /// Spreading factor 16 : 96 values Spreading factor 8 : 192 values;
        /// Spreading factor 4 : 384 values Spreading factor 2 : 768 values;
        /// 
        ///   &lt;Value in % symbol 0&gt;, &lt;Value in % symbol 1&gt;,.....;
        /// 
        /// POWER VS SYMBOL (TRACE2):
        /// 
        /// The number of values depends on the spreading factor:
        /// 
        /// Spreading factor 64 : 24 values Spreading factor 32 : 48 values:
        /// Spreading factor 16 : 96 values Spreading factor 8 : 192 values;
        /// Spreading factor 4 : 384 values Spreading factor 2 : 768 values;
        /// 
        ///   &lt;Value in dBm symbol 0&gt;, &lt;Value in dBm symbol 1&gt;,.....;
        /// 
        /// SYMBOL CONST (TRACE2):
        /// 
        /// The number of value pairs depends on the spreading factor:
        /// 
        /// Spreading factor 64 : 24 pairs Spreading factor 32 : 48 pairs:
        /// Spreading factor 16 : 96 pairs Spreading factor 8 : 192 pairs;
        /// Spreading factor 4 : 384 pairs Spreading factor 2 : 768 pairs;
        /// 
        /// The real and imaginary component are transferred as a value pair.
        /// 
        ///   &lt;re 0&gt;,&lt;im 0&gt;,&lt;re 1&gt;,&lt;im 1&gt;,.....&lt;re n&gt;, &lt;im n&gt;
        /// 
        /// COMPOSITE CONST (TRACe2):
        /// 
        /// The number of value pairs corresponds to the number of chips from the 1536 chips in a power control group. Real and imaginary components are transferred as value pairs.
        /// 
        ///   &lt;re Chip 0&gt;, &lt;in Chip 0&gt;, &lt;re Chip 1&gt;, &lt;in Chip 1&gt;,.....;
        /// 
        /// 
        /// BITSTREAM (TRACE2):
        /// 
        /// The bit stream of a slot is output. A value is read out for each bit (value range 0,1), each symbol consists of one bit for BPSK channels.
        /// 
        /// Spreading factor 64 : 24 values Spreading factor 32 : 48 values:
        /// Spreading factor 16 : 96 values Spreading factor 8 : 192 values;
        /// Spreading factor 4 : 384 values Spreading factor 2 : 768 values;
        /// 
        /// If a channel is detected as being active, the invalid bits in the bit stream are identified by "9".
        /// 
        /// Example of bit stream trace: 0, 0, 1, 0, 1, 1, 0 ....
        /// 
        /// 
        /// ---------- Option FS-K84 ----------
        /// 
        /// 
        /// CODE DOMAIN POWER ABSOLUT/CODE DOMAIN POWER RELATIVE (TRACE1/TRACE2):
        /// 
        /// The following is output for each channel of the selected channel type:
        /// 
        /// Code number:
        ///   Code number of the channel, values between 0 to 31/15/63
        ///   depending on the channel type.
        /// 
        /// Level:
        ///   - For CODE DOMAIN POWER ABSOLUTE in unit dBm
        ///   - For CODE DOMAIN POWER RELATIVE in unit dB
        /// 
        /// Power ID:
        ///   0 - inactive channel
        ///   1 - active channel
        /// 
        /// Three values are thus transferred for all channels:
        /// 
        ///   &lt;Code number&gt;, &lt;Level&gt;, &lt;Power ID&gt;, etc.
        /// 
        /// The number of output codes corresponds to the spreading factor, which depends on the channel type: PILOT/PREAMBLE: 32, DATA: 16, MAC: 64. (See table above.) Depending on the mapping settings, the I or Q branch or the complex signal is returned. With Code Domain Overview, the I branch in TRACE1 and the Q branch in TRACE2 are always returned.
        /// 
        /// CODE DOMAIN ERROR POWER (TRACE1/TRACE2):
        /// 
        /// The following is output for each channel of the selected channel type:
        /// 
        /// Code number:
        ///   Code number of the channel, values between 0 to 31/15/63
        ///   depending on the channel type.
        /// 
        /// Error power:
        ///   In unit dB
        /// 
        /// Power ID:
        ///   0 - inactive channel
        ///   1 - active channel
        /// 
        /// Three values are thus transferred for all channels:
        /// 
        ///   &lt;Code number&gt;, &lt;Level&gt;, &lt;Power ID&gt;, etc.
        /// 
        /// The number of output codes corresponds to the spreading factor, which depends on the channel type: PILOT/PREAMBLE: 32, DATA: 16, MAC: 64. (See table above.) Depending on the mapping settings, the I or Q branch or the complex signal is returned. With Code Domain Overview, the I branch in TRACE1 and the Q branch in TRACE2 are always returned.
        /// 
        /// CHANNEL TABLE (TRACE1):
        /// 
        /// The following is output for each active channel:
        /// 
        /// Channel type:
        ///   The channel type is numerically coded as follows:
        ///   0 = PILOT
        ///   1 = MAC
        ///   2 = PREAMBLE with 64 chip length
        ///   3 = PREAMBLE with 128 chip length
        ///   4 = PREAMBLE with 256 chip length
        ///   5 = PREAMBLE with 512 chip length
        ///   6 = PREAMBLE with 1024 chip length
        ///   7 = DATA
        /// 
        /// Code class:
        ///   Fixed, depending on channel type: PILOT: 5, MAC: 6, PREAMBLE:
        ///   5 and DATA: 4 (spreading factor = 2^code class)
        /// 
        /// Code number:
        ///   Code number of channel, values between 0 and spreading
        ///   factor 1.
        /// 
        /// Modulation:
        ///   Modulation type including mapping:
        ///   0 = BPSK-I
        ///   1 = BPSK-Q
        ///   2 = QPSK
        ///   3 = 8-PSK
        ///   4 = 16-QAM
        /// 
        ///   Modulation types QPSK/8-PSK/16-QAM have complex values.
        /// 
        /// Absolute level:
        ///   In unit dBm
        /// 
        /// Relative level:
        ///   In unit dB, referred to the total power in the channel type.
        /// 
        /// Timing offset:
        ///   Referred to the first active channel in seconds.
        /// 
        /// Phase offset:
        ///   Referred to the first active channel in rad. If the analysis 
        ///   of the timing and phase offsets is not active (refer to
        ///   CDPower:TPMeas) or if there are more than 50 active channels
        ///   in the signal, a value of 9 is returned. A value of 9 is 
        ///   generally returned for inactive channels.
        /// 
        /// This means that 8 values are transmitted for all active channels:
        /// 
        ///   &lt;Channel type&gt;, &lt;Code class&gt;, &lt;Code number&gt;, &lt;Modulation&gt;, 
        ///   &lt;Absolute level&gt;, &lt;Relative level&gt;, &lt;Timing offset&gt;, &lt;Phase
        ///   offset&gt;, etc.
        /// 
        /// The channels are listed in the following channel type order: PILOT, MAC, PREAMBLE, DATA. Within the channel types, the channels are sorted in ascending code number order.
        /// 
        /// GENERAL RESULTS/CHANNEL RESULTS (TRACE2):
        /// 
        /// The results included in GENERAL RESULTS and CHANNEL RESULTS are read out in the following order:
        /// 
        ///   &lt;FERRor&gt;, &lt;FERPpm&gt;, &lt;CERRor&gt;, &lt;TFRame&gt;, &lt;RHOPilot&gt;, &lt;RHO1&gt;, 
        ///   &lt;RHO2&gt;, &lt;PPILot&gt;, &lt;PMAC&gt;, &lt;PDATa&gt;, &lt;PPReamble&gt;, 
        ///   &lt;MACCuracy&gt;,&lt;DMTYpe&gt;, &lt;MACTive&gt;, &lt;DACTive&gt;, &lt;PLENGth&gt;, &lt;RHO&gt;,
        ///   &lt;PCDerror&gt; , &lt;IQIMbalance&gt;, &lt;IQOFfset&gt;, &lt;SRATe&gt;, &lt;CHANnel&gt;,
        ///   &lt;SFACtor&gt; &lt;TOFFset&gt;, &lt;POFFset&gt;, &lt;CDPRelative&gt;, &lt;CDPabsolute&gt;,
        ///   &lt;EVMRms&gt;, &lt;EVMPeak&gt;, &lt;MTYPe&gt;
        /// 
        /// In this instance the results have the following meaning and units:
        /// 
        /// Global results for all slots:
        ///   FERRor - Frequency error in Hz
        ///   FERPpm - Frequency error in ppm
        ///   CERRor - Chip rate error in ppm
        ///   TFRame - Trigger to Frame
        ///   RHOPilot - RHO over all slots for the pilot area
        ///   RHO1 - RHOoverall-1 over all slots over all chips with start
        ///          of averaging at the half-slot limit
        ///   RHO2 - RHOoverall-2 over all slots over all chips with start
        ///          of averaging at the quarter-slot limit
        /// 
        /// Results for the selected slot:
        ///   PPILot - Absolute power in the PILOT channel type
        ///   PMAC - Absolute power in the MAC channel type
        ///   PDATa - Absolute power in the DATA channel type
        ///   PPReamble - Absolute power in the PREAMBLE channel type
        ///   MACCuracy - Composite EVM in %
        ///   DMTYpe - Modulation type in the DATA channel type:
        ///     2 = QPSK
        ///     3 = 8-PSK
        ///     4 = 16-QAM
        ///   MACTive - Number of active MAC channels
        ///   DACTive - Number of active DATA channels
        ///   PLENGth - Length of preamble in chips
        ///   RHO - RHO value for the selected channel type/slot
        /// 
        /// Results for the selected channel type in the selected slot:
        ///   PCDerror - Peak code domain error in dB
        ///   IQIMbalance - IQ imbalance in %
        ///   IQOFfset - IQ offset in %
        /// 
        /// Channel results:
        ///   SRATe - Symbol rate in ksps
        ///   CHANnel - Channel number
        ///   SFACtor - Spreading factor of the channel
        ///   TOFFset - Timing offset in s
        ///   POFFset - Phase offset in rad
        ///   CDPRelative - Channel power, relative, in dB
        ///   CDPabsolute - Channel power, absolute, in dBm
        ///   EVMRms - Error vector magnitude RMS in %
        ///   EVMPeak - Error vector mag. peak in %
        ///   MTYPe - Modulation type:
        ///     0 = BPSK-I
        ///     1 = BPSK-Q
        ///     2 = QPSK
        ///     3 = 8-PSK
        ///     4 = 16-QAM
        ///     5 = 2BPSK (if complex analysis selected for PILOT, PREAMBLE
        ///         or MAC)
        /// 
        /// POWER VS CHIP
        /// 
        /// The number of returned values is 2048, since one level value is available per chip.
        /// 
        ///   &lt;Level value in dBm&gt;, &lt;Level value in dBm&gt;, etc.;
        /// 
        /// PEAK CODE DOMAIN ERR and COMPOSITE EVM (TRACE2):
        /// 
        /// The number of returned value pairs corresponds to the IQ capture length. (Refer to the command CDPower:IQLength).
        /// 
        ///   PEAK CODE DOMAIN ERROR:
        /// 
        ///     &lt;Slot number&gt;, &lt;Level value in dB&gt;, .....;
        /// 
        ///   COMPOSITE EVM:
        /// 
        ///     &lt;Slot number&gt;, &lt;Value in %&gt;, .....;
        /// 
        /// SYMBOL EVM (TRACE2):
        /// 
        /// The number of values depends on the number of symbols and is between 2 and 100. Refer in this respect to the table above in the column 'Number of symbols per slot and code'.
        /// 
        ///   &lt;Value in % symbol 0&gt;, &lt;Value in % symbol 1&gt;,.....;
        /// 
        /// POWER VS SYMBOL (TRACE2):
        /// 
        /// The number of values depends on the number of symbols and is between 2 and 100. Refer in this respect to the table above in the column 'Number of symbols per slot and code'.
        /// 
        ///   &lt;Value in dBm symbol 0&gt;, &lt;Value in dBm symbol 1&gt;,.....;
        /// 
        /// SYMBOL CONST (TRACE2):
        /// 
        /// The number of value pairs depends on the number of symbols and is between 2 and 100. Refer in this respect to the table above in the column 'Number of symbols per slot and code'. Real and imaginary parts are transferred as value pairs.
        /// 
        ///   &lt;re 0&gt;,&lt;im 0&gt;,&lt;re 1&gt;,&lt;im 1&gt;,.....&lt;re n&gt;, &lt;im n&gt;
        /// 
        /// COMPOSITE CONST (TRACe2):
        /// 
        /// The number of value pairs corresponds to the number of chips for the channel type. The value is between 64 and 1600 ships per slot. Refer in this respect to the table above in the column 'Chips per slot'.
        /// 
        ///   &lt;re Chip 0&gt;, &lt;in Chip 0&gt;, &lt;re Chip 1&gt;, &lt;in Chip 1&gt;,.....;
        /// 
        /// BITSTREAM (TRACE2):
        /// 
        /// The bit stream of a slot is output. A value is read out for each bit (range 0,1), each symbol consists of 1 bit for BPSK-I/BPSK-Q, 2 bits for QPSK and 2BPSK (Complex BPSK-I/BPSK-Q analysis), 3 bits for 8-PSK and 4 bits for 16-QAM. The bit stream per slot is accordingly of different lengths. Refer in this respect to the table above in the column 'Bits per slot and code'. The number of bits is between 2 and 400.
        /// 
        /// Example of bit stream trace: 0, 0, 1, 0, 1, 1, 0 ....
        /// 
        /// 
        /// ---------- Option FS-K85 ----------
        /// 
        /// 
        /// CODE DOMAIN POWER ABSOLUTE / CODE DOMAIN POWER RELATIVE (TRACE1):
        /// 
        /// The following is output for each channel:
        /// 
        /// Code class:
        ///   Code class of the channel; with Hadamard order it is usually
        ///   code class 4. With BitReverse order, values are between
        ///   2 and 4.
        /// 
        /// Code number:
        ///   Code number of channel, values between 0 and 15.
        /// 
        /// Level:
        ///   - For CODE DOMAIN POWER ABSOLUTE, units are dBm
        ///   - For CODE DOMAIN POWER RELATIVE, units are dB
        ///     (referred to the total or pilot power, see the command 
        ///     CDPower:PREFerence). Power values of the individual codes
        ///     are usually given in Hadamard order; the consolidated 
        ///     channel power is returned in BitReverse order.
        /// 
        /// Power ID:
        ///   0 - Inactive channel
        ///   1 - Active channel
        ///   3 - Quasi-inactive channel (on the analyzed branch, the
        ///       channel is not occupied, but an active channel exists on
        ///       the other branch)
        /// 
        /// Four values are thus transferred for all channels:
        /// 
        ///   &lt;Code class&gt;, &lt;Code number&gt;, &lt;Level&gt;, &lt;Power ID&gt;
        /// 
        /// The Hadamard or BitReverse order is important for sorting the channels and consolidation (see the command CDPower:ORDer). With Hadamard, the individual codes are output in ascending order with their code power. The number of codes which are output corresponds to spreading factor 16. With BitReverse, codes which belong to a particular channel are adjacent to each other and are therefore output in the class of the channel together with the channel power. The maximum number of codes or channels that are output cannot be higher than spreading factor 16, and decreases with each concentrated channel.
        /// 
        /// CODE DOMAIN ERROR POWER (TRACE1):
        /// 
        /// The following is output for each channel:
        /// 
        /// Code class:
        ///   Code class of the channel is usually 4 since the CDEP is
        ///   displayed in base spreading factor 16.
        /// 
        /// Code number:
        ///   Code number of channel, values between 0 and 15.
        /// 
        /// Error power:
        ///   In dB. No difference of power between the Hadamard and
        ///   BitReverse order.
        /// 
        /// Power ID:
        ///   0 -   Inactive channel 
        ///   1 - Active channel
        ///   3 - Quasi-inactive channel (on the analyzed branch, the
        ///       channel is not occupied, but an active channel exists on
        ///       the other branch)
        /// 
        /// Four values are thus transferred for all channels:
        /// 
        ///   &lt;Code class&gt;, &lt;Code number&gt;, &lt;Level&gt;, &lt;Power ID&gt;
        /// 
        /// The Hadamard or BitReverse order is important for sorting the channels (see the CDPower:ORDer command). With Hadamard order, the individual codes are output in ascending order. With BitReverse order, codes which belong to a particular channel are adjacent to each other. Since an error power is output for the code domain error power, consolidation of the power values is not appropriate. The number of codes that are output therefore generally corresponds to base spreading factor 16.
        /// 
        /// CHANNEL TABLE (TRACE1):
        /// 
        /// The following is output for each channel:
        /// 
        /// Channel type:
        ///   The channel type is coded by numbers as follows:
        ///   0 = PICH 
        ///   1 = RRI
        ///   2 = DATA
        ///   3 = ACK
        ///   4 = DRC
        ///   5 = INACTIVE
        /// 
        /// Code class:
        ///   Code class of channel, values between 2 and 4.
        /// 
        /// Code number:
        ///   Code number of channel, values between 0 and 15.
        /// 
        /// Mapping:
        ///   0 = I branch
        ///   1 = Q branch
        /// 
        /// Absolute level:
        ///   In dBm
        /// 
        /// Relative level:
        ///   In dB, referred to the total or pilot power (see the 
        ///   CDPower:PREFerence command)
        /// 
        /// Timing offset:
        ///   Referred to the pilot in seconds.
        /// 
        /// Phase offset:
        ///   Referred to the pilot in rad. If the evaluation of the timing
        ///   and phase offset is not active (see CDPower:TPMeas) or more
        ///   than 50 active channels are in the signal, the value 9 is
        ///   returned. For inactive channels, the value 9 is usually
        ///   returned.
        /// 
        /// The class specifies the spreading factor of the channel:
        /// 
        /// Class 4 corresponds to spreading factor 16 (symbol rate 76.8 ksps), class 2 to the lowest permissible spreading factor 4 (symbol rate 307.2 ksps).
        /// 
        /// Eight values are thus transferred for all channels:
        /// 
        ///   &lt;Channel type&gt;, &lt;Code class&gt;, &lt;Code number&gt;, &lt;Mapping&gt;,
        ///   &lt;Absolute level&gt;, &lt;Relative level&gt;, &lt;Timing offset&gt;, &lt;Phase
        ///   offset&gt;
        /// 
        /// All detected active channels are output first, followed by the inactive or quasi-active channels. The channels are sorted in ascending code number order (with identical code numbers: the I branch first, followed by the Q branch). The unassigned codes are displayed together with code class 4.
        /// 
        /// RESULT SUMMARY (TRACE2):
        /// 
        /// The results of RESULT SUMMARY are output in the following order:
        /// 
        ///   &lt;SLOT&gt;, &lt;PTOTal&gt;, &lt;PPICh&gt;, &lt;PRRI&gt;,  &lt;RHO&gt;, &lt;MACCuracy&gt;,
        ///   &lt;PCDerror&gt;, &lt;ACTive&gt;, &lt;FERRor&gt;, &lt;FERPpm&gt;, &lt;DRPich&gt;,
        ///   &lt;RHOVerall&gt;, &lt;TFRame&gt;, &lt;CERRor&gt;, &lt;IQOFfset&gt;, &lt;IQIMbalance&gt;,
        ///   &lt;SRATe&gt;, &lt;CHANnel&gt;, &lt;SFACtor&gt;, &lt;TOFFset&gt;, &lt;POFFset&gt;, 
        ///   &lt;CDPRelative&gt;, &lt;CDPabsolute&gt;, &lt;EVMRms&gt;, &lt;EVMPeak&gt;
        /// 
        /// The results have the following meanings and units:
        /// 
        /// Global results of selected half slot:
        /// 
        ///   SLOT - Half-slot number
        ///   PTOTal - Total power in dBm
        ///   PPICh - Pilot power in dBm
        ///   PRRI - RRI power in dBm
        ///   RHO - RHO
        ///   MACCuracy - Composite EVM in %
        ///   PCDerror - Peak code domain error in dB
        ///   IQOFfset - IQ offset in %
        ///   IQIMbalance - IQ imbalance in %
        /// 
        /// Global results of all half slots:
        /// 
        ///   FERRor - Frequency error in Hz
        ///   FERPpm - Frequency error in ppm
        ///   DRPich - Delta RRI/PICH in dB
        ///   CERRor - Chip rate error in ppm
        ///   TFRame - Trigger to frame
        ///   ACTive - Number of active channels
        ///   RHOverall - RHO over all half slots
        /// 
        /// Channel results:
        /// 
        ///   SRATe - Symbol rate in ksps
        ///   CHANnel - Channel number
        ///   SFACtor - Spreading factor of channel
        ///   CDPRelative - Channel power relative in dB
        ///     (relative to total or PICH power, see command CDP:PREF)
        ///   EVMRms - Error vector magnitude RMS in %
        ///   TOFFset - Timing offset in s
        ///   POFFset - Phase offset in rad
        ///   CDPabsolute - Channel power absolute in dBm
        ///   EVMPeak - Error vector magnitude peak in %
        /// 
        /// POWER VS HALFSLOT, PEAK CODE DOMAIN ERR and COMPOSITE EVM (TRACE2):
        /// 
        /// The number of returned value pairs corresponds to the IQ capture length. (See command CDPower:IQLength).
        /// 
        ///   POWER VS HALFSLOT: &lt;Half-slot number&gt;,&lt;Level value in dB&gt;,
        ///                      &lt;Half-slot number&gt;, &lt;Level value in
        ///                      dB&gt;,.....;
        /// 
        ///   PEAK CODE DOMAIN ERROR: &lt;Half-slot number&gt;, &lt;Level value in
        ///                           dB&gt;, .....;
        /// 
        ///   COMPOSITE EVM: &lt;Half-slot number&gt;, &lt;Value in %&gt;, .....;
        /// 
        /// SYMBOL EVM (TRACE2):
        /// 
        /// The number of values depends on the spreading factor:
        ///   Spreading factor 16 : 64 values
        ///   Spreading factor 8  : 128 values
        ///   Spreading factor 4  : 256 values
        /// 
        ///   &lt;Value in % symbol 0&gt;, &lt;Value in % symbol 1&gt;,.....;
        /// 
        /// POWER VS SYMBOL (TRACE2):
        /// 
        /// The number of values depends on the spreading factor:
        /// 
        ///   Spreading factor 16 : 64 values
        ///   Spreading factor 8  : 128 values
        ///   Spreading factor 4  : 256 values
        /// 
        ///   &lt;Value in dBm symbol 0&gt;, &lt;Value in dBm symbol 1&gt;,.....;
        /// 
        /// SYMBOL CONST (TRACE2):
        /// 
        /// The number of value pairs depends on the spreading factor:
        ///   Spreading factor 16 : 64  values
        ///   Spreading factor 8  : 128 values
        ///   Spreading factor 4  : 256 values
        /// 
        /// Real and imaginary components are transferred as value pairs.
        /// 
        ///   &lt;re 0&gt;,&lt;im 0&gt;,&lt;re 1&gt;,&lt;im 1&gt;,.....&lt;re n&gt;, &lt;im n&gt;
        /// 
        /// BIT STREAM (TRACE2):
        /// 
        /// The bit stream of a slot is output. A value is read out for each bit (value range 0,1); each symbol consists of one bit for BPSK channels.
        /// 
        ///   Spreading factor 16 : 64  values
        ///   Spreading factor 8  : 128 values
        ///   Spreading factor 4  : 256 values
        /// 
        /// If a channel is detected as being inactive, the invalid bits in the bit stream are identified by "9".
        /// 
        /// Example of a bit stream trace: 0, 0, 1, 0 , 1, 1, 0 ....
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadC2KTraceData(int Window, int Source_Trace, int Array_Length, out int Actual_Points, double[] Values)
        {
            int pInvokeResult = PInvoke.ReadC2KTraceData(this._handle, Window, Source_Trace, Array_Length, out Actual_Points, Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads trace data out of the instrument. Data can be queried depending on the display mode.
        /// 
        /// Notes:
        /// 
        /// (1) CTABLE:
        /// In addition to the results of the channel table which are output using the :TRACe[:DATA]? TRACE1 command, active timing and phase offset measurement (refer to CDPower:TPMeas) also has the CTABle query command which displays the maximum values of the TIMING and PHASE OFFSET together with the associated channel.
        /// 
        /// (2) This function is available for option FS-K82, FS-K83, FS-K84 or FS-K85 only.
        /// 
        /// (3) The associated measurement window is selected with the function Display Set Active Window (rsspecan_DisplaySetActiveWindow).
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;[:DATA]? CTABle
        /// FORMat[:DATA] REAL,32
        /// *CLS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Source_Trace">
        /// This parameter indicates which trace will be read out.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_CTABLE (0) - Channel Table (Default Value)
        /// 
        /// </param>
        /// <param name="Array_Length">
        /// Number of array points requested.
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Actual_Points">
        /// Number of points actually returned in the Amplitude array and X Array array.
        /// </param>
        /// <param name="Values">
        /// A user allocated buffer into which the trace values are stored.
        /// 
        /// Returned Values:
        /// 
        /// Option FS-K82, FS-K83 - the following values are output:
        /// 
        /// &lt;Max. time offset in s&gt;, &lt;Code number for max. time&gt;, &lt;Code
        /// class for max. time&gt;, &lt;Max. phase offset in rad&gt;, &lt;Code number for max. phase&gt;, &lt;Code class for max. phase&gt;, &lt;Reserved 1&gt;, ..., &lt;Reserved 6&gt;
        /// 
        /// Option FS-K84 - the following values are output:
        /// 
        /// &lt;Max. time offset in s&gt;, &lt;Channel Type&gt;, &lt;Code number for max. time&gt;, &lt;Code class for max. time&gt;, &lt;Max. phase offset in rad&gt;, &lt;Channe type&gt;, &lt;Code number for max. phase&gt;, &lt;Code class for max. phase&gt;, &lt;Reserved 1&gt;, ..., &lt;Reserved 4&gt;
        /// 
        /// Option FS-K85 - the following values are output:
        /// 
        /// &lt;Max. time offset in s&gt;, &lt;Code number for max. time&gt;, &lt;Code class for max. time&gt;, &lt;Max. phase offset in rad&gt;, &lt;Code number for max. phase&gt;, &lt;Code class for max. phase&gt;, &lt;Reserved 1&gt;, ..., &lt;Reserved 6&gt;
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadC2KCDPTraceData(int Window, int Source_Trace, int Array_Length, out int Actual_Points, double[] Values)
        {
            int pInvokeResult = PInvoke.ReadC2KCDPTraceData(this._handle, Window, Source_Trace, Array_Length, out Actual_Points, Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the CDMA2000 Base Station Test mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument[:SELect] BC2K
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int BC2KMode()
        {
            int pInvokeResult = PInvoke.BC2KMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the values of the selected channel table in the code domain power measurement mode. The whole table is defined in one go. The inactive channels (INACtive) do not have to be defined. Eight values are specified for a line of table.
        /// 
        /// Prior to this function, the name of the channel table has to be defined with function Configure CDMA2000 Channel Table Operations (ConfigureC2KChannelTableOperations) - attribute RSSPECAN_ATTR_C2K_CTABLE_NAME.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for option K82.
        /// 
        /// Attribute(s):
        /// None
        /// 
        /// Remote-control command(s):
        /// CONFigure:CDPower[:BTS]:CTABle:DATA 0..11, 2..7, 0..127, 0..5, 0, 0, 0 | 1, ...
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Channel_Type">
        /// This control selects the Channel Type. 
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0   - PICH
        /// 1   - SYNC
        /// 2   - PCH
        /// 3   - TDPICH
        /// 4   - APICH
        /// 5   - ATDPICH
        /// 6   - BCH
        /// 7   - CPCCH
        /// 8   - CACH
        /// 9   - CCCH
        /// 10  - CHAN
        /// 11  - INACTIVE
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Code_Class">
        /// This control sets the Code Class.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 2 to 7
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Code_Number">
        /// This control sets the Code Number.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0 to 127
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Radio_Configuration">
        /// This control sets the Radio Configuration.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0 to 5
        /// 
        /// Default Value: None
        /// 
        /// Notes:
        /// 
        /// (1) 0 for all special channels; for the others (CHAN), the values 1, 2, 3, 4 and 5 are allowed.
        /// </param>
        /// <param name="Reserved_1">
        /// This control is reserver for future use.
        /// </param>
        /// <param name="Reserved_2">
        /// This control is reserver for future use.
        /// </param>
        /// <param name="Status">
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </param>
        /// <param name="CDP_Relative">
        /// This control sets the CDP Relative.
        /// 
        /// Units: dB
        /// 
        /// Valid Values:
        /// None
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Num_of_Elements">
        /// Sets number of elements for parameter arrays.
        /// 
        /// Valid Values: &gt;= 1
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBC2KChannelTableData(int[] Channel_Type, int[] Code_Class, int[] Code_Number, int[] Radio_Configuration, int[] Reserved_1, int[] Reserved_2, ushort[] Status, double[] CDP_Relative, int Num_of_Elements)
        {
            int pInvokeResult = PInvoke.ConfigureBC2KChannelTableData(this._handle, Channel_Type, Code_Class, Code_Number, Radio_Configuration, Reserved_1, Reserved_2, Status, CDP_Relative, Num_of_Elements);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the advanced settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_CDP_PNOFFSET
        /// RSSPECAN_ATTR_C2K_CDP_SFACTOR
        /// RSSPECAN_ATTR_C2K_CDP_PREFERENCE
        /// RSSPECAN_ATTR_C2K_CDP_ORDER
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:PNOFfset 0 to 511
        /// [SENSe:]CDPower:SFACtor 64 | 128
        /// [SENSe:]CDPower:PREFerence TOTal | PICH
        /// [SENSe:]CDPower:ORDer HADamard | BITReverse
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="PN_Offset">
        /// This control sets the PN offset of the base station in multiples of 64 chips.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0 to 511
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_CDP_PNOFFSET
        /// 
        /// </param>
        /// <param name="Base_Spreading_Factor">
        /// This control defines the base spreading factor.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_SFACTOR_64   (0) - 64
        /// RSSPECAN_VAL_C2K_SFACTOR_128  (1) - 128
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_SFACTOR_64
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_CDP_SFACTOR
        /// 
        /// Notes:
        /// 
        /// (1) The base spreading factor should be set to 128 when there are channels of code class 7 (spreading factor 128), since alias power values will otherwise be measured for the code domain power analysis.
        /// </param>
        /// <param name="Power_Reference">
        /// This control sets the reference for the relative CDP measured values to the total power or the PICH power (with antenna 2 the TDPICH power is used).
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_PREF_TOTAL   (0) - Total
        /// RSSPECAN_VAL_C2K_PREF_PICH    (1) - PICH
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_PREF_TOTAL
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_CDP_PREFERENCE
        /// </param>
        /// <param name="Order">
        /// This control sets the order of the code domain evaluation.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_ORDER_HADAMARD   (0) - Hadamard
        /// RSSPECAN_VAL_C2K_ORDER_BITREVERSE (1) - Bitreverse
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_ORDER_HADAMARD
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_CDP_ORDER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBC2KAdvancedSettings(int PN_Offset, int Base_Spreading_Factor, int Power_Reference, int Order)
        {
            int pInvokeResult = PInvoke.ConfigureBC2KAdvancedSettings(this._handle, PN_Offset, Base_Spreading_Factor, Power_Reference, Order);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the 1xEV-DV system.
        /// 
        /// Note(s):
        /// 
        /// (1) This is necessary in the automatic search mode to support the channel detection. If there is a QPSK modulated channel with spreading-factor 32 it is not possible to distinguish whether it is a data channel (CHAN with radio configuration 3-5) or a PACKET DATA CHANNEL (PDCH with radio configuration 10). This decision is made for the PDCH of the 1xEV-DV system if the value is ON. If the value is OFF (default) the CHAN is detected.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_FORCE_DV
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:FDV ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="_1xEV_DV">
        /// This control switches the 1xEV-DV system.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_FORCE_DV
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBC2KForce1xEVDV(bool _1xEV_DV)
        {
            int pInvokeResult = PInvoke.ConfigureBC2KForce1xEVDV(this._handle, System.Convert.ToUInt16(_1xEV_DV));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Antenna Settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_CDP_ANTENNA
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:ANTenna OFF | 1 | 2
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Antenna_Diversity">
        /// This control selects Antenna mode for which the evaluation has to be made. With OFF, no signals with transmit diversity are expected.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_ANTENNA_OFF  (0) - Off
        /// VI_TRUE                       (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_CDP_ANTENNA
        /// 
        /// Notes:
        /// 
        /// (1) If this parameter is turned off, the antenna number is ignored.
        /// </param>
        /// <param name="Antenna_Number">
        /// This control sets the antenna number.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_CDP_ANTENNA
        /// 
        /// Notes:
        /// 
        /// (1) The value from this control is applied only when the Antenna Diversity is set to On.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBC2KAntennaSettings(bool Antenna_Diversity, int Antenna_Number)
        {
            int pInvokeResult = PInvoke.ConfigureBC2KAntennaSettings(this._handle, System.Convert.ToUInt16(Antenna_Diversity), Antenna_Number);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Multi Carrier settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_MCARRIER_STATE
        /// RSSPECAN_ATTR_C2K_MCARRIER_ENH_ALG
        /// RSSPECAN_ATTR_C2K_MCARRIER_FILTER_STATE
        /// RSSPECAN_ATTR_C2K_MCARRIER_FILTER_TYPE
        /// RSSPECAN_ATTR_C2K_MCARRIER_ROLL_OFF
        /// RSSPECAN_ATTR_C2K_MCARRIER_CUT_OFF_FREQ
        /// 
        /// Remote-control command(s):
        /// CONFigure:CDPower:MCARier ON | OFF
        /// CONFigure:CDPower:MCARier:MALGo ON | OFF
        /// CONFigure:CDPower:MCARier:FILTer[:STATe] ON | OFF
        /// CONFigure:CDPower:MCARier:FILTer:TYPE LPASs | RRC
        /// CONFigure:CDPower:MCARier:FILTer:ROFF &lt;numeric_value&gt;
        /// CONFigure:CDPower:MCARier:FILTer:COFRequency &lt;numeric_value&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Multi_Carrier">
        /// This control selects Multi Carrier mode.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE  (0) - Off
        /// VI_TRUE   (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_MCARRIER_STATE
        /// 
        /// Notes:
        /// 
        /// (1) If this parameter is turned off, the Enhanced Algorithm, MC Filter and Filter Type parameters are ignored.
        /// </param>
        /// <param name="Enhanced_Algorithm">
        /// This control selects Enhanced Algorithm.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE  (0) - Off
        /// VI_TRUE   (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_MCARRIER_ENH_ALG
        /// </param>
        /// <param name="MC_Filter">
        /// This control selects MC Filter state.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE  (0) - Off
        /// VI_TRUE   (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_MCARRIER_FILTER_STATE
        /// 
        /// Notes:
        /// 
        /// (1) If this parameter is turned off, the Filter Type parameter is ignored.
        /// 
        /// (2) If the Enhanced Algorithm is set to Off and this parameter is then set to Off, it will turn off the whole Multi Carrier system.
        /// </param>
        /// <param name="Filter_Type">
        /// This control selects the Filter Type.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_MCARRIER_FTYPE_LPASS  (0) - Low pass
        /// RSSPECAN_VAL_C2K_MCARRIER_FTYPE_RRC    (1) - RRC
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_MCARRIER_FTYPE_LPASS
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_MCARRIER_FILTER_TYPE
        /// 
        /// Notes: 
        /// 
        /// (1) If this parameter is set to Low pass, the Roll Off and Cut Off Frequency parameters are ignored.
        /// </param>
        /// <param name="Roll_Off">
        /// This control sets Roll Off Factor. 
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0.01 to 0.99
        /// 
        /// Default Value: 0.02
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_MCARRIER_ROLL_OFF
        /// 
        /// Notes: 
        /// 
        /// (1) If the Filter Type parameter is set to Low pass, this parameter is ignored.
        /// </param>
        /// <param name="Cut_Off_Frequency">
        /// This control sets Cut Off Frequency.
        /// 
        /// Units: Hz
        /// 
        /// Valid Values:
        /// 100.0e+3 to 2.4e+6
        /// 
        /// Default Value: 1.25e+6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_MCARRIER_CUT_OFF_FREQ
        /// 
        /// Notes: 
        /// 
        /// (1) If the Filter Type parameter is set to Low pass, this parameter is ignored.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBC2KMultiCarrier(bool Multi_Carrier, bool Enhanced_Algorithm, bool MC_Filter, int Filter_Type, double Roll_Off, double Cut_Off_Frequency)
        {
            int pInvokeResult = PInvoke.ConfigureBC2KMultiCarrier(this._handle, System.Convert.ToUInt16(Multi_Carrier), System.Convert.ToUInt16(Enhanced_Algorithm), System.Convert.ToUInt16(MC_Filter), Filter_Type, Roll_Off, Cut_Off_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets selected marker to channel 0.64.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_MARKER_PICH
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:PICH
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBC2KPichMarkerSettings()
        {
            int pInvokeResult = PInvoke.ConfigureBC2KPichMarkerSettings(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets selected marker to channel 16.128.
        /// 
        /// Notes:
        /// 
        /// (1) The base spreading factor (RSSPECAN_ATTR_C2K_CDP_SFACTOR attribute) should be set to 128 before hand.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_MARKER_TDPICH
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:TDPIch
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBC2KTDPichMarkerSettings()
        {
            int pInvokeResult = PInvoke.ConfigureBC2KTDPichMarkerSettings(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the measured data that will be displayed.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_RESULT_TYPE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FEED 'XPOW:CDP' | 'XPOW:CDP:RAT' | 'XPOW:CDEP' | 'XTIM:CDP:MACCuracy' | 'XTIM:CDP:PVSLot' | 'XTIM:CDP:PVSYmbol' | 'XTIM:CDP:BSTReam' | 'XTIM:CDP:ERR:SUMM' | 'XTIM:CDP:ERR:CTABle' | 'XTIM:CDP:ERR:PCDomain' | 'XTIM:CDP:SYMB:CONSt' | 'XTIM:CDP:SYMB:EVM' | 'XTIM:CDP:COMP:CONSt'
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Result_Type">
        /// This control selects the results to be displayed on screen A.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_FEED_CDP       (0)  -  Code Domain Power Absolute   
        /// RSSPECAN_VAL_C2K_FEED_CDP_RAT   (1)  -  Code Domain Power Relative   
        /// RSSPECAN_VAL_C2K_FEED_CDEP      (2)  -  Code Domain Error            
        /// RSSPECAN_VAL_C2K_FEED_ERR_CTAB  (3)  -  Channel Table                
        /// RSSPECAN_VAL_C2K_FEED_MACC      (4)  -  Composite EVM                
        /// RSSPECAN_VAL_C2K_FEED_PVSL      (5)  -  Power vs PCG                 
        /// RSSPECAN_VAL_C2K_FEED_PVSY      (6)  -  Power vs Symbol              
        /// RSSPECAN_VAL_C2K_FEED_BSTR      (7)  -  Bitsream                     
        /// RSSPECAN_VAL_C2K_FEED_ERR_SUMM  (8)  -  Result Summary               
        /// RSSPECAN_VAL_C2K_FEED_ERR_PCD   (9)  -  Peak Code Domain Error       
        /// RSSPECAN_VAL_C2K_FEED_SYMB_CONS (10) -  Symbol Const                 
        /// RSSPECAN_VAL_C2K_FEED_SYMB_EVM  (11) -  Symbol EVM                   
        /// RSSPECAN_VAL_C2K_FEED_COMP_CONS (12) 	-  Composite Const             
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_FEED_CDP
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_RESULT_TYPE
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBC2KCodeDomainAnalyzerResults(int Result_Type)
        {
            int pInvokeResult = PInvoke.ConfigureBC2KCodeDomainAnalyzerResults(this._handle, Result_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the Code Domain Analyzer Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for option K82.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_CDP_IQLENGTH_FSQ_FSU
        /// RSSPECAN_ATTR_C2K_CDP_IQLENGTH_FSP
        /// RSSPECAN_ATTR_C2K_CDP_CODE
        /// RSSPECAN_ATTR_C2K_CDP_SLOT
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:IQLength
        /// [SENSe:]CDPower:CODE 0 to (BASE SF-1)
        /// [SENSe:]CDPower:SLOT 0  to (IQ_CAPTURE_LENGTH-1)
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Capture_Length">
        /// This control sets the Captured Length.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 2 to 64 for FSU, FSQ
        /// 2 to 12 for FSP
        /// 
        /// 
        /// Default Value: 3
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_CDP_IQLENGTH_FSP RSSPECAN_ATTR_C2K_CDP_IQLENGTH_FSQ_FSU
        /// 
        /// </param>
        /// <param name="Select_Channel">
        /// This control selects the code number..
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0 to (Base Spreading Factor - 1)
        /// 
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_CDP_CODE
        /// 
        /// Notes:
        /// 
        /// (1) The values are shown on the instrument like this: &lt;channel number&gt;,&lt;spreading factor&gt;
        /// </param>
        /// <param name="Select_PCG">
        /// This control selects the Power Control Group. 
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0 to (IQ_CAPTURE_LENGTH-1)
        /// 
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_CDP_SLOT
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBC2KCodeDomainAnalyzerSettings(int Capture_Length, int Select_Channel, int Select_PCG)
        {
            int pInvokeResult = PInvoke.ConfigureBC2KCodeDomainAnalyzerSettings(this._handle, Capture_Length, Select_Channel, Select_PCG);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the measured and calculated values of the code domain power analysis.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for option K82.
        /// 
        /// Attribute(s):
        /// None
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:FUNCtion:CDPower[:BTS]:RESult? SLOT | PTOTal | PPICh | RHO | MACCuracy | PCDerror | ACTive | FERRor | FERPpm | CERRor | TFRame I IQOFfset | IQIMbalance | SRATe | CHANnel | SFACtor | TOFFset | POFFset | CDPabsolute | CDPRelative | EVMRms | EVMPeak | DMTYpe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Result_Type">
        /// This control selects the result type.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_SLOT          (0) - PCG
        /// RSSPECAN_VAL_C2K_PTOTAL        (1) - Total PWR
        /// RSSPECAN_VAL_C2K_PPICH         (2) - Pilot PWR
        /// RSSPECAN_VAL_C2K_RHO           (3) - RHO
        /// RSSPECAN_VAL_C2K_MACCURACY     (4) - Composite EVM
        /// RSSPECAN_VAL_C2K_PCDERROR      (5) - Peak Code Domain Error
        /// RSSPECAN_VAL_C2K_IQIMBALANCE   (6) - IQ Imbalance
        /// RSSPECAN_VAL_C2K_IQOFFSET      (7) - IQ Offset
        /// RSSPECAN_VAL_C2K_FERROR        (8) - Carrier Freq Error Hz
        /// RSSPECAN_VAL_C2K_FERPPM        (9) - Carrier Freq Error ppm
        /// RSSPECAN_VAL_C2K_CERROR        (10) - Chip Rate Error
        /// RSSPECAN_VAL_C2K_TFRAME        (11) - Trigger to Frame
        /// RSSPECAN_VAL_C2K_ACTIVE        (12) - No of Active Chan
        /// RSSPECAN_VAL_C2K_SRATE         (13) - Symbol Rate
        /// RSSPECAN_VAL_C2K_CHANNEL       (14) - Channel
        /// RSSPECAN_VAL_C2K_SFACTOR       (15) - Spreading Factor
        /// RSSPECAN_VAL_C2K_TOFFSET       (16) - Timing Offset
        /// RSSPECAN_VAL_C2K_POFFSET       (17) - Phase Offset
        /// RSSPECAN_VAL_C2K_CDPRELATIVE   (18) - Channel Power Rel
        /// RSSPECAN_VAL_C2K_CDPABSOLUTE   (19) - Channel Power Abs
        /// RSSPECAN_VAL_C2K_EVMRMS        (20) - Symbol EVM rms
        /// RSSPECAN_VAL_C2K_EVMPEAK       (21) - Symbol EVM Pk
        /// RSSPECAN_VAL_C2K_DMTYPE        (29) - Modulation Type
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_SLOT
        /// 
        /// Notes:
        /// 
        /// (1) Total Power: Specifies the total power of the signal.
        /// 
        /// (2) Pilot power: Specifies the pilot power. If antenna 2 (refer to ANT NO softkey) is selected, the power of the TDPICH is displayed, in all other cases that of the PICH.
        /// 
        /// (3) RHO: Specifies RHO, the quality parameter. According to the
        /// cdma2000 standards, RHO is the normalized, correlated power between the measured and the ideally generated reference signal. When RHO is measured, the cdma2000 standards require that only the pilot channel be supplied.
        /// 
        /// (4) Composite EVM: The composite EVM value is the difference between the test signal and the ideal reference signal (refer to the COMPOSITE EVM softkey).
        /// 
        /// (5) Pk CDE: The PEAK CODE DOMAIN ERR measurement specifies a
        /// projection of the difference between the test signal and the ideal reference signal to the selected base spreading factor (refer to softkeys PEAK CODE DOMAIN ERR and SELECT BASE SF). The base spreading factor, to which the projection is made, is specified in addition to the measured value.
        /// 
        /// (6) IQ Imbalance: IQ imbalance of the signal in %
        /// 
        /// (7) IQ Offset: DC offset of the signal in %
        /// 
        /// (8) Carrier Freq Error: Specifies the frequency error referred to the set center frequency of the analyzer. The absolute frequency error is the sum of the frequency error of the analyzer and that of the device under test. Frequency differences between the transmitter and receiver of more than 1.5 kHz impair synchronization of the CDP measurement. If at all possible, the transmitter and the receiver should be synchronized (refer to the Getting Started chapter). The frequency error is available in the unit hertz on the one hand and in ppm referred to the carrier frequency on the other. 
        /// 
        /// (9) Chip Rate Error: Specifies the chip rate error (1.2288 Mcps) in ppm. A large chip rate error results in symbol errors and, consequently, possibly in the CDP measurement not being able to perform synchronization. This measurement result is also valid when the analyzer could not synchronize to the cdma2000 signal.
        /// 
        /// (10) Trigger to Frame: This measurement result reflects the time offset from the beginning of the recorded signal section until the start of the first PCG. In the case of triggered data recording, this corresponds to the timing offset "frame trigger (+ trigger offset) - start of first PCG". If it was not possible to synchronize the analyzer to the cdma2000 signal, the value of Trg to Frame is meaningless. Is the trigger FREE RUN selected, dashes (-.--) are displayed.
        /// 
        /// (11) No of Active Chan: Specifies the number of active channels found in the signal. In this case the detected data channels as well as the special channels are regarded as active. The result applies with transmit diversity to the selected antenna (refer to softkeys ANT DIV and ANT NO.).
        /// 
        /// (12) Symbol Rate: Symbol rate with which the channel is transmitted.
        /// 
        /// (13) Channel.SF: Number of the channel and its associated spreading factor 
        /// 
        /// (14) Timing Offset: Timing offset between the selected channel and the pilot channel. This measurement can be enabled by means of the TIME/PHASE softkey.
        /// 
        /// (15) Phase Offset: Phase offset between the selected channel and the pilot channel. This measurement can be enabled by means of the TIME/PHASE softkey.
        /// 
        /// (16) Chan Pow rel. / abs.: Channel power relative (referred to the pilot or total power depending on the POWER REF TOT/PICH) softkey and the absolute channel power.
        /// 
        /// (17) Symbol EVM Pk / rms: Peak or mean value of the results of the error vector magnitude measurement (refer to the SYMBOL EVM
        /// softkey). The measurement provides information on the EVM of the selected channel for the selected PCG at symbol level.
        /// 
        /// (18) Modulation Type: 1 = BPSK, 2 = QPSK, 3 = 8-PSK, 4= 16-QAM
        /// </param>
        /// <param name="Result">
        /// This control returns the measured and calculated values of the code domain power analysis. 
        /// 
        /// Units:
        /// Total power in dBm 
        /// Frequency error in Hz
        /// Pilot power in dBm 
        /// Frequency error in ppm
        /// Chip rate error in ppm
        /// Composite EVM in % 
        /// Peak code domain error in dB 
        /// IQ offset in %
        /// IQ imbalance in %
        /// Symbol rate in ksps 
        /// Timing offset in s
        /// Phase offset in rad
        /// Channel power, relative, in dB 
        /// Channel power, abs., in dBm
        /// Error vector magnitude rms in % 
        /// Error Vector Mag. Peak in %
        /// 
        /// All other parameters have no units.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryBC2KCodeDomainAnalyzerResultSummary(int Window, int Result_Type, out double Result)
        {
            int pInvokeResult = PInvoke.QueryBC2KCodeDomainAnalyzerResultSummary(this._handle, Window, Result_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Spectrum EM Mask Measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_LIM_MODE
        /// RSSPECAN_ATTR_C2K_LIM_VAL
        /// RSSPECAN_ATTR_C2K_LIM_TRANS
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:ESPectrum:MODE AUTO | MANual | USER
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:ESPectrum:VALue &lt;numeric_value&gt; dBm
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:ESPectrum:TRANsition
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Limit_Line_Type">
        /// This control selects the Limit Line Type.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_LIM_MODE_AUTO  (0) - Auto
        /// RSSPECAN_VAL_C2K_LIM_MODE_MAN   (1) - Manual
        /// RSSPECAN_VAL_C2K_LIM_MODE_USER  (2) - User
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_LIM_MODE_AUTO
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_LIM_MODE
        /// 
        /// Note(s):
        /// 
        /// (1) The user mode enables the creation of user-defined limit lines. The softkey opens the menus of the limit-line editor, which may be familiar from the basic unit. The limit lines created by the user are included in the table for the manual mode.
        /// </param>
        /// <param name="Limit_Line_Value">
        /// This control sets the Limit Line Value.
        /// 
        /// Units: dBm
        /// 
        /// Valid Values:
        /// None
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_LIM_VAL
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available if the Limit Line Type is set to Manual.
        /// 
        /// (2) The limit line is selected by specifying the expected power as a value. One of the four possible limit lines is selected depending on the value entered:
        /// Specified value in dBm
        /// Value &gt;= 33; 28 &lt;= value &lt; 33; Value &lt; 28
        /// Selected limit line
        /// "P &gt;= 33"; "28 &lt;= P &lt; 33"; "P &lt; 28"
        /// Value when queried
        /// 33; 28; 0
        /// </param>
        /// <param name="_30kHz_1MHz_Transition">
        /// This control sets the 30kHz/1MHz Transition.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 100.0 to 1.0e+9
        /// 
        /// Default Value: 4.0e+6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_LIM_TRANS
        /// 
        /// Note(s):
        /// 
        /// (1) This attribute specifies the offset frequency at which the resolution bandwidth is changed between 30k and 1MHz. For multicarrier spectrum emission mask measurement this value can be extended. It is recommended to define user specific limit lines in this case.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBC2KSEM(int Limit_Line_Type, double Limit_Line_Value, double _30kHz_1MHz_Transition)
        {
            int pInvokeResult = PInvoke.ConfigureBC2KSEM(this._handle, Limit_Line_Type, Limit_Line_Value, _30kHz_1MHz_Transition);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function stores the peak list of selected trace in ASCII format to a file.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_LIM_STORE_FILE
        /// 
        /// Remote-control command(s):
        /// MMEMory:STORe:FINal &lt;file_name&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="File_Name">
        /// Specifies full path of the file, where spectrum emission mask data will be stored.
        /// 
        /// Valid Values:
        /// any valid file name
        /// 
        /// Default Value: "c:\\test.txt"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_LIM_STORE_FILE
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int StoreBC2KSEMToFile(string File_Name)
        {
            int pInvokeResult = PInvoke.StoreBC2KSEMToFile(this._handle, File_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the spectrum emission mask peak list.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;[:DATA] FINal1
        /// FORMat[:DATA] REAL,32
        /// *CLS
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="No_Of_Values">
        /// This control sets the number of values returned in arrays.
        /// The array size has to be at least equal to this number.
        /// 
        /// Valid Values: &gt; 0
        /// 
        /// Default Value: 10
        /// 
        /// </param>
        /// <param name="Frequency">
        /// Returns an array of frequency list in Hz.
        /// 
        /// </param>
        /// <param name="Level">
        /// Returns an array of level list in dBm.
        /// 
        /// </param>
        /// <param name="Delta_Level">
        /// Returns an array of delta to limit line values in dB. A delta limit of +200dB indicates no limit check is active.
        /// 
        /// </param>
        /// <param name="Returned_Values">
        /// Returns number of value sets (frequency, level and delta to limit line) read (available to return) from the instrument. Number of values returned in arrays is limited with the parameter No of Values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadBC2KSEMPeakList(int Window, int No_Of_Values, double[] Frequency, double[] Level, double[] Delta_Level, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.ReadBC2KSEMPeakList(this._handle, Window, No_Of_Values, Frequency, Level, Delta_Level, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the CDMA2000 Mobile Station Test mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MC2K_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument[:SELect] MC2K
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int MC2KMode()
        {
            int pInvokeResult = PInvoke.MC2KMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the values of the selected channel table in the code domain power measurement mode. The whole table is defined in one go. The inactive channels (INACtive) do not have to be defined. Eight values are specified for a line of table.
        /// 
        /// Prior to this function, the name of the channel table has to be defined with function Configure CDMA2000 Channel Table Operations (ConfigureC2KChannelTableOperations) - attribute RSSPECAN_ATTR_C2K_CTABLE_NAME.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for option K83.
        /// 
        /// Attribute(s):
        /// None
        /// 
        /// Remote-control command(s):
        /// CONFigure:CDPower[:BTS]:CTABle:DATA 0..9, 1..6, 0..63, 0..1, 0, 0, 0 | 1, ...
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Channel_Type">
        /// This control selects the Channel Type. 
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0 = PICH
        /// 1 = EACH
        /// 2 = CCCH
        /// 3 = DCCH
        /// 4 = ACKCH
        /// 5 = CQICH
        /// 6 = FCH
        /// 7 = S1CH
        /// 8 = S2CH
        /// 9 = INACTIVE
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Code_Class">
        /// This control sets the Code Class.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 1 to 6
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Code_Number">
        /// This control sets the Code Number.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0 to 63
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Mapping">
        /// This control sets the Mapping.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0 = I branch
        /// 1 = Q branch
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Reserved_1">
        /// This control is reserver for future use.
        /// </param>
        /// <param name="Reserved_2">
        /// This control is reserver for future use.
        /// </param>
        /// <param name="Status">
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </param>
        /// <param name="CDP_Relative">
        /// This control sets the CDP Relative.
        /// 
        /// Units: dB
        /// 
        /// Valid Values:
        /// None
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Num_of_Elements">
        /// Sets number of elements for parameter arrays.
        /// 
        /// Valid Values: &gt;= 1
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMC2KChannelTableData(int[] Channel_Type, int[] Code_Class, int[] Code_Number, int[] Mapping, int[] Reserved_1, int[] Reserved_2, ushort[] Status, double[] CDP_Relative, int Num_of_Elements)
        {
            int pInvokeResult = PInvoke.ConfigureMC2KChannelTableData(this._handle, Channel_Type, Code_Class, Code_Number, Mapping, Reserved_1, Reserved_2, Status, CDP_Relative, Num_of_Elements);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Advanced Settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MC2K_CDP_SFACTOR
        /// RSSPECAN_ATTR_MC2K_CDP_PREFERENCE
        /// RSSPECAN_ATTR_MC2K_CDP_ORDER
        /// RSSPECAN_ATTR_MC2K_CDP_MAPPING
        /// RSSPECAN_ATTR_MC2K_LCODE_MODE
        /// RSSPECAN_ATTR_MC2K_CDP_LCODE_MASK
        /// RSSPECAN_ATTR_MC2K_CDP_LCODE_OFFSET
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:SFACtor 16 | 32 | 64
        /// [SENSe:]CDPower:PREFerence TOTal | PICH
        /// [SENSe:]CDPower:ORDer HADamard | BITReverse
        /// [SENSe:]CDPower:MAPPing I | Q
        /// [SENSe:]CDPower:LCODe:MODE STANdard | ESG101
        /// [SENSe:]CDPower:LCODe:MASK '#H0'  to  '#H4FFFFFFFFFF'
        /// [SENSe:]CDPower:LCODe:OFFSet '#H0'  to  '#H1FFFFFFFFFF8000'
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Base_Spreading_Factor">
        /// This control defines the base spreading factor.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MC2K_SFACTOR_16   (0) - 16
        /// RSSPECAN_VAL_MC2K_SFACTOR_32   (1) - 32
        /// RSSPECAN_VAL_MC2K_SFACTOR_64   (2) - 64
        /// 
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_MC2K_SFACTOR_32
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MC2K_CDP_SFACTOR
        /// 
        /// Notes:
        /// 
        /// (1) The base spreading factor should be set to 64 if channels of code class 6 (spreading factor 64) exist, otherwise alias power values will be measured in the code domain power analysis.
        /// </param>
        /// <param name="Power_Reference">
        /// This control sets the reference for the relative CDP measured values to the total power or the PICH power.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_PREF_TOTAL   (0) - Total
        /// RSSPECAN_VAL_C2K_PREF_PICH    (1) - PICH
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_PREF_TOTAL
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MC2K_CDP_PREFERENCE
        /// </param>
        /// <param name="Order">
        /// This control sets the order of the code domain evaluation.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_ORDER_HADAMARD   (0) - Hadamard
        /// RSSPECAN_VAL_C2K_ORDER_BITREVERSE (1) - Bitreverse
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_ORDER_HADAMARD
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MC2K_CDP_ORDER
        /// </param>
        /// <param name="Select_I___Q">
        /// This control selects whether the I or Q branch should be analyzed.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MC2K_CDP_MAPPING_I   (0) - I
        /// RSSPECAN_VAL_MC2K_CDP_MAPPING_Q   (1) - Q
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_MC2K_CDP_MAPPING_I
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MC2K_CDP_MAPPING
        /// 
        /// </param>
        /// <param name="Long_Code_Mode">
        /// This control defines the mode of the long code generation.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MC2K_LCODE_STANDARD (0) - Standard
        /// RSSPECAN_VAL_MC2K_LCODE_ESG101   (1) - ESG Option 101
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_MC2K_LCODE_STANDARD
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MC2K_LCODE_MODE
        /// 
        /// Notes:
        /// 
        /// (1) If the the ESG option 101 long code is used, only signals from that generator are possible to analyse then.
        /// 
        /// (2) If the the ESG option 101 long code is used, the long code mask and long code offset parameters are ignored.
        /// </param>
        /// <param name="Long_Code_Mask">
        /// This control defines the mask of the long code. It is changed into the hexadecimal format and then sent to the instrument.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// "0" to "4FFFFFFFFFF"
        /// 
        /// Default Value: "0"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MC2K_CDP_LCODE_MASK
        /// 
        /// Notes:
        /// 
        /// (1) If the the ESG option 101 long code mode is used, this parameter is ignored.
        /// </param>
        /// <param name="Long_Code_Offset">
        /// This control defines the offset of the long code including the PN offset in chips in hexadecimal format with a 52-bit resolution.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// "0" to "1FFFFFFFFFF8000"
        /// 
        /// Default Value: "0"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MC2K_CDP_LCODE_OFFSET
        /// 
        /// Notes:
        /// 
        /// (1) If the the ESG option 101 long code mode is used, this parameter is ignored.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMC2KAdvancedSettings(int Base_Spreading_Factor, int Power_Reference, int Order, int Select_I___Q, int Long_Code_Mode, string Long_Code_Mask, string Long_Code_Offset)
        {
            int pInvokeResult = PInvoke.ConfigureMC2KAdvancedSettings(this._handle, Base_Spreading_Factor, Power_Reference, Order, Select_I___Q, Long_Code_Mode, Long_Code_Mask, Long_Code_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the CDP Overview
        /// 
        /// Note(s):
        /// 
        /// (1) This function is valid only for Code Domain Power or the Code Domain Error Power analysis
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MC2K_CDP_OVERVIEW
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:OVERview ON | OFF
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Code_Domain_Overview">
        /// This control can be enabled by means of ON when either the Code Domain Power or the Code Domain Error Power analysis is active. (Refer to the attributes RSSPECAN_ATTR_C2K_RESULT_TYPE). In Overview mode, the I branch of the signal is normally displayed on screen A and the Q branch of the signal on screen B with the CDP/CDEP.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE   (0) - Off
        /// VI_TRUE    (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MC2K_CDP_OVERVIEW
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMC2KCDPOverview(bool Code_Domain_Overview)
        {
            int pInvokeResult = PInvoke.ConfigureMC2KCDPOverview(this._handle, System.Convert.ToUInt16(Code_Domain_Overview));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets marker to channel 0.16, 0.32 or 0.64, depending on the base spreading factor.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MC2K_MARKER_PICH
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:PICH
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMC2KPichMarkerSettings()
        {
            int pInvokeResult = PInvoke.ConfigureMC2KPichMarkerSettings(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Code Domain Analyzer.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_RESULT_TYPE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FEED 'XPOW:CDP' | 'XPOW:CDP:RAT' | 'XPOW:CDEP' | 'XTIM:CDP:MACCuracy' | 'XTIM:CDP:PVSLot' | 'XTIM:CDP:PVSYmbol' | 'XTIM:CDP:BSTReam' | 'XTIM:CDP:ERR:SUMM' | 'XTIM:CDP:ERR:CTABle' | 'XTIM:CDP:ERR:PCDomain' | 'XTIM:CDP:SYMB:CONSt' | 'XTIM:CDP:SYMB:EVM' | 'XTIM:CDP:COMP:CONSt'
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Result_Type">
        /// This control selects the results to be displayed on screen A.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_FEED_CDP       (0)  -  Code Domain Power Absolute   
        /// RSSPECAN_VAL_C2K_FEED_CDP_RAT   (1)  -  Code Domain Power Relative   
        /// RSSPECAN_VAL_C2K_FEED_CDEP      (2)  -  Code Domain Error            
        /// RSSPECAN_VAL_C2K_FEED_ERR_CTAB  (3)  -  Channel Table                
        /// RSSPECAN_VAL_C2K_FEED_MACC      (4)  -  Composite EVM                
        /// RSSPECAN_VAL_C2K_FEED_PVSL      (5)  -  Power vs PCG                 
        /// RSSPECAN_VAL_C2K_FEED_PVSY      (6)  -  Power vs Symbol              
        /// RSSPECAN_VAL_C2K_FEED_BSTR      (7)  -  Bitsream                     
        /// RSSPECAN_VAL_C2K_FEED_ERR_SUMM  (8)  -  Result Summary               
        /// RSSPECAN_VAL_C2K_FEED_ERR_PCD   (9)  -  Peak Code Domain Error       
        /// RSSPECAN_VAL_C2K_FEED_SYMB_CONS (10) -  Symbol Const                 
        /// RSSPECAN_VAL_C2K_FEED_SYMB_EVM  (11) -  Symbol EVM                   
        /// RSSPECAN_VAL_C2K_FEED_COMP_CONS (12) 	-  Composite Const              
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_FEED_CDP
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_RESULT_TYPE
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMC2KCodeDomainAnalyzerResults(int Result_Type)
        {
            int pInvokeResult = PInvoke.ConfigureMC2KCodeDomainAnalyzerResults(this._handle, Result_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the Code Domain Analyzer Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for option K83.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MC2K_CDP_IQLENGTH_FSQ_FSU
        /// RSSPECAN_ATTR_MC2K_CDP_IQLENGTH_FSP
        /// RSSPECAN_ATTR_MC2K_CDP_CODE
        /// RSSPECAN_ATTR_C2K_CDP_SLOT
        /// RSSPECAN_ATTR_MC2K_CDP_MAPPING
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:IQLength
        /// [SENSe:]CDPower:CODE 0 to (BASE SF-1)
        /// [SENSe:]CDPower:SLOT 0  to (IQ_CAPTURE_LENGTH-1)
        /// [SENSe:]CDPower:MAPPing I | Q
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Capture_Length">
        /// This control selects the Captured Length. The driver uses this value to set the RSSPECAN_ATTR_MC2K_CDP_IQLENGTH_FSP or RSSPECAN_ATTR_MC2K_CDP_IQLENGTH_FSQ_FSU  attribute.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 2 to 64 for FSU, FSQ
        /// 2 to 12 for FSP
        /// 
        /// 
        /// Default Value: 3
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MC2K_CDP_IQLENGTH_FSP RSSPECAN_ATTR_MC2K_CDP_IQLENGTH_FSQ_FSU
        /// </param>
        /// <param name="Select_Channel">
        /// This control selects the channel. The driver uses this value to set the RSSPECAN_ATTR_MC2K_CDP_CODE  attribute.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0 to (Base Spreading Factor - 1)
        /// 
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MC2K_CDP_CODE
        /// 
        /// Notes:
        /// 
        /// (1) The values are shown on the instrument like this: &lt;channel number&gt;,&lt;spreading factor&gt;
        /// </param>
        /// <param name="Select_PCG">
        /// This control selects the Power Control Group.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0 to (IQ_CAPTURE_LENGTH-1)
        /// 
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_CDP_SLOT
        /// </param>
        /// <param name="Select_Mapping">
        /// This control selects the Mapping.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MC2K_CDP_MAPPING_Q   (0) - Q
        /// RSSPECAN_VAL_MC2K_CDP_MAPPING_I   (1) - I
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_MC2K_CDP_MAPPING_I
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MC2K_CDP_MAPPING
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMC2KCodeDomainAnalyzerSettings(int Capture_Length, int Select_Channel, int Select_PCG, int Select_Mapping)
        {
            int pInvokeResult = PInvoke.ConfigureMC2KCodeDomainAnalyzerSettings(this._handle, Capture_Length, Select_Channel, Select_PCG, Select_Mapping);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the measured and calculated values of the code domain power analysis.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for option K83.
        /// 
        /// Attribute(s):
        /// None
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:FUNCtion:CDPower:RESult? PTOTal | FERRor | RHO | PPICh | FERPpm | CERRor| TFRame | IQOFfset | IQIMbalance | MACCuracy | PCDerror | SLOT | ACTive | SRATe | TOFFset | CHANnel | POFFset | SFACtor | CDPabsolute | CDPRelative |EVMRms | EVMPeak
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Result_Type">
        /// This control selects the result type.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_SLOT          (0) - PCG
        /// RSSPECAN_VAL_C2K_PTOTAL        (1) - Total PWR
        /// RSSPECAN_VAL_C2K_PPICH         (2) - Pilot PWR
        /// RSSPECAN_VAL_C2K_RHO           (3) - RHO
        /// RSSPECAN_VAL_C2K_MACCURACY     (4) - Composite EVM
        /// RSSPECAN_VAL_C2K_PCDERROR      (5) - Peak Code Domain Error
        /// RSSPECAN_VAL_C2K_IQIMBALANCE   (6) - IQ Imbalance
        /// RSSPECAN_VAL_C2K_IQOFFSET      (7) - IQ Offset
        /// RSSPECAN_VAL_C2K_FERROR        (8) - Carrier Freq Error Hz
        /// RSSPECAN_VAL_C2K_FERPPM        (9) - Carrier Freq Error ppm
        /// RSSPECAN_VAL_C2K_CERROR        (10) - Chip Rate Error
        /// RSSPECAN_VAL_C2K_TFRAME        (11) - Trigger to Frame
        /// RSSPECAN_VAL_C2K_ACTIVE        (12) - No of Active Chan
        /// RSSPECAN_VAL_C2K_SRATE         (13) - Symbol Rate
        /// RSSPECAN_VAL_C2K_CHANNEL       (14) - Channel
        /// RSSPECAN_VAL_C2K_SFACTOR       (15) - Spreading Factor
        /// RSSPECAN_VAL_C2K_TOFFSET       (16) - Timing Offset
        /// RSSPECAN_VAL_C2K_POFFSET       (17) - Phase Offset
        /// RSSPECAN_VAL_C2K_CDPRELATIVE   (18) - Channel Power Rel
        /// RSSPECAN_VAL_C2K_CDPABSOLUTE   (19) - Channel Power Abs
        /// RSSPECAN_VAL_C2K_EVMRMS        (20) - Symbol EVM rms
        /// RSSPECAN_VAL_C2K_EVMPEAK       (21) - Symbol EVM Pk
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_SLOT
        /// 
        /// Notes:
        /// 
        /// (1) Total Power: Specifies the total power of the signal.
        /// 
        /// (2) Pilot power: Specifies the pilot power. If antenna 2 (refer to ANT NO softkey) is selected, the power of the TDPICH is displayed, in all other cases that of the PICH.
        /// 
        /// (3) RHO: Specifies RHO, the quality parameter. According to the
        /// cdma2000 standards, RHO is the normalized, correlated power between the measured and the ideally generated reference signal. When RHO is measured, the cdma2000 standards require that only the pilot channel be supplied.
        /// 
        /// (4) Composite EVM: The composite EVM value is the difference between the test signal and the ideal reference signal (refer to the COMPOSITE EVM softkey).
        /// 
        /// (5) Pk CDE: The PEAK CODE DOMAIN ERR measurement specifies a
        /// projection of the difference between the test signal and the ideal reference signal to the selected base spreading factor (refer to softkeys PEAK CODE DOMAIN ERR and SELECT BASE SF). The base spreading factor, to which the projection is made, is specified in addition to the measured value.
        /// 
        /// (6) IQ Imbalance: IQ imbalance of the signal in %
        /// 
        /// (7) IQ Offset: DC offset of the signal in %
        /// 
        /// (8) Carrier Freq Error: Specifies the frequency error referred to the set center frequency of the analyzer. The absolute frequency error is the sum of the frequency error of the analyzer and that of the device under test. Frequency differences between the transmitter and receiver of more than 1.5 kHz impair synchronization of the CDP measurement. If at all possible, the transmitter and the receiver should be synchronized (refer to the Getting Started chapter). The frequency error is available in the unit hertz on the one hand and in ppm referred to the carrier frequency on the other. 
        /// 
        /// (9) Chip Rate Error: Specifies the chip rate error (1.2288 Mcps) in ppm. A large chip rate error results in symbol errors and, consequently, possibly in the CDP measurement not being able to perform synchronization. This measurement result is also valid when the analyzer could not synchronize to the cdma2000 signal.
        /// 
        /// (10) Trigger to Frame: This measurement result reflects the time offset from the beginning of the recorded signal section until the start of the first PCG. In the case of triggered data recording, this corresponds to the timing offset "frame trigger (+ trigger offset) - start of first PCG". If it was not possible to synchronize the analyzer to the cdma2000 signal, the value of Trg to Frame is meaningless. Is the trigger FREE RUN selected, dashes (-.--) are displayed.
        /// 
        /// (11) No of Active Chan: Specifies the number of active channels found in the signal. In this case the detected data channels as well as the special channels are regarded as active. The result applies with transmit diversity to the selected antenna (refer to softkeys ANT DIV and ANT NO.).
        /// 
        /// (12) Symbol Rate: Symbol rate with which the channel is transmitted.
        /// 
        /// (13) Channel.SF: Number of the channel and its associated spreading factor 
        /// 
        /// (14) Timing Offset: Timing offset between the selected channel and the pilot channel. This measurement can be enabled by means of the TIME/PHASE softkey.
        /// 
        /// (15) Phase Offset: Phase offset between the selected channel and the pilot channel. This measurement can be enabled by means of the TIME/PHASE softkey.
        /// 
        /// (16) Chan Pow rel. / abs.: Channel power relative (referred to the pilot or total power depending on the POWER REF TOT/PICH) softkey and the absolute channel power.
        /// 
        /// (17) Symbol EVM Pk / rms: Peak or mean value of the results of the error vector magnitude measurement (refer to the SYMBOL EVM
        /// softkey). The measurement provides information on the EVM of the selected channel for the selected PCG at symbol level.
        /// </param>
        /// <param name="Result">
        /// This control returns the measured and calculated values of the code domain power analysis. 
        /// 
        /// Units:
        /// Total power in dBm 
        /// Frequency error in Hz
        /// Pilot power in dBm 
        /// Frequency error in ppm
        /// Chip rate error in ppm
        /// Composite EVM in % 
        /// Peak code domain error in dB 
        /// IQ offset in %
        /// IQ imbalance in %
        /// Symbol rate in ksps 
        /// Timing offset in s
        /// Phase offset in rad
        /// Channel power, relative, in dB 
        /// Channel power, abs., in dBm
        /// Error vector magnitude rms in % 
        /// Error Vector Mag. Peak in %
        /// 
        /// All other parameters have no units.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryMC2KCodeDomainAnalyzerResultSummary(int Window, int Result_Type, out double Result)
        {
            int pInvokeResult = PInvoke.QueryMC2KCodeDomainAnalyzerResultSummary(this._handle, Window, Result_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Spectrum EM Mask Measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MC2K_LIM_MODE
        /// RSSPECAN_ATTR_C2K_LIM_TRANS
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:ESPectrum:MODE AUTO | USER
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:ESPectrum:TRANsition
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Limit_Line_Type">
        /// This control selects the Limit Line Type.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MC2K_LIM_MODE_AUTO  (0) - Auto
        /// RSSPECAN_VAL_MC2K_LIM_MODE_USER  (1) - User
        /// 
        /// Default Value: RSSPECAN_VAL_MC2K_LIM_MODE_AUTO
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MC2K_LIM_MODE
        /// 
        /// Note(s):
        /// 
        /// (1) The user mode enables the creation of user-defined limit lines. The softkey opens the menus of the limit-line editor, which may be familiar from the basic unit.
        /// </param>
        /// <param name="_30kHz_1MHz_Transition">
        /// This control sets the 30kHz/1MHz Transition.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 100.0 to 1.0e+9
        /// 
        /// Default Value: 4.0e+6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_LIM_TRANS
        /// 
        /// Note(s):
        /// 
        /// (1) This attribute specifies the offset frequency at which the resolution bandwidth is changed between 30k and 1MHz. For multicarrier spectrum emission mask measurement this value can be extended. It is recommended to define user specific limit lines in this case.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMC2KSEM(int Limit_Line_Type, double _30kHz_1MHz_Transition)
        {
            int pInvokeResult = PInvoke.ConfigureMC2KSEM(this._handle, Limit_Line_Type, _30kHz_1MHz_Transition);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the 1xEV-DO Base Station Test mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BDO_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument[:SELect] BDO
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int BDOMode()
        {
            int pInvokeResult = PInvoke.BDOMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Power vs Time Settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NUMBER_OF_SWEEPS
        /// RSSPECAN_ATTR_BDO_RFSLOT
        /// RSSPECAN_ATTR_BDO_PVT_BURST_FIT
        /// RSSPECAN_ATTR_BDO_LIM_PVT_REFERENCE
        /// RSSPECAN_ATTR_BDO_LIM_PVT_VALUE
        /// RSSPECAN_ATTR_BDO_PVT_FAIL_RESTART
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:COUNt
        /// CONFigure:CDPower[:BTS]:RFSLot FULL | IDLE
        /// CONFigure:CDPower[:BTS]:PVTime:BURSt:CENTer ON | OFF
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:PVTime:REFerence AUTO | MANual | ONCE
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:PVTime:RVALue &lt;numeric_value&gt;
        /// CONFigure:CDPower[:BTS]:PVTime:FREStart ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Number_Of_Halfslots">
        /// This control defines the number of sweeps started with single sweep, which are used for calculating the average or maximum value.
        /// 
        /// Units: dBm
        /// 
        /// Valid Values:
        /// 0 to 32767
        /// 
        /// Default Value: 100
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NUMBER_OF_SWEEPS
        /// </param>
        /// <param name="Reference_Slot">
        /// This control defines the expected signal: FULL slot or IDLE slot.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_CDP_RFSLOT_FULL  (0) - Full
        /// RSSPECAN_VAL_C2K_CDP_RFSLOT_IDLE  (1) - Idle
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_CDP_RFSLOT_FULL
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BDO_RFSLOT
        /// </param>
        /// <param name="Burst_Fit">
        /// This control activates an automatic burst alignment to the center of the diagram.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BDO_PVT_BURST_FIT
        /// 
        /// Notes:
        /// 
        /// (1) This parameter is ignored when the Reference Slot is set to FULL.
        /// </param>
        /// <param name="Reference_Mean_Power">
        /// This control selects the Reference Mean Power Type.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_LIM_PVT_REF_AUTO  (0) - Auto
        /// RSSPECAN_VAL_C2K_LIM_PVT_REF_MAN   (1) - Manual
        /// RSSPECAN_VAL_C2K_LIM_PVT_REF_ONCE  (2) - Once
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_LIM_MODE_AUTO
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BDO_LIM_PVT_REFERENCE
        /// 
        /// </param>
        /// <param name="Reference_Value">
        /// This control sets the reference level for calculating the limit lines. 
        /// 
        /// Units: dBm
        /// 
        /// Valid Values:
        /// -200.0 to 200.0
        /// 
        /// Default Value: -20.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BDO_LIM_PVT_VALUE
        /// 
        /// Note(s);
        /// 
        /// (1) This control is ignored if the Reference Mean Power is set to AUTO.
        /// </param>
        /// <param name="Restart_On_Fail">
        /// This control evaluates the limit line over all results at the end of the single sweep.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE   (0) - Off
        /// VI_TRUE    (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BDO_PVT_FAIL_RESTART
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBDOPowerVsTime(int Number_Of_Halfslots, int Reference_Slot, bool Burst_Fit, int Reference_Mean_Power, double Reference_Value, bool Restart_On_Fail)
        {
            int pInvokeResult = PInvoke.ConfigureBDOPowerVsTime(this._handle, Number_Of_Halfslots, Reference_Slot, System.Convert.ToUInt16(Burst_Fit), Reference_Mean_Power, Reference_Value, System.Convert.ToUInt16(Restart_On_Fail));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function enables/disables Power vs Time list evaluation.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BDO_LIM_PVT_LIST
        /// 
        /// Remote-control command(s):
        /// CONFigure:CDP:BST:PVTime:LIST[:STATe] OFF | ON
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="State">
        /// This control enables/disables the list evaluation.
        /// 
        /// 
        /// Valid Values:
        /// VI_FALSE   (0) - Off
        /// VI_TRUE    (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BDO_LIM_PVT_LIST
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBDOPowerVsTimeListEvaluation(bool State)
        {
            int pInvokeResult = PInvoke.ConfigureBDOPowerVsTimeListEvaluation(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function restores the standard limit lines for the power versus time measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BDO_LIM_PVT_RESTORE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:PVTime:RESTore
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int RestoreBDOPowerVsTimeStandardLines()
        {
            int pInvokeResult = PInvoke.RestoreBDOPowerVsTimeStandardLines(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the values of the selected channel table in the code domain power measurement mode. The whole table is defined in one go. The inactive channels (INACtive) do not have to be defined. Eight values are specified for a line of table.
        /// 
        /// Prior to this function, the name of the channel table has to be defined with function Configure CDMA2000 Channel Table Operations (ConfigureC2KChannelTableOperations) - attribute RSSPECAN_ATTR_C2K_CTABLE_NAME.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for option K84.
        /// 
        /// Attribute(s):
        /// None
        /// 
        /// Remote-control command(s):
        /// CONFigure:CDPower[:BTS]:CTABle:DATA 0..7, 4..6, 0..63, 0..4, 0, 0, 0 | 1, ...
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Channel_Type">
        /// This control selects the Channel Type. 
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0 = PILOT
        /// 1 = MAC
        /// 2 = PREAMBLE with 64 chip length
        /// 3 = PREAMBLE with 128 chip length
        /// 4 = PREAMBLE with 256 chip length
        /// 5 = PREAMBLE with 512 chip length
        /// 6 = PREAMBLE with 1024 chip length
        /// 7 = DATA
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Code_Class">
        /// This control sets the Code Class.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// Depending on channel type, fixed: PILOT: 5, MAC: 6,
        /// PREAMBLE: 5 and DATA: 4 (spreading factor = 2^code class)
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Code_Number">
        /// This control sets the Code Number.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0 to spreading factor-1
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Modulation">
        /// This control sets the Modulation.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0 = BPSK-I
        /// 1 = BPSK-Q
        /// 2 = QPSK
        /// 3 = 8-PSK
        /// 4 = 16-QAM
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Reserved_1">
        /// This control is reserver for future use.
        /// </param>
        /// <param name="Reserved_2">
        /// This control is reserver for future use.
        /// </param>
        /// <param name="Status">
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </param>
        /// <param name="CDP_Relative">
        /// This control sets the CDP Relative.
        /// 
        /// Units: dB
        /// 
        /// Valid Values:
        /// None
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Num_of_Elements">
        /// Sets number of elements for parameter arrays.
        /// 
        /// Valid Values: &gt;= 1
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBDOChannelTableData(int[] Channel_Type, int[] Code_Class, int[] Code_Number, int[] Modulation, int[] Reserved_1, int[] Reserved_2, ushort[] Status, double[] CDP_Relative, int Num_of_Elements)
        {
            int pInvokeResult = PInvoke.ConfigureBDOChannelTableData(this._handle, Channel_Type, Code_Class, Code_Number, Modulation, Reserved_1, Reserved_2, Status, CDP_Relative, Num_of_Elements);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Advanced Settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_CDP_PNOFFSET
        /// RSSPECAN_ATTR_BDO_CDP_AVERAGE
        /// RSSPECAN_ATTR_BDO_REVISION
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:PNOFfset 0 to 511
        /// [SENSe:]CDPower:AVERage ON | OFF
        /// CONFigure:CDPower[:BTS]:REVision
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="PN_Offset">
        /// This control sets the PN offset of the base station in multiples of 64 chips.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0 to 511
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_CDP_PNOFFSET
        /// </param>
        /// <param name="CDP_Average">
        /// This control enables CDP averaging.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE   (0) - Off
        /// VI_TRUE    (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BDO_CDP_AVERAGE
        /// 
        /// </param>
        /// <param name="Revision">
        /// This control defines whether a revision 0 (subtype 0/1) signal or a revision A (subtype 2) signal is analyzed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_BDO_REVISION_0 (0) - 0
        /// RSSPECAN_VAL_BDO_REVISION_A (1) - A
        /// 
        /// Default Value: RSSPECAN_VAL_BDO_REVISION_0 (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BDO_REVISION
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBDOAdvancedSettings(int PN_Offset, bool CDP_Average, int Revision)
        {
            int pInvokeResult = PInvoke.ConfigureBDOAdvancedSettings(this._handle, PN_Offset, System.Convert.ToUInt16(CDP_Average), Revision);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Multi Carrier settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_MCARRIER_STATE
        /// RSSPECAN_ATTR_C2K_MCARRIER_ENH_ALG
        /// RSSPECAN_ATTR_C2K_MCARRIER_FILTER_STATE
        /// RSSPECAN_ATTR_C2K_MCARRIER_FILTER_TYPE
        /// RSSPECAN_ATTR_C2K_MCARRIER_ROLL_OFF
        /// RSSPECAN_ATTR_C2K_MCARRIER_CUT_OFF_FREQ
        /// 
        /// Remote-control command(s):
        /// CONFigure:CDPower:MCARier ON | OFF
        /// CONFigure:CDPower:MCARier:MALGo ON | OFF
        /// CONFigure:CDPower:MCARier:FILTer[:STATe] ON | OFF
        /// CONFigure:CDPower:MCARier:FILTer:TYPE LPASs | RRC
        /// CONFigure:CDPower:MCARier:FILTer:ROFF &lt;numeric_value&gt;
        /// CONFigure:CDPower:MCARier:FILTer:COFRequency &lt;numeric_value&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Multi_Carrier">
        /// This control selects Multi Carrier mode.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE  (0) - Off
        /// VI_TRUE   (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_MCARRIER_STATE
        /// 
        /// Notes:
        /// 
        /// (1) If this parameter is turned off, the Enhanced Algorithm, MC Filter and Filter Type parameters are ignored.
        /// </param>
        /// <param name="Enhanced_Algorithm">
        /// This control selects Enhanced Algorithm.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE  (0) - Off
        /// VI_TRUE   (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_MCARRIER_ENH_ALG
        /// 
        /// Notes:
        /// 
        /// (1) If this parameter is turned off and the MC Filter was previously turned off, then the MC Filter and Filter Type parameters are ignored.
        /// </param>
        /// <param name="MC_Filter">
        /// This control selects MC Filter state.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE  (0) - Off
        /// VI_TRUE   (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_MCARRIER_FILTER_STATE
        /// 
        /// Notes:
        /// 
        /// (1) If this parameter is turned off, the Filter Type parameter is ignored.
        /// </param>
        /// <param name="Filter_Type">
        /// This control selects the Filter Type.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_MCARRIER_FTYPE_LPASS  (0) - Low pass
        /// RSSPECAN_VAL_C2K_MCARRIER_FTYPE_RRC    (1) - RRC
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_MCARRIER_FILTER_TYPE
        /// 
        /// Default Value: 
        /// 
        /// Notes: RSSPECAN_VAL_C2K_MCARRIER_FTYPE_LPASS
        /// 
        /// (1) If this parameter is set to Lowpa, the Roll Off and Cut Off Frequency parameters are ignored.
        /// </param>
        /// <param name="Roll_Off">
        /// This control sets Roll Off Factor.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0.01 to 0.99
        /// 
        /// Default Value: 0.02
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_MCARRIER_ROLL_OFF
        /// 
        /// </param>
        /// <param name="Cut_Off_Frequency">
        /// This control sets Cut Off Frequency.
        /// 
        /// Units: Hz
        /// 
        /// Valid Values:
        /// 100.0e+3 to 2.4e+6
        /// 
        /// Default Value: 1.25e+6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_MCARRIER_CUT_OFF_FREQ
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBDOMultiCarrier(bool Multi_Carrier, bool Enhanced_Algorithm, bool MC_Filter, int Filter_Type, double Roll_Off, double Cut_Off_Frequency)
        {
            int pInvokeResult = PInvoke.ConfigureBDOMultiCarrier(this._handle, System.Convert.ToUInt16(Multi_Carrier), System.Convert.ToUInt16(Enhanced_Algorithm), System.Convert.ToUInt16(MC_Filter), Filter_Type, Roll_Off, Cut_Off_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the measured data that will be displayed.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_RESULT_TYPE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FEED 'XPOW:CDP' | 'XPOW:CDP:RAT' | 'XPOW:CDEP' | 'XTIM:CDP:MACCuracy' | 'XTIM:CDP:PVCHip' | 'XTIM:CDP:PVSYmbol' | 'XTIM:CDP:BSTReam' | 'XTIM:CDP:ERR:SUMM' | 'XTIM:CDP:ERR:CTABle' | 'XTIM:CDP:ERR:PCDomain' | 'XTIM:CDP:SYMB:CONSt' | 'XTIM:CDP:SYMB:EVM' | 'XTIM:CDP:COMP:CONSt' | 'XTIM:CDP:PVCH'
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Result_Type">
        /// This control selects the results to be displayed on screen A.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_FEED_CDP       (0)  -  Code Domain Power Absolute   
        /// RSSPECAN_VAL_C2K_FEED_CDP_RAT   (1)  -  Code Domain Power Relative   
        /// RSSPECAN_VAL_C2K_FEED_CDEP      (2)  -  Code Domain Error            
        /// RSSPECAN_VAL_C2K_FEED_ERR_CTAB  (3)  -  Channel Table                
        /// RSSPECAN_VAL_C2K_FEED_MACC      (4)  -  Composite EVM                
        /// RSSPECAN_VAL_C2K_FEED_PVSY      (6)  -  Power vs Symbol              
        /// RSSPECAN_VAL_C2K_FEED_BSTR      (7)  -  Bitsream                     
        /// RSSPECAN_VAL_C2K_FEED_ERR_SUMM  (8)  -  Result Summary               
        /// RSSPECAN_VAL_C2K_FEED_ERR_PCD   (9)  -  Peak Code Domain Error       
        /// RSSPECAN_VAL_C2K_FEED_SYMB_CONS (10) -  Symbol Const                 
        /// RSSPECAN_VAL_C2K_FEED_SYMB_EVM  (11) -  Symbol EVM                   
        /// RSSPECAN_VAL_C2K_FEED_COMP_CONS (12) 	-  Composite Const              
        /// RSSPECAN_VAL_C2K_FEED_PVCH      (13) 	-  Power Vs. Chip
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_FEED_CDP
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_RESULT_TYPE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBDOCodeDomainAnalyzerResults(int Result_Type)
        {
            int pInvokeResult = PInvoke.ConfigureBDOCodeDomainAnalyzerResults(this._handle, Result_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the Code Domain Analyzer Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for option K84.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BDO_CDP_IQLENGTH_FSQ_FSU
        /// RSSPECAN_ATTR_C2K_CDP_IQLENGTH_FSP
        /// RSSPECAN_ATTR_BDO_CDP_CODE
        /// RSSPECAN_ATTR_C2K_CDP_SLOT
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:IQLength
        /// [SENSe:]CDPower:CODE 0 to (BASE SF-1)
        /// [SENSe:]CDPower:SLOT 0  to (IQ_CAPTURE_LENGTH-1)
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Capture_Length">
        /// This control sets the Captured Length.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 2 to 36 for FSU, FSQ
        /// 2 to 12 for FSP
        /// 
        /// 
        /// Default Value: 3
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_CDP_IQLENGTH_FSP RSSPECAN_ATTR_BDO_CDP_IQLENGTH_FSQ_FSU
        /// 
        /// </param>
        /// <param name="Select_Code">
        /// This control selects the channel. The driver uses this value to set the RSSPECAN_ATTR_BDO_CDP_CODE  attribute.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0 to (Base Spreading Factor - 1)
        /// 
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BDO_CDP_CODE
        /// 
        /// Notes:
        /// 
        /// (1) The values are shown on the instrument like this: &lt;channel number&gt;,&lt;spreading factor&gt;
        /// 
        /// (2) The maximum value depends on the spreading factor of the channel type. PILOT/PREAMBLE: 0..31, DATA: 0..15, MAC: 0..63 (attribute  RSSPECAN_ATTR_BDO_CDP_CTYPE).
        /// </param>
        /// <param name="Select_Slot">
        /// This control selects the Power Control Group.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0 to (IQ_CAPTURE_LENGTH-1)
        /// 
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_ADJUST_REF_LEVEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBDOCodeDomainAnalyzerSettings(int Capture_Length, int Select_Code, int Select_Slot)
        {
            int pInvokeResult = PInvoke.ConfigureBDOCodeDomainAnalyzerSettings(this._handle, Capture_Length, Select_Code, Select_Slot);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Channel Type for Code Domain Power or Code Domain Error Power analysis.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BDO_CDP_CTYPE
        /// RSSPECAN_ATTR_BDO_CDP_OVERVIEW
        /// RSSPECAN_ATTR_BDO_CDP_MMODE
        /// RSSPECAN_ATTR_BDO_CDP_MAPPING
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:CTYPe PILot | MAC | PREamble | DATA
        /// [SENSe:]CDPower:OVERview ON | OFF
        /// [SENSe:]CDPower:MMODe AUTO | IOQ | COMPlex
        /// [SENSe:]CDPower:MAPPing I | Q
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Channel_Type">
        /// This control selects the channel type.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_CDP_CTYPE_PILOT    (0) - Pilot
        /// RSSPECAN_VAL_C2K_CDP_CTYPE_MAC      (1) - MAC
        /// RSSPECAN_VAL_C2K_CDP_CTYPE_PREAMBLE (2) - Preamble
        /// RSSPECAN_VAL_C2K_CDP_CTYPE_DATA     (3) - Data
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_CDP_CTYPE_PILOT
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BDO_CDP_CTYPE
        /// 
        /// </param>
        /// <param name="Code_Domain_Overview">
        /// This control can be enabled by means of ON when either the Code Domain Power or the Code Domain Error Power analysis is active. (Refer to the attributes RSSPECAN_ATTR_C2K_RESULT_TYPE). In Overview mode, the I branch of the signal is normally displayed on screen A and the Q branch of the signal on screen B with the CDP/CDEP.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE   (0) - Off
        /// VI_TRUE    (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BDO_CDP_OVERVIEW
        /// 
        /// </param>
        /// <param name="Mapping">
        /// This control defines the mapping mode.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_CDP_MMODE_AUTO     (0) - Auto
        /// RSSPECAN_VAL_C2K_CDP_MMODE_IOQ      (1) - I or Q
        /// RSSPECAN_VAL_C2K_CDP_MMODE_COMPLEX  (2) - Complex
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_CDP_MMODE_AUTO
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BDO_CDP_MMODE
        /// 
        /// Notes:
        /// 
        /// (1) This parameter is ignored when Code Domain Overview is set to ON.
        /// </param>
        /// <param name="Select">
        /// This control selects, whether the I or Q branch should be analyzed.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_CDP_MAPPING_I   (0) - I
        /// RSSPECAN_VAL_C2K_CDP_MAPPING_Q   (1) - Q
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_CDP_MAPPING_I
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BDO_CDP_MAPPING
        /// 
        /// Notes:
        /// 
        /// (1) The parameter is ignored when Complex Mapping is selected.
        /// 
        /// (2) The parameter is ignored when both Auto Mapping and Data Channel Type is selected at once.
        /// 
        /// (3) This parameter is ignored when Code Domain Overview is set to ON.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBDOCDPChannelType(int Channel_Type, bool Code_Domain_Overview, int Mapping, int Select)
        {
            int pInvokeResult = PInvoke.ConfigureBDOCDPChannelType(this._handle, Channel_Type, System.Convert.ToUInt16(Code_Domain_Overview), Mapping, Select);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Channel Type without CDP Overview.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BDO_CDP_CTYPE
        /// RSSPECAN_ATTR_BDO_CDP_MMODE
        /// RSSPECAN_ATTR_BDO_CDP_MAPPING
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:CTYPe PILot | MAC | PREamble | DATA
        /// [SENSe:]CDPower:MMODe AUTO | IOQ | COMPlex
        /// [SENSe:]CDPower:MAPPing I | Q
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Channel_Type">
        /// This control selects the channel type. 
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_CDP_CTYPE_PILOT    (0) - Pilot
        /// RSSPECAN_VAL_C2K_CDP_CTYPE_MAC      (1) - MAC
        /// RSSPECAN_VAL_C2K_CDP_CTYPE_PREAMBLE (2) - Preamble
        /// RSSPECAN_VAL_C2K_CDP_CTYPE_DATA     (3) - Data
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_CDP_CTYPE_PILOT
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BDO_CDP_CTYPE
        /// </param>
        /// <param name="Mapping">
        /// This control defines the mapping mode.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_CDP_MMODE_AUTO     (0) - Auto
        /// RSSPECAN_VAL_C2K_CDP_MMODE_IOQ      (1) - I or Q
        /// RSSPECAN_VAL_C2K_CDP_MMODE_COMPLEX  (2) - Complex
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_CDP_MMODE_AUTO
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BDO_CDP_MMODE
        /// 
        /// Notes:
        /// 
        /// (1) This parameter is ignored when Code Domain Overview is set to ON.
        /// </param>
        /// <param name="Select">
        /// This control selects, whether the I or Q branch should be analyzed.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_CDP_MAPPING_I   (0) - I
        /// RSSPECAN_VAL_C2K_CDP_MAPPING_Q   (1) - Q
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_CDP_MAPPING_I
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BDO_CDP_MAPPING
        /// 
        /// Notes:
        /// 
        /// (1) The parameter is ignored when Complex Mapping is selected.
        /// 
        /// (2) The parameter is ignored when both Auto Mapping and Data Channel Type is selected at once.
        /// 
        /// (3) This parameter is ignored when Code Domain Overview is set to ON.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBDOChannelType(int Channel_Type, int Mapping, int Select)
        {
            int pInvokeResult = PInvoke.ConfigureBDOChannelType(this._handle, Channel_Type, Mapping, Select);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the measured and calculated values of the code domain power analysis.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for option K84.
        /// 
        /// Attribute(s):
        /// None
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:FUNCtion:CDPower[:BTS]:RESult? RHO | MACCuracy | FERRor | FERPpm | CERRor | TFRame | RHOPilot | RHO1 | RHO2 | PPILot | PMAC | PDATa | PPReamble | MACTive | DACTive | PLENgth | DMTYpe | PDMax | PDMin | IPMMax | RHOData | RHOMac
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Result_Type">
        /// This control selects the result type.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_RHO           (3) - RHO
        /// RSSPECAN_VAL_C2K_MACCURACY     (4) - Composite EVM
        /// RSSPECAN_VAL_C2K_FERROR        (8) - Carrier Freq Error Hz
        /// RSSPECAN_VAL_C2K_FERPPM        (9) - Carrier Freq Error ppm
        /// RSSPECAN_VAL_C2K_CERROR        (10) - Chip Rate Error
        /// RSSPECAN_VAL_C2K_TFRAME        (11) - Trigger to Frame
        /// RSSPECAN_VAL_C2K_RHOPILOT      (22) - RHO Pilot
        /// RSSPECAN_VAL_C2K_RHO1          (23) - RHO Overall-1
        /// RSSPECAN_VAL_C2K_RHO2          (24) - RHO Overall-2
        /// RSSPECAN_VAL_C2K_PPILOT        (25) - Power PILOT
        /// RSSPECAN_VAL_C2K_PMAC          (26) - Power MAC
        /// RSSPECAN_VAL_C2K_PDATA         (27) - Power DATA
        /// RSSPECAN_VAL_C2K_PPREAMBLE     (28) - Power PREAMBLE
        /// RSSPECAN_VAL_C2K_DMTYPE        (29) - Data Modulation Type
        /// RSSPECAN_VAL_C2K_MACTIVE       (30) - Act. MAC Channels
        /// RSSPECAN_VAL_C2K_DACTIVE       (31) - Act. DATA Channels
        /// RSSPECAN_VAL_C2K_PLENGTH       (32) - Preamble Length
        /// RSSPECAN_VAL_C2K_PDMAX         (33) - Max. Pwr DATA
        /// RSSPECAN_VAL_C2K_PDMIN         (34) - Min. Pwr DATA
        /// RSSPECAN_VAL_C2K_IPMMAX        (35) - Max inact. Pwr MAC
        /// RSSPECAN_VAL_C2K_RHODATA       (40) - RHO DATA
        /// RSSPECAN_VAL_C2K_RHOMAC        (41) - RHO MAC
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_RHO
        /// 
        /// Notes:
        /// 
        /// (1) Carrier Freq Error: Specifies the frequency error referred to the set center frequency of the analyzer. The absolute frequency error is the sum of the frequency error of the analyzer and that of the device under test. Differences of more than 8.0 kHz between transmitter and receiver frequency impair the synchronization of the CDP measurement. If at all possible, the transmitter and the receiver should be synchronized (refer to the Getting Started chapter). The frequency error is available in Hz as the unit, on the one hand, and in ppm referred to the carrier frequency, on the other.
        /// 
        /// (2) Chip Rate Error: Specifies the chip rate error (1.2288 Mcps) in ppm. A large chip rate error results in symbol errors and, consequently, possibly in the CDP measurement not being able to perform synchronization. This measurement result is also valid if the analyzer was not able to synchronize onto the 1xEV-DO signal.
        /// 
        /// (3) Trigger to Frame: This measurement result reproduces the timing offset from the beginning of the acquired signal section until the start of the first slot. In the case of triggered data acquisition, this corresponds to the timing offset frame-trigger (+ triggeroffset) - start of the first slot. If the analyzer was not able to synchronize onto the 1xEV-DO signal, the value of Trg to Frame is not meaningful. If the FREE RUN trigger is selected, dashes (---) are displayed.
        /// 
        /// (4) RHO Pilot: Specifies the RHO quality parameter for the area of the pilot channel calculated over all slots, as called for in the Standard.
        /// 
        /// (5) RHO overall-1: Specifies the RHO quality parameter calculated for over all chips and over all slots, where the averaging limit is on the half-slot limit, as called for in the Standard.
        /// 
        /// (6) RHO overall-2: Specifies the RHO quality parameter calculated for over all chips and over all slots, where the averaging limit is on the quarter-slot limit, as called for in the Standard.
        /// 
        /// (7) Power PILOT: Absolute power measurements in dBm in the PILOT channel type.
        /// 
        /// (8) Power MAC: Absolute power measurements in dBm in the MAC channel type.
        /// 
        /// (9) Power DATA: Absolute power measurements in dBm in the DATA channel type.
        /// 
        /// (10) Power PREAMBLE: Absolute power measurements in dBm in the PREAMBLE channel type.
        /// 
        /// (11) Composite EVM: The composite EVM value is the difference between the test signal and ideal reference signal (refer to the COMPOSITE EVM softkey).
        /// 
        /// (12) Data Modulation Type: Modulation type of the DATA channel type.
        /// 
        /// (13) Act. MAC Channels: Number of active MAC channels.
        /// 
        /// (14) Act. DATA Channels: Number of active DATA channels.
        /// 
        /// (15) Preamble Length: Length of the preamble in chips, 0 if no preamble is present in this slot.
        /// 
        /// (16) RHO: Specifies the RHO quality parameter calculated over a slot.
        /// </param>
        /// <param name="Result">
        /// This control returns the measured and calculated values of the code domain power analysis. 
        /// 
        /// Units:
        /// Frequency error in Hz
        /// Frequency error in ppm
        /// Chip rate error in ppm
        /// Absolute power in the PILOT channel type in dBm
        /// PMAC Absolute power in the MAC channel type in dBm
        /// PDATa Absolute power in the DATA channel type in dBm
        /// PPReamble Absolute power in the PREAMBLE channel type in dBm
        /// MACCuracy Composite EVM in %
        /// PLENGth Length of preamble in chips
        /// Power Data Max in dB
        /// Power Data Min in dB
        /// Inactiv Power MAC Max in dB
        /// 
        /// All other parameters have no units.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryBDOCodeDomainAnalyzerGeneralResults(int Window, int Result_Type, out double Result)
        {
            int pInvokeResult = PInvoke.QueryBDOCodeDomainAnalyzerGeneralResults(this._handle, Window, Result_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the measured and calculated values of the code domain power analysis.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for option K84.
        /// 
        /// Attribute(s):
        /// None
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:FUNCtion:CDPower[:BTS]:RESult? PTOTal | PCDerror | IQOFfset | IQIMbalance | SRATe | CHANnel | SFACtor | TOFFset | POFFset | CDPabsolute | CDPRelative | EVMRms | EVMPeak | MTYPe | DMTYpe | RHOData | RHOMac
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Result_Type">
        /// This control selects the result type.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_PCDERROR      (5) - Peak Code Domain Error
        /// RSSPECAN_VAL_C2K_IQIMBALANCE   (6) - IQ Imbalance
        /// RSSPECAN_VAL_C2K_IQOFFSET      (7) - IQ Offset
        /// RSSPECAN_VAL_C2K_SRATE         (13) - Symbol Rate
        /// RSSPECAN_VAL_C2K_CHANNEL       (14) - Channel
        /// RSSPECAN_VAL_C2K_SFACTOR       (15) - Spreading Factor
        /// RSSPECAN_VAL_C2K_TOFFSET       (16) - Timing Offset
        /// RSSPECAN_VAL_C2K_POFFSET       (17) - Phase Offset
        /// RSSPECAN_VAL_C2K_CDPRELATIVE   (18) - Channel Power Rel
        /// RSSPECAN_VAL_C2K_CDPABSOLUTE   (19) - Channel Power Abs
        /// RSSPECAN_VAL_C2K_EVMRMS        (20) - Symbol EVM rms
        /// RSSPECAN_VAL_C2K_EVMPEAK       (21) - Symbol EVM Pk
        /// RSSPECAN_VAL_C2K_MTYPE         (36) - Modulation Type
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_SLOT
        /// 
        /// Notes:
        /// 
        /// (1) Power: The total power of the channel type. This result is identical to one from the bottom half of General Results and is listed again, because the Channel Power Rel relative to this value is displayed from this point on.
        /// 
        /// (2) Peak Code Domain Error: The PEAK CODE DOMAIN ERR measurement specifies a projection of the difference between test signal and ideal reference signal to the spreading factor that belongs to the channel type. This spreading factor is output in brackets.
        /// 
        /// (3) IQ Imbalance: IQ imbalance of the signal in %
        /// 
        /// (4) IQ Offset: DC offset of the signal in %
        /// 
        /// (5) Symbol Rate: Symbol rate with which the channel is transmitted
        /// 
        /// (6) Code.SF: Number of the code and its associated spreading factor.
        /// 
        /// (7) Timing Offset: Timing offset between the selected channel and the first active channel in the channel type. This measurement can be enabled by means of the TIME/PHASE softkey.
        /// 
        /// (8) Phase Offset: Phase offset between the selected channel and the first active channel in the channel type. This measurement can be enabled by means of the TIME/PHASE softkey.
        /// 
        /// (9) Channel Power rel. / abs.: Channel power relative (referred to the total power of the channel type) and absolute channel power.
        /// 
        /// (10) Symbol EVM Pk / RMS: Peak or mean value of the results of the error vector magnitude measurement (refer to the SYMBOL EVM softkey). The measurement provides information about the EVM of the selected channel for the selected slot at symbol level.
        /// 
        /// (11) Modulation type:
        /// 0 = BPSK-I
        /// 1 = BPSK-Q
        /// 2 = QPSK
        /// 3 = 8-PSK
        /// 4 = 16-QAM
        /// 5 = 2BPSK (if complex analysis selected for PILOT, PREAMBLE or MAC)
        /// 
        /// 
        /// 
        /// </param>
        /// <param name="Result">
        /// This control returns the measured and calculated values of the code domain power analysis. 
        /// 
        /// Units:
        /// Peak code domain error in dB
        /// IQ imbalance in %
        /// IQ offset in %
        /// Symbol rate in ksps
        /// Timing offset in s
        /// Phase offset in rad
        /// Channel power, relative, in dB
        /// Channel power, absolute, in dBm
        /// Error vector magnitude RMS in %
        /// Error vector mag. peak in %
        /// Modulation type:
        /// 0 = BPSK-I
        /// 1 = BPSK-Q
        /// 2 = QPSK
        /// 3 = 8-PSK
        /// 4 = 16-QAM
        /// 5 = 2BPSK
        /// 
        /// All other parameters have no units.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryBDOCodeDomainAnalyzerChannelResults(int Window, int Result_Type, out double Result)
        {
            int pInvokeResult = PInvoke.QueryBDOCodeDomainAnalyzerChannelResults(this._handle, Window, Result_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Spectrum EM Mask Measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_LIM_MODE
        /// RSSPECAN_ATTR_C2K_LIM_VAL
        /// RSSPECAN_ATTR_C2K_LIM_TRANS
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:ESPectrum:MODE AUTO | MANual | USER
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:ESPectrum:VALue &lt;numeric_value&gt; dBm
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:ESPectrum:TRANsition
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Limit_Line_Type">
        /// This control selects the Limit Line Type.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_LIM_MODE_AUTO  (0) - Auto
        /// RSSPECAN_VAL_C2K_LIM_MODE_MAN   (1) - Manual
        /// RSSPECAN_VAL_C2K_LIM_MODE_USER  (2) - User
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_LIM_MODE_AUTO
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_LIM_MODE
        /// 
        /// Note(s):
        /// 
        /// (1) The user mode enables the creation of user-defined limit lines. The softkey opens the menus of the limit-line editor, which may be familiar from the basic unit. The limit lines created by the user are included in the table for the manual mode.
        /// </param>
        /// <param name="Limit_Line_Value">
        /// This control sets the Limit Line Value.
        /// 
        /// Units: dBm
        /// 
        /// Valid Values:
        /// None
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_LIM_VAL
        /// 
        /// Note(s):
        /// 
        /// (1) This control switches to manual limit line selection. The limit line is selected by specifying the expected power as a value. One of the four possible limit lines is selected depending on the value entered:
        /// Specified value in dBm
        /// Value &gt;= 33; 28 &lt;= value &lt; 33; Value &lt; 28
        /// Selected limit line
        /// "P &gt;= 33"; "28 &lt;= P &lt; 33"; "P &lt; 28"
        /// </param>
        /// <param name="_30kHz_1MHz_Transition">
        /// This control sets the 30kHz/1MHz Transition.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 100.0 to 1.0e+9
        /// 
        /// Default Value: 4.0e+6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_LIM_TRANS
        /// 
        /// 
        /// Note(s):
        /// 
        /// (1) This attribute specifies the offset frequency at which the resolution bandwidth is changed between 30k and 1MHz. For multicarrier spectrum emission mask measurement this value can be extended. It is recommended to define user specific limit lines in this case.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureBDOSEM(int Limit_Line_Type, double Limit_Line_Value, double _30kHz_1MHz_Transition)
        {
            int pInvokeResult = PInvoke.ConfigureBDOSEM(this._handle, Limit_Line_Type, Limit_Line_Value, _30kHz_1MHz_Transition);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function stores the peak list of selected trace in ASCII format to a file.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_LIM_STORE_FILE
        /// 
        /// Remote-control command(s):
        /// MMEMory:STORe:FINal &lt;file_name&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="File_Name">
        /// Specifies full path of the file, where spectrum emission mask data will be stored.
        /// 
        /// Valid Values:
        /// any valid file name
        /// 
        /// Default Value: "c:\\test.txt"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_C2K_LIM_STORE_FILE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int StoreBDOSEMToFile(string File_Name)
        {
            int pInvokeResult = PInvoke.StoreBDOSEMToFile(this._handle, File_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the spectrum emission mask peak list.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;[:DATA] FINal1
        /// FORMat[:DATA] REAL,32
        /// *CLS
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="No_Of_Values">
        /// This control sets the number of values returned in arrays.
        /// The array size has to be at least equal to this number.
        /// 
        /// Valid Values: &gt; 0
        /// 
        /// Default Value: 10
        /// 
        /// </param>
        /// <param name="Frequency">
        /// Returns an array of frequency list in Hz.
        /// 
        /// </param>
        /// <param name="Level">
        /// Returns an array of level list in dBm.
        /// 
        /// </param>
        /// <param name="Delta_Level">
        /// Returns an array of delta to limit line values in dB. A delta limit of +200dB indicates no limit check is active.
        /// 
        /// </param>
        /// <param name="Returned_Values">
        /// Returns number of value sets (frequency, level and delta to limit line) read (available to return) from the instrument. Number of values returned in arrays is limited with the parameter No of Values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadBDOSEMPeakList(int Window, int No_Of_Values, double[] Frequency, double[] Level, double[] Delta_Level, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.ReadBDOSEMPeakList(this._handle, Window, No_Of_Values, Frequency, Level, Delta_Level, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the list evaluation results
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CONFigure:CDP:BTS:PVTime:LIST:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="No_Of_Values">
        /// This control sets the number of values returned in arrays.
        /// The array size has to be at least equal to this number.
        /// 
        /// Valid Values: &gt; 0
        /// 
        /// Default Value: 10
        /// 
        /// </param>
        /// <param name="Range_Number">
        /// Returns an array of range numbers
        /// 
        /// 
        /// </param>
        /// <param name="Start_Time">
        /// Returns an array of start time
        /// 
        /// </param>
        /// <param name="Stop_Time">
        /// Returns an array of stop time.
        /// </param>
        /// <param name="Avg_Power_Abs">
        /// Returns an array of average absolute power in dBm.
        /// </param>
        /// <param name="Avg_Power_Rel">
        /// Returns an array of average relative power in dB (relative to mean power)
        /// 
        /// 
        /// </param>
        /// <param name="Max_Power_Abs">
        /// Returns an array of average absolute power in dBm
        /// 
        /// </param>
        /// <param name="Max_Power_Rel">
        /// Returns an array of average relative power in dB (relative to mean power)
        /// </param>
        /// <param name="Min_Power_Abs">
        /// Returns an array of average absolute power in dBm
        /// 
        /// </param>
        /// <param name="Min_Power_Rel">
        /// Returns an array of average relative power in dB (relative to mean power)
        /// </param>
        /// <param name="Limit_Check">
        /// Returns an array of limit check state (0 = PASSED, 1 = FAILED
        /// </param>
        /// <param name="Reserved1">
        /// Reserved (0.0)
        /// 
        /// 
        /// </param>
        /// <param name="Reserved2">
        /// Reserved (0.0)
        /// 
        /// </param>
        /// <param name="Returned_Values">
        /// Returns number of value sets (frequency, level and delta to limit line) read (available to return) from the instrument. Number of values returned in arrays is limited with the parameter No of Values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadBDOPowerVsTimeListEvaluation(int No_Of_Values, double[] Range_Number, double[] Start_Time, double[] Stop_Time, double[] Avg_Power_Abs, double[] Avg_Power_Rel, double[] Max_Power_Abs, double[] Max_Power_Rel, double[] Min_Power_Abs, double[] Min_Power_Rel, double[] Limit_Check, double[] Reserved1, double[] Reserved2, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.ReadBDOPowerVsTimeListEvaluation(this._handle, No_Of_Values, Range_Number, Start_Time, Stop_Time, Avg_Power_Abs, Avg_Power_Rel, Max_Power_Abs, Max_Power_Rel, Min_Power_Abs, Min_Power_Rel, Limit_Check, Reserved1, Reserved2, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the 1xEV-DO Mobile Station Test mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MDO_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument[:SELect] MDO
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int MDOMode()
        {
            int pInvokeResult = PInvoke.MDOMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the values of the selected channel table in the code domain power measurement mode. The whole table is defined in one go. The inactive channels (INACtive) do not have to be defined. Eight values are specified for a line of table.
        /// 
        /// Prior to this function, the name of the channel table has to be defined with function configure CDMA2000 Channel Table Operations (ConfigureC2KChannelTableOperations) - attribute RSSPECAN_ATTR_C2K_CTABLE_NAME.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for option K85.
        /// 
        /// Attribute(s):
        /// None
        /// 
        /// Remote-control command(s):
        /// CONFigure:CDPower[:BTS]:CTABle:DATA 0..5, 2..4, 0..15, 0..1, 0..65535, 0 , 0 | 1, 0, ...
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Channel_Type">
        /// This control selects the Channel Type. 
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0 = PICH
        /// 1 = RRI
        /// 2 = DATA
        /// 3 = ACK
        /// 4 = DRC
        /// 5 = INACTIVE
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Code_Class">
        /// This control sets the Code Class.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 2 to 4
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Code_Number">
        /// This control sets the Code Number.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0 to 15
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Mapping">
        /// This control sets the Mapping.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0 = I branch
        /// 1 = Q branch
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Activity">
        /// This control sets the Activity.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0 to 65535 (decimal)
        /// 
        /// Default Value: None
        /// 
        /// Notes:
        /// 
        /// (1) The decimal number interpreted as a binary number in 16 bits, determines the half slot in which the channel is active (value 1) or inactive (value 0).
        /// Examples:
        /// 65535 dec. = 1111 1111 1111 1111 bin. (e.g. DATA)
        /// Channel is active in each half slot
        /// 43690 dec. = 1010 1010 1010 1010 bin. (e.g. RRI)
        /// Channel is active in half slot 0, 2, 4 etc
        /// 24576 dec. = 0110 0000 0000 0000 bin. (e.g. DRC)
        /// Channel is active in half slot 1 and 2
        /// </param>
        /// <param name="Reserved_1">
        /// This control is reserver for future use.
        /// </param>
        /// <param name="Status">
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </param>
        /// <param name="Reserved_2">
        /// This control is reserver for future use.
        /// </param>
        /// <param name="Num_of_Elements">
        /// Sets number of elements for parameter arrays.
        /// 
        /// Valid Values: &gt;= 1
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMDOChannelTableData(int[] Channel_Type, int[] Code_Class, int[] Code_Number, int[] Mapping, int[] Activity, int[] Reserved_1, ushort[] Status, int[] Reserved_2, int Num_of_Elements)
        {
            int pInvokeResult = PInvoke.ConfigureMDOChannelTableData(this._handle, Channel_Type, Code_Class, Code_Number, Mapping, Activity, Reserved_1, Status, Reserved_2, Num_of_Elements);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Advanced Settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MC2K_CDP_PREFERENCE
        /// RSSPECAN_ATTR_MC2K_CDP_ORDER
        /// RSSPECAN_ATTR_MC2K_CDP_OVERVIEW
        /// RSSPECAN_ATTR_MC2K_CDP_MAPPING
        /// RSSPECAN_ATTR_MDO_CDP_AVERAGE
        /// RSSPECAN_ATTR_MDO_CDP_OPERATION
        /// RSSPECAN_ATTR_MDO_CDP_LCODE_I
        /// RSSPECAN_ATTR_MDO_CDP_LCODE_Q
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:PREFerence TOTal | PICH
        /// [SENSe:]CDPower:ORDer HADamard | BITReverse
        /// [SENSe:]CDPower:OVERview ON | OFF
        /// [SENSe:]CDPower:MAPPing I | Q
        /// [SENSe:]CDPower:AVERage ON | OFF
        /// [SENSe:]CDPower:OPERation ACCess | TRAFfic
        /// [SENSe:]CDPower:LCODe:I '#H0' .. '#H3FFFFFFFFFF'
        /// [SENSe:]CDPower:LCODe:Q '#H0' .. '#H3FFFFFFFFFF'
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Power_Reference">
        /// This control sets the reference for the relative CDP measured values to the total power or the PICH power. The driver uses this value to set the RSSPECAN_ATTR_MC2K_CDP_PREFERENCE attribute.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_PREF_TOTAL   (0) - Total
        /// RSSPECAN_VAL_C2K_PREF_PICH    (1) - PICH
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_PREF_TOTAL
        /// </param>
        /// <param name="Order">
        /// This control sets the order of the code domain evaluation. The driver uses this value to set the RSSPECAN_ATTR_MC2K_CDP_ORDER attribute.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_ORDER_HADAMARD   (0) - Hadamard
        /// RSSPECAN_VAL_C2K_ORDER_BITREVERSE (1) - Bitreverse
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_ORDER_HADAMARD
        /// </param>
        /// <param name="Select_I___Q">
        /// This control selects whether the I or Q branch should be analyzed. The driver uses this value to set the RSSPECAN_ATTR_MC2K_CDP_MAPPING attribute.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MC2K_CDP_MAPPING_I   (0) - I
        /// RSSPECAN_VAL_MC2K_CDP_MAPPING_Q   (1) - Q
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_MC2K_CDP_MAPPING_I
        /// </param>
        /// <param name="CDP_Average">
        /// This control is used to enable averaging of the CDP evaluation over all recorded half slots. The driver uses this value to set the RSSPECAN_ATTR_MDO_CDP_AVERAGE attribute.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE   (0) - Off
        /// VI_TRUE    (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Notes:
        /// 
        /// (1) The attribute is only available in the CDP measurement.
        /// </param>
        /// <param name="Operation">
        /// This control is used to set the operation mode. The driver uses this value to set the RSSPECAN_ATTR_MDO_CDP_OPERATION attribute.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MDO_CDP_OPER_ACCESS  (0) - Acces
        /// RSSPECAN_VAL_MDO_CDP_OPER_TRAFFIC (1) - Traffic
        /// 
        /// Default Value: RSSPECAN_VAL_MDO_CDP_OPER_TRAFFIC
        /// 
        /// Notes:
        /// 
        /// (1) In TRAFFIC mode all channels (PICH/RRI/DATA/ACK and DRC) can exist. PICH and RRI are always in the signal. In ACCESS mode only PICH (always available) and DATA channel can exist.
        /// </param>
        /// <param name="Long_Code_I">
        /// This control defines the mask of the long code in hexadecimal format for the I branch. The driver uses this value to set the RSSPECAN_ATTR_MDO_CDP_LCODE_I attribute.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// "0" to "3FFFFFFFFFF"
        /// 
        /// Default Value: "0"
        /// </param>
        /// <param name="Long_Code_Q">
        /// This control defines the mask of the long code in hexadecimal format for the Q branch. The driver uses this value to set the RSSPECAN_ATTR_MDO_CDP_LCODE_Q attribute.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// "0" to "3FFFFFFFFFF"
        /// 
        /// Default Value: "0"
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMDOAdvancedSettings(int Power_Reference, int Order, int Select_I___Q, bool CDP_Average, int Operation, string Long_Code_I, string Long_Code_Q)
        {
            int pInvokeResult = PInvoke.ConfigureMDOAdvancedSettings(this._handle, Power_Reference, Order, Select_I___Q, System.Convert.ToUInt16(CDP_Average), Operation, Long_Code_I, Long_Code_Q);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the CDP Overview.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for Code Domain Power or Code Domain Error Power analysis.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MC2K_CDP_OVERVIEW
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:OVERview ON | OFF
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Code_Domain_Overview">
        /// This control can be enabled by means of ON when either the Code Domain Power or the Code Domain Error Power analysis is active. (Refer to the attributes RSSPECAN_ATTR_C2K_RESULT_A and RSSPECAN_ATTR_C2K_RESULT_B). In Overview mode, the I branch of the signal is normally displayed on screen A and the Q branch of the signal on screen B with the CDP/CDEP. The driver uses this value to set the RSSPECAN_ATTR_MC2K_CDP_OVERVIEW attribute.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// VI_FALSE   (0) - Off
        /// VI_TRUE    (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMDOCDPOverview(bool Code_Domain_Overview)
        {
            int pInvokeResult = PInvoke.ConfigureMDOCDPOverview(this._handle, System.Convert.ToUInt16(Code_Domain_Overview));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets marker to channel 0.16, 0.32 or 0.64, depending on the base spreading factor.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MC2K_MARKER_PICH
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:PICH
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMDOPichMarkerSettings()
        {
            int pInvokeResult = PInvoke.ConfigureMDOPichMarkerSettings(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Code Domain Analyzer.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_C2K_RESULT_TYPE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FEED 'XPOW:CDP' | 'XPOW:CDP:RAT' | 'XPOW:CDEP' | 'XTIM:CDP:MACCuracy' | 'XTIM:CDP:PVSLot' | 'XTIM:CDP:PVSYmbol' | 'XTIM:CDP:BSTReam' | 'XTIM:CDP:ERR:SUMM' | 'XTIM:CDP:ERR:CTABle' | 'XTIM:CDP:ERR:PCDomain' | 'XTIM:CDP:SYMB:CONSt' | 'XTIM:CDP:SYMB:EVM' | 'XTIM:CDP:COMP:CONSt'
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Result_Type">
        /// This control selects the results to be displayed on screen A. The driver uses this value to set the RSSPECAN_ATTR_C2K_RESULT_TYPE attribute.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_FEED_CDP       (0)  -  Code Domain Power Absolute   
        /// RSSPECAN_VAL_C2K_FEED_CDP_RAT   (1)  -  Code Domain Power Relative   
        /// RSSPECAN_VAL_C2K_FEED_CDEP      (2)  -  Code Domain Error            
        /// RSSPECAN_VAL_C2K_FEED_ERR_CTAB  (3)  -  Channel Table                
        /// RSSPECAN_VAL_C2K_FEED_MACC      (4)  -  Composite EVM                
        /// RSSPECAN_VAL_C2K_FEED_PVSL      (5)  -  Power vs PCG                 
        /// RSSPECAN_VAL_C2K_FEED_PVSY      (6)  -  Power vs Symbol              
        /// RSSPECAN_VAL_C2K_FEED_BSTR      (7)  -  Bitsream                     
        /// RSSPECAN_VAL_C2K_FEED_ERR_SUMM  (8)  -  Result Summary               
        /// RSSPECAN_VAL_C2K_FEED_ERR_PCD   (9)  -  Peak Code Domain Error       
        /// RSSPECAN_VAL_C2K_FEED_SYMB_CONS (10) -  Symbol Const                 
        /// RSSPECAN_VAL_C2K_FEED_SYMB_EVM  (11) -  Symbol EVM                   
        /// RSSPECAN_VAL_C2K_FEED_COMP_CONS (12) 	-  Composite Const              
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_FEED_CDP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMDOCodeDomainAnalyzerResults(int Result_Type)
        {
            int pInvokeResult = PInvoke.ConfigureMDOCodeDomainAnalyzerResults(this._handle, Result_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the Code Domain Analyzer Settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for option K85.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MDO_CDP_IQLENGTH_FSQ_FSU
        /// RSSPECAN_ATTR_MDO_CDP_IQLENGTH_FSP
        /// RSSPECAN_ATTR_MDO_CDP_CODE
        /// RSSPECAN_ATTR_C2K_CDP_SLOT
        /// RSSPECAN_ATTR_MC2K_CDP_MAPPING
        /// 
        /// Remote-control command(s):
        /// [SENSe:]CDPower:IQLength
        /// [SENSe:]CDPower:CODE 0 to (BASE SF-1)
        /// [SENSe:]CDPower:SLOT 0  to (IQ_CAPTURE_LENGTH-1)
        /// [SENSe:]CDPower:MAPPing I | Q
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Capture_Length">
        /// This control sets the Captured Length. The driver uses this value to set the RSSPECAN_ATTR_MDO_CDP_IQLENGTH_FSP or RSSPECAN_ATTR_MDO_CDP_IQLENGTH_FSQ_FSU  attribute.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 4 to 70 for FSU, FSQ
        /// 4 to 24 for FSP
        /// 
        /// 
        /// Default Value: 6
        /// </param>
        /// <param name="Select_Channel">
        /// This control selects the code number. The driver uses this value to set the RSSPECAN_ATTR_MDO_CDP_CODE  attribute.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0 to 15
        /// 
        /// 
        /// Default Value: 0
        /// 
        /// Notes:
        /// 
        /// (1) The values are shown on the instrument like this: &lt;channel number&gt;,&lt;spreading factor&gt;
        /// </param>
        /// <param name="Select_Halfslot">
        /// This control selects a half slot. The driver uses this value to set the RSSPECAN_ATTR_C2K_CDP_SLOT attribute.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 0 to (IQ_CAPTURE_LENGTH-1)
        /// 
        /// 
        /// Default Value: 0
        /// </param>
        /// <param name="Select">
        /// This control selects the Mapping. The driver uses this value to set the RSSPECAN_ATTR_MC2K_CDP_MAPPING attribute.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MC2K_CDP_MAPPING_Q   (0) - Q
        /// RSSPECAN_VAL_MC2K_CDP_MAPPING_I   (1) - I
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_MC2K_CDP_MAPPING_I
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMDOCodeDomainAnalyzerSettings(int Capture_Length, int Select_Channel, int Select_Halfslot, int Select)
        {
            int pInvokeResult = PInvoke.ConfigureMDOCodeDomainAnalyzerSettings(this._handle, Capture_Length, Select_Channel, Select_Halfslot, Select);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the measured and calculated values of the code domain power analysis.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for option K85.
        /// 
        /// Attribute(s):
        /// None
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:FUNCtion:CDPower:RESult? SLOT | PTOTal | PPICh | PRRI | RHO | MACCuracy | PCDerror | ACTive | FERRor | FERPpm | DRPich | RHOVerall | CERRor | TFRame | IQOFfset | IQIMbalance | SRATe | CHANnel | SFACtor | TOFFset | POFFset | CDPabsolute | CDPRelative | EVMRms | EVMPeak
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Result_Type">
        /// This control selects the result type.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_C2K_SLOT          (0) - Halfslot
        /// RSSPECAN_VAL_C2K_PTOTAL        (1) - Total PWR
        /// RSSPECAN_VAL_C2K_PPICH         (2) - Pilot PWR
        /// RSSPECAN_VAL_C2K_RHO           (3) - RHO
        /// RSSPECAN_VAL_C2K_MACCURACY     (4) - Composite EVM
        /// RSSPECAN_VAL_C2K_PCDERROR      (5) - Peak Code Domain Error
        /// RSSPECAN_VAL_C2K_IQIMBALANCE   (6) - IQ Imbalance
        /// RSSPECAN_VAL_C2K_IQOFFSET      (7) - IQ Offset
        /// RSSPECAN_VAL_C2K_FERROR        (8) - Carrier Freq Error Hz
        /// RSSPECAN_VAL_C2K_FERPPM        (9) - Carrier Freq Error ppm
        /// RSSPECAN_VAL_C2K_CERROR        (10) - Chip Rate Error
        /// RSSPECAN_VAL_C2K_TFRAME        (11) - Trigger to Frame
        /// RSSPECAN_VAL_C2K_ACTIVE        (12) - No of Active Chan
        /// RSSPECAN_VAL_C2K_SRATE         (13) - Symbol Rate
        /// RSSPECAN_VAL_C2K_CHANNEL       (14) - Channel
        /// RSSPECAN_VAL_C2K_SFACTOR       (15) - Spreading Factor
        /// RSSPECAN_VAL_C2K_TOFFSET       (16) - Timing Offset
        /// RSSPECAN_VAL_C2K_POFFSET       (17) - Phase Offset
        /// RSSPECAN_VAL_C2K_CDPRELATIVE   (18) - Channel Power Rel
        /// RSSPECAN_VAL_C2K_CDPABSOLUTE   (19) - Channel Power Abs
        /// RSSPECAN_VAL_C2K_EVMRMS        (20) - Symbol EVM rms
        /// RSSPECAN_VAL_C2K_EVMPEAK       (21) - Symbol EVM Pk
        /// RSSPECAN_VAL_C2K_PRRI          (37) - RRI Power
        /// RSSPECAN_VAL_C2K_DRPICH        (38) - DELTA RRI/PICH
        /// RSSPECAN_VAL_C2K_RHOVERALL     (39) - RHO overal
        /// 
        /// Default Value: RSSPECAN_VAL_C2K_SLOT
        /// 
        /// Notes:
        /// 
        /// (1) Total Power: Specifies the total power of the signal.
        /// 
        /// (2) Pilot Power: Specifies the pilot power.
        /// 
        /// (3) RRI Power: Specifies the RRI power; dashes are displayed if there is no RRI channel.
        /// 
        /// (4) RHO: Specifies the RHO quality parameter. According to the 1xEVDO specification, RHO is the normalized, correlated power between the measured and the ideally generated reference signal. The 1xEV-DO specification requires that only the pilot channel be input during measurement of RHO.
        /// 
        /// (5) Composite EVM: The composite EVM value is the difference between the test signal and the ideal reference signal (see the COMPOSITE EVM softkey).
        /// 
        /// (6) Pk CDE: The PEAK CODE DOMAIN ERR measurement specifies a projection of the difference between the test signal and the ideal reference signal to spreading factor 16 (see softkeys PEAK CODE DOMAIN ERR). The spreading factor, for which the projection is made, is specified next to the measured value.
        /// 
        /// (7) IQ Imbalance: IQ imbalance of the signal in %
        /// 
        /// (8) IQ Offset: DC offset of the signal in %
        /// 
        /// (9) Carrier Freq Error: Specifies the frequency error referred to the set center frequency of the analyzer. The absolute frequency error is the sum of the frequency error of the analyzer and that of the device under test. Excessive differences between transmitter and receiver frequency impair synchronization of the CDP measurement. If at all possible, the transmitter and the receiver should therefore be synchronized to a common reference frequency (see the chapter "Getting Started"). The frequency error is available both in Hz and in ppm referred to the carrier frequency.
        /// 
        /// (10) DELTA RRI/PICH: This value specifies a logarithmic correlation between RRI and pilot power. The specification requires that 16 half slots be measured; this can be adjusted using the CAPTURE LENGTH softkey. The formula of the standard has been extended to any given number of half slots.
        /// 
        /// (11) RHO overall: RHO determined over all half slots.
        /// 
        /// (12) Trigger to Frame: This measurement result reproduces the timing offset from the beginning of the acquired signal section until the start of the first even-numbered half slot. In the case of triggered data acquisition, this corresponds to the timing offset frametrigger (+ trigger-offset) - start of the first even-numbered half slot. If the analyzer was not able to synchronize to the 1xEVDO signal, the value of Trg to Frame is not meaningful. If the FREE RUN trigger is selected, dashes (-.--) are shown.
        /// 
        /// (13) Chip Rate Error: Specifies the chip rate error (1.2288 Mcps) in ppm. A high chip rate error causes symbol errors and this may prevent the CDP measurement from performing synchronization. This measurement result is valid even if the analyzer was not able to synchronize to the 1xEV-DO signal.
        /// 
        /// (14) Active Channels: Specifies the number of active channels found in the signal. The PICH and RRI each count as an individual channel. (Display for each half slot)
        /// 
        /// (15) Symbol Rate: Symbol rate with which the channel is transmitted.
        /// 
        /// (16) Mapping: Indicates whether the I or Q branch is being evaluated.
        /// 
        /// (17) Channel.SF: Number of the channel and its associated spreading factor.
        /// 
        /// (18) Timing Offset: Timing offset between the selected channel and the pilot channel. This measurement can be enabled by means of the TIME/PHASE softkey.
        /// 
        /// (19) Phase Offset: Phase offset between the selected channel and the pilot channel. This measurement can be enabled by means of the TIME/PHASE softkey.
        /// 
        /// (20) Chan Pow rel. / abs.: Relative channel power (referred to the pilot or total power, depending on the POWER REF TOT/PICH softkey) and the absolute channel power.
        /// 
        /// (21) Symbol EVM Pk / rms: Peak or mean value of the results of the error vector magnitude measurement (see the SYMBOL EVM softkey). The measurement provides information about the EVM of the selected channel for the selected half slot at symbol level.
        /// </param>
        /// <param name="Result">
        /// This control returns the measured and calculated values of the code domain power analysis. 
        /// 
        /// Units:
        /// Total power in dBm 
        /// Frequency error in Hz
        /// Pilot power in dBm 
        /// Frequency error in ppm
        /// RRI power in dBm 
        /// Delta RRI/PICH in dB
        /// Composite EVM in % 
        /// PCDerror Peak code domain error in dB 
        /// Chip rate error in ppm
        /// IQ imbalance in %
        /// IQ offset in %
        /// Symbol rate in ksps 
        /// Timing offset in s
        /// Phase offset in rad
        /// Channel power relative in dB 
        /// Channel power absolute in dBm
        /// Error vector magnitude RMS in % 
        /// Error vector magnitude peak in %
        /// 
        /// All other parameters have no units.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryMDOCodeDomainAnalyzerResultSummary(int Window, int Result_Type, out double Result)
        {
            int pInvokeResult = PInvoke.QueryMDOCodeDomainAnalyzerResultSummary(this._handle, Window, Result_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Spectrum EM Mask Measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MC2K_LIM_MODE
        /// RSSPECAN_ATTR_C2K_LIM_TRANS
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:ESPectrum:MODE AUTO | MANual | USER
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1 to 8&gt;:ESPectrum:TRANsition
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Limit_Line_Type">
        /// This control selects the Limit Line Type. The driver uses this value to set the RSSPECAN_ATTR_MC2K_LIM_MODE attribute.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MC2K_LIM_MODE_AUTO  (0) - Auto
        /// RSSPECAN_VAL_MC2K_LIM_MODE_USER  (1) - User
        /// 
        /// Default Value: RSSPECAN_VAL_MC2K_LIM_MODE_AUTO
        /// 
        /// Note(s):
        /// 
        /// (1) The user mode enables the creation of user-defined limit lines. The softkey opens the menus of the limit-line editor, which may be familiar from the basic unit.
        /// </param>
        /// <param name="_30kHz_1MHz_Transition">
        /// This control sets the 30kHz/1MHz Transition. The driver uses this value to set the RSSPECAN_ATTR_C2K_LIM_TRANS attribute.
        /// 
        /// Units: None
        /// 
        /// Valid Values:
        /// 100.0 to 1.0e+9
        /// 
        /// Default Value: 4.0e+6
        /// 
        /// Note(s):
        /// 
        /// (1) This attribute specifies the offset frequency at which the resolution bandwidth is changed between 30k and 1MHz. For multicarrier spectrum emission mask measurement this value can be extended. It is recommended to define user specific limit lines in this case.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureMDOSEM(int Limit_Line_Type, double _30kHz_1MHz_Transition)
        {
            int pInvokeResult = PInvoke.ConfigureMDOSEM(this._handle, Limit_Line_Type, _30kHz_1MHz_Transition);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the WLAN analyzer mode.
        /// 
        /// Note:
        /// (1) This function is not valid for FS-K92 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument:SELect WLAN
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int WlanSetWlanMode()
        {
            int pInvokeResult = PInvoke.WlanSetWlanMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Wireless LAN standard.
        /// 
        /// Note:
        /// (1) This function is not valid for FS-K92 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_STAN
        /// 
        /// Remote-control command(s):
        /// CONFigure:STANdard
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Standard">
        /// Sets the standard.
        /// 
        /// Valid Values:
        /// 0 - IEEE 802.11a
        /// 1 - IEEE 802.11b
        /// 2 - IEEE 802.11j (10 MHz)
        /// 3 - IEEE 802.11j (20 MHz)
        /// 4 - IEEE 802.11g
        /// 5 - 802.11 OFDM Turbo Mode / IEEE 802.11n for FSV only
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_STAN
        /// 
        /// Note:
        /// (1) For option FSQ-K90 is valid only 0 - IEEE 802.11a
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanStandard(int Standard)
        {
            int pInvokeResult = PInvoke.ConfigureWlanStandard(this._handle, Standard);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the signal characteristics.
        /// 
        /// Note:
        /// (1) This function is available only with option B25.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AMPL_EATT_STATE
        /// RSSPECAN_ATTR_AMPL_EATT_AUTO
        /// RSSPECAN_ATTR_AMPL_EATT_VALUE
        /// 
        /// Remote-control command(s):
        /// INPut:EATT:STATe
        /// INPut:EATT:AUTO ON | OFF
        /// INPut:EATT
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="State">
        /// This contorl switches the electronic input attenuation into the signal path (state ON) or removes it from the signal path (state OFF).
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value) 
        /// VI_TRUE  (1) - On 
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_INP_EATT_STAT
        /// </param>
        /// <param name="Electronic_Attenuator_Auto">
        /// This control specifies, that instrument automatically couples the electronic input attenuation to the reference level and the attenuation of the mechanical attenuator (state ON) or switches the input attenuation to manual entry (state OFF).
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_INP_EATT_AUTO
        /// 
        /// Note:
        /// (1) This parameter is valid only if State is set On.
        /// 
        /// </param>
        /// <param name="Manual__Value__dB_">
        /// This control sets the manual input electronic attenuation level. 
        /// 
        /// Valid Values: 0 to 30 dB
        /// 
        /// Default Value: 0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_INP_EATT
        /// 
        /// Note:
        /// 
        /// (1) The attenuation can be varied in 5 dB steps.
        /// 
        /// (2) This parameter is valid only if Attenuator Auto is set Off.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanElectronicAtt(bool State, bool Electronic_Attenuator_Auto, int Manual__Value__dB_)
        {
            int pInvokeResult = PInvoke.ConfigureWlanElectronicAtt(this._handle, System.Convert.ToUInt16(State), System.Convert.ToUInt16(Electronic_Attenuator_Auto), Manual__Value__dB_);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the YIG filter for image frequency suppression into the signal path (condition ON) and/or removes it from the signal path (condition OFF).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SERVICE_YIG_FILTER
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:FILTer:YIG[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="YIG_Filter">
        /// Switches the YIG filter for image rejection into the signal path (ON) or removes it from the signal path (OFF). 
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SERVICE_YIG_FILTER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWLANYIGFilter(int Window, bool YIG_Filter)
        {
            int pInvokeResult = PInvoke.ConfigureWLANYIGFilter(this._handle, Window, System.Convert.ToUInt16(YIG_Filter));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the trigger parameters. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_TRIG_MODE
        /// RSSPECAN_ATTR_TRIGGER_DELAY
        /// RSSPECAN_ATTR_WLAN_TRIG_POW_LEV_AUTO
        /// RSSPECAN_ATTR_WLAN_TRIG_POW_LEV
        /// RSSPECAN_ATTR_EXTERNAL_TRIGGER_LEVEL
        /// 
        /// Remote-control command(s):
        /// TRIGger[:SEQuence]:MODE IMMediate | EXTernal | POWer
        /// TRIGger&lt;1|2&gt;[:SEQuence]:HOLDoff
        /// TRIGger[:SEQuence]:LEVel:POWer:AUTO
        /// TRIGger[:SEQuence]:LEVel:POWer
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel[:EXTernal]
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Mode">
        /// Sets the trigger mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TRIG_MODE_IMM (0) - Immediate
        /// RSSPECAN_VAL_TRIG_MODE_EXT (1) - External
        /// RSSPECAN_VAL_TRIG_MODE_POW (2) - Power 
        /// 
        /// Default Value: RSSPECAN_VAL_TRIG_MODE_IMM (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_TRIG_MODE
        /// </param>
        /// <param name="Offset">
        /// Defines the length of the trigger delay. A negative delay time (pretrigger) can be set.
        /// 
        /// Valid Values:
        /// -100.0 to 100.0
        /// 
        /// Default Value: 10.0e-6 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_DELAY
        /// 
        /// Note:
        /// (1) This parameter has effect only if Trigger Mode isn't Free Run
        /// 
        /// 
        /// </param>
        /// <param name="Auto_Trigger_Level">
        /// Specifies whether or not an automatic power trigger level calculation is performed before each main measurement.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_TRIG_POW_LEV_AUTO
        /// 
        /// Note:
        /// (1) This parameter has effect only for Trigger Mode set to Power
        /// 
        /// </param>
        /// <param name="Power_Trigger_Level">
        /// Specifies the level of the input signal for which triggering will occur. The value is specified in dBm for RF input and V for baseband inputs.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_TRIG_POW_LEV
        /// 
        /// Note:
        /// (1) This parameter has effect only for Trigger Mode set to Power
        /// 
        /// 
        /// </param>
        /// <param name="Ext__Trigger_Level">
        /// Specifies the level of the external trigger input for which triggering will occur.
        /// 
        /// Valid Values:
        /// 500.0e-3 to 3.5 V
        /// 
        /// Default Value: 1.4 V
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_TRIGGER_LEVEL
        /// 
        /// Note:
        /// (1) This parameter has effect only for Trigger Mode set to External
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanTrigger(int Mode, double Offset, bool Auto_Trigger_Level, double Power_Trigger_Level, double Ext__Trigger_Level)
        {
            int pInvokeResult = PInvoke.ConfigureWlanTrigger(this._handle, Mode, Offset, System.Convert.ToUInt16(Auto_Trigger_Level), Power_Trigger_Level, Ext__Trigger_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the trigger parameters. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_EGAT
        /// RSSPECAN_ATTR_WLAN_EGAT_HOLD_TIME
        /// RSSPECAN_ATTR_WLAN_EGAT_HOLD_SAMP
        /// RSSPECAN_ATTR_WLAN_EGAT_LENG_TIME
        /// RSSPECAN_ATTR_WLAN_EGAT_LENG_SAMP
        /// RSSPECAN_ATTR_WLAN_EGAT_LINK
        /// 
        /// Remote-control command(s):
        /// [SENSe:]SWEep:EGATe ON | OFF
        /// [SENSe:]SWEep:EGATe:HOLDoff[:TIMe]
        /// [SENSe:]SWEep:EGATe:HOLDoff:SAMPle
        /// [SENSe:]SWEep:EGATe:LENGth[:TIMe]
        /// [SENSe:]SWEep:EGATe:LENGth:SAMPle
        /// [SENSe:]SWEep:EGATe:LINK ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Gating">
        /// This parameter switches on/off the gating. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_EGAT
        /// </param>
        /// <param name="Mode">
        /// Sets the way for delay and length specification. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WLAN_GATE_TIME   (0) - Time
        /// RSSPECAN_VAL_WLAN_GATE_SAMPLE (1) - Samples
        /// 
        /// Default Value: RSSPECAN_VAL_GATE_TIME (0)
        /// 
        /// Note:
        /// (1) Time - Length and Delay are set in time units
        ///     Samples - Length and Delay are set in samples
        /// 
        /// (2) This parameter has effect only for Gating set to On
        /// 
        /// </param>
        /// <param name="Delay">
        /// Defines the gate delay in the capture buffer in time  units or samples. The range of this value is dependent on the measurement that was last run. The driver uses this value to set the RSSPECAN_ATTR_WLAN_EGAT_HOLD_TIME or RSSPECAN_ATTR_WLAN_EGAT_HOLD_SAMP attribute.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 100.0e-6 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_EGAT_HOLD_TIME RSSPECAN_ATTR_WLAN_EGAT_HOLD_SAMP
        /// 
        /// Note:
        /// (1) This parameter has effect only for Gating set to On
        /// 
        /// </param>
        /// <param name="Length">
        /// Specifies defines the gate time in the capture buffer in time units or samples. The range of this value is dependent on the measurement that was last run. The driver uses this value to set the RSSPECAN_ATTR_WLAN_EGAT_LENG_TIME or RSSPECAN_ATTR_WLAN_EGAT_LENG_SAMP attribute.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 400.0e-6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_EGAT_LENG_TIME RSSPECAN_ATTR_WLAN_EGAT_LENG_SAMP
        /// 
        /// Note:
        /// (1) This parameter has effect only for Gating set to On
        /// 
        /// 
        /// 
        /// </param>
        /// <param name="Link_Gate_And_Marker">
        /// Links together the movement of the gating lines and the capture buffer marker.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_EGAT_LINK
        /// 
        /// Note:
        /// (1) This parameter has effect only for Gating set to On
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanGate(bool Gating, int Mode, double Delay, double Length, bool Link_Gate_And_Marker)
        {
            int pInvokeResult = PInvoke.ConfigureWlanGate(this._handle, System.Convert.ToUInt16(Gating), Mode, Delay, Length, System.Convert.ToUInt16(Link_Gate_And_Marker));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the ACP measurement mode to either absolute or relative.
        /// 
        /// Note:
        /// 
        /// (1) This function is available only for IEEE 802.11j standards.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_ACH_MODE
        /// 
        /// Remote-control command(s):
        /// [SENSe:]POWer:ACHannel:MODE ABS | REL
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="ACP_Mode">
        /// This control sets the ACP measurement mode to either absolute or relative.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ABS (0) - Absolute 
        /// RSSPECAN_VAL_REL (1) - Relative
        /// 
        /// Default Value: RSSPECAN_VAL_ABS
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_ACH_MODE
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanACPMode(int ACP_Mode)
        {
            int pInvokeResult = PInvoke.ConfigureWlanACPMode(this._handle, ACP_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the units for gain imbalance results.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_UNIT_GIMB
        /// 
        /// Remote-control command(s):
        /// UNIT:GIMBalance DB | PCT
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Unit">
        /// This parameter specifies the units for gain imbalance results. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DB  (8)  - dB 
        /// RSSPECAN_VAL_UNIT_PCT (13) - %
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_DB (13)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_UNIT_GIMB
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanGainImbalanceUnit(int Unit)
        {
            int pInvokeResult = PInvoke.ConfigureWlanGainImbalanceUnit(this._handle, Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the acquisition.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SWEEP_MODE_CONTINUOUS
        /// 
        /// Remote-control command(s):
        /// INITiate&lt;1|2&gt;:CONTinuous ON | OFF
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Sweep_Mode_Continuous">
        /// Turns on/off continuous sweeping.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SWEEP_MODE_CONTINUOUS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWLANSweep(bool Sweep_Mode_Continuous)
        {
            int pInvokeResult = PInvoke.ConfigureWLANSweep(this._handle, System.Convert.ToUInt16(Sweep_Mode_Continuous));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function performs the auto level routine immediately one time regardless of the current state of the Auto Level State.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_POW_AUTO_ONCE
        /// 
        /// Remote-control command(s):
        /// CONFigure:POWer:AUTO ONCE
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanAutoLevelOnce()
        {
            int pInvokeResult = PInvoke.ConfigureWlanAutoLevelOnce(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the signal characteristics.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_POW_AUTO
        /// RSSPECAN_ATTR_WLAN_POW_AUTO_SWE_TIME
        /// RSSPECAN_ATTR_WLAN_POW_EXP_IQ
        /// RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET
        /// 
        /// Remote-control command(s):
        /// CONFigure:POWer:AUTO ON | OFF
        /// CONFigure:POWer:AUTO:SWEep:TIME
        /// CONFigure:POWer:EXPected:IQ
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:RLEVel:OFFSet
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Auto_Level">
        /// Switches on or off automatic power level detection.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_POW_AUTO
        /// 
        /// </param>
        /// <param name="Auto_Level_Time">
        /// Sets the seep time for the automatic power level detection.
        /// 
        /// Valid Values: 1.0e-3 to 1.0 s
        /// 
        /// Default Value: 100.0e-3 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_POW_AUTO_SWE_TIME
        /// </param>
        /// <param name="Signal_Level">
        /// Sets the mean power level of the source signal as supplied to the Analyzer IQ inputs. This value will be overwritten if Auto Level is turned on.
        /// 
        /// Valid Values:
        /// 0.0316,0.0562,0.1,0.178,0.316,0.562,1.0,1.78,3.16,5.62
        /// 
        /// Default Value: 0.0316 V
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_POW_EXP_IQ
        /// 
        /// Note:
        /// (1) Another value will be coerced to allowed
        /// 
        /// (2) This control has no effect for FSL
        /// </param>
        /// <param name="Ext_Att">
        /// Specifies the external attenuation or gain applied to the RF signal.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 0.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanBasebandSignalLevel(bool Auto_Level, double Auto_Level_Time, double Signal_Level, double Ext_Att)
        {
            int pInvokeResult = PInvoke.ConfigureWlanBasebandSignalLevel(this._handle, System.Convert.ToUInt16(Auto_Level), Auto_Level_Time, Signal_Level, Ext_Att);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the channel frequency.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FREQUENCY_CENTER
        /// RSSPECAN_ATTR_WLAN_CHAN
        /// 
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer
        /// CONFigure:CHANnel
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Channel_No">
        /// Sets the input channel.
        /// 
        /// Valid Values:
        /// &gt;0
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_CHAN
        /// 
        /// Note:
        /// (1) If channel number is set to 0, the value is not set and Frequency is set instead of channel.
        /// 
        /// </param>
        /// <param name="Frequency">
        /// Sets the center frequency.
        /// 
        /// Valid Values:
        /// FSQ 3  - 0.0 Hz to 3.6 GHz
        /// FSQ 8  - 0.0 Hz to 8.0 GHz
        /// FSQ 26 - 0.0 Hz to 26.5 GHz
        /// 
        /// Default Value: 1.5e+9 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_CENTER
        /// 
        /// Note:
        /// (1) This avalue is set only if channel number is set to 0. The instrument modifies the Channel number accordingly.
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanRFChannelFrequency(int Channel_No, double Frequency)
        {
            int pInvokeResult = PInvoke.ConfigureWlanRFChannelFrequency(this._handle, Channel_No, Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the signal characteristics.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_POW_AUTO
        /// RSSPECAN_ATTR_WLAN_POW_AUTO_SWE_TIME
        /// RSSPECAN_ATTR_WLAN_POW_EXP_RF
        /// RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET
        /// RSSPECAN_ATTR_ATTENUATION
        /// 
        /// 
        /// Remote-control command(s):
        /// CONFigure:POWer:AUTO ON | OFF
        /// CONFigure:POWer:AUTO:SWEep:TIME
        /// CONFigure:POWer:EXPected:RF
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:RLEVel:OFFSet
        /// INPut&lt;1|2&gt;:ATTenuation
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Auto_Level">
        /// Switches on or off automatic power level detection.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_POW_AUTO
        /// 
        /// </param>
        /// <param name="Auto_Level_Time">
        /// Sets the seep time for the automatic power level detection.
        /// 
        /// Valid Values: 1.0e-3 to 1.0 s
        /// 
        /// Default Value: 100.0e-3 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_POW_AUTO_SWE_TIME
        /// </param>
        /// <param name="Signal_Level">
        /// Sets the mean power level of the source signal as supplied to the analyzer RF input.
        /// 
        /// Valid Values: no range checking (depends on RF Att setting)
        /// 
        /// Default Value: -15.0 dBm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_POW_EXP_RF
        /// 
        /// Note:
        /// (1) This parameter has effect only if Auto Level is set Off.
        /// 
        /// (2) Not available for R&amp;S FMU instrument.
        /// </param>
        /// <param name="RF_Att">
        /// Specifies the RF attenuation value.
        /// 
        /// Valid Values:
        /// 0 to 70 dB (without the option electronic attenuator, step 10dB)
        /// 0 to 75 dB (with the option electronic attenuator, step 5dB)
        /// 
        /// Default Value: 0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ATTENUATION
        /// 
        /// Note:
        /// (1) This parameter has effect only if Auto Level is set Off.
        /// 
        /// (2) Not available for R&amp;S FMU instrument.
        /// </param>
        /// <param name="Ext_Att">
        /// Specifies the external attenuation or gain applied to the RF signal.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 0.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanRFSignalLevel(bool Auto_Level, double Auto_Level_Time, double Signal_Level, double RF_Att, double Ext_Att)
        {
            int pInvokeResult = PInvoke.ConfigureWlanRFSignalLevel(this._handle, System.Convert.ToUInt16(Auto_Level), Auto_Level_Time, Signal_Level, RF_Att, Ext_Att);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function enables the active marker on the specified trace.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_ENABLED
        /// RSSPECAN_ATTR_MARKER_TRACE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;[:STATe]
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:TRACe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects delta marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker_Enabled">
        /// Specifies whether the specified delta marker is enabled.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_ENABLED
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_TRACE
        /// 
        /// Note:
        /// (1) This control has effect only if Marker Enabled is set On.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWLANMarkerEnabled(int Window, int Marker, bool Marker_Enabled, int Trace)
        {
            int pInvokeResult = PInvoke.ConfigureWLANMarkerEnabled(this._handle, Window, Marker, System.Convert.ToUInt16(Marker_Enabled), Trace);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the marker position.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_POSITION
        /// RSSPECAN_ATTR_WLAN_MARK_Y
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:X
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:Y
        /// 
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// Sets the marker. The driver uses this value to set the repeated capability Marker of the attributes.
        /// 
        /// Valid Values:
        /// 1
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Position">
        /// Sets the position. The driver uses this value to select the attribute to be set.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WLAN_POS_X (0) - X
        /// RSSPECAN_VAL_WLAN_POS_Y (1) - Y
        /// 
        /// Default Value: RSSPECAN_VAL_WLAN_POS_X
        /// 
        /// </param>
        /// <param name="Value">
        /// Sets the marker position value.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value:
        /// 0.0
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_POSITION
        /// RSSPECAN_ATTR_WLAN_MARK_Y
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanMarkPosition(int Window, int Marker, int Position, double Value)
        {
            int pInvokeResult = PInvoke.ConfigureWlanMarkPosition(this._handle, Window, Marker, Position, Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function positions the selected marker to the indicated symbol/carrier.
        /// 
        /// 
        /// Note:
        /// (1) This function is valid only for the following result displays:
        /// constellation Vs symbol
        /// constellation Vs carrier
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_MARK_SYMB
        /// RSSPECAN_ATTR_WLAN_MARK_CARR
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:SYMBol
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:CARRier
        /// 
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// Sets the marker. The driver uses this value to set the repeated capability Marker of the attributes.
        /// 
        /// Valid Values:
        /// 1
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Type">
        /// Sets the type. The driver uses this value to select the attribute to be set.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WLAN_CONS_SYM  (0) - Symbol
        /// RSSPECAN_VAL_WLAN_CONS_CARR (1) - Carrier
        /// 
        /// Default Value: RSSPECAN_VAL_WLAN_CONS_SYM
        /// 
        /// </param>
        /// <param name="Position">
        /// Sets the marker position value.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value:
        /// 1
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_MARK_SYMB
        /// RSSPECAN_ATTR_WLAN_MARK_CARR
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanMarkerPositionConstellation(int Window, int Marker, int Type, int Position)
        {
            int pInvokeResult = PInvoke.ConfigureWlanMarkerPositionConstellation(this._handle, Window, Marker, Type, Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the ratio to be zoomed around the selected marker in the selected measurement window.
        /// 
        /// Note:
        /// (1) This function is only available for the following result displays:
        /// 
        ///   Constellation versus Carrier
        ///   Constellation versus Symbol
        ///   PVT Full Burst
        ///   PVT Rising / Falling
        ///   Magnitude Capture Buffer
        /// 
        /// and for FSQ-K92 option is available in folowing results too:
        ///   EVM versus Carrier
        ///   Frequency error versus Preamble
        ///   Phase error versus Preamble
        ///   Spectrum Flatness
        ///   Spectrum Flatness Difference
        ///   Spectrum Group Delay
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_ZOOM
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:FUNCtion:ZOOM
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// Sets the marker. The driver uses this value to set the repeated capability Marker of the attributes.
        /// 
        /// Valid Values:
        /// 1
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Zoom_Ratio">
        /// This control defines the ratio to be zoomed around the selected marker in the selected measurement window.
        /// 
        /// Valid Values: 0.0 % to 100.0 %
        /// 
        /// Default Value: 100.0 %
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_ZOOM
        /// 
        /// Note:
        /// 
        /// The default value is 100.0 %, where the full trace is shown.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanMarkerZoom(int Window, int Marker, double Zoom_Ratio)
        {
            int pInvokeResult = PInvoke.ConfigureWlanMarkerZoom(this._handle, Window, Marker, Zoom_Ratio);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function positions the selected marker to the indicated symbol in the indicated burst (Constellation vs Symbol and EVM vs Symbol).
        /// 
        /// Note:
        /// (1) This function is only valid for the following 802.11b measurements:
        /// 
        ///        Constellation vs Symbol 
        ///        EVM vs Symbol
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:BSYMbol
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 2
        /// </param>
        /// <param name="Marker">
        /// Selects the marker to configure.
        /// 
        /// Valid Values:
        /// 1 
        /// 
        /// Default Value: 1
        /// 
        /// Note:
        /// 
        /// (1) Currently there is only one marker available and it is always on.
        /// </param>
        /// <param name="Burst_Number">
        /// Specifies burst, where the specified marker will be set.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0
        /// 
        /// </param>
        /// <param name="Symbol_Number">
        /// Specifies symbol number in specified burst.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanMarkerBurstSymbol(int Window, int Marker, int Burst_Number, int Symbol_Number)
        {
            int pInvokeResult = PInvoke.ConfigureWlanMarkerBurstSymbol(this._handle, Window, Marker, Burst_Number, Symbol_Number);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function set the selected marker to the min/max peak in the current trace.
        /// 
        /// Note:
        /// (1) This function is only available for the Spectrum Flatness result display.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_SEARCH_MIN
        /// RSSPECAN_ATTR_MARKER_SEARCH_PEAK
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:MINimum
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:MAXimum
        /// 
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 2 to 2
        /// 
        /// Default Value: 2
        /// </param>
        /// <param name="Marker">
        /// Sets the marker. The driver uses this value to set the repeated capability MArker of the attributes.
        /// 
        /// Valid Values:
        /// 1
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Position">
        /// Sets min/max peak value. The driver uses this value to select the attribute to be set.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MARKER_SEARCH_HIGHEST (1) - Max
        /// RSSPECAN_VAL_MARKER_SEARCH_MINIMUM (2) - Min
        /// 
        /// Default Value: RSSPECAN_VAL_MARKER_SEARCH_MINIMUM
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_SEARCH_MIN
        /// RSSPECAN_ATTR_MARKER_SEARCH_PEAK
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SetWlanMarkerPeak(int Window, int Marker, int Position)
        {
            int pInvokeResult = PInvoke.SetWlanMarkerPeak(this._handle, Window, Marker, Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function turns off all the markers in selected measurement window.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_AOFF
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:AOFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int WlanDisableAllMarkers(int Window)
        {
            int pInvokeResult = PInvoke.WlanDisableAllMarkers(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the demodulation parameters. 
        /// 
        /// Note:
        /// (1) This function is not available for IEEE 802.11b and IEEE 802.11g (PLCP) standard.
        /// (2) This function is not available for FS-K92 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_SIGS
        /// RSSPECAN_ATTR_WLAN_BCON_AUTO
        /// RSSPECAN_ATTR_WLAN_BAN
        /// RSSPECAN_ATTR_WLAN_BTYP
        /// RSSPECAN_ATTR_WLAN_SYM_EQU
        /// RSSPECAN_ATTR_WLAN_SYM
        /// RSSPECAN_ATTR_WLAN_CEST
        /// 
        /// Remote-control command(s):
        /// [SENSe:]DEMod:FORMat:SIGSymbol ON | OFF
        /// [SENSe:]DEMod:FORMat[:BCONtent]:AUTo ON | OFF
        /// [SENSe:]DEMod:FORMat:BANalyze
        /// [SENSe:]DEMod:FORMat:BANalyze:BTYPe
        /// [SENSe:]DEMod:FORMat:BANalyze:SYMbols:EQUal ON | OFF
        /// [SENSe:]DEMod:FORMat:BANalyze:SYMbols:MIN
        /// [SENSe:]DEMod:FORMat:BANalyze:SYMbols:MAX
        /// [SENSe:]DEMod:CESTimation
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Use_Signal_Field_Content">
        /// Sets the signal symbol field.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_SIGS
        /// 
        /// Note:
        /// (1) If Auto Demodulation is set On, this control has no effect. Signal Field Content is set On automatically.
        /// 
        /// </param>
        /// <param name="Burst_Type">
        /// This control specifies the type of burst to be analyzed. The driver uses this value to set the RSSPECAN_ATTR_WLAN_BTYP attribute.
        /// 
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_WLAN_BTYP_DIRECT (0) - Direct Link Burst
        /// RSSPECAN_VAL_WLAN_BTYP_LOFDM  (1) - Long DSSS OFDM
        /// RSSPECAN_VAL_WLAN_BTYP_SOFDM  (2) - Short DSSS OFDM
        /// 
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_BTYP
        /// 
        /// Notes:
        /// 
        /// (1) Values 
        ///      Long DSSS OFDM
        ///      Long DSSS OFDM
        /// are valid only for IEEE 802.11g standard
        /// 
        /// (6) The R&amp;S FSQ-K91 option will only analyze DIRECT bursts
        /// </param>
        /// <param name="Auto_Demodulation">
        /// If this parameter sets the burst to analyze auto demodulation state.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_BCON_AUTO
        /// 
        /// Note:
        /// (1) This parameter has effect only if Use Signal Field Content is set On. 
        /// 
        /// </param>
        /// <param name="Demodulator">
        /// This control is used to specify the modulation to be applied to the measured data.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WLAN_BPSK    (0)  - BPSK (K90)
        /// RSSPECAN_VAL_WLAN_QPSK    (1)  - QPSK (K90)
        /// RSSPECAN_VAL_WLAN_QAM16   (2)  - QAM16 (K90)
        /// RSSPECAN_VAL_WLAN_QAM64   (3)  - QAM64 (K90)
        /// RSSPECAN_VAL_WLAN_BPSK6   (4)  - BPSK6
        /// RSSPECAN_VAL_WLAN_BPSK9   (5)  - BPSK9
        /// RSSPECAN_VAL_WLAN_QPSK12  (6)  - QPSK12
        /// RSSPECAN_VAL_WLAN_QPSK18  (7)  - QPSK18
        /// RSSPECAN_VAL_WLAN_QAM1624 (8)  - QAM1624
        /// RSSPECAN_VAL_WLAN_QAM1636 (9)  - QAM1636
        /// RSSPECAN_VAL_WLAN_QAM6448 (10) - QAM6448
        /// RSSPECAN_VAL_WLAN_QAM6454 (11) - QAM6454
        /// RSSPECAN_VAL_WLAN_DBPSK   (12) - DBPSK
        /// RSSPECAN_VAL_WLAN_DQPSK   (13) - DQPSK
        /// RSSPECAN_VAL_WLAN_CCK55   (14) - CCK55
        /// RSSPECAN_VAL_WLAN_CCK11   (15) - CCK11
        /// RSSPECAN_VAL_WLAN_PBCC55  (16) - PBCC55 
        /// RSSPECAN_VAL_WLAN_PBCC11  (17) - PBCC11
        /// RSSPECAN_VAL_WLAN_PBCC22  (18) - PBCC22
        /// RSSPECAN_VAL_WLAN_BPSK3   (19) - BPSK3
        /// RSSPECAN_VAL_WLAN_BPSK45  (20) - BPSK45
        /// RSSPECAN_VAL_WLAN_QPSK6   (21) - QPSK6
        /// RSSPECAN_VAL_WLAN_QPSK9   (22) - QPSK9
        /// RSSPECAN_VAL_WLAN_QAM1612 (23) - QAM1612
        /// RSSPECAN_VAL_WLAN_QAM1618 (24) - QAM1618
        /// RSSPECAN_VAL_WLAN_QAM6424 (25) - QAM6424
        /// RSSPECAN_VAL_WLAN_QAM6427 (26) - QAM6427
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_WLAN_BPSK6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_BAN
        /// 
        /// Notes:
        /// (1) Values RSSPECAN_VAL_WLAN_BPSK6 to RSSPECAN_VAL_WLAN_QAM6454 are valid for standards IEEE 802.11a, g (OFDM), j (20 MHz)
        /// 
        /// BPSK6 - BI-Phase shift keying at 6 Mbps
        /// BPSK9 - BI-Phase shift keying at 9 Mbps
        /// QPSK12 - Quadrature phase shift keying at 12 Mbps
        /// QPSK18 - Quadrature phase shift keying at 18 Mbps
        /// QAM1624 - Quadrature Amplitude Modulation at 24 Mbps
        /// QAM1636 - Quadrature Amplitude Modulation at 36 Mbps
        /// QAM6448 - Quadrature Amplitude Modulation at 48 Mbps
        /// QAM6454 - Quadrature Amplitude Modulation at 54 Mbps
        /// 
        /// (2) Values RSSPECAN_VAL_WLAN_BPSK3 to RSSPECAN_VAL_WLAN_QAM6427 are valid for standard IEEE 802.11j (10 MHz) 
        /// 
        /// BPSK3 - BI-Phase shift keying at 3 Mbps
        /// BPSK45 - BI-Phase shift keying at 6 Mbps
        /// QPSK6 - Quadrature phase shift keying at 6 Mbps
        /// QPSK9 - Quadrature phase shift keying at 9 Mbps
        /// QAM1612 - Quadrature Amplitude Modulation at 12 Mbps
        /// QAM1618 - Quadrature Amplitude Modulation at 18 Mbps
        /// QAM6424 - Quadrature Amplitude Modulation at 24 Mbps
        /// QAM6427 - Quadrature Amplitude Modulation at 27 Mbps
        /// 
        /// (3) Values RSSPECAN_VAL_WLAN_DBPSK to RSSPECAN_VAL_WLAN_PBCC22 are valid for IEEE 802.11g standard
        /// 
        /// DBPSK - Differential BI-Phase shift keying
        /// DQPSK - Differential Quadrature phase shift keying
        /// CCK55 - Complementary Code Keying at 5.5 Mbps
        /// CCK11 - Complementary Code Keying at 11 Mbps
        /// PBCC55 - PBCC at 5.5 Mbps
        /// PBCC11 - PBCC at 11 Mbps
        /// PBCC22 - PBCC at 11 Mbps
        /// 
        /// (4) This parameter has effect only if Auto Demodulation is set Off or Use Signal Field Content is set Off.
        /// 
        /// 
        /// </param>
        /// <param name="Channel_Estimation">
        /// Specifies defines whether channel estimation will be done in the preamble and payload.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_CEST
        /// </param>
        /// <param name="Equal_Burst_Length">
        /// Enables/disables equal burst length to analyze.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_SYM_EQU
        /// 
        /// </param>
        /// <param name="Min_Number_of_Data_Symbols">
        /// Specifies the number of data symbols required for bursts to qualify for measurement analysis.
        /// 
        /// Valid Values:
        /// 1 to 1366
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_SYM_MIN
        /// 
        /// Note:
        /// (1) If Equal Burst Length is set ON, this value sets number of data symbol for equal operation.
        /// 
        /// 
        /// </param>
        /// <param name="Max_Number_of_Data_Symbols">
        /// Specifies the maximum number of data symbols required for bursts to qualify for measurement analysis.
        /// 
        /// Valid Values:
        /// 1 to 1366
        /// 
        /// Default Value: 64
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_SYM_MAX
        /// 
        /// Note:
        /// (1) If Equal Burst Length is set ON, this value has no effect.
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanDemodulation(bool Use_Signal_Field_Content, int Burst_Type, bool Auto_Demodulation, int Demodulator, bool Channel_Estimation, bool Equal_Burst_Length, int Min_Number_of_Data_Symbols, int Max_Number_of_Data_Symbols)
        {
            int pInvokeResult = PInvoke.ConfigureWlanDemodulation(this._handle, System.Convert.ToUInt16(Use_Signal_Field_Content), Burst_Type, System.Convert.ToUInt16(Auto_Demodulation), Demodulator, System.Convert.ToUInt16(Channel_Estimation), System.Convert.ToUInt16(Equal_Burst_Length), Min_Number_of_Data_Symbols, Max_Number_of_Data_Symbols);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the demodulation parameters for PLCP Burst Type demodulation. 
        /// 
        /// Note:
        /// (1)  This function is valid only if standard IEEE 802.11b or 
        ///     IEEE 802.11g (PLCP) is selected.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_SIGS
        /// RSSPECAN_ATTR_WLAN_BCON_AUTO
        /// RSSPECAN_ATTR_WLAN_BAN
        /// RSSPECAN_ATTR_WLAN_BTYP
        /// RSSPECAN_ATTR_WLAN_DBYT_EQU
        /// RSSPECAN_ATTR_WLAN_DUR_EQU
        /// RSSPECAN_ATTR_WLAN_DBYT
        /// RSSPECAN_ATTR_WLAN_DUR
        /// 
        /// Remote-control command(s):
        /// [SENSe:]DEMod:FORMat:SIGSymbol ON | OFF
        /// [SENSe:]DEMod:FORMat[:BCONtent]:AUTo ON | OFF
        /// [SENSe:]DEMod:FORMat:BANalyze
        /// [SENSe:]DEMod:FORMat:BANalyze:BTYPe
        /// [SENSe:]DEMod:FORMat:BANalyze:DBYTes:EQUal
        /// [SENSe:]DEMod:FORMat:BANalyze:DURation:EQUal
        /// [SENSe:]DEMod:FORMat:BANalyze:DBYTes:MIN
        /// [SENSe:]DEMod:FORMat:BANalyze:DURation:MIN
        /// [SENSe:]DEMod:FORMat:BANalyze:DBYTes:MAX
        /// [SENSe:]DEMod:FORMat:BANalyze:DURation:MAX
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Use_PLCP_Header_Content">
        /// Sets the signal symbol field.
        /// 
        /// Note:
        /// (1) If Auto Demodulation is set On, this control has no effect. Signal Field Content is set On automatically.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_SIGS
        /// </param>
        /// <param name="Burst_Type">
        /// This control specifies the type of burst to be analyzed.
        /// 
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_WLAN_BTYP_LONG  (3) - Long PLCP Burst
        /// RSSPECAN_VAL_WLAN_BTYP_SHORT (4) - Short PLCP Burst
        /// 
        /// Default Value: RSSPECAN_VAL_WLAN_BTYP_LONG
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_BTYP
        /// </param>
        /// <param name="Auto_Demodulation">
        /// If this parameter sets the burst to analyze auto demodulation state.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_BCON_AUTO
        /// 
        /// Note:
        /// (1) This parameter has effect only if Use PLCP Header Content is set On.
        /// 
        /// </param>
        /// <param name="Demodulator">
        /// This control is used to specify the modulation to be applied to the measured data.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WLAN_DBPSK   (16) - DBPSK
        /// RSSPECAN_VAL_WLAN_DQPSK   (17) - DQPSK
        /// RSSPECAN_VAL_WLAN_CCK55   (18) - CCK55
        /// RSSPECAN_VAL_WLAN_CCK11   (19) - CCK11
        /// RSSPECAN_VAL_WLAN_PBCC55  (20) - PBCC55 
        /// RSSPECAN_VAL_WLAN_PBCC11  (21) - PBCC11
        /// RSSPECAN_VAL_WLAN_PBCC22  (22) - PBCC22
        /// 
        /// Default Value: RSSPECAN_VAL_WLAN_DBPSK
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_BAN
        /// 
        /// Notes:
        /// (1) 
        /// 
        /// DBPSK - Differential BI-Phase shift keying
        /// DQPSK - Differential Quadrature phase shift keying
        /// CCK55 - Complementary Code Keying at 5.5 Mbps
        /// CCK11 - Complementary Code Keying at 11 Mbps
        /// PBCC55 - PBCC at 5.5 Mbps
        /// PBCC11 - PBCC at 11 Mbps
        /// 
        /// (2) This parameter has effect only if Auto Demodulation is set Off or Use PLCP Header Content is set Off.
        /// 
        /// 
        /// 
        /// </param>
        /// <param name="Length_Type">
        /// Specifies the units for min/max payload length. The driver uses this value to select the attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WLAN_DEM_DUR_MS   (0) - Miliseconds
        /// RSSPECAN_VAL_WLAN_DEM_DUR_BYTE (1) - Bytes
        /// 
        /// Default Value: RSSPECAN_VAL_WLAN_DEM_DUR_BYTE
        /// 
        /// </param>
        /// <param name="Equal_Burst_Length">
        /// Enables/disables equal burst length to analyze.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_DBYT_EQU
        /// RSSPECAN_ATTR_WLAN_DUR_EQU
        /// </param>
        /// <param name="Min_Payload_Length">
        /// Specifies the duration in microseconds or bytes required for bursts to qualify for measurement analysis.The driver uses this value to set the RSSPECAN_ATTR_WLAN_DBYT_MIN or RSSPECAN_ATTR_WLAN_DUR_MIN attribute.
        /// 
        /// Valid Values:
        /// 1 to 4095 for bytes
        /// 1 to 32760 for microseconds
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_DBYT_MIN
        /// RSSPECAN_ATTR_WLAN_DUR_MIN
        /// 
        /// Note:
        /// (1) If Equal Burst Length is set ON, this value sets number of microseconds for equal operation.
        /// 
        /// 
        /// </param>
        /// <param name="Max_Payload_Length">
        /// Specifies the maximum duration in microseconds or bytes required for bursts to qualify for measurement analysis.
        /// 
        /// Valid Values:
        /// 1 to 4095 for bytes
        /// 1 to 32760 for microseconds
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_DBYT_MAX 
        /// RSSPECAN_ATTR_WLAN_DUR_MAX
        /// 
        /// Note:
        /// (1) If Equal Burst Length is set ON, this value has no effect.
        /// 
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanPLCPDemodulation(bool Use_PLCP_Header_Content, int Burst_Type, bool Auto_Demodulation, int Demodulator, int Length_Type, bool Equal_Burst_Length, int Min_Payload_Length, int Max_Payload_Length)
        {
            int pInvokeResult = PInvoke.ConfigureWlanPLCPDemodulation(this._handle, System.Convert.ToUInt16(Use_PLCP_Header_Content), Burst_Type, System.Convert.ToUInt16(Auto_Demodulation), Demodulator, Length_Type, System.Convert.ToUInt16(Equal_Burst_Length), Min_Payload_Length, Max_Payload_Length);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the tracking parameters. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_TRAC_PHAS
        /// RSSPECAN_ATTR_WLAN_TRAC_TIME
        /// RSSPECAN_ATTR_WLAN_TRAC_LEV
        /// 
        /// 
        /// Remote-control command(s):
        /// [SENSe:]TRACking:PHASe ON | OFF
        /// [SENSe:]TRACking:TIME ON | OFF
        /// [SENSe:]TRACking:LEVel ON | OFF
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Phase">
        /// Specifies defines whether or not the measurement results should be compensated for phase.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_TRAC_PHAS
        /// </param>
        /// <param name="Timing">
        /// Specifies defineswhether or not the measurement results should be compensated for time.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_TRAC_TIME
        /// 
        /// </param>
        /// <param name="Level">
        /// Specifies defines whether or not the measurement results should be compensated for level.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_TRAC_LEV
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanTracking(bool Phase, bool Timing, bool Level)
        {
            int pInvokeResult = PInvoke.ConfigureWlanTracking(this._handle, System.Convert.ToUInt16(Phase), System.Convert.ToUInt16(Timing), System.Convert.ToUInt16(Level));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the TX and RX filters. The names of the filters correspond to the file names returned by RSSPECAN_ATTR_WLAN_FILT_CAT attribute.
        /// 
        /// Note:
        /// (1) This function is available only for IEEE 802.11b &amp; g standards
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// [SENSe:]DEMod:FILTer:MODulation
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="TX_Filter_Name">
        /// This parameter selects the TX filters name. The name of the filter correspond to the file names returned by rsspecan_GetWlanFilterCatalog function.
        /// 
        /// Valid Values: any filter name
        /// 
        /// Default Value: "AUTO"
        /// 
        /// </param>
        /// <param name="RX_Filter_Name">
        /// This parameter selects the RX filters name. The name of the filter correspond to the file names returned by rsspecan_GetWlanFilterCatalog function.
        /// 
        /// Valid Values: any filter name
        /// 
        /// Default Value: "AUTO"
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanSelectFilter(string TX_Filter_Name, string RX_Filter_Name)
        {
            int pInvokeResult = PInvoke.ConfigureWlanSelectFilter(this._handle, TX_Filter_Name, RX_Filter_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads the names of all available filters.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// [SENSe:]DEMod:FILTer:CATalog?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the string you specify for the catalog list parameter.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 512
        /// 
        /// </param>
        /// <param name="Filter_List">
        /// Returns the names of all available filters. The file names are output without file extension.
        /// 
        /// Syntax of output format: filter_1,filter_2, .. ,filter_n
        /// 
        /// </param>
        /// <param name="Returned_Length">
        /// Returns the length of filter string readed from instrument.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetWlanFilterCatalog(int Buffer_Size, System.Text.StringBuilder Filter_List, out int Returned_Length)
        {
            int pInvokeResult = PInvoke.GetWlanFilterCatalog(this._handle, Buffer_Size, Filter_List, out Returned_Length);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the FFT start offset.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_FFT_START_OFFSET
        /// 
        /// Remote-control command(s):
        /// [SENSe:]DEMod:FFT:OFFSet AUTO | GICenter
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="FFT_Start_Offset">
        /// This control specifies the FFT start offset.
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_WLAN_FFT_START_OFFSET_AUTO (0) - Auto
        /// RSSPECAN_VAL_WLAN_FFT_START_OFFSET_GIC  (1) - Guard Interval Center
        /// 
        /// Default Value: RSSPECAN_VAL_WLAN_FFT_START_OFFSET_AUTO (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_FFT_START_OFFSET
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWLANFFTStartOffset(int FFT_Start_Offset)
        {
            int pInvokeResult = PInvoke.ConfigureWLANFFTStartOffset(this._handle, FFT_Start_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the display parameters. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_DISP_TABL
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TABLe ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Display_Table">
        /// This parameter selects whether the results table is displayed. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_DISP_TABL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanDisplay(bool Display_Table)
        {
            int pInvokeResult = PInvoke.ConfigureWlanDisplay(this._handle, System.Convert.ToUInt16(Display_Table));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the limit.
        /// 
        /// Note:
        /// (1) This function is not valid for standard IEEE 802.11b 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_FERR
        /// RSSPECAN_ATTR_WLAN_SYMB
        /// RSSPECAN_ATTR_WLAN_IQOF
        /// RSSPECAN_ATTR_WLAN_EVM_ALL
        /// RSSPECAN_ATTR_WLAN_EVM_DATA
        /// RSSPECAN_ATTR_WLAN_EVM_PIL
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:FERRor[:AVERage]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:FERRor:MAXimum
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:SYMBolerror[:AVERage]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:SYMBolerror:MAXimum
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:IQOFfset[:AVERage]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:IQOFfset:MAXimum
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:EVM:ALL[:AVERage]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:EVM:ALL:MAXimum
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:EVM:DATA[:AVERage]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:EVM:DATA:MAXimum
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:EVM:PILot[:AVERage]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:EVM:PILot:MAXimum
        /// 
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// Sets the limit. The driver uses this value to set the repeated capability Limit of the attributes.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Limit_Type">
        /// Sets the limit type. The driver uses this value to select the attribute to be set.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WLAN_LIM_FERR     (0) - Frequency Error
        /// RSSPECAN_VAL_WLAN_LIM_SYMB     (1) - Symbol Error
        /// RSSPECAN_VAL_WLAN_LIM_IQOFF    (2) - IQ Offset Error
        /// RSSPECAN_VAL_WLAN_LIM_EVM_ALL  (3) - Combined EVM
        /// RSSPECAN_VAL_WLAN_LIM_EVM_DATA (4) - Data Carrier EVM
        /// RSSPECAN_VAL_WLAN_LIM_EVM_PIL  (5) - Pilot Carrier EVM
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_WLAN_LIM_FERR
        /// 
        /// </param>
        /// <param name="Mode">
        /// Sets the limit measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MAX  (1) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MAX
        /// 
        /// 
        /// </param>
        /// <param name="Value">
        /// Sets the limit value.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value:
        /// 0.0
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_FERR
        /// RSSPECAN_ATTR_WLAN_SYMB
        /// RSSPECAN_ATTR_WLAN_IQOF
        /// RSSPECAN_ATTR_WLAN_EVM_ALL
        /// RSSPECAN_ATTR_WLAN_EVM_DATA
        /// RSSPECAN_ATTR_WLAN_EVM_PIL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanLimit(int Window, int Limit, int Limit_Type, int Mode, double Value)
        {
            int pInvokeResult = PInvoke.ConfigureWlanLimit(this._handle, Window, Limit, Limit_Type, Mode, Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets all limits.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:ALL &lt;numeric value&gt;, ...
        /// 
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// Sets the limit. The driver uses this value to set the repeated capability Limit of the attributes.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// This control sets the size of array in control Values.
        /// 
        /// Valid Values:
        /// &gt;0
        /// 
        /// Default Value: 12
        /// </param>
        /// <param name="Values">
        /// Sets the array of limit values.
        /// 
        /// Values:
        /// for WLAN:
        /// &lt;average frequency error&gt;
        /// &lt;max frequency error&gt;
        /// &lt;average symbol error&gt;
        /// &lt;max symbol error&gt;
        /// &lt;average IQ offset&gt;
        /// &lt;maximum IQ offset&gt;
        /// &lt;average EVM all bursts&gt;
        /// &lt;max EVM all bursts&gt;
        /// &lt;average EVM data carriers&gt;
        /// &lt;max EVM data carriers&gt;
        /// &lt;average EVM pilots&gt;
        /// &lt;max EVM pilots&gt;
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWLANAllLimits(int Window, int Limit, int Array_Size, double[] Values)
        {
            int pInvokeResult = PInvoke.ConfigureWLANAllLimits(this._handle, Window, Limit, Array_Size, Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the limit.
        /// 
        /// Note:
        /// (1)  This function is valid only if standard IEEE 802.11b or 
        ///     IEEE 802.11g (PLCP) is selected.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_TRIS
        /// RSSPECAN_ATTR_WLAN_TFAL
        /// RSSPECAN_ATTR_WLAN_EVM
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:TRISe[:AVERage]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:TRISe:MAXimum
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:TFALl[:AVERage]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:TFALl:MAXimum
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:EVM[:AVERage]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:EVM:MAXimum
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// Sets the limit. The driver uses this value to set the repeated capability Limit of the attributes.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Limit_Type">
        /// Sets the limit type. The driver uses this value to select the attribute to be set.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WLAN_LIM_RTIME    (6) - Rise Time
        /// RSSPECAN_VAL_WLAN_LIM_FTIME    (7) - Fall Time
        /// RSSPECAN_VAL_WLAN_LIM_EVM      (8) - EVM
        /// 
        /// Default Value: RSSPECAN_VAL_WLAN_LIM_RTIME
        /// 
        /// </param>
        /// <param name="Mode">
        /// Sets the limit measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MAX  (1) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MAX
        /// 
        /// 
        /// </param>
        /// <param name="Value">
        /// Sets the limit value.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value:
        /// 0.0
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_TRIS
        /// RSSPECAN_ATTR_WLAN_TFAL
        /// RSSPECAN_ATTR_WLAN_EVM
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanPLCPLimit(int Window, int Limit, int Limit_Type, int Mode, double Value)
        {
            int pInvokeResult = PInvoke.ConfigureWlanPLCPLimit(this._handle, Window, Limit, Limit_Type, Mode, Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the ACP channel limit.
        /// 
        /// Note:
        /// (1) This function is available only for IEEE 802.11j standard.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_ACP_ADJ
        /// RSSPECAN_ATTR_WLAN_ACP_ALT
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:ACPower:ACHannel
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:ACPower:ALTernate
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// Sets the limit. The driver uses this value to set the repeated capability Limit of the attributes.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Channel_Type">
        /// Sets the ACP channel type. The driver uses this value to select the attribute to be set.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WLAN_ACP_ADJ     (0) - Adjacent
        /// RSSPECAN_VAL_WLAN_ACP_ALT     (1) - Alternate
        /// 
        /// Default Value: RSSPECAN_VAL_WLAN_ACP_ADJ
        /// 
        /// </param>
        /// <param name="Value">
        /// Returns the ACP limit value.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetWlanACPChannelLimit(int Window, int Limit, int Channel_Type, out double Value)
        {
            int pInvokeResult = PInvoke.GetWlanACPChannelLimit(this._handle, Window, Limit, Channel_Type, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the value at the maximum overstepping of the spectrum mask limits.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_SPEC_MASK_X
        /// RSSPECAN_ATTR_WLAN_SPEC_MASK_Y
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:SPECtrum:MASK:CHECk:X
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:SPECtrum:MASK:CHECk:Y
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// Sets the limit. The driver uses this value to set the repeated capability Limit of the attributes.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Channel_Type">
        /// Sets the value type. The driver uses this value to select the attribute to be set.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FREQ (0) - Frequency
        /// RSSPECAN_VAL_POW  (1) - Power
        /// 
        /// Default Value: RSSPECAN_VAL_FREQ
        /// 
        /// </param>
        /// <param name="Value">
        /// Returns the value.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetWlanSpectrumMaxOverstepping(int Window, int Limit, int Channel_Type, out double Value)
        {
            int pInvokeResult = PInvoke.GetWlanSpectrumMaxOverstepping(this._handle, Window, Limit, Channel_Type, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns all limits.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:ALL
        /// 
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// Sets the limit. The driver uses this value to set the repeated capability Limit of the attributes.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Values">
        /// Returns the array of limit values.
        /// 
        /// Note:
        /// (1) The array size has to be at least 12
        /// 
        /// Returned Values:
        /// for WLAN:
        /// &lt;average frequency error&gt;
        /// &lt;max frequency error&gt;
        /// &lt;average symbol error&gt;
        /// &lt;max symbol error&gt;
        /// &lt;average IQ offset&gt;
        /// &lt;maximum IQ offset&gt;
        /// &lt;average EVM all bursts&gt;
        /// &lt;max EVM all bursts&gt;
        /// &lt;average EVM data carriers&gt;
        /// &lt;max EVM data carriers&gt;
        /// &lt;average EVM pilots&gt;
        /// &lt;max EVM pilots&gt;
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetWlanAllLimits(int Window, int Limit, double[] Values)
        {
            int pInvokeResult = PInvoke.GetWlanAllLimits(this._handle, Window, Limit, Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the measurement parameters. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SWEEP_TIME
        /// RSSPECAN_ATTR_WLAN_BURS_COUN_STAT
        /// RSSPECAN_ATTR_WLAN_BURS_COUN
        /// RSSPECAN_ATTR_NUMBER_OF_SWEEPS
        /// RSSPECAN_ATTR_WLAN_SWAP_IQ
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:TIME
        /// [SENSe:]BURSt:COUNt:STATe ON | OFF
        /// [SENSe:]BURSt:COUNt
        /// [SENSe&lt;1|2&gt;:]SWEep:COUNt
        /// [SENSe:]SWAPiq ON|OFF
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Overall_Burst_Count">
        /// Specifies whether a specified number of bursts is to be captured and analyzed..
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_BURS_COUN_STAT
        /// </param>
        /// <param name="No_of_Burst_to_Analyze">
        /// Sets defines the number of bursts that will be analyzed by the measurement.
        /// 
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_BURS_COUN
        /// 
        /// Note:
        /// (1) This parameter is ignored, if Overall Burst Count is set to Off.
        /// 
        /// 
        /// </param>
        /// <param name="Capture_Time">
        /// Sets the sweep acquisition period (capture time) for which the input signal is to be measured.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 5.0e-3 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SWEEP_TIME
        /// </param>
        /// <param name="Swap_I_Q">
        /// Defines whether or not the recorded IQ pairs should be swapped (I/Q) before being processed. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_SWAP_IQ
        /// </param>
        /// <param name="Sweep_Count">
        /// Specifies the number of sweeps that should be performed for Spectrum ACP/ACPR and Spectrum Mask measurements. 
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NUMBER_OF_SWEEPS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanMeas(bool Overall_Burst_Count, int No_of_Burst_to_Analyze, double Capture_Time, bool Swap_I_Q, int Sweep_Count)
        {
            int pInvokeResult = PInvoke.ConfigureWlanMeas(this._handle, System.Convert.ToUInt16(Overall_Burst_Count), No_of_Burst_to_Analyze, Capture_Time, System.Convert.ToUInt16(Swap_I_Q), Sweep_Count);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets measurement type to be Power Versus Time and configures measurement parameters.
        /// 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_PVT
        /// RSSPECAN_ATTR_WLAN_PVT_SEL
        /// 
        /// Remote-control command(s):
        /// CONFigure:BURSt:PVT[:IMMediate]
        /// CONFigure:BURSt:PVT:SELect EDGE | FULL
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Rising_Falling">
        /// Sets the power versus time measurement results interpreting.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_PVT_EDGE (0) - Both
        /// RSSPECAN_VAL_PVT_FULL (1) - Full
        /// 
        /// Default Value: RSSPECAN_VAL_PVT_EDGE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_PVT_SEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanPVTMeas(int Rising_Falling)
        {
            int pInvokeResult = PInvoke.ConfigureWlanPVTMeas(this._handle, Rising_Falling);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets measurement type to be Power Versus Time and configures measurement parameters for IEEE802.11b/g (PLCP) standard. 
        /// 
        /// Note:
        /// (1) This function is valid only if standard IEEE 802.11b or 
        ///     IEEE 802.11g (PLCP) is selected.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_PVT_RPOW
        /// RSSPECAN_ATTR_WLAN_PVT_SEL
        /// RSSPECAN_ATTR_WLAN_PVT_AVER
        /// RSSPECAN_ATTR_WLAN_PVT
        /// 
        /// Remote-control command(s):
        /// CONFigure:BURSt:PVT:RPOWer MEAN | MAX
        /// CONFigure:BURSt:PVT:SELect EDGE | RISE | FALL
        /// CONFigure:BURSt:PVT:AVERage
        /// CONFigure:BURSt:PVT[:IMMediate]
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Reference_Power">
        /// Sets the burst power as the reference power.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_PVT_MEAN (0) - Mean
        /// RSSPECAN_VAL_PVT_MAX  (1) - Max
        /// 
        /// Default Value: RSSPECAN_VAL_PVT_MEAN (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_PVT_RPOW
        /// </param>
        /// <param name="Ramp">
        /// Sets the power versus time measurement results interpreting.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_PVT_EDGE (0) - Both
        /// RSSPECAN_VAL_PVT_RISE (2) - Up
        /// RSSPECAN_VAL_PVT_FALL (3) - Down
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_PVT_EDGE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_PVT_SEL
        /// </param>
        /// <param name="Average_Length">
        /// Specifies measurement type to set the burst power averaging length to the desired value.
        /// 
        /// Valid Values:
        /// 1 to 399
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_PVT_AVER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanPLCPPVT(int Reference_Power, int Ramp, int Average_Length)
        {
            int pInvokeResult = PInvoke.ConfigureWlanPLCPPVT(this._handle, Reference_Power, Ramp, Average_Length);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets measurement type to be EVM.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_EVM_ECAR
        /// RSSPECAN_ATTR_WLAN_EVM_ESYM
        /// RSSPECAN_ATTR_WLAN_PRE
        /// RSSPECAN_ATTR_WLAN_PRE_SEL
        /// RSSPECAN_ATTR_WLAN_DISP_AUTO_Y
        /// RSSPECAN_ATTR_WLAN_Y_PDIV
        /// RSSPECAN_ATTR_WLAN_UNIT_EVM
        /// RSSPECAN_ATTR_WLAN_UNIT_PRE
        /// 
        /// Remote-control command(s):
        /// CONFigure:BURSt:EVM:ECARrier[:IMMediate]
        /// CONFigure:BURSt:EVM:ESYMbol[:IMMediate]
        /// CONFigure:BURSt:PREamble[:IMMediate]
        /// CONFigure:BURSt:PREamble:SELect PHAS | FREQ
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:AUTO
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:PDIVision
        /// UNIT:EVM DB | PCT
        /// UNIT:PREamble HZ | PCT
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Mode">
        /// Sets the EVT measurement mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WLAN_EVM_MODE_SYMBOL   (0) - EVM Vs Symbol
        /// RSSPECAN_VAL_WLAN_EVM_MODE_CARRIER  (1) - EVM Vs Carrier
        /// RSSPECAN_VAL_WLAN_EVM_MODE_PHAS (2) - Phase Vs Preamble
        /// RSSPECAN_VAL_WLAN_EVM_MODE_FREQ (3) - Frequency Vs Preamble
        /// 
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_WLAN_EVM_MODE_SYMBOL
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_EVM_ECAR 
        /// RSSPECAN_ATTR_WLAN_EVM_ESYM 
        /// RSSPECAN_ATTR_WLAN_PRE
        /// </param>
        /// <param name="Auto_Scaling">
        /// This parameter switches on or off automatic scaling of the Y-axis for the specified trace display.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_DISP_AUTO_Y
        /// </param>
        /// <param name="Per_Division">
        /// This parameter sets the size of each Y scale division for the specified trace display.
        /// 
        /// Note:
        /// (1) This attribute has no affect if automatic scaling of the Y-axis is enabled. 
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 3.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_Y_PDIV
        /// </param>
        /// <param name="Unit">
        /// This parameter specifies the units for EVM trace results.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DB  (8)  - dB 
        /// RSSPECAN_VAL_UNIT_HZ  (12) - Hz
        /// RSSPECAN_VAL_UNIT_PCT (13) - %
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_PCT (13)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_UNIT_EVM
        /// 
        /// Note:
        /// (1) Unit dB is valid only for Mode EVM vs Carrier and EVM vs Symbols
        /// 
        /// (2) Mode Phase Vs Preamble or Frequency Vs Preamble this control has no effect.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanEVMMeasurement(int Mode, bool Auto_Scaling, double Per_Division, int Unit)
        {
            int pInvokeResult = PInvoke.ConfigureWlanEVMMeasurement(this._handle, Mode, System.Convert.ToUInt16(Auto_Scaling), Per_Division, Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets measurement type to be Spectrum.
        /// 
        /// Note:
        /// (1) This function is not valid for FSQ-K92 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_SPEC_FLAT
        /// RSSPECAN_ATTR_WLAN_SPEC_MASK
        /// RSSPECAN_ATTR_WLAN_SPEC_MASK_SEL
        /// RSSPECAN_ATTR_WLAN_SPEC_FFT
        /// RSSPECAN_ATTR_WLAN_SPEC_ACPR
        /// 
        /// Remote-control command(s):
        /// CONFigure:BURSt:SPECtrum:FLATness[:IMMediate]
        /// CONFigure:BURSt:SPECtrum:MASK[:IMMediate]
        /// CONFigure:BURSt:SPECtrum:MASK:SELect IEEE | ETSI
        /// CONFigure:BURSt:SPECtrum:FFT[:IMMediate]
        /// CONFigure:BURSt:SPECtrum:ACPR[:IMMediate]
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Mode">
        /// Sets the spectrum measurement mode. The driver uses this value to set the RSSPECAN_ATTR_WLAN_SPEC_FLAT or RSSPECAN_ATTR_WLAN_SPEC_MASK and RSSPECAN_ATTR_WLAN_SPEC_MASK_SEL or RSSPECAN_ATTR_WLAN_SPEC_FFT or RSSPECAN_ATTR_WLAN_SPEC_ACPR attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WLAN_SPEC_MODE_IEEE  (0) - IEEE
        /// RSSPECAN_VAL_WLAN_SPEC_MODE_ETSI  (1) - ETSI
        /// RSSPECAN_VAL_WLAN_SPEC_MODE_FFT   (2) - FFT
        /// RSSPECAN_VAL_WLAN_SPEC_MODE_ACPR  (3) - ACPR
        /// RSSPECAN_VAL_WLAN_SPEC_MODE_FLAT  (4) - Flatness
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_WLAN_SPEC_MODE_FLAT
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_SPEC_FLAT
        /// RSSPECAN_ATTR_WLAN_SPEC_MASK
        /// RSSPECAN_ATTR_WLAN_SPEC_MASK_SEL
        /// RSSPECAN_ATTR_WLAN_SPEC_FFT
        /// RSSPECAN_ATTR_WLAN_SPEC_ACPR
        /// 
        /// Note:
        /// (1) Values RSSPECAN_VAL_WLAN_SPEC_MODE_IEEE and RSSPECAN_VAL_WLAN_SPEC_MODE_ETSI are available only for standard IEEE 802.11a
        /// 
        /// 
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanSpectrumMeasurement(int Mode)
        {
            int pInvokeResult = PInvoke.ConfigureWlanSpectrumMeasurement(this._handle, Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Constellation measurement.
        /// 
        /// Note:
        /// (1) This function is not valid for IEEE 802.11b stanadard.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_CONS_CSYM
        /// RSSPECAN_ATTR_WLAN_CONS_CCAR
        /// RSSPECAN_ATTR_WLAN_CONS_CARR_SEL
        /// RSSPECAN_ATTR_WLAN_CONS_CARR_SEL_NR
        /// 
        /// Remote-control command(s):
        /// CONFigure:BURSt:CONStellation:CSYMbol[:IMMediate]
        /// CONFigure:BURSt:CONStellation:CCARrier[:IMMediate]
        /// CONFigure:BURSt:CONStellation:CARRier:SELect
        /// CONFigure:BURSt:CONStellation:CARRier:SELect nr
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Constellation_Mode">
        /// Sets the measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WLAN_CONS_SYM  (0) - Constellation vs. Symbol
        /// RSSPECAN_VAL_WLAN_CONS_CARR (1) - Constellation vs. Carrier
        /// 
        /// Default Value: RSSPECAN_VAL_WLAN_CONS_SYM
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_CONS_CCAR
        /// RSSPECAN_ATTR_WLAN_CONS_CSYM
        /// </param>
        /// <param name="Carrier_Selection">
        /// Defines the result calculation of the selected carrier.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SELECTION_ALL (0) - All
        /// RSSPECAN_VAL_SELECTION_PIL (1) - Pilots
        /// RSSPECAN_VAL_SELECTION_NUM (2) - Carrier Number
        /// 
        /// Default Value: RSSPECAN_VAL_SELECTION_NUM
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_CONS_CARR_SEL RSSPECAN_ATTR_WLAN_CONS_CARR_SEL_NUM
        /// 
        /// Note:
        /// (1) This parameter is valid only if Constellation vs. Symbol is selected
        /// </param>
        /// <param name="Carrier_Number">
        /// Defines the carrier number.
        /// 
        /// Valid Values:
        /// -26 to 26 exclude 0
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_CONS_CARR_SEL_NUM
        /// 
        /// Note:
        /// (1) This parameter is valid only if Carrier Number is selected in Carrier Selection control
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanConstellationMeasurement(int Constellation_Mode, int Carrier_Selection, int Carrier_Number)
        {
            int pInvokeResult = PInvoke.ConfigureWlanConstellationMeasurement(this._handle, Constellation_Mode, Carrier_Selection, Carrier_Number);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Constellation measurement.
        /// 
        /// Note:
        /// (1)  This function is valid only if standard IEEE 802.11b or 
        ///     IEEE 802.11g (PLCP) is selected.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_CONS_CSYM
        /// 
        /// Remote-control command(s):
        /// CONFigure:BURSt:CONStellation:CSYMbol[:IMMediate]
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanPLCPConstellation()
        {
            int pInvokeResult = PInvoke.ConfigureWlanPLCPConstellation(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the statistics measurement.
        /// 
        /// Note:
        /// (1) This function is available only for option FS-K90 or FS-K91
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_STAT_CCDF
        /// RSSPECAN_ATTR_WLAN_STAT_BSTR
        /// RSSPECAN_ATTR_WLAN_STAT_SFI
        /// 
        /// Remote-control command(s):
        /// CONFigure:BURSt:STATistics:CCDF[:IMMediate]
        /// CONFigure:BURSt:STATistics:BSTReam[:IMMediate]
        /// CONFigure:BURSt:STATistics:SFIeld[:IMMediate]
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Mode">
        /// Sets the measurement type. The driver uses this value to selects the attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WLAN_STAT_CCDF (0) - CCDF
        /// RSSPECAN_VAL_WLAN_STAT_BSTR (1) - Bit Stream
        /// RSSPECAN_VAL_WLAN_STAT_SFI  (2) - Signal Field
        /// 
        /// Default Value: RSSPECAN_VAL_WLAN_STAT_CCDF
        /// 
        /// Note:
        /// (1) Value RSSPECAN_VAL_WLAN_STAT_SFI is not available for option FS-K92
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanStatisticsMeasurement(int Mode)
        {
            int pInvokeResult = PInvoke.ConfigureWlanStatisticsMeasurement(this._handle, Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the IQ Trace parameters. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_SRAT
        /// RSSPECAN_ATTR_WLAN_FLAT
        /// 
        /// Remote-control command(s):
        /// TRACE:IQ:SRATe
        /// TRACE:IQ:FILTer:FLATness
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Sample_Rate">
        /// Sets the sampling rate for the I/Q data acquisition.
        /// 
        /// Valid Values:
        /// 10.0e3 Hz to 100.0e6 Hz (326.4e6 Hz with option R&amp;S FSQ-B72)
        /// 2.0e+6 to 32.0e+6 Hz for FS-K92 option
        /// 
        /// Default Value: 32.0e6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_SRAT
        /// </param>
        /// <param name="Filter">
        /// This control determines whether the wideband detector filters are too be used in preference to the wideband detector board, (B72 option), for a specified sample rate.
        /// 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_IQ_FILT_NORM (0) - Normal 
        /// RSSPECAN_VAL_IQ_FILT_WIDE (1) - Wide
        /// 
        /// Default Value: RSSPECAN_VAL_IQ_FILT_WIDE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_FLAT
        /// 
        /// Note(s):
        /// (1) This value has effect only if option B72 is installed. Otherwise it is ignored.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanIQMeasurementParameters(double Sample_Rate, int Filter)
        {
            int pInvokeResult = PInvoke.ConfigureWlanIQMeasurementParameters(this._handle, Sample_Rate, Filter);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Spectrum Emission Mask measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_SEM_TYPE
        /// RSSPECAN_ATTR_WLAN_SEM_FILE
        /// RSSPECAN_ATTR_WLAN_SEM_CLASS
        /// 
        /// Remote-control command(s):
        /// [SENSe:]POWer:SEM IEEE | ETSI | USER
        /// MMEMory:LOAD:SEM:STATe
        /// [SENSe:]POWer:SEM:CLASs
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Type">
        /// Sets the Spectrum Emission Mask (SEM) measurement type. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SEM_IEEE (0) - IEEE
        /// RSSPECAN_VAL_SEM_ETSI (1) - ETSI
        /// RSSPECAN_VAL_SEM_USER (2) - User
        /// 
        /// Default Value: RSSPECAN_VAL_SEM_IEEE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_SEM_TYPE
        /// </param>
        /// <param name="File_Name">
        /// This control is used to load a K91 spectrum emission mask setup from an xml file.
        /// 
        /// Valid Values:
        /// any valid file name
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_SEM_FILE
        /// 
        /// Note:
        /// 
        /// (1) This control has effect only if Type is set to User.
        /// 
        /// </param>
        /// <param name="Power_Class">
        /// This control sets the Spectrum Emission Mask (SEM) power class index. The index represents the power classes to be applied. The index is directly related to the entries displayed in the power class drop down combo box, within the SEM settings configuration page.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_SEM_CLASS
        /// 
        /// Note:
        /// 
        /// (1) Class 0 sets the Power Class to automatic
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWlanSEMMeasurement(int Type, string File_Name, int Power_Class)
        {
            int pInvokeResult = PInvoke.ConfigureWlanSEMMeasurement(this._handle, Type, File_Name, Power_Class);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the result values of the adjacent channel power measurement.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:FUNCtion:POWer:RESult[:CURRent]?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:FUNCtion:POWer:RESult:MAXHold?
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// Sets the marker. The driver uses this value to set the repeated capability Marker of the attributes.
        /// 
        /// Valid Values:
        /// 1
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Mode">
        /// Sets the ACP measurement results. The driver uses this value to select the attribute to be set.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ACP_CURR    (0) - Current
        /// RSSPECAN_VAL_ACP_MAX     (1) - Maximum
        /// 
        /// Default Value: RSSPECAN_VAL_ACP_CURR
        /// 
        /// </param>
        /// <param name="Values">
        /// Returns the measured values.
        /// 
        /// Returned Value:
        /// values i dB in order
        /// 
        /// Power of main channel
        /// Power of lower adjacent channel
        /// Power of upper adjacent channel
        /// Power of lower alternate adjacent channel 1
        /// Power of upper alternate adjacent channel 1
        /// Power of lower alternate adjacent channel 2
        /// Power of upper alternate adjacent channel 2
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetWlanACPMeasurement(int Window, int Marker, int Mode, double[] Values)
        {
            int pInvokeResult = PInvoke.GetWlanACPMeasurement(this._handle, Window, Marker, Mode, Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function loads the IQ data from the specified .iqw file.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_IQ_LOAD
        /// 
        /// Remote-control command(s):
        /// MMEMory:LOAD:IQ:STATe
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="File_Name">
        /// Specifies full path of the file, the IQ data are load from.
        /// 
        /// Valid Values:
        /// any valid file name
        /// 
        /// Default Value: "d:\\user\\data.iqw"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_IQ_LOAD
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int LoadWlanIQData(string File_Name)
        {
            int pInvokeResult = PInvoke.LoadWlanIQData(this._handle, File_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function stores the IQ data from/to the specified .iqw file.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_IQ_SAVE
        /// 
        /// Remote-control command(s):
        /// MMEMory:STORe:IQ:STATe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="File_Name">
        /// Specifies full path of the file, where IQ data are stored.
        /// 
        /// Valid Values:
        /// any valid file name
        /// 
        /// Default Value: "d:\\user\\data.iqw"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_IQ_SAVE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int StoreWlanIQData(string File_Name)
        {
            int pInvokeResult = PInvoke.StoreWlanIQData(this._handle, File_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function forces the IQ measurement results to be recalculated according to the current settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_CALC_BURS
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:BURSt[:IMMediate]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int WlanBurstImm()
        {
            int pInvokeResult = PInvoke.WlanBurstImm(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns all the measured data that relates to the currently selected measurement type. The returned data is particular to the currently selected measurement type and is specified in the Trace Data parameter description.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRACe[:DATA]? TRACE1 | TRACE2 | TRACE3 | TRACE4 | TRACE5 | TRACE6
        /// FORMat[:DATA] REAL,32
        /// *CLS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Trace">
        /// This parameter indicates which trace will be read out.
        /// 
        /// Valid Values:
        /// 1 to 6
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Array_Size">
        /// Number of array points requested.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 256
        /// </param>
        /// <param name="Trace_Data">
        /// Returns an array of trace data. Number and format of the measurement values for operating mode Wireless LAN (WLAN):
        /// 
        /// --- IQ Measurements:
        /// 
        /// There are a number of measurements that can be performed in IQ mode. No data will be returned for any of the following measurements, should it be requested, until such time as a measurement belonging to the IQ group has been previously run. Running a frequency sweep measurement for example, Spectrum Mask, will not generate results for this measurement group.
        /// 
        /// --- Constellation vs Symbol:
        /// 
        /// This measurement represents I and Q data. For IEEE 802.11a and j 
        /// the data will be returned as a repeating array of interleaved I and Q data in groups of selected carriers, until all the data is exhausted. For IEEE 802.11b the data will be returned as a repeating array of interleaved I and Q data in symbol order until all the data is exhausted. Each I and Q point will be returned in floating point format. TRACE1 is used for this measurement results.
        /// 
        /// IEEE 802.11a &amp; j only:
        /// 
        /// If "All Carriers" is selected, it will return 52 per of I and Q data per symbol. If "Pilots Only" is selected, it will return 4 per of I and Q per symbol in the following order: Carrier -21, Carrier -7, Carrier 7, Carrier 21. If a single carrier is selected, it will return 1 per of I and Q data per symbol.
        /// 
        /// --- Constellation vs Carrier:
        /// 
        /// This measurement represents I and Q data. Data will be returned as a repeating array of interleaved I and Q data in groups of 53 channels including the channel 0, until all the data is exhausted. Each I and Q point will be returned in floating point format. TRACE1 is used for this measurement results.
        /// 
        /// --- Power vs Time - Full Burst and Rising / falling data:
        /// 
        /// Both measurement results are once again simply slightly different views of the same results data. All fully complete bursts within the capture time are analyzed into three master bursts. The three master bursts relate to the minimum, maximum and average values across all complete bursts. This data is returned in dBm values to the user on a per sample basis. Each sample will in some way relate to an analyzis of each corresponding sample within each processed burst.
        /// 
        /// The type of PVT data returned will be determined by the TRACE number passed as an argument to the SCPI command, in addition to the graphic type that is selected. If the graphic type selected is full burst, then the return data is as follows.
        /// 
        ///   TRACE1 - full burst, minimum burst data values.
        ///   TRACE2 - full burst, mean burst data values.
        ///   TRACE3 - full burst, maximum burst data values.
        /// 
        /// If the graphic type selected is EDGe, then the return data is as follows.
        /// 
        ///   TRACE1 - rising edge, minimum burst data values.
        ///   TRACE2 - rising edge, mean burst data values.
        ///   TRACE3 - rising edge, maximum burst data values.
        ///   TRACE4 - falling edge, minimum burst data values.
        ///   TRACE5 - falling edge, mean burst data values.
        ///   TRACE6 - falling edge, maximum burst data values.
        /// 
        /// For IEEE 802.11b:
        /// 
        /// If the graphic type selected is RISing or FALLing then only 3 traces are available (1 to 3) and represent the minimum, mean and maximum bursts data for the respective graph selection. The number of samples returned during full burst analysis will depend on the modulation type and will typically be 5000. The number of samples returned when the rising and falling graphic type is selected will be less than what is returned for full burst and will be approximately 400 samples. The samples will be returned in floating point format as a single sequence of comma delimited values.
        /// 
        /// --- Spectrum Flatness:
        /// 
        /// There are four separate traces that are available with this measurement. Trace data for a particular trace will only be returnable by querying the appropriate trace Spectrum flatness provides two basic graph types. These are an absolute power value graph (ABS) and a relative group delay graph, which are both plotted on a per carrier basis. All 52 carriers are drawn, in addition to the unused 0 carrier. Both the absolute power and group delay graph groups will allow all the data points to be returned as one trace and an average of all the channels as the other trace. E.g. the return data will either be one single group of 53 carriers if the average trace is selected or a repeating group of 53 carriers if all the data is requested.
        /// 
        ///   TRACE1 - ABS (all analyzed trains)
        ///   TRACE2 - Group Delay (all analyzed trains)
        ///   TRACE3 - ABS (average trace)
        ///   TRACE4 - Group Delay (average trace)
        /// 
        /// Absolute power results are returned in dB and group delay results are returned in ns.
        /// 
        /// --- Spectrum FFT:
        /// 
        /// All FFT points will be returned if the data for this measurement is requested. This will be an exhaustive call, due to the fact that there are nearly always more FFT points than IQ samples. The number of FFT points is the number presented by a power of 2 that is higher than the total number of samples. E.g. if there were 20000 samples, then 32768 FFT points would be returned. Data will be returned in floating point format in dBm. TRACE1 is used for this measurement results.
        /// 
        /// --- Statistics Bitstream data:
        /// 
        /// IEEE 802.11a &amp; j only:
        /// 
        /// Data will be returned in repeating groups of 52 data channels where each symbol value will be represented by an integer value within one byte. Channel 0 is unused and will therefore not have any data associated with it, with no return data being provided. The number of repeating groups that are returned will be equal to the number of measured symbols. Currently, 64QAM has the highest data rate and it contains symbol values up to 63, making the proposal of one byte sufficient in size to represent all symbol data values, regardless of the modulation type in use. Data will be returned in ascii printable hexadecimal character format. TRACE1 is used for this measurement results.
        /// 
        /// IEEE 802.11b only:
        /// 
        /// Data will be returned in burst order. Each burst will be represented as a series of bytes. For each burst the first 9 or 18 bytes represents the PLCP Preamble for Short and Long burst types respectively. The next 6 bytes represents the PLCP Header. The remaining bytes represent the PSDU. Data will be returned in ascii printable hexadecimal character format. TRACE1 is used for this measurement results.
        /// 
        /// --- Statistics CCDF - Complementary cumulative distribution function:
        /// 
        /// Up to a maximum of 201 data points will be returned in addition to a data count value. The first value in the return data will represent the quantity of probability values that follow. Each of the potential 201 data points will be returned as probability value and will represent the total number of samples that are equal to or exceed the corresponding power level. Probability data will be returned up to the power level that contains at least one sample. It is highly unlikely that the full 201 data values will ever be returned. Each probability value will be returned as a floating point number, with a value less than 1.
        /// 
        /// --- Statistics Signal Field data:
        /// 
        /// IEEE 802.11a &amp; j only Data will be returned as an array of hexidecimal values, with each hexidecimal value representing the 24 bit long signal field for a single burst.
        /// 
        /// IEEE 802.11b only Data will be returned as an array of hexidecimal values, with each hexidecimal value representing the 48 bit long signal field for a single burst.
        /// 
        /// --- EVM vs Carrier:
        /// 
        /// Two trace types are provided with this measurement. There is an average EVM value for each of the 53 channels or a repeating group of EVM values for each channel. The number of repeating groups will correspond to the number of fully analyzed trains. Each EVM value will be returned as a floating point number, expressed in units of dBm.
        /// 
        ///   TRACE1 - Average EVM values per channel
        ///   TRACE2 - All EVM values per channel for each full train of the
        ///            capture period
        /// 
        /// --- EVM vs Symbol:
        /// 
        /// Three traces types are available with this measurement. The basic trace types show either the minimum, mean or maximum EVM value, as measured over the complete capture period. The number of repeating groups that are returned will be equal to the number of measured symbols. Each EVM value will be returned as a floating point number, expressed in units of dBm.
        /// 
        /// IEEE 802.11a &amp; j only:
        /// 
        ///   TRACE1 - Minimum EVM values
        ///   TRACE2 - Mean EVM values
        ///   TRACE3 - Maximum EVM values
        /// 
        /// IEEE 802.11b only:
        /// 
        ///   TRACE1 - EVM IEEE values
        ///   TRACE2 - EVM Direct values
        /// 
        /// --- Frequency Sweep Measurements:
        /// 
        /// No data will be returned for this type of measurement, should it be requested, until such time as a measurement has been previously run. Running an IQ measurement will not generate results for this type of measurement.
        /// 
        /// --- Spectrum Mask:
        /// 
        /// Result data will be returned as 625 trace points in floating point format. These trace points are obtained directly from the base system via the measurement API and the quantity is therefore a fixed value. Only an array of Y data will be returned. TRACE1 is used for this measurement results.
        /// 
        /// --- Spectrum ACPR:
        /// 
        /// Result data will be returned as 625 trace points in floating point format. These trace points are obtained directly from the base system via the measurement API and the quantity is therefore a fixed value. Only an array of Y data will be returned. TRACE1 is used for this measurement results.
        /// 
        /// --- Error Vs Preamble:
        /// Three traces types are available with this measurement. The basic trace types show either the
        /// minimum, mean or maximum frequency or phase value as measured over the preamble part of the
        /// burst.
        /// </param>
        /// <param name="No_of_Values">
        /// Returns the number of trace data values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadWlanTraceData(int Trace, int Array_Size, double[] Trace_Data, out int No_of_Values)
        {
            int pInvokeResult = PInvoke.ReadWlanTraceData(this._handle, Trace, Array_Size, Trace_Data, out No_of_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns all the IQ data associated with the measurement acquisition time. The result values are scaled linear in unit Volt and correspond to the voltage at the RF input of the instrument. The command returns a comma-separated list of the measured voltage values in floating point format (Comma Separated Values = CSV). The number of values returned is 2 * number of samples, the first half being the I values, the second half the Q values.
        /// 
        /// Notes:
        /// 
        /// (1) If there are not I/Q data available in memory because the corresponding measurement had not been started, the function will cause a Query Error.
        /// 
        /// Remote-control command(s):
        /// TRACe:IQ:DATA:MEMory?
        /// *CLS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Offset_Samples">
        /// This control sets offset of the values to be read related to the start of the acquired data.
        /// 
        /// Valid Values: 0 to &lt;# of samples&gt; - 1
        /// 
        /// Default Value: 0
        /// 
        /// Note:
        /// 
        /// Where &lt;# of samples&gt; is the value indicated with function Trace I/Q Set (rsspecan_TraceIQSet).
        /// 
        /// </param>
        /// <param name="No_of_Samples">
        /// This control sets the number of measurement values to be read.
        /// 
        /// Valid Values: 1 to &lt;# of samples&gt; - &lt;offset samples&gt;
        /// 
        /// Default Value: 1
        /// 
        /// Note:
        /// 
        /// Where &lt;# of samples&gt; is the value indicated with function Trace I/Q Set (rsspecan_TraceIQSet).
        /// 
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of doubles in the array you specify for the real parts and imaginary parameters.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the data contains more elemens than you specify with the Buffer Size parameter, the function returns Buffer Size elements into the arrays and returns the buffer size you must pass to get the all data.  For example, if the data contains 16000 elements for real parts and 16000 elements for imaginary part and the Buffer Size is 10000, the function places 10000 elements into the arrays and returns 16000.
        /// 
        /// Notes:
        /// 
        /// (1) If VI_ATTR_INTF_TYPE is equal to VI_INTF_ASRL, then use this parameter to pass the expected number of points to be read from the instrument. Data points will be read until instruments output buffer is empty. If Buffer Size is lower than actual number of results available in the instruments output buffer, then this function returns an error code (BFFC0002  Parameter 2 (Buffer Size) out of range.) and measurement results will not be valid. This is due to limitation if the unit is remote-controlled via the RS-232-C interface.
        /// 
        /// (2) This function requires that all response data are read out completely before the instrument accepts further commands.
        /// 
        /// </param>
        /// <param name="No_of_Points">
        /// Returns the number of I/Q trace data points.
        /// 
        /// </param>
        /// <param name="Real_Parts__I_">
        /// Returns an array of trace data (I-value x, real part) point. The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter.
        /// 
        /// </param>
        /// <param name="Imaginary_Parts__Q_">
        /// Returns an array of trace data (Q-value y, imaginary part) point. The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadWlanMemoryIQData(int Offset_Samples, int No_of_Samples, int Buffer_Size, out int No_of_Points, double[] Real_Parts__I_, double[] Imaginary_Parts__Q_)
        {
            int pInvokeResult = PInvoke.ReadWlanMemoryIQData(this._handle, Offset_Samples, No_of_Samples, Buffer_Size, out No_of_Points, Real_Parts__I_, Imaginary_Parts__Q_);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the Spectrum Emission Mask summary results.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:DATA? LIST
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="No_Of_Values">
        /// This control sets the number of values returned in arrays.
        /// The array size has to be at least equal to this number.
        /// 
        /// Valid Values: &gt; 0
        /// 
        /// Default Value: 10
        /// 
        /// </param>
        /// <param name="Index">
        /// Returns an array of index into table of results.
        /// 
        /// 
        /// </param>
        /// <param name="Start_Frequency_Band">
        /// Returns an array of start frequency band in Hz.
        /// 
        /// </param>
        /// <param name="Stop_Frequency_Band">
        /// Returns an array of stop frequency band in Hz.
        /// </param>
        /// <param name="Resolution_Bandwidth">
        /// Returns an array of resolution bandwidth
        /// </param>
        /// <param name="Limit_Fail_Frequency">
        /// Returns an array of limit fail frequency.
        /// 
        /// 
        /// </param>
        /// <param name="Power_Abs">
        /// Returns an array of absolute power in dBm.
        /// 
        /// </param>
        /// <param name="Power_Rel">
        /// Returns the relative power in dBc.
        /// </param>
        /// <param name="Limit_Distance">
        /// Returns an array of limit distance in dB.
        /// </param>
        /// <param name="Failure_Flag">
        /// Returns an array of failure flag (0 = PASSED, 1 = FAILED
        /// </param>
        /// <param name="Returned_Values">
        /// Returns number of value sets (frequency, level and delta to limit line) read (available to return) from the instrument. Number of values returned in arrays is limited with the parameter No of Values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadWlanSEMResults(int No_Of_Values, double[] Index, double[] Start_Frequency_Band, double[] Stop_Frequency_Band, double[] Resolution_Bandwidth, double[] Limit_Fail_Frequency, double[] Power_Abs, double[] Power_Rel, double[] Limit_Distance, double[] Failure_Flag, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.ReadWlanSEMResults(this._handle, No_Of_Values, Index, Start_Frequency_Band, Stop_Frequency_Band, Resolution_Bandwidth, Limit_Fail_Frequency, Power_Abs, Power_Rel, Limit_Distance, Failure_Flag, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns all the results.
        /// 
        /// Note:
        /// (1) If no measurement has been performed yet, a query error will occur.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:ALL?
        /// *CLS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Results">
        /// This control returns all the results. 
        /// 
        /// Results are returned in order:
        /// for FSQ-K91
        /// IEEE 802.11a, j, g (OFDM) &amp; Turbo Mode
        /// Idx 0 - preamble power   
        /// Idx 1 - payload power   
        /// Idx 2 - min rms power
        /// Idx 3 - average rms power
        /// Idx 4 - max rms power   
        /// Idx 5 - peak power   
        /// Idx 6 - min crest factor
        /// Idx 7 - average crest factor
        /// Idx 8 - max crest factor  
        /// Idx 9 - min frequency error
        /// Idx 10 - average frequency error
        /// Idx 11 - max frequency error    
        /// Idx 12 - min symbol error
        /// Idx 13 - average symbol error
        /// Idx 14 - max symbol error    
        /// Idx 15 - min IQ offset
        /// Idx 16 - average IQ offset
        /// Idx 17 - maximum IQ offset   
        /// Idx 18 - min gain imbalance
        /// Idx 19 - average gain imbalance
        /// Idx 20 - max gain imbalance   
        /// Idx 21 - min quadrature offset
        /// Idx 22 - average quadrature offset
        /// Idx 23 - max quadrature offset  
        /// Idx 24 - min EVM all bursts
        /// Idx 25 - average EVM all bursts
        /// Idx 26 - max EVM all bursts   
        /// Idx 27 - min EVM data carriers
        /// Idx 28 - average EVM data carriers 
        /// Idx 29 - max EVM data carriers  
        /// Idx 30 - min EVM pilots
        /// Idx 31 - average EVM pilots 
        /// Idx 32 - max EVM pilots
        /// 
        /// IEEE 802.11b &amp; g (CCK)
        /// Idx 0 - min rise time
        /// Idx 1 - average rise time
        /// Idx 2 - max rise time 
        /// Idx 3 - min fall time
        /// Idx 4 - average fall time
        /// Idx 5 - max fall time
        /// Idx 6 - min rms power
        /// Idx 7 - average rms power
        /// Idx 8 - max rms power
        /// Idx 9 - min peak power
        /// Idx 10 - average peak power
        /// Idx 11 - max peak power
        /// Idx 12 - min crest factor
        /// Idx 13 - average crest factor
        /// Idx 14 - max crest factor
        /// Idx 15 - min frequency error
        /// Idx 16 - average frequency error
        /// Idx 17 - max frequency error
        /// Idx 18 - min chip clock error
        /// Idx 19 - average chip clock error
        /// Idx 20 - max chip clock error
        /// Idx 21 - min phase error
        /// Idx 22 - average phase error
        /// Idx 23 - max phase error
        /// Idx 24 - min IQ offset
        /// Idx 25 - average IQ offset
        /// Idx 26 - maximum IQ offset
        /// Idx 27 - min gain imbalance
        /// Idx 28 - average gain imbalance
        /// Idx 29 - max gain imbalance
        /// Idx 30 - min quadrature offset
        /// Idx 31 - average quadrature offset
        /// Idx 32 - max quadrature offset
        /// Idx 33 - min EVM IEEE
        /// Idx 34 - average EVM IEEE
        /// Idx 35 - max EVM IEEE
        /// Idx 36 - min EVM Direct
        /// Idx 37 - average EVM Direct 
        /// Idx 38 - max EVM Direct 
        /// </param>
        /// <param name="Returned_Values">
        /// Returns the number of values in Results array.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchWlanBurstAll(double[] Results, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.FetchWlanBurstAll(this._handle, Results, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the power measurement value, measured during the measurement.
        /// 
        /// Note:
        /// (1) If no measurement has been performed yet, a query error will occur.
        /// 
        /// (2) This function is not available only for IEEE 802.11b standard
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_FETC_PRE
        /// RSSPECAN_ATTR_WLAN_FETC_PAYL
        /// RSSPECAN_ATTR_WLAN_FETC_RMS
        /// RSSPECAN_ATTR_WLAN_FETC_PEAK
        /// RSSPECAN_ATTR_WLAN_FETC_CRES
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:PREamble?
        /// FETCh:BURSt:PAYLoad?
        /// FETCh:BURSt:RMS:MINimum?
        /// FETCh:BURSt:RMS:MAXimum?
        /// FETCh:BURSt:RMS:AVERage?
        /// FETCh:BURSt:PEAK?
        /// FETCh:BURSt:CRESt:MINimum?
        /// FETCh:BURSt:CRESt:MAXimum?
        /// FETCh:BURSt:CRESt[:AVERage]?
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Power_Measurement">
        /// Sets the power measurement. The driver uses this value to select the attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WLAN_MEAS_PREA       (0) - Preamble
        /// RSSPECAN_VAL_WLAN_MEAS_PAYL       (1) - Payload
        /// RSSPECAN_VAL_WLAN_MEAS_RMS_MAX    (2) - Maximum RMS
        /// RSSPECAN_VAL_WLAN_MEAS_RMS_MIN    (3) - Minimum RMS
        /// RSSPECAN_VAL_WLAN_MEAS_RMS_AVER   (4) - Average RMS
        /// RSSPECAN_VAL_WLAN_MEAS_PEAK       (5) - Peak
        /// RSSPECAN_VAL_WLAN_MEAS_CREST_MAX  (6) - Maximum Crest Factor
        /// RSSPECAN_VAL_WLAN_MEAS_CREST_MIN  (7) - Minimum Crest Factor
        /// RSSPECAN_VAL_WLAN_MEAS_CREST_AVER (8) - Average Crest Factor
        /// 
        /// Default Value: RSSPECAN_VAL_WLAN_MEAS_PREA
        /// 
        /// 
        /// </param>
        /// <param name="Value">
        /// This control returns the selected power value .
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchWlanBurstPower(int Power_Measurement, out double Value)
        {
            int pInvokeResult = PInvoke.FetchWlanBurstPower(this._handle, Power_Measurement, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the burst rise/falling time.
        /// 
        /// Note:
        /// (1) If no measurement has been performed yet, a query error will occur.
        /// 
        /// (2) This function is available only for IEEE 802.11b standard
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_FETC_TRIS
        /// RSSPECAN_ATTR_WLAN_FETC_TFAL
        /// 
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:TRISe:AVERage?
        /// FETCh:BURSt:TRISe:MINimum?
        /// FETCh:BURSt:TRISe:MAXimum?
        /// FETCh:BURSt:TFALl:AVERage?
        /// FETCh:BURSt:TFALl:MINimum?
        /// FETCh:BURSt:TFALl:MAXimum?
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Time">
        /// Sets the time measurement. The driver uses this value to select the attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WLAN_TIME_FALL (0) - Fall
        /// RSSPECAN_VAL_WLAN_TIME_RISE (1) - Rise
        /// 
        /// Default Value: RSSPECAN_VAL_WLAN_TIME_FALL
        /// 
        /// 
        /// </param>
        /// <param name="Mode">
        /// Sets the measurement mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MAX
        /// 
        /// </param>
        /// <param name="Value">
        /// This control returns the selected power value .
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchWlanBurstTime(int Time, int Mode, out double Value)
        {
            int pInvokeResult = PInvoke.FetchWlanBurstTime(this._handle, Time, Mode, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the burst errors.
        /// 
        /// Note:
        /// (1) If no measurement has been performed yet, a query error will occur.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_FETC_FERR
        /// RSSPECAN_ATTR_WLAN_FETC_SYMB
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:FERRor:AVERage?
        /// FETCh:BURSt:FERRor:MINimum?
        /// FETCh:BURSt:FERRor:MAXimum?
        /// FETCh:BURSt:SYMBolerror:AVERage?
        /// FETCh:BURSt:SYMBolerror:MINimum?
        /// FETCh:BURSt:SYMBolerror:MAXimum?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Error_Type">
        /// Sets the power measurement. The driver uses this value to select teh attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WLAN_ERR_FREQ (0) - Frequency
        /// RSSPECAN_VAL_WLAN_ERR_SYMB (1) - Symbol
        /// 
        /// Default Value: RSSPECAN_VAL_WLAN_ERR_FREQ
        /// 
        /// 
        /// </param>
        /// <param name="Mode">
        /// Sets the measurement mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MAX
        /// 
        /// </param>
        /// <param name="Value">
        /// This control returns the selected power value .
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchWlanBurstError(int Error_Type, int Mode, out double Value)
        {
            int pInvokeResult = PInvoke.FetchWlanBurstError(this._handle, Error_Type, Mode, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the burst errors.
        /// 
        /// Note:
        /// (1) If no measurement has been performed yet, a query error will occur.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_FETC_IQOF
        /// RSSPECAN_ATTR_WLAN_FETC_GIMB
        /// RSSPECAN_ATTR_WLAN_FETC_QUAD
        /// 
        /// Remote-control command(s):
        /// *CLS
        /// FETCh:BURSt:IQOFfset:AVERage?
        /// FETCh:BURSt:IQOFfset:MAXimum?
        /// FETCh:BURSt:IQOFfset:MINimum?
        /// FETCh:BURSt:GIMBalance:AVERage?
        /// FETCh:BURSt:GIMBalance:MAXimum?
        /// FETCh:BURSt:GIMBalance:MINimum?
        /// FETCh:BURSt:QUADoffset:AVERage?
        /// FETCh:BURSt:QUADoffset:MAXimum?
        /// FETCh:BURSt:QUADoffset:MINimum?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="IQ_Impairment">
        /// Sets the IQ Impairment. The driver uses this value to select teh attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WLAN_IQ_OFFS (0) - IQ Offset
        /// RSSPECAN_VAL_WLAN_IQ_IMBA (1) - Gain Imbalance
        /// RSSPECAN_VAL_WLAN_IQ_QUAD (2) - Quadrature Offset
        /// 
        /// Default Value: RSSPECAN_VAL_WLAN_IQ_OFFS
        /// 
        /// Note:
        /// (1) Value RSSPECAN_VAL_WLAN_IQ_QUAD (2) - Quadrature Offset is valid only for IEEE 802.11a &amp; j standards.
        /// 
        /// 
        /// </param>
        /// <param name="Mode">
        /// Sets the measurement mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MAX
        /// 
        /// </param>
        /// <param name="Value">
        /// This control returns the selected power value .
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchWlanIQImp(int IQ_Impairment, int Mode, out double Value)
        {
            int pInvokeResult = PInvoke.FetchWlanIQImp(this._handle, IQ_Impairment, Mode, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the EVM measurement value.
        /// 
        /// Note:
        /// (1) If no measurement has been performed yet, a query error will occur.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_FETC_EVM_ALL
        /// RSSPECAN_ATTR_WLAN_FETC_EVM_DATA
        /// RSSPECAN_ATTR_WLAN_FETC_EVM_PILOT
        /// RSSPECAN_ATTR_WLAN_FETC_EVM_DIR
        /// RSSPECAN_ATTR_WLAN_FETC_EVM
        /// 
        /// Remote-control command(s):
        /// *CLS
        /// FETCh:BURSt:EVM:ALL:AVERage?
        /// FETCh:BURSt:EVM:ALL:MAXimum?
        /// FETCh:BURSt:EVM:ALL:MINimum?
        /// FETCh:BURSt:EVM:DATA:AVERage?
        /// FETCh:BURSt:EVM:DATA:MAXimum?
        /// FETCh:BURSt:EVM:DATA:MINimum?
        /// FETCh:BURSt:EVM:PILot:AVERage?
        /// FETCh:BURSt:EVM:PILot:MAXimum?
        /// FETCh:BURSt:EVM:PILot:MINimum?
        /// FETCh:BURSt:EVM:DIRect:AVERage?
        /// FETCh:BURSt:EVM:DIRect:MAXimum?
        /// FETCh:BURSt:EVM:DIRect:MINimum?
        /// FETCh:BURSt:EVM:[IEEE]:AVERage?
        /// FETCh:BURSt:EVM:[IEEE]:MAXimum?
        /// FETCh:BURSt:EVM:[IEEE]:MINimum?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="EVM">
        /// Sets the EVM measurement. The driver uses this value to select teh attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WLAN_EVM_ALL   (0) - All Carriers
        /// RSSPECAN_VAL_WLAN_EVM_DATA  (1) - Data Carriers
        /// RSSPECAN_VAL_WLAN_EVM_PILOT (2) - Pilot Carriers
        /// RSSPECAN_VAL_WLAN_EVM       (3) - Burst EVM
        /// RSSPECAN_VAL_WLAN_EVM_PEAK  (4) - Peak EVM (IEEE)
        /// 
        /// Default Value: RSSPECAN_VAL_WLAN_EVM_ALL
        /// 
        /// Note:
        /// (1) Values RSSPECAN_VAL_WLAN_EVM (3) and RSSPECAN_VAL_WLAN_EVM_PEAK (4) are valid only for IEEE 802.11b standard.
        /// 
        /// 
        /// </param>
        /// <param name="Mode">
        /// Sets the measurement mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MAX
        /// 
        /// </param>
        /// <param name="Value">
        /// This control returns the selected power value .
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchWlanEVM(int EVM, int Mode, out double Value)
        {
            int pInvokeResult = PInvoke.FetchWlanEVM(this._handle, EVM, Mode, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the count of analyzed burst or number of symbols of analyzed burst.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_FETC_BURS_COUNT
        /// RSSPECAN_ATTR_WLAN_FETC_SYMB_COUNT
        /// 
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:COUNt?
        /// FETCh:SYMBol:COUNt?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Type">
        /// Sets the count type. The driver uses this value to select the attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WLAN_BURS (0) - Burst
        /// RSSPECAN_VAL_WLAN_SYMB (1) - Symbol
        /// 
        /// Default Value: RSSPECAN_VAL_WLAN_BURS
        /// 
        /// 
        /// </param>
        /// <param name="Count">
        /// This control returns the count value.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchWlanCount(int Type, out int Count)
        {
            int pInvokeResult = PInvoke.FetchWlanCount(this._handle, Type, out Count);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the limit results
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_FERR_RES
        /// RSSPECAN_ATTR_WLAN_SYMB_RES
        /// RSSPECAN_ATTR_WLAN_IQOF_RES
        /// RSSPECAN_ATTR_WLAN_EVM_ALL_RES
        /// RSSPECAN_ATTR_WLAN_EVM_DATA_RES
        /// RSSPECAN_ATTR_WLAN_EVM_PIL_RES
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:FERRor[:AVERage]:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:FERRor:MAXimum:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:SYMBolerror[:AVERage]:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:SYMBolerror:MAXimum:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:IQOFfset[:AVERage]:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:IQOFfset:MAXimum:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:EVM:ALL[:AVERage]:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:EVM:ALL:MAXimum:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:EVM:DATA[:AVERage]:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:EVM:DATA:MAXimum:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:EVM:PILot[:AVERage]:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:EVM:PILot:MAXimum:RESult
        /// 
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// Sets the limit. The driver uses this value to set the repeated capability Limit of the attributes.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Limit_Type">
        /// Sets the limit type. The driver uses this value to select the attribute to be set.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WLAN_LIM_FERR     (0) - Frequency Error
        /// RSSPECAN_VAL_WLAN_LIM_SYMB     (1) - Symbol Error
        /// RSSPECAN_VAL_WLAN_LIM_IQOFF    (2) - IQ Offset Error
        /// RSSPECAN_VAL_WLAN_LIM_EVM_ALL  (3) - Combined EVM
        /// RSSPECAN_VAL_WLAN_LIM_EVM_DATA (4) - Data Carrier EVM
        /// RSSPECAN_VAL_WLAN_LIM_EVM_PIL  (5) - Pilot Carrier EVM
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_WLAN_LIM_FERR
        /// 
        /// Note:
        /// (1) Value RSSPECAN_VAL_WLAN_LIM_EVM_PIL is not valid for FS-K92 option.
        /// 
        /// </param>
        /// <param name="Mode">
        /// Sets the limit measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MAX
        /// 
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns the result.
        /// 
        /// Returned Value:
        /// 0 - Failed
        /// 1 - Passed
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetWlanLimitCheckResult(int Window, int Limit, int Limit_Type, int Mode, out int Result)
        {
            int pInvokeResult = PInvoke.GetWlanLimitCheckResult(this._handle, Window, Limit, Limit_Type, Mode, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the limit result
        /// 
        /// Note:
        /// (1)  This function is valid only if standard IEEE 802.11b or 
        ///     IEEE 802.11g (PLCP) is selected.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_TRIS_AVER_RES
        /// RSSPECAN_ATTR_WLAN_TRIS_MAX_RES
        /// RSSPECAN_ATTR_WLAN_TFAL_AVER_RES
        /// RSSPECAN_ATTR_WLAN_TFAL_MAX_RES
        /// RSSPECAN_ATTR_WLAN_EVM_AVER_RES
        /// RSSPECAN_ATTR_WLAN_EVM_MAX_RES
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:TRISe[:AVERage]:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:TRISe:MAXimum:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:TFALl[:AVERage]:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:TFALl:MAXimum:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:EVM[:AVERage]:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:EVM:MAXimum:RESult
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// Sets the limit. The driver uses this value to set the repeated capability Limit of the attributes.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Limit_Type">
        /// Sets the limit type. The driver uses this value to select the attribute to be set.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WLAN_LIM_RTIME    (6) - Rise Time
        /// RSSPECAN_VAL_WLAN_LIM_FTIME    (7) - Fall Time
        /// RSSPECAN_VAL_WLAN_LIM_EVM      (8) - EVM
        /// 
        /// Default Value: RSSPECAN_VAL_WLAN_LIM_RTIME
        /// 
        /// </param>
        /// <param name="Mode">
        /// Sets the limit measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MAX
        /// 
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns the result.
        /// 
        /// Returned Value:
        /// 0 - Failed
        /// 1 - Passed
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetWlanPLCPLimitCheckResult(int Window, int Limit, int Limit_Type, int Mode, out int Result)
        {
            int pInvokeResult = PInvoke.GetWlanPLCPLimitCheckResult(this._handle, Window, Limit, Limit_Type, Mode, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the ACP channel limit.
        /// 
        /// Note:
        /// (1) This function is available only for IEEE 802.11j standard.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_ACP_ADJ_RES
        /// RSSPECAN_ATTR_WLAN_ACP_ALT_RES
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:ACPower:ACHannel:RESult?
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:ACPower:ALTernate:RESult?
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// Sets the limit. The driver uses this value to set the repeated capability Limit of the attributes.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Channel_Type">
        /// Sets the ACP channel type. The driver uses this value to select the attribute to be set.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WLAN_ACP_ADJ (0) - Adjacent
        /// RSSPECAN_VAL_WLAN_ACP_ALT (1) - Alternate
        /// 
        /// Default Value: RSSPECAN_VAL_WLAN_ACP_ADJ
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns the result.
        /// 
        /// Returned Value:
        /// 0 - Failed
        /// 1 - Passed
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetWlanACPChannLimitCheckResult(int Window, int Limit, int Channel_Type, out int Result)
        {
            int pInvokeResult = PInvoke.GetWlanACPChannLimitCheckResult(this._handle, Window, Limit, Channel_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns all limit results.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:ALL:RESult?
        /// 
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// Sets the limit. The driver uses this value to set the repeated capability Limit of the attributes.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Results">
        /// Returns the array of limit resutls.
        /// 
        /// Note:
        /// (1) The array size has to be at least 12
        /// 
        /// Returned Values:
        /// 0 - Failed
        /// 1 - Passed
        /// 
        /// in order:
        /// 
        /// &lt;average frequency error&gt;
        /// &lt;max frequency error&gt;
        /// &lt;average symbol error&gt;
        /// &lt;max symbol error&gt;
        /// &lt;average IQ offset&gt;
        /// &lt;maximum IQ offset&gt;
        /// &lt;average EVM all bursts&gt;
        /// &lt;max EVM all bursts&gt;
        /// &lt;average EVM data carriers&gt;
        /// &lt;max EVM data carriers&gt;
        /// &lt;average EVM pilots&gt;
        /// &lt;max EVM pilots&gt;
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetWlanAllLimitCheckResults(int Window, int Limit, int[] Results)
        {
            int pInvokeResult = PInvoke.GetWlanAllLimitCheckResults(this._handle, Window, Limit, Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the result of the limit check of the limit line indicated in the selected measurement window.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WLAN_LIM_FAIL_RES
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:FAIL?
        /// 
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// Sets the limit. The driver uses this value to set the repeated capability Limit of the attributes.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Result">
        /// Return the of limit result.
        /// 
        /// Returned Values:
        /// 0 - Failed
        /// 1 - Passed
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetWlanLimitCheck(int Window, int Limit, out int Result)
        {
            int pInvokeResult = PInvoke.GetWlanLimitCheck(this._handle, Window, Limit, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the WiMAX analyzer mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument:SELect WIMax
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SetWiMAXMode()
        {
            int pInvokeResult = PInvoke.SetWiMAXMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Wireless LAN standard.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_STAN
        /// 
        /// Remote-control command(s):
        /// CONFigure:STANdard
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Standard">
        /// Sets the standard.
        /// 
        /// Valid Values:
        /// 0 - IEEE 802.16-2004 OFDM
        /// 1 - IEEE 802.16e-2005 OFDMA
        /// 2 - IEEE 802.16e-2005 WiBro
        /// 3 - IEEE 802.16e-2005 MIMO
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_STAN
        /// 
        /// Note(s):
        /// 
        /// (1) MIMO - only available with K94.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXStandard(int Standard)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXStandard(this._handle, Standard);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the signal characteristics.
        /// 
        /// Note:
        /// (1) This function is available only with option B25.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AMPL_EATT_STATE
        /// RSSPECAN_ATTR_AMPL_EATT_AUTO
        /// RSSPECAN_ATTR_AMPL_EATT_VALUE
        /// 
        /// Remote-control command(s):
        /// INPut:EATT:STATe
        /// INPut:EATT:AUTO ON | OFF
        /// INPut:EATT
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="State">
        /// This control switches the electronic input attenuation into the signal path (state ON) or removes it from the signal path (state OFF).
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value) 
        /// VI_TRUE  (1) - On 
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AMPL_EATT_STATE
        /// </param>
        /// <param name="Electronic_Attenuator_Auto">
        /// This control specifies, that instrument automatically couples the electronic input attenuation to the reference level and the attenuation of the mechanical attenuator (state ON) or switches the input attenuation to manual entry (state OFF).
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AMPL_EATT_AUTO
        /// 
        /// Note:
        /// (1) This parameter is valid only if State is set On.
        /// 
        /// </param>
        /// <param name="Manual__Value__dB_">
        /// This control sets the manual input electronic attenuation level. 
        /// 
        /// Valid Values: 0 to 30 dB
        /// 
        /// Default Value: 0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AMPL_EATT_STATE_VALUE
        /// 
        /// Note:
        /// 
        /// (1) The attenuation can be varied in 5 dB steps.
        /// 
        /// (2) This parameter is valid only if Attenuator Auto is set Off.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXElectronicAtt(bool State, bool Electronic_Attenuator_Auto, int Manual__Value__dB_)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXElectronicAtt(this._handle, System.Convert.ToUInt16(State), System.Convert.ToUInt16(Electronic_Attenuator_Auto), Manual__Value__dB_);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the YIG filter for image frequency suppression into the signal path (condition ON) and/or removes it from the signal path (condition OFF).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_AMPL_YIG_FILTER   
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:FILTer:YIG[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="YIG_Filter">
        /// Switches the YIG filter for image rejection into the signal path (ON) or removes it from the signal path (OFF).
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AMPL_YIG_FILTER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXYIGFilter(int Window, bool YIG_Filter)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXYIGFilter(this._handle, Window, System.Convert.ToUInt16(YIG_Filter));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies whether automatic setting of the YIG filter to the optimum setting is performed by the application (ON), or wether the YIG filter settings is set manually (OFF).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_YIG_AUTO
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:FILTer:YIG:AUTO
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="YIG_Filter_Auto">
        /// Switches the YIG filter for image rejection into the signal path (ON) or removes it from the signal path (OFF).
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_AMPL_YIG_FILTER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXYIGFilterAuto(int Window, bool YIG_Filter_Auto)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXYIGFilterAuto(this._handle, Window, System.Convert.ToUInt16(YIG_Filter_Auto));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the trigger parameters. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_TRIG_MODE
        /// RSSPECAN_ATTR_TRIGGER_DELAY
        /// RSSPECAN_ATTR_WIMAX_TRIG_POW_LEV_AUTO
        /// RSSPECAN_ATTR_WIMAX_TRIG_POW_LEV
        /// RSSPECAN_ATTR_EXTERNAL_TRIGGER_LEVEL
        /// 
        /// Remote-control command(s):
        /// TRIGger[:SEQuence]:MODE IMMediate | EXTernal | POWer
        /// TRIGger&lt;1|2&gt;[:SEQuence]:HOLDoff
        /// TRIGger[:SEQuence]:LEVel:POWer:AUTO
        /// TRIGger[:SEQuence]:LEVel:POWer
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel[:EXTernal]
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Mode">
        /// Sets the trigger mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TRIG_MODE_IMM (0) - Immediate
        /// RSSPECAN_VAL_TRIG_MODE_EXT (1) - External
        /// RSSPECAN_VAL_TRIG_MODE_POW (2) - Power 
        /// 
        /// Default Value: RSSPECAN_VAL_TRIG_MODE_IMM (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_TRIG_MODE
        /// </param>
        /// <param name="Offset">
        /// Defines the length of the trigger delay. A negative delay time (pretrigger) can be set.
        /// 
        /// Valid Values:
        /// -100.0 to 100.0
        /// 
        /// Default Value: 10.0e-6 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_DELAY
        /// 
        /// Note:
        /// (1) This parameter has effect only if Trigger Mode isn't Free Run
        /// 
        /// 
        /// </param>
        /// <param name="Auto_Trigger_Level">
        /// Specifies whether or not an automatic power trigger level calculation is performed before each main measurement.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_TRIG_POW_LEV_AUTO
        /// 
        /// Note:
        /// (1) This parameter has effect only for Trigger Mode set to Power
        /// 
        /// </param>
        /// <param name="Power_Trigger_Level">
        /// Specifies the level of the input signal for which triggering will occur. The value is specified in dBm for RF input and V for baseband inputs.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_TRIG_POW_LEV
        /// 
        /// 
        /// Note:
        /// (1) This parameter has effect only for Trigger Mode set to Power
        /// 
        /// 
        /// </param>
        /// <param name="Ext__Trigger_Level">
        /// Specifies the level of the external trigger input for which triggering will occur.
        /// 
        /// Valid Values:
        /// 500.0e-3 to 3.5 V
        /// 
        /// Default Value: 1.4 V
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_TRIGGER_LEVEL
        /// 
        /// Note:
        /// (1) This parameter has effect only for Trigger Mode set to External
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXTrigger(int Mode, double Offset, bool Auto_Trigger_Level, double Power_Trigger_Level, double Ext__Trigger_Level)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXTrigger(this._handle, Mode, Offset, System.Convert.ToUInt16(Auto_Trigger_Level), Power_Trigger_Level, Ext__Trigger_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the trigger parameters. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_EGAT
        /// RSSPECAN_ATTR_WIMAX_EGAT_HOLD_TIME
        /// RSSPECAN_ATTR_WIMAX_EGAT_HOLD_SAMP
        /// RSSPECAN_ATTR_WIMAX_EGAT_LENG_TIME
        /// RSSPECAN_ATTR_WIMAX_EGAT_LENG_SAMP
        /// RSSPECAN_ATTR_WIMAX_EGAT_LINK
        /// 
        /// Remote-control command(s):
        /// [SENSe:]SWEep:EGATe ON | OFF
        /// [SENSe:]SWEep:EGATe:HOLDoff[:TIMe]
        /// [SENSe:]SWEep:EGATe:HOLDoff:SAMPle
        /// [SENSe:]SWEep:EGATe:LENGth[:TIMe]
        /// [SENSe:]SWEep:EGATe:LENGth:SAMPle
        /// [SENSe:]SWEep:EGATe:LINK ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Gating">
        /// This parameter switches on/off the gating.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_EGAT
        /// </param>
        /// <param name="Mode">
        /// Sets the way for delay and length specification. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_GATE_TIME   (0) - Time
        /// RSSPECAN_VAL_WIMAX_GATE_SAMPLE (1) - Samples
        /// 
        /// Default Value: RSSPECAN_VAL_GATE_TIME (0)
        /// 
        /// Note:
        /// (1) Time - Length and Delay are set in time units
        ///     Samples - Length and Delay are set in samples
        /// 
        /// (2) This parameter has effect only for Gating set to On
        /// 
        /// </param>
        /// <param name="Delay">
        /// Defines the gate delay in the capture buffer in time  units or samples. The range of this value is dependent on the measurement that was last run. The driver uses this value to set the RSSPECAN_ATTR_WIMAX_EGAT_HOLD_TIME or RSSPECAN_ATTR_WIMAX_EGAT_HOLD_SAMP attribute.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 100.0e-6 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_EGAT_HOLD_TIME RSSPECAN_ATTR_WIMAX_EGAT_HOLD_SAMP
        /// 
        /// Note:
        /// (1) This parameter has effect only for Gating set to On
        /// 
        /// </param>
        /// <param name="Length">
        /// Specifies defines the gate time in the capture buffer in time units or samples. The range of this value is dependent on the measurement that was last run. The driver uses this value to set the RSSPECAN_ATTR_WIMAX_EGAT_LENG_TIME or RSSPECAN_ATTR_WIMAX_EGAT_LENG_SAMP attribute.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 400.0e-6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_EGAT_LENG_TIME RSSPECAN_ATTR_WIMAX_EGAT_LENG_SAMP
        /// 
        /// Note:
        /// (1) This parameter has effect only for Gating set to On
        /// 
        /// 
        /// 
        /// </param>
        /// <param name="Link_Gate_And_Marker">
        /// Links together the movement of the gating lines and the capture buffer marker.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_EGAT_LINK
        /// 
        /// Note:
        /// (1) This parameter has effect only for Gating set to On
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXGate(bool Gating, int Mode, double Delay, double Length, bool Link_Gate_And_Marker)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXGate(this._handle, System.Convert.ToUInt16(Gating), Mode, Delay, Length, System.Convert.ToUInt16(Link_Gate_And_Marker));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the ACP measurement.
        /// 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_ACH_MODE
        /// RSSPECAN_ATTR_WIMAX_ACH_ACP
        /// RSSPECAN_ATTR_WIMAX_ACP_NOISE_CORR
        /// 
        /// 
        /// Remote-control command(s):
        /// [SENSe:]POWer:ACHannel:MODE ABS | REL
        /// [SENSe:]POWer:ACHannel:ACPairs
        /// [SENSe:]POWer:NCORrection
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="ACP_Mode">
        /// This control sets the ACP measurement mode to either absolute or relative.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ABS (0) - Absolute 
        /// RSSPECAN_VAL_REL (1) - Relative
        /// 
        /// Default Value: RSSPECAN_VAL_ABS
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ACH_MODE
        /// </param>
        /// <param name="Number_Of_Channels">
        /// Specifies the number of adjacent channels (upper and lower channel in pairs).The figure 0 stands for pure channel power measurement.
        /// 
        /// Valid Values:
        /// 0 to 12
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ACH_ACP
        /// </param>
        /// <param name="Noise_Correction">
        /// This control sets the noise correction on/off for future spectrum ACP measurements.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ACP_NOISE_CORR
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXACP(int ACP_Mode, int Number_Of_Channels, bool Noise_Correction)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXACP(this._handle, ACP_Mode, Number_Of_Channels, System.Convert.ToUInt16(Noise_Correction));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the ACP channel bandwidth.
        /// 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_ACH_BW_CHAN
        /// RSSPECAN_ATTR_WIMAX_ACH_BW_ACH
        /// RSSPECAN_ATTR_WIMAX_ACH_BW_ALT
        /// 
        /// Remote-control command(s):
        /// [SENSe:]POWer:ACHannel:BANDwidth|BWIDth[:CHANnel]
        /// [SENSe:]POWer:ACHannel:BANDwidth|BWIDth:ACHannel
        /// [SENSe:]POWer:ACHannel:BANDwidth|BWIDth:ALTernate&lt;1 to 4&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Channel">
        /// Selects the channel type. The driver uses this value to select the  RSSPECAN_ATTR_POW_CHANNEL_SPACING or RSSPECAN_ATTR_POW_ADJ_CHANNEL_SPACING or 
        /// RSSPECAN_ATTR_POW_ALT_CHANNEL_SPACING attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ACP_TX  (0) - Separate Channel
        /// RSSPECAN_VAL_ACP_ADJ (1) - Adjacent Channel
        /// RSSPECAN_VAL_ACP_ALT (2) - Alternate Channel
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_ACP_TX  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_POW_CHANNEL_SPACING
        /// RSSPECAN_ATTR_POW_ADJ_CHANNEL_SPACING
        /// RSSPECAN_ATTR_POW_ALT_CHANNEL_SPACING
        /// </param>
        /// <param name="Channel_Number">
        /// Sets the channel number.
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value:
        /// 1
        /// 
        /// Note:
        /// (1) This control is valid for Alternate Channels. For TX Channel and Adjacent Channel has no effect.
        /// </param>
        /// <param name="Bandwidth">
        /// Sets the channel bandwidth of the radio communication system.
        /// 
        /// Valid Values:
        /// 0 to 100.0e+9
        /// 
        /// Default Value: 1.75e+6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ACH_BW_CHAN
        /// RSSPECAN_ATTR_WIMAX_ACH_BW_ACH
        /// RSSPECAN_ATTR_WIMAX_ACH_BW_ALT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXACPChannelBandwidth(int Channel, int Channel_Number, double Bandwidth)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXACPChannelBandwidth(this._handle, Channel, Channel_Number, Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the ACP channel spacing.
        /// 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_ACH_SPAC_ACH
        /// RSSPECAN_ATTR_WIMAX_ACH_SPAC_ALT
        /// 
        /// Remote-control command(s):
        /// [SENSe:]POWer:ACHannel:SPACing:ACHannel
        /// [SENSe:]POWer:ACHannel:SPACing:ALTernate&lt;1 to 4&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Channel">
        /// Selects the channel type. The driver uses this value to selects the  RSSPECAN_ATTR_WIMAX_ACH_SPAC_ACH or
        /// RSSPECAN_ATTR_WIMAX_ACH_SPAC_ALT attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ACP_ADJ (1) - Adjacent Channel
        /// RSSPECAN_VAL_ACP_ALT (2) - Alternate Channel
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_ACP_ADJ (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ACH_SPAC_ACH
        /// RSSPECAN_ATTR_WIMAX_ACH_SPAC_ALT
        /// </param>
        /// <param name="Channel_Number">
        /// Sets the channel number.
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value:
        /// 1
        /// 
        /// Note:
        /// (1) This control is valid for Alternate Channels. For Adjacent Channel has no effect.
        /// </param>
        /// <param name="Spacing">
        /// Sets the spacing between the second alternate adjacent
        /// channel and the TX channel.
        /// 
        /// Valid Values:
        /// 0 to 100.0e+9
        /// 
        /// Default Value: 3.5e+6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ACH_SPAC_ALT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXACPChannelSpacing(int Channel, int Channel_Number, double Spacing)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXACPChannelSpacing(this._handle, Channel, Channel_Number, Spacing);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the reference level, and reference level offset.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_POW_AUTO
        /// RSSPECAN_ATTR_WIMAX_RLEV_RF
        /// RSSPECAN_ATTR_WIMAX_RLEV_IQ
        /// RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET
        /// 
        /// Remote-control command(s):
        /// CONFigure:POWer:AUTO ON | OFF
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:RLEVel[:RF]
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:RLEVel:IQ
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:RLEVel:OFFSet
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement">
        /// Specifies the power level measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_MEASUREMENT_RF (0) - RF 
        /// RSSPECAN_VAL_WIMAX_MEASUREMENT_IQ (1) - IQ
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_MEASUREMENT_RF (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Reference level is applied to a RF measurements.
        /// 
        /// (2) Reference level is applied to a IQ measurements..
        /// 
        /// </param>
        /// <param name="Auto_Level">
        /// Switches on or off automatic power level detection.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_POW_AUTO
        /// </param>
        /// <param name="Reference_Level">
        /// The reference level for selected measurements.
        /// 
        /// Units: dB for RF measurement
        ///        V for IQ measurement
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_RLEV_RF
        /// RSSPECAN_ATTR_WIMAX_RLEV_IQ
        /// </param>
        /// <param name="External_Attenuation">
        /// Specifies the external attenuation/gain applied to measurements. The value corresponds to the reference level offset in spectrum analyzer mode.
        /// 
        /// Units: dB
        /// 
        /// Valid Values: -200.0 to 200.0
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXLevel(int Measurement, bool Auto_Level, double Reference_Level, double External_Attenuation)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXLevel(this._handle, Measurement, System.Convert.ToUInt16(Auto_Level), Reference_Level, External_Attenuation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the Spectrum Emission Mask (SEM) analysis to be UL or DL (uplink or downlink).
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the R&amp;S FSL and FS-K93 option.
        /// 
        /// (2) This function is available only for WiBro standard.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_ESP_ANALYSIS
        /// 
        /// Remote-control command(s):
        /// [SENSe:]POWer:SEM:MODE DL | UL
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Link_Mode">
        /// This control sets the link mode to be used.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SPEM_ANALYSIS_DL (0) - Down Link
        /// RSSPECAN_VAL_SPEM_ANALYSIS_UL (1) - Up Link
        /// 
        /// Default Value: RSSPECAN_VAL_SPEM_ANALYSIS_DL (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ESP_ANALYSIS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXSEMMode(int Link_Mode)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXSEMMode(this._handle, Link_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function allows a user-defined Channel Bandwidth to Sampling Rate relationship. 
        /// 
        /// Note:
        /// 
        /// (1) This function is only available when the Frequency Band is "Unspecified".
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_FSBW_RATIO_AUTO 
        /// RSSPECAN_ATTR_WIMAX_FSBW_SRATE
        /// RSSPECAN_ATTR_WIMAX_FSBW_CHANNEL_BW
        /// 
        /// Remote-control command(s):
        /// CONFigure:WIMax:FSBWratio:AUTo
        /// CONFigure:WIMax:FSBWratio:FS
        /// CONFigure:WIMax:FSBWratio:BW
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="FS_BW_Auto">
        /// This control switches the Fs/BS ratio to automatic (as per standard) or to user defined values.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_FSBW_RATIO_AUTO
        /// </param>
        /// <param name="Sampling_Rate_Fs">
        /// This control sets the Sample Rate (Fs) part of the ratio used to calculate the relationship between the Channel Bandwidth (BW) and the Sample Rate (FS). Ratio = Fs/BW. 
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 2.5
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_FSBW_SRATE
        /// 
        /// Note:
        /// 
        /// (1) This control has effect only if FS/BW Auto is set Off
        /// </param>
        /// <param name="Channel_Bandwidth_BW">
        /// This control sets the Channel Bandwidth (BW) part of the ratio used to calculate the relationship between the Channel Bandwidth (BW) and the Sample Rate (Fs). Ratio = Fs/BW.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 1.5
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_FSBW_CHANNEL_BW
        /// 
        /// Note:
        /// 
        /// (1) This control has effect only if FS/BW Auto is set Off
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXFSBW(bool FS_BW_Auto, double Sampling_Rate_Fs, double Channel_Bandwidth_BW)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXFSBW(this._handle, System.Convert.ToUInt16(FS_BW_Auto), Sampling_Rate_Fs, Channel_Bandwidth_BW);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the number of antennas.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_NUMBER_OF_ANTENNAS
        /// 
        /// Remote-control command(s):
        /// CONFigure:WIMax:DUTConfigure TX1 | TX2
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Number_Of_Antennas">
        /// This control specifies the number of antennas.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_DUTC_TX1 (0) - One Antenna
        /// RSSPECAN_VAL_WIMAX_DUTC_TX2 (1) - Two Antennas
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_DUTC_TX1 (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_NUMBER_OF_ANTENNAS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXNumberOfAntennas(int Number_Of_Antennas)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXNumberOfAntennas(this._handle, Number_Of_Antennas);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function performs the auto level routine immediately one time regardless of the current state of the Auto Level State.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_POW_AUTO_ONCE
        /// 
        /// Remote-control command(s):
        /// CONFigure:POWer:AUTO ONCE
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXAutoLevelOnce()
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXAutoLevelOnce(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the signal characteristics.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_POW_AUTO
        /// RSSPECAN_ATTR_WIMAX_POW_AUTO_SWE_TIME
        /// RSSPECAN_ATTR_WIMAX_POW_EXP_IQ
        /// RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET
        /// 
        /// Remote-control command(s):
        /// CONFigure:POWer:AUTO ON | OFF
        /// CONFigure:POWer:AUTO:SWEep:TIME
        /// CONFigure:POWer:EXPected:IQ
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:RLEVel:OFFSet
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Auto_Level">
        /// Switches on or off automatic power level detection.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_POW_AUTO
        /// </param>
        /// <param name="Auto_Level_Time">
        /// Sets the seep time for the automatic power level detection.
        /// 
        /// Valid Values: 1.0e-3 to 1.0 s
        /// 
        /// Default Value: 100.0e-3 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_POW_AUTO_SWE_TIME
        /// </param>
        /// <param name="Signal_Level">
        /// Sets the mean power level of the source signal as supplied to the Analyzer IQ inputs. This value will be overwritten if Auto Level is turned on.
        /// 
        /// Valid Values:
        /// 0.0316,0.0562,0.1,0.178,0.316,0.562,1.0,1.78,3.16,5.62
        /// 
        /// Default Value: 0.0316 V
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_POW_EXP_IQ
        /// 
        /// Note:
        /// (1) Another value will be coerced to allowed
        /// 
        /// (2) This value has no effect for R&amp;S FSL
        /// </param>
        /// <param name="Ext_Att">
        /// Specifies the external attenuation or gain applied to
        /// the RF signal.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 0.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXBasebandSignalLevel(bool Auto_Level, double Auto_Level_Time, double Signal_Level, double Ext_Att)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXBasebandSignalLevel(this._handle, System.Convert.ToUInt16(Auto_Level), Auto_Level_Time, Signal_Level, Ext_Att);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the channel frequency.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FREQUENCY_CENTER
        /// RSSPECAN_ATTR_WIMAX_CHAN
        /// 
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer
        /// CONFigure:CHANnel
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Channel_No">
        /// Sets the input channel.
        /// 
        /// Valid Values:
        /// &gt;0
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_CHAN
        /// 
        /// Note:
        /// (1) If channel number is set to 0, the value is not set and Frequency is set instead of channel.
        /// 
        /// </param>
        /// <param name="Frequency">
        /// Sets the center frequency.
        /// 
        /// Valid Values:
        /// FSQ 3  - 0.0 Hz to 3.6 GHz
        /// FSQ 8  - 0.0 Hz to 8.0 GHz
        /// FSQ 26 - 0.0 Hz to 26.5 GHz
        /// 
        /// Default Value: 1.5e+9 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_CENTER
        /// 
        /// Note:
        /// (1) This avalue is set only if channel number is set to 0. The instrument modifies the Channel number accordingly.
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXRFChannelFrequency(int Channel_No, double Frequency)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXRFChannelFrequency(this._handle, Channel_No, Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the signal characteristics.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_POW_AUTO
        /// RSSPECAN_ATTR_WIMAX_POW_AUTO_SWE_TIME
        /// RSSPECAN_ATTR_WIMAX_POW_EXP_RF
        /// RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET
        /// RSSPECAN_ATTR_ATTENUATION
        /// 
        /// 
        /// Remote-control command(s):
        /// CONFigure:POWer:AUTO ON | OFF
        /// CONFigure:POWer:AUTO:SWEep:TIME
        /// CONFigure:POWer:EXPected:RF
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:RLEVel:OFFSet
        /// INPut&lt;1|2&gt;:ATTenuation
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Auto_Level">
        /// Switches on or off automatic power level detection.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_POW_AUTO
        /// </param>
        /// <param name="Auto_Level_Time">
        /// Sets the seep time for the automatic power level detection..
        /// 
        /// Valid Values: 1.0e-3 to 1.0 s
        /// 
        /// Default Value: 100.0e-3 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_POW_AUTO_SWE_TIME
        /// </param>
        /// <param name="Signal_Level">
        /// Sets the mean power level of the source signal as supplied to the analyzer RF input.
        /// 
        /// Valid Values: no range checking (depends on RF Att setting)
        /// 
        /// Default Value: -15.0 dBm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_POW_EXP_RF
        /// 
        /// Note:
        /// (1) This parameter has effect only if Auto Level is set Off.
        /// 
        /// (2) Not available for R&amp;S FMU instrument.
        /// </param>
        /// <param name="RF_Att">
        /// Specifies the RF attenuation value.
        /// 
        /// Valid Values:
        /// 0 to 70 dB (without the option electronic attenuator, step 10dB)
        /// 0 to 75 dB (with the option electronic attenuator, step 5dB)
        /// 
        /// Default Value: 0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_ATTENUATION
        /// 
        /// Note:
        /// (1) This parameter has effect only if Auto Level is set Off.
        /// 
        /// (2) Not available for R&amp;S FMU instrument.
        /// </param>
        /// <param name="Ext_Att">
        /// Specifies the external attenuation or gain applied to the RF signal.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 0.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXRFSignalLevel(bool Auto_Level, double Auto_Level_Time, double Signal_Level, double RF_Att, double Ext_Att)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXRFSignalLevel(this._handle, System.Convert.ToUInt16(Auto_Level), Auto_Level_Time, Signal_Level, RF_Att, Ext_Att);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function the current internal instrument reference level
        /// used when performing measurements.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_REFERENCE_LEVEL
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1?3&gt;:Y[:SCALe]:RLEVel?
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Reference_Level">
        /// Returns the instrument internal reference level.
        /// 
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetWiMAXReferenceLevel(out double Reference_Level)
        {
            int pInvokeResult = PInvoke.GetWiMAXReferenceLevel(this._handle, out Reference_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function enables the active marker on the specified trace.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_ENABLED
        /// RSSPECAN_ATTR_MARKER_TRACE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;[:STATe]
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:TRACe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects delta marker.
        /// 
        /// Valid Values:
        /// 1..1
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker_Enabled">
        /// Specifies whether the specified delta marker is enabled. 
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_ENABLED
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_TRACE
        /// 
        /// Note:
        /// (1) This control has effect only if Marker Enabled is set On.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXMarkerEnabled(int Window, int Marker, bool Marker_Enabled, int Trace)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXMarkerEnabled(this._handle, Window, Marker, System.Convert.ToUInt16(Marker_Enabled), Trace);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the marker position.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_POSITION
        /// RSSPECAN_ATTR_WIMAX_MARK_Y
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:X
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:Y
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// Sets the marker. The driver uses this value to set the repeated capability Marker of the attributes.
        /// 
        /// Valid Values:
        /// 1
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Position">
        /// Sets the position. The driver uses this value to select the attribute to be set.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_X (0) - X
        /// RSSPECAN_VAL_Y (1) - Y
        /// 
        /// Default Value: RSSPECAN_VAL_X
        /// 
        /// Note(s):
        /// 
        /// 
        /// 
        /// </param>
        /// <param name="Value">
        /// Sets the marker position value.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value:
        /// 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_POSITION
        /// RSSPECAN_ATTR_WIMAX_MARK_Y
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXMarkPosition(int Window, int Marker, int Position, double Value)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXMarkPosition(this._handle, Window, Marker, Position, Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function positions the selected marker to the indicated symbol/carrier.
        /// 
        /// Note:
        /// (1) This function is valid only for the following result displays:
        /// constellation Vs symbol
        /// constellation Vs carrier
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_MARK_SYMB
        /// RSSPECAN_ATTR_WIMAX_MARK_CARR
        /// RSSPECAN_ATTR_WIMAX_MARK_BURST
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:SYMBol
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:CARRier
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:BURSt
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// Sets the marker. The driver uses this value to set the repeated capability Marker of the attributes.
        /// 
        /// Valid Values:
        /// 1
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Type">
        /// Sets the type. The driver uses this value to select the attribute to be set.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_CONS_SYM   (0) - Symbol
        /// RSSPECAN_VAL_WIMAX_CONS_CARR  (1) - Carrier
        /// RSSPECAN_VAL_WIMAX_CONS_BURST (2) - Burst
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_CONS_SYM
        /// 
        /// Note(s):
        /// (1) Value Burst is available only for Constellation Vs Symbol result display
        /// 
        /// </param>
        /// <param name="Position">
        /// Sets the marker position value.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value:
        /// 0
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_MARK_SYMB
        /// RSSPECAN_ATTR_WIMAX_MARK_CARR
        /// RSSPECAN_ATTR_WIMAX_MARK_BURST
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXMarkerPositionConstellation(int Window, int Marker, int Type, int Position)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXMarkerPositionConstellation(this._handle, Window, Marker, Type, Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the ratio to be zoomed around the selected marker in the selected measurement window.
        /// 
        /// Note:
        /// (1) This function is only available for the following result displays:
        /// 
        ///   Constellation versus Carrier
        ///   Constellation versus Symbol
        ///   PVT Full Burst
        ///   PVT Rising / Falling
        ///   Magnitude Capture Buffer
        ///   EVM versus Carrier
        ///   Frequency error versus Preamble
        ///   Phase error versus Preamble
        ///   Spectrum Flatness
        ///   Spectrum Flatness Difference
        ///   Spectrum Group Delay
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_ZOOM
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:FUNCtion:ZOOM
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Zoom_Ratio">
        /// This control defines the ratio to be zoomed around the selected marker in the selected measurement window.
        /// 
        /// Valid Values: 0.0 % to 100.0 %
        /// 
        /// Default Value: 100.0 %
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_ZOOM
        /// 
        /// Note:
        /// 
        /// The default value is 100.0 %, where the full trace is shown.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXMarkerZoom(int Window, double Zoom_Ratio)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXMarkerZoom(this._handle, Window, Zoom_Ratio);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function set the selected marker to the min/max peak in the current trace.
        /// 
        /// Note:
        /// (1) This function is only available for the Spectrum Flatness result display.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_SEARCH_MIN
        /// RSSPECAN_ATTR_MARKER_SEARCH_PEAK
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:MINimum
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:MAXimum
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// Sets the marker. The driver uses this value to set the repeated capability MArker of the attributes.
        /// 
        /// Valid Values:
        /// 1
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Position">
        /// Sets min/max peak value. The driver uses this value to select the attribute to be set.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MARKER_SEARCH_HIGHEST (1) - Max
        /// RSSPECAN_VAL_MARKER_SEARCH_MINIMUM (2) - Min
        /// 
        /// Default Value: RSSPECAN_VAL_MARKER_SEARCH_MINIMUM
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_SEARCH_MIN
        /// RSSPECAN_ATTR_MARKER_SEARCH_PEAK
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SetWiMAXMarkerPeak(int Window, int Marker, int Position)
        {
            int pInvokeResult = PInvoke.SetWiMAXMarkerPeak(this._handle, Window, Marker, Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function turns off all the markers in selected measurement window.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_AOFF
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:AOFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int WiMAXDisableAllMarkers(int Window)
        {
            int pInvokeResult = PInvoke.WiMAXDisableAllMarkers(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the WiMax demodulation parameters. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_SIGS
        /// RSSPECAN_ATTR_WIMAX_BCON_AUTO
        /// RSSPECAN_ATTR_WIMAX_BAN
        /// RSSPECAN_ATTR_WIMAX_LMODE
        /// RSSPECAN_ATTR_WIMAX_CEST
        /// RSSPECAN_ATTR_WIMAX_SYM_EQU
        /// RSSPECAN_ATTR_WIMAX_SYM
        /// 
        /// Remote-control command(s):
        /// [SENSe:]DEMod:FORMat:SIGSymbol ON | OFF
        /// [SENSe:]DEMod:FORMat[:BCONtent]:AUTO NONE | FIRSt | USER | ALL
        /// [SENSe:]DEMod:FORMat:BANalyze
        /// CONFigure:WIMAX:LMODe
        /// [SENSe:]DEMod:CESTimation
        /// [SENSe:]DEMod:FORMat:BANalyze:SYMbols:EQUal ON | OFF
        /// [SENSe:]DEMod:FORMat:BANalyze:SYMbols:MIN
        /// [SENSe:]DEMod:FORMat:BANalyze:SYMbols:MAX
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Use_Signal_Field_Content">
        /// Sets the signal symbol field.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_SIGS
        /// 
        /// Note:
        /// (1) This control has effect only for FS-K92 option. For FS-K93 option is ignored.
        /// 
        /// 
        /// </param>
        /// <param name="Modulation_Detection_Mode">
        /// If this parameter sets the burst to analyze auto demodulation state.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_BCON_NONE  (0) - None
        /// RSSPECAN_VAL_WIMAX_BCON_FIRST (1) - First
        /// RSSPECAN_VAL_WIMAX_BCON_USER  (2) - User
        /// RSSPECAN_VAL_WIMAX_BCON_ALL   (3) - All
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_BCON_NONE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_BCON_AUTO
        /// 
        /// </param>
        /// <param name="Demodulator">
        /// This control is used to specify the modulation to be applied to the measured data.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_BPSK1_2  (0)  - BPSK1/2
        /// RSSPECAN_VAL_WIMAX_QPSK1_2  (1)  - QPSK1/2
        /// RSSPECAN_VAL_WIMAX_QPSK3_4  (2)  - QPSK3/4
        /// RSSPECAN_VAL_WIMAX_16QAM1_2 (3)  - 16QAM1/2
        /// RSSPECAN_VAL_WIMAX_16QAM3_4 (4)  - 16QAM3/4
        /// RSSPECAN_VAL_WIMAX_64QAM2_3 (5)  - 64QAM2/3
        /// RSSPECAN_VAL_WIMAX_64QAM3_4 (6)  - 64QAM3/4
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_64QAM2_3
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_BAN
        /// 
        /// Note:
        /// (1) This control has effect only for Modulation Detection Mode None or User
        /// 
        /// </param>
        /// <param name="Link_Mode">
        /// This control specifies the Link Mode to be used.
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_WIMAX_LMOD_DL  (0) - Down Link
        /// RSSPECAN_VAL_WIMAX_LMOD_UL  (1) - Up Link
        /// RSSPECAN_VAL_WIMAX_LMOD_ANY (2) - Up Link or Down Link
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_LMOD_DL
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_LMODE
        /// 
        /// </param>
        /// <param name="Channel_Estimation">
        /// Specifies defines whether channel estimation will be done in the preamble and payload.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_CEST
        /// 
        /// 
        /// </param>
        /// <param name="Equal_Burst_Length">
        /// Enables/disables equal burst length to analyze.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_SYM_EQU
        /// 
        /// </param>
        /// <param name="Min_Number_of_Data_Symbols">
        /// Specifies the number of data symbols required for bursts to qualify for measurement analysis.
        /// 
        /// Valid Values:
        /// 1 to 1366
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_SYM_MIN
        /// 
        /// Note:
        /// (1) If Equal Burst Length is set ON, this value sets number of data symbol for equal operation.
        /// 
        /// 
        /// </param>
        /// <param name="Max_Number_of_Data_Symbols">
        /// Specifies the maximum number of data symbols required for bursts to qualify for measurement analysis.
        /// 
        /// Valid Values:
        /// 1 to 2425
        /// 
        /// Default Value: 64
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_SYM_MAX
        /// 
        /// 
        /// Note:
        /// (1) If Equal Burst Length is set ON, this value has no effect.
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXDemodulation(bool Use_Signal_Field_Content, int Modulation_Detection_Mode, int Demodulator, int Link_Mode, bool Channel_Estimation, bool Equal_Burst_Length, int Min_Number_of_Data_Symbols, int Max_Number_of_Data_Symbols)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXDemodulation(this._handle, System.Convert.ToUInt16(Use_Signal_Field_Content), Modulation_Detection_Mode, Demodulator, Link_Mode, System.Convert.ToUInt16(Channel_Estimation), System.Convert.ToUInt16(Equal_Burst_Length), Min_Number_of_Data_Symbols, Max_Number_of_Data_Symbols);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the WiMAX OFDMA demodulation parameters. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_CEST_OFDMA
        /// RSSPECAN_ATTR_WIMAX_FFT_OFFSET 
        /// 
        /// Remote-control command(s):
        /// [SENSe:]DEMod:CESTimation PREAMONLY | PREAMPAYL | PAYLONLY
        /// [SENSe:]FFT:OFFSet
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Channel_Estimation">
        /// If this parameter defines whether improved channel estimation will be used for IQ measurements.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_CEST_PREAMONLY (0) - Preamble Only
        /// RSSPECAN_VAL_WIMAX_CEST_PREAMPAYL (1) - Preamble and Payload
        /// RSSPECAN_VAL_WIMAX_CEST_PAYLONLY  (2) - Payload Only
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_CEST_PREAMPAYL
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_CEST_OFDMA
        /// </param>
        /// <param name="FFT_Offset">
        /// Specifies FFT start offset relative to the GP centre.
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_FFT_OFFSET
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXDemodulationOFDMA(int Channel_Estimation, double FFT_Offset)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXDemodulationOFDMA(this._handle, Channel_Estimation, FFT_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the WiMax subchannelization parameters. 
        /// 
        /// Note:
        /// (1) This function is available only in Uplink mode
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_SUBCHANNEL
        /// RSSPECAN_ATTR_WIMAX_SUBCHANNEL_STATE
        /// RSSPECAN_ATTR_WIMAX_SUBCHANNEL_ULPH
        /// 
        /// 
        /// Remote-control command(s):
        /// [SENSe:]SUBChannel
        /// [SENSe:]SUBChannel:STATe
        /// [SENSe:]SUBChannel:ULPHysmod
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="State">
        /// Enables or disables the subchannelization used in the measurement.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_SUBCHANNEL_STATE
        /// </param>
        /// <param name="Subchannel">
        /// Sets the sub-channel to be used in the measurement..
        /// 
        /// Valid Values:
        /// 1 to 31
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_SUBCHANNEL
        /// </param>
        /// <param name="Up_Link_Physical_Modifier">
        /// Sets the Up Link Physical Modifier to be used in the measurement.
        /// 
        /// Valid Values:
        /// 0 to 255
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_SUBCHANNEL_ULPH
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXSubchannelization(bool State, int Subchannel, int Up_Link_Physical_Modifier)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXSubchannelization(this._handle, System.Convert.ToUInt16(State), Subchannel, Up_Link_Physical_Modifier);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the tracking parameters. 
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available for OFDM Standard only.
        /// 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_TRAC_PHAS
        /// RSSPECAN_ATTR_WIMAX_TRAC_TIME
        /// RSSPECAN_ATTR_WIMAX_TRAC_LEV
        /// RSSPECAN_ATTR_WIMAX_TRAC_CEST
        /// 
        /// Remote-control command(s):
        /// [SENSe:]TRACking:PHASe ON | OFF
        /// [SENSe:]TRACking:TIME ON | OFF
        /// [SENSe:]TRACking:LEVel ON | OFF
        /// [SENSe:]TRACking:CESTimation[:BASE] USER | FULL
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Based_On">
        /// Specifies if the processing of the Channel Estimation uses the user selected Phase, Timing and Level settings or whether these settings are determined by the DSP, in which case all tracking options are used..
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_TRAC_CEST_FULL (0) - Full 
        /// RSSPECAN_VAL_WIMAX_TRAC_CEST_USER (1) - User
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_TRAC_CEST_USER
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_TRAC_CEST
        /// </param>
        /// <param name="Phase">
        /// Specifies defines whether or not the measurement results should be compensated for phase.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_TRAC_PHAS
        /// </param>
        /// <param name="Timing">
        /// Specifies whether or not the measurement results should be compensated for time.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_TRAC_TIME
        /// </param>
        /// <param name="Level">
        /// Specifies defines whether or not the measurement results should be compensated for level.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_TRAC_LEV
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXTracking(int Based_On, bool Phase, bool Timing, bool Level)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXTracking(this._handle, Based_On, System.Convert.ToUInt16(Phase), System.Convert.ToUInt16(Timing), System.Convert.ToUInt16(Level));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the tracking parameters. 
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available for OFDMA or WiBro Standard only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_TRAC_PHAS
        /// RSSPECAN_ATTR_WIMAX_TRAC_TIME
        /// RSSPECAN_ATTR_WIMAX_TRACK_PILOTS
        /// RSSPECAN_ATTR_WIMAX_TRAC_CEST
        /// 
        /// Remote-control command(s):
        /// [SENSe:]TRACking:PHASe ON | OFF
        /// [SENSe:]TRACking:TIME ON | OFF
        /// [SENSe:]TRACking:PILot PRED | DET
        /// [SENSe:]TRACking:CESTimation[:BASE] USER | FULL
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Based_On">
        /// Specifies if the processing of the Channel Estimation uses the user selected Phase, Timing and Level settings or whether these settings are determined by the DSP, in which case all tracking options are used.. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_TRAC_CEST_FULL (0) - Full 
        /// RSSPECAN_VAL_WIMAX_TRAC_CEST_USER (1) - User
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_TRAC_CEST_USER
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_TRAC_CEST
        /// 
        /// </param>
        /// <param name="Phase">
        /// Specifies defines whether or not the measurement results should be compensated for phase.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_TRAC_PHAS
        /// </param>
        /// <param name="Timing">
        /// Specifies whether or not the measurement results should be compensated for time. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_TRAC_TIME
        /// </param>
        /// <param name="Tracking_Pilot">
        /// This parameter defines whether or not the measurement results should have predefined pilot tracking (PRED) or whether the tracking should be determined when the measurement is run (DET). 
        /// 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_TRAC_PIL_PRED (0) - Predefined tracking 
        /// RSSPECAN_VAL_WIMAX_TRAC_PIL_DET  (1) - Determined at run
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_TRAC_PIL_PRED (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_TRACK_PILOTS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXOFDMATracking(int Based_On, bool Phase, bool Timing, int Tracking_Pilot)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXOFDMATracking(this._handle, Based_On, System.Convert.ToUInt16(Phase), System.Convert.ToUInt16(Timing), Tracking_Pilot);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the TX and RX filters. The names of the filters correspond to the file names returned by RSSPECAN_ATTR_WIMAX_FILT_CAT attribute.
        /// 
        /// Note:
        /// (1) This function is available only for FS-K92 of R&amp;S FSL with K93 option.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// [SENSe:]DEMod:FILTer:MODulation
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="TX_Filter_Name">
        /// This parameter selects the TX filters name. The name of the filter correspond to the file names returned by rsspecan_GetWiMAXFilterCatalog function.
        /// 
        /// Valid Values: any filter name
        /// 
        /// Default Value: "AUTO"
        /// 
        /// </param>
        /// <param name="RX_Filter_Name">
        /// This parameter selects the RX filters name. The name of the filter correspond to the file names returned by rsspecan_GetWiMAXFilterCatalog function.
        /// 
        /// Valid Values: any filter name
        /// 
        /// Default Value: "AUTO"
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXSelectFilter(string TX_Filter_Name, string RX_Filter_Name)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXSelectFilter(this._handle, TX_Filter_Name, RX_Filter_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads the names of all available filters.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// [SENSe:]DEMod:FILTer:CATalog?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the string you specify for the catalog list parameter.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 512
        /// 
        /// </param>
        /// <param name="Filter_List">
        /// Returns the names of all available filters. The file names are output without file extension.
        /// 
        /// Syntax of output format: filter_1,filter_2, .. ,filter_n
        /// 
        /// </param>
        /// <param name="Returned_Length">
        /// Returns the length of filter string readed from instrument.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetWiMAXFilterCatalog(int Buffer_Size, System.Text.StringBuilder Filter_List, out int Returned_Length)
        {
            int pInvokeResult = PInvoke.GetWiMAXFilterCatalog(this._handle, Buffer_Size, Filter_List, out Returned_Length);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the display parameters. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_DISP_TABL
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TABLe ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Display_Table">
        /// This parameter selects whether the results table is displayed.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_DISP_TABL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXDisplay(bool Display_Table)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXDisplay(this._handle, System.Convert.ToUInt16(Display_Table));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function displays the requested page for the results table. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_DISP_TABL_LIST
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TABLe:LIST
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Page">
        /// This parameter selects the result page.
        /// 
        /// Valid Values:
        /// 0 to 1 for OFDM
        /// 0 to 2 for OFDMA
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_DISP_TABL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXDisplayResultTable(int Page)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXDisplayResultTable(this._handle, Page);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the limit.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_FERR
        /// RSSPECAN_ATTR_WIMAX_SYMB
        /// RSSPECAN_ATTR_WIMAX_SST
        /// RSSPECAN_ATTR_WIMAX_EVM_ALL
        /// RSSPECAN_ATTR_WIMAX_EVM_DATA
        /// RSSPECAN_ATTR_WIMAX_IQOF
        /// RSSPECAN_ATTR_WIMAX_BERP
        /// RSSPECAN_ATTR_WIMAX_USERR
        /// 
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:FERRor[:AVERage]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:FERRor:MAXimum
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:SYMBolerror[:AVERage]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:SYMBolerror:MAXimum
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:SSTiming[:AVERage]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:SSTiming:MAXimum
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:EVM:ALL[:AVERage]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:EVM:ALL:MAXimum
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:EVM:DATA[:AVERage]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:EVM:DATA:MAXimum
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:IQOFfset[:AVERage]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:IQOFfset:MAXimum
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1..8&gt;:BURSt:BERPilot[:AVERage]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1..8&gt;:BURSt:BERPilot:MAXimum
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:USERror[:AVERage]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:USERror:MAXimum
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// Sets the limit. The driver uses this value to set the repeated capability Limit of the attributes.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Limit_Type">
        /// Sets the limit type. The driver uses this value to select the attribute to be set.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_LIM_FERR     (0) - Frequency Error
        /// RSSPECAN_VAL_WIMAX_LIM_SYMB     (1) - Symbol Error
        /// RSSPECAN_VAL_WIMAX_LIM_SST      (2) - Subscriber Station Timing
        /// RSSPECAN_VAL_WIMAX_LIM_EVM_ALL  (3) - Combined EVM
        /// RSSPECAN_VAL_WIMAX_LIM_EVM_DATA (4) - Data Carrier EVM
        /// RSSPECAN_VAL_WIMAX_LIM_IQOFF    (5) - IQ Offset Error Limit
        /// RSSPECAN_VAL_WIMAX_LIM_BERP     (6) - Bit Error Rate for Pilot Carriers
        /// RSSPECAN_VAL_WIMAX_LIM_USERR    (7) - Unmodulated Subcarrier Error
        ///  
        /// Default Value: RSSPECAN_VAL_WIMAX_LIM_FERR
        /// 
        /// Note(s):
        /// 
        /// (1) Value RSSPECAN_VAL_WIMAX_LIM_BERP is valid for R&amp;S FSL with K93 option only.
        /// 
        /// (2) Value RSSPECAN_VAL_WIMAX_LIM_USERR is valid for Up Link Zone only.
        /// </param>
        /// <param name="Mode">
        /// Sets the limit measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MAX  (1) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MAX
        /// 
        /// 
        /// </param>
        /// <param name="Value">
        /// Sets the limit value.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value:
        /// 0.0
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_FERR
        /// RSSPECAN_ATTR_WIMAX_SYMB
        /// RSSPECAN_ATTR_WIMAX_SST
        /// RSSPECAN_ATTR_WIMAX_EVM_ALL
        /// RSSPECAN_ATTR_WIMAX_EVM_DATA
        /// RSSPECAN_ATTR_WIMAX_IQOF
        /// RSSPECAN_ATTR_WIMAX_BERP
        /// RSSPECAN_ATTR_WIMAX_USERR
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXLimit(int Window, int Limit, int Limit_Type, int Mode, double Value)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXLimit(this._handle, Window, Limit, Limit_Type, Mode, Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets all limits.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:ALL &lt;numeric value&gt;, ...
        /// 
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// Sets the limit. The driver uses this value to set the repeated capability Limit of the attributes.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// This control sets the size of array in control Values.
        /// 
        /// Valid Values:
        /// &gt;0
        /// 
        /// Default Value: 12
        /// </param>
        /// <param name="Values">
        /// Sets the array of limit values.
        /// 
        /// Values:
        /// for WiMAX - K92:
        /// &lt;average EVM all bursts&gt;
        /// &lt;max EVM all bursts&gt;
        /// &lt;average EVM data carriers&gt;
        /// &lt;max EVM data carriers&gt;
        /// &lt;average frequency error&gt;
        /// &lt;max frequency error&gt;
        /// &lt;average symbol error&gt;
        /// &lt;max symbol error&gt;
        /// 
        /// for WiMAX - K93:
        /// &lt;average EVM all bursts&gt;
        /// &lt;max EVM all bursts&gt;
        /// &lt;average EVM data carriers&gt;
        /// &lt;max EVM data carriers&gt;
        /// &lt;average Frequency Error&gt;
        /// &lt;max Frequency Error&gt;
        /// &lt;average Symbol Error&gt;
        /// &lt;max Symbol Error&gt;
        /// &lt;average SS Timing&gt;
        /// &lt;maximum SS Timing&gt;
        /// &lt;average IQ Offset&gt;
        /// &lt;max IQ Offset&gt;
        /// 
        /// Note: 
        /// (1) The two SS Timing values are only present in Up Link mode, and are maintained for backward compatibility. When setting, use a value of 0, and ignore when querying.
        /// 
        /// OFDM:
        /// &lt;average EVM All Carriers&gt;, &lt;max EVM All Carriers&gt;,
        /// &lt;average EVM Data&gt;, &lt;max EVM Data&gt;, --- Not used
        /// &lt;average Frequency Error&gt;, &lt;max Frequency Error&gt;,
        /// &lt;average Symbol Error&gt;, &lt;max Symbol Error&gt;,
        /// &lt;average SS Timing&gt;, &lt;max SS Timing&gt;, 
        /// &lt;average IQ Offset&gt;, &lt;max IQ Offset&gt;
        /// 
        /// Note: 
        /// (1) EVM Data values are no longer used for OFDM, but are kept to maintain positions for backward compatibility.
        /// (2) The two SS Timing values are only present in Up Link mode, and are maintained for backward compatibility. When setting, use a value of 0, and ignore when querying.
        /// 
        /// OFDMA:
        /// &lt;average EVM Data and Pilots &gt;, &lt;max EVM Data and Pilots &gt;,
        /// &lt;average EVM Data&gt;, &lt;max EVM Data&gt;,
        /// &lt;average Frequency Error&gt;, &lt;max Frequency Error&gt;,
        /// &lt;average Symbol Error&gt;, &lt;max Symbol Error&gt;,
        /// &lt;average IQ Offset&gt;, &lt;max IQ Offset&gt;,
        /// &lt;average BER Pilots&gt;, &lt;max BER Pilots&gt; -- Optional when setting 
        /// &lt;average Unmod Subcarrier Error&gt;, &lt;max Unmod Subcarrier Error&gt; -- Optional when setting, Returned in Uplink only
        /// 
        /// Note:
        /// (1) BER Pilots values do not have to be provided when setting OFDMA limits, unless the Unmod. Subcarrier Error values are following (in which case they are required to maintain the correct positions).
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXAllLimits(int Window, int Limit, int Array_Size, double[] Values)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXAllLimits(this._handle, Window, Limit, Array_Size, Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the value at the maximum overstepping of the spectrum mask limits.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_SPEC_MASK_X
        /// RSSPECAN_ATTR_WIMAX_SPEC_MASK_Y
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:SPECtrum:MASK:CHECk:X
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:SPECtrum:MASK:CHECk:Y
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// Sets the limit. The driver uses this value to set the repeated capability Limit of the attributes.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Channel_Type">
        /// Sets the value type. The driver uses this value to select the attribute to be set.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FREQ (0) - Frequency
        /// RSSPECAN_VAL_POW  (1) - Power
        /// 
        /// Default Value: RSSPECAN_VAL_FREQ
        /// 
        /// </param>
        /// <param name="Value">
        /// Returns the value.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetWiMAXSpectrumMaxOverstepping(int Window, int Limit, int Channel_Type, out double Value)
        {
            int pInvokeResult = PInvoke.GetWiMAXSpectrumMaxOverstepping(this._handle, Window, Limit, Channel_Type, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns all limits.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:ALL
        /// 
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// Sets the limit. The driver uses this value to set the repeated capability Limit of the attributes.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Values">
        /// Returns the array of limit values.
        /// 
        /// Note:
        /// (1) The array size has to be at least 12
        /// 
        /// (2) The two SS Timing values are only present in Up Link mode, and are maintained for backward compatibility. When setting, use a value of 0, and ignore when querying.
        /// 
        /// Returned Values:
        /// for WiMAX - K92:
        /// &lt;average EVM all bursts&gt;
        /// &lt;max EVM all bursts&gt;
        /// &lt;average EVM data carriers&gt;
        /// &lt;max EVM data carriers&gt;
        /// &lt;average frequency error&gt;
        /// &lt;max frequency error&gt;
        /// &lt;average symbol error&gt;
        /// &lt;max symbol error&gt;
        /// 
        /// for WiMAX - K93:
        /// &lt;average EVM all bursts&gt;
        /// &lt;max EVM all bursts&gt;
        /// &lt;average EVM data carriers&gt;
        /// &lt;max EVM data carriers&gt;
        /// &lt;average Frequency Error&gt;
        /// &lt;max Frequency Error&gt;
        /// &lt;average Symbol Error&gt;
        /// &lt;max Symbol Error&gt;
        /// &lt;average SS Timing&gt;
        /// &lt;maximum SS Timing&gt;
        /// &lt;average IQ Offset&gt;
        /// &lt;max IQ Offset&gt;
        /// 
        /// OFDM:
        /// &lt;average EVM All Carriers&gt;, &lt;max EVM All Carriers&gt;,
        /// &lt;average EVM Data&gt;, &lt;max EVM data&gt;,
        /// &lt;average Frequency Error&gt;, &lt;max Frequency Error&gt;,
        /// &lt;average Symbol Error&gt;, &lt;max Symbol Error&gt;,
        /// &lt;average SS Timing&gt;, &lt;max SS Timing&gt;,
        /// &lt;average IQ Offset&gt;, &lt;max IQ Offset&gt;,
        /// 
        /// Note: 
        /// 
        /// (1) SS Timing and EVM Data are maintained for backward compatibility.
        /// 
        /// OFDMA:
        /// &lt;average EVM Data and Pilots&gt;, &lt;max EVM Data and Pilots&gt;,
        /// &lt;average EVM Data&gt;, &lt;max EVM Data&gt;,
        /// &lt;average Frequency Error&gt;, &lt;max Frequency Error&gt;,
        /// &lt;average Symbol Error&gt;, &lt;max Symbol Error&gt;,
        /// &lt;average IQ Offset&gt;, &lt;max IQ Offset&gt;,
        /// &lt;average BER Pilots&gt;, &lt;max BER Pilots&gt;
        /// &lt;average Unmod Subcarrier Error&gt;,&lt;max Unmod Subcarrier Error&gt; -- returned for UL zones
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetWiMAXAllLimits(int Window, int Limit, double[] Values)
        {
            int pInvokeResult = PInvoke.GetWiMAXAllLimits(this._handle, Window, Limit, Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the frame parameters
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the FSQ-K93 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_DL_IDCELL
        /// RSSPECAN_ATTR_WIMAX_UL_FRAME
        /// RSSPECAN_ATTR_WIMAX_TDD_TTG
        /// RSSPECAN_ATTR_WIMAX_UL_FRAME_CRLEN
        /// 
        /// Remote-control command(s):
        /// CONFigure:WIMax:DLSFrame:IDCell
        /// CONFigure:WIMax:ULSFrame:FRAMe
        /// CONFigure:WIMax:TDDFrame:TTG
        /// CONFigure:WIMax:ULSFrame:CRLength
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="ID_Cell">
        /// This control sets the channel bandwidth of the radio communication system.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_DL_IDCELL
        /// </param>
        /// <param name="UL_Subframe_Frame_Number">
        /// This control sets the frame number of the uplink frame.
        /// 
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_UL_FRAME
        /// </param>
        /// <param name="TDD_Frame">
        /// Specifies TDD frame Tx Transition Gap TTG. 
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 5.0e-6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_TDD_TTG
        /// </param>
        /// <param name="UL_Control_Region_Length">
        /// Specifies specifies the length, in symbols, of a control region to be found at the start of any UL zone. 
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_UL_FRAME_CRLEN
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXFrame(int ID_Cell, int UL_Subframe_Frame_Number, double TDD_Frame, int UL_Control_Region_Length)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXFrame(this._handle, ID_Cell, UL_Subframe_Frame_Number, TDD_Frame, UL_Control_Region_Length);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures that the IDCell number is determined by analysis.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the FSQ-K93 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_DOWN_LINK_ID_CELL_AUTO
        /// 
        /// Remote-control command(s):
        /// CONFigure:WIMax:FRAMe:IDCell:AUTO ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="ID_Cell_Auto">
        /// This control specifies that the IDCell number is determined by analysis.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_DOWN_LINK_ID_CELL_AUTO
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXFrameIDCellAuto(bool ID_Cell_Auto)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXFrameIDCellAuto(this._handle, System.Convert.ToUInt16(ID_Cell_Auto));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the predefined map mode for auto demod.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the FSx-K93 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_FRAME_PREDEFINED_MAP
        /// 
        /// Remote-control command(s):
        /// CONFigure:WIMax:FRAMe:PREDefmap:CONFig ON | OFF
        /// CONFigure:WIMax:FRAMe:PREDefmap:IQDL ON | OFF
        /// CONFigure:WIMax:FRAMe:PREDefmap:IQUL ON | OFF
        /// CONFigure:WIMax:FRAMe:PREDefmap:SMU ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Predefined_Map">
        /// This control selects the predefined map type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_PREDEF_MAP_CONFIG (0) - Frame Configuration
        /// RSSPECAN_VAL_WIMAX_PREDEF_MAP_IQDL   (1) - Downling IQW File
        /// RSSPECAN_VAL_WIMAX_PREDEF_MAP_IQUL   (2) - Uplink IQW File
        /// RSSPECAN_VAL_WIMAX_PREDEF_MAP_SMU    (3) - SMU Settings
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_PREDEF_MAP_CONFIG (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Frame Configuration: frame configuration settings used
        /// 
        /// (2) Downling IQW File: downlink; determined by a loaded IQW file
        /// 
        /// (3) Uplink IQW File: uplink, determined by an IQW file
        /// 
        /// (4) SMU Settings: the current SMU or equivalent generator settings are used
        /// </param>
        /// <param name="Predefined_Map_State">
        /// This control sets the state of the predefined map mode for auto demod.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_FRAME_PREDEFINED_MAP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXFramePredefinedMap(int Predefined_Map, bool Predefined_Map_State)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXFramePredefinedMap(this._handle, Predefined_Map, System.Convert.ToUInt16(Predefined_Map_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures whether the DSP should perform a pre-analysis to determine the Zone, Burst and Downlink
        /// sub?Channel Map for the current captured signal prior to calculating the results for OFDMA.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_BCON_AUTO
        /// 
        /// Remote-control command(s):
        /// [SENSe:]DEMod:FORMat[:BCONtent]:AUTO
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Use_for_Analysis">
        /// If this parameter specifies whether the analysis should be
        /// performed using the current user-defined configuration, or
        /// whether automatic determination of the frame configuration
        /// should be performed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_BCON_SIGNAL (4) - Signal MAP
        /// RSSPECAN_VAL_WIMAX_BCON_PREDEF (5) - Predefined MAP
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_BCON_PREDEF
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_BCON_AUTO
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXFrameOFDMAMAP(int Use_for_Analysis)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXFrameOFDMAMAP(this._handle, Use_for_Analysis);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the preamble index calculation.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the FS-K93 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_DL_PREAMBLE_MODE
        /// RSSPECAN_ATTR_WIMAX_DL_PREAMBLE_INDEX
        /// 
        /// Remote-control command(s):
        /// CONFigure:WIMax:DLSFrame:PREamble:MODE AUTO | USER
        /// CONFigure:WIMax:DLSFrame:PREamble:INDex
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Mode">
        /// Specifies how the preamble index is calculated.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_PRE_AUTO (0) - Auto
        /// RSSPECAN_VAL_WIMAX_PRE_USER (1) - User 
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_PRE_USER (1)
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_DL_PREAMBLE_MODE
        /// 
        /// Note(s):
        /// 
        /// (1) Auto - The preamble index is automatically calculated according to the parameters set by the CONFigure:WIMax:DLSFrame:IDCell and CONFigure:WIMax:DLSFrame:SEGMent&lt;1 to 3&gt; commands.
        /// 
        /// (2) User - The preamble index can be specified manually, i.e. the preamble pattern is chosen.
        /// </param>
        /// <param name="Preamble_Pattern">
        /// This control specifies the preamble pattern (according to the standard) to be used.
        /// 
        /// Valid Values: 0 to 113
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_DL_PREAMBLE_INDEX
        /// 
        /// Note:
        /// 
        /// (1) This control has effect only if Mode is set to User
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXDLFramePreambleIndex(int Mode, int Preamble_Pattern)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXDLFramePreambleIndex(this._handle, Mode, Preamble_Pattern);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the allowable logical sub channel usage of the transmission spectrum for one of the three downlink PUSC segments.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_DL_SUBCHANNEL 
        /// 
        /// Remote-control command(s):
        /// CONFigure:WIMax:DLSFrame:SEGMent&lt;1...3&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Segment">
        /// This control sets the PUSC segment.
        /// 
        /// Valid Values: 1 to 3
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Subchannel_Usage">
        /// This control sets the allowable logical sub channel usage of the
        /// transmission spectrum for one of the three downlink PUSC segments. 
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_DL_SUBCHANNEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXFrameDLUsedSubchannel(int Segment, int Subchannel_Usage)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXFrameDLUsedSubchannel(this._handle, Segment, Subchannel_Usage);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the allowable logical sub channel bitmap usage of the transmission spectrum for one of the three uplink segments. Note that the indexes specified in the GUI are 0 based where as under remote control they a 1 based, i.e The first segment in the GUI is labelled segement 0 and would be accessed with the command.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_UL_SUBCHANNEL_BITMAP 
        /// 
        /// Remote-control command(s):
        /// CONFigure:WIMax:ULSFrame:SEGMent&lt;1...3&gt;:BITMap&lt;1...18&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Segment">
        /// This control sets the PUSC segment.
        /// 
        /// Valid Values: 1 to 3
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Bitmap">
        /// This control sets the bitmap.
        /// 
        /// Valid Values: 1 to 18
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Logical_Subchannel">
        /// This control sets the allowable logical sub channel usage of the
        /// transmission spectrum for one of the three downlink PUSC segments. 
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_UL_SUBCHANNEL_BITMAP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXFrameULAllocatedSubchannel(int Segment, int Bitmap, int Logical_Subchannel)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXFrameULAllocatedSubchannel(this._handle, Segment, Bitmap, Logical_Subchannel);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function loads a K93 zone frame setup from an xml file created from a SMU signal generator, with support for the IEEE 80216e/D11 standard.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_FRAME_LOAD 
        /// 
        /// Remote-control command(s):
        /// MMEMory:LOAD:FRAMe:STATe 1,&lt;file_name&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="File_Name">
        /// Specifies full path of the file, where IQ data are load/stored.
        /// 
        /// Valid Values:
        /// any valid file name
        /// 
        /// Default Value: "d:\\user\\data.xml"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_FRAME_LOAD
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int LoadWiMAXFrameData(string File_Name)
        {
            int pInvokeResult = PInvoke.LoadWiMAXFrameData(this._handle, File_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sends a locally created and stored K93 zone frame file name, with support for the IEEE 80216e/D11 standard, to the SMU signal generator. If the command succeeds, the SMU will generate the waveform associated with the file sent.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_FRAME_SAVE
        /// 
        /// Remote-control command(s):
        /// MMEMory:STORe:FRAMe:STATe 1,&lt;file_name&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="File_Name">
        /// Specifies full path of the file, where frame data are stored.
        /// 
        /// Valid Values:
        /// any valid file name
        /// 
        /// Default Value: "d:\\user\\data.wimax"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_FRAME_SAVE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int StoreWiMAXFrameData(string File_Name)
        {
            int pInvokeResult = PInvoke.StoreWiMAXFrameData(this._handle, File_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines a new zone. It accepts only seven arguments which make up all the input settings to create a new zone. Up to 26 user zones can be defined.
        /// New zones can only be appended to the end of the existing defined zones. For example if 4 zones are already defined, then the suffix required to enter a new zone is 5.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the FSQ-K93 option.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:CONTrol[:DATA] &lt;boolean&gt;,DLFUSC | DLPUSC | ULPUSC, &lt;numeric value&gt;,&lt;numeric value&gt;,&lt;numeric value&gt;,&lt;numeric value&gt;,&lt;numeric value&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Zone">
        /// Selects the user defined zone.
        /// 
        /// Valid Values: 1 to 26
        /// 
        /// Default Value: 1
        /// 
        /// 
        /// </param>
        /// <param name="Analyze">
        /// Specifies whether the zone is too be marked for analysis.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// </param>
        /// <param name="Type">
        /// Sets the zone type. The driver uses this value to set the RSSPECAN_ATTR_WIMAX_ZONE_TYPE attribute. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_ZONE_TYPE_DLFUSC  (0) - DL_FUSC
        /// RSSPECAN_VAL_WIMAX_ZONE_TYPE_DLPUSC  (1) - DL_PUSC
        /// RSSPECAN_VAL_WIMAX_ZONE_TYPE_ULPUSC  (2) - UL_PUSC
        /// RSSPECAN_VAL_WIMAX_ZONE_TYPE_DLAMC2X (3) - Downlink AMC 2x
        /// RSSPECAN_VAL_WIMAX_ZONE_TYPE_ULAMC2X (4) - Uplink AMC 2x
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_ZONE_TYPE_DLFUSC (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Value Downlink AMC 2x and Uplink AMC 2x are only available for R&amp;S FSV.
        /// </param>
        /// <param name="Segment">
        /// This control sets downlink PUSC zone segment
        /// 
        /// Valid Values: 0 to 2
        /// 
        /// Default Value: 0
        /// </param>
        /// <param name="Length">
        /// This control sets zone symbol length
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0
        /// </param>
        /// <param name="Offset">
        /// This control sets zone symbol offset
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0
        /// </param>
        /// <param name="Permbase">
        /// This control sets permbase to be used for channel decoding
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0
        /// </param>
        /// <param name="PRBS_ID">
        /// This control sets PRBS to be used for channel decoding
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CreateWiMAXNewZone(int Zone, bool Analyze, int Type, int Segment, int Length, int Offset, int Permbase, int PRBS_ID)
        {
            int pInvokeResult = PInvoke.CreateWiMAXNewZone(this._handle, Zone, System.Convert.ToUInt16(Analyze), Type, Segment, Length, Offset, Permbase, PRBS_ID);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures zone parameters.
        /// 
        /// Attributes(s):
        /// RSSPECAN_ATTR_WIMAX_ZONE_ANALYZE_STATE 
        /// RSSPECAN_ATTR_WIMAX_ZONE_TYPE 
        /// RSSPECAN_ATTR_WIMAX_ZONE_SEGMENT_NUM 
        /// RSSPECAN_ATTR_WIMAX_ZONE_SYMBOL_COUNT
        /// RSSPECAN_ATTR_WIMAX_ZONE_SYMBOL_OFFSET 
        /// RSSPECAN_ATTR_WIMAX_ZONE_PERM_BASE
        /// RSSPECAN_ATTR_WIMAX_ZONE_PRBS 
        /// 
        /// Remote-control command(s):
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;[:ANALyze]:STATe
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:TYPE DLFUSC | DLPUSC | ULPUSC
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:IDSegment
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:SYMB:COUNt
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:SYMB:OFFSet
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:PERMbase
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:PRBS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Zone">
        /// Selects the user defined zone. 
        /// 
        /// Valid Values: 1 to 26
        /// 
        /// Default Value: 1
        /// 
        /// 
        /// </param>
        /// <param name="Analyze">
        /// Specifies whether the zone is too be marked for analysis.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ZONE_ANALYZE_STATE
        /// </param>
        /// <param name="Type">
        /// Sets the zone type. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_ZONE_TYPE_DLFUSC  (0) - DL_FUSC
        /// RSSPECAN_VAL_WIMAX_ZONE_TYPE_DLPUSC  (1) - DL_PUSC
        /// RSSPECAN_VAL_WIMAX_ZONE_TYPE_ULPUSC  (2) - UL_PUSC
        /// RSSPECAN_VAL_WIMAX_ZONE_TYPE_DLAMC2X (3) - Downlink AMC 2x
        /// RSSPECAN_VAL_WIMAX_ZONE_TYPE_ULAMC2X (4) - Uplink AMC 2x
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_ZONE_TYPE_DLFUSC (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ZONE_TYPE
        /// 
        /// Note(s):
        /// 
        /// (1) Value Downlink AMC 2x and Uplink AMC 2x are only available for R&amp;S FSV.
        /// </param>
        /// <param name="Segment">
        /// This control sets downlink PUSC zone segment.
        /// 
        /// Valid Values: 0 to 2
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ZONE_SEGMENT_NUM
        /// 
        /// Note:
        /// 
        /// (1) This control has effect only if Type DL_PUSC is selected.
        /// </param>
        /// <param name="Length">
        /// This control sets zone symbol length. 
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ZONE_SYMBOL_COUNT
        /// </param>
        /// <param name="Offset">
        /// This control sets zone symbol offset. 
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ZONE_SYMBOL_OFFSET
        /// 
        /// Note:
        /// 
        /// (1) This parameter is ignored. It is not allowed to set offset manually now.
        /// </param>
        /// <param name="Permbase">
        /// This control sets permbase to be used for channel decoding.
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ZONE_PERM_BASE
        /// 
        /// </param>
        /// <param name="PRBS_ID">
        /// This control sets PRBS to be used for channel decoding.
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ZONE_PRBS
        /// 
        /// Note:
        /// 
        /// (1) This control has effect only if Type DL_PUSC or DL_FUSC is seslected.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXZone(int Zone, bool Analyze, int Type, int Segment, int Length, int Offset, int Permbase, int PRBS_ID)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXZone(this._handle, Zone, System.Convert.ToUInt16(Analyze), Type, Segment, Length, Offset, Permbase, PRBS_ID);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the space time coding.
        /// 
        /// Attributes(s):
        /// RSSPECAN_ATTR_WIMAX_ZONE_SPACE_TIME_CODING 
        /// 
        /// Remote-control command(s):
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:CONTrol:STCCoding NOSTc | TX2
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Zone">
        /// Selects the user defined zone. 
        /// 
        /// Valid Values: 1 to 26
        /// 
        /// Default Value: 1
        /// 
        /// 
        /// </param>
        /// <param name="Space_Time_Coding">
        /// Specifies the space time coding. 
        /// 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_ZONE_STCC_NOST (0) - No Coding
        /// RSSPECAN_VAL_WIMAX_ZONE_STCC_TX2  (1) - Two Antennas
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_ZONE_STCC_NOST (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ZONE_SPACE_TIME_CODING
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXZoneSpaceTimeCoding(int Zone, int Space_Time_Coding)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXZoneSpaceTimeCoding(this._handle, Zone, Space_Time_Coding);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the matrix configuration for the zone.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with the FSQ-K93 option.
        /// 
        /// Attributes(s):
        /// RSSPECAN_ATTR_WIMAX_ZONE_MATRIX 
        /// 
        /// Remote-control command(s):
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:CONTrol:MXINdicator BURStdefined | A | B
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Zone">
        /// Selects the user defined zone. 
        /// 
        /// Valid Values: 1 to 26
        /// 
        /// Default Value: 1
        /// 
        /// 
        /// </param>
        /// <param name="Matrix_Type">
        /// Specifies the matrix configuration for the zone.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_MATRIX_BURS (0) - Burst Defined
        /// RSSPECAN_VAL_WIMAX_MATRIX_A    (1) - Matrix A
        /// RSSPECAN_VAL_WIMAX_MATRIX_B    (2) - Matrix B
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_MATRIX_BURS (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ZONE_MATRIX
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXZoneMatrix(int Zone, int Matrix_Type)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXZoneMatrix(this._handle, Zone, Matrix_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function can be used to enter a burst definition that is to be associated with a specific zone. It accepts eight arguments which make up all the input settings to create a new burst.
        /// A zone may have up to 32 bursts defined within it.
        /// New bursts can only be appended to the end of the existing burst list. For example if 4 bursts are already defined, then the suffix required to enter a new burst is 5.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the FSQ-K93 option.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:BURSt&lt;1...32&gt;:CONTrol[:DATA] QPSK_1_2 | QPSK_3_4 | QAM16_1_2 | QAM16_3_4 | QAM64_1_2 | QAM64_2_3 | QAM64_3_4 | QAM64_5_6, &lt;numeric value&gt;,&lt;numeric value&gt;, &lt;numeric value&gt;,&lt;numeric value&gt;,&lt;numeric value&gt;, &lt;numeric value&gt;, FCH | DLMAP | ULMAP | DATA | HARQ | FASTfeedback
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Zone">
        /// Selects the user defined zone.
        /// 
        /// Valid Values: 1 to 26
        /// 
        /// Default Value: 1
        /// 
        /// 
        /// </param>
        /// <param name="Burst">
        /// Selects the user defined burst.
        /// 
        /// Valid Values: 1 to 32
        /// 
        /// Default Value: 1
        /// 
        /// 
        /// </param>
        /// <param name="Modulation">
        /// Sets the burst modulation format. The driver uses this value to set the RSSPECAN_ATTR_WIMAX_ZONE_BURST_FORMAT attribute. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_ZONE_BURST_FORM_QPSK_12  (0) - QPSK code rate 1/2
        /// RSSPECAN_VAL_WIMAX_ZONE_BURST_FORM_QPSK_34  (1) - QPSK code rate 3/4
        /// RSSPECAN_VAL_WIMAX_ZONE_BURST_FORM_QAM16_12 (2) - QAM code rate 1/2
        /// RSSPECAN_VAL_WIMAX_ZONE_BURST_FORM_QAM16_34 (3) - QAM code rate 3/4
        /// RSSPECAN_VAL_WIMAX_ZONE_BURST_FORM_QAM64_12 (4) - 64 QAM code rate 1/2
        /// RSSPECAN_VAL_WIMAX_ZONE_BURST_FORM_QAM64_23 (5) - 64 QAM code rate 2/3
        /// RSSPECAN_VAL_WIMAX_ZONE_BURST_FORM_QAM64_34 (6) - 64 QAM code rate 3/4
        /// RSSPECAN_VAL_WIMAX_ZONE_BURST_FORM_QAM64_56 (7) - 64 QAM code rate 5/6
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_ZONE_BURST_FORM_QPSK_12  (0)
        /// </param>
        /// <param name="Subchannels">
        /// This control sets number of logical sub channels used by the burst.
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0
        /// </param>
        /// <param name="Symbols">
        /// This control sets number of symbols used by the burst.
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0
        /// </param>
        /// <param name="Slot_Duration">
        /// This control sets the duration of an uplink burst in slots. Has no effect on downlink bursts.
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0
        /// </param>
        /// <param name="Subchannel_Offset">
        /// This control sets logical sub channels offset for the burst.
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0
        /// </param>
        /// <param name="Symbol_Offset">
        /// This control sets the symbol offset for the burst.
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0
        /// </param>
        /// <param name="Burst_Power">
        /// This control sets the boosting power associated with the burst.
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <param name="Burst_Type">
        /// Sets the type of burst from a higher layer point of view. The driver uses this value to set the RSSPECAN_ATTR_WIMAX_ZONE_BURST_TYPE attribute. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_ZONE_BURS_TYPE_FCH   (0) - FCH 
        /// RSSPECAN_VAL_WIMAX_ZONE_BURS_TYPE_DLMAP (1) - DLMAP
        /// RSSPECAN_VAL_WIMAX_ZONE_BURS_TYPE_ULMAP (2) - ULMAP
        /// RSSPECAN_VAL_WIMAX_ZONE_BURS_TYPE_DATA  (3) - DATA
        /// RSSPECAN_VAL_WIMAX_ZONE_BURS_TYPE_HARQ  (4) - HARQ
        /// RSSPECAN_VAL_WIMAX_ZONE_BURS_TYPE_FASTF (5) - Fast Feedback
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_ZONE_BURS_TYPE_FCH (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int CreateWiMAXNewZoneBurst(int Zone, int Burst, int Modulation, int Subchannels, int Symbols, int Slot_Duration, int Subchannel_Offset, int Symbol_Offset, double Burst_Power, int Burst_Type)
        {
            int pInvokeResult = PInvoke.CreateWiMAXNewZoneBurst(this._handle, Zone, Burst, Modulation, Subchannels, Symbols, Slot_Duration, Subchannel_Offset, Symbol_Offset, Burst_Power, Burst_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures burst with a specific zone. A zone may have up to 32 bursts defined within it.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the FSQ-K93 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_ZONE_BURST_FORMAT
        /// RSSPECAN_ATTR_WIMAX_ZONE_BURST_SUBC_COUNT
        /// RSSPECAN_ATTR_WIMAX_ZONE_BURST_SYMBOL_COUNT 
        /// RSSPECAN_ATTR_WIMAX_ZONE_BURST_SLOT_DURATION
        /// RSSPECAN_ATTR_WIMAX_ZONE_BURST_POWER 
        /// RSSPECAN_ATTR_WIMAX_ZONE_BURST_TYPE
        /// RSSPECAN_ATTR_WIMAX_ZONE_BURST_OFFSET_AUTO 
        /// RSSPECAN_ATTR_WIMAX_ZONE_BURST_SUBC_OFFSET
        /// RSSPECAN_ATTR_WIMAX_ZONE_BURST_SYMBOL_OFFSET 
        /// 
        /// Remote-control command(s):
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:BURSt&lt;1...32&gt;:FORMat QPSK_1_2 | QPSK_3_4 | QAM16_1_2 | QAM16_3_4 | QAM64_1_2 | QAM64_2_3 | QAM64_3_4 | QAM64_5_6
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:BURSt&lt;1...32&gt;:SUBChannel:COUNt
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:BURSt&lt;1...32&gt;:SYMBol:COUNt
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:BURSt&lt;1...32&gt;:SLOT:DURation
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:BURSt&lt;1...32&gt;:POWer
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:BURSt&lt;1...32&gt;:TYPE FCH | DLMAP | ULMAP | DATA | HARQ | FASTfeedback
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:BURSt&lt;1...32&gt;:OFFSet:AUTO ON | OFF
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:BURSt&lt;1...32&gt;:SUBChannel:OFFSet
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:BURSt&lt;1...32&gt;:SYMBol:OFFSet
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Zone">
        /// Selects the user defined zone.
        /// 
        /// Valid Values: 1 to 26
        /// 
        /// Default Value: 1
        /// 
        /// 
        /// </param>
        /// <param name="Burst">
        /// Selects the user defined burst.
        /// 
        /// Valid Values: 1 to 32
        /// 
        /// Default Value: 1
        /// 
        /// 
        /// </param>
        /// <param name="Modulation">
        /// Sets the burst modulation format. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_ZONE_BURST_FORM_QPSK_12  (0) - QPSK code rate 1/2
        /// RSSPECAN_VAL_WIMAX_ZONE_BURST_FORM_QPSK_34  (1) - QPSK code rate 3/4
        /// RSSPECAN_VAL_WIMAX_ZONE_BURST_FORM_QAM16_12 (2) - QAM code rate 1/2
        /// RSSPECAN_VAL_WIMAX_ZONE_BURST_FORM_QAM16_34 (3) - QAM code rate 3/4
        /// RSSPECAN_VAL_WIMAX_ZONE_BURST_FORM_QAM64_12 (4) - 64 QAM code rate 1/2
        /// RSSPECAN_VAL_WIMAX_ZONE_BURST_FORM_QAM64_23 (5) - 64 QAM code rate 2/3
        /// RSSPECAN_VAL_WIMAX_ZONE_BURST_FORM_QAM64_34 (6) - 64 QAM code rate 3/4
        /// RSSPECAN_VAL_WIMAX_ZONE_BURST_FORM_QAM64_56 (7) - 64 QAM code rate 5/6
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_ZONE_BURST_FORM_QPSK_12  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ZONE_BURST_FORMAT
        /// </param>
        /// <param name="Subchannels">
        /// This control sets number of logical sub channels used by the burst.
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ZONE_BURST_SUBC_COUNT
        /// </param>
        /// <param name="Symbols">
        /// This control sets number of symbols used by the burst. 
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ZONE_BURST_SYMBOL_COUNT
        /// </param>
        /// <param name="Slot_Duration">
        /// This control sets the duration of an uplink burst in slots. Has no effect on downlink bursts.
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ZONE_BURST_SLOT_DURATION
        /// 
        /// </param>
        /// <param name="Burst_Power">
        /// This control sets the boosting power associated with the burst. 
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ZONE_BURST_POWER
        /// 
        /// </param>
        /// <param name="Burst_Type">
        /// Sets the type of burst from a higher layer point of view. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_ZONE_BURS_TYPE_FCH   (0) - FCH 
        /// RSSPECAN_VAL_WIMAX_ZONE_BURS_TYPE_DLMAP (1) - DLMAP
        /// RSSPECAN_VAL_WIMAX_ZONE_BURS_TYPE_ULMAP (2) - ULMAP
        /// RSSPECAN_VAL_WIMAX_ZONE_BURS_TYPE_DATA  (3) - DATA
        /// RSSPECAN_VAL_WIMAX_ZONE_BURS_TYPE_HARQ  (4) - HARQ
        /// RSSPECAN_VAL_WIMAX_ZONE_BURS_TYPE_FASTF (5) - Fast Feedback
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_ZONE_BURS_TYPE_FCH (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ZONE_BURST_TYPE
        /// 
        /// </param>
        /// <param name="Offset_Auto">
        /// Allows the logical sub channel and symbol offsets to be automatically calculated so that they are contiguous.  
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On (Default Value)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ZONE_BURST_OFFSET_AUTO
        /// 
        /// </param>
        /// <param name="Subchannel_Offset">
        /// This control sets logical sub channels offset for the burst. 
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ZONE_BURST_SUBC_OFFSET
        /// 
        /// Note:
        /// 
        /// (1) This control has effect only if Offset Auto is set Off.
        /// </param>
        /// <param name="Symbol_Offset">
        /// This control sets the symbol offset for the burst. 
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ZONE_BURST_SYMBOL_OFFSET
        /// 
        /// Note:
        /// 
        /// (1) This control has effect only if Offset Auto is set Off.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXZoneBurst(int Zone, int Burst, int Modulation, int Subchannels, int Symbols, int Slot_Duration, double Burst_Power, int Burst_Type, bool Offset_Auto, int Subchannel_Offset, int Symbol_Offset)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXZoneBurst(this._handle, Zone, Burst, Modulation, Subchannels, Symbols, Slot_Duration, Burst_Power, Burst_Type, System.Convert.ToUInt16(Offset_Auto), Subchannel_Offset, Symbol_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the matrix configuration for the burst.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with the FSQ-K93 option.
        /// 
        /// Attributes(s):
        /// RSSPECAN_ATTR_WIMAX_ZONE_BURST_MATRIX 
        /// 
        /// Remote-control command(s):
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:BURSt&lt;1...32&gt;:CONTrol:MXINdicator BURStdefined | A | B
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Zone">
        /// Selects the user defined zone. 
        /// 
        /// Valid Values: 1 to 26
        /// 
        /// Default Value: 1
        /// 
        /// 
        /// </param>
        /// <param name="Burst">
        /// Selects the user defined burst.
        /// 
        /// Valid Values: 1 to 32
        /// 
        /// Default Value: 1
        /// 
        /// 
        /// </param>
        /// <param name="Matrix_Type">
        /// Specifies the matrix configuration for the zone.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_MATRIX_BURS (0) - Burst Defined
        /// RSSPECAN_VAL_WIMAX_MATRIX_A    (1) - Matrix A
        /// RSSPECAN_VAL_WIMAX_MATRIX_B    (2) - Matrix B
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_MATRIX_BURS (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ZONE_MATRIX
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXZoneBurstMatrix(int Zone, int Burst, int Matrix_Type)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXZoneBurstMatrix(this._handle, Zone, Burst, Matrix_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function allows an external SMU signal generator to be connected via TCPIP to K93, so that frame zone setup can be downloaded directly to K93.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_SMU_ADDRESS
        /// 
        /// Remote-control command(s):
        /// SYSTem:COMMunicate:TCPip:ADDRess
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="TCPIP_Address">
        /// This parameter defines TCPIP address of external SMU generator to be frame zone downloaded. 
        /// 
        /// Valid Values:
        /// any TCPIP address
        /// 
        /// Default Value: "192.168.1.1"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_SMU_ADDRESS
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXExtFrameZoneGeneratorAddress(string TCPIP_Address)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXExtFrameZoneGeneratorAddress(this._handle, TCPIP_Address);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the current count of user defined zones.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_ZONE_COUNT_QUERY 
        /// 
        /// Remote-control command(s):
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:COUNt?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Count">
        /// Returns the current count of user defined zones.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetWiMAXZoneCount(out int Count)
        {
            int pInvokeResult = PInvoke.GetWiMAXZoneCount(this._handle, out Count);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the current count of user defined bursts within the specified zone.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_ZONE_BURST_COUNT_QUERY 
        /// 
        /// Remote-control command(s):
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:BURSt&lt;1...32&gt;:COUNt?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Zone">
        /// Selects the user defined zone.
        /// 
        /// Valid Values: 1 to 26
        /// 
        /// Default Value: 1
        /// 
        /// 
        /// </param>
        /// <param name="Count">
        /// Returns the current count of the burst in user defined zones.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetWiMAXZoneBurstCount(int Zone, out int Count)
        {
            int pInvokeResult = PInvoke.GetWiMAXZoneBurstCount(this._handle, Zone, out Count);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function deletes a specific zone. If the specified zone is within a larger list of zones, then all following zones will be shuffled down to take up the space occupied by the deleted zone.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_ZONE_DELETE
        /// 
        /// Remote-control command(s):
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:DELete
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Zone">
        /// Selects the user defined zone.
        /// 
        /// Valid Values: 1 to 26
        /// 
        /// Default Value: 1
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int WiMAXZoneDelete(int Zone)
        {
            int pInvokeResult = PInvoke.WiMAXZoneDelete(this._handle, Zone);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function deletes a specific burst from within the specified zone.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_ZONE_BURST_DELETE
        /// 
        /// Remote-control command(s):
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:BURSt&lt;1...32&gt;:DELete
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Zone">
        /// Selects the user defined zone.
        /// 
        /// Valid Values: 1 to 26
        /// 
        /// Default Value: 1
        /// 
        /// 
        /// </param>
        /// <param name="Burst">
        /// Selects the user defined burst.
        /// 
        /// Valid Values: 1 to 32
        /// 
        /// Default Value: 1
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int WiMAXZoneBurstDelete(int Zone, int Burst)
        {
            int pInvokeResult = PInvoke.WiMAXZoneBurstDelete(this._handle, Zone, Burst);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function removes all zone and burst data.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_ZONE_RESET
        /// 
        /// Remote-control command(s):
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:RESet
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int WiMAXZoneReset()
        {
            int pInvokeResult = PInvoke.WiMAXZoneReset(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function removes all burst data associated with a specific zone.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_ZONE_BURST_RESET
        /// 
        /// Remote-control command(s):
        /// CONFigure:WIMax:ZONE&lt;1...26&gt;:BURSt&lt;1...32&gt;:RESet
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Zone">
        /// Selects the user defined zone.
        /// 
        /// Valid Values: 1 to 26
        /// 
        /// Default Value: 1
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int WiMAXZoneBurstReset(int Zone)
        {
            int pInvokeResult = PInvoke.WiMAXZoneBurstReset(this._handle, Zone);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the measurement parameters. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SWEEP_TIME
        /// RSSPECAN_ATTR_WIMAX_BURS_COUN_STAT
        /// RSSPECAN_ATTR_WIMAX_BURS_COUN
        /// RSSPECAN_ATTR_NUMBER_OF_SWEEPS
        /// RSSPECAN_ATTR_WIMAX_ACPR_TIME_AUTO
        /// RSSPECAN_ATTR_WIMAX_ACPR_TIME
        /// RSSPECAN_ATTR_WIMAX_SWAP_IQ
        /// RSSPECAN_ATTR_WIMAX_FBAN
        /// RSSPECAN_ATTR_WIMAX_BAND_CHAN
        /// RSSPECAN_ATTR_WIMAX_IGR
        /// 
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:TIME
        /// [SENSe:]BURSt:COUNt:STATe ON | OFF
        /// [SENSe:]BURSt:COUNt
        /// [SENSe&lt;1|2&gt;:]SWEep:COUNt
        /// [SENSe:]SWEep:ACPR:TIME:AUTO
        /// [SENSe:]SWEep:ACPR:TIME
        /// [SENSe:]SWAPiq ON|OFF
        /// CONFigure:WIMAX:FBANd
        /// [SENSe:]BANDwidth:CHANnel
        /// CONFigure:WIMAX:IGRatio
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Overall_Burst_Count">
        /// Specifies whether a specified number of bursts is to be captured and analyzed..
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_BURS_COUN_STAT
        /// </param>
        /// <param name="No_of_Burst_to_Analyze">
        /// Sets defines the number of bursts that will be analyzed by the measurement.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_BURS_COUN
        /// 
        /// Note:
        /// (1) This parameter is ignored, if Overall Burst Count is set to Off.
        /// 
        /// 
        /// </param>
        /// <param name="Capture_Time">
        /// Sets the sweep acquisition period (capture time) for which the input signal is to be measured. 
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 5.0e-3 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SWEEP_TIME
        /// 
        /// </param>
        /// <param name="Swap_I_Q">
        /// Defines whether or not the recorded IQ pairs should be swapped (I/Q) before being processed.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_SWAP_IQ
        /// </param>
        /// <param name="Sweep_Count">
        /// Specifies the number of sweeps that should be performed for Spectrum ACP/ACPR and Spectrum Mask measurements.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NUMBER_OF_SWEEPS
        /// 
        /// </param>
        /// <param name="Sweep_Time_Auto__Mask_ACP_">
        /// Specifies automatic sweep time calculation for Spectrum ACPR and .MASK measurements. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ACPR_TIME_AUTO
        /// 
        /// </param>
        /// <param name="Sweep_Time__Mask_ACP_">
        /// Sets the the sweep time for Spectrum ACP and .MASK measurements.
        /// 
        /// Valid Values: 
        /// 15.0e-3 to 16.0e3 s
        /// 
        /// Default Value: 2.0 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ACPR_TIME
        /// 
        /// Note:
        /// 
        /// (1) This control has effect only if Time Auto is set Off
        /// </param>
        /// <param name="Frequency_Band">
        /// Sets the frequency band to be used.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_FBAN_UNSP  (0) - Unspecified
        /// RSSPECAN_VAL_WIMAX_FBAN_ETSI  (1) - ETSI 3.410-4.200 GHz
        /// RSSPECAN_VAL_WIMAX_FBAN_MMDS  (2) - MMDS 2.150-2.162 GHz
        /// RSSPECAN_VAL_WIMAX_FBAN_WCS   (3) - WCS 2.305-2.320 GHz
        /// RSSPECAN_VAL_WIMAX_FBAN_CEPT  (4) - CEPT 5.470-5.725 GHz
        /// RSSPECAN_VAL_WIMAX_FBAN_UNI   (5) - U-NII 5.250-5.360 GHz
        /// RSSPECAN_VAL_WIMAX_FBAN_ETSI2 (6) - ETSI 10.000-10.680 GHz
        /// RSSPECAN_VAL_WIMAX_FBAN_MMDS2 (7) - MMDS 2.500-2.690 GHz
        /// RSSPECAN_VAL_WIMAX_FBAN_WCS2  (8) - WCS 2.345-2.360 GHz
        /// RSSPECAN_VAL_WIMAX_FBAN_CEPT2 (9) - CEPT 5.725-5.875 GHz
        /// RSSPECAN_VAL_WIMAX_FBAN_UNI2 (10) - U-NII 5.725-5.825 GHz
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_FBAN_UNSP (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_FBAN
        /// 
        /// </param>
        /// <param name="Channel_Bandwidth">
        /// Sets the channel bandwidth of the signal to be measured.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 1.75e+6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_BAND_CHAN
        /// 
        /// </param>
        /// <param name="Guard_Ratio">
        /// Specifies the inverse guard ratio for burst to be analyzed.
        /// 
        /// Valid Values:
        /// 4  - 1/4
        /// 8  - 1/8
        /// 16 - 1/16
        /// 32 - 1/32
        ///  
        /// 
        /// Default Value: 4
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_IGR
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXMeas(bool Overall_Burst_Count, int No_of_Burst_to_Analyze, double Capture_Time, bool Swap_I_Q, int Sweep_Count, bool Sweep_Time_Auto__Mask_ACP_, double Sweep_Time__Mask_ACP_, int Frequency_Band, double Channel_Bandwidth, int Guard_Ratio)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXMeas(this._handle, System.Convert.ToUInt16(Overall_Burst_Count), No_of_Burst_to_Analyze, Capture_Time, System.Convert.ToUInt16(Swap_I_Q), Sweep_Count, System.Convert.ToUInt16(Sweep_Time_Auto__Mask_ACP_), Sweep_Time__Mask_ACP_, Frequency_Band, Channel_Bandwidth, Guard_Ratio);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the measurement parameters. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SWEEP_TIME
        /// RSSPECAN_ATTR_NUMBER_OF_SWEEPS
        /// RSSPECAN_ATTR_WIMAX_ACPR_TIME_AUTO
        /// RSSPECAN_ATTR_WIMAX_ACPR_TIME
        /// RSSPECAN_ATTR_WIMAX_SWAP_IQ
        /// RSSPECAN_ATTR_WIMAX_FBAN
        /// RSSPECAN_ATTR_WIMAX_BAND_CHAN
        /// RSSPECAN_ATTR_WIMAX_IGR
        /// RSSPECAN_ATTR_WIMAX_NFFT
        /// RSSPECAN_ATTR_WIMAX_ZONE_COUNT_STATE
        /// RSSPECAN_ATTR_WIMAX_ZONE_COUNT
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:TIME
        /// [SENSe&lt;1|2&gt;:]SWEep:COUNt
        /// [SENSe:]SWEep:ACPR:TIME:AUTO
        /// [SENSe:]SWEep:ACPR:TIME
        /// [SENSe:]SWAPiq ON|OFF
        /// CONFigure:WIMAX:FBANd
        /// [SENSe:]BANDwidth:CHANnel
        /// CONFigure:WIMAX:IGRatio
        /// CONFigure:WIMax:NFFT
        /// [SENSe:]ZONE:COUNt:STATe
        /// [SENSe:]ZONE:COUNt
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Capture_Time">
        /// Sets the sweep acquisition period (capture time) for which the input signal is to be measured.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 5.0e-3 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SWEEP_TIME
        /// 
        /// </param>
        /// <param name="Swap_I_Q">
        /// Defines whether or not the recorded IQ pairs should be swapped (I/Q) before being processed.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_SWAP_IQ
        /// 
        /// </param>
        /// <param name="Sweep_Count">
        /// Specifies the number of sweeps that should be performed for Spectrum ACP/ACPR and Spectrum Mask measurements. 
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NUMBER_OF_SWEEPS
        /// 
        /// </param>
        /// <param name="Sweep_Time_Auto__Mask_ACP_">
        /// Specifies automatic sweep time calculation for Spectrum ACPR and .MASK measurements. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ACPR_TIME_AUTO
        /// 
        /// </param>
        /// <param name="Sweep_Time__Mask_ACP_">
        /// Sets the the sweep time for Spectrum ACP and .MASK measurements.
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0.1 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ACPR_TIME
        /// 
        /// Note:
        /// 
        /// (1) This control has effect only if Time Auto is set Off
        /// </param>
        /// <param name="Frequency_Band">
        /// Sets the frequency band to be used.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_FBAN_UNSP  (0) - Unspecified
        /// RSSPECAN_VAL_WIMAX_FBAN_ETSI  (1) - ETSI
        /// RSSPECAN_VAL_WIMAX_FBAN_MMDS  (2) - MMDS
        /// RSSPECAN_VAL_WIMAX_FBAN_WCS   (3) - WCS
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_FBAN_UNSP (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_FBAN
        /// 
        /// </param>
        /// <param name="Channel_Bandwidth">
        /// Sets the channel bandwidth of the signal to be measured. 
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 1.75e+6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_BAND_CHAN
        /// 
        /// 
        /// </param>
        /// <param name="Guard_Ratio">
        /// Specifies the inverse guard ratio for burst to be analyzed.
        /// 
        /// Valid Values:
        /// 4  - 1/4
        /// 8  - 1/8
        /// 16 - 1/16
        /// 32 - 1/32
        ///  
        /// 
        /// Default Value: 4
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_IGR
        /// 
        /// </param>
        /// <param name="FFT_Size">
        /// Specifies FFT size. 
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 1024
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_NFFT
        /// 
        /// </param>
        /// <param name="Capture_Count_State">
        /// Specifies if the zone count parameter will be used by the measurement (On), otherwise the zone count parameter will be ignored (Off). 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ZONE_COUNT_STATE
        /// 
        /// </param>
        /// <param name="Number_of_Subframes_to_Analyze">
        /// Specifies defines the number of zones that will be analyzed by the measurement.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ZONE_COUNT
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXMeasOFDMA(double Capture_Time, bool Swap_I_Q, int Sweep_Count, bool Sweep_Time_Auto__Mask_ACP_, double Sweep_Time__Mask_ACP_, int Frequency_Band, double Channel_Bandwidth, int Guard_Ratio, int FFT_Size, bool Capture_Count_State, int Number_of_Subframes_to_Analyze)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXMeasOFDMA(this._handle, Capture_Time, System.Convert.ToUInt16(Swap_I_Q), Sweep_Count, System.Convert.ToUInt16(Sweep_Time_Auto__Mask_ACP_), Sweep_Time__Mask_ACP_, Frequency_Band, Channel_Bandwidth, Guard_Ratio, FFT_Size, System.Convert.ToUInt16(Capture_Count_State), Number_of_Subframes_to_Analyze);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the measurement parameters. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SWEEP_TIME
        /// RSSPECAN_ATTR_NUMBER_OF_SWEEPS
        /// RSSPECAN_ATTR_WIMAX_ACPR_TIME_AUTO
        /// RSSPECAN_ATTR_WIMAX_ACPR_TIME
        /// RSSPECAN_ATTR_WIMAX_SWAP_IQ
        /// RSSPECAN_ATTR_WIMAX_BAND_CHAN
        /// RSSPECAN_ATTR_WIMAX_ZONE_COUNT_STATE
        /// RSSPECAN_ATTR_WIMAX_ZONE_COUNT
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:TIME
        /// [SENSe&lt;1|2&gt;:]SWEep:COUNt
        /// [SENSe:]SWEep:ACPR:TIME:AUTO
        /// [SENSe:]SWEep:ACPR:TIME
        /// [SENSe:]SWAPiq ON|OFF
        /// [SENSe:]BANDwidth:CHANnel
        /// [SENSe:]ZONE:COUNt:STATe
        /// [SENSe:]ZONE:COUNt
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Capture_Time">
        /// Sets the sweep acquisition period (capture time) for which the input signal is to be measured.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 5.0e-3 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SWEEP_TIME
        /// 
        /// </param>
        /// <param name="Swap_I_Q">
        /// Defines whether or not the recorded IQ pairs should be swapped (I/Q) before being processed. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_SWAP_IQ
        /// 
        /// </param>
        /// <param name="Sweep_Count">
        /// Specifies the number of sweeps that should be performed for Spectrum ACP/ACPR and Spectrum Mask measurements. 
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NUMBER_OF_SWEEPS
        /// 
        /// </param>
        /// <param name="Sweep_Time_Auto__Mask_ACP_">
        /// Specifies automatic sweep time calculation for Spectrum ACPR and .MASK measurements.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ACPR_TIME_AUTO
        /// 
        /// </param>
        /// <param name="Sweep_Time__Mask_ACP_">
        /// Sets the the sweep time for Spectrum ACP and .MASK measurements.
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 24.0e-6 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ACPR_TIME
        /// 
        /// Note:
        /// 
        /// (1) This control has effect only if Time Auto is set Off
        /// </param>
        /// <param name="Channel_Bandwidth">
        /// Sets the channel bandwidth of the signal to be measured.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 1.75e+6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_BAND_CHAN
        /// 
        /// </param>
        /// <param name="Capture_Count_State">
        /// Specifies if the zone count parameter will be used by the measurement (On), otherwise the zone count parameter will be ignored (Off).
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ZONE_COUNT_STATE
        /// 
        /// </param>
        /// <param name="Number_of_Subframes_to_Analyze">
        /// Specifies defines the number of zones that will be analyzed by the measurement.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_ZONE_COUNT
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXMeasWiBro(double Capture_Time, bool Swap_I_Q, int Sweep_Count, bool Sweep_Time_Auto__Mask_ACP_, double Sweep_Time__Mask_ACP_, double Channel_Bandwidth, bool Capture_Count_State, int Number_of_Subframes_to_Analyze)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXMeasWiBro(this._handle, Capture_Time, System.Convert.ToUInt16(Swap_I_Q), Sweep_Count, System.Convert.ToUInt16(Sweep_Time_Auto__Mask_ACP_), Sweep_Time__Mask_ACP_, Channel_Bandwidth, System.Convert.ToUInt16(Capture_Count_State), Number_of_Subframes_to_Analyze);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets measurement type to be Power Versus Time and configures measurement parameters.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_PVT
        /// RSSPECAN_ATTR_WIMAX_PVT_SEL
        /// RSSPECAN_ATTR_WIMAX_BURS_PVT_BURS
        /// 
        /// Remote-control command(s):
        /// CONFigure:BURSt:PVT[:IMMediate]
        /// CONFigure:BURSt:PVT:SELect EDGE | FULL
        /// CONFigure:BURSt:PVT:BURSt
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Rising_Falling">
        /// Sets the power versus time measurement results interpreting.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_PVT_EDGE (0) - Both
        /// RSSPECAN_VAL_PVT_FULL (1) - Full
        /// 
        /// Default Value: RSSPECAN_VAL_PVT_EDGE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_PVT_SEL
        /// 
        /// </param>
        /// <param name="Burst_Selection">
        /// Sets the PVT to use the specified burst for its measurement results.
        /// 
        /// Valid Values:
        /// no checked
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_BURS_PVT_BURS
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXPVTMeas(int Rising_Falling, int Burst_Selection)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXPVTMeas(this._handle, Rising_Falling, Burst_Selection);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets measurement type to be Power Versus Time and configures measurement parameters.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_PVT
        /// RSSPECAN_ATTR_WIMAX_PVT_SEL
        /// 
        /// Remote-control command(s):
        /// CONFigure:BURSt:PVT[:IMMediate]
        /// CONFigure:BURSt:PVT:SELect EDGE | FULL
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Rising_Falling">
        /// Sets the power versus time measurement results interpreting.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_PVT_EDGE (0) - Both
        /// RSSPECAN_VAL_PVT_FULL (1) - Full
        /// 
        /// Default Value: RSSPECAN_VAL_PVT_EDGE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_PVT_SEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXPVTMeasOFDMA(int Rising_Falling)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXPVTMeasOFDMA(this._handle, Rising_Falling);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets measurement type to be EVM.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_EVM_ECAR
        /// RSSPECAN_ATTR_WIMAX_EVM_ESYM
        /// RSSPECAN_ATTR_WIMAX_PRE
        /// RSSPECAN_ATTR_WIMAX_PRE_SEL
        /// RSSPECAN_ATTR_WIMAX_DISP_AUTO_Y
        /// RSSPECAN_ATTR_WIMAX_Y_PDIV
        /// RSSPECAN_ATTR_WIMAX_UNIT_EVM
        /// RSSPECAN_ATTR_WIMAX_UNIT_PRE
        /// 
        /// Remote-control command(s):
        /// CONFigure:BURSt:EVM:ECARrier[:IMMediate]
        /// CONFigure:BURSt:EVM:ESYMbol[:IMMediate]
        /// CONFigure:BURSt:PREamble[:IMMediate]
        /// CONFigure:BURSt:PREamble:SELect PHAS | FREQ
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:AUTO
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:PDIVision
        /// UNIT:EVM DB | PCT
        /// UNIT:PREamble HZ | PCT
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Mode">
        /// Sets the EVT measurement mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_EVM_MODE_SYMBOL   (0) - EVM Vs Symbol
        /// RSSPECAN_VAL_WIMAX_EVM_MODE_CARRIER  (1) - EVM Vs Carrier
        /// RSSPECAN_VAL_WIMAX_EVM_MODE_PHAS     (2) - Phase Vs Preamble
        /// RSSPECAN_VAL_WIMAX_EVM_MODE_FREQ     (3) - Frequency Vs Preamble
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_EVM_MODE_SYMBOL
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_EVM_ECAR
        /// RSSPECAN_ATTR_WIMAX_EVM_ESYM 
        /// RSSPECAN_ATTR_WIMAX_PRE 
        /// RSSPECAN_ATTR_WIMAX_PRE_SEL
        /// </param>
        /// <param name="Auto_Scaling">
        /// This parameter switches on or off automatic scaling of the Y-axis for the specified trace display.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_DISP_AUTO_Y
        /// </param>
        /// <param name="Per_Division">
        /// This parameter sets the size of each Y scale division for the specified trace display. 
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 3.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_Y_PDIV
        /// 
        /// Note:
        /// (1) This attribute has no affect if automatic scaling of the Y-axis is enabled. 
        /// </param>
        /// <param name="Unit">
        /// This parameter specifies the units for EVM trace results.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DB  (8)  - dB 
        /// RSSPECAN_VAL_UNIT_PCT (13) - %
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_PCT (13)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_UNIT_EVM
        /// 
        /// 
        /// Note:
        /// (1) This control has effect only for Mode EVM vs Carrier and EVM vs Symbols.
        /// 
        /// (2) RSSPECAN_ATTR_WIMAX_UNIT_PRE attribute is set automatically ... DEG for Phase vs. Preamble and Hz for Frequency vs. Preamble
        /// 
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXEVMMeasurement(int Mode, bool Auto_Scaling, double Per_Division, int Unit)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXEVMMeasurement(this._handle, Mode, System.Convert.ToUInt16(Auto_Scaling), Per_Division, Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets measurement type to be Spectrum.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_SPEC_MASK
        /// RSSPECAN_ATTR_WIMAX_SPEC_MASK_SEL
        /// RSSPECAN_ATTR_WIMAX_SPEC_FFT
        /// RSSPECAN_ATTR_WIMAX_SPEC_ACPR
        /// RSSPECAN_ATTR_WIMAX_SPEC_ACPR_SEL
        /// RSSPECAN_ATTR_WIMAX_SPEC_FLAT
        /// RSSPECAN_ATTR_WIMAX_SPEC_PREAMBLE
        /// 
        /// Remote-control command(s):
        /// CONFigure:BURSt:SPECtrum:MASK[:IMMediate]
        /// CONFigure:BURSt:SPECtrum:MASK:SELect IEEE | ETSI
        /// CONFigure:BURSt:SPECtrum:FFT[:IMMediate]
        /// CONFigure:BURSt:SPECtrum:ACPR[:IMMediate]
        /// CONFigure:BURSt:SPECtrum:ACPR:SELect REL | ABS
        /// CONFigure:BURSt:SPECtrum:FLATness:SELect FLAT | GRD | DIFF
        /// CONFigure:BURSt:SPECtrum:PHASe:PREamble[:IMMediate]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Mode">
        /// Sets the spectrum measurement mode. The driver uses this value to set the RSSPECAN_ATTR_WIMAX_SPEC_MASK and RSSPECAN_ATTR_WIMAX_SPEC_MASK_SEL or RSSPECAN_ATTR_WIMAX_SPEC_FFT or 
        /// RSSPECAN_ATTR_WIMAX_SPEC_FLAT or
        /// RSSPECAN_ATTR_WIMAX_SPEC_ACPR or RSSPECAN_ATTR_WIMAX_SPEC_ACPR_SEL and RSSPECAN_ATTR_WIMAX_SPEC_PREAMBLE attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_MODE_IEEE       (0) - IEEE
        /// RSSPECAN_VAL_WIMAX_MODE_ETSI       (1) - ETSI
        /// RSSPECAN_VAL_WIMAX_MODE_FFT        (2) - FFT
        /// RSSPECAN_VAL_WIMAX_MODE_FLAT       (4) - Flatness
        /// RSSPECAN_VAL_WIMAX_MODE_FLAT_GRDEL (5) - Flatness Group Delay
        /// RSSPECAN_VAL_WIMAX_MODE_FLAT_DIFF  (6) - Flatness Difference
        /// RSSPECAN_VAL_WIMAX_MODE_ACPR_ABS   (7) - ACPR Absolute
        /// RSSPECAN_VAL_WIMAX_MODE_ACPR_REL   (8) - ACPR Relative
        /// RSSPECAN_VAL_WIMAX_MODE_MASK       (9) - Spectrum Mask
        /// RSSPECAN_VAL_WIMAX_MODE_PREAMBLE   (10) - Preamble Channel Frequency Response
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_MODE_FLAT
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_SPEC_MASK
        /// RSSPECAN_ATTR_WIMAX_SPEC_MASK_SEL
        /// RSSPECAN_ATTR_WIMAX_SPEC_FFT
        /// RSSPECAN_ATTR_WIMAX_SPEC_FLAT
        /// RSSPECAN_ATTR_WIMAX_SPEC_ACPR
        /// RSSPECAN_ATTR_WIMAX_SPEC_ACPR_SEL
        /// RSSPECAN_ATTR_WIMAX_SPEC_PREAMBLE
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXSpectrumMeasurement(int Mode)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXSpectrumMeasurement(this._handle, Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Constellation measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_CONS_CSYM
        /// RSSPECAN_ATTR_WIMAX_CONS_CCAR
        /// RSSPECAN_ATTR_WIMAX_CONS_SEL
        /// RSSPECAN_ATTR_WIMAX_CONS_SEL_NR
        /// 
        /// Remote-control command(s):
        /// CONFigure:BURSt:CONStellation:CSYMbol[:IMMediate]
        /// CONFigure:BURSt:CONStellation:CCARrier[:IMMediate]
        /// CONFigure:BURSt:CONStellation:CARRier:SELect
        /// CONFigure:BURSt:CONStellation:CARRier:SELect nr
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Constellation_Mode">
        /// Sets the measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_CONS_SYM  (0) - Constellation vs. Symbol
        /// RSSPECAN_VAL_WIMAX_CONS_CARR (1) - Constellation vs. Carrier
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_CONS_SYM
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_CONS_CCAR
        /// RSSPECAN_ATTR_WIMAX_CONS_CSYM
        /// </param>
        /// <param name="Carrier_Selection">
        /// Defines the result calculation of the selected carrier.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SELECTION_ALL (0) - All
        /// RSSPECAN_VAL_SELECTION_PIL (1) - Pilots
        /// RSSPECAN_VAL_SELECTION_NUM (2) - Carrier Number
        /// 
        /// Default Value: RSSPECAN_VAL_SELECTION_NUM
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_CONS_CARR_SEL
        /// RSSPECAN_ATTR_WIMAX_CONS_CARR_SEL_NUM
        /// 
        /// Note:
        /// (1) This parameter is valid only if Constellation vs. Symbol is selected
        /// </param>
        /// <param name="Carrier_Number">
        /// Defines the carrier number.
        /// 
        /// Valid Values:
        /// -100 to 100 exclude 0
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_CONS_CARR_SEL_NUM
        /// 
        /// Note:
        /// (1) This parameter is valid only if Carrier Number is selected in Carrier Selection control
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXConstellationMeasurement(int Constellation_Mode, int Carrier_Selection, int Carrier_Number)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXConstellationMeasurement(this._handle, Constellation_Mode, Carrier_Selection, Carrier_Number);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Constellation Selection for IEEE 802.16e-2005 OFDMA &amp; WiBro.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the FSQ-K93 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_FORMAT_SELECT 
        /// RSSPECAN_ATTR_WIMAX_CONS_SEL
        /// RSSPECAN_ATTR_WIMAX_CONS_SEL_NR
        /// 
        /// Remote-control command(s):
        /// CONFigure:BURSt:CONSt:FORMat:SELect ALL | QPSK | QAM16 | QAM64
        /// CONFigure:BURSt:CONSt:BURSt:SELect ALL | PILOTS 
        /// CONFigure:BURSt:CONSt:BURSt:SELect &lt;numeric value &gt;
        /// CONFigure:BURSt:CONSt:CARRier:SELect ALL | PILOTS
        /// CONFigure:BURSt:CONSt:CARRier:SELect -100 to 100
        /// CONFigure:BURSt:CONSt:SYMBol:SELect ALL
        /// CONFigure:BURSt:CONSt:SYMBol:SELect &lt;numeric value&gt;
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Modulation">
        /// This control sets the modulation format.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_MODUL_ALL   (0) - All
        /// RSSPECAN_VAL_WIMAX_MODUL_QPSK  (1) - QPSK
        /// RSSPECAN_VAL_WIMAX_MODUL_QAM16 (2) - QAM16
        /// RSSPECAN_VAL_WIMAX_MODUL_QAM64 (3) - QAM64
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_MODUL_ALL   (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_FORMAT_SELECT
        /// 
        /// Notes:
        /// 
        /// (1) All - all modulation formats
        /// 
        /// (2) QPSK - QPSK modulation format
        /// 
        /// (3) QAM16 - QAM16 modulation format
        /// 
        /// (4) QAM64 - QAM64 modulation format
        /// </param>
        /// <param name="Burst_Selection">
        /// This control handles burst selection.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SELECTION_ALL (0) - All
        /// RSSPECAN_VAL_SELECTION_PIL (1) - Pilots
        /// RSSPECAN_VAL_SELECTION_NUM (2) - Burst Number
        /// 
        /// Default Value: RSSPECAN_VAL_SELECTION_NUM (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_CONS_SEL
        /// RSSPECAN_ATTR_WIMAX_CONS_SEL_NR
        /// 
        /// Notes:
        /// 
        /// (1) Burst Number:
        /// Burst from valid range is selected.
        /// 
        /// (2) All:
        /// All bursts are selected.
        /// 
        /// (3) Pilots:
        /// Pilots only.
        /// 
        /// </param>
        /// <param name="Burst_Number">
        /// Burst number is to be selected.
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_CONS_BURS_SELECT_NUM
        /// 
        /// Note:
        /// 
        /// (1) This control has effect only if Burst Number is select.
        /// </param>
        /// <param name="Carrier_Selection">
        /// Defines the result calculation of the selected carrier.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SELECTION_ALL (0) - All
        /// RSSPECAN_VAL_SELECTION_PIL (1) - Pilots
        /// RSSPECAN_VAL_SELECTION_NUM (2) - Carrier Number
        /// 
        /// Default Value: RSSPECAN_VAL_SELECTION_NUM
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_CONS_SEL
        /// RSSPECAN_ATTR_WIMAX_CONS_SEL_NR 
        /// 
        /// 
        /// Note:
        /// (1) This parameter is valid only if Constellation vs. Symbol is selected
        /// </param>
        /// <param name="Carrier_Number">
        /// Defines the carrier number.
        /// 
        /// Valid Values:
        /// -100 to 100 exclude 0
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_CONS_CARR_SEL_NR
        /// 
        /// Note:
        /// (1) This parameter is valid only if Carrier Number is selected in Carrier Selection control
        /// </param>
        /// <param name="Symbol_Selection">
        /// Defines the sets the symbol results calculation.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SELECTION_ALL (0) - All
        /// RSSPECAN_VAL_SELECTION_NUM (2) - Symbol Number
        /// 
        /// Default Value: RSSPECAN_VAL_SELECTION_NUM
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_CONS_SEL
        /// RSSPECAN_ATTR_WIMAX_CONS_SEL_NR
        /// 
        /// </param>
        /// <param name="Symbol_Number">
        /// Defines the carrier number.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_CONS_SYMBOL_SELECT_NUM
        /// 
        /// Note:
        /// (1) This parameter is valid only if Symbol Number is selected in Symbol Selection control
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXConstellationSelection(int Modulation, int Burst_Selection, int Burst_Number, int Carrier_Selection, int Carrier_Number, int Symbol_Selection, int Symbol_Number)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXConstellationSelection(this._handle, Modulation, Burst_Selection, Burst_Number, Carrier_Selection, Carrier_Number, Symbol_Selection, Symbol_Number);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Bitstream Selection for IEEE 802.16e-2005 OFDMA &amp; WiBro.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the FSQ-K93 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_FORMAT_SELECT 
        /// RSSPECAN_ATTR_WIMAX_BSTR_SEL
        /// RSSPECAN_ATTR_WIMAX_BSTR_SEL_NR
        /// 
        /// Remote-control command(s):
        /// CONFigure:BURSt:BSTReam:FORMat:SELect
        /// CONFigure:BURSt:BSTReam:BURSt:SELect ALL | PILOTS | num
        /// CONFigure:BURSt:BSTReam:SYMBol:SELect ALL | PILOTS | num
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Modulation">
        /// This control sets the modulation format.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_MODUL_ALL   (0) - All
        /// RSSPECAN_VAL_WIMAX_MODUL_QPSK  (1) - QPSK
        /// RSSPECAN_VAL_WIMAX_MODUL_QAM16 (2) - QAM16
        /// RSSPECAN_VAL_WIMAX_MODUL_QAM64 (3) - QAM64
        /// RSSPECAN_VAL_WIMAX_MODUL_BPSK  (4) - BPSK
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_MODUL_ALL   (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_FORMAT_SELECT
        /// 
        /// Notes:
        /// 
        /// (1) All - all modulation formats
        /// 
        /// (2) QPSK - QPSK modulation format
        /// 
        /// (3) QAM16 - QAM16 modulation format
        /// 
        /// (4) QAM64 - QAM64 modulation format
        /// 
        /// (4) BPSK - BPSK modulation format
        /// </param>
        /// <param name="Burst_Selection">
        /// This control handles burst selection.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SELECTION_ALL (0) - All
        /// RSSPECAN_VAL_SELECTION_PIL (1) - Pilots
        /// RSSPECAN_VAL_SELECTION_NUM (2) - Burst Number
        /// 
        /// Default Value: RSSPECAN_VAL_SELECTION_NUM (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_BSTR_SEL
        /// RSSPECAN_ATTR_WIMAX_BSTR_SEL_NUM
        /// 
        /// Notes:
        /// 
        /// (1) Burst Number:
        /// Burst from valid range is selected.
        /// 
        /// (2) All:
        /// All bursts are selected.
        /// 
        /// (3) Pilots:
        /// Pilots only.
        /// 
        /// </param>
        /// <param name="Burst_Number">
        /// Burst number is to be selected.
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_BSTR_SEL_NUM
        /// 
        /// Note:
        /// 
        /// (1) This control has effect only if Burst Number is select.
        /// </param>
        /// <param name="Symbol_Selection">
        /// Defines the sets the symbol results calculation.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SELECTION_ALL (0) - All
        /// RSSPECAN_VAL_SELECTION_NUM (2) - Symbol Number
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_SELECTION_NUM
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_BSTR_SEL
        /// RSSPECAN_ATTR_WIMAX_BSTR_SEL_NUM
        /// 
        /// </param>
        /// <param name="Symbol_Number">
        /// Defines the carrier number.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 1
        /// 
        /// Attrbibute:
        /// RSSPECAN_ATTR_WIMAX_BSTR_SEL_NUM
        /// 
        /// Note:
        /// (1) This parameter is valid only if Symbol Number is selected in Symbol Selection control
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXBitstreamSelection(int Modulation, int Burst_Selection, int Burst_Number, int Symbol_Selection, int Symbol_Number)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXBitstreamSelection(this._handle, Modulation, Burst_Selection, Burst_Number, Symbol_Selection, Symbol_Number);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the statistics measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_STAT_CCDF
        /// RSSPECAN_ATTR_WIMAX_STAT_BSTR
        /// RSSPECAN_ATTR_WIMAX_STAT_BSUM
        /// 
        /// Remote-control command(s):
        /// CONFigure:BURSt:STATistics:CCDF[:IMMediate]
        /// CONFigure:BURSt:STATistics:BSTReam[:IMMediate]
        /// CONFigure:BURSt:STATistics:BSUMmary[:IMMediate]
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Mode">
        /// Sets the measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_STAT_CCDF (0) - CCDF
        /// RSSPECAN_VAL_WIMAX_STAT_BSTR (1) - Bit Stream
        /// RSSPECAN_VAL_WIMAX_STAT_BSUM (2) - Burst Summary
        /// 
        /// Default Value: RSSPECAN_VAL_WLAN_STAT_CCDF
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_STAT_CCDF
        /// RSSPECAN_ATTR_WIMAX_STAT_BSTR
        /// RSSPECAN_ATTR_WIMAX_STAT_BSUM
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXStatisticsMeasurement(int Mode)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXStatisticsMeasurement(this._handle, Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the IQ Trace parameters. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_SRAT
        /// RSSPECAN_ATTR_WIMAX_FLAT
        /// 
        /// Remote-control command(s):
        /// TRACE:IQ:SRATe
        /// TRACE:IQ:FILTer:FLATness
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Sample_Rate">
        /// Sets the sampling rate for the I/Q data acquisition.
        /// 
        /// Valid Values:
        /// 10.0e3 Hz to 100.0e6 Hz (326.4e6 Hz with option R&amp;S FSQ-B72)
        /// 2.0e+6 to 32.0e+6 Hz for FS-K92 option
        /// 
        /// Default Value: 32.0e6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_SRAT
        /// 
        /// </param>
        /// <param name="Filter">
        /// This control determines whether the wideband detector filters are too be used in preference to the wideband detector board, (B72 option), for a specified sample rate.
        /// 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_IQ_FILT_NORM (0) - Normal 
        /// RSSPECAN_VAL_IQ_FILT_WIDE (1) - Wide
        /// 
        /// Default Value: RSSPECAN_VAL_IQ_FILT_WIDE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_FLAT
        /// 
        /// Note(s):
        /// (1) This value has effect only if option B72 is installed. Otherwise it is ignored.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXIQMeasurementParameters(double Sample_Rate, int Filter)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXIQMeasurementParameters(this._handle, Sample_Rate, Filter);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Spectrum Emission Mask measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_SEM_TYPE
        /// RSSPECAN_ATTR_WIMAX_SEM_FILE
        /// RSSPECAN_ATTR_WIMAX_SEM_CLASS
        /// 
        /// Remote-control command(s):
        /// [SENSe:]POWer:SEM IEEE | ETSI | USER | STANDARD
        /// MMEMory:LOAD:SEM:STATe
        /// [SENSe:]POWer:SEM:CLASs
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Type">
        /// Sets the Spectrum Emission Mask (SEM) measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SEM_IEEE     (0) - IEEE
        /// RSSPECAN_VAL_SEM_ETSI     (1) - ETSI
        /// RSSPECAN_VAL_SEM_USER     (2) - User
        /// RSSPECAN_VAL_SEM_STANDARD (3) - Standard
        /// 
        /// Default Value: RSSPECAN_VAL_SEM_IEEE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_SEM_TYPE
        /// 
        /// </param>
        /// <param name="File_Name">
        /// This control is used to load a K91 spectrum emission mask setup from an xml file.
        /// 
        /// Valid Values:
        /// any valid file name
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_SEM_FILE
        /// 
        /// Note:
        /// 
        /// (1) This control has effect only if Type is set to User.
        /// 
        /// </param>
        /// <param name="Power_Class">
        /// This control sets the Spectrum Emission Mask (SEM) power class index. The index represents the power classes to be applied. The index is directly related to the entries displayed in the power class drop down combo box, within the SEM settings configuration page. 
        /// 
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WLAN_SEM_CLASS
        /// 
        /// Note:
        /// 
        /// (1) Class 0 sets the Power Class to automatic
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXSEMMeasurement(int Type, string File_Name, int Power_Class)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXSEMMeasurement(this._handle, Type, File_Name, Power_Class);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the frame that the Time to Capture Buffer marker is to be set to.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_TTC_FRAME
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:FUNCtion:TTCapture:FRAMe
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Frame">
        /// Sets the frame that the Time to Capture Buffer marker is to be set to.
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_TTC_FRAME
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXTTCFrame(int Frame)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXTTCFrame(this._handle, Frame);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the 'time to capture buffer' calculation to be with respect to either FFT or frame.
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the FSQ-K93 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_TTC_START
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:FUNCtion:TTCapture:STARt FFT | FRAMe
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Start">
        /// Sets the ?time to capture buffer? calculation to be with respect to either FFT or frame. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_TTC_FFT  (0) - FFT
        /// RSSPECAN_VAL_WIMAX_TTC_FRAM (1) - Frame
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_TTC_START
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXTTCStart(int Start)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXTTCStart(this._handle, Start);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the result values of the adjacent channel power measurement.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:FUNCtion:POWer:RESult[:CURRent]?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:FUNCtion:POWer:RESult:MAXHold?
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// Sets the marker. The driver uses this value to set the repeated capability Marker of the attributes.
        /// 
        /// Valid Values:
        /// 1
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Mode">
        /// Sets the ACP measurement results. The driver uses this value to select the attribute to be set.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_ACP_CURR    (0) - Current
        /// RSSPECAN_VAL_ACP_MAX     (1) - Maximum
        /// 
        /// Default Value: RSSPECAN_VAL_ACP_CURR
        /// 
        /// </param>
        /// <param name="Values">
        /// Returns the measured values.
        /// 
        /// Returned Value:
        /// values i dB in order
        /// 
        /// Power of main channel
        /// Power of lower adjacent channel
        /// Power of upper adjacent channel
        /// Power of lower alternate adjacent channel 1
        /// Power of upper alternate adjacent channel 1
        /// Power of lower alternate adjacent channel 2
        /// Power of upper alternate adjacent channel 2
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetWiMAXACPMeasurement(int Window, int Marker, int Mode, double[] Values)
        {
            int pInvokeResult = PInvoke.GetWiMAXACPMeasurement(this._handle, Window, Marker, Mode, Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function loads the IQ data from the specified .iqw file.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_IQ_LOAD
        /// 
        /// Remote-control command(s):
        /// MMEMory:LOAD:IQ:STATe
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="File_Name">
        /// Specifies full path of the file, the IQ data are load from.
        /// 
        /// Valid Values:
        /// any valid file name
        /// 
        /// Default Value: "d:\\user\\data.iqw"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_IQ_LOAD
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int LoadWiMAXIQData(string File_Name)
        {
            int pInvokeResult = PInvoke.LoadWiMAXIQData(this._handle, File_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function stores the IQ data from/to the specified .iqw file.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_IQ_SAVE
        /// 
        /// Remote-control command(s):
        /// MMEMory:STORe:IQ:STATe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="File_Name">
        /// Specifies full path of the file, where IQ data are stored.
        /// 
        /// Valid Values:
        /// any valid file name
        /// 
        /// Default Value: "d:\\user\\data.iqw"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_IQ_SAVE
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int StoreWiMAXIQData(string File_Name)
        {
            int pInvokeResult = PInvoke.StoreWiMAXIQData(this._handle, File_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function forces the IQ measurement results to be recalculated according to the current settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_CALC_BURS
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:BURSt[:IMMediate]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int WiMAXBurstImm()
        {
            int pInvokeResult = PInvoke.WiMAXBurstImm(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function updates the current IQ measurement results to reflect the current measurement settings. Note no new IQ data is captured. I.e. the measurement settings apply to the IQ data being currently in the capture buffer. The command applies exclusively to IQ measurements. It requires available IQ data.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_REFRESH 
        /// 
        /// Remote-control command(s):
        /// INITiate:REFResh
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int WiMAXRefreshIQMeasurement()
        {
            int pInvokeResult = PInvoke.WiMAXRefreshIQMeasurement(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the time to the start of the first frame in the capture buffer.
        /// 
        /// Note:
        /// 
        /// This function is only available with the R&amp;S FSL and FS-K93 option.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_QUERY_TTC
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:FUNCtion:TTCapture?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker_Number">
        /// Selects the marker to configure.
        /// 
        /// Valid Values:
        /// 1 to 1 
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Time">
        /// This control returns the time to the start of the first frame in the capture buffer.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int WiMAXGetMarkerTTC(int Window, int Marker_Number, out double Time)
        {
            int pInvokeResult = PInvoke.WiMAXGetMarkerTTC(this._handle, Window, Marker_Number, out Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns all the measured data that relates to the currently selected measurement type. The returned data is particular to the currently selected measurement type and is specified in the Trace Data parameter description.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRACe[:DATA]? TRACE1 | TRACE2 | TRACE3 | TRACE4 | TRACE5 | TRACE6
        /// FORMat[:DATA] REAL,32
        /// *CLS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Trace">
        /// This parameter indicates which trace will be read out.
        /// 
        /// Valid Values:
        /// 1 to 6
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Array_Size">
        /// Number of array points requested.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 256
        /// </param>
        /// <param name="Trace_Data">
        /// Returns an array of trace data. Number and format of the measurement values for operating mode WiMAX:
        /// 
        /// --- IQ Measurements:
        /// 
        /// There are a number of measurements that can be performed in IQ mode. No data will be returned for any of the following measurements, should it be requested, until such time as a measurement belonging to the IQ group has been previously run. Running a frequency sweep measurement for example, Spectrum Mask, will not generate results for this measurement group.
        /// 
        /// --- Constellation vs Symbol:
        /// 
        /// This measurement represents I and Q data. Data will be returned as a repeating array of interleaved I and Q data in groups of selected carriers, until all the data is exhausted.
        /// Each I and Q point will be returned in floating point format. TRACE1 is used for this measurement results.
        /// If "All Carriers" is selected, it will return 52 per of I and Q data per symbol.
        /// If "Pilots Only" is selected, it will return 4 per of I and Q per symbol in the following order: Carrier ? 21, Carrier ?7, Carrier 7, Carrier 21.
        /// If a single carrier is selected, it will return 1 per of I and Q data per symbol.
        /// For 802.16e/D11 OFDMS the following rules apply:
        /// The results are retuned in repeating groups of the FFT size, for all symbols.
        /// For example if the FFT size was 1024 and 12 symbols were found then 12288 I/Q pairs worth of data would be returned. Carriers that do not exist or are filtered out by the current filter settings, are denoted by the text "NAN".
        /// 
        /// --- Constellation vs Carrier:
        /// 
        /// This measurement represents I and Q data. Data will be returned as a repeating array of interleaved I and Q data in groups of 53 channels including the channel 0, until all the data is exhausted. Each I and Q point will be returned in floating point format. TRACE1 is used for this measurement results.
        /// 
        /// --- Power vs Time ? Full Burst and Start / End data
        /// 
        /// This description applies to measurement results from the 802.16 2004 OFDM standard. Both measurement results are once again simply slightly different views of the same results data.
        /// All fully complete bursts within the capture time are analyzed. This data is returned in dBm values to the user on a per sample basis. Each sample will in some way relate to an analyzis of each corresponding sample within each processed burst. The type of PVT data returned will be determined by the TRACE number passed as an argument to the SCPI command, in addition to the
        /// graphic type that is selected.
        /// If the graphic type selected is full burst, then the return data is as follows.
        /// TRACE1 ? full burst, burst data values.
        /// If the graphic type selected is rising/falling, then the return data is as follows.
        /// TRACE1 ? start, burst data values.
        /// TRACE2 ? end, burst data values.
        /// The number of samples returned during full burst analysis will depend on the modulation type and will typically be 5000.
        /// The number of samples returned when the rising and falling graphic type is selected will be less than what is returned for full burst and will be approximately 400 samples.
        /// The samples will be returned in floating point format as a single sequence of comma delimited values.
        /// 
        /// --- Power vs Time - Full Burst and Rising / falling data:
        /// 
        /// This description applies to measurement results from the 802.16e/D11 standard. Both measurement results are once again simply slightly different views of the same results data.
        /// All fully complete frames within the capture time are analyzed into three master frames. The three master frames relate to the minimum, maximum and average values across all complete frames.
        /// This data is returned in dBm values to the user on a per sample basis. Each sample will in some way relate to an analysis of each corresponding sample within each processed frame. The type of PVT data returned will be determined by the TRACE number passed as an argument to the SCPI command, in addition to the graphic type that is selected.
        /// 
        /// If the graphic type selected is full burst, then the return data is as follows.
        /// TRACE1 ? full sub frame, minimum frame data values.
        /// TRACE2 ? full sub frame, mean frame data values.
        /// TRACE3 ? full sub frame, maximum frame data values.
        /// If the graphic type selected is rising/falling, then the return data is as follows.
        /// TRACE1 ? rising edge, minimum frame data values.
        /// TRACE2 ? rising edge, mean frame data values.
        /// TRACE3 ? rising edge, maximum frame data values.
        /// TRACE4 ? falling edge, minimum frame data values.
        /// TRACE5 ? falling edge, mean frame data values.
        /// TRACE6 ? falling edge, maximum frame data values.
        /// The number of samples returned during full frame analysis will depend on the modulation type and will typically be 5000.
        /// The number of samples returned when the start / end graphic type is selected will be less than what is returned for full burst and will be approximately 400 samples.
        /// The samples will be returned in floating point format as a single sequence of comma delimited values.
        /// 
        /// --- Spectrum Flatness:
        /// 
        /// There are four separate traces that are available with this measurement. Trace data for a particular trace will only be returnable by querying the appropriate trace Spectrum flatness provides two basic graph types. These are an absolute power value graph (ABS) and a relative group delay graph, which are both plotted on a per carrier basis. All 52 carriers are drawn, in addition to the unused 0 carrier. Both the absolute power and group delay graph groups will allow all the data points to be returned as one trace and an average of all the channels as the other trace. E.g. the return data will either be one single group of 53 carriers if the average trace is selected or a repeating group of 53 carriers if all the data is requested.
        /// 
        ///   TRACE1 - ABS (all analyzed trains)
        ///   TRACE2 - Group Delay (all analyzed trains)
        ///   TRACE3 - ABS (average trace)
        ///   TRACE4 - Group Delay (average trace)
        /// 
        /// Absolute power results are returned in dB and group delay results are returned in ns.
        /// 
        /// --- Spectrum FFT:
        /// 
        /// All FFT points will be returned if the data for this measurement is requested. This will be an exhaustive call, due to the fact that there are nearly always more FFT points than IQ samples. The number of FFT points is the number presented by a power of 2 that is higher than the total number of samples.
        /// e.g. if there were 20000 samples, then 32768 FFT points would be returned. Data will be returned in floating point format in dBm. TRACE1 is used for this measurement results.
        /// 
        /// --- Statistics Bitstream data:
        /// 
        /// Data will be returned depends on the selected standard from which the measurement was executed.
        /// For the 802.16 2004 OFDM standard, data is returned in repeating groups of 200 data channels where each symbol value will be represented by an integer value within one byte. Channel 0 is
        /// unused and will therefore not have any data associated with it, with no return data being provided. For the 802e16e/D11 OFDMA standard, the data is returned in groups of the selected FFT size, where each symbol value will be represented by an integer value within one byte. The FFT size is either, 128, 512, 1024, 2048 sub carriers. Unused carriers, including the zero carrier, are also retuned and these are denoted by the string value NAN.
        /// The number of repeating groups that are returned will be equal to the number of measured symbols. 64QAM has the highest data rate and it contains symbol values up to 63, making one byte sufficient in size to represent all symbol data values, regardless of the modulation type in use.
        /// Data will be returned in ASCII printable hexadecimal character format. TRACE1 is used for this measurement results.
        /// 
        /// --- Statistics Burst Summary data
        /// 
        /// The return data depends on the current standard and measurement results. 
        /// For the IEEE 802.16 2004 OFDM standard, the data will be returned in repeating groups of 6 comma separated values as follows:
        /// - 1st value ? Burst number. If this value is 0 then it is an FCH burst.
        /// - 2nd value ? Area where:
        ///   0 = Preamble
        ///   1 = Data.
        /// - 3rd value ? Modulation where:
        ///   0 = BPSK
        ///   1 = QPSK
        ///   2 = 16QAM
        ///   3 = 64QAM
        /// - 4th value ? Symbol length. This is an integer value giving the number of symbols in the current area.
        /// - 5th value ? Power in dBm for the current area. This is returned as a float.
        /// - 6th value ? EVM in dB for the current area. This is returned as a float.
        /// 
        /// For the IEEE 802.16e/D11 OFDMA standard, the data will be returned in repeating groups of 7 comma separated values as follows:
        /// - 1st value ? Sub Frame number.
        /// - 2nd value ? Burst number.
        /// - 3rd value ? Burst Type:
        ///   1 = FCH.
        ///   2 = DL Map.
        ///   3 = UL Map.
        ///   4 = Data.
        /// - 4th value ? Modulation where:
        ///   1 = QPSK
        ///   2 = 16QAM
        ///   3 = 64QAM
        /// - 5th value ? Number of slots. This is an integer value giving the number of slots associated with the burst.
        /// - 6th value ? Power in dBm for the current area. This is returned as a float.
        /// - 7th value ? EVM in dB for the current area. This is returned as a float.
        /// 
        /// --- Statistics CCDF - Complementary cumulative distribution function:
        /// 
        /// Up to a maximum of 201 data points will be returned in addition to a data count value. The first value in the return data will represent the quantity of probability values that follow. Each of the potential 201 data points will be returned as probability value and will represent the total number of samples that are equal to or exceed the corresponding power level. Probability data will be returned up to the power level that contains at least one sample. It is highly unlikely that the full 201 data values will ever be returned.
        /// Each probability value will be returned as a floating point number, with a value less than 1.
        /// 
        /// --- EVM vs Carrier:
        /// 
        /// Two trace types are provided with this measurement. There is an average EVM value for each of the 53 channels or a repeating group of EVM values for each channel. The number of repeating groups will correspond to the number of fully analyzed trains.
        /// Each EVM value will be returned as a floating point number, expressed in units of dBm.
        /// 
        /// TRACE1 ? Average EVM values per channel
        /// TRACE2 ? All EVM values per channel for each full train of the capture period
        /// 
        /// For the 802.16e/D11 standard, the number of sub carriers returned varies according to the FFT size. Each EVM value will be returned as a floating point number, expressed in units of dBm or percentage.
        /// 
        /// TRACE1 ? Minimum EVM values
        /// TRACE2 ? Mean EVM values
        /// TRACE3 ? Maximum EVM values
        /// 
        /// --- EVM vs Symbol:
        /// 
        /// Three traces types are available with this measurement. The basic trace types show either the minimum, mean or maximum EVM value, as measured over the complete capture period. The number of repeating groups that are returned will be equal to the number of measured symbols. Each EVM value will be returned as a floating point number, expressed in units of dBm.
        /// 
        /// TRACE1 ? Minimum EVM values
        /// TRACE2 ? Mean EVM values
        /// TRACE3 ? Maximum EVM values
        /// 
        /// --- Frequency Sweep Measurements:
        /// 
        /// Currently, there is only one measurement that is performed in frequency sweep mode. This is the Spectrum Mask measurement. No data will be returned for this measurement, should it be
        /// requested, until such time as a measurement has been previously run. Running an IQ measurement will not generate results for this measurement.
        /// 
        /// --- Spectrum Mask:
        /// 
        /// Result data will be returned as 625 trace points for R&amp;S FSQ or 501 trace points for R&amp;S FSP in floating point format. These trace points are obtained directly from the base system via the
        /// measurement API and the quantity is therefore a fixed value. Only an array of Y data will be returned.
        /// 
        /// TRACE1 ? Clear write values
        /// TRACE2 ? Max hold values
        /// 
        /// --- Spectrum ACPR:
        /// 
        /// Result data will be returned as 625 trace points for R&amp;S or 501 trace points for R&amp;S FSP in floating point format. These trace points are obtained directly from the base system via the measurement API and the quantity is therefore a fixed value. Only an array of Y data will be returned.
        /// 
        /// TRACE1 ? Clear write values
        /// TRACE2 ? Max hold values
        /// 
        /// </param>
        /// <param name="No_of_Values">
        /// Returns the number of trace data values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadWiMAXTraceData(int Trace, int Array_Size, double[] Trace_Data, out int No_of_Values)
        {
            int pInvokeResult = PInvoke.ReadWiMAXTraceData(this._handle, Trace, Array_Size, Trace_Data, out No_of_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns all the IQ data associated with the measurement acquisition time. The result values are scaled linear in unit Volt and correspond to the voltage at the RF input of the instrument. The command returns a comma-separated list of the measured voltage values in floating point format (Comma Separated Values = CSV). The number of values returned is 2 * number of samples, the first half being the I values, the second half the Q values.
        /// 
        /// Notes:
        /// 
        /// (1) If there are not I/Q data available in memory because the corresponding measurement had not been started, the function will cause a Query Error.
        /// 
        /// Remote-control command(s):
        /// TRACe:IQ:DATA:MEMory?
        /// *CLS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Offset_Samples">
        /// This control sets offset of the values to be read related to the start of the acquired data.
        /// 
        /// Valid Values: 0 to &lt;# of samples&gt; - 1
        /// 
        /// Default Value: 0
        /// 
        /// Note:
        /// 
        /// Where &lt;# of samples&gt; is the value indicated with function Trace I/Q Set (rsspecan_TraceIQSet).
        /// 
        /// </param>
        /// <param name="No_of_Samples">
        /// This control sets the number of measurement values to be read.
        /// 
        /// Valid Values: 1 to &lt;# of samples&gt; - &lt;offset samples&gt;
        /// 
        /// Default Value: 1
        /// 
        /// Note:
        /// 
        /// Where &lt;# of samples&gt; is the value indicated with function Trace I/Q Set (rsspecan_TraceIQSet).
        /// 
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of doubles in the array you specify for the real parts and imaginary parameters.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the data contains more elemens than you specify with the Buffer Size parameter, the function returns Buffer Size elements into the arrays and returns the buffer size you must pass to get the all data.  For example, if the data contains 16000 elements for real parts and 16000 elements for imaginary part and the Buffer Size is 10000, the function places 10000 elements into the arrays and returns 16000.
        /// 
        /// Notes:
        /// 
        /// (1) If VI_ATTR_INTF_TYPE is equal to VI_INTF_ASRL, then use this parameter to pass the expected number of points to be read from the instrument. Data points will be read until instruments output buffer is empty. If Buffer Size is lower than actual number of results available in the instruments output buffer, then this function returns an error code (BFFC0002  Parameter 2 (Buffer Size) out of range.) and measurement results will not be valid. This is due to limitation if the unit is remote-controlled via the RS-232-C interface.
        /// 
        /// (2) This function requires that all response data are read out completely before the instrument accepts further commands.
        /// 
        /// </param>
        /// <param name="No_of_Points">
        /// Returns the number of I/Q trace data points.
        /// 
        /// </param>
        /// <param name="Real_Parts__I_">
        /// Returns an array of trace data (I-value x, real part) point. The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter.
        /// 
        /// </param>
        /// <param name="Imaginary_Parts__Q_">
        /// Returns an array of trace data (Q-value y, imaginary part) point. The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadWiMAXMemoryIQData(int Offset_Samples, int No_of_Samples, int Buffer_Size, out int No_of_Points, double[] Real_Parts__I_, double[] Imaginary_Parts__Q_)
        {
            int pInvokeResult = PInvoke.ReadWiMAXMemoryIQData(this._handle, Offset_Samples, No_of_Samples, Buffer_Size, out No_of_Points, Real_Parts__I_, Imaginary_Parts__Q_);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the Spectrum Emission Mask summary results.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:DATA? LIST
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="No_Of_Values">
        /// This control sets the number of values returned in arrays.
        /// The array size has to be at least equal to this number.
        /// 
        /// Valid Values: &gt; 0
        /// 
        /// Default Value: 10
        /// 
        /// </param>
        /// <param name="Index">
        /// Returns an array of index into table of results.
        /// 
        /// 
        /// </param>
        /// <param name="Start_Frequency_Band">
        /// Returns an array of start frequency band in Hz.
        /// 
        /// </param>
        /// <param name="Stop_Frequency_Band">
        /// Returns an array of stop frequency band in Hz.
        /// </param>
        /// <param name="Resolution_Bandwidth">
        /// Returns an array of resolution bandwidth
        /// </param>
        /// <param name="Limit_Fail_Frequency">
        /// Returns an array of limit fail frequency.
        /// 
        /// 
        /// </param>
        /// <param name="Power_Abs">
        /// Returns an array of absolute power in dBm.
        /// 
        /// </param>
        /// <param name="Power_Rel">
        /// Returns the relative power in dBc.
        /// </param>
        /// <param name="Limit_Distance">
        /// Returns an array of limit distance in dB.
        /// </param>
        /// <param name="Failure_Flag">
        /// Returns an array of failure flag (0 = PASSED, 1 = FAILED
        /// </param>
        /// <param name="Returned_Values">
        /// Returns number of value sets (frequency, level and delta to limit line) read (available to return) from the instrument. Number of values returned in arrays is limited with the parameter No of Values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadWiMAXSEMResults(int No_Of_Values, double[] Index, double[] Start_Frequency_Band, double[] Stop_Frequency_Band, double[] Resolution_Bandwidth, double[] Limit_Fail_Frequency, double[] Power_Abs, double[] Power_Rel, double[] Limit_Distance, double[] Failure_Flag, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.ReadWiMAXSEMResults(this._handle, No_Of_Values, Index, Start_Frequency_Band, Stop_Frequency_Band, Resolution_Bandwidth, Limit_Fail_Frequency, Power_Abs, Power_Rel, Limit_Distance, Failure_Flag, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns all the results.
        /// 
        /// Note:
        /// (1) If no measurement has been performed yet, a query error will occur.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:ALL?
        /// *CLS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Results">
        /// This control returns all the results. Up to 59 values can be returned! Be sure, that the allocated memory has correct size.
        /// 
        /// Results are returned in order:
        /// for FSQ-K92
        /// Idx 0 - min EVM all bursts
        /// Idx 1 - average EVM all bursts
        /// Idx 2 - max EVM all bursts      
        /// Idx 3 - min EVM data carriers
        /// Idx 4 - average EVM data carriers 
        /// Idx 5 - max EVM data carriers 
        /// Idx 6 - min EVM pilots
        /// Idx 7 - average EVM pilots 
        /// Idx 8 - max EVM pilots     
        /// Idx 9 - min IQ offset
        /// Idx 10 - average IQ offset
        /// Idx 11 - maximum IQ offset    
        /// Idx 12 - min gain imbalance
        /// Idx 13 - average gain imbalance
        /// Idx 14 - max gain imbalance    
        /// Idx 15 - min quadrature offset
        /// Idx 16 - average quadrature offset
        /// Idx 17 - max quadrature offset  
        /// Idx 18 - min frequency error
        /// Idx 19 - average frequency error
        /// Idx 20 - max frequency error    
        /// Idx 21 - min symbol error
        /// Idx 22 - average symbol error
        /// Idx 23 - max symbol error    
        /// Idx 24 - min burst power
        /// Idx 25 - average burst power
        /// Idx 26 - max burst power    
        /// Idx 27 - min crest factor
        /// Idx 28 - average crest factor
        /// Idx 29 - max crest factor  
        /// Idx 30 - min SS timing
        /// Idx 31 - average SS timing
        /// Idx 32 - max SS timing   
        /// Idx 33 - min RSSI
        /// Idx 34 - average RSSI
        /// Idx 35 - max RSSI   
        /// Idx 36 - min RSSI SD
        /// Idx 37 - average RSSI SD
        /// Idx 38 - max RSSI SD   
        /// Idx 39 - min CINR
        /// Idx 40 - average CINR
        /// Idx 41 - max CINR
        /// Idx 42 - min CINR SD
        /// Idx 43 - average CINR SD
        /// Idx 44 - max CINR SD
        /// 
        /// FSQ-K93 OFDM:
        /// Idx 0 - min EVM all bursts
        /// Idx 1 - average EVM all bursts
        /// Idx 2 - max EVM all bursts
        /// Idx 3 - min EVM data carriers
        /// Idx 4 - average EVM data carriers
        /// Idx 5 - max EVM data carriers
        /// Idx 6 - min EVM pilots
        /// Idx 7 - average EVM pilots
        /// Idx 8 - max EVM pilots
        /// Idx 9 - min IQ offset
        /// Idx 10 - average IQ offset
        /// Idx 11 - maximum IQ offset
        /// Idx 12 - min gain imbalance
        /// Idx 13 - average gain imbalance
        /// Idx 14 - max gain imbalance
        /// Idx 15 - min quadrature offset
        /// Idx 16 - average quadrature offset
        /// Idx 17 - max quadrature offset
        /// Idx 18 - min frequency error
        /// Idx 19 - average frequency error
        /// Idx 20 - max frequency error
        /// Idx 21 - min symbol error
        /// Idx 22 - average symbol error
        /// Idx 23 - max symbol error
        /// Idx 24 - min burst power
        /// Idx 25 - average burst power
        /// Idx 26 - max burst power
        /// Idx 27 - min crest factor
        /// Idx 28 - average crest factor
        /// Idx 29 - max crest factor
        /// Idx 30 - min SS timing
        /// Idx 31 - average SS timing
        /// Idx 32 - max SS timing
        /// Idx 33 - min RSSI
        /// Idx 34 - average RSSI
        /// Idx 35 - max RSSI
        /// Idx 36 - min RSSI SD
        /// Idx 37 - average RSSI SD
        /// Idx 38 - max RSSI SD
        /// Idx 39 - min CINR
        /// Idx 40 - average CINR
        /// Idx 41 - max CINR
        /// Idx 42 - min CINR SD
        /// Idx 43 - average CINR SD
        /// Idx 44 - max CINR SD
        /// 
        /// FSQ-K93, OFMDA Uplink:
        /// Idx 0 - min BER pilots
        /// Idx 1 - average BER pilots
        /// Idx 2 - max BER pilots
        /// Idx 3 - min EVM all bursts
        /// Idx 4 - average EVM all bursts
        /// Idx 5  - max EVM all bursts
        /// Idx 6 - min EVM data carriers
        /// Idx 7 - average EVM data carriers
        /// Idx 8 - max EVM data carriers
        /// Idx 9 - min EVM pilots
        /// Idx 10 - average EVM pilots
        /// Idx 11 - max EVM pilots
        /// Idx 12 - min IQ offset
        /// Idx 13 - average IQ offset
        /// Idx 14 - maximum IQ offset
        /// Idx 15 - min gain imbalance
        /// Idx 16 - average gain imbalance
        /// Idx 17 - max gain imbalance
        /// Idx 18 - min quadrature offset
        /// Idx 19 - average quadrature offset
        /// Idx 20 - max quadrature offset
        /// Idx 21 - min frequency error
        /// Idx 22 - average frequency error
        /// Idx 23 - max frequency error
        /// Idx 24 - min symbol error
        /// Idx 25 - average symbol error
        /// Idx 26 - max symbol error
        /// Idx 27 - min power all
        /// Idx 28 - average power all
        /// Idx 29 - max power all
        /// Idx 30 - min power data
        /// Idx 31 - average power data
        /// Idx 32 - max power data
        /// Idx 33 - min power pilots
        /// Idx 34 - average power pilots
        /// Idx 35 - max power pilots
        /// Idx 36 - min crest factor
        /// Idx 37 - average crest factor
        /// Idx 38 - max crest factor
        /// Idx 39 - min unmod. subcarrier error
        /// Idx 40 - average unmod. subcarrier error
        /// Idx 41 - max unmod. subcarrier error
        /// 
        /// FSQ-K93, MIMO Uplink:
        /// Idx 0 - min BER pilots
        /// Idx 1 - average BER pilots
        /// Idx 2 - max BER pilots
        /// Idx 3 - min EVM all bursts
        /// Idx 4 - average EVM all bursts
        /// Idx 5 - max EVM all bursts
        /// Idx 6 - min EVM data carriers
        /// Idx 7 - average EVM data carriers
        /// Idx 8 - max EVM data carriers
        /// Idx 9 - min EVM pilots
        /// Idx 10 - average EVM pilots
        /// Idx 11 - max EVM pilots
        /// Idx 12 - min IQ offset
        /// Idx 13 - average IQ offset
        /// Idx 14 - maximum IQ offset
        /// Idx 15 - min gain imbalance
        /// Idx 16 - average gain imbalance
        /// Idx 17 - max gain imbalance
        /// Idx 18 - min quadrature offset
        /// Idx 19 - average quadrature offset
        /// Idx 20 - max quadrature offset
        /// Idx 21 - min frequency error
        /// Idx 22 - average frequency error
        /// Idx 23 - max frequency error
        /// Idx 24 - min symbol error
        /// Idx 25 - average symbol error
        /// Idx 26 - max symbol error
        /// Idx 27 - min power all
        /// Idx 28 - average power all
        /// Idx 29 - max power all
        /// Idx 30 - min power data
        /// Idx 31 - average power data
        /// Idx 32 - max power data
        /// Idx 33 - min power pilots
        /// Idx 34 - average power pilots
        /// Idx 35 - max power pilots
        /// Idx 36 - min power null pilots
        /// Idx 37 - average power null pilots
        /// Idx 38 - max power null pilots
        /// Idx 39 - min crest factor
        /// Idx 40 - average crest factor
        /// Idx 41 - max crest factor
        /// Idx 42 - min unmod. subcarrier error
        /// Idx 43 - average unmod. subcarrier error
        /// Idx 44 - max unmod. subcarrier error
        /// 
        /// FSQ-K93, OFDMA Downlink:
        /// Idx 0 - min BER pilots
        /// Idx 1 - average BER pilots
        /// Idx 2 - max BER pilots
        /// Idx 3 - min EVM all bursts
        /// Idx 4 - average EVM all bursts
        /// Idx 5 - max EVM all bursts
        /// Idx 6 - min EVM data carriers
        /// Idx 7 - average EVM data carriers
        /// Idx 8 - max EVM data carriers
        /// Idx 9 - min EVM pilots
        /// Idx 10 - average EVM pilots
        /// Idx 11 - max EVM pilots
        /// Idx 12 - min IQ offset
        /// Idx 13 - average IQ offset
        /// Idx 14 - maximum IQ offset
        /// Idx 15 - min gain imbalance
        /// Idx 16 - average gain imbalance
        /// Idx 17 - max gain imbalance
        /// Idx 18 - min quadrature offset
        /// Idx 19 - average quadrature offset
        /// Idx 20 - max quadrature offset
        /// Idx 21 - min frequency error
        /// Idx 22 - average frequency error
        /// Idx 23 - max frequency error
        /// Idx 24 - min symbol error
        /// Idx 25 - average symbol error
        /// Idx 26 - max symbol error
        /// Idx 27 - min power DL preamble
        /// Idx 28 - average power DL preamble
        /// Idx 29 - max power DL preamble
        /// Idx 30 - min power all
        /// Idx 31 - average power all
        /// Idx 32 - max power all
        /// Idx 33 - min power data
        /// Idx 34 - average power data
        /// Idx 35 - max power data
        /// Idx 36 - min power pilots
        /// Idx 37 - average power pilots
        /// Idx 38 - max power pilots
        /// Idx 39 - min crest factor
        /// Idx 40 - average crest factor
        /// Idx 41 - max crest factor
        /// Idx 42 - min RSSI
        /// Idx 43 - average RSSI
        /// Idx 44 - max RSSI
        /// Idx 45 - RSSI SD
        /// Idx 46 - min CINR
        /// Idx 47 - average CINR
        /// Idx 48 - max CINR
        /// Idx 49 - CINR SD
        /// Idx 50 - min unmod. subcarrier error
        /// Idx 51 - average unmod. subcarrier error
        /// Idx 52 - max unmod. subcarrier error
        /// Idx 53 - min EVM preamble
        /// Idx 54 - average EVM preamble
        /// Idx 55 - max EVM preamble
        /// 
        /// FSQ-K93, MIMO Downlink:
        /// Idx 0 - min BER pilots
        /// Idx 1 - average BER pilots
        /// Idx 2 - max BER pilots
        /// Idx 3 - min EVM all bursts
        /// Idx 4 - average EVM all bursts
        /// Idx 5 - max EVM all bursts
        /// Idx 6 - min EVM data carriers
        /// Idx 7 - average EVM data carriers
        /// Idx 8 - max EVM data carriers
        /// Idx 9 - min EVM pilots
        /// Idx 10 - average EVM pilots
        /// Idx 11 - max EVM pilots
        /// Idx 12 - min IQ offset
        /// Idx 13 - average IQ offset
        /// Idx 14 - maximum IQ offset
        /// Idx 15 - min gain imbalance
        /// Idx 16 - average gain imbalance
        /// Idx 17 - max gain imbalance
        /// Idx 18 - min quadrature offset
        /// Idx 19 - average quadrature offset
        /// Idx 20 - max quadrature offset
        /// Idx 21 - min frequency error
        /// Idx 22 - average frequency error
        /// Idx 23 - max frequency error
        /// Idx 24 - min symbol error
        /// Idx 25 - average symbol error
        /// Idx 26 - max symbol error
        /// Idx 27 - min power DL preamble
        /// Idx 28 - average power DL preamble
        /// Idx 29 - max power DL preamble
        /// Idx 30 - min power all
        /// Idx 31 - average power all
        /// Idx 32 - max power all
        /// Idx 33 - min power data
        /// Idx 34 - average power data
        /// Idx 35 - max power data
        /// Idx 36 - min power pilots
        /// Idx 37 - average power pilots
        /// Idx 38 - max power pilots
        /// Idx 39 - min power null pilots
        /// Idx 40 - average power null pilots
        /// Idx 41 - max power null pilots 
        /// Idx 42 - min crest factor
        /// Idx 43 - average crest factor
        /// Idx 44 - max crest factor
        /// Idx 45 - min RSSI
        /// Idx 46 - average RSSI
        /// Idx 47 - max RSSI
        /// Idx 48 - RSSI SD 
        /// Idx 49 - min CINR
        /// Idx 50 - average CINR
        /// Idx 51 - max CINR
        /// Idx 52 - CINR SD
        /// Idx 53 - min unmod. subcarrier error
        /// Idx 54 - average unmod. subcarrier error
        /// Idx 55 - max unmod. subcarrier error
        /// Idx 56 - min EVM preamble
        /// Idx 57 - average EVM preamble
        /// Idx 58 - max EVM preamble
        /// </param>
        /// <param name="Returned_Values">
        /// Returns the number of values in Results array.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchWiMAXBurstAll(double[] Results, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.FetchWiMAXBurstAll(this._handle, Results, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the power measurement value, measured during the measurement.
        /// 
        /// Note:
        /// (1) If no measurement has been performed yet, a query error will occur.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_FETC_RMS
        /// RSSPECAN_ATTR_WIMAX_FETC_CRES
        /// RSSPECAN_ATTR_WIMAX_FETC_RMS_ALL
        /// RSSPECAN_ATTR_WIMAX_FETC_RMS_PILOT
        /// RSSPECAN_ATTR_WIMAX_FETC_RMS_DATA
        /// RSSPECAN_ATTR_WIMAX_FETC_RMS_DLPR
        /// RSSPECAN_ATTR_WIMAX_FETCH_NPIL_RMS
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:RMS:MINimum?
        /// FETCh:BURSt:RMS:MAXimum?
        /// FETCh:BURSt:RMS:AVERage?
        /// FETCh:BURSt:CRESt:MINimum?
        /// FETCh:BURSt:CRESt:MAXimum?
        /// FETCh:BURSt:CRESt[:AVERage]?
        /// FETCh:BURSt:RMS:ALL:AVERage?
        /// FETCh:BURSt:RMS:ALL:MINimum?
        /// FETCh:BURSt:RMS:ALL:MAXimum?
        /// FETCh:BURSt:RMS:PILot:AVERage?
        /// FETCh:BURSt:RMS:PILot:MINimum?
        /// FETCh:BURSt:RMS:PILot:MAXimum?
        /// FETCh:BURSt:RMS:DATA:AVERage?
        /// FETCh:BURSt:RMS:DATA:MINimum?
        /// FETCh:BURSt:RMS:DATA:MAXimum?
        /// FETCh:BURSt:RMS:DLPR:AVERage?
        /// FETCh:BURSt:RMS:DLPR:MINimum?
        /// FETCh:BURSt:RMS:DLPR:MAXimum?
        /// FETCh:BURSt:RMS:NPILot:AVERage?
        /// FETCh:BURSt:RMS:NPILot:MINimum?
        /// FETCh:BURSt:RMS:NPILot:MAXimum?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Power_Measurement">
        /// Sets the power measurement. The driver uses this value to select the attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_MEAS_RMS       (0) - RMS
        /// RSSPECAN_VAL_WIMAX_MEAS_CREST     (1) - Crest Factor
        /// RSSPECAN_VAL_WIMAX_MEAS_RMS_ALL   (2) - RMS All Carriers
        /// RSSPECAN_VAL_WIMAX_MEAS_RMS_PILOT (3) - RMS Pilot Carriers
        /// RSSPECAN_VAL_WIMAX_MEAS_RMS_DATA  (4) - RMS Data Carriers
        /// RSSPECAN_VAL_WIMAX_MEAS_RMS_DLPR  (5) - RMS Downlink Preamble
        /// RSSPECAN_VAL_WIMAX_MEAS_RMS_NPIL  (6) - RMS Null Pilot Carriers
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_MEAS_RMS
        /// 
        /// 
        /// </param>
        /// <param name="Mode">
        /// Sets the measurement mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MAX
        /// 
        /// </param>
        /// <param name="Value">
        /// This control returns the selected power value .
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchWiMAXBurstPower(int Power_Measurement, int Mode, out double Value)
        {
            int pInvokeResult = PInvoke.FetchWiMAXBurstPower(this._handle, Power_Measurement, Mode, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the burst errors.
        /// 
        /// Note:
        /// (1) If no measurement has been performed yet, a query error will occur.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_FETC_FERR
        /// RSSPECAN_ATTR_WIMAX_FETC_SYMB
        /// 
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:FERRor:AVERage?
        /// FETCh:BURSt:FERRor:MINimum?
        /// FETCh:BURSt:FERRor:MAXimum?
        /// FETCh:BURSt:SYMBolerror:AVERage?
        /// FETCh:BURSt:SYMBolerror:MINimum?
        /// FETCh:BURSt:SYMBolerror:MAXimum?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Error_Type">
        /// Sets the power measurement. The driver uses this value to select teh attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_ERR_FREQ (0) - Frequency
        /// RSSPECAN_VAL_WIMAX_ERR_SYMB (1) - Symbol
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_ERR_FREQ
        /// 
        /// 
        /// </param>
        /// <param name="Mode">
        /// Sets the measurement mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MAX
        /// 
        /// </param>
        /// <param name="Value">
        /// This control returns the selected power value .
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchWiMAXBurstError(int Error_Type, int Mode, out double Value)
        {
            int pInvokeResult = PInvoke.FetchWiMAXBurstError(this._handle, Error_Type, Mode, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the burst errors.
        /// 
        /// Note:
        /// (1) If no measurement has been performed yet, a query error will occur.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_FETC_IQOF
        /// RSSPECAN_ATTR_WIMAX_FETC_GIMB
        /// RSSPECAN_ATTR_WIMAX_FETC_QUAD
        /// 
        /// Remote-control command(s):
        /// *CLS
        /// FETCh:BURSt:IQOFfset:AVERage?
        /// FETCh:BURSt:IQOFfset:MAXimum?
        /// FETCh:BURSt:IQOFfset:MINimum?
        /// FETCh:BURSt:GIMBalance:AVERage?
        /// FETCh:BURSt:GIMBalance:MAXimum?
        /// FETCh:BURSt:GIMBalance:MINimum?
        /// FETCh:BURSt:QUADoffset:AVERage?
        /// FETCh:BURSt:QUADoffset:MAXimum?
        /// FETCh:BURSt:QUADoffset:MINimum?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="IQ_Impairment">
        /// Sets the IQ Impairment. The driver uses this value to select the attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_IQ_OFFS (0) - IQ Offset
        /// RSSPECAN_VAL_WIMAX_IQ_IMBA (1) - Gain Imbalance
        /// RSSPECAN_VAL_WIMAX_IQ_QUAD (2) - Quadrature Offset
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_IQ_OFFS
        /// 
        /// 
        /// 
        /// 
        /// </param>
        /// <param name="Mode">
        /// Sets the measurement mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MAX
        /// 
        /// </param>
        /// <param name="Value">
        /// This control returns the selected power value .
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchWiMAXIQImp(int IQ_Impairment, int Mode, out double Value)
        {
            int pInvokeResult = PInvoke.FetchWiMAXIQImp(this._handle, IQ_Impairment, Mode, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the EVM measurement value.
        /// 
        /// Note:
        /// (1) If no measurement has been performed yet, a query error will occur.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_FETC_EVM_ALL
        /// RSSPECAN_ATTR_WIMAX_FETC_EVM_DATA
        /// RSSPECAN_ATTR_WIMAX_FETC_EVM_PILOT
        /// RSSPECAN_ATTR_WIMAX_FETC_EVM
        /// RSSPECAN_ATTR_WIMAX_FETC_EVM_PREAMBLE
        /// 
        /// Remote-control command(s):
        /// *CLS
        /// FETCh:BURSt:EVM:ALL:AVERage?
        /// FETCh:BURSt:EVM:ALL:MAXimum?
        /// FETCh:BURSt:EVM:ALL:MINimum?
        /// FETCh:BURSt:EVM:DATA:AVERage?
        /// FETCh:BURSt:EVM:DATA:MAXimum?
        /// FETCh:BURSt:EVM:DATA:MINimum?
        /// FETCh:BURSt:EVM:PILot:AVERage?
        /// FETCh:BURSt:EVM:PILot:MAXimum?
        /// FETCh:BURSt:EVM:PILot:MINimum?
        /// FETCh:BURSt:EVM:AVERage?
        /// FETCh:BURSt:EVM:MAXimum?
        /// FETCh:BURSt:EVM:MINimum?
        /// FETCh:BURSt:EVM:PREamble:AVERage?
        /// FETCh:BURSt:EVM:PREamble:MINimum?
        /// FETCh:BURSt:EVM:PREamble:MAXimum?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="EVM">
        /// Sets the EVM measurement. The driver uses this value to select teh attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_EVM_ALL   (0) - All Carriers
        /// RSSPECAN_VAL_WIMAX_EVM_DATA  (1) - Data Carriers
        /// RSSPECAN_VAL_WIMAX_EVM_PILOT (2) - Pilot Carriers
        /// RSSPECAN_VAL_WIMAX_EVM       (3) - EVM without RX Filters
        /// RSSPECAN_VAL_WIMAX_EVM_PREAM (4) - Preamble Carriers
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_EVM_ALL
        /// 
        /// 
        /// </param>
        /// <param name="Mode">
        /// Sets the measurement mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MAX
        /// 
        /// </param>
        /// <param name="Value">
        /// This control returns the selected power value .
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchWiMAXEVM(int EVM, int Mode, out double Value)
        {
            int pInvokeResult = PInvoke.FetchWiMAXEVM(this._handle, EVM, Mode, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the WiMax specific timing value.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_FETC_SST
        /// 
        /// Remote-control command(s):
        /// *CLS
        /// FETCh:BURSt:SSTiming:MAXimum?
        /// FETCh:BURSt:SSTiming:MINimum?
        /// FETCh:BURSt:SSTiming:AVERage?
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Mode">
        /// Sets the measurement mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MAX
        /// 
        /// </param>
        /// <param name="Value">
        /// This control returns the selected power value .
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchWiMAXSubscriberStationTime(int Mode, out double Value)
        {
            int pInvokeResult = PInvoke.FetchWiMAXSubscriberStationTime(this._handle, Mode, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the Carrier Interference Noise Ratio values.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:CINR?
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Values">
        /// This control returns the Carrier Interference Noise Ratio values.
        /// 
        /// Values are returned in order:
        /// 
        /// minimum
        /// average
        /// maximum
        /// standard deviation
        /// 
        /// Note:
        /// (1) Array size has to be at least 4 values
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchWiMAXCINRValues(double[] Values)
        {
            int pInvokeResult = PInvoke.FetchWiMAXCINRValues(this._handle, Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the Received Signal Strength Indication values.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:RSSi?
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Values">
        /// This control returns the Received Signal Strength Indication values.
        /// 
        /// Values are returned in order:
        /// 
        /// minimum
        /// average
        /// maximum
        /// standard deviation
        /// 
        /// Note:
        /// (1) Array size has to be at least 4 values
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchWiMAXRSSIValues(double[] Values)
        {
            int pInvokeResult = PInvoke.FetchWiMAXRSSIValues(this._handle, Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the count of analyzed burst or number of symbols of analyzed burst or current number of zones found during measurement analysis.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_FETC_BURS_COUNT
        /// RSSPECAN_ATTR_WIMAX_FETC_SYMB_COUNT
        /// RSSPECAN_ATTR_WIMAX_FETC_ZONE_COUNT
        /// RSSPECAN_ATTR_WIMAX_FETC_ZONE_STAT_COUNT
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:COUNt?
        /// FETCh:SYMBol:COUNt?
        /// FETCh:ZONE:COUNt?
        /// FETCh:ZONE:STATistic:COUNt?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Type">
        /// Sets the count type. The driver uses this value to select the attribute.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_BURS      (0) - Burst
        /// RSSPECAN_VAL_WIMAX_SYMB      (1) - Symbol
        /// RSSPECAN_VAL_WIMAX_ZONE      (2) - Zone
        /// RSSPECAN_VAL_WIMAX_STAT_ZONE (3) - Zone for Statistical Meas
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_BURS
        /// 
        /// 
        /// </param>
        /// <param name="Count">
        /// This control returns the count value.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchWiMAXCount(int Type, out int Count)
        {
            int pInvokeResult = PInvoke.FetchWiMAXCount(this._handle, Type, out Count);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the minimum, average and maximum time domain values. 
        /// 
        /// Note:
        /// 
        /// (1) This function is only available with the R&amp;S FSL and FS-K93 option.
        /// 
        /// (2) If no measurement has been performed yet, a query error will occur.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_FETC_TDOM_PREAMBLE
        /// RSSPECAN_ATTR_WIMAX_FETC_TDOM_SUBFRAME
        /// RSSPECAN_ATTR_WIMAX_FETC_TDOM_ZONE
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:TDOMain:PREamble?
        /// FETCh:BURSt:TDOMain:SUBFrame?
        /// FETCh:BURSt:TDOMain:ZONE?
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Result">
        /// This control selects time domain result.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_PREA (0) - Preamble 
        /// RSSPECAN_VAL_WIMAX_SUBF (1) - Subframe
        /// RSSPECAN_VAL_WIMAX_ZONE (2) - Zone
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_PREA (0)
        /// 
        /// Notes:
        /// 
        /// (1) Preamble:
        /// Returns the preamble time domain values (average, minimum or maximum value).
        /// 
        /// (2) Subframe:
        /// Returns the subframe time domain values (average, minimum or maximum value).
        /// 
        /// (3) Zone
        /// Returns the zone time domain values (average, minimum or maximum value).
        /// 
        /// 
        /// </param>
        /// <param name="Minimum">
        /// This control returns the minimum of the selected measurement.
        /// 
        /// You can use VI_NULL for this parameter.
        /// </param>
        /// <param name="Average">
        /// This control returns the average of the selected measurement.
        /// 
        /// You can use VI_NULL for this parameter.
        /// </param>
        /// <param name="Maximum">
        /// This control returns the maximum of the selected measurement.
        /// 
        /// You can use VI_NULL for this parameter.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchWiMAXTimeDomainPreamble(int Result, out double Minimum, out double Average, out double Maximum)
        {
            int pInvokeResult = PInvoke.FetchWiMAXTimeDomainPreamble(this._handle, Result, out Minimum, out Average, out Maximum);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the Unmodulate Subcarrier Error for the analysed zone.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_FETC_USERR
        /// 
        /// Remote-control command(s):
        /// *CLS
        /// FETCh:BURSt:USERror:MAXimum?
        /// FETCh:BURSt:USERror:MINimum?
        /// FETCh:BURSt:USERror:AVERage?
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Mode">
        /// Sets the measurement mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MAX
        /// 
        /// </param>
        /// <param name="Value">
        /// This control returns the selected power value .
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchWiMAXUnmodulatedSubcarrierError(int Mode, out double Value)
        {
            int pInvokeResult = PInvoke.FetchWiMAXUnmodulatedSubcarrierError(this._handle, Mode, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the units for table results.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_TABLE_UNIT
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TABLe:UNIT DB | PCT
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Unit">
        /// This parameter specifies the units for table results.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DB  (8)  - dB 
        /// RSSPECAN_VAL_UNIT_PCT (13) - %
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_PCT (13)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_TABLE_UNIT
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXListResultUnit(int Unit)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXListResultUnit(this._handle, Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the RSSI, CINR averaging parameter.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_RSSI_AVG_PARAM
        /// 
        /// Remote-control command(s):
        /// CONFigure:WIMax:AVERaging
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Averaging_Parameter">
        /// This parameter specifies the RSSI, CINR averaging parameter. 
        /// 
        /// Valid Values:
        /// no checking
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_WIMAX_RSSI_AVG_PARAM
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureWiMAXRSSICINRAvg(double Averaging_Parameter)
        {
            int pInvokeResult = PInvoke.ConfigureWiMAXRSSICINRAvg(this._handle, Averaging_Parameter);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the limit results
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_FERR_RES
        /// RSSPECAN_ATTR_WIMAX_SYMB_RES
        /// RSSPECAN_ATTR_WIMAX_SST_RES
        /// RSSPECAN_ATTR_WIMAX_EVM_ALL_RES
        /// RSSPECAN_ATTR_WIMAX_EVM_DATA_RES
        /// RSSPECAN_ATTR_WIMAX_BERP_RES
        /// RSSPECAN_ATTR_WIMAX_USERR_RES
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:FERRor[:AVERage]:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:FERRor:MAXimum:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:SYMBolerror[:AVERage]:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:SYMBolerror:MAXimum:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:SSTiming[:AVERage]:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:SSTiming:MAXimum:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:EVM:ALL[:AVERage]:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:EVM:ALL:MAXimum:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:EVM:DATA[:AVERage]:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:EVM:DATA:MAXimum:RESult
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1..8&gt;:BURSt:BERPilot[:AVERage]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1..8&gt;:BURSt:BERPilot:MAXimum:RESult?
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1..8&gt;:BURSt:USERror[:AVERage]:RESult?
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1..8&gt;:BURSt:USERror:MAXimum:RESult?
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// Sets the limit. The driver uses this value to set the repeated capability Limit of the attributes.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Limit_Type">
        /// Sets the limit type. The driver uses this value to select the attribute to be set.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_WIMAX_LIM_FERR     (0) - Frequency Error
        /// RSSPECAN_VAL_WIMAX_LIM_SYMB     (1) - Symbol Error
        /// RSSPECAN_VAL_WIMAX_LIM_SST      (2) - Subscriber Station Timing
        /// RSSPECAN_VAL_WIMAX_LIM_EVM_ALL  (3) - Combined EVM
        /// RSSPECAN_VAL_WIMAX_LIM_EVM_DATA (4) - Data Carrier EVM
        /// RSSPECAN_VAL_WIMAX_LIM_IQOFF   (5) - IQ Offset Error Limit
        /// RSSPECAN_VAL_WIMAX_LIM_BERP     (6) - Bit Error Rate for Pilot Carriers
        /// RSSPECAN_VAL_WIMAX_LIM_USERR    (7) - Unmodulated Subcarrier Error
        /// 
        /// Default Value: RSSPECAN_VAL_WIMAX_LIM_FERR
        /// 
        /// Note(s):
        /// 
        /// (1) Value RSSPECAN_VAL_WIMAX_LIM_BERP is valid for R&amp;S FSL with K93 option only.
        /// 
        /// (2) Value RSSPECAN_VAL_WIMAX_LIM_USERR is valid for Up Link Zone only.
        /// </param>
        /// <param name="Mode">
        /// Sets the limit measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MAX
        /// 
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns the result.
        /// 
        /// Returned Value:
        /// 0 - Failed
        /// 1 - Passed
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetWiMAXLimitCheckResult(int Window, int Limit, int Limit_Type, int Mode, out int Result)
        {
            int pInvokeResult = PInvoke.GetWiMAXLimitCheckResult(this._handle, Window, Limit, Limit_Type, Mode, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns all limit results.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:BURSt:ALL:RESult?
        /// 
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// Sets the limit. The driver uses this value to set the repeated capability Limit of the attributes.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Results">
        /// Returns the array of limit resutls.
        /// 
        /// Note:
        /// (1) The array size has to be at least 12
        /// 
        /// (2) &lt;average SS Timing&gt; and &lt;maximum SS Timing &gt; are only returned in Up Link mode.
        /// 
        /// Returned Values:
        /// 0 - Failed
        /// 1 - Passed
        /// 
        /// in order:
        /// 
        /// &lt;average EVM all bursts&gt;
        /// &lt;max EVM all bursts&gt;
        /// &lt;average EVM data carriers&gt;
        /// &lt;max EVM data carriers&gt;
        /// &lt;average Frequency Error&gt;
        /// &lt;max Frequency Error&gt;
        /// &lt;average Symbol Error&gt;
        /// &lt;max Symbol Error&gt;
        /// &lt;average SS Timing&gt;
        /// &lt;maximum SS Timing &gt;
        /// &lt;average IQ Offset&gt;
        /// &lt;max IQ Offset&gt;
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetWiMAXAllLimitCheckResults(int Window, int Limit, int[] Results)
        {
            int pInvokeResult = PInvoke.GetWiMAXAllLimitCheckResults(this._handle, Window, Limit, Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the result of the limit check of the limit line indicated in the selected measurement window.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_WIMAX_LIM_FAIL_RES
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1&gt;:FAIL?
        /// 
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit">
        /// Sets the limit. The driver uses this value to set the repeated capability Limit of the attributes.
        /// 
        /// Valid Values:
        /// 1 to 8
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Result">
        /// Return the of limit result.
        /// 
        /// Returned Values:
        /// 0 - Failed
        /// 1 - Passed
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetWiMAXLimitCheck(int Window, int Limit, out int Result)
        {
            int pInvokeResult = PInvoke.GetWiMAXLimitCheck(this._handle, Window, Limit, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects LTE Downlink analyzer mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument:SELect LTE
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SetLTEDownlinkMode()
        {
            int pInvokeResult = PInvoke.SetLTEDownlinkMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the general settings concerning the physical attributes of the signals to be measured.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_DUPLEXING
        /// RSSPECAN_ATTR_FREQUENCY_CENTER
        /// RSSPECAN_ATTR_LTE_DOWNLINK_CHANNEL_BANDWIDTH
        /// RSSPECAN_ATTR_LTE_DOWNLINK_NUMBER_OF_RESOURCE_BLOCKS
        /// RSSPECAN_ATTR_LTE_DOWNLINK_CYCLIC_PREFIX
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer
        /// CONFigure[:LTE]:DUPLexing
        /// CONFigure[:LTE]:DL:BW
        /// CONFigure[:LTE]:DL:NORB
        /// CONFigure[:LTE]:DL:CYCPrefix
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Duplexing">
        /// Configures the duplexing mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LTE_DUPLEXING_TDD (0) - Time Division Duplex
        /// RSSPECAN_VAL_LTE_DUPLEXING_FDD (1) - Frequency Divison Duplex
        /// 
        /// Default Value: RSSPECAN_VAL_LTE_DUPLEXING_FDD (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_DUPLEXING
        /// </param>
        /// <param name="Frequency">
        /// The control defines the center frequency of the analyzer.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_CENTER
        /// </param>
        /// <param name="Bandwidth_Setting_Type">
        /// Setting type of bandwidth
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LTE_DOWNLINK_CHBW (0) - Channel Bandwidth 
        /// RSSPECAN_VAL_LTE_DOWNLINK_NRB  (1) - Number of Resource Blocks 
        /// 
        /// Default Value: RSSPECAN_VAL_LTE_DOWNLINK_NRB
        /// 
        /// </param>
        /// <param name="Channel_Bandwidth">
        /// Setting cannel bandwidth.
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_LTE_DOWNLINK_BW1_40 (0) - 1.4 MHz 
        /// RSSPECAN_VAL_LTE_DOWNLINK_BW2_50 (1) - 2.5 MHz
        /// RSSPECAN_VAL_LTE_DOWNLINK_BW3_00 (2) - 3 MHz 
        /// RSSPECAN_VAL_LTE_DOWNLINK_BW5_00 (3) - 5 MHz
        /// RSSPECAN_VAL_LTE_DOWNLINK_BW10_00 (4) - 10 MHz
        /// RSSPECAN_VAL_LTE_DOWNLINK_BW15_00 (5) - 15 MHz
        /// RSSPECAN_VAL_LTE_DOWNLINK_BW20_00 (6) - 20 MHz
        /// 
        /// Default Value: RSSPECAN_VAL_LTE_DOWNLINK_BW1_40
        /// </param>
        /// <param name="Number_of_Resource_Blocks">
        /// Configures the number of resource blocks used for DL.
        /// 
        /// Valid Values:
        /// 6 to 110
        /// 
        /// Default Value: 25
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_NUMBER_OF_RESOURCE_BLOCKS
        /// </param>
        /// <param name="Cyclic_Prefix">
        /// Configures the cyclic prefix type for DL.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LTE_DOWNLINK_CPREFIX_NORMAL   (0) - Normal
        /// RSSPECAN_VAL_LTE_DOWNLINK_CPREFIX_EXTENDED (1) - Extended
        /// RSSPECAN_VAL_LTE_DOWNLINK_CPREFIX_AUTO     (2) - Auto
        /// 
        /// Default Value: RSSPECAN_VAL_LTE_DOWNLINK_CPREFIX_AUTO (2)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_CYCLIC_PREFIX
        /// 
        /// Note(s):
        /// 
        /// (1) Normal: Normal cyclic prefix length
        /// 
        /// (2) Extended: Extended cyclic prefix length
        /// 
        /// (3) Auto: Automatic cyclic prefix length detection
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkSignalCharacteristics(int Duplexing, double Frequency, int Bandwidth_Setting_Type, int Channel_Bandwidth, int Number_of_Resource_Blocks, int Cyclic_Prefix)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkSignalCharacteristics(this._handle, Duplexing, Frequency, Bandwidth_Setting_Type, Channel_Bandwidth, Number_of_Resource_Blocks, Cyclic_Prefix);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures settings related to the reference level and the RF attenuation.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_AUTO_REFERENCE_LEVEL
        /// RSSPECAN_ATTR_LTE_DOWNLINK_REFERENCE_LEVEL
        /// RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET
        /// 
        /// Remote-control command(s):
        /// [SENSe]:POWer:AUTO&lt;1..4&gt;
        /// CONFigure:POWer:EXPected:RF&lt;1..4&gt;
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:RLEVel:OFFSet
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Reference_Level">
        /// Configures the reference level of the analyzers used in RF mode.
        /// 
        /// Valid Values:
        /// -1.3E+002 to 3.0E+001
        /// 
        /// Default Value: -20.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_REFERENCE_LEVEL
        /// 
        /// Note(s):
        /// 
        /// (1) This control is valid only if Auto Reference Level is set to Off.
        /// </param>
        /// <param name="Auto_Reference_Level">
        /// Activates or deactivates auto level.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_AUTO_REFERENCE_LEVEL
        /// </param>
        /// <param name="External_Attenuation">
        /// This control specifies the external attenuation or gain applied to the RF signal. A positive value indicates attenuation, a negative value indicates gain.
        /// 
        /// Valid Values:
        /// -2.0E+002 dB to 2.0E+002 dB
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkLevel(double Reference_Level, bool Auto_Reference_Level, double External_Attenuation)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkLevel(this._handle, Reference_Level, System.Convert.ToUInt16(Auto_Reference_Level), External_Attenuation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the reference level of the analyzers used in BB-mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_REFERENCE_LEVEL_IN_BB_MODE
        /// 
        /// Remote-control command(s):
        /// CONFigure:POWer:EXPected:IQ&lt;1..4&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Reference_Level">
        /// Configures the reference level of the analyzers used in BB-mode.
        /// 
        /// Valid Values:
        /// -25.0 to 20.0 dBm
        /// 
        /// Default Value: 0.0 dBm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_REFERENCE_LEVEL_IN_BB_MODE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkLevelInBBMode(double Reference_Level)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkLevelInBBMode(this._handle, Reference_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures essential antenna configuration parameters.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_MIMO_CONFIGURATION
        /// RSSPECAN_ATTR_LTE_DOWNLINK_MIMO_ANTENNA
        /// 
        /// Remote-control command(s):
        /// CONFigure[:LTE]:DL:MIMO:CONF
        /// CONFigure[:LTE]:DL:MIMO:ASELection
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Configuration">
        /// Configures the current MIMO antenna configuration.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LTE_DOWNLINK_MIMO_TX1 (0) - 1 Tx Antenna
        /// RSSPECAN_VAL_LTE_DOWNLINK_MIMO_TX2 (1) - 2 Tx Antennas
        /// RSSPECAN_VAL_LTE_DOWNLINK_MIMO_TX4 (2) - 4 Tx Antennas
        /// 
        /// Default Value: RSSPECAN_VAL_LTE_DOWNLINK_MIMO_TX1 (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_MIMO_CONFIGURATION
        /// </param>
        /// <param name="Antenna_Selection">
        /// Configures the antenna selection.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LTE_DOWNLINK_MIMO_ANT_ALL (0) - All
        /// RSSPECAN_VAL_LTE_DOWNLINK_MIMO_ANT1    (1) - Antenna 1
        /// RSSPECAN_VAL_LTE_DOWNLINK_MIMO_ANT2    (2) - Antenna 2
        /// RSSPECAN_VAL_LTE_DOWNLINK_MIMO_ANT3    (3) - Antenna 3
        /// RSSPECAN_VAL_LTE_DOWNLINK_MIMO_ANT4    (4) - Antenna 4
        /// 
        /// Default Value: RSSPECAN_VAL_LTE_DOWNLINK_MIMO_ANT1 (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_MIMO_ANTENNA
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkMIMO(int Configuration, int Antenna_Selection)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkMIMO(this._handle, Configuration, Antenna_Selection);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures I/Q settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_IQ_SWAP
        /// 
        /// Remote-control command(s):
        /// [SENSe]:SWAPiq[?]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Swap_IQ">
        /// Specifies if the IQ data shall be swapped.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_IQ_SWAP
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkIQ(bool Swap_IQ)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkIQ(this._handle, System.Convert.ToUInt16(Swap_IQ));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Configures settings related to the input source of the signal to be measured.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_INPUT
        /// 
        /// Remote-control command(s):
        /// SENSe:INPut
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Source">
        /// Configures the current signal source. Signal sources can be baseband, digital IQ and Radio Frequency.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_BB_INPUT_RF  (0) - RF
        /// RSSPECAN_VAL_BB_INPUT_AIQ (1) - Analog Baseband Input
        /// RSSPECAN_VAL_BB_INPUT_DIQ (2) - Digital Baseband Input
        /// 
        /// Default Value: RSSPECAN_VAL_BB_INPUT_RF (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_INPUT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkInput(int Source)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkInput(this._handle, Source);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Restores previously saved demodulation settings. The input file must be of type "*.allocation" and depends on the link direction that was currently selected when the file was saved. Only files with correct link directions can get loaded.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_LOAD_DEMODULATION_SETTINGS
        /// 
        /// Remote-control command(s):
        /// MMEMory:LOAD:DEM
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="File_Path">
        /// Restores previously saved demodulation settings. The input file must be of type "*.allocation" and depends on the link direction that was currently selected when the file was saved. Only files with correct link directions can get loaded.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: "D:\\USER\\Settingsfile.allocation"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_LOAD_DEMODULATION_SETTINGS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int LoadLTEDownlinkDemodulationSettings(string File_Path)
        {
            int pInvokeResult = PInvoke.LoadLTEDownlinkDemodulationSettings(this._handle, File_Path);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Stores the current demodulation settings to a file. The resulting file type is "*.allocation". Existing files will be overwritten.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_STORE_DEMODULATION_SETTINGS
        /// 
        /// Remote-control command(s):
        /// MMEMory:STORe:DEM
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="File_Path">
        /// Stores the current demodulation settings to a file. The resulting file type is "*.allocation". Existing files will be overwritten.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: "D:\\USER\\Settingsfile.allocation"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_STORE_DEMODULATION_SETTINGS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int StoreLTEDownlinkDemodulationSettings(string File_Path)
        {
            int pInvokeResult = PInvoke.StoreLTEDownlinkDemodulationSettings(this._handle, File_Path);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Configures processing configuration with respect to how the signal is to be measured.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_CHANNEL_ESTIMATION
        /// RSSPECAN_ATTR_LTE_DOWNLINK_EVM_CALCULATION_METHOD
        /// RSSPECAN_ATTR_LTE_DOWNLINK_SCRAMBLING_OF_CODED_BITS
        /// RSSPECAN_ATTR_LTE_DOWNLINK_AUTO_DEMODULATION
        /// 
        /// Remote-control command(s):
        /// [SENSe][:LTE]:DL:DEMod:CESTimation
        /// [SENSe][:LTE]:DL:DEMod:EVMCalc
        /// [SENSe][:LTE]:DL:DEMod:CBSCrambling
        /// [SENSe][:LTE]:DL:DEMod:AUTO[?]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Channel_Estimation">
        /// Configures the channel estimation type for DL.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LTE_DOWNLINK_CEST_TGPP   (0) - 3GPP EVM definition
        /// RSSPECAN_VAL_LTE_DOWNLINK_CEST_PILOT  (1) - Optimal, Pilot Only
        /// RSSPECAN_VAL_LTE_DOWNLINK_CEST_PILPAY (2) - Optimal, Pilot and Payload
        /// 
        /// Default Value: RSSPECAN_VAL_LTE_DOWNLINK_CEST_TGPP (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_CHANNEL_ESTIMATION
        /// </param>
        /// <param name="EVM_Calculation_Method">
        /// Sets the EVM calculation method for DL.
        /// 
        /// Valid Values:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_EVM_CALC_TGPP (0) - 3GPP Definition
        /// RSSPECAN_ATTR_LTE_DOWNLINK_EVM_CALC_OTP  (1) - Optimal Timing Position
        /// 
        /// Default Value: RSSPECAN_ATTR_LTE_DOWNLINK_EVM_CALC_TGPP (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_EVM_CALCULATION_METHOD
        /// </param>
        /// <param name="Scrambling_of_Coded_Bits">
        /// Activates or deactivates scrambling of coded bits for DL.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_SCRAMBLING_OF_CODED_BITS
        /// </param>
        /// <param name="Auto_Demodulation">
        /// Activates or deactivates automatic demodulation for DL.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_AUTO_DEMODULATION
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkDataAnalysis(int Channel_Estimation, int EVM_Calculation_Method, bool Scrambling_of_Coded_Bits, bool Auto_Demodulation)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkDataAnalysis(this._handle, Channel_Estimation, EVM_Calculation_Method, System.Convert.ToUInt16(Scrambling_of_Coded_Bits), System.Convert.ToUInt16(Auto_Demodulation));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Configures phase and timing tracking.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PHASE_TRACKING
        /// RSSPECAN_ATTR_LTE_DOWNLINK_TIMING
        /// 
        /// Remote-control command(s):
        /// [SENSe][:LTE]:DL:TRACking:PHASe
        /// [SENSe][:LTE]:DL:TRACking:TIME
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Phase_Tracking">
        /// Configures the phase tracking type for DL.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LTE_DOWNLINK_PHASE_TRACKING_OFF    (0) - Deactivate Phase Tracking
        /// RSSPECAN_VAL_LTE_DOWNLINK_PHASE_TRACKING_PILOT  (1) - Pilot Only
        /// RSSPECAN_VAL_LTE_DOWNLINK_PHASE_TRACKING_PILPAY (2) - Pilot and Payload
        /// 
        /// Default Value: RSSPECAN_VAL_LTE_DOWNLINK_PHASE_TRACKING_OFF (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PHASE_TRACKING
        /// </param>
        /// <param name="Timing">
        /// Activates or deactivates timing tracking for DL.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_TIMING
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkTracking(int Phase_Tracking, bool Timing)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkTracking(this._handle, Phase_Tracking, System.Convert.ToUInt16(Timing));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Configures the TDD UL/DL Allocations for DL.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_TDD_UL_DL_ALLOCATIONS
        /// 
        /// Remote-control command(s):
        /// CONFigure[:LTE]:DL:TDD:UDConf
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="TDD_UL_DL_Allocations">
        /// Configures the TDD UL/DL Allocations for DL.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_TDD_UL_DL_ALLOCATIONS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkTDDAllocation(int TDD_UL_DL_Allocations)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkTDDAllocation(this._handle, TDD_UL_DL_Allocations);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Configures physical layer cell identity. There are 504 unique physical-layer cell identities. The physical-layer cell identities are  grouped into 168 unique physical-layer cell-identity groups, each group containing  three unique identities.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_CID_GROUP_AUTO
        /// RSSPECAN_ATTR_LTE_DOWNLINK_CELL_IDENTITY_GROUP
        /// RSSPECAN_ATTR_LTE_DOWNLINK_CELL_IDENTITY_AUTO
        /// RSSPECAN_ATTR_LTE_DOWNLINK_CELL_IDENTITY
        /// 
        /// Remote-control command(s):
        /// CONFigure[:LTE]:DL:PLCI:CIDGroup
        /// CONFigure[:LTE]:DL:PLCI:PLID
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Auto_Cell_Identity_Group">
        /// Enables/disables automatic cell identity group for DL.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_CID_GROUP_AUTO
        /// </param>
        /// <param name="Cell_Identity_Group">
        /// Configures the current cell identity group for DL.
        /// 
        /// Valid Values:
        /// 0 to 167
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_CELL_IDENTITY_GROUP
        /// </param>
        /// <param name="Cell_Identity">
        /// Configures the current cell identity for DL.
        /// 
        /// Valid Values:
        /// 0 to 2
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_CELL_IDENTITY
        /// </param>
        /// <param name="Auto_Cell_Identity">
        /// Enables/disables automatic cell identity for DL.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_CELL_IDENTITY_AUTO
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkPhysicalLayerCellIdentity(bool Auto_Cell_Identity_Group, int Cell_Identity_Group, int Cell_Identity, bool Auto_Cell_Identity)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkPhysicalLayerCellIdentity(this._handle, System.Convert.ToUInt16(Auto_Cell_Identity_Group), Cell_Identity_Group, Cell_Identity, System.Convert.ToUInt16(Auto_Cell_Identity));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Sets the number of configurable subframes for DL.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_CONFIGURABLE_SUBFRAMES
        /// 
        /// Remote-control command(s):
        /// CONFigure[:LTE]:DL:CSUBframes
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Configurable_Subframes">
        /// Sets the number of configurable subframes for DL.
        /// 
        /// Valid Values:
        /// 0 to 10
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_CONFIGURABLE_SUBFRAMES
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkPDSCHConfigurableSubframes(int Configurable_Subframes)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkPDSCHConfigurableSubframes(this._handle, Configurable_Subframes);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Configures the number of used allocations for a specific DL subframe. The suffix stands for the corresponding subframe number.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_USED_ALLOCATIONS
        /// 
        /// Remote-control command(s):
        /// CONFigure[:LTE]:DL:CSUBframes
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Subframe">
        /// Selects configurable subframe.
        /// 
        /// Valid Values:
        /// 0 to 9
        /// 
        /// Default Value: 0
        /// </param>
        /// <param name="Used_Allocations">
        /// Configures the number of used allocations for a specific DL subframe.
        /// 
        /// Valid Values:
        /// 0 to 110
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_USED_ALLOCATIONS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkPDSCHUsedAllocations(int Subframe, int Used_Allocations)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkPDSCHUsedAllocations(this._handle, Subframe, Used_Allocations);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Configures specified allocation in specified subframe.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_MODULATION
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_RESOURCE_BLOCKS
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_RESOURCE_BLOCKS_OFFSET
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_POWER
        /// 
        /// Remote-control command(s):
        /// CONFigure[:LTE]:DL:SUBFrame&lt;LTESubframe&gt;:ALLoc&lt;LTEAllocation&gt;[:CW&lt;[1]|2&gt;]:MODulation
        /// CONFigure[:LTE]:DL:SUBFrame&lt;LTESubframe&gt;:ALLoc&lt;LTEAllocation&gt;:RBCount
        /// CONFigure[:LTE]:DL:SUBFrame&lt;LTESubframe&gt;:ALLoc&lt;LTEAllocation&gt;:RBOFfset
        /// CONFigure[:LTE]:DL:SUBFrame&lt;LTESubframe&gt;:ALLoc&lt;LTEAllocation&gt;:POWer
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Subframe">
        /// Selects configurable subframe.
        /// 
        /// Valid Values:
        /// 0 to 9
        /// 
        /// Default Value: 0
        /// </param>
        /// <param name="Used_Allocation">
        /// Selects allocation.
        /// 
        /// Valid Values:
        /// 0 to 99
        /// 
        /// Default Value: 0
        /// </param>
        /// <param name="Codeword">
        /// Selects codeword.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Modulation">
        /// Configures the modulation type for an allocation in a specific DL subframe. In case of MIMO there might be more than one codewords which can be configured separately. The first suffix stands for the corresponding subframe number and the second one stands for the corresponding allocation number. The third suffix stands for the codeword index.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LTE_DOWNLINK_MODULATION_QPSK  (0) - QPSK
        /// RSSPECAN_VAL_LTE_DOWNLINK_MODULATION_16QAM (1) - 16QAM
        /// RSSPECAN_VAL_LTE_DOWNLINK_MODULATION_64QAM (2) - 64QAM
        /// 
        /// Default Value: RSSPECAN_VAL_LTE_DOWNLINK_MODULATION_QPSK (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_MODULATION
        /// </param>
        /// <param name="Resource_Blocks">
        /// Configures the number of resource blocks for an allocation in a specific DL subframe.
        /// 
        /// Valid Values:
        /// 1 to 110
        /// 
        /// Default Value: 6
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_RESOURCE_BLOCKS
        /// </param>
        /// <param name="Resource_Blocks_Offset">
        /// Configures the resource block offset for an allocation in a specific DL subframe.
        /// 
        /// Valid Values:
        /// 0 to 109
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_RESOURCE_BLOCKS_OFFSET
        /// </param>
        /// <param name="Power">
        /// Configures the relative power of an allocation in a specific DL subframe.
        /// 
        /// Valid Values:
        /// -12.0 to 12.0 dB
        /// 
        /// Default Value: 0.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_POWER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkPDSCHUsedAllocation(int Subframe, int Used_Allocation, int Codeword, int Modulation, int Resource_Blocks, int Resource_Blocks_Offset, double Power)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkPDSCHUsedAllocation(this._handle, Subframe, Used_Allocation, Codeword, Modulation, Resource_Blocks, Resource_Blocks_Offset, Power);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Configures the relative power of the reference signal for DL.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_REFERENCE_POWER
        /// 
        /// Remote-control command(s):
        /// CONFigure[:LTE]:DL:REFSig:POWer
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Reference_Power">
        /// Configures the relative power of the reference signal for DL.
        /// 
        /// Valid Values:
        /// -12.0 to 12.0 dB
        /// 
        /// Default Value: 0.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_REFERENCE_POWER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkReferenceSignal(double Reference_Power)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkReferenceSignal(this._handle, Reference_Power);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Configures synchronisation signal.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PSYNC_RELATIVE_POWER
        /// RSSPECAN_ATTR_LTE_DOWNLINK_SSYNC_RELATIVE_POWER
        /// 
        /// Remote-control command(s):
        /// CONFigure[:LTE]:DL:SYNC:PPOWer
        /// CONFigure[:LTE]:DL:SYNC:SPOWer
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="P_SYNC_Relative_Power">
        /// Configures the P-SYNC relative power for DL.
        /// 
        /// Valid Values:
        /// -12.0 to 12.0 dB
        /// 
        /// Default Value: 0.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PSYNC_RELATIVE_POWER
        /// </param>
        /// <param name="S_SYNC_Relative_Power">
        /// Configures the S-SYNC relative power for DL.
        /// 
        /// Valid Values:
        /// -12.0 to 12.0 dB
        /// 
        /// Default Value: 0.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_SSYNC_RELATIVE_POWER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkSynchronisationSignal(double P_SYNC_Relative_Power, double S_SYNC_Relative_Power)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkSynchronisationSignal(this._handle, P_SYNC_Relative_Power, S_SYNC_Relative_Power);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Configures physical broadcast channel.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PBCH_STATE
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PBCH_RELATIVE_POWER
        /// 
        /// Remote-control command(s):
        /// CONFigure[:LTE]:DL:PBCH:STATe
        /// CONFigure[:LTE]:DL:PBCH:POWer
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="PBCH_Present">
        /// Activates or deactivates the physical broadcast channel.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PBCH_STATE
        /// </param>
        /// <param name="PBCH_Relative_Power">
        /// Configures the PBCH relative power for DL.
        /// 
        /// Valid Values:
        /// -12.0 to 12.0 dB
        /// 
        /// Default Value: 0.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PBCH_RELATIVE_POWER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkPBCH(bool PBCH_Present, double PBCH_Relative_Power)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkPBCH(this._handle, System.Convert.ToUInt16(PBCH_Present), PBCH_Relative_Power);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Configures PCFICH.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PCFICH_STATE
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PCFICH_RELATIVE_POWER
        /// 
        /// Remote-control command(s):
        /// CONFigure[:LTE]:DL:PCFICH:STATe
        /// CONFigure[:LTE]:DL:PCFICH:POWer
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="PCFICH_Present">
        /// Activates or deactivates the PCFICH for DL.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PCFICH_STATE
        /// </param>
        /// <param name="PCFICH_Relative_Power">
        /// Configures the PCFICH relative power for DL.
        /// 
        /// Valid Values:
        /// -12.0 to 12.0 dB
        /// 
        /// Default Value: 0.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_RSSPECAN_ATTR_LTE_DOWNLINK_PCFICH_RELATIVE_POWER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkPCFICH(bool PCFICH_Present, double PCFICH_Relative_Power)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkPCFICH(this._handle, System.Convert.ToUInt16(PCFICH_Present), PCFICH_Relative_Power);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Configures PHICH.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PHICH_NUMBER_OF_GROUPS
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PHICH_DURATION
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PHICH_RELATIVE_POWER
        /// 
        /// Remote-control command(s):
        /// CONFigure[:LTE]:DL:PHICh:NOGRoups
        /// CONFigure[:LTE]:DL:PHICh:DURation
        /// CONFigure[:LTE]:DL:PHICh:POWer
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="PHICH_Number_of_Groups">
        /// Configures the number of PHICH groups for DL.
        /// 
        /// Valid Values:
        /// 0 to 1000
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PHICH_NUMBER_OF_GROUPS
        /// </param>
        /// <param name="PHICH_Duration">
        /// Configure the PHICH duration for DL.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LTE_DOWNLINK_PHICH_DURATION_NORMAL   (0) - Normal
        /// RSSPECAN_VAL_LTE_DOWNLINK_PHICH_DURATION_EXTENDED (1) - Extended
        /// 
        /// Default Value: RSSPECAN_VAL_LTE_DOWNLINK_PHICH_DURATION_NORMAL (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PHICH_DURATION
        /// </param>
        /// <param name="PHICH_Relative_Power">
        /// Configures the PHICH relative power for DL.
        /// 
        /// Valid Values:
        /// -12.0 to 12.0 dB
        /// 
        /// Default Value: 0.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PHICH_RELATIVE_POWER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkPHICH(int PHICH_Number_of_Groups, int PHICH_Duration, double PHICH_Relative_Power)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkPHICH(this._handle, PHICH_Number_of_Groups, PHICH_Duration, PHICH_Relative_Power);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Configures PDCCH.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PDCCH_NUMBER_OF_PDCCHS
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PDCCH_FORMAT
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PDCCH_RELATIVE_POWER
        /// 
        /// Remote-control command(s):
        /// CONFigure[:LTE]:DL:PDCCh:NOPD
        /// CONFigure[:LTE]:DL:PDCCh:FORMat
        /// CONFigure[:LTE]:DL:PDCCh:POWer
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Number_of_PDCCHs">
        /// Configures the number of physical downlink control channels.
        /// 
        /// Valid Values:
        /// 0 to 1000
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PDCCH_NUMBER_OF_PDCCHS
        /// </param>
        /// <param name="PDCCH_Format">
        /// Configures the physical downlink control channel format.
        /// 
        /// Valid Values:
        /// -1 to 3
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PDCCH_FORMAT
        /// </param>
        /// <param name="PDCCH_Relative_Power">
        /// Configures the PDCCH relative power for DL.
        /// 
        /// Valid Values:
        /// -12.0 to 12.0 dB
        /// 
        /// Default Value: 0.0 dB
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_PDCCH_RELATIVE_POWER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkPDCCH(int Number_of_PDCCHs, int PDCCH_Format, double PDCCH_Relative_Power)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkPDCCH(this._handle, Number_of_PDCCHs, PDCCH_Format, PDCCH_Relative_Power);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This command updates the current IQ measurement results to reflect the current measurement settings. Note no new IQ data is captured. I.e. the measurement settings apply to the IQ data being currently in the capture buffer. The command applies exclusively to IQ measurements. It requires available IQ data.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_UPDATE_IQ_RESULTS
        /// 
        /// Remote-control command(s):
        /// INITiate:REFResh
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int UpdateIQResults()
        {
            int pInvokeResult = PInvoke.UpdateIQResults(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Selects LTE Downlink measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_MEASUREMENT_TYPE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;Window&gt;:FEED
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Measurement_Type">
        /// Selects LTE Downlink measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LTE_DOWNLINK_CBUF (0) - Capture Buffer
        /// RSSPECAN_VAL_LTE_DOWNLINK_EVCA (1) - EVM versus Carrier
        /// RSSPECAN_VAL_LTE_DOWNLINK_EVSY (2) - EVM versus Symbol
        /// RSSPECAN_VAL_LTE_DOWNLINK_FEVS (3) - Frequency Error versus Symbol
        /// RSSPECAN_VAL_LTE_DOWNLINK_EVSU (4) - EVM versus Subframe
        /// RSSPECAN_VAL_LTE_DOWNLINK_SEM  (5) - Spectrum Mask
        /// RSSPECAN_VAL_LTE_DOWNLINK_ACP  (6) - ACP
        /// RSSPECAN_VAL_LTE_DOWNLINK_PSPE (7) - Power Spectrum
        /// RSSPECAN_VAL_LTE_DOWNLINK_PVRP (8) - Power versus RB PDSCH
        /// RSSPECAN_VAL_LTE_DOWNLINK_PVRR (9) - Power versus RB RS
        /// RSSPECAN_VAL_LTE_DOWNLINK_FLAT (10) - Spectrum Flatness
        /// RSSPECAN_VAL_LTE_DOWNLINK_GDEL (11) - Group Delay
        /// RSSPECAN_VAL_LTE_DOWNLINK_FDIF (12) - Flatness Difference
        /// RSSPECAN_VAL_LTE_DOWNLINK_CONS (13) - Constellation Diagram
        /// RSSPECAN_VAL_LTE_DOWNLINK_DFTC (14) - DFT Precoded Constellation Diagram
        /// RSSPECAN_VAL_LTE_DOWNLINK_CCDF (15) - CCDF
        /// RSSPECAN_VAL_LTE_DOWNLINK_SFLO (16) - Signal Flow
        /// RSSPECAN_VAL_LTE_DOWNLINK_ASUM (17) - Allocation Summary
        /// RSSPECAN_VAL_LTE_DOWNLINK_BSTR (18) - Bit Stream
        /// 
        /// Default Value: RSSPECAN_VAL_LTE_DOWNLINK_CBUF (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_MEASUREMENT_TYPE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkMeasurementType(int Window, int Measurement_Type)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkMeasurementType(this._handle, Window, Measurement_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Configures the subframe to be analyzed.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_SUBFRAME_SELECTION_ALL
        /// RSSPECAN_ATTR_LTE_DOWNLINK_SUBFRAME_SELECTION
        /// 
        /// Remote-control command(s):
        /// [SENSe][:LTE]:SUBFrame:SELect
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="All_Subframes">
        /// Selects either single subframe or all subframes to be analyzed.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off, Single Subframe
        /// VI_TRUE  (1) - On, All Subframes
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_SUBFRAME_SELECTION_ALL
        /// </param>
        /// <param name="Single_Subframe_Selection">
        /// Configures the subframe to be analyzed.
        /// 
        /// Valid Values:
        /// 0 to 9
        /// 
        /// Default Value: 0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_SUBFRAME_SELECTION
        /// 
        /// Note(s):
        /// 
        /// (1) This control is valid only if 'All Subframes' is turned off.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkSubframeSelection(bool All_Subframes, int Single_Subframe_Selection)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkSubframeSelection(this._handle, System.Convert.ToUInt16(All_Subframes), Single_Subframe_Selection);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Specifies the units for EVM results.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_EVM_UNITS
        /// 
        /// Remote-control command(s):
        /// UNIT:EVM
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="EVM_Units">
        /// Specifies the units for EVM results.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DB  (8)  - dB
        /// RSSPECAN_VAL_UNIT_PCT (13) - %
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_DB (8)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_EVM_UNITS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkEVMUnits(int EVM_Units)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkEVMUnits(this._handle, EVM_Units);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Specifies if the bit stream gets displayed using bits or using symbols.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_BIT_STREAM_FORMAT
        /// 
        /// Remote-control command(s):
        /// UNIT:BSTR
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Bit_Stream_Format">
        /// Specifies if the bit stream gets displayed using bits or using symbols.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LTE_DOWNLINK_STREAM_SYMBOLS (0) - Symbols
        /// RSSPECAN_VAL_LTE_DOWNLINK_STREAM_BITS    (1) - Bits
        /// 
        /// Default Value: RSSPECAN_VAL_LTE_DOWNLINK_STREAM_SYMBOLS (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_BIT_STREAM_FORMAT
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkBitStream(int Bit_Stream_Format)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkBitStream(this._handle, Bit_Stream_Format);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Specifies the SEM category defined in 3GPP TS 36.101 for Uplink.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_SEM_CHANNEL
        /// 
        /// Remote-control command(s):
        /// [SENSe]:POWer:SEM:CATegory
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="SEM_Channel_Category">
        /// Specifies the SEM category defined in 3GPP TS 36.101 for Uplink.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LTE_DOWNLINK_CATEGORY_A (0) - Category A
        /// RSSPECAN_VAL_LTE_DOWNLINK_CATEGORY_B (1) - Category B
        /// 
        /// Default Value: RSSPECAN_VAL_LTE_DOWNLINK_CATEGORY_A (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_SEM_CHANNEL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkSEMSettings(int SEM_Channel_Category)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkSEMSettings(this._handle, SEM_Channel_Category);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Configures adjacent channel power measurement settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_ACP_ASSUMED_ADJ_CHANNEL_CARRIER
        /// RSSPECAN_ATTR_LTE_DOWNLINK_ACP_NOISE_CORRECTION
        /// 
        /// Remote-control command(s):
        /// [SENSe]:POWer:ACHannel:AACHannel
        /// [SENSe]:POWer:NCORrection
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Assumed_Adj_Channel_Carrier">
        /// Selects the ACP Assumed Adj. Channel Carrier in the general settings menu.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LTE_DOWNLINK_ASSUMED_EUTRA   (0) - EUTRA
        /// RSSPECAN_VAL_LTE_DOWNLINK_ASSUMED_UTRA128 (1) - UTRA128
        /// RSSPECAN_VAL_LTE_DOWNLINK_ASSUMED_UTRA384 (2) - UTRA384
        /// RSSPECAN_VAL_LTE_DOWNLINK_ASSUMED_UTRA768 (3) - UTRA768
        /// 
        /// Default Value: RSSPECAN_VAL_LTE_DOWNLINK_ASSUMED_EUTRA (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_ACP_ASSUMED_ADJ_CHANNEL_CARRIER
        /// 
        /// Note(s):
        /// 
        /// (1) EUTRA: Selects  an  EUTRA  signal  of  the  same  bandwidth  like  the  TX channel as assumed adjacent channel carrier. 
        /// 
        /// (2) UTRA128: Selects an UTRA signal with a bandwidth of 1.28MHz as assumed adjacent channel carrier.
        /// 
        /// (3) UTRA384: Selects an UTRA signal with a bandwidth of 3.84MHz as assumed adjacent channel carrier.
        /// 
        /// (4) UTRA768: Selects an UTRA signal with a bandwidth of 7.68MHz as assumed adjacent channel carrier.
        /// </param>
        /// <param name="Noise_Correction">
        /// Activates or deactivates Noise Correction for ACP measurements.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_ACP_NOISE_CORRECTION
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkACPSettings(int Assumed_Adj_Channel_Carrier, bool Noise_Correction)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkACPSettings(this._handle, Assumed_Adj_Channel_Carrier, System.Convert.ToUInt16(Noise_Correction));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Defines if the constellation diagram shall use data before or after the MIMO decoder.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_LOCATION
        /// 
        /// Remote-control command(s):
        /// CONFigure[:LTE]:DL:CONS:LOCation
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Constellation_Location">
        /// Defines if the constellation diagram shall use data before or after the MIMO decoder.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LTE_DOWNLINK_LOC_AMD (0) - After MIMO Decoder
        /// RSSPECAN_VAL_LTE_DOWNLINK_LOC_BMD (1) - Before MIMO Decoder
        /// 
        /// Default Value: RSSPECAN_VAL_LTE_DOWNLINK_LOC_AMD (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_LOCATION
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureLTEDownlinkConstellationLocation(int Constellation_Location)
        {
            int pInvokeResult = PInvoke.ConfigureLTEDownlinkConstellationLocation(this._handle, Constellation_Location);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Queries LTE Downlink measurement result summary.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_QPSK_RESULT
        /// RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_16QAM_RESULT
        /// RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_64QAM_RESULT
        /// 
        /// Remote-control command(s):
        /// FETCh:SUMMary:EVM:DSQP[:AVERage]?
        /// FETCh:SUMMary:EVM:DSST[:AVERage]?
        /// FETCh:SUMMary:EVM:DSSF[:AVERage]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Frame_Result">
        /// Selects result type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_LTE_DOWNLINK_QPSK  (0) - EVM PDSCH QPSK
        /// RSSPECAN_VAL_LTE_DOWNLINK_16QAM (1) - EVM PDSCH 16QAM
        /// RSSPECAN_VAL_LTE_DOWNLINK_64QAM (2) - EVM PDSCH 64QAM
        /// 
        /// Default Value: RSSPECAN_VAL_LTE_DOWNLINK_QPSK (0)
        /// 
        /// Note(s):
        /// 
        /// (1) EVM PDSCH QPSK: returns the "EVM PDSCH QPSK" value from the result summary list.
        /// 
        /// (2) EVM PDSCH 16QAM: returns the "EVM PDSCH 16QAM" value from the result summary list.
        /// 
        /// (3) EVM PDSCH 64QAM: Returns the "EVM PDSCH 64QAM" value from the result summary list.
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns selected result. Depending on the EVM unit selected in the "General Settings" menu the value will be either in dB or in percent.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryLTEDownlinkMeasurementResultSummary(int Frame_Result, out double Result)
        {
            int pInvokeResult = PInvoke.QueryLTEDownlinkMeasurementResultSummary(this._handle, Frame_Result, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Queries LTE Downlink measurement EVM All result.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_EVM_ALL_RESULT
        /// 
        /// Remote-control command(s):
        /// FETCh:SUMMary:EVM[:ALL][:AVERage]?
        /// FETCh:SUMMary:EVM[:ALL]:MAXimum?
        /// FETCh:SUMMary:EVM[:ALL]:MINimum?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement_Type">
        /// Selects measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MIN (1)
        /// </param>
        /// <param name="Result">
        /// Returns selected result. Depending on the EVM unit selected in the "General Settings" menu the value will be either in dB or in percent.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryLTEDownlinkMeasurementEVMAll(int Measurement_Type, out double Result)
        {
            int pInvokeResult = PInvoke.QueryLTEDownlinkMeasurementEVMAll(this._handle, Measurement_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Queries the "EVM Physical Channel" value from the result summary list.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_CHANNEL_RESULT
        /// 
        /// Remote-control command(s):
        /// FETCh:SUMMary:EVM:PCHannel[:AVERage]?
        /// FETCh:SUMMary:EVM:PCHannel:MAXimum?
        /// FETCh:SUMMary:EVM:PCHannel:MINimum?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement_Type">
        /// Selects measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MIN (1)
        /// </param>
        /// <param name="Result">
        /// Returns selected result. Depending on the EVM unit selected in the "General Settings" menu the value will be either in dB or in percent.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryLTEDownlinkMeasurementEVMPhysicalChannel(int Measurement_Type, out double Result)
        {
            int pInvokeResult = PInvoke.QueryLTEDownlinkMeasurementEVMPhysicalChannel(this._handle, Measurement_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Queries the "EVM Physical Signal" value from the result summary list.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_SIGNAL_RESULT
        /// 
        /// Remote-control command(s):
        /// FETCh:SUMMary:EVM:PSIGnal[:AVERage]?
        /// FETCh:SUMMary:EVM:PSIGnal:MAXimum?
        /// FETCh:SUMMary:EVM:PSIGnal:MINimum?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement_Type">
        /// Selects measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MIN (1)
        /// </param>
        /// <param name="Result">
        /// Returns selected result. Depending on the EVM unit selected in the "General Settings" menu the value will be either in dB or in percent.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryLTEDownlinkMeasurementEVMPhysicalSignal(int Measurement_Type, out double Result)
        {
            int pInvokeResult = PInvoke.QueryLTEDownlinkMeasurementEVMPhysicalSignal(this._handle, Measurement_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Queries "Frequency Error" value from the result summary list in Hz.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_FREQUENCY_ERROR_RESULT
        /// 
        /// Remote-control command(s):
        /// FETCh:SUMMary:FERRor[:AVERage]?
        /// FETCh:SUMMary:FERRor:MAXimum?
        /// FETCh:SUMMary:FERRor:MINimum?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement_Type">
        ///     Selects measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MIN (1)
        /// </param>
        /// <param name="Result">
        /// Returns selected result.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryLTEDownlinkMeasurementFrequencyError(int Measurement_Type, out double Result)
        {
            int pInvokeResult = PInvoke.QueryLTEDownlinkMeasurementFrequencyError(this._handle, Measurement_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Queries "Sampling Error" value from the result summary list in ppm.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_SAMPLING_ERROR_RESULT
        /// 
        /// Remote-control command(s):
        /// FETCh:SUMMary:SERRor[:AVERage]?
        /// FETCh:SUMMary:SERRor:MAXimum?
        /// FETCh:SUMMary:SERRor:MINimum?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement_Type">
        /// Selects measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MIN (1)
        /// </param>
        /// <param name="Result">
        /// Returns selected result.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryLTEDownlinkMeasurementSamplingError(int Measurement_Type, out double Result)
        {
            int pInvokeResult = PInvoke.QueryLTEDownlinkMeasurementSamplingError(this._handle, Measurement_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the IQ offset from the result summary list in dB.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_IQ_OFFSET_RESULT
        /// 
        /// Remote-control command(s):
        /// FETCh:SUMMary:IQOFfset[:AVERage]?
        /// FETCh:SUMMary:IQOFfset:MAXimum?
        /// FETCh:SUMMary:IQOFfset:MINimum?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement_Type">
        /// Selects measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MIN (1)
        /// </param>
        /// <param name="Result">
        /// Returns selected result.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryLTEDownlinkMeasurementIQOffset(int Measurement_Type, out double Result)
        {
            int pInvokeResult = PInvoke.QueryLTEDownlinkMeasurementIQOffset(this._handle, Measurement_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the IQ gain imbalance from the result summary list in dB.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_IQ_GAIN_IMBALANCE_RESULT
        /// 
        /// Remote-control command(s):
        /// FETCh:SUMMary:GIMBalance[:AVERage]?
        /// FETCh:SUMMary:GIMBalance:MAXimum?
        /// FETCh:SUMMary:GIMBalance:MINimum?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement_Type">
        /// Selects measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MIN (1)
        /// </param>
        /// <param name="Result">
        /// Returns selected result.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryLTEDownlinkMeasurementIQGainImbalance(int Measurement_Type, out double Result)
        {
            int pInvokeResult = PInvoke.QueryLTEDownlinkMeasurementIQGainImbalance(this._handle, Measurement_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the IQ quadrature error from the result summary list in degrees.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_IQ_QUADRATURE_ERROR_RESULT
        /// 
        /// Remote-control command(s):
        /// FETCh:SUMMary:QUADerror[:AVERage]?
        /// FETCh:SUMMary:QUADerror:MAXimum?
        /// FETCh:SUMMary:QUADerror:MINimum?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement_Type">
        /// Selects measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MIN (1)
        /// </param>
        /// <param name="Result">
        /// Returns selected result.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryLTEDownlinkMeasurementIQQuadratureError(int Measurement_Type, out double Result)
        {
            int pInvokeResult = PInvoke.QueryLTEDownlinkMeasurementIQQuadratureError(this._handle, Measurement_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the "Frame Power" from the result summary list in dBm.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_FRAME_POWER_RESULT
        /// 
        /// Remote-control command(s):
        /// FETCh:SUMMary:POWer[:AVERage]?
        /// FETCh:SUMMary:POWer:MAXimum?
        /// FETCh:SUMMary:POWer:MINimum?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement_Type">
        /// Selects measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MIN (1)
        /// </param>
        /// <param name="Result">
        /// Returns selected result.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryLTEDownlinkMeasurementFramePower(int Measurement_Type, out double Result)
        {
            int pInvokeResult = PInvoke.QueryLTEDownlinkMeasurementFramePower(this._handle, Measurement_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the crest factor from the result summary list in dB.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_CREST_FACTOR_RESULT
        /// 
        /// Remote-control command(s):
        /// FETCh:SUMMary:CRESt?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns the crest factor from the result summary list in dB.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryLTEDownlinkMeasurementCrestFactor(out double Result)
        {
            int pInvokeResult = PInvoke.QueryLTEDownlinkMeasurementCrestFactor(this._handle, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the trigger to frame value.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_TRIGGER_TO_FRAME_RESULT
        /// 
        /// Remote-control command(s):
        /// FETCh:SUMMary:TFRame?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns the trigger to frame value.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryLTEDownlinkMeasurementTriggerToFrame(out double Result)
        {
            int pInvokeResult = PInvoke.QueryLTEDownlinkMeasurementTriggerToFrame(this._handle, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the OSTP value from the result list in dBm.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_OSTP_RESULT
        /// 
        /// Remote-control command(s):
        /// FETCh:SUMMary:OSTP?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns the OSTP value from the result list in dBm.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryLTEDownlinkMeasurementOSTP(out double Result)
        {
            int pInvokeResult = PInvoke.QueryLTEDownlinkMeasurementOSTP(this._handle, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the current result values of the adjacent channel power measurement. An ACPR (Adjacent channel power relative) measurement must have previously been run for there to be summary data available. 
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1&gt;:FUNCtion:POWer:RESult[:CURRent]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// This control defines size of 'Result' array.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 5
        /// </param>
        /// <param name="Result">
        /// Adjacent channel power values are output in dB. The returned list is variable length depending on the number of channels to be measured, i.e. if the number of channels is 2 then the list will contain 5 results (main channel plus two results each for each adjacent channel specified in following order).
        /// 
        /// 1. Power of main channel in dBm
        /// 2. Rel. Power of lower adjacent channel in dB
        /// 3. Rel. Power of upper adjacent channel in dB
        /// 4. Rel. Power of lower alternate adjacent channel 1 in dB
        /// 5. Rel. Power of upper alternate adjacent channel 1 in dB
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryLTEDownlinkMeasurementACPResult(int Array_Size, double[] Result)
        {
            int pInvokeResult = PInvoke.QueryLTEDownlinkMeasurementACPResult(this._handle, Array_Size, Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the cell identity group detected by the DSP kernel. If no valid value has been detected yet, the function will return -1.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_DETECTED_CELL_IDENTITY_GROUP
        /// 
        /// Remote-control command(s):
        /// FETCh:PLCI:CIDGroup?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns the cell identity group detected by the DSP kernel. If no valid value has been detected yet, the control will return -1.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryLTEDownlinkDetectedCellIdentityGroup(out int Result)
        {
            int pInvokeResult = PInvoke.QueryLTEDownlinkDetectedCellIdentityGroup(this._handle, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the cell identity detected by the DSP kernel. If no valid value has been detected yet, the function will return -1.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_DETECTED_CELL_IDENTITY
        /// 
        /// Remote-control command(s):
        /// FETCh:PLCI:PLID?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns the cell identity detected by the DSP kernel. If no valid value has been detected yet, the control will return -1.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryLTEDownlinkDetectedCellIdentity(out int Result)
        {
            int pInvokeResult = PInvoke.QueryLTEDownlinkDetectedCellIdentity(this._handle, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the cyclic prefix detected by the DSP kernel. If no valid value has been detected yet, the function will return -1.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_LTE_DOWNLINK_DETECTED_CYCLIC_PREFIX
        /// 
        /// Remote-control command(s):
        /// FETCh:CYCPrefix?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns the cyclic prefix detected by the DSP kernel. If no valid value has been detected yet, the control will return -1.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryLTEDownlinkDetectedCyclicPrefix(out int Result)
        {
            int pInvokeResult = PInvoke.QueryLTEDownlinkDetectedCyclicPrefix(this._handle, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the EVM-vs.-Carrier-values as list over all carriers. Depends on the Subframe selection that can be made in the General Settings.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// TRACe[:DATA]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement_Type">
        /// Selects measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MIN (1)
        /// </param>
        /// <param name="Array_Size">
        /// Number of array points requested.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 1024
        /// </param>
        /// <param name="Trace_Data">
        /// Returns the EVM-vs.-Carrier-values as list over all carriers. Depends on the Subframe selection that can be made in the General Settings.
        /// </param>
        /// <param name="No_of_Values">
        /// Returns the number of trace data values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadLTEDownlinkEVMVersusCarrierTrace(int Measurement_Type, int Array_Size, double[] Trace_Data, out int No_of_Values)
        {
            int pInvokeResult = PInvoke.ReadLTEDownlinkEVMVersusCarrierTrace(this._handle, Measurement_Type, Array_Size, Trace_Data, out No_of_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the EVM-vs.-Symbol-values as list over all symbols.  If a single subframe is selected in the General Settings, only the symbols of the selected subframe will be returned.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// TRACe[:DATA]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// Number of array points requested.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 1024
        /// </param>
        /// <param name="Trace_Data">
        /// Returns the EVM-vs.-Symbol-values as list over all symbols.  If a single subframe is selected in the General Settings, only the symbols of the selected subframe will be returned.
        /// </param>
        /// <param name="No_of_Values">
        /// Returns the number of trace data values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadLTEDownlinkEVMVersusSymbolTrace(int Array_Size, double[] Trace_Data, out int No_of_Values)
        {
            int pInvokeResult = PInvoke.ReadLTEDownlinkEVMVersusSymbolTrace(this._handle, Array_Size, Trace_Data, out No_of_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the Frequency Error vs. Symbol values as list over all symbols in increasing order. If a single subframe is selected in the "General Settings", only the symbols of the selected subframe are returned.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// TRACe[:DATA]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// Number of array points requested.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 1024
        /// </param>
        /// <param name="Trace_Data">
        /// Returns the Frequency Error vs. Symbol values as list over all symbols in increasing order. If a single subframe is selected in the "General Settings", only the symbols of the selected subframe are returned.
        /// </param>
        /// <param name="No_of_Values">
        /// Returns the number of trace data values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadLTEDownlinkFrequencyErrorVersusSymbolTrace(int Array_Size, double[] Trace_Data, out int No_of_Values)
        {
            int pInvokeResult = PInvoke.ReadLTEDownlinkFrequencyErrorVersusSymbolTrace(this._handle, Array_Size, Trace_Data, out No_of_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the EVM-vs.-Symbol-values as list over all subframes.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// TRACe[:DATA]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// Number of array points requested.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 1024
        /// </param>
        /// <param name="Trace_Data">
        /// Returns the EVM-vs.-Symbol-values as list over all subframes.
        /// </param>
        /// <param name="No_of_Values">
        /// Returns the number of trace data values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadLTEDownlinkEVMVersusSubframeTrace(int Array_Size, double[] Trace_Data, out int No_of_Values)
        {
            int pInvokeResult = PInvoke.ReadLTEDownlinkEVMVersusSubframeTrace(this._handle, Array_Size, Trace_Data, out No_of_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the spectrum emission mask results.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// TRACe[:DATA]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement_Type">
        /// Selects measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_VALUES  (0) - Clear Power Values
        /// RSSPECAN_VAL_MEASTYPE_SUMMARY (1) - Summary Results
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_VALUES (1)
        /// 
        /// Note(s):
        /// 
        /// (1) Clear Power Values: power values in dBm
        /// (2) Summary Results:
        /// 1st Value - Index in the table of results 
        /// 2nd Value - Start frequency band (Hz) 
        /// 3rd Value - Stop frequency band (Hz) 
        /// 4th Value - RBW (Hz) 
        /// 5th Value - limit fail frequency (Hz) 
        /// 6th Value - Power absolute (dBm) 
        /// 7th Value - Power relative (dBc) 
        /// 8th Value - Limit distance (dB) 
        /// 9th Value - Failure flag (1 = fail, 0 = pass) 
        /// </param>
        /// <param name="Array_Size">
        /// Number of array points requested.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 1024
        /// </param>
        /// <param name="Trace_Data">
        /// Returns the spectrum emission mask results.
        /// </param>
        /// <param name="No_of_Values">
        /// Returns the number of trace data values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadLTEDownlinkSEMTrace(int Measurement_Type, int Array_Size, double[] Trace_Data, out int No_of_Values)
        {
            int pInvokeResult = PInvoke.ReadLTEDownlinkSEMTrace(this._handle, Measurement_Type, Array_Size, Trace_Data, out No_of_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the ACP power values as vector.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// TRACe[:DATA]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// Number of array points requested.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 1024
        /// </param>
        /// <param name="Trace_Data">
        /// Returns the ACP power values as vector.
        /// </param>
        /// <param name="No_of_Values">
        /// Returns the number of trace data values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadLTEDownlinkACPTrace(int Array_Size, double[] Trace_Data, out int No_of_Values)
        {
            int pInvokeResult = PInvoke.ReadLTEDownlinkACPTrace(this._handle, Array_Size, Trace_Data, out No_of_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the signal power in dBm/Hz as list over the considered frequency span.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// TRACe[:DATA]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// Number of array points requested.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 1024
        /// </param>
        /// <param name="Trace_Data">
        /// Returns the signal power in dBm/Hz as list over the considered frequency span.
        /// </param>
        /// <param name="No_of_Values">
        /// Returns the number of trace data values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadLTEDownlinkPowerSpectrumTrace(int Array_Size, double[] Trace_Data, out int No_of_Values)
        {
            int pInvokeResult = PInvoke.ReadLTEDownlinkPowerSpectrumTrace(this._handle, Array_Size, Trace_Data, out No_of_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the reference signal power per RB in dBm as list over the currently available RBs.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// TRACe[:DATA]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement_Type">
        /// Selects measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MIN (1)
        /// </param>
        /// <param name="Array_Size">
        /// Number of array points requested.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 1024
        /// </param>
        /// <param name="Trace_Data">
        /// Returns the reference signal power per RB in dBm as list over the currently available RBs.
        /// </param>
        /// <param name="No_of_Values">
        /// Returns the number of trace data values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadLTEDownlinkPowerVersusRBReferenceSignalTrace(int Measurement_Type, int Array_Size, double[] Trace_Data, out int No_of_Values)
        {
            int pInvokeResult = PInvoke.ReadLTEDownlinkPowerVersusRBReferenceSignalTrace(this._handle, Measurement_Type, Array_Size, Trace_Data, out No_of_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the PDSCH signal power per RB in dBm as list over the currently available RBs.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// TRACe[:DATA]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement_Type">
        /// Selects measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MIN (1)
        /// </param>
        /// <param name="Array_Size">
        /// Number of array points requested.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 1024
        /// </param>
        /// <param name="Trace_Data">
        /// Returns the PDSCH signal power per RB in dBm as list over the currently available RBs.
        /// </param>
        /// <param name="No_of_Values">
        /// Returns the number of trace data values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadLTEDownlinkPowerVersusRBPDSCHTrace(int Measurement_Type, int Array_Size, double[] Trace_Data, out int No_of_Values)
        {
            int pInvokeResult = PInvoke.ReadLTEDownlinkPowerVersusRBPDSCHTrace(this._handle, Measurement_Type, Array_Size, Trace_Data, out No_of_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the spectrum flatness in dB as list over the considered frequency span. Depends on the subframe selection that can be made in the General Settings.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// TRACe[:DATA]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement_Type">
        /// Selects measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MIN (1)
        /// </param>
        /// <param name="Array_Size">
        /// Number of array points requested.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 1024
        /// </param>
        /// <param name="Trace_Data">
        /// Returns the spectrum flatness in dB as list over the considered frequency span. Depends on the subframe selection that can be made in the General Settings.
        /// </param>
        /// <param name="No_of_Values">
        /// Returns the number of trace data values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadLTEDownlinkChannelFlatnessTrace(int Measurement_Type, int Array_Size, double[] Trace_Data, out int No_of_Values)
        {
            int pInvokeResult = PInvoke.ReadLTEDownlinkChannelFlatnessTrace(this._handle, Measurement_Type, Array_Size, Trace_Data, out No_of_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the channel group delay in ns as list over the considered frequency span. Depends on the Subframe selection that can be made in the General Settings.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// TRACe[:DATA]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement_Type">
        /// Selects measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MIN (1)
        /// </param>
        /// <param name="Array_Size">
        /// Number of array points requested.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 1024
        /// </param>
        /// <param name="Trace_Data">
        /// Returns the channel group delay in ns as list over the considered frequency span. Depends on the Subframe selection that can be made in the General Settings.
        /// </param>
        /// <param name="No_of_Values">
        /// Returns the number of trace data values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadLTEDownlinkChannelGroupDelayTrace(int Measurement_Type, int Array_Size, double[] Trace_Data, out int No_of_Values)
        {
            int pInvokeResult = PInvoke.ReadLTEDownlinkChannelGroupDelayTrace(this._handle, Measurement_Type, Array_Size, Trace_Data, out No_of_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the channel flatness difference in dB as list over the considered frequency span. Depends on the Subframe selection that can be made in the General Settings.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// TRACe[:DATA]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement_Type">
        /// Selects measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_MIN  (1) - Minimum
        /// RSSPECAN_VAL_MEASTYPE_MAX  (2) - Maximum
        /// RSSPECAN_VAL_MEASTYPE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_MIN (1)
        /// </param>
        /// <param name="Array_Size">
        /// Number of array points requested.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 1024
        /// </param>
        /// <param name="Trace_Data">
        /// Returns the channel flatness difference in dB as list over the considered frequency span. Depends on the Subframe selection that can be made in the General Settings.
        /// </param>
        /// <param name="No_of_Values">
        /// Returns the number of trace data values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadLTEDownlinkChannelFlatnessDifferenceTrace(int Measurement_Type, int Array_Size, double[] Trace_Data, out int No_of_Values)
        {
            int pInvokeResult = PInvoke.ReadLTEDownlinkChannelFlatnessDifferenceTrace(this._handle, Measurement_Type, Array_Size, Trace_Data, out No_of_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This measurement represents I and Q data. Data will be returned as an array of interleaved I and Q data until all data is exhausted. By default all detected data-points will be returned. The amount of data can be narrowed by changing settings in the "Constellation Selection" menu. Only data points that meet the requirements specified in the "Constellation Selection" menu will be returned. The the constellation data will be sweeped and returned in the following order: 
        /// Subframe0 -&gt; Symbol0 -&gt; First Carrier to last Carrier of Symbol 0 
        /// Subframe0 -&gt; Symbol1 -&gt; First Carrier0 to last Carrier of Symbol 1
        /// Subframe0 -&gt; ...
        /// Subframe0 -&gt; Last Symbol of Subframe0 -&gt; First Carrier to last Carrier
        /// Subframe1 -&gt; Symbol0 -&gt; First Carrier to last Carrier of Symbol 0
        /// Subframe1 -&gt; Symbol1 -&gt; First Carrier to last Carrier of Symbol 1
        /// Subframe1 -&gt; ...
        /// Subframe1 -&gt; Last Symbol of Subframe0 -&gt; First Carrier to last Carrier
        /// ...
        /// Last Subframe 
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;[:DATA]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of doubles in the array you specify for the real parts and imaginary parameters.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the data contains more elemens than you specify with the Buffer Size parameter, the function returns Buffer Size elements into the arrays and returns the buffer size you must pass to get the all data.  For example, if the data contains 16000 elements for real parts and 16000 elements for imaginary part and the Buffer Size is 10000, the function places 10000 elements into the arrays and returns 16000.
        /// 
        /// Notes:
        /// 
        /// (1) If VI_ATTR_INTF_TYPE is equal to VI_INTF_ASRL, then use this parameter to pass the expected number of points to be read from the instrument. Data points will be read until instruments output buffer is empty. If Buffer Size is lower than actual number of results available in the instruments output buffer, then this function returns an error code (BFFC0002  Parameter 2 (Buffer Size) out of range.) and measurement results will not be valid. This is due to limitation if the unit is remote-controlled via the RS-232-C interface.
        /// 
        /// (2) This function requires that all response data are read out completely before the instrument accepts further commands.
        /// 
        /// </param>
        /// <param name="No_of_Points">
        /// Returns the number of I/Q trace data points.
        /// 
        /// </param>
        /// <param name="Real_Parts__I_">
        /// Returns an array of trace data (I-value x, real part) point. The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter.
        /// 
        /// </param>
        /// <param name="Imaginary_Parts__Q_">
        /// Returns an array of trace data (Q-value y, imaginary part) point. The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadLTEDownlinkConstellationDiagram(int Buffer_Size, out int No_of_Points, double[] Real_Parts__I_, double[] Imaginary_Parts__Q_)
        {
            int pInvokeResult = PInvoke.ReadLTEDownlinkConstellationDiagram(this._handle, Buffer_Size, out No_of_Points, Real_Parts__I_, Imaginary_Parts__Q_);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the complementary cumulative distribution function results in percent as list over the power level in dB. The first value returned represents the number of following values. Trace1 will return the probablility-values (Y-Axis) while Trace2 will return the corresponding power-level-steps (X-Axis).
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// TRACe[:DATA]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement_Type">
        /// Selects measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEASTYPE_PROBABILITY (0) - Probability Values (Y-Axis)
        /// RSSPECAN_VAL_MEASTYPE_POWER_STEPS (1) - Corresponding Power Steps (X-Axis)
        /// 
        /// Default Value: RSSPECAN_VAL_MEASTYPE_PROBABILITY (0)
        /// </param>
        /// <param name="Array_Size">
        /// Number of array points requested.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 1024
        /// </param>
        /// <param name="Trace_Data">
        /// Returns the complementary cumulative distribution function results in percent as list over the power level in dB. The first value returned represents the number of following values. Trace1 will return the probablility-values (Y-Axis) while Trace2 will return the corresponding power-level-steps (X-Axis).
        /// </param>
        /// <param name="No_of_Values">
        /// Returns the number of trace data values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadLTEDownlinkCCDF(int Measurement_Type, int Array_Size, double[] Trace_Data, out int No_of_Values)
        {
            int pInvokeResult = PInvoke.ReadLTEDownlinkCCDF(this._handle, Measurement_Type, Array_Size, Trace_Data, out No_of_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the complementary cumulative distribution function results in percent as list over the power level in dB. The first value returned represents the number of following values. Trace1 will return the probablility-values (Y-Axis) while Trace2 will return the corresponding power-level-steps (X-Axis).
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// TRACe[:DATA]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// Number of array points requested.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 256
        /// </param>
        /// <param name="Subframe_Number">
        /// Subframe number.
        /// </param>
        /// <param name="Allocation_ID">
        ///  Allocation ID where: 
        /// -1  = INVALID 
        /// -2  = ALL 
        /// -3  = P-SYNC 
        /// -4  = S-SYNC 
        /// -5  = PILOTS_ANT1 
        /// -6  = PILOTS_ANT2 
        /// -7  = PILOTS_ANT3 
        /// -8  = PILOTS_ANT4  
        /// -9  = PCFICH 
        /// -10 = PHICH 
        /// -11 = PDCCH 
        /// -12 = PBCH 
        /// -13 = PMCH
        /// </param>
        /// <param name="Number_of_RB">
        /// Number of RB.
        /// </param>
        /// <param name="Offset_RB">
        /// Number of RB.
        /// </param>
        /// <param name="Modulation">
        ///  Modulation 
        /// 0    = Unrecognized 
        /// 1    = RBPSK (both constellation points are located on the X-axis) 
        /// 2    = QPSK 
        /// 3    = 16QAM 
        /// 4    = 64QAM 
        /// 5    = 8PSK 
        /// 6    = PSK 
        /// 7    = Modulation Mixture 
        /// 8    = BPSK
        /// </param>
        /// <param name="Power">
        /// Power in dBm.
        /// </param>
        /// <param name="EVM">
        /// EVM in dB or percent.
        /// </param>
        /// <param name="No_of_Values">
        /// Returns the number of trace data values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadLTEDownlinkAllocationSummary(int Array_Size, int[] Subframe_Number, int[] Allocation_ID, int[] Number_of_RB, int[] Offset_RB, int[] Modulation, double[] Power, double[] EVM, out int No_of_Values)
        {
            int pInvokeResult = PInvoke.ReadLTEDownlinkAllocationSummary(this._handle, Array_Size, Subframe_Number, Allocation_ID, Number_of_RB, Offset_RB, Modulation, Power, EVM, out No_of_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Bitsteam data will be returned in groups where each group represents one line of the table. The result data has to be interpreted differently for downlink and uplink and additionally it is important to distinguish between bitstream and hexadecimal data format.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// TRACe[:DATA]?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// Number of array points requested.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 256
        /// </param>
        /// <param name="Bit_stream_Size">
        /// Size of bitstream array.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 4096
        /// </param>
        /// <param name="Subframe_Number">
        /// Subframe number.
        /// </param>
        /// <param name="Allocation_ID">
        ///  Allocation ID where: 
        /// -1  = INVALID 
        /// -2  = ALL 
        /// -3  = P-SYNC 
        /// -4  = S-SYNC 
        /// -5  = PILOTS_ANT1 
        /// -6  = PILOTS_ANT2 
        /// -7  = PILOTS_ANT3 
        /// -8  = PILOTS_ANT4  
        /// -9  = PCFICH 
        /// -10 = PHICH 
        /// -11 = PDCCH 
        /// -12 = PBCH 
        /// -13 = PMCH
        /// </param>
        /// <param name="Codeword">
        /// Number of RB.
        /// 
        /// 0    = '1/1' 
        /// 1    = '1/2' 
        /// 2    = '2/2'
        /// </param>
        /// <param name="Modulation">
        ///  Modulation 
        /// 0    = Unrecognized 
        /// 1    = RBPSK (both constellation points are located on the X-axis) 
        /// 2    = QPSK 
        /// 3    = 16QAM 
        /// 4    = 64QAM 
        /// 5    = 8PSK 
        /// 6    = PSK 
        /// 7    = Modulation Mixture 
        /// 8    = BPSK
        /// </param>
        /// <param name="Number_of_Symbols">
        /// Power in dBm.
        /// </param>
        /// <param name="Bit_Streams">
        /// Bit stream of binary numbers. Overall number of bytes required for this array is defined by multiplication of number of datablocks and its corresponding sizes.
        /// </param>
        /// <param name="No_of_Values">
        /// Returns the number of trace data values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadLTEDownlinkBitstream(int Array_Size, int Bit_stream_Size, int[] Subframe_Number, int[] Allocation_ID, int[] Codeword, int[] Modulation, int[] Number_of_Symbols, System.Text.StringBuilder Bit_Streams, out int No_of_Values)
        {
            int pInvokeResult = PInvoke.ReadLTEDownlinkBitstream(this._handle, Array_Size, Bit_stream_Size, Subframe_Number, Allocation_ID, Codeword, Modulation, Number_of_Symbols, Bit_Streams, out No_of_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects TETRA 2 analysis mode.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_MODE
        /// 
        /// Remote-control command(s):
        /// INSTrument:SELect TETRA2
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SetTETRAMode()
        {
            int pInvokeResult = PInvoke.SetTETRAMode(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the general settings concerning the physical attributes of the signals to be measured.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FREQUENCY_CENTER
        /// RSSPECAN_ATTR_TETRA_BURST_TYPE
        /// RSSPECAN_ATTR_TETRA_CHANNEL_BANDWIDTH
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer
        /// [SENSe:]DEMod:FORMat:BANalyze:BTYPe NDB | NUB | CUB | RAB
        /// [SENSe:]DEMod:FORMat:BANalyze:BANDwidth  B25 | B50 | B100 | B150
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Frequency">
        /// The control defines the center frequency of the analyzer.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 1.0e+9 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FREQUENCY_CENTER
        /// 
        /// Note(s):
        /// 
        /// (1) When using the digital Baseband input, the Frequency parameter is ignored.
        /// </param>
        /// <param name="Slot_Type">
        /// This control specifies the type of burst to be analyzed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_SLOT_TYPE_NDB (0) - Normal Downling Burst
        /// RSSPECAN_VAL_TETRA_SLOT_TYPE_NUB (1) - Normal Upling Burst
        /// RSSPECAN_VAL_TETRA_SLOT_TYPE_CUB (2) - Control Uplink Burst
        /// RSSPECAN_VAL_TETRA_SLOT_TYPE_RAB (3) - Random Access Burst
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_SLOT_TYPE_NDB (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_BURST_TYPE
        /// </param>
        /// <param name="Channel_Bandwidth">
        /// Configures the analysis channel bandwidth.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_CHANN_BAND_25  (0) - 25 kHz
        /// RSSPECAN_VAL_TETRA_CHANN_BAND_50  (1) - 50 kHz
        /// RSSPECAN_VAL_TETRA_CHANN_BAND_100 (2) - 100 kHz
        /// RSSPECAN_VAL_TETRA_CHANN_BAND_150 (3) - 150 kHz
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_CHANN_BAND_25 (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_CHANNEL_BANDWIDTH
        /// 
        /// Note(s):
        /// 
        /// (1) When slot type is Random Access Burst (RAB), only the 25kHz channel bandwidth is supported.
        /// 
        /// (2) 25 kHz channel bandwidth uses 8 carriers.
        /// 
        /// (3) 50 kHz channel bandwidth uses 16 carriers.
        /// 
        /// (4) 100 kHz channel bandwidth uses 32 carriers.
        /// 
        /// (5) 150 kHz channel bandwidth uses 48 carriers.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTETRASignalCharacteristics(double Frequency, int Slot_Type, int Channel_Bandwidth)
        {
            int pInvokeResult = PInvoke.ConfigureTETRASignalCharacteristics(this._handle, Frequency, Slot_Type, Channel_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures settings related to the reference level and the RF attenuation.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_REFERENCE_LEVEL_FOR_RF_MEASUREMENT
        /// RSSPECAN_ATTR_TETRA_REFERENCE_LEVEL_FOR_IQ_MEASUREMENT
        /// RSSPECAN_ATTR_DIGITAL_INPUT_RANGE
        /// RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET
        /// 
        /// Remote-control command(s):
        /// DISPlay:WINDow&lt;1|2&gt;:TRACe1:Y[:SCALe]RLEVel:RF
        /// DISPlay:WINDow&lt;1|2&gt;:TRACe1:Y[:SCALe]RLEVel:IQ
        /// INPut&lt;1|2&gt;:DIQ:RANGe[:UPPer]
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1 to 3&gt;:Y[:SCALe]:RLEVel:OFFSet
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Auto_Level">
        /// This control is reserved for the future use.
        /// </param>
        /// <param name="Level">
        /// Configures the signal level of the analyzers used in RF mode and for analog baseband input or full scale level for digital baseband input.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_REFERENCE_LEVEL_FOR_RF_MEASUREMENT
        /// RSSPECAN_ATTR_TETRA_REFERENCE_LEVEL_FOR_IQ_MEASUREMENT
        /// RSSPECAN_ATTR_DIGITAL_INPUT_RANGE
        /// 
        /// Note(s):
        /// 
        /// (1) Unit for RF input is dBm, for analog and digital baseband input is volt.
        /// </param>
        /// <param name="External_Attenuation">
        /// This control specifies the external attenuation or gain applied to the RF signal. A positive value indicates attenuation, a negative value indicates gain.
        /// 
        /// Valid Values:
        /// -2.0E+002 dB to 2.0E+002 dB
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET
        /// 
        /// Note(s):
        /// 
        /// (1) When using the analog or digital baseband input, the External Attenuation parameter is ignored.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTETRALevelSettings(bool Auto_Level, double Level, double External_Attenuation)
        {
            int pInvokeResult = PInvoke.ConfigureTETRALevelSettings(this._handle, System.Convert.ToUInt16(Auto_Level), Level, External_Attenuation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures how much data is to be captured.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SWEEP_TIME
        /// RSSPECAN_ATTR_TETRA_TEDS_SLOT_DURATION
        /// RSSPECAN_ATTR_TETRA_OVERALL_BURST_COUNT
        /// RSSPECAN_ATTR_TETRA_NUMBER_OF_BURST
        /// 
        /// Remote-control command(s):
        /// [SENSe]:SWEep:TIME
        /// [SENSe]:SWEep:SLOTs
        /// [SENSe:]BURSt:COUNt:STATe
        /// [SENSe:]BURSt:COUNt
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Capture_Time">
        /// Configures the time (and therefore the amount of data) to be captured in each capture buffer. The Capture Time can be specified in TEDS full time-slots  or seconds.
        /// 
        /// Valid Values:
        /// 34.0e-3 to 1.53 s (when unit Seconds is selected)
        /// 2.4     to 108    (when unit Slots is selected)
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SWEEP_TIME
        /// RSSPECAN_ATTR_TETRA_TEDS_SLOT_DURATION
        /// 
        /// </param>
        /// <param name="Capture_Time_Unit">
        /// This control specifies whether the capture time is set in seconds or in slots.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_UNIT_SECONDS (0) - Seconds
        /// RSSPECAN_VAL_TETRA_UNIT_SLOTS   (1) - Slots
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_UNIT_SECONDS (0)
        /// 
        /// </param>
        /// <param name="Use_Number_Of_Slots_To_Analyzer">
        /// This control sets whether the number of slots to analyze control is used or not.
        /// 
        /// Valid Value:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_OVERALL_BURST_COUNT
        /// </param>
        /// <param name="Number_Of_Slots_To_Analyze">
        /// This control specifies the number of slots to be measured.
        /// 
        /// Valid Values:
        /// 1 to 100000
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_NUMBER_OF_BURST
        /// 
        /// Note(s):
        /// 
        /// (1) When the control Use Number Of Slots To Analyze is set to OFF, then this parameter is ignored.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTETRADataCaptureSettings(double Capture_Time, int Capture_Time_Unit, bool Use_Number_Of_Slots_To_Analyzer, int Number_Of_Slots_To_Analyze)
        {
            int pInvokeResult = PInvoke.ConfigureTETRADataCaptureSettings(this._handle, Capture_Time, Capture_Time_Unit, System.Convert.ToUInt16(Use_Number_Of_Slots_To_Analyzer), Number_Of_Slots_To_Analyze);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the trigger parameters. 
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available for RF input and analog baseband input.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_TRIGGER_MODE
        /// RSSPECAN_ATTR_TRIGGER_DELAY
        /// RSSPECAN_ATTR_TETRA_TRIGGER_TEDS_SLOT_DURATION
        /// RSSPECAN_ATTR_EXTERNAL_TRIGGER_LEVEL
        /// RSSPECAN_ATTR_TETRA_TRIGGER_POWER_LEVEL
        /// RSSPECAN_ATTR_TETRA_TRIGGER_IQ_LEVEL
        /// 
        /// 
        /// Remote-control command(s):
        /// TRIGger[:SEQuence]:MODE IMMediate | EXTernal | POWer
        /// TRIGger[:SEQuence]:HOLDoff
        /// TRIGger[:SEQuence]:HOLDoff:SLOT
        /// TRIGger[:SEQuence]:LEVel:POWer
        /// TRIGger[:SEQuence]:LEVel:IQ
        /// TRIGger[:SEQuence]:LEVel[:EXTernal]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Mode">
        /// Sets the trigger mode.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TRIG_MODE_IMM (0) - Immediate
        /// RSSPECAN_VAL_TRIG_MODE_EXT (1) - External
        /// RSSPECAN_VAL_TRIG_MODE_POW (2) - Power 
        /// 
        /// Default Value: RSSPECAN_VAL_TRIG_MODE_IMM (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_TRIGGER_MODE
        /// </param>
        /// <param name="Offset">
        /// Defines the length of the trigger delay. A negative delay time (pretrigger) can be set.
        /// 
        /// Valid Values:
        /// -155.83 to 42.5 s (when unit is Seconds)
        /// -11     to 3000   (when unit is Slots)
        /// 
        /// Default Value: 0.0 s
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRIGGER_DELAY
        /// RSSPECAN_ATTR_TETRA_TRIGGER_TEDS_SLOT_DURATION
        /// 
        /// Note:
        /// (1) This parameter has effect only if Trigger Mode isn't Free Run
        /// 
        /// 
        /// </param>
        /// <param name="Offset_Unit">
        /// This control specifies whether the trigger offset is set in seconds or in slots.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_UNIT_SECONDS (0) - Seconds
        /// RSSPECAN_VAL_TETRA_UNIT_SLOTS   (1) - Slots
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_UNIT_SECONDS (0)
        /// </param>
        /// <param name="Ext__Trigger_Level">
        /// Specifies the level of the external trigger input for which triggering will occur.
        /// 
        /// Valid Values:
        /// 500.0e-3 to 3.5 V
        /// 
        /// Default Value: 1.4 V
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_EXTERNAL_TRIGGER_LEVEL
        /// 
        /// Note:
        /// (1) This parameter has effect only for Trigger Mode set to External
        /// 
        /// 
        /// </param>
        /// <param name="Auto_Trigger_Level">
        /// This control is reserved for the future use.
        /// 
        /// </param>
        /// <param name="Power_Trigger_Level">
        /// Specifies the level of the input signal for which triggering will occur. The value is specified in dBm for RF input and V for baseband inputs.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_TRIGGER_POWER_LEVEL
        /// RSSPECAN_ATTR_TETRA_TRIGGER_IQ_LEVEL
        /// 
        /// 
        /// Note:
        /// (1) This parameter has effect only for Trigger Mode set to Power
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTETRATriggerSettings(int Mode, double Offset, int Offset_Unit, double Ext__Trigger_Level, bool Auto_Trigger_Level, double Power_Trigger_Level)
        {
            int pInvokeResult = PInvoke.ConfigureTETRATriggerSettings(this._handle, Mode, Offset, Offset_Unit, Ext__Trigger_Level, System.Convert.ToUInt16(Auto_Trigger_Level), Power_Trigger_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures I/Q settings.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_IQ_SETTING
        /// 
        /// Remote-control command(s):
        /// [SENSe]:SWAPiq
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Swap_IQ">
        /// Specifies if the IQ data shall be swapped.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_IQ_SETTING
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTETRAIQSettings(bool Swap_IQ)
        {
            int pInvokeResult = PInvoke.ConfigureTETRAIQSettings(this._handle, System.Convert.ToUInt16(Swap_IQ));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures whether the RF input or the analog Baseband input or the digital Baseband input is the currently selected signal input. 
        /// 
        /// Note(s):
        /// 
        /// (1) Analog Baseband input requires option FSQ-B71.
        /// 
        /// (2) Digital Baseband input requires option FSQ-B17.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_INPUT_SELECT
        /// 
        /// Remote-control command(s):
        /// INPut:SELect RF | BB | DIQ
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Input">
        /// Selects between RF and analog Baseband inputs and digital Baseband input.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_BB_INPUT_RF  (0) - RF
        /// RSSPECAN_VAL_BB_INPUT_AIQ (1) - Baseband Analog
        /// RSSPECAN_VAL_BB_INPUT_DIQ (2) - IQ Digital
        /// 
        /// Default Value: RSSPECAN_VAL_BB_INPUT_RF (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_INPUT_SELECT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTETRAInputSettings(int Input)
        {
            int pInvokeResult = PInvoke.ConfigureTETRAInputSettings(this._handle, Input);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures details of how the instrument operates and how measurements are performed. 
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available if Analog Baseband input is selected (option FSQ-B71).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_BB_INPUT_IMPEDANCE
        /// RSSPECAN_ATTR_BB_INPUT_SIGNAL_PATH
        /// RSSPECAN_ATTR_BB_INPUT_BALANCED
        /// RSSPECAN_ATTR_BB_INPUT_IQ_LPAS
        /// RSSPECAN_ATTR_BB_INPUT_IQ_DITH
        /// 
        /// Remote-control command(s):
        /// INPut:IQ:IMPedance LOW | HIGH
        /// INPut:IQ:TYPE IQ | I | Q
        /// INPut:IQ:BALanced[:STATe] ON | OFF
        /// [SENSe]:IQ:LPASs[:STATe] ON | OFF
        /// [SENSe]:IQ:DITHer[:STATe] ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Impedance">
        /// Selects the impedance of the baseband inputs.
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_BB_INPUT_IMP_LOW  (0) - Low  (50 Ohm)
        /// RSSPECAN_VAL_BB_INPUT_IMP_HIGH (1) - High (1 kOhm)
        /// 
        /// Default Value: RSSPECAN_VAL_BB_INPUT_IMP_LOW (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BB_INPUT_IMPEDANCE
        /// </param>
        /// <param name="Path">
        /// Defines the input signal path used.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_BB_SIG_PATH_I  (0) - I Only
        /// RSSPECAN_VAL_BB_SIG_PATH_Q  (1) - Q Only
        /// RSSPECAN_VAL_BB_SIG_PATH_IQ (2) - I+j*Q
        /// 
        /// Default Value: RSSPECAN_VAL_BB_SIG_PATH_IQ (2)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BB_INPUT_SIGNAL_PATH
        /// 
        /// Note:
        /// (1) I Only - Real signal I only
        ///     Q Only - Real signal Q only
        ///     I+j*Q  - Complex signal I+j*Q
        /// </param>
        /// <param name="Balanced">
        /// This attribute toggles the baseband inputs between balanced and unbalanced.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off (Unbalanced)
        /// VI_TRUE  (1) - On  (Balanced)
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BB_INPUT_BALANCED
        /// </param>
        /// <param name="Low_Pass">
        /// Switches an anti-aliasing filter into the I and Q branch of the analog Baseband inputs.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// 
        /// Default Value: VI_TRUE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BB_INPUT_IQ_LPAS
        /// </param>
        /// <param name="IQ_Dither">
        /// Links a 2 MHz broad dithering signal at 42.67 MHz into the signal path of the analog Baseband inputs.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_BB_INPUT_IQ_DITH
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTETRAAdvancedBasebandSettings(int Impedance, int Path, bool Balanced, bool Low_Pass, bool IQ_Dither)
        {
            int pInvokeResult = PInvoke.ConfigureTETRAAdvancedBasebandSettings(this._handle, Impedance, Path, System.Convert.ToUInt16(Balanced), System.Convert.ToUInt16(Low_Pass), System.Convert.ToUInt16(IQ_Dither));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures settings related to the reference level and the RF attenuation.
        /// 
        /// Note(s):
        /// This function is available only if input is set to Digital Baseband.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DIGITAL_INPUT_RANGE_AUTO
        /// RSSPECAN_ATTR_DIGITAL_INPUT_RANGE
        /// RSSPECAN_ATTR_DIGITAL_INPUT_SRATE
        /// RSSPECAN_ATTR_DIGITAL_INPUT_SAMPLE_RATE_AUTO
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:DIQ:RANGe[:UPPer]:AUTO ON | OFF
        /// INPut&lt;1|2&gt;:DIQ:RANGe[:UPPer]
        /// INPut&lt;1|2&gt;:DIQ:SRATe:AUTO ON | OFF
        /// INPut&lt;1|2&gt;:DIQ:SRATe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Auto_Level">
        /// This control sets whether the Full Scale Level in digital baseband input is set automatically or manually.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DIGITAL_INPUT_RANGE_AUTO
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available when Digital Input is selected (function rsspecan_ConfigureTETRAInputSettings), otherwise it is ignored.
        /// </param>
        /// <param name="Level">
        /// Configures the signal level of the analyzers used in RF mode and for analog baseband input or full scale level for digital baseband input.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_REFERENCE_LEVEL_FOR_RF_MEASUREMENT
        /// RSSPECAN_ATTR_TETRA_REFERENCE_LEVEL_FOR_IQ_MEASUREMENT
        /// RSSPECAN_ATTR_DIGITAL_INPUT_RANGE
        /// 
        /// Note(s):
        /// 
        /// (1) Unit for RF input is dBm, for analog and digital baseband input is volt.
        /// 
        /// (2) If Auto Level is set to On and the Digital Input is selected (function rsspecan_ConfigureTETRAInputSettings), then this control is ignored.
        /// </param>
        /// <param name="Auto_Sampling_Rate">
        /// This control sets whether the Sampling Rate in digital baseband input is set automatically or manually.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DIGITAL_INPUT_SAMPLE_RATE_AUTO
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available when Digital Input is selected (function rsspecan_ConfigureTETRAInputSettings), otherwise it is ignored.
        /// </param>
        /// <param name="Sampling_Rate">
        /// This control sets the input date sampling rate read by the digital baseband input.  
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 10.0e+6 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DIGITAL_INPUT_SRATE
        /// 
        /// Note(s):
        /// 
        /// (1) If Auto Sampling Rate is set to On and the Digital Input is selected (function rsspecan_ConfigureTETRAInputSettings), then this control is ignored.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTETRAAdvancedIQDigital(bool Auto_Level, double Level, bool Auto_Sampling_Rate, double Sampling_Rate)
        {
            int pInvokeResult = PInvoke.ConfigureTETRAAdvancedIQDigital(this._handle, System.Convert.ToUInt16(Auto_Level), Level, System.Convert.ToUInt16(Auto_Sampling_Rate), Sampling_Rate);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the IQ value for Center Frequency. 
        /// 
        /// Note(s):
        /// 
        /// (1) This function requires option B71.
        /// 
        /// (2) This is an independent value for center frequency that is used while dithering is active. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_IQ_CENTER_FREQUENCY
        /// 
        /// Remote-control command(s):
        /// [SENSe]:IQ:FREQuency[:CENTer]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="IQ_Center_Frequency">
        /// This control sets the IQ value for Center Frequency.  
        /// 
        /// Valid Values:
        /// -35.0e-6 to 35.0e6 Hz
        /// 
        /// Default Value: 0.0 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_IQ_CENTER_FREQUENCY
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTETRAIQCenterFrequency(double IQ_Center_Frequency)
        {
            int pInvokeResult = PInvoke.ConfigureTETRAIQCenterFrequency(this._handle, IQ_Center_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the characteristics of the slots to be considered in the measurement results.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_BURST_TYPE
        /// RSSPECAN_ATTR_TETRA_PAYLOAD_MODULATION
        /// RSSPECAN_ATTR_TETRA_MAXIMUM_CARRIER_OFFSET
        /// 
        /// Remote-control command(s):
        /// [SENSe:]DEMod:FORMat:BANalyze:BTYPe NDB | NUB | CUB | RAB
        /// [SENSe:]DEMod:FORMat:BANalyze:MODulation QM4 | QM16 | QM64
        /// [SENSe:]DEMod:FORMat:BANalyze:MCARrier KH1 | KH10
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Slot_Type">
        /// This control specifies the type of burst to be analyzed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_SLOT_TYPE_NDB (0) - Normal Downling Burst
        /// RSSPECAN_VAL_TETRA_SLOT_TYPE_NUB (1) - Normal Upling Burst
        /// RSSPECAN_VAL_TETRA_SLOT_TYPE_CUB (2) - Control Uplink Burst
        /// RSSPECAN_VAL_TETRA_SLOT_TYPE_RAB (3) - Random Access Burst
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_SLOT_TYPE_NDB (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_BURST_TYPE
        /// </param>
        /// <param name="Payload_Modulation">
        /// Selects the analysis modulation format.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_MOD_TYPE_QM4  (0) - 4 QAM
        /// RSSPECAN_VAL_TETRA_MOD_TYPE_QM16 (1) - 16 QAM
        /// RSSPECAN_VAL_TETRA_MOD_TYPE_QM64 (2) - 64 QAM
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_MOD_TYPE_QM64 (2)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_PAYLOAD_MODULATION
        /// 
        /// Note(s):
        /// 
        /// (1) When slot type is Random Access Burst (RAB), only the 4 QAM modulation is supported.
        /// 
        /// (2) 4 QAM: Quadrature Amplitude Modulation 4 (2 bits).
        /// 
        /// (3) 16 QAM: Quadrature Amplitude Modulation 16 (4 bits).
        /// 
        /// (4) 64 QAM: Quadrature Amplitude Modulation 64 (6 bits).
        /// </param>
        /// <param name="Demod_RRC_Roll_off">
        /// This control is reserved for the future use.
        /// </param>
        /// <param name="Max_Carrier_Offset">
        /// Selects the maximum carrier offset.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_CARRIER_OFFSET_KH1  (0) - 1 kHz
        /// RSSPECAN_VAL_TETRA_CARRIER_OFFSET_KH10 (1) - 10 kHz
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_CARRIER_OFFSET_KH10 (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_MAXIMUM_CARRIER_OFFSET
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTETRADemodulationSettings(int Slot_Type, int Payload_Modulation, double Demod_RRC_Roll_off, int Max_Carrier_Offset)
        {
            int pInvokeResult = PInvoke.ConfigureTETRADemodulationSettings(this._handle, Slot_Type, Payload_Modulation, Demod_RRC_Roll_off, Max_Carrier_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the type of calculations and normalizations that will be performed on the  demodulator's output.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_PILOT_TRACKING
        /// RSSPECAN_ATTR_TETRA_COMPENSATE_AMPLITUDE_DROOP
        /// RSSPECAN_ATTR_TETRA_COMPENSATE_IQ_OFFSET
        /// 
        /// Remote-control command(s):
        /// [SENSe:]DEMod:FORMat:BANalyze:PTRacking
        /// [SENSe:]DEMod:FORMat:BANalyze:CADRoop
        /// [SENSe:]DEMod:FORMat:BANalyze:CIQoffset
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Pilot_Tracking">
        /// Specifies whether Pilot Tracking will be used.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_PILOT_TRACKING
        /// </param>
        /// <param name="Compensate_Amplitude_Droop">
        /// Specifies whether amplitude droop compensation will be used.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_COMPENSATE_AMPLITUDE_DROOP
        /// </param>
        /// <param name="Compensate_IQ_Offset">
        /// Specifies whether IQ offset compensation will be used.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_COMPENSATE_IQ_OFFSET
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTETRAEVMSettings(bool Pilot_Tracking, bool Compensate_Amplitude_Droop, bool Compensate_IQ_Offset)
        {
            int pInvokeResult = PInvoke.ConfigureTETRAEVMSettings(this._handle, System.Convert.ToUInt16(Pilot_Tracking), System.Convert.ToUInt16(Compensate_Amplitude_Droop), System.Convert.ToUInt16(Compensate_IQ_Offset));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures which traces are displayed.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TRACE_STATE
        /// 
        /// Remote-control command(s):
        /// DISPlay:WINDow&lt;1|2&gt;:TRACe&lt;1..4&gt;:STATe ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Trace">
        /// Selects the trace type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_TRACE_CURR (1) - Current
        /// RSSPECAN_VAL_TETRA_TRACE_MIN  (2) - Minimum
        /// RSSPECAN_VAL_TETRA_TRACE_PEAK (3) - Peak
        /// RSSPECAN_VAL_TETRA_TRACE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_TRACE_CURR (1)
        /// 
        /// Note(s):
        /// 
        /// (1) Minimum: not available in ACP Trans and Spectrum FFT.
        /// 
        /// (2) Peak: not avalaible in Spectrum FFT.
        /// 
        /// (3) Average: not available in ACP Trans.
        /// </param>
        /// <param name="Trace_State">
        /// Controls the state of the trace.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRACE_STATE
        /// 
        /// Note(s):
        /// 
        /// (1) Availabe in measurements:
        /// 
        /// - PVT 
        /// - EVM vs Symbol 
        /// - EVM vs Carrier 
        /// - Magnitude Error vs Symbol 
        /// - Phase Error vs Symbol 
        /// - Spectrum FFT 
        /// - ACP Due to Modulation 
        /// - ACP Due to Transients
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTETRATraceSettings(int Trace, bool Trace_State)
        {
            int pInvokeResult = PInvoke.ConfigureTETRATraceSettings(this._handle, Trace, System.Convert.ToUInt16(Trace_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function enables the active marker on the specified trace.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_ENABLED
        /// RSSPECAN_ATTR_MARKER_TRACE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;[:STATe]
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:TRACe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker_Enabled">
        /// Specifies whether the specified marker is enabled. 
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_ENABLED
        /// </param>
        /// <param name="Trace">
        /// Assigns the market to the trace.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_TRACE_CURR (1) - Current
        /// RSSPECAN_VAL_TETRA_TRACE_MIN  (2) - Minimum
        /// RSSPECAN_VAL_TETRA_TRACE_PEAK (3) - Peak
        /// RSSPECAN_VAL_TETRA_TRACE_AVER (4) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_TRACE_CURR (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_TRACE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTETRAMarker(int Window, int Marker, bool Marker_Enabled, int Trace)
        {
            int pInvokeResult = PInvoke.ConfigureTETRAMarker(this._handle, Window, Marker, System.Convert.ToUInt16(Marker_Enabled), Trace);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the type of marker search and performs the search.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_MARKER_SEARCH_PEAK
        /// RSSPECAN_ATTR_MARKER_SEARCH_MIN
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:MAXimum[:PEAK]
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:MINimum[:PEAK]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 1..16 for R&amp;S FSV
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker_Search">
        /// Specifies the type of marker search type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MARKER_SEARCH_HIGHEST (1) - Peak Maximum
        /// RSSPECAN_VAL_MARKER_SEARCH_MINIMUM (2) - Peak Minimum
        /// 
        /// Default Value:  RSSPECAN_VAL_MARKER_SEARCH_HIGHEST
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_MARKER_SEARCH_PEAK
        /// RSSPECAN_ATTR_MARKER_SEARCH_MIN
        /// 
        /// Note(s):
        /// 
        /// (1) Peak Maximum - Sets marker search for the highest amplitude
        /// 
        /// (2) Peak Minimum - Sets marker search for the minimum amplitude.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int TETRAMarkerSearch(int Window, int Marker, int Marker_Search)
        {
            int pInvokeResult = PInvoke.TETRAMarkerSearch(this._handle, Window, Marker, Marker_Search);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function moves the active marker to the specified horizontal position.
        /// 
        /// Note(s):
        /// 
        /// (1) The marker must be ON (use function rsspecan_ConfigureTETRAMarker).
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_MOVE_MARKER
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:XFRequency
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:XSYMbol
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:XTIMe
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:XCARrier
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Measurement">
        /// This control selects the measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEAS_PVT  (0) - Power vs. Time
        /// RSSPECAN_VAL_MEAS_EVMS (1) - EVM vs. Symbol
        /// RSSPECAN_VAL_MEAS_EVMC (2) - EVM vs. Carrier
        /// RSSPECAN_VAL_MEAS_MERR (3) - Magniture Error
        /// RSSPECAN_VAL_MEAS_PERR (4) - Phase Error
        /// RSSPECAN_VAL_MEAS_SPEC (5) - Spectrum FFT
        /// RSSPECAN_VAL_MEAS_ACPM (6) - ACP Modulation
        /// RSSPECAN_VAL_MEAS_ACPT (7) - ACP Transients
        /// RSSPECAN_VAL_MEAS_CAPT (8) - Capture Memory
        /// 
        /// Default Value: RSSPECAN_VAL_MEAS_PVT  (0)
        /// </param>
        /// <param name="Marker_Position">
        /// Sets the horizontal position.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_MOVE_MARKER
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int TETRAMoveMarker(int Window, int Marker, int Measurement, double Marker_Position)
        {
            int pInvokeResult = PInvoke.TETRAMoveMarker(this._handle, Window, Marker, Measurement, Marker_Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the marker amplitude level of the selected marker.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_MARKER_AMPLITUDE
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:YRESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker">
        /// This control selects marker.
        /// 
        /// Valid Values:
        /// 1..4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Marker_Amplitude">
        /// The amplitude value of the selected marker.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_MARKER_AMPLITUDE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryTETRAMarkerAmplitude(int Window, int Marker, out double Marker_Amplitude)
        {
            int pInvokeResult = PInvoke.QueryTETRAMarkerAmplitude(this._handle, Window, Marker, out Marker_Amplitude);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the measurement result display between FULL SCREEN and SPLIT SCREEN.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_DISP_FORMAT
        /// 
        /// Remote-control command(s):
        /// DISPlay:FORMat SINGle | SPLit
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Format">
        /// Switches the measurement result display between FULL SCREEN and SPLIT SCREEN. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DISP_FORMAT_SINGLE (0) - Single
        /// RSSPECAN_VAL_DISP_FORMAT_SPLIT  (1) - Split
        /// 
        /// Default Value:
        /// RSSPECAN_VAL_DISP_FORMAT_SINGLE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_DISP_FORMAT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTETRADisplayFormat(int Format)
        {
            int pInvokeResult = PInvoke.ConfigureTETRADisplayFormat(this._handle, Format);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the active measurement window.
        /// In FULL SCREEN mode, the measurements are only performed in the active measurement window. Measurements are therefore initiated in the active window and result queries (marker, trace data and other results) answered also in the active window.
        /// Initiating measurements and querying results in the inactive window yields an error message (execution error).
        /// In split screen mode, the selection of the active window for result queries is irrelevant.
        /// 
        /// Note(s):
        /// 
        /// (1) In FULL SCREEN mode, settings can also be performed in the inactive measurement window. They become effective as soon as the corresponding window becomes active.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SELECT_WINDOW
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:SELect
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTETRADisplayActiveWindow(int Window)
        {
            int pInvokeResult = PInvoke.ConfigureTETRADisplayActiveWindow(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the limits.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_EVM_SUMMARY_LIMITS
        /// 
        /// Remote-control command(s):
        /// CALCulate:LIMit:BURSt:EVM:ALL[:AVERage]
        /// CALCulate:LIMit:BURSt:EVM:ALL:MAXimum
        /// CALCulate:LIMit:BURSt:EVM:DATA[:AVERage]
        /// CALCulate:LIMit:BURSt:EVM:DATA:MAXimum
        /// CALCulate:LIMit:BURSt:EVM:DHEA[:AVERage]
        /// CALCulate:LIMit:BURSt:EVM:DHEA:MAXimum
        /// CALCulate:LIMit:BURSt:EVM:DSPilot[:AVERage]
        /// CALCulate:LIMit:BURSt:EVM:DSPilot:MAXimum
        /// CALCulate:LIMit:BURSt:EVM:SPILot[:AVERage]
        /// CALCulate:LIMit:BURSt:EVM:SPILot:MAXimum
        /// CALCulate:LIMit:BURSt:IQOFfset[:AVERage]
        /// CALCulate:LIMit:BURSt:IQOFfset:MAXimum
        /// CALCulate:LIMit:BURSt:FERRor[:AVERage]
        /// CALCulate:LIMit:BURSt:FERRor:MAXimum
        /// CALCulate:LIMit:BURSt:PHASerror[:AVERage]
        /// CALCulate:LIMit:BURSt:PHASerror:MAXimum
        /// CALCulate:LIMit:BURSt:MAGError[:AVERage]
        /// CALCulate:LIMit:BURSt:MAGError:MAXimum
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Limit_Type">
        /// Selects the limit type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_LIM_EVM_ALL  (0) - Combined EVM
        /// RSSPECAN_VAL_TETRA_LIM_EVM_DATA (1) - Data EVM
        /// RSSPECAN_VAL_TETRA_LIM_EVM_DHEA (2) - Data + Header EVM
        /// RSSPECAN_VAL_TETRA_LIM_EVM_DSP  (3) - Data + Sync + Pilot EVM
        /// RSSPECAN_VAL_TETRA_LIM_EVM_SPIL (4) - Sync + Pilot EVM
        /// RSSPECAN_VAL_TETRA_LIM_IQOF     (5) - IQ Offset Limit
        /// RSSPECAN_VAL_TETRA_LIM_FERR     (6) - Center Frequency Error
        /// RSSPECAN_VAL_TETRA_LIM_PHAS     (7) - Phase Error
        /// RSSPECAN_VAL_TETRA_LIM_MAGN     (8) - Magnitude Error
        ///  
        /// Default Value: RSSPECAN_VAL_TETRA_LIM_EVM_ALL  (0)
        /// </param>
        /// <param name="Mode">
        /// Sets the limit measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_MEAS_MAX  (2) - Maximum
        /// RSSPECAN_VAL_TETRA_MEAS_AVER (3) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_MEAS_MAX (2)
        /// 
        /// 
        /// </param>
        /// <param name="Value">
        /// Sets the limit value.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value:
        /// 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_EVM_SUMMARY_LIMITS
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTETRAResultSummaryLimits(int Limit_Type, int Mode, double Value)
        {
            int pInvokeResult = PInvoke.ConfigureTETRAResultSummaryLimits(this._handle, Limit_Type, Mode, Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets all limits.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate:LIMit:BURSt:ALL &lt;numeric value&gt;, ...
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// This control sets the size of array in control Values.
        /// 
        /// Valid Values:
        /// &gt;0
        /// 
        /// Default Value: 18
        /// </param>
        /// <param name="Limit_Values">
        /// Sets the array of limit values.
        /// 
        /// Note(s):
        /// 
        /// (1) Order of values:
        /// &lt;average EVM All Result&gt;,
        /// &lt;max EVM All Limit&gt;,
        /// &lt;average EVM Data+Sync+Pilot Result&gt;,
        /// &lt;max EVM Data+Sync+Pilot Result&gt;,
        /// &lt;average EVM Data+Header Result&gt;,
        /// &lt;max EVM Data+Header Result&gt;,
        /// &lt;average EVM Data Result&gt;,
        /// &lt;max EVM Data Result&gt;,
        /// &lt;average EVM Sync+Pilot Result&gt;,
        /// &lt;max EVM Sync+Pilot Result&gt;,
        /// &lt;average IQ Offset Result&gt;,
        /// &lt;max IQ Offset Result&gt;,
        /// &lt;average Frequency Error Result&gt;,
        /// &lt;max Frequency Error Result&gt;,
        /// &lt;average Phase Error Result&gt;,
        /// &lt;max Phase Error Result&gt;
        /// &lt;average Magnitude Error Result&gt;,
        /// &lt;max Magnitude Error Result&gt;
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTETRAResultSummaryLimitsAll(int Array_Size, double[] Limit_Values)
        {
            int pInvokeResult = PInvoke.ConfigureTETRAResultSummaryLimitsAll(this._handle, Array_Size, Limit_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns all limits.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate:LIMit:BURSt:ALL?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// This control sets the size of array in control Limit Values.
        /// 
        /// Valid Values:
        /// &gt;0
        /// 
        /// Default Value: 18
        /// </param>
        /// <param name="Limit_Values">
        /// Returns the array of limit values.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should contain at least 18 elements.
        /// 
        /// (2) Order of values:
        /// &lt;average EVM All Result&gt;,
        /// &lt;max EVM All Limit&gt;,
        /// &lt;average EVM Data+Sync+Pilot Result&gt;,
        /// &lt;max EVM Data+Sync+Pilot Result&gt;,
        /// &lt;average EVM Data+Header Result&gt;,
        /// &lt;max EVM Data+Header Result&gt;,
        /// &lt;average EVM Data Result&gt;,
        /// &lt;max EVM Data Result&gt;,
        /// &lt;average EVM Sync+Pilot Result&gt;,
        /// &lt;max EVM Sync+Pilot Result&gt;,
        /// &lt;average IQ Offset Result&gt;,
        /// &lt;max IQ Offset Result&gt;,
        /// &lt;average Frequency Error Result&gt;,
        /// &lt;max Frequency Error Result&gt;,
        /// &lt;average Phase Error Result&gt;,
        /// &lt;max Phase Error Result&gt;
        /// &lt;average Magnitude Error Result&gt;,
        /// &lt;max Magnitude Error Result&gt;
        /// </param>
        /// <param name="Returned_Values">
        /// Returns the number of values in Limit Values array.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetTETRAResultSummaryLimitsAll(int Array_Size, double[] Limit_Values, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.GetTETRAResultSummaryLimitsAll(this._handle, Array_Size, Limit_Values, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the ACP Due to Modulation and ACP Due to Transients relative power limit for the specified channel and 
        /// bandwidth in dB.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_ACP_LIMITS
        /// 
        /// Remote-control command(s):
        /// CALCulate:LIMit:BURSt:ACPMod:CHAN&lt;0..6&gt;:RELPower:BANDwidth&lt;0..3&gt;
        /// CALCulate:LIMit:BURSt:ACPTrans:CHAN&lt;0..6&gt;:RELPower:BANDwidth&lt;0..3&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="ACP">
        /// Selects ACP measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_ACP_MODULATION (0) - ACP Modulation
        /// RSSPECAN_VAL_TETRA_ACP_TRANSIENT  (1) - ACP Transient
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_ACP_MODULATION (0)
        /// </param>
        /// <param name="Channel_Number">
        /// This control selects the ACP measurements channels.
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_NEG_3 (0) - -3
        /// RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_NEG_2 (1) - -2
        /// RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_NEG_1 (2) - -1
        /// RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_POS_1 (4) - 1
        /// RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_POS_2 (5) - 2
        /// RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_POS_3 (6) - 3
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_NEG_1 (2)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_ACP_LIMITS
        /// 
        /// Note(s):
        /// 
        /// (1) ACP due to Modulation Relative Channel Number:
        /// 
        /// 0 = -3
        /// 1 = -2
        /// 2 = -1
        /// 4 = +1
        /// 5 = +2
        /// 6 = +3
        /// 
        /// (1) ACP due to Transient Relative Channel Number:
        /// 
        /// 0 = Reserved - not available
        /// 1 = Reserved - not available
        /// 2 = -1 (Lower)
        /// 4 = 1 (Upper)
        /// 5 = Reserved - not available
        /// 6 = Reserved - not available
        /// </param>
        /// <param name="Channel_Bandwidth">
        /// Selets the channel bandwidth.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_CHANN_BAND_25  (0) - 25 kHz
        /// RSSPECAN_VAL_TETRA_CHANN_BAND_50  (1) - 50 kHz
        /// RSSPECAN_VAL_TETRA_CHANN_BAND_100 (2) - 100 kHz
        /// RSSPECAN_VAL_TETRA_CHANN_BAND_150 (3) - 150 kHz
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_CHANN_BAND_25 (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_ACP_LIMITS
        /// 
        /// Note(s):
        /// 
        /// (1)25 kHz channel bandwidth uses 8 carriers.
        /// 
        /// (2) 50 kHz channel bandwidth uses 16 carriers.
        /// 
        /// (3) 100 kHz channel bandwidth uses 32 carriers.
        /// 
        /// (4) 150 kHz channel bandwidth uses 48 carriers.
        /// </param>
        /// <param name="Limit_Value">
        /// Sets the limit value.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value:
        /// 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_ACP_LIMITS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTETRAACPLimits(int ACP, int Channel_Number, int Channel_Bandwidth, double Limit_Value)
        {
            int pInvokeResult = PInvoke.ConfigureTETRAACPLimits(this._handle, ACP, Channel_Number, Channel_Bandwidth, Limit_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the TETRA measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_MEASUREMENT_TYPE
        /// 
        /// 
        /// Remote-control command(s):
        /// CONFigure:BURSt:PVT[:IMMediate]
        /// CONFigure:BURSt:EVM:ESYMbol[:IMMediate]
        /// CONFigure:BURSt:EVM:ECARrier[:IMMediate]
        /// CONFigure:BURSt:MAGNitude[:IMMediate]
        /// CONFigure:BURSt:PHASe[:IMMediate]
        /// CONFigure:BURSt:SPECtrum:FFT[:IMMediate]
        /// CONFigure:BURSt:SPECtrum:ACPR:MODulation[:IMMediate]
        /// CONFigure:BURSt:SPECtrum:ACPR:TUPPer[:IMMediate]
        /// CONFigure:BURSt:SPECtrum:ACPR:TLOWer[:IMMediate]
        /// CONFigure:BURSt:CONStell:CSYMbol[:IMMediate]
        /// CONFigure:BURSt:CONStell:CCARrier[:IMMediate]
        /// CONFigure:BURSt:STATistics:BSTReam[:IMMediate]
        /// CONFigure:BURSt:SUMTable[:IMMediate]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Measurement">
        /// This control selects the TETRA measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_PVT      (0) - Power vs. Time
        /// RSSPECAN_VAL_TETRA_EVMS     (1) - EVM vs. Symbol
        /// RSSPECAN_VAL_TETRA_EVMC     (2) - EVM vs. Carrier
        /// RSSPECAN_VAL_TETRA_MAGN     (3) - Magnitude Error
        /// RSSPECAN_VAL_TETRA_PHAS     (4) - Phase Error
        /// RSSPECAN_VAL_TETRA_FFT      (5) - Spectrum FFT
        /// RSSPECAN_VAL_TETRA_ACPM     (6) - ACP due to Modulation
        /// RSSPECAN_VAL_TETRA_ACPT_UPP (7) - ACP due to Transients - Upper Channel
        /// RSSPECAN_VAL_TETRA_ACPT_LOW (8) - ACP due to Transients - Lower Channel
        /// RSSPECAN_VAL_TETRA_CSYM     (9) - Constellation vs. Symbol
        /// RSSPECAN_VAL_TETRA_CCAR     (10) - Constellation vs. Carrier
        /// RSSPECAN_VAL_TETRA_BSTR     (11) - Statistics Bit Stream
        /// RSSPECAN_VAL_TETRA_SUMM     (12) - Result Summary 
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_PVT  (0)
        /// 
        /// Attribute:
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTETRAMeasurementType(int Measurement)
        {
            int pInvokeResult = PInvoke.ConfigureTETRAMeasurementType(this._handle, Measurement);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the Y-axis for the specified trace display and measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_Y_AXIS_TRACE
        /// RSSPECAN_ATTR_TETRA_Y_AXIS_TRACE_UNIT
        /// RSSPECAN_ATTR_TETRA_Y_AXIS_TRACE_AUTO
        /// 
        /// Remote-control command(s):
        /// DISPlay:WINDow&lt;1|2&gt;:TRACe1:Y[:SCALe]:ECARrier:AUTO
        /// DISPlay:WINDow&lt;1|2&gt;:TRACe1:Y[:SCALe]:ESYMbol:AUTO
        /// DISPlay:WINDow&lt;1|2&gt;:TRACe1:Y[:SCALe]:MAGNitude:AUTO
        /// DISPlay:WINDow&lt;1|2&gt;:TRACe1:Y[:SCALe]:PHASE:AUTO
        /// DISPlay:WINDow&lt;1|2&gt;:TRACe1:Y[:SCALe]:ECARrier:UNIT
        /// DISPlay:WINDow&lt;1|2&gt;:TRACe1:Y[:SCALe]:ESYMbol:UNIT
        /// DISPlay:WINDow&lt;1|2&gt;:TRACe1:Y[:SCALe]:ECARrier:CENTer
        /// DISPlay:WINDow&lt;1|2&gt;:TRACe1:Y[:SCALe]:ESYMbol:CENTer
        /// DISPlay:WINDow&lt;1|2&gt;:TRACe1:Y[:SCALe]:MAGNitude:CENTer
        /// DISPlay:WINDow&lt;1|2&gt;:TRACe1:Y[:SCALe]:PHASE:CENTer
        /// DISPlay:WINDow&lt;1|2&gt;:TRACe1:Y[:SCALe]:ECARrier:SPAN
        /// DISPlay:WINDow&lt;1|2&gt;:TRACe1:Y[:SCALe]:ESYMbol:SPAN
        /// DISPlay:WINDow&lt;1|2&gt;:TRACe1:Y[:SCALe]:MAGNitude:SPAN
        /// DISPlay:WINDow&lt;1|2&gt;:TRACe1:Y[:SCALe]:PHASE:SPAN
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement">
        /// Selects measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_EVMS (1) - EVM vs. Symbol
        /// RSSPECAN_VAL_TETRA_EVMC (2) - EVM vs. Carrier
        /// RSSPECAN_VAL_TETRA_MAGN (3) - Magnitude Error
        /// RSSPECAN_VAL_TETRA_PHAS (4) - Phase Error
        /// 
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_EVMS (1)
        /// </param>
        /// <param name="Auto_Scaling">
        /// This control toggles the automatic scaling of the Y-axis for the specified measurement.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_Y_AXIS_TRACE_AUTO
        /// </param>
        /// <param name="Unit">
        /// Specifies the units for Y-axis for the specified measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DB  (8)  - dB
        /// RSSPECAN_VAL_UNIT_PCT (13) - %
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_DB (8)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_Y_AXIS_TRACE_UNIT
        /// 
        /// Note(s):
        /// 
        /// (1) This control is only available when Measurement control is set to EVM vs. Symbol or EVM vs. Carrier.
        /// </param>
        /// <param name="Y_Center">
        /// Sets the center value of the Y-axis for the specified measurement.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_Y_AXIS_TRACE
        /// 
        /// Note(s):
        /// 1) This control is available only if Auto Scalling is Off.
        /// </param>
        /// <param name="Y_Span">
        /// Sets the span of the Y-axis for the specified measurement
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 0.0
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_Y_AXIS_TRACE
        /// 
        /// Note(s):
        /// 1) This control is available only if Auto Scalling is Off.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTETRAYAxis(int Measurement, bool Auto_Scaling, int Unit, double Y_Center, double Y_Span)
        {
            int pInvokeResult = PInvoke.ConfigureTETRAYAxis(this._handle, Measurement, System.Convert.ToUInt16(Auto_Scaling), Unit, Y_Center, Y_Span);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function determines whether the trigger system is continuously initiated (continuous) or performs single measurements (single)
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SWEEP_MODE_CONTINUOUS
        /// 
        /// Remote-control command(s):
        /// INITiate&lt;1|2&gt;:CONTinuous OFF | ON
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Continuous">
        /// This control toggles the sweep.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SWEEP_MODE_CONTINUOUS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTETRAContinuousMeasurement(bool Continuous)
        {
            int pInvokeResult = PInvoke.ConfigureTETRAContinuousMeasurement(this._handle, System.Convert.ToUInt16(Continuous));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the constellation settings.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only if Constellation vs. Symbol or Constellation vs. Carrier measurement is selected.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_CONSTELLATION_SYMBOL_SETTINGS
        /// RSSPECAN_ATTR_TETRA_CONSTELLATION_CARRIER_NUMBER
        /// 
        /// Remote-control command(s):
        /// CONFigure:BURSt:CONStell:SYMbol:SELect  ALL | DATA | HDR | SYNC | PILOT
        /// CONFigure:BURSt:CONStell:CSYMbol:CARRier:SELect
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Symbols_to_Plot_Selection">
        /// Selects the symbol to be displayed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_SYMBOLS_ALL  (0) - All
        /// RSSPECAN_VAL_TETRA_SYMBOLS_DATA (1) - Data
        /// RSSPECAN_VAL_TETRA_SYMBOLS_HEAD (2) - Header
        /// RSSPECAN_VAL_TETRA_SYMBOLS_SYNC (3) - Sync
        /// RSSPECAN_VAL_TETRA_SYMBOLS_PIL  (4) - Pilot
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_SYMBOLS_ALL (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_CONSTELLATION_SYMBOL_SETTINGS
        /// </param>
        /// <param name="Carrier_Selection">
        /// Selects whether all carriers or only single carrier is display.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_CARRIER_ALL (0) - All
        /// RSSPECAN_VAL_CARRIER_NUM (1) - Carrier Number
        /// 
        /// Default Value: RSSPECAN_VAL_SELECTION_NUM (1)
        /// 
        /// 
        /// Note(s):
        /// 
        /// (1) When selecting a single carrier, set this parameter to Carrier Number and then use parameter Carrier Number to set a desired value.
        /// </param>
        /// <param name="Carrier_Number">
        /// Defines the carrier number.
        /// 
        /// Valid Values:
        /// -24 to -1, 1 to 24 (see notes below)
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_CONSTELLATION_CARRIER_NUMBER
        /// 
        /// Note(s):
        /// 
        /// (1) This parameter is valid only if Carrier Number is selected in Carrier Selection control.
        /// 
        /// (2) The valid range depends on the selected bandwidth of slots to analyze:
        /// 
        /// For 25 KHz (8 sub carriers) valid parameters are -4 to -1, +1 to +4.
        /// 
        /// For 50 kHz (16 sub carriers) valid parameters are -8 to -1, +1 to +8.
        /// 
        /// For 100 KHz (32 sub carriers) valid parameters are -16 to -1, +1 to +16.
        /// 
        /// For 150 kHz (48 sub carriers) valid parameters are -24 to -1, +1 to +24.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTETRAConstellationSettings(int Symbols_to_Plot_Selection, int Carrier_Selection, int Carrier_Number)
        {
            int pInvokeResult = PInvoke.ConfigureTETRAConstellationSettings(this._handle, Symbols_to_Plot_Selection, Carrier_Selection, Carrier_Number);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Specifies the units for EVM results.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_EVM_RESULT_UNITS
        /// 
        /// Remote-control command(s):
        /// UNIT:EVM
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="EVM_Units">
        /// Specifies the units for EVM results.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DB  (8)  - dB
        /// RSSPECAN_VAL_UNIT_PCT (13) - %
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_DB (8)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_EVM_RESULT_UNITS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTETRAEVMResultUnits(int EVM_Units)
        {
            int pInvokeResult = PInvoke.ConfigureTETRAEVMResultUnits(this._handle, EVM_Units);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Specifies the units for Gain Imbalance Summary Table results.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_GAIN_IMBALANCE_RESULT_UNITS
        /// 
        /// Remote-control command(s):
        /// UNIT:GIMBalance
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Gain_Imbalance_Units">
        /// Specifies the units for Gain Imbalance Summary Table results.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_UNIT_DB  (8)  - dB
        /// RSSPECAN_VAL_UNIT_PCT (13) - %
        /// 
        /// Default Value: RSSPECAN_VAL_UNIT_DB (8)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_GAIN_IMBALANCE_RESULT_UNITS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConfigureTETRAGainImbalanceResultUnits(int Gain_Imbalance_Units)
        {
            int pInvokeResult = PInvoke.ConfigureTETRAGainImbalanceResultUnits(this._handle, Gain_Imbalance_Units);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function causes the measurement results to be recalculated using the current settings and acquisition data.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_BURST_RECALC
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:BURSt[:IMMediate]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int TETRABurstRecalc()
        {
            int pInvokeResult = PInvoke.TETRABurstRecalc(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns all the measured data that relates to the currently selected measurement type. The returned data is particular to the currently selected measurement type and is specified in the Trace Data parameter description.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;[:DATA]? TRACE1 | TRACE2 | TRACE3 | TRACE4
        /// FORMat[:DATA] REAL,32
        /// *CLS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// This parameter indicates which trace will be read out.
        /// 
        /// Valid Values:
        /// 1 to 4
        /// 
        /// Default Value: 1
        /// 
        /// Note(s):
        /// 
        /// (1) Measurements with 4 Traces:
        /// TRACE1 = Current
        /// TRACE2 = Minimum
        /// TRACE3 = Peak
        /// TRACE4 = Average
        /// 
        /// (2) Capture Memory: only Trace 1 available.
        /// 
        /// (3) ACP Mod: only Trace 1 available.
        /// 
        /// (4) ACP Trans: only Trace 1 and Trace 3 available.
        /// 
        /// (5) Spectrum FFT: only Trace 1 and Trace 4 available.
        /// </param>
        /// <param name="Array_Size">
        /// Number of array points requested.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 642
        /// </param>
        /// <param name="Trace_Data">
        /// Returns an array of trace data. Number of the measurement values for operating mode TETRA:
        /// 
        /// - ACP due to Modulation: 642
        /// 
        /// - ACP due to Transients - Lower: 642
        /// 
        /// - ACP due to Transients - Upper: 642
        /// 
        /// - Capture Memory: 4000
        /// 
        /// - Constellation vs. Carrier: Currently not available yet.
        /// 
        /// - Constellation vs. Symbol: Currently not available yet.
        /// 
        /// - EVM vs. Carrier: Channel Bandwidth
        ///  8 - 25 kHz (8 carriers)
        /// 16 - 50 kHz (16 carriers)
        /// 32 - 100 kHz (32 carriers)
        /// 48 - 150 kHz (48 carriers)
        /// 
        /// - EVM vs. Symbol, Magnitude Error vs. Symbol, Phase Error vs. Symbol:
        /// Slot Type
        /// 34 - Normal Downlink Burst
        /// 31 - Normal Uplink Burst
        /// 14 - Control Uplink Burst
        /// 14 - Random Access Burst
        /// 
        /// - Power vs. Time: 642
        /// 
        /// - Spectrum FFT: 642
        /// </param>
        /// <param name="No_of_Values">
        /// Returns the number of trace data values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadTETRATraceData(int Window, int Trace, int Array_Size, double[] Trace_Data, out int No_of_Values)
        {
            int pInvokeResult = PInvoke.ReadTETRATraceData(this._handle, Window, Trace, Array_Size, Trace_Data, out No_of_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns all the measured data that relates to the currently selected measurement type. The returned data is particular to the currently selected measurement type and is specified in the Trace Data parameter description.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;[:DATA]? TRACE1 | TRACE2 | TRACE3 | TRACE4
        /// FORMat[:DATA] ASCII
        /// *CLS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// This parameter indicates which trace will be read out.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// 
        /// Note(s):
        /// 
        /// (1) 
        /// TRACE1 = Header Symbols
        /// TRACE2 = Payload Symbols
        /// </param>
        /// <param name="Array_Size">
        /// Number of array points requested.
        /// 
        /// Valid Values:
        /// no range checking
        /// 
        /// Default Value: 1384
        /// </param>
        /// <param name="Trace_Data">
        /// Returns an array of the Header symbols or payload data symbols. Number of the measurement values for operating mode TETRA:
        /// 
        /// Number of values depends on channel type and slot type (set with function rsspecan_ConfigureTETRASignalCharacteristics).
        /// 
        /// Random Access
        /// - 25 KHz: Payload: 84 values, Header: 0 values
        /// - 50 KHz: N.A.
        /// - 100 KHz: N.A.
        /// - 150 KHz: N.A.
        /// 
        /// Control uplink
        /// - 25 KHz: Payload: 76 values, Header: 8 values
        /// - 50 KHz: Payload: 160 values, Header: 8 values
        /// - 100 KHz: Payload: 328 values, Header: 8 values
        /// - 150 KHz: Payload: 496 values, Header: 8 values
        /// 
        /// Normal uplink
        /// - 25 KHz: Payload: 200 values, Header: 8 values
        /// - 50 KHz: Payload: 408 values, Header: 8 values
        /// - 100 KHz: Payload: 824 values, Header: 8 values
        /// - 150 KHz: Payload: 1240 values, Header: 8 values
        /// 
        /// Normal downlink
        /// - 25 KHz: Payload: 204 values, Header: 32 values
        /// - 50 KHz: Payload: 440 values, Header: 32 values
        /// - 100 KHz: Payload: 91 values, Header: 32 values
        /// - 150 KHz: Payload: 1384 values, Header: 32 values
        /// </param>
        /// <param name="No_of_Values">
        /// Returns the number of trace data values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadTETRABitstream(int Window, int Trace, int Array_Size, double[] Trace_Data, out int No_of_Values)
        {
            int pInvokeResult = PInvoke.ReadTETRABitstream(this._handle, Window, Trace, Array_Size, Trace_Data, out No_of_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the total number of burst found and analyzed.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_NUMBER_OF_BURST
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:COUNt?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Number_Of_Burst">
        /// Returns the total number of burst found and analyzed.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchTETRANumberOfBursts(out int Number_Of_Burst)
        {
            int pInvokeResult = PInvoke.FetchTETRANumberOfBursts(this._handle, out Number_Of_Burst);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the measured current (i.e. of last found slot), average, minimum or maximum result values.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_SUMMARY_TABLE_MEASUREMENT_RESULT
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:SUMTable:EVM:ALL[:CURRent]?
        /// FETCh:BURSt:SUMTable:EVM:ALL:AVERage?
        /// FETCh:BURSt:SUMTable:EVM:ALL:MAXimum?
        /// FETCh:BURSt:SUMTable:EVM:ALL:MINimum?
        /// FETCh:BURSt:SUMTable:EVM:DATA[:CURRent]?
        /// FETCh:BURSt:SUMTable:EVM:DATA:AVERage?
        /// FETCh:BURSt:SUMTable:EVM:DATA:MAXimum?
        /// FETCh:BURSt:SUMTable:EVM:DATA:MINimum?
        /// FETCh:BURSt:SUMTable:EVM:DHEAder[:CURRent]?
        /// FETCh:BURSt:SUMTable:EVM:DHEAder:AVERage?
        /// FETCh:BURSt:SUMTable:EVM:DHEAder:MAXimum?
        /// FETCh:BURSt:SUMTable:EVM:DHEAder:MINimum?
        /// FETCh:BURSt:SUMTable:EVM:DSPilot[:CURRent]?
        /// FETCh:BURSt:SUMTable:EVM:DSPilot:AVERage?
        /// FETCh:BURSt:SUMTable:EVM:DSPilot:MAXimum?
        /// FETCh:BURSt:SUMTable:EVM:DSPilot:MINimum?
        /// FETCh:BURSt:SUMTable:EVM:SPILot[:CURRent]?
        /// FETCh:BURSt:SUMTable:EVM:SPILot:AVERage?
        /// FETCh:BURSt:SUMTable:EVM:SPILot:MAXimum?
        /// FETCh:BURSt:SUMTable:EVM:SPILot:MINimum?
        /// FETCh:BURSt:SUMTable:IQOFfset[:CURRent]?
        /// FETCh:BURSt:SUMTable:IQOFfset:AVERage?
        /// FETCh:BURSt:SUMTable:IQOFfset:MAXimum?
        /// FETCh:BURSt:SUMTable:IQOFfset:MINimum?
        /// FETCh:BURSt:SUMTable:GIMBalance[:CURRent]?
        /// FETCh:BURSt:SUMTable:GIMBalance:AVERage?
        /// FETCh:BURSt:SUMTable:GIMBalance:MAXimum?
        /// FETCh:BURSt:SUMTable:GIMBalance:MINimum?
        /// FETCh:BURSt:SUMTable:QUADerror[:CURRent]?
        /// FETCh:BURSt:SUMTable:QUADerror:AVERage?
        /// FETCh:BURSt:SUMTable:QUADerror:MAXimum?
        /// FETCh:BURSt:SUMTable:QUADerror:MINimum?
        /// FETCh:BURSt:SUMTable:AMPDroop[:CURRent]?
        /// FETCh:BURSt:SUMTable:AMPDroop:AVERage?
        /// FETCh:BURSt:SUMTable:AMPDroop:MAXimum?
        /// FETCh:BURSt:SUMTable:AMPDroop:MINimum?
        /// FETCh:BURSt:SUMTable:FERRor[:CURRent]?
        /// FETCh:BURSt:SUMTable:FERRor:AVERage?
        /// FETCh:BURSt:SUMTable:FERRor:MAXimum?
        /// FETCh:BURSt:SUMTable:FERRor:MINimum?
        /// FETCh:BURSt:SUMTable:PHASeerror[:CURRent]?
        /// FETCh:BURSt:SUMTable:PHASeerror:AVERage?
        /// FETCh:BURSt:SUMTable:PHASeerror:MAXimum?
        /// FETCh:BURSt:SUMTable:PHASeerror:MINimum?
        /// FETCh:BURSt:SUMTable:MAGerror[:CURRent]?
        /// FETCh:BURSt:SUMTable:MAGerror:AVERage?
        /// FETCh:BURSt:SUMTable:MAGerror:MAXimum?
        /// FETCh:BURSt:SUMTable:MAGerror:MINimum?
        /// FETCh:BURSt:SUMTable:BPWR[:CURRent]?
        /// FETCh:BURSt:SUMTable:BPWR:AVERage?
        /// FETCh:BURSt:SUMTable:BPWR:MAXimum?
        /// FETCh:BURSt:SUMTable:BPWR:MINimum?
        /// FETCh:BURSt:SUMTable:CRESt[:CURRent]?
        /// FETCh:BURSt:SUMTable:CRESt:AVERage?
        /// FETCh:BURSt:SUMTable:CRESt:MAXimum?
        /// FETCh:BURSt:SUMTable:CRESt:MINimum?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement">
        /// This control selects the measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_MEAS_EVM_ALL  (0) - EVM All Symbols
        /// RSSPECAN_VAL_TETRA_MEAS_EVM_DSP  (1) - EVM Data + Sync + Pilot
        /// RSSPECAN_VAL_TETRA_MEAS_EVM_DHEA (2) - EVM Data + Header
        /// RSSPECAN_VAL_TETRA_MEAS_EVM_DATA (3) - EVM Data
        /// RSSPECAN_VAL_TETRA_MEAS_EVM_SPIL (4) - EVM Sync + Pilot
        /// RSSPECAN_VAL_TETRA_MEAS_EVM_IQOF (5) - IQ Offset
        /// RSSPECAN_VAL_TETRA_MEAS_EVM_GIMB (6) - Gain Imbalance
        /// RSSPECAN_VAL_TETRA_MEAS_EVM_QUAD (7) - Quadrature Error
        /// RSSPECAN_VAL_TETRA_MEAS_EVM_AMPD (8) - Amplitude Droop
        /// RSSPECAN_VAL_TETRA_MEAS_EVM_FERR (9) - Center Frequency Error
        /// RSSPECAN_VAL_TETRA_MEAS_EVM_PHAS (10) - Phase Error
        /// RSSPECAN_VAL_TETRA_MEAS_EVM_MAG  (11) - Magnitude Error
        /// RSSPECAN_VAL_TETRA_MEAS_EVM_BPWR (12) - Burst Power
        /// RSSPECAN_VAL_TETRA_MEAS_EVM_CRES (13) - Crest Factor
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_MEAS_EVM_ALL (0)
        /// </param>
        /// <param name="Measurement_Type">
        /// Selects measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_MEAS_MIN  (1) - Minimum
        /// RSSPECAN_VAL_TETRA_MEAS_MAX  (2) - Maximum
        /// RSSPECAN_VAL_TETRA_MEAS_AVER (3) - Average
        /// RSSPECAN_VAL_TETRA_MEAS_CURR (4) - Current
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_MEAS_MIN (1)
        /// </param>
        /// <param name="Result">
        /// Returns selected result.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchTETRAMeasuremenResults(int Measurement, int Measurement_Type, out double Result)
        {
            int pInvokeResult = PInvoke.FetchTETRAMeasuremenResults(this._handle, Measurement, Measurement_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the results from the R&amp;S FS-K110 Summary Table measurement. 
        /// 
        /// Note(s):
        /// 
        /// (1) The units for the EVM results have to be specified before with the rsspecan_ConfigureTETRAEVMResultUnits function. The units 
        /// for the gain imbalance results have to be specified before with the rsspecan_ConfigureTETRAGainImbalanceResultUnits function.
        /// 
        /// (2) This function does not return all results that the rsspecan_FetchTETRAMeasuremenResults function can return. Especially the "current" results (i.e. results of the 
        /// last found slot) and the subcarrier results are excluded.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:SUMTable:ALL?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// This control sets the size of array in control Result.
        /// 
        /// Valid Values:
        /// &gt;0
        /// 
        /// Default Value: 42
        /// </param>
        /// <param name="Result">
        /// Returns selected result.
        /// 
        /// Note(s): 
        /// 
        /// (1) The array should contain at least 42 elements.
        /// 
        /// (2) "All symbols" includes: data, header, sync, and pilot symbols.
        /// 
        /// (3) Order of values:
        /// 
        /// &lt;min EVM all symbols&gt;, &lt;average EVM all symbols&gt;, &lt;max EVM all symbols&gt;, &lt;min EVM data+sync+pilot&gt;, &lt;average EVM data+sync+pilot&gt;, &lt;max EVM data+sync+pilot&gt;, &lt;min EVM data+header&gt;, &lt;average EVM data+header&gt;, &lt;max EVM data+header&gt;, &lt;min EVM data&gt;, &lt;average EVM data&gt;, &lt;max EVM data&gt;, &lt;min EVM sync+pilot&gt;, &lt;average EVM sync+pilot&gt;, &lt;max EVM sync+pilot&gt;, &lt;min IQ offset&gt;, &lt;average IQ offset&gt;, &lt;maximum IQ offset&gt;, &lt;min gain imbalance&gt;, &lt;average gain imbalance&gt;, &lt;max gain imbalance&gt;, &lt;min quadrature error&gt;, &lt;average quadrature error&gt;, &lt;max quadrature error&gt;, &lt;min amplitude droop&gt;, &lt;average amplitude droop&gt;, &lt;max amplitude droop&gt;, &lt;min frequency error&gt;, &lt;average frequency error&gt;, &lt;max frequency error&gt;, &lt;min phase error&gt;, &lt;average phase error&gt;, &lt;max phase error&gt;, &lt;min magnitude error&gt;, &lt;average magnitude error&gt;, &lt;max magnitude error&gt;, &lt;min burst power&gt;, &lt;average burst power&gt;, &lt;max burst power&gt;, &lt;min crest factor&gt;, &lt;average crest factor&gt;, &lt;max crest factor&gt;
        /// </param>
        /// <param name="Returned_Values">
        /// Returns the number of values in Result array.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchTETRAMeasurementResultsAll(int Array_Size, double[] Result, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.FetchTETRAMeasurementResultsAll(this._handle, Array_Size, Result, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the measured current, average, minimum or maximum of the Mean power in each sub-carrier in dBm.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:SUMTable:SCMPower[:CURRent?]
        /// FETCh:BURSt:SUMTable:SCMPower:AVERage?
        /// FETCh:BURSt:SUMTable:SCMPower:MAXimum?
        /// FETCh:BURSt:SUMTable:SCMPower:MINimum?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement_Type">
        /// Selects measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_MEAS_MIN  (1) - Minimum
        /// RSSPECAN_VAL_TETRA_MEAS_MAX  (2) - Maximum
        /// RSSPECAN_VAL_TETRA_MEAS_AVER (3) - Average
        /// RSSPECAN_VAL_TETRA_MEAS_CURR (4) - Current
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_MEAS_MIN (1)
        /// </param>
        /// <param name="Array_Size">
        /// This control sets the size of array in control Result.
        /// 
        /// Valid Values:
        /// &gt;0
        /// 
        /// Default Value: 48
        /// </param>
        /// <param name="Result">
        /// Returns selected result.
        /// 
        /// Note(s):
        /// 
        /// (1) The amount of values returned depends on the selected Channel Bandwidth (rsspecan_ConfigureTETRASignalCharacteristics) - 8, 16, 32, 48.
        /// </param>
        /// <param name="Returned_Values">
        /// Returns the number of values in Result array.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchTETRASubcarrierMeanPower(int Measurement_Type, int Array_Size, double[] Result, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.FetchTETRASubcarrierMeanPower(this._handle, Measurement_Type, Array_Size, Result, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the measured current, average, minimum or maximum of the Reference power in each sub-carrier in dBm.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:SUMTable:SCRPower[:CURRent?]
        /// FETCh:BURSt:SUMTable:SCRPower:AVERage?
        /// FETCh:BURSt:SUMTable:SCRPower:MAXimum?
        /// FETCh:BURSt:SUMTable:SCRPower:MINimum
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Measurement_Type">
        /// Selects measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_MEAS_MIN  (1) - Minimum
        /// RSSPECAN_VAL_TETRA_MEAS_MAX  (2) - Maximum
        /// RSSPECAN_VAL_TETRA_MEAS_AVER (3) - Average
        /// RSSPECAN_VAL_TETRA_MEAS_CURR (4) - Current
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_MEAS_MIN (1)
        /// </param>
        /// <param name="Array_Size">
        /// This control sets the size of array in control Result.
        /// 
        /// Valid Values:
        /// &gt;0
        /// 
        /// Default Value: 48
        /// </param>
        /// <param name="Result">
        /// Returns selected result.
        /// 
        /// Note(s):
        /// 
        /// (1) The amount of values returned depends on the selected Channel Bandwidth (rsspecan_ConfigureTETRASignalCharacteristics) - 8, 16, 32, 48.
        /// </param>
        /// <param name="Returned_Values">
        /// Returns the number of values in Result array.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchTETRASubcarrierReferencePower(int Measurement_Type, int Array_Size, double[] Result, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.FetchTETRASubcarrierReferencePower(this._handle, Measurement_Type, Array_Size, Result, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries value for the Power versus Time measurements from the result summary list.
        /// 
        /// Note(s):
        /// 
        /// (1) The Power versus Time measurement must be selected to obtain results.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_PVT_MEASUREMENT
        /// RSSPECAN_ATTR_TETRA_PVT_TRIGGER_TO_SYNC_TIME
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:PVTTable:REFPower[:CURRent]?
        /// FETCh:BURSt:PVTTable:REFPower:AVERage?
        /// FETCh:BURSt:PVTTable:REFPower:MINimum?
        /// FETCh:BURSt:PVTTable:REFPower:MAXimum?
        /// FETCh:BURSt:PVTTable:SLOTpower[:CURRent]?
        /// FETCh:BURSt:PVTTable:SLOTpower:AVERage?
        /// FETCh:BURSt:PVTTable:SLOTpower:MINimum?
        /// FETCh:BURSt:PVTTable:SLOTpower:MAXimum?
        /// FETCh:BURSt:PVTTable:BEFBpower[:CURRent]?
        /// FETCh:BURSt:PVTTable:BEFBpower:AVERage?
        /// FETCh:BURSt:PVTTable:BEFBpower:MINimum?
        /// FETCh:BURSt:PVTTable:BEFBpower:MAXimum?
        /// FETCh:BURSt:PVTTable:RELBeforepower[:CURRent]?
        /// FETCh:BURSt:PVTTable:RELBeforepower:AVERage? 
        /// FETCh:BURSt:PVTTable:RELBeforepower:MINimum? 
        /// FETCh:BURSt:PVTTable:RELBeforepower:MAXimum?
        /// FETCh:BURSt:PVTTable:AFTBpower[:CURRent]?
        /// FETCh:BURSt:PVTTable:AFTBpower:AVERage?
        /// FETCh:BURSt:PVTTable:AFTBpower:MINimum?
        /// FETCh:BURSt:PVTTable:AFTBpower:MAXimum?
        /// FETCh:BURSt:PVTTable:RELafterpower[:CURRent]?
        /// FETCh:BURSt:PVTTable:RELafterpower:AVERage?
        /// FETCh:BURSt:PVTTable:RELafterpower:MINimum?
        /// FETCh:BURSt:PVTTable:RELafterpower:MAXimum?
        /// FETCh:BURSt:PVTTable:TRIGsync?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="PVT">
        /// Selects Power versus Time measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_PVT_REF         (0) - Reference Power 
        /// RSSPECAN_VAL_TETRA_PVT_SLOT        (1) - Slot Power
        /// RSSPECAN_VAL_TETRA_PVT_BEFB        (2) - Power Before Burst
        /// RSSPECAN_VAL_TETRA_PVT_REL_BEFB    (3) - Relative Power Before Burst
        /// RSSPECAN_VAL_TETRA_PVT_AFTB        (4) - Power After Burst
        /// RSSPECAN_VAL_TETRA_PVT_REL_AFTB    (5) - Relative Power After Burst
        /// RSSPECAN_VAL_TETRA_PVT_TRG_TO_SYNC (6) - Trigger To Sync Time
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_PVT_REF (0)
        /// 
        /// Note(s):
        /// 
        /// (1) For the Power versus Time measurements, this function queries "Reference Power" value from the result summary list.
        /// 
        /// (2) For the Power versus Time measurements, this function queries "Slot Power" value from the result summary list.
        /// 
        /// (3) For the Power versus Time measurements, this function queries "Before Burst Power" value from the result summary list.
        /// 
        /// (4) For the Power versus Time measurements, this function queries "Relative Before Burst Power" value from the result summary list.
        /// 
        /// (5) For the Power versus Time measurements, this function queries "After Burst Power" value from the result summary list.
        /// 
        /// (6) For the Power versus Time measurements, this function queries "Relative After Burst Power" value from the result summary list.
        /// 
        /// (7) Trigger To Sync Time: returns the measured trigger to sync time in seconds from the signal that triggered capturing of IQ data to the first symbol instant of the first found slot in the current capture buffer. This result is only useful if an external trigger is used.
        /// </param>
        /// <param name="Measurement_Type">
        /// Selects measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_MEAS_MIN  (1) - Minimum
        /// RSSPECAN_VAL_TETRA_MEAS_MAX  (2) - Maximum
        /// RSSPECAN_VAL_TETRA_MEAS_AVER (3) - Average
        /// RSSPECAN_VAL_TETRA_MEAS_CURR (4) - Current
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_MEAS_MIN (1)
        /// </param>
        /// <param name="Result">
        /// Returns selected result.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchTETRAPVTMeasurement(int PVT, int Measurement_Type, out double Result)
        {
            int pInvokeResult = PInvoke.FetchTETRAPVTMeasurement(this._handle, PVT, Measurement_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the results from the R&amp;S FS-K110 Power versus Time Summary Table measurement. 
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:PVTTable:ALL?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Results">
        /// Returns all Power vs. Time summary table result.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should contain at least 25 elements.
        /// 
        /// (2) Order of results:
        /// 
        /// &lt;current AFTBpower&gt;,
        /// &lt;min AFTBpower&gt;,
        /// &lt;average AFTBpower&gt;,
        /// &lt;max AFTBpower&gt;,
        /// &lt;current BEFBpower&gt;,
        /// &lt;min BEFBpower&gt;,
        /// &lt;average BEFBpower&gt;,
        /// &lt;max BEFBpower&gt;,
        /// &lt;current REFPower&gt;,
        /// &lt;min REFPower&gt;,
        /// &lt;average REFPower&gt;,
        /// &lt;max REFPower&gt;,
        /// &lt;current RELafterpow&gt;,
        /// &lt;min RELafterpow&gt;,
        /// &lt;average RELafterpow&gt;,
        /// &lt;max RELafterpow&gt;,
        /// &lt;current RELBeforepow&gt;,
        /// &lt;min RELBeforepow&gt;,
        /// &lt;average RELBeforepow&gt;,
        /// &lt;max RELBeforepow&gt;,
        /// &lt;current SLOTpower&gt;,
        /// &lt;min SLOTpower&gt;,
        /// &lt;average SLOTpower&gt;,
        /// &lt;max SLOTpower&gt;,
        /// &lt;TRIGsync&gt;
        /// </param>
        /// <param name="Array_Size">
        /// This control sets the size of array in control Results.
        /// 
        /// Valid Values:
        /// &gt;0
        /// 
        /// Default Value: 25
        /// </param>
        /// <param name="Returned_Values">
        /// Returns the number of values in Results array.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchTETRAPVTMeasurementAll(double[] Results, int Array_Size, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.FetchTETRAPVTMeasurementAll(this._handle, Results, Array_Size, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the RBW used for the most recent Spectrum FFT measurement. Units are Hertz.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is valid only if the Spectrum FFT measurement is selected.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_SPECTRUM_FFT_MEASUREMENT_RBW
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:SPECtrum:FFT:RBW?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns the RBW used for the most recent Spectrum FFT measurement.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchTETRASpectrumFFTMeasurementRBW(out double Result)
        {
            int pInvokeResult = PInvoke.FetchTETRASpectrumFFTMeasurementRBW(this._handle, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Returns the measured ACP due to Modulation and ACP due to Transient results.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_FETCH_ACP_MEASUREMENT
        /// RSSPECAN_ATTR_TETRA_FETCH_ACP_MODULATION_RBW
        /// 
        /// Remote-control command(s):
        /// FETCh:BURSt:ACPMod:CHAN&lt;0..6&gt;:BANDwidth?
        /// FETCh:BURSt:ACPMod:CHAN&lt;0..6&gt;:RRCosine?
        /// FETCh:BURSt:ACPMod:CHAN&lt;0..6&gt;:OFFSet?
        /// FETCh:BURSt:ACPMod:CHAN&lt;0..6&gt;:ABSPower[:CURRent]?
        /// FETCh:BURSt:ACPMod:CHAN&lt;0..6&gt;:ABSPower:MAXimum? 
        /// FETCh:BURSt:ACPMod:CHAN&lt;0..6&gt;:RELPower[:CURRent]? 
        /// FETCh:BURSt:ACPMod:CHAN&lt;0..6&gt;:RELPower:MAXimum? 
        /// FETCh:BURSt:ACPMod:CHAN&lt;0..6&gt;:DELLimit[:CURRent]? 
        /// FETCh:BURSt:ACPMod:CHAN&lt;0..6&gt;:DELLimit:MAXimum?
        /// FETCh:BURSt:ACPMod:RBW?
        /// FETCh:BURSt:ACPTrans:CHAN&lt;0..6&gt;:BANDwidth?
        /// FETCh:BURSt:ACPTrans:CHAN&lt;0..6&gt;:RRCosine?
        /// FETCh:BURSt:ACPTrans:CHAN&lt;0..6&gt;:OFFSet?
        /// FETCh:BURSt:ACPTrans:CHAN&lt;0..6&gt;:ABSPower[:CURRent]?
        /// FETCh:BURSt:ACPTrans:CHAN&lt;0..6&gt;:ABSPower:MAXimum? 
        /// FETCh:BURSt:ACPTrans:CHAN&lt;0..6&gt;:RELPower[:CURRent]? 
        /// FETCh:BURSt:ACPTrans:CHAN&lt;0..6&gt;:RELPower:MAXimum? 
        /// FETCh:BURSt:ACPTrans:CHAN&lt;0..6&gt;:DELLimit[:CURRent]? 
        /// FETCh:BURSt:ACPTrans:CHAN&lt;0..6&gt;:DELLimit:MAXimum?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="ACP">
        /// Selects ACP measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_ACP_MODULATION (0) - ACP Modulation
        /// RSSPECAN_VAL_TETRA_ACP_TRANSIENT  (1) - ACP Transient
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_ACP_MODULATION (0)
        /// </param>
        /// <param name="ACP_Measurement">
        /// Selects ACP measurement.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_ACP_BW         (0) - Bandwidth
        /// RSSPECAN_VAL_TETRA_ACP_RCC        (1) - RCC
        /// RSSPECAN_VAL_TETRA_ACP_OFFS       (2) - Offset
        /// RSSPECAN_VAL_TETRA_ACP_ABS_POW    (3) - Abolute Max Power
        /// RSSPECAN_VAL_TETRA_ACP_REL_POW    (4) - Relative Max Power
        /// RSSPECAN_VAL_TETRA_ACP_DLIMIT_POW (5) - Delta to Limit
        /// RSSPECAN_VAL_TETRA_ACP_RBW        (6) - RBW
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_ACP_BW (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Bandwidth: returns the measured ACP Modulation bandwidth in Hz.
        /// 
        /// (2) RCC: returns the measured Root Raised Cosine for the specified channel.
        /// 
        /// (3) Offset: returns the measured ACP Modulation offset in Hz.
        /// 
        /// (4) Abolute Max Power: returns the measured current and maximum determined absolute power in dBm.
        /// 
        /// (5) Relative Max Power: returns the measured current and maximum determined relative power in dB.
        /// 
        /// (6) Delta to Limit: returns the measured Delta to Limit current and maximum power in dB.
        /// 
        /// (7) RBW: returns the RBW used for the most recent ACP due to Modulation measurement. Units are Hertz. This is valid only if the ACP due to Modulation measurement is selected.
        /// </param>
        /// <param name="Channel_Number">
        /// This control selects the ACP measurements channels.
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_NEG_3 (0) - -3
        /// RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_NEG_2 (1) - -2
        /// RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_NEG_1 (2) - -1
        /// RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_TX    (3) - Transmit Channel, TX
        /// RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_POS_1 (4) - 1
        /// RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_POS_2 (5) - 2
        /// RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_POS_3 (6) - 3
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_TX    (3)
        /// 
        /// Note(s):
        /// 
        /// (1) ACP due to Modulation Relative Channel Number:
        /// 
        /// 0 = -3
        /// 1 = -2
        /// 2 = -1
        /// 3 = Transmit Channel, TX
        /// 4 = +1
        /// 5 = +2
        /// 6 = +3
        /// 
        /// (1) ACP due to Transient Relative Channel Number:
        /// 
        /// 0 = Reserved - not available
        /// 1 = Reserved - not available
        /// 2 = -1 (Lower)
        /// 3 = Transmit Channel, TX
        /// 4 = 1 (Upper)
        /// 5 = Reserved - not available
        /// 6 = Reserved - not available
        /// </param>
        /// <param name="Measurement_Type">
        /// Selects measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_MEAS_NONE (0) - None
        /// RSSPECAN_VAL_TETRA_MEAS_MAX  (2) - Maximum
        /// RSSPECAN_VAL_TETRA_MEAS_CURR (4) - Current
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_MEAS_MAX (2)
        /// </param>
        /// <param name="Result">
        /// Returns selected result.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FetchTETRAACP(int ACP, int ACP_Measurement, int Channel_Number, int Measurement_Type, out double Result)
        {
            int pInvokeResult = PInvoke.FetchTETRAACP(this._handle, ACP, ACP_Measurement, Channel_Number, Measurement_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the minimum and maximum values respectively for the X-axis data for the specified trace display and measurement.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_TRACE_X_AXIS_MIN_MAX
        /// 
        /// Remote-control command(s):
        /// DISPlay:WINDow&lt;1|2&gt;:TRACe&lt;1..4&gt;:X:MINimum?
        /// DISPlay:WINDow&lt;1|2&gt;:TRACe&lt;1..4&gt;:X:MAXimum?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// Selects the trace for Detector definition.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 1 to 6 for R&amp;S FSL
        /// 
        /// Default Value: 1
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TRACE_STATE
        /// </param>
        /// <param name="Value_Type">
        /// Selects measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_MEAS_MIN  (1) - Minimum
        /// RSSPECAN_VAL_TETRA_MEAS_MAX  (2) - Maximum
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_MEAS_MIN (1)
        /// </param>
        /// <param name="X_Axis_Value">
        /// Returns the RBW used for the most recent ACP due to Modulation measurement. Units are Hertz.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetTETRATraceXAxisMinMax(int Window, int Trace, int Value_Type, out double X_Axis_Value)
        {
            int pInvokeResult = PInvoke.GetTETRATraceXAxisMinMax(this._handle, Window, Trace, Value_Type, out X_Axis_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the sampling rate in Hz for the current IQ data.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_IQ_SAMPLE_RATE
        /// 
        /// Remote-control command(s):
        /// TRACe:IQ:DATA:SRATe?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="IQ_Data_Sampling_Rate">
        /// This control returns the sampling rate in Hz for the current IQ data.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_TETRA_IQ_SAMPLE_RATE
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetTETRAIQDataSamplingRate(out double IQ_Data_Sampling_Rate)
        {
            int pInvokeResult = PInvoke.GetTETRAIQDataSamplingRate(this._handle, out IQ_Data_Sampling_Rate);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the limit check results.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_RESULT_SUMMARY_LIMIT_CHECK_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate:LIMit:BURSt:EVM:ALL[:AVERage]:RESult?
        /// CALCulate:LIMit:BURSt:EVM:ALL:MAXimum:RESult?
        /// CALCulate:LIMit:BURSt:EVM:DATA[:AVERage]:RESult?
        /// CALCulate:LIMit:BURSt:EVM:DATA:MAXimum:RESult?
        /// CALCulate:LIMit:BURSt:EVM:DHEA[:AVERage]:RESult?
        /// CALCulate:LIMit:BURSt:EVM:DHEA:MAXimum:RESult?
        /// CALCulate:LIMit:BURSt:EVM:DSPilot[:AVERage]:RESult?
        /// CALCulate:LIMit:BURSt:EVM:DSPilot:MAXimum:RESult?
        /// CALCulate:LIMit:BURSt:EVM:SPILot[:AVERage]:RESult?
        /// CALCulate:LIMit:BURSt:EVM:SPILot:MAXimum:RESult?
        /// CALCulate:LIMit:BURSt:IQOFfset[:AVERage]:RESult?
        /// CALCulate:LIMit:BURSt:IQOFfset:MAXimum:RESult?
        /// CALCulate:LIMit:BURSt:FERRor[:AVERage]:RESult?
        /// CALCulate:LIMit:BURSt:FERRor:MAXimum:RESult?
        /// CALCulate:LIMit:BURSt:PHASerror[:AVERage]:RESult?
        /// CALCulate:LIMit:BURSt:PHASerror:MAXimum:RESult?
        /// CALCulate:LIMit:BURSt:MAGError[:AVERage]:RESult?
        /// CALCulate:LIMit:BURSt:MAGError:MAXimum:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Limit_Type">
        /// Selects the limit type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_LIM_EVM_ALL  (0) - Combined EVM
        /// RSSPECAN_VAL_TETRA_LIM_EVM_DATA (1) - Data EVM
        /// RSSPECAN_VAL_TETRA_LIM_EVM_DHEA (2) - Data + Header EVM
        /// RSSPECAN_VAL_TETRA_LIM_EVM_DSP  (3) - Data + Sync + Pilot EVM
        /// RSSPECAN_VAL_TETRA_LIM_EVM_SPIL (4) - Sync + Pilot EVM
        /// RSSPECAN_VAL_TETRA_LIM_IQOF     (5) - IQ Offset Limit
        /// RSSPECAN_VAL_TETRA_LIM_FERR     (6) - Center Frequency Error
        /// RSSPECAN_VAL_TETRA_LIM_PHAS     (7) - Phase Error
        /// RSSPECAN_VAL_TETRA_LIM_MAGN     (8) - Magnitude Error
        ///  
        /// Default Value: RSSPECAN_VAL_TETRA_LIM_EVM_ALL  (0)
        /// </param>
        /// <param name="Mode">
        /// Sets the limit measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_MEAS_MAX  (2) - Maximum
        /// RSSPECAN_VAL_TETRA_MEAS_AVER (3) - Average
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_MEAS_MAX (2)
        /// 
        /// 
        /// </param>
        /// <param name="Result">
        /// Returns the limit check result.
        /// 
        /// Returned Value:
        /// 0 - Passed
        /// 1 - Failed
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetTETRAResultSummaryLimitCheckResult(int Limit_Type, int Mode, out int Result)
        {
            int pInvokeResult = PInvoke.GetTETRAResultSummaryLimitCheckResult(this._handle, Limit_Type, Mode, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns all limit check results.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// CALCulate:LIMit:BURSt:ALL:RESult?
        /// 
        /// 
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// This control sets the size of array in control Results.
        /// 
        /// Valid Values:
        /// &gt;0
        /// 
        /// Default Value: 18
        /// </param>
        /// <param name="Results">
        /// Returns the array of limit resutls.
        /// 
        /// Returned Values:
        /// 0 - Passed
        /// 1 - Failed
        /// 
        /// in order:
        /// 
        /// &lt;average EVM All Result&gt;,
        /// &lt;max EVM All Limit&gt;,
        /// &lt;average EVM Data+Sync+Pilot Result&gt;,
        /// &lt;max EVM Data+Sync+Pilot Result&gt;,
        /// &lt;average EVM Data+Header Result&gt;,
        /// &lt;max EVM Data+Header Result&gt;,
        /// &lt;average EVM Data Result&gt;,
        /// &lt;max EVM Data Result&gt;,
        /// &lt;average EVM Sync+Pilot Result&gt;,
        /// &lt;max EVM Sync+Pilot Result&gt;,
        /// &lt;average IQ Offset Result&gt;,
        /// &lt;max IQ Offset Result&gt;,
        /// &lt;average Frequency Error Result&gt;,
        /// &lt;max Frequency Error Result&gt;,
        /// &lt;average Phase Error Result&gt;,
        /// &lt;max Phase Error Result&gt;
        /// &lt;average Magnitude Error Result&gt;,
        /// &lt;max Magnitude Error Result&gt;
        /// 
        /// Note(s):
        /// 
        /// (1) The array should contain at least 18 elements.
        /// </param>
        /// <param name="Returned_Values">
        /// Returns the number of values in Results array.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetTETRAResultSummaryLimitCheckResultsAll(int Array_Size, int[] Results, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.GetTETRAResultSummaryLimitCheckResultsAll(this._handle, Array_Size, Results, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the ACP due to Modulation and ACP Due to Transients relative power limit check result.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_TETRA_ACP_LIMIT_CHECK_RESULT
        /// 
        /// Remote-control command(s):
        /// CALCulate:LIMit:BURSt:ACPMod:CHAN&lt;0..6&gt;:RELPower[:CURRent]:RESult? 
        /// CALCulate:LIMit:BURSt:ACPMod:CHAN&lt;0..6&gt;:RELPower:MAXimum:RESult?
        /// CALCulate:LIMit:BURSt:ACPTrans:CHAN&lt;0..6&gt;:RELPower[:CURRent]:RESult? 
        /// CALCulate:LIMit:BURSt:ACPTrans:CHAN&lt;0..6&gt;:RELPower:MAXimum:RESult? 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="ACP">
        /// Selects ACP measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_ACP_MODULATION (0) - ACP Modulation
        /// RSSPECAN_VAL_TETRA_ACP_TRANSIENT  (1) - ACP Transient
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_ACP_MODULATION (0)
        /// </param>
        /// <param name="Channel_Number">
        /// This control selects the ACP measurements channels.
        /// 
        /// Valid Values: 
        /// RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_NEG_3 (0) - -3
        /// RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_NEG_2 (1) - -2
        /// RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_NEG_1 (2) - -1
        /// RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_POS_1 (4) - 1
        /// RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_POS_2 (5) - 2
        /// RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_POS_3 (6) - 3
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_NEG_1 (2)
        /// 
        /// Note(s):
        /// 
        /// (1) ACP due to Modulation Relative Channel Number:
        /// 
        /// 0 = -3
        /// 1 = -2
        /// 2 = -1
        /// 4 = +1
        /// 5 = +2
        /// 6 = +3
        /// 
        /// (1) ACP due to Transient Relative Channel Number:
        /// 
        /// 0 = Reserved - not available
        /// 1 = Reserved - not available
        /// 2 = -1 (Lower)
        /// 4 = 1 (Upper)
        /// 5 = Reserved - not available
        /// 6 = Reserved - not available
        /// </param>
        /// <param name="Measurement_Type">
        /// Selects measurement type.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_TETRA_MEAS_MAX  (2) - Maximum
        /// RSSPECAN_VAL_TETRA_MEAS_CURR (4) - Current
        /// 
        /// Default Value: RSSPECAN_VAL_TETRA_MEAS_MAX (2)
        /// </param>
        /// <param name="Result">
        /// Returns the limit check result.
        /// 
        /// Returned Value:
        /// 0 - Passed
        /// 1 - Failed
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetTETRAACPLimitCheckResult(int ACP, int Channel_Number, int Measurement_Type, out int Result)
        {
            int pInvokeResult = PInvoke.GetTETRAACPLimitCheckResult(this._handle, ACP, Channel_Number, Measurement_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the input for servicing purposes.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SERVICE_INPUT_SOURCE
        /// RSSPECAN_ATTR_SERVICE_INPUT_LEVEL
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:INPut[:SELect] CALibration | RF | TG
        /// DIAGnostic&lt;1|2&gt;:SERVice:CSOurce[:POWer]
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Input">
        /// Toggles between the RF input on the front panel and the internal 128-MHz reference signal in the selected measurement window.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_INPUT_RF  (0) - Input RF
        /// RSSPECAN_VAL_INPUT_CAL (1) - Input Cal
        /// RSSPECAN_VAL_INPUT_TG  (2) - Input TG
        /// 
        /// Default Value: RSSPECAN_VAL_INPUT_RF (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SERVICE_INPUT_SOURCE
        /// 
        /// Note(s):
        /// 
        /// (1) RF  - switches the input to the input connector (normal position). This is the default setting.
        /// 
        /// (2) Cal - switches the RF input to the internal calibration source (65.83 MHz) and activates the data entry of the output level of the calibration source. Possible
        /// values are 0 dB and ?30 dB.
        /// 
        /// (3) TG - switches the RF input to the tracking generator output. It is valid only for R&amp;S FSL or R&amp;S ESL or R&amp;S FSV.
        /// </param>
        /// <param name="Level">
        /// This control switches the level of the 128 MHz reference signal source between 0 dBm and -30 dBm in the selected measurement window.
        /// 
        /// Valid Values:
        /// 0.0 and -30.0 dBm
        /// 
        /// Default Value: -30.0 dBm
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SERVICE_INPUT_LEVEL
        /// 
        /// Note(s):
        /// (1) This value is valid only for Input Cal
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ServiceConfigureInput(int Window, int Input, double Level)
        {
            int pInvokeResult = PInvoke.ServiceConfigureInput(this._handle, Window, Input, Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures pulsed calibration signal. This function takes effect only if the RF input has been set to the internal reference signal using rsspecan_ServiceConfigureInput.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SERVICE_INPUT_PULSED_STATE
        /// RSSPECAN_ATTR_SERVICE_INPUT_PULSED_RATE
        /// RSSPECAN_ATTR_SERVICE_PULSE_CAL_FREQ_FSV
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:INPut:PULSed[:STATe] ON | OFF
        /// DIAGnostic&lt;1|2&gt;:SERVice:INPut:PULSed:PRATe
        /// DIAGnostic&lt;1|2&gt;:SERVice:INPut:PULSed:CFRequency
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// Toggles the calibration signal in the selected measurement window between pulsed (On) and non-pulsed (Off).
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SERVICE_INPUT_PULSED_STATE
        /// </param>
        /// <param name="Frequency">
        /// Selects the pulse rate (or calibration frequency for R&amp;S FSV) for the pulsed calibration signal in the selected measurement window.
        /// 
        /// Valid Values:
        /// 10.0e3, 62.5e3, 100.0e3, 500.0e3, 1.0e6, 128.e6, 640e6 Hz
        /// 15.62882 to 64.0e6 Hz - for R&amp;S FSV
        /// 
        /// Default Value: 128.0e6 Hz (64.0e6 Hz for R&amp;S FSV)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SERVICE_INPUT_PULSED_RATE RSSPECAN_ATTR_SERVICE_PULSE_CAL_FREQ_FSV
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ServiceConfigurePulsedInput(int Window, bool State, double Frequency)
        {
            int pInvokeResult = PInvoke.ServiceConfigurePulsedInput(this._handle, Window, System.Convert.ToUInt16(State), Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures rectangle calibration signal. This function takes effect only if the RF input has been set to the internal reference signal using rsspecan_ServiceConfigureInput.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SERVICE_INPUT_RECTANGLE_STATE
        /// RSSPECAN_ATTR_SERVICE_INPUT_RECTANGLE_RATE
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:INPut:RECTangle[:STATe] ON | OFF
        /// DIAGnostic&lt;1|2&gt;:SERVice:INPut:RECTangle:PRATe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// Switches the rectangle calibration signal on or off in the selected measurement window.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SERVICE_INPUT_RECTANGLE_STATE
        /// 
        /// </param>
        /// <param name="Frequency">
        /// Sets the frequency of the rectangle calibration signal in the selected measurement window.
        /// 
        /// Valid Values:
        /// 5.0e3, 31.25e3, 50.0e3, 250.0e3, 500.0e3 Hz.
        /// 
        /// Default Value: 5.0e3 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SERVICE_INPUT_RECTANGLE_RATE
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ServiceConfigureRectangleInput(int Window, bool State, double Frequency)
        {
            int pInvokeResult = PInvoke.ServiceConfigureRectangleInput(this._handle, Window, System.Convert.ToUInt16(State), Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the 28-V supply of the noise source at the rear panel on or off.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SERVICE_NOISE_SOURCE
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:NSOurce ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Noise">
        /// Switches the 28-V supply of the noise source at the rear panel on or off.
        ///  
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SERVICE_NOISE_SOURCE
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ServiceConfigureNoise(bool Noise)
        {
            int pInvokeResult = PInvoke.ServiceConfigureNoise(this._handle, System.Convert.ToUInt16(Noise));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the baseband signal source.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SERVICE_BASEBAND_SIGNAL_SOURCE
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:IQ:INPut IQ | GND | CALDc | CALPulse
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Source">
        /// Selects the baseband signal source.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SERVICE_BB_IQ       (0) - IQ
        /// RSSPECAN_VAL_SERVICE_BB_GND      (1) - GND
        /// RSSPECAN_VAL_SERVICE_BB_CALDC    (2) - Calibration DC
        /// RSSPECAN_VAL_SERVICE_BB_CALPULSE (3) - Calibration Pulse
        /// 
        /// Default Value: RSSPECAN_VAL_SERVICE_BB_IQ (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SERVICE_BASEBAND_SIGNAL_SOURCE
        /// 
        /// Note(s):
        /// 
        /// (1) IQ - The female I and Q connectors of the instrument are the baseband signal sources.
        /// 
        /// (2) GND - The baseband inputs are internally connected to ground.
        /// 
        /// (3) Calibration DC - The baseband signal source is the DC Cal signal. The voltage of this signal can be set with diag:serv:iq:cal:dc.
        /// 
        /// (4) Calibration Pulse - The baseband signal source is the Pulse Cal signal. The frequency of this signal can be set with diag:serv:iq:prat.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ServiceConfigureBasebandSource(int Source)
        {
            int pInvokeResult = PInvoke.ServiceConfigureBasebandSource(this._handle, Source);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures calibration signals. This function takes effect only if the RF input has been set to the internal reference signal using rsspecan_ServiceConfigureInput.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SERVICE_PULSE_CAL_DESTINATION
        /// RSSPECAN_ATTR_SERVICE_PULSE_CAL_FREQ
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:IQ:CALibration:DESTination IHIGh | ILOW | QHIGh | QLOW
        /// DIAGnostic&lt;1|2&gt;:SERVice:IQ:CALibration:PULSe:PRATe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Destination">
        /// The calibration signals (DC Cal signal and Pulse Cal signal) can only be switched to one input at a time. This attribute switches the calibration signal to the I or Q path.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SERVICE_FEED_IHIG (0) - I Positive
        /// RSSPECAN_VAL_SERVICE_FEED_ILOW (1) - I Negative
        /// RSSPECAN_VAL_SERVICE_FEED_QHIG (2) - Q Positive
        /// RSSPECAN_VAL_SERVICE_FEED_QLOW (3) - Q Negative
        /// 
        /// Default Value: RSSPECAN_VAL_SERVICE_FEED_IHIG (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SERVICE_PULSE_CAL_DESTINATION
        /// 
        /// Note(s):
        /// 
        /// (1) I Positive - Feed the calibration signal into the positive I path.
        /// 
        /// (2) I Negative - Feed the calibration signal into the negative I path.
        /// 
        /// (3) Q Positive - Feed the calibration signal into the positive Q path.
        /// 
        /// (4) Q Negative - Feed the calibration signal into the negative Q path.
        /// </param>
        /// <param name="Frequency">
        /// Sets the frequency of the Pulse Cal signal.
        /// 
        /// Valid Values:
        /// 10.0e3, 62.5e3, 80.0e3, 100.0e3, 102.4e3, 200.e3, 500e3, 1e6, 2e6, 4e6 Hz
        /// 
        /// Default Value: 62.5e3 Hz
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SERVICE_PULSE_CAL_FREQ
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ServiceConfigurePulsedIQCalibrationSignal(int Destination, double Frequency)
        {
            int pInvokeResult = PInvoke.ServiceConfigurePulsedIQCalibrationSignal(this._handle, Destination, Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the voltage for the DC Cal signal in Volt.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SERVICE_DC_CAL_VOLTAGE
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:IQ:CALibration:DC
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Voltage">
        /// Selects the voltage for the DC Cal signal in Volt.
        /// 
        /// Valid Values:
        /// 0, 0.1, 0.178, 0.316, 0.562, 1.0 V
        /// 
        /// Default Value: 0.0 V
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SERVICE_DC_CAL_VOLTAGE
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ServiceConfigureDCIQCalibrationSignal(double Voltage)
        {
            int pInvokeResult = PInvoke.ServiceConfigureDCIQCalibrationSignal(this._handle, Voltage);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the comb generator. This function takes effect only if the internal reference signal is set using rsspecan_ServiceConfigureInput.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is valid for R&amp;S FSL or R&amp;S ESL only.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SERVICE_INPUT_COMB_FREQUENCY
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:INPut:PULSed:PRATe
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Comb_Frequency">
        /// Selects comb generator frequency for the internal calibration. 
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_SERV_COMB_FREQ_COMB1  (0) - Comb/1
        /// RSSPECAN_VAL_SERV_COMB_FREQ_COMB64 (1) - Comb/64
        /// RSSPECAN_VAL_SERV_COMB_FREQ_COMB65 (2) - Comb/65
        /// 
        /// Default Value: 
        /// RSSPECAN_VAL_SERV_COMB_FREQ_COMB1 (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SERVICE_INPUT_COMB_FREQUENCY
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ServiceConfigureCombGenerator(int Window, int Comb_Frequency)
        {
            int pInvokeResult = PInvoke.ServiceConfigureCombGenerator(this._handle, Window, Comb_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the contents of the module info table. Table lines are output as string data and are separated by commas.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:HWINfo?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the size of array passed to Result argument.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 256
        /// </param>
        /// <param name="HW_Info">
        /// This control returns the contents of the module info table. Table lines are output as string data and are separated by commas.
        /// 
        /// "&lt;component 1&gt;|&lt;serial #&gt;|&lt;order #&gt;|&lt;model&gt;|&lt;HWC&gt;|&lt;rev&gt;|&lt;sub rev&gt;",
        /// "&lt;component 2&gt;|&lt;serial #&gt;|&lt;order #&gt;|&lt;model&gt;|&lt;HWC&gt;|&lt;rev&gt;|&lt;sub rev&gt;"
        /// The individual columns of the table are separated from each other by '|'.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ServiceHWInfo(int Array_Size, System.Text.StringBuilder HW_Info)
        {
            int pInvokeResult = PInvoke.ServiceHWInfo(this._handle, Array_Size, HW_Info);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the instrument firmware and option versions.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:VERSinfo?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the size of array passed to Version Info argument.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 1024
        /// </param>
        /// <param name="Version_Info">
        /// This control returns the instrument firmware and option versions.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ServiceVersionInfo(int Array_Size, System.Text.StringBuilder Version_Info)
        {
            int pInvokeResult = PInvoke.ServiceVersionInfo(this._handle, Array_Size, Version_Info);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the decimal separator (decimal point or comma) for floating-point numerals contained in the file.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FILE_DEC_SEPARATOR
        /// 
        /// Remote-control command(s):
        /// FORMat:DEXPort:DSEParator POINt | COMMa
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Decimal_Separator">
        /// This control sets the decimal separator for the ASCII data stored to the file.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_DEC_SEP_POIN  (0) - Point '.' 
        /// RSSPECAN_VAL_DEC_SEP_COMMA (1) - Comma ','
        /// 
        /// Default Value: RSSPECAN_VAL_DEC_SEP_POIN  (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FILE_DEC_SEPARATOR
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FileDecimalSeparator(int Decimal_Separator)
        {
            int pInvokeResult = PInvoke.FileDecimalSeparator(this._handle, Decimal_Separator);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// The MMEMory (mass memory) subsystem function provides commands which allow for access to the storage media of the instrument and for storing and loading various instrument settings.
        /// The File to be Printed/Plotted command stores the HCOPy outputs in a file. The various drives can be addressed via the mass storage unit specifier &lt;msus&gt; using the conventional DOS syntax. The internal hard disk is addressed by "C:", the floppy-disk drive installed by "A:". The file names &lt;file_name&gt; are indicated as string parameters with the commands being enclosed in quotation marks. They correspond to the DOS conventions.
        /// DOS file names consist of max. 8 ASCII characters and an extension of up to three characters separated from the file name by a colon "." Both, the colon and the extension are optional. The colon is not part of the file name. DOS file names do not differ between uppercase and lowercase notation. All letters and digits are permitted as well as the special characters "_", "^", "$", "~", "!", "#", "%", "&amp;", "-", "{","}", "(", ")", "@" and "' ". Reserved file names are CLOCK$, CON, AUX, COM1 to COM4, LPT1 to LPT3, NUL and PRN. The two characters "*" and "?" have the function of so-called "wildcards", i.e., they are variables for
        /// selection of several files. The question mark "?" replaces exactly one character which may be any, the asterisk means any of the remaining characters in the file name. "*.*" thus means all files in a directory.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FILE_MANAGER_MAKE_DIR
        /// -
        /// -
        /// RSSPECAN_ATTR_FILE_MANAGER_DELETE
        /// RSSPECAN_ATTR_FILE_MANAGER_DELETE_DIR
        /// RSSPECAN_ATTR_FILE_MANAGER_FORMAT_DISK
        /// RSSPECAN_ATTR_FILE_MANAGER_EDIT_PATH
        /// RSSPECAN_ATTR_FILE_MANAGER_EDIT_PATH_DEVICE
        /// 
        /// Remote-control command(s):
        /// MMEMory:MDIRectory &lt;file_name&gt;
        /// MMEMory:COPY &lt;file_source&gt;,&lt;file_destination&gt;
        /// MMEMory:MOVE &lt;file_source&gt;,&lt;file_destination&gt;
        /// MMEMory:DELete
        /// MMEMory:RDIRectory
        /// MMEMory:INITialize
        /// MMEMory:CDIRectory
        /// MMEMory:MSIS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Operation">
        /// This control selects the type of memory operation to be performed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FILE_NEW    (0) - New Folder (Default Value)
        /// RSSPECAN_VAL_FILE_COPY   (1) - Copy
        /// RSSPECAN_VAL_FILE_RENAME (2) - Rename
        /// RSSPECAN_VAL_FILE_DELETE (3) - Delete File
        /// RSSPECAN_VAL_FILE_RDIR   (4) - Delete Directory
        /// RSSPECAN_VAL_FILE_FORMAT (5) - Format Disk
        /// RSSPECAN_VAL_FILE_CDIR   (6) - Change Directory
        /// RSSPECAN_VAL_FILE_CDISC  (7) - Change Drives
        /// 
        /// Notes:
        /// 
        /// (1) New Folder
        /// This command creates a new directory. Put the directory name into Source control (Example 'D:\\USER\\DATA').
        /// 
        /// (2) Copy:
        /// This command copies the files indicated. The indication of the file name may include the path and the drive. The file names and path information must be in accordance with the DOS conventions. Use Source (Example 'D:\\USER\\DATA\\SETUP.CFG') and Destination (Example 'A:') controls to define file names.
        /// 
        /// (3) Rename
        /// This command renames existing files. Use Source (Example 'TEST01.CFG') and Destination (Example 'SETUP.CFG') controls to define file names.
        /// 
        /// (3) Delete File:
        /// This command deletes the files indicated. The indication of the file name contains the path and, optionally, the drive. Indication of the path corresponds to the DOS conventions. The file name includes indication of the path and may also include the drive. The path name corresponds to the DOS conventions. Use Source control to define files to be deleted (Example 'TEST01.HCP').
        /// 
        /// (4) Delete Directory:
        /// This command deletes the directory indicated. The directory name includes indication of the path and may also include the drive. The path name corresponds to the DOS conventions. Use Source control to define directory name (Example 'D:\\TEST'). 
        /// 
        /// (5) Format Disk:
        /// This command formats the disk in drive A. Formatting deletes all data stored on the floppy disk. Use Source control to define disk drive (Example 'A:'). Not available for R&amp;S FMU instrument.
        /// 
        /// (5) Change Directory:
        /// This command changes the current directory. In addition to the path name, the indication of the directory may contain the drive name. The path name complies with the DOS conventions. Put the directory name into Source control (Example 'D:\\USER\\DATA').
        /// 
        /// (6) Change Drives:
        /// This command changes to the drive indicated. The drive may be the internal hard disk 'D:' or the floppy-disk drive 'A:'. The drive is indicated according to the DOS conventions. Use Source control to define disk drive.
        /// 
        /// </param>
        /// <param name="Source">
        /// This control specifies the source of the operation to be performed. If the operation requires a single parameter, this control specifies that parameter.
        /// 
        /// Valid Values: not tested
        /// 
        /// Default Value: "D:\\user\\data"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FILE_MANAGER_MAKE_DIR
        /// RSSPECAN_ATTR_FILE_MANAGER_DELETE
        /// RSSPECAN_ATTR_FILE_MANAGER_DELETE_DIR
        /// RSSPECAN_ATTR_FILE_MANAGER_FORMAT_DISK
        /// RSSPECAN_ATTR_FILE_MANAGER_EDIT_PATH
        /// RSSPECAN_ATTR_FILE_MANAGER_EDIT_PATH_DEVICE
        /// 
        /// </param>
        /// <param name="Destination">
        /// This control specifies the destination of the operation to be performed.
        /// 
        /// Valid Values: not tested
        /// 
        /// Default Value: ""
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int FileManagerOperations(int Operation, string Source, string Destination)
        {
            int pInvokeResult = PInvoke.FileManagerOperations(this._handle, Operation, Source, Destination);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads the indicated directory. According to DOS convention, wild card characters can be entered in order to query eg. a list of all file of a certain type.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// MMEMory:CATalog?
        /// *CLS
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Directory">
        /// This control specifies the directory in which the command will be performed.
        /// 
        /// Valid Values: not tested
        /// 
        /// Default Value: "D:\\user\\data"
        /// 
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the ViChar array you specify for the Output parameter.  
        /// 
        /// If the current value of the attribute, including the terminating NUL byte, contains more bytes that you indicate in this parameter, the function copies Buffer Size - 1 bytes into the buffer, places an ASCII NUL byte at the end of the buffer, and returns the buffer size you must pass to get the entire value.  For example, if the value is "123456" and the Buffer Size is 4, the function places "123" into the buffer and returns 7.
        /// 
        /// If you pass a negative number, the function copies the value to the buffer regardless of the number of bytes in the value.
        /// 
        /// If you pass 0, you can pass VI_NULL for the Attribute Value buffer parameter.
        /// 
        /// </param>
        /// <param name="Output">
        /// This indicator returns the files in the indicated directory as comma separated strings, for example: 'SPOOL1.WMF','SPOOL2.WMF','SPOOL3.WMF'
        /// 
        /// The buffer must be of type ViChar and have at least as many bytes as indicated in the Buffer Size parameter.
        /// 
        /// If the current value of the attribute, including the terminating NUL byte, contains more bytes that you indicate in this parameter, the function copies Buffer Size - 1 bytes into the buffer, places an ASCII NUL byte at the end of the buffer, and returns the buffer size you must pass to get the entire value.  For example, if the value is "123456" and the Buffer Size is 4, the function places "123" into the buffer and returns 7.
        /// 
        /// If you specify 0 for the Buffer Size parameter, you can pass VI_NULL for this parameter.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Directory) out of range.
        /// BFFC0003  Parameter 3 (Buffer Size) out of range.
        /// BFFC0004  Parameter 4 (Output) is null pointer.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int FileDirectory(string Directory, int Buffer_Size, System.Text.StringBuilder Output)
        {
            int pInvokeResult = PInvoke.FileDirectory(this._handle, Directory, Buffer_Size, Output);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the directories and files in the given path. According to DOS convention, wild card characters can be entered in order to query eg. a list of all file of a certain type.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// MMEMory:CATalog:LONG? &lt;path&gt;
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Directory">
        /// This control specifies the directory in which the command will be performed.
        /// 
        /// Valid Values: not tested
        /// 
        /// Default Value: "D:\\user\\data"
        /// 
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the ViChar array you specify for the Output parameter.  
        /// 
        /// If the current value of the attribute, including the terminating NUL byte, contains more bytes that you indicate in this parameter, the function copies Buffer Size - 1 bytes into the buffer, places an ASCII NUL byte at the end of the buffer, and returns the buffer size you must pass to get the entire value in Returned Values.  For example, if the value is "123456" and the Buffer Size is 4, the function places "123" into the buffer and returns 7 in Returned Values.
        /// 
        /// If you specify 0 for the Buffer Size parameter, you can pass VI_NULL for this parameter.
        /// 
        /// Valid Values: any number
        /// 
        /// Default Value: 512
        /// </param>
        /// <param name="Output">
        /// This indicator returns the file/directory information in the indicated directory as comma separated strings, in format: &lt;used_bytes_in_this_directory&gt;,&lt;free_bytes_on_this_disk&gt;,
        /// "&lt;file_name&gt;,&lt;file_type&gt;,&lt;filesize_in_bytes&gt;",
        /// "&lt;file_name&gt;,&lt;file_type&gt;,&lt;filesize_in_bytes&gt;", ..
        /// 
        /// with
        /// 
        /// &lt;file_name&gt; name of file or directory
        /// &lt;file_type&gt; file type: DIR (directory), ASCii (ASCII file),
        /// BINary (binary file) and STATe (file with device
        /// settings)
        /// &lt;filesize_in_bytes&gt; size of file, 0 for directories
        /// 
        /// The buffer must be of type ViChar and have at least as many bytes as indicated in the Buffer Size parameter.
        /// 
        /// If the current value of the attribute, including the terminating NUL byte, contains more bytes that you indicate in this parameter, the function copies Buffer Size - 1 bytes into the buffer, places an ASCII NUL byte at the end of the buffer, and returns the buffer size you must pass to get the entire value in Returned Values.  For example, if the value is "123456" and the Buffer Size is 4, the function places "123" into the buffer and returns 7 in Returned Values.
        /// 
        /// If you specify 0 for the Buffer Size parameter, you can pass VI_NULL for this parameter.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Directory) out of range.
        /// BFFC0003  Parameter 3 (Buffer Size) out of range.
        /// BFFC0004  Parameter 4 (Output) is null pointer.
        /// BFFC0005  Parameter 5 (Returned Values) is null pointer.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int FileDirectoryPath(string Directory, int Buffer_Size, System.Text.StringBuilder Output)
        {
            int pInvokeResult = PInvoke.FileDirectoryPath(this._handle, Directory, Buffer_Size, Output);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function allows to provide data set operation for Save/Recall items of memory subsystem.
        /// 
        /// The various drives can be addressed via the "mass storage unit specifier" &lt;msus&gt; using the conventional DOS syntax. The internal hard disk is addressed by "D:", the floppy disk drive by "A:".
        /// 
        /// For reasons of compatibility with the FSE instruments, addressing the hard disk by "C:" is also accepted. Since hard disk "C:" is reserved for instrument software, all read and write operations are rerouted to hard disk "D:" in normal operation (service level 0).
        /// 
        /// The file names &lt;file_name&gt; are indicated as string parameters with the commands being enclosed in quotation marks. They also comply with DOS conventions. DOS file names consist of max. 8 ASCII characters and an extension of up to three characters
        /// separated from the file name by a dot "." Both, the dot and the extension are optional. The dot is not part of the file name. DOS file names do not distinguish between uppercase and lowercase notation. All letters and digits are permitted as well as the special characters "_", "^", "$", "~", "!", "#", "%", "&amp;", "-", "{", "}", "(", ")", "@" and "`". Reserved file names are CLOCK$, CON, AUX, COM1 to COM4, LPT1 to LPT3,
        /// NUL and PRN. The two characters "*" and "?" have the function of so-called "wildcards", i.e., they are variables for
        /// selection of several files. The question mark "?" replaces exactly one character, the asterisk means any
        /// of the remaining characters in the file name. "*.*" thus means all files in a directory.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FILE_SAVE
        /// RSSPECAN_ATTR_FILE_RECALL
        /// RSSPECAN_ATTR_FILE_DATA_CLEAR
        /// RSSPECAN_ATTR_FILE_STARTUP_RECALL
        /// 
        /// Remote-control command(s):
        /// MMEMory:STORe&lt;1|2&gt;:STATe 1,&lt;file name&gt;
        /// MMEMory:LOAD:STATe 1,&lt;file_name&gt;
        /// MMEMory:CLEar:STATe 1,&lt;file_name&gt;
        /// MMEMory:LOAD:AUTO 1,&lt;file_name&gt;
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Operation">
        /// This control selects the type of memory operation to be performed.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_MEM_SAVE    (0) - Save
        /// RSSPECAN_VAL_MEM_RECALL (1) - Recall
        /// RSSPECAN_VAL_MEM_DELETE  (2) - Delete
        /// RSSPECAN_VAL_MEM_STARTUP (3) - Startup Recall
        /// 
        /// Notes:
        /// 
        /// (1) Save:
        /// This command stores the current device settings in a series of files which have the indicated file name, but different extensions. The file name includes indication of the path and may also include the drive name. The path name complies with DOS conventions. (Example 'D:\\USER\\DATA').
        /// 
        /// (2) Recall:
        /// This command loads device settings from files.The contents of the file are loaded and set as the new device state. The file name includes indication of the path and may also include the drive name. The path name complies with DOS conventions. (Example 'D:\\USER\\file_name.fsp').
        /// 
        /// (3) Delete:
        /// This command deletes the instrument setting selected by &lt;file_name&gt;. All associated files on the mass
        /// memory storage are cleared. A list of the extensions used is included under MMEMory:LOAD:STATe. The file name includes indication of the path and may also include the drive. The path name complies with DOS conventions. (Example 'D:\\USER\\file_name.fsp').
        /// 
        /// (4) Startup Recall:
        /// This command defines which device setting is automatically loaded after the device is switched on. The contents of the file are read after switching on the device and used to define the new device state. The file name includes indication of the path and may also include the drive. The path name complies with DOS
        /// conventions. (Example 'D:\\USER\\file_name.fsp'). 'FACTORY' denotes the data set previously in the instrument.
        /// 
        /// </param>
        /// <param name="Path">
        /// This control specifies the source of the operation to be performed. 
        /// 
        /// Valid Values: not tested
        /// 
        /// Default Value: "D:\\user\\data"
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FILE_SAVE
        /// RSSPECAN_ATTR_FILE_RECALL
        /// RSSPECAN_ATTR_FILE_DATA_CLEAR
        /// RSSPECAN_ATTR_FILE_STARTUP_RECALL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int DataSetFileOperations(int Operation, string Path)
        {
            int pInvokeResult = PInvoke.DataSetFileOperations(this._handle, Operation, Path);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function stores the current device settings in a *.FSL.dfl file. The file name used in the last saving process is automatically counted up to the next unused name.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available for R&amp;S FSL or R&amp;S ESL.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FILE_SAVE_NEXT
        /// 
        /// Remote-control command(s):
        /// MMEMory:STORe&lt;1|2&gt;:STATe:NEXT
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int DataSetFileSaveNext()
        {
            int pInvokeResult = PInvoke.DataSetFileSaveNext(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines a comment ( max. 60 characters) for a device setting to be stored.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FILE_EDIT_COMM
        /// 
        /// Remote-control command(s):
        /// MMEMory:COMMent &lt;string&gt;
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Comment">
        /// This control specifies the file comment.
        /// 
        /// Valid Values: not tested
        /// 
        /// Default Value: ""
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FILE_EDIT_COMM
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int DataSetFileComment(string Comment)
        {
            int pInvokeResult = PInvoke.DataSetFileComment(this._handle, Comment);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function deletes all device settings in the selected directory.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FILE_MANAGER_EDIT_PATH
        /// RSSPECAN_ATTR_FILE_DATA_CLEAR_ALL
        /// 
        /// Remote-control command(s):
        /// MMEMory:CDIRectory
        /// MMEMory:CLEar:ALL
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Directory">
        /// This control specifies the directory.
        /// 
        /// Valid Values: not tested
        /// 
        /// Default Value: "D:\\user\\data"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_FILE_MANAGER_EDIT_PATH
        /// RSSPECAN_ATTR_FILE_DATA_CLEAR_ALL
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int DataSetFileClearAll(string Directory)
        {
            int pInvokeResult = PInvoke.DataSetFileClearAll(this._handle, Directory);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function add/remove settings to/from the list of partial datasets of a device setting to be stored/loaded. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_NONE
        /// RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_DEFAULT
        /// RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_ALL
        /// RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_HWSETTINGS
        /// RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_TRAC
        /// RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_LINE
        /// RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_SOURCE_CAL_DATA
        /// RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_TRAN
        /// RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_SPEM
        /// RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_CLIST
        /// RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_FINAL
        /// 
        /// Remote-control command(s):
        /// MMEMory:SELect[:ITEM]:NONE
        /// MMEMory:SELect[:ITEM]:DEFault
        /// MMEMory:SELect[:ITEM]:ALL
        /// MMEMory:SELect[:ITEM]:HWSettings ON|OFF
        /// MMEMory:SELect[:ITEM]:TRACe[:ACTive] ON|OFF
        /// MMEMory:SELect[:ITEM]:LINes:ALL ON|OFF
        /// MMEMory:SELect[:ITEM]:TRANsducer:ALL ON | OFF
        /// MMEMory:SELect[:ITEM]:SCData ON | OFF
        /// MMEMory:SELect[:ITEM]:SPECtrogram ON | OFF
        /// MMEMory:SELect[:ITEM]:CLISt ON | OFF
        /// MMEMory:SELect[:ITEM]:FINal ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Item_Selector">
        /// This control selects the items to save or recall.
        /// 
        /// Valid Values:
        /// RSSPECAN_VAL_FILE_ITEMS_NONE          (0) - Select None
        /// RSSPECAN_VAL_FILE_ITEMS_DEFAULT       (1) - Select Default RSSPECAN_VAL_FILE_ITEMS_ALL           (2) - Select All
        /// RSSPECAN_VAL_FILE_ITEMS_HWSETTINGS    (3) - HW Settings
        /// RSSPECAN_VAL_FILE_ITEMS_ALLTRACES     (4) - All Traces
        /// RSSPECAN_VAL_FILE_ITEMS_ALLLINES      (5) - All Lines
        /// RSSPECAN_VAL_FILE_ITEMS_SOUR_CAL_DATA (6) - Source Cal Data
        /// RSSPECAN_VAL_FILE_ITEMS_ALLTRAN       (7) - All Transducers
        /// RSSPECAN_VAL_FILE_ITEMS_SPECTROGRAM   (8) - Spectrogram
        /// RSSPECAN_VAL_FILE_ITEMS_CLIST         (9) - Scan Channel List
        /// RSSPECAN_VAL_FILE_ITEMS_FINAL         (10) - Final Measurement
        /// 
        /// Default Value: RSSPECAN_VAL_FILE_ITEMS_DEFAULT (1)
        /// 
        /// Note(s):
        /// 
        /// (1) Value RSSPECAN_VAL_FILE_ITEMS_SPECTROGRAM is valid only for R&amp;S FSL with K14 option
        /// 
        /// (2) Value RSSPECAN_VAL_FILE_ITEMS_ALLTRAN is not available for R&amp;S FMU.
        /// 
        /// (3) Values RSSPECAN_VAL_FILE_ITEMS_CLIST and RSSPECAN_VAL_FILE_ITEMS_FINAL are valid only for R&amp;S ESL.
        /// </param>
        /// <param name="Item_State">
        /// This control sets if selected item is active or not for to save/recall.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE  (1)
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_NONE
        /// RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_DEFAULT
        /// RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_ALL
        /// RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_HWSETTINGS
        /// RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_TRAC
        /// RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_LINE
        /// RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_SOURCE_CAL_DATA
        /// RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_TRAN
        /// RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_SPEM
        /// RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_CLIST
        /// RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_FINAL
        /// 
        /// Note(s): 
        /// (1) This control has no effect if None, Default or All selected in Item Selector
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int DataSetFileSelectItems(int Item_Selector, bool Item_State)
        {
            int pInvokeResult = PInvoke.DataSetFileSelectItems(this._handle, Item_Selector, System.Convert.ToUInt16(Item_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function maps a drive to a server or server folder of the network. As a prerequisite in Microsoft networks, sharing of this server or server folder must be enabled.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// MMEMory:NETWork:MAP
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Drive_Letter">
        /// This control sets the drive letter.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: "T:"
        /// </param>
        /// <param name="Host_Name___IP_Address">
        /// This control sets the host name or IP address and shared folder.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: "\\server\\folder"
        /// </param>
        /// <param name="Use_Optional_Parameters">
        /// This control sets whether the user name, password and reconnect at logon controls are used.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <param name="User_Name">
        /// This control sets the user name.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// </param>
        /// <param name="Password">
        /// This control sets the password.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// </param>
        /// <param name="Reconnect_At_Logon">
        /// This control sets the whether tha mapped drive should be reconnected at the logon time.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int NetworkMapDrive(string Drive_Letter, string Host_Name___IP_Address, bool Use_Optional_Parameters, string User_Name, string Password, bool Reconnect_At_Logon)
        {
            int pInvokeResult = PInvoke.NetworkMapDrive(this._handle, Drive_Letter, Host_Name___IP_Address, System.Convert.ToUInt16(Use_Optional_Parameters), User_Name, Password, System.Convert.ToUInt16(Reconnect_At_Logon));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function disconnects the selected drive.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_NETWORK_DISCONNECT_DRIVE
        /// 
        /// Remote-control command(s):
        /// MMEMory:NETWork:DISConnect
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Drive_Letter">
        /// This control sets the drive letter to be disconnected.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: "T:"
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_NETWORK_DISCONNECT_DRIVE
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int NetworkDisconnectDrive(string Drive_Letter)
        {
            int pInvokeResult = PInvoke.NetworkDisconnectDrive(this._handle, Drive_Letter);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function lists all unused network drive names.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_UNUSED_NETWORK_DRIVES
        /// 
        /// Remote-control command(s):
        /// MMEMory:NETWork:UNUSeddrives?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the size of Unused Drives array.
        /// 
        /// Valid Values:
        /// &gt; 0
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Unused_Drives">
        /// This control lists all unused network drive names.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_UNUSED_NETWORK_DRIVES
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int NetworkUnusedDrivesList(int Array_Size, System.Text.StringBuilder Unused_Drives)
        {
            int pInvokeResult = PInvoke.NetworkUnusedDrivesList(this._handle, Array_Size, Unused_Drives);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function lists all mapped network drives.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_USED_NETWORK_DRIVES
        /// 
        /// Remote-control command(s):
        /// MMEMory:NETWork:USEDdrives?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the size of Used Drives array.
        /// 
        /// Valid Values:
        /// &gt; 0
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Used_Drives">
        /// This control lists all mapped network drives.
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_USED_NETWORK_DRIVES
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int NetworkUsedDrivesList(int Array_Size, System.Text.StringBuilder Used_Drives)
        {
            int pInvokeResult = PInvoke.NetworkUsedDrivesList(this._handle, Array_Size, Used_Drives);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets if the folder info is showed in the list.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SHOW_NETWORK_FOLDER_INFO
        /// 
        /// Remote-control command(s):
        /// MMEMory:NETWork:USEDdrives ON | OFF
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Show_Folder_Info">
        /// This control sets if the folder info is showed in the list.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Attribute:
        /// RSSPECAN_ATTR_SHOW_NETWORK_FOLDER_INFO
        /// 
        /// Note(s):
        /// 
        /// (1) ON - Lists all mapped network drives including the folder information.
        /// 
        /// (2) OFF - Lists all mapped network drive names.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int NetworkShowFolderInfo(bool Show_Folder_Info)
        {
            int pInvokeResult = PInvoke.NetworkShowFolderInfo(this._handle, System.Convert.ToUInt16(Show_Folder_Info));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function resets the instrument.  
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// *RST
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int reset()
        {
            int pInvokeResult = PInvoke.reset(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function places the instrument in a quiescent state where it has minimal or no impact on the system to which it is connected.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// Not supported
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Disable()
        {
            int pInvokeResult = PInvoke.Disable(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches ON/OFF state checking of the instrument (reading of the Standard Event Register and checking it for error). All driver functions are using state checking. Switch this function to ON when debug your application. For better bus throughput and instrument performance switch it OFF. As default is state checking switched to ON!
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="State">
        /// This control selects ON or OFF instrument state checking.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On (Default Value)
        /// 
        /// Attribute:
        /// RS_ATTR_QUERY_INSTRUMENT_STATUS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int setCheckStatus(bool State)
        {
            int pInvokeResult = PInvoke.setCheckStatus(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function runs the instrument's self test routine and returns the test result(s). 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SERVICE_STEST
        /// 
        /// Remote-control command(s):
        /// *TST?
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Self_Test_Result">
        /// This control contains the value returned from the instrument self test.  Zero means success.  For any other code, see the device's operator's manual.
        /// 
        /// </param>
        /// <param name="Self_Test_Message">
        /// This control contains the string returned from the self test. See the device's operation manual for an explanation of the string's contents.
        /// 
        /// Notes:
        /// 
        /// (1) The array must contain at least 256 elements ViChar[256].
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int self_test(out short Self_Test_Result, System.Text.StringBuilder Self_Test_Message)
        {
            int pInvokeResult = PInvoke.self_test(this._handle, out Self_Test_Result, Self_Test_Message);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads the results of the selftest out of the instrument. The lines of the result table are output as string data separated by commas.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SERVICE_STEST_RESULTS
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:STESt:RESult?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Buffer_Size">
        /// This control specifies the size of the buffer.
        /// 
        /// Note:
        /// This value is not range checked.
        /// </param>
        /// <param name="Results">
        /// Returns contains the results of the selftest out of the instrument. The lines of the restlt table are output as string data separated by commas.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SelfTestResults(int Buffer_Size, System.Text.StringBuilder Results)
        {
            int pInvokeResult = PInvoke.SelfTestResults(this._handle, Buffer_Size, Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the revision numbers of the instrument driver and instrument firmware.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// *IDN?
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Instrument_Driver_Revision">
        /// Returns the instrument driver software revision numbers in the form of a string.
        /// 
        /// You must pass a ViChar array with at least 256 bytes.
        /// </param>
        /// <param name="Firmware_Revision">
        /// Returns the instrument firmware revision numbers in the form of a string.
        /// 
        /// You must pass a ViChar array with at least 256 bytes.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int revision_query(System.Text.StringBuilder Instrument_Driver_Revision, System.Text.StringBuilder Firmware_Revision)
        {
            int pInvokeResult = PInvoke.revision_query(this._handle, Instrument_Driver_Revision, Firmware_Revision);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches option checking of the instrument.
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsetl_init or rsetl_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Option_Checking">
        /// This control switches option checking On or Off.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int setCheckOption(bool Option_Checking)
        {
            int pInvokeResult = PInvoke.setCheckOption(this._handle, System.Convert.ToUInt16(Option_Checking));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the number of the SCPI version, which is relevant for the instrument.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SYST_VERSION
        /// 
        /// Remote-control command(s):
        /// SYSTem:VERSion?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Length">
        /// Sets the allocated memory length of System Version string.
        /// 
        /// Valid Values: &gt;0
        /// 
        /// Default Value: 255
        /// </param>
        /// <param name="System_Version">
        /// Returns the number of the SCPI version, which is relevant for the instrument.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SystemVersion(int Length, System.Text.StringBuilder System_Version)
        {
            int pInvokeResult = PInvoke.SystemVersion(this._handle, Length, System_Version);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the memory size installed on related boards.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is not available for R&amp;S FSV and R&amp;S FSL.
        /// 
        /// Attribute(s):
        /// no attribute
        /// 
        /// Remote-control command(s):
        /// SYSTem:MSIZe? MBOard
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Memory_Size">
        /// Returns the number of the SCPI version, which is relevant for the instrument.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int MemorySize(out int Memory_Size)
        {
            int pInvokeResult = PInvoke.MemorySize(this._handle, out Memory_Size);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads an error code and a message from the instrument's error queue.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SYST_ERR
        /// 
        /// Remote-control command(s):
        /// SYSTem:ERRor?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Error_Code">
        /// Returns the error code read from the instrument's error queue.
        /// 
        /// 
        /// </param>
        /// <param name="Error_Message">
        /// Returns the error message string read from the instrument's error message queue.
        /// 
        /// You must pass a ViChar array with at least 256 bytes.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int error_query(out int Error_Code, System.Text.StringBuilder Error_Message)
        {
            int pInvokeResult = PInvoke.error_query(this._handle, out Error_Code, Error_Message);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads all system messages and returns a list of comma separated strings. Each string corresponds to an entry in the table SYSTEM MESSAGES.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SYST_ERR_LIST
        /// 
        /// Remote-control command(s):
        /// SYSTem:ERRor:LIST?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Buffer_Size">
        /// Sets the size of Errors string.
        /// 
        /// Valid Values:
        /// &gt; 0
        /// 
        /// Default Value:
        /// 1024
        /// </param>
        /// <param name="Errors">
        /// Returns the error message string read from the instrument's error message queue.
        /// 
        /// You must pass a ViChar array with at least 256 bytes.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ErrorList(int Buffer_Size, System.Text.StringBuilder Errors)
        {
            int pInvokeResult = PInvoke.ErrorList(this._handle, Buffer_Size, Errors);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function deletes all entries in the table SYSTEM MESSAGES.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SYST_ERR_CLEAR
        /// 
        /// Remote-control command(s):
        /// SYSTem:ERRor:CLEar:ALL
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ClearInstrumentErrors()
        {
            int pInvokeResult = PInvoke.ClearInstrumentErrors(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function initiates an instrument reset.
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SYST_PRESET
        /// 
        /// Remote-control command(s):
        /// SYSTem:PRESet
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int Preset()
        {
            int pInvokeResult = PInvoke.Preset(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the *IDN string to the factory default (Softkey: ID STRING FACTORY). 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SYST_ID_FACTORY
        /// 
        /// Remote-control command(s):
        /// SYSTem:IDENtify:FACTory
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int IDStringFactory()
        {
            int pInvokeResult = PInvoke.IDStringFactory(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns 1" for the factory default state and "0" if the ID string is changed. 
        /// 
        /// Attribute(s):
        /// RSSPECAN_ATTR_SYST_ID_FACTORY_QUERY
        /// 
        /// Remote-control command(s):
        /// SYSTem:IDENtify:FACTory?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Factory_Default">
        /// Returns 1" for the factory default state and "0" if the ID
        /// string is changed.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetIDStringFactory(out int Factory_Default)
        {
            int pInvokeResult = PInvoke.GetIDStringFactory(this._handle, out Factory_Default);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function is used to read data from the instrument and write it to a user specified file on the host computer.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// FORMat[:DATA] REAL,32
        /// MMEMory:DATA? &lt;file_name&gt;
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Source">
        /// This control selects file for the data transfer from instrument to control computer.
        /// 
        /// Valid Values: not tested
        /// 
        /// Default Value: "D:\\user\\data\\test.hcp"
        /// 
        /// </param>
        /// <param name="Destination">
        /// This control defines destination file to which the data transfer from instrument to control computer applies.
        /// 
        /// Valid Values: not tested
        /// 
        /// Default Value: "\\test.hcp"
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadToFileFromInstrument(string Source, string Destination)
        {
            int pInvokeResult = PInvoke.ReadToFileFromInstrument(this._handle, Source, Destination);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function is used to read data from the host computer and write it to a user specified file in the instrument.
        /// 
        /// Attribute(s):
        /// -
        /// 
        /// Remote-control command(s):
        /// MMEMory:DATA &lt;file_name&gt;[,&lt;block data&gt;]
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Source">
        /// This control selects file for the data transfer from control computer to instrument.
        /// 
        /// Valid Values: not tested
        /// 
        /// Default Value: "\\test.hcp"
        /// 
        /// </param>
        /// <param name="Destination">
        /// This control defines destination file to which the data transfer from control computer to instrument applies.
        /// 
        /// Valid Values: not tested
        /// 
        /// Default Value: "D:\\user\\data\\test.hcp"
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int WriteFromFileToInstrument(string Source, string Destination)
        {
            int pInvokeResult = PInvoke.WriteFromFileToInstrument(this._handle, Source, Destination);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function writes a user-specified string to the instrument.
        /// 
        /// Note:
        /// 
        /// This function bypasses attribute state caching. Therefore, when you call this function, the cached values for all attributes will be invalidated.
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Write_Buffer">
        /// Pass the string to be written to the instrument.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int WriteInstrData(string Write_Buffer)
        {
            int pInvokeResult = PInvoke.WriteInstrData(this._handle, Write_Buffer);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads data from the instrument.
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Number_of_Bytes_To_Read">
        /// Pass the maximum number of bytes to read from the instruments.  
        /// 
        /// Valid Values:  0 to the number of elements in the Read Buffer.
        /// 
        /// Default Value:  0
        /// 
        /// 
        /// </param>
        /// <param name="Read_Buffer">
        /// After this function executes, this parameter contains the data that was read from the instrument.
        /// </param>
        /// <param name="Num_Bytes_Read">
        /// Returns the number of bytes actually read from the instrument and stored in the Read Buffer.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ReadInstrData(int Number_of_Bytes_To_Read, System.Text.StringBuilder Read_Buffer, out uint Num_Bytes_Read)
        {
            int pInvokeResult = PInvoke.ReadInstrData(this._handle, Number_of_Bytes_To_Read, Read_Buffer, out Num_Bytes_Read);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the ViBoolean value.
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Command">
        /// The query to be sent to the instrument.
        /// 
        /// Default Value:  ""
        /// </param>
        /// <param name="Value">
        /// Returns the ViBoolean value
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryViBoolean(string Command, out bool Value)
        {
            ushort ValueAsUShort;
            int pInvokeResult = PInvoke.QueryViBoolean(this._handle, Command, out ValueAsUShort);
            Value = System.Convert.ToBoolean(ValueAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the ViInt32 value.
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Command">
        /// The query to be sent to the instrument.
        /// 
        /// Default Value:  ""
        /// </param>
        /// <param name="Value">
        /// Returns the ViInt32 value
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryViInt32(string Command, out int Value)
        {
            int pInvokeResult = PInvoke.QueryViInt32(this._handle, Command, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the ViReal64 value.
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Command">
        /// The query to be sent to the instrument.
        /// 
        /// Default Value:  ""
        /// </param>
        /// <param name="Value">
        /// Returns the ViReal64 value
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryViReal64(string Command, out double Value)
        {
            int pInvokeResult = PInvoke.QueryViReal64(this._handle, Command, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the ViString value.
        /// 
        /// 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.  The handle identifies a particular instrument session.
        /// 
        /// Default Value:  None
        /// 
        /// </param>
        /// <param name="Command">
        /// The query to be sent to the instrument.
        /// 
        /// Default Value:  ""
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the ViChar array you specify for the  Value parameter.  
        /// 
        /// If the current value of the attribute, including the terminating NUL byte, contains more bytes that you indicate in this parameter, the function copies Buffer Size - 1 bytes into the buffer, places an ASCII NUL byte at the end of the buffer, and returns the buffer size you must pass to get the entire value.  For example, if the value is "123456" and the Buffer Size is 4, the function places "123" into the buffer and returns 7.
        /// 
        /// If you pass a negative number, the function copies the value to the buffer regardless of the number of bytes in the value.
        /// 
        /// If you pass 0, you can pass VI_NULL for the Value buffer parameter.
        /// 
        /// Default Value: 512
        /// </param>
        /// <param name="Value">
        /// Returns the ViBoolean value
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int QueryViString(string Command, int Buffer_Size, System.Text.StringBuilder Value)
        {
            int pInvokeResult = PInvoke.QueryViString(this._handle, Command, Buffer_Size, Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the Enable, NTransition, and PTransition bits     of the Main, Power, Limit, LMargin, ACP Limit, Frequency, status questionable and operating registers.
        /// 
        /// Note:
        /// 
        /// (1) For detailed description of Status Reporting System see also operating manual.
        /// 
        /// Remote-control command(s):
        /// STATus:PRESet
        /// STATus:QUEStionable:POWer:ENABle
        /// STATus:QUEStionable:LIMit&lt;1|2|3&gt;:ENABle 
        /// STATus:QUEStionable:LMARgin&lt;1|2&gt;:ENABle 
        /// STATus:QUEStionable:SYNC:ENABle 
        /// STATus:QUEStionable:ACPLimit:ENABle 
        /// STATus:QUEStionable:FREQuency:ENABle
        /// STATus:QUEStionable:PNOise:ENABle
        /// STATus:QUEStionable:DIQ:ENABle
        /// STATus:QUEStionable:DCPNoise:ENABle 
        /// STATus:QUEStionable:LPNoise:ENABle
        /// STATus:QUEStionable:POWer:PTRansition
        /// STATus:QUEStionable:LIMit&lt;1|2|3&gt;:PTRansition 
        /// STATus:QUEStionable:LMARgin&lt;1|2&gt;:PTRansition 
        /// STATus:QUEStionable:SYNC:PTRansition 
        /// STATus:QUEStionable:ACPLimit:PTRansition 
        /// STATus:QUEStionable:FREQuency:PTRansition
        /// STATus:QUEStionable:PNOise:PTRansition
        /// STATus:QUEStionable:DIQ:PTRansition
        /// STATus:QUEStionable:DCPNoise:PTRansition
        /// STATus:QUEStionable:LPNoise:PTRansition
        /// STATus:QUEStionable:POWer:NTRansition
        /// STATus:QUEStionable:LIMit&lt;1|2|3&gt;:NTRansition 
        /// STATus:QUEStionable:LMARgin&lt;1|2&gt;:NTRansition 
        /// STATus:QUEStionable:SYNC:NTRansition 
        /// STATus:QUEStionable:ACPLimit:NTRansition 
        /// STATus:QUEStionable:FREQuency:NTRansition 
        /// STATus:QUEStionable:PNOise:NTRansition
        /// STATus:QUEStionable:DIQ:NTRansition
        /// STATus:QUEStionable:DCPNoise:NTRansition
        /// STATus:QUEStionable:LPNoise:NTRansition
        /// STATus:OPERation:PCALibration:ENABle
        /// STATus:OPERation:PCALibration:PTRansition
        /// STATus:OPERation:PCALibration:NTRansition 
        /// STATus:OPERation:ENABle
        /// STATus:OPERation:PTRansition
        /// STATus:OPERation:NTRansition
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Register_Operation">
        /// Specifies the operation to perform.
        /// 
        /// Valid Values:
        /// 0 - Preset Registers (Default Value)
        /// 1 - Set Questionable Register
        /// 2 - Set Operation Register
        /// 3 - Set Power Calibration Register
        /// 
        /// Notes:
        /// 
        /// (1) Preset Registers:
        /// This command resets the edge detectors and ENABle parts of all registers to a defined value. All PTRansition parts are set to FFFFh, i.e., all transitions from 0 to 1 are detected. All NTRansition parts are set to 0, i.e., a transition from 1 to 0 in a CONDition bit is not detected. The ENABle part of the STATus:OPERation and STATus:QUEStionable registers are set to 0, i.e., all events in these registers are not passed on.
        /// 
        /// (2) Set Power Calibration Register:
        /// This selection sets the ENABle part, PTRansition part and NTRansition Part of the STATus:OPERation:PCALibration register.
        /// This selection is available only for FSMR.
        /// 
        /// </param>
        /// <param name="Questionable_Register">
        /// This control selects the registers that will be configured. 
        /// 
        /// Valid Values:
        /// 0 - Main(Default Value)
        /// 1 - Power
        /// 2 - Limit Screen A
        /// 3 - LMargin Screen A
        /// 4 - Sync
        /// 5 - ACP Limit
        /// 6 - Frequency
        /// 7 - Limit Screen B
        /// 8 - LMargin Screen B
        /// 9 - Phase Noise
        /// 10 - DIQ
        /// 11 - DC and Phase Noise
        /// 12 - Low Phase Noise
        /// 13 - Limit Screen C
        /// 
        /// Note:
        /// 
        /// (1) Phase Noise - this selection is available only in conjunction with FS-K40.
        /// 
        /// (2) DC and Phase Noise, Low Phase Noise - these selections are available only in conjunction with R&amp;S FSUP instrument.
        /// 
        /// (3) Limit Screen C - this selection is available only in conjuction with FS-K110 (TETRA II).
        /// </param>
        /// <param name="Enable">
        /// This control sets the enable bits of the status register.  
        /// 
        /// Valid range: 0 to 65535
        /// 
        /// Default Value: 0
        /// 
        /// Note:
        /// 
        /// (1) The ENABle register selectively enables the individual events of the associated EVENt section for the sum bit in the status byte.
        /// 
        /// </param>
        /// <param name="PTransition">
        /// This control sets the PTransition bits (edge detectors) of the status register.  
        /// 
        /// Valid range: 0 to 65535
        /// 
        /// Default Value: 0
        /// 
        /// </param>
        /// <param name="NTransition">
        /// This control sets the NTransition bits (edge detectors) of the status register.  
        /// 
        /// Valid range: 0 to 65535
        /// 
        /// Default Value: 0
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int setStatusRegister(int Register_Operation, int Questionable_Register, int Enable, int PTransition, int NTransition)
        {
            int pInvokeResult = PInvoke.setStatusRegister(this._handle, Register_Operation, Questionable_Register, Enable, PTransition, NTransition);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function contains the commands for the querying status reporting system.
        /// 
        /// Note:
        /// 
        /// For detailed description of Status Reporting System see also operating manual.
        /// 
        /// Remote-control command(s):
        /// STATus:PRESet
        /// STATus:OPERation:EVENt?
        /// STATus:OPERation:CONDition?
        /// STATus:QUEStionable:EVENt?
        /// STATus:QUEStionable:CONDition?
        /// STATus:QUEStionable:POWer:EVENt? 
        /// STATus:QUEStionable:POWer:CONDition? 
        /// STATus:QUEStionable:LIMit&lt;1|2|3&gt;:EVENt?
        /// STATus:QUEStionable:LIMit&lt;1|2|3&gt;:CONDition? 
        /// STATus:QUEStionable:SYNC:EVENt? 
        /// STATus:QUEStionable:SYNC:CONDition? 
        /// STATus:QUEStionable:ACPLimit:EVENt? 
        /// STATus:QUEStionable:ACPLimit:CONDition? 
        /// STATus:QUEStionable:FREQuency:EVENt? 
        /// STATus:QUEStionable:FREQuency:CONDition? 
        /// STATus:QUEStionable:TRANsducer:EVENt? 
        /// STATus:QUEStionable:TRANsducer:CONDition? 
        /// STATus:QUEStionable?
        /// STATus:QUEStionable:LMARgin&lt;1|2&gt;:EVENt? 
        /// STATus:QUEStionable:LMARgin&lt;1|2&gt;:CONDition? 
        /// STATus:QUEStionable:PNOise:EVENt?
        /// STATus:QUEStionable:PNOise:CONDition?
        /// STATus:QUEStionable:DIQ:EVENt?
        /// STATus:QUEStionable:DIQ:CONDition?
        /// STATus:QUEStionable:DCPNoise[:EVENt]?
        /// STATus:QUEStionable:DCPNoise:CONDition?
        /// STATus:QUEStionable:LPNoise[:EVENt]?
        /// STATus:QUEStionable:LPNoise:CONDition?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Status_Registers_Query">
        /// This control selects the registers to be queried.
        /// 
        /// Valid Values:
        /// 0 - Operation Event (Default Value)
        /// 1 - Operation Condition
        /// 2 - Main Event
        /// 3 - Main Condition
        /// 4 - Power Event
        /// 5 - Power Condition
        /// 6 - Limit Event Screen A
        /// 7 - Limit Condition Screen A
        /// 8 - LMargin Event Screen A
        /// 9 - LMargin Condition Screen A
        /// 10 - Sync Event
        /// 11 - Sync Condition
        /// 12 - ACP Limit Event
        /// 13 - ACP Limit Condition
        /// 14 - Frequency Event
        /// 15 - Frequency Condition
        /// 16 - RESERVED
        /// 17 - RESERVED
        /// 18 - Queue Next
        /// 19 - Limit Event Screen B
        /// 20 - Limit Condition Screen B
        /// 21 - LMargin Event Screen B
        /// 22 - LMargin Condition Screen B
        /// 23 - Phase Noise Event
        /// 24 - Phase Noise Condition
        /// 25 - Power Calibration Event
        /// 26 - Power Calibration Condition
        /// 27 - DIQ Event
        /// 28 - DIQ Condition
        /// 29 - DC and Phase Noise Event
        /// 30 - DC and Phase Noise Condition
        /// 31 - Low Phase Noise Event
        /// 32 - Low Phase Noise Condition
        /// 33 - Limit Event Screen C
        /// 34 - Limit Condition Screen C
        /// 
        /// Notes:
        /// 
        /// (1) Operation Event:
        /// This command queries the contents of the EVENt section of the STATus:OPERation register.
        /// 
        /// (2) Operation Condition:
        /// This command queries the CONDition section of the STATus:OPERation register.
        /// 
        /// (3) Main Event:
        /// This command queries the contents of the EVENt section of the STATus:QUEStionable register.
        /// 
        /// (4) Main Condition:
        /// This command queries the CONDition section of the STATus:QUEStionable register.
        /// 
        /// (5) Power Event:
        /// This command queries the contents of the EVENt section of the STATus:QUEStionable:POWer register.
        /// 
        /// (6) Power Condition:
        /// This command queries the content of the CONDition section of the STATus:QUEStionable:POWer register.
        /// 
        /// (7) Limit Event Screen A(B,C):
        /// This command queries the contents of the EVENt section of the STATus:QUEStionable:LIMit register. Limit Screen C selection is available only in conjunction with FS-K110 (TETRA II).
        /// 
        /// (8) Limit Condition Screen A(B,C):
        /// This command queries the contents of the CONDition section of the STATus:QUEStionable:LIMit register. Limit Screen C selection is available only in conjunction with FS-K110 (TETRA II).
        /// 
        /// (9) LMargin Event Screen A(B):
        /// This command queries the contents of the EVENt section of the STATus:QUEStionable:LMARgin register.
        /// 
        /// (10) LMargin Condition Screen A(B):
        /// This command queries the contents of the CONDition section of the STATus:QUEStionable:LMARgin register.
        /// 
        /// (11) ACP Limit Event:
        /// This command queries the contents of the EVENt section of the STATus:QUEStionable:ACPLimit register.
        /// 
        /// (12) ACP Limit Condition:
        /// This command queries the contents of the CONDition section of the STATus:QUEStionable: ACPLimit register.
        /// 
        /// (13) Frequency Event:
        /// This command queries the contents of the EVENt section of the STATus:QUEStionableFREQuency register.
        /// 
        /// (14) Frequency Condition:
        /// This command queries the contents of the CONDition section of the STATus:QUEStionable:FREQuency register.
        /// 
        /// (15) Queue Next:
        /// This command queries the earliest entry to the error queue, thus deleting it.
        /// 
        /// (16) Sync Event:
        /// This command queries the contents of the EVENt section of the STATus:QUEStionable:SYNC register. 
        /// 
        /// (17) Sync Condition:
        /// This command queries the contents of the CONDition section of the STATus:QUEStionable:SYNC register. 
        /// 
        /// (18) Phase Noise Event:
        /// This command queries the contents of the EVENt section of the STATus:QUEStionable:PNOIse Register. Readout deletes the contents of the EVENt section. This selection is available only in conjunction with FS-K40.
        /// 
        /// (19) Phase Noise Condition:
        /// This command queries the contents of the CONDition section of the STATus:QUEStionable:PNOIse register. Readout does not delete the contents of the CONDition section. This selection is available only in conjunction with FS-K40.
        /// 
        /// (20) Power Caibration Event:
        /// This command queries the contents of the EVENt section of the STATus:OPERation:PCALibration Register. Readout deletes the contents of the EVENt section. This selection is available only in conjunction with FSMR.
        /// 
        /// (21) Power Calibration Condition:
        /// This command queries the contents of the CONDition section of the STATus:OPERation:PCALibration register. Readout does not delete the contents of the CONDition section. This selection is available only in conjunction with FSMR.
        /// 
        /// (22) DIQ Event:
        /// This command queries the contents of the EVENt section of the STATus:QUEStionable:DIQ register. 
        /// 
        /// (23) DIQ Condition:
        /// This command queries the contents of the CONDition section of the STATus:QUEStionable:DIQ register.
        /// 
        /// (24) DC and Phase Noise Event:
        /// This command queries the contents of the EVENt section of the STATus:QUEStionable:DCPNoise register. Readout deletes the contents of the EVENt section. This selection is available only in conjunction with FSUP.
        /// 
        /// (25) DC and Phase Noise Condition: 
        /// This command queries the contents of the CONDition section of the STATus:QUEStionable:DCPNoise register. Readout does not delete the contents of the CONDition section. This selection is available only in conjunction with FSUP.
        /// 
        /// (26) Low Phase Noise Event:
        /// This command queries the contents of the EVENt section of the STATus:QUEStionable:LPNoise register. Readout deletes the contents of the EVENt section. This selection is available only in conjunction with FSUP. This selection is available only in conjunction with FS-B60.
        /// 
        /// (27) Low Phase Noise Condition:
        /// This command queries the contents of the CONDition section of the STATus:QUEStionable:LPNoise register. Readout does not delete the contents of the CONDition section. This selection is available only in conjunction with FSUP. This selection is available only in conjunction with FS-B60. 
        /// </param>
        /// <param name="Register_Value">
        /// Returns content of selected register.
        /// 
        /// Note:
        /// 
        /// For detailed description of Status Reporting System see also operating manual.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int getStatusRegister(int Status_Registers_Query, out int Register_Value)
        {
            int pInvokeResult = PInvoke.getStatusRegister(this._handle, Status_Registers_Query, out Register_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads Condition Frequency Register.
        /// 
        /// Remote-control command(s):
        /// STATus:QUEStionable:FREQuency:CONDition? 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Status_Register_Value">
        /// This control returns Condition Frequency Register value.
        /// 
        /// Valid Values:
        /// bit 0: OVEN COLD
        /// bit 1: LO UNLocked (Screen A)
        /// bit 2: not used
        /// bit 3: not used
        /// bit 4: not used
        /// bit 5: not used
        /// bit 6: not used
        /// bit 7: not used
        /// bit 8: not used
        /// bit 9: LO UNLocked (Screen B)
        /// bit 10: not used
        /// bit 11: not used
        /// bit 12: not used
        /// bit 13: not used
        /// bit 14: not used
        /// bit 15: This bit is always 0.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConditionFrequencyRegister(out int Status_Register_Value)
        {
            int pInvokeResult = PInvoke.ConditionFrequencyRegister(this._handle, out Status_Register_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads Condition Limit Register.
        /// 
        /// Remote-control command(s):
        /// STATus:QUEStionable:LIMit&lt;1|2&gt;:CONDition? 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Status_Register_Value">
        /// This control returns Condition Limit Register value. 
        /// 
        /// Valid Values:
        /// bit 0: LIMit 1 FAIL
        /// bit 1: LIMit 2 FAIL
        /// bit 2: LIMit 3 FAIL
        /// bit 3: LIMit 4 FAIL
        /// bit 4: LIMit 5 FAIL
        /// bit 5: LIMit 6 FAIL
        /// bit 6: LIMit 7 FAIL
        /// bit 7: LIMit 8 FAIL
        /// bit 8: not used
        /// bit 9: not used
        /// bit 10: not used
        /// bit 11: not used
        /// bit 12: not used
        /// bit 13: not used
        /// bit 14: not used
        /// bit 15: This bit is always 0.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConditionLimitRegister(int Window, out int Status_Register_Value)
        {
            int pInvokeResult = PInvoke.ConditionLimitRegister(this._handle, Window, out Status_Register_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads Condition Limit Margin Register.
        /// 
        /// Remote-control command(s):
        /// STATus:QUEStionable:LMARgin&lt;1|2&gt;:CONDition? 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Window">
        /// This control selects the measurement window.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Status_Register_Value">
        /// This control returns Condition Limit Margin Register value.
        /// 
        /// Valid Values:
        /// bit 0: LMARgin 1 FAIL
        /// bit 1: LMARgin 2 FAIL
        /// bit 2: LMARgin 3 FAIL
        /// bit 3: LMARgin 4 FAIL
        /// bit 4: LMARgin 5 FAIL
        /// bit 5: LMARgin 6 FAIL
        /// bit 6: LMARgin 7 FAIL
        /// bit 7: LMARgin 8 FAIL
        /// bit 8: not used
        /// bit 9: not used
        /// bit 10: not used
        /// bit 11: not used
        /// bit 12: not used
        /// bit 13: not used
        /// bit 14: not used
        /// bit 15: This bit is always 0.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConditionLimitMarginRegister(int Window, out int Status_Register_Value)
        {
            int pInvokeResult = PInvoke.ConditionLimitMarginRegister(this._handle, Window, out Status_Register_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads Condition ACP Limit Register.
        /// 
        /// Remote-control command(s):
        /// STATus:QUEStionable:ACPLimit:CONDition? 
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Status_Register_Value">
        /// This control returns Condition ACP Limit Register value.
        /// 
        /// Valid Values:
        /// bit 0: ADJ UPPer FAIL(Screen A) 
        /// bit 1: ADJ LOWer FAIL (Screen A)
        /// bit 2: ALT1 UPPer FAIL (Screen A).
        /// bit 3: ALT1 LOWer FAIL (Screen A)
        /// bit 4: ALT2 UPPer FAIL (Screen A)
        /// bit 5: ALT2 LOWer FAIL (Screen A)
        /// bit 6: not used
        /// bit 7: not used
        /// bit 8: ADJ UPPer FAIL (Screen B)
        /// bit 9: ADJ LOWer FAIL (Screen B)
        /// bit 10: ALT1 UPPer FAIL (Screen B)
        /// bit 11: ALT1 LOWer FAIL (Screen B)
        /// bit 12: ALT2 UPPer FAIL (Screen B)
        /// bit 13: ALT2 LOWer FAIL (Screen B
        /// bit 14: not used
        /// bit 15: This bit is always set to 0.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConditionACPLimitRegister(out int Status_Register_Value)
        {
            int pInvokeResult = PInvoke.ConditionACPLimitRegister(this._handle, out Status_Register_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads Condition Power Register.
        /// 
        /// Remote-control command(s):
        /// STATus:QUEStionable:POWer:CONDition?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Status_Register_Value">
        /// This control returns Condition Power Register value.
        /// 
        /// Valid Values:
        /// bit 0: OVERload (Screen A)
        /// bit 1: UNDerload (Screen A) 
        /// bit 2: IF_OVerload (Screen A)
        /// bit 3: not used
        /// bit 4: not used
        /// bit 5: not used
        /// bit 6: not used
        /// bit 7: not used
        /// bit 8: OVERload (Screen B)
        /// bit 9: UNDerload (Screen B) 
        /// bit 10: IF_OVerload (Screen B)
        /// bit 11: not used
        /// bit 12: not used
        /// bit 13: not used
        /// bit 14: not used
        /// bit 15: This bit is always 0.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConditionPowerRegister(out int Status_Register_Value)
        {
            int pInvokeResult = PInvoke.ConditionPowerRegister(this._handle, out Status_Register_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads Condition Sync Register.
        /// 
        /// Remote-control command(s):
        /// STATus:QUEStionable:SYNC:CONDition?
        /// </summary>
        /// <param name="Instrument_Handle">
        /// This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
        /// 
        /// Default Value:  None
        /// </param>
        /// <param name="Status_Register_Value">
        /// This control displays Condition Sync Register.
        /// 
        /// Valid Values:
        /// 
        /// --- K5 ---
        /// 
        /// bit 0: BURSt not found (screen A)
        /// bit 1: SYNC not found (screen A)
        /// bit 2: No carrier (screen A)
        /// bit 3: Carrier overload (screen A)
        /// bit 4: Not used
        /// bit 5: Not used
        /// bit 6: Not used 
        /// bit 7: Not used 
        /// bit 8: Not used
        /// bit 9: Not used
        /// bit 10:Not used
        /// bit 11:Not used
        /// bit 12:Not used
        /// bit 13:Not used
        /// bit 14:Not used
        /// bit 15:This bit is always 0
        /// 
        /// --- K8 ---
        /// 
        /// bit 0: BURSt not found (screen A)
        /// bit 1: SYNC not found (screen A)
        /// bit 2: Not used 
        /// bit 3: Not used
        /// bit 4: Not used
        /// bit 5: Not used
        /// bit 6: Not used 
        /// bit 7: Not used 
        /// bit 8: Not used
        /// bit 9: Not used
        /// bit 10:Not used
        /// bit 11:Not used
        /// bit 12:Not used
        /// bit 13:Not used
        /// bit 14:Not used
        /// bit 15:This bit is always 0
        /// 
        /// --- K70 ---
        /// 
        /// bit 0: BURSt not found (screen A)
        /// bit 1: SYNC not found (screen A)
        /// bit 2: Not used 
        /// bit 3: Not used
        /// bit 4: Not used
        /// bit 5: Not used
        /// bit 6: DEMod failed
        /// bit 7: End of buffer reached (FW 3.50 and higher)
        /// bit 8: Not used
        /// bit 9: Not used
        /// bit 10:Not used
        /// bit 11:Not used
        /// bit 12:Not used
        /// bit 13:Not used
        /// bit 14:Not used
        /// bit 15:This bit is always 0
        /// 
        /// --- K72/K73/K74 ---
        /// 
        /// bit 0: Not used 
        /// bit 1: K72/K74 Frame Sync failed
        /// bit 2: Not used
        /// bit 3: Not used
        /// bit 4: Not used
        /// bit 5: K72/K74 Incorrect Pilot Symbol 
        /// bit 6: Not used 
        /// bit 7: Not used 
        /// bit 8: Not used
        /// bit 9: Not used
        /// bit 10:Not used
        /// bit 11:Not used
        /// bit 12:Not used
        /// bit 13:Not used
        /// bit 14:Not used
        /// bit 15:This bit is always 0
        /// 
        /// --- K76/K82/K83 ---
        /// 
        /// bit 0: Not used 
        /// bit 1: K76 Frame Sync failed
        /// bit 2: Not used
        /// bit 3: Not used
        /// bit 4: Not used
        /// bit 5: Not used
        /// bit 6: Not used
        /// bit 7: Not used
        /// bit 8: Not used
        /// bit 9: Not used
        /// bit 10:Not used
        /// bit 11:Not used
        /// bit 12:Not used
        /// bit 13:Not used
        /// bit 14:Not used
        /// bit 15:This bit is always 0
        /// 
        /// --- K84 ---
        /// 
        /// bit 0: Not used in the FS-K84 application
        /// bit 1: K84 Frame Sync failed
        /// bit 2: Not used in the FS-K84 application
        /// bit 3: Not used in the FS-K84 application
        /// bit 4: K84 Preamble Current Slot missing
        /// bit 5: K84 Preamble Overall missing
        /// bit 6: Not used in the FS-K84 application
        /// bit 7: Not used in the FS-K84 application
        /// bit 8: Not used in the FS-K84 application
        /// bit 9: Not used in the FS-K84 application
        /// bit 10:Not used in the FS-K84 application
        /// bit 11:Not used in the FS-K84 application
        /// bit 12:Not used in the FS-K84 application
        /// bit 13:Not used in the FS-K84 application
        /// bit 14:Not used in the FS-K84 application
        /// bit 15:This bit is always 0
        /// 
        /// --- K91 ---
        /// 
        /// bit 0: BURSt not found
        /// bit 1: Not used in the FS-K91 application
        /// bit 2: no bursts of REQuired type
        /// bit 3: GATE length too small
        /// bit 4: BURSt count too small
        /// bit 5: auto level OVERload
        /// bit 6: auto level NoSIGnal
        /// bit 7: Not used in the FS-K91 application
        /// bit 8: Not used in the FS-K91 application
        /// bit 9: Not used in the FS-K91 application
        /// bit 10:Not used in the FS-K91 application
        /// bit 11:Not used in the FS-K91 application
        /// bit 12:Not used in the FS-K91 application
        /// bit 13:Not used in the FS-K91 application
        /// bit 14:Not used in the FS-K91 application
        /// bit 15:This bit is always 0
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ConditionSyncRegister(out int Status_Register_Value)
        {
            int pInvokeResult = PInvoke.ConditionSyncRegister(this._handle, out Status_Register_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        public void Dispose()
        {
            this.Dispose(true);
            System.GC.SuppressFinalize(this);
        }

        private void Dispose(bool disposing)
        {
            if ((this._disposed == false))
            {
                PInvoke.close(this._handle);
                this._handle = System.IntPtr.Zero;
            }
            this._disposed = true;
        }

        public void SetInt32(rsspecanProperties propertyId, string repeatedCapabilityOrChannel, int val)
        {
            PInvoke.TestForError(this._handle, PInvoke.SetAttributeViInt32(this._handle, repeatedCapabilityOrChannel, ((uint)(propertyId)), val));
        }

        public void SetInt32(rsspecanProperties propertyId, int val)
        {
            this.SetInt32(propertyId, "", val);
        }

        public int GetInt32(rsspecanProperties propertyId, string repeatedCapabilityOrChannel)
        {
            int val;
            PInvoke.TestForError(this._handle, PInvoke.GetAttributeViInt32(this._handle, repeatedCapabilityOrChannel, ((uint)(propertyId)), out val));
            return val;
        }

        public int GetInt32(rsspecanProperties propertyId)
        {
            return this.GetInt32(propertyId, "");
        }

        public void SetDouble(rsspecanProperties propertyId, string repeatedCapabilityOrChannel, double val)
        {
            PInvoke.TestForError(this._handle, PInvoke.SetAttributeViReal64(this._handle, repeatedCapabilityOrChannel, ((uint)(propertyId)), val));
        }

        public void SetDouble(rsspecanProperties propertyId, double val)
        {
            this.SetDouble(propertyId, "", val);
        }

        public double GetDouble(rsspecanProperties propertyId, string repeatedCapabilityOrChannel)
        {
            double val;
            PInvoke.TestForError(this._handle, PInvoke.GetAttributeViReal64(this._handle, repeatedCapabilityOrChannel, ((uint)(propertyId)), out val));
            return val;
        }

        public double GetDouble(rsspecanProperties propertyId)
        {
            return this.GetDouble(propertyId, "");
        }

        public void SetBoolean(rsspecanProperties propertyId, string repeatedCapabilityOrChannel, bool val)
        {
            PInvoke.TestForError(this._handle, PInvoke.SetAttributeViBoolean(this._handle, repeatedCapabilityOrChannel, ((uint)(propertyId)), System.Convert.ToUInt16(val)));
        }

        public void SetBoolean(rsspecanProperties propertyId, bool val)
        {
            this.SetBoolean(propertyId, "", val);
        }

        public bool GetBoolean(rsspecanProperties propertyId, string repeatedCapabilityOrChannel)
        {
            ushort val;
            PInvoke.TestForError(this._handle, PInvoke.GetAttributeViBoolean(this._handle, repeatedCapabilityOrChannel, ((uint)(propertyId)), out val));
            return System.Convert.ToBoolean(val);
        }

        public bool GetBoolean(rsspecanProperties propertyId)
        {
            return this.GetBoolean(propertyId, "");
        }

        public void SetString(rsspecanProperties propertyId, string repeatedCapabilityOrChannel, string val)
        {
            PInvoke.TestForError(this._handle, PInvoke.SetAttributeViString(this._handle, repeatedCapabilityOrChannel, ((uint)(propertyId)), val));
        }

        public void SetString(rsspecanProperties propertyId, string val)
        {
            this.SetString(propertyId, "", val);
        }

        public string GetString(rsspecanProperties propertyId, string repeatedCapabilityOrChannel)
        {
            System.Text.StringBuilder newVal = new System.Text.StringBuilder(512);
            int size = PInvoke.GetAttributeViString(this._handle, repeatedCapabilityOrChannel, ((uint)(propertyId)), 512, newVal);
            if ((size < 0))
            {
                PInvoke.ThrowError(this._handle, size);
            }
            else
            {
                if ((size > 0))
                {
                    newVal.Capacity = size;
                    PInvoke.TestForError(this._handle, PInvoke.GetAttributeViString(this._handle, repeatedCapabilityOrChannel, ((uint)(propertyId)), size, newVal));
                }
            }
            return newVal.ToString();
        }

        public string GetString(rsspecanProperties propertyId)
        {
            return this.GetString(propertyId, "");
        }

        public void SetSession(rsspecanProperties propertyId, string repeatedCapabilityOrChannel, System.IntPtr val)
        {
            PInvoke.TestForError(this._handle, PInvoke.SetAttributeViSession(this._handle, repeatedCapabilityOrChannel, ((uint)(propertyId)), val));
        }

        public void SetSession(rsspecanProperties propertyId, System.IntPtr val)
        {
            this.SetSession(propertyId, "", val);
        }

        public System.IntPtr GetSession(rsspecanProperties propertyId, string repeatedCapabilityOrChannel)
        {
            System.IntPtr val;
            PInvoke.TestForError(this._handle, PInvoke.GetAttributeViSession(this._handle, repeatedCapabilityOrChannel, ((uint)(propertyId)), out val));
            return val;
        }

        public System.IntPtr GetSession(rsspecanProperties propertyId)
        {
            return this.GetSession(propertyId, "");
        }

        private class PInvoke
        {

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_init", CallingConvention = CallingConvention.StdCall)]
            public static extern int init(string Resource_Name, ushort ID_Query, ushort Reset_Device, out System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_InitWithOptions", CallingConvention = CallingConvention.StdCall)]
            public static extern int InitWithOptions(string Resource_Name, ushort ID_Query, ushort Reset_Device, string Option_String, out System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SetSANMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSANMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFrequencyStartStop", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFrequencyStartStop(System.IntPtr Instrument_Handle, int Window, double Start_Frequency, double Stop_Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFrequencyCenter", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFrequencyCenter(System.IntPtr Instrument_Handle, int Window, double Center_Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFrequencyCenterSpan", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFrequencyCenterSpan(System.IntPtr Instrument_Handle, int Window, double Center_Frequency, double Span);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFrequencyStepSize", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFrequencyStepSize(System.IntPtr Instrument_Handle, int Window, double Step_Size);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFrequencyStepSizeAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFrequencyStepSizeAuto(System.IntPtr Instrument_Handle, int Window, ushort Step_Size_Auto);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFrequencyCouplingFactor", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFrequencyCouplingFactor(System.IntPtr Instrument_Handle, int Window, int Coupling, int Coupling_Factor);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFrequencySpanFull", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFrequencySpanFull(System.IntPtr Instrument_Handle, int Window);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFrequencyZeroSpan", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFrequencyZeroSpan(System.IntPtr Instrument_Handle, int Window);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAutoSpanMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAutoSpanMode(System.IntPtr Instrument_Handle, ushort Span_Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFrequencyOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFrequencyOffset(System.IntPtr Instrument_Handle, int Window, double Frequency_Offset);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFrequencyMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFrequencyMode(System.IntPtr Instrument_Handle, int Window, int Frequency_Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFrequencyInputMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFrequencyInputMode(System.IntPtr Instrument_Handle, int Input_Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureChannelTable", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureChannelTable(System.IntPtr Instrument_Handle, string Channel_Table);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSweepTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSweepTime(System.IntPtr Instrument_Handle, int Window, ushort Sweep_Time_Auto, double Sweep_Time);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSweepCouplingAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSweepCouplingAuto(System.IntPtr Instrument_Handle, int Window, int Sweep_Coupling);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSweepCoupling", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSweepCoupling(System.IntPtr Instrument_Handle, int Window, int Sweep_Coupling, double Coupling_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSweepCouplingAdvanced", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSweepCouplingAdvanced(System.IntPtr Instrument_Handle, int Window, int Ratio_Selection, double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMainPLLBwid", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMainPLLBwid(System.IntPtr Instrument_Handle, int Window, int PLL);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFilterType", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFilterType(System.IntPtr Instrument_Handle, int Window, int Filter_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVBWMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVBWMode(System.IntPtr Instrument_Handle, int Window, int Video_Filter_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFFTFilterMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFFTFilterMode(System.IntPtr Instrument_Handle, int Window, int FFT_Filter_Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAveraging", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAveraging(System.IntPtr Instrument_Handle, int Window, int Trace, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAveragingType", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAveragingType(System.IntPtr Instrument_Handle, int Window, int Averaging_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAveragingCount", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAveragingCount(System.IntPtr Instrument_Handle, int Window, int Count);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAcquisition", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAcquisition(System.IntPtr Instrument_Handle, int Window, ushort Sweep_Mode_Continuous, int Number_Of_Sweeps);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVerticalScale", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVerticalScale(System.IntPtr Instrument_Handle, int Window, int Vertical_Scale);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureHorizontalScale", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureHorizontalScale(System.IntPtr Instrument_Handle, int Window, int Horizontal_Scale);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSweepPoints", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSweepPoints(System.IntPtr Instrument_Handle, int Window, int Sweep_Points);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QuerySweepPoints", CallingConvention = CallingConvention.StdCall)]
            public static extern int QuerySweepPoints(System.IntPtr Instrument_Handle, int Window, out int Sweep_Points);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSweepType", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSweepType(System.IntPtr Instrument_Handle, int Sweep_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAttenuation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAttenuation(System.IntPtr Instrument_Handle, int Window, ushort Attenuation_Auto, double Attenuation);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReferenceLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReferenceLevel(System.IntPtr Instrument_Handle, int Window, double Reference_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReferenceLevelOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReferenceLevelOffset(System.IntPtr Instrument_Handle, int Window, double Reference_Level_Offset);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLevel(System.IntPtr Instrument_Handle, int Window, int Amplitude_Units, double Input_Impedance, double Reference_Level, double Reference_Level_Offset, ushort Attenuation_Auto, double Attenuation);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAttenuationMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAttenuationMode(System.IntPtr Instrument_Handle, int Window, int Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureParameterCoupling", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureParameterCoupling(System.IntPtr Instrument_Handle, int Parameter_Coupling);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReferenceLevelCoupling", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReferenceLevelCoupling(System.IntPtr Instrument_Handle, int Window, ushort Coupled_Reference_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureIFOutputSource", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureIFOutputSource(System.IntPtr Instrument_Handle, int Window, int Output_Source);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureUnitPower", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureUnitPower(System.IntPtr Instrument_Handle, int Window, int Unit_Power);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureUnitLength", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureUnitLength(System.IntPtr Instrument_Handle, int Unit_Length);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetChannelNumber", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetChannelNumber(System.IntPtr Instrument_Handle, out int Number_Of_Channel);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTrace(System.IntPtr Instrument_Handle, int Window, int Trace, int Trace_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTraceDetector", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTraceDetector(System.IntPtr Instrument_Handle, int Window, int Trace, ushort Detector_Type_Auto, int Detector_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTraceResetBehavior", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTraceResetBehavior(System.IntPtr Instrument_Handle, int Window, int Trace, ushort Reset_at_Change);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_StoreTraceToFile", CallingConvention = CallingConvention.StdCall)]
            public static extern int StoreTraceToFile(System.IntPtr Instrument_Handle, int Window, int Trace, string File_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CopyTraceToMemory", CallingConvention = CallingConvention.StdCall)]
            public static extern int CopyTraceToMemory(System.IntPtr Instrument_Handle, int Trace);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMemoryTraceState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMemoryTraceState(System.IntPtr Instrument_Handle, int Trace, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_TraceIQSet", CallingConvention = CallingConvention.StdCall)]
            public static extern int TraceIQSet(System.IntPtr Instrument_Handle, int Window, double Sampling_Rate, int Trigger_Mode, int Trigger_Slope, double Bandwidth, int Pretrigger_Samples, int No_of_Samples);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_TraceIQSamplingRate", CallingConvention = CallingConvention.StdCall)]
            public static extern int TraceIQSamplingRate(System.IntPtr Instrument_Handle, int Window, double Sampling_Rate);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_TraceIQAveraging", CallingConvention = CallingConvention.StdCall)]
            public static extern int TraceIQAveraging(System.IntPtr Instrument_Handle, int Window, ushort Average_State, int Average_Count);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_TraceIQExtendedBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int TraceIQExtendedBandwidth(System.IntPtr Instrument_Handle, int Window, int Filter);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTraceIQDataAcquisition", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTraceIQDataAcquisition(System.IntPtr Instrument_Handle, int Window, ushort Trace_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTraceIQStartSynchronization", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTraceIQStartSynchronization(System.IntPtr Instrument_Handle, int Window, ushort Start_Synchronization);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTraceIQBWExtension", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTraceIQBWExtension(System.IntPtr Instrument_Handle, int Window, ushort BW_Extension_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTraceIQGate", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTraceIQGate(System.IntPtr Instrument_Handle, ushort Gate, int Type, int Gate_Length, int Gate_Gap, int Number_Of_Gate_Periods);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTraceIQDataDisplayType", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTraceIQDataDisplayType(System.IntPtr Instrument_Handle, int Display_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryTraceIQFilterBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryTraceIQFilterBandwidth(System.IntPtr Instrument_Handle, out double Filter_Bandwidth);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CopyTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int CopyTrace(System.IntPtr Instrument_Handle, int Window, int Destination_Trace, int Source_Trace);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSubtractTraces", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSubtractTraces(System.IntPtr Instrument_Handle, int Window, double Position, int Mode, ushort Trace_Math_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SubtractTraces", CallingConvention = CallingConvention.StdCall)]
            public static extern int SubtractTraces(System.IntPtr Instrument_Handle, int Window, int Trace_2);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMarker", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMarker(System.IntPtr Instrument_Handle, int Window, int Marker, ushort Marker_Enabled, int Trace);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSignalTrack", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSignalTrack(System.IntPtr Instrument_Handle, int Window, ushort Signal_Track_Enabled, double Signal_Track_Bandwidth, double Signal_Track_Threshold, int Trace);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_DisableAllMarkers", CallingConvention = CallingConvention.StdCall)]
            public static extern int DisableAllMarkers(System.IntPtr Instrument_Handle, int Window);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_MarkerSearch", CallingConvention = CallingConvention.StdCall)]
            public static extern int MarkerSearch(System.IntPtr Instrument_Handle, int Window, int Marker, int Marker_Search);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_MarkerPeakAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int MarkerPeakAuto(System.IntPtr Instrument_Handle, int Window, int Marker_Search, ushort Auto_Peak);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_MoveMarker", CallingConvention = CallingConvention.StdCall)]
            public static extern int MoveMarker(System.IntPtr Instrument_Handle, int Window, int Marker, double Marker_Position);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SetInstrumentFromMarker", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetInstrumentFromMarker(System.IntPtr Instrument_Handle, int Window, int Marker, int Instrument_Setting);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMarkerZoomState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMarkerZoomState(System.IntPtr Instrument_Handle, ushort Marker_Zoom_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMarkerZoom", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMarkerZoom(System.IntPtr Instrument_Handle, int Window, double Marker_Zoom, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMarkerZoomPortable", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMarkerZoomPortable(System.IntPtr Instrument_Handle, ushort Zoom, int Zoom_Factor, double X);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_LinkMarkerAndDeltamarker", CallingConvention = CallingConvention.StdCall)]
            public static extern int LinkMarkerAndDeltamarker(System.IntPtr Instrument_Handle, int Window, int Marker, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryMarker", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryMarker(System.IntPtr Instrument_Handle, int Window, int Marker, out double Marker_Position, out double Marker_Amplitude);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMarkerStepSize", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMarkerStepSize(System.IntPtr Instrument_Handle, int Window, int Step_Size);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ProbabilityMarkerPosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int ProbabilityMarkerPosition(System.IntPtr Instrument_Handle, int Window, int Marker, double Probability);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMarkerDisplayMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMarkerDisplayMode(System.IntPtr Instrument_Handle, int Marker, int Display_Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMarkerReceiverFrequencyCoupling", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMarkerReceiverFrequencyCoupling(System.IntPtr Instrument_Handle, int Window, int Marker, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMarkerSubscanCoupling", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMarkerSubscanCoupling(System.IntPtr Instrument_Handle, int Window, int Marker, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMarkerSearchThreshold", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMarkerSearchThreshold(System.IntPtr Instrument_Handle, int Window, ushort Threshold_State, double Marker_Threshold);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMarkerSearchPeakExcursion", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMarkerSearchPeakExcursion(System.IntPtr Instrument_Handle, int Window, double Peak_Excursion);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMarkerSearchLocalOscillator", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMarkerSearchLocalOscillator(System.IntPtr Instrument_Handle, int Window, ushort Local_Oscillator_Supression);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMarkerSearchLimits", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMarkerSearchLimits(System.IntPtr Instrument_Handle, int Window, ushort Search_Limits, double Search_Limit_Left, double Search_Limit_Right);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMarkerSearchLimitsZoom", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMarkerSearchLimitsZoom(System.IntPtr Instrument_Handle, int Window, ushort Search_Limits_Zoom);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMarkerDemodulation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMarkerDemodulation(System.IntPtr Instrument_Handle, int Window, int Marker, ushort State, int Demodulation_Type, double Marker_Stop_Time, ushort Continuous_Demodulation, ushort Squelch_State, double Squelch_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMarkerPeakList", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMarkerPeakList(System.IntPtr Instrument_Handle, int Window, int Marker, int Peak_List_Count, int Peak_List_Sort);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_StoreMarkerPeakList", CallingConvention = CallingConvention.StdCall)]
            public static extern int StoreMarkerPeakList(System.IntPtr Instrument_Handle, string Path);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryMarkerPeakListFound", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryMarkerPeakListFound(System.IntPtr Instrument_Handle, int Window, int Marker, out int Peak_List_Found);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryMarkerPeakList", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryMarkerPeakList(System.IntPtr Instrument_Handle, int Window, int Marker, int Array_Size, int Peak_List_Selection, [In, Out] double[] Peak_List);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_MarkerSaveToFile", CallingConvention = CallingConvention.StdCall)]
            public static extern int MarkerSaveToFile(System.IntPtr Instrument_Handle, string Path);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDeltaMarker", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDeltaMarker(System.IntPtr Instrument_Handle, int Window, int Delta_Marker, ushort State, int Trace);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDeltaMarkerPosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDeltaMarkerPosition(System.IntPtr Instrument_Handle, int Window, int Delta_Marker, int Mode, double Position);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_DisableAllDeltaMarkers", CallingConvention = CallingConvention.StdCall)]
            public static extern int DisableAllDeltaMarkers(System.IntPtr Instrument_Handle, int Window);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_DeltaMarkerSearch", CallingConvention = CallingConvention.StdCall)]
            public static extern int DeltaMarkerSearch(System.IntPtr Instrument_Handle, int Window, int Delta_Marker, int Marker_Search);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryDeltaMarker", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryDeltaMarker(System.IntPtr Instrument_Handle, int Window, int Delta_Marker, int Mode, out double Position, out double Amplitude);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReferenceFixed", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReferenceFixed(System.IntPtr Instrument_Handle, int Window, ushort Fixed_Reference);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReferenceFixedPoint", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReferenceFixedPoint(System.IntPtr Instrument_Handle, int Window, double Ref_Point_Level, double Ref_Point_Level_Offset, double Ref_Point_Frequency_Time);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReferenceFixedPeakSearch", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReferenceFixedPeakSearch(System.IntPtr Instrument_Handle, int Window);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReferenceOscillator", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReferenceOscillator(System.IntPtr Instrument_Handle, int Source, double External_Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QuerySelectedReferenceOscillatorSource", CallingConvention = CallingConvention.StdCall)]
            public static extern int QuerySelectedReferenceOscillatorSource(System.IntPtr Instrument_Handle, out int Source);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReferenceOscillatorExternalPLLBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReferenceOscillatorExternalPLLBandwidth(System.IntPtr Instrument_Handle, int Bandwidth);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SpeakerVolume", CallingConvention = CallingConvention.StdCall)]
            public static extern int SpeakerVolume(System.IntPtr Instrument_Handle, double Speaker_Volume);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBeeperVolume", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBeeperVolume(System.IntPtr Instrument_Handle, double Beeper_Volume);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureKeyVolume", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureKeyVolume(System.IntPtr Instrument_Handle, double Key_Volume);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBNCMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBNCMode(System.IntPtr Instrument_Handle, int BNC, int BNC_Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureInputImpedancePad", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureInputImpedancePad(System.IntPtr Instrument_Handle, int Pad);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ApplicationSetupRecovery", CallingConvention = CallingConvention.StdCall)]
            public static extern int ApplicationSetupRecovery(System.IntPtr Instrument_Handle, ushort Application_Setup_Recovery);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureIFShift", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureIFShift(System.IntPtr Instrument_Handle, int IF_Shift, double IF_Shift_Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryAutoOffTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryAutoOffTime(System.IntPtr Instrument_Handle, out int Auto_Off_Time);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryPowerSource", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryPowerSource(System.IntPtr Instrument_Handle, System.Text.StringBuilder Power_Source);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTriggerSource", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTriggerSource(System.IntPtr Instrument_Handle, int Window, int Trigger_Source);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTVRFPTriggerSource", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTVRFPTriggerSource(System.IntPtr Instrument_Handle, int Window, int Trigger_Source);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTrigger", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTrigger(System.IntPtr Instrument_Handle, int Window, double Trigger_Delay, int Trigger_Polarity);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureExternalTrigger", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureExternalTrigger(System.IntPtr Instrument_Handle, int Window, double External_Trigger_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDelayCompensation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDelayCompensation(System.IntPtr Instrument_Handle, int Window, ushort Delay_Compensation);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVideoTrigger", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVideoTrigger(System.IntPtr Instrument_Handle, int Window, double Video_Trigger_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureIFPowerTrigger", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureIFPowerTrigger(System.IntPtr Instrument_Handle, int Window, double Trigger_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureIFPowerTriggerParameters", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureIFPowerTriggerParameters(System.IntPtr Instrument_Handle, int Window, double Trigger_Level, double Holdoff, double Hysteresis);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureRFPowerTrigger", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureRFPowerTrigger(System.IntPtr Instrument_Handle, int Window, double Trigger_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTimeTrigger", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTimeTrigger(System.IntPtr Instrument_Handle, int Window, double Time_Interval);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTVTrigger", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTVTrigger(System.IntPtr Instrument_Handle, int Window, int Line_System, int Synchronization, int Horizontal_Sync_Line, int Polarity, ushort TV_Free_Run_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureExternalGate", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureExternalGate(System.IntPtr Instrument_Handle, int Window, ushort Gating, int Gate_Source, int Mode, int Polarity, double Delay, double Length);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureExternalGateTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureExternalGateTrace(System.IntPtr Instrument_Handle, int Trace, int Range, ushort Trace_State, string Comment, double Trace_Start, double Trace_Stop, double Period_Length);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureExternalMixerEnabled", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureExternalMixerEnabled(System.IntPtr Instrument_Handle, ushort External_Mixing_Enabled);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureExternalMixerNumberOfPorts", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureExternalMixerNumberOfPorts(System.IntPtr Instrument_Handle, int Number_of_Ports);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureExternalMixer", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureExternalMixer(System.IntPtr Instrument_Handle, int Band, int Harmonic, int Harmonic_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryExternalMixerRFStartStop", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryExternalMixerRFStartStop(System.IntPtr Instrument_Handle, out double RF_Start, out double RF_Stop);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureExternalMixerHandoverFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureExternalMixerHandoverFrequency(System.IntPtr Instrument_Handle, double Handover_Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureExternalMixerRFOverrange", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureExternalMixerRFOverrange(System.IntPtr Instrument_Handle, ushort RF_Overrange);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureExternalMixerPresetBand", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureExternalMixerPresetBand(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureExternalMixerHighHarmonic", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureExternalMixerHighHarmonic(System.IntPtr Instrument_Handle, ushort High_Harmonic, int Harmonic_Order);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureExternalMixerSignalIDMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureExternalMixerSignalIDMode(System.IntPtr Instrument_Handle, int Mode, double Auto_ID_Threshold);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureExternalMixerBias", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureExternalMixerBias(System.IntPtr Instrument_Handle, int Range, double Bias);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureExternalMixerConversionLoss", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureExternalMixerConversionLoss(System.IntPtr Instrument_Handle, int Range, int Conversion_Loss_Mode, double Conversion_Loss_Average, string Conversion_Loss_Table);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureExternalMixerLowPower", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureExternalMixerLowPower(System.IntPtr Instrument_Handle, double Low_Power);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CreateExternalMixerConversionLossTable", CallingConvention = CallingConvention.StdCall)]
            public static extern int CreateExternalMixerConversionLossTable(System.IntPtr Instrument_Handle, int Window, string Table_Name, string Mixer_Type, int Band, string Serial_Number, int Harmonic, int Ports, double Bias, string Comment, int Number_Of_Values, double[] Frequency_Values, double[] Conversion_Loss_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ExternalMixerConversionLossTableSelect", CallingConvention = CallingConvention.StdCall)]
            public static extern int ExternalMixerConversionLossTableSelect(System.IntPtr Instrument_Handle, int Window, string Table_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ExternalMixerConversionLossTableDelete", CallingConvention = CallingConvention.StdCall)]
            public static extern int ExternalMixerConversionLossTableDelete(System.IntPtr Instrument_Handle, int Window, string Table_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ExternalMixerConversionLossTableCatalog", CallingConvention = CallingConvention.StdCall)]
            public static extern int ExternalMixerConversionLossTableCatalog(System.IntPtr Instrument_Handle, out int Number_of_CVL, int Buffer_Size, System.Text.StringBuilder CVL_Names_List);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureRFInputCoupling", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureRFInputCoupling(System.IntPtr Instrument_Handle, int RF_Input_Coupling);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureElectronicAttenuator", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureElectronicAttenuator(System.IntPtr Instrument_Handle, int Input, int Mode, int Manual_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureInputMixerLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureInputMixerLevel(System.IntPtr Instrument_Handle, int Input, ushort Automatic_Setup, double Power);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePreamplifierState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePreamplifierState(System.IntPtr Instrument_Handle, int Window, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureYIGFilter", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureYIGFilter(System.IntPtr Instrument_Handle, int Window, ushort YIG_Filter);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_InputProtectionReset", CallingConvention = CallingConvention.StdCall)]
            public static extern int InputProtectionReset(System.IntPtr Instrument_Handle, int Window);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CreateLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int CreateLimitLine(System.IntPtr Instrument_Handle, int Window, int Limit, string Name, int Domain, string Comment, int Assign_to_Trace, ushort Delete_Existing_Limit_Line);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLimitLine(System.IntPtr Instrument_Handle, int Limit, int Type, int Units, int X_Axis_Interpolation, int Y_Axis_Interpolation, int X_Axis_Scaling, double Margin, double Threshold);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_DefineLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int DefineLimitLine(System.IntPtr Instrument_Handle, int Limit, int Type, int Count, double[] X_Axis, double[] Amplitude);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SelectLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int SelectLimitLine(System.IntPtr Instrument_Handle, int Window, int Limit, string Name, ushort Limit_Enabled);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SelectLimitLineUpperPortable", CallingConvention = CallingConvention.StdCall)]
            public static extern int SelectLimitLineUpperPortable(System.IntPtr Instrument_Handle, string Limit_Line_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SelectLimitLineLowerPortable", CallingConvention = CallingConvention.StdCall)]
            public static extern int SelectLimitLineLowerPortable(System.IntPtr Instrument_Handle, string Limit_Line_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_EnableLimitCheck", CallingConvention = CallingConvention.StdCall)]
            public static extern int EnableLimitCheck(System.IntPtr Instrument_Handle, int Window, int Limit, int Type, ushort Limit_Enabled, ushort Check_Enabled);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_MoveLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int MoveLimitLine(System.IntPtr Instrument_Handle, int Limit, int Type, int Method, double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CopyLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int CopyLimitLine(System.IntPtr Instrument_Handle, int Limit, int Copy_To);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_DeleteLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int DeleteLimitLine(System.IntPtr Instrument_Handle, int Limit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ClearLimitLineResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int ClearLimitLineResults(System.IntPtr Instrument_Handle, int Window, int Limit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDisplayLineState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDisplayLineState(System.IntPtr Instrument_Handle, int Window, int Line, int Type, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDisplayLinePosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDisplayLinePosition(System.IntPtr Instrument_Handle, int Window, int Line, int Type, double Position);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLimitLineAudioBeep", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLimitLineAudioBeep(System.IntPtr Instrument_Handle, ushort Audio_Beep);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetActiveLimitLines", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetActiveLimitLines(System.IntPtr Instrument_Handle, int String_Size, System.Text.StringBuilder Active_Limit_Lines, out int Returned_String_Length);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetLimitLineCatalog", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetLimitLineCatalog(System.IntPtr Instrument_Handle, out int Number_of_Limit_Lines, int Buffer_Size, System.Text.StringBuilder Limit_Lines_List);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_DisplaySetActiveWindow", CallingConvention = CallingConvention.StdCall)]
            public static extern int DisplaySetActiveWindow(System.IntPtr Instrument_Handle, int Window);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_DisplayGetActiveWindow", CallingConvention = CallingConvention.StdCall)]
            public static extern int DisplayGetActiveWindow(System.IntPtr Instrument_Handle, out int Window);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDisplayControl", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDisplayControl(System.IntPtr Instrument_Handle, ushort Frequency, ushort Logo, ushort Time);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDisplayComment", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDisplayComment(System.IntPtr Instrument_Handle, int Window, ushort State, string Comment);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDisplayPowerSave", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDisplayPowerSave(System.IntPtr Instrument_Handle, ushort State, int Holdoff);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDisplayFormat", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDisplayFormat(System.IntPtr Instrument_Handle, int Format);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDisplayWindowSize", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDisplayWindowSize(System.IntPtr Instrument_Handle, int Window_Size);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDisplayLogRange", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDisplayLogRange(System.IntPtr Instrument_Handle, int Window, double Range);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDisplayAmplitudeGridMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDisplayAmplitudeGridMode(System.IntPtr Instrument_Handle, int Window, int Y_Axis_Grid_Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDisplayReferenceValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDisplayReferenceValue(System.IntPtr Instrument_Handle, int Window, double Reference_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDisplayReferencePosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDisplayReferencePosition(System.IntPtr Instrument_Handle, int Window, double Reference_Position);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDisplaySingleSweep", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDisplaySingleSweep(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDisplayUpdate", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDisplayUpdate(System.IntPtr Instrument_Handle, ushort Display_In_Remote);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDisplaySoftFrontpanel", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDisplaySoftFrontpanel(System.IntPtr Instrument_Handle, ushort Soft_Frontpanel);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDisplayAutoscaleOnce", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDisplayAutoscaleOnce(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDisplayYAxisScaling", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDisplayYAxisScaling(System.IntPtr Instrument_Handle, int Window, double Scaling);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDisplaySettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDisplaySettings(System.IntPtr Instrument_Handle, ushort Status_Bar_State, ushort Softkey_Bar_State, ushort Tool_Bar_State, int Touchscreen_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetDisplayThemeCatalog", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetDisplayThemeCatalog(System.IntPtr Instrument_Handle, out int Number_of_Themes, int Buffer_Size, System.Text.StringBuilder Theme_List);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDisplayThemeSelect", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDisplayThemeSelect(System.IntPtr Instrument_Handle, string Theme);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDisplayMarkerTable", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDisplayMarkerTable(System.IntPtr Instrument_Handle, int Marker_Table);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDisplayDateFormat", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDisplayDateFormat(System.IntPtr Instrument_Handle, int Date_Format);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CreateDisplayScreenshot", CallingConvention = CallingConvention.StdCall)]
            public static extern int CreateDisplayScreenshot(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDisplayZoom", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDisplayZoom(System.IntPtr Instrument_Handle, int Window, ushort Zoom_State, double x1, double y1, double x2, double y2);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_DisplayColorDefault", CallingConvention = CallingConvention.StdCall)]
            public static extern int DisplayColorDefault(System.IntPtr Instrument_Handle, int Default_Setting);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_DisplayColorDefaultPortable", CallingConvention = CallingConvention.StdCall)]
            public static extern int DisplayColorDefaultPortable(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDisplayColor", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDisplayColor(System.IntPtr Instrument_Handle, int Color_Map, double TINT, double SATURATION, double BRIGHTNESS);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDisplayColorPortable", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDisplayColorPortable(System.IntPtr Instrument_Handle, int Color_Scheme);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDisplayColorByPredefined", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDisplayColorByPredefined(System.IntPtr Instrument_Handle, int Color_Map, int Predefined_Colors);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SelectTransducerFactor", CallingConvention = CallingConvention.StdCall)]
            public static extern int SelectTransducerFactor(System.IntPtr Instrument_Handle, int Window, ushort Transducer_State, string Transducer_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTransducerFactor", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTransducerFactor(System.IntPtr Instrument_Handle, int Window, string Name, int Unit, int Interpolation, string Comment, int No_of_Test_Points, double[] Frequency_Data, double[] Level_Data);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTransducerFactorRefLevAdj", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTransducerFactorRefLevAdj(System.IntPtr Instrument_Handle, int Window, ushort Ref_Lev_Adj);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDisplayTransducerState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDisplayTransducerState(System.IntPtr Instrument_Handle, int Window, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetTransducerFactorCatalog", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTransducerFactorCatalog(System.IntPtr Instrument_Handle, out int Number_of_Transducer_Factors, int Buffer_Size, System.Text.StringBuilder Transducer_Factors_List);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_DeleteTransducerFactor", CallingConvention = CallingConvention.StdCall)]
            public static extern int DeleteTransducerFactor(System.IntPtr Instrument_Handle, int Window, string Transducer_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetTransducerActive", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTransducerActive(System.IntPtr Instrument_Handle, System.Text.StringBuilder Active_Transducer);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryIsotropicAntennaState", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryIsotropicAntennaState(System.IntPtr Instrument_Handle, int Transducer, out ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureHardcopyDevice", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureHardcopyDevice(System.IntPtr Instrument_Handle, int Device, int Destination, int Page_Orientation);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_HardcopyAbort", CallingConvention = CallingConvention.StdCall)]
            public static extern int HardcopyAbort(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_HardcopyPrint", CallingConvention = CallingConvention.StdCall)]
            public static extern int HardcopyPrint(System.IntPtr Instrument_Handle, int Device, int Items);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_HardcopyPrintNext", CallingConvention = CallingConvention.StdCall)]
            public static extern int HardcopyPrintNext(System.IntPtr Instrument_Handle, int Device);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_HardcopySetFileName", CallingConvention = CallingConvention.StdCall)]
            public static extern int HardcopySetFileName(System.IntPtr Instrument_Handle, string Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_HardcopyGetPrinterList", CallingConvention = CallingConvention.StdCall)]
            public static extern int HardcopyGetPrinterList(System.IntPtr Instrument_Handle, int Buffer_Size, System.Text.StringBuilder Printer_List);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_HardcopySetPrinter", CallingConvention = CallingConvention.StdCall)]
            public static extern int HardcopySetPrinter(System.IntPtr Instrument_Handle, int Device, string Printer_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_HardcopyComment", CallingConvention = CallingConvention.StdCall)]
            public static extern int HardcopyComment(System.IntPtr Instrument_Handle, string Comment);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_HardcopyTimeDate", CallingConvention = CallingConvention.StdCall)]
            public static extern int HardcopyTimeDate(System.IntPtr Instrument_Handle, int Device, ushort Time_and_Date_Stamp);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_HardcopyColor", CallingConvention = CallingConvention.StdCall)]
            public static extern int HardcopyColor(System.IntPtr Instrument_Handle, ushort Color, int Default_Setting);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureHardcopyColor", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureHardcopyColor(System.IntPtr Instrument_Handle, int Color_Map, double TINT, double SATURATION, double BRIGHTNESS);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureHardcopyColorByPredefined", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureHardcopyColorByPredefined(System.IntPtr Instrument_Handle, int Color_Map, int Predefined_Colors);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Calibration", CallingConvention = CallingConvention.StdCall)]
            public static extern int Calibration(System.IntPtr Instrument_Handle, ushort Sync, uint Timeout__ms_, out int Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CalibrationAbort", CallingConvention = CallingConvention.StdCall)]
            public static extern int CalibrationAbort(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CalibrationResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int CalibrationResult(System.IntPtr Instrument_Handle, int Array_Size, System.Text.StringBuilder Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CalibrationState", CallingConvention = CallingConvention.StdCall)]
            public static extern int CalibrationState(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_YIGCorrection", CallingConvention = CallingConvention.StdCall)]
            public static extern int YIGCorrection(System.IntPtr Instrument_Handle, int Window, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetCalibrationProbeCatalog", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetCalibrationProbeCatalog(System.IntPtr Instrument_Handle, out int Number_of_Data_Sets, int Buffer_Size, System.Text.StringBuilder Data_Set_List);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CalibrationProbeState", CallingConvention = CallingConvention.StdCall)]
            public static extern int CalibrationProbeState(System.IntPtr Instrument_Handle, string Probe_Data_Set_Name, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CalibrationProbeSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int CalibrationProbeSettings(System.IntPtr Instrument_Handle, int Probe_Mode, double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CalibrationProbeManualCompensation", CallingConvention = CallingConvention.StdCall)]
            public static extern int CalibrationProbeManualCompensation(System.IntPtr Instrument_Handle, ushort Manual_Probe_Adjustment);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CalibrationProbeCompensation", CallingConvention = CallingConvention.StdCall)]
            public static extern int CalibrationProbeCompensation(System.IntPtr Instrument_Handle, ushort Frequency_Response_Comp, ushort Gain_Error_Compensation);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_StartProbeCalibration", CallingConvention = CallingConvention.StdCall)]
            public static extern int StartProbeCalibration(System.IntPtr Instrument_Handle, string Probe_Data_Set_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetProbeCalibrationStatus", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetProbeCalibrationStatus(System.IntPtr Instrument_Handle, uint Timeout, out ushort Probe_Calibration_Status_I, out ushort Probe_Calibration_Status_Q);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CalibrationProbeCheck", CallingConvention = CallingConvention.StdCall)]
            public static extern int CalibrationProbeCheck(System.IntPtr Instrument_Handle, string Probe_Data_Set_Name, out int Probe_Check);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CalibrationProbeResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int CalibrationProbeResult(System.IntPtr Instrument_Handle, string Probe_Data_Set_Name, int Array_Size, System.Text.StringBuilder Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CalibrationProbeRename", CallingConvention = CallingConvention.StdCall)]
            public static extern int CalibrationProbeRename(System.IntPtr Instrument_Handle, string Probe_Data_Set_Source, string Probe_Data_Set_Destination);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CalibrationProbeDelete", CallingConvention = CallingConvention.StdCall)]
            public static extern int CalibrationProbeDelete(System.IntPtr Instrument_Handle, string Probe_Data_Set_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureControlLinesState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureControlLinesState(System.IntPtr Instrument_Handle, int Window, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureOutputControlLines", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureOutputControlLines(System.IntPtr Instrument_Handle, int Window, string User_Ports);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetControlLines", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetControlLines(System.IntPtr Instrument_Handle, int Window, int Lines, System.Text.StringBuilder User_Ports);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAutoAdjust", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAutoAdjust(System.IntPtr Instrument_Handle, int Adjust_Settings, int Measurement_Time_Mode, double Measurement_Time);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_YAxisAutoScale", CallingConvention = CallingConvention.StdCall)]
            public static extern int YAxisAutoScale(System.IntPtr Instrument_Handle, int Auto_Scale_Screen);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGPSState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGPSState(System.IntPtr Instrument_Handle, ushort GPS);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_getGPSConnected", CallingConvention = CallingConvention.StdCall)]
            public static extern int getGPSConnected(System.IntPtr Instrument_Handle, out ushort GPS_Connected);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_getGPSFrequencyCorrection", CallingConvention = CallingConvention.StdCall)]
            public static extern int getGPSFrequencyCorrection(System.IntPtr Instrument_Handle, out double Correction_Factor);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_getGPSSignalQuality", CallingConvention = CallingConvention.StdCall)]
            public static extern int getGPSSignalQuality(System.IntPtr Instrument_Handle, out int Signal_Quality);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_getGPSSatellites", CallingConvention = CallingConvention.StdCall)]
            public static extern int getGPSSatellites(System.IntPtr Instrument_Handle, out int Satellites);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_getGPSLatitude", CallingConvention = CallingConvention.StdCall)]
            public static extern int getGPSLatitude(System.IntPtr Instrument_Handle, int Buffer_Size, System.Text.StringBuilder Latitude);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_getGPSLongitude", CallingConvention = CallingConvention.StdCall)]
            public static extern int getGPSLongitude(System.IntPtr Instrument_Handle, int Buffer_Size, System.Text.StringBuilder Longitude);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_getGPSPositionInformation", CallingConvention = CallingConvention.StdCall)]
            public static extern int getGPSPositionInformation(System.IntPtr Instrument_Handle, out ushort Position_Information);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadYTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadYTrace(System.IntPtr Instrument_Handle, int Window, int Trace, uint Maximum_Time__ms_, int Array_Length, out int Actual_Points, [In, Out] double[] Amplitude);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchYTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchYTrace(System.IntPtr Instrument_Handle, int Window, int Trace, int Array_Length, out int Actual_Points, [In, Out] double[] Amplitude);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetCurrentResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetCurrentResults(System.IntPtr Instrument_Handle, int Window, out double X_Value, out double Y_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetCurrentYResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetCurrentYResult(System.IntPtr Instrument_Handle, int Window, out double Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadTraceIQData", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadTraceIQData(System.IntPtr Instrument_Handle, int Window, uint Timeout, int Buffer_Size, out int No_of_Points, [In, Out] double[] Real_Parts__I_, [In, Out] double[] Imaginary_Parts__Q_);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchTraceIQData", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchTraceIQData(System.IntPtr Instrument_Handle, int Offset_Samples, int No_of_Samples, int Buffer_Size, out int No_of_Points, [In, Out] double[] Real_Parts__I_, [In, Out] double[] Imaginary_Parts__Q_);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadMemoryIQLargeData", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadMemoryIQLargeData(System.IntPtr Instrument_Handle, int Window, uint Timeout, int Format, int Data_Order, out int No_of_Points);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadMemoryIQLargeDataBlock", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadMemoryIQLargeDataBlock(System.IntPtr Instrument_Handle, int Buffer_Size, System.Text.StringBuilder Returned_Data, out int Returned_Counts);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetLimitCheckResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetLimitCheckResult(System.IntPtr Instrument_Handle, int Window, int Limit, out int State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Initiate", CallingConvention = CallingConvention.StdCall)]
            public static extern int Initiate(System.IntPtr Instrument_Handle, int Window, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_InitiateSpurious", CallingConvention = CallingConvention.StdCall)]
            public static extern int InitiateSpurious(System.IntPtr Instrument_Handle, int Window, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_InitiateEMITest", CallingConvention = CallingConvention.StdCall)]
            public static extern int InitiateEMITest(System.IntPtr Instrument_Handle, int Window, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_InitiateFinalMeasurementTest", CallingConvention = CallingConvention.StdCall)]
            public static extern int InitiateFinalMeasurementTest(System.IntPtr Instrument_Handle, int Window, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_InitiateSEMMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int InitiateSEMMeasurement(System.IntPtr Instrument_Handle, int Window, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Abort", CallingConvention = CallingConvention.StdCall)]
            public static extern int Abort(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Hold", CallingConvention = CallingConvention.StdCall)]
            public static extern int Hold(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Continue", CallingConvention = CallingConvention.StdCall)]
            public static extern int Continue(System.IntPtr Instrument_Handle, int Window, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SendSoftwareTrigger", CallingConvention = CallingConvention.StdCall)]
            public static extern int SendSoftwareTrigger(System.IntPtr Instrument_Handle, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetSweepNumber", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSweepNumber(System.IntPtr Instrument_Handle, int Window, out int Number_Of_Sweeps);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMarkerNoiseMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMarkerNoiseMeasurement(System.IntPtr Instrument_Handle, int Window, int Marker, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryMarkerNoiseMeasurementResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryMarkerNoiseMeasurementResult(System.IntPtr Instrument_Handle, int Window, int Marker, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMarkerPhaseNoiseMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMarkerPhaseNoiseMeasurement(System.IntPtr Instrument_Handle, int Window, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_PhaseNoisePeakSearch", CallingConvention = CallingConvention.StdCall)]
            public static extern int PhaseNoisePeakSearch(System.IntPtr Instrument_Handle, int Window);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_PhaseNoisePeakSearchAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int PhaseNoisePeakSearchAuto(System.IntPtr Instrument_Handle, int Window, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryPhaseNoiseResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryPhaseNoiseResult(System.IntPtr Instrument_Handle, int Window, out double Phase_Noise_Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMarkerFrequencyCounter", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMarkerFrequencyCounter(System.IntPtr Instrument_Handle, int Window, int Marker, ushort Marker_Frequency_Counter, double Frequency_Counter_Resolution);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryMarkerFrequencyCounterResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryMarkerFrequencyCounterResult(System.IntPtr Instrument_Handle, int Window, int Marker, out double Counter_Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureNdBPoints", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureNdBPoints(System.IntPtr Instrument_Handle, int Window, ushort N_dB_State, double N_dB_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryNdBResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryNdBResult(System.IntPtr Instrument_Handle, int Window, out double N_dB_Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryNdBFrequencies", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryNdBFrequencies(System.IntPtr Instrument_Handle, int Window, out double N_dB_Frequency_Lower, out double N_dB_Frequency_Higher);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryNdBTimes", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryNdBTimes(System.IntPtr Instrument_Handle, int Window, out double N_dB_Time_Lower, out double N_dB_Time_Higher);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryNdBQFactor", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryNdBQFactor(System.IntPtr Instrument_Handle, int Window, out double N_dB_Q_Factor);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureListPowerAverageType", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureListPowerAverageType(System.IntPtr Instrument_Handle, int Window, int Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureListPowerSequence", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureListPowerSequence(System.IntPtr Instrument_Handle, int Window, int No_of_List_Items, double[] Analyzer_Frequency, double[] Reference_Level, double[] RF_Input_Attenuation, double[] RF_Input_Electronic_Attn, int[] Filter_Type, double[] Resolution_Bandwidth, double[] Video_Bandwidth, double[] Meas_Time, double[] Trigger_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureListPowerSet", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureListPowerSet(System.IntPtr Instrument_Handle, int Window, ushort Peak_Meas, ushort RMS_Meas, ushort AVG_Meas, int Trigger_Mode, int Trigger_Slope, double Trigger_Offset, double Gate_Length);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ListPowerMeasurementOff", CallingConvention = CallingConvention.StdCall)]
            public static extern int ListPowerMeasurementOff(System.IntPtr Instrument_Handle, int Window);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryListPowerSequence", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryListPowerSequence(System.IntPtr Instrument_Handle, int Window, int No_of_List_Items, double[] Analyzer_Frequency, double[] Reference_Level, double[] RF_Input_Attenuation, double[] RF_Input_Electronic_Attn, int[] Filter_Type, double[] Resolution_Bandwidth, double[] Video_Bandwidth, double[] Meas_Time, double[] Trigger_Level, [In, Out] double[] List_Power_Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryListPowerResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryListPowerResult(System.IntPtr Instrument_Handle, int Window, int No_of_Results, [In, Out] double[] List_Power_Results, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTimeDomainPowerMeasurementState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTimeDomainPowerMeasurementState(System.IntPtr Instrument_Handle, int Window, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTimeDomainPowerMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTimeDomainPowerMeasurement(System.IntPtr Instrument_Handle, int Window, ushort Peak, ushort RMS, ushort Mean, ushort Standard_Deviation, ushort Average, ushort Max_Hold, int Power);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_TimeDomainPowerMeasurementAllOff", CallingConvention = CallingConvention.StdCall)]
            public static extern int TimeDomainPowerMeasurementAllOff(System.IntPtr Instrument_Handle, int Window);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_TimeDomainPowerMeasurementSetReference", CallingConvention = CallingConvention.StdCall)]
            public static extern int TimeDomainPowerMeasurementSetReference(System.IntPtr Instrument_Handle, int Window);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryTDomPowerResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryTDomPowerResult(System.IntPtr Instrument_Handle, int Window, int Measurement, int Result_Type, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryTDomPowerPulseSequenceResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryTDomPowerPulseSequenceResult(System.IntPtr Instrument_Handle, int Window, double Time_Offset_of_First_Pulse, double Measurement_Time, double Period, int Number_of_Pulses, [In, Out] double[] Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SelectPowerMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int SelectPowerMeasurement(System.IntPtr Instrument_Handle, int Window, ushort State, int Power_Measurement);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePowerMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePowerMeasurement(System.IntPtr Instrument_Handle, int Window, int No_Of_Adj_Channels, int Channel_Mode, int Power_Mode, ushort Result_Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePowerChannelSpacing", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePowerChannelSpacing(System.IntPtr Instrument_Handle, int Window, int Channel, int Channel_Number, double Spacing);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePowerChannelBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePowerChannelBandwidth(System.IntPtr Instrument_Handle, int Window, int Channel, int Channel_Number, double Bandwidth);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureChannelPowerUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureChannelPowerUnit(System.IntPtr Instrument_Handle, int Channel_Power_Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureChannelPowerNoiseCorrection", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureChannelPowerNoiseCorrection(System.IntPtr Instrument_Handle, int Window, ushort Noise_Correction_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureChannelPowerTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureChannelPowerTrace(System.IntPtr Instrument_Handle, int Window, int Trace_Number);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureChannelPowerStandard", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureChannelPowerStandard(System.IntPtr Instrument_Handle, int Window, int ACP_Standard);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryChannelPowerStandardCatalog", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryChannelPowerStandardCatalog(System.IntPtr Instrument_Handle, int Buffer_Size, System.Text.StringBuilder Power_Standard_Catalog);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureChannelPowerUserStandard", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureChannelPowerUserStandard(System.IntPtr Instrument_Handle, int Window, int Operation, string User_Standard_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_PowerAdjustReferenceLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int PowerAdjustReferenceLevel(System.IntPtr Instrument_Handle, int Window, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_PowerPresetMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int PowerPresetMeasurement(System.IntPtr Instrument_Handle, int Window, int Channel_Power_Type, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_PowerStandardCheck", CallingConvention = CallingConvention.StdCall)]
            public static extern int PowerStandardCheck(System.IntPtr Instrument_Handle, out int Check);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryPowerResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryPowerResults(System.IntPtr Instrument_Handle, int Window, int Power_Measurement, int Array_Size, [In, Out] double[] Results, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePowerChannelWeightingFilters", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePowerChannelWeightingFilters(System.IntPtr Instrument_Handle, int Filter_Type, ushort Filter_State, double Filter_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureACPFastACPState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureACPFastACPState(System.IntPtr Instrument_Handle, int Window, ushort Fast_ACP_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ACPChannelPowerAutoAdjust", CallingConvention = CallingConvention.StdCall)]
            public static extern int ACPChannelPowerAutoAdjust(System.IntPtr Instrument_Handle, int Window, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ACPChannelPowerAdjustRefLevelOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int ACPChannelPowerAdjustRefLevelOffset(System.IntPtr Instrument_Handle, int Window, double Adjust_Ref_Level_Offset);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ACPAdjustReferenceLevelToChannelPower", CallingConvention = CallingConvention.StdCall)]
            public static extern int ACPAdjustReferenceLevelToChannelPower(System.IntPtr Instrument_Handle, int Window, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureACPLimitState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureACPLimitState(System.IntPtr Instrument_Handle, int Window, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureACPAdjacentChannelLimit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureACPAdjacentChannelLimit(System.IntPtr Instrument_Handle, int Window, ushort Relative_State, double Relative_Value, ushort Absolute_State, double Absolute_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureACPAlternateChannelLimit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureACPAlternateChannelLimit(System.IntPtr Instrument_Handle, int Window, int Alternate_Channel_Number, ushort Relative_State, double Relative_Value, ushort Absolute_State, double Absolute_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryACPAdjacentChannelLimitCheckResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryACPAdjacentChannelLimitCheckResults(System.IntPtr Instrument_Handle, int Window, out int Lower_Result, out int Upper_Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryACPAlternateChannelLimitCheckResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryACPAlternateChannelLimitCheckResults(System.IntPtr Instrument_Handle, int Window, int Alternate_Channel_Number, out int Lower_Result, out int Upper_Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMulticarrierACPPower", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMulticarrierACPPower(System.IntPtr Instrument_Handle, int Window, int No_Of_Adj_Channels, int No_Of_TX_Channels, int Channel_Mode, int Power_Mode, ushort Result_Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureACPReferenceChannelAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureACPReferenceChannelAuto(System.IntPtr Instrument_Handle, int Window, int Reference);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureACPReferenceChannelMan", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureACPReferenceChannelMan(System.IntPtr Instrument_Handle, int Window, int Channel);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureOccupiedBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureOccupiedBandwidth(System.IntPtr Instrument_Handle, int Window, double Power_Bandwidth, double Channel_Bandwidth);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSignalStatisticMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSignalStatisticMeasurement(System.IntPtr Instrument_Handle, int Measurement, int Number_of_Samples, double Resolution_Bandwidth);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSignalStatisticMeasurementXAxis", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSignalStatisticMeasurementXAxis(System.IntPtr Instrument_Handle, double X_Axis_Ref_Level, double X_Axis_Range);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSignalStatisticMeasurementYAxis", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSignalStatisticMeasurementYAxis(System.IntPtr Instrument_Handle, int Y_Unit, double Y_Axis_Max_Value, double Y_Axis_Min_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSignalStatisticSweep", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSignalStatisticSweep(System.IntPtr Instrument_Handle, ushort Sweep_Mode_Continuous);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSignalStatisticMeanPowerPosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSignalStatisticMeanPowerPosition(System.IntPtr Instrument_Handle, double Mean_Power_Position);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SignalStatisticMeasurementAdjustSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int SignalStatisticMeasurementAdjustSettings(System.IntPtr Instrument_Handle, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SignalStatisticMeasurementRefLevelAdjust", CallingConvention = CallingConvention.StdCall)]
            public static extern int SignalStatisticMeasurementRefLevelAdjust(System.IntPtr Instrument_Handle, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SignalStatisticMeasurementPresetScaling", CallingConvention = CallingConvention.StdCall)]
            public static extern int SignalStatisticMeasurementPresetScaling(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QuerySignalStatisticCCDFLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int QuerySignalStatisticCCDFLevel(System.IntPtr Instrument_Handle, int Trace, int Probability, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QuerySignalStatisticResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QuerySignalStatisticResult(System.IntPtr Instrument_Handle, int Trace, int Result_Type, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureModDepthMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureModDepthMeasurement(System.IntPtr Instrument_Handle, int Window, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryModulationDepth", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryModulationDepth(System.IntPtr Instrument_Handle, int Window, out double Modulation_Depth);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ModulationDepthSignalSearch", CallingConvention = CallingConvention.StdCall)]
            public static extern int ModulationDepthSignalSearch(System.IntPtr Instrument_Handle, int Window, int Marker);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTOIMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTOIMeasurement(System.IntPtr Instrument_Handle, int Window, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryTOI", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryTOI(System.IntPtr Instrument_Handle, int Window, out double TOI_Measurement);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_TOISignalSearch", CallingConvention = CallingConvention.StdCall)]
            public static extern int TOISignalSearch(System.IntPtr Instrument_Handle, int Window, int Marker);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTOIPositioning", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTOIPositioning(System.IntPtr Instrument_Handle, int Window, int Intercept_Markers_Positioning);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureHDistMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureHDistMeasurement(System.IntPtr Instrument_Handle, ushort State, int No_of_Harmonics, ushort Harmonic_RBW_Auto);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_AdjustHDistSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int AdjustHDistSettings(System.IntPtr Instrument_Handle, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryHDist", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryHDist(System.IntPtr Instrument_Handle, out double Harmonic_Distortion_in_percent, out double Harmonic_Distortion_in_dB);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryHDistHarmonicsList", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryHDistHarmonicsList(System.IntPtr Instrument_Handle, [In, Out] double[] Harmonics_List);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryHDistBandwidthList", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryHDistBandwidthList(System.IntPtr Instrument_Handle, int Array_Size, [In, Out] double[] Result_Values, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_BurstPowerSequence", CallingConvention = CallingConvention.StdCall)]
            public static extern int BurstPowerSequence(System.IntPtr Instrument_Handle, int Window, double Analyzer_Frequency__Hz_, double Resolution_Bandwidth__Hz_, double Meas_Time__s_, int Trigger_Source, double Trigger_Level____, double Trigger_Offset__s_, int Type_of_Meas, int No_of_Measurements);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_BurstPowerFilterType", CallingConvention = CallingConvention.StdCall)]
            public static extern int BurstPowerFilterType(System.IntPtr Instrument_Handle, int Window, int Filter_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetBurstPowerSequence", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetBurstPowerSequence(System.IntPtr Instrument_Handle, int Window, double Analyzer_Frequency__Hz_, double Resolution_Bandwidth__Hz_, double Meas_Time__s_, int Trigger_Source, double Trigger_Level____, double Trigger_Offset__s_, int Type_of_Meas, int No_of_Measurements, uint Timeout, [In, Out] double[] Burst_Power_Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetBurstPowerResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetBurstPowerResults(System.IntPtr Instrument_Handle, int Window, int No_of_Results, [In, Out] double[] Burst_Power_Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetBurstPwrResultMin", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetBurstPwrResultMin(System.IntPtr Instrument_Handle, int Window, out double Power_Result_Min);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSEState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSEState(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSEPeakSearch", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSEPeakSearch(System.IntPtr Instrument_Handle, int Window, double Margin, int Peaks_Per_Range);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSESweepListStartStop", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSESweepListStartStop(System.IntPtr Instrument_Handle, int Range, double Start_Frequency, double Stop_Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSESweepListFilterType", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSESweepListFilterType(System.IntPtr Instrument_Handle, int Range, int Filter);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSESweepListBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSESweepListBandwidth(System.IntPtr Instrument_Handle, int Range, double Resolution_Bandwidth, double Video_Bandwidth);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSESweepSweepListTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSESweepSweepListTime(System.IntPtr Instrument_Handle, int Range, ushort Sweep_Time_Auto, double Sweep_Time);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSESweepListDetector", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSESweepListDetector(System.IntPtr Instrument_Handle, int Range, int Detector);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSESweepListReferenceLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSESweepListReferenceLevel(System.IntPtr Instrument_Handle, int Range, double Ref_Level_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSESweepListRFAttenuation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSESweepListRFAttenuation(System.IntPtr Instrument_Handle, int Range, ushort Attenuator_Auto, double Manual_Attenuator_Value__dB_);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSESweepListPreamplifier", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSESweepListPreamplifier(System.IntPtr Instrument_Handle, int Range, ushort Pre_amplifier);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSESweepListSweep", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSESweepListSweep(System.IntPtr Instrument_Handle, int Range, int Sweep_Points, ushort Stop_After_Sweep);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSESweepListTransducerFactor", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSESweepListTransducerFactor(System.IntPtr Instrument_Handle, int Range, string Transducer_Factor);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSESweepListLimits", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSESweepListLimits(System.IntPtr Instrument_Handle, int Range, ushort Limit_Checking, double Limit, double Absolute_Limit_Start, double Absolute_Limit_Stop);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SEDeleteRange", CallingConvention = CallingConvention.StdCall)]
            public static extern int SEDeleteRange(System.IntPtr Instrument_Handle, int Range);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetSENumberOfListRanges", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSENumberOfListRanges(System.IntPtr Instrument_Handle, out int Number_Of_Ranges);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SESearchPeak", CallingConvention = CallingConvention.StdCall)]
            public static extern int SESearchPeak(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_StoreSEToFile", CallingConvention = CallingConvention.StdCall)]
            public static extern int StoreSEToFile(System.IntPtr Instrument_Handle, string File_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadSEPeakList", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadSEPeakList(System.IntPtr Instrument_Handle, int No_Of_Values, [In, Out] double[] Frequency, [In, Out] double[] Level, [In, Out] double[] Delta_Limit, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SEListEvaluation", CallingConvention = CallingConvention.StdCall)]
            public static extern int SEListEvaluation(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadSEListEvaluation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadSEListEvaluation(System.IntPtr Instrument_Handle, int No_Of_Values, [In, Out] double[] Range_Number, [In, Out] double[] Start_Frequency, [In, Out] double[] Stop_Frequency, [In, Out] double[] Resolution_Bandwidth, [In, Out] double[] Frequency, [In, Out] double[] Level_Abs, [In, Out] double[] Level_Rel, [In, Out] double[] Delta, [In, Out] double[] Limit_Check, [In, Out] double[] Reserved1, [In, Out] double[] Reserved2, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSEMState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSEMState(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSEM", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSEM(System.IntPtr Instrument_Handle, int Limit_Line_Type, double Limit_Line_Value, double _30kHz_1MHz_Transition);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSEMPeakSearch", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSEMPeakSearch(System.IntPtr Instrument_Handle, double Margin, int Peaks_Per_Range);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_RestoreSEMStandardLines", CallingConvention = CallingConvention.StdCall)]
            public static extern int RestoreSEMStandardLines(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SEMSearchPeak", CallingConvention = CallingConvention.StdCall)]
            public static extern int SEMSearchPeak(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_StoreSEMToFile", CallingConvention = CallingConvention.StdCall)]
            public static extern int StoreSEMToFile(System.IntPtr Instrument_Handle, string File_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadSEMPeakList", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadSEMPeakList(System.IntPtr Instrument_Handle, int Window, int Final_Measurement, int No_Of_Values, [In, Out] double[] Frequency, [In, Out] double[] Level, [In, Out] double[] Delta_Level, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetSEMValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSEMValue(System.IntPtr Instrument_Handle, int Selection, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SEMListEvaluation", CallingConvention = CallingConvention.StdCall)]
            public static extern int SEMListEvaluation(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadSEMListEvaluationResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadSEMListEvaluationResults(System.IntPtr Instrument_Handle, int No_Of_Values, [In, Out] double[] Range_Number, [In, Out] double[] Start_Frequency, [In, Out] double[] Stop_Frequency, [In, Out] double[] Resolution_Bandwidth, [In, Out] double[] Frequency, [In, Out] double[] Power_Abs, [In, Out] double[] Power_Rel, [In, Out] double[] Delta, [In, Out] double[] Limit_Check, [In, Out] double[] Reserved1, [In, Out] double[] Reserved2, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SEMMarkAllPeaks", CallingConvention = CallingConvention.StdCall)]
            public static extern int SEMMarkAllPeaks(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSEMReferenceRange", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSEMReferenceRange(System.IntPtr Instrument_Handle, int Power_Reference_Type, double Tx_Bandwidth, ushort RRC_Filter_State, double RRC_Filter_Alpha);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QuerySEMReferenceRangePosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int QuerySEMReferenceRangePosition(System.IntPtr Instrument_Handle, out int Reference_Range_Position);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFastSEM", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFastSEM(System.IntPtr Instrument_Handle, ushort Fast_SEM);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSEMPresetStandard", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSEMPresetStandard(System.IntPtr Instrument_Handle, string Preset_Standard);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_RestoreSEMStandardFiles", CallingConvention = CallingConvention.StdCall)]
            public static extern int RestoreSEMStandardFiles(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_StoreSEMPresetSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int StoreSEMPresetSettings(System.IntPtr Instrument_Handle, string Settings_File_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SEMSaveListToFile", CallingConvention = CallingConvention.StdCall)]
            public static extern int SEMSaveListToFile(System.IntPtr Instrument_Handle, string File_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSEMRangeStartStop", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSEMRangeStartStop(System.IntPtr Instrument_Handle, int Range, double Range_Start, double Range_Stop);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSEMRangeFilterType", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSEMRangeFilterType(System.IntPtr Instrument_Handle, int Range, int Filter_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSEMRangeBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSEMRangeBandwidth(System.IntPtr Instrument_Handle, int Range, double Resolution_Bandwidth, double Video_Bandwidth);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSEMRangeSweepTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSEMRangeSweepTime(System.IntPtr Instrument_Handle, int Range, ushort Sweep_Time_Mode, double Sweep_Time);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSEMRangeReferenceLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSEMRangeReferenceLevel(System.IntPtr Instrument_Handle, int Range, double Reference_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSEMRangeRFAttenuation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSEMRangeRFAttenuation(System.IntPtr Instrument_Handle, int Range, ushort RF_Attenuation_Mode, double RF_Attenuator, ushort Preamplifiier);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSEMRangeTransducerFactor", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSEMRangeTransducerFactor(System.IntPtr Instrument_Handle, int Range, string Transducer_Factor);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSEMRangeLimits", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSEMRangeLimits(System.IntPtr Instrument_Handle, int Range, int Limit_Check_Type, double Absolute_Limit_Start, double Absolute_Limit_Stop, double Relative_Limit_Start, double Relative_Limit_Stop);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetSEMRangeCount", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSEMRangeCount(System.IntPtr Instrument_Handle, out int Range_Count);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SEMInsertNewRange", CallingConvention = CallingConvention.StdCall)]
            public static extern int SEMInsertNewRange(System.IntPtr Instrument_Handle, int Range, int Insert_Range);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SEMDeleteRange", CallingConvention = CallingConvention.StdCall)]
            public static extern int SEMDeleteRange(System.IntPtr Instrument_Handle, int Range);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSEMUsedPowerClasses", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSEMUsedPowerClasses(System.IntPtr Instrument_Handle, int Power_Class_Number, int Used_Power_Class);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSEMPowerClassCount", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSEMPowerClassCount(System.IntPtr Instrument_Handle, int Count);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSEMPowerClassLimits", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSEMPowerClassLimits(System.IntPtr Instrument_Handle, int Power_Class_Number, int Limit_State, double Lower_Level_Limit, double Upper_Level_Limit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSEMPowerClassAllLimits", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSEMPowerClassAllLimits(System.IntPtr Instrument_Handle, int Number_Of_Limits, double[] Limits);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QuerySEMPowerClassAllLimits", CallingConvention = CallingConvention.StdCall)]
            public static extern int QuerySEMPowerClassAllLimits(System.IntPtr Instrument_Handle, [In, Out] double[] Limits);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTDMAPowerMeasurementTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTDMAPowerMeasurementTime(System.IntPtr Instrument_Handle, int Marker, double Measurement_Time);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReceiverMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReceiverMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_AddReceiverFrequencyToPeakList", CallingConvention = CallingConvention.StdCall)]
            public static extern int AddReceiverFrequencyToPeakList(System.IntPtr Instrument_Handle, double Receiver_Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReceiverPeakListDisplayState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReceiverPeakListDisplayState(System.IntPtr Instrument_Handle, int Trace, int Peak_Symbol);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFinalReceiverDetector", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFinalReceiverDetector(System.IntPtr Instrument_Handle, int Window, int Trace, int Detector_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReceiverDetector", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReceiverDetector(System.IntPtr Instrument_Handle, int Window, int Trace, ushort Average, ushort Max_Peak, ushort Min_Peak, ushort RMS, ushort Quasipeak, ushort CISPR_RMS, ushort CISPR_Average);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryReceiverDetector", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryReceiverDetector(System.IntPtr Instrument_Handle, int Window, int Trace, out ushort Average, out ushort Max_Peak, out ushort Min_Peak, out ushort RMS, out ushort Quasipeak, out ushort CISPR_RMS, out ushort CISPR_Average);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReceiverDemodulationType", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReceiverDemodulationType(System.IntPtr Instrument_Handle, int Window, int Demodulation_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReceiverSquelch", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReceiverSquelch(System.IntPtr Instrument_Handle, int Window, ushort Squelch_State, double Trigger_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReceiverTraceFeedControl", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReceiverTraceFeedControl(System.IntPtr Instrument_Handle, int Feed_Control);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReceiverTracePoints", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReceiverTracePoints(System.IntPtr Instrument_Handle, int Trace_Points);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReceiverStepMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReceiverStepMode(System.IntPtr Instrument_Handle, int Step_Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReceiverPeakSearchMethod", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReceiverPeakSearchMethod(System.IntPtr Instrument_Handle, int Method);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReceiverNumberOfPeaksSubranges", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReceiverNumberOfPeaksSubranges(System.IntPtr Instrument_Handle, int Number_of_Peaks_Subranges);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReceiverPeaksPerSubrange", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReceiverPeaksPerSubrange(System.IntPtr Instrument_Handle, int Peak_Per_Subrange);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReceiverMargin", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReceiverMargin(System.IntPtr Instrument_Handle, double Margin);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReceiverPeakExcursion", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReceiverPeakExcursion(System.IntPtr Instrument_Handle, double Peak_Excursion);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReceiverFinalMeasurementFinalTest", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReceiverFinalMeasurementFinalTest(System.IntPtr Instrument_Handle, int Window, double Final_Measurement_Time, ushort Final_Measurement);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReceiverPeakSearchAfterPrescan", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReceiverPeakSearchAfterPrescan(System.IntPtr Instrument_Handle, ushort Peak_Search_After_Prescan);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReceiverVnetworkType", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReceiverVnetworkType(System.IntPtr Instrument_Handle, int Window, int Measurement, int V_Network_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReceiverVnetworkHighPassFilter", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReceiverVnetworkHighPassFilter(System.IntPtr Instrument_Handle, int Window, int Measurement, ushort High_Pass_Filter_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReceiverVnetworkPhase", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReceiverVnetworkPhase(System.IntPtr Instrument_Handle, int Window, int Measurement, int Phase);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReceiverVnetworkMultiPhaseFinalMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReceiverVnetworkMultiPhaseFinalMeasurement(System.IntPtr Instrument_Handle, int Window, int Number_Of_Phases, int[] Phases);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryReceiverVnetworkMultiPhaseFinalMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryReceiverVnetworkMultiPhaseFinalMeasurement(System.IntPtr Instrument_Handle, int Window, out int Number_Of_Phasees, [In, Out] int[] Phase);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReceiverPeakSearch", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReceiverPeakSearch(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReceiverAmplitudeLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReceiverAmplitudeLevel(System.IntPtr Instrument_Handle, double Minimum_Level, double Maximum_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReceiverBargraphMaxhold", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReceiverBargraphMaxhold(System.IntPtr Instrument_Handle, ushort Maxhold_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ResetReceiverBargraphMaxhold", CallingConvention = CallingConvention.StdCall)]
            public static extern int ResetReceiverBargraphMaxhold(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryReceiverBargraphLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryReceiverBargraphLevel(System.IntPtr Instrument_Handle, out double Lower_Level, out double Upper_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureScanRanges", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureScanRanges(System.IntPtr Instrument_Handle, int Number_Of_Ranges);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureScanTableStartStop", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureScanTableStartStop(System.IntPtr Instrument_Handle, int Scan_Range, double Frequency_Start, double Frequency_Stop, double Step_Size);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureScanTableResBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureScanTableResBandwidth(System.IntPtr Instrument_Handle, int Scan_Range, double Bandwidth);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureScanTableMeasTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureScanTableMeasTime(System.IntPtr Instrument_Handle, int Scan_Range, double Measurement_Time);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureScanTableAttenuation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureScanTableAttenuation(System.IntPtr Instrument_Handle, int Scan_Range, ushort Auto_Ranging, double RF_Attenuation);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureScanTablePreamplifier", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureScanTablePreamplifier(System.IntPtr Instrument_Handle, int Scan_Range, ushort Auto_Preamplifier, ushort Preamplifier);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReceiverRFAttenuation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReceiverRFAttenuation(System.IntPtr Instrument_Handle, int Window, double Attenuation);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReceiverPreamplifierState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReceiverPreamplifierState(System.IntPtr Instrument_Handle, int Window, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReceiver10dBMin", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReceiver10dBMin(System.IntPtr Instrument_Handle, int Window, ushort _10_dB_Min);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReceiverAutoRange", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReceiverAutoRange(System.IntPtr Instrument_Handle, int Window, ushort Auto_Range);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReceiverAutoPreamplifier", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReceiverAutoPreamplifier(System.IntPtr Instrument_Handle, int Window, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_RunReceiverMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int RunReceiverMeasurement(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadReceiverDetector", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadReceiverDetector(System.IntPtr Instrument_Handle, uint Timeout, out double First_Detector, out double Second_Detector, out double Third_Detector);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadReceiverDetectorExt", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadReceiverDetectorExt(System.IntPtr Instrument_Handle, uint Timeout, out double First_Detector, out double Second_Detector, out double Third_Detector, out double Fourth_Detector);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadScanMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadScanMeasurement(System.IntPtr Instrument_Handle, int Array_Size, out int Trace_Status, out int Number_of_Results, System.Text.StringBuilder Status_Data, [In, Out] double[] Results_of_Trace_1, [In, Out] double[] Results_of_Trace_2, [In, Out] double[] Results_of_Trace_3, [In, Out] double[] Results_of_Trace_4, [In, Out] double[] Results_of_Trace_5, [In, Out] double[] Results_of_Trace_6);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadScanMeasurementStatus", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadScanMeasurementStatus(System.IntPtr Instrument_Handle, System.Text.StringBuilder Status_Data);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadPeakResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadPeakResults(System.IntPtr Instrument_Handle, uint Timeout, int Array_Size, out int Number_of_Results, [In, Out] double[] Peak_Level, [In, Out] double[] Peak_Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SSAPhaseMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SSAPhaseMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SSAMeasurementConfiguration", CallingConvention = CallingConvention.StdCall)]
            public static extern int SSAMeasurementConfiguration(System.IntPtr Instrument_Handle, int Test_Setup);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhaseSignalSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhaseSignalSettings(System.IntPtr Instrument_Handle, double Frequency, double Level, ushort Sweep_Forward, ushort Verify_Frequency___Level, double Frequency_Tolerance, double Level_Tolerance);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhaseFrequencyMultiplier", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhaseFrequencyMultiplier(System.IntPtr Instrument_Handle, int Frequency_Multiplier);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhaseAFCTrackSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhaseAFCTrackSettings(System.IntPtr Instrument_Handle, ushort Track_Frequency, ushort Track_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhaseAFCSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhaseAFCSettings(System.IntPtr Instrument_Handle, ushort AFC_State, double Target_Frequency, double Tolerance, int Iterations);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhaseDisplaySettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhaseDisplaySettings(System.IntPtr Instrument_Handle, double X_Axis_Start, double X_Axis_Stop, ushort Autoscale_Once, double Y_Axis_Top, double Y_Axis_Range);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhaseTraceSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhaseTraceSettings(System.IntPtr Instrument_Handle, double Trace_Offset, int Trace_Smoothing, int Smoothing_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhaseResidualCalculations", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhaseResidualCalculations(System.IntPtr Instrument_Handle, ushort Use_Measurement_Settings, double Evaluate_From, double Evaluate_To);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhaseLevelSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhaseLevelSettings(System.IntPtr Instrument_Handle, ushort Preamplifier___Atten_Auto, ushort Preamplifier, double Attenuation);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhaseBasebandSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhaseBasebandSettings(System.IntPtr Instrument_Handle, int Coupling, double Impedance);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhaseSpotNoiseSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhaseSpotNoiseSettings(System.IntPtr Instrument_Handle, ushort Enable, double Offset_Frequency_1, double Offset_Frequency_2, double Offset_Frequency_3, double Offset_Frequency_4, double Offset_Frequency_5);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhaseInternalGeneratorSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhaseInternalGeneratorSettings(System.IntPtr Instrument_Handle, ushort Enable, double Frequency, double Gen_Out_1_Level, double Gen_Out_2_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SSAPhaseDisableAllSpotNoiseMarkers", CallingConvention = CallingConvention.StdCall)]
            public static extern int SSAPhaseDisableAllSpotNoiseMarkers(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhaseDetectorScope", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhaseDetectorScope(System.IntPtr Instrument_Handle, ushort Phase_Detector_Display);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhaseNoiseMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhaseNoiseMeasurement(System.IntPtr Instrument_Handle, int Phase_Noise);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhaseSweepSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhaseSweepSettings(System.IntPtr Instrument_Handle, int Sweep_Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhaseFFTWindowpSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhaseFFTWindowpSettings(System.IntPtr Instrument_Handle, int FFT_Window);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhaseSpanSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhaseSpanSettings(System.IntPtr Instrument_Handle, double Start_Offset, double Stop_Offset);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhaseCarrierFrequencyOffsetTable", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhaseCarrierFrequencyOffsetTable(System.IntPtr Instrument_Handle, int Sub_band, double RBW, int Average, int FFT);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhaseTuningSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhaseTuningSettings(System.IntPtr Instrument_Handle, int Measurement_Points, double Settle_Time_Vdc, double Settle_Time_Vtune);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhasePresetSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhasePresetSettings(System.IntPtr Instrument_Handle, double RBW, int Average, int FFT);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhaseOptimizeCrossCorrelation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhaseOptimizeCrossCorrelation(System.IntPtr Instrument_Handle, ushort Optimize_Cross_Correlation);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhaseTracking", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhaseTracking(System.IntPtr Instrument_Handle, double Tracking);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhasePulsedFrequencySettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhasePulsedFrequencySettings(System.IntPtr Instrument_Handle, ushort Pulsed_Frequency_Meas_Mode, double Duty_Cycle, double Repetition_Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAVCOCarrierSearch", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAVCOCarrierSearch(System.IntPtr Instrument_Handle, ushort Autosearch, double Start_Frequency, double Stop_Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAVCOLevelSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAVCOLevelSettings(System.IntPtr Instrument_Handle, ushort Preamplifier___Atten_Auto, ushort Preamplifier, double Attenuation);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAVCOTraceSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAVCOTraceSettings(System.IntPtr Instrument_Handle, int Trace, int Harmonics);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAVCOYAxisAutoscaleY", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAVCOYAxisAutoscaleY(System.IntPtr Instrument_Handle, int Autoscale_Y);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAVCOYAxisSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAVCOYAxisSettings(System.IntPtr Instrument_Handle, int Trace, double Top, double Range);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAVCOYAxisTraceOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAVCOYAxisTraceOffset(System.IntPtr Instrument_Handle, double Trace_Offset);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAVCOMeasurementType", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAVCOMeasurementType(System.IntPtr Instrument_Handle, int VCO_Measurement_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAVCOMeasurementUnits", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAVCOMeasurementUnits(System.IntPtr Instrument_Handle, int VCO_Meas_Units);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAVCOPushPullScale", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAVCOPushPullScale(System.IntPtr Instrument_Handle, int VCO_Type, int Scale);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAVCOFrequencyCounter", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAVCOFrequencyCounter(System.IntPtr Instrument_Handle, ushort Coupled, double Resolution);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAVCOMeasurementPoints", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAVCOMeasurementPoints(System.IntPtr Instrument_Handle, int Tuning_Meas, int Pushing_Meas);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAVCOMeasurementTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAVCOMeasurementTime(System.IntPtr Instrument_Handle, double Sweep_Time, double Settle_Time_Vdc, double Settle_Time_Vtune);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSATransientSignalSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSATransientSignalSettings(System.IntPtr Instrument_Handle, double Frequency, double Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSATransientLevelSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSATransientLevelSettings(System.IntPtr Instrument_Handle, ushort Preamplifier___Atten_Auto, ushort Preamplifier, double Attenuation);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSATransientYAxisAutoscaleY", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSATransientYAxisAutoscaleY(System.IntPtr Instrument_Handle, int Autoscale_Y);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSATransientYAxisSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSATransientYAxisSettings(System.IntPtr Instrument_Handle, double Top, double Range);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSATransientYAxisTraceOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSATransientYAxisTraceOffset(System.IntPtr Instrument_Handle, double Trace_Offset);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSATransientUnits", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSATransientUnits(System.IntPtr Instrument_Handle, int Units);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSATransientMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSATransientMeasurement(System.IntPtr Instrument_Handle, int Transient_Measurement);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSATransientBandwidthSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSATransientBandwidthSettings(System.IntPtr Instrument_Handle, double Demod_BW, int Low_Pass, double Measurement_Time);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSATransientTriggerSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSATransientTriggerSettings(System.IntPtr Instrument_Handle, int Trigger_Mode, double Trigger_Offset, double Trigger_Level, int Trigger_Polarity);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSATransientDemodulatedTriggerSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSATransientDemodulatedTriggerSettings(System.IntPtr Instrument_Handle, int Demod_Trigger_Source, double Demod_Trigger_Level, double Trigger_Offset, int Trigger_Polarity);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSATransientSquelchSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSATransientSquelchSettings(System.IntPtr Instrument_Handle, ushort Squelch_Enabled, double Squelch_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSATransientZeroPhaseReference", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSATransientZeroPhaseReference(System.IntPtr Instrument_Handle, double Zero_Phase_Reference);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSADCPower", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSADCPower(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSADCPortsSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSADCPortsSettings(System.IntPtr Instrument_Handle, int Port_Select, ushort Enable_Port, double Set_Voltage, double Min_Voltage, double Max_Voltage, double Vcc1_Pushing, double Max_Current);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSADCTuningPortsSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSADCTuningPortsSettings(System.IntPtr Instrument_Handle, int Port_Select, ushort Enable_Port, double Set_Voltage, double Min_Voltage, double Max_Voltage);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSADCAUXPortSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSADCAUXPortSettings(System.IntPtr Instrument_Handle, ushort Enable_Port, double Set_Voltage, double Max_Current);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSADCSwitchingSequenceState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSADCSwitchingSequenceState(System.IntPtr Instrument_Handle, ushort Switching_Seq_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSADCSwitchingSequenceList", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSADCSwitchingSequenceList(System.IntPtr Instrument_Handle, int[] Port_List, double[] Delay_List);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSADCMeasurementPort", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSADCMeasurementPort(System.IntPtr Instrument_Handle, int Use_DC_Port);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAVCOLoopDUTSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAVCOLoopDUTSettings(System.IntPtr Instrument_Handle, double DUT_Center_Frequency, double DUT_Level, double DUT_Drift);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAVCOLoopDUTPremeasSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAVCOLoopDUTPremeasSettings(System.IntPtr Instrument_Handle, double DUT_Tuning_Range, double DUT_Slope, ushort Premeasurement);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAVCOLoopPhaseDetectorSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAVCOLoopPhaseDetectorSettings(System.IntPtr Instrument_Handle, ushort Phase_Detector_Premeasurement, double Phase_Detector_Slope, double Phase_Detector_Level, double Phase_Detector_DC_Offset, ushort Auto_Harmonic, int Harmonic);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAVCOLoopGeneratorSlope", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAVCOLoopGeneratorSlope(System.IntPtr Instrument_Handle, double Generator_Slope);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAVCOLoopPLLBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAVCOLoopPLLBandwidth(System.IntPtr Instrument_Handle, ushort BW_Auto, double PLL_Bandwidth);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAVCOLoopPhaseDetectorCharacteristic", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAVCOLoopPhaseDetectorCharacteristic(System.IntPtr Instrument_Handle, int Use_Voltage, double Max_Phase_Detector_Voltage, double Min_Phase_Detector_Voltage, int Phase_Detector_Characteristic, double Phase_Detector_Slope, double Phase_Detector_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAVCOLoopAMDetectorCharacteristic", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAVCOLoopAMDetectorCharacteristic(System.IntPtr Instrument_Handle, string File_Name, double Exetrnal_LNA_Gain);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAVCOLoopModulationIndex", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAVCOLoopModulationIndex(System.IntPtr Instrument_Handle, double Modulation_Index);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAVCOLoopAMDetectorGain", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAVCOLoopAMDetectorGain(System.IntPtr Instrument_Handle, double AM_Detector_Gain);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSASpursDisplaySettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSASpursDisplaySettings(System.IntPtr Instrument_Handle, int Display_Mode, ushort Highlight_Spurs);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSASpursSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSASpursSettings(System.IntPtr Instrument_Handle, int Group, ushort Enable_Spurs, double Fundamental_Frequency, int Max_Order);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSASpursThreshold", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSASpursThreshold(System.IntPtr Instrument_Handle, int Spur_Threshold);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SSAStoreSpursList", CallingConvention = CallingConvention.StdCall)]
            public static extern int SSAStoreSpursList(System.IntPtr Instrument_Handle, string File_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhaseInputLNAGain", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhaseInputLNAGain(System.IntPtr Instrument_Handle, ushort Automatic_LNA_Gain, int Low_Noise_Amplifier_Gain);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhaseMarkerZoom", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhaseMarkerZoom(System.IntPtr Instrument_Handle, int Zoom_Factor);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SSACreateLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int SSACreateLimitLine(System.IntPtr Instrument_Handle, int Limit, string Name, string Comment, int Assign_to_Trace);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SSADefineLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int SSADefineLimitLine(System.IntPtr Instrument_Handle, int Limit, int Type, int Count, double[] X_Axis, double[] Amplitude);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SSALimitLineDomain", CallingConvention = CallingConvention.StdCall)]
            public static extern int SSALimitLineDomain(System.IntPtr Instrument_Handle, int Limit, int Domain);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SSALimitLineUnits", CallingConvention = CallingConvention.StdCall)]
            public static extern int SSALimitLineUnits(System.IntPtr Instrument_Handle, int Limit, int Units);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SSAEnableLimitCheck", CallingConvention = CallingConvention.StdCall)]
            public static extern int SSAEnableLimitCheck(System.IntPtr Instrument_Handle, int Limit, int Type, ushort Limit_Enabled, ushort Check_Enabled);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SSAMoveLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int SSAMoveLimitLine(System.IntPtr Instrument_Handle, int Limit, int Type, double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SSACopyLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int SSACopyLimitLine(System.IntPtr Instrument_Handle, int Limit, int Copy_To);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SSADeleteLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int SSADeleteLimitLine(System.IntPtr Instrument_Handle, int Limit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSATraceSmoothingState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSATraceSmoothingState(System.IntPtr Instrument_Handle, int Trace, ushort Smoothing_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhaseAutoLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhaseAutoLevel(System.IntPtr Instrument_Handle, ushort Auto_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhaseSignalLevelRF", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhaseSignalLevelRF(System.IntPtr Instrument_Handle, double Signal_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPresetCompatibility", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPresetCompatibility(System.IntPtr Instrument_Handle, int Preset_Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadSSATraceData", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadSSATraceData(System.IntPtr Instrument_Handle, int Source_Trace, int Array_Length, out int No_of_Values, [In, Out] double[] Trace_Data);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SSAAutoscaleY", CallingConvention = CallingConvention.StdCall)]
            public static extern int SSAAutoscaleY(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAPhaseReferenceMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAPhaseReferenceMeasurement(System.IntPtr Instrument_Handle, int Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAVCOPushPullState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAVCOPushPullState(System.IntPtr Instrument_Handle, int VCO_Type, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAVCOLoopStartPremeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAVCOLoopStartPremeasurement(System.IntPtr Instrument_Handle, int Timeout__ms_);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAVCOLoopPremeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAVCOLoopPremeasurement(System.IntPtr Instrument_Handle, int Action);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAAMNoiseMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAAMNoiseMeasurement(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAAMNoiseOperatingPointPremeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAAMNoiseOperatingPointPremeasurement(System.IntPtr Instrument_Handle, int Timeout__ms_);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSSAAMNoiseNormalizationPremeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSSAAMNoiseNormalizationPremeasurement(System.IntPtr Instrument_Handle, int Timeout__ms_);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QuerySSAMarkerOnSpur", CallingConvention = CallingConvention.StdCall)]
            public static extern int QuerySSAMarkerOnSpur(System.IntPtr Instrument_Handle, int Marker_Number, out int Marker_Position_On_Spur);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QuerySSADeltaMarkerOnSpur", CallingConvention = CallingConvention.StdCall)]
            public static extern int QuerySSADeltaMarkerOnSpur(System.IntPtr Instrument_Handle, int Marker_Number, out int DMarker_Position_On_Spur);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QuerySSASpotNoiseValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int QuerySSASpotNoiseValue(System.IntPtr Instrument_Handle, int Spot_Noise_Marker, out double Position__y_);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QuerySSALimitLineCompatibility", CallingConvention = CallingConvention.StdCall)]
            public static extern int QuerySSALimitLineCompatibility(System.IntPtr Instrument_Handle, int Limit, out int Limit_Line_Compatibility);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SSAClearLimitLineResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int SSAClearLimitLineResults(System.IntPtr Instrument_Handle, int Limit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QuerySSASweepPoints", CallingConvention = CallingConvention.StdCall)]
            public static extern int QuerySSASweepPoints(System.IntPtr Instrument_Handle, out int Sweep_Points);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchSSAPhaseNoiseResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchSSAPhaseNoiseResults(System.IntPtr Instrument_Handle, int Trace_Number, int Type, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchSSAPhaseNoiseSpursResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchSSAPhaseNoiseSpursResults(System.IntPtr Instrument_Handle, int Trace_Number, int Array_Size, [In, Out] double[] Frequency_Results, [In, Out] double[] Power_Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchSSAVCOTuningCharResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchSSAVCOTuningCharResults(System.IntPtr Instrument_Handle, int Type, [In, Out] double[] Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchSSAVCOPushingResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchSSAVCOPushingResults(System.IntPtr Instrument_Handle, int Type, [In, Out] double[] Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchSSAVCOPullingResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchSSAVCOPullingResults(System.IntPtr Instrument_Handle, int Type, [In, Out] double[] Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchSSAVCOHarmonicResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchSSAVCOHarmonicResults(System.IntPtr Instrument_Handle, int Trace_Number, int Type, [In, Out] double[] Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchSSAVCODCCharResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchSSAVCODCCharResults(System.IntPtr Instrument_Handle, int Type, [In, Out] double[] Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetSSAVCOTransientResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSSAVCOTransientResults(System.IntPtr Instrument_Handle, int Modulation_Type, int Parameter_To_Read, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetSSADCVoltageResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSSADCVoltageResults(System.IntPtr Instrument_Handle, [In, Out] double[] Voltage_Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetSSADCCurrentResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSSADCCurrentResults(System.IntPtr Instrument_Handle, [In, Out] double[] Current_Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetSSADCPowerResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSSADCPowerResults(System.IntPtr Instrument_Handle, [In, Out] double[] Power_Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetSSAVCOLoopLowPhaseNoise", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSSAVCOLoopLowPhaseNoise(System.IntPtr Instrument_Handle, out double Phase_Detector_Slope, out double Phase_Detector_Level, out double Phase_Detector_DC_Offset);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetSSAVCOLoopPhaseDetectorVoltage", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSSAVCOLoopPhaseDetectorVoltage(System.IntPtr Instrument_Handle, out double Phase_Detector_Voltage);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetSSAVCOLoopAMDetectorInputLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSSAVCOLoopAMDetectorInputLevel(System.IntPtr Instrument_Handle, out double Input_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_MeasuringReceiverMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int MeasuringReceiverMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverFrequency(System.IntPtr Instrument_Handle, double Receiver_Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverFrequencyStepSize", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverFrequencyStepSize(System.IntPtr Instrument_Handle, double Frequency_Step_Size);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverFrequencySpan", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverFrequencySpan(System.IntPtr Instrument_Handle, double Frequency_Span);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverFrequencyFullSpan", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverFrequencyFullSpan(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverFrequencyMaxSpan", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverFrequencyMaxSpan(System.IntPtr Instrument_Handle, double Frequency_Max_Span);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverAFCenterSpan", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverAFCenterSpan(System.IntPtr Instrument_Handle, double AF_Center, double AF_Span);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverAFStartStop", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverAFStartStop(System.IntPtr Instrument_Handle, double AF_Start, double AF_Stop);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverAFFullSpan", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverAFFullSpan(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_MeasuringReceiverAutoSignalSearch", CallingConvention = CallingConvention.StdCall)]
            public static extern int MeasuringReceiverAutoSignalSearch(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_MeasuringReceiverSignalTrack", CallingConvention = CallingConvention.StdCall)]
            public static extern int MeasuringReceiverSignalTrack(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverRefLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverRefLevel(System.IntPtr Instrument_Handle, int Window, double Ref_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverAutoRange", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverAutoRange(System.IntPtr Instrument_Handle, int Window, ushort Auto_Range);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiver10dBMin", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiver10dBMin(System.IntPtr Instrument_Handle, int Window, ushort _10_dB_Min);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverAutoPreamplifier", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverAutoPreamplifier(System.IntPtr Instrument_Handle, int Window, ushort Auto_Preamplifier);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverDisplayScalePerDiv", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverDisplayScalePerDiv(System.IntPtr Instrument_Handle, int Window, double Scale_Per_Div);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverDisplayReferencePosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverDisplayReferencePosition(System.IntPtr Instrument_Handle, int Window, double Reference_Position);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverDisplayReferenceValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverDisplayReferenceValue(System.IntPtr Instrument_Handle, int Window, double Reference_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverZeroPhaseRefPosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverZeroPhaseRefPosition(System.IntPtr Instrument_Handle, double Reference_Point);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverDisplaySpacing", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverDisplaySpacing(System.IntPtr Instrument_Handle, int Window, int Display_Spacing);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverDisplayPhaseWrap", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverDisplayPhaseWrap(System.IntPtr Instrument_Handle, ushort Phase_Wrap);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverUnitPM", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverUnitPM(System.IntPtr Instrument_Handle, int PM_Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverUnitTHD_SINAD", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverUnitTHD_SINAD(System.IntPtr Instrument_Handle, int THD_SINAD_Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverRFInputCoupling", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverRFInputCoupling(System.IntPtr Instrument_Handle, int Input_Coupling);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverRFAttenuation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverRFAttenuation(System.IntPtr Instrument_Handle, int Window, ushort Input_Attenuation_Auto, double Input_Attenuation);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverMixerLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverMixerLevel(System.IntPtr Instrument_Handle, int Window, ushort Mixer_Level_Auto, double Mixer_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverInputImpedance", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverInputImpedance(System.IntPtr Instrument_Handle, int Window, double Input_Impedance);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverDemodBW", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverDemodBW(System.IntPtr Instrument_Handle, int Window, ushort Demod_BW_Auto, double Demod_BW);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverIFBW", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverIFBW(System.IntPtr Instrument_Handle, int Window, ushort IF_BW_Auto, double IF_BW);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverMeasTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverMeasTime(System.IntPtr Instrument_Handle, int Window, ushort Meas_Time_Auto, double Meas_Time);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverADemodMeasuringTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverADemodMeasuringTime(System.IntPtr Instrument_Handle, double Measuring_Time);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverResolutionBWManual", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverResolutionBWManual(System.IntPtr Instrument_Handle, double Resolution_BW);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverTrigger", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverTrigger(System.IntPtr Instrument_Handle, int Trigger_Source, double Trigger_Offset, int Trigger_Polarity, double Trigger_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_MeasuringReceiverRFLevelMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int MeasuringReceiverRFLevelMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverCalibrationAbsPower", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverCalibrationAbsPower(System.IntPtr Instrument_Handle, int Cal_Abs_Power);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverExpandCalibratedRange", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverExpandCalibratedRange(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverAdjustInputRange", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverAdjustInputRange(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryMeasuringReceiverCorrectionTable", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryMeasuringReceiverCorrectionTable(System.IntPtr Instrument_Handle, int Buffer_Size, System.Text.StringBuilder Correction_List);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverCorrection", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverCorrection(System.IntPtr Instrument_Handle, int Operation, string Path);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverRelativeLevelMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverRelativeLevelMeasurement(System.IntPtr Instrument_Handle, ushort Relative_Level_Measurement, double Reference_Value, int Relative_Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverMeasuredLevelToReference", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverMeasuredLevelToReference(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverLevelMeasurementAveraging", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverLevelMeasurementAveraging(System.IntPtr Instrument_Handle, ushort Averaging, int Averaging_Count);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverLevelAutoAverage", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverLevelAutoAverage(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverLevelAutoAverageData", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverLevelAutoAverageData(System.IntPtr Instrument_Handle, int Array_Size, double[] Level, int[] Count);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverLevelDetector", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverLevelDetector(System.IntPtr Instrument_Handle, int Detector_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverAutoRecal", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverAutoRecal(System.IntPtr Instrument_Handle, ushort Auto_Recal);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_MeasuringReceiverPWRMeter", CallingConvention = CallingConvention.StdCall)]
            public static extern int MeasuringReceiverPWRMeter(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverPWMMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverPWMMeasurement(System.IntPtr Instrument_Handle, int Measurement_Time, int Number_Of_Readings);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverPWMZero", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverPWMZero(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverPWMRelative", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverPWMRelative(System.IntPtr Instrument_Handle, double Reference_Value, int Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverPWMAbsolute", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverPWMAbsolute(System.IntPtr Instrument_Handle, int Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverPWMMeasToRef", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverPWMMeasToRef(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverPWMUsePowerSplitter", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverPWMUsePowerSplitter(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverPWMPowerSplitterInsertionLoss", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverPWMPowerSplitterInsertionLoss(System.IntPtr Instrument_Handle, int Array_Size, double[] Frequency, double[] Insertion_Loss);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverPWMPath2InsertionLoss", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverPWMPath2InsertionLoss(System.IntPtr Instrument_Handle, double Insertion_Loss);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverPWMVSWRCorrection", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverPWMVSWRCorrection(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverPWRExternalMeter", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverPWRExternalMeter(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverPWMRemoteSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverPWMRemoteSettings(System.IntPtr Instrument_Handle, int Type, int GPIB_Address);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverPWMSensorCalFactor", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverPWMSensorCalFactor(System.IntPtr Instrument_Handle, int Use_Sensor, string Sensor_Label, int Array_Size, double[] Frequency, double[] Cal_Factor);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverPWMReferenceOutput", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverPWMReferenceOutput(System.IntPtr Instrument_Handle, ushort Reference_Output);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfMeasuringReceiverAFFilter", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfMeasuringReceiverAFFilter(System.IntPtr Instrument_Handle, int Filter_Type, ushort Filter_State, double Filter_Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverDeemphasisAFFilter", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverDeemphasisAFFilter(System.IntPtr Instrument_Handle, ushort Filter_State, int Deemphasis_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverWeightingAFFilter", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverWeightingAFFilter(System.IntPtr Instrument_Handle, int Filter_Type, ushort Filter_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverAFInputCoupling", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverAFInputCoupling(System.IntPtr Instrument_Handle, int Input_Coupling);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_MeasuringReceiverModulationMeasurementMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int MeasuringReceiverModulationMeasurementMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverModulationType", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverModulationType(System.IntPtr Instrument_Handle, int Modulation_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverDetectorType", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverDetectorType(System.IntPtr Instrument_Handle, int Detector_Type, ushort Detector_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverDetectorTypeMultiple", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverDetectorTypeMultiple(System.IntPtr Instrument_Handle, ushort _PK, ushort _PK_2, ushort _PK_2_2, ushort Average, ushort RMS, ushort Sqrt_2__RMS, ushort THD___SINAD);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverModulationMeasToRef", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverModulationMeasToRef(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverPeakHold", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverPeakHold(System.IntPtr Instrument_Handle, ushort Peak_Hold);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverAveraging", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverAveraging(System.IntPtr Instrument_Handle, ushort Average);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverModulationResultDisplay", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverModulationResultDisplay(System.IntPtr Instrument_Handle, int Result_Display, int Modulation_Type, int Trace);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverRelativeMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverRelativeMeasurement(System.IntPtr Instrument_Handle, int Detector_Type, int Measurement_Mode, int Relative_Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverRelativeMeasurementRef", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverRelativeMeasurementRef(System.IntPtr Instrument_Handle, int Detector_Type, ushort Reference_Auto, double Reference_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryMeasuringReceiverRelativeMeasurementRef", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryMeasuringReceiverRelativeMeasurementRef(System.IntPtr Instrument_Handle, int Detector_Type, out double Reference_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_MeasuringReceiverAudioInputMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int MeasuringReceiverAudioInputMode(System.IntPtr Instrument_Handle, int Input);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverAudioInput", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverAudioInput(System.IntPtr Instrument_Handle, int Input_Level, double Input_Impedance);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverAudioInputResultDisplay", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverAudioInputResultDisplay(System.IntPtr Instrument_Handle, int Result_Display);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverAudioInputResultDisplayMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverAudioInputResultDisplayMode(System.IntPtr Instrument_Handle, int Display_Mode, int Relative_Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMeasuringReceiverAudioInputMeasToRef", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMeasuringReceiverAudioInputMeasToRef(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryMeasuringReceiverModulationMeasurementResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryMeasuringReceiverModulationMeasurementResults(System.IntPtr Instrument_Handle, int Detector_Type, int Modulation_Type, out double Modulation_Measurement_Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryMeasuringReceiverModulationAveragedMeasurementResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryMeasuringReceiverModulationAveragedMeasurementResults(System.IntPtr Instrument_Handle, int Detector_Type, int Modulation_Type, out double Modulation_Measurement_Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryMeasuringReceiverModulationPeakHoldMeasurementResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryMeasuringReceiverModulationPeakHoldMeasurementResults(System.IntPtr Instrument_Handle, int Detector_Type, int Modulation_Type, out double Modulation_Measurement_Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryMeasuringReceiverModulationAudioFrequencyResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryMeasuringReceiverModulationAudioFrequencyResult(System.IntPtr Instrument_Handle, out double Audio_Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryMeasuringReceiverModulationFrequencyErrorResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryMeasuringReceiverModulationFrequencyErrorResult(System.IntPtr Instrument_Handle, out double Frequency_Error);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryMeasuringReceiverModulationCarrierPowerResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryMeasuringReceiverModulationCarrierPowerResult(System.IntPtr Instrument_Handle, out double Carrier_Power);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryMeasuringReceiverModulationSINADMeasurementResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryMeasuringReceiverModulationSINADMeasurementResults(System.IntPtr Instrument_Handle, int Result_Type, out double Measurement_Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryMeasuringReceiverModulationTHDMeasurementResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryMeasuringReceiverModulationTHDMeasurementResults(System.IntPtr Instrument_Handle, int Result_Type, out double Measurement_Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryMeasuringReceiverModulationStandardUncertainty", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryMeasuringReceiverModulationStandardUncertainty(System.IntPtr Instrument_Handle, out double Standard_Uncertainty);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryMeasuringReceiverAudioMeasurementResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryMeasuringReceiverAudioMeasurementResults(System.IntPtr Instrument_Handle, int Detector_Type, out double Audio_Measurement_Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadMeasuringReceiverPWMResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadMeasuringReceiverPWMResult(System.IntPtr Instrument_Handle, uint Timeout, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchMeasuringReceiverPWMResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchMeasuringReceiverPWMResult(System.IntPtr Instrument_Handle, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTrackingGenerator", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTrackingGenerator(System.IntPtr Instrument_Handle, int Window, double Source_Power, double Power_Offset, double Frequency_Offset);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SetTrackingGeneratorState", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTrackingGeneratorState(System.IntPtr Instrument_Handle, int Window, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTrackingGeneratorExternalModulation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTrackingGeneratorExternalModulation(System.IntPtr Instrument_Handle, int Window, int Modulation, ushort Mod_State, double FM_Deviation);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTrackingGeneratorPowerSweep", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTrackingGeneratorPowerSweep(System.IntPtr Instrument_Handle, int Window, int Power_Sweep, double Power_Start, double Power_Stop);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureExternalGeneratorPower", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureExternalGeneratorPower(System.IntPtr Instrument_Handle, int Window, int Generator, double Power);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureExternalGeneratorSource", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureExternalGeneratorSource(System.IntPtr Instrument_Handle, int Window, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SelectExternalGenerator", CallingConvention = CallingConvention.StdCall)]
            public static extern int SelectExternalGenerator(System.IntPtr Instrument_Handle, int Generator, int Generator_Address, int Generator_Link, string Generator_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureExternalGeneratorFrequencySweep", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureExternalGeneratorFrequencySweep(System.IntPtr Instrument_Handle, int Window, int Generator, ushort Frequency_Sweep, double Frequency_Offset, int Denominator, int Numerator);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureExternalGeneratorRefOscillator", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureExternalGeneratorRefOscillator(System.IntPtr Instrument_Handle, int Window, int Reference);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureExternalGain", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureExternalGain(System.IntPtr Instrument_Handle, int Window, double Gain);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SendCommandToExternalGenerator", CallingConvention = CallingConvention.StdCall)]
            public static extern int SendCommandToExternalGenerator(System.IntPtr Instrument_Handle, int Device_Address, string Command_String);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ExternalGeneratorNormalization", CallingConvention = CallingConvention.StdCall)]
            public static extern int ExternalGeneratorNormalization(System.IntPtr Instrument_Handle, int Window, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ExternalGeneratorCalibration", CallingConvention = CallingConvention.StdCall)]
            public static extern int ExternalGeneratorCalibration(System.IntPtr Instrument_Handle, int Window, int Method, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ExternalGeneratorRecall", CallingConvention = CallingConvention.StdCall)]
            public static extern int ExternalGeneratorRecall(System.IntPtr Instrument_Handle, int Window);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ExternalGeneratorSaveAsTrdFactor", CallingConvention = CallingConvention.StdCall)]
            public static extern int ExternalGeneratorSaveAsTrdFactor(System.IntPtr Instrument_Handle, int Window, string Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_NetworkAnalyzerMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int NetworkAnalyzerMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureNetworkAnalyzerMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureNetworkAnalyzerMeasurement(System.IntPtr Instrument_Handle, int Measurement_Mode, ushort Transmission_State, int Measurement_Format, int Measurement_Port);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureNetworkAnalyzerReflectionState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureNetworkAnalyzerReflectionState(System.IntPtr Instrument_Handle, ushort Reflection_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureElectricalCableState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureElectricalCableState(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMagnitudeSpacing", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMagnitudeSpacing(System.IntPtr Instrument_Handle, int Spacing);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMarkerOutputFormat", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMarkerOutputFormat(System.IntPtr Instrument_Handle, int Marker, int Output_Format);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSmithChartReferenceImpedance", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSmithChartReferenceImpedance(System.IntPtr Instrument_Handle, double Reference_Impedance);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSWRYScale", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSWRYScale(System.IntPtr Instrument_Handle, double Y_Scale);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePhaseUnwrap", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePhaseUnwrap(System.IntPtr Instrument_Handle, ushort Phase_Unwrap);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureReflectionCoefficientUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureReflectionCoefficientUnit(System.IntPtr Instrument_Handle, int Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTrackingGeneratorYScale", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTrackingGeneratorYScale(System.IntPtr Instrument_Handle, int Type, double Y_Scale);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTrackingGeneratorReferenceLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTrackingGeneratorReferenceLevel(System.IntPtr Instrument_Handle, int Type, double Reference_Level, int Reference_Position);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTrackingGeneratorOutputLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTrackingGeneratorOutputLevel(System.IntPtr Instrument_Handle, double Output_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryCalibrationMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryCalibrationMode(System.IntPtr Instrument_Handle, out int Calibration_Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryCalibrationStatus", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryCalibrationStatus(System.IntPtr Instrument_Handle, System.Text.StringBuilder Calibration_Status);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryElectricalCableLength", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryElectricalCableLength(System.IntPtr Instrument_Handle, out double Electrical_Cable_Length);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryVSWRFail", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryVSWRFail(System.IntPtr Instrument_Handle, System.Text.StringBuilder Fail);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureUserCalibrate", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureUserCalibrate(System.IntPtr Instrument_Handle, ushort User_Calkit, double User_Length, double Offset_Length);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_UserCalkitRename", CallingConvention = CallingConvention.StdCall)]
            public static extern int UserCalkitRename(System.IntPtr Instrument_Handle, string Rename_User);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGroupDelay", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGroupDelay(System.IntPtr Instrument_Handle, int Aperture_Steps, int Reference_Level, int Reference_Position, double Y_Scale);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVectorVoltmeterReferenceState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVectorVoltmeterReferenceState(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryVectorVoltmeterMagnitudeReferenceValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryVectorVoltmeterMagnitudeReferenceValue(System.IntPtr Instrument_Handle, out double Reference_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryVectorVoltmeterMagnitudeValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryVectorVoltmeterMagnitudeValue(System.IntPtr Instrument_Handle, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryVectorVoltmeterPhaseReferenceValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryVectorVoltmeterPhaseReferenceValue(System.IntPtr Instrument_Handle, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryVectorVoltmeterPhaseValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryVectorVoltmeterPhaseValue(System.IntPtr Instrument_Handle, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_DTFMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int DTFMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCableLength", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCableLength(System.IntPtr Instrument_Handle, double Cable_Length);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDistancetoFaultCableModel", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDistancetoFaultCableModel(System.IntPtr Instrument_Handle, string Cable_Model);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDistancetoFaultThreshold", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDistancetoFaultThreshold(System.IntPtr Instrument_Handle, double Threshold);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryDistancetoFaultResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryDistancetoFaultResult(System.IntPtr Instrument_Handle, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFMDemodAnalogState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFMDemodAnalogState(System.IntPtr Instrument_Handle, int Window, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFMDemodAnalogLowpassFilter", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFMDemodAnalogLowpassFilter(System.IntPtr Instrument_Handle, int Window, ushort AF_Lowpass, double Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFMDemodAnalogHighpassFilter", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFMDemodAnalogHighpassFilter(System.IntPtr Instrument_Handle, int Window, double Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFMDemodAnalogRange", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFMDemodAnalogRange(System.IntPtr Instrument_Handle, int Window, ushort Span_Auto_Manual, double Span);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFFTDiagnostic", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFFTDiagnostic(System.IntPtr Instrument_Handle, int Signal_Path, int Signal_Source, double Voltage, double Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAnalogBasebandInput", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAnalogBasebandInput(System.IntPtr Instrument_Handle, int Baseband_Inputs);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAnalogBasebandInputMeasurementRange", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAnalogBasebandInputMeasurementRange(System.IntPtr Instrument_Handle, double Measurement_Range);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAnalogBasebandTrigger", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAnalogBasebandTrigger(System.IntPtr Instrument_Handle, int Trigger);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAnalogBasebandTriggerLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAnalogBasebandTriggerLevel(System.IntPtr Instrument_Handle, int Trigger, double Trigger_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAnalogBasebandParameters", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAnalogBasebandParameters(System.IntPtr Instrument_Handle, ushort Balanced, int Impedance);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAnalogBasebandSignalPath", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAnalogBasebandSignalPath(System.IntPtr Instrument_Handle, int Path);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAnalogBasebandSignalSourceLowPass", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAnalogBasebandSignalSourceLowPass(System.IntPtr Instrument_Handle, ushort Low_Pass);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAnalogBasebandSignalSourceDither", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAnalogBasebandSignalSourceDither(System.IntPtr Instrument_Handle, ushort IQ_Dither);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FFTSetFFTMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int FFTSetFFTMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FFTPreset", CallingConvention = CallingConvention.StdCall)]
            public static extern int FFTPreset(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FFTRecalc", CallingConvention = CallingConvention.StdCall)]
            public static extern int FFTRecalc(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFFTReferenceLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFFTReferenceLevel(System.IntPtr Instrument_Handle, int Window, int Amplitude_Units, double Reference_Level, double Reference_Level_Offset);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFFTFrequencyDomain", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFFTFrequencyDomain(System.IntPtr Instrument_Handle, int Type, int Window);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFFTTimeDomain", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFFTTimeDomain(System.IntPtr Instrument_Handle, int Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFFTPhaseDiagram", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFFTPhaseDiagram(System.IntPtr Instrument_Handle, int Phase_Wrap, double Phase_Offset, int Phase_Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFFTPhaseLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFFTPhaseLine(System.IntPtr Instrument_Handle, int Window, int Line, ushort Phase_Line_State, double Phase_Line_Position);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFFTSignalSource", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFFTSignalSource(System.IntPtr Instrument_Handle, int Path, int IQ_Input, ushort IQ_Balanced, ushort Low_Pass, ushort IQ_Dither);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFFTSweep", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFFTSweep(System.IntPtr Instrument_Handle, ushort Sweep_Mode_Continuous, int Sweep_Count, int Sweep_Points, ushort Capure_Both_Domain, ushort Recalculate_Auto);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFFTBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFFTBandwidth(System.IntPtr Instrument_Handle, ushort Resolution_Bandwidth_Auto, double Resolution_Bandwidth, double Resolution_Bandwidth_Ratio, int Resolution_BW_1235);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDigitalBasebandInput", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDigitalBasebandInput(System.IntPtr Instrument_Handle, int Baseband_Inputs);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDigitalBasebandInputParameters", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDigitalBasebandInputParameters(System.IntPtr Instrument_Handle, double Range, double Sample_Rate);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDigitalBasebandOutput", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDigitalBasebandOutput(System.IntPtr Instrument_Handle, ushort Output);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDigitalBasebandFullScaleLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDigitalBasebandFullScaleLevel(System.IntPtr Instrument_Handle, double Full_Scale_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryDigitalBasebandInputOutputConfiguration", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryDigitalBasebandInputOutputConfiguration(System.IntPtr Instrument_Handle, int Digital_Baseband_Select, int Array_Size, System.Text.StringBuilder Configuration);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDigitalBasebandInputTriggerSource", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDigitalBasebandInputTriggerSource(System.IntPtr Instrument_Handle, int Trigger_Source);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureDigitalBasebandInputTriggerParameters", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureDigitalBasebandInputTriggerParameters(System.IntPtr Instrument_Handle, double Trigger_Level, double Holdoff);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GsmMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int GsmMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGsmTrigger", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGsmTrigger(System.IntPtr Instrument_Handle, int Trigger_Source, double Trigger_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGsmDemodulation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGsmDemodulation(System.IntPtr Instrument_Handle, int Modulation_Type, int Multi_Slot, int Samples_Per_Symbols, ushort Sync_Search, ushort Burst_Search, double Burst_Search_Threshold, int Midamble, string User_Sequence);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGsmMultiCarrierMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGsmMultiCarrierMode(System.IntPtr Instrument_Handle, ushort Multi_Carrier_Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGsmTransSpecLimitType", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGsmTransSpecLimitType(System.IntPtr Instrument_Handle, int Limit_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGsmIFRFTriggerAsIQPower", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGsmIFRFTriggerAsIQPower(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGsmModulationSpectrumListAverage", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGsmModulationSpectrumListAverage(System.IntPtr Instrument_Handle, int List_Average);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_RestoreGsmLimitLines", CallingConvention = CallingConvention.StdCall)]
            public static extern int RestoreGsmLimitLines(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGsmExtendedSlotState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGsmExtendedSlotState(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGsmExtendedSlot", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGsmExtendedSlot(System.IntPtr Instrument_Handle, int Standard, int Trigger_Reference, int Reference_Midamble, ushort Absolute_Level_State, double Limit_Base_1, double Limit_Base_2);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGsmExtendedSlotCommonSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGsmExtendedSlotCommonSettings(System.IntPtr Instrument_Handle, ushort Long_Slots, int Array_Size, int[] Long_Slots_Values, ushort Only_One_Frame);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGsmExtendedSlotParameters", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGsmExtendedSlotParameters(System.IntPtr Instrument_Handle, int Slot_Number, int Modulation, int Reference_Level_Mode, double Reference_Level_Value, int Limit_Line_Ctrl_Level, string Limit_Line_Lower, string Limit_Line_Upper);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGsmMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGsmMeasurement(System.IntPtr Instrument_Handle, int Measurement_Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGsmPVTMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGsmPVTMeasurement(System.IntPtr Instrument_Handle, int Burst_Section, ushort High_Resolution_Mode, int Filter_Bandwidth, int Transition_No);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadGsmLevelTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadGsmLevelTime(System.IntPtr Instrument_Handle, uint Timeout, out ushort Status, out double Signal_Power__dBm_, out double Trigger_Time__s_, out double Trigger_Level__dBm_V_, out double Reserved);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadGsmPhaseFreqErrorValues", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadGsmPhaseFreqErrorValues(System.IntPtr Instrument_Handle, uint Timeout, int Measurement, int Modifier, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchGsmPhaseFreqErrorValues", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchGsmPhaseFreqErrorValues(System.IntPtr Instrument_Handle, int Measurement, int Modifier, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadGsmModulAccValues", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadGsmModulAccValues(System.IntPtr Instrument_Handle, uint Timeout, int Measurement, int Modifier, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchGsmModulAccValues", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchGsmModulAccValues(System.IntPtr Instrument_Handle, int Measurement, int Modifier, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadGsmPtempRef", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadGsmPtempRef(System.IntPtr Instrument_Handle, uint Timeout, [In, Out] double[] Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadGsmPowVsTimeValues", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadGsmPowVsTimeValues(System.IntPtr Instrument_Handle, uint Timeout, int Measurement, int Modifier, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadGsmExtendedSlotPtempRef", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadGsmExtendedSlotPtempRef(System.IntPtr Instrument_Handle, uint Timeout, int Array_Size, [In, Out] int[] Slot_Number, [In, Out] double[] Measured_Level, [In, Out] double[] Corrected_Level, [In, Out] double[] Resolution_Bandwidth);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchGsmPtempRef", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchGsmPtempRef(System.IntPtr Instrument_Handle, [In, Out] double[] Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchGsmPowVsTimeValues", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchGsmPowVsTimeValues(System.IntPtr Instrument_Handle, int Measurement, int Modifier, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchGsmExtendedSlotPtempRef", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchGsmExtendedSlotPtempRef(System.IntPtr Instrument_Handle, int Array_Size, [In, Out] int[] Slot_Number, [In, Out] double[] Measured_Level, [In, Out] double[] Corrected_Level, [In, Out] double[] Resolution_Bandwidth);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadGsmSpecModRef", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadGsmSpecModRef(System.IntPtr Instrument_Handle, uint Timeout, [In, Out] double[] Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadGsmModSpectrum", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadGsmModSpectrum(System.IntPtr Instrument_Handle, int Array_Size, uint Timeout, [In, Out] int[] Index, [In, Out] double[] Start_Frequencies, [In, Out] double[] Stop_Frequencies, [In, Out] double[] Levels, [In, Out] double[] Limits, [In, Out] int[] Abs_Rel, [In, Out] int[] Status, out int Num_of_Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchGsmSpecModRef", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchGsmSpecModRef(System.IntPtr Instrument_Handle, [In, Out] double[] Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchGsmModSpectrum", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchGsmModSpectrum(System.IntPtr Instrument_Handle, int Array_Size, [In, Out] int[] Index, [In, Out] double[] Start_Frequencies, [In, Out] double[] Stop_Frequencies, [In, Out] double[] Levels, [In, Out] double[] Limits, [In, Out] int[] Abs_Rel, [In, Out] int[] Status, out int Num_of_Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadGsmSpecTransRef", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadGsmSpecTransRef(System.IntPtr Instrument_Handle, uint Timeout, [In, Out] double[] Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadGsmTransSpectrum", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadGsmTransSpectrum(System.IntPtr Instrument_Handle, int Array_Size, uint Timeout, [In, Out] int[] Index, [In, Out] double[] Start_Frequencies, [In, Out] double[] Stop_Frequencies, [In, Out] double[] Levels, [In, Out] double[] Limits, [In, Out] int[] Abs_Rel, [In, Out] int[] Status, out int Num_of_Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchGsmTransSpectrum", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchGsmTransSpectrum(System.IntPtr Instrument_Handle, int Array_Size, [In, Out] int[] Index, [In, Out] double[] Start_Frequencies, [In, Out] double[] Stop_Frequencies, [In, Out] double[] Levels, [In, Out] double[] Limits, [In, Out] int[] Abs_Rel, [In, Out] int[] Status, out int Num_of_Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ADemodMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int ADemodMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SetADemodState", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetADemodState(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SelectADemodResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int SelectADemodResult(System.IntPtr Instrument_Handle, int Operating_Mode, int Trace);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureADemodTrigger", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureADemodTrigger(System.IntPtr Instrument_Handle, int Trigger_Source, double Trigger_Offset, int Trigger_Polarity, double Trigger_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureADemodSet", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureADemodSet(System.IntPtr Instrument_Handle, double Sample_Rate, int Record_Length, int Trigger_Source, int Trigger_Slope, int Pretrigger_Samples, int Number_Of_Meas);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureADemodFilter", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureADemodFilter(System.IntPtr Instrument_Handle, int Filter_Type, ushort Filter_State, double Filter_Frequency, double AM_FM_Deemphasis_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureADemodFilterWeighting", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureADemodFilterWeighting(System.IntPtr Instrument_Handle, ushort CCITT, ushort CCIR);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureADemodZoom", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureADemodZoom(System.IntPtr Instrument_Handle, ushort Zoom_State, double Start_Time);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureADemodZoomLength", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureADemodZoomLength(System.IntPtr Instrument_Handle, int Zoom_Length_Mode, double Zoom_Length);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureADemodTHDUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureADemodTHDUnit(System.IntPtr Instrument_Handle, int Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ADemAutoTune", CallingConvention = CallingConvention.StdCall)]
            public static extern int ADemAutoTune(System.IntPtr Instrument_Handle, uint Timeout__ms_);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureADemodBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureADemodBandwidth(System.IntPtr Instrument_Handle, double Demodulation_Bandwidth);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureADemodAFCoupling", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureADemodAFCoupling(System.IntPtr Instrument_Handle, int AF_Coupling);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureADemodMeasuringTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureADemodMeasuringTime(System.IntPtr Instrument_Handle, double Measuring_Time);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureADemodAFFrequencyStartStop", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureADemodAFFrequencyStartStop(System.IntPtr Instrument_Handle, double Start_Frequency, double Stop_Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureADemodAFFrequencyCenterSpan", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureADemodAFFrequencyCenterSpan(System.IntPtr Instrument_Handle, double Center_Frequency, double Span);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureADemodAFFullSpan", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureADemodAFFullSpan(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureADemodAFParam", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureADemodAFParam(System.IntPtr Instrument_Handle, double Resolution_Bandwidth);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureADemodRFParam", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureADemodRFParam(System.IntPtr Instrument_Handle, double Resolution_Bandwidth, double Span);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureADemodRFZoom", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureADemodRFZoom(System.IntPtr Instrument_Handle, double Zoom);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAnalogDemodPMPhaseWrap", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAnalogDemodPMPhaseWrap(System.IntPtr Instrument_Handle, int Phase_Wrap);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAnalogDemodPMPhaseUnits", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAnalogDemodPMPhaseUnits(System.IntPtr Instrument_Handle, int PM_Units);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAnalogDemodPMZeroPhaseReferencePoint", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAnalogDemodPMZeroPhaseReferencePoint(System.IntPtr Instrument_Handle, double Reference_Point);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureADemodResultType", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureADemodResultType(System.IntPtr Instrument_Handle, int Demodulation_Type, int[] Result_Types);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetADemodCurrentlySetValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetADemodCurrentlySetValue(System.IntPtr Instrument_Handle, int Value, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetADemodResultValues", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetADemodResultValues(System.IntPtr Instrument_Handle, int Demodulation_Type, int Result_Type, uint Timeout, int Array_Size, [In, Out] double[] Result_Values, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetADemodOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetADemodOffset(System.IntPtr Instrument_Handle, int Result_Type, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetADemodMarkerModulationValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetADemodMarkerModulationValue(System.IntPtr Instrument_Handle, int Marker_Number, int Demodulation_Type, int Detector_Type, int Trace, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetADemodMarkerMeasValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetADemodMarkerMeasValue(System.IntPtr Instrument_Handle, int Marker_Number, int Measurement, int Trace, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FMStereoMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int FMStereoMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SetFMStereoState", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetFMStereoState(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFMStereoTrigger", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFMStereoTrigger(System.IntPtr Instrument_Handle, int Trigger_Source, double Trigger_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFMStereoPhaseNoiseMarker", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFMStereoPhaseNoiseMarker(System.IntPtr Instrument_Handle, int Window, int Marker, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFMStereoDifferenceFrequencyDistortion", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFMStereoDifferenceFrequencyDistortion(System.IntPtr Instrument_Handle, int Window, ushort State, int Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFMStereoIntermodulationDistortion", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFMStereoIntermodulationDistortion(System.IntPtr Instrument_Handle, int Window, ushort State, int Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFMStereoAFAutoScale", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFMStereoAFAutoScale(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFMStereoDisplayConfig", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFMStereoDisplayConfig(System.IntPtr Instrument_Handle, int Window, int Display_Configuration);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFMStereoReferenceDeviation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFMStereoReferenceDeviation(System.IntPtr Instrument_Handle, double Reference_Deviation);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFMStereoResultSummarySetup", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFMStereoResultSummarySetup(System.IntPtr Instrument_Handle, int Channel_Type, int Detector, int Mode, ushort Coupled);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFMStereoMeasToRef", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFMStereoMeasToRef(System.IntPtr Instrument_Handle, int Channel_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFMStereoMeasurementTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFMStereoMeasurementTime(System.IntPtr Instrument_Handle, double Measuring_Time);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFMStereoAFFilter", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFMStereoAFFilter(System.IntPtr Instrument_Handle, int Channel_Type, int High_Pass, int Low_Pass, int Weighting, int Deemphasis, ushort Coupled);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_AllFMStereoAFFiltersOff", CallingConvention = CallingConvention.StdCall)]
            public static extern int AllFMStereoAFFiltersOff(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFMStereoAFRange", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFMStereoAFRange(System.IntPtr Instrument_Handle, int Deviation, double Dev_per_Division);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFMStereoAFRangeTHDUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFMStereoAFRangeTHDUnit(System.IntPtr Instrument_Handle, int Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFMStereoAFRangeDevUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFMStereoAFRangeDevUnit(System.IntPtr Instrument_Handle, int Absolute_Dev_Unit, int Relative_Dev_Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFMStereoTimeDomainZoom", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFMStereoTimeDomainZoom(System.IntPtr Instrument_Handle, ushort Zoom_State, double Start_Time);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFMStereoTimeDomainZoomLength", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFMStereoTimeDomainZoomLength(System.IntPtr Instrument_Handle, int Zoom_Length_Mode, double Zoom_Length);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryFMStereoCarrierFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryFMStereoCarrierFrequency(System.IntPtr Instrument_Handle, out double Carrier_Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureFMStereoTraceModes", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureFMStereoTraceModes(System.IntPtr Instrument_Handle, int Channel_Type, int Measurement_Type, int[] Trace_Modes);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FMStereoDifferenceFrequencyDistortionSearchSignal", CallingConvention = CallingConvention.StdCall)]
            public static extern int FMStereoDifferenceFrequencyDistortionSearchSignal(System.IntPtr Instrument_Handle, int Window, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FMStereoIntermodulationDistortionSearchSignal", CallingConvention = CallingConvention.StdCall)]
            public static extern int FMStereoIntermodulationDistortionSearchSignal(System.IntPtr Instrument_Handle, int Window, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryFMStereoPhaseNoiseResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryFMStereoPhaseNoiseResult(System.IntPtr Instrument_Handle, int Window, int Marker, out double Phase_Noise_Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryFMStereoDifferenceFrequencyDistortionResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryFMStereoDifferenceFrequencyDistortionResult(System.IntPtr Instrument_Handle, int Window, [In, Out] double[] Difference_Frequency_Distortion);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryFMStereoIntermodulationDistortionResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryFMStereoIntermodulationDistortionResult(System.IntPtr Instrument_Handle, int Window, [In, Out] double[] Intermodulation_Distortion);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryFMStereoMeasurementResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryFMStereoMeasurementResults(System.IntPtr Instrument_Handle, [In, Out] double[] Absolute_Deviation, [In, Out] double[] Relative_Deviation, [In, Out] double[] SINAD, [In, Out] double[] THD, [In, Out] double[] Modulation_Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryFMStereoChannelTypeResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryFMStereoChannelTypeResults(System.IntPtr Instrument_Handle, int Window, int Channel_Type, int Measurement_Type, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryFMStereoChannelTypeAllResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryFMStereoChannelTypeAllResults(System.IntPtr Instrument_Handle, int Window, int Channel_Type, [In, Out] double[] Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryFMStereoResultValues", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryFMStereoResultValues(System.IntPtr Instrument_Handle, int Channel_Type, int Measurement_Type, int Trace_Mode, uint Timeout, int Array_Size, [In, Out] double[] Result_Values, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_BTOMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int BTOMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBTOMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBTOMeasurement(System.IntPtr Instrument_Handle, int Geographical_Region, int Channel, int Output_Power_Class, int Samples_Per_Symbols, int Packet_Type, double Antenna_Gain);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBTOAveragingMethod", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBTOAveragingMethod(System.IntPtr Instrument_Handle, double Start_Position, double Stop_Position);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBTOSynchronization", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBTOSynchronization(System.IntPtr Instrument_Handle, ushort Find_Sync, int Sync_Offset, ushort Find_Burst, double Burst_Offset, ushort Search_Length_Auto, double Search_Length);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBTOLAP", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBTOLAP(System.IntPtr Instrument_Handle, int Lowest_24_Bits_of_Dev__Address);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBTOMeasurementMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBTOMeasurementMode(System.IntPtr Instrument_Handle, int Measurement_Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBTONumberOfAdjacentChannels", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBTONumberOfAdjacentChannels(System.IntPtr Instrument_Handle, int Number_of_Adjacent_Channels);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBTOReceiveFilter", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBTOReceiveFilter(System.IntPtr Instrument_Handle, int Filter);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBTOSweepCouplingAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBTOSweepCouplingAuto(System.IntPtr Instrument_Handle, int Sweep_Coupling);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBTOSweepCoupling", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBTOSweepCoupling(System.IntPtr Instrument_Handle, int Sweep_Coupling, double Coupling_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBTOAcquisition", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBTOAcquisition(System.IntPtr Instrument_Handle, ushort Sweep_Mode_Continuous, int Number_Of_Sweeps);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SelectBTOTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int SelectBTOTrace(System.IntPtr Instrument_Handle, int Trace);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBTOTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBTOTrace(System.IntPtr Instrument_Handle, int Trace, int Trace_Mode, int Trace_Detector);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBTOPacketBytesSCO", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBTOPacketBytesSCO(System.IntPtr Instrument_Handle, int Packet_Bytes_SCO);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBTODisplayYAxisScaling", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBTODisplayYAxisScaling(System.IntPtr Instrument_Handle, double Scaling);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBTOAnalogDemodulationZoom", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBTOAnalogDemodulationZoom(System.IntPtr Instrument_Handle, ushort Zoom_State, double Start_Time);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetBTOMesurementStatus", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetBTOMesurementStatus(System.IntPtr Instrument_Handle, out int Status);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetBTOPacketInfo", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetBTOPacketInfo(System.IntPtr Instrument_Handle, out int Type, out int Length);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetBTOOutputPower", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetBTOOutputPower(System.IntPtr Instrument_Handle, int Type, out double Power);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetBTONumberOfExceptions", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetBTONumberOfExceptions(System.IntPtr Instrument_Handle, out int Number_Of_Exceptions);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetBTOPowerOfChannels", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetBTOPowerOfChannels(System.IntPtr Instrument_Handle, int Array_Size, [In, Out] double[] Power, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetBTOFrequencyDeviation", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetBTOFrequencyDeviation(System.IntPtr Instrument_Handle, int Frequency_Deviation, int Type, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetBTORatioAverageFrequencyDeviation", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetBTORatioAverageFrequencyDeviation(System.IntPtr Instrument_Handle, int Type, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetBTOPercentageDeviation", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetBTOPercentageDeviation(System.IntPtr Instrument_Handle, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetBTOPacketDataBits", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetBTOPacketDataBits(System.IntPtr Instrument_Handle, uint Maximum_Time__ms_, int Array_Length, out int Actual_Points, [In, Out] double[] Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadBTOFMTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadBTOFMTrace(System.IntPtr Instrument_Handle, uint Maximum_Time__ms_, int Array_Length, out int Actual_Points, [In, Out] double[] Amplitude);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetBTOInitCarrFreqTol", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetBTOInitCarrFreqTol(System.IntPtr Instrument_Handle, int Type, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetBTOCarrFreqDrift", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetBTOCarrFreqDrift(System.IntPtr Instrument_Handle, int Type, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBTOSENumAdjacentChannels", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBTOSENumAdjacentChannels(System.IntPtr Instrument_Handle, int Number);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_AdjustBTOSEGateSetting", CallingConvention = CallingConvention.StdCall)]
            public static extern int AdjustBTOSEGateSetting(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetBTOChannelPowerOfSEMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetBTOChannelPowerOfSEMeasurement(System.IntPtr Instrument_Handle, int Parameter, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetBTOSEPowerOfAdjacentChannels", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetBTOSEPowerOfAdjacentChannels(System.IntPtr Instrument_Handle, int Array_Size, [In, Out] double[] Power, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetBTOSENumberOfExceptions", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetBTOSENumberOfExceptions(System.IntPtr Instrument_Handle, out int Exceptions);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBTOCFSBlockCount", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBTOCFSBlockCount(System.IntPtr Instrument_Handle, int Block_Count);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetBTOCFSDeviation", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetBTOCFSDeviation(System.IntPtr Instrument_Handle, int Parameter, int Type, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetBTOCFSModulAccuracy", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetBTOCFSModulAccuracy(System.IntPtr Instrument_Handle, int Parameter, int Type, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetBTODifferentialPhaseEncodingValues", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetBTODifferentialPhaseEncodingValues(System.IntPtr Instrument_Handle, int Parameter, out int Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBTORTPMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBTORTPMeasurement(System.IntPtr Instrument_Handle, int Packet_Section, double Start, double Stop);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetBTORelativeTransmissionPower", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetBTORelativeTransmissionPower(System.IntPtr Instrument_Handle, int Parameter, int Modificator, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_PWMMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int PWMMode(System.IntPtr Instrument_Handle, int Window, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePWMMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePWMMeasurement(System.IntPtr Instrument_Handle, int Window, int Measurement_Time, int Coupling, double Frequency, int Average_Count);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePWMRelative", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePWMRelative(System.IntPtr Instrument_Handle, int Window, double Reference_Value, int Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePWMAbsolute", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePWMAbsolute(System.IntPtr Instrument_Handle, int Window, int Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePWMReferenceLevelOffsetState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePWMReferenceLevelOffsetState(System.IntPtr Instrument_Handle, int Window, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePWMExternalPowerTrigger", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePWMExternalPowerTrigger(System.IntPtr Instrument_Handle, int Window, ushort State, double Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePWMBarGraphView", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePWMBarGraphView(System.IntPtr Instrument_Handle, ushort Bar_Graph_View);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePWMDutyCycle", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePWMDutyCycle(System.IntPtr Instrument_Handle, ushort Duty_Cycle, double Duty_Cycle_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePWMAbsoluteUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePWMAbsoluteUnit(System.IntPtr Instrument_Handle, int Window, int Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePowerSensorPowerDisplay", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePowerSensorPowerDisplay(System.IntPtr Instrument_Handle, int Power_Display);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePowerSensorStandard", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePowerSensorStandard(System.IntPtr Instrument_Handle, int Channel_Power_Standard);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePowerSensorUseStandard", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePowerSensorUseStandard(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePWMMeasToRef", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePWMMeasToRef(System.IntPtr Instrument_Handle, int Window);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePWMZero", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePWMZero(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePowerSensorReferenceLevelOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePowerSensorReferenceLevelOffset(System.IntPtr Instrument_Handle, double Offset);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadPWMResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadPWMResult(System.IntPtr Instrument_Handle, int Window, uint Timeout, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchPWMResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchPWMResult(System.IntPtr Instrument_Handle, int Window, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GSMK10Mode", CallingConvention = CallingConvention.StdCall)]
            public static extern int GSMK10Mode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGSMK10DUT", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGSMK10DUT(System.IntPtr Instrument_Handle, int Device_Under_Test);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGSMK10SignalCharacteristics", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGSMK10SignalCharacteristics(System.IntPtr Instrument_Handle, int Band_Class, int ARFCN);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGSMK10LevelSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGSMK10LevelSettings(System.IntPtr Instrument_Handle, int Automatic_Reference_Level, double Reference_Level, double External_Attenuation, int Power_Class, int Static_PCL);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGSMK10CaptureSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGSMK10CaptureSettings(System.IntPtr Instrument_Handle, int Signal_Source, double Capture_Time, int Trigger_Mode, double Trigger_Offset, double Trigger_Level, int Statistic_Count);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGSMK10Synchronization", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGSMK10Synchronization(System.IntPtr Instrument_Handle, int Synchronization, ushort Measure_Only_On_Sync);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGSMK10AutoTrackTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGSMK10AutoTrackTime(System.IntPtr Instrument_Handle, double Auto_Track_Time);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGSMK10AutoSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGSMK10AutoSettings(System.IntPtr Instrument_Handle, int Level, int Frame_Configuration, int Trigger);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGSMK10DemodulationSingleSlotMeasurements", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGSMK10DemodulationSingleSlotMeasurements(System.IntPtr Instrument_Handle, int Slot_To_Measure);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGSMK10DemodulationMultiSlotMeasurements", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGSMK10DemodulationMultiSlotMeasurements(System.IntPtr Instrument_Handle, int Number_Of_Slots_To_Measure, int First_Slot_To_Measure);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGSMK10DemodulationEqualTimeslotLength", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGSMK10DemodulationEqualTimeslotLength(System.IntPtr Instrument_Handle, ushort Equal_Timeslot_Length);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGSMK10Burst", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGSMK10Burst(System.IntPtr Instrument_Handle, int Slot, ushort Active, int Burst_Type, int PCL, int Modulation, int Filter, int TSC);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGSMK10Measurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGSMK10Measurement(System.IntPtr Instrument_Handle, int Measurement_Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGsmK10PVTMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGsmK10PVTMeasurement(System.IntPtr Instrument_Handle, int Burst_Section);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGSMK10PVTFilter", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGSMK10PVTFilter(System.IntPtr Instrument_Handle, int Power_Vs_Time_Filter);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGSMK10ModulationTransientSpectrum", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGSMK10ModulationTransientSpectrum(System.IntPtr Instrument_Handle, ushort Enable_Left_Limit, ushort Enable_Right_Limit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureGSMK10MultiCarrier", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureGSMK10MultiCarrier(System.IntPtr Instrument_Handle, ushort Multi_Carrier_BTS, int Number_Of_Active_Carriers, int BTS_Class, int PvT_Filter);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadGSMK10MeasurementResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadGSMK10MeasurementResults(System.IntPtr Instrument_Handle, uint Timeout, int Measurement, int Modifier, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchGSMK10MeasurementResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchGSMK10MeasurementResults(System.IntPtr Instrument_Handle, int Measurement, int Modifier, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadGSMK10ModulationAccuracyResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadGSMK10ModulationAccuracyResults(System.IntPtr Instrument_Handle, uint Timeout, int Array_Size, [In, Out] double[] Result, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchGSMK10ModulationAccuracyResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchGSMK10ModulationAccuracyResults(System.IntPtr Instrument_Handle, int Array_Size, [In, Out] double[] Result, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadGSMK10BurstSlotResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadGSMK10BurstSlotResults(System.IntPtr Instrument_Handle, uint Timeout, int Slot, int Burst_Selection, int Modifier, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchGSMK10BurstSlotResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchGSMK10BurstSlotResults(System.IntPtr Instrument_Handle, int Slot, int Burst_Selection, int Modifier, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadGSMK10ModulationSpectrumResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadGSMK10ModulationSpectrumResults(System.IntPtr Instrument_Handle, int Array_Size, uint Timeout, [In, Out] int[] Index, [In, Out] double[] Start_Frequencies, [In, Out] double[] Stop_Frequencies, [In, Out] double[] Levels, [In, Out] double[] Limits, [In, Out] int[] Abs_Rel, [In, Out] int[] Status, out int Num_of_Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchGSMK10ModulationSpectrumResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchGSMK10ModulationSpectrumResults(System.IntPtr Instrument_Handle, int Array_Size, [In, Out] int[] Index, [In, Out] double[] Start_Frequencies, [In, Out] double[] Stop_Frequencies, [In, Out] double[] Levels, [In, Out] double[] Limits, [In, Out] int[] Abs_Rel, [In, Out] int[] Status, out int Num_of_Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadGSMK10TransientSpectrumResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadGSMK10TransientSpectrumResults(System.IntPtr Instrument_Handle, int Array_Size, uint Timeout, [In, Out] int[] Index, [In, Out] double[] Start_Frequencies, [In, Out] double[] Stop_Frequencies, [In, Out] double[] Levels, [In, Out] double[] Limits, [In, Out] int[] Abs_Rel, [In, Out] int[] Status, out int Num_of_Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchGSMK10TransientSpectrumResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchGSMK10TransientSpectrumResults(System.IntPtr Instrument_Handle, int Array_Size, [In, Out] int[] Index, [In, Out] double[] Start_Frequencies, [In, Out] double[] Stop_Frequencies, [In, Out] double[] Levels, [In, Out] double[] Limits, [In, Out] int[] Abs_Rel, [In, Out] int[] Status, out int Num_of_Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SpectrogramMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SpectrogramMode(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSpectrogramSweep", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSpectrogramSweep(System.IntPtr Instrument_Handle, ushort Continuous);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSpectrogramDisplayParameters", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSpectrogramDisplayParameters(System.IntPtr Instrument_Handle, int Color, int Size);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSpectrogramMeasurementParameters", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSpectrogramMeasurementParameters(System.IntPtr Instrument_Handle, int History_Depth, ushort Time_Stamp, int Frame_Count, ushort Continue_Frame);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SpectrogramClear", CallingConvention = CallingConvention.StdCall)]
            public static extern int SpectrogramClear(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSpectrogramMarkerFrameNumber", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSpectrogramMarkerFrameNumber(System.IntPtr Instrument_Handle, int Marker_Number, int Frame_Number);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetSpectrogramMarkerFrameNumber", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSpectrogramMarkerFrameNumber(System.IntPtr Instrument_Handle, int Marker_Number, out int Frame_Number);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSpectrogramMarkerSearchArea", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSpectrogramMarkerSearchArea(System.IntPtr Instrument_Handle, int Marker_Number, int Search_Area);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SpectrogramMarkerSearch", CallingConvention = CallingConvention.StdCall)]
            public static extern int SpectrogramMarkerSearch(System.IntPtr Instrument_Handle, int Marker_Number, int Search_Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSpectrogramDeltaMarkerFrameNumber", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSpectrogramDeltaMarkerFrameNumber(System.IntPtr Instrument_Handle, int Delta_Marker_Number, int Frame_Number);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetSpectrogramDeltaMarkerFrameNumber", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSpectrogramDeltaMarkerFrameNumber(System.IntPtr Instrument_Handle, int Delta_Marker, out int Frame_Number);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureSpectrogramDeltaMarkerSearchArea", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSpectrogramDeltaMarkerSearchArea(System.IntPtr Instrument_Handle, int Delta_Marker, int Search_Area);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SpectrogramDeltaMarkerSearch", CallingConvention = CallingConvention.StdCall)]
            public static extern int SpectrogramDeltaMarkerSearch(System.IntPtr Instrument_Handle, int Delta_Marker, int Search_Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetSpectrogramTimeStamp", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSpectrogramTimeStamp(System.IntPtr Instrument_Handle, int Frame, int Array_Size, [In, Out] int[] Part_1, [In, Out] int[] Part_2, [In, Out] int[] Part_3, [In, Out] int[] Part_4, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadSpectrogramTraceData", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadSpectrogramTraceData(System.IntPtr Instrument_Handle, int Array_Size, [In, Out] double[] Results, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_AvionicsMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int AvionicsMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAvionicsDemodulationMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAvionicsDemodulationMode(System.IntPtr Instrument_Handle, int Demodulation_Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAvionicsDistortionResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAvionicsDistortionResult(System.IntPtr Instrument_Handle, int Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAvionicsTHDUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAvionicsTHDUnit(System.IntPtr Instrument_Handle, int Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAvionicsDDMUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAvionicsDDMUnit(System.IntPtr Instrument_Handle, int Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAvionicsInput", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAvionicsInput(System.IntPtr Instrument_Handle, int Window, int Input);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAvionicsDemodulationBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAvionicsDemodulationBandwidth(System.IntPtr Instrument_Handle, int Window, ushort Auto, double Demodulation_Bandwidth);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAvionicsBasebandInput", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAvionicsBasebandInput(System.IntPtr Instrument_Handle, int Level, double Sensitivity);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAvionicsAFFrequencyStartStop", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAvionicsAFFrequencyStartStop(System.IntPtr Instrument_Handle, double Start_Frequency, double Stop_Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAvionicsAFFrequencyCenterSpan", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAvionicsAFFrequencyCenterSpan(System.IntPtr Instrument_Handle, double Center_Frequency, double Span);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAvionicsAFFullSpan", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAvionicsAFFullSpan(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAvionicsAFParam", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAvionicsAFParam(System.IntPtr Instrument_Handle, double Resolution_Bandwidth);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureAvionicsDistortionMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAvionicsDistortionMeasurement(System.IntPtr Instrument_Handle, ushort SHD_State, double SHD_Harmonic_Frequency, int THD_Fundamental_Frequency, double THD_Max_Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryAvionicsRFFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryAvionicsRFFrequency(System.IntPtr Instrument_Handle, out double RF_Frequency_Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryAvionicsCarrierOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryAvionicsCarrierOffset(System.IntPtr Instrument_Handle, out double Carrier_Offset_Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryAvionicsRFLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryAvionicsRFLevel(System.IntPtr Instrument_Handle, out double RF_Level_Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryAvionicsAM", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryAvionicsAM(System.IntPtr Instrument_Handle, int AM_Depth, out double AM_Depth_Result, int AM_Frequency, out double AM_Frequency_Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryAvionicsTHD", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryAvionicsTHD(System.IntPtr Instrument_Handle, int THD, out double THD_Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryAvionicsDifferenceInDepth", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryAvionicsDifferenceInDepth(System.IntPtr Instrument_Handle, out double DDM_Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryAvionicsFM", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryAvionicsFM(System.IntPtr Instrument_Handle, out double FM_Deviation_Result, out double FM_Frequency_Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryAvionicsPhase", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryAvionicsPhase(System.IntPtr Instrument_Handle, out double Phase_Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryAvionicsInputLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryAvionicsInputLevel(System.IntPtr Instrument_Handle, out double Input_Level_Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryAvionicsSDM", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryAvionicsSDM(System.IntPtr Instrument_Handle, out double SDM_Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryAvionicsSHD", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryAvionicsSHD(System.IntPtr Instrument_Handle, out double SHD_Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CATVMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int CATVMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVChannelSetup", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVChannelSetup(System.IntPtr Instrument_Handle, string Channel_Table_Name, int Measurement_Channel);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVChannelFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVChannelFrequency(System.IntPtr Instrument_Handle, double RF_Frequency, int Axis_Labeling);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVChannelFrequencyStepSize", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVChannelFrequencyStepSize(System.IntPtr Instrument_Handle, double Step_Size);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVModulationStandard", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVModulationStandard(System.IntPtr Instrument_Handle, string Modulation_Standard, int Signal_Type, int Sideband_Position);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CATVChannelAdjustAtt", CallingConvention = CallingConvention.StdCall)]
            public static extern int CATVChannelAdjustAtt(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CATVAutoscale", CallingConvention = CallingConvention.StdCall)]
            public static extern int CATVAutoscale(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVMarkerState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVMarkerState(System.IntPtr Instrument_Handle, int Marker_Number, ushort Marker_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVDeltaMarkerState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVDeltaMarkerState(System.IntPtr Instrument_Handle, int Delta_Marker_Number, ushort Delta_Marker_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVAMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVAMeasurement(System.IntPtr Instrument_Handle, int Measurement);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVAModulStandard", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVAModulStandard(System.IntPtr Instrument_Handle, int TV_Standard, int Sound_System, int Group_Delay, int Color_System, int Bar_Field, int Bar_Line, int Bar_Line_Type, int Quiet_Field, int Quiet_Line);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVAVideoScopeLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVAVideoScopeLine(System.IntPtr Instrument_Handle, int Line);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVAVideoScopeField", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVAVideoScopeField(System.IntPtr Instrument_Handle, int Active_Field);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVACNMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVACNMeasurement(System.IntPtr Instrument_Handle, int Meas_Carrier, int Measurement_Method, double Noise_Reference_Bandwidth);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVACNReferencePower", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVACNReferencePower(System.IntPtr Instrument_Handle, int Reference_Power, int Reference_Channel, double Power_Manual);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVACNMeasurementFrequencies", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVACNMeasurementFrequencies(System.IntPtr Instrument_Handle, int Table_Row, ushort State, double Center_Frequency, double Span);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVACNNoiseFloorCorrection", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVACNNoiseFloorCorrection(System.IntPtr Instrument_Handle, ushort Noise_Floor_Correction, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CATVACNNextMeasFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int CATVACNNextMeasFrequency(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVACSOMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVACSOMeasurement(System.IntPtr Instrument_Handle, int Meas_Carrier, int Measurement_Method);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVACSOReferencePower", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVACSOReferencePower(System.IntPtr Instrument_Handle, int Reference_Power, int Reference_Channel, double Power_Manual);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVACSOMeasurementFrequencies", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVACSOMeasurementFrequencies(System.IntPtr Instrument_Handle, int Table_Row, ushort State, double Center_Frequency, double Span);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVACSONoiseFloorCorrection", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVACSONoiseFloorCorrection(System.IntPtr Instrument_Handle, ushort Noise_Floor_Correction, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CATVACSONextMeasFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int CATVACSONextMeasFrequency(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVACTBMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVACTBMeasurement(System.IntPtr Instrument_Handle, int Meas_Carrier);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVACTBReferencePower", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVACTBReferencePower(System.IntPtr Instrument_Handle, int Reference_Power, int Reference_Channel, double Power_Manual);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVACTBMeasurementFrequencies", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVACTBMeasurementFrequencies(System.IntPtr Instrument_Handle, int Table_Row, ushort State, double Center_Frequency, double Span);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVACTBNoiseFloorCorrection", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVACTBNoiseFloorCorrection(System.IntPtr Instrument_Handle, ushort Noise_Floor_Correction, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CATVACTBNextMeasFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int CATVACTBNextMeasFrequency(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVACarrierLimit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVACarrierLimit(System.IntPtr Instrument_Handle, int Measurement, int Limit_Type, double Limit_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVACarrierRatioLimit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVACarrierRatioLimit(System.IntPtr Instrument_Handle, int Measurement, int Limit_Type, double Limit_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVAHumLimit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVAHumLimit(System.IntPtr Instrument_Handle, int Limit_Type, double Limit_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVAVisionModLimit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVAVisionModLimit(System.IntPtr Instrument_Handle, int Limit, int Limit_Type, double Limit_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVAVisionModulationCarrierPowerUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVAVisionModulationCarrierPowerUnit(System.IntPtr Instrument_Handle, int Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVAHumUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVAHumUnit(System.IntPtr Instrument_Handle, int Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVDMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVDMeasurement(System.IntPtr Instrument_Handle, int Measurement);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVDModulStandard", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVDModulStandard(System.IntPtr Instrument_Handle, int TV_Standard, int Roll_Off, int Constellation_Parameter, double Symbol_Rate);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVDShoulderAttenuation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVDShoulderAttenuation(System.IntPtr Instrument_Handle, ushort Shoulder_Attenuation);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVDEchoPattern", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVDEchoPattern(System.IntPtr Instrument_Handle, ushort Zoom_State, int Zoom_Factor, double Velocity_Factor);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVDModulErrorZoom", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVDModulErrorZoom(System.IntPtr Instrument_Handle, int Zoom);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVDOverviewZoom", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVDOverviewZoom(System.IntPtr Instrument_Handle, int Zoom);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVDConstellationDiagramZoom", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVDConstellationDiagramZoom(System.IntPtr Instrument_Handle, int Zoom_Diagram);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVDEqualizer", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVDEqualizer(System.IntPtr Instrument_Handle, ushort State, ushort Freeze);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVDSignalStatisticsNoSamples", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVDSignalStatisticsNoSamples(System.IntPtr Instrument_Handle, int No_of_Samples);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVDSignalStatisticsScaling", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVDSignalStatisticsScaling(System.IntPtr Instrument_Handle, int Scaling_Mode, double Scaling_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVDCCDFPercentMark", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVDCCDFPercentMark(System.IntPtr Instrument_Handle, int Marker_Number, double Position_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CATVDResetEqualizer", CallingConvention = CallingConvention.StdCall)]
            public static extern int CATVDResetEqualizer(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVDOverLimit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVDOverLimit(System.IntPtr Instrument_Handle, int Measurement, int Limit_Type, double Limit_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVDChanPowLimit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVDChanPowLimit(System.IntPtr Instrument_Handle, int Limit_Type, double Limit_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVDModErrLimit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVDModErrLimit(System.IntPtr Instrument_Handle, int Measurement, int Limit_Type, double Limit_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVDEchoPatternUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVDEchoPatternUnit(System.IntPtr Instrument_Handle, int Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVDUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVDUnit(System.IntPtr Instrument_Handle, int Measurement, int Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVTVMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVTVMeasurement(System.IntPtr Instrument_Handle, int Measurement);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureCATVTVStandardForTitlMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureCATVTVStandardForTitlMeasurement(System.IntPtr Instrument_Handle, string Modulation_Standard, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryCATVACarrierResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryCATVACarrierResult(System.IntPtr Instrument_Handle, int Measurement, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryCATVACarrierAllResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryCATVACarrierAllResults(System.IntPtr Instrument_Handle, [In, Out] double[] Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryCATVACarrierRatioResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryCATVACarrierRatioResult(System.IntPtr Instrument_Handle, int Measurement, int Result_Type, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryCATVACarrierAllRatioResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryCATVACarrierAllRatioResults(System.IntPtr Instrument_Handle, int Measurement, [In, Out] double[] Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryCATVAHumResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryCATVAHumResult(System.IntPtr Instrument_Handle, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryCATVAVisionModulationResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryCATVAVisionModulationResult(System.IntPtr Instrument_Handle, int Measurement, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryCATVAVisionAllModulationResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryCATVAVisionAllModulationResults(System.IntPtr Instrument_Handle, [In, Out] double[] Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryCATVACarrierLimitResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryCATVACarrierLimitResult(System.IntPtr Instrument_Handle, int Measurement, int Limit_Type, out int Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryCATVACarrierRatioLimitResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryCATVACarrierRatioLimitResult(System.IntPtr Instrument_Handle, int Measurement, int Limit_Type, out int Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryCATVAHumLimitResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryCATVAHumLimitResult(System.IntPtr Instrument_Handle, int Limit_Type, out int Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryCATVAVisionModulationLimitResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryCATVAVisionModulationLimitResult(System.IntPtr Instrument_Handle, int Measurement, int Limit_Type, out int Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryCATVDSpectrumResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryCATVDSpectrumResult(System.IntPtr Instrument_Handle, int Measurement, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryCATVDChanelPowerResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryCATVDChanelPowerResult(System.IntPtr Instrument_Handle, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryCATVDOverviewResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryCATVDOverviewResult(System.IntPtr Instrument_Handle, int Measurement, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryCATVDOverviewAllResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryCATVDOverviewAllResults(System.IntPtr Instrument_Handle, [In, Out] double[] Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryCATVDModulationErrorsResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryCATVDModulationErrorsResult(System.IntPtr Instrument_Handle, int Measurement, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryCATVDModulationErrorsAllResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryCATVDModulationErrorsAllResults(System.IntPtr Instrument_Handle, [In, Out] double[] Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryCATVDSignalStatisticsResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryCATVDSignalStatisticsResult(System.IntPtr Instrument_Handle, int Trace_Number, int Result_Type, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryCATVDSignalStatisticsAllResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryCATVDSignalStatisticsAllResults(System.IntPtr Instrument_Handle, int Trace_Number, [In, Out] double[] Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryCATVDOverviewLimitResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryCATVDOverviewLimitResult(System.IntPtr Instrument_Handle, int Measurement, int Limit_Type, out int Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryCATVDChannelPowerLimitResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryCATVDChannelPowerLimitResult(System.IntPtr Instrument_Handle, out int Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryCATVDModulationErrorsLimitResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryCATVDModulationErrorsLimitResult(System.IntPtr Instrument_Handle, int Measurement, int Limit_Type, out int Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_NoiseMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int NoiseMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureNoiseFrequencySettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureNoiseFrequencySettings(System.IntPtr Instrument_Handle, double Start_Frequency, double Stop_Frequency, double Step_Frequency, int Mode, double Fixed_LO, double Fixed_IF, double Image_Rejection, double Fixed_Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureNoiseFrequencyTable", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureNoiseFrequencyTable(System.IntPtr Instrument_Handle, int Number_Of_Values, double[] Receiver_Frequency_Values, double[] Local_Oscilator_Frequency_Val_, double[] Intermediate_Frequency_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureNoiseMeasurementMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureNoiseMeasurementMode(System.IntPtr Instrument_Handle, int Measurement_Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureNoiseCalibration", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureNoiseCalibration(System.IntPtr Instrument_Handle, ushort _2nd_Stage_Correction);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureNoiseAnalyzerSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureNoiseAnalyzerSettings(System.IntPtr Instrument_Handle, double Resolution_Bandwidth, double Sweep_Time, double Settling_Time, int Average, double RF_Attenuation, double Range, ushort Automatic_Ref_Level, double Ref_Level, ushort Pre_selector, ushort Pre_amplifier__B2_B25_);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureNoiseGeneratorSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureNoiseGeneratorSettings(System.IntPtr Instrument_Handle, ushort Automatic_Control, double Level, string Generator_Select, int GPIB_Address, ushort Init_Before_Measurement);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureNoiseGeneratorFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureNoiseGeneratorFrequency(System.IntPtr Instrument_Handle, double Offset_Nr__1, double Offset_Nr__2, int Numerator, int Denominator);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureNoiseENRSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureNoiseENRSettings(System.IntPtr Instrument_Handle, int Selection_Mode, double ENR_Constant, double Room_Temperature);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureNoiseENRTable", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureNoiseENRTable(System.IntPtr Instrument_Handle, int Number_Of_Values, double[] Receiver_Frequency_Values, double[] ENR_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureNoiseLossInputSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureNoiseLossInputSettings(System.IntPtr Instrument_Handle, int Selection_Mode, double Loss_Input_Constant);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureNoiseLossInputTable", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureNoiseLossInputTable(System.IntPtr Instrument_Handle, int Number_Of_Values, double[] Receiver_Frequency_Values, double[] Loss_Input_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureNoiseLossOutputSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureNoiseLossOutputSettings(System.IntPtr Instrument_Handle, int Selection_Mode, double Loss_Output_Constant);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureNoiseLossOutputTable", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureNoiseLossOutputTable(System.IntPtr Instrument_Handle, int Number_Of_Values, double[] Receiver_Frequency_Values, double[] Loss_Output_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureNoiseResultSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureNoiseResultSettings(System.IntPtr Instrument_Handle, int Combined_Trace_Display, ushort Table_Form_Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureNoiseTraceSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureNoiseTraceSettings(System.IntPtr Instrument_Handle, int Y_Axis, ushort Symbols, ushort Automatic_Scaling, double Min_Y_Axis, double Max_Y_Axis);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureNoiseGainTraceSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureNoiseGainTraceSettings(System.IntPtr Instrument_Handle, ushort Y_Axis, ushort Symbols, ushort Automatic_Scaling, double Min_Y_Axis, double Max_Y_Axis);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureNoiseTraceMemoryDisplay", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureNoiseTraceMemoryDisplay(System.IntPtr Instrument_Handle, int Memory, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureNoiseCurrentResultsDisplay", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureNoiseCurrentResultsDisplay(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureXAxisFrequencyDisplay", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureXAxisFrequencyDisplay(System.IntPtr Instrument_Handle, int Measurement_Results, int Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CreateNoiseLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int CreateNoiseLimitLine(System.IntPtr Instrument_Handle, int Limit, string Name, string Comment);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_DefineNoiseLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int DefineNoiseLimitLine(System.IntPtr Instrument_Handle, int Limit, int Result_Type, int Type, int Count, double[] X_Axis, double[] Amplitude);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_EnableNoiseLimitCheck", CallingConvention = CallingConvention.StdCall)]
            public static extern int EnableNoiseLimitCheck(System.IntPtr Instrument_Handle, int Limit, int Type, ushort Limit_Enabled, ushort Check_Enabled);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_MoveNoiseLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int MoveNoiseLimitLine(System.IntPtr Instrument_Handle, int Limit, int Type, double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CopyNoiseLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int CopyNoiseLimitLine(System.IntPtr Instrument_Handle, int Limit, int Copy_To);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ClearNoiseLimitLineResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int ClearNoiseLimitLineResults(System.IntPtr Instrument_Handle, int Limit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_DeleteNoiseLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int DeleteNoiseLimitLine(System.IntPtr Instrument_Handle, int Limit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SaveNoiseTraceDataToMemory", CallingConvention = CallingConvention.StdCall)]
            public static extern int SaveNoiseTraceDataToMemory(System.IntPtr Instrument_Handle, int Memory);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchNoiseArray", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchNoiseArray(System.IntPtr Instrument_Handle, int Modifier, int Array_Size, [In, Out] double[] Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchNoiseScalar", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchNoiseScalar(System.IntPtr Instrument_Handle, int Modifier, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchNoiseMemoryArray", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchNoiseMemoryArray(System.IntPtr Instrument_Handle, int Modifier, int Memory, int Array_Size, [In, Out] double[] Meas_Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetNoiseLimitCheckResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetNoiseLimitCheckResult(System.IntPtr Instrument_Handle, int Limit, out int State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_PhaseMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int PhaseMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePhaseSignalSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePhaseSignalSettings(System.IntPtr Instrument_Handle, double Frequency, double Level, ushort Sweep_Forward, ushort Verify_Frequency___Level, double Frequency_Tolerance, double Level_Tolerance);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePhaseFrequencyTolerance", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePhaseFrequencyTolerance(System.IntPtr Instrument_Handle, double Absolute_Frequency_Tolerance);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePhaseAFCSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePhaseAFCSettings(System.IntPtr Instrument_Handle, ushort Track_Frequency, ushort Track_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePhaseDisplaySettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePhaseDisplaySettings(System.IntPtr Instrument_Handle, double X_Axis_Start, double X_Axis_Stop, ushort Autoscale_Once, double Y_Axis_Top, double Y_Axis_Range);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePhaseTraceSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePhaseTraceSettings(System.IntPtr Instrument_Handle, double Trace_Offset, int Trace_Smoothing, int Smoothing_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePhaseResidualCalculations", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePhaseResidualCalculations(System.IntPtr Instrument_Handle, ushort Use_Measurement_Settings, double Evaluate_From, double Evaluate_To);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePhaseSpotNoiseSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePhaseSpotNoiseSettings(System.IntPtr Instrument_Handle, ushort Enable, double Offset_Frequency_1, double Offset_Frequency_2, double Offset_Frequency_3, double Offset_Frequency_4);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePhaseAnalyzerSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePhaseAnalyzerSettings(System.IntPtr Instrument_Handle, ushort Pre_selector, ushort Pre_amplifier__B2_B25_B23_);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SetPhaseAllSpotNoiseMarkersOff", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetPhaseAllSpotNoiseMarkersOff(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePhaseSweepSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePhaseSweepSettings(System.IntPtr Instrument_Handle, int Sweep_Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePhaseSpanSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePhaseSpanSettings(System.IntPtr Instrument_Handle, double Start_Offset, double Stop_Offset);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePhaseCarrierFrequencyOffsetTable", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePhaseCarrierFrequencyOffsetTable(System.IntPtr Instrument_Handle, int Sub_band, double RBW, int Average, int FFT);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigurePhasePresetSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigurePhasePresetSettings(System.IntPtr Instrument_Handle, double RBW, int Average, int FFT);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CreatePhaseLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int CreatePhaseLimitLine(System.IntPtr Instrument_Handle, int Window, int Limit, string Name, string Comment, int Assign_to_Trace);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_DefinePhaseLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int DefinePhaseLimitLine(System.IntPtr Instrument_Handle, int Limit, int Type, int Count, double[] X_Axis, double[] Amplitude);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_EnablePhaseLimitCheck", CallingConvention = CallingConvention.StdCall)]
            public static extern int EnablePhaseLimitCheck(System.IntPtr Instrument_Handle, int Window, int Limit, int Type, ushort Limit_Enabled, ushort Check_Enabled);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_MovePhaseLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int MovePhaseLimitLine(System.IntPtr Instrument_Handle, int Limit, int Type, double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CopyPhaseLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int CopyPhaseLimitLine(System.IntPtr Instrument_Handle, int Limit, int Copy_To);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ClearPhaseLimitLineResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int ClearPhaseLimitLineResults(System.IntPtr Instrument_Handle, int Limit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_DeletePhaseLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int DeletePhaseLimitLine(System.IntPtr Instrument_Handle, int Limit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_TracePhaseSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int TracePhaseSettings(System.IntPtr Instrument_Handle, int Trace_Number, ushort Trace_State, int Trace_Mode, ushort Smoothing);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReferencePhaseMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReferencePhaseMeasurement(System.IntPtr Instrument_Handle, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetPhaseEstimatedMeasurementTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPhaseEstimatedMeasurementTime(System.IntPtr Instrument_Handle, out double Estimated_Measurement_Time);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadPhaseTraceData", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadPhaseTraceData(System.IntPtr Instrument_Handle, int Source_Trace, out int No_of_Points, [In, Out] double[] Trace_Data_X, [In, Out] double[] Trace_Data_Y);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchPhaseResidualResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchPhaseResidualResults(System.IntPtr Instrument_Handle, int Modifier, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetPhaseSpotNoiseYPosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPhaseSpotNoiseYPosition(System.IntPtr Instrument_Handle, int Spot_Noise_Marker_Number, out double Spot_Noise_Marker_Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetPhaseLimitCheckResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPhaseLimitCheckResult(System.IntPtr Instrument_Handle, int Window, int Limit, out int State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SetVSAMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetVSAMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SetVSAFactoryDefaults", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetVSAFactoryDefaults(System.IntPtr Instrument_Handle, int Defaults);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_VSARefLevelAdjust", CallingConvention = CallingConvention.StdCall)]
            public static extern int VSARefLevelAdjust(System.IntPtr Instrument_Handle, int Window, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAMarkerState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAMarkerState(System.IntPtr Instrument_Handle, int Window, int Marker_Number, ushort Marker_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAScreenCoupling", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAScreenCoupling(System.IntPtr Instrument_Handle, int Window, int Marker_Number, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAMarkerSearch", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAMarkerSearch(System.IntPtr Instrument_Handle, int Window, int Search);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAMarkerPeakSearch", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAMarkerPeakSearch(System.IntPtr Instrument_Handle, int Window, int Marker_Number);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_VSAAdjustReferenceLevelToFullScaleLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int VSAAdjustReferenceLevelToFullScaleLevel(System.IntPtr Instrument_Handle, ushort Adjust_Reference_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAFullScaleLevelUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAFullScaleLevelUnit(System.IntPtr Instrument_Handle, int Full_Scale_Level_Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_PresetVSAScreens", CallingConvention = CallingConvention.StdCall)]
            public static extern int PresetVSAScreens(System.IntPtr Instrument_Handle, int Window);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSADigitalStandard", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSADigitalStandard(System.IntPtr Instrument_Handle, int Window, string Standard, ushort Step_Width_Auto, double Step_Size);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_VSADigitalStandardSave", CallingConvention = CallingConvention.StdCall)]
            public static extern int VSADigitalStandardSave(System.IntPtr Instrument_Handle, int Window, string User_Defined_Standard, string Group, string Comment, string Prefix);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_VSADigitalStandardDelete", CallingConvention = CallingConvention.StdCall)]
            public static extern int VSADigitalStandardDelete(System.IntPtr Instrument_Handle, int Window, string Digital_Standard);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_VSADigitalStandardDefaults", CallingConvention = CallingConvention.StdCall)]
            public static extern int VSADigitalStandardDefaults(System.IntPtr Instrument_Handle, int Window);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetVSADigitalStandardCatalog", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetVSADigitalStandardCatalog(System.IntPtr Instrument_Handle, int Window, out int Number_of_Digital_Standards, int Buffer_Size, System.Text.StringBuilder Digital_Standards_List);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSABurstSearchAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSABurstSearchAuto(System.IntPtr Instrument_Handle, int Window, int Burst_Search);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSABurstSearch", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSABurstSearch(System.IntPtr Instrument_Handle, int Window, ushort Search_State, int Burst_Search_Mode, ushort Threshold_Auto, int Threshold_Mode, double Threshold_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSABurstSearchHysteresis", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSABurstSearchHysteresis(System.IntPtr Instrument_Handle, int Window, double Hysteresis_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSABurstSearchExpertSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSABurstSearchExpertSettings(System.IntPtr Instrument_Handle, int Window, int Useful_Length, int Max_Length, int Run_In, int Run_Out, int Min_Gaplen);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSABurstSearchAutoConfiguration", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSABurstSearchAutoConfiguration(System.IntPtr Instrument_Handle, int Window, ushort Auto_Configuration, int Tolerance, int Min_Gaplen);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetVSABurstFound", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetVSABurstFound(System.IntPtr Instrument_Handle, out ushort Found);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAPatternSearchAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAPatternSearchAuto(System.IntPtr Instrument_Handle, int Window, int Pattern_Search);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAPatternSearch", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAPatternSearch(System.IntPtr Instrument_Handle, int Window, ushort Search_State, int Pattern_Search_Mode, string Pattern);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAPatternSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAPatternSettings(System.IntPtr Instrument_Handle, int Window, string Pattern_Name, string Description, string Comment, int Degree_of_Modulation, string Pattern_Data);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAPatternOperations", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAPatternOperations(System.IntPtr Instrument_Handle, int Window, int Operation, string Pattern_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAPatternSearchAutoConfiguration", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAPatternSearchAutoConfiguration(System.IntPtr Instrument_Handle, int Window, ushort Auto_Configuration, double IQ_Correlation_Threshold);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetVSAPatternFound", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetVSAPatternFound(System.IntPtr Instrument_Handle, int Window, int Buffer_Size, System.Text.StringBuilder Found);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetVSAPatternCatalog", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetVSAPatternCatalog(System.IntPtr Instrument_Handle, int Window, int Selection, out int Number_of_Patterns, int Buffer_Size, System.Text.StringBuilder Patterns_List);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAEqualizer", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAEqualizer(System.IntPtr Instrument_Handle, int Window, double Speed_of_Conversion, int Length);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAEqualizerMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAEqualizerMode(System.IntPtr Instrument_Handle, int Window, ushort State, ushort Training);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_VSAEqualizerOperations", CallingConvention = CallingConvention.StdCall)]
            public static extern int VSAEqualizerOperations(System.IntPtr Instrument_Handle, int Window, int Operation, string Equalizer_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSASignalType", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSASignalType(System.IntPtr Instrument_Handle, int Window, int Signal_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAPatternSignalState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAPatternSignalState(System.IntPtr Instrument_Handle, int Window, ushort Pattern_Signal);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAPatternOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAPatternOffset(System.IntPtr Instrument_Handle, int Window, ushort Pattern_Offset_State, int Pattern_Offset);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAModulation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAModulation(System.IntPtr Instrument_Handle, int Window, double Symbol_Rate, int Mode, string Mapping_Name, ushort Signal_Filtering, int Points_Symbol);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAFSKModulation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAFSKModulation(System.IntPtr Instrument_Handle, int Window, ushort Deviation_Compensation, double FSK_Deviation);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAEVMModulation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAEVMModulation(System.IntPtr Instrument_Handle, int Window, int Calculation);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAEVMOffsetState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAEVMOffsetState(System.IntPtr Instrument_Handle, int Window, ushort Calculation);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAModulationFilters", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAModulationFilters(System.IntPtr Instrument_Handle, int Window, string TX_Filter, string Receive_Filter, string Meas_Filter);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAFilterCharacteristic", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAFilterCharacteristic(System.IntPtr Instrument_Handle, int Window, double Alpha_BT);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetVSAModulationFilterCatalog", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetVSAModulationFilterCatalog(System.IntPtr Instrument_Handle, int Window, out int Number_of_Filters, int Buffer_Size, System.Text.StringBuilder Modulation_Filters_List);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetVSAModulationMappingCatalog", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetVSAModulationMappingCatalog(System.IntPtr Instrument_Handle, int Window, out int Number_of_Mappings, int Buffer_Size, System.Text.StringBuilder Mappings_List);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetVSAModulationUQAMLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetVSAModulationUQAMLevel(System.IntPtr Instrument_Handle, int Window, out int Specific_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSATXFilter", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSATXFilter(System.IntPtr Instrument_Handle, int Window, int Type, double Alpha_BT);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSADemodulation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSADemodulation(System.IntPtr Instrument_Handle, int Window, int Recording_Length, double Record_Length, ushort Multiburst_Search, int Side_Band, ushort Normalize, int Points_Symbol);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAResultLength", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAResultLength(System.IntPtr Instrument_Handle, int Window, int Result_Length);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAEvaluationLinesState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAEvaluationLinesState(System.IntPtr Instrument_Handle, int Window, ushort Evaluation_Lines_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAEvaluationLines", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAEvaluationLines(System.IntPtr Instrument_Handle, int Window, int Evaluation_Line, int Line_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAForceWBPath", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAForceWBPath(System.IntPtr Instrument_Handle, int Window, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSACompensation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSACompensation(System.IntPtr Instrument_Handle, int Window, int Compensate_For, ushort Compensation_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAMeasurementFilter", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAMeasurementFilter(System.IntPtr Instrument_Handle, int Window, ushort Auto_Measurement_Filter, int Type, double Alpha_BT);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAUnitTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAUnitTime(System.IntPtr Instrument_Handle, int Window, int Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAUnitAngle", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAUnitAngle(System.IntPtr Instrument_Handle, int Window, int Unit_Angle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAUnitPower", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAUnitPower(System.IntPtr Instrument_Handle, int Window, int Unit_Power);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSADisplayAlignment", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSADisplayAlignment(System.IntPtr Instrument_Handle, int Window, int Trace, int Alignment, double Align_Value, int Align_Offset);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSADisplayFitTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSADisplayFitTrace(System.IntPtr Instrument_Handle, int Window, int Trace, int Fit, int Symbol_Number, int Pattern_Position);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSADisplayTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSADisplayTrace(System.IntPtr Instrument_Handle, int Window, int Trace, int Symbol, int Spacing);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSADisplayXAxis", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSADisplayXAxis(System.IntPtr Instrument_Handle, int Window, int Trace, int Quantize, int Scaling);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSADisplayXAxisReference", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSADisplayXAxisReference(System.IntPtr Instrument_Handle, int Window, int Trace, int Reference_Value, int Reference_Position);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSADisplayYAxis", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSADisplayYAxis(System.IntPtr Instrument_Handle, int Window, double Scaling, double Reference_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSADisplayYAxisRefPosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSADisplayYAxisRefPosition(System.IntPtr Instrument_Handle, int Window, double Reference_Position);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSADisplayPointPerSymbol", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSADisplayPointPerSymbol(System.IntPtr Instrument_Handle, int Window, int Points_Per_Symbol, int Number_of_Points_Per_Symbol);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSACaptureOversampling", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSACaptureOversampling(System.IntPtr Instrument_Handle, int Window, int Oversampling, int Statistical_Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSATraceEvaluation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSATraceEvaluation(System.IntPtr Instrument_Handle, int Window, int Trace_Evaluation);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSADisplayWindowState", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSADisplayWindowState(System.IntPtr Instrument_Handle, int Window, ushort Window_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAMultipleEvaluation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAMultipleEvaluation(System.IntPtr Instrument_Handle, int Window, ushort Capture_Auto, ushort Multiburst_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAMultipleEvaluationZoom", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAMultipleEvaluationZoom(System.IntPtr Instrument_Handle, int Window, int Zoom_Start, int Zoom_Length);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_VSASearchBurstInCaptureRam", CallingConvention = CallingConvention.StdCall)]
            public static extern int VSASearchBurstInCaptureRam(System.IntPtr Instrument_Handle, int Window, int Action);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_VSAImport", CallingConvention = CallingConvention.StdCall)]
            public static extern int VSAImport(System.IntPtr Instrument_Handle, int Type, string Name, string Path);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_VSAExport", CallingConvention = CallingConvention.StdCall)]
            public static extern int VSAExport(System.IntPtr Instrument_Handle, int Type, string Name, string Path);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAFileExport", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAFileExport(System.IntPtr Instrument_Handle, int Window, int Trace, int Decimal_Separator, ushort Header, int Data_Mode, int Data_Raw, string File_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAFileImport", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAFileImport(System.IntPtr Instrument_Handle, string File_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAMeasurementResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAMeasurementResults(System.IntPtr Instrument_Handle, int Window, int Measurement_Signal, int Result_Display, ushort Spectrum, ushort Statistics);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAErrorResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAErrorResults(System.IntPtr Instrument_Handle, int Window, int Result_Display, ushort Spectrum, ushort Statistics);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSASymbolTableResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSASymbolTableResults(System.IntPtr Instrument_Handle, int Window);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAEqualizerResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAEqualizerResults(System.IntPtr Instrument_Handle, int Window, int Result_Display);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSARawSignal", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSARawSignal(System.IntPtr Instrument_Handle, int Window, int Result_Display, ushort Spectrum, ushort Statistics);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSAModulationAccuracy", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSAModulationAccuracy(System.IntPtr Instrument_Handle, int Window);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetVSAXAxisFirstValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetVSAXAxisFirstValue(System.IntPtr Instrument_Handle, int Window, int Trace, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetVSAResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetVSAResult(System.IntPtr Instrument_Handle, int Window, int Type, int Modifier, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetVSAAllResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetVSAAllResults(System.IntPtr Instrument_Handle, int Window, [In, Out] double[] Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetVSAXMarkerPosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetVSAXMarkerPosition(System.IntPtr Instrument_Handle, int Window, int Marker, out double Position);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadVSATraceData", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadVSATraceData(System.IntPtr Instrument_Handle, int Window, int Source_Trace, int Array_Size, [In, Out] double[] Trace_Data, out int No_of_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureVSACompressionPointMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureVSACompressionPointMeasurement(System.IntPtr Instrument_Handle, int Window, ushort State, double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetVSACompressionPointResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetVSACompressionPointResults(System.IntPtr Instrument_Handle, int Window, int Type, [In, Out] double[] Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Set3GPPBSMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int Set3GPPBSMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Configure3GPPBSMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int Configure3GPPBSMeasurement(System.IntPtr Instrument_Handle, int Scrambling_Code, int Power_Reference, int Compare_Mode, int Sync_Type, int Side_Band, ushort Normalize, ushort HSDPA_UPA, double Inactive_Channel_Treshold, ushort Invert_Q, int Antenna, ushort Power_Difference, int Constellation_Parameter_B, int Timing_Offset);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Configure3GPPBSMultiframe", CallingConvention = CallingConvention.StdCall)]
            public static extern int Configure3GPPBSMultiframe(System.IntPtr Instrument_Handle, int Capture_Length, int Frame_To_Analyze);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Configure3GPPBSMulticarrier", CallingConvention = CallingConvention.StdCall)]
            public static extern int Configure3GPPBSMulticarrier(System.IntPtr Instrument_Handle, ushort Multicarrier_State, ushort Autoscale);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Configure3GPPBSResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int Configure3GPPBSResults(System.IntPtr Instrument_Handle, int Channel, int CPICH_Slot, int Spreading_Factor);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Configure3GPPBSResultType", CallingConvention = CallingConvention.StdCall)]
            public static extern int Configure3GPPBSResultType(System.IntPtr Instrument_Handle, int Result_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Configure3GPPBSResultSummaryView", CallingConvention = CallingConvention.StdCall)]
            public static extern int Configure3GPPBSResultSummaryView(System.IntPtr Instrument_Handle, int View);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Configure3GPPBSYScaleDivision", CallingConvention = CallingConvention.StdCall)]
            public static extern int Configure3GPPBSYScaleDivision(System.IntPtr Instrument_Handle, double Y_Scale_Division);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Set3GPPBSASequence", CallingConvention = CallingConvention.StdCall)]
            public static extern int Set3GPPBSASequence(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Search3GPPBSScramblingCode", CallingConvention = CallingConvention.StdCall)]
            public static extern int Search3GPPBSScramblingCode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Configure3GPPFDDBSChannelTableOperations", CallingConvention = CallingConvention.StdCall)]
            public static extern int Configure3GPPFDDBSChannelTableOperations(System.IntPtr Instrument_Handle, int Operation, string Channel_Table);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Configure3GPPFDDBSChannelTableData", CallingConvention = CallingConvention.StdCall)]
            public static extern int Configure3GPPFDDBSChannelTableData(System.IntPtr Instrument_Handle, int Array_Size, int[] Code_Class, int[] Code_Number, ushort[] TFCI, int[] Timing_Offset, int[] Pilot_Length, int[] Channel_Type, ushort[] Status, double[] CDP_Relative);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Set3GPPBSMarkerToChannel", CallingConvention = CallingConvention.StdCall)]
            public static extern int Set3GPPBSMarkerToChannel(System.IntPtr Instrument_Handle, int Window, int Channel);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Set3GPPBSDeltaMarkerToChannel", CallingConvention = CallingConvention.StdCall)]
            public static extern int Set3GPPBSDeltaMarkerToChannel(System.IntPtr Instrument_Handle, int Window, int Channel);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Set3GPPBSMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int Set3GPPBSMeasurement(System.IntPtr Instrument_Handle, int Measurement);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Configure3GPPFDDBSChannelTableComment", CallingConvention = CallingConvention.StdCall)]
            public static extern int Configure3GPPFDDBSChannelTableComment(System.IntPtr Instrument_Handle, string Channel_Table_Comment);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Query3GPPFDDBSChannelTableCatalog", CallingConvention = CallingConvention.StdCall)]
            public static extern int Query3GPPFDDBSChannelTableCatalog(System.IntPtr Instrument_Handle, out int Number_of_Channel_Tables, int Buffer_Size, System.Text.StringBuilder Channel_Tables_List);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Get3GPPBSTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int Get3GPPBSTrace(System.IntPtr Instrument_Handle, int Window, int Trace, int Array_Length, out int Actual_Points, [In, Out] double[] Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Get3GPPBSResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int Get3GPPBSResult(System.IntPtr Instrument_Handle, int Type, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Get3GPPBSCDPResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int Get3GPPBSCDPResult(System.IntPtr Instrument_Handle, int Type, int Array_Size, [In, Out] double[] Results, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Get3GPPBSCDPScramblingCodeResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int Get3GPPBSCDPScramblingCodeResult(System.IntPtr Instrument_Handle, int Array_Size, [In, Out] int[] Scrambing_Code, [In, Out] double[] Power, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Get3GPPBSCDPListEvaluationResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int Get3GPPBSCDPListEvaluationResult(System.IntPtr Instrument_Handle, int No_Of_Values, [In, Out] double[] Range_Number, [In, Out] double[] Start_Frequency, [In, Out] double[] Stop_Frequency, [In, Out] double[] Resolution_Bandwidth, [In, Out] double[] Frequency, [In, Out] double[] Power_Abs, [In, Out] double[] Power_Rel, [In, Out] double[] Delta, [In, Out] int[] Limit_Check, [In, Out] double[] Reserved1, [In, Out] double[] Reserved2, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Set3GPPUEMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int Set3GPPUEMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Configure3GPPUEMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int Configure3GPPUEMeasurement(System.IntPtr Instrument_Handle, int Scrambling_Code, int Scrambling_Type, int Measurement_Base, int Slot, int Side_Band, ushort Normalize, ushort RRC_Filter, ushort HSDPA_UPA, ushort Eliminate_Tail_Chips, double Inactive_Channel_Treshold, ushort Invert_Q, int EVM_Meas_Interval);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Configure3GPPUEMultiframe", CallingConvention = CallingConvention.StdCall)]
            public static extern int Configure3GPPUEMultiframe(System.IntPtr Instrument_Handle, int Capture_Length, int Frame_To_Analyze);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Configure3GPPUEResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int Configure3GPPUEResults(System.IntPtr Instrument_Handle, int Select_I_Q, int Channel, int Spreading_Factor);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Configure3GPPUEResultType", CallingConvention = CallingConvention.StdCall)]
            public static extern int Configure3GPPUEResultType(System.IntPtr Instrument_Handle, int Result_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Configure3GPPFDDUEChannelTableOperations", CallingConvention = CallingConvention.StdCall)]
            public static extern int Configure3GPPFDDUEChannelTableOperations(System.IntPtr Instrument_Handle, int Operation, string Channel_Table);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Configure3GPPFDDUEChannelTableData", CallingConvention = CallingConvention.StdCall)]
            public static extern int Configure3GPPFDDUEChannelTableData(System.IntPtr Instrument_Handle, int Array_Size, int[] Code_Class, int[] Code_Number, int[] Channel_Mapping, int[] Pilot_Length, ushort[] Status, double[] CDP_Relative);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Configure3GPPFDDUEChannelTableHighSpeedChannel", CallingConvention = CallingConvention.StdCall)]
            public static extern int Configure3GPPFDDUEChannelTableHighSpeedChannel(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Set3GPPUEMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int Set3GPPUEMeasurement(System.IntPtr Instrument_Handle, int Measurement);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Configure3GPPFDDUEChannelTableComment", CallingConvention = CallingConvention.StdCall)]
            public static extern int Configure3GPPFDDUEChannelTableComment(System.IntPtr Instrument_Handle, string Channel_Table_Comment);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Query3GPPFDDUEChannelTableCatalog", CallingConvention = CallingConvention.StdCall)]
            public static extern int Query3GPPFDDUEChannelTableCatalog(System.IntPtr Instrument_Handle, out int Number_of_Channel_Tables, int Buffer_Size, System.Text.StringBuilder Channel_Tables_List);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Get3GPPUETrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int Get3GPPUETrace(System.IntPtr Instrument_Handle, int Window, int Trace, int Array_Length, out int Actual_Points, [In, Out] double[] Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Get3GPPUEResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int Get3GPPUEResult(System.IntPtr Instrument_Handle, int Type, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SetTDSBSMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTDSBSMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SetTDSBSMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTDSBSMeasurement(System.IntPtr Instrument_Handle, int Measurement);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTDSBSAdaptToSignal", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTDSBSAdaptToSignal(System.IntPtr Instrument_Handle, int Start_Slot, int Stop_Slot);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTDSBSCapture", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTDSBSCapture(System.IntPtr Instrument_Handle, int Set_Count, int Set_Value, int Capture_Length);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTDSBSCDACapture", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTDSBSCDACapture(System.IntPtr Instrument_Handle, int Set_Count, int Set_Value, int Capture_Length, int Channel, int Slot);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTDSBSResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTDSBSResults(System.IntPtr Instrument_Handle, int Result_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_TDSBSAdaptSignalAutoLevelTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int TDSBSAdaptSignalAutoLevelTime(System.IntPtr Instrument_Handle, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_TDSBSCDPLevelAdjust", CallingConvention = CallingConvention.StdCall)]
            public static extern int TDSBSCDPLevelAdjust(System.IntPtr Instrument_Handle, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_TDSBSChannelTableOperations", CallingConvention = CallingConvention.StdCall)]
            public static extern int TDSBSChannelTableOperations(System.IntPtr Instrument_Handle, int Operation, string Channel_Table);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTDSBSChannelTableComment", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTDSBSChannelTableComment(System.IntPtr Instrument_Handle, string Channel_Table_Comment);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTDSBSChannelTableData", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTDSBSChannelTableData(System.IntPtr Instrument_Handle, int Array_Size, int[] Channel_Type, int[] Code_Class, int[] Code_Number, int[] Modulation_Type, int[] Midamble_Shift, ushort[] Status, int[] Reserved_1, int[] Reserved_2);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTDSBSChannelTableOrder", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTDSBSChannelTableOrder(System.IntPtr Instrument_Handle, int Order);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTDSBSChannelTableMidambleShift", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTDSBSChannelTableMidambleShift(System.IntPtr Instrument_Handle, int Maximum_of_Midamble_Shifts);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetTDSBSChannelTableCatalog", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTDSBSChannelTableCatalog(System.IntPtr Instrument_Handle, out int Number_of_Channel_Tables, int Buffer_Size, System.Text.StringBuilder Channel_Tables_List);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTDSBSMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTDSBSMeasurement(System.IntPtr Instrument_Handle, int Standard, int Scrambling_Code, int Midamble_Shift_Cells, int Side_Band, ushort Normalize, double Inactive_Channel_Treshold, ushort Invert_Q);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTDSBSSyncToSlot", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTDSBSSyncToSlot(System.IntPtr Instrument_Handle, ushort Sync_To_Slot);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTDSBSUCPICH", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTDSBSUCPICH(System.IntPtr Instrument_Handle, ushort User_Defined_CPICH, int CPICH_Code, int CPICH_Pattern);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTDSBSPVTMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTDSBSPVTMeasurement(System.IntPtr Instrument_Handle, int Switching_Point, int Subframes, ushort High_Dynamic_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetTDSBSResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTDSBSResult(System.IntPtr Instrument_Handle, int Type, uint Timeout, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetTDSBSReferenceLevelAdjustmentResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTDSBSReferenceLevelAdjustmentResult(System.IntPtr Instrument_Handle, uint Timeout, out int Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchTDSBSTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchTDSBSTrace(System.IntPtr Instrument_Handle, int Trace, int Array_Length, out int Actual_Points, [In, Out] double[] Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SetTDSUEMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTDSUEMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SetTDSUEMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTDSUEMeasurement(System.IntPtr Instrument_Handle, int Measurement);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTDSUEAdaptToSignal", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTDSUEAdaptToSignal(System.IntPtr Instrument_Handle, int Start_Slot, int Stop_Slot);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTDSUECapture", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTDSUECapture(System.IntPtr Instrument_Handle, int Set_Count, int Set_Value, int Capture_Length, int Channel, int Slot);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTDSUEResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTDSUEResults(System.IntPtr Instrument_Handle, int Result_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_TDSUEAdaptSignalAutoLevelTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int TDSUEAdaptSignalAutoLevelTime(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_TDSUECDPLevelAdjust", CallingConvention = CallingConvention.StdCall)]
            public static extern int TDSUECDPLevelAdjust(System.IntPtr Instrument_Handle, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_TDSUEChannelTableOperations", CallingConvention = CallingConvention.StdCall)]
            public static extern int TDSUEChannelTableOperations(System.IntPtr Instrument_Handle, int Operation, string Channel_Table);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTDSUEChannelTableComment", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTDSUEChannelTableComment(System.IntPtr Instrument_Handle, string Channel_Table_Comment);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTDSUEChannelTableData", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTDSUEChannelTableData(System.IntPtr Instrument_Handle, int Array_Size, int[] Channel_Type, int[] Code_Class, int[] Code_Number, int[] Modulation_Type, int[] Midamble_Shift, ushort[] Status, int[] Reserved_1, int[] Reserved_2);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTDSUEChannelTableOrder", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTDSUEChannelTableOrder(System.IntPtr Instrument_Handle, int Order);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTDSUEChannelTableMidambleShift", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTDSUEChannelTableMidambleShift(System.IntPtr Instrument_Handle, int Maximum_of_Midamble_Shifts);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetTDSUEChannelTableCatalog", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTDSUEChannelTableCatalog(System.IntPtr Instrument_Handle, out int Number_of_Channel_Tables, int Buffer_Size, System.Text.StringBuilder Channel_Tables_List);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTDSUEMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTDSUEMeasurement(System.IntPtr Instrument_Handle, int Scrambling_Code, int Midamble_Shift_Cells, int Side_Band, ushort Normalize, double Inactive_Channel_Treshold, ushort Invert_Q);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTDSUESyncToSlot", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTDSUESyncToSlot(System.IntPtr Instrument_Handle, ushort Sync_To_Slot);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTDSUEUCPICH", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTDSUEUCPICH(System.IntPtr Instrument_Handle, ushort User_Defined_CPICH, int CPICH_Code, int CPICH_Pattern);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTDSUEPVTMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTDSUEPVTMeasurement(System.IntPtr Instrument_Handle, int Subframes, ushort High_Dynamic_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetTDSUEResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTDSUEResult(System.IntPtr Instrument_Handle, int Type, uint Timeout, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetTDSUEReferenceLevelAdjustmentResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTDSUEReferenceLevelAdjustmentResult(System.IntPtr Instrument_Handle, uint Timeout, out int Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchTDSUETrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchTDSUETrace(System.IntPtr Instrument_Handle, int Trace, int Array_Length, out int Actual_Points, [In, Out] double[] Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_AdjustC2KSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int AdjustC2KSettings(System.IntPtr Instrument_Handle, int Window, int Adjust_Settings, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_AdjustC2KReferenceLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int AdjustC2KReferenceLevel(System.IntPtr Instrument_Handle, int Window, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_RestoreC2KSEMStandardLines", CallingConvention = CallingConvention.StdCall)]
            public static extern int RestoreC2KSEMStandardLines(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureC2KSignalStatisticMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureC2KSignalStatisticMeasurement(System.IntPtr Instrument_Handle, int Measurement, int Number_of_Samples);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureC2KSignalStatisticsPercentMarker", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureC2KSignalStatisticsPercentMarker(System.IntPtr Instrument_Handle, int Window, int Marker, double Percent_Marker);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureC2KChannelTableOperations", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureC2KChannelTableOperations(System.IntPtr Instrument_Handle, int Operation, string Channel_Table);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_RestoreC2KChannelTable", CallingConvention = CallingConvention.StdCall)]
            public static extern int RestoreC2KChannelTable(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureC2KBasicSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureC2KBasicSettings(System.IntPtr Instrument_Handle, int Band_Class, ushort Time_Phase, double Inact_Channel_Threshold, ushort Invert_Q, int Side_Band, ushort Normalize);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureC2KAmplitudeYScaleDivision", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureC2KAmplitudeYScaleDivision(System.IntPtr Instrument_Handle, double Y_Scale_Div);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_AdjustC2KAmplitudeReferenceLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int AdjustC2KAmplitudeReferenceLevel(System.IntPtr Instrument_Handle, uint Timeout);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SelectC2KMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int SelectC2KMeasurement(System.IntPtr Instrument_Handle, int Measurement_Select);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureC2KPCGSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureC2KPCGSettings(System.IntPtr Instrument_Handle, int Set_Count, int Set_To_Analyze);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureC2KChannelTableComment", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureC2KChannelTableComment(System.IntPtr Instrument_Handle, string Channel_Table_Comment);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryC2KChannelTableCatalog", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryC2KChannelTableCatalog(System.IntPtr Instrument_Handle, out int Number_of_Channel_Tables, int Buffer_Size, System.Text.StringBuilder Channel_Tables_List);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadC2KTraceData", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadC2KTraceData(System.IntPtr Instrument_Handle, int Window, int Source_Trace, int Array_Length, out int Actual_Points, [In, Out] double[] Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadC2KCDPTraceData", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadC2KCDPTraceData(System.IntPtr Instrument_Handle, int Window, int Source_Trace, int Array_Length, out int Actual_Points, [In, Out] double[] Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_BC2KMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int BC2KMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBC2KChannelTableData", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBC2KChannelTableData(System.IntPtr Instrument_Handle, int[] Channel_Type, int[] Code_Class, int[] Code_Number, int[] Radio_Configuration, int[] Reserved_1, int[] Reserved_2, ushort[] Status, double[] CDP_Relative, int Num_of_Elements);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBC2KAdvancedSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBC2KAdvancedSettings(System.IntPtr Instrument_Handle, int PN_Offset, int Base_Spreading_Factor, int Power_Reference, int Order);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBC2KForce1xEVDV", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBC2KForce1xEVDV(System.IntPtr Instrument_Handle, ushort _1xEV_DV);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBC2KAntennaSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBC2KAntennaSettings(System.IntPtr Instrument_Handle, ushort Antenna_Diversity, int Antenna_Number);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBC2KMultiCarrier", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBC2KMultiCarrier(System.IntPtr Instrument_Handle, ushort Multi_Carrier, ushort Enhanced_Algorithm, ushort MC_Filter, int Filter_Type, double Roll_Off, double Cut_Off_Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBC2KPichMarkerSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBC2KPichMarkerSettings(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBC2KTDPichMarkerSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBC2KTDPichMarkerSettings(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBC2KCodeDomainAnalyzerResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBC2KCodeDomainAnalyzerResults(System.IntPtr Instrument_Handle, int Result_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBC2KCodeDomainAnalyzerSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBC2KCodeDomainAnalyzerSettings(System.IntPtr Instrument_Handle, int Capture_Length, int Select_Channel, int Select_PCG);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryBC2KCodeDomainAnalyzerResultSummary", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryBC2KCodeDomainAnalyzerResultSummary(System.IntPtr Instrument_Handle, int Window, int Result_Type, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBC2KSEM", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBC2KSEM(System.IntPtr Instrument_Handle, int Limit_Line_Type, double Limit_Line_Value, double _30kHz_1MHz_Transition);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_StoreBC2KSEMToFile", CallingConvention = CallingConvention.StdCall)]
            public static extern int StoreBC2KSEMToFile(System.IntPtr Instrument_Handle, string File_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadBC2KSEMPeakList", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadBC2KSEMPeakList(System.IntPtr Instrument_Handle, int Window, int No_Of_Values, [In, Out] double[] Frequency, [In, Out] double[] Level, [In, Out] double[] Delta_Level, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_MC2KMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int MC2KMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMC2KChannelTableData", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMC2KChannelTableData(System.IntPtr Instrument_Handle, int[] Channel_Type, int[] Code_Class, int[] Code_Number, int[] Mapping, int[] Reserved_1, int[] Reserved_2, ushort[] Status, double[] CDP_Relative, int Num_of_Elements);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMC2KAdvancedSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMC2KAdvancedSettings(System.IntPtr Instrument_Handle, int Base_Spreading_Factor, int Power_Reference, int Order, int Select_I___Q, int Long_Code_Mode, string Long_Code_Mask, string Long_Code_Offset);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMC2KCDPOverview", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMC2KCDPOverview(System.IntPtr Instrument_Handle, ushort Code_Domain_Overview);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMC2KPichMarkerSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMC2KPichMarkerSettings(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMC2KCodeDomainAnalyzerResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMC2KCodeDomainAnalyzerResults(System.IntPtr Instrument_Handle, int Result_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMC2KCodeDomainAnalyzerSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMC2KCodeDomainAnalyzerSettings(System.IntPtr Instrument_Handle, int Capture_Length, int Select_Channel, int Select_PCG, int Select_Mapping);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryMC2KCodeDomainAnalyzerResultSummary", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryMC2KCodeDomainAnalyzerResultSummary(System.IntPtr Instrument_Handle, int Window, int Result_Type, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMC2KSEM", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMC2KSEM(System.IntPtr Instrument_Handle, int Limit_Line_Type, double _30kHz_1MHz_Transition);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_BDOMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int BDOMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBDOPowerVsTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBDOPowerVsTime(System.IntPtr Instrument_Handle, int Number_Of_Halfslots, int Reference_Slot, ushort Burst_Fit, int Reference_Mean_Power, double Reference_Value, ushort Restart_On_Fail);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBDOPowerVsTimeListEvaluation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBDOPowerVsTimeListEvaluation(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_RestoreBDOPowerVsTimeStandardLines", CallingConvention = CallingConvention.StdCall)]
            public static extern int RestoreBDOPowerVsTimeStandardLines(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBDOChannelTableData", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBDOChannelTableData(System.IntPtr Instrument_Handle, int[] Channel_Type, int[] Code_Class, int[] Code_Number, int[] Modulation, int[] Reserved_1, int[] Reserved_2, ushort[] Status, double[] CDP_Relative, int Num_of_Elements);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBDOAdvancedSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBDOAdvancedSettings(System.IntPtr Instrument_Handle, int PN_Offset, ushort CDP_Average, int Revision);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBDOMultiCarrier", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBDOMultiCarrier(System.IntPtr Instrument_Handle, ushort Multi_Carrier, ushort Enhanced_Algorithm, ushort MC_Filter, int Filter_Type, double Roll_Off, double Cut_Off_Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBDOCodeDomainAnalyzerResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBDOCodeDomainAnalyzerResults(System.IntPtr Instrument_Handle, int Result_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBDOCodeDomainAnalyzerSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBDOCodeDomainAnalyzerSettings(System.IntPtr Instrument_Handle, int Capture_Length, int Select_Code, int Select_Slot);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBDOCDPChannelType", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBDOCDPChannelType(System.IntPtr Instrument_Handle, int Channel_Type, ushort Code_Domain_Overview, int Mapping, int Select);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBDOChannelType", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBDOChannelType(System.IntPtr Instrument_Handle, int Channel_Type, int Mapping, int Select);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryBDOCodeDomainAnalyzerGeneralResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryBDOCodeDomainAnalyzerGeneralResults(System.IntPtr Instrument_Handle, int Window, int Result_Type, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryBDOCodeDomainAnalyzerChannelResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryBDOCodeDomainAnalyzerChannelResults(System.IntPtr Instrument_Handle, int Window, int Result_Type, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureBDOSEM", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureBDOSEM(System.IntPtr Instrument_Handle, int Limit_Line_Type, double Limit_Line_Value, double _30kHz_1MHz_Transition);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_StoreBDOSEMToFile", CallingConvention = CallingConvention.StdCall)]
            public static extern int StoreBDOSEMToFile(System.IntPtr Instrument_Handle, string File_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadBDOSEMPeakList", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadBDOSEMPeakList(System.IntPtr Instrument_Handle, int Window, int No_Of_Values, [In, Out] double[] Frequency, [In, Out] double[] Level, [In, Out] double[] Delta_Level, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadBDOPowerVsTimeListEvaluation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadBDOPowerVsTimeListEvaluation(System.IntPtr Instrument_Handle, int No_Of_Values, [In, Out] double[] Range_Number, [In, Out] double[] Start_Time, [In, Out] double[] Stop_Time, [In, Out] double[] Avg_Power_Abs, [In, Out] double[] Avg_Power_Rel, [In, Out] double[] Max_Power_Abs, [In, Out] double[] Max_Power_Rel, [In, Out] double[] Min_Power_Abs, [In, Out] double[] Min_Power_Rel, [In, Out] double[] Limit_Check, [In, Out] double[] Reserved1, [In, Out] double[] Reserved2, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_MDOMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int MDOMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMDOChannelTableData", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMDOChannelTableData(System.IntPtr Instrument_Handle, int[] Channel_Type, int[] Code_Class, int[] Code_Number, int[] Mapping, int[] Activity, int[] Reserved_1, ushort[] Status, int[] Reserved_2, int Num_of_Elements);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMDOAdvancedSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMDOAdvancedSettings(System.IntPtr Instrument_Handle, int Power_Reference, int Order, int Select_I___Q, ushort CDP_Average, int Operation, string Long_Code_I, string Long_Code_Q);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMDOCDPOverview", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMDOCDPOverview(System.IntPtr Instrument_Handle, ushort Code_Domain_Overview);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMDOPichMarkerSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMDOPichMarkerSettings(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMDOCodeDomainAnalyzerResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMDOCodeDomainAnalyzerResults(System.IntPtr Instrument_Handle, int Result_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMDOCodeDomainAnalyzerSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMDOCodeDomainAnalyzerSettings(System.IntPtr Instrument_Handle, int Capture_Length, int Select_Channel, int Select_Halfslot, int Select);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryMDOCodeDomainAnalyzerResultSummary", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryMDOCodeDomainAnalyzerResultSummary(System.IntPtr Instrument_Handle, int Window, int Result_Type, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureMDOSEM", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureMDOSEM(System.IntPtr Instrument_Handle, int Limit_Line_Type, double _30kHz_1MHz_Transition);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_WlanSetWlanMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int WlanSetWlanMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanStandard", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanStandard(System.IntPtr Instrument_Handle, int Standard);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanElectronicAtt", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanElectronicAtt(System.IntPtr Instrument_Handle, ushort State, ushort Electronic_Attenuator_Auto, int Manual__Value__dB_);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWLANYIGFilter", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWLANYIGFilter(System.IntPtr Instrument_Handle, int Window, ushort YIG_Filter);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanTrigger", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanTrigger(System.IntPtr Instrument_Handle, int Mode, double Offset, ushort Auto_Trigger_Level, double Power_Trigger_Level, double Ext__Trigger_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanGate", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanGate(System.IntPtr Instrument_Handle, ushort Gating, int Mode, double Delay, double Length, ushort Link_Gate_And_Marker);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanACPMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanACPMode(System.IntPtr Instrument_Handle, int ACP_Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanGainImbalanceUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanGainImbalanceUnit(System.IntPtr Instrument_Handle, int Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWLANSweep", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWLANSweep(System.IntPtr Instrument_Handle, ushort Sweep_Mode_Continuous);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanAutoLevelOnce", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanAutoLevelOnce(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanBasebandSignalLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanBasebandSignalLevel(System.IntPtr Instrument_Handle, ushort Auto_Level, double Auto_Level_Time, double Signal_Level, double Ext_Att);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanRFChannelFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanRFChannelFrequency(System.IntPtr Instrument_Handle, int Channel_No, double Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanRFSignalLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanRFSignalLevel(System.IntPtr Instrument_Handle, ushort Auto_Level, double Auto_Level_Time, double Signal_Level, double RF_Att, double Ext_Att);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWLANMarkerEnabled", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWLANMarkerEnabled(System.IntPtr Instrument_Handle, int Window, int Marker, ushort Marker_Enabled, int Trace);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanMarkPosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanMarkPosition(System.IntPtr Instrument_Handle, int Window, int Marker, int Position, double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanMarkerPositionConstellation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanMarkerPositionConstellation(System.IntPtr Instrument_Handle, int Window, int Marker, int Type, int Position);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanMarkerZoom", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanMarkerZoom(System.IntPtr Instrument_Handle, int Window, int Marker, double Zoom_Ratio);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanMarkerBurstSymbol", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanMarkerBurstSymbol(System.IntPtr Instrument_Handle, int Window, int Marker, int Burst_Number, int Symbol_Number);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SetWlanMarkerPeak", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetWlanMarkerPeak(System.IntPtr Instrument_Handle, int Window, int Marker, int Position);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_WlanDisableAllMarkers", CallingConvention = CallingConvention.StdCall)]
            public static extern int WlanDisableAllMarkers(System.IntPtr Instrument_Handle, int Window);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanDemodulation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanDemodulation(System.IntPtr Instrument_Handle, ushort Use_Signal_Field_Content, int Burst_Type, ushort Auto_Demodulation, int Demodulator, ushort Channel_Estimation, ushort Equal_Burst_Length, int Min_Number_of_Data_Symbols, int Max_Number_of_Data_Symbols);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanPLCPDemodulation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanPLCPDemodulation(System.IntPtr Instrument_Handle, ushort Use_PLCP_Header_Content, int Burst_Type, ushort Auto_Demodulation, int Demodulator, int Length_Type, ushort Equal_Burst_Length, int Min_Payload_Length, int Max_Payload_Length);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanTracking", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanTracking(System.IntPtr Instrument_Handle, ushort Phase, ushort Timing, ushort Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanSelectFilter", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanSelectFilter(System.IntPtr Instrument_Handle, string TX_Filter_Name, string RX_Filter_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetWlanFilterCatalog", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetWlanFilterCatalog(System.IntPtr Instrument_Handle, int Buffer_Size, System.Text.StringBuilder Filter_List, out int Returned_Length);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWLANFFTStartOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWLANFFTStartOffset(System.IntPtr Instrument_Handle, int FFT_Start_Offset);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanDisplay", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanDisplay(System.IntPtr Instrument_Handle, ushort Display_Table);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanLimit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanLimit(System.IntPtr Instrument_Handle, int Window, int Limit, int Limit_Type, int Mode, double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWLANAllLimits", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWLANAllLimits(System.IntPtr Instrument_Handle, int Window, int Limit, int Array_Size, double[] Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanPLCPLimit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanPLCPLimit(System.IntPtr Instrument_Handle, int Window, int Limit, int Limit_Type, int Mode, double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetWlanACPChannelLimit", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetWlanACPChannelLimit(System.IntPtr Instrument_Handle, int Window, int Limit, int Channel_Type, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetWlanSpectrumMaxOverstepping", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetWlanSpectrumMaxOverstepping(System.IntPtr Instrument_Handle, int Window, int Limit, int Channel_Type, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetWlanAllLimits", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetWlanAllLimits(System.IntPtr Instrument_Handle, int Window, int Limit, [In, Out] double[] Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanMeas", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanMeas(System.IntPtr Instrument_Handle, ushort Overall_Burst_Count, int No_of_Burst_to_Analyze, double Capture_Time, ushort Swap_I_Q, int Sweep_Count);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanPVTMeas", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanPVTMeas(System.IntPtr Instrument_Handle, int Rising_Falling);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanPLCPPVT", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanPLCPPVT(System.IntPtr Instrument_Handle, int Reference_Power, int Ramp, int Average_Length);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanEVMMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanEVMMeasurement(System.IntPtr Instrument_Handle, int Mode, ushort Auto_Scaling, double Per_Division, int Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanSpectrumMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanSpectrumMeasurement(System.IntPtr Instrument_Handle, int Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanConstellationMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanConstellationMeasurement(System.IntPtr Instrument_Handle, int Constellation_Mode, int Carrier_Selection, int Carrier_Number);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanPLCPConstellation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanPLCPConstellation(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanStatisticsMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanStatisticsMeasurement(System.IntPtr Instrument_Handle, int Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanIQMeasurementParameters", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanIQMeasurementParameters(System.IntPtr Instrument_Handle, double Sample_Rate, int Filter);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWlanSEMMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWlanSEMMeasurement(System.IntPtr Instrument_Handle, int Type, string File_Name, int Power_Class);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetWlanACPMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetWlanACPMeasurement(System.IntPtr Instrument_Handle, int Window, int Marker, int Mode, [In, Out] double[] Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_LoadWlanIQData", CallingConvention = CallingConvention.StdCall)]
            public static extern int LoadWlanIQData(System.IntPtr Instrument_Handle, string File_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_StoreWlanIQData", CallingConvention = CallingConvention.StdCall)]
            public static extern int StoreWlanIQData(System.IntPtr Instrument_Handle, string File_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_WlanBurstImm", CallingConvention = CallingConvention.StdCall)]
            public static extern int WlanBurstImm(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadWlanTraceData", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadWlanTraceData(System.IntPtr Instrument_Handle, int Trace, int Array_Size, [In, Out] double[] Trace_Data, out int No_of_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadWlanMemoryIQData", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadWlanMemoryIQData(System.IntPtr Instrument_Handle, int Offset_Samples, int No_of_Samples, int Buffer_Size, out int No_of_Points, [In, Out] double[] Real_Parts__I_, [In, Out] double[] Imaginary_Parts__Q_);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadWlanSEMResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadWlanSEMResults(System.IntPtr Instrument_Handle, int No_Of_Values, [In, Out] double[] Index, [In, Out] double[] Start_Frequency_Band, [In, Out] double[] Stop_Frequency_Band, [In, Out] double[] Resolution_Bandwidth, [In, Out] double[] Limit_Fail_Frequency, [In, Out] double[] Power_Abs, [In, Out] double[] Power_Rel, [In, Out] double[] Limit_Distance, [In, Out] double[] Failure_Flag, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchWlanBurstAll", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchWlanBurstAll(System.IntPtr Instrument_Handle, [In, Out] double[] Results, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchWlanBurstPower", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchWlanBurstPower(System.IntPtr Instrument_Handle, int Power_Measurement, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchWlanBurstTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchWlanBurstTime(System.IntPtr Instrument_Handle, int Time, int Mode, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchWlanBurstError", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchWlanBurstError(System.IntPtr Instrument_Handle, int Error_Type, int Mode, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchWlanIQImp", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchWlanIQImp(System.IntPtr Instrument_Handle, int IQ_Impairment, int Mode, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchWlanEVM", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchWlanEVM(System.IntPtr Instrument_Handle, int EVM, int Mode, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchWlanCount", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchWlanCount(System.IntPtr Instrument_Handle, int Type, out int Count);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetWlanLimitCheckResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetWlanLimitCheckResult(System.IntPtr Instrument_Handle, int Window, int Limit, int Limit_Type, int Mode, out int Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetWlanPLCPLimitCheckResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetWlanPLCPLimitCheckResult(System.IntPtr Instrument_Handle, int Window, int Limit, int Limit_Type, int Mode, out int Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetWlanACPChannLimitCheckResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetWlanACPChannLimitCheckResult(System.IntPtr Instrument_Handle, int Window, int Limit, int Channel_Type, out int Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetWlanAllLimitCheckResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetWlanAllLimitCheckResults(System.IntPtr Instrument_Handle, int Window, int Limit, [In, Out] int[] Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetWlanLimitCheck", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetWlanLimitCheck(System.IntPtr Instrument_Handle, int Window, int Limit, out int Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SetWiMAXMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetWiMAXMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXStandard", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXStandard(System.IntPtr Instrument_Handle, int Standard);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXElectronicAtt", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXElectronicAtt(System.IntPtr Instrument_Handle, ushort State, ushort Electronic_Attenuator_Auto, int Manual__Value__dB_);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXYIGFilter", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXYIGFilter(System.IntPtr Instrument_Handle, int Window, ushort YIG_Filter);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXYIGFilterAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXYIGFilterAuto(System.IntPtr Instrument_Handle, int Window, ushort YIG_Filter_Auto);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXTrigger", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXTrigger(System.IntPtr Instrument_Handle, int Mode, double Offset, ushort Auto_Trigger_Level, double Power_Trigger_Level, double Ext__Trigger_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXGate", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXGate(System.IntPtr Instrument_Handle, ushort Gating, int Mode, double Delay, double Length, ushort Link_Gate_And_Marker);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXACP", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXACP(System.IntPtr Instrument_Handle, int ACP_Mode, int Number_Of_Channels, ushort Noise_Correction);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXACPChannelBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXACPChannelBandwidth(System.IntPtr Instrument_Handle, int Channel, int Channel_Number, double Bandwidth);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXACPChannelSpacing", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXACPChannelSpacing(System.IntPtr Instrument_Handle, int Channel, int Channel_Number, double Spacing);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXLevel(System.IntPtr Instrument_Handle, int Measurement, ushort Auto_Level, double Reference_Level, double External_Attenuation);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXSEMMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXSEMMode(System.IntPtr Instrument_Handle, int Link_Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXFSBW", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXFSBW(System.IntPtr Instrument_Handle, ushort FS_BW_Auto, double Sampling_Rate_Fs, double Channel_Bandwidth_BW);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXNumberOfAntennas", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXNumberOfAntennas(System.IntPtr Instrument_Handle, int Number_Of_Antennas);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXAutoLevelOnce", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXAutoLevelOnce(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXBasebandSignalLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXBasebandSignalLevel(System.IntPtr Instrument_Handle, ushort Auto_Level, double Auto_Level_Time, double Signal_Level, double Ext_Att);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXRFChannelFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXRFChannelFrequency(System.IntPtr Instrument_Handle, int Channel_No, double Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXRFSignalLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXRFSignalLevel(System.IntPtr Instrument_Handle, ushort Auto_Level, double Auto_Level_Time, double Signal_Level, double RF_Att, double Ext_Att);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetWiMAXReferenceLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetWiMAXReferenceLevel(System.IntPtr Instrument_Handle, out double Reference_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXMarkerEnabled", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXMarkerEnabled(System.IntPtr Instrument_Handle, int Window, int Marker, ushort Marker_Enabled, int Trace);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXMarkPosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXMarkPosition(System.IntPtr Instrument_Handle, int Window, int Marker, int Position, double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXMarkerPositionConstellation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXMarkerPositionConstellation(System.IntPtr Instrument_Handle, int Window, int Marker, int Type, int Position);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXMarkerZoom", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXMarkerZoom(System.IntPtr Instrument_Handle, int Window, double Zoom_Ratio);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SetWiMAXMarkerPeak", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetWiMAXMarkerPeak(System.IntPtr Instrument_Handle, int Window, int Marker, int Position);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_WiMAXDisableAllMarkers", CallingConvention = CallingConvention.StdCall)]
            public static extern int WiMAXDisableAllMarkers(System.IntPtr Instrument_Handle, int Window);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXDemodulation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXDemodulation(System.IntPtr Instrument_Handle, ushort Use_Signal_Field_Content, int Modulation_Detection_Mode, int Demodulator, int Link_Mode, ushort Channel_Estimation, ushort Equal_Burst_Length, int Min_Number_of_Data_Symbols, int Max_Number_of_Data_Symbols);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXDemodulationOFDMA", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXDemodulationOFDMA(System.IntPtr Instrument_Handle, int Channel_Estimation, double FFT_Offset);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXSubchannelization", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXSubchannelization(System.IntPtr Instrument_Handle, ushort State, int Subchannel, int Up_Link_Physical_Modifier);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXTracking", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXTracking(System.IntPtr Instrument_Handle, int Based_On, ushort Phase, ushort Timing, ushort Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXOFDMATracking", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXOFDMATracking(System.IntPtr Instrument_Handle, int Based_On, ushort Phase, ushort Timing, int Tracking_Pilot);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXSelectFilter", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXSelectFilter(System.IntPtr Instrument_Handle, string TX_Filter_Name, string RX_Filter_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetWiMAXFilterCatalog", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetWiMAXFilterCatalog(System.IntPtr Instrument_Handle, int Buffer_Size, System.Text.StringBuilder Filter_List, out int Returned_Length);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXDisplay", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXDisplay(System.IntPtr Instrument_Handle, ushort Display_Table);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXDisplayResultTable", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXDisplayResultTable(System.IntPtr Instrument_Handle, int Page);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXLimit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXLimit(System.IntPtr Instrument_Handle, int Window, int Limit, int Limit_Type, int Mode, double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXAllLimits", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXAllLimits(System.IntPtr Instrument_Handle, int Window, int Limit, int Array_Size, double[] Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetWiMAXSpectrumMaxOverstepping", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetWiMAXSpectrumMaxOverstepping(System.IntPtr Instrument_Handle, int Window, int Limit, int Channel_Type, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetWiMAXAllLimits", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetWiMAXAllLimits(System.IntPtr Instrument_Handle, int Window, int Limit, [In, Out] double[] Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXFrame", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXFrame(System.IntPtr Instrument_Handle, int ID_Cell, int UL_Subframe_Frame_Number, double TDD_Frame, int UL_Control_Region_Length);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXFrameIDCellAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXFrameIDCellAuto(System.IntPtr Instrument_Handle, ushort ID_Cell_Auto);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXFramePredefinedMap", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXFramePredefinedMap(System.IntPtr Instrument_Handle, int Predefined_Map, ushort Predefined_Map_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXFrameOFDMAMAP", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXFrameOFDMAMAP(System.IntPtr Instrument_Handle, int Use_for_Analysis);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXDLFramePreambleIndex", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXDLFramePreambleIndex(System.IntPtr Instrument_Handle, int Mode, int Preamble_Pattern);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXFrameDLUsedSubchannel", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXFrameDLUsedSubchannel(System.IntPtr Instrument_Handle, int Segment, int Subchannel_Usage);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXFrameULAllocatedSubchannel", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXFrameULAllocatedSubchannel(System.IntPtr Instrument_Handle, int Segment, int Bitmap, int Logical_Subchannel);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_LoadWiMAXFrameData", CallingConvention = CallingConvention.StdCall)]
            public static extern int LoadWiMAXFrameData(System.IntPtr Instrument_Handle, string File_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_StoreWiMAXFrameData", CallingConvention = CallingConvention.StdCall)]
            public static extern int StoreWiMAXFrameData(System.IntPtr Instrument_Handle, string File_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CreateWiMAXNewZone", CallingConvention = CallingConvention.StdCall)]
            public static extern int CreateWiMAXNewZone(System.IntPtr Instrument_Handle, int Zone, ushort Analyze, int Type, int Segment, int Length, int Offset, int Permbase, int PRBS_ID);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXZone", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXZone(System.IntPtr Instrument_Handle, int Zone, ushort Analyze, int Type, int Segment, int Length, int Offset, int Permbase, int PRBS_ID);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXZoneSpaceTimeCoding", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXZoneSpaceTimeCoding(System.IntPtr Instrument_Handle, int Zone, int Space_Time_Coding);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXZoneMatrix", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXZoneMatrix(System.IntPtr Instrument_Handle, int Zone, int Matrix_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_CreateWiMAXNewZoneBurst", CallingConvention = CallingConvention.StdCall)]
            public static extern int CreateWiMAXNewZoneBurst(System.IntPtr Instrument_Handle, int Zone, int Burst, int Modulation, int Subchannels, int Symbols, int Slot_Duration, int Subchannel_Offset, int Symbol_Offset, double Burst_Power, int Burst_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXZoneBurst", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXZoneBurst(System.IntPtr Instrument_Handle, int Zone, int Burst, int Modulation, int Subchannels, int Symbols, int Slot_Duration, double Burst_Power, int Burst_Type, ushort Offset_Auto, int Subchannel_Offset, int Symbol_Offset);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXZoneBurstMatrix", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXZoneBurstMatrix(System.IntPtr Instrument_Handle, int Zone, int Burst, int Matrix_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXExtFrameZoneGeneratorAddress", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXExtFrameZoneGeneratorAddress(System.IntPtr Instrument_Handle, string TCPIP_Address);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetWiMAXZoneCount", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetWiMAXZoneCount(System.IntPtr Instrument_Handle, out int Count);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetWiMAXZoneBurstCount", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetWiMAXZoneBurstCount(System.IntPtr Instrument_Handle, int Zone, out int Count);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_WiMAXZoneDelete", CallingConvention = CallingConvention.StdCall)]
            public static extern int WiMAXZoneDelete(System.IntPtr Instrument_Handle, int Zone);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_WiMAXZoneBurstDelete", CallingConvention = CallingConvention.StdCall)]
            public static extern int WiMAXZoneBurstDelete(System.IntPtr Instrument_Handle, int Zone, int Burst);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_WiMAXZoneReset", CallingConvention = CallingConvention.StdCall)]
            public static extern int WiMAXZoneReset(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_WiMAXZoneBurstReset", CallingConvention = CallingConvention.StdCall)]
            public static extern int WiMAXZoneBurstReset(System.IntPtr Instrument_Handle, int Zone);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXMeas", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXMeas(System.IntPtr Instrument_Handle, ushort Overall_Burst_Count, int No_of_Burst_to_Analyze, double Capture_Time, ushort Swap_I_Q, int Sweep_Count, ushort Sweep_Time_Auto__Mask_ACP_, double Sweep_Time__Mask_ACP_, int Frequency_Band, double Channel_Bandwidth, int Guard_Ratio);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXMeasOFDMA", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXMeasOFDMA(System.IntPtr Instrument_Handle, double Capture_Time, ushort Swap_I_Q, int Sweep_Count, ushort Sweep_Time_Auto__Mask_ACP_, double Sweep_Time__Mask_ACP_, int Frequency_Band, double Channel_Bandwidth, int Guard_Ratio, int FFT_Size, ushort Capture_Count_State, int Number_of_Subframes_to_Analyze);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXMeasWiBro", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXMeasWiBro(System.IntPtr Instrument_Handle, double Capture_Time, ushort Swap_I_Q, int Sweep_Count, ushort Sweep_Time_Auto__Mask_ACP_, double Sweep_Time__Mask_ACP_, double Channel_Bandwidth, ushort Capture_Count_State, int Number_of_Subframes_to_Analyze);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXPVTMeas", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXPVTMeas(System.IntPtr Instrument_Handle, int Rising_Falling, int Burst_Selection);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXPVTMeasOFDMA", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXPVTMeasOFDMA(System.IntPtr Instrument_Handle, int Rising_Falling);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXEVMMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXEVMMeasurement(System.IntPtr Instrument_Handle, int Mode, ushort Auto_Scaling, double Per_Division, int Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXSpectrumMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXSpectrumMeasurement(System.IntPtr Instrument_Handle, int Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXConstellationMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXConstellationMeasurement(System.IntPtr Instrument_Handle, int Constellation_Mode, int Carrier_Selection, int Carrier_Number);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXConstellationSelection", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXConstellationSelection(System.IntPtr Instrument_Handle, int Modulation, int Burst_Selection, int Burst_Number, int Carrier_Selection, int Carrier_Number, int Symbol_Selection, int Symbol_Number);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXBitstreamSelection", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXBitstreamSelection(System.IntPtr Instrument_Handle, int Modulation, int Burst_Selection, int Burst_Number, int Symbol_Selection, int Symbol_Number);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXStatisticsMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXStatisticsMeasurement(System.IntPtr Instrument_Handle, int Mode);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXIQMeasurementParameters", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXIQMeasurementParameters(System.IntPtr Instrument_Handle, double Sample_Rate, int Filter);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXSEMMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXSEMMeasurement(System.IntPtr Instrument_Handle, int Type, string File_Name, int Power_Class);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXTTCFrame", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXTTCFrame(System.IntPtr Instrument_Handle, int Frame);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXTTCStart", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXTTCStart(System.IntPtr Instrument_Handle, int Start);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetWiMAXACPMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetWiMAXACPMeasurement(System.IntPtr Instrument_Handle, int Window, int Marker, int Mode, [In, Out] double[] Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_LoadWiMAXIQData", CallingConvention = CallingConvention.StdCall)]
            public static extern int LoadWiMAXIQData(System.IntPtr Instrument_Handle, string File_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_StoreWiMAXIQData", CallingConvention = CallingConvention.StdCall)]
            public static extern int StoreWiMAXIQData(System.IntPtr Instrument_Handle, string File_Name);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_WiMAXBurstImm", CallingConvention = CallingConvention.StdCall)]
            public static extern int WiMAXBurstImm(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_WiMAXRefreshIQMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int WiMAXRefreshIQMeasurement(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_WiMAXGetMarkerTTC", CallingConvention = CallingConvention.StdCall)]
            public static extern int WiMAXGetMarkerTTC(System.IntPtr Instrument_Handle, int Window, int Marker_Number, out double Time);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadWiMAXTraceData", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadWiMAXTraceData(System.IntPtr Instrument_Handle, int Trace, int Array_Size, [In, Out] double[] Trace_Data, out int No_of_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadWiMAXMemoryIQData", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadWiMAXMemoryIQData(System.IntPtr Instrument_Handle, int Offset_Samples, int No_of_Samples, int Buffer_Size, out int No_of_Points, [In, Out] double[] Real_Parts__I_, [In, Out] double[] Imaginary_Parts__Q_);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadWiMAXSEMResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadWiMAXSEMResults(System.IntPtr Instrument_Handle, int No_Of_Values, [In, Out] double[] Index, [In, Out] double[] Start_Frequency_Band, [In, Out] double[] Stop_Frequency_Band, [In, Out] double[] Resolution_Bandwidth, [In, Out] double[] Limit_Fail_Frequency, [In, Out] double[] Power_Abs, [In, Out] double[] Power_Rel, [In, Out] double[] Limit_Distance, [In, Out] double[] Failure_Flag, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchWiMAXBurstAll", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchWiMAXBurstAll(System.IntPtr Instrument_Handle, [In, Out] double[] Results, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchWiMAXBurstPower", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchWiMAXBurstPower(System.IntPtr Instrument_Handle, int Power_Measurement, int Mode, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchWiMAXBurstError", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchWiMAXBurstError(System.IntPtr Instrument_Handle, int Error_Type, int Mode, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchWiMAXIQImp", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchWiMAXIQImp(System.IntPtr Instrument_Handle, int IQ_Impairment, int Mode, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchWiMAXEVM", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchWiMAXEVM(System.IntPtr Instrument_Handle, int EVM, int Mode, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchWiMAXSubscriberStationTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchWiMAXSubscriberStationTime(System.IntPtr Instrument_Handle, int Mode, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchWiMAXCINRValues", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchWiMAXCINRValues(System.IntPtr Instrument_Handle, [In, Out] double[] Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchWiMAXRSSIValues", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchWiMAXRSSIValues(System.IntPtr Instrument_Handle, [In, Out] double[] Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchWiMAXCount", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchWiMAXCount(System.IntPtr Instrument_Handle, int Type, out int Count);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchWiMAXTimeDomainPreamble", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchWiMAXTimeDomainPreamble(System.IntPtr Instrument_Handle, int Result, out double Minimum, out double Average, out double Maximum);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchWiMAXUnmodulatedSubcarrierError", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchWiMAXUnmodulatedSubcarrierError(System.IntPtr Instrument_Handle, int Mode, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXListResultUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXListResultUnit(System.IntPtr Instrument_Handle, int Unit);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureWiMAXRSSICINRAvg", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureWiMAXRSSICINRAvg(System.IntPtr Instrument_Handle, double Averaging_Parameter);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetWiMAXLimitCheckResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetWiMAXLimitCheckResult(System.IntPtr Instrument_Handle, int Window, int Limit, int Limit_Type, int Mode, out int Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetWiMAXAllLimitCheckResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetWiMAXAllLimitCheckResults(System.IntPtr Instrument_Handle, int Window, int Limit, [In, Out] int[] Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetWiMAXLimitCheck", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetWiMAXLimitCheck(System.IntPtr Instrument_Handle, int Window, int Limit, out int Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SetLTEDownlinkMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetLTEDownlinkMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkSignalCharacteristics", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkSignalCharacteristics(System.IntPtr Instrument_Handle, int Duplexing, double Frequency, int Bandwidth_Setting_Type, int Channel_Bandwidth, int Number_of_Resource_Blocks, int Cyclic_Prefix);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkLevel(System.IntPtr Instrument_Handle, double Reference_Level, ushort Auto_Reference_Level, double External_Attenuation);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkLevelInBBMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkLevelInBBMode(System.IntPtr Instrument_Handle, double Reference_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkMIMO", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkMIMO(System.IntPtr Instrument_Handle, int Configuration, int Antenna_Selection);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkIQ", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkIQ(System.IntPtr Instrument_Handle, ushort Swap_IQ);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkInput", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkInput(System.IntPtr Instrument_Handle, int Source);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_LoadLTEDownlinkDemodulationSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int LoadLTEDownlinkDemodulationSettings(System.IntPtr Instrument_Handle, string File_Path);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_StoreLTEDownlinkDemodulationSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int StoreLTEDownlinkDemodulationSettings(System.IntPtr Instrument_Handle, string File_Path);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkDataAnalysis", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkDataAnalysis(System.IntPtr Instrument_Handle, int Channel_Estimation, int EVM_Calculation_Method, ushort Scrambling_of_Coded_Bits, ushort Auto_Demodulation);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkTracking", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkTracking(System.IntPtr Instrument_Handle, int Phase_Tracking, ushort Timing);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkTDDAllocation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkTDDAllocation(System.IntPtr Instrument_Handle, int TDD_UL_DL_Allocations);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkPhysicalLayerCellIdentity", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkPhysicalLayerCellIdentity(System.IntPtr Instrument_Handle, ushort Auto_Cell_Identity_Group, int Cell_Identity_Group, int Cell_Identity, ushort Auto_Cell_Identity);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkPDSCHConfigurableSubframes", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkPDSCHConfigurableSubframes(System.IntPtr Instrument_Handle, int Configurable_Subframes);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkPDSCHUsedAllocations", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkPDSCHUsedAllocations(System.IntPtr Instrument_Handle, int Subframe, int Used_Allocations);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkPDSCHUsedAllocation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkPDSCHUsedAllocation(System.IntPtr Instrument_Handle, int Subframe, int Used_Allocation, int Codeword, int Modulation, int Resource_Blocks, int Resource_Blocks_Offset, double Power);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkReferenceSignal", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkReferenceSignal(System.IntPtr Instrument_Handle, double Reference_Power);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkSynchronisationSignal", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkSynchronisationSignal(System.IntPtr Instrument_Handle, double P_SYNC_Relative_Power, double S_SYNC_Relative_Power);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkPBCH", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkPBCH(System.IntPtr Instrument_Handle, ushort PBCH_Present, double PBCH_Relative_Power);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkPCFICH", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkPCFICH(System.IntPtr Instrument_Handle, ushort PCFICH_Present, double PCFICH_Relative_Power);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkPHICH", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkPHICH(System.IntPtr Instrument_Handle, int PHICH_Number_of_Groups, int PHICH_Duration, double PHICH_Relative_Power);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkPDCCH", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkPDCCH(System.IntPtr Instrument_Handle, int Number_of_PDCCHs, int PDCCH_Format, double PDCCH_Relative_Power);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_UpdateIQResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int UpdateIQResults(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkMeasurementType", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkMeasurementType(System.IntPtr Instrument_Handle, int Window, int Measurement_Type);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkSubframeSelection", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkSubframeSelection(System.IntPtr Instrument_Handle, ushort All_Subframes, int Single_Subframe_Selection);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkEVMUnits", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkEVMUnits(System.IntPtr Instrument_Handle, int EVM_Units);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkBitStream", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkBitStream(System.IntPtr Instrument_Handle, int Bit_Stream_Format);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkSEMSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkSEMSettings(System.IntPtr Instrument_Handle, int SEM_Channel_Category);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkACPSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkACPSettings(System.IntPtr Instrument_Handle, int Assumed_Adj_Channel_Carrier, ushort Noise_Correction);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureLTEDownlinkConstellationLocation", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureLTEDownlinkConstellationLocation(System.IntPtr Instrument_Handle, int Constellation_Location);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryLTEDownlinkMeasurementResultSummary", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryLTEDownlinkMeasurementResultSummary(System.IntPtr Instrument_Handle, int Frame_Result, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryLTEDownlinkMeasurementEVMAll", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryLTEDownlinkMeasurementEVMAll(System.IntPtr Instrument_Handle, int Measurement_Type, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryLTEDownlinkMeasurementEVMPhysicalChannel", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryLTEDownlinkMeasurementEVMPhysicalChannel(System.IntPtr Instrument_Handle, int Measurement_Type, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryLTEDownlinkMeasurementEVMPhysicalSignal", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryLTEDownlinkMeasurementEVMPhysicalSignal(System.IntPtr Instrument_Handle, int Measurement_Type, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryLTEDownlinkMeasurementFrequencyError", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryLTEDownlinkMeasurementFrequencyError(System.IntPtr Instrument_Handle, int Measurement_Type, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryLTEDownlinkMeasurementSamplingError", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryLTEDownlinkMeasurementSamplingError(System.IntPtr Instrument_Handle, int Measurement_Type, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryLTEDownlinkMeasurementIQOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryLTEDownlinkMeasurementIQOffset(System.IntPtr Instrument_Handle, int Measurement_Type, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryLTEDownlinkMeasurementIQGainImbalance", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryLTEDownlinkMeasurementIQGainImbalance(System.IntPtr Instrument_Handle, int Measurement_Type, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryLTEDownlinkMeasurementIQQuadratureError", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryLTEDownlinkMeasurementIQQuadratureError(System.IntPtr Instrument_Handle, int Measurement_Type, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryLTEDownlinkMeasurementFramePower", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryLTEDownlinkMeasurementFramePower(System.IntPtr Instrument_Handle, int Measurement_Type, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryLTEDownlinkMeasurementCrestFactor", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryLTEDownlinkMeasurementCrestFactor(System.IntPtr Instrument_Handle, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryLTEDownlinkMeasurementTriggerToFrame", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryLTEDownlinkMeasurementTriggerToFrame(System.IntPtr Instrument_Handle, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryLTEDownlinkMeasurementOSTP", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryLTEDownlinkMeasurementOSTP(System.IntPtr Instrument_Handle, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryLTEDownlinkMeasurementACPResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryLTEDownlinkMeasurementACPResult(System.IntPtr Instrument_Handle, int Array_Size, [In, Out] double[] Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryLTEDownlinkDetectedCellIdentityGroup", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryLTEDownlinkDetectedCellIdentityGroup(System.IntPtr Instrument_Handle, out int Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryLTEDownlinkDetectedCellIdentity", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryLTEDownlinkDetectedCellIdentity(System.IntPtr Instrument_Handle, out int Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryLTEDownlinkDetectedCyclicPrefix", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryLTEDownlinkDetectedCyclicPrefix(System.IntPtr Instrument_Handle, out int Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadLTEDownlinkEVMVersusCarrierTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadLTEDownlinkEVMVersusCarrierTrace(System.IntPtr Instrument_Handle, int Measurement_Type, int Array_Size, [In, Out] double[] Trace_Data, out int No_of_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadLTEDownlinkEVMVersusSymbolTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadLTEDownlinkEVMVersusSymbolTrace(System.IntPtr Instrument_Handle, int Array_Size, [In, Out] double[] Trace_Data, out int No_of_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadLTEDownlinkFrequencyErrorVersusSymbolTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadLTEDownlinkFrequencyErrorVersusSymbolTrace(System.IntPtr Instrument_Handle, int Array_Size, [In, Out] double[] Trace_Data, out int No_of_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadLTEDownlinkEVMVersusSubframeTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadLTEDownlinkEVMVersusSubframeTrace(System.IntPtr Instrument_Handle, int Array_Size, [In, Out] double[] Trace_Data, out int No_of_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadLTEDownlinkSEMTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadLTEDownlinkSEMTrace(System.IntPtr Instrument_Handle, int Measurement_Type, int Array_Size, [In, Out] double[] Trace_Data, out int No_of_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadLTEDownlinkACPTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadLTEDownlinkACPTrace(System.IntPtr Instrument_Handle, int Array_Size, [In, Out] double[] Trace_Data, out int No_of_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadLTEDownlinkPowerSpectrumTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadLTEDownlinkPowerSpectrumTrace(System.IntPtr Instrument_Handle, int Array_Size, [In, Out] double[] Trace_Data, out int No_of_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadLTEDownlinkPowerVersusRBReferenceSignalTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadLTEDownlinkPowerVersusRBReferenceSignalTrace(System.IntPtr Instrument_Handle, int Measurement_Type, int Array_Size, [In, Out] double[] Trace_Data, out int No_of_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadLTEDownlinkPowerVersusRBPDSCHTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadLTEDownlinkPowerVersusRBPDSCHTrace(System.IntPtr Instrument_Handle, int Measurement_Type, int Array_Size, [In, Out] double[] Trace_Data, out int No_of_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadLTEDownlinkChannelFlatnessTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadLTEDownlinkChannelFlatnessTrace(System.IntPtr Instrument_Handle, int Measurement_Type, int Array_Size, [In, Out] double[] Trace_Data, out int No_of_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadLTEDownlinkChannelGroupDelayTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadLTEDownlinkChannelGroupDelayTrace(System.IntPtr Instrument_Handle, int Measurement_Type, int Array_Size, [In, Out] double[] Trace_Data, out int No_of_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadLTEDownlinkChannelFlatnessDifferenceTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadLTEDownlinkChannelFlatnessDifferenceTrace(System.IntPtr Instrument_Handle, int Measurement_Type, int Array_Size, [In, Out] double[] Trace_Data, out int No_of_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadLTEDownlinkConstellationDiagram", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadLTEDownlinkConstellationDiagram(System.IntPtr Instrument_Handle, int Buffer_Size, out int No_of_Points, [In, Out] double[] Real_Parts__I_, [In, Out] double[] Imaginary_Parts__Q_);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadLTEDownlinkCCDF", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadLTEDownlinkCCDF(System.IntPtr Instrument_Handle, int Measurement_Type, int Array_Size, [In, Out] double[] Trace_Data, out int No_of_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadLTEDownlinkAllocationSummary", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadLTEDownlinkAllocationSummary(System.IntPtr Instrument_Handle, int Array_Size, [In, Out] int[] Subframe_Number, [In, Out] int[] Allocation_ID, [In, Out] int[] Number_of_RB, [In, Out] int[] Offset_RB, [In, Out] int[] Modulation, [In, Out] double[] Power, [In, Out] double[] EVM, out int No_of_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadLTEDownlinkBitstream", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadLTEDownlinkBitstream(System.IntPtr Instrument_Handle, int Array_Size, int Bit_stream_Size, [In, Out] int[] Subframe_Number, [In, Out] int[] Allocation_ID, [In, Out] int[] Codeword, [In, Out] int[] Modulation, [In, Out] int[] Number_of_Symbols, System.Text.StringBuilder Bit_Streams, out int No_of_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SetTETRAMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTETRAMode(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTETRASignalCharacteristics", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTETRASignalCharacteristics(System.IntPtr Instrument_Handle, double Frequency, int Slot_Type, int Channel_Bandwidth);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTETRALevelSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTETRALevelSettings(System.IntPtr Instrument_Handle, ushort Auto_Level, double Level, double External_Attenuation);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTETRADataCaptureSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTETRADataCaptureSettings(System.IntPtr Instrument_Handle, double Capture_Time, int Capture_Time_Unit, ushort Use_Number_Of_Slots_To_Analyzer, int Number_Of_Slots_To_Analyze);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTETRATriggerSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTETRATriggerSettings(System.IntPtr Instrument_Handle, int Mode, double Offset, int Offset_Unit, double Ext__Trigger_Level, ushort Auto_Trigger_Level, double Power_Trigger_Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTETRAIQSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTETRAIQSettings(System.IntPtr Instrument_Handle, ushort Swap_IQ);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTETRAInputSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTETRAInputSettings(System.IntPtr Instrument_Handle, int Input);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTETRAAdvancedBasebandSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTETRAAdvancedBasebandSettings(System.IntPtr Instrument_Handle, int Impedance, int Path, ushort Balanced, ushort Low_Pass, ushort IQ_Dither);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTETRAAdvancedIQDigital", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTETRAAdvancedIQDigital(System.IntPtr Instrument_Handle, ushort Auto_Level, double Level, ushort Auto_Sampling_Rate, double Sampling_Rate);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTETRAIQCenterFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTETRAIQCenterFrequency(System.IntPtr Instrument_Handle, double IQ_Center_Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTETRADemodulationSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTETRADemodulationSettings(System.IntPtr Instrument_Handle, int Slot_Type, int Payload_Modulation, double Demod_RRC_Roll_off, int Max_Carrier_Offset);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTETRAEVMSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTETRAEVMSettings(System.IntPtr Instrument_Handle, ushort Pilot_Tracking, ushort Compensate_Amplitude_Droop, ushort Compensate_IQ_Offset);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTETRATraceSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTETRATraceSettings(System.IntPtr Instrument_Handle, int Trace, ushort Trace_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTETRAMarker", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTETRAMarker(System.IntPtr Instrument_Handle, int Window, int Marker, ushort Marker_Enabled, int Trace);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_TETRAMarkerSearch", CallingConvention = CallingConvention.StdCall)]
            public static extern int TETRAMarkerSearch(System.IntPtr Instrument_Handle, int Window, int Marker, int Marker_Search);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_TETRAMoveMarker", CallingConvention = CallingConvention.StdCall)]
            public static extern int TETRAMoveMarker(System.IntPtr Instrument_Handle, int Window, int Marker, int Measurement, double Marker_Position);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryTETRAMarkerAmplitude", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryTETRAMarkerAmplitude(System.IntPtr Instrument_Handle, int Window, int Marker, out double Marker_Amplitude);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTETRADisplayFormat", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTETRADisplayFormat(System.IntPtr Instrument_Handle, int Format);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTETRADisplayActiveWindow", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTETRADisplayActiveWindow(System.IntPtr Instrument_Handle, int Window);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTETRAResultSummaryLimits", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTETRAResultSummaryLimits(System.IntPtr Instrument_Handle, int Limit_Type, int Mode, double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTETRAResultSummaryLimitsAll", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTETRAResultSummaryLimitsAll(System.IntPtr Instrument_Handle, int Array_Size, double[] Limit_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetTETRAResultSummaryLimitsAll", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTETRAResultSummaryLimitsAll(System.IntPtr Instrument_Handle, int Array_Size, [In, Out] double[] Limit_Values, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTETRAACPLimits", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTETRAACPLimits(System.IntPtr Instrument_Handle, int ACP, int Channel_Number, int Channel_Bandwidth, double Limit_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTETRAMeasurementType", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTETRAMeasurementType(System.IntPtr Instrument_Handle, int Measurement);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTETRAYAxis", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTETRAYAxis(System.IntPtr Instrument_Handle, int Measurement, ushort Auto_Scaling, int Unit, double Y_Center, double Y_Span);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTETRAContinuousMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTETRAContinuousMeasurement(System.IntPtr Instrument_Handle, ushort Continuous);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTETRAConstellationSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTETRAConstellationSettings(System.IntPtr Instrument_Handle, int Symbols_to_Plot_Selection, int Carrier_Selection, int Carrier_Number);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTETRAEVMResultUnits", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTETRAEVMResultUnits(System.IntPtr Instrument_Handle, int EVM_Units);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConfigureTETRAGainImbalanceResultUnits", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTETRAGainImbalanceResultUnits(System.IntPtr Instrument_Handle, int Gain_Imbalance_Units);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_TETRABurstRecalc", CallingConvention = CallingConvention.StdCall)]
            public static extern int TETRABurstRecalc(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadTETRATraceData", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadTETRATraceData(System.IntPtr Instrument_Handle, int Window, int Trace, int Array_Size, [In, Out] double[] Trace_Data, out int No_of_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadTETRABitstream", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadTETRABitstream(System.IntPtr Instrument_Handle, int Window, int Trace, int Array_Size, [In, Out] double[] Trace_Data, out int No_of_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchTETRANumberOfBursts", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchTETRANumberOfBursts(System.IntPtr Instrument_Handle, out int Number_Of_Burst);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchTETRAMeasuremenResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchTETRAMeasuremenResults(System.IntPtr Instrument_Handle, int Measurement, int Measurement_Type, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchTETRAMeasurementResultsAll", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchTETRAMeasurementResultsAll(System.IntPtr Instrument_Handle, int Array_Size, [In, Out] double[] Result, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchTETRASubcarrierMeanPower", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchTETRASubcarrierMeanPower(System.IntPtr Instrument_Handle, int Measurement_Type, int Array_Size, [In, Out] double[] Result, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchTETRASubcarrierReferencePower", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchTETRASubcarrierReferencePower(System.IntPtr Instrument_Handle, int Measurement_Type, int Array_Size, [In, Out] double[] Result, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchTETRAPVTMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchTETRAPVTMeasurement(System.IntPtr Instrument_Handle, int PVT, int Measurement_Type, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchTETRAPVTMeasurementAll", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchTETRAPVTMeasurementAll(System.IntPtr Instrument_Handle, [In, Out] double[] Results, int Array_Size, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchTETRASpectrumFFTMeasurementRBW", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchTETRASpectrumFFTMeasurementRBW(System.IntPtr Instrument_Handle, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FetchTETRAACP", CallingConvention = CallingConvention.StdCall)]
            public static extern int FetchTETRAACP(System.IntPtr Instrument_Handle, int ACP, int ACP_Measurement, int Channel_Number, int Measurement_Type, out double Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetTETRATraceXAxisMinMax", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTETRATraceXAxisMinMax(System.IntPtr Instrument_Handle, int Window, int Trace, int Value_Type, out double X_Axis_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetTETRAIQDataSamplingRate", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTETRAIQDataSamplingRate(System.IntPtr Instrument_Handle, out double IQ_Data_Sampling_Rate);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetTETRAResultSummaryLimitCheckResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTETRAResultSummaryLimitCheckResult(System.IntPtr Instrument_Handle, int Limit_Type, int Mode, out int Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetTETRAResultSummaryLimitCheckResultsAll", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTETRAResultSummaryLimitCheckResultsAll(System.IntPtr Instrument_Handle, int Array_Size, [In, Out] int[] Results, out int Returned_Values);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetTETRAACPLimitCheckResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTETRAACPLimitCheckResult(System.IntPtr Instrument_Handle, int ACP, int Channel_Number, int Measurement_Type, out int Result);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ServiceConfigureInput", CallingConvention = CallingConvention.StdCall)]
            public static extern int ServiceConfigureInput(System.IntPtr Instrument_Handle, int Window, int Input, double Level);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ServiceConfigurePulsedInput", CallingConvention = CallingConvention.StdCall)]
            public static extern int ServiceConfigurePulsedInput(System.IntPtr Instrument_Handle, int Window, ushort State, double Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ServiceConfigureRectangleInput", CallingConvention = CallingConvention.StdCall)]
            public static extern int ServiceConfigureRectangleInput(System.IntPtr Instrument_Handle, int Window, ushort State, double Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ServiceConfigureNoise", CallingConvention = CallingConvention.StdCall)]
            public static extern int ServiceConfigureNoise(System.IntPtr Instrument_Handle, ushort Noise);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ServiceConfigureBasebandSource", CallingConvention = CallingConvention.StdCall)]
            public static extern int ServiceConfigureBasebandSource(System.IntPtr Instrument_Handle, int Source);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ServiceConfigurePulsedIQCalibrationSignal", CallingConvention = CallingConvention.StdCall)]
            public static extern int ServiceConfigurePulsedIQCalibrationSignal(System.IntPtr Instrument_Handle, int Destination, double Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ServiceConfigureDCIQCalibrationSignal", CallingConvention = CallingConvention.StdCall)]
            public static extern int ServiceConfigureDCIQCalibrationSignal(System.IntPtr Instrument_Handle, double Voltage);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ServiceConfigureCombGenerator", CallingConvention = CallingConvention.StdCall)]
            public static extern int ServiceConfigureCombGenerator(System.IntPtr Instrument_Handle, int Window, int Comb_Frequency);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ServiceHWInfo", CallingConvention = CallingConvention.StdCall)]
            public static extern int ServiceHWInfo(System.IntPtr Instrument_Handle, int Array_Size, System.Text.StringBuilder HW_Info);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ServiceVersionInfo", CallingConvention = CallingConvention.StdCall)]
            public static extern int ServiceVersionInfo(System.IntPtr Instrument_Handle, int Array_Size, System.Text.StringBuilder Version_Info);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FileDecimalSeparator", CallingConvention = CallingConvention.StdCall)]
            public static extern int FileDecimalSeparator(System.IntPtr Instrument_Handle, int Decimal_Separator);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FileManagerOperations", CallingConvention = CallingConvention.StdCall)]
            public static extern int FileManagerOperations(System.IntPtr Instrument_Handle, int Operation, string Source, string Destination);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FileDirectory", CallingConvention = CallingConvention.StdCall)]
            public static extern int FileDirectory(System.IntPtr Instrument_Handle, string Directory, int Buffer_Size, System.Text.StringBuilder Output);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_FileDirectoryPath", CallingConvention = CallingConvention.StdCall)]
            public static extern int FileDirectoryPath(System.IntPtr Instrument_Handle, string Directory, int Buffer_Size, System.Text.StringBuilder Output);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_DataSetFileOperations", CallingConvention = CallingConvention.StdCall)]
            public static extern int DataSetFileOperations(System.IntPtr Instrument_Handle, int Operation, string Path);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_DataSetFileSaveNext", CallingConvention = CallingConvention.StdCall)]
            public static extern int DataSetFileSaveNext(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_DataSetFileComment", CallingConvention = CallingConvention.StdCall)]
            public static extern int DataSetFileComment(System.IntPtr Instrument_Handle, string Comment);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_DataSetFileClearAll", CallingConvention = CallingConvention.StdCall)]
            public static extern int DataSetFileClearAll(System.IntPtr Instrument_Handle, string Directory);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_DataSetFileSelectItems", CallingConvention = CallingConvention.StdCall)]
            public static extern int DataSetFileSelectItems(System.IntPtr Instrument_Handle, int Item_Selector, ushort Item_State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_NetworkMapDrive", CallingConvention = CallingConvention.StdCall)]
            public static extern int NetworkMapDrive(System.IntPtr Instrument_Handle, string Drive_Letter, string Host_Name___IP_Address, ushort Use_Optional_Parameters, string User_Name, string Password, ushort Reconnect_At_Logon);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_NetworkDisconnectDrive", CallingConvention = CallingConvention.StdCall)]
            public static extern int NetworkDisconnectDrive(System.IntPtr Instrument_Handle, string Drive_Letter);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_NetworkUnusedDrivesList", CallingConvention = CallingConvention.StdCall)]
            public static extern int NetworkUnusedDrivesList(System.IntPtr Instrument_Handle, int Array_Size, System.Text.StringBuilder Unused_Drives);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_NetworkUsedDrivesList", CallingConvention = CallingConvention.StdCall)]
            public static extern int NetworkUsedDrivesList(System.IntPtr Instrument_Handle, int Array_Size, System.Text.StringBuilder Used_Drives);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_NetworkShowFolderInfo", CallingConvention = CallingConvention.StdCall)]
            public static extern int NetworkShowFolderInfo(System.IntPtr Instrument_Handle, ushort Show_Folder_Info);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_reset", CallingConvention = CallingConvention.StdCall)]
            public static extern int reset(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Disable", CallingConvention = CallingConvention.StdCall)]
            public static extern int Disable(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_setCheckStatus", CallingConvention = CallingConvention.StdCall)]
            public static extern int setCheckStatus(System.IntPtr Instrument_Handle, ushort State);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_self_test", CallingConvention = CallingConvention.StdCall)]
            public static extern int self_test(System.IntPtr Instrument_Handle, out short Self_Test_Result, System.Text.StringBuilder Self_Test_Message);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SelfTestResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int SelfTestResults(System.IntPtr Instrument_Handle, int Buffer_Size, System.Text.StringBuilder Results);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_revision_query", CallingConvention = CallingConvention.StdCall)]
            public static extern int revision_query(System.IntPtr Instrument_Handle, System.Text.StringBuilder Instrument_Driver_Revision, System.Text.StringBuilder Firmware_Revision);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_setCheckOption", CallingConvention = CallingConvention.StdCall)]
            public static extern int setCheckOption(System.IntPtr Instrument_Handle, ushort Option_Checking);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SystemVersion", CallingConvention = CallingConvention.StdCall)]
            public static extern int SystemVersion(System.IntPtr Instrument_Handle, int Length, System.Text.StringBuilder System_Version);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_MemorySize", CallingConvention = CallingConvention.StdCall)]
            public static extern int MemorySize(System.IntPtr Instrument_Handle, out int Memory_Size);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_error_query", CallingConvention = CallingConvention.StdCall)]
            public static extern int error_query(System.IntPtr Instrument_Handle, out int Error_Code, System.Text.StringBuilder Error_Message);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ErrorList", CallingConvention = CallingConvention.StdCall)]
            public static extern int ErrorList(System.IntPtr Instrument_Handle, int Buffer_Size, System.Text.StringBuilder Errors);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ClearInstrumentErrors", CallingConvention = CallingConvention.StdCall)]
            public static extern int ClearInstrumentErrors(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_Preset", CallingConvention = CallingConvention.StdCall)]
            public static extern int Preset(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_IDStringFactory", CallingConvention = CallingConvention.StdCall)]
            public static extern int IDStringFactory(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetIDStringFactory", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetIDStringFactory(System.IntPtr Instrument_Handle, out int Factory_Default);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadToFileFromInstrument", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadToFileFromInstrument(System.IntPtr Instrument_Handle, string Source, string Destination);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_WriteFromFileToInstrument", CallingConvention = CallingConvention.StdCall)]
            public static extern int WriteFromFileToInstrument(System.IntPtr Instrument_Handle, string Source, string Destination);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_WriteInstrData", CallingConvention = CallingConvention.StdCall)]
            public static extern int WriteInstrData(System.IntPtr Instrument_Handle, string Write_Buffer);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ReadInstrData", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadInstrData(System.IntPtr Instrument_Handle, int Number_of_Bytes_To_Read, System.Text.StringBuilder Read_Buffer, out uint Num_Bytes_Read);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryViBoolean", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryViBoolean(System.IntPtr Instrument_Handle, string Command, out ushort Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryViInt32", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryViInt32(System.IntPtr Instrument_Handle, string Command, out int Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryViReal64", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryViReal64(System.IntPtr Instrument_Handle, string Command, out double Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_QueryViString", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryViString(System.IntPtr Instrument_Handle, string Command, int Buffer_Size, System.Text.StringBuilder Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_setStatusRegister", CallingConvention = CallingConvention.StdCall)]
            public static extern int setStatusRegister(System.IntPtr Instrument_Handle, int Register_Operation, int Questionable_Register, int Enable, int PTransition, int NTransition);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_getStatusRegister", CallingConvention = CallingConvention.StdCall)]
            public static extern int getStatusRegister(System.IntPtr Instrument_Handle, int Status_Registers_Query, out int Register_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConditionFrequencyRegister", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConditionFrequencyRegister(System.IntPtr Instrument_Handle, out int Status_Register_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConditionLimitRegister", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConditionLimitRegister(System.IntPtr Instrument_Handle, int Window, out int Status_Register_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConditionLimitMarginRegister", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConditionLimitMarginRegister(System.IntPtr Instrument_Handle, int Window, out int Status_Register_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConditionACPLimitRegister", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConditionACPLimitRegister(System.IntPtr Instrument_Handle, out int Status_Register_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConditionPowerRegister", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConditionPowerRegister(System.IntPtr Instrument_Handle, out int Status_Register_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_ConditionSyncRegister", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConditionSyncRegister(System.IntPtr Instrument_Handle, out int Status_Register_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_close", CallingConvention = CallingConvention.StdCall)]
            public static extern int close(System.IntPtr Instrument_Handle);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_error_message", CallingConvention = CallingConvention.StdCall)]
            public static extern int error_message(System.IntPtr Instrument_Handle, int Error_Code, System.Text.StringBuilder Error_Message_2);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetAttributeViInt32", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAttributeViInt32(System.IntPtr Instrument_Handle, string Repeated_Capability_Name, uint Attribute_ID, out int Attribute_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetAttributeViReal64", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAttributeViReal64(System.IntPtr Instrument_Handle, string Repeated_Capability_Name, uint Attribute_ID, out double Attribute_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetAttributeViString", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAttributeViString(System.IntPtr Instrument_Handle, string Repeated_Capability_Name, uint Attribute_ID, int Buffer_Size, System.Text.StringBuilder Attribute_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetAttributeViBoolean", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAttributeViBoolean(System.IntPtr Instrument_Handle, string Repeated_Capability_Name, uint Attribute_ID, out ushort Attribute_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetAttributeViSession", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAttributeViSession(System.IntPtr Instrument_Handle, string Repeated_Capability_Name, uint Attribute_ID, out System.IntPtr Attribute_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetAttributeRepeatedCapabilityIds", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAttributeRepeatedCapabilityIds(System.IntPtr Instrument_Handle, uint Attribute_ID, int Buffer_Size, System.Text.StringBuilder Repeated_Capability_Id_s_);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetAttributeRepeatedCapabilityIdNames", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAttributeRepeatedCapabilityIdNames(System.IntPtr Instrument_Handle, uint Attribute_ID, string Repeated_Capability_Id, int Buffer_Size, System.Text.StringBuilder Repeated_Capability_Id_Name_s_);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SetAttributeViInt32", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAttributeViInt32(System.IntPtr Instrument_Handle, string Repeated_Capability_Name, uint Attribute_ID, int Attribute_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SetAttributeViReal64", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAttributeViReal64(System.IntPtr Instrument_Handle, string Repeated_Capability_Name, uint Attribute_ID, double Attribute_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SetAttributeViString", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAttributeViString(System.IntPtr Instrument_Handle, string Repeated_Capability_Name, uint Attribute_ID, string Attribute_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SetAttributeViBoolean", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAttributeViBoolean(System.IntPtr Instrument_Handle, string Repeated_Capability_Name, uint Attribute_ID, ushort Attribute_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_SetAttributeViSession", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAttributeViSession(System.IntPtr Instrument_Handle, string Repeated_Capability_Name, uint Attribute_ID, System.IntPtr Attribute_Value);

            [DllImport("rsspecan_32.dll", EntryPoint = "rsspecan_GetError", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetError(System.IntPtr Instrument_Handle, out int Code, int BufferSize, System.Text.StringBuilder Description);


            public static int TestForError(System.IntPtr handle, int status)
            {
                if ((status < 0))
                {
                    PInvoke.ThrowError(handle, status);
                }
                return status;
            }

            public static int ThrowError(System.IntPtr handle, int code)
            {
                int status;
                int size = PInvoke.GetError(handle, out status, 0, null);
                System.Text.StringBuilder msg = new System.Text.StringBuilder();
                if ((size >= 0))
                {
                    msg.Capacity = size;
                    PInvoke.GetError(handle, out status, size, msg);
                }
                throw new System.Runtime.InteropServices.ExternalException(msg.ToString(), code);
            }
        }
    }

    public class rsspecanConstants
    {

        public const int CentFreqLinkOff = 0;

        public const int CentFreqLinkSpan = 1;

        public const int CentFreqLinkRbw = 2;

        public const int CentFreqLinkDivt = 3;

        public const int FreqModeCw = 0;

        public const int FreqModeFix = 1;

        public const int FreqModeSwe = 2;

        public const int FreqModeScan = 3;

        public const int FreqInputModeChan = 0;

        public const int FreqInputModeFreq = 1;

        public const int CouplingRbw = 0;

        public const int CouplingVbw = 1;

        public const int CouplingSweepTime = 2;

        public const int CouplingRatioRbw = 0;

        public const int CouplingRatioVbw = 1;

        public const int PllAuto = 0;

        public const int PllLow = 1;

        public const int PllMed = 2;

        public const int PllHigh = 3;

        public const int FiltTypeNorm = 0;

        public const int FiltTypeFft = 1;

        public const int FiltTypeCfil = 2;

        public const int FiltTypeRrc = 3;

        public const int FiltTypeP5 = 4;

        public const int FiltTypePulse = 5;

        public const int FiltTypeNoise = 6;

        public const int FiltTypeP5d = 7;

        public const int VbwFiltLin = 0;

        public const int VbwFiltLog = 1;

        public const int FftModeAuto = 0;

        public const int FftModeWide = 1;

        public const int FftModeNarrow = 2;

        public const int AvgLin = 0;

        public const int AvgVid = 1;

        public const int AvgPow = 2;

        public const int TraceSpaceLin = 0;

        public const int TraceSpaceLog = 1;

        public const int TraceSpacePct = 2;

        public const int TraceSpaceLdb = 3;

        public const int Lin = 0;

        public const int Log = 1;

        public const int SweepTypeAuto = 0;

        public const int SweepTypeSweep = 1;

        public const int SweepTypeFft = 2;

        public const int AmplitudeUnitsDbm = 0;

        public const int AmplitudeUnitsDbmv = 1;

        public const int AmplitudeUnitsDbuv = 2;

        public const int AmplitudeUnitsVolt = 3;

        public const int AmplitudeUnitsWatt = 4;

        public const int AmplitudeUnitsDbpw = 5;

        public const int AmplitudeUnitsDbua = 6;

        public const int AmplitudeUnitsA = 7;

        public const int AmplitudeUnitsDb = 8;

        public const int AmplitudeUnitsPct = 9;

        public const int AmplitudeUnitsUnitless = 10;

        public const int AmplitudeUnitsDbpt = 11;

        public const int AmplitudeUnitsDbuvMhz = 12;

        public const int AmplitudeUnitsDbmvMhz = 13;

        public const int AmplitudeUnitsDbuaMhz = 14;

        public const int AmplitudeUnitsDbuvM = 15;

        public const int AmplitudeUnitsDbuaM = 16;

        public const int AmplitudeUnitsDbuvMmhz = 17;

        public const int AmplitudeUnitsDbuaMmhz = 18;

        public const int AttModeNorm = 0;

        public const int AttModeLdis = 1;

        public const int CouplingNone = 0;

        public const int CouplingRlev = 1;

        public const int CouplingCfb = 2;

        public const int CouplingCfa = 3;

        public const int IfOutputSourceIf = 0;

        public const int IfOutputSourceVid = 1;

        public const int IfOutputSourceHvid = 2;

        public const int UnitDbm = 0;

        public const int UnitDbpw = 1;

        public const int UnitWatt = 2;

        public const int UnitDbuv = 3;

        public const int UnitDbmv = 4;

        public const int UnitVolt = 5;

        public const int UnitDbua = 6;

        public const int UnitAmp = 7;

        public const int UnitDb = 8;

        public const int UnitPct = 13;

        public const int UnitUnitless = 14;

        public const int UnitDbpt = 15;

        public const int UnitDbmvMhz = 16;

        public const int UnitDbuvMhz = 17;

        public const int UnitDbuaMhz = 18;

        public const int UnitDbmvM = 19;

        public const int UnitDbuaM = 20;

        public const int UnitDbmvMmhz = 21;

        public const int UnitDbuaMmhz = 22;

        public const int UnitDbuvM = 23;

        public const int UnitLengthMeter = 0;

        public const int UnitLengthFeet = 1;

        public const int TracModWrite = 0;

        public const int TracModView = 1;

        public const int TracModAver = 2;

        public const int TracModMaxh = 3;

        public const int TracModMinh = 4;

        public const int TracModBlank = 5;

        public const int DetectorTypeAutoPeak = 0;

        public const int DetectorTypeAverage = 1;

        public const int DetectorTypeMaxPeak = 2;

        public const int DetectorTypeMinPeak = 3;

        public const int DetectorTypeSample = 4;

        public const int DetectorTypeRms = 5;

        public const int DetectorTypeQpk = 6;

        public const int DetectorTypeCrms = 7;

        public const int DetectorTypeCav = 8;

        public const int TrgImm = 0;

        public const int TrgExt = 1;

        public const int TrgIfp = 2;

        public const int TrgVid = 3;

        public const int TrgTime = 4;

        public const int Neg = 0;

        public const int Pos = 1;

        public const int IqFiltNorm = 0;

        public const int IqFiltWide = 1;

        public const int GateTypeLevel = 0;

        public const int GateTypeEdge = 1;

        public const int IqDataTypeRimag = 0;

        public const int IqDataTypeMagn = 1;

        public const int IqDataTypeFreq = 2;

        public const int IqDataTypeVect = 3;

        public const int Mpow = 2;

        public const int MarkerSearchHighest = 1;

        public const int MarkerSearchMinimum = 2;

        public const int MarkerSearchNextPeak = 3;

        public const int MarkerSearchNextPeakLeft = 4;

        public const int MarkerSearchNextPeakRight = 5;

        public const int MarkerSearchNextMinimum = 6;

        public const int MarkerSearchNextMinimumLeft = 7;

        public const int MarkerSearchNextMinimumRight = 8;

        public const int InstrumentSettingFrequencyCenter = 1;

        public const int InstrumentSettingFrequencyStep = 2;

        public const int InstrumentSettingReferenceLevel = 3;

        public const int DisplayZoomFactor2 = 0;

        public const int DisplayZoomFactor4 = 1;

        public const int DisplayZoomFactor8 = 2;

        public const int MarkStepSizeStan = 0;

        public const int MarkStepSizePoin = 1;

        public const int MarkerDisplayModeFreq = 0;

        public const int MarkerDisplayModeChan = 1;

        public const int MarkerDemodAm = 0;

        public const int MarkerDemodFm = 1;

        public const int MarkerSortX = 0;

        public const int MarkerSortY = 1;

        public const int Abs = 0;

        public const int Rel = 1;

        public const int SourInt = 0;

        public const int SourExt = 1;

        public const int SourEaut = 2;

        public const int PllBwidNorm = 0;

        public const int PllBwidWide = 1;

        public const int BncModeRef = 0;

        public const int BncModeTrig = 1;

        public const int BncModeBias = 2;

        public const int BncModeIf3 = 3;

        public const int BncModeIfd = 4;

        public const int ImpedancePadRam = 0;

        public const int ImpedancePadRaz = 1;

        public const int ImpedancePadHzte = 2;

        public const int IfShiftOff = 0;

        public const int IfShiftA = 1;

        public const int IfShiftB = 2;

        public const int IfShiftAuto = 3;

        public const int TrgB6Rfp = 0;

        public const int TrgB6Tv = 1;

        public const int TrigTvVfielAll = 0;

        public const int TrigTvVfielOdd = 1;

        public const int TrigTvVfielEven = 2;

        public const int TrigTvHorizontal = 3;

        public const int EgatSourIfp = 1;

        public const int EgatSourExt = 0;

        public const int EgatSourVid = 2;

        public const int EgatTrigLev = 0;

        public const int EgatTrigEdge = 1;

        public const int ExtMixBandA = 0;

        public const int ExtMixBandQ = 1;

        public const int ExtMixBandU = 2;

        public const int ExtMixBandV = 3;

        public const int ExtMixBandE = 4;

        public const int ExtMixBandW = 5;

        public const int ExtMixBandF = 6;

        public const int ExtMixBandD = 7;

        public const int ExtMixBandG = 8;

        public const int ExtMixBandY = 9;

        public const int ExtMixBandJ = 10;

        public const int ExtMixBandUser = 11;

        public const int ExtMixHarmOdd = 0;

        public const int ExtMixHarmEven = 1;

        public const int ExtMixHarmEodd = 2;

        public const int StateOff = 0;

        public const int StateOn = 1;

        public const int StateAuto = 2;

        public const int StateAll = 3;

        public const int MixerLossAver = 0;

        public const int MixerLossTabl = 1;

        public const int CvlBandA = 0;

        public const int CvlBandQ = 1;

        public const int CvlBandU = 2;

        public const int CvlBandV = 3;

        public const int CvlBandE = 4;

        public const int CvlBandW = 5;

        public const int CvlBandF = 6;

        public const int CvlBandD = 7;

        public const int CvlBandG = 8;

        public const int CvlBandY = 9;

        public const int CvlBandJ = 10;

        public const int CvlBandUser = 11;

        public const int Ac = 1;

        public const int Dc = 0;

        public const int EattOff = 0;

        public const int EattAuto = 1;

        public const int EattMan = 2;

        public const int Freq = 0;

        public const int Time = 1;

        public const int LimitLower = 0;

        public const int LimitUpper = 1;

        public const int UnitDeg = 9;

        public const int UnitRad = 10;

        public const int UnitS = 11;

        public const int UnitHz = 12;

        public const int LimitControl = 2;

        public const int LimitMoveOffset = 0;

        public const int LimitMoveShift = 1;

        public const int LineThr = 0;

        public const int LineDisplay = 1;

        public const int LineFreq = 2;

        public const int LineTime = 3;

        public const int DispFormatSingle = 0;

        public const int DispFormatSplit = 1;

        public const int DispSizeLarge = 0;

        public const int DispSizeSmall = 1;

        public const int TouchOff = 0;

        public const int TouchOn = 1;

        public const int TouchFram = 2;

        public const int DisplayDateFormatDdmm = 0;

        public const int DisplayDateFormatMmdd = 1;

        public const int DisplayColorSchemeCol = 0;

        public const int DisplayColorSchemeBw = 1;

        public const int DispColBlac = 0;

        public const int DispColBlue = 1;

        public const int DispColBrow = 2;

        public const int DispColGre = 3;

        public const int DispColCyan = 4;

        public const int DispColRed = 5;

        public const int DispColMag = 6;

        public const int DispColYell = 7;

        public const int DispColWhit = 8;

        public const int DispColDgra = 9;

        public const int DispColLgra = 10;

        public const int DispColLblu = 11;

        public const int DispColLgre = 12;

        public const int DispColLcy = 13;

        public const int DispColLred = 14;

        public const int DispColLmag = 15;

        public const int TfacUnitDb = 0;

        public const int TfacUnitDbm = 1;

        public const int TfacUnitDbmv = 2;

        public const int TfacUnitDbuv = 3;

        public const int TfacUnitDbuvm = 4;

        public const int TfacUnitDbua = 5;

        public const int TfacUnitDbuam = 6;

        public const int TfacUnitDbpw = 7;

        public const int TfacUnitDbpt = 8;

        public const int HcopyDestBmp = 0;

        public const int HcopyDestWmf = 1;

        public const int HcopyDestEmf = 2;

        public const int HcopyDestClp = 3;

        public const int HcopyDestPrn = 4;

        public const int HcopyDestJpg = 5;

        public const int HcopyDestPng = 6;

        public const int HcopyDeviceOrientLand = 0;

        public const int HcopyDeviceOrientPort = 1;

        public const int HcopAll = 0;

        public const int HcopTable = 1;

        public const int HcopTrace = 2;

        public const int ProbeModeDc = 0;

        public const int ProbeModePuls = 1;

        public const int ProbeModeComp = 2;

        public const int UserPortInp = 0;

        public const int UserPortOut = 1;

        public const int AdjAll = 0;

        public const int AdjFreq = 1;

        public const int AdjLev = 2;

        public const int Auto = 0;

        public const int Man = 1;

        public const int AutoscaleAll = 0;

        public const int AutoscaleA = 1;

        public const int AutoscaleB = 2;

        public const int AutoscaleC = 3;

        public const int AutoscaleD = 4;

        public const int FormatAsc = 0;

        public const int FormatReal = 1;

        public const int TraceFormComp = 0;

        public const int TraceFormIqbl = 1;

        public const int TraceFormIqp = 2;

        public const int TdomPeak = 0;

        public const int TdomRms = 1;

        public const int TdomMean = 2;

        public const int TdomSdev = 3;

        public const int TdomResult = 0;

        public const int TdomAvgResult = 1;

        public const int TdomMaxHoldResult = 2;

        public const int MeasPowAcp = 0;

        public const int MeasPowCpow = 1;

        public const int MeasPowMcac = 2;

        public const int MeasPowOban = 3;

        public const int MeasPowCn = 4;

        public const int MeasPowCn0 = 5;

        public const int MeasPowModeWrite = 0;

        public const int MeasPowModeMaxh = 1;

        public const int AcpTx = 0;

        public const int AcpAdj = 1;

        public const int AcpAlt = 2;

        public const int CpowUnitDbm = 0;

        public const int CpowUnitDbmv = 1;

        public const int CpowUnitDbuv = 2;

        public const int CpowUnitVolt = 3;

        public const int CpowUnitWatt = 4;

        public const int CpowUnitDuvm = 5;

        public const int CpowUnitDuam = 6;

        public const int CpowUnitV = 7;

        public const int CpowUnitW = 8;

        public const int CpowUnitVM = 9;

        public const int CpowUnitWM2 = 10;

        public const int MeasPowStdNone = 0;

        public const int MeasPowStdNadc = 1;

        public const int MeasPowStdTetra = 2;

        public const int MeasPowStdPdc = 3;

        public const int MeasPowStdPhs = 4;

        public const int MeasPowStdCdpd = 5;

        public const int MeasPowStdFis95a = 6;

        public const int MeasPowStdRis95a = 7;

        public const int MeasPowStdFis95c0 = 8;

        public const int MeasPowStdRis95c0 = 9;

        public const int MeasPowStdFj008 = 10;

        public const int MeasPowStdRj008 = 11;

        public const int MeasPowStdFis95c1 = 12;

        public const int MeasPowStdRis95c1 = 13;

        public const int MeasPowStdFwcd = 14;

        public const int MeasPowStdRwcd = 15;

        public const int MeasPowStdFw3g = 16;

        public const int MeasPowStdRw3g = 17;

        public const int MeasPowStdD2cd = 18;

        public const int MeasPowStdS2cd = 19;

        public const int MeasPowStdM2cd = 20;

        public const int MeasPowStdTcdm = 21;

        public const int MeasPowStdAwl = 22;

        public const int MeasPowStdBwl = 23;

        public const int MeasPowStdWim = 24;

        public const int MeasPowStdWibro = 25;

        public const int MeasPowStdRtcdma = 26;

        public const int MeasPowStdRfid = 27;

        public const int MeasPowStdGsm = 28;

        public const int MeasPowStdEurt = 29;

        public const int MeasPowStdReut = 30;

        public const int UstdSet = 0;

        public const int UstdSave = 1;

        public const int UstdDel = 2;

        public const int AdjPreAcp = 0;

        public const int AdjPreCpow = 1;

        public const int AdjPreMcac = 2;

        public const int AdjPreOban = 3;

        public const int AdjPreCn = 4;

        public const int AdjPreCn0 = 5;

        public const int WfilterAll = 0;

        public const int WfilterAdj = 1;

        public const int WfilterAlt = 2;

        public const int WfilterTx = 3;

        public const int AdjRchanAutoMin = 0;

        public const int AdjRchanAutoMax = 1;

        public const int AdjRchanAutoLhig = 2;

        public const int SstatOff = 0;

        public const int SstatApd = 1;

        public const int SstatCcdf = 2;

        public const int Pct = 1;

        public const int SstatProb01 = 0;

        public const int SstatProb1 = 1;

        public const int SstatProb10 = 2;

        public const int SstatResultMean = 0;

        public const int SstatResultPeak = 1;

        public const int SstatResultCrest = 2;

        public const int ToiPosCalc = 0;

        public const int ToiPosSear = 1;

        public const int BursMeasMean = 0;

        public const int BursMeasPeak = 1;

        public const int MpowFtypeNorm = 0;

        public const int MpowFtypeCfil = 1;

        public const int MpowFtypeRrc = 2;

        public const int ListFtypeNorm = 0;

        public const int ListFtypeChan = 1;

        public const int ListFtypeRrc = 2;

        public const int ListFtypeP5 = 3;

        public const int ListFtypePuls = 4;

        public const int ListFtypeP5d = 6;

        public const int ListRangDetApe = 0;

        public const int ListRangDetNeg = 1;

        public const int ListRangDetPos = 2;

        public const int ListRangDetSamp = 3;

        public const int ListRangDetRms = 4;

        public const int ListRangDetAver = 5;

        public const int EspModeAuto = 0;

        public const int EspModeUser = 1;

        public const int EspModeMan = 2;

        public const int X = 0;

        public const int Y = 1;

        public const int SemRefTypePeak = 0;

        public const int SemRefTypeCpow = 1;

        public const int SemFilterTypeNorm = 0;

        public const int SemFilterTypeCfil = 1;

        public const int SemFilterTypeRrc = 2;

        public const int SemFilterTypePuls = 3;

        public const int SemLimCheckTypeAbs = 0;

        public const int SemLimCheckTypeRel = 1;

        public const int SemLimCheckTypeAnd = 2;

        public const int SemLimCheckTypeOr = 3;

        public const int SemInsertRangeAfter = 0;

        public const int SemInsertRangeBefore = 1;

        public const int ViFalse = 0;

        public const int ViTrue = 1;

        public const int All = 2;

        public const int SemPeakSymbOff = 0;

        public const int SemPeakSymbCros = 1;

        public const int RecDemodOff = 0;

        public const int RecDemodAm = 1;

        public const int RecDemodFm = 2;

        public const int TracFeedNever = 0;

        public const int TracFeedAlways = 1;

        public const int SweepSpacLog = 0;

        public const int SweepSpacLin = 1;

        public const int SweepSpacAuto = 2;

        public const int RecPsearchMethodPeak = 0;

        public const int RecPsearchMethodSubr = 1;

        public const int VnetMeasPrescan = 0;

        public const int VnetMeasFinal = 1;

        public const int RecVnetTypeOff = 0;

        public const int RecVnetTypeEsh3z5 = 1;

        public const int RecVnetTypeEsh2z5 = 2;

        public const int RecVnetType4200 = 3;

        public const int RecVnetType216 = 4;

        public const int RecVnetPhasN = 0;

        public const int RecVnetPhasL1 = 1;

        public const int RecVnetPhasL2 = 2;

        public const int RecVnetPhasL3 = 3;

        public const int SsaMeasConfInt1 = 0;

        public const int SsaMeasConfInt2 = 1;

        public const int SsaMeasConfInt3 = 2;

        public const int SsaMeasConfExt1 = 3;

        public const int SsaMeasConfExt2 = 4;

        public const int SsaMeasConfDut1 = 5;

        public const int SsaMeasConfDcon1 = 6;

        public const int SsaMeasConfDcon2 = 7;

        public const int SsaMeasConfRpn1 = 8;

        public const int SsaMeasConfRpn2 = 9;

        public const int SsaSmoothTypeLin = 0;

        public const int SsaSmoothTypeLog = 1;

        public const int SsaSmoothTypeMed = 2;

        public const int SsaPhasNoisMeasSpec = 0;

        public const int SsaPhasNoisMeasPll = 1;

        public const int SsaPhasNoisMeasCcor = 2;

        public const int SsaPhasNoisMeasSnt = 3;

        public const int SsaPhasNoisMeasRes = 4;

        public const int SsaPhasNoisMeasBas = 5;

        public const int SweModeFast = 0;

        public const int SweModeNorm = 1;

        public const int SweModeAver = 2;

        public const int SweModeMan = 3;

        public const int SsaFftWindowCheb = 0;

        public const int SsaFftWindowGaus = 1;

        public const int SsaFftWindowRect = 2;

        public const int SsaFftWindowBhar = 3;

        public const int BandTypeNorm = 0;

        public const int BandTypeFft = 1;

        public const int SsaVcoAutoscaleOff = 0;

        public const int SsaVcoAutoscaleOn = 1;

        public const int SsaVcoAutoscaleOnce = 2;

        public const int SsaVcoMeasTypeTch = 0;

        public const int SsaVcoMeasTypeTsen = 1;

        public const int SsaVcoMeasTypeRfp = 2;

        public const int SsaVcoMeasTypeTcur = 3;

        public const int SsaVcoMeasTypeHpow = 4;

        public const int SsaVcoMeasTypeDcch = 5;

        public const int SsaVcoMeasTypeSumm = 6;

        public const int SsaVcoXAxisUnitVolt = 0;

        public const int SsaVcoXAxisUnitFreq = 1;

        public const int SsaVcoTypePush = 0;

        public const int SsaVcoTypePull = 1;

        public const int UnitAngleDeg = 0;

        public const int UnitAngleRad = 1;

        public const int SsaTransMeasFreq = 0;

        public const int SsaTransMeasPhas = 1;

        public const int SsaTransMeasAmpl = 2;

        public const int SsaTransMeasPow = 3;

        public const int SsaTrigModeImm = 0;

        public const int SsaTrigModeExt = 1;

        public const int SsaTrigModePow = 2;

        public const int SsaTrigModeAm = 3;

        public const int SsaTrigModeAmr = 4;

        public const int SsaTrigModeFm = 5;

        public const int SsaTrigModePm = 6;

        public const int SsaUseVoltMax = 0;

        public const int SsaUseVoltBoth = 1;

        public const int SsaPdetCharSine = 0;

        public const int SsaPdetCharLin = 1;

        public const int SsaSpursDispModeAll = 0;

        public const int SsaSpursDispModeNone = 1;

        public const int SsaSpursDispModeSel = 2;

        public const int SsaSpursDispModeUns = 3;

        public const int UnitDbc = 23;

        public const int UnitDbchz = 24;

        public const int SsaPresetPno = 0;

        public const int SsaPresetSan = 1;

        public const int SsaPremeasCont = 0;

        public const int SsaPremeasSkip = 1;

        public const int SsaPremeasFin = 2;

        public const int SsaPhasNoisResRfm = 0;

        public const int SsaPhasNoisResRpm = 1;

        public const int SsaPhasNoisResRms = 2;

        public const int SsaPhasNoisResIpn = 3;

        public const int SsaVcoTchResTypeFreq = 0;

        public const int SsaVcoTchResTypePow = 1;

        public const int SsaVcoTchResTypeSens = 2;

        public const int SsaVcoTchResTypeCurr = 3;

        public const int SsaVcoPushPullResFdev = 0;

        public const int SsaVcoPushPullResPdev = 1;

        public const int SsaVcoPushPullResSens = 2;

        public const int SsaVcoHarmResFreq = 0;

        public const int SsaVcoHarmResApow = 1;

        public const int SsaVcoHarmResRpow = 2;

        public const int SsaVcoHarmResSens = 3;

        public const int SsaVcoDchResFreq = 0;

        public const int SsaVcoDchResPow = 1;

        public const int SsaModAm = 0;

        public const int SsaModFm = 1;

        public const int SsaModPm = 2;

        public const int SsaDetPpe = 0;

        public const int SsaDetMpe = 1;

        public const int SsaDetMid = 2;

        public const int SsaDetRms = 3;

        public const int SsaDetFerr = 4;

        public const int SsaDetCarr = 5;

        public const int WinA = 1;

        public const int WinB = 2;

        public const int WinC = 3;

        public const int WinD = 4;

        public const int TrSrcFree = 0;

        public const int TrSrcExt = 1;

        public const int TrSrcIf = 2;

        public const int TrSrcFm = 3;

        public const int TrSrcPm = 4;

        public const int TrSrcAm = 5;

        public const int TrSrcVideo = 6;

        public const int TrSrcAudio = 7;

        public const int CalAbsPowInput = 0;

        public const int CalAbsPowMeter = 1;

        public const int CalAbsPowSplit = 2;

        public const int CorrSel = 0;

        public const int CorrDel = 1;

        public const int CorrDelAll = 2;

        public const int RecLevUnitDb = 0;

        public const int RecLevUnitPct = 1;

        public const int DetWdNarrow = 0;

        public const int DetWdWide = 1;

        public const int PwmetMeastimeNorm = 0;

        public const int PwmetMeastimeShort = 1;

        public const int PwmetMeastimeLong = 2;

        public const int PwmetMeastimeManual = 3;

        public const int RemoteSetTypeEpm441a = 0;

        public const int RemoteSetTypeHp437b = 1;

        public const int RemoteSetTypeHp438a = 2;

        public const int RemoteSetTypeMll2438a = 3;

        public const int RemoteSetTypeNrvd = 4;

        public const int RemoteSetTypeNrvs = 5;

        public const int SensorA = 0;

        public const int SensorB = 1;

        public const int AfHp = 0;

        public const int AfLp = 1;

        public const int FilDempCons25 = 0;

        public const int FilDempCons50 = 1;

        public const int FilDempCons75 = 2;

        public const int FilDempCons750 = 3;

        public const int WeightingFilCcit = 0;

        public const int WeightingFilCcir = 1;

        public const int ModTypeAm = 0;

        public const int ModTypeFm = 1;

        public const int ModTypePm = 2;

        public const int DetTypePlusPeak = 0;

        public const int DetTypeMinusPeak = 1;

        public const int DetTypeAvgPeak = 2;

        public const int DetTypeAvg = 3;

        public const int DetTypeRms = 4;

        public const int DetTypeSrms = 5;

        public const int DetTypeThd = 6;

        public const int DetTypeSinad = 7;

        public const int ResDispBargraf = 0;

        public const int ResDispTimeDomain = 1;

        public const int ResDispModSpect = 2;

        public const int ResDispRfSpect = 3;

        public const int DetTypePpeak = 0;

        public const int DetTypeMpeak = 1;

        public const int DetTypePaverage = 2;

        public const int DetTypeAverage = 3;

        public const int MeasModeAbs = 0;

        public const int MeasModeRel = 1;

        public const int MeasModeRelAllOff = 2;

        public const int RelUnitDb = 0;

        public const int RelUnitPtc = 1;

        public const int InputTypeRf = 0;

        public const int InputTypeAudio = 1;

        public const int InLev4v = 0;

        public const int InLev400mv = 1;

        public const int InpResDispTd = 0;

        public const int InpResDispSpect = 1;

        public const int DispModeAbs = 0;

        public const int DispModeRel = 1;

        public const int ResTypeCur = 0;

        public const int ResTypeAver = 2;

        public const int ResTypePhold = 3;

        public const int AudioMeasResPpeak = 0;

        public const int AudioMeasResRms = 1;

        public const int AudioMeasResThd = 2;

        public const int AudioMeasResSinad = 3;

        public const int TgenExtAm = 0;

        public const int TgenExtFm = 1;

        public const int TgenExtIq = 2;

        public const int TgenPowFix = 0;

        public const int TgenPowSwe = 1;

        public const int ExtGenIntfGpib = 0;

        public const int ExtGenIntfTtl = 1;

        public const int TgenCalTran = 0;

        public const int TgenCalReflo = 1;

        public const int TgenCalRefls = 2;

        public const int NanMeasModeScalar = 0;

        public const int NanMeasModeVector = 1;

        public const int NanMeasModeVvm = 2;

        public const int DtfMeasModeDtf = 3;

        public const int DtfMeasModeReflection = 4;

        public const int DtfMeasModeSpectrum = 5;

        public const int MeasFormatMagnitude = 0;

        public const int MeasFormatPhase = 1;

        public const int MeasFormatMphase = 2;

        public const int MeasFormatGdelay = 3;

        public const int MagnitudeSpacingLin = 0;

        public const int MagnitudeSpacingLog = 1;

        public const int MarkerOutputFormatNorm = 0;

        public const int MarkerOutputFormatRpdb = 1;

        public const int MarkerOutputFormatRpl = 2;

        public const int MarkerOutputFormatRsc = 3;

        public const int MarkerOutputFormatImp = 4;

        public const int MarkerOutputFormatAdm = 5;

        public const int MarkerOutputFormatNimp = 6;

        public const int MarkerOutputFormatNadm = 7;

        public const int DisplayReflectionUnitRho = 0;

        public const int DisplayReflectionUnitMrho = 1;

        public const int TrackingGenTypeCloss = 0;

        public const int TrackingGenTypeMagn = 1;

        public const int TrackingGenTypePhas = 2;

        public const int TrackingGenTypeRefl = 3;

        public const int FftCalIhig = 0;

        public const int FftCalIlow = 1;

        public const int FftCalQhig = 2;

        public const int FftCalQlow = 3;

        public const int FftSigSourIq = 0;

        public const int FftSigSourGnd = 1;

        public const int FftSigSourCald = 2;

        public const int FftSigSourCalp = 3;

        public const int BbInputRf = 0;

        public const int BbInputAiq = 1;

        public const int BbTrigIonly = 0;

        public const int BbTrigQonly = 1;

        public const int BbInputImpLow = 0;

        public const int BbInputImpHigh = 1;

        public const int BbSigPathI = 0;

        public const int BbSigPathQ = 1;

        public const int BbSigPathIq = 2;

        public const int FftMagn = 0;

        public const int FftRim = 3;

        public const int FftMph = 4;

        public const int FftWindFlat = 0;

        public const int FftWindExp = 1;

        public const int FftWindHamm = 2;

        public const int FftWindHann = 3;

        public const int FftWindRect = 4;

        public const int FftWindCheb = 5;

        public const int FftVolt = 5;

        public const int FftPhas = 1;

        public const int FftUph = 2;

        public const int BstepModeLin = 0;

        public const int BstepModeL1235 = 1;

        public const int BbInputDiq = 2;

        public const int DiqIn = 0;

        public const int DiqOut = 1;

        public const int TrgBbp = 5;

        public const int GsmTrigFrun = 0;

        public const int GsmTrigExt = 1;

        public const int GsmTrigIfp = 2;

        public const int GsmTrigRfp = 3;

        public const int GsmMtypGmsk = 0;

        public const int GsmMtypEdge = 1;

        public const int GsmSlot11 = 0;

        public const int GsmSlot21 = 1;

        public const int GsmSlot22 = 2;

        public const int GsmSlot31 = 3;

        public const int GsmSlot32 = 4;

        public const int GsmSlot33 = 5;

        public const int GsmSlot41 = 6;

        public const int GsmSlot42 = 7;

        public const int GsmSlot43 = 8;

        public const int GsmSlot44 = 9;

        public const int GsmSlot81 = 10;

        public const int GsmSlot82 = 11;

        public const int GsmSlot83 = 12;

        public const int GsmSlot84 = 13;

        public const int GsmSlot85 = 14;

        public const int GsmSlot86 = 15;

        public const int GsmSlot87 = 16;

        public const int GsmSlot88 = 17;

        public const int GsmTsc0 = 0;

        public const int GsmTsc1 = 1;

        public const int GsmTsc2 = 2;

        public const int GsmTsc3 = 3;

        public const int GsmTsc4 = 4;

        public const int GsmTsc5 = 5;

        public const int GsmTsc6 = 6;

        public const int GsmTsc7 = 7;

        public const int GsmUser = 8;

        public const int GsmTscAb0 = 9;

        public const int GsmTscAb1 = 10;

        public const int GsmTscAb2 = 11;

        public const int GsmExtslotDynamic = 0;

        public const int GsmExtslotGsm900 = 1;

        public const int GsmExtslotGsm1800 = 2;

        public const int GsmExtslotGsm1900 = 3;

        public const int GsmExtslotGsm850 = 4;

        public const int ExtSlotModulOff = 0;

        public const int ExtSlotModulGmsk = 1;

        public const int ExtSlotModulEdge = 2;

        public const int GsmMeasPfer = 0;

        public const int GsmMeasMacc = 1;

        public const int GsmMeasCpw = 2;

        public const int GsmMeasPvt = 3;

        public const int GsmMeasSpecMod = 4;

        public const int GsmMeasSpecTra = 5;

        public const int GsmMeasSpur = 6;

        public const int GsmBurstSecFull = 0;

        public const int GsmBurstSecTop = 1;

        public const int GsmBurstSecRis = 2;

        public const int GsmBurstSecFall = 3;

        public const int GsmBurstSecFrz = 4;

        public const int GsmFiltG500 = 0;

        public const int GsmFiltB600 = 1;

        public const int GsmPerrRms = 0;

        public const int GsmPerrPeak = 1;

        public const int GsmFerr = 2;

        public const int GsmIqof = 3;

        public const int GsmIqim = 4;

        public const int MeastypeMax = 2;

        public const int MeastypeAver = 4;

        public const int GsmMaccRms = 0;

        public const int GsmMaccPeak = 1;

        public const int GsmMaccOsup = 2;

        public const int GsmMaccPerc = 3;

        public const int GsmMaccFreq = 4;

        public const int GsmMaccIqof = 5;

        public const int GsmMaccIqim = 6;

        public const int GsmPvtTrgs = 0;

        public const int FmFeedAmRel = 0;

        public const int FmFeedAm = 1;

        public const int FmFeedSpec = 2;

        public const int FmFeedFm = 3;

        public const int FmFeedPm = 4;

        public const int FmFeedAmsRel = 6;

        public const int FmFeedAms = 5;

        public const int FmFeedFms = 7;

        public const int FmFeedPms = 8;

        public const int FmFeedFmAfsp = 9;

        public const int FmFeedPmAfsp = 10;

        public const int FmFeedAmRelAfsp = 11;

        public const int FmFeedRfpAfsp = 12;

        public const int FsmFeedSumm = 13;

        public const int TrgFm = 3;

        public const int TrgAm = 4;

        public const int TrgPm = 5;

        public const int TrgAmr = 6;

        public const int FmdemFilterHp = 0;

        public const int FmdemFilterLp = 1;

        public const int FmdemFilterDeemp = 2;

        public const int FmdemFilterLpRel = 3;

        public const int AdemUphas = 0;

        public const int AdemPhas = 1;

        public const int FmdemAm = 0;

        public const int FmdemFm = 1;

        public const int FmdemPm = 2;

        public const int FmdemAfsam = 3;

        public const int FmdemAfsfm = 4;

        public const int FmdemAfspm = 5;

        public const int FmdemRf = 6;

        public const int FmdemAmrel = 7;

        public const int FmdemAfsamrel = 8;

        public const int FmdemRlen = 0;

        public const int FmdemSrate = 1;

        public const int FmdemResultCurr = 1;

        public const int FmdemResultAvg = 2;

        public const int FmdemResultMax = 3;

        public const int FmdemResultMin = 4;

        public const int FmdemImm = 0;

        public const int FmdemAvg = 1;

        public const int FmdemDetPpe = 0;

        public const int FmdemDetMpe = 1;

        public const int FmdemDetMid = 2;

        public const int FmdemDetRms = 3;

        public const int FmdemMeasAfr = 0;

        public const int FmdemMeasFer = 1;

        public const int FmdemMeasSin = 2;

        public const int FmdemMeasThd = 3;

        public const int FmdemMeasCar = 4;

        public const int FmsTrgImm = 0;

        public const int FmsTrgExt = 1;

        public const int FmsTrgIfp = 2;

        public const int FmsTrgTime = 3;

        public const int FmsTrgLeft = 4;

        public const int FmsTrgRigh = 5;

        public const int FmsTrgMpx = 6;

        public const int FmsTrgMono = 7;

        public const int FmsTrgSter = 8;

        public const int FmsTrgRds = 9;

        public const int FmsTrgPil = 10;

        public const int FmsDispOff = 0;

        public const int FmsDispSumm = 1;

        public const int FmsDispTdRf = 2;

        public const int FmsDispTdLeft = 3;

        public const int FmsDispTdRight = 4;

        public const int FmsDispTdMpx = 5;

        public const int FmsDispTdMono = 6;

        public const int FmsDispTdSter = 7;

        public const int FmsDispTdRds = 8;

        public const int FmsDispTdPilot = 9;

        public const int FmsDispSpecRf = 10;

        public const int FmsDispSpecLeft = 11;

        public const int FmsDispSpecRight = 12;

        public const int FmsDispSpecMpx = 13;

        public const int FmsDispSpecMono = 14;

        public const int FmsDispSpecSter = 15;

        public const int FmsDispSpecRds = 16;

        public const int FmsDispSpecPilot = 17;

        public const int FsmChanTypeLeft = 0;

        public const int FsmChanTypeRight = 1;

        public const int FsmChanTypeMpx = 2;

        public const int FsmChanTypeMono = 3;

        public const int FsmChanTypeSter = 4;

        public const int FsmChanTypeRds = 5;

        public const int FsmChanTypePilot = 6;

        public const int FmsRsumDetecRms = 0;

        public const int FmsRsumDetecSrms = 1;

        public const int FmsRsumDetecPpe = 2;

        public const int FmsRsumDetecNpe = 3;

        public const int FmsRsumDetecPav = 4;

        public const int FmsRsumDetecQpe = 5;

        public const int FmsRsumDetecSqp = 6;

        public const int FmsRsumModeWrit = 0;

        public const int FmsRsumModeAver = 1;

        public const int FmsRsumModePhol = 2;

        public const int FmsHpasFiltNone = 0;

        public const int FmsHpasFilt20hz = 1;

        public const int FmsHpasFilt50hz = 2;

        public const int FmsHpasFilt300hz = 3;

        public const int FmsLpasFiltNone = 0;

        public const int FmsLpasFilt10pct = 1;

        public const int FmsLpasFilt15khz = 2;

        public const int FmsLpasFilt150khz = 3;

        public const int FmsLpasFilt23khz = 4;

        public const int FmsLpasFilt25pct = 5;

        public const int FmsLpasFilt3khz = 6;

        public const int FmsLpasFilt5pct = 7;

        public const int FmsWeightingNone = 0;

        public const int FmsWeightingCcit = 1;

        public const int FmsWeightingCciru = 2;

        public const int FmsWeightingCcirw = 3;

        public const int FmsDeemphNone = 0;

        public const int FmsDeemph25 = 1;

        public const int FmsDeemph50 = 2;

        public const int FmsDeemph75 = 3;

        public const int FmsDeemph750 = 4;

        public const int FmsAfsp = 0;

        public const int FmsTdom = 1;

        public const int FmsMeasTypeAdev = 0;

        public const int FmsMeasTypeRdev = 1;

        public const int FmsMeasTypeSinad = 2;

        public const int FmsMeasTypeThd = 3;

        public const int FmsMeasTypeAfr = 4;

        public const int FmsResultCurr = 1;

        public const int BtoGeogEur = 0;

        public const int BtoGeogUsa = 1;

        public const int BtoGeogFran = 2;

        public const int BtoPtypeAuto = 0;

        public const int BtoPtypeDh1 = 1;

        public const int BtoPtypeDh3 = 2;

        public const int BtoPtypeDh5 = 3;

        public const int BtoMeasModeOpow = 0;

        public const int BtoMeasModeAclr = 1;

        public const int BtoMeasModeMch = 2;

        public const int BtoMeasModeIcft = 3;

        public const int BtoMeasModeCfdr = 4;

        public const int BtoMeasModeRtp = 5;

        public const int BtoMeasModeIbs = 6;

        public const int BtoMeasModeDpen = 7;

        public const int BtoMeasModeCfst = 8;

        public const int BtoMeasfiltOff = 0;

        public const int BtoMeasfiltBto = 1;

        public const int BtoTracDetApeak = 0;

        public const int BtoTracDetNeg = 1;

        public const int BtoTracDetPos = 2;

        public const int BtoTracDetSample = 3;

        public const int BtoTracDetRms = 4;

        public const int BtoTracDetAver = 5;

        public const int MeastypeMin = 1;

        public const int MeastypePeak = 3;

        public const int MeastypeMaxmin = 0;

        public const int MeastypeMaxmax = 1;

        public const int MeastypeMaxaver = 2;

        public const int MeastypeAvermin = 3;

        public const int MeastypeAvermax = 4;

        public const int MeastypeNone = 0;

        public const int BtoSeTx = 0;

        public const int BtoSeHadjLow = 1;

        public const int BtoSeHadjUpp = 2;

        public const int BtoCfsFerrInitial = 0;

        public const int BtoCfsFerrBlock = 1;

        public const int BtoCfsFerrTotal = 2;

        public const int BtoCfsDevmRms = 0;

        public const int BtoCfsDevm99pct = 1;

        public const int BtoCfsDevmPeak = 2;

        public const int BtoDpenTested = 0;

        public const int BtoDpenBer = 1;

        public const int BtoDpenPassed = 2;

        public const int BtoRtpGfsk = 0;

        public const int BtoRtpDpsk = 1;

        public const int BtoRtpRatio = 2;

        public const int PmetCoupOff = 0;

        public const int PmetCoupCent = 1;

        public const int PmetCoupMark = 2;

        public const int PowersensorPwrDispAver = 0;

        public const int PowersensorPwrDispPenv = 1;

        public const int GsmK10BtsNormal = 0;

        public const int GsmK10BtsMicro = 1;

        public const int GsmK10BtsPico = 2;

        public const int GsmK10MsNormal = 3;

        public const int GsmK10MsSmall = 4;

        public const int GsmK10FreqBandTgsm380 = 0;

        public const int GsmK10FreqBandTgsm410 = 1;

        public const int GsmK10FreqBandGsm450 = 2;

        public const int GsmK10FreqBandGsm480 = 3;

        public const int GsmK10FreqBandGsm710 = 4;

        public const int GsmK10FreqBandGsm750 = 5;

        public const int GsmK10FreqBandTgsm810 = 6;

        public const int GsmK10FreqBandGsm850 = 7;

        public const int GsmK10FreqBandPgsm900 = 8;

        public const int GsmK10FreqBandEgsm900 = 9;

        public const int GsmK10FreqBandRgsm900 = 10;

        public const int GsmK10FreqBandTgsm900 = 11;

        public const int GsmK10FreqBandDcs1800 = 12;

        public const int GsmK10FreqBandPcs1900 = 13;

        public const int GsmK10PowClass1 = 0;

        public const int GsmK10PowClass2 = 1;

        public const int GsmK10PowClass3 = 2;

        public const int GsmK10PowClass4 = 3;

        public const int GsmK10PowClass5 = 4;

        public const int GsmK10PowClass6 = 5;

        public const int GsmK10PowClass7 = 6;

        public const int GsmK10PowClass8 = 7;

        public const int GsmK10PowClassE1 = 8;

        public const int GsmK10PowClassE2 = 9;

        public const int GsmK10PowClassE3 = 10;

        public const int GsmK10PowClassM1 = 11;

        public const int GsmK10PowClassM2 = 12;

        public const int GsmK10PowClassM3 = 13;

        public const int GsmK10PowClassP1 = 14;

        public const int GsmK10AutoOff = 0;

        public const int GsmK10AutoOn = 1;

        public const int GsmK10AutoOnce = 2;

        public const int GsmK10SyncAll = 0;

        public const int GsmK10SyncTsc = 1;

        public const int GsmK10SyncBurs = 2;

        public const int GsmK10SyncNone = 3;

        public const int GsmK10BurstNormal = 0;

        public const int GsmK10BurstHigherRate = 1;

        public const int GsmK10ModGmsk = 0;

        public const int GsmK10ModQpsk = 1;

        public const int GsmK10Mod8psk = 2;

        public const int GsmK10Mod16qam = 3;

        public const int GsmK10Mod32qam = 4;

        public const int GsmK10FilterNarrow = 2;

        public const int GsmK10FilterWide = 3;

        public const int GsmK10Macc = 0;

        public const int GsmK10Pfer = 1;

        public const int GsmK10Etim = 2;

        public const int GsmK10Merr = 3;

        public const int GsmK10Const = 4;

        public const int GsmK10Ptem = 5;

        public const int GsmK10Mod = 6;

        public const int GsmK10Swit = 7;

        public const int GsmK10BurstSelFull = 0;

        public const int GsmK10BurstSelThRes = 1;

        public const int GsmK10BurstSelRs = 2;

        public const int GsmK10Filter1mhz = 0;

        public const int GsmK10Filter500khz = 1;

        public const int GsmK10Filter600khz = 2;

        public const int GsmK10McarFilt400 = 0;

        public const int GsmK10McarFilt300 = 1;

        public const int GsmK10MeasEvmRms = 0;

        public const int GsmK10MeasEvmPeak = 1;

        public const int GsmK10MeasMerrRms = 2;

        public const int GsmK10MeasMerrPeak = 3;

        public const int GsmK10MeasPerrRms = 4;

        public const int GsmK10MeasPerrPeak = 5;

        public const int GsmK10MeasOsupp = 6;

        public const int GsmK10MeasIqof = 7;

        public const int GsmK10MeasIqim = 8;

        public const int GsmK10MeasFreq = 9;

        public const int GsmK10MeasBpow = 10;

        public const int GsmK10MeasAdr = 11;

        public const int GsmK10MeasPercEvm = 12;

        public const int GsmK10MeasPercMerr = 13;

        public const int GsmK10MeasPercPerr = 14;

        public const int GsmK10ModAvg = 0;

        public const int GsmK10ModCur = 1;

        public const int GsmK10ModMax = 2;

        public const int GsmK10ModDev = 3;

        public const int GsmK10BurstAll = 0;

        public const int GsmK10BurstCurr = 1;

        public const int GsmK10BurstModAvgPwr = 0;

        public const int GsmK10BurstModCrest = 1;

        public const int GsmK10BurstModMaxPwr = 2;

        public const int SpemColColor = 0;

        public const int SpemColRadar = 1;

        public const int SpemColGray = 2;

        public const int SpemSarMemory = 1;

        public const int SpemSarVisible = 0;

        public const int SpecmMarkMaxPeak = 0;

        public const int SpecmMarkMaxNext = 1;

        public const int SpecmMarkMaxLeft = 2;

        public const int SpecmMarkMaxRight = 3;

        public const int SpecmMarkMinPeak = 4;

        public const int SpecmMarkMinNext = 5;

        public const int SpecmMarkMinLeft = 6;

        public const int SpecmMarkMinRight = 7;

        public const int SpecmMarkYMaxPeak = 8;

        public const int SpecmMarkYMaxNext = 9;

        public const int SpecmMarkYMaxAbove = 10;

        public const int SpecmMarkYMaxBelow = 11;

        public const int SpecmMarkYMinPeak = 12;

        public const int SpecmMarkYMinNext = 13;

        public const int SpecmMarkYMinAbove = 14;

        public const int SpecmMarkYMinBelow = 15;

        public const int SpecmMarkXyMax = 16;

        public const int SpecmMarkXyMin = 17;

        public const int AviStanVor = 0;

        public const int AviStanIls = 1;

        public const int AviFeedSpec = 0;

        public const int AviFeedAms = 1;

        public const int BbInputAud = 3;

        public const int AviInputVol4v = 0;

        public const int AviInputVol400mv = 1;

        public const int AviThdFundFreq90 = 0;

        public const int AviThdFundFreq150 = 1;

        public const int AviThdFundFreq90150 = 2;

        public const int AviThdFundFreqId = 3;

        public const int VarAviDemodSumm30 = 0;

        public const int VarAviDemodSumm996 = 1;

        public const int VarAviDemodSummId = 2;

        public const int VarAviDemodSumm90 = 3;

        public const int VarAviDemodSumm150 = 4;

        public const int VarAviDemodSumm90150 = 5;

        public const int CatvSigTypeAtv = 0;

        public const int CatvSigTypeDtv = 1;

        public const int CatvSbandNorm = 0;

        public const int CatvSbandInv = 1;

        public const int CatvSbandAuto = 2;

        public const int AtvMeasAsp = 0;

        public const int AtvMeasCarr = 1;

        public const int AtvMeasCn = 2;

        public const int AtvMeasCso = 3;

        public const int AtvMeasCtb = 4;

        public const int AtvMeasHum = 5;

        public const int AtvMeasVmod = 6;

        public const int AtvMeasVsc = 7;

        public const int AtvStanBg = 0;

        public const int AtvStanDk = 1;

        public const int AtvStanI = 2;

        public const int AtvStanK1 = 3;

        public const int AtvStanL = 4;

        public const int AtvStanM = 5;

        public const int AtvStanN = 6;

        public const int AtvAudioFm55nicam585 = 0;

        public const int AtvAudioFm55fm5742 = 1;

        public const int AtvAudioFm55mono = 2;

        public const int AtvAudioFm65nicam585 = 3;

        public const int AtvAudioFm65fm6258 = 4;

        public const int AtvAudioFm65fm6742 = 5;

        public const int AtvAudioFm65mono = 6;

        public const int AtvAudioFm60nicam6552 = 7;

        public const int AtvAudioFm60mono = 8;

        public const int AtvAudioAm65nicam585 = 9;

        public const int AtvAudioAm60mono = 10;

        public const int AtvAudioFm45btsc = 11;

        public const int AtvAudioFm45eiaJ = 12;

        public const int AtvAudioFm45fm4742 = 13;

        public const int AtvAudioFm45mono = 14;

        public const int CatvGdelayGeneral = 0;

        public const int CatvGdelayGhalf = 1;

        public const int CatvGdelayAustralia = 2;

        public const int CatvGdelayDanmark = 3;

        public const int CatvGdelayNewzeland = 4;

        public const int CatvGdelayNorway = 5;

        public const int CatvGdelaySwedenfull = 6;

        public const int CatvGdelayFlat = 7;

        public const int CatvGdelayOirt = 8;

        public const int CatvGdelayChina = 9;

        public const int CatvGdelayCcir = 10;

        public const int CatvGdelayTdf = 11;

        public const int CatvGdelayFfc = 12;

        public const int AtvColorSystemPal = 0;

        public const int AtvColorSystemNtsc = 1;

        public const int AtvColorSystemSecam = 2;

        public const int AtvBlinTypeCcir17 = 0;

        public const int AtvBlinTypeFcc = 1;

        public const int AtvBlinTypeNtc7 = 2;

        public const int AtvMeasCarrCarr = 0;

        public const int AtvMeasCarrNois = 1;

        public const int AtvMeasModeInservice = 0;

        public const int AtvMeasModeOffservice = 1;

        public const int AtvMeasModeQuietline = 2;

        public const int AtvRpowModeRch = 0;

        public const int AtvRpowModeMch = 1;

        public const int AtvRpowModeMan = 2;

        public const int AtvVcp = 0;

        public const int AtvVcf = 1;

        public const int AtvSc1pr = 2;

        public const int AtvSc1if = 3;

        public const int AtvSc2pr = 4;

        public const int AtvSc2if = 5;

        public const int LimLow = 0;

        public const int LimUpp = 1;

        public const int AtvRpc = 1;

        public const int AtvMdep = 2;

        public const int DtvMeasDsp = 0;

        public const int DtvMeasChp = 1;

        public const int DtvMeasCons = 2;

        public const int DtvMeasMerr = 3;

        public const int DtvMeasEpatt = 4;

        public const int DtvMeasApd = 5;

        public const int DtvMeasCcdf = 6;

        public const int DtvMeasOver = 7;

        public const int DtvStanJ83a = 0;

        public const int DtvStanJ83b = 1;

        public const int DtvStanJ83c = 2;

        public const int CatvDtvAlphaR012 = 0;

        public const int CatvDtvAlphaR013 = 1;

        public const int CatvDtvAlphaR015 = 2;

        public const int CatvDtvAlphaR018 = 3;

        public const int DtvFormatQam4 = 0;

        public const int DtvFormatQam16 = 1;

        public const int DtvFormatQam32 = 2;

        public const int DtvFormatQam64 = 3;

        public const int DtvFormatQam128 = 4;

        public const int DtvFormatQam256 = 5;

        public const int DtvFormatQam512 = 6;

        public const int DtvFormatQam1024 = 7;

        public const int CatvMerrZoomNone = 0;

        public const int CatvMerrZoomMerr = 1;

        public const int CatvMerrZoomMerp = 2;

        public const int CatvMerrZoomEvmr = 3;

        public const int CatvMerrZoomEvmp = 4;

        public const int CatvMerrZoomImb = 5;

        public const int CatvMerrZoomQerr = 6;

        public const int CatvMerrZoomSup = 7;

        public const int CatvMerrZoomPjit = 8;

        public const int CatvOverZoomNone = 0;

        public const int CatvOverZoomMerr = 1;

        public const int CatvOverZoomMerp = 2;

        public const int CatvOverZoomEvmr = 3;

        public const int CatvOverZoomEvmp = 4;

        public const int CatvOverZoomCfof = 5;

        public const int CatvOverZoomSrof = 6;

        public const int CatvQuadZoomNone = 0;

        public const int CatvQuadZoom1 = 1;

        public const int CatvQuadZoom2 = 2;

        public const int CatvQuadZoom3 = 3;

        public const int CatvQuadZoom4 = 4;

        public const int StatDefault = 0;

        public const int StatAdj = 1;

        public const int StatXrlev = 2;

        public const int StatXrange = 3;

        public const int StatYmax = 4;

        public const int StatYmin = 5;

        public const int DtvMerr = 0;

        public const int DtvMerp = 1;

        public const int DtvEvmr = 2;

        public const int DtvEvmp = 3;

        public const int DtvCfof = 4;

        public const int DtvSrof = 5;

        public const int DtvImb = 4;

        public const int DtvQerr = 5;

        public const int DtvSup = 6;

        public const int DtvPjit = 7;

        public const int UnitSec = 0;

        public const int UnitMin = 1;

        public const int TvMeasTilt = 0;

        public const int AtvResRpow = 0;

        public const int AtvResNfc = 1;

        public const int AtvResMfr = 2;

        public const int AtvResCr = 3;

        public const int AtvResNrbw = 4;

        public const int DtvSpecSal = 0;

        public const int DtvSpecSaup = 1;

        public const int StatMean = 0;

        public const int StatPeak = 1;

        public const int StatCfac = 2;

        public const int NoiseDutModeDirect = 0;

        public const int NoiseDutModeFixedIfPlus = 1;

        public const int NoiseDutModeFixedIfAbs = 2;

        public const int NoiseDutModeFixedLoPlus = 3;

        public const int NoiseDutModeFixedLoAbs = 4;

        public const int NoiseSecondStageCorrMeas = 0;

        public const int NoiseContFreqListMeas = 1;

        public const int NoiseSingleFreqListMeas = 2;

        public const int NoiseSingleFreqMeas = 3;

        public const int NoiseErnModeSelSpot = 1;

        public const int NoiseErnModeSelTable = 0;

        public const int NoiseLossModeSpot = 1;

        public const int NoiseLossModeTable = 0;

        public const int NoiseDispDataOff = 0;

        public const int NoiseDispDataTeff = 1;

        public const int NoiseDispDataNfigure = 2;

        public const int NoiseNoise = 1;

        public const int NoiseGain = 2;

        public const int NoiseXaxisIf = 0;

        public const int NoiseXaxisRf = 1;

        public const int NoiseCalcLimLineNfigure = 0;

        public const int NoiseCalcLimLineTeffective = 1;

        public const int NoiseCalcLimLineGain = 2;

        public const int NoiseFetcFigure = 0;

        public const int NoiseFetcTemperature = 1;

        public const int NoiseFetcGain = 2;

        public const int PhaseSmoTypeLin = 0;

        public const int PhaseSmoTypeLog = 1;

        public const int PhaseRfm = 0;

        public const int PhaseRpm = 1;

        public const int PhaseRms = 2;

        public const int VsaFactoryGst = 0;

        public const int VsaFactoryStan = 1;

        public const int VsaFactoryMapp = 2;

        public const int VsaFactoryPatt = 3;

        public const int VsaFactoryFilt = 4;

        public const int VsaMarkerSearchReal = 0;

        public const int VsaMarkerSearchImag = 1;

        public const int VsaBursModeMeas = 0;

        public const int VsaBursModeBurs = 1;

        public const int VsaSyncModeMeas = 0;

        public const int VsaSyncModeSync = 1;

        public const int SyncPattAdd = 0;

        public const int SyncPattSelect = 1;

        public const int SyncPattDelete = 2;

        public const int SyncPattRemove = 3;

        public const int SyncPattRemoveAll = 4;

        public const int SyncPattCopy = 5;

        public const int VsaPatternAll = 0;

        public const int VsaPatternCurrent = 1;

        public const int VsaEquaLoad = 0;

        public const int VsaEquaSave = 1;

        public const int VsaEquaDelete = 2;

        public const int VsaEquaReset = 3;

        public const int VsaSignalTypeCont = 0;

        public const int VsaSignalTypeBurst = 1;

        public const int VsaModulationFormatQpsk = 0;

        public const int VsaModulationFormatDqpsk = 1;

        public const int VsaModulationFormatOqpsk = 2;

        public const int VsaModulationFormatDpi4qpsk = 3;

        public const int VsaModulationFormatBpsk = 4;

        public const int VsaModulationFormat8psk = 5;

        public const int VsaModulationFormatD8psk = 6;

        public const int VsaModulationFormatN3pi8psk = 7;

        public const int VsaModulationFormatMsk = 8;

        public const int VsaModulationFormatDmsk = 9;

        public const int VsaModulationFormat16qam = 10;

        public const int VsaModulationFormat32qam = 11;

        public const int VsaModulationFormat64qam = 12;

        public const int VsaModulationFormat128qam = 13;

        public const int VsaModulationFormat256qam = 14;

        public const int VsaModulationFormat2fsk = 15;

        public const int VsaModulationFormat4fsk = 16;

        public const int VsaModulationFormat8vsb = 17;

        public const int VsaModulationFormat16dqam = 18;

        public const int VsaModulationFormat32dqam = 19;

        public const int VsaModulationFormat64dqam = 20;

        public const int VsaModulationFormat128dqam = 21;

        public const int VsaModulationFormat256dqam = 22;

        public const int VsaModulationFormatUserqam = 23;

        public const int VsaModEvmSymbol = 0;

        public const int VsaModEvmSignal = 1;

        public const int VsaModEvmMacp = 2;

        public const int VsaModEvmMecp = 3;

        public const int VsaTxFilterTypeNone = 0;

        public const int VsaTxFilterTypeRc = 1;

        public const int VsaTxFilterTypeRrc = 2;

        public const int VsaTxFilterTypeGauss = 3;

        public const int VsaTxFilterTypeGmsk = 4;

        public const int VsaTxFilterTypeLinGmsk = 5;

        public const int VsaTxFilterTypeApco25 = 6;

        public const int VsaTxFilterTypeCdma2kFwd = 7;

        public const int VsaTxFilterTypeCdma2kRev = 8;

        public const int VsaTxFilterTypeRect = 9;

        public const int VsaTxFilterTypeHalfSine = 10;

        public const int VsaDemRlenAutoOn = 0;

        public const int VsaDemRlenSymbol = 1;

        public const int VsaDemRlenTime = 2;

        public const int VsaSbandNorm = 0;

        public const int VsaSbandInv = 1;

        public const int VsaCompensateIqOffset = 0;

        public const int VsaCompensateIqImbal = 1;

        public const int VsaCompensateAdroop = 2;

        public const int VsaMeasFilterTypeNone = 0;

        public const int VsaMeasFilterTypeRrc = 1;

        public const int VsaMeasFilterTypeEdge = 2;

        public const int VsaMeasFilterTypeApco25 = 3;

        public const int VsaMeasFilterTypeCdma2kFwd = 4;

        public const int VsaMeasFilterTypeCdma2kRev = 5;

        public const int VsaUnitTimeS = 0;

        public const int VsaUnitTimeSym = 1;

        public const int FitAlignLeft = 0;

        public const int FitAlignCenter = 1;

        public const int FitAlignRight = 2;

        public const int VsaFitTrigger = 0;

        public const int VsaFitBurst = 1;

        public const int VsaFitPattern = 2;

        public const int VsaTraceOff = 0;

        public const int VsaTraceDots = 1;

        public const int VsaTraceBars = 2;

        public const int StatModeSonly = 0;

        public const int StatModeInfinite = 1;

        public const int TracEvalMeas = 0;

        public const int TracEvalRef = 1;

        public const int VsaMbursSearchFirst = 0;

        public const int VsaMbursSearchNext = 1;

        public const int VsaMbursSearchStart = 2;

        public const int VsaImpexpStandards = 0;

        public const int VsaImpexpMappings = 1;

        public const int VsaImpexpPatterns = 2;

        public const int VsaImpexpFilters = 3;

        public const int VsaImpexpEqualizers = 4;

        public const int DecSepPoin = 0;

        public const int DecSepComma = 1;

        public const int ExpModeRaw = 0;

        public const int ExpModeTrace = 1;

        public const int ExpRawModeAsc = 0;

        public const int ExpRawModeWav = 1;

        public const int ExpRawModeBin = 2;

        public const int VsaDataSignalMeas = 0;

        public const int VsaDataSignalRef = 1;

        public const int ResultDispFreqAbs = 0;

        public const int ResultDispFreqRel = 1;

        public const int ResultDispRim = 2;

        public const int ResultDispIqConst = 3;

        public const int ResultDispIqVect = 4;

        public const int ResultDispMagnAbs = 5;

        public const int ResultDispMagnRel = 6;

        public const int ResultDispPhasWrap = 7;

        public const int ResultDispPhasUwrap = 8;

        public const int ResultDispEyeI = 9;

        public const int ResultDispEyeQ = 10;

        public const int ResultDispMagnErr = 5;

        public const int ResultDispPhasErr = 6;

        public const int ResultDispAmpmConv = 7;

        public const int ResultDispEvm = 8;

        public const int ResultEqdispMagnLin = 0;

        public const int ResultEqdispMagnLog = 1;

        public const int ResultEqdispPhasWrap = 2;

        public const int ResultEqdispPhasUwrap = 3;

        public const int ResultEqdispRim = 4;

        public const int ResultEqdispGdelay = 5;

        public const int ResultEqdispPrespWrap = 6;

        public const int ResultEqdispPrespUwrap = 7;

        public const int ResultEqdispFrespLin = 8;

        public const int ResultEqdispFrespLog = 9;

        public const int ResultEqdispChrespLin = 10;

        public const int ResultEqdispChrespLog = 11;

        public const int ResultDispMagnRaw = 3;

        public const int VsaMpow = 0;

        public const int VsaDtts = 1;

        public const int VsaAdr = 2;

        public const int VsaCfer = 3;

        public const int VsaEvm = 4;

        public const int VsaFskCdr = 5;

        public const int VsaFskDerr = 6;

        public const int VsaFskMdev = 7;

        public const int VsaIqim = 8;

        public const int VsaGimb = 9;

        public const int VsaMerr = 10;

        public const int VsaOoff = 11;

        public const int VsaPerr = 12;

        public const int VsaPler = 13;

        public const int VsaQerr = 14;

        public const int VsaRho = 15;

        public const int VsaSnr = 16;

        public const int VsaModifNone = 0;

        public const int VsaModifRms = 1;

        public const int VsaModifAvg = 2;

        public const int VsaModifSdev = 3;

        public const int VsaModifTpe = 4;

        public const int VsaModifPeak = 5;

        public const int VsaModifAsym = 6;

        public const int VsaModifPctl = 7;

        public const int VsaModifFsvCurr = 8;

        public const int VsaModifFsvAvg = 9;

        public const int VsaModifFsvRpe = 10;

        public const int VsaModifFsvSdev = 11;

        public const int VsaModifFsvPctl = 12;

        public const int VsaModifFsvPeak = 13;

        public const int VsaModifFsvPavg = 14;

        public const int VsaModifFsvTpe = 15;

        public const int VsaModifFsvPsd = 16;

        public const int VsaModifFsvPpct = 17;

        public const int VsaCpointPshift = 0;

        public const int VsaCpointPower = 1;

        public const int VsaCpointData = 2;

        public const int _3gppBsSettingsPowRefTot = 0;

        public const int _3gppBsSettingsPowRefCpic = 1;

        public const int _3gppBsSettingsCompModePilot = 0;

        public const int _3gppBsSettingsCompModeSlot = 1;

        public const int _3gppBsSettingsSyncTypeCpic = 0;

        public const int _3gppBsSettingsSyncTypeSch = 1;

        public const int _3gppSbandNorm = 0;

        public const int _3gppSbandInv = 1;

        public const int _3gppBsSettingsAntennaOff = 0;

        public const int _3gppBsSettingsAntenna1 = 1;

        public const int _3gppBsSettingsAntenna2 = 2;

        public const int _3gppToffsetPred = 0;

        public const int _3gppToffsetMeas = 1;

        public const int _3gbsResCdp = 0;

        public const int _3gbsResCdprat = 1;

        public const int _3gbsResCdep = 2;

        public const int _3gbsResSum = 3;

        public const int _3gbsResCtab = 4;

        public const int _3gbsResPcd = 5;

        public const int _3gbsResMacc = 6;

        public const int _3gbsResPvsla = 7;

        public const int _3gbsResPvslr = 8;

        public const int _3gbsResBstr = 9;

        public const int _3gbsResSymbconst = 10;

        public const int _3gbsResSymbevm = 11;

        public const int _3gbsResFvsl = 12;

        public const int _3gbsResPvsy = 13;

        public const int _3gbsResEvmMagn = 14;

        public const int _3gbsResEvmPhas = 15;

        public const int _3gbsResCompCons = 16;

        public const int _3gbsResChipEvm = 17;

        public const int _3gbsResChipMagn = 18;

        public const int _3gbsResCdpover = 21;

        public const int _3gppResSumNormal = 0;

        public const int _3gppResSumExtended = 1;

        public const int CdpTableOff = 0;

        public const int CdpTableSelect = 1;

        public const int CdpTableNew = 2;

        public const int CdpTableDelete = 3;

        public const int CdpTableCopy = 4;

        public const int _3gppMarkerCpich = 0;

        public const int _3gppMarkerPccpch = 1;

        public const int _3gppbsMeasPow = 0;

        public const int _3gppbsMeasAclr = 1;

        public const int _3gppbsMeasEsp = 2;

        public const int _3gppbsMeasObw = 3;

        public const int _3gppbsMeasWcdp = 4;

        public const int _3gppbsMeasCcdf = 5;

        public const int _3gppbsMeasMcac = 6;

        public const int _3gppbsMeasRfc = 7;

        public const int _3gppbsMeasFdom = 8;

        public const int _3gppbsMeasTdom = 9;

        public const int _3gbsTracCdp = 0;

        public const int _3gbsTracCdprat = 1;

        public const int _3gbsTracCdep = 2;

        public const int _3gbsTracCtab = 3;

        public const int _3gbsTracCwcd = 4;

        public const int _3gbsTracPwcd = 5;

        public const int _3gbsTracPcd = 6;

        public const int _3gbsTracMacc = 7;

        public const int _3gbsTracPvsla = 8;

        public const int _3gbsTracPvslr = 9;

        public const int _3gbsTracBstr = 10;

        public const int _3gbsTracSymbconst = 11;

        public const int _3gbsTracSymbevm = 12;

        public const int _3gbsTracFvsl = 13;

        public const int _3gbsTracPvsy = 14;

        public const int _3gbsTracEvmMagn = 15;

        public const int _3gbsTracEvmPhas = 16;

        public const int _3gbsTracCompCons = 17;

        public const int _3gbsTracChipEvm = 18;

        public const int _3gbsTracChipMagn = 19;

        public const int _3gbsTracChipPhas = 20;

        public const int _3gbsTracAtrace2 = 21;

        public const int WcdpResPtot = 0;

        public const int WcdpResFerr = 1;

        public const int WcdpResTfr = 2;

        public const int WcdpResMacc = 3;

        public const int WcdpResPcd = 4;

        public const int WcdpResEvmr = 5;

        public const int WcdpResEvmp = 6;

        public const int WcdpResCerr = 7;

        public const int WcdpResCsl = 8;

        public const int WcdpResSrat = 9;

        public const int WcdpResChan = 10;

        public const int WcdpResCdpa = 11;

        public const int WcdpResCdpr = 12;

        public const int WcdpResIqof = 13;

        public const int WcdpResIqim = 14;

        public const int WcdpResMtyp = 15;

        public const int WcdpResRho = 16;

        public const int WcdpResToff = 17;

        public const int WcdpResPsym = 18;

        public const int WcdpResAch = 19;

        public const int WcdpResMpic = 20;

        public const int WcdpResRcde = 21;

        public const int WcdpResArcd = 22;

        public const int WcdpResRcdq = 23;

        public const int PowResAcp = 0;

        public const int PowResCpow = 1;

        public const int PowResMcac = 2;

        public const int PowResOban = 3;

        public const int _3gueCdpScrLong = 0;

        public const int _3gueCdpScrShort = 1;

        public const int _3gueCdpBaseSlot = 0;

        public const int _3gueCdpBaseFrame = 1;

        public const int _3gueCdpEvmIntSlot = 0;

        public const int _3gueCdpEvmIntMeas = 1;

        public const int _3gueCdpEvmIntFhal = 2;

        public const int _3gueCdpEvmIntShal = 3;

        public const int I = 0;

        public const int Q = 1;

        public const int _3gueResCdp = 0;

        public const int _3gueResCdprat = 1;

        public const int _3gueResCdpover = 2;

        public const int _3gueResCdep = 3;

        public const int _3gueResSum = 4;

        public const int _3gueResCtab = 5;

        public const int _3gueResPcd = 6;

        public const int _3gueResMacc = 7;

        public const int _3gueResPvsla = 8;

        public const int _3gueResPvslr = 9;

        public const int _3gueResBstr = 10;

        public const int _3gueResPvsy = 11;

        public const int _3gueResSymbconst = 12;

        public const int _3gueResSymbevm = 13;

        public const int _3gueResFvsl = 14;

        public const int _3gueResPsvsl = 15;

        public const int _3gueResEvmMagn = 16;

        public const int _3gueResEvmPhas = 17;

        public const int _3gueResCompCons = 18;

        public const int _3gueResChipEvm = 19;

        public const int _3gueResChipMagn = 20;

        public const int _3gueResChipPhas = 21;

        public const int _3gppueMeasPow = 0;

        public const int _3gppueMeasAclr = 1;

        public const int _3gppueMeasEsp = 2;

        public const int _3gppueMeasObw = 3;

        public const int _3gppueMeasWcdp = 4;

        public const int _3gppueMeasCcdf = 5;

        public const int _3gueTracCdp = 0;

        public const int _3gueTracCdprat = 1;

        public const int _3gueTracCdpover = 2;

        public const int _3gueTracCdep = 3;

        public const int _3gueTracCtab = 4;

        public const int _3gueTracPcd = 5;

        public const int _3gueTracMacc = 6;

        public const int _3gueTracPvsla = 7;

        public const int _3gueTracPvslr = 8;

        public const int _3gueTracBstr = 9;

        public const int _3gueTracPvsy = 10;

        public const int _3gueTracSymbconst = 11;

        public const int _3gueTracSymbevm = 12;

        public const int _3gueTracFvsl = 13;

        public const int _3gueTracPsvsl = 14;

        public const int _3gueTracEvmMagn = 15;

        public const int _3gueTracEvmPhas = 16;

        public const int _3gueTracCompCons = 17;

        public const int _3gueTracChipEvm = 18;

        public const int _3gueTracChipMagn = 19;

        public const int _3gueTracChipPhas = 20;

        public const int _3gueTracEvmRms = 21;

        public const int CdpResPtot = 0;

        public const int CdpResFerr = 1;

        public const int CdpResTfr = 2;

        public const int CdpResMacc = 3;

        public const int CdpResPcd = 4;

        public const int CdpResEvmr = 5;

        public const int CdpResEvmp = 6;

        public const int CdpResCerr = 7;

        public const int CdpResCsl = 8;

        public const int CdpResSrat = 9;

        public const int CdpResChan = 10;

        public const int CdpResCdpa = 11;

        public const int CdpResCdpr = 12;

        public const int CdpResIqof = 13;

        public const int CdpResIqim = 14;

        public const int CdpResCmap = 15;

        public const int CdpResPsym = 16;

        public const int CdpResRho = 17;

        public const int CdpResToff = 18;

        public const int CdpResEvmb = 19;

        public const int CdpResEvm = 20;

        public const int CdpResMtyp = 21;

        public const int CdpResAch = 22;

        public const int CdpResMpic = 23;

        public const int CdpResArcd = 24;

        public const int TdsMeasPow = 0;

        public const int TdsMeasAclr = 1;

        public const int TdsMeasEsp = 2;

        public const int TdsMeasOban = 3;

        public const int TdsMeasPvt = 4;

        public const int TdsMeasCdp = 5;

        public const int TdsMeasCcdf = 6;

        public const int TdsMeasMcac = 7;

        public const int TdsResCdp = 0;

        public const int TdsResCdprat = 1;

        public const int TdsResCdep = 2;

        public const int TdsResMacc = 3;

        public const int TdsResPvsla = 4;

        public const int TdsResPvslr = 5;

        public const int TdsResPvsym = 6;

        public const int TdsResBstr = 7;

        public const int TdsResSum = 8;

        public const int TdsResCtab = 9;

        public const int TdsResPcd = 10;

        public const int TdsResSymbconst = 11;

        public const int TdsResSymbevm = 12;

        public const int TdsResCompconst = 13;

        public const int TdsTableOff = 0;

        public const int TdsTableSelect = 1;

        public const int TdsTableNew = 2;

        public const int TdsTableDelete = 3;

        public const int TdsTableCopy = 4;

        public const int TdsCtabOrderCode = 0;

        public const int TdsCtabOrderMid = 1;

        public const int TdbsSettingsStandardGpp = 0;

        public const int TdbsSettingsStandardTsm = 1;

        public const int TdsCdpSbandNorm = 0;

        public const int TdsCdpSbandInv = 1;

        public const int TdsCpichPattOff = 0;

        public const int TdsCpichPatt1 = 1;

        public const int TdsCpichPatt2 = 2;

        public const int TdsResultSlot = 0;

        public const int TdsResultPdat = 1;

        public const int TdsResultPd1 = 2;

        public const int TdsResultPd2 = 3;

        public const int TdsResultPmid = 4;

        public const int TdsResultRho = 5;

        public const int TdsResultMacc = 6;

        public const int TdsResultPcd = 7;

        public const int TdsResultFerr = 8;

        public const int TdsResultCerr = 9;

        public const int TdsResultTfr = 10;

        public const int TdsResultIqim = 11;

        public const int TdsResultIqof = 12;

        public const int TdsResultAct = 13;

        public const int TdsResultSrat = 14;

        public const int TdsResultChan = 15;

        public const int TdsResultSfac = 16;

        public const int TdsResultCdpr = 17;

        public const int TdsResultCdp = 18;

        public const int TdsResultEvmr = 19;

        public const int TdsResultEvmp = 20;

        public const int C2kAchPresAcp = 0;

        public const int C2kAchPresCpow = 1;

        public const int C2kAchPresMcac = 2;

        public const int C2kAchPresObw = 3;

        public const int TableOff = 0;

        public const int TableSelect = 1;

        public const int TableNew = 2;

        public const int TableDelete = 3;

        public const int TableCopy = 4;

        public const int C2kSbandInvers = 1;

        public const int C2kSbandNormal = 0;

        public const int C2kMeasSelPower = 0;

        public const int C2kMeasSelAclr = 1;

        public const int C2kMeasSelMcAclr = 2;

        public const int C2kMeasSelEsp = 3;

        public const int C2kMeasSelObw = 4;

        public const int C2kMeasSelCdp = 5;

        public const int C2kMeasSelCcdf = 6;

        public const int C2kMeasSelPvtime = 7;

        public const int C2kCtable = 0;

        public const int C2kSfactor128 = 1;

        public const int C2kSfactor64 = 0;

        public const int C2kPrefPich = 1;

        public const int C2kPrefTotal = 0;

        public const int C2kOrderBitreverse = 1;

        public const int C2kOrderHadamard = 0;

        public const int C2kMcarrierFtypeLpass = 0;

        public const int C2kMcarrierFtypeRrc = 1;

        public const int C2kFeedCdp = 0;

        public const int C2kFeedCdpRat = 1;

        public const int C2kFeedCdep = 2;

        public const int C2kFeedErrCtab = 3;

        public const int C2kFeedMacc = 4;

        public const int C2kFeedPvsl = 5;

        public const int C2kFeedPvsy = 6;

        public const int C2kFeedBstr = 7;

        public const int C2kFeedErrSumm = 8;

        public const int C2kFeedErrPcd = 9;

        public const int C2kFeedSymbCons = 10;

        public const int C2kFeedSymbEvm = 11;

        public const int C2kFeedCompCons = 12;

        public const int C2kSlot = 0;

        public const int C2kPtotal = 1;

        public const int C2kPpich = 2;

        public const int C2kRho = 3;

        public const int C2kMaccuracy = 4;

        public const int C2kPcderror = 5;

        public const int C2kIqimbalance = 6;

        public const int C2kIqoffset = 7;

        public const int C2kFerror = 8;

        public const int C2kFerppm = 9;

        public const int C2kCerror = 10;

        public const int C2kTframe = 11;

        public const int C2kActive = 12;

        public const int C2kSrate = 13;

        public const int C2kChannel = 14;

        public const int C2kSfactor = 15;

        public const int C2kToffset = 16;

        public const int C2kPoffset = 17;

        public const int C2kCdprelative = 18;

        public const int C2kCdpabsolute = 19;

        public const int C2kEvmrms = 20;

        public const int C2kEvmpeak = 21;

        public const int C2kDmtype = 29;

        public const int C2kLimModeAuto = 0;

        public const int C2kLimModeMan = 1;

        public const int C2kLimModeUser = 2;

        public const int Mc2kSfactor16 = 0;

        public const int Mc2kSfactor32 = 1;

        public const int Mc2kSfactor64 = 2;

        public const int Mc2kCdpMappingQ = 1;

        public const int Mc2kCdpMappingI = 0;

        public const int Mc2kLcodeStandard = 0;

        public const int Mc2kLcodeEsg101 = 1;

        public const int Mc2kLimModeAuto = 0;

        public const int Mc2kLimModeUser = 1;

        public const int C2kCdpRfslotFull = 0;

        public const int C2kCdpRfslotIdle = 1;

        public const int C2kLimPvtRefAuto = 0;

        public const int C2kLimPvtRefMan = 1;

        public const int C2kLimPvtRefOnce = 2;

        public const int BdoRevision0 = 0;

        public const int BdoRevisionA = 1;

        public const int C2kFeedPvch = 13;

        public const int C2kCdpCtypePilot = 0;

        public const int C2kCdpCtypeMac = 1;

        public const int C2kCdpCtypePreamble = 2;

        public const int C2kCdpCtypeData = 3;

        public const int C2kCdpMmodeAuto = 0;

        public const int C2kCdpMmodeIoq = 1;

        public const int C2kCdpMmodeComplex = 2;

        public const int C2kCdpMappingQ = 1;

        public const int C2kCdpMappingI = 0;

        public const int C2kRhopilot = 22;

        public const int C2kRho1 = 23;

        public const int C2kRho2 = 24;

        public const int C2kPpilot = 25;

        public const int C2kPmac = 26;

        public const int C2kPdata = 27;

        public const int C2kPpreamble = 28;

        public const int C2kMactive = 30;

        public const int C2kDactive = 31;

        public const int C2kPlength = 32;

        public const int C2kPdmax = 33;

        public const int C2kPdmin = 34;

        public const int C2kIpmmax = 35;

        public const int C2kRhodata = 40;

        public const int C2kRhomac = 41;

        public const int C2kMtype = 36;

        public const int MdoCdpOperAccess = 0;

        public const int MdoCdpOperTraffic = 1;

        public const int C2kPrri = 37;

        public const int C2kDrpich = 38;

        public const int C2kRhoverall = 39;

        public const int TrigModeImm = 0;

        public const int TrigModeExt = 1;

        public const int TrigModePow = 2;

        public const int WlanGateTime = 0;

        public const int WlanGateSample = 1;

        public const int WlanPosX = 0;

        public const int WlanPosY = 1;

        public const int WlanConsSym = 0;

        public const int WlanConsCarr = 1;

        public const int WlanBtypDirect = 0;

        public const int WlanBtypLofdm = 1;

        public const int WlanBtypSofdm = 2;

        public const int WlanBpsk = 0;

        public const int WlanQpsk = 1;

        public const int WlanQam16 = 2;

        public const int WlanQam64 = 3;

        public const int WlanBpsk6 = 4;

        public const int WlanBpsk9 = 5;

        public const int WlanQpsk12 = 6;

        public const int WlanQpsk18 = 7;

        public const int WlanQam1624 = 8;

        public const int WlanQam1636 = 9;

        public const int WlanQam6448 = 10;

        public const int WlanQam6454 = 11;

        public const int WlanDbpsk = 12;

        public const int WlanDqpsk = 13;

        public const int WlanCck55 = 14;

        public const int WlanCck11 = 15;

        public const int WlanPbcc55 = 16;

        public const int WlanPbcc11 = 17;

        public const int WlanPbcc22 = 18;

        public const int WlanBpsk3 = 19;

        public const int WlanBpsk45 = 20;

        public const int WlanQpsk6 = 21;

        public const int WlanQpsk9 = 22;

        public const int WlanQam1612 = 23;

        public const int WlanQam1618 = 24;

        public const int WlanQam6424 = 25;

        public const int WlanQam6427 = 26;

        public const int WlanBtypLong = 3;

        public const int WlanBtypShort = 4;

        public const int WlanDemDurMs = 0;

        public const int WlanDemDurByte = 1;

        public const int WlanFftStartOffsetAuto = 0;

        public const int WlanFftStartOffsetGic = 1;

        public const int WlanLimFerr = 0;

        public const int WlanLimSymb = 1;

        public const int WlanLimIqoff = 2;

        public const int WlanLimEvmAll = 3;

        public const int WlanLimEvmData = 4;

        public const int WlanLimEvmPil = 5;

        public const int WlanLimRtime = 6;

        public const int WlanLimFtime = 7;

        public const int WlanLimEvm = 8;

        public const int WlanAcpAdj = 0;

        public const int WlanAcpAlt = 1;

        public const int Pow = 1;

        public const int PvtEdge = 0;

        public const int PvtFull = 1;

        public const int PvtMean = 0;

        public const int PvtMax = 1;

        public const int PvtRise = 2;

        public const int PvtFall = 3;

        public const int WlanEvmModeSymbol = 0;

        public const int WlanEvmModeCarrier = 1;

        public const int WlanEvmModePhas = 2;

        public const int WlanEvmModeFreq = 3;

        public const int WlanSpecModeIeee = 0;

        public const int WlanSpecModeEtsi = 1;

        public const int WlanSpecModeFft = 2;

        public const int WlanSpecModeAcpr = 3;

        public const int WlanSpecModeFlat = 4;

        public const int SelectionAll = 0;

        public const int SelectionPil = 1;

        public const int SelectionNum = 2;

        public const int WlanStatCcdf = 0;

        public const int WlanStatBstr = 1;

        public const int WlanStatSfi = 2;

        public const int SemIeee = 0;

        public const int SemEtsi = 1;

        public const int SemUser = 2;

        public const int AcpCurr = 0;

        public const int AcpMax = 1;

        public const int WlanMeasPrea = 0;

        public const int WlanMeasPayl = 1;

        public const int WlanMeasRmsMax = 2;

        public const int WlanMeasRmsMin = 3;

        public const int WlanMeasRmsAver = 4;

        public const int WlanMeasPeak = 5;

        public const int WlanMeasCrestMax = 6;

        public const int WlanMeasCrestMin = 7;

        public const int WlanMeasCrestAver = 8;

        public const int WlanTimeFall = 0;

        public const int WlanTimeRise = 1;

        public const int WlanErrFreq = 0;

        public const int WlanErrSymb = 1;

        public const int WlanIqOffs = 0;

        public const int WlanIqImba = 1;

        public const int WlanIqQuad = 2;

        public const int WlanEvmAll = 0;

        public const int WlanEvmData = 1;

        public const int WlanEvmPilot = 2;

        public const int WlanEvm = 3;

        public const int WlanEvmPeak = 4;

        public const int WlanBurs = 0;

        public const int WlanSymb = 1;

        public const int WimaxGateTime = 0;

        public const int WimaxGateSample = 1;

        public const int WimaxMeasurementRf = 0;

        public const int WimaxMeasurementIq = 1;

        public const int SpemAnalysisDl = 0;

        public const int SpemAnalysisUl = 1;

        public const int WimaxDutcTx1 = 0;

        public const int WimaxDutcTx2 = 1;

        public const int WimaxConsSym = 0;

        public const int WimaxConsCarr = 1;

        public const int WimaxConsBurst = 2;

        public const int WimaxBconNone = 0;

        public const int WimaxBconFirst = 1;

        public const int WimaxBconUser = 2;

        public const int WimaxBconAll = 3;

        public const int WimaxBpsk12 = 0;

        public const int WimaxQpsk12 = 1;

        public const int WimaxQpsk34 = 2;

        public const int Wimax16qam12 = 3;

        public const int Wimax16qam34 = 4;

        public const int Wimax64qam23 = 5;

        public const int Wimax64qam34 = 6;

        public const int WimaxLmodDl = 0;

        public const int WimaxLmodUl = 1;

        public const int WimaxLmodAny = 2;

        public const int WimaxCestPreamonly = 0;

        public const int WimaxCestPreampayl = 1;

        public const int WimaxCestPaylonly = 2;

        public const int WimaxTracCestFull = 0;

        public const int WimaxTracCestUser = 1;

        public const int WimaxTracPilPred = 0;

        public const int WimaxTracPilDet = 1;

        public const int WimaxLimFerr = 0;

        public const int WimaxLimSymb = 1;

        public const int WimaxLimSst = 2;

        public const int WimaxLimEvmAll = 3;

        public const int WimaxLimEvmData = 4;

        public const int WimaxLimIqoff = 5;

        public const int WimaxLimBerp = 6;

        public const int WimaxLimUserr = 7;

        public const int WimaxPredefMapConfig = 0;

        public const int WimaxPredefMapIqdl = 1;

        public const int WimaxPredefMapIqul = 2;

        public const int WimaxPredefMapSmu = 3;

        public const int WimaxBconSignal = 4;

        public const int WimaxBconPredef = 5;

        public const int WimaxPreAuto = 0;

        public const int WimaxPreUser = 1;

        public const int WimaxZoneTypeDlfusc = 0;

        public const int WimaxZoneTypeDlpusc = 1;

        public const int WimaxZoneTypeUlpusc = 2;

        public const int WimaxZoneTypeDlamc2x = 3;

        public const int WimaxZoneTypeUlamc2x = 4;

        public const int WimaxZoneStccNost = 0;

        public const int WimaxZoneStccTx2 = 1;

        public const int WimaxMatrixBurs = 0;

        public const int WimaxMatrixA = 1;

        public const int WimaxMatrixB = 2;

        public const int WimaxZoneBurstFormQpsk12 = 0;

        public const int WimaxZoneBurstFormQpsk34 = 1;

        public const int WimaxZoneBurstFormQam1612 = 2;

        public const int WimaxZoneBurstFormQam1634 = 3;

        public const int WimaxZoneBurstFormQam6412 = 4;

        public const int WimaxZoneBurstFormQam6423 = 5;

        public const int WimaxZoneBurstFormQam6434 = 6;

        public const int WimaxZoneBurstFormQam6456 = 7;

        public const int WimaxZoneBursTypeFch = 0;

        public const int WimaxZoneBursTypeDlmap = 1;

        public const int WimaxZoneBursTypeUlmap = 2;

        public const int WimaxZoneBursTypeData = 3;

        public const int WimaxZoneBursTypeHarq = 4;

        public const int WimaxZoneBursTypeFastf = 5;

        public const int WimaxFbanUnsp = 0;

        public const int WimaxFbanEtsi = 1;

        public const int WimaxFbanMmds = 2;

        public const int WimaxFbanWcs = 3;

        public const int WimaxFbanUni = 5;

        public const int WimaxFbanEtsi2 = 6;

        public const int WimaxFbanMmds2 = 7;

        public const int WimaxFbanWcs2 = 8;

        public const int WimaxFbanCept2 = 9;

        public const int WimaxFbanUni2 = 10;

        public const int WimaxEvmModeSymbol = 0;

        public const int WimaxEvmModeCarrier = 1;

        public const int WimaxEvmModePhas = 2;

        public const int WimaxEvmModeFreq = 3;

        public const int WimaxModeIeee = 0;

        public const int WimaxModeEtsi = 1;

        public const int WimaxModeFft = 2;

        public const int WimaxModeFlat = 4;

        public const int WimaxModeFlatGrdel = 5;

        public const int WimaxModeFlatDiff = 6;

        public const int WimaxModeAcprAbs = 7;

        public const int WimaxModeAcprRel = 8;

        public const int WimaxModeMask = 9;

        public const int WimaxModePreamble = 10;

        public const int WimaxModulAll = 0;

        public const int WimaxModulQpsk = 1;

        public const int WimaxModulQam16 = 2;

        public const int WimaxModulQam64 = 3;

        public const int WimaxModulBpsk = 4;

        public const int WimaxStatCcdf = 0;

        public const int WimaxStatBstr = 1;

        public const int WimaxStatBsum = 2;

        public const int SemStandard = 3;

        public const int WimaxTtcFft = 0;

        public const int WimaxTtcFram = 1;

        public const int WimaxMeasRms = 0;

        public const int WimaxMeasCrest = 1;

        public const int WimaxMeasRmsAll = 2;

        public const int WimaxMeasRmsPilot = 3;

        public const int WimaxMeasRmsData = 4;

        public const int WimaxMeasRmsDlpr = 5;

        public const int WimaxMeasRmsNpil = 6;

        public const int WimaxErrFreq = 0;

        public const int WimaxErrSymb = 1;

        public const int WimaxIqOffs = 0;

        public const int WimaxIqImba = 1;

        public const int WimaxIqQuad = 2;

        public const int WimaxEvmAll = 0;

        public const int WimaxEvmData = 1;

        public const int WimaxEvmPilot = 2;

        public const int WimaxEvm = 3;

        public const int WimaxEvmPream = 4;

        public const int WimaxBurs = 0;

        public const int WimaxSymb = 1;

        public const int WimaxZone = 2;

        public const int WimaxStatZone = 3;

        public const int WimaxPrea = 0;

        public const int WimaxSubf = 1;

        public const int LteDuplexingTdd = 0;

        public const int LteDuplexingFdd = 1;

        public const int LteDownlinkChbw = 0;

        public const int LteDownlinkNrb = 1;

        public const int LteDownlinkBw140 = 0;

        public const int LteDownlinkBw250 = 1;

        public const int LteDownlinkBw300 = 2;

        public const int LteDownlinkBw500 = 3;

        public const int LteDownlinkBw1000 = 4;

        public const int LteDownlinkBw1500 = 5;

        public const int LteDownlinkBw2000 = 6;

        public const int LteDownlinkCprefixNormal = 0;

        public const int LteDownlinkCprefixExtended = 1;

        public const int LteDownlinkCprefixAuto = 2;

        public const int LteDownlinkMimoTx1 = 0;

        public const int LteDownlinkMimoTx2 = 1;

        public const int LteDownlinkMimoTx4 = 2;

        public const int LteDownlinkMimoAntAll = 0;

        public const int LteDownlinkMimoAnt1 = 1;

        public const int LteDownlinkMimoAnt2 = 2;

        public const int LteDownlinkMimoAnt3 = 3;

        public const int LteDownlinkMimoAnt4 = 4;

        public const int LteDownlinkCestTgpp = 0;

        public const int LteDownlinkCestPilot = 1;

        public const int LteDownlinkCestPilpay = 2;

        public const int AttrLteDownlinkEvmCalcTgpp = 0;

        public const int AttrLteDownlinkEvmCalcOtp = 1;

        public const int LteDownlinkPhaseTrackingOff = 0;

        public const int LteDownlinkPhaseTrackingPilot = 1;

        public const int LteDownlinkPhaseTrackingPilpay = 2;

        public const int LteDownlinkModulationQpsk = 0;

        public const int LteDownlinkModulation16qam = 1;

        public const int LteDownlinkModulation64qam = 2;

        public const int LteDownlinkPhichDurationNormal = 0;

        public const int LteDownlinkPhichDurationExtended = 1;

        public const int LteDownlinkCbuf = 0;

        public const int LteDownlinkEvca = 1;

        public const int LteDownlinkEvsy = 2;

        public const int LteDownlinkFevs = 3;

        public const int LteDownlinkEvsu = 4;

        public const int LteDownlinkSem = 5;

        public const int LteDownlinkAcp = 6;

        public const int LteDownlinkPspe = 7;

        public const int LteDownlinkPvrp = 8;

        public const int LteDownlinkPvrr = 9;

        public const int LteDownlinkFlat = 10;

        public const int LteDownlinkGdel = 11;

        public const int LteDownlinkFdif = 12;

        public const int LteDownlinkCons = 13;

        public const int LteDownlinkDftc = 14;

        public const int LteDownlinkCcdf = 15;

        public const int LteDownlinkSflo = 16;

        public const int LteDownlinkAsum = 17;

        public const int LteDownlinkBstr = 18;

        public const int LteDownlinkStreamSymbols = 0;

        public const int LteDownlinkStreamBits = 1;

        public const int LteDownlinkCategoryA = 0;

        public const int LteDownlinkCategoryB = 1;

        public const int LteDownlinkAssumedEutra = 0;

        public const int LteDownlinkAssumedUtra128 = 1;

        public const int LteDownlinkAssumedUtra384 = 2;

        public const int LteDownlinkAssumedUtra768 = 3;

        public const int LteDownlinkLocAmd = 0;

        public const int LteDownlinkLocBmd = 1;

        public const int LteDownlinkQpsk = 0;

        public const int LteDownlink16qam = 1;

        public const int LteDownlink64qam = 2;

        public const int MeastypeValues = 0;

        public const int MeastypeSummary = 1;

        public const int MeastypeProbability = 0;

        public const int MeastypePowerSteps = 1;

        public const int TetraChannBand25 = 0;

        public const int TetraChannBand50 = 1;

        public const int TetraChannBand100 = 2;

        public const int TetraChannBand150 = 3;

        public const int TetraSlotTypeNdb = 0;

        public const int TetraSlotTypeNub = 1;

        public const int TetraSlotTypeCub = 2;

        public const int TetraSlotTypeRab = 3;

        public const int TetraUnitSeconds = 0;

        public const int TetraUnitSlots = 1;

        public const int TetraModTypeQm4 = 0;

        public const int TetraModTypeQm16 = 1;

        public const int TetraModTypeQm64 = 2;

        public const int TetraCarrierOffsetKh1 = 0;

        public const int TetraCarrierOffsetKh10 = 1;

        public const int TetraTraceCurr = 1;

        public const int TetraTraceMin = 2;

        public const int TetraTracePeak = 3;

        public const int TetraTraceAver = 4;

        public const int MeasPvt = 0;

        public const int MeasEvms = 1;

        public const int MeasEvmc = 2;

        public const int MeasMerr = 3;

        public const int MeasPerr = 4;

        public const int MeasSpec = 5;

        public const int MeasAcpm = 6;

        public const int MeasAcpt = 7;

        public const int MeasCapt = 8;

        public const int TetraLimEvmAll = 0;

        public const int TetraLimEvmData = 1;

        public const int TetraLimEvmDhea = 2;

        public const int TetraLimEvmDsp = 3;

        public const int TetraLimEvmSpil = 4;

        public const int TetraLimIqof = 5;

        public const int TetraLimFerr = 6;

        public const int TetraLimPhas = 7;

        public const int TetraLimMagn = 8;

        public const int TetraMeasMax = 2;

        public const int TetraMeasAver = 3;

        public const int TetraAcpModulation = 0;

        public const int TetraAcpTransient = 1;

        public const int TetraAcpChannelNumberNeg3 = 0;

        public const int TetraAcpChannelNumberNeg2 = 1;

        public const int TetraAcpChannelNumberNeg1 = 2;

        public const int TetraAcpChannelNumberPos1 = 4;

        public const int TetraAcpChannelNumberPos2 = 5;

        public const int TetraAcpChannelNumberPos3 = 6;

        public const int TetraPvt = 0;

        public const int TetraEvms = 1;

        public const int TetraEvmc = 2;

        public const int TetraMagn = 3;

        public const int TetraPhas = 4;

        public const int TetraFft = 5;

        public const int TetraAcpm = 6;

        public const int TetraAcptUpp = 7;

        public const int TetraAcptLow = 8;

        public const int TetraCsym = 9;

        public const int TetraCcar = 10;

        public const int TetraBstr = 11;

        public const int TetraSumm = 12;

        public const int TetraSymbolsAll = 0;

        public const int TetraSymbolsData = 1;

        public const int TetraSymbolsHead = 2;

        public const int TetraSymbolsSync = 3;

        public const int TetraSymbolsPil = 4;

        public const int CarrierAll = 0;

        public const int CarrierNum = 1;

        public const int TetraMeasMin = 1;

        public const int TetraMeasCurr = 4;

        public const int TetraMeasEvmAll = 0;

        public const int TetraMeasEvmDsp = 1;

        public const int TetraMeasEvmDhea = 2;

        public const int TetraMeasEvmData = 3;

        public const int TetraMeasEvmSpil = 4;

        public const int TetraMeasEvmIqof = 5;

        public const int TetraMeasEvmGimb = 6;

        public const int TetraMeasEvmQuad = 7;

        public const int TetraMeasEvmAmpd = 8;

        public const int TetraMeasEvmFerr = 9;

        public const int TetraMeasEvmPhas = 10;

        public const int TetraMeasEvmMag = 11;

        public const int TetraMeasEvmBpwr = 12;

        public const int TetraMeasEvmCres = 13;

        public const int TetraPvtRef = 0;

        public const int TetraPvtSlot = 1;

        public const int TetraPvtBefb = 2;

        public const int TetraPvtRelBefb = 3;

        public const int TetraPvtAftb = 4;

        public const int TetraPvtRelAftb = 5;

        public const int TetraPvtTrgToSync = 6;

        public const int TetraAcpBw = 0;

        public const int TetraAcpRcc = 1;

        public const int TetraAcpOffs = 2;

        public const int TetraAcpAbsPow = 3;

        public const int TetraAcpRelPow = 4;

        public const int TetraAcpRbw = 6;

        public const int TetraMeasNone = 0;

        public const int TetraAcpChannelNumberTx = 3;

        public const int InputRf = 0;

        public const int InputCal = 1;

        public const int InputTg = 2;

        public const int ServiceBbIq = 0;

        public const int ServiceBbGnd = 1;

        public const int ServiceBbCaldc = 2;

        public const int ServiceBbCalpulse = 3;

        public const int ServiceFeedIhig = 0;

        public const int ServiceFeedIlow = 1;

        public const int ServiceFeedQhig = 2;

        public const int ServiceFeedQlow = 3;

        public const int ServCombFreqComb1 = 0;

        public const int ServCombFreqComb64 = 1;

        public const int ServCombFreqComb65 = 2;

        public const int FileNew = 0;

        public const int FileCopy = 1;

        public const int FileRename = 2;

        public const int FileDelete = 3;

        public const int FileRdir = 4;

        public const int FileFormat = 5;

        public const int FileCdir = 6;

        public const int FileCdisc = 7;

        public const int MemSave = 0;

        public const int MemRecall = 1;

        public const int MemDelete = 2;

        public const int MemStartup = 3;

        public const int FileItemsNone = 0;

        public const int FileItemsDefault = 1;

        public const int FileItemsAll = 2;

        public const int FileItemsHwsettings = 3;

        public const int FileItemsAlltraces = 4;

        public const int FileItemsAlllines = 5;

        public const int FileItemsSourCalData = 6;

        public const int FileItemsAlltran = 7;

        public const int FileItemsSpectrogram = 8;

        public const int FileItemsClist = 9;

        public const int FileItemsFinal = 10;

        public const int AttModeLno = 2;

        public const int SweModeAuto = 0;

        public const int SweModeList = 1;

        public const int SweModeEsync = 2;

        public const int SweModeEsp = 3;

        public const int TracModRms = 5;

        public const int DetectorTypePortableAutoPeak = 0;

        public const int DetectorTypePortableNegative = 1;

        public const int DetectorTypePortablePositive = 2;

        public const int DetectorTypePortableSample = 3;

        public const int DetectorTypePortableRms = 4;

        public const int CalAll = 0;

        public const int HcopyDeviceMem = 0;

        public const int HcopyDevicePrn = 1;

        public const int HcopyDeviceClp = 2;

        public const int HcopyDeviceLangGdi = 0;

        public const int HcopyDeviceLangWmf = 1;

        public const int HcopyDeviceLangEwmf = 2;

        public const int HcopyDeviceLangBmp = 3;

        public const int ListFtypeNois = 5;

        public const int SemFilterTypeP5 = 4;

        public const int MrecPhas = 1;

        public const int MrecUph = 0;

        public const int MrecTrgSrcRun = 0;

        public const int MrecTrgSrcExt = 1;

        public const int MrecTrgSrcIf = 2;

        public const int MrecTrgSrcFm = 3;

        public const int MrecTrgSrcPm = 4;

        public const int MrecTrgSrcAm = 5;

        public const int MrecTrgSrcVideo = 6;

        public const int MrecTrgSrcAudio = 7;

        public const int LevDetNar = 0;

        public const int LevDetWide = 1;

        public const int PmetMeastimeNorm = 0;

        public const int PmetMeastimeShort = 1;

        public const int PmetMeastimeLong = 2;

        public const int PwmSensorA = 0;

        public const int PwmSensorB = 1;

        public const int ModulationTypeAm = 0;

        public const int ModulationTypeFm = 1;

        public const int ModulationTypePm = 2;

        public const int MeasPpeakModeAbs = 0;

        public const int MeasPpeakModeRel = 1;

        public const int MeasMpeakModeAbs = 0;

        public const int MeasMpeakModeRel = 1;

        public const int MeasPaverageModeAbs = 0;

        public const int MeasPaverageModeRel = 1;

        public const int MeasRmsModeAbs = 0;

        public const int MeasRmsModeRel = 1;

        public const int MeasAverageModeAbs = 0;

        public const int MeasAverageModeRel = 1;

        public const int MeasSrmsModeAbs = 0;

        public const int MeasSrmsModeRel = 1;

        public const int AudioInModeRf = 0;

        public const int AudioInModeAudio = 1;

        public const int AudioInLevel4v = 0;

        public const int AudioInLevel400mv = 1;

        public const int CorrTran = 0;

        public const int CorrRefl = 1;

        public const int CorrAcqThr = 0;

        public const int CorrAcqOpen = 1;

        public const int GsmTscUser = 8;

        public const int BtoPtypeDm1 = 4;

        public const int BtoPtypeDm3 = 5;

        public const int BtoPtypeDm5 = 6;

        public const int BtoPtypeFhs = 7;

        public const int BtoPtypeHv1 = 8;

        public const int BtoPtypeHv2 = 9;

        public const int BtoPtypeHv3 = 10;

        public const int BtoPtypeDv = 11;

        public const int BtoPtypeNull = 12;

        public const int BtoPtypePoll = 13;

        public const int BtoPtypeAux1 = 14;

        public const int BtoPtypeUndef = 15;

        public const int AbsunitDb = 3;

        public const int AbsunitVswr = 4;

        public const int PmetStdUser = 0;

        public const int PmetStdGsm = 1;

        public const int PmetStdEdge = 2;

        public const int PmetStdWcdma = 3;

        public const int PmetStdCdma1 = 4;

        public const int PmetStdCdma2k = 5;

        public const int PmetStdDvbt = 6;

        public const int PmetStdDab = 7;

        public const int PmetStdTetra = 8;

        public const int GsmK10BandClassPgsm = 0;

        public const int GsmK10BandClassEgsm = 1;

        public const int GsmK10BandClassDcs = 2;

        public const int GsmK10BandClassPcs = 3;

        public const int GsmK10BandClassTgsm = 4;

        public const int GsmK10BandClassRgsm = 5;

        public const int GsmK10BandClassGsm = 6;

        public const int GsmK10FilterGmsk = 0;

        public const int GsmK10FilterLinGmsk = 1;

        public const int ResFail = 0;

        public const int ResPass = 1;

        public const int NoiseConfSystLoscFixed = 0;

        public const int NoiseConfSystLoscVariable = 1;

        public const int NoiseConfDutAmplifier = 0;

        public const int NoiseConfDutDownconv = 1;

        public const int NoiseConfDutUpconv = 2;

        public const int VsaFactoryAll = 5;

        public const int ResultXtimDdemMeas = 0;

        public const int ResultXtimDdemRef = 1;

        public const int ResultXtimDdemErrMph = 2;

        public const int ResultXtimDdemErrVect = 3;

        public const int ResultXtimDdemImp = 4;

        public const int ResultXfrDdemRat = 5;

        public const int ResultXfrDdemIrat = 6;

        public const int ResultXtimDdemSymb = 7;

        public const int ResultTcap = 8;

        public const int ResultMacc = 9;

        public const int VsaResultFormatMagn = 0;

        public const int VsaResultFormatPhas = 1;

        public const int VsaResultFormatUphas = 2;

        public const int VsaResultFormatRim = 3;

        public const int VsaResultFormatFreq = 4;

        public const int VsaResultFormatComp = 5;

        public const int VsaResultFormatCons = 6;

        public const int VsaResultFormatGdel = 7;

        public const int VsaResultFormatIeye = 8;

        public const int VsaResultFormatQeye = 9;

        public const int VsaResultFormatConv = 10;

        public const int VsaModFormatQpsk = 0;

        public const int VsaModFormatPsk = 1;

        public const int VsaModFormatMsk = 2;

        public const int VsaModFormatQam = 3;

        public const int VsaModFormatFsk = 4;

        public const int VsaModFormatVsb = 5;

        public const int VsaModFormatUqam = 6;

        public const int ModQpskNorm = 0;

        public const int ModQpskDiff = 1;

        public const int ModQpskOffs = 2;

        public const int ModQpskDpi4 = 3;

        public const int ModPskNorm = 0;

        public const int ModPskDiff = 1;

        public const int ModPskN3pi8 = 2;

        public const int ModMskNorm = 0;

        public const int ModMskDiff = 1;

        public const int ModQamNorm = 0;

        public const int ModQamDiff = 1;

        public const int ModEvmSymbol = 0;

        public const int ModEvmSignal = 1;

        public const int _3gppBsAclrModeAbs = 0;

        public const int _3gppBsAclrModeRel = 1;

        public const int _3gppBsAclrCpacpConfAcpRefSettingsMin = 0;

        public const int _3gppBsAclrCpacpConfAcpRefSettingsMax = 1;

        public const int _3gppBsAclrCpacpConfAcpRefSettingsLhig = 2;

        public const int _3gbsResChipPhas = 19;

        public const int _3gbsResPsvsl = 20;

        public const int TdsResCompevm = 13;

        public const int C2kAntennaOff = 0;

        public const int C2kAntenna1 = 1;

        public const int C2kAntenna2 = 2;

        public const int SpecIeee = 0;

        public const int SpecEtsi = 1;

        public const int PreFreq = 0;

        public const int PrePhas = 1;

        public const int WimaxFbanCept = 4;

        public const int WimaxFlatFlat = 0;

        public const int WimaxFlatGrd = 1;

        public const int WimaxFlatDif = 2;

        public const int WimaxOfdm = 0;

        public const int WimaxOfdma = 1;
    }

    public enum rsspecanProperties
    {

        /// <summary>
        /// System.Int32
        /// </summary>
        RsAttrRangeCheck = 1050002,

        /// <summary>
        /// System.Boolean
        /// </summary>
        RsAttrQueryInstrumentStatus = 1050003,

        /// <summary>
        /// System.Boolean
        /// </summary>
        RsAttrCache = 1050004,

        /// <summary>
        /// System.Boolean
        /// </summary>
        RsAttrSimulate = 1050005,

        /// <summary>
        /// System.Boolean
        /// </summary>
        RsAttrRecordCoercions = 1050006,

        /// <summary>
        /// System.Boolean
        /// </summary>
        RsAttrInterchangeCheck = 1050021,

        /// <summary>
        /// System.Boolean
        /// </summary>
        RsAttrSpy = 1050022,

        /// <summary>
        /// System.Boolean
        /// </summary>
        RsAttrUseSpecificSimulation = 1050023,

        /// <summary>
        /// System.String
        /// </summary>
        RsAttrClassDriverDescription = 1050518,

        /// <summary>
        /// System.String
        /// </summary>
        RsAttrClassDriverPrefix = 1050301,

        /// <summary>
        /// System.String
        /// </summary>
        RsAttrClassDriverVendor = 1050517,

        /// <summary>
        /// System.String
        /// </summary>
        RsAttrClassDriverRevision = 1050552,

        /// <summary>
        /// System.Int32
        /// </summary>
        RsAttrClassDriverClassSpecMajorVersion = 1050519,

        /// <summary>
        /// System.Int32
        /// </summary>
        RsAttrClassDriverClassSpecMinorVersion = 1050520,

        /// <summary>
        /// System.String
        /// </summary>
        RsAttrSpecificDriverDescription = 1050514,

        /// <summary>
        /// System.String
        /// </summary>
        RsAttrSpecificDriverPrefix = 1050302,

        /// <summary>
        /// System.String
        /// </summary>
        RsAttrSpecificDriverLocator = 1050303,

        /// <summary>
        /// System.String
        /// </summary>
        RsAttrSpecificDriverVendor = 1050513,

        /// <summary>
        /// System.String
        /// </summary>
        RsAttrSpecificDriverRevision = 1050551,

        /// <summary>
        /// System.Int32
        /// </summary>
        RsAttrSpecificDriverClassSpecMajorVersion = 1050515,

        /// <summary>
        /// System.Int32
        /// </summary>
        RsAttrSpecificDriverClassSpecMinorVersion = 1050516,

        /// <summary>
        /// System.String
        /// </summary>
        RsAttrSupportedInstrumentModels = 1050327,

        /// <summary>
        /// System.String
        /// </summary>
        RsAttrGroupCapabilities = 1050401,

        /// <summary>
        /// System.String
        /// </summary>
        RsAttrFunctionCapabilities = 1050402,

        /// <summary>
        /// System.Int32
        /// </summary>
        RsAttrChannelCount = 1050203,

        /// <summary>
        /// System.String
        /// </summary>
        RsAttrDriverSetup = 1050007,

        /// <summary>
        /// System.String
        /// </summary>
        RsAttrInstrumentManufacturer = 1050511,

        /// <summary>
        /// System.String
        /// </summary>
        RsAttrInstrumentModel = 1050512,

        /// <summary>
        /// System.String
        /// </summary>
        RsAttrInstrumentFirmwareRevision = 1050510,

        /// <summary>
        /// System.String
        /// </summary>
        RsAttrOptionsList = 1050591,

        /// <summary>
        /// System.String
        /// </summary>
        RsAttrIoResourceDescriptor = 1050304,

        /// <summary>
        /// System.String
        /// </summary>
        RsAttrLogicalName = 1050305,

        /// <summary>
        /// System.Int32
        /// </summary>
        RsAttrPrimaryError = 1050101,

        /// <summary>
        /// System.Int32
        /// </summary>
        RsAttrSecondaryError = 1050102,

        /// <summary>
        /// System.String
        /// </summary>
        RsAttrErrorElaboration = 1050103,

        /// <summary>
        /// System.IntPtr
        /// </summary>
        RsAttrVisaRmSession = 1050321,

        /// <summary>
        /// System.IntPtr
        /// </summary>
        RsAttrIoSession = 1050322,

        /// <summary>
        /// System.String
        /// </summary>
        RsAttrIoSessionType = 1050324,

        /// <summary>
        /// System.Int32
        /// </summary>
        RsAttrSpecificDriverMajorVersion = 1050503,

        /// <summary>
        /// System.Int32
        /// </summary>
        RsAttrSpecificDriverMinorVersion = 1050504,

        /// <summary>
        /// System.Int32
        /// </summary>
        RsAttrSpecificDriverMinorMinorVersion = 1050590,

        /// <summary>
        /// System.Int32
        /// </summary>
        RsAttrClassDriverMajorVersion = 1050505,

        /// <summary>
        /// System.Int32
        /// </summary>
        RsAttrClassDriverMinorVersion = 1050506,

        /// <summary>
        /// System.Int32
        /// </summary>
        RsAttrEngineMajorVersion = 1050501,

        /// <summary>
        /// System.Int32
        /// </summary>
        RsAttrEngineMinorVersion = 1050502,

        /// <summary>
        /// System.String
        /// </summary>
        RsAttrEngineRevision = 1050553,

        /// <summary>
        /// System.Int32
        /// </summary>
        RsAttrOpcCallback = 1050602,

        /// <summary>
        /// System.Int32
        /// </summary>
        RsAttrCheckStatusCallback = 1050603,

        /// <summary>
        /// System.Int32
        /// </summary>
        RsAttrOpcTimeout = 1051001,

        /// <summary>
        /// System.Int32
        /// </summary>
        AmplitudeUnits = 1150001,

        /// <summary>
        /// System.Double
        /// </summary>
        Attenuation = 1150002,

        /// <summary>
        /// System.Boolean
        /// </summary>
        AttenuationAuto = 1150003,

        /// <summary>
        /// System.Int32
        /// </summary>
        AttenuationMode = 1150004,

        /// <summary>
        /// System.Int32
        /// </summary>
        AttenuationModePortable = 1150005,

        /// <summary>
        /// System.Double
        /// </summary>
        FrequencyStart = 1150006,

        /// <summary>
        /// System.Double
        /// </summary>
        FrequencyStop = 1150007,

        /// <summary>
        /// System.Double
        /// </summary>
        FrequencyOffset = 1150008,

        /// <summary>
        /// System.Double
        /// </summary>
        FrequencyCenter = 1150009,

        /// <summary>
        /// System.Double
        /// </summary>
        FrequencyCenterStep = 1150010,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FrequencyStepAuto = 1150011,

        /// <summary>
        /// System.Int32
        /// </summary>
        FrequencyCenterLink = 1150012,

        /// <summary>
        /// System.Int32
        /// </summary>
        FrequencyCenterLinkFactor = 1150013,

        /// <summary>
        /// System.Double
        /// </summary>
        FrequencySpan = 1150014,

        /// <summary>
        /// System.String
        /// </summary>
        FrequencySpanFull = 1150015,

        /// <summary>
        /// System.Boolean
        /// </summary>
        AutoSpanMode = 1150016,

        /// <summary>
        /// System.Int32
        /// </summary>
        FrequencyMode = 1150017,

        /// <summary>
        /// System.Int32
        /// </summary>
        FrequencyInputMode = 1150018,

        /// <summary>
        /// System.String
        /// </summary>
        ChannelTable = 1150019,

        /// <summary>
        /// System.Double
        /// </summary>
        InputImpedance = 1150020,

        /// <summary>
        /// System.Int32
        /// </summary>
        NumberOfSweeps = 1150021,

        /// <summary>
        /// System.Double
        /// </summary>
        ReferenceLevel = 1150022,

        /// <summary>
        /// System.Double
        /// </summary>
        ReferenceLevelOffset = 1150023,

        /// <summary>
        /// System.Double
        /// </summary>
        ResolutionBandwidth = 1150024,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ResolutionBandwidthAuto = 1150025,

        /// <summary>
        /// System.Double
        /// </summary>
        ResolutionBandwidthRatio = 1150026,

        /// <summary>
        /// System.Int32
        /// </summary>
        SweepMode = 1150027,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SweepModeContinuous = 1150028,

        /// <summary>
        /// System.Double
        /// </summary>
        SweepTime = 1150029,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SweepTimeAuto = 1150030,

        /// <summary>
        /// System.Int32
        /// </summary>
        SweepPoints = 1150031,

        /// <summary>
        /// System.Int32
        /// </summary>
        SweepType = 1150032,

        /// <summary>
        /// System.Int32
        /// </summary>
        SweepCountCurrent = 1150033,

        /// <summary>
        /// System.Int32
        /// </summary>
        VerticalScale = 1150034,

        /// <summary>
        /// System.Double
        /// </summary>
        VideoBandwidth = 1150035,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VideoBandwidthAuto = 1150036,

        /// <summary>
        /// System.Double
        /// </summary>
        VideoBandwidthRatio = 1150037,

        /// <summary>
        /// System.Int32
        /// </summary>
        FftFilterMode = 1150038,

        /// <summary>
        /// System.Int32
        /// </summary>
        ParameterCoupling = 1150039,

        /// <summary>
        /// System.Boolean
        /// </summary>
        CorrYig = 1150040,

        /// <summary>
        /// System.Double
        /// </summary>
        YResult = 1150041,

        /// <summary>
        /// System.Int32
        /// </summary>
        HorizontalScale = 1150042,

        /// <summary>
        /// System.Int32
        /// </summary>
        IfOutputSource = 1150043,

        /// <summary>
        /// System.String
        /// </summary>
        SanMode = 1150044,

        /// <summary>
        /// System.String
        /// </summary>
        GetInstrMode = 1150045,

        /// <summary>
        /// System.Int32
        /// </summary>
        ChannelFrequencyModeChannelNumber = 1150046,

        /// <summary>
        /// System.String
        /// </summary>
        MeasAbort = 1150047,

        /// <summary>
        /// System.String
        /// </summary>
        Init = 1150048,

        /// <summary>
        /// System.String
        /// </summary>
        InitWithoutOpc = 1150049,

        /// <summary>
        /// System.String
        /// </summary>
        InitConmeas = 1150050,

        /// <summary>
        /// System.String
        /// </summary>
        Hold = 1150051,

        /// <summary>
        /// System.String
        /// </summary>
        InitSwTrigger = 1150052,

        /// <summary>
        /// System.String
        /// </summary>
        InitSpurious = 1150053,

        /// <summary>
        /// System.String
        /// </summary>
        InitEmiTest = 1150054,

        /// <summary>
        /// System.String
        /// </summary>
        InitFinalMeasurementTest = 1150055,

        /// <summary>
        /// System.String
        /// </summary>
        InitSemMeas = 1150056,

        /// <summary>
        /// System.String
        /// </summary>
        UpdateIqResults = 1150057,

        /// <summary>
        /// System.Boolean
        /// </summary>
        AvgState = 1150058,

        /// <summary>
        /// System.Int32
        /// </summary>
        AvgType = 1150059,

        /// <summary>
        /// System.Int32
        /// </summary>
        AvgCount = 1150060,

        /// <summary>
        /// System.Int32
        /// </summary>
        ResolutionBandwidthFilterType = 1150061,

        /// <summary>
        /// System.Int32
        /// </summary>
        VideoBandwidthType = 1150062,

        /// <summary>
        /// System.Int32
        /// </summary>
        PllBandwidth = 1150063,

        /// <summary>
        /// System.String
        /// </summary>
        TfacGenName = 1150064,

        /// <summary>
        /// System.String
        /// </summary>
        TfacSelName = 1150065,

        /// <summary>
        /// System.Int32
        /// </summary>
        TfacUnit = 1150066,

        /// <summary>
        /// System.Int32
        /// </summary>
        TfacScaling = 1150067,

        /// <summary>
        /// System.String
        /// </summary>
        TfacComment = 1150068,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TfacState = 1150069,

        /// <summary>
        /// System.String
        /// </summary>
        TfacDelete = 1150070,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TfacDisplay = 1150071,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TfacAdjState = 1150072,

        /// <summary>
        /// System.String
        /// </summary>
        TfacSetComment = 1150073,

        /// <summary>
        /// System.String
        /// </summary>
        TfacActName = 1150074,

        /// <summary>
        /// System.String
        /// </summary>
        CvlName = 1150075,

        /// <summary>
        /// System.String
        /// </summary>
        CvlMixer = 1150076,

        /// <summary>
        /// System.String
        /// </summary>
        CvlMixerSnum = 1150077,

        /// <summary>
        /// System.Int32
        /// </summary>
        CvlBand = 1150078,

        /// <summary>
        /// System.Int32
        /// </summary>
        CvlHarm = 1150079,

        /// <summary>
        /// System.Int32
        /// </summary>
        CvlPortMixer = 1150080,

        /// <summary>
        /// System.Double
        /// </summary>
        CvlBias = 1150081,

        /// <summary>
        /// System.String
        /// </summary>
        CvlComment = 1150082,

        /// <summary>
        /// System.String
        /// </summary>
        CvlClear = 1150083,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TraceState = 1150084,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TraceResetBehavior = 1150085,

        /// <summary>
        /// System.Int32
        /// </summary>
        TraceType = 1150086,

        /// <summary>
        /// System.String
        /// </summary>
        TraceToMemory = 1150087,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TraceMathState = 1150088,

        /// <summary>
        /// System.Double
        /// </summary>
        TraceMathPosition = 1150089,

        /// <summary>
        /// System.Int32
        /// </summary>
        TraceMathMode = 1150090,

        /// <summary>
        /// System.Int32
        /// </summary>
        DetectorType = 1150091,

        /// <summary>
        /// System.Boolean
        /// </summary>
        DetectorTypeAuto = 1150092,

        /// <summary>
        /// System.Int32
        /// </summary>
        DetectorTypePortable = 1150093,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MemoryDisplayTraceState = 1150094,

        /// <summary>
        /// System.Int32
        /// </summary>
        Calibration = 1150095,

        /// <summary>
        /// System.String
        /// </summary>
        CalibrationAbort = 1150096,

        /// <summary>
        /// System.String
        /// </summary>
        CalibrationResultQuery = 1150097,

        /// <summary>
        /// System.Boolean
        /// </summary>
        CalibrationState = 1150098,

        /// <summary>
        /// System.Boolean
        /// </summary>
        CalProbeState = 1150099,

        /// <summary>
        /// System.String
        /// </summary>
        CalProbeDataSelect = 1150100,

        /// <summary>
        /// System.Double
        /// </summary>
        CalProbeDcVoltage = 1150101,

        /// <summary>
        /// System.Double
        /// </summary>
        CalProbePulseFreq = 1150102,

        /// <summary>
        /// System.Int32
        /// </summary>
        CalProbeAdjustMode = 1150103,

        /// <summary>
        /// System.Int32
        /// </summary>
        CalProbeCheck = 1150104,

        /// <summary>
        /// System.String
        /// </summary>
        CalProbeDelete = 1150105,

        /// <summary>
        /// System.Boolean
        /// </summary>
        CalProbeManCompensation = 1150106,

        /// <summary>
        /// System.Boolean
        /// </summary>
        CalProbeFreqRespCompensation = 1150107,

        /// <summary>
        /// System.Boolean
        /// </summary>
        CalProbeGainErrorCompensation = 1150108,

        /// <summary>
        /// System.String
        /// </summary>
        CalProbeStart = 1150109,

        /// <summary>
        /// System.Boolean
        /// </summary>
        InpUportState = 1150110,

        /// <summary>
        /// System.String
        /// </summary>
        InpUportValue = 1150111,

        /// <summary>
        /// System.String
        /// </summary>
        OutUportValue = 1150112,

        /// <summary>
        /// System.String
        /// </summary>
        AutoAdjustAll = 1150113,

        /// <summary>
        /// System.String
        /// </summary>
        AutoAdjustFreq = 1150114,

        /// <summary>
        /// System.String
        /// </summary>
        AutoAdjustLevel = 1150115,

        /// <summary>
        /// System.Double
        /// </summary>
        AutoAdjustLevelMeasTime = 1150116,

        /// <summary>
        /// System.Int32
        /// </summary>
        AutoAdjustLevelMeasTimeMode = 1150117,

        /// <summary>
        /// System.String
        /// </summary>
        AutoScaleSingleScreen = 1150118,

        /// <summary>
        /// System.String
        /// </summary>
        AutoScaleAllScreens = 1150119,

        /// <summary>
        /// System.Double
        /// </summary>
        MarkerAmplitude = 1150120,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MarkerEnabled = 1150121,

        /// <summary>
        /// System.Double
        /// </summary>
        MarkerPosition = 1150122,

        /// <summary>
        /// System.Int32
        /// </summary>
        MarkerTrace = 1150123,

        /// <summary>
        /// System.String
        /// </summary>
        MarkerAoff = 1150124,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MarkerLoex = 1150125,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SignalTrackEnabled = 1150126,

        /// <summary>
        /// System.Double
        /// </summary>
        SignalTrackBwid = 1150127,

        /// <summary>
        /// System.Double
        /// </summary>
        SignalTrackThreshold = 1150128,

        /// <summary>
        /// System.Int32
        /// </summary>
        SignalTrackTrace = 1150129,

        /// <summary>
        /// System.Int32
        /// </summary>
        MarkerStepSize = 1150130,

        /// <summary>
        /// System.Double
        /// </summary>
        MarkerProbability = 1150131,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MarkerReceiverFreqCoupling = 1150132,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MarkerSubscanCoupling = 1150133,

        /// <summary>
        /// System.String
        /// </summary>
        MarkerSearchMinRight = 1150134,

        /// <summary>
        /// System.String
        /// </summary>
        MarkerSearchMinLeft = 1150135,

        /// <summary>
        /// System.String
        /// </summary>
        MarkerSearchPeak = 1150136,

        /// <summary>
        /// System.String
        /// </summary>
        MarkerSearchPeakNext = 1150137,

        /// <summary>
        /// System.String
        /// </summary>
        MarkerSearchPeakRight = 1150138,

        /// <summary>
        /// System.String
        /// </summary>
        MarkerSearchPeakLeft = 1150139,

        /// <summary>
        /// System.String
        /// </summary>
        MarkerSearchMin = 1150140,

        /// <summary>
        /// System.String
        /// </summary>
        MarkerSearchMinNext = 1150141,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MarkerSearchPeakAuto = 1150142,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MarkerSearchMinAuto = 1150143,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MarkerThresholdState = 1150144,

        /// <summary>
        /// System.Double
        /// </summary>
        MarkerThreshold = 1150145,

        /// <summary>
        /// System.Double
        /// </summary>
        PeakExcursion = 1150146,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MarkerSearchLimitsState = 1150147,

        /// <summary>
        /// System.Double
        /// </summary>
        MarkerSearchLimitsLeft = 1150148,

        /// <summary>
        /// System.Double
        /// </summary>
        MarkerSearchLimitsRight = 1150149,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MarkerSearchLimitsZoom = 1150150,

        /// <summary>
        /// System.Int32
        /// </summary>
        MarkerPeakListCount = 1150151,

        /// <summary>
        /// System.Int32
        /// </summary>
        MarkerPeakListFound = 1150152,

        /// <summary>
        /// System.Int32
        /// </summary>
        MarkerPeakListSort = 1150153,

        /// <summary>
        /// System.Double
        /// </summary>
        MarkerZoom = 1150154,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MarkerZoomState = 1150155,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MarkerZoomStatePortable = 1150156,

        /// <summary>
        /// System.Int32
        /// </summary>
        MarkerZoomFactorPortable = 1150157,

        /// <summary>
        /// System.Double
        /// </summary>
        MarkerZoomXPortable = 1150158,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MarkerDemodState = 1150159,

        /// <summary>
        /// System.Int32
        /// </summary>
        MarkerDemodType = 1150160,

        /// <summary>
        /// System.Double
        /// </summary>
        MarkerDemodHoldoff = 1150161,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MarkerDemodCont = 1150162,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MarkerDemodSquelchState = 1150163,

        /// <summary>
        /// System.Double
        /// </summary>
        MarkerDemodSquelchLevel = 1150164,

        /// <summary>
        /// System.String
        /// </summary>
        MarkerToCenter = 1150165,

        /// <summary>
        /// System.String
        /// </summary>
        MarkerToStep = 1150166,

        /// <summary>
        /// System.String
        /// </summary>
        MarkerToReference = 1150167,

        /// <summary>
        /// System.String
        /// </summary>
        MarkerToReferencePortable = 1150168,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MarkerLink = 1150169,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MarkerRefState = 1150170,

        /// <summary>
        /// System.Double
        /// </summary>
        MarkerRefLevel = 1150171,

        /// <summary>
        /// System.Double
        /// </summary>
        MarkerRefLevelOffset = 1150172,

        /// <summary>
        /// System.Double
        /// </summary>
        MarkerRefFreq = 1150173,

        /// <summary>
        /// System.String
        /// </summary>
        MarkerRefPeak = 1150174,

        /// <summary>
        /// System.String
        /// </summary>
        MarkerStorePeakList = 1150175,

        /// <summary>
        /// System.String
        /// </summary>
        MarkerFileExport = 1150176,

        /// <summary>
        /// System.Int32
        /// </summary>
        MarkerDisplayMode = 1150177,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ReferenceMarkerState = 1150178,

        /// <summary>
        /// System.Int32
        /// </summary>
        ReferenceMarkerMode = 1150179,

        /// <summary>
        /// System.Double
        /// </summary>
        ReferenceMarkerPosition = 1150180,

        /// <summary>
        /// System.Double
        /// </summary>
        ReferenceMarkerRelPosition = 1150181,

        /// <summary>
        /// System.Double
        /// </summary>
        ReferenceMarkerAmplitude = 1150182,

        /// <summary>
        /// System.Int32
        /// </summary>
        ReferenceMarkerTrace = 1150183,

        /// <summary>
        /// System.String
        /// </summary>
        ReferenceMarkerAoff = 1150184,

        /// <summary>
        /// System.String
        /// </summary>
        ReferenceMarkerPeak = 1150185,

        /// <summary>
        /// System.String
        /// </summary>
        ReferenceMarkerPeakNext = 1150186,

        /// <summary>
        /// System.String
        /// </summary>
        ReferenceMarkerPeakNextRight = 1150187,

        /// <summary>
        /// System.String
        /// </summary>
        ReferenceMarkerPeakNextLeft = 1150188,

        /// <summary>
        /// System.String
        /// </summary>
        ReferenceMarkerMin = 1150189,

        /// <summary>
        /// System.String
        /// </summary>
        ReferenceMarkerMinNext = 1150190,

        /// <summary>
        /// System.String
        /// </summary>
        ReferenceMarkerMinNextRight = 1150191,

        /// <summary>
        /// System.String
        /// </summary>
        ReferenceMarkerMinNextLeft = 1150192,

        /// <summary>
        /// System.String
        /// </summary>
        SystErr = 1150193,

        /// <summary>
        /// System.String
        /// </summary>
        SystErrList = 1150194,

        /// <summary>
        /// System.String
        /// </summary>
        SystErrClear = 1150195,

        /// <summary>
        /// System.String
        /// </summary>
        SystPreset = 1150196,

        /// <summary>
        /// System.Double
        /// </summary>
        SystSpeaker = 1150197,

        /// <summary>
        /// System.Double
        /// </summary>
        SystSpeakerPortable = 1150198,

        /// <summary>
        /// System.Double
        /// </summary>
        BeeperVolume = 1150199,

        /// <summary>
        /// System.Double
        /// </summary>
        KeyVolume = 1150200,

        /// <summary>
        /// System.Int32
        /// </summary>
        BncMode = 1150201,

        /// <summary>
        /// System.Int32
        /// </summary>
        InputImpedancePad = 1150202,

        /// <summary>
        /// System.String
        /// </summary>
        SystVersion = 1150203,

        /// <summary>
        /// System.String
        /// </summary>
        SystIdFactory = 1150204,

        /// <summary>
        /// System.Int32
        /// </summary>
        SystIdFactoryQuery = 1150205,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ApplSetupRec = 1150206,

        /// <summary>
        /// System.Int32
        /// </summary>
        IfShift = 1150207,

        /// <summary>
        /// System.Double
        /// </summary>
        IfShiftFreq = 1150208,

        /// <summary>
        /// System.Int32
        /// </summary>
        AutoOffTime = 1150209,

        /// <summary>
        /// System.String
        /// </summary>
        PowerSource = 1150210,

        /// <summary>
        /// System.Int32
        /// </summary>
        RoscSource = 1150211,

        /// <summary>
        /// System.Int32
        /// </summary>
        RoscSourceEauto = 1150212,

        /// <summary>
        /// System.Double
        /// </summary>
        RoscFrequency = 1150213,

        /// <summary>
        /// System.Int32
        /// </summary>
        RoscPllBwid = 1150214,

        /// <summary>
        /// System.Int32
        /// </summary>
        TriggerSource = 1150215,

        /// <summary>
        /// System.Double
        /// </summary>
        TriggerDelay = 1150216,

        /// <summary>
        /// System.Int32
        /// </summary>
        TriggerSlope = 1150217,

        /// <summary>
        /// System.Double
        /// </summary>
        ExternalTriggerLevel = 1150218,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ExternalTriggerDelayCompensation = 1150219,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ExternalGate = 1150220,

        /// <summary>
        /// System.Double
        /// </summary>
        ExternalGateHold = 1150221,

        /// <summary>
        /// System.Int32
        /// </summary>
        ExternalGateTriggerType = 1150222,

        /// <summary>
        /// System.Int32
        /// </summary>
        ExternalGatePolarity = 1150223,

        /// <summary>
        /// System.Double
        /// </summary>
        ExternalGateLength = 1150224,

        /// <summary>
        /// System.Int32
        /// </summary>
        ExternalGateSignalSource = 1150225,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ExternalGateTraceState = 1150226,

        /// <summary>
        /// System.Double
        /// </summary>
        ExternalGateTraceStart = 1150227,

        /// <summary>
        /// System.Double
        /// </summary>
        ExternalGateTraceStop = 1150228,

        /// <summary>
        /// System.Double
        /// </summary>
        ExternalGateTracePeriodLength = 1150229,

        /// <summary>
        /// System.String
        /// </summary>
        ExternalGateTraceComment = 1150230,

        /// <summary>
        /// System.Double
        /// </summary>
        VideoTriggerLevel = 1150231,

        /// <summary>
        /// System.Double
        /// </summary>
        TriggerIfpLevel = 1150232,

        /// <summary>
        /// System.Double
        /// </summary>
        TriggerIfpOffset = 1150233,

        /// <summary>
        /// System.Double
        /// </summary>
        TriggerIfpHysteresis = 1150234,

        /// <summary>
        /// System.Double
        /// </summary>
        TriggerTimeInterval = 1150235,

        /// <summary>
        /// System.Int32
        /// </summary>
        ExternalMixerSignalIdMode = 1150236,

        /// <summary>
        /// System.Double
        /// </summary>
        ExternalMixerConversionLossLow = 1150237,

        /// <summary>
        /// System.Double
        /// </summary>
        ExternalMixerLoPower = 1150238,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ExternalMixerEnabled = 1150239,

        /// <summary>
        /// System.Int32
        /// </summary>
        ExternalMixerHarmonic = 1150240,

        /// <summary>
        /// System.Int32
        /// </summary>
        ExternalMixerNumberOfPorts = 1150241,

        /// <summary>
        /// System.Int32
        /// </summary>
        ExternalMixerHarmonicType = 1150242,

        /// <summary>
        /// System.Int32
        /// </summary>
        ExternalMixerHarmonicBand = 1150243,

        /// <summary>
        /// System.Double
        /// </summary>
        ExternalMixerConversionLossHigh = 1150244,

        /// <summary>
        /// System.String
        /// </summary>
        ExternalMixerConversionLossTableHigh = 1150245,

        /// <summary>
        /// System.String
        /// </summary>
        ExternalMixerConversionLossTableLow = 1150246,

        /// <summary>
        /// System.Double
        /// </summary>
        ExternalMixerBiasLow = 1150247,

        /// <summary>
        /// System.Double
        /// </summary>
        ExternalMixerBiasHigh = 1150248,

        /// <summary>
        /// System.Double
        /// </summary>
        ExternalMixerThreshold = 1150249,

        /// <summary>
        /// System.Double
        /// </summary>
        GetExternalMixerRfFrequencyStart = 1150250,

        /// <summary>
        /// System.Double
        /// </summary>
        GetExternalMixerRfFrequencyStop = 1150251,

        /// <summary>
        /// System.Double
        /// </summary>
        ExternalMixerHandoverFrequency = 1150252,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ExternalMixerRfOverrange = 1150253,

        /// <summary>
        /// System.String
        /// </summary>
        ExternalMixerPresetBand = 1150254,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ExternalMixerHighHarmonicState = 1150255,

        /// <summary>
        /// System.Int32
        /// </summary>
        ExternalMixerHighHarmonic = 1150256,

        /// <summary>
        /// System.Int32
        /// </summary>
        AmplRfCoupling = 1150257,

        /// <summary>
        /// System.Boolean
        /// </summary>
        AmplEattState = 1150258,

        /// <summary>
        /// System.Boolean
        /// </summary>
        AmplEattAuto = 1150259,

        /// <summary>
        /// System.Int32
        /// </summary>
        AmplEattValue = 1150260,

        /// <summary>
        /// System.Double
        /// </summary>
        AmplMixerLevel = 1150261,

        /// <summary>
        /// System.Boolean
        /// </summary>
        AmplMixerAuto = 1150262,

        /// <summary>
        /// System.Boolean
        /// </summary>
        AmplPreamplifier = 1150263,

        /// <summary>
        /// System.String
        /// </summary>
        AmplProtectionReset = 1150264,

        /// <summary>
        /// System.Boolean
        /// </summary>
        AmplYigFilter = 1150265,

        /// <summary>
        /// System.Boolean
        /// </summary>
        LimitState = 1150266,

        /// <summary>
        /// System.String
        /// </summary>
        LimitClear = 1150267,

        /// <summary>
        /// System.Int32
        /// </summary>
        LimitAssignTrace = 1150268,

        /// <summary>
        /// System.Int32
        /// </summary>
        LimitUnits = 1150269,

        /// <summary>
        /// System.String
        /// </summary>
        LimitComment = 1150270,

        /// <summary>
        /// System.Int32
        /// </summary>
        LimitCopy = 1150271,

        /// <summary>
        /// System.String
        /// </summary>
        LimitName = 1150272,

        /// <summary>
        /// System.String
        /// </summary>
        LimitDelete = 1150273,

        /// <summary>
        /// System.String
        /// </summary>
        LimitUpperPortable = 1150274,

        /// <summary>
        /// System.String
        /// </summary>
        LimitLowPortable = 1150275,

        /// <summary>
        /// System.Boolean
        /// </summary>
        LimitAudioBeep = 1150276,

        /// <summary>
        /// System.Int32
        /// </summary>
        LimitControlDomain = 1150277,

        /// <summary>
        /// System.Int32
        /// </summary>
        LimitControlSpacing = 1150278,

        /// <summary>
        /// System.Int32
        /// </summary>
        LimitControlScaling = 1150279,

        /// <summary>
        /// System.Double
        /// </summary>
        LimitControlOffset = 1150280,

        /// <summary>
        /// System.Double
        /// </summary>
        LimitControlShift = 1150281,

        /// <summary>
        /// System.Boolean
        /// </summary>
        LimitLowerState = 1150282,

        /// <summary>
        /// System.Int32
        /// </summary>
        LimitLowerScaling = 1150283,

        /// <summary>
        /// System.Int32
        /// </summary>
        LimitLowerSpacing = 1150284,

        /// <summary>
        /// System.Double
        /// </summary>
        LimitLowerMargin = 1150285,

        /// <summary>
        /// System.Double
        /// </summary>
        LimitLowerThreshold = 1150286,

        /// <summary>
        /// System.Double
        /// </summary>
        LimitLowerOffset = 1150287,

        /// <summary>
        /// System.Double
        /// </summary>
        LimitLowerShift = 1150288,

        /// <summary>
        /// System.Boolean
        /// </summary>
        LimitUpperState = 1150289,

        /// <summary>
        /// System.Int32
        /// </summary>
        LimitUpperScaling = 1150290,

        /// <summary>
        /// System.Int32
        /// </summary>
        LimitUpperSpacing = 1150291,

        /// <summary>
        /// System.Double
        /// </summary>
        LimitUpperMargin = 1150292,

        /// <summary>
        /// System.Double
        /// </summary>
        LimitUpperThreshold = 1150293,

        /// <summary>
        /// System.Double
        /// </summary>
        LimitUpperOffset = 1150294,

        /// <summary>
        /// System.Double
        /// </summary>
        LimitUpperShift = 1150295,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ThrlineState = 1150296,

        /// <summary>
        /// System.Double
        /// </summary>
        ThrlinePosition = 1150297,

        /// <summary>
        /// System.Boolean
        /// </summary>
        DlineState = 1150298,

        /// <summary>
        /// System.Double
        /// </summary>
        DlinePosition = 1150299,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FlineState = 1150300,

        /// <summary>
        /// System.Double
        /// </summary>
        FlinePosition = 1150301,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TlineState = 1150302,

        /// <summary>
        /// System.Double
        /// </summary>
        TlinePosition = 1150303,

        /// <summary>
        /// System.Double
        /// </summary>
        DispRefValue = 1150304,

        /// <summary>
        /// System.Double
        /// </summary>
        DispRefPosition = 1150305,

        /// <summary>
        /// System.Boolean
        /// </summary>
        DispFpKeys = 1150306,

        /// <summary>
        /// System.String
        /// </summary>
        SelectWindow = 1150307,

        /// <summary>
        /// System.Boolean
        /// </summary>
        DispFreqState = 1150308,

        /// <summary>
        /// System.Boolean
        /// </summary>
        DispLogoState = 1150309,

        /// <summary>
        /// System.Boolean
        /// </summary>
        DispTimeState = 1150310,

        /// <summary>
        /// System.String
        /// </summary>
        DispComment = 1150311,

        /// <summary>
        /// System.Boolean
        /// </summary>
        DispCommentState = 1150312,

        /// <summary>
        /// System.Boolean
        /// </summary>
        DispPwrSaveState = 1150313,

        /// <summary>
        /// System.Int32
        /// </summary>
        DispPwrSaveHoldoff = 1150314,

        /// <summary>
        /// System.Int32
        /// </summary>
        DispFormat = 1150315,

        /// <summary>
        /// System.Int32
        /// </summary>
        DispWindowSize = 1150316,

        /// <summary>
        /// System.String
        /// </summary>
        DispColPreset = 1150317,

        /// <summary>
        /// System.String
        /// </summary>
        DispColPresPortable = 1150318,

        /// <summary>
        /// System.Int32
        /// </summary>
        DispColPortable = 1150319,

        /// <summary>
        /// System.Int32
        /// </summary>
        DispColPredefined = 1150320,

        /// <summary>
        /// System.Double
        /// </summary>
        DispLogRange = 1150321,

        /// <summary>
        /// System.Int32
        /// </summary>
        DispAmplitudeGridMode = 1150322,

        /// <summary>
        /// System.Double
        /// </summary>
        DispAmplitudeMinLevel = 1150323,

        /// <summary>
        /// System.Double
        /// </summary>
        DispAmplitudeMaxLevel = 1150324,

        /// <summary>
        /// System.Boolean
        /// </summary>
        DispSingleSweep = 1150325,

        /// <summary>
        /// System.Boolean
        /// </summary>
        DispUpdate = 1150326,

        /// <summary>
        /// System.Double
        /// </summary>
        DispBrightness = 1150327,

        /// <summary>
        /// System.Int32
        /// </summary>
        QueryActiveWindow = 1150328,

        /// <summary>
        /// System.Int32
        /// </summary>
        UnitPower = 1150329,

        /// <summary>
        /// System.Int32
        /// </summary>
        UnitLength = 1150330,

        /// <summary>
        /// System.Boolean
        /// </summary>
        DispRefValueAuto = 1150331,

        /// <summary>
        /// System.String
        /// </summary>
        DispTraceYAuto = 1150332,

        /// <summary>
        /// System.String
        /// </summary>
        DispTracYAutoPortable = 1150333,

        /// <summary>
        /// System.Double
        /// </summary>
        DispYAxisScaling = 1150334,

        /// <summary>
        /// System.Boolean
        /// </summary>
        DispStatusBar = 1150335,

        /// <summary>
        /// System.Boolean
        /// </summary>
        DispSoftkeyBar = 1150336,

        /// <summary>
        /// System.Boolean
        /// </summary>
        DispToolBar = 1150337,

        /// <summary>
        /// System.Int32
        /// </summary>
        DispTouchscreenState = 1150338,

        /// <summary>
        /// System.String
        /// </summary>
        DispSelectTheme = 1150339,

        /// <summary>
        /// System.Int32
        /// </summary>
        DispMarkerTable = 1150340,

        /// <summary>
        /// System.Boolean
        /// </summary>
        DisplayState = 1150341,

        /// <summary>
        /// System.Int32
        /// </summary>
        DispDateFormat = 1150342,

        /// <summary>
        /// System.String
        /// </summary>
        DispScreenshot = 1150343,

        /// <summary>
        /// System.Boolean
        /// </summary>
        DisplayZoomState = 1150344,

        /// <summary>
        /// System.String
        /// </summary>
        HcopyAbort = 1150345,

        /// <summary>
        /// System.Int32
        /// </summary>
        HcopyColorDef = 1150346,

        /// <summary>
        /// System.Boolean
        /// </summary>
        HcopyColor = 1150347,

        /// <summary>
        /// System.String
        /// </summary>
        HcopyPrint = 1150348,

        /// <summary>
        /// System.String
        /// </summary>
        HcopyPrintNext = 1150349,

        /// <summary>
        /// System.Int32
        /// </summary>
        HcopyDeviceDestination = 1150350,

        /// <summary>
        /// System.Int32
        /// </summary>
        HcopyColorPredefined = 1150351,

        /// <summary>
        /// System.Int32
        /// </summary>
        HcopyDeviceLangOutForm = 1150352,

        /// <summary>
        /// System.String
        /// </summary>
        HcopyPrintScreen = 1150353,

        /// <summary>
        /// System.Boolean
        /// </summary>
        HcopyPrintTab = 1150354,

        /// <summary>
        /// System.String
        /// </summary>
        HcopyCommScr = 1150355,

        /// <summary>
        /// System.Boolean
        /// </summary>
        HcopyPrintTrace = 1150356,

        /// <summary>
        /// System.Int32
        /// </summary>
        HcopyDeviceOrientation = 1150357,

        /// <summary>
        /// System.String
        /// </summary>
        HcopyFileName = 1150358,

        /// <summary>
        /// System.String
        /// </summary>
        HcopyPrinter = 1150359,

        /// <summary>
        /// System.String
        /// </summary>
        HcopyPrinterNext = 1150360,

        /// <summary>
        /// System.String
        /// </summary>
        HcopyPrinterFirst = 1150361,

        /// <summary>
        /// System.Boolean
        /// </summary>
        HcopyTimeDate = 1150362,

        /// <summary>
        /// System.String
        /// </summary>
        FileRecall = 1150363,

        /// <summary>
        /// System.String
        /// </summary>
        FileStartupRecall = 1150364,

        /// <summary>
        /// System.String
        /// </summary>
        FileSave = 1150365,

        /// <summary>
        /// System.String
        /// </summary>
        FileSaveNext = 1150366,

        /// <summary>
        /// System.String
        /// </summary>
        FileDataClear = 1150367,

        /// <summary>
        /// System.String
        /// </summary>
        FileDataClearAll = 1150368,

        /// <summary>
        /// System.String
        /// </summary>
        FileEditComm = 1150369,

        /// <summary>
        /// System.Int32
        /// </summary>
        FileDecSeparator = 1150370,

        /// <summary>
        /// System.String
        /// </summary>
        FileManagerEditPath = 1150371,

        /// <summary>
        /// System.String
        /// </summary>
        FileManagerDelete = 1150372,

        /// <summary>
        /// System.String
        /// </summary>
        FileManagerFormatDisk = 1150373,

        /// <summary>
        /// System.String
        /// </summary>
        FileManagerMakeDir = 1150374,

        /// <summary>
        /// System.String
        /// </summary>
        FileManagerEditPathDevice = 1150375,

        /// <summary>
        /// System.String
        /// </summary>
        FileManagerDeleteDir = 1150376,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FileItemsSaveRecalSelSourceCalData = 1150377,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FileItemsSaveRecalSelTran = 1150378,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FileItemsSaveRecalSelHwsettings = 1150379,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FileItemsSaveRecalSelTrac = 1150380,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FileItemsSaveRecalSelLine = 1150381,

        /// <summary>
        /// System.String
        /// </summary>
        FileItemsSaveRecalAll = 1150382,

        /// <summary>
        /// System.String
        /// </summary>
        FileItemsSaveRecalNone = 1150383,

        /// <summary>
        /// System.String
        /// </summary>
        FileItemsSaveRecalDefault = 1150384,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FileItemsSaveRecalFinal = 1150385,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FileItemsSaveRecalClist = 1150386,

        /// <summary>
        /// System.String
        /// </summary>
        NetworkDisconnectDrive = 1150387,

        /// <summary>
        /// System.String
        /// </summary>
        UnusedNetworkDrives = 1150388,

        /// <summary>
        /// System.String
        /// </summary>
        UsedNetworkDrives = 1150389,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ShowNetworkFolderInfo = 1150390,

        /// <summary>
        /// System.Int32
        /// </summary>
        ServiceInputSource = 1150391,

        /// <summary>
        /// System.Double
        /// </summary>
        ServiceInputLevel = 1150392,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ServiceInputPulsedState = 1150393,

        /// <summary>
        /// System.Double
        /// </summary>
        ServiceInputPulsedRate = 1150394,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ServiceInputRectangleState = 1150395,

        /// <summary>
        /// System.Double
        /// </summary>
        ServiceInputRectangleRate = 1150396,

        /// <summary>
        /// System.Int32
        /// </summary>
        ServiceInputCombFrequency = 1150397,

        /// <summary>
        /// System.Int32
        /// </summary>
        ServiceBasebandSignalSource = 1150398,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ServiceNoiseSource = 1150399,

        /// <summary>
        /// System.Double
        /// </summary>
        ServiceDcCalVoltage = 1150400,

        /// <summary>
        /// System.Int32
        /// </summary>
        ServicePulseCalDestination = 1150401,

        /// <summary>
        /// System.Double
        /// </summary>
        ServicePulseCalFreq = 1150402,

        /// <summary>
        /// System.Double
        /// </summary>
        ServicePulseCalFreqFsv = 1150403,

        /// <summary>
        /// System.String
        /// </summary>
        ServiceHwInfo = 1150404,

        /// <summary>
        /// System.String
        /// </summary>
        ServiceStestResults = 1150405,

        /// <summary>
        /// System.Int32
        /// </summary>
        ServiceStest = 1150406,

        /// <summary>
        /// System.Int32
        /// </summary>
        ServiceRoscIntTune = 1150407,

        /// <summary>
        /// System.String
        /// </summary>
        ServiceRoscIntTuneSave = 1150408,

        /// <summary>
        /// System.Int32
        /// </summary>
        LimitCheckResult = 1150409,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MarkerNoiseState = 1150410,

        /// <summary>
        /// System.Double
        /// </summary>
        MarkerNoiseResult = 1150411,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ReferenceMarkerPnoState = 1150412,

        /// <summary>
        /// System.Double
        /// </summary>
        ReferenceMarkerPnoResult = 1150413,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ReferenceMarkerPnoAuto = 1150414,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MarkerFrequencyCounterEnabled = 1150415,

        /// <summary>
        /// System.Double
        /// </summary>
        MarkerFrequencyCounterResolution = 1150416,

        /// <summary>
        /// System.Double
        /// </summary>
        MarkerCount = 1150417,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MarkerNdbState = 1150418,

        /// <summary>
        /// System.Double
        /// </summary>
        MarkerNdbVal = 1150419,

        /// <summary>
        /// System.Double
        /// </summary>
        MarkerNdbResult = 1150420,

        /// <summary>
        /// System.Double
        /// </summary>
        MarkerNdbQFactor = 1150421,

        /// <summary>
        /// System.Int32
        /// </summary>
        ListPowType = 1150422,

        /// <summary>
        /// System.String
        /// </summary>
        ListPowStateOff = 1150423,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MeasTdomState = 1150424,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MeasTdomPeak = 1150425,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MeasTdomRms = 1150426,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MeasTdomMean = 1150427,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MeasTdomSdev = 1150428,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MeasTdomAvg = 1150429,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MeasTdomMaxHold = 1150430,

        /// <summary>
        /// System.Int32
        /// </summary>
        MeasTdomMode = 1150431,

        /// <summary>
        /// System.String
        /// </summary>
        MeasTdomAoff = 1150432,

        /// <summary>
        /// System.String
        /// </summary>
        MeasTdomSetReference = 1150433,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasTdomPeakResult = 1150434,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasTdomRmsResult = 1150435,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasTdomMeanResult = 1150436,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasTdomSdevResult = 1150437,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasTdomAvgPeakResult = 1150438,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasTdomAvgRmsResult = 1150439,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasTdomAvgMeanResult = 1150440,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasTdomAvgSdevResult = 1150441,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasTdomPeakHoldResult = 1150442,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasTdomPeakHoldRmsResult = 1150443,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasTdomPeakHoldMeanResult = 1150444,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasTdomPeakHoldSdevResult = 1150445,

        /// <summary>
        /// System.Int32
        /// </summary>
        MeasPowSelect = 1150446,

        /// <summary>
        /// System.String
        /// </summary>
        MeasPowOff = 1150447,

        /// <summary>
        /// System.Boolean
        /// </summary>
        PowerMeasurementState = 1150448,

        /// <summary>
        /// System.Int32
        /// </summary>
        MeasPowAdjNum = 1150449,

        /// <summary>
        /// System.Int32
        /// </summary>
        MeasPowAdjMode = 1150450,

        /// <summary>
        /// System.Int32
        /// </summary>
        MeasPowMode = 1150451,

        /// <summary>
        /// System.Int32
        /// </summary>
        MeasPowModePortable = 1150452,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MeasPowResultMode = 1150453,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasPowChannelSpacing = 1150454,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasPowAdjChannelSpacing = 1150455,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasPowAltChannelSpacing = 1150456,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasPowChannelBandwidth = 1150457,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasPowChannelBandwidthPortable = 1150458,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasPowAdjChannelBandwidth = 1150459,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasPowAltChannelBandwidth = 1150460,

        /// <summary>
        /// System.Int32
        /// </summary>
        MeasPowChannelUnit = 1150461,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasChanPowBandwidth = 1150462,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MeasPowNoiseCorr = 1150463,

        /// <summary>
        /// System.Int32
        /// </summary>
        MeasPowTrace = 1150464,

        /// <summary>
        /// System.Int32
        /// </summary>
        MeasPowStandard = 1150465,

        /// <summary>
        /// System.String
        /// </summary>
        MeasPowRefValue = 1150466,

        /// <summary>
        /// System.Int32
        /// </summary>
        MeasPowAdjPreset = 1150467,

        /// <summary>
        /// System.Int32
        /// </summary>
        MeasPowStandardCheck = 1150468,

        /// <summary>
        /// System.String
        /// </summary>
        AcpUserStdSet = 1150469,

        /// <summary>
        /// System.String
        /// </summary>
        AcpUserStdSave = 1150470,

        /// <summary>
        /// System.String
        /// </summary>
        AcpUserStdDelete = 1150471,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MeasAcpHsp = 1150472,

        /// <summary>
        /// System.String
        /// </summary>
        MeasAcpPresetRefLevel = 1150473,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasAcpPresetRefLevelOffset = 1150474,

        /// <summary>
        /// System.String
        /// </summary>
        MeasAcpPresetRefLevelAdj = 1150475,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MeasAcpLimitState = 1150476,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MeasAcpLimitRelState = 1150477,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MeasAcpLimitAbsState = 1150478,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MeasAcpAltLimitRelState = 1150479,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MeasAcpAltLimitAbsState = 1150480,

        /// <summary>
        /// System.Int32
        /// </summary>
        MeasPowCarrSigNum = 1150481,

        /// <summary>
        /// System.Int32
        /// </summary>
        MeasPowRefChanSelAuto = 1150482,

        /// <summary>
        /// System.Int32
        /// </summary>
        MeasPowAdjRefTxchannel = 1150483,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasPowBandwidth = 1150484,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasPowBandwidthPortable = 1150485,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MeasStatApdState = 1150486,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MeasStatCcdfState = 1150487,

        /// <summary>
        /// System.Int32
        /// </summary>
        MeasStatSamples = 1150488,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasStatXRef = 1150489,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasStatXRange = 1150490,

        /// <summary>
        /// System.Int32
        /// </summary>
        MeasStatYUnit = 1150491,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasStatYMin = 1150492,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasStatYMax = 1150493,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasStatMeanPowerPos = 1150494,

        /// <summary>
        /// System.String
        /// </summary>
        MeasStatAdj = 1150495,

        /// <summary>
        /// System.String
        /// </summary>
        MeasStatRefLevelAdjust = 1150496,

        /// <summary>
        /// System.String
        /// </summary>
        MeasStatPreset = 1150497,

        /// <summary>
        /// System.Double
        /// </summary>
        StatResult = 1150498,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MeasMdepthState = 1150499,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasMdepthResult = 1150500,

        /// <summary>
        /// System.String
        /// </summary>
        MeasMdepthSearch = 1150501,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MeasToiState = 1150502,

        /// <summary>
        /// System.Double
        /// </summary>
        MeasToiResult = 1150503,

        /// <summary>
        /// System.String
        /// </summary>
        MeasToiSearch = 1150504,

        /// <summary>
        /// System.Int32
        /// </summary>
        MeasToiPositioning = 1150505,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MeasHdistState = 1150506,

        /// <summary>
        /// System.Int32
        /// </summary>
        MeasHdistNoofharm = 1150507,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MeasHdistRbwauto = 1150508,

        /// <summary>
        /// System.String
        /// </summary>
        MeasHdistPreset = 1150509,

        /// <summary>
        /// System.Double
        /// </summary>
        MpowMin = 1150510,

        /// <summary>
        /// System.Int32
        /// </summary>
        MpowFtype = 1150511,

        /// <summary>
        /// System.Double
        /// </summary>
        SePeakSearchMargin = 1150512,

        /// <summary>
        /// System.Int32
        /// </summary>
        SePeakSearchPeaks = 1150513,

        /// <summary>
        /// System.Double
        /// </summary>
        SeListRangStart = 1150514,

        /// <summary>
        /// System.Double
        /// </summary>
        SeListRangStop = 1150515,

        /// <summary>
        /// System.Int32
        /// </summary>
        SeListRangFiltType = 1150516,

        /// <summary>
        /// System.Double
        /// </summary>
        SeListRangBand = 1150517,

        /// <summary>
        /// System.Double
        /// </summary>
        SeListRangBandVid = 1150518,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SeListRangSweTimeAuto = 1150519,

        /// <summary>
        /// System.Double
        /// </summary>
        SeListRangSweTime = 1150520,

        /// <summary>
        /// System.Int32
        /// </summary>
        SeListRangDet = 1150521,

        /// <summary>
        /// System.Double
        /// </summary>
        SeListRangRlev = 1150522,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SeListRangInpAttAuto = 1150523,

        /// <summary>
        /// System.Double
        /// </summary>
        SeListRangInpAtt = 1150524,

        /// <summary>
        /// System.Int32
        /// </summary>
        SeListRangPoints = 1150525,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SeListRangBreak = 1150526,

        /// <summary>
        /// System.String
        /// </summary>
        SeListRangTran = 1150527,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SeListRangInpGainStat = 1150528,

        /// <summary>
        /// System.String
        /// </summary>
        SeListRangDel = 1150529,

        /// <summary>
        /// System.Int32
        /// </summary>
        SeListRangCount = 1150530,

        /// <summary>
        /// System.String
        /// </summary>
        SePeakSearch = 1150531,

        /// <summary>
        /// System.String
        /// </summary>
        SeStoreFile = 1150532,

        /// <summary>
        /// System.Double
        /// </summary>
        SeListRangLimit = 1150533,

        /// <summary>
        /// System.Double
        /// </summary>
        SeListRangLimitStart = 1150534,

        /// <summary>
        /// System.Double
        /// </summary>
        SeListRangLimitStop = 1150535,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SeListRangLimitState = 1150536,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SeListEvalState = 1150537,

        /// <summary>
        /// System.Int32
        /// </summary>
        IqDataFormat = 1150538,

        /// <summary>
        /// System.Double
        /// </summary>
        IqSampleRate = 1150539,

        /// <summary>
        /// System.Int32
        /// </summary>
        IqExtBand = 1150540,

        /// <summary>
        /// System.Boolean
        /// </summary>
        IqDataState = 1150541,

        /// <summary>
        /// System.Boolean
        /// </summary>
        IqStartSynchronization = 1150542,

        /// <summary>
        /// System.Boolean
        /// </summary>
        IqDataWbandState = 1150543,

        /// <summary>
        /// System.Boolean
        /// </summary>
        IqDataAverState = 1150544,

        /// <summary>
        /// System.Int32
        /// </summary>
        IqDataAverCount = 1150545,

        /// <summary>
        /// System.Boolean
        /// </summary>
        IqDataGateState = 1150546,

        /// <summary>
        /// System.Int32
        /// </summary>
        IqDataGateType = 1150547,

        /// <summary>
        /// System.Int32
        /// </summary>
        IqDataGateLength = 1150548,

        /// <summary>
        /// System.Int32
        /// </summary>
        IqDataGateGap = 1150549,

        /// <summary>
        /// System.Int32
        /// </summary>
        IqDataGateNumberOfPeriods = 1150550,

        /// <summary>
        /// System.Int32
        /// </summary>
        IqDataDisplayType = 1150551,

        /// <summary>
        /// System.Double
        /// </summary>
        IqDataFilterBandwidth = 1150552,

        /// <summary>
        /// System.String
        /// </summary>
        EspRestore = 1150553,

        /// <summary>
        /// System.Int32
        /// </summary>
        EspMode = 1150554,

        /// <summary>
        /// System.String
        /// </summary>
        EspPeakSearch = 1150555,

        /// <summary>
        /// System.Double
        /// </summary>
        EspManVal = 1150556,

        /// <summary>
        /// System.Double
        /// </summary>
        EspQueryX = 1150557,

        /// <summary>
        /// System.Double
        /// </summary>
        EspQueryY = 1150558,

        /// <summary>
        /// System.Double
        /// </summary>
        EspTran = 1150559,

        /// <summary>
        /// System.String
        /// </summary>
        LimStoreFile = 1150560,

        /// <summary>
        /// System.Int32
        /// </summary>
        EspAnalysis = 1150561,

        /// <summary>
        /// System.Double
        /// </summary>
        SemPeakSearchMargin = 1150562,

        /// <summary>
        /// System.Int32
        /// </summary>
        SemPeakSearchPeaks = 1150563,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SemListEvalState = 1150564,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SemMarkAllPeaks = 1150565,

        /// <summary>
        /// System.Int32
        /// </summary>
        SemPowerRefType = 1150566,

        /// <summary>
        /// System.Double
        /// </summary>
        SemBandwidth = 1150567,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SemRrcFilterState = 1150568,

        /// <summary>
        /// System.Double
        /// </summary>
        SemRrcFilterAlphaValue = 1150569,

        /// <summary>
        /// System.Int32
        /// </summary>
        SemRefRangePosition = 1150570,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SemFastSem = 1150571,

        /// <summary>
        /// System.String
        /// </summary>
        SemPresetStandard = 1150572,

        /// <summary>
        /// System.String
        /// </summary>
        SemRestoreStandard = 1150573,

        /// <summary>
        /// System.String
        /// </summary>
        SemStorePreset = 1150574,

        /// <summary>
        /// System.String
        /// </summary>
        SemListFileExport = 1150575,

        /// <summary>
        /// System.Double
        /// </summary>
        SemRangeBandwidth = 1150576,

        /// <summary>
        /// System.Double
        /// </summary>
        SemRangeVbw = 1150577,

        /// <summary>
        /// System.Double
        /// </summary>
        SemRangeFreqStart = 1150578,

        /// <summary>
        /// System.Double
        /// </summary>
        SemRangeFreqStop = 1150579,

        /// <summary>
        /// System.Double
        /// </summary>
        SemRangeStartFreqAbsLimit = 1150580,

        /// <summary>
        /// System.Double
        /// </summary>
        SemRangeStopFreqAbsLimit = 1150581,

        /// <summary>
        /// System.Double
        /// </summary>
        SemRangeStartFreqRelLimit = 1150582,

        /// <summary>
        /// System.Double
        /// </summary>
        SemRangeStopFreqRelLimit = 1150583,

        /// <summary>
        /// System.Int32
        /// </summary>
        SemRangeLimitCheckType = 1150584,

        /// <summary>
        /// System.Int32
        /// </summary>
        SemRangeFilterType = 1150585,

        /// <summary>
        /// System.Double
        /// </summary>
        SemRangeInputAttenuation = 1150586,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SemRangeInputAttenuationAuto = 1150587,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SemRangePreamplifier = 1150588,

        /// <summary>
        /// System.Double
        /// </summary>
        SemRangeRefLevel = 1150589,

        /// <summary>
        /// System.Double
        /// </summary>
        SemRangeSweepTime = 1150590,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SemRangeSweepTimeAuto = 1150591,

        /// <summary>
        /// System.String
        /// </summary>
        SemRangeTransducer = 1150592,

        /// <summary>
        /// System.Int32
        /// </summary>
        SemRangeCount = 1150593,

        /// <summary>
        /// System.Int32
        /// </summary>
        SemInsertNewRange = 1150594,

        /// <summary>
        /// System.String
        /// </summary>
        SemDeleteRange = 1150595,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SemPowerClassUsed = 1150596,

        /// <summary>
        /// System.Double
        /// </summary>
        SemPowerClassLowerLevelLimit = 1150597,

        /// <summary>
        /// System.Double
        /// </summary>
        SemPowerClassUpperLevelLimit = 1150598,

        /// <summary>
        /// System.Int32
        /// </summary>
        SemPowerClassLimitState = 1150599,

        /// <summary>
        /// System.Int32
        /// </summary>
        SemPowerClassCount = 1150600,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SemWeightFilterAllState = 1150601,

        /// <summary>
        /// System.Double
        /// </summary>
        SemWeightFilterAllAlpha = 1150602,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SemWeightFilterAchState = 1150603,

        /// <summary>
        /// System.Double
        /// </summary>
        SemWeightFilterAchAlpha = 1150604,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SemWeightFilterAltState = 1150605,

        /// <summary>
        /// System.Double
        /// </summary>
        SemWeightFilterAltAlpha = 1150606,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SemWeightFilterTxChannelState = 1150607,

        /// <summary>
        /// System.Double
        /// </summary>
        SemWeightFilterTxChannelAlpha = 1150608,

        /// <summary>
        /// System.Double
        /// </summary>
        TdmaPowMeasTime = 1150609,

        /// <summary>
        /// System.String
        /// </summary>
        ReceiverMode = 1150610,

        /// <summary>
        /// System.Double
        /// </summary>
        AddReceiverFrequency = 1150611,

        /// <summary>
        /// System.Int32
        /// </summary>
        ReceiverPeaksPerSubrange = 1150612,

        /// <summary>
        /// System.Int32
        /// </summary>
        ReceiverPeakListDisplayState = 1150613,

        /// <summary>
        /// System.Int32
        /// </summary>
        ReceiverPeakSearchMethod = 1150614,

        /// <summary>
        /// System.Int32
        /// </summary>
        ReceiverNumberOfPeaksSubranges = 1150615,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ReceiverFinalMeasAuto = 1150616,

        /// <summary>
        /// System.Double
        /// </summary>
        ReceiverFinalMeasTime = 1150617,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ReceiverPeakSearchAfterPrescan = 1150618,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ReceiverPreHighPassFilter = 1150619,

        /// <summary>
        /// System.Int32
        /// </summary>
        ReceiverPreVNetworkType = 1150620,

        /// <summary>
        /// System.Int32
        /// </summary>
        ReceiverPresVNetworkPhase = 1150621,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ReceiverFinalMeasHighPassFilter = 1150622,

        /// <summary>
        /// System.Int32
        /// </summary>
        ReceiverFinalMeasVNetworkType = 1150623,

        /// <summary>
        /// System.Int32
        /// </summary>
        ReceiverFinalMeasVNetworkPhase = 1150624,

        /// <summary>
        /// System.Int32
        /// </summary>
        ReceiverDemodType = 1150625,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ReceiverDemodSquelchState = 1150626,

        /// <summary>
        /// System.Int32
        /// </summary>
        ReceiverFinalMeasDetectorType = 1150627,

        /// <summary>
        /// System.Double
        /// </summary>
        ReceiverMinBargraphLevel = 1150628,

        /// <summary>
        /// System.Double
        /// </summary>
        ReceiverMaxBargraphLevel = 1150629,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ReceiverBargraphMaxhold = 1150630,

        /// <summary>
        /// System.String
        /// </summary>
        ReceiverBargraphMaxholdReset = 1150631,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ReceiverProtectionState = 1150632,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ReceiverAutoPreamplifier = 1150633,

        /// <summary>
        /// System.Int32
        /// </summary>
        ReceiverTraceFeedControl = 1150634,

        /// <summary>
        /// System.Int32
        /// </summary>
        ReceiverTracePoints = 1150635,

        /// <summary>
        /// System.Int32
        /// </summary>
        ReceiverStepMode = 1150636,

        /// <summary>
        /// System.Int32
        /// </summary>
        ReceiverScanRanges = 1150637,

        /// <summary>
        /// System.Double
        /// </summary>
        ReceiverScanStart = 1150638,

        /// <summary>
        /// System.Double
        /// </summary>
        ReceiverScanStop = 1150639,

        /// <summary>
        /// System.Double
        /// </summary>
        ReceiverScanStepSize = 1150640,

        /// <summary>
        /// System.Double
        /// </summary>
        ReceiverScanBandwidth = 1150641,

        /// <summary>
        /// System.Double
        /// </summary>
        ReceiverScanMeasurementTime = 1150642,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ReceiverScanAutoRanging = 1150643,

        /// <summary>
        /// System.Double
        /// </summary>
        ReceiverScanRfAttenuation = 1150644,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ReceiverScanPreamplifier = 1150645,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ReceiverScanAutoPreamplifier = 1150646,

        /// <summary>
        /// System.String
        /// </summary>
        SsaPhaseMode = 1150647,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaMeasConf = 1150648,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaPhaseNoiseMeas = 1150649,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaFreqMulti = 1150650,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaExpectedPower = 1150651,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaSweepForward = 1150652,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaVerifyFreq = 1150653,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaFreqTol = 1150654,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaLevelTol = 1150655,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaTrackFreq = 1150656,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaTrackLevel = 1150657,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaAfcState = 1150658,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaAfcTargetFreq = 1150659,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaAfcTol = 1150660,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaAfcIter = 1150661,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaAutoScaling = 1150662,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaYAxisTop = 1150663,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaYAxisRange = 1150664,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaTraceOffset = 1150665,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaSmoothingState = 1150666,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaTraceSmoothingAper = 1150667,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaSmoothingType = 1150668,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaCalcEval = 1150669,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaCalcEvalStart = 1150670,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaCalcEvalStop = 1150671,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaPreampAndAttenAuto = 1150672,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaInputImp = 1150673,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaCalcSnoState = 1150674,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaCalcSnoX = 1150675,

        /// <summary>
        /// System.String
        /// </summary>
        SsaCalcSnoAoff = 1150676,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaGetSpotNoiseYPos = 1150677,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaIntGenState = 1150678,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaIntGenFreq = 1150679,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaIntGenOutpLevel = 1150680,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaPhaseDetDisp = 1150681,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaSweepMode = 1150682,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaFftWindow = 1150683,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaListRangBand = 1150684,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaListRangSweCoun = 1150685,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaListRangFiltType = 1150686,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaTunMeasPoints = 1150687,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaSettlTimeDcPort = 1150688,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaSettlTimeTunPort = 1150689,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaBandRat = 1150690,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaSweepCount = 1150691,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaBandFft = 1150692,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaXoptimize = 1150693,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaTracking = 1150694,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaPulsedFreqMeasMode = 1150695,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaPulsedDutyCycle = 1150696,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaPulsedRepetFreq = 1150697,

        /// <summary>
        /// System.String
        /// </summary>
        SsaRefMeas = 1150698,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaFetchPhaseNoiseResults = 1150699,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaVcoMeasType = 1150700,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaAutoCarrierSearch = 1150701,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaCarrierSearchFreqStar = 1150702,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaCarrierSearchFreqStop = 1150703,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaHarmonics = 1150704,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaVcoYAxisAutoscale = 1150705,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaVcoYAxisTop = 1150706,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaVcoYAxisRange = 1150707,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaVcoYAxisOffs = 1150708,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaVcoXAxisUnit = 1150709,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaVcoPushState = 1150710,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaVcoPushScal = 1150711,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaVcoPullState = 1150712,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaVcoPullScal = 1150713,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaVcoFreqCountCoupl = 1150714,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaVcoFreqCountRes = 1150715,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaPushMeasPoints = 1150716,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaVcoSweepTime = 1150717,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaTransMeasType = 1150718,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaTransYAxisAutoscale = 1150719,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaTransYAxisTop = 1150720,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaTransYAxisRange = 1150721,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaTransYAxisOffs = 1150722,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaTransUnit = 1150723,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaDemodBand = 1150724,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaRelLpassFilt = 1150725,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaMeasTime = 1150726,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaTrigMode = 1150727,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaTrigLevel = 1150728,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaDemodTrigLev = 1150729,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaSquelchState = 1150730,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaSquelchLevel = 1150731,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaZeroPhaseRef = 1150732,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaDemodSummRes = 1150733,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaFreqErrRes = 1150734,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaCarrPowRes = 1150735,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaDcPowState = 1150736,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaDcPortState = 1150737,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaDcPortVolt = 1150738,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaDcPortVoltMax = 1150739,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaDcPortVoltMin = 1150740,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaDcPortCurrMax = 1150741,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaDcPortPushVolt = 1150742,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaDcTunPortState = 1150743,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaDcTunPortVolt = 1150744,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaDcTunPortVoltMax = 1150745,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaDcTunPortVoltMin = 1150746,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaDcAuxPortState = 1150747,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaDcAuxPortVolt = 1150748,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaDcAuxPortCurrMax = 1150749,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaDcSwitchSeqState = 1150750,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaUseDcPort = 1150751,

        /// <summary>
        /// System.String
        /// </summary>
        SsaStartPremeas = 1150752,

        /// <summary>
        /// System.String
        /// </summary>
        SsaPremeasNext = 1150753,

        /// <summary>
        /// System.String
        /// </summary>
        SsaPremeasSkip = 1150754,

        /// <summary>
        /// System.String
        /// </summary>
        SsaPremeasFin = 1150755,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaDutPremeasState = 1150756,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaDutDrift = 1150757,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaDutTunRange = 1150758,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaDutSlope = 1150759,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaPdetPremeasState = 1150760,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaPdetSlope = 1150761,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaPdetLevel = 1150762,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaPdetOffs = 1150763,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaPdetHarmState = 1150764,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaPdetHarm = 1150765,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaLpnSlopeRes = 1150766,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaLpnLevelRes = 1150767,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaLpnDcOffsetRes = 1150768,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaGenSlope = 1150769,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaPllBandAuto = 1150770,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaPllBand = 1150771,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaPdetVoltBoth = 1150772,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaPdetVoltMax = 1150773,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaPdetVoltMin = 1150774,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaPdetVoltRes = 1150775,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaPdetChar = 1150776,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaAnoiseMeasStat = 1150777,

        /// <summary>
        /// System.String
        /// </summary>
        SsaAnoiseOpointPremeas = 1150778,

        /// <summary>
        /// System.String
        /// </summary>
        SsaAnoiseNormPremeas = 1150779,

        /// <summary>
        /// System.String
        /// </summary>
        SsaAdetCfile = 1150780,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaAdetExtLnaGain = 1150781,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaAdetMindex = 1150782,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaAdetGain = 1150783,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaAdetLevelRes = 1150784,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaSpursDispMode = 1150785,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaSpursHighlight = 1150786,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaSpursEnab = 1150787,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaSpursFundFreq = 1150788,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaSpursCount = 1150789,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaSpursThr = 1150790,

        /// <summary>
        /// System.String
        /// </summary>
        SsaStoreSpursList = 1150791,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaAutoLnaGain = 1150792,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaLnaGain = 1150793,

        /// <summary>
        /// System.String
        /// </summary>
        SsaAutoScalOnce = 1150794,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaDeltaMarkerPosOnSpur = 1150795,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaMarkerPosOnSpur = 1150796,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaQueryLimitLineComp = 1150797,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SsaAutoLevel = 1150798,

        /// <summary>
        /// System.Double
        /// </summary>
        SsaSignalLevel = 1150799,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaQuerySweepPoints = 1150800,

        /// <summary>
        /// System.Int32
        /// </summary>
        SsaPresetComp = 1150801,

        /// <summary>
        /// System.String
        /// </summary>
        MreceiverMode = 1150802,

        /// <summary>
        /// System.Double
        /// </summary>
        MreceiverFreqSpan = 1150803,

        /// <summary>
        /// System.String
        /// </summary>
        MreceiverFreqFullSpan = 1150804,

        /// <summary>
        /// System.Double
        /// </summary>
        MreceiverFreqMaxSpan = 1150805,

        /// <summary>
        /// System.String
        /// </summary>
        MreceiverAfFullSpan = 1150806,

        /// <summary>
        /// System.String
        /// </summary>
        MreceiverAutoSignalSearch = 1150807,

        /// <summary>
        /// System.String
        /// </summary>
        MreceiverSignalTrack = 1150808,

        /// <summary>
        /// System.Double
        /// </summary>
        MreceiverRefLevel = 1150809,

        /// <summary>
        /// System.Boolean
        /// </summary>
        Mreceiver10DbMin = 1150810,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MreceiverAutoPreamplifier = 1150811,

        /// <summary>
        /// System.Double
        /// </summary>
        MreceiverZeroPhaseRefPos = 1150812,

        /// <summary>
        /// System.Int32
        /// </summary>
        MreceiverDisplayPhaseWrap = 1150813,

        /// <summary>
        /// System.Int32
        /// </summary>
        MreceiverUnitPm = 1150814,

        /// <summary>
        /// System.Int32
        /// </summary>
        MreceiverUnitThdSinad = 1150815,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MreceiverDemodBwAuto = 1150816,

        /// <summary>
        /// System.Double
        /// </summary>
        MreceiverDemodBw = 1150817,

        /// <summary>
        /// System.Double
        /// </summary>
        MreceiverAnalogDemodMeasTime = 1150818,

        /// <summary>
        /// System.Double
        /// </summary>
        MreceiverResBwManual = 1150819,

        /// <summary>
        /// System.Int32
        /// </summary>
        MreceiverTriggerSource = 1150820,

        /// <summary>
        /// System.Double
        /// </summary>
        MreceiverTriggerOffset = 1150821,

        /// <summary>
        /// System.Double
        /// </summary>
        MreceiverTriggerLevelAudio = 1150822,

        /// <summary>
        /// System.Double
        /// </summary>
        MreceiverTriggerLevelAm = 1150823,

        /// <summary>
        /// System.String
        /// </summary>
        MreceiverRfLevelMode = 1150824,

        /// <summary>
        /// System.Int32
        /// </summary>
        MreceiverCalAbsPower = 1150825,

        /// <summary>
        /// System.String
        /// </summary>
        MreceiverExpandCalRange = 1150826,

        /// <summary>
        /// System.String
        /// </summary>
        MreceiverAdjustInputRange = 1150827,

        /// <summary>
        /// System.String
        /// </summary>
        MreceiverCorrSel = 1150828,

        /// <summary>
        /// System.String
        /// </summary>
        MreceiverCorrDel = 1150829,

        /// <summary>
        /// System.String
        /// </summary>
        MreceiverCorrDelAll = 1150830,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MreceiverRelLevelMeasState = 1150831,

        /// <summary>
        /// System.Double
        /// </summary>
        MreceiverRelLevelMeasValue = 1150832,

        /// <summary>
        /// System.Int32
        /// </summary>
        MreceiverRelLevelMeasUnit = 1150833,

        /// <summary>
        /// System.String
        /// </summary>
        MreceiverRelLevelMeas = 1150834,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MreceiverLevelMeasAverState = 1150835,

        /// <summary>
        /// System.Int32
        /// </summary>
        MreceiverLevelMeasAverCount = 1150836,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MreceiverLevelAutoAver = 1150837,

        /// <summary>
        /// System.Int32
        /// </summary>
        MreceiverLevelDetector = 1150838,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MreceiverAutoRecal = 1150839,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ReceiverPwrMeter = 1150840,

        /// <summary>
        /// System.Int32
        /// </summary>
        MreceiverPwmMeasTime = 1150841,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MreceiverPwmMeasCountManual = 1150842,

        /// <summary>
        /// System.Int32
        /// </summary>
        MreceiverPwmMeasCount = 1150843,

        /// <summary>
        /// System.String
        /// </summary>
        MreceiverPwmZero = 1150844,

        /// <summary>
        /// System.Double
        /// </summary>
        MreceiverPwmRelativeValue = 1150845,

        /// <summary>
        /// System.Int32
        /// </summary>
        MreceiverPwmRelativeUnit = 1150846,

        /// <summary>
        /// System.Int32
        /// </summary>
        MreceiverPwmAbsoluteUnit = 1150847,

        /// <summary>
        /// System.String
        /// </summary>
        MreceiverPwmMeasToReference = 1150848,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MreceiverPwmUsePowerSplitter = 1150849,

        /// <summary>
        /// System.Double
        /// </summary>
        MreceiverPwmPath2InsertionLoss = 1150850,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MreceiverPwmVswrCorrection = 1150851,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MreceiverPwrExternalMeter = 1150852,

        /// <summary>
        /// System.Int32
        /// </summary>
        MreceiverPwmRemoteSetType = 1150853,

        /// <summary>
        /// System.Int32
        /// </summary>
        MereceiverPwmRemoteSetAddress = 1150854,

        /// <summary>
        /// System.Int32
        /// </summary>
        MreceiverPwmSensorCalFactorSensor = 1150855,

        /// <summary>
        /// System.String
        /// </summary>
        MreceiverPwmSensorCalFactorALabel = 1150856,

        /// <summary>
        /// System.String
        /// </summary>
        MreceiverPwmSensorCalFactorBLabel = 1150857,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MreceiverPwmRefOutput = 1150858,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MreceiverAfFilHp = 1150859,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MreceiverAfFilLp = 1150860,

        /// <summary>
        /// System.Double
        /// </summary>
        MreceiverAfFilLpFreq = 1150861,

        /// <summary>
        /// System.Double
        /// </summary>
        MreceiverAfFilHpFreq = 1150862,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MreceiverDeemAfFilState = 1150863,

        /// <summary>
        /// System.Int32
        /// </summary>
        MreceiverDeemAfFilValue = 1150864,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MreceiverWeightingAfFilCcit = 1150865,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MreceiverWeightingAfFilCcir = 1150866,

        /// <summary>
        /// System.Int32
        /// </summary>
        MreceiverAfInputCoupling = 1150867,

        /// <summary>
        /// System.String
        /// </summary>
        MreceiverModMeasMode = 1150868,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MreceiverDetectorType = 1150869,

        /// <summary>
        /// System.Int32
        /// </summary>
        MreceiverModulationType = 1150870,

        /// <summary>
        /// System.String
        /// </summary>
        MreceiverModMeasToRef = 1150871,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MreceiverPeakHold = 1150872,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MreceiverAveraging = 1150873,

        /// <summary>
        /// System.Int32
        /// </summary>
        MreceiverRelMeasPpeakMode = 1150874,

        /// <summary>
        /// System.Int32
        /// </summary>
        MreceiverRelMeasMpeakMode = 1150875,

        /// <summary>
        /// System.Int32
        /// </summary>
        MreceiverRelMeasPaverageMode = 1150876,

        /// <summary>
        /// System.Int32
        /// </summary>
        MreceiverRelMeasAverageMode = 1150877,

        /// <summary>
        /// System.Int32
        /// </summary>
        MreceiverRelMeasRmsMode = 1150878,

        /// <summary>
        /// System.Int32
        /// </summary>
        MreceiverRelMeasSrmsMode = 1150879,

        /// <summary>
        /// System.String
        /// </summary>
        MreceiverRelMeasAbsoluteMode = 1150880,

        /// <summary>
        /// System.Double
        /// </summary>
        MreceiverRelMeasRefValue = 1150881,

        /// <summary>
        /// System.String
        /// </summary>
        MreceiverRelMeasRefAuto = 1150882,

        /// <summary>
        /// System.Int32
        /// </summary>
        MreceiverAudioInputMode = 1150883,

        /// <summary>
        /// System.Int32
        /// </summary>
        MreceiverAudioInputLevel = 1150884,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MreceiverAudioInputResDisplayMode = 1150885,

        /// <summary>
        /// System.String
        /// </summary>
        MreceiverAudioInputMeasToRef = 1150886,

        /// <summary>
        /// System.Double
        /// </summary>
        QueryMreceiverModMeasurementRes = 1150887,

        /// <summary>
        /// System.Double
        /// </summary>
        QueryMreceiverModAveragedMeasRes = 1150888,

        /// <summary>
        /// System.Double
        /// </summary>
        QueryMreceiverModPeakHoldMeasRes = 1150889,

        /// <summary>
        /// System.Double
        /// </summary>
        QueryMreceiverModAudioFreqRes = 1150890,

        /// <summary>
        /// System.Double
        /// </summary>
        QueryMreceiverModFreqErrorRes = 1150891,

        /// <summary>
        /// System.Double
        /// </summary>
        QueryMreceiverModCarPowerRes = 1150892,

        /// <summary>
        /// System.Double
        /// </summary>
        QueryMreceiverModSinadMeasResults = 1150893,

        /// <summary>
        /// System.Double
        /// </summary>
        QueryMreceiverModSinadMeasAverage = 1150894,

        /// <summary>
        /// System.Double
        /// </summary>
        QueryMreceiverModSinadMeasPhold = 1150895,

        /// <summary>
        /// System.Double
        /// </summary>
        QueryMreceiverModThdMeasResults = 1150896,

        /// <summary>
        /// System.Double
        /// </summary>
        QueryMreceiverModThdMeasAverage = 1150897,

        /// <summary>
        /// System.Double
        /// </summary>
        QueryMreceiverModThdMeasPhold = 1150898,

        /// <summary>
        /// System.Double
        /// </summary>
        QueryMreceiverModStandardUncertainty = 1150899,

        /// <summary>
        /// System.Double
        /// </summary>
        QueryMreceiverAudioMeasRes = 1150900,

        /// <summary>
        /// System.Double
        /// </summary>
        ReadMreceiverPwmResult = 1150901,

        /// <summary>
        /// System.Double
        /// </summary>
        FetchMreceiverPwmResult = 1150902,

        /// <summary>
        /// System.String
        /// </summary>
        NanMode = 1150903,

        /// <summary>
        /// System.Int32
        /// </summary>
        NanMeasMode = 1150904,

        /// <summary>
        /// System.Boolean
        /// </summary>
        NanTransmissionState = 1150905,

        /// <summary>
        /// System.Int32
        /// </summary>
        NanMeasFormat = 1150906,

        /// <summary>
        /// System.Int32
        /// </summary>
        NanMeasPort = 1150907,

        /// <summary>
        /// System.Boolean
        /// </summary>
        NanReflectionState = 1150908,

        /// <summary>
        /// System.Boolean
        /// </summary>
        NanElectricalCableState = 1150909,

        /// <summary>
        /// System.Int32
        /// </summary>
        NanMagnSpacing = 1150910,

        /// <summary>
        /// System.Int32
        /// </summary>
        NetworkAnalyzerMarkerOutputFormat = 1150911,

        /// <summary>
        /// System.Double
        /// </summary>
        NanSmithRefImpedance = 1150912,

        /// <summary>
        /// System.Double
        /// </summary>
        NanVswrYScale = 1150913,

        /// <summary>
        /// System.Boolean
        /// </summary>
        NanPhaseUnwrap = 1150914,

        /// <summary>
        /// System.Int32
        /// </summary>
        NanReflCoefficientUnit = 1150915,

        /// <summary>
        /// System.Double
        /// </summary>
        NanCableYScale = 1150916,

        /// <summary>
        /// System.Double
        /// </summary>
        NanMagnYScale = 1150917,

        /// <summary>
        /// System.Int32
        /// </summary>
        NanPhaseYScale = 1150918,

        /// <summary>
        /// System.Int32
        /// </summary>
        NanReflCoefficientYScale = 1150919,

        /// <summary>
        /// System.Double
        /// </summary>
        NanCableRefLevel = 1150920,

        /// <summary>
        /// System.Double
        /// </summary>
        NanMagnRefLevel = 1150921,

        /// <summary>
        /// System.Double
        /// </summary>
        NanPhaseRefLevel = 1150922,

        /// <summary>
        /// System.Int32
        /// </summary>
        NanCableRefPosition = 1150923,

        /// <summary>
        /// System.Int32
        /// </summary>
        NanMagnRefPosition = 1150924,

        /// <summary>
        /// System.Int32
        /// </summary>
        NanPhaseRefPosition = 1150925,

        /// <summary>
        /// System.Double
        /// </summary>
        NanOutputLevel = 1150926,

        /// <summary>
        /// System.Int32
        /// </summary>
        NanCalibrationMode = 1150927,

        /// <summary>
        /// System.String
        /// </summary>
        NanCalibrationStatus = 1150928,

        /// <summary>
        /// System.Double
        /// </summary>
        NanCableLength = 1150929,

        /// <summary>
        /// System.String
        /// </summary>
        NanVswrFail = 1150930,

        /// <summary>
        /// System.Boolean
        /// </summary>
        UserCalibrateState = 1150931,

        /// <summary>
        /// System.Double
        /// </summary>
        UserCalibrateLength = 1150932,

        /// <summary>
        /// System.Double
        /// </summary>
        UserCalibrateOffs = 1150933,

        /// <summary>
        /// System.String
        /// </summary>
        UserCalibrateRename = 1150934,

        /// <summary>
        /// System.Int32
        /// </summary>
        GdelApertureSteps = 1150935,

        /// <summary>
        /// System.Int32
        /// </summary>
        GdelRefLevel = 1150936,

        /// <summary>
        /// System.Int32
        /// </summary>
        GdelRefPosition = 1150937,

        /// <summary>
        /// System.Double
        /// </summary>
        GdelYScale = 1150938,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VvmRefState = 1150939,

        /// <summary>
        /// System.Double
        /// </summary>
        VvmMagnRefValue = 1150940,

        /// <summary>
        /// System.Double
        /// </summary>
        VvmMagnValue = 1150941,

        /// <summary>
        /// System.Double
        /// </summary>
        VvmPhaseRefValue = 1150942,

        /// <summary>
        /// System.Double
        /// </summary>
        VvmPhaseValue = 1150943,

        /// <summary>
        /// System.String
        /// </summary>
        DistanceToFaultMode = 1152356,

        /// <summary>
        /// System.Double
        /// </summary>
        DtfCableLength = 1150944,

        /// <summary>
        /// System.String
        /// </summary>
        DtfCableModel = 1150945,

        /// <summary>
        /// System.Double
        /// </summary>
        DtfThreshold = 1150946,

        /// <summary>
        /// System.Double
        /// </summary>
        DtfResult = 1150947,

        /// <summary>
        /// System.Boolean
        /// </summary>
        PositionGpsState = 1150948,

        /// <summary>
        /// System.Boolean
        /// </summary>
        GpsConnectionState = 1150949,

        /// <summary>
        /// System.Double
        /// </summary>
        GpsFrequencyCorrection = 1150950,

        /// <summary>
        /// System.Int32
        /// </summary>
        GpsSignalQuality = 1150951,

        /// <summary>
        /// System.Int32
        /// </summary>
        GpsSatellites = 1150952,

        /// <summary>
        /// System.String
        /// </summary>
        GpsLatitude = 1150953,

        /// <summary>
        /// System.String
        /// </summary>
        GpsLongitude = 1150954,

        /// <summary>
        /// System.Boolean
        /// </summary>
        GpsPositionInformation = 1150955,

        /// <summary>
        /// System.Double
        /// </summary>
        TriggerRfpLevel = 1150956,

        /// <summary>
        /// System.Int32
        /// </summary>
        TriggerSourceTvrf = 1150957,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TriggerTvVidCont = 1150958,

        /// <summary>
        /// System.Int32
        /// </summary>
        TriggerTvVerticalSignal = 1150959,

        /// <summary>
        /// System.Int32
        /// </summary>
        TriggerTvLineSystem = 1150960,

        /// <summary>
        /// System.Int32
        /// </summary>
        TriggerTvHorizontalSignal = 1150961,

        /// <summary>
        /// System.Int32
        /// </summary>
        TriggerTvVideoPolarity = 1150962,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TgenNormalizationState = 1150963,

        /// <summary>
        /// System.Int32
        /// </summary>
        TgenMethod = 1150964,

        /// <summary>
        /// System.Int32
        /// </summary>
        TgenAcquire = 1150965,

        /// <summary>
        /// System.String
        /// </summary>
        TgenRecall = 1150966,

        /// <summary>
        /// System.Int32
        /// </summary>
        TgenPowMode = 1150967,

        /// <summary>
        /// System.Int32
        /// </summary>
        ExtGenIntfType = 1150968,

        /// <summary>
        /// System.Double
        /// </summary>
        TgenPowStart = 1150969,

        /// <summary>
        /// System.Double
        /// </summary>
        TgenPowStop = 1150970,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ExtGenState = 1150971,

        /// <summary>
        /// System.Int32
        /// </summary>
        ExtGenDenominator = 1150972,

        /// <summary>
        /// System.Int32
        /// </summary>
        ExtGenNumerator = 1150973,

        /// <summary>
        /// System.Double
        /// </summary>
        ExtGenFreqOffset = 1150974,

        /// <summary>
        /// System.Boolean
        /// </summary>
        ExtGenFreqSweepState = 1150975,

        /// <summary>
        /// System.Double
        /// </summary>
        ExtGenOutputPower = 1150976,

        /// <summary>
        /// System.Int32
        /// </summary>
        ExtGenRoscSource = 1150977,

        /// <summary>
        /// System.String
        /// </summary>
        ExtGenType = 1150978,

        /// <summary>
        /// System.Int32
        /// </summary>
        ExtGenIeeeAddr = 1150979,

        /// <summary>
        /// System.Double
        /// </summary>
        CorrEgaInp = 1150980,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TgenDmState = 1150981,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TgenFmState = 1150982,

        /// <summary>
        /// System.Double
        /// </summary>
        TgenFmDeviation = 1150983,

        /// <summary>
        /// System.Double
        /// </summary>
        TgenFrequencyOffset = 1150984,

        /// <summary>
        /// System.Double
        /// </summary>
        TgenOutputLevel = 1150985,

        /// <summary>
        /// System.Double
        /// </summary>
        TgenLevelOffset = 1150986,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TgenState = 1150987,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TgenAmState = 1150988,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TrackingGeneratorIsotropicAntenaState = 1150989,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FmLpasMan = 1150990,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FmSpanMan = 1150991,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FmState = 1150992,

        /// <summary>
        /// System.Double
        /// </summary>
        FmLpasFilter = 1150993,

        /// <summary>
        /// System.Double
        /// </summary>
        FmHpasFilter = 1150994,

        /// <summary>
        /// System.Double
        /// </summary>
        FmSpan = 1150995,

        /// <summary>
        /// System.Int32
        /// </summary>
        BbInputSelection = 1150996,

        /// <summary>
        /// System.Boolean
        /// </summary>
        BbInputBalanced = 1150997,

        /// <summary>
        /// System.Int32
        /// </summary>
        BbInputImpedance = 1150998,

        /// <summary>
        /// System.Int32
        /// </summary>
        BbInputSignalPath = 1150999,

        /// <summary>
        /// System.Boolean
        /// </summary>
        BbInputIqDith = 1151000,

        /// <summary>
        /// System.Boolean
        /// </summary>
        BbInputIqLpas = 1151001,

        /// <summary>
        /// System.Double
        /// </summary>
        BbInputMeasRange = 1151002,

        /// <summary>
        /// System.Int32
        /// </summary>
        BbInputTrigger = 1151003,

        /// <summary>
        /// System.Double
        /// </summary>
        BbInputTriggerILevel = 1151004,

        /// <summary>
        /// System.Double
        /// </summary>
        BbInputTriggerQLevel = 1151005,

        /// <summary>
        /// System.Int32
        /// </summary>
        FftBandStepMode = 1151006,

        /// <summary>
        /// System.Int32
        /// </summary>
        FftForm = 1151007,

        /// <summary>
        /// System.Double
        /// </summary>
        FftPline = 1151008,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FftPlineState = 1151009,

        /// <summary>
        /// System.String
        /// </summary>
        Fft = 1151010,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FftCapt = 1151011,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FftCaptAuto = 1151012,

        /// <summary>
        /// System.String
        /// </summary>
        FftCaptCalc = 1151013,

        /// <summary>
        /// System.String
        /// </summary>
        FftPreset = 1151014,

        /// <summary>
        /// System.Int32
        /// </summary>
        FftWinType = 1151015,

        /// <summary>
        /// System.Int32
        /// </summary>
        FftCalSig = 1151016,

        /// <summary>
        /// System.Double
        /// </summary>
        FftCalDc = 1151017,

        /// <summary>
        /// System.Double
        /// </summary>
        FftCalFreq = 1151018,

        /// <summary>
        /// System.Int32
        /// </summary>
        FftCalSigSour = 1151019,

        /// <summary>
        /// System.Double
        /// </summary>
        FftOffsPhas = 1151020,

        /// <summary>
        /// System.Int32
        /// </summary>
        FftUnit = 1151021,

        /// <summary>
        /// System.Double
        /// </summary>
        DigitalInputRange = 1151022,

        /// <summary>
        /// System.Boolean
        /// </summary>
        DigitalInputRangeAuto = 1151023,

        /// <summary>
        /// System.Double
        /// </summary>
        DigitalInputSrate = 1151024,

        /// <summary>
        /// System.Boolean
        /// </summary>
        DigitalInputSampleRateAuto = 1151025,

        /// <summary>
        /// System.String
        /// </summary>
        QueryDigitalInputConf = 1151026,

        /// <summary>
        /// System.String
        /// </summary>
        QueryDigitalOutputConf = 1151027,

        /// <summary>
        /// System.Boolean
        /// </summary>
        DigitalBasebandOutputState = 1151028,

        /// <summary>
        /// System.Double
        /// </summary>
        DigitalBasebandFullScaleLevel = 1151029,

        /// <summary>
        /// System.Int32
        /// </summary>
        DigitalBasebandInputTriggerSource = 1151030,

        /// <summary>
        /// System.Double
        /// </summary>
        DigitalBasebandInputTriggerLevel = 1151031,

        /// <summary>
        /// System.Double
        /// </summary>
        DigitalBasebandInputTriggerHoldoff = 1151032,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmMtyp = 1151033,

        /// <summary>
        /// System.String
        /// </summary>
        GsmRest = 1151034,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmPrat = 1151035,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmSlotMult = 1151036,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmChanTsc = 1151037,

        /// <summary>
        /// System.String
        /// </summary>
        GsmTscUser = 1151038,

        /// <summary>
        /// System.Boolean
        /// </summary>
        GsmSse = 1151039,

        /// <summary>
        /// System.Boolean
        /// </summary>
        GsmBse = 1151040,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmBsth = 1151041,

        /// <summary>
        /// System.Boolean
        /// </summary>
        GsmBursPtemTmhr = 1151042,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmBursPtemFrz = 1151043,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmBursPtemFilt = 1151044,

        /// <summary>
        /// System.Boolean
        /// </summary>
        GsmMcar = 1151045,

        /// <summary>
        /// System.String
        /// </summary>
        GsmMode = 1151046,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmBursPtemSel = 1151047,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmSyncAdjExt = 1151048,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmSyncAdjIfp = 1151049,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmSyncAdjRfp = 1151050,

        /// <summary>
        /// System.String
        /// </summary>
        GsmSyncAdjImm = 1151051,

        /// <summary>
        /// System.Boolean
        /// </summary>
        GsmSyncIqpower = 1151052,

        /// <summary>
        /// System.String
        /// </summary>
        GsmBursPfer = 1151053,

        /// <summary>
        /// System.String
        /// </summary>
        GsmBursMacc = 1151054,

        /// <summary>
        /// System.String
        /// </summary>
        GsmBursPow = 1151055,

        /// <summary>
        /// System.String
        /// </summary>
        GsmBursPtem = 1151056,

        /// <summary>
        /// System.String
        /// </summary>
        GsmSpecMod = 1151057,

        /// <summary>
        /// System.String
        /// </summary>
        GsmSpecSwit = 1151058,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmSwitLim = 1151059,

        /// <summary>
        /// System.String
        /// </summary>
        GsmSpur = 1151060,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmModulationSpectrumListAverType = 1151061,

        /// <summary>
        /// System.Boolean
        /// </summary>
        GsmExtendedSlotState = 1151062,

        /// <summary>
        /// System.Boolean
        /// </summary>
        GsmExtendedSlotLongSlot = 1151063,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmExtendedSlotTrigReference = 1151064,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmExtendedSlotMidambleReference = 1151065,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmExtendedSlotStandard = 1151066,

        /// <summary>
        /// System.Boolean
        /// </summary>
        GsmExtendedSlotOframe = 1151067,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmExtendedSlotModulation = 1151068,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmExtendedSlotRlevMode = 1151069,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmExtendedSlotRlevValue = 1151070,

        /// <summary>
        /// System.String
        /// </summary>
        GsmExtendedSlotLimit = 1151071,

        /// <summary>
        /// System.Boolean
        /// </summary>
        GsmExtendedSlotAbsLimitState = 1151072,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmExtendedSlotAbsLimitValue = 1151073,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmExtendedSlotLimitClev = 1151074,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstPerrRmsAver = 1151075,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstPerrRmsMax = 1151076,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstPerrPeakAver = 1151077,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstPerrPeakMax = 1151078,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstFerrAver = 1151079,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstFerrMax = 1151080,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstIqofAver = 1151081,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstIqofMax = 1151082,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstIqimAver = 1151083,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstIqimMax = 1151084,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstMaccRmsAver = 1151085,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstMaccRmsMax = 1151086,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstMaccPeakAver = 1151087,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstMaccPeakMax = 1151088,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstMaccOsupAver = 1151089,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstMaccOsupMax = 1151090,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstMaccPercAver = 1151091,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstMaccPercMax = 1151092,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstMaccFreqAver = 1151093,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstMaccFreqMax = 1151094,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstMaccIqofAver = 1151095,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstMaccIqofMax = 1151096,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstMaccIqimAver = 1151097,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstMaccIqimMax = 1151098,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstPerrRmsAver = 1151099,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstPerrRmsMax = 1151100,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstPerrPeakAver = 1151101,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstPerrPeakMax = 1151102,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstFerrAver = 1151103,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstFerrMax = 1151104,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstIqofAver = 1151105,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstIqofMax = 1151106,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstIqimAver = 1151107,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstIqimMax = 1151108,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstMaccRmsAver = 1151109,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstMaccRmsMax = 1151110,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstMaccPeakAver = 1151111,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstMaccPeakMax = 1151112,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstMaccOsupAver = 1151113,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstMaccOsupMax = 1151114,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstMaccPercAver = 1151115,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstMaccPercMax = 1151116,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstMaccFreqAver = 1151117,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstMaccFreqMax = 1151118,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstMaccIqofAver = 1151119,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstMaccIqofMax = 1151120,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstMaccIqimAver = 1151121,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstMaccIqimMax = 1151122,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstPtemTrgsAver = 1151123,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmFetcBurstPtemTrgsMax = 1151124,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstPtemTrgsAver = 1151125,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmReadBurstPtemTrgsMax = 1151126,

        /// <summary>
        /// System.String
        /// </summary>
        FmdemMode = 1151127,

        /// <summary>
        /// System.Int32
        /// </summary>
        FmdemTriggerSource = 1151128,

        /// <summary>
        /// System.Double
        /// </summary>
        AdemTriggerAmLevelAbs = 1151129,

        /// <summary>
        /// System.Double
        /// </summary>
        AdemTriggerAmLevelRel = 1151130,

        /// <summary>
        /// System.Double
        /// </summary>
        AdemTriggerFmLevel = 1151131,

        /// <summary>
        /// System.Double
        /// </summary>
        AdemTriggerPmLevel = 1151132,

        /// <summary>
        /// System.String
        /// </summary>
        AdemSignalSearch = 1151133,

        /// <summary>
        /// System.Double
        /// </summary>
        FmdemDisplayPdiv = 1151134,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FmdemState = 1151135,

        /// <summary>
        /// System.Int32
        /// </summary>
        FmdemAfCoup = 1151136,

        /// <summary>
        /// System.Double
        /// </summary>
        FmdemMtim = 1151137,

        /// <summary>
        /// System.Double
        /// </summary>
        FmdemBandDem = 1151138,

        /// <summary>
        /// System.Double
        /// </summary>
        FmdemRlen = 1151139,

        /// <summary>
        /// System.Double
        /// </summary>
        FmdemSrate = 1151140,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FmdemZoom = 1151141,

        /// <summary>
        /// System.Double
        /// </summary>
        FmdemZoomStart = 1151142,

        /// <summary>
        /// System.Int32
        /// </summary>
        FmdemZoomLengthMode = 1151143,

        /// <summary>
        /// System.Double
        /// </summary>
        FmdemZoomLength = 1151144,

        /// <summary>
        /// System.Int32
        /// </summary>
        AdemPhaseWrap = 1151145,

        /// <summary>
        /// System.Int32
        /// </summary>
        AdemUnitAngle = 1151146,

        /// <summary>
        /// System.Double
        /// </summary>
        AdemPmRpoX = 1151147,

        /// <summary>
        /// System.Int32
        /// </summary>
        AdemUnitThd = 1151148,

        /// <summary>
        /// System.Double
        /// </summary>
        FmdemAfCenter = 1151149,

        /// <summary>
        /// System.Double
        /// </summary>
        FmdemAfSpan = 1151150,

        /// <summary>
        /// System.Double
        /// </summary>
        FmdemAfStart = 1151151,

        /// <summary>
        /// System.Double
        /// </summary>
        FmdemAfStop = 1151152,

        /// <summary>
        /// System.String
        /// </summary>
        FmdemAfFullSpan = 1151153,

        /// <summary>
        /// System.Double
        /// </summary>
        FmdemBandRes = 1151154,

        /// <summary>
        /// System.Double
        /// </summary>
        FmdemSpecSpan = 1151155,

        /// <summary>
        /// System.Double
        /// </summary>
        FmdemSpecZoom = 1151156,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FmdemFiltHpasStat = 1151157,

        /// <summary>
        /// System.Double
        /// </summary>
        FmdemFiltHpasFreq = 1151158,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FmdemFiltLpasStat = 1151159,

        /// <summary>
        /// System.Double
        /// </summary>
        FmdemFiltLpasFreq = 1151160,

        /// <summary>
        /// System.Int32
        /// </summary>
        AdemFiltLpasFreqRel = 1151161,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FmdemFiltDempStat = 1151162,

        /// <summary>
        /// System.Double
        /// </summary>
        FmdemFiltDempTcon = 1151163,

        /// <summary>
        /// System.Boolean
        /// </summary>
        AdemFiltCcitt = 1151164,

        /// <summary>
        /// System.Boolean
        /// </summary>
        AdemFiltCcir = 1151165,

        /// <summary>
        /// System.Double
        /// </summary>
        FmdemAfrRes = 1151166,

        /// <summary>
        /// System.Double
        /// </summary>
        FmdemFerrRes = 1151167,

        /// <summary>
        /// System.Double
        /// </summary>
        FmdemSinRes = 1151168,

        /// <summary>
        /// System.Double
        /// </summary>
        FmdemThdRes = 1151169,

        /// <summary>
        /// System.Double
        /// </summary>
        FmdemCarrRes = 1151170,

        /// <summary>
        /// System.Double
        /// </summary>
        AdemSummRes = 1151171,

        /// <summary>
        /// System.Double
        /// </summary>
        AdemFmOffset = 1151172,

        /// <summary>
        /// System.String
        /// </summary>
        FmsMode = 1151173,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FmsState = 1151174,

        /// <summary>
        /// System.Int32
        /// </summary>
        FmsTriggerSource = 1151175,

        /// <summary>
        /// System.Double
        /// </summary>
        FmsTriggerLevel = 1151176,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FmsPhaseNoiseMarkerState = 1151177,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FmsDifferenceFrequencyDistortionState = 1151178,

        /// <summary>
        /// System.Int32
        /// </summary>
        FmsDifferenceFrequencyDistortionUnit = 1151179,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FmsIntermodulationDistortionState = 1151180,

        /// <summary>
        /// System.Int32
        /// </summary>
        FmsIntermodulationDistortionUnit = 1151181,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FmsAfAutoScale = 1151182,

        /// <summary>
        /// System.Int32
        /// </summary>
        FmsDisplayConfig = 1151183,

        /// <summary>
        /// System.Double
        /// </summary>
        FmsReferenceDeviation = 1151184,

        /// <summary>
        /// System.Int32
        /// </summary>
        FmsResultSummaryDetector = 1151185,

        /// <summary>
        /// System.Int32
        /// </summary>
        FmsResultSummaryMode = 1151186,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FmsResultSummaryCoupled = 1151187,

        /// <summary>
        /// System.String
        /// </summary>
        FmsMeasToRef = 1151188,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FmsAfFilterHighPassState = 1151189,

        /// <summary>
        /// System.Double
        /// </summary>
        FmsAfFilterHighPassFrequency = 1151190,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FmsAfFilterLowPassState = 1151191,

        /// <summary>
        /// System.Double
        /// </summary>
        FmsAfFilterLowPassFrequencyAbsolute = 1151192,

        /// <summary>
        /// System.Int32
        /// </summary>
        FmsAfFilterLowPassFrequencyRelative = 1151193,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FmsAfFilterCcitt = 1151194,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FmsAfFilterCcirUnweighted = 1151195,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FmsAfFilterCcirWeighted = 1151196,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FmsAfFilterDeemphasisState = 1151197,

        /// <summary>
        /// System.Double
        /// </summary>
        FmsAfFilterDeemphasisTimeConstant = 1151198,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FmsAfFilterCoupled = 1151199,

        /// <summary>
        /// System.String
        /// </summary>
        FmsAllAfFiltersOff = 1151200,

        /// <summary>
        /// System.Int32
        /// </summary>
        FmsAfRangeDevUnitAbsolute = 1151201,

        /// <summary>
        /// System.Int32
        /// </summary>
        FmsAfRangeDevUnitRelative = 1151202,

        /// <summary>
        /// System.String
        /// </summary>
        FmsDifferenceFrequencyDistortionSearchSignal = 1151203,

        /// <summary>
        /// System.String
        /// </summary>
        FmsIntermodulationDistortionSearchSignal = 1151204,

        /// <summary>
        /// System.Double
        /// </summary>
        QueryFmsCarrierFrequency = 1151205,

        /// <summary>
        /// System.Double
        /// </summary>
        QueryFmsPhaseNoiseResult = 1151206,

        /// <summary>
        /// System.Double
        /// </summary>
        QueryFmsChannelTypeResult = 1151207,

        /// <summary>
        /// System.Int32
        /// </summary>
        BtoPtype = 1151208,

        /// <summary>
        /// System.Int32
        /// </summary>
        BtoPrat = 1151209,

        /// <summary>
        /// System.Int32
        /// </summary>
        BtoPcl = 1151210,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoPowAver = 1151211,

        /// <summary>
        /// System.Boolean
        /// </summary>
        BtoBwidAuto = 1151212,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoBwid = 1151213,

        /// <summary>
        /// System.Boolean
        /// </summary>
        BtoVidAuto = 1151214,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoVid = 1151215,

        /// <summary>
        /// System.Int32
        /// </summary>
        BtoFiltMeas = 1151216,

        /// <summary>
        /// System.Int32
        /// </summary>
        BtoTracMod = 1151217,

        /// <summary>
        /// System.String
        /// </summary>
        BtoTrac = 1151218,

        /// <summary>
        /// System.Int32
        /// </summary>
        BtoTracDet = 1151219,

        /// <summary>
        /// System.Int32
        /// </summary>
        BtoSwe = 1151220,

        /// <summary>
        /// System.Boolean
        /// </summary>
        BtoSweTimeAuto = 1151221,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoSweTime = 1151222,

        /// <summary>
        /// System.String
        /// </summary>
        BtoMode = 1151223,

        /// <summary>
        /// System.Int32
        /// </summary>
        Bto = 1151224,

        /// <summary>
        /// System.Int32
        /// </summary>
        BtoLap = 1151225,

        /// <summary>
        /// System.Int32
        /// </summary>
        BtoChannel = 1151226,

        /// <summary>
        /// System.Int32
        /// </summary>
        BtoAclrAcp = 1151227,

        /// <summary>
        /// System.Int32
        /// </summary>
        BtoGeog = 1151228,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoEgaInp = 1151229,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoPulsOffs = 1151230,

        /// <summary>
        /// System.Boolean
        /// </summary>
        BtoSearchPuls = 1151231,

        /// <summary>
        /// System.Int32
        /// </summary>
        BtoSyncOffs = 1151232,

        /// <summary>
        /// System.Boolean
        /// </summary>
        BtoSearchSync = 1151233,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoSearchTime = 1151234,

        /// <summary>
        /// System.Boolean
        /// </summary>
        BtoSearchTimeAuto = 1151235,

        /// <summary>
        /// System.Int32
        /// </summary>
        BtoCfstBlockCount = 1151236,

        /// <summary>
        /// System.Int32
        /// </summary>
        BtoIbsAcp = 1151237,

        /// <summary>
        /// System.String
        /// </summary>
        BtoIbsGateAuto = 1151238,

        /// <summary>
        /// System.Int32
        /// </summary>
        BtoPbsc = 1151239,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoRtpTime = 1151240,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoPdiv = 1151241,

        /// <summary>
        /// System.Boolean
        /// </summary>
        BtoZoomState = 1151242,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoZoomStart = 1151243,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoOpowPeak = 1151244,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoOpowAver = 1151245,

        /// <summary>
        /// System.Int32
        /// </summary>
        BtoStat = 1151246,

        /// <summary>
        /// System.Int32
        /// </summary>
        BtoAclrExc = 1151247,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoMchDfAver = 1151248,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoMchDfMax = 1151249,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoMchRat = 1151250,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoMchDf2Perc = 1151251,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoIcft = 1151252,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoCfdrMax = 1151253,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoCfdrRate = 1151254,

        /// <summary>
        /// System.Int32
        /// </summary>
        BtoPlen = 1151255,

        /// <summary>
        /// System.Int32
        /// </summary>
        BtoPtyp = 1151256,

        /// <summary>
        /// System.Int32
        /// </summary>
        BtoCfstBlockCountValue = 1151257,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoCfstDevmRms = 1151258,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoCfstDevmPeak = 1151259,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoCfstDevm99pct = 1151260,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoCfstFerrTotal = 1151261,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoCfstFerrBlock = 1151262,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoCfstFerrInitial = 1151263,

        /// <summary>
        /// System.Int32
        /// </summary>
        BtoDpenPackets = 1151264,

        /// <summary>
        /// System.Int32
        /// </summary>
        BtoDpenPacketsPassed = 1151265,

        /// <summary>
        /// System.Int32
        /// </summary>
        BtoDpenBer = 1151266,

        /// <summary>
        /// System.Int32
        /// </summary>
        BtoIbsExc = 1151267,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoIbsHadj = 1151268,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoIbsTxchannel = 1151269,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoRtp = 1151270,

        /// <summary>
        /// System.Double
        /// </summary>
        BtoRtpRatio = 1151271,

        /// <summary>
        /// System.String
        /// </summary>
        PmetMode = 1152357,

        /// <summary>
        /// System.Double
        /// </summary>
        PmetRel = 1151272,

        /// <summary>
        /// System.String
        /// </summary>
        PmetRelAuto = 1151273,

        /// <summary>
        /// System.Double
        /// </summary>
        PwmRelativeOffset = 1151274,

        /// <summary>
        /// System.Double
        /// </summary>
        PmetFreq = 1151275,

        /// <summary>
        /// System.Int32
        /// </summary>
        PmetMeasTime = 1151276,

        /// <summary>
        /// System.Int32
        /// </summary>
        PmetCoupling = 1151277,

        /// <summary>
        /// System.String
        /// </summary>
        PmetZero = 1151278,

        /// <summary>
        /// System.Int32
        /// </summary>
        PmetUnitAbs = 1151279,

        /// <summary>
        /// System.Int32
        /// </summary>
        PmetUnitRel = 1151280,

        /// <summary>
        /// System.Boolean
        /// </summary>
        PmetState = 1151281,

        /// <summary>
        /// System.Boolean
        /// </summary>
        PmetAverageAuto = 1151282,

        /// <summary>
        /// System.Int32
        /// </summary>
        PmetAverageCount = 1151283,

        /// <summary>
        /// System.Boolean
        /// </summary>
        PmetRefLevelOffsetState = 1151284,

        /// <summary>
        /// System.Boolean
        /// </summary>
        PwmExternalPowerTriggerState = 1151285,

        /// <summary>
        /// System.Double
        /// </summary>
        PwrExternalPowerTriggerLevel = 1151286,

        /// <summary>
        /// System.Boolean
        /// </summary>
        PmetDutyCycleState = 1151287,

        /// <summary>
        /// System.Double
        /// </summary>
        PmetDutyCycle = 1151288,

        /// <summary>
        /// System.Boolean
        /// </summary>
        PmetUseStandard = 1151289,

        /// <summary>
        /// System.Int32
        /// </summary>
        PmetStandard = 1151290,

        /// <summary>
        /// System.Boolean
        /// </summary>
        PmetBargraphState = 1151291,

        /// <summary>
        /// System.Boolean
        /// </summary>
        PmetRelative = 1151292,

        /// <summary>
        /// System.Int32
        /// </summary>
        PmetPowerDisplay = 1151293,

        /// <summary>
        /// System.Double
        /// </summary>
        PmetRead = 1151294,

        /// <summary>
        /// System.Double
        /// </summary>
        PmetFetch = 1151295,

        /// <summary>
        /// System.String
        /// </summary>
        GsmK10Mode = 1151296,

        /// <summary>
        /// System.String
        /// </summary>
        GsmK10ModeFsv = 1151297,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmK10Dut = 1151298,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmK10SigCharMode = 1151299,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmK10SigCharBand = 1151300,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmK10SigCharBandArfc = 1151301,

        /// <summary>
        /// System.Boolean
        /// </summary>
        GsmK10AutoPwrLev = 1151302,

        /// <summary>
        /// System.String
        /// </summary>
        GsmK10AutoPwrLevOnce = 1151303,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmK10LevelSettingsPwrClass = 1151304,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmK10LevelSettingsPowStat = 1151305,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmK10Synchronization = 1151306,

        /// <summary>
        /// System.Boolean
        /// </summary>
        GsmK10MeasureOnlyOnSync = 1151307,

        /// <summary>
        /// System.Double
        /// </summary>
        GsmK10CapSetTrackTime = 1151308,

        /// <summary>
        /// System.Boolean
        /// </summary>
        GsmK10AutoSetLevel = 1151309,

        /// <summary>
        /// System.String
        /// </summary>
        GsmK10AutoSetLevelOnce = 1151310,

        /// <summary>
        /// System.Boolean
        /// </summary>
        GsmK10AutoSetFrame = 1151311,

        /// <summary>
        /// System.String
        /// </summary>
        GsmK10AutoSetFrameOnce = 1151312,

        /// <summary>
        /// System.Boolean
        /// </summary>
        GsmK10AutoSetTrigger = 1151313,

        /// <summary>
        /// System.String
        /// </summary>
        GsmK10AutoSetTriggerOnce = 1151314,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmK10DemodSingleSlotMeas = 1151315,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmK10DemMultiSlotMeasSlotNum = 1151316,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmK10DemMultiSlotMeasFirstSlot = 1151317,

        /// <summary>
        /// System.Boolean
        /// </summary>
        GsmK10DemodulationEqualTimeslotLength = 1151318,

        /// <summary>
        /// System.Boolean
        /// </summary>
        GsmK10BurstSlotState = 1151319,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmK10BurstTypeBurst = 1151320,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmK10BurstPcl = 1151321,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmK10BurstMod = 1151322,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmK10BurstFilter = 1151323,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmK10BurstTsc = 1151324,

        /// <summary>
        /// System.String
        /// </summary>
        GsmK10MeasBurstType = 1151325,

        /// <summary>
        /// System.String
        /// </summary>
        GsmK10MeasSpectType = 1151326,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmK10PwrVsTimeMeas = 1151327,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmK10PowerVsTimeFilter = 1151328,

        /// <summary>
        /// System.Boolean
        /// </summary>
        GsmK10ModulationTransientSpectrumEnableLeftLimit = 1151329,

        /// <summary>
        /// System.Boolean
        /// </summary>
        GsmK10ModulationTransientSpectrumEnableRightLimit = 1151330,

        /// <summary>
        /// System.Boolean
        /// </summary>
        GsmK10MultiCarrierBts = 1151331,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmK10MultiCarrierNumberOfActiveChannels = 1151332,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmK10MultiCarrierBtsClass = 1151333,

        /// <summary>
        /// System.Int32
        /// </summary>
        GsmK10MultiCarrierFilter = 1151334,

        /// <summary>
        /// System.Double
        /// </summary>
        ReadGsmK10MeasRes = 1151335,

        /// <summary>
        /// System.Double
        /// </summary>
        ReadGsmK10MeasRes95Perc = 1151336,

        /// <summary>
        /// System.Double
        /// </summary>
        FetchGsmK10MeasRes = 1151337,

        /// <summary>
        /// System.Double
        /// </summary>
        FetchGsmK10MeasRes95Perc = 1151338,

        /// <summary>
        /// System.Double
        /// </summary>
        ReadGsmK10BurstSlotRes = 1151339,

        /// <summary>
        /// System.Double
        /// </summary>
        FetchGsmK10BurstSlotRes = 1151340,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SpemState = 1151341,

        /// <summary>
        /// System.Int32
        /// </summary>
        SpemColor = 1151342,

        /// <summary>
        /// System.Int32
        /// </summary>
        SpemSize = 1151343,

        /// <summary>
        /// System.Int32
        /// </summary>
        SpemHdep = 1151344,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SpemTimeStampState = 1151345,

        /// <summary>
        /// System.Int32
        /// </summary>
        SpemFrameCount = 1151346,

        /// <summary>
        /// System.Int32
        /// </summary>
        SpemFrameSelect = 1151347,

        /// <summary>
        /// System.Boolean
        /// </summary>
        SpemContinuous = 1151348,

        /// <summary>
        /// System.String
        /// </summary>
        SpemClear = 1151349,

        /// <summary>
        /// System.Boolean
        /// </summary>
        FileItemsSaveRecalSpem = 1151350,

        /// <summary>
        /// System.Int32
        /// </summary>
        SpemMarkFrameNumber = 1151351,

        /// <summary>
        /// System.Int32
        /// </summary>
        SpemMarkerSearchArea = 1151352,

        /// <summary>
        /// System.String
        /// </summary>
        SpemMarkerSearchXy = 1151353,

        /// <summary>
        /// System.String
        /// </summary>
        SpemMarkerSearchY = 1151354,

        /// <summary>
        /// System.Int32
        /// </summary>
        SpemDeltFrameNumber = 1151355,

        /// <summary>
        /// System.Int32
        /// </summary>
        SpemDeltSearchArea = 1151356,

        /// <summary>
        /// System.String
        /// </summary>
        SpemDeltSearchXy = 1151357,

        /// <summary>
        /// System.String
        /// </summary>
        SpemDeltSearchY = 1151358,

        /// <summary>
        /// System.String
        /// </summary>
        AviMode = 1151359,

        /// <summary>
        /// System.Int32
        /// </summary>
        AviDemodMode = 1151360,

        /// <summary>
        /// System.Int32
        /// </summary>
        AviDistortionResult = 1151361,

        /// <summary>
        /// System.Int32
        /// </summary>
        AviDdmUnit = 1151362,

        /// <summary>
        /// System.Int32
        /// </summary>
        AviInput = 1151363,

        /// <summary>
        /// System.Boolean
        /// </summary>
        AviDemodBwidAuto = 1151364,

        /// <summary>
        /// System.Double
        /// </summary>
        AviDemodBwid = 1151365,

        /// <summary>
        /// System.Int32
        /// </summary>
        AviBbInputRange = 1151366,

        /// <summary>
        /// System.Double
        /// </summary>
        AviBbInputSens = 1151367,

        /// <summary>
        /// System.Boolean
        /// </summary>
        AviHarmDistState = 1151368,

        /// <summary>
        /// System.Double
        /// </summary>
        AviHarmDistFreq = 1151369,

        /// <summary>
        /// System.Int32
        /// </summary>
        AviThdFundFreq = 1151370,

        /// <summary>
        /// System.Double
        /// </summary>
        AviThdMaxFreq = 1151371,

        /// <summary>
        /// System.Double
        /// </summary>
        AviRfFreqResult = 1151372,

        /// <summary>
        /// System.Double
        /// </summary>
        AviCarrOffsResult = 1151373,

        /// <summary>
        /// System.Double
        /// </summary>
        AviRfLevelResult = 1151374,

        /// <summary>
        /// System.Double
        /// </summary>
        AviDdmResult = 1151375,

        /// <summary>
        /// System.Double
        /// </summary>
        AviFmDeviationResult = 1151376,

        /// <summary>
        /// System.Double
        /// </summary>
        AviFmFreqResult = 1151377,

        /// <summary>
        /// System.Double
        /// </summary>
        AviPhaseResult = 1151378,

        /// <summary>
        /// System.Double
        /// </summary>
        AviInputLevelResult = 1151379,

        /// <summary>
        /// System.Double
        /// </summary>
        AviSdmResult = 1151380,

        /// <summary>
        /// System.Double
        /// </summary>
        AviShdResult = 1151381,

        /// <summary>
        /// System.String
        /// </summary>
        CatvMode = 1151382,

        /// <summary>
        /// System.String
        /// </summary>
        CatvYScaleAuto = 1151383,

        /// <summary>
        /// System.Boolean
        /// </summary>
        CatvEpatZoomState = 1151384,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvEpatZoomFactor = 1151385,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvMerrZoom = 1151386,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvOverZoom = 1151387,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvQuadZoom = 1151388,

        /// <summary>
        /// System.Boolean
        /// </summary>
        CatvTraceModeFreeze = 1151389,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvChannel = 1151390,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvRfFrequency = 1151391,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvRfFrequencyStepSize = 1151392,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvSweepSpacing = 1151393,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvSideBand = 1151394,

        /// <summary>
        /// System.String
        /// </summary>
        CatvModulStandard = 1151395,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvModulStandardGdelay = 1151396,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvModulStandardSigType = 1151397,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvMeas = 1151398,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvCarrMeas = 1151399,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvMeasMode = 1151400,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvAtvBwid = 1151401,

        /// <summary>
        /// System.String
        /// </summary>
        CatvAtvFreqNext = 1151402,

        /// <summary>
        /// System.Boolean
        /// </summary>
        CatvAtvNoiseFloorCorrection = 1151403,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvRefChannelManual = 1151404,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvAtvRefPowerManual = 1151405,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvRefPowerMode = 1151406,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvHumUnit = 1151407,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvVcpUnit = 1151408,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvStandard = 1151409,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvSoundSystem = 1151410,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvColor = 1151411,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvTriggerBarField = 1151412,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvTriggerBarLine = 1151413,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvTriggerBarLineType = 1151414,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvTriggerVideoScopeField = 1151415,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvTriggerVideoScopeLine = 1151416,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvTriggerQlineField = 1151417,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvTriggerQuietLine = 1151418,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvAtvCarrLimFreqOffset = 1151419,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvAtvCarrLimRelPower = 1151420,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvAtvCarrLimVcf = 1151421,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvAtvCarrLimVcp = 1151422,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvAtvLimCn = 1151423,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvAtvLimCso = 1151424,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvAtvLimCtb = 1151425,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvAtvLimHumLow = 1151426,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvAtvLimHumUpp = 1151427,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvAtvLimVmodMdep = 1151428,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvAtvLimVmodRpc = 1151429,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvAtvLimVmodVcp = 1151430,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvCarrLimFreqOffsetResult = 1151431,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvCarrLimRelPowerResult = 1151432,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvCarrLimVcfResult = 1151433,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvCarrLimVcpResult = 1151434,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvLimCnResult = 1151435,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvLimCsoResult = 1151436,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvLimCtbResult = 1151437,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvLimHumResult = 1151438,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvAtvLimVmodMdepResult = 1151439,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvAtvCarrVcpa = 1151440,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvAtvCarrVcfValue = 1151441,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvAtvCarrRelPowerValue = 1151442,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvAtvCarrFreqOffsetValue = 1151443,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvAtvValue = 1151444,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvAtvCnCnValue = 1151445,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvAtvCsoCsoValue = 1151446,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvAtvCtbCtbValue = 1151447,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvAtvHumValue = 1151448,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvAtvVmodResultValue = 1151449,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvDtvMeas = 1151450,

        /// <summary>
        /// System.Boolean
        /// </summary>
        CatvDtvSatt = 1151451,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvDtvEpatUnit = 1151452,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvDtvEpatRvel = 1151453,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvDtvErrorUnit = 1151454,

        /// <summary>
        /// System.Boolean
        /// </summary>
        CatvDtvEqualizerState = 1151455,

        /// <summary>
        /// System.Boolean
        /// </summary>
        CatvDtvEqualizerFreeze = 1151456,

        /// <summary>
        /// System.String
        /// </summary>
        CatvDtvEqualizerReset = 1151457,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvDtvFilterAlpha = 1151458,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvDtvSrate = 1151459,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvDtvStandard = 1151460,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvDtvFormat = 1151461,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvDtvCarrLimFreqOffset = 1151462,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvDtvLimChannelPower = 1151463,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvDtvLimSupp = 1151464,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvDtvLimErrLow = 1151465,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvDtvLimErrUpp = 1151466,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvDtvLimImb = 1151467,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvDtvLimPjit = 1151468,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvDtvLimQerr = 1151469,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvDtvLimSymbRateOffset = 1151470,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvDtvCarrLimFreqOffsetResult = 1151471,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvDtvLimChannelPowerResult = 1151472,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvDtvCarrLimCarrSuppResult = 1151473,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvDtvLimErrorResult = 1151474,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvDtvLimImbResult = 1151475,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvDtvLimPjitResult = 1151476,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvDtvLimQerrResult = 1151477,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvDtvLimSymbRateOffsetResult = 1151478,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvDtvSpec = 1151479,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvDtvError = 1151480,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvDtvOverCarrFreqOffset = 1151481,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvDtvOverSrateOffset = 1151482,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvDtvMerrImb = 1151483,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvDtvMerrQerr = 1151484,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvDtvMerrCsup = 1151485,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvDtvMerrPjit = 1151486,

        /// <summary>
        /// System.Double
        /// </summary>
        CatvDtvChannelPowerResult = 1151487,

        /// <summary>
        /// System.Int32
        /// </summary>
        CatvTvMeas = 1151488,

        /// <summary>
        /// System.String
        /// </summary>
        CatvTvChannel = 1151489,

        /// <summary>
        /// System.String
        /// </summary>
        CatvTvModStandardTiltActivate = 1151490,

        /// <summary>
        /// System.String
        /// </summary>
        CatvTvModStandardTiltDeactivate = 1151491,

        /// <summary>
        /// System.String
        /// </summary>
        NoiseMode = 1151492,

        /// <summary>
        /// System.Int32
        /// </summary>
        NoiseCalcLimTrac = 1151493,

        /// <summary>
        /// System.String
        /// </summary>
        NoiseConfCorr = 1151494,

        /// <summary>
        /// System.String
        /// </summary>
        NoiseConfListCont = 1151495,

        /// <summary>
        /// System.String
        /// </summary>
        NoiseConfListSing = 1151496,

        /// <summary>
        /// System.String
        /// </summary>
        NoiseConfSing = 1151497,

        /// <summary>
        /// System.Boolean
        /// </summary>
        NoiseInpPresStat = 1151498,

        /// <summary>
        /// System.Boolean
        /// </summary>
        NoiseInpGainStat = 1151499,

        /// <summary>
        /// System.Int32
        /// </summary>
        NoiseCorrEnrMode = 1151500,

        /// <summary>
        /// System.Double
        /// </summary>
        NoiseCorrEnrSpot = 1151501,

        /// <summary>
        /// System.Double
        /// </summary>
        NoiseCorrTemp = 1151502,

        /// <summary>
        /// System.Int32
        /// </summary>
        NoiseCorrLossMode = 1151503,

        /// <summary>
        /// System.Double
        /// </summary>
        NoiseCorrLossSpot = 1151504,

        /// <summary>
        /// System.Double
        /// </summary>
        NoiseCorrErej = 1151505,

        /// <summary>
        /// System.Boolean
        /// </summary>
        NoiseCorrStat = 1151506,

        /// <summary>
        /// System.Double
        /// </summary>
        NoiseFreqStep = 1151507,

        /// <summary>
        /// System.Double
        /// </summary>
        NoiseFreqFixed = 1151508,

        /// <summary>
        /// System.Int32
        /// </summary>
        NoiseConfSystLosc = 1151509,

        /// <summary>
        /// System.Double
        /// </summary>
        NoiseConfSystLoscFreq = 1151510,

        /// <summary>
        /// System.Double
        /// </summary>
        NoiseConfSystIntFreq = 1151511,

        /// <summary>
        /// System.Int32
        /// </summary>
        NoiseConfModeDut = 1151512,

        /// <summary>
        /// System.Double
        /// </summary>
        NoiseExtFreqOffs = 1151513,

        /// <summary>
        /// System.Int32
        /// </summary>
        NoiseExtFreqNum = 1151514,

        /// <summary>
        /// System.Int32
        /// </summary>
        NoiseExtFreqDen = 1151515,

        /// <summary>
        /// System.Double
        /// </summary>
        NoiseExtPow = 1151516,

        /// <summary>
        /// System.Int32
        /// </summary>
        NoiseGpibGenAddr = 1151517,

        /// <summary>
        /// System.String
        /// </summary>
        NoiseGenType = 1151518,

        /// <summary>
        /// System.Boolean
        /// </summary>
        NoiseConfGenContStat = 1151519,

        /// <summary>
        /// System.Boolean
        /// </summary>
        NoiseConfGenInitAuto = 1151520,

        /// <summary>
        /// System.Double
        /// </summary>
        NoiseConfDutGain = 1151521,

        /// <summary>
        /// System.Double
        /// </summary>
        NoiseConfDutSettTime = 1151522,

        /// <summary>
        /// System.String
        /// </summary>
        NoiseArrayMemory = 1151523,

        /// <summary>
        /// System.Int32
        /// </summary>
        NoiseDispDataTrac = 1151524,

        /// <summary>
        /// System.Boolean
        /// </summary>
        NoiseDispTracStat = 1151525,

        /// <summary>
        /// System.Boolean
        /// </summary>
        NoiseDispSymb = 1151526,

        /// <summary>
        /// System.Boolean
        /// </summary>
        NoiseDispAutoScal = 1151527,

        /// <summary>
        /// System.Double
        /// </summary>
        NoiseDispTracBottom = 1151528,

        /// <summary>
        /// System.Double
        /// </summary>
        NoiseDispTracTop = 1151529,

        /// <summary>
        /// System.Boolean
        /// </summary>
        NoiseDispTracRefLevStat = 1151530,

        /// <summary>
        /// System.Boolean
        /// </summary>
        NoiseDispTable = 1151531,

        /// <summary>
        /// System.Boolean
        /// </summary>
        NoiseDispTraceMem = 1151532,

        /// <summary>
        /// System.Boolean
        /// </summary>
        NoiseDisplCurrResults = 1151533,

        /// <summary>
        /// System.Int32
        /// </summary>
        NoiseDispXAxis = 1151534,

        /// <summary>
        /// System.String
        /// </summary>
        PhaseMode = 1151535,

        /// <summary>
        /// System.Int32
        /// </summary>
        PhaseSweepCount = 1151536,

        /// <summary>
        /// System.Boolean
        /// </summary>
        PhaseConfPowAuto = 1151537,

        /// <summary>
        /// System.Int32
        /// </summary>
        PhaseBandType = 1151538,

        /// <summary>
        /// System.Double
        /// </summary>
        PhaseBandRat = 1151539,

        /// <summary>
        /// System.Double
        /// </summary>
        PhaseConfPowExpRf = 1151540,

        /// <summary>
        /// System.Int32
        /// </summary>
        PhaseSweepMode = 1151541,

        /// <summary>
        /// System.Boolean
        /// </summary>
        PhaseSweForw = 1151542,

        /// <summary>
        /// System.Double
        /// </summary>
        PhaseListRangBand = 1151543,

        /// <summary>
        /// System.Int32
        /// </summary>
        PhaseListRangFiltType = 1151544,

        /// <summary>
        /// System.Int32
        /// </summary>
        PhaseListRangSweCoun = 1151545,

        /// <summary>
        /// System.Boolean
        /// </summary>
        PhaseFreqVer = 1151546,

        /// <summary>
        /// System.Double
        /// </summary>
        PhaseFreqVerTol = 1151547,

        /// <summary>
        /// System.Double
        /// </summary>
        PhaseFreqVerTolAbs = 1151548,

        /// <summary>
        /// System.Boolean
        /// </summary>
        PhaseFreqTrack = 1151549,

        /// <summary>
        /// System.Boolean
        /// </summary>
        PhasePowRlevVer = 1151550,

        /// <summary>
        /// System.Double
        /// </summary>
        PhasePowRlevVerTol = 1151551,

        /// <summary>
        /// System.Boolean
        /// </summary>
        PhasePowTrack = 1151552,

        /// <summary>
        /// System.Boolean
        /// </summary>
        PhaseCalcEval = 1151553,

        /// <summary>
        /// System.Double
        /// </summary>
        PhaseCalcEvalStar = 1151554,

        /// <summary>
        /// System.Double
        /// </summary>
        PhaseCalcEvalStop = 1151555,

        /// <summary>
        /// System.Double
        /// </summary>
        PhaseCalcSnoX = 1151556,

        /// <summary>
        /// System.Double
        /// </summary>
        PhaseCalcSnoY = 1151557,

        /// <summary>
        /// System.String
        /// </summary>
        PhaseCalcSnoAoff = 1151558,

        /// <summary>
        /// System.Boolean
        /// </summary>
        PhaseCalcSnoStat = 1151559,

        /// <summary>
        /// System.String
        /// </summary>
        PhaseConfRefm = 1151560,

        /// <summary>
        /// System.Double
        /// </summary>
        PhaseDispTraceY = 1151561,

        /// <summary>
        /// System.Boolean
        /// </summary>
        PhaseDispTraceYAuto = 1151562,

        /// <summary>
        /// System.String
        /// </summary>
        PhaseDispTraceYAutoOnce = 1151563,

        /// <summary>
        /// System.Double
        /// </summary>
        PhaseDispTraceYRlev = 1151564,

        /// <summary>
        /// System.Double
        /// </summary>
        PhaseDispTraceYRlevOffs = 1151565,

        /// <summary>
        /// System.Boolean
        /// </summary>
        PhaseDispTraceSmo = 1151566,

        /// <summary>
        /// System.Int32
        /// </summary>
        PhaseDispTraceSmoAper = 1151567,

        /// <summary>
        /// System.Int32
        /// </summary>
        PhaseDispTraceSmoType = 1151568,

        /// <summary>
        /// System.String
        /// </summary>
        VsaMode = 1151569,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaFactoryDefaults = 1151570,

        /// <summary>
        /// System.String
        /// </summary>
        VsaRefLevelAdjust = 1151571,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaScreenCoupling = 1151572,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaMarkerSearch = 1151573,

        /// <summary>
        /// System.String
        /// </summary>
        VsaMarkerPeakSearch = 1151574,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaAdjustReferenceLevelToFullScaleLevel = 1151575,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaFullScaleLevelUnit = 1151576,

        /// <summary>
        /// System.String
        /// </summary>
        VsaPresetScreens = 1151577,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaResult = 1151578,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaResultFormat = 1151579,

        /// <summary>
        /// System.String
        /// </summary>
        VsaDigitalStandard = 1151580,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaFrequencyStepAuto = 1151581,

        /// <summary>
        /// System.String
        /// </summary>
        VsaDigitalStandardSave = 1151582,

        /// <summary>
        /// System.String
        /// </summary>
        VsaDigitalStandardGroup = 1151583,

        /// <summary>
        /// System.String
        /// </summary>
        VsaDigitalStandardComment = 1151584,

        /// <summary>
        /// System.String
        /// </summary>
        VsaDigitalStandardPrefix = 1151585,

        /// <summary>
        /// System.String
        /// </summary>
        VsaDigitalStandardDelete = 1151586,

        /// <summary>
        /// System.String
        /// </summary>
        VsaDigitalStandardDefaults = 1151587,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaBurstSearchAuto = 1151588,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaBurstSearchState = 1151589,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaBurstSearchMode = 1151590,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaBurstSearchThresholdAuto = 1151591,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaBurstSearchThresholdMode = 1151592,

        /// <summary>
        /// System.Double
        /// </summary>
        VsaBurstSearchThresholdLevel = 1151593,

        /// <summary>
        /// System.Double
        /// </summary>
        VsaBurstSearchHysteresisLevel = 1151594,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaBurstSearchLengthMin = 1151595,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaBurstSearchLengthMax = 1151596,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaBurstSearchSkipRise = 1151597,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaBurstSearchSkipFall = 1151598,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaBurstSearchGlen = 1151599,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaSearchBurstAutoConfiguration = 1151600,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaSearchBurstTolerance = 1151601,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaSyncSearchAuto = 1151602,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaSyncSearchState = 1151603,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaSyncSearchMode = 1151604,

        /// <summary>
        /// System.String
        /// </summary>
        VsaSyncName = 1151605,

        /// <summary>
        /// System.String
        /// </summary>
        VsaSyncText = 1151606,

        /// <summary>
        /// System.String
        /// </summary>
        VsaSyncComment = 1151607,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaSyncNstat = 1151608,

        /// <summary>
        /// System.String
        /// </summary>
        VsaSyncPattAdd = 1151609,

        /// <summary>
        /// System.String
        /// </summary>
        VsaSyncDelete = 1151610,

        /// <summary>
        /// System.String
        /// </summary>
        VsaSyncPattRemove = 1151611,

        /// <summary>
        /// System.String
        /// </summary>
        VsaSyncPattRemoveAll = 1151612,

        /// <summary>
        /// System.String
        /// </summary>
        VsaSyncCopy = 1151613,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaSyncSearchPatternAutoConfiguration = 1151614,

        /// <summary>
        /// System.Double
        /// </summary>
        VsaSyncSearchIqCorrelationThreshold = 1151615,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaEqualizerState = 1151616,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaEqualizerLearning = 1151617,

        /// <summary>
        /// System.Double
        /// </summary>
        VsaEqualizerSpeedConversion = 1151618,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaEqualizerLength = 1151619,

        /// <summary>
        /// System.String
        /// </summary>
        VsaEqualizerLoad = 1151620,

        /// <summary>
        /// System.String
        /// </summary>
        VsaEqualizerSave = 1151621,

        /// <summary>
        /// System.String
        /// </summary>
        VsaEqualizerDelete = 1151622,

        /// <summary>
        /// System.String
        /// </summary>
        VsaEqualizerReset = 1151623,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaSignalType = 1151624,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaPatternSignalState = 1151625,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaPatternOffsetState = 1151626,

        /// <summary>
        /// System.Double
        /// </summary>
        VsaModulationSrate = 1151627,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaModulationFormat = 1151628,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaModulationQpskFormat = 1151629,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaModulationPskFormat = 1151630,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaModulationMskFormat = 1151631,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaModulationQamFormat = 1151632,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaModulationPskNstat = 1151633,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaModulationQamNstat = 1151634,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaModulationFskNstat = 1151635,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaModulationVsbNstat = 1151636,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaModulationUqamNstat = 1151637,

        /// <summary>
        /// System.String
        /// </summary>
        VsaModulationMapping = 1151638,

        /// <summary>
        /// System.String
        /// </summary>
        VsaModulationUqamMapping = 1151639,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaModulationFilterState = 1151640,

        /// <summary>
        /// System.Double
        /// </summary>
        VsaModulationAlphabt = 1151641,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaModulationPrate = 1151642,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaModulationFskDevComp = 1151643,

        /// <summary>
        /// System.Double
        /// </summary>
        VsaModulationFskDev = 1151644,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaModulationEvmCalculation = 1151645,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaModulationEvmOffset = 1151646,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaTxFilterType = 1151647,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaTxFilterState = 1151648,

        /// <summary>
        /// System.Double
        /// </summary>
        VsaTxFilterAlpha = 1151649,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaDemodulationRecLengthAuto = 1151650,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaDemodulationRecLengthSym = 1151651,

        /// <summary>
        /// System.Double
        /// </summary>
        VsaDemodulationRecLengthS = 1151652,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaDemodulationResultLength = 1151653,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaDemodulationMultiburstState = 1151654,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaDemodulationSideBand = 1151655,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaDemodulationNormalize = 1151656,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaEvalLineState = 1151657,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaEvalLine = 1151658,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaDemodulationWbanState = 1151659,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaCompensateIqOffset = 1151660,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaCompensateIqImbalance = 1151661,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaCompensateAmplitudeDroop = 1151662,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaAutoMeasurementFilter = 1151663,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaMeasurementFilterType = 1151664,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaMeasurementFilterState = 1151665,

        /// <summary>
        /// System.Double
        /// </summary>
        VsaMeasurementFilterAlpha = 1151666,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaUnitTime = 1151667,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaUnitAngle = 1151668,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaFitRefPoint = 1151669,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaFitAlign = 1151670,

        /// <summary>
        /// System.Double
        /// </summary>
        VsaFitAlignValue = 1151671,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaFitAlignOffset = 1151672,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaFitSymbolNumber = 1151673,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaFitSyncOffset = 1151674,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaDisplayTraceSymbol = 1151675,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaTraceDisplaySpacing = 1151676,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaScaleXBarCount = 1151677,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaScaleXPdiv = 1151678,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaScaleXRval = 1151679,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaScaleXRpos = 1151680,

        /// <summary>
        /// System.Double
        /// </summary>
        VsaScaleYPdiv = 1151681,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaConfigureDisplayPointsPerSymbolAuto = 1151682,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaConfigureDisplayPointsPerSymbol = 1151683,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaCaptureOversampling = 1151684,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaTraceEvaluation = 1151685,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaMburstStart = 1151686,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaMburstLength = 1151687,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaMburstCaptureAuto = 1151688,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaMburstState = 1151689,

        /// <summary>
        /// System.String
        /// </summary>
        VsaMburstSearchFirst = 1151690,

        /// <summary>
        /// System.String
        /// </summary>
        VsaMburstSearchNext = 1151691,

        /// <summary>
        /// System.String
        /// </summary>
        VsaMburstSearchStart = 1151692,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaFileExpHeader = 1151693,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaFileDataMode = 1151694,

        /// <summary>
        /// System.Int32
        /// </summary>
        VsaFileDataRawMode = 1151695,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaSpectrumState = 1151696,

        /// <summary>
        /// System.Boolean
        /// </summary>
        VsaCpointState = 1151697,

        /// <summary>
        /// System.Double
        /// </summary>
        VsaCpointValue = 1151698,

        /// <summary>
        /// System.Double
        /// </summary>
        VsaCpointPhaseShift = 1151699,

        /// <summary>
        /// System.Double
        /// </summary>
        VsaCpointPower = 1151700,

        /// <summary>
        /// System.Double
        /// </summary>
        VsaXFirstValueQuery = 1151701,

        /// <summary>
        /// System.Double
        /// </summary>
        VsaXMarkerPosition = 1151702,

        /// <summary>
        /// System.String
        /// </summary>
        BwcdMode = 1151703,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gbsMeas = 1151704,

        /// <summary>
        /// System.Boolean
        /// </summary>
        _3gbsWcdpHsdpaupaState = 1151705,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gbsWcdpSyncType = 1151706,

        /// <summary>
        /// System.Boolean
        /// </summary>
        _3gbsWcdpAutoScale = 1151707,

        /// <summary>
        /// System.Boolean
        /// </summary>
        _3gbsWcdpMcaclr = 1151708,

        /// <summary>
        /// System.Boolean
        /// </summary>
        _3gbsWcdpNormalize = 1151709,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gbsWcdpCompareMode = 1151710,

        /// <summary>
        /// System.Boolean
        /// </summary>
        _3gbsWcdpInvertQ = 1151711,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gbsWcdpPowRef = 1151712,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gbsWcdpSideBand = 1151713,

        /// <summary>
        /// System.Double
        /// </summary>
        _3gbsWcdpInactChanThr = 1151714,

        /// <summary>
        /// System.Boolean
        /// </summary>
        _3gbsWcdpPowDifference = 1151715,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gbsWcdpConstB = 1151716,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gbsWcdpMultiframeCaptureLength = 1151717,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gbsWcdpMultiframeFrameAnalyze = 1151718,

        /// <summary>
        /// System.Boolean
        /// </summary>
        _3gbsMcaclrAcpLimit = 1151719,

        /// <summary>
        /// System.Boolean
        /// </summary>
        _3gbsMcaclrAcpLimitRelState = 1151720,

        /// <summary>
        /// System.Boolean
        /// </summary>
        _3gbsMcaclrAcpLimitAbsState = 1151721,

        /// <summary>
        /// System.Boolean
        /// </summary>
        _3gbsMcaclrAcpEditAcpLimit = 1151722,

        /// <summary>
        /// System.Boolean
        /// </summary>
        _3gbsMcaclrAcpEditAcpLimitAbs = 1151723,

        /// <summary>
        /// System.Boolean
        /// </summary>
        _3gbsMcaclrAclrAcpChanPow = 1151724,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gbsMcaclrAcpAbsRel = 1151725,

        /// <summary>
        /// System.String
        /// </summary>
        _3gbsMcaclrAcpAdjustSettings = 1151726,

        /// <summary>
        /// System.String
        /// </summary>
        _3gbsMcaclrAdjustRefLevel = 1151727,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gbsMcaclrAcpAcpRefSettings = 1151728,

        /// <summary>
        /// System.Boolean
        /// </summary>
        _3gbsMcaclrNoiseCorrection = 1151729,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gbsAntenna = 1151730,

        /// <summary>
        /// System.String
        /// </summary>
        _3gbsMarkerToChannel = 1151731,

        /// <summary>
        /// System.String
        /// </summary>
        _3gbsReferenceMarkerToChannel = 1151732,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gbsScode = 1151733,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gbsTimingOffset = 1151734,

        /// <summary>
        /// System.Double
        /// </summary>
        _3gppPdiv = 1151735,

        /// <summary>
        /// System.String
        /// </summary>
        _3gppAseq = 1151736,

        /// <summary>
        /// System.String
        /// </summary>
        _3gppScodeSearch = 1151737,

        /// <summary>
        /// System.String
        /// </summary>
        _3gbsWcdpCtabHeadComm = 1151738,

        /// <summary>
        /// System.String
        /// </summary>
        _3gbsWcdpCtabDel = 1151739,

        /// <summary>
        /// System.String
        /// </summary>
        _3gbsWcdpCtabCopy = 1151740,

        /// <summary>
        /// System.String
        /// </summary>
        _3gbsWcdpCtabName = 1151741,

        /// <summary>
        /// System.Boolean
        /// </summary>
        _3gbsWcdpCtab = 1151742,

        /// <summary>
        /// System.String
        /// </summary>
        _3gbsWcdpCtabSel = 1151743,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gbsResultsSfac = 1151744,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gbsResultsSelectChan = 1151745,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gbsResultsSelectCpichSlot = 1151746,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gbsResult = 1151747,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gbsResultSumView = 1151748,

        /// <summary>
        /// System.String
        /// </summary>
        _3gueMode = 1151749,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gueResult = 1151750,

        /// <summary>
        /// System.Boolean
        /// </summary>
        _3gueRrcFilter = 1151751,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gueMeas = 1151752,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gueCdpBase = 1151753,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gueCdpSideband = 1151754,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gueCdpSelComp = 1151755,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gueCdpChanCode = 1151756,

        /// <summary>
        /// System.Double
        /// </summary>
        _3gueCdpInactChanThr = 1151757,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gueCdpScrCode = 1151758,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gueCdpScrType = 1151759,

        /// <summary>
        /// System.Boolean
        /// </summary>
        _3gueCdpNorm = 1151760,

        /// <summary>
        /// System.Boolean
        /// </summary>
        _3gueCdpInvertQ = 1151761,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gueCdpSfac = 1151762,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gueCdpSlot = 1151763,

        /// <summary>
        /// System.Boolean
        /// </summary>
        _3gueCdpEliminateTailChips = 1151764,

        /// <summary>
        /// System.Boolean
        /// </summary>
        _3gueCdpHsdpaupaState = 1151765,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gueCdpMultiframeCaptureLength = 1151766,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gueCdpMultiframeFrameAnalyze = 1151767,

        /// <summary>
        /// System.Int32
        /// </summary>
        _3gueCdpEvmMeasInterval = 1151768,

        /// <summary>
        /// System.Boolean
        /// </summary>
        _3gueWcdpCtabHsdp = 1151769,

        /// <summary>
        /// System.String
        /// </summary>
        _3gueWcdpCtabComment = 1151770,

        /// <summary>
        /// System.String
        /// </summary>
        _3gueWcdpCtabCopy = 1151771,

        /// <summary>
        /// System.String
        /// </summary>
        _3gueWcdpCtabDel = 1151772,

        /// <summary>
        /// System.String
        /// </summary>
        _3gueWcdpCtabName = 1151773,

        /// <summary>
        /// System.String
        /// </summary>
        _3gueWcdpCtabSel = 1151774,

        /// <summary>
        /// System.Boolean
        /// </summary>
        _3gueWcdpCtab = 1151775,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdbsMeas = 1151776,

        /// <summary>
        /// System.String
        /// </summary>
        TdbsMode = 1151777,

        /// <summary>
        /// System.String
        /// </summary>
        TdbsAdaptSignAutLevelTime = 1151778,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdbsAdaptSignSlotStart = 1151779,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdbsAdaptSignSlotStop = 1151780,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdbsAdjRlevQuery = 1151781,

        /// <summary>
        /// System.String
        /// </summary>
        TdbsCdpLevAdjust = 1151782,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdbsStandard = 1151783,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdbsCaptureLength = 1151784,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdbsScode = 1151785,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdbsCellShifts = 1151786,

        /// <summary>
        /// System.Double
        /// </summary>
        TdbsInactChanThr = 1151787,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdbsSideBand = 1151788,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TdbsInvertQ = 1151789,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TdbsNormalize = 1151790,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TdbsSyncToSlot = 1151791,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TdbsCpichState = 1151792,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdbsCpichCode = 1151793,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdbsCpichPattern = 1151794,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdbsResultType = 1151795,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdbsResultSelChan = 1151796,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdbsResultSelSlot = 1151797,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdbsResultSetCount = 1151798,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdbsResultSetValue = 1151799,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdbsCtabOrder = 1151800,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TdbsCtabState = 1151801,

        /// <summary>
        /// System.String
        /// </summary>
        TdbsCtabSel = 1151802,

        /// <summary>
        /// System.String
        /// </summary>
        TdbsCtabName = 1151803,

        /// <summary>
        /// System.String
        /// </summary>
        TdbsCtabComm = 1151804,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdbsCtabMshift = 1151805,

        /// <summary>
        /// System.String
        /// </summary>
        TdbsCtabCopy = 1151806,

        /// <summary>
        /// System.String
        /// </summary>
        TdbsCtabDel = 1151807,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdbsMeasPvtSwitchPoint = 1151808,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdbsMeasPvtSframes = 1151809,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TdbsMeasPvtHdyn = 1151810,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdueMeas = 1151811,

        /// <summary>
        /// System.String
        /// </summary>
        TdueMode = 1151812,

        /// <summary>
        /// System.String
        /// </summary>
        TdueAdaptSignAutLevelTime = 1151813,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdueAdaptSignSlotStart = 1151814,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdueAdaptSignSlotStop = 1151815,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdueAdjRlevQuery = 1151816,

        /// <summary>
        /// System.String
        /// </summary>
        TdueCdpLevAdjust = 1151817,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdueCaptureLength = 1151818,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdueScode = 1151819,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdueCellShifts = 1151820,

        /// <summary>
        /// System.Double
        /// </summary>
        TdueInactChanThr = 1151821,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdueSideBand = 1151822,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TdueInvertQ = 1151823,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TdueNormalize = 1151824,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TdueSyncToSlot = 1151825,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TdueCpichState = 1151826,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdueCpichCode = 1151827,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdueCpichPattern = 1151828,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdueResultType = 1151829,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdueResultSelChan = 1151830,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdueResultSelSlot = 1151831,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdueResultSetCount = 1151832,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdueResultSetValue = 1151833,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdueCtabOrder = 1151834,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TdueCtabState = 1151835,

        /// <summary>
        /// System.String
        /// </summary>
        TdueCtabSel = 1151836,

        /// <summary>
        /// System.String
        /// </summary>
        TdueCtabName = 1151837,

        /// <summary>
        /// System.String
        /// </summary>
        TdueCtabComm = 1151838,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdueCtabMshift = 1151839,

        /// <summary>
        /// System.String
        /// </summary>
        TdueCtabCopy = 1151840,

        /// <summary>
        /// System.String
        /// </summary>
        TdueCtabDel = 1151841,

        /// <summary>
        /// System.Int32
        /// </summary>
        TdueMeasPvtSframes = 1151842,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TdueMeasPvtHdyn = 1151843,

        /// <summary>
        /// System.String
        /// </summary>
        C2kMode = 1151844,

        /// <summary>
        /// System.String
        /// </summary>
        Mc2kMode = 1151845,

        /// <summary>
        /// System.String
        /// </summary>
        BdoMode = 1151846,

        /// <summary>
        /// System.String
        /// </summary>
        MdoMode = 1151847,

        /// <summary>
        /// System.Boolean
        /// </summary>
        C2kCtableState = 1151848,

        /// <summary>
        /// System.String
        /// </summary>
        C2kCtableSelect = 1151849,

        /// <summary>
        /// System.String
        /// </summary>
        C2kCtableName = 1151850,

        /// <summary>
        /// System.String
        /// </summary>
        C2kCtableComment = 1151851,

        /// <summary>
        /// System.String
        /// </summary>
        C2kCtableCopy = 1151852,

        /// <summary>
        /// System.String
        /// </summary>
        C2kCtableDelete = 1151853,

        /// <summary>
        /// System.String
        /// </summary>
        C2kCtableRestore = 1151854,

        /// <summary>
        /// System.Boolean
        /// </summary>
        C2kCdpNormalize = 1151855,

        /// <summary>
        /// System.Boolean
        /// </summary>
        C2kCdpQinvert = 1151856,

        /// <summary>
        /// System.Int32
        /// </summary>
        C2kCdpSlot = 1151857,

        /// <summary>
        /// System.Int32
        /// </summary>
        C2kCdpSband = 1151858,

        /// <summary>
        /// System.Double
        /// </summary>
        C2kCdpIcthres = 1151859,

        /// <summary>
        /// System.String
        /// </summary>
        C2kCdpLevelAdjust = 1151860,

        /// <summary>
        /// System.Boolean
        /// </summary>
        C2kCdpTpmeas = 1151861,

        /// <summary>
        /// System.Int32
        /// </summary>
        C2kAchPres = 1151862,

        /// <summary>
        /// System.String
        /// </summary>
        C2kAdjustRefLevel = 1151863,

        /// <summary>
        /// System.Double
        /// </summary>
        C2kPdiv = 1151864,

        /// <summary>
        /// System.String
        /// </summary>
        C2kAchRefAuto = 1151865,

        /// <summary>
        /// System.Double
        /// </summary>
        C2kLimTrans = 1151866,

        /// <summary>
        /// System.Int32
        /// </summary>
        C2kSetToAnalyze = 1151867,

        /// <summary>
        /// System.Double
        /// </summary>
        C2kPercentMarker = 1151868,

        /// <summary>
        /// System.String
        /// </summary>
        C2kLimRestore = 1151869,

        /// <summary>
        /// System.Int32
        /// </summary>
        C2kMeasSelect = 1151870,

        /// <summary>
        /// System.Int32
        /// </summary>
        C2kBclass = 1151871,

        /// <summary>
        /// System.Int32
        /// </summary>
        C2kSetCount = 1151872,

        /// <summary>
        /// System.Int32
        /// </summary>
        C2kResultType = 1151873,

        /// <summary>
        /// System.Double
        /// </summary>
        C2kLimVal = 1151874,

        /// <summary>
        /// System.Int32
        /// </summary>
        C2kLimMode = 1151875,

        /// <summary>
        /// System.String
        /// </summary>
        C2kLimStoreFile = 1151876,

        /// <summary>
        /// System.Int32
        /// </summary>
        C2kCdpPnoffset = 1151877,

        /// <summary>
        /// System.Int32
        /// </summary>
        C2kCdpIqlengthFsp = 1151878,

        /// <summary>
        /// System.Boolean
        /// </summary>
        C2kMcarrierState = 1151879,

        /// <summary>
        /// System.Boolean
        /// </summary>
        C2kMcarrierEnhAlg = 1151880,

        /// <summary>
        /// System.Boolean
        /// </summary>
        C2kMcarrierFilterState = 1151881,

        /// <summary>
        /// System.Int32
        /// </summary>
        C2kMcarrierFilterType = 1151882,

        /// <summary>
        /// System.Double
        /// </summary>
        C2kMcarrierRollOff = 1151883,

        /// <summary>
        /// System.Double
        /// </summary>
        C2kMcarrierCutOffFreq = 1151884,

        /// <summary>
        /// System.String
        /// </summary>
        C2kMarkerTdpich = 1151885,

        /// <summary>
        /// System.String
        /// </summary>
        C2kMarkerPich = 1151886,

        /// <summary>
        /// System.Int32
        /// </summary>
        C2kCdpSfactor = 1151887,

        /// <summary>
        /// System.Int32
        /// </summary>
        C2kCdpPreference = 1151888,

        /// <summary>
        /// System.Int32
        /// </summary>
        C2kCdpOrder = 1151889,

        /// <summary>
        /// System.Int32
        /// </summary>
        C2kCdpIqlengthFsqFsu = 1151890,

        /// <summary>
        /// System.Int32
        /// </summary>
        C2kCdpCode = 1151891,

        /// <summary>
        /// System.Int32
        /// </summary>
        C2kCdpAntenna = 1151892,

        /// <summary>
        /// System.Boolean
        /// </summary>
        C2kForceDv = 1151893,

        /// <summary>
        /// System.Boolean
        /// </summary>
        BdoPvtBurstFit = 1151894,

        /// <summary>
        /// System.Boolean
        /// </summary>
        BdoPvtFailRestart = 1151895,

        /// <summary>
        /// System.Int32
        /// </summary>
        BdoLimPvtReference = 1151896,

        /// <summary>
        /// System.Double
        /// </summary>
        BdoLimPvtValue = 1151897,

        /// <summary>
        /// System.Boolean
        /// </summary>
        BdoLimPvtList = 1151898,

        /// <summary>
        /// System.String
        /// </summary>
        BdoLimPvtRestore = 1151899,

        /// <summary>
        /// System.Int32
        /// </summary>
        BdoRfslot = 1151900,

        /// <summary>
        /// System.Int32
        /// </summary>
        BdoCdpMmode = 1151901,

        /// <summary>
        /// System.Int32
        /// </summary>
        BdoCdpMapping = 1151902,

        /// <summary>
        /// System.Int32
        /// </summary>
        BdoCdpCtype = 1151903,

        /// <summary>
        /// System.Boolean
        /// </summary>
        BdoCdpOverview = 1151904,

        /// <summary>
        /// System.Boolean
        /// </summary>
        BdoCdpAverage = 1151905,

        /// <summary>
        /// System.Int32
        /// </summary>
        BdoCdpIqlengthFsqFsu = 1151906,

        /// <summary>
        /// System.Int32
        /// </summary>
        BdoCdpCode = 1151907,

        /// <summary>
        /// System.Int32
        /// </summary>
        BdoRevision = 1151908,

        /// <summary>
        /// System.Int32
        /// </summary>
        Mc2kLimMode = 1151909,

        /// <summary>
        /// System.String
        /// </summary>
        Mc2kMarkerPich = 1151910,

        /// <summary>
        /// System.Int32
        /// </summary>
        Mc2kCdpMapping = 1151911,

        /// <summary>
        /// System.Int32
        /// </summary>
        Mc2kCdpPreference = 1151912,

        /// <summary>
        /// System.Int32
        /// </summary>
        Mc2kCdpOrder = 1151913,

        /// <summary>
        /// System.Boolean
        /// </summary>
        Mc2kCdpOverview = 1151914,

        /// <summary>
        /// System.String
        /// </summary>
        Mc2kCdpLcodeMask = 1151915,

        /// <summary>
        /// System.String
        /// </summary>
        Mc2kCdpLcodeOffset = 1151916,

        /// <summary>
        /// System.Int32
        /// </summary>
        Mc2kCdpCode = 1151917,

        /// <summary>
        /// System.Int32
        /// </summary>
        Mc2kCdpSfactor = 1151918,

        /// <summary>
        /// System.Int32
        /// </summary>
        Mc2kCdpIqlengthFsqFsu = 1151919,

        /// <summary>
        /// System.Int32
        /// </summary>
        Mc2kCdpIqlengthFsp = 1151920,

        /// <summary>
        /// System.Int32
        /// </summary>
        Mc2kLcodeMode = 1151921,

        /// <summary>
        /// System.String
        /// </summary>
        MdoCdpLcodeI = 1151922,

        /// <summary>
        /// System.String
        /// </summary>
        MdoCdpLcodeQ = 1151923,

        /// <summary>
        /// System.Int32
        /// </summary>
        MdoCdpCode = 1151924,

        /// <summary>
        /// System.Int32
        /// </summary>
        MdoCdpIqlengthFsqFsu = 1151925,

        /// <summary>
        /// System.Int32
        /// </summary>
        MdoCdpIqlengthFsp = 1151926,

        /// <summary>
        /// System.Int32
        /// </summary>
        MdoCdpOperation = 1151927,

        /// <summary>
        /// System.Boolean
        /// </summary>
        MdoCdpAverage = 1151928,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanStan = 1151929,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanPvtRpow = 1151930,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanPvtSel = 1151931,

        /// <summary>
        /// System.String
        /// </summary>
        WlanPvt = 1151932,

        /// <summary>
        /// System.String
        /// </summary>
        WlanSpecFlat = 1151933,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanBtyp = 1151934,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanPvtAver = 1151935,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanPowExpIq = 1151936,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanPowExpRf = 1151937,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WlanPowAuto = 1151938,

        /// <summary>
        /// System.String
        /// </summary>
        WlanPowAutoOnce = 1151939,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanChan = 1151940,

        /// <summary>
        /// System.String
        /// </summary>
        WlanEvmEcar = 1151941,

        /// <summary>
        /// System.String
        /// </summary>
        WlanEvmEsym = 1151942,

        /// <summary>
        /// System.String
        /// </summary>
        WlanSpecMask = 1151943,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanSpecMaskSel = 1151944,

        /// <summary>
        /// System.String
        /// </summary>
        WlanSpecFft = 1151945,

        /// <summary>
        /// System.String
        /// </summary>
        WlanSpecAcpr = 1151946,

        /// <summary>
        /// System.String
        /// </summary>
        WlanConsCsym = 1151947,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WlanBursCounStat = 1151948,

        /// <summary>
        /// System.String
        /// </summary>
        WlanConsCcar = 1151949,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanConsCarrSel = 1151950,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanConsCarrSelNr = 1151951,

        /// <summary>
        /// System.String
        /// </summary>
        WlanStatCcdf = 1151952,

        /// <summary>
        /// System.String
        /// </summary>
        WlanStatBstr = 1151953,

        /// <summary>
        /// System.String
        /// </summary>
        WlanStatSfi = 1151954,

        /// <summary>
        /// System.String
        /// </summary>
        WlanInit = 1151955,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WlanInitCont = 1151956,

        /// <summary>
        /// System.String
        /// </summary>
        WlanMode = 1151957,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanBursCoun = 1151958,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WlanSwapIq = 1151959,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanAchMode = 1151960,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanUnitPre = 1151961,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanUnitEvm = 1151962,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanUnitGimb = 1151963,

        /// <summary>
        /// System.String
        /// </summary>
        WlanPre = 1151964,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanSrat = 1151965,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanFlat = 1151966,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanPowAutoSweTime = 1151967,

        /// <summary>
        /// System.String
        /// </summary>
        WlanIqLoad = 1151968,

        /// <summary>
        /// System.String
        /// </summary>
        WlanIqSave = 1151969,

        /// <summary>
        /// System.String
        /// </summary>
        WlanCalcBurs = 1151970,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanPreSel = 1151971,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanFftStartOffset = 1151972,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WlanDispTabl = 1151973,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WlanDispAutoY = 1151974,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanYPdiv = 1151975,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanTris = 1151976,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanTfal = 1151977,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanFerr = 1151978,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanSymb = 1151979,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanIqof = 1151980,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanEvm = 1151981,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanEvmAll = 1151982,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanEvmData = 1151983,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanEvmPil = 1151984,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanTfalRes = 1151985,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanTrisRes = 1151986,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanFerrRes = 1151987,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanSymbRes = 1151988,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanIqofRes = 1151989,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanEvmRes = 1151990,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanLimFailRes = 1151991,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanEvmAllRes = 1151992,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanEvmDataRes = 1151993,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanEvmPilRes = 1151994,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanAcpAdj = 1151995,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanAcpAdjRes = 1151996,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanAcpAlt = 1151997,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanAcpAltRes = 1151998,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanSpecMaskX = 1151999,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanSpecMaskY = 1152000,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanMarkSymb = 1152001,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanMarkCarr = 1152002,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanMarkY = 1152003,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanFetcBursCount = 1152004,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanFetcPre = 1152005,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanFetcPayl = 1152006,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanFetcRms = 1152007,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanFetcPeak = 1152008,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanFetcCres = 1152009,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanFetcTris = 1152010,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanFetcTfal = 1152011,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanFetcFerr = 1152012,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanFetcSymb = 1152013,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanFetcIqof = 1152014,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanFetcGimb = 1152015,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanFetcQuad = 1152016,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanFetcEvm = 1152017,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanFetcEvmDir = 1152018,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanFetcEvmAll = 1152019,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanFetcEvmData = 1152020,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanFetcEvmPilot = 1152021,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanFetcSymbCount = 1152022,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanEgatHoldSamp = 1152023,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanEgatHoldTime = 1152024,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanEgatLengSamp = 1152025,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanEgatLengTime = 1152026,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WlanEgatLink = 1152027,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WlanEgat = 1152028,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanTrigMode = 1152029,

        /// <summary>
        /// System.Double
        /// </summary>
        WlanTrigPowLev = 1152030,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WlanTrigPowLevAuto = 1152031,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WlanCest = 1152032,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WlanSigs = 1152033,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WlanBconAuto = 1152034,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanBan = 1152035,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WlanSymEqu = 1152036,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanSym = 1152037,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WlanDbytEqu = 1152038,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanDbyt = 1152039,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WlanDurEqu = 1152040,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanDur = 1152041,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WlanTracLev = 1152042,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WlanTracPhas = 1152043,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WlanTracTime = 1152044,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanSemType = 1152045,

        /// <summary>
        /// System.Int32
        /// </summary>
        WlanSemClass = 1152046,

        /// <summary>
        /// System.String
        /// </summary>
        WlanSemFile = 1152047,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxStan = 1152048,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxMode = 1152049,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxBandChan = 1152050,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WimaxBursCounStat = 1152051,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxBursCoun = 1152052,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxFban = 1152053,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxChan = 1152054,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxIgr = 1152055,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WimaxPowAuto = 1152056,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxPowAutoOnce = 1152057,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxPowAutoSweTime = 1152058,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxPowExpIq = 1152059,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxPowExpRf = 1152060,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxRlevIq = 1152061,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxRlevRf = 1152062,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxSrat = 1152063,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxFlat = 1152064,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxIqLoad = 1152065,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxIqSave = 1152066,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WimaxSwapIq = 1152067,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxTableUnit = 1152068,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxRssiAvgParam = 1152069,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxCalcBurs = 1152070,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxFormatSelect = 1152071,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxNfft = 1152072,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WimaxYigAuto = 1152073,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxFrameLoad = 1152074,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxFrameSave = 1152075,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxSmuAddress = 1152076,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxTtcFrame = 1152077,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxTtcStart = 1152078,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WimaxFsbwRatioAuto = 1152079,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFsbwSrate = 1152080,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFsbwChannelBw = 1152081,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxNumberOfAntennas = 1152082,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxStatBstr = 1152083,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxBstrSel = 1152084,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxBstrSelNr = 1152085,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxStatCcdf = 1152086,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxStatBsum = 1152087,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxPvt = 1152088,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxBursPvtBurs = 1152089,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxPvtSel = 1152090,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxConsSel = 1152091,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxConsSelNr = 1152092,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxConsCcar = 1152093,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxConsCsym = 1152094,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxPre = 1152095,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxPreSel = 1152096,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxUnitPre = 1152097,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxEvmEcar = 1152098,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxEvmEsym = 1152099,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxUnitEvm = 1152100,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxSpecAcprSel = 1152101,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxAchMode = 1152102,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxAchAcp = 1152103,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxAchBwAch = 1152104,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxAchBwAlt = 1152105,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxAchBwChan = 1152106,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxAchSpacAch = 1152107,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxAchSpacAlt = 1152108,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WimaxAcpNoiseCorr = 1152109,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxSpecAcpr = 1152110,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxSpecFft = 1152111,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxSpecFlat = 1152112,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxSpecMask = 1152113,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxSpecMaskSel = 1152114,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxSpecPreamble = 1152115,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxAcprTime = 1152116,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WimaxAcprTimeAuto = 1152117,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxSemType = 1152118,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxSemClass = 1152119,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxSemFile = 1152120,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WimaxDispAutoY = 1152121,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WimaxDispTabl = 1152122,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxDispTablList = 1152123,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxYPdiv = 1152124,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxEvmAll = 1152125,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxEvmData = 1152126,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFerr = 1152127,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxSst = 1152128,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxSymb = 1152129,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxIqof = 1152130,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxBerp = 1152131,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxUserr = 1152132,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxEvmAllRes = 1152133,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxEvmDataRes = 1152134,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxFerrRes = 1152135,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxLimFailRes = 1152136,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxSpecMaskX = 1152137,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxSpecMaskY = 1152138,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxSstRes = 1152139,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxSymbRes = 1152140,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxIqofRes = 1152141,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxBerpRes = 1152142,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxUserrRes = 1152143,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxMarkBurst = 1152144,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxMarkCarr = 1152145,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxMarkSymb = 1152146,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxMarkY = 1152147,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxFetcBursCount = 1152148,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFetcCres = 1152149,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFetcEvmAll = 1152150,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFetcEvm = 1152151,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFetcEvmData = 1152152,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFetcEvmPilot = 1152153,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFetcEvmPreamble = 1152154,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFetcFerr = 1152155,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFetcGimb = 1152156,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFetcIqof = 1152157,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFetcQuad = 1152158,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFetcRms = 1152159,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFetcSst = 1152160,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFetcSymb = 1152161,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxFetcSymbCount = 1152162,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFetcRmsAll = 1152163,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFetcRmsPilot = 1152164,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFetcRmsData = 1152165,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFetcRmsDlpr = 1152166,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFetchNpilRms = 1152167,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxFetcZoneCount = 1152168,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFetcTdomPreamble = 1152169,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFetcTdomSubframe = 1152170,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFetcTdomZone = 1152171,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFetcUserr = 1152172,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxFetcZoneStatCount = 1152173,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WimaxEgat = 1152174,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxEgatHoldSamp = 1152175,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxEgatHoldTime = 1152176,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxEgatLengSamp = 1152177,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxEgatLengTime = 1152178,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WimaxEgatLink = 1152179,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxTrigMode = 1152180,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxTrigPowLev = 1152181,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WimaxTrigPowLevAuto = 1152182,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxBan = 1152183,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxBconAuto = 1152184,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxBtyp = 1152185,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WimaxCest = 1152186,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxLmode = 1152187,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WimaxSigs = 1152188,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxSubchannel = 1152189,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WimaxSubchannelState = 1152190,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxSubchannelUlph = 1152191,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WimaxSymEqu = 1152192,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxSym = 1152193,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WimaxTracLev = 1152194,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WimaxTracPhas = 1152195,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WimaxTracTime = 1152196,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxTddTtg = 1152197,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxFftOffset = 1152198,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxCestOfdma = 1152199,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxTrackPilots = 1152200,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxTracCest = 1152201,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxDlIdcell = 1152202,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WimaxDownLinkIdCellAuto = 1152203,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WimaxFramePredefinedMap = 1152204,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxDlPreambleMode = 1152205,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxDlPreambleIndex = 1152206,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxDlSubchannel = 1152207,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxUlFrame = 1152208,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxUlSubchannelBitmap = 1152209,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxUlFrameCrlen = 1152210,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxZoneCountQuery = 1152211,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WimaxZoneAnalyzeState = 1152212,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxZoneType = 1152213,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxZoneSegmentNum = 1152214,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxZonePermBase = 1152215,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxZonePrbs = 1152216,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxZoneSpaceTimeCoding = 1152217,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxZoneMatrix = 1152218,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxZoneDelete = 1152219,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxZoneReset = 1152220,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxZoneCount = 1152221,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WimaxZoneCountState = 1152222,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxZoneSymbolCount = 1152223,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxZoneSymbolOffset = 1152224,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxZoneBurstCountQuery = 1152225,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxZoneBurstFormat = 1152226,

        /// <summary>
        /// System.Boolean
        /// </summary>
        WimaxZoneBurstOffsetAuto = 1152227,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxZoneBurstSlotDuration = 1152228,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxZoneBurstSubcCount = 1152229,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxZoneBurstSubcOffset = 1152230,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxZoneBurstSymbolCount = 1152231,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxZoneBurstSymbolOffset = 1152232,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxZoneBurstPower = 1152233,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxZoneBurstType = 1152234,

        /// <summary>
        /// System.Int32
        /// </summary>
        WimaxZoneBurstMatrix = 1152235,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxZoneBurstDelete = 1152236,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxZoneBurstReset = 1152237,

        /// <summary>
        /// System.String
        /// </summary>
        WimaxRefresh = 1152238,

        /// <summary>
        /// System.Double
        /// </summary>
        WimaxQueryTtc = 1152239,

        /// <summary>
        /// System.String
        /// </summary>
        LteDownlinkMode = 1152240,

        /// <summary>
        /// System.String
        /// </summary>
        LteDownlinkStoreDemodulationSettings = 1152241,

        /// <summary>
        /// System.String
        /// </summary>
        LteDownlinkLoadDemodulationSettings = 1152242,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkDuplexing = 1152243,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkChannelBandwidth = 1152244,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkNumberOfResourceBlocks = 1152245,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkCyclicPrefix = 1152246,

        /// <summary>
        /// System.Boolean
        /// </summary>
        LteDownlinkAutoReferenceLevel = 1152247,

        /// <summary>
        /// System.Double
        /// </summary>
        LteDownlinkReferenceLevel = 1152248,

        /// <summary>
        /// System.Double
        /// </summary>
        LteDownlinkReferenceLevelInBbMode = 1152249,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkMimoConfiguration = 1152250,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkMimoAntenna = 1152251,

        /// <summary>
        /// System.Boolean
        /// </summary>
        LteDownlinkIqSwap = 1152252,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkInput = 1152253,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkChannelEstimation = 1152254,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkEvmCalculationMethod = 1152255,

        /// <summary>
        /// System.Boolean
        /// </summary>
        LteDownlinkScramblingOfCodedBits = 1152256,

        /// <summary>
        /// System.Boolean
        /// </summary>
        LteDownlinkAutoDemodulation = 1152257,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkPhaseTracking = 1152258,

        /// <summary>
        /// System.Boolean
        /// </summary>
        LteDownlinkTiming = 1152259,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkTddUlDlAllocations = 1152260,

        /// <summary>
        /// System.String
        /// </summary>
        LteDownlinkCidGroupAuto = 1152261,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkCellIdentityGroup = 1152262,

        /// <summary>
        /// System.String
        /// </summary>
        LteDownlinkCellIdentityAuto = 1152263,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkCellIdentity = 1152264,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkPdschConfigurableSubframes = 1152265,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkPdschUsedAllocations = 1152266,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkPdschAllocationModulation = 1152267,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkPdschAllocationResourceBlocks = 1152268,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkPdschAllocationResourceBlocksOffset = 1152269,

        /// <summary>
        /// System.Double
        /// </summary>
        LteDownlinkPdschAllocationPower = 1152270,

        /// <summary>
        /// System.Double
        /// </summary>
        LteDownlinkReferencePower = 1152271,

        /// <summary>
        /// System.Double
        /// </summary>
        LteDownlinkPsyncRelativePower = 1152272,

        /// <summary>
        /// System.Double
        /// </summary>
        LteDownlinkSsyncRelativePower = 1152273,

        /// <summary>
        /// System.Boolean
        /// </summary>
        LteDownlinkPbchState = 1152274,

        /// <summary>
        /// System.Double
        /// </summary>
        LteDownlinkPbchRelativePower = 1152275,

        /// <summary>
        /// System.Boolean
        /// </summary>
        LteDownlinkPcfichState = 1152276,

        /// <summary>
        /// System.Double
        /// </summary>
        LteDownlinkPcfichRelativePower = 1152277,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkPhichNumberOfGroups = 1152278,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkPhichDuration = 1152279,

        /// <summary>
        /// System.Double
        /// </summary>
        LteDownlinkPhichRelativePower = 1152280,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkPdcchNumberOfPdcchs = 1152281,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkPdcchFormat = 1152282,

        /// <summary>
        /// System.Double
        /// </summary>
        LteDownlinkPdcchRelativePower = 1152283,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkMeasurementType = 1152284,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkSubframeSelection = 1152285,

        /// <summary>
        /// System.String
        /// </summary>
        LteDownlinkSubframeSelectionAll = 1152286,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkEvmUnits = 1152287,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkBitStreamFormat = 1152288,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkSemChannel = 1152289,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkAcpAssumedAdjChannelCarrier = 1152290,

        /// <summary>
        /// System.Boolean
        /// </summary>
        LteDownlinkAcpNoiseCorrection = 1152291,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkConstellationLocation = 1152292,

        /// <summary>
        /// System.Double
        /// </summary>
        LteDownlinkEvmPdschQpskResult = 1152293,

        /// <summary>
        /// System.Double
        /// </summary>
        LteDownlinkEvmPdsch16qamResult = 1152294,

        /// <summary>
        /// System.Double
        /// </summary>
        LteDownlinkEvmPdsch64qamResult = 1152295,

        /// <summary>
        /// System.Double
        /// </summary>
        LteDownlinkEvmAllResult = 1152296,

        /// <summary>
        /// System.Double
        /// </summary>
        LteDownlinkEvmPhysicalChannelResult = 1152297,

        /// <summary>
        /// System.Double
        /// </summary>
        LteDownlinkEvmPhysicalSignalResult = 1152298,

        /// <summary>
        /// System.Double
        /// </summary>
        LteDownlinkFrequencyErrorResult = 1152299,

        /// <summary>
        /// System.Double
        /// </summary>
        LteDownlinkSamplingErrorResult = 1152300,

        /// <summary>
        /// System.Double
        /// </summary>
        LteDownlinkIqOffsetResult = 1152301,

        /// <summary>
        /// System.Double
        /// </summary>
        LteDownlinkIqGainImbalanceResult = 1152302,

        /// <summary>
        /// System.Double
        /// </summary>
        LteDownlinkIqQuadratureErrorResult = 1152303,

        /// <summary>
        /// System.Double
        /// </summary>
        LteDownlinkFramePowerResult = 1152304,

        /// <summary>
        /// System.Double
        /// </summary>
        LteDownlinkCrestFactorResult = 1152305,

        /// <summary>
        /// System.Double
        /// </summary>
        LteDownlinkTriggerToFrameResult = 1152306,

        /// <summary>
        /// System.Double
        /// </summary>
        LteDownlinkOstpResult = 1152307,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkDetectedCellIdentityGroup = 1152308,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkDetectedCellIdentity = 1152309,

        /// <summary>
        /// System.Int32
        /// </summary>
        LteDownlinkDetectedCyclicPrefix = 1152310,

        /// <summary>
        /// System.String
        /// </summary>
        TetraMode = 1152311,

        /// <summary>
        /// System.Int32
        /// </summary>
        TetraBurstType = 1152312,

        /// <summary>
        /// System.Int32
        /// </summary>
        TetraChannelBandwidth = 1152313,

        /// <summary>
        /// System.Double
        /// </summary>
        TetraReferenceLevelForRfMeasurement = 1152314,

        /// <summary>
        /// System.Int32
        /// </summary>
        TetraReferenceLevelForIqMeasurement = 1152315,

        /// <summary>
        /// System.Double
        /// </summary>
        TetraTedsSlotDuration = 1152316,

        /// <summary>
        /// System.Int32
        /// </summary>
        TetraNumberOfBurst = 1152317,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TetraOverallBurstCount = 1152318,

        /// <summary>
        /// System.Int32
        /// </summary>
        TetraTriggerMode = 1152319,

        /// <summary>
        /// System.Double
        /// </summary>
        TetraTriggerPowerLevel = 1152320,

        /// <summary>
        /// System.Double
        /// </summary>
        TetraTriggerIqLevel = 1152321,

        /// <summary>
        /// System.Int32
        /// </summary>
        TetraTriggerTedsSlotDuration = 1152322,

        /// <summary>
        /// System.Int32
        /// </summary>
        TetraInputSelect = 1152323,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TetraIqSetting = 1152324,

        /// <summary>
        /// System.Double
        /// </summary>
        TetraIqCenterFrequency = 1152325,

        /// <summary>
        /// System.Int32
        /// </summary>
        TetraPayloadModulation = 1152326,

        /// <summary>
        /// System.Int32
        /// </summary>
        TetraMaximumCarrierOffset = 1152327,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TetraPilotTracking = 1152328,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TetraCompensateAmplitudeDroop = 1152329,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TetraCompensateIqOffset = 1152330,

        /// <summary>
        /// System.Double
        /// </summary>
        TetraMoveMarker = 1152331,

        /// <summary>
        /// System.Double
        /// </summary>
        TetraMarkerAmplitude = 1152332,

        /// <summary>
        /// System.Double
        /// </summary>
        TetraEvmSummaryLimits = 1152333,

        /// <summary>
        /// System.Double
        /// </summary>
        TetraAcpLimits = 1152334,

        /// <summary>
        /// System.Double
        /// </summary>
        TetraSummaryTableMeasurementResult = 1152335,

        /// <summary>
        /// System.Double
        /// </summary>
        TetraFetchAcpMeasurement = 1152336,

        /// <summary>
        /// System.Double
        /// </summary>
        TetraFetchAcpModulationRbw = 1152337,

        /// <summary>
        /// System.Double
        /// </summary>
        TetraPvtMeasurement = 1152338,

        /// <summary>
        /// System.Double
        /// </summary>
        TetraPvtTriggerToSyncTime = 1152339,

        /// <summary>
        /// System.Double
        /// </summary>
        TetraSpectrumFftMeasurementRbw = 1152340,

        /// <summary>
        /// System.Double
        /// </summary>
        TetraTraceXAxisMinMax = 1152341,

        /// <summary>
        /// System.Double
        /// </summary>
        TetraIqSampleRate = 1152342,

        /// <summary>
        /// System.Int32
        /// </summary>
        TetraResultSummaryLimitCheckResult = 1152343,

        /// <summary>
        /// System.Int32
        /// </summary>
        TetraAcpLimitCheckResult = 1152344,

        /// <summary>
        /// System.Double
        /// </summary>
        TetraYAxisTrace = 1152345,

        /// <summary>
        /// System.Boolean
        /// </summary>
        TetraYAxisTraceAuto = 1152346,

        /// <summary>
        /// System.String
        /// </summary>
        TetraMeasurementType = 1152347,

        /// <summary>
        /// System.Int32
        /// </summary>
        TetraYAxisTraceUnit = 1152348,

        /// <summary>
        /// System.Int32
        /// </summary>
        TetraConstellationSymbolSettings = 1152349,

        /// <summary>
        /// System.Int32
        /// </summary>
        TetraConstellationCarrierNumber = 1152350,

        /// <summary>
        /// System.Int32
        /// </summary>
        TetraEvmResultUnits = 1152351,

        /// <summary>
        /// System.Int32
        /// </summary>
        TetraGainImbalanceResultUnits = 1152352,

        /// <summary>
        /// System.String
        /// </summary>
        TetraBurstRecalc = 1152353,

        /// <summary>
        /// System.String
        /// </summary>
        IdQueryResponse = 1152354,

        /// <summary>
        /// System.Int32
        /// </summary>
        MarkerOutputFormat = 1152355,
    }
}
